/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef ACLMANAGER_I_H
#define ACLMANAGER_I_H

#include <QtCore/QObject>
#include <QtCore/QHash>
#include <QtCore/QReadWriteLock>

#include <diamesh/qiceobj_accessrights.h>

#include "diamesh/usermanager.h"


/*! \namespace Diamesh
 *
 * Diamesh dedicated namespace
 */
namespace Diamesh
{

class SqlDatabase;
class AccessRightsHelper;
class ACLStorage;
class HSetManagerI;

/*! \class ACLManagerI
 * \brief Access Control List manager implementation
 *
 */
class ACLManagerI : public _Diamesh_ACLManager
{
    Q_OBJECT

    friend class MasterI;
public:
    explicit ACLManagerI(ACLStorage *storage, HSetManagerI *hSetManager, Ice::ObjectAdapterPtr adapter);
    virtual ~ACLManagerI();

    ACLManagerPrx proxy() const;
    AccessRightsHelper *helper() const;

    void registerAccessRight(const QString &rightName);
    void registerACLTargetType(const QString &targetIceId, const QString &targetType);
    virtual AccessRightList getRegisteredRights(const ::Ice::Current &current = Ice::Current());
    virtual StringStringMap getTargetTypes(const ::Ice::Current &current = Ice::Current());
    virtual AccessControlList getACL(const std::string &target, const std::string &subject, const ::Ice::Current &current = Ice::Current());
    virtual StringList getSubjectsForTarget(const ACLTargetPtr &target, const ::Ice::Current &current = Ice::Current());
    virtual ACLList getACLsForTarget(const ACLTargetPtr &target, const ::Ice::Current &current = Ice::Current());
    virtual bool setACL(const std::string &target, const std::string &subject, const LongList &rightsIds, Ice::Long hSetId = -1, const ::Ice::Current &current = Ice::Current());
    virtual bool initACL(const std::string &target, const std::string &subject, const LongList &rightsIds, Ice::Long hSetId = -1, const Diamesh::UserPtr &currentUser = 0);
    virtual bool removeACL(const std::string &target, const std::string &subject, const ::Ice::Current &current = Ice::Current());
    virtual void targetRemoved(const std::string &target, const ::Ice::Current &current = Ice::Current());
    virtual void subjectRemoved(const std::string &subject, const ::Ice::Current &current = Ice::Current());

    virtual bool testRightForObject(const ACLTargetPtr &target, const UserPtr &subject, Ice::Long rightId, const ::Ice::Current &current = Ice::Current());
    virtual bool testRight(const std::string &target, const UserPtr &subject, Ice::Long rightId, const ::Ice::Current &current = Ice::Current());

    bool testRightForObject(const ACLTargetPtr &target, const UserPtr &subject, const QString &rigthName);
    bool testRight(const QString &target, const UserPtr &subject, const QString &rightName);

    AccessRightsHelper *accessRightsHelper() const;

protected:
    void _init();
    virtual bool _setACL(const std::string &target, const std::string &subject, const LongList &rightsIds, Ice::Long hSetId = -1, const Diamesh::UserPtr &currentUser = 0, bool silent = false);

protected slots:
    void _aclAddedSlot(QString target, QString subject);
    void _aclChangedSlot(QString target, QString subject);
    void _aclRemovedSlot(QString target, QString subject);

private:
    QStringList _targetsFromDetails(const QPair<QString, QString> &targetDetails) const;
    bool _testRight(const QString &target, const UserPtr &subject, Ice::Long rightId) const;
    bool _testRight(const QStringList &targetList, const UserPtr &subject, Ice::Long rightId, qlonglong targetRank = -1) const;
    bool _canUserUpdateAcl(const QString &target, const QString &subject, const UserPtr &user) const;

    ACLStorage *_storage;
    HSetManagerI *_hSetManager;
    Ice::ObjectAdapterPtr _adapter;
    Ice::Identity _id;
    AccessRightsHelper *_helper;
    QHash<QString, QString> _targetTypeHash;
};

/*! \class ACLStorage
 * \brief Access Control List storage interface
 *
 */
class ACLStorage : public QObject
{
    Q_OBJECT

    friend class ACLManagerI;
public:
    ACLStorage(QObject *parent = 0);
    virtual ~ACLStorage();

    virtual bool hasACL(const QString &target, const QString &subject);
    virtual AccessControlList getACL(const QString &target, const QString &subject);
    virtual QStringList getSubjectsForTarget(const QString &target);
    virtual QList<AccessControlList> getACLsForTarget(const QString &target);

    bool addAcl(const QString &target, const QString &subject, const LongList &rightsIds, qlonglong hSetId, bool silent = false);
    bool updateACL(const QString &target, const QString &subject, const LongList &rightsIds, qlonglong hSetId);
    bool removeACL(const QString &target, const QString &subject);

    virtual void targetRemoved(const QString &target);
    virtual void subjectRemoved(const QString &subject);

signals:
    void aclAdded(QString target, QString subject);
    void aclChanged(QString target, QString subject);
    void aclRemoved(QString target, QString subject);

protected:
    virtual void _init(AccessRightsHelper *helper);
    virtual void _initStorage() = 0;

    virtual bool _storageAddAcl(const AccessControlList &newAcl) = 0;
    virtual bool _storageUpdateAcl(const AccessControlList &updatedAcl) = 0;
    virtual bool _storageRemoveAcl(const QString &target, const QString &subject) = 0;

    AccessControlList _getACL(const QString &target, const QString &subject) const;
    virtual bool _insertACL(const AccessControlList &acl, bool silent = false);

    AccessRightsHelper *_helper;

private:
    QReadWriteLock _lock;
    QHash<QPair<QString, QString>, AccessControlList> _aclHash;
    QHash<QString, QStringList> _targetToSubjectsHash;
};

/*! \class SqlACLStorage
 * \brief Access Control List storage implementation based on Sql database
 *
 */
class SqlACLStorage : public ACLStorage
{
    Q_OBJECT

public:
    explicit SqlACLStorage(SqlDatabase *db, QObject *parent = 0);
    virtual ~SqlACLStorage();

protected:
    virtual void _initStorage();
    virtual bool _storageAddAcl(const AccessControlList &newAcl);
    virtual bool _storageUpdateAcl(const AccessControlList &updatedAcl);
    virtual bool _storageRemoveAcl(const QString &target, const QString &subject);

private:
    SqlDatabase *_db;
};

} // namespace Diamesh

#endif // ACLMANAGER_I_H
