/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HSETMANAGER_I_H
#define HSETMANAGER_I_H

#include <QtCore/QObject>
#include <QtCore/QReadWriteLock>

#include <diamesh/qiceobj_hsetmanager.h>


/*! \namespace Diamesh
 *
 * Diamesh dedicated namespace
 */
namespace Diamesh
{

class SqlDatabase;
class HSetStorage;

/*! \class HSetManagerI
 * \brief Hierarchical Set manager implementation
 *
 */
class HSetManagerI : public _Diamesh_HSetManager
{
    Q_OBJECT

public:
    explicit HSetManagerI(HSetStorage *storage, Ice::ObjectAdapterPtr adapter);
    virtual ~HSetManagerI();

    HSetManagerPrx proxy() const;

    Ice::Long addHSet(Ice::Long rank, const std::string &name, const std::string &description, const::Ice::Current &current = Ice::Current());
    bool updateHSet(const HierarchicalSet &updatedHSet, const::Ice::Current &current = Ice::Current());
    bool removeHSet(Ice::Long hSetId, const::Ice::Current &current = Ice::Current());
    HierarchicalSet getHSet(Ice::Long hSetId, const::Ice::Current &current = Ice::Current());
    HierarchicalSetList getHSets(const::Ice::Current &current = Ice::Current());
    Ice::Long highestHSetId() const;
    Ice::Long lowestHSetId() const;
    QList<Ice::Long> getHSetsIdsList();

protected slots:
    void _hSetAddedSlot(qlonglong hSetId);
    void _hSetChangedSlot(qlonglong hSetId);
    void _hSetRemovedSlot(qlonglong hSetId);

protected:
    HSetStorage *_storage;
    Ice::ObjectAdapterPtr _adapter;
    Ice::Identity _id;
};


/*! \class HSetStorage
 * \brief Hierarchical Set storage interface
 *
 */
class HSetStorage : public QObject
{
    Q_OBJECT

    friend class HSetManagerI;
public:
    explicit HSetStorage(QObject *parent = 0);
    virtual ~HSetStorage();

    virtual bool hasHSet(qlonglong hSetId);
    virtual HierarchicalSet getHSet(qlonglong hSetId);
    virtual HierarchicalSetList getHSets();
    virtual qlonglong addHSet(qlonglong rank, const QString &name, const QString &description);
    virtual bool updateHSet(const HierarchicalSet &updatedHSet);
    virtual bool removeHSet(qlonglong hSetId);
    qlonglong highestHSetId() const;
    qlonglong lowestHSetId() const;

signals:
    void hSetAdded(qlonglong hSetId);
    void hSetChanged(qlonglong hSetId);
    void hSetRemoved(qlonglong hSetId);

protected:
    virtual void _init();
    virtual bool _initStorage() = 0;
    virtual bool _insertHSet(const HierarchicalSet &hSet);
    virtual qlonglong _addHSet(qlonglong rank, const QString &name, const QString &description) = 0;
    virtual bool _updateHSet(const HierarchicalSet &updatedHSet) = 0;
    virtual bool _removeHSet(qlonglong hSetId) = 0;

private:
    bool _preInsertHSet(qlonglong rank, QList<qlonglong> &updatedIds);
    bool _postRemoveHSet(qlonglong rank, QList<qlonglong> &updatedIds);
    bool _updateHSetRank(qlonglong hSetId, qlonglong newRank);
    bool _decreaseHsetRankRange(qlonglong startRank, qlonglong endRank, QList<qlonglong> &updatedIds);
    bool _decreaseHSetRank(qlonglong hSetId, QList<qlonglong> &updatedIds, qlonglong rankLimit = -1);
    bool _decreaseRankRevert(QList<qlonglong> &updatedIds);
    bool _increaseHSetRankRange(qlonglong startRank, qlonglong endRank, QList<qlonglong> &updatedIds);
    bool _increaseHSetRank(qlonglong hSetId, QList<qlonglong> &updatedIds, qlonglong rankLimit = -1);
    bool _increaseRankRevert(QList<qlonglong> &updatedIds);
    QReadWriteLock _lock;
    QHash<qlonglong, HierarchicalSet> _hSetHash;
    QMap<qlonglong, qlonglong> _hierarchyToSet;
};


/*! \class SqlHSetStorage
 * \brief Hierarchical Set storage implementation based on Sql database
 *
 */
class SqlHSetStorage : public HSetStorage
{
    Q_OBJECT

public:
    explicit SqlHSetStorage(SqlDatabase *db, QObject *parent = 0);
    virtual ~SqlHSetStorage();

protected:
    virtual bool _initStorage();
    virtual qlonglong _addHSet(qlonglong rank, const QString &name, const QString &description);
    virtual bool _updateHSet(const HierarchicalSet &updatedHSet);
    virtual bool _removeHSet(qlonglong hSetId);

private:
    SqlDatabase *_db;
};

} // namespace Diamesh

#endif // HSETMANAGER_I_H
