/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include <QtCore/QStringList>
#include <QtCore/QDebug>

#include "diamesh/accessrightshelper.h"
#include "diamesh/constants.h"

namespace Diamesh
{

AccessRightsHelper::AccessRightsHelper(const AccessRightList &rightList, const StringStringMap &targetTypeMap, QObject *parent)
    : QObject(parent)
    , _rootId(0)
    , _nextId(0)
{
    _initSubjectTypes();
    QMap<std::string, std::string> targetQMap(targetTypeMap);
    QMapIterator<std::string, std::string> iterator(targetQMap);
    while(iterator.hasNext()) {
        iterator.next();
        _targetTypeHash.insert(QString::fromStdString(iterator.key()), QString::fromStdString(iterator.value()));
    }
    QList<AccessRightPtr> list(QList<AccessRightPtr>::fromStdList(rightList));
    foreach(const AccessRightPtr &right, list) {
        Ice::Long rightId = right->id;
        _rightsHash.insert(rightId, right);
        _nameToIdHash.insert(QString::fromStdString(right->name), rightId);
        if (rightId >_nextId)
            _nextId = rightId + 1;
    }
}

AccessRightsHelper::AccessRightsHelper(QObject *parent)
    : QObject(parent)
    , _rootId(0)
    , _nextId(0)
{
    _initSubjectTypes();
    _registerRootAccessRight();
}

AccessRightsHelper::~AccessRightsHelper()
{
    _rightsHash.clear();
}

bool AccessRightsHelper::hasRight(const QString &rightName) const
{
    return _nameToIdHash.contains(rightName);
}

AccessRightList AccessRightsHelper::getRightList() const
{
    return _rightsHash.values().toStdList();
}

AccessRightPtr AccessRightsHelper::getRight(const QString &rightName) const
{
    return _getRight(rightFromName(rightName));
}

LongList AccessRightsHelper::addAccessRights(const LongList &rightsIds, LongList additionnalRightsIds) const
{
    LongList result = rightsIds;
    result.merge(additionnalRightsIds);
    result.unique();
    result.sort();
    return result;
}

LongList AccessRightsHelper::addAccessRight(const LongList &rightsIds, long additionnalRightId) const
{
    LongList additionalList;
    additionalList.push_back(additionnalRightId);
    return addAccessRights(rightsIds, additionalList);
}

LongList AccessRightsHelper::removeAccessRights(const LongList &rightsIds, LongList toRemoveIds) const
{
    LongList result = rightsIds;
    QList<Ice::Long> toRemoveList = QList<Ice::Long>::fromStdList(toRemoveIds);
    foreach(Ice::Long toRemoveId, toRemoveList)
        result.remove(toRemoveId);
    result.sort();
    return result;
}

LongList AccessRightsHelper::removeAccessRight(const LongList &rightsIds, long toRemoveId) const
{
    LongList toRemoveList;
    toRemoveList.push_back(toRemoveId);
    return removeAccessRights(rightsIds, toRemoveList);
}

Ice::Long AccessRightsHelper::rightFromName(const QString &name) const
{
    return _nameToIdHash.value(name, 0);
}

LongList AccessRightsHelper::rightsFromNames(const QString &names) const
{
    return rightsFromNames(names.split(Constants::RIGHT_NAME_SEPARATOR, QString::SkipEmptyParts));
}

LongList AccessRightsHelper::rightsFromNames(const QStringList &names) const
{
    LongList result;
    foreach(const QString &name, names) {
        Ice::Long rightId = rightFromName(name);
        if (rightId != 0)
            result.push_back(rightId);
    }
    result.sort();
    return result;
}

QString AccessRightsHelper::nameFromRight(Ice::Long rightId) const
{
    QString result;
    AccessRightPtr right = _getRight(rightId);
    if (right != 0)
        result = QString::fromStdString(right->name);
    return result;
}

QStringList AccessRightsHelper::nameListFromRights(const LongList &rightsIds) const
{
    QStringList result;
    QList<Ice::Long> rightList = QList<Ice::Long>::fromStdList(rightsIds);
    foreach(Ice::Long rightId, rightList) {
        QString rightName = nameFromRight(rightId);
        if (!rightName.isEmpty())
            result.append(rightName);
        else
            qWarning() << tr("Invalid access right identifier: %1").arg(QString::number(rightId));
    }
    return result;
}

QString AccessRightsHelper::namesFromRights(const LongList &rightsIds) const
{
    return nameListFromRights(rightsIds).join(Constants::RIGHT_NAME_SEPARATOR);
}

LongList AccessRightsHelper::rightsIdsFromAccessRightList(const AccessRightList &rights) const
{
    LongList rightsIds;
    QList<AccessRightPtr> list(QList<AccessRightPtr>::fromStdList(rights));
    foreach(const AccessRightPtr &right, list)
        rightsIds.push_back(right->id);
    return rightsIds;
}

LongList AccessRightsHelper::validRightsIdsFromRightId(Ice::Long rightId) const
{
    LongList validRightsIds;
    Ice::Long rId = rightId;
    while(rId != 0) {
        if (_rightsHash.contains(rId)) {
            validRightsIds.push_back(rId);
            AccessRightPtr right = _rightsHash.value(rId);
            if (right->parent)
                rId = right->parent->id;
            else
                rId = 0;
        }
    }
    return validRightsIds;
}

QPair<AccessRightsHelper::SubjectType, qlonglong> AccessRightsHelper::subjectDetails(const QString &subject)
{
    AccessRightsHelper::SubjectType type = AccessRightsHelper::INVALID_SUBJECT;
    qlonglong id = -1;
    QStringList split = subject.split(".", QString::SkipEmptyParts);
    if (split.count() == 1)
        type = _subjectTypeHash.value(split.at(0), AccessRightsHelper::INVALID_SUBJECT);
    else if (split.count() == 2) {
        type = _composedSubjectTypeHash.value(split.at(0), AccessRightsHelper::INVALID_SUBJECT);
        if (type != AccessRightsHelper::INVALID_SUBJECT)
            id = split.at(1).toLongLong();
    }
    return qMakePair(type, id);
}

QPair<QString, qlonglong> AccessRightsHelper::targetDetails(const QString &target)
{
    QString type;
    qlonglong id = -1;
    QStringList split = target.split(".", QString::SkipEmptyParts);
    QStringList typeValues = _targetTypeHash.values();
    if (split.count() == 2 && typeValues.contains(split.at(0))) {
        type = split.at(0);
        id = split.at(1).toLongLong();
    }
    return qMakePair(type, id);
}

QPair<QString, QString> Diamesh::AccessRightsHelper::targetDetailsFromObject(const ACLTargetPtr &target)
{
    QPair<QString, QString> result;
    QString iceId = QString::fromStdString(target->ice_id());

    QString targetType;
    if (_targetTypeHash.contains(iceId))
        targetType = _targetTypeHash.value(iceId);
    else {
        QVector<std::string> ice_ids = QVector<std::string>::fromStdVector(target->ice_ids());
        for(int i = ice_ids.size() - 1; i >= 0; i--) {
            QString parentClassName = QString::fromStdString(ice_ids.at(i));
            if (_targetTypeHash.contains(parentClassName)) {
                targetType = _targetTypeHash.value(parentClassName);
                _targetTypeHash.insert(iceId, targetType);
                break;
            }
        }
    }
    if (targetType.isEmpty())
        qWarning() << tr("Could not build a valid target for object of type '%1'.").arg(iceId);
    else
        result = qMakePair(targetType, QString::number(target->id));
    return result;
}

QString Diamesh::AccessRightsHelper::targetFromObject(const ACLTargetPtr &target)
{
    QPair<QString, QString> details = targetDetailsFromObject(target);
    QString targetString = details.first;
    if (!targetString.isEmpty() && !details.second.isEmpty())
        targetString.append(QString(".%1").arg(details.second));
    return targetString;
}

bool AccessRightsHelper::rightsMatchesRightsNames(const LongList &rightsIds, const QStringList &rightsNames) const
{
    bool result = false;
    QList<Ice::Long> rightsList = QList<Ice::Long>::fromStdList(rightsIds);
    if (rightsList.count() == rightsNames.count()) {
        result = true;
        foreach(const QString &rightName, rightsNames) {
            if (!rightsList.contains(rightFromName(rightName))) {
                result = false;
                break;
            }
        }
    }
    return result;
}

AccessRightPtr AccessRightsHelper::_getRight(Ice::Long rightId) const
{
    return _rightsHash.value(rightId, 0);
}

void AccessRightsHelper::_initSubjectTypes()
{
    _composedSubjectTypeHash.insert("u", AccessRightsHelper::USER_SUBJECT);
    _composedSubjectTypeHash.insert("g", AccessRightsHelper::GROUP_SUBJECT);
    _composedSubjectTypeHash.insert("h", AccessRightsHelper::HSET_SUBJECT);
    _subjectTypeHash.insert("o", AccessRightsHelper::OTHER_SUBJECT);
}

Ice::Long AccessRightsHelper::_checkRight(const QString &rightName, Ice::Long parentId)
{
    Ice::Long result = rightFromName(rightName);
    if (result == 0) {
        AccessRightPtr right = new AccessRight(++_nextId, rightName.toStdString(), _getRight(parentId), AccessRightList());
        result = right->id;
        if (parentId != 0)
            _addChild(parentId, right);
        _rightsHash.insert(result, right);
        _nameToIdHash.insert(rightName, result);
    }
    return result;
}

void AccessRightsHelper::_addChild(Ice::Long rightId, const AccessRightPtr &child)
{
    AccessRightPtr right = _getRight(rightId);
    if (right)
        right->children.push_back(child);
}

void AccessRightsHelper::_registerRootAccessRight()
{
    _rootId = _checkRight(Constants::ACL_ALL_RIGHTS, 0);
}

void AccessRightsHelper::_registerAccessRight(const QString &rightName)
{
    if (hasRight(rightName))
        return;

    QStringList rightHierarchy = rightName.split(Constants::ACCESS_RIGHT_SEPARATOR, QString::SkipEmptyParts);
    QString fullRight;
    Ice::Long parentId = _rootId;
    foreach(const QString &rightLevel, rightHierarchy) {
        fullRight.append(rightLevel);
        parentId = _checkRight(fullRight, parentId);
        fullRight.append(Constants::ACCESS_RIGHT_SEPARATOR);
    }
}

void AccessRightsHelper::_registerTargetType(const QString &targetIceId, const QString &targetType)
{
    _targetTypeHash.insert(targetIceId, targetType);
}

} // namespace Diamesh
