/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include <QtCore/QStringList>
#include <QtCore/QDebug>
#include <QtSql/QSqlRecord>

#include <Ice/ObjectAdapter.h>

#include "diamesh/aclmanagerI.h"
#include "diamesh/constants.h"
#include "diamesh/sqldatabase.h"
#include "diamesh/accessrightshelper.h"
#include "diamesh/hsetmanagerI.h"
#include "diamesh/masterI.h"
#include "diamesh/sessionManagerI.h"

#include <stdexcept>


namespace Diamesh
{

ACLManagerI::ACLManagerI(ACLStorage *storage, HSetManagerI *hSetManager, Ice::ObjectAdapterPtr adapter)
    : _Diamesh_ACLManager()
    , _storage(storage)
    , _hSetManager(hSetManager)
    , _adapter(adapter)
    , _helper(new AccessRightsHelper(this))
{
    _id.name = Constants::ACL_MANAGER_PROXY_NAME;
    _adapter->add(this, _id);
}

ACLManagerI::~ACLManagerI()
{
}

ACLManagerPrx ACLManagerI::proxy() const
{
    return ACLManagerPrx::uncheckedCast(_adapter->createProxy(_id));
}

AccessRightsHelper *ACLManagerI::helper() const
{
    return _helper;
}

void ACLManagerI::registerAccessRight(const QString &rightName)
{
    _helper->_registerAccessRight(rightName);
}

void ACLManagerI::registerACLTargetType(const QString &targetIceId, const QString &targetType)
{
    if (_targetTypeHash.contains(targetIceId))
        qWarning() << tr("ACL Target type '%1' is already registered.").arg(targetIceId);
    else {
        _targetTypeHash.insert(targetIceId, targetType);
        _helper->_registerTargetType(targetIceId, targetType);
    }
}

AccessRightList ACLManagerI::getRegisteredRights(const Ice::Current &current)
{
    Q_UNUSED(current)
    return _helper->getRightList();
}

StringStringMap ACLManagerI::getTargetTypes(const Ice::Current &current)
{
    Q_UNUSED(current)
    StringStringMap result;
    QHashIterator<QString, QString>iterator(_targetTypeHash);
    while(iterator.hasNext()) {
        iterator.next();
        result[iterator.key().toStdString()] = iterator.value().toStdString();
    }
    return result;
}

AccessControlList ACLManagerI::getACL(const std::string &target, const std::string &subject, const Ice::Current &current)
{
    Q_UNUSED(current)
    return _storage->getACL(QString::fromStdString(target), QString::fromStdString(subject));
}

StringList ACLManagerI::getSubjectsForTarget(const ACLTargetPtr &target, const Ice::Current &current)
{
    Q_UNUSED(current)
    QPair<QString, QString> targetDetails = _helper->targetDetailsFromObject(target);
    QStringList list = _storage->getSubjectsForTarget(QString("%1.%2").arg(targetDetails.first).arg(targetDetails.second));
    StringList result;
    foreach(const QString &subject, list)
            result.push_back(subject.toStdString());
    return result;
}

ACLList ACLManagerI::getACLsForTarget(const ACLTargetPtr &target, const Ice::Current &current)
{
    Q_UNUSED(current)
    QPair<QString, QString> targetDetails = _helper->targetDetailsFromObject(target);
    return _storage->getACLsForTarget(QString("%1.%2").arg(targetDetails.first).arg(targetDetails.second)).toStdList();
}

bool ACLManagerI::setACL(const std::string &target, const std::string &subject, const LongList &rightsIds, Ice::Long hSetId, const Ice::Current &current)
{
    return _setACL(target, subject, rightsIds, hSetId, MasterI::instance()->sessionManager()->currentUser(current));
}

bool ACLManagerI::initACL(const std::string &target, const std::string &subject, const LongList &rightsIds, Ice::Long hSetId, const UserPtr &currentUser)
{
    return _setACL(target, subject, rightsIds, hSetId, currentUser, true);
}

bool ACLManagerI::removeACL(const std::string &target, const std::string &subject, const Ice::Current &current)
{
    bool result = false;
    QString qTarget = QString::fromStdString(target);
    QString qSubject = QString::fromStdString(subject);
    if (!_storage->hasACL(qTarget, qSubject))
        qCritical() << tr("Could not locate Access Control List for key '%1/%2'").arg(qTarget).arg(qSubject);
    else {
        UserPtr user = MasterI::instance()->sessionManager()->currentUser(current);
        if (!user || _canUserUpdateAcl(qTarget, qSubject, user))
            result = _storage->removeACL(qTarget, qSubject);
    }
    return result;
}

void ACLManagerI::targetRemoved(const std::string &target, const Ice::Current &current)
{
    Q_UNUSED(current)
    return _storage->targetRemoved(QString::fromStdString(target));
}

void ACLManagerI::subjectRemoved(const std::string &subject, const Ice::Current &current)
{
    Q_UNUSED(current)
    return _storage->subjectRemoved(QString::fromStdString(subject));
}

bool ACLManagerI::testRightForObject(const ACLTargetPtr &target, const UserPtr &subject, Ice::Long rightId, const Ice::Current &current)
{
    Q_UNUSED(current)
    QPair<QString, QString> targetDetails = _helper->targetDetailsFromObject(target);
    if (targetDetails.first.isEmpty()) {
        qCritical() << tr("Trying to test right for an unregistered object. Aborting...");
        return false;
    }

    QStringList targetList = _targetsFromDetails(targetDetails);

    HierarchicalSet targetHSet = _hSetManager->getHSet(target->hSetId);
    if (targetHSet.id == -1) {
        qCritical() << tr("Trying to test right for object with Id '%1' which has an invalid Hierachical set. Aborting...")
                       .arg(targetDetails.second);
        return false;
    }
    return _testRight(targetList, subject, rightId, targetHSet.rank);
}

bool ACLManagerI::testRight(const std::string &target, const UserPtr &subject, Ice::Long rightId, const Ice::Current &current)
{
    Q_UNUSED(current)
    return _testRight(QString::fromStdString(target), subject, rightId);
}

bool ACLManagerI::testRightForObject(const ACLTargetPtr &target, const UserPtr &subject, const QString &rightName)
{
    qlonglong rightId = _helper->rightFromName(rightName);
    if (rightId != 0)
        return testRightForObject(target, subject, rightId);
    return false;
}

bool ACLManagerI::testRight(const QString &target, const UserPtr &subject, const QString &rightName)
{
    qlonglong rightId = _helper->rightFromName(rightName);
    if (rightId != 0)
        return _testRight(target, subject, rightId);
    return false;
}

AccessRightsHelper *ACLManagerI::accessRightsHelper() const
{
    return _helper;
}

void ACLManagerI::_init()
{
    _storage->_init(_helper);
    QObject::connect(_storage, SIGNAL(aclAdded(QString, QString)), this, SLOT(_aclAddedSlot(QString, QString)));
    QObject::connect(_storage, SIGNAL(aclChanged(QString, QString)), this, SLOT(_aclChangedSlot(QString, QString)));
    QObject::connect(_storage, SIGNAL(aclRemoved(QString, QString)), this, SLOT(_aclRemovedSlot(QString, QString)));
}

bool ACLManagerI::_setACL(const std::string &target, const std::string &subject, const LongList &rightsIds, Ice::Long hSetId, const Diamesh::UserPtr &currentUser, bool silent)
{
    QString qTarget = QString::fromStdString(target);
    QString qSubject = QString::fromStdString(subject);
    bool shouldUpdate = _storage->hasACL(qTarget, qSubject);
    bool shouldDelete = (rightsIds.empty());
    if (!shouldUpdate && shouldDelete) {
        qWarning() << tr("Could not insert an empty Access Control List for key'%1/%2'.").arg(qTarget, qSubject);
        return false;
    }

    if (currentUser) {
        if (hSetId == -1)
            hSetId = currentUser->hSetId;

        HierarchicalSet userHSet = _hSetManager->getHSet(currentUser->hSetId);
        if (userHSet.id == -1) {
            qCritical() << tr("Trying to set an Access Control List by a user with Id '%1' who has an invalid Hierachical set. Aborting...")
                           .arg(QString::number(currentUser->id));
            return false;
        }

        HierarchicalSet requestedACLHSet = _hSetManager->getHSet(hSetId);
        if (requestedACLHSet.id == -1) {
            qCritical() << tr("Trying to set an Access Control List (Key '%1/%2') with an invalid Hierachical set. Aborting...")
                           .arg(QString::fromStdString(target))
                           .arg(QString::fromStdString(subject));
            return false;
        }

        if (requestedACLHSet.rank < userHSet.rank) {
            qWarning() << tr("Could not set an Access Control List with an unauthorized Hierarchical Set level.");
            return false;
        }
    }

    if (shouldUpdate && (!currentUser || _canUserUpdateAcl(qTarget, qSubject, currentUser))) {
        if (shouldDelete)
            return _storage->removeACL(qTarget, qSubject);
        else
            return _storage->updateACL(qTarget, qSubject, rightsIds, hSetId);
    }
    return _storage->addAcl(qTarget, qSubject, rightsIds, hSetId, silent);
}

void ACLManagerI::_aclAddedSlot(QString target, QString subject)
{
    emit aclAdded(target, subject);
}

void ACLManagerI::_aclChangedSlot(QString target, QString subject)
{
    emit aclChanged(target, subject);
}

void ACLManagerI::_aclRemovedSlot(QString target, QString subject)
{
    emit aclRemoved(target, subject);
}

QStringList ACLManagerI::_targetsFromDetails(const QPair<QString, QString> &targetDetails) const
{
    QStringList targetList;
    if (targetDetails.first.isEmpty() || targetDetails.second.isEmpty())
        return targetList;
    targetList.append(QString("%1.%2")
                      .arg(targetDetails.first)
                      .arg(targetDetails.second));
    targetList.append(QString("%1.%2")
                      .arg(targetDetails.first)
                      .arg(Constants::ACL_TARGET_GENERIC_ID));
    targetList.append(Constants::ACL_TARGET_ALL_TARGETS);
    return targetList;
}

bool ACLManagerI::_testRight(const QStringList &targetList, const UserPtr &subject, Ice::Long rightId, qlonglong targetRank) const
{
    QStringList subjectList;
    subjectList.append(QString("u.%1").arg(QString::number(subject->id)));
    foreach(qlonglong groupId, subject->groupList)
        subjectList.append(QString("g.%1").arg(QString::number(groupId)));
    subjectList.append("o");

    HierarchicalSet userHSet = _hSetManager->getHSet(subject->hSetId);
    if (userHSet.id == -1) {
        qCritical() << tr("Trying to test right for user with Id '%1' who has an invalid Hierachical set. Aborting...")
                       .arg(QString::number(subject->id));
        return false;
    }

    // Add user's HSet as potential subject if target HSet priority is inferior to user HSet priority
    // Highest rank is lowest priority
    if (targetRank > userHSet.rank)
        subjectList.append(QString("h.%1").arg(QString::number(userHSet.id)));

    QList<Ice::Long> validRightIds = QList<Ice::Long>::fromStdList(_helper->validRightsIdsFromRightId(rightId));
    foreach(const QString &aclSubject, subjectList) {
        foreach(const QString &aclTarget, targetList) {
            AccessControlList acl = _storage->getACL(aclTarget, aclSubject);
            if (!acl.target.empty()) {
                HierarchicalSet aclHSet = _hSetManager->getHSet(acl.hSetId);
                if (aclHSet.id == -1) {
                    qWarning() << tr("Trying to test an ACL with key '%1/%2' which has an invalid Hierachical set. Ignoring ACL...")
                                  .arg(QString::fromStdString(acl.target))
                                  .arg(QString::fromStdString(acl.subject));
                } else if (targetRank != -1 && targetRank < aclHSet.rank) {
                    qWarning() << tr("Found an invalid ACL which match key '%1/%2'. Ignoring ACL...")
                                  .arg(QString::fromStdString(acl.target))
                                  .arg(QString::fromStdString(acl.subject));
                } else {
                    QList<Ice::Long> rightList = QList<Ice::Long>::fromStdList(acl.rights);
                    foreach(Ice::Long validRightId, validRightIds) {
                        if (rightList.contains(validRightId))
                            return true;
                    }
                }
            }
        }
    }
    return false;
}

bool ACLManagerI::_testRight(const QString &target, const UserPtr &subject, Ice::Long rightId) const
{
    QStringList targetList;
    targetList.append(target);
    targetList.append(Constants::ACL_TARGET_ALL_TARGETS);
    return _testRight(targetList, subject, rightId, -1);
}

bool ACLManagerI::_canUserUpdateAcl(const QString &target, const QString &subject, const UserPtr &user) const
{
    AccessControlList targetAcl = _storage->getACL(target, subject);
    if (!targetAcl.target.empty()) {
        HierarchicalSet ACLHSet = _hSetManager->getHSet(targetAcl.hSetId);
        HierarchicalSet userHSet = _hSetManager->getHSet(user->hSetId);
        QPair<AccessRightsHelper::SubjectType, qlonglong> subjectDetails = _helper->subjectDetails(subject);
        if ((subjectDetails.first == AccessRightsHelper::USER_SUBJECT &&
             subjectDetails.second == user->id &&
             userHSet.rank != 0) ||
            (ACLHSet.rank < userHSet.rank)) {
            qWarning() << tr("User '%1' is trying to update an unauthorized Access Control List with key '%2/%3'.")
                          .arg(QString::fromStdString(user->name))
                          .arg(target)
                          .arg(subject);
            return false;
        }
    }
    return true;
}


ACLStorage::ACLStorage(QObject *parent)
    : QObject(parent)
{
}

ACLStorage::~ACLStorage()
{
}

bool ACLStorage::hasACL(const QString &target, const QString &subject)
{
    QReadLocker rLocker(&_lock);
    return _aclHash.contains(qMakePair(target, subject));
}

AccessControlList ACLStorage::getACL(const QString &target, const QString &subject)
{
    QReadLocker rLocker(&_lock);
    return _getACL(target, subject);
}

QStringList ACLStorage::getSubjectsForTarget(const QString &target)
{
    QReadLocker rLocker(&_lock);
    return _targetToSubjectsHash.value(target);
}

QList<AccessControlList> ACLStorage::getACLsForTarget(const QString &target)
{
    QList<AccessControlList> result;
    QReadLocker rLocker(&_lock);
    foreach(const QString &subject, _targetToSubjectsHash.value(target)) {
        AccessControlList acl = _getACL(target, subject);
        if (!acl.target.empty())
            result.append(acl);
    }
    return result;
}

bool ACLStorage::addAcl(const QString &target, const QString &subject, const LongList &rightsIds, qlonglong hSetId, bool silent)
{
    bool result = false;
    QWriteLocker wLocker(&_lock);
    AccessControlList acl;
    acl.target = target.toStdString();
    acl.subject = subject.toStdString();
    acl.rights = rightsIds;
    acl.hSetId = hSetId;
    if (_storageAddAcl(acl)) {
        result = _insertACL(acl, silent);
        if (!result)
            _storageRemoveAcl(target, subject);
    }
    return result;
}

bool ACLStorage::updateACL(const QString &target, const QString &subject, const LongList &rightsIds, qlonglong hSetId)
{
    QWriteLocker wLocker(&_lock);
    AccessControlList updatedAcl;
    updatedAcl.target = target.toStdString();
    updatedAcl.subject = subject.toStdString();
    updatedAcl.rights = rightsIds;
    updatedAcl.hSetId = hSetId;
    bool result = _storageUpdateAcl(updatedAcl);
    if (result) {
        QPair<QString, QString> key = qMakePair(target, subject);
        _aclHash[key] = updatedAcl;
        emit aclChanged(target, subject);
    }
    return result;
}

bool ACLStorage::removeACL(const QString &target, const QString &subject)
{
    bool result = false;
    QWriteLocker wLocker(&_lock);
    if (_storageRemoveAcl(target, subject)) {
        QPair<QString, QString> key = qMakePair(target, subject);
        _targetToSubjectsHash[target].removeAll(subject);
        if (_targetToSubjectsHash[target].count() == 0)
            _targetToSubjectsHash.remove(target);
        _aclHash.remove(key);
        emit aclRemoved(target, subject);
        result = true;
    }
    return result;
}

void ACLStorage::targetRemoved(const QString &target)
{
    Q_UNUSED(target)
    qDebug() << "TODO: ACLStorage::targetRemoved";
}

void ACLStorage::subjectRemoved(const QString &subject)
{
    Q_UNUSED(subject)
    qDebug() << "TODO: ACLStorage::subjectRemoved";
}

void ACLStorage::_init(AccessRightsHelper *helper)
{
    QWriteLocker locker(&_lock);
    _helper = helper;
    _initStorage();
    qDebug() << _targetToSubjectsHash;
}

AccessControlList ACLStorage::_getACL(const QString &target, const QString &subject) const
{
    return _aclHash.value(qMakePair(target, subject));
}

bool ACLStorage::_insertACL(const AccessControlList &acl, bool silent)
{
    bool result = false;
    QPair<QString, QString> key = qMakePair(QString::fromStdString(acl.target), QString::fromStdString(acl.subject));
    if (!_aclHash.contains(key)) {
        _aclHash.insert(key, acl);
        _targetToSubjectsHash[key.first].append(key.second);
        if (!silent)
            emit aclAdded(key.first, key.second);
        result = true;
    } else
        qWarning() << tr("Duplicate Access Control List found for key '%1/%2'.").arg(key.first, key.second);
    return result;
}



SqlACLStorage::SqlACLStorage(SqlDatabase *db, QObject *parent)
    : ACLStorage(parent)
    , _db(db)
{
}

SqlACLStorage::~SqlACLStorage()
{
}

void SqlACLStorage::_initStorage()
{
    QList<QSqlRecord> results;
    if (_db->executeQuery("SELECT * FROM acls", QList<QVariant>(), &results) == -1) {
        throw std::runtime_error("SqlACLStorage: Unable to read Access Control List from database.");
    }

    foreach(const QSqlRecord &r, results) {
        AccessControlList acl;
        acl.target = r.value(Constants::ACL_TARGET_FIELD_NAME).toString().toStdString();
        acl.subject = r.value(Constants::ACL_SUBJECT_FIELD_NAME).toString().toStdString();
        acl.rights = _helper->rightsFromNames(r.value(Constants::ACL_RIGHTS_FIELD_NAME).toString());
        acl.hSetId = r.value(Constants::ACL_HSETID_FIELD_NAME).toLongLong();
        _insertACL(acl);
    }
}

bool SqlACLStorage::_storageAddAcl(const AccessControlList &newAcl)
{
    QString target = QString::fromStdString(newAcl.target);
    QString subject = QString::fromStdString(newAcl.subject);
    QString query = QString("INSERT INTO acls (%1, %2, %3, %4) VALUES (?, ?, ?, ?)")
            .arg(Constants::ACL_TARGET_FIELD_NAME)
            .arg(Constants::ACL_SUBJECT_FIELD_NAME)
            .arg(Constants::ACL_RIGHTS_FIELD_NAME)
            .arg(Constants::ACL_HSETID_FIELD_NAME);
    QList<QVariant> values;
    values.append(target);
    values.append(subject);
    values.append(_helper->namesFromRights(newAcl.rights));
    values.append(QVariant::fromValue(newAcl.hSetId));

    if (_db->executeQuery(query, values) == -1) {
        qCritical() << tr("An error occured while trying to add Access Control List for key'%1/%2' into database.").arg(target).arg(subject);
        return false;
    }
    return true;
}

bool SqlACLStorage::_storageUpdateAcl(const AccessControlList &updatedAcl)
{
    QString target = QString::fromStdString(updatedAcl.target);
    QString subject = QString::fromStdString(updatedAcl.subject);
    QString query = QString("UPDATE acls SET %1=?, %2=? WHERE %3=? AND %4=?")
            .arg(Constants::ACL_RIGHTS_FIELD_NAME)
            .arg(Constants::ACL_HSETID_FIELD_NAME)
            .arg(Constants::ACL_TARGET_FIELD_NAME)
            .arg(Constants::ACL_SUBJECT_FIELD_NAME);
    QList<QVariant> values;
    values.append(_helper->namesFromRights(updatedAcl.rights));
    qlonglong hSetId = updatedAcl.hSetId;
    values.append(hSetId);
    values.append(target);
    values.append(subject);

    if (_db->executeQuery(query, values) == -1) {
        qCritical() << tr("An error occured while trying to update Access Control List for key'%1/%2' into database.").arg(target).arg(subject);
        return false;
    }
    return true;
}

bool SqlACLStorage::_storageRemoveAcl(const QString &target, const QString &subject)
{
    QList<QVariant> values;
    values.append(target);
    values.append(subject);
    QString query = QString("DELETE FROM acls WHERE %1=? AND %2=?")
            .arg(Constants::ACL_TARGET_FIELD_NAME)
            .arg(Constants::ACL_SUBJECT_FIELD_NAME);
    if (_db->executeQuery(query, values) == -1) {
        qCritical() << tr("An error occured while trying to delete Access Control List for key '%1/%2' from database.").arg(target).arg(subject);
        return false;
    }
    return true;
}


} // namespace Diamesh
