/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include <QtCore/QDebug>
#include <QtCore/QSettings>
#include <QtCore/QStringList>
#include <QtCore/QCryptographicHash>
#include <QtCore/QDateTime>
#include <Ice/Ice.h>
#include <Glacier2/Glacier2.h>
#include <QtSql/QSqlField>
#include <QtSql/QSqlRecord>

#include "diamesh/constants.h"
#include "diamesh/masterI.h"
#include "diamesh/authmanagerI.h"
#include "diamesh/sessionManagerI.h"
#include "diamesh/sqldatabase.h"
#include "diamesh/usermanagerI.h"
#include "diamesh/aclmanagerI.h"

namespace Diamesh
{

AuthManagerI::AuthManagerI(const QString &configFileName, UserManagerI *userManager) :
    _userManager(userManager)
{
    QSettings settings(configFileName, QSettings::IniFormat);
    settings.beginGroup(Constants::SETTINGS_GROUP);
    QString source = settings.value("authDataSource", "QSettings").toString().toLower();
    if (source == "database") {
        QString dbType=settings.value("authDBType", "QSQLITE").toString();
        QString dbName=settings.value("authDBName", "/tmp/diamesh.db").toString();
        QString dbTable=settings.value("authDBTable", "users").toString();
        _storage = new SqlPermissionsStorage(dbType, dbName, dbTable);
    } else {
        _storage = new QSettingsPermissionsStorage(configFileName);
    }

    connect(_userManager, SIGNAL(userRenamed(QString,QString)), _storage, SLOT(_userRenamed(QString,QString)));
    connect(_userManager, SIGNAL(userRemoved(QString)), _storage, SLOT(_userRemoved(QString)));
}

AuthManagerI::~AuthManagerI()
{
    delete _storage;
}

bool AuthManagerI::checkPermissions(const std::string &userName, const std::string &password, std::string &reason, const Ice::Current &) const
{
    Ice::Long userId = 0;
    if ((userId = _userManager->getUserIdFromName(userName)) == 0) {
        reason = QObject::tr("User %1 doesn't exists").arg(userName.c_str()).toStdString();
        qDebug() << reason.c_str();
        return false;
    }
    UserPtr user = _userManager->getUser(userId);
    if (!user->isEnabled) {
        reason = QObject::tr("User %1 is disabled").arg(userName.c_str()).toStdString();
        qDebug() << reason.c_str();
        return false;
    }

    if (!_storage->checkPermissions(userName.c_str(), password.c_str())) {
        reason = QObject::tr("User %1 failed to authenticate").arg(userName.c_str()).toStdString();
        qDebug() << reason.c_str();
        return false;
    }

    qDebug() << QObject::tr("User %1 authenticated successfully.").arg(userName.c_str());
    return true;
}

bool AuthManagerI::setPassword(const std::string &userName, const std::string &password, const Ice::Current &current)
{
    Ice::Long userId = _userManager->getUserIdFromName(userName);

    if (userId == 0) {
        qWarning() << tr("Trying to set password for unknown user with name '%1'.").arg(QString::fromStdString(userName));
        return false;
    }

    UserPtr targetUser = _userManager->getUser(userId);
    if (!targetUser) {
        qWarning() << tr("Trying to set password for unknown user with Id '%1'.").arg(userId);
        return false;
    }

    UserPtr currentUser = MasterI::instance()->sessionManager()->currentUser(current);

    if (!MasterI::instance()->aclManager()->testRightForObject(targetUser, currentUser, Constants::RIGHT_DIAMESH_USER_PASSWORD)) {
        qWarning() << tr("setPassword: permission denied");
        return false;
    }

    if (_storage->setPassword(userName.c_str(), password.c_str())) {
        qDebug() << QObject::tr("Password set successfully for user %1").arg(userName.c_str());
        // When a user changes his own password he shouldn't be disconnected by the server
        if (userId != currentUser->id)
            emit userPasswordSet(userId);
        return true;
    } else {
        qCritical() << QObject::tr("Error while setting the password for user %1").arg(userName.c_str());
        return false;
    }
}

QSettingsPermissionsStorage::QSettingsPermissionsStorage(const QString &configFileName) :
    _settings(new QSettings(configFileName, QSettings::IniFormat))
{
    _settings->beginGroup(Constants::SETTINGS_GROUP);
    _settings->beginGroup("UsersAuth");
}

bool QSettingsPermissionsStorage::checkPermissions(const QString &userName, const QString &password)
{
    if (!_settings->contains(userName)) {
        qDebug() << QObject::tr("Unknown user \"%1\"").arg(userName);
        return false;
    }

    if (_settings->value(userName).toString() != password) {
        qDebug() << QObject::tr("Invalid password given for user \"%1\"").arg(userName);
        return false;
    }

    return true;
}

bool QSettingsPermissionsStorage::setPassword(const QString &userName, const QString &password, const QString &oldName)
{
    if (!oldName.isEmpty())
        _settings->remove(oldName);

    _settings->setValue(userName, password);
    return true;
}

void QSettingsPermissionsStorage::_userRenamed(QString oldName, QString newName)
{
    QString password = _settings->value(oldName).toString();
    // No password registered for this user
    if (password.isEmpty())
        return;

    setPassword(newName, password, oldName);
}

void QSettingsPermissionsStorage::_userRemoved(QString name)
{
    _settings->remove(name);
}

SqlPermissionsStorage::SqlPermissionsStorage(const QString &dbType, const QString &dbName, const QString &tableName) :
    _lock(QReadWriteLock::Recursive), _tableName(tableName), _db(new SqlDatabase(dbType, dbName))
{
    QList<QSqlRecord> results;
    _db->executeQuery(QString("SELECT * FROM %1;").arg(_tableName), QList<QVariant>(), &results);
    foreach (const QSqlRecord &record, results) {
        QString userName(record.field("userName").value().toString());
        QString password(record.field("password").value().toString());
        QString salt(record.field("salt").value().toString());
        if (userName.isEmpty() || password.isEmpty()) {
            qWarning() << QObject::tr("Empty username or password in authentication database");
            break;
        }
        if (_userPasswordHash.contains(userName)) {
            qWarning() << QObject::tr("Duplicate username \"%1\" in authentication database").arg(userName);
            break;
        }
        _userPasswordHash.insert(userName, Password(password, salt));
    }
}

bool SqlPermissionsStorage::checkPermissions(const QString &userName, const QString &password)
{
    QReadLocker locker(&_lock);
    if (!_userPasswordHash.contains(userName)) {
        qDebug() << QObject::tr("Unknown user \"%1\"").arg(userName);
        return false;
    }

    const Password &pass = _userPasswordHash.value(userName);
    if (pass.hash != _hashPassword(password, pass.salt)) {
        qDebug() << QObject::tr("Invalid password given for user \"%1\"").arg(userName);
        return false;
    }

    return true;
}

bool SqlPermissionsStorage::setPassword(const QString &userName, const QString &password, const QString &oldName)
{
    QWriteLocker locker(&_lock);
    QString query;
    QList<QVariant> values;
    if (!oldName.isEmpty()) {
        // The user was renamed
        query = QString("UPDATE %1 SET userName=? WHERE userName=?").arg(_tableName);
        values.append(userName);
        values.append(oldName);
        if (_db->executeQuery(query, values) == -1)
            return false;
        _userPasswordHash.insert(userName, _userPasswordHash.take(oldName));
        return true;
    } else {
        qsrand(QDateTime::currentDateTime().toMSecsSinceEpoch());
        QByteArray saltByteArray;
        // Generate a 40 hexadecimal characters salt
        for (int i = 0 ; i < 20 ; ++i)
            saltByteArray.append((char)qrand());

        QString salt(saltByteArray.toHex());
        QString hash = _hashPassword(password, salt);

        if (_userPasswordHash.contains(userName))
            query = QString("UPDATE %1 SET password=?, salt=? WHERE userName=?").arg(_tableName);
        else
            query = QString("INSERT INTO %1 (password, salt, userName) VALUES (?, ?, ?)").arg(_tableName);

        values.append(hash);
        values.append(salt);
        values.append(userName);

        if (_db->executeQuery(query, values) == -1)
            return false;

        _userPasswordHash.insert(userName, Password(hash, salt));
        return true;
    }
}

void SqlPermissionsStorage::_userRenamed(QString oldName, QString newName)
{
    QWriteLocker locker(&_lock);
    QString hash = _userPasswordHash.value(oldName).hash;
    // No password registered for this user
    if (hash.isEmpty())
        return;

    setPassword(newName, hash, oldName);
}

void SqlPermissionsStorage::_userRemoved(QString name)
{
    QWriteLocker locker(&_lock);
    // No password registered for this user
    if (_userPasswordHash.remove(name) == 0)
        return;

    QString query = QString("DELETE FROM %1 WHERE userName=?").arg(_tableName);
    QVariantList values = QVariantList() << name;
    _db->executeQuery(query, values);
}

QString SqlPermissionsStorage::_hashPassword(const QString &password, const QString &salt)
{
    QString passwordToHash = salt + password;
    return QCryptographicHash::hash(passwordToHash.toUtf8(), QCryptographicHash::Sha1).toHex();
}

}
