/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include <diamesh/coreapplication.h>
#include <diamesh/constants.h>
#include <diamesh/logger.h>
#include <diamesh/diamesh.h>
#include <diamesh/utils.h>

#include <getopt.h>
#include <unistd.h>
#include <errno.h>
#include <pwd.h>
#include <sys/socket.h>
#include <stdlib.h>

#include <Ice/Properties.h>
#include <Ice/Ice.h>
#include <QtCore/QDateTime>
#include <QtCore/QSettings>
#include <QtCore/QStringList>
#include <QtCore/QTextCodec>
#include <QtCore/QFile>
#include <QtCore/QSocketNotifier>
#include <QtCore/QVector>
#include <QtCore/QDebug>

struct option app_options[] = {
    { "pidfile", required_argument, NULL, 'p' },
    { "daemonize", no_argument, NULL, 'd' },
    { "help", no_argument, NULL, 'h' },
    { NULL, 0, NULL, 0 }
};

namespace Diamesh {

int CoreApplication::_sigHupFd[2] = {0,0};
int CoreApplication::_sigTermFd[2] = {0,0};
int CoreApplication::_sigFatalFd[2] = {0,0};
volatile sig_atomic_t CoreApplication::_sigTermHandlerOnce = false;
volatile sig_atomic_t CoreApplication::_sigFatalHandlerOnce = false;

typedef void (*SignalHandler)(int);

static SignalHandler installSignalHandler(int signum, SignalHandler handler)
{
    struct sigaction act, old;
    sigemptyset(&act.sa_mask);
    act.sa_handler = reinterpret_cast<void (*)(int)>(handler);
    act.sa_flags = 0;
    int r;
    do {
        r = sigaction(signum, &act, &old);
    } while (( r == -1 ) && ( errno == EINTR ));

    if (r != -1)
        return old.sa_handler;
    else
        return 0;
}

#define SIG_CASE(SUFFIX) case SIG ## SUFFIX : return ( #SUFFIX );

static QString signalToString(int signum)
{
    switch(signum)
    {
    SIG_CASE(HUP);
    SIG_CASE(INT);
    SIG_CASE(QUIT);
    SIG_CASE(ILL);
    SIG_CASE(TRAP);
    SIG_CASE(ABRT);
    SIG_CASE(BUS);
    SIG_CASE(FPE);
    SIG_CASE(KILL);
    SIG_CASE(USR1);
    SIG_CASE(SEGV);
    SIG_CASE(USR2);
    SIG_CASE(PIPE);
    SIG_CASE(ALRM);
    SIG_CASE(TERM);
    SIG_CASE(CHLD);
    SIG_CASE(CONT);
    SIG_CASE(STOP);
    SIG_CASE(TSTP);
    SIG_CASE(TTIN);
    SIG_CASE(TTOU);
    SIG_CASE(URG);
    SIG_CASE(XCPU);
    SIG_CASE(XFSZ);
    SIG_CASE(VTALRM);
    SIG_CASE(PROF);
    SIG_CASE(WINCH);
    SIG_CASE(POLL);
    SIG_CASE(SYS);
    }
    return(QString("SIG %1").arg(signum));
}

CoreApplication::CoreApplication(int &argc, char **argv, Instance *diameshInstance)
    : QCoreApplication(argc, argv)
    , _sigHupSocketNotifier(NULL)
    , _sigTermSocketNotifier(NULL)
    , _sigFatalSocketNotifier(NULL)
    , _pidFile(NULL)
    , _foreground(true)
    , _logger(NULL)
{
    QHash<QString, QString> appConfigHash = diameshInstance->appConfigHash();
    setApplicationName(appConfigHash.value(Diamesh::Constants::APP_CONFIG_NAME));
    setOrganizationName(appConfigHash.value(Diamesh::Constants::APP_CONFIG_ORGANIZATION));
    qInstallMsgHandler(&CoreApplication::_qtMessageHandler);

    QHash<QString, QString> opts = diameshInstance->parsedOpts();

    if (opts.contains("daemonize"))
        _foreground = false;

    if (opts.contains("pidfile"))
        _setPidFile(opts.value("pidfile"));

    if (optind < argc) {
        _execCommand(argv[optind]);
        deleteLater();
        ::quick_exit(0);
    } else {
        QSettings config(diameshInstance->configFileName(), QSettings::IniFormat);
        config.setIniCodec(QTextCodec::codecForName("UTF-8"));
        config.beginGroup(Constants::SETTINGS_GROUP);
        _logger = Logger::create(config.value(Constants::SETTINGS_LOG_METHOD).toString(),
                                 config.value(Constants::SETTINGS_LOG_FILENAME).toString(),
                                 config.value(Constants::SETTINGS_LOG_LEVEL).toInt());
    }
}

CoreApplication::~CoreApplication()
{
    _cleanup();
}

CoreApplication *CoreApplication::create(int &argc, char **argv, const option *opts, const QHash<QString, QString> &appConfigHash)
{
    QVector<option> options;
    for (const option *o = app_options ; o->name ; ++o)
        options.append(*o);
    if (opts) {
        for (const option *o = opts ; o->name ; ++o)
            options.append(*o);
    }

    // Parse and consume Ice arguments before creating the CoreApplication
    Instance *diameshInstance = Diamesh::initialize(argc, argv, options, appConfigHash);

    QFile configFile(diameshInstance->configFileName());
    if (!configFile.exists() || !configFile.open(QIODevice::ReadOnly))
        qFatal("The configuration file is not readable: %s", qPrintable(diameshInstance->configFileName()));
    configFile.close();

    CoreApplication *app = new CoreApplication(argc, argv, diameshInstance);

    app->_init();
    return app;
}

CoreApplication *CoreApplication::instance()
{
    return qobject_cast<CoreApplication *>(QCoreApplication::instance());
}

void CoreApplication::logToFile(const QString &logFile)
{
    if (_logger)
        _logger->logToFile(logFile);
}

bool CoreApplication::notify(QObject *receiver, QEvent *event)
{
    bool ret = false;
    try {
        ret = QCoreApplication::notify(receiver, event);
    } catch(...) {
        Diamesh::instance()->handleException();
    }

    return ret;
}

void CoreApplication::restoreSigChldHandler()
{
    installSignalHandler(SIGCHLD, _qt_handler);
}

void CoreApplication::_handleSighup()
{
    _sigHupSocketNotifier->setEnabled(false);
    char tmp;
    saferead(_sigHupFd[1], &tmp, sizeof(tmp));
    emit reload();
    _sigHupSocketNotifier->setEnabled(true);
}

void CoreApplication::_handleTermSignals()
{
    _sigHupSocketNotifier->setEnabled(false);
    int sig;
    saferead(_sigTermFd[1], &sig, sizeof(sig));
    qWarning("We received signal [%s], stopping.\n",  qPrintable(signalToString(sig)));
    _cleanup();

    ::signal(sig, SIG_DFL);
    ::raise(sig);
}

void CoreApplication::_handleFatalSignals()
{
    _sigHupSocketNotifier->setEnabled(false);
    int sig;
    saferead(_sigFatalFd[1], &sig, sizeof(sig));
    qWarning("We received fatal signal [%s], stopping.\n",  qPrintable(signalToString(sig)));

    ::signal(sig, SIG_DFL);
    ::raise(sig);
}

void CoreApplication::_log(QtMsgType type, const QString &msg) const
{
    if (_logger)
        _logger->log(type, msg);
}

bool CoreApplication::_setPidFile(const QString &path)
{
    if (_pidFile)
        return false;
    _pidFilePath = path;
    if (!_pidFilePath.startsWith('/'))
        _pidFilePath.prepend("/var/run/");
    return true;
}

void CoreApplication::_execCommand(const QString &cmd)
{
    if (_pidFilePath.isEmpty())
        qFatal("Can't locate running instance without a pidfile");

    QFile pidFile(_pidFilePath);
    bool status = (cmd == "status");
    if (!pidFile.exists()) {
        if (status)
            _displayStatus(0);
        else
            _displayAbortCmd();
        return;
    }

    if (!pidFile.open(QIODevice::ReadOnly))
        qFatal("Cannot open pidfile %s", qPrintable(_pidFilePath));
    QByteArray pidStr = pidFile.readAll();
    bool ok = false;
    int pid = pidStr.toInt(&ok);
    if (!ok)
        qFatal("Pidfile %s does not contain a pid.", qPrintable(_pidFilePath));

    bool running = (::kill(pid, 0) == 0);

    if (status) {
        _displayStatus(running ? pid : 0);
        return;
    }

    if (!running) {
        qWarning("No such process %d, removing pidfile.", pid);
        pidFile.remove();
        return;
    }

    if (cmd == "reload") {
        if (::kill(pid, SIGHUP) < 0)
            qFatal("Reloading failed. pid: %d error: %s", pid, strerror(errno));
        _displayMessage(QString("Reloading %1 (pid %2) successful.\n").arg(applicationName()).arg(QString::number(pid)));
    } else if (cmd == "stop") {
        if (::kill(pid, SIGINT) < 0)
            qFatal("Stopping failed. pid: %d error: %s", pid, strerror(errno));
        _displayMessage(QString("Stopping %1 (pid %2) successful.\n").arg(applicationName()).arg(QString::number(pid)));
    } else {
        qFatal("Unknown command: %s", qPrintable(cmd));
    }
}

void CoreApplication::_init()
{
    this->_daemonize();
    qDebug("%s starting.", qPrintable(applicationName()));
    QTextCodec::setCodecForCStrings(QTextCodec::codecForName("UTF-8"));
    Diamesh::instance()->loadTranslations();

    if (!_pidFilePath.isEmpty()) {
        _pidFile = new QFile(_pidFilePath);
        if (!_pidFile->open(QIODevice::ReadWrite))
            qFatal("Cannot open pidfile %s",qPrintable(_pidFilePath));
        if (::lockf(_pidFile->handle(), F_TLOCK, 0) < 0)
            qFatal("Can't get a lock on %s. another instance is already running.",qPrintable(_pidFilePath));
        QByteArray pid = QByteArray::number(getpid());
        if (_pidFile->write(pid) != pid.size())
            qFatal("Can't write pid in pidfile %s.",qPrintable(_pidFilePath));
        _pidFile->flush();
    }

    // SIGHUP

    if (::socketpair(AF_UNIX, SOCK_STREAM|SOCK_NONBLOCK, 0, _sigHupFd))
        qFatal("Couldn't create HUP socketpair");

    _sigHupSocketNotifier = new QSocketNotifier(_sigHupFd[1], QSocketNotifier::Read, this);
    connect(_sigHupSocketNotifier, SIGNAL(activated(int)), this, SLOT(_handleSighup()));

    installSignalHandler(SIGHUP, CoreApplication::_hupSignalHandler);

    // Termination signals

    if (::socketpair(AF_UNIX, SOCK_STREAM|SOCK_NONBLOCK, 0, _sigTermFd))
        qFatal("Couldn't create termination signals socketpair");

    _sigTermSocketNotifier = new QSocketNotifier(_sigTermFd[1], QSocketNotifier::Read, this);
    connect(_sigTermSocketNotifier, SIGNAL(activated(int)), this, SLOT(_handleTermSignals()));

    installSignalHandler(SIGINT, CoreApplication::_termSignalHandler);
    installSignalHandler(SIGQUIT, CoreApplication::_termSignalHandler);
    installSignalHandler(SIGTERM, CoreApplication::_termSignalHandler);

    // Fatal signals

    if (::socketpair(AF_UNIX, SOCK_STREAM, 0, _sigFatalFd))
        qFatal("Couldn't create fatal signals socketpair");

    _sigFatalSocketNotifier = new QSocketNotifier(_sigFatalFd[1], QSocketNotifier::Read, this);
    connect(_sigFatalSocketNotifier, SIGNAL(activated(int)), this, SLOT(_handleFatalSignals()));

    installSignalHandler(SIGFPE, CoreApplication::_fatalSignalHandler);
    installSignalHandler(SIGILL, CoreApplication::_fatalSignalHandler);
    installSignalHandler(SIGSEGV, CoreApplication::_fatalSignalHandler);
    installSignalHandler(SIGBUS, CoreApplication::_fatalSignalHandler);
    installSignalHandler(SIGABRT, CoreApplication::_fatalSignalHandler);
    installSignalHandler(SIGIOT, CoreApplication::_fatalSignalHandler);
    installSignalHandler(SIGTRAP, CoreApplication::_fatalSignalHandler);
    installSignalHandler(SIGSYS, CoreApplication::_fatalSignalHandler);
    installSignalHandler(SIGFPE, CoreApplication::_fatalSignalHandler);

    _qt_handler = installSignalHandler(SIGCHLD, SIG_IGN);
    installSignalHandler(SIGCHLD, _qt_handler);
}

void CoreApplication::_daemonize()
{
    // Fork is called twice to prevents zombies
    if (!_foreground) {
        switch (::fork()) {
        case 0:
            break;
        case -1:
            // Error
            qFatal("Error demonizing (fork)! %d - %s\n", errno, ::strerror(errno));
            break;
        default:
            ::_exit(0);
        }

        if (::setsid() < 0)
            qFatal("Error demonizing (setsid)! %d - %s\n", errno, ::strerror(errno));

        switch (::fork()) {
        case 0:
            break;
        case -1:
            // Error
            qFatal("Error demonizing (second fork)! %d - %s\n", errno, ::strerror(errno));
            break;
        default:
            ::_exit(0);
        }
        ::freopen( "/dev/null", "r", stdin);
        ::freopen( "/dev/null", "w", stdout);
        ::freopen( "/dev/null", "w", stderr);
    }
}

void CoreApplication::_cleanup()
{
    emit shutdown();
    delete _sigHupSocketNotifier;
    delete _sigTermSocketNotifier;
    delete _sigFatalSocketNotifier;
    close(_sigHupFd[0]);
    close(_sigHupFd[1]);
    close(_sigTermFd[0]);
    close(_sigTermFd[1]);
    close(_sigFatalFd[0]);
    close(_sigFatalFd[1]);

    if (!_pidFilePath.isEmpty() && _pidFile && _pidFile->exists()) {
        if (!_pidFile->remove())
            qCritical("Can't remove pidfile %s", qPrintable(_pidFilePath));
    }
    qDebug("%s stopped.", qPrintable(applicationName()));
    delete _pidFile;
    delete _logger;
    _logger = NULL;

    qInstallMsgHandler(NULL);
}

void CoreApplication::_displayMessage(const QString &message)
{
    printf("%s", qPrintable(message));
}

void CoreApplication::_displayStatus(int pid)
{
    QString message;
    if (pid)
        message = QString("%1 is running. (pid %2)\n").arg(applicationName()).arg(QString::number(pid));
    else
        message = QString("%1 is NOT running.\n").arg(applicationName());
    _displayMessage(message);
}

void CoreApplication::_displayAbortCmd()
{
    _displayMessage(QString("%1 is not running.\n").arg(applicationName()));
}

void CoreApplication::_hupSignalHandler(int /* sig */)
{
    char a = 1;
    safewrite(_sigHupFd[0], &a, sizeof(a));
}

void CoreApplication::_termSignalHandler(int signum)
{
    if (_sigTermHandlerOnce)
        ::raise(signum);

    _sigTermHandlerOnce = 1;

    safewrite(_sigTermFd[0], &signum, sizeof(signum));
}

void CoreApplication::_fatalSignalHandler(int signum)
{
    if (_sigFatalHandlerOnce)
        ::raise(signum);

    _sigFatalHandlerOnce = 1;

    safewrite(_sigFatalFd[0], &signum, sizeof(signum));
}

void CoreApplication::_qtMessageHandler(QtMsgType type, const char * msg)
{
    CoreApplication *app = CoreApplication::instance();
    if (app)
        app->_log(type, msg);
  else
    ::fprintf(stderr, "*** No logger available. Unable to log the following message:\n%s\n", msg);

  if (type == QtFatalMsg)
      ::abort();
}

} // namespace Diamesh
