/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include <diamesh/diamesh.h>
#include <diamesh/types.h>
#include <diamesh/accessrights.h>
#include <diamesh/constants.h>
#include <QtIce/operators.h>

#include <QtCore/QCoreApplication>
#include <QtCore/QLibraryInfo>
#include <QtCore/QLocale>
#include <QtCore/QMetaType>
#include <QtCore/QSettings>
#include <QtCore/QStringList>
#include <QtCore/QTranslator>
#include <QtCore/QVector>
#include <QDomDocument>
#include <QFile>
#include <QDebug>
#include <Ice/Initialize.h>
#include <Ice/LocalException.h>
#include <Ice/Properties.h>
#include <getopt.h>

option nullOpt = { NULL, 0, NULL, 0 };

namespace Diamesh
{

QDataStream &operator<<(QDataStream &out, const Diamesh::AccessControlList &acl)
{
    out << QString::fromStdString(acl.target)
        << QString::fromStdString(acl.subject)
        << QList<Ice::Long>::fromStdList(acl.rights)
        << qulonglong(acl.hSetId);
    return out;
}

QDataStream &operator>>(QDataStream &in, Diamesh::AccessControlList &acl)
{
    QString target;
    QString subject;
    QList<Ice::Long> rights;
    qulonglong hSetId;
    in >> target >> subject >> rights >> hSetId;
    acl.target = target.toStdString();
    acl.subject = subject.toStdString();
    acl.rights = rights.toStdList();
    acl.hSetId = hSetId;
    return in;
}

class RegisterMetaTypes
{
public:
    RegisterMetaTypes()
    {
        qRegisterMetaType<Ice::Long>("Ice::Long");
        qRegisterMetaTypeStreamOperators< ::Diamesh::IntList >("::Diamesh::IntList");
        qRegisterMetaTypeStreamOperators< ::Diamesh::LongList >("::Diamesh::LongList");
        qRegisterMetaTypeStreamOperators< ::Diamesh::StringList >("::Diamesh::StringList");
        qRegisterMetaType<Diamesh::AccessControlList>("Diamesh::AccessControlList");
        qRegisterMetaTypeStreamOperators<Diamesh::AccessControlList>("Diamesh::AccessControlList");
        qRegisterMetaTypeStreamOperators<Diamesh::ACLList>("Diamesh::ACLList");
    }
} _register;

static Instance *diameshInstance = 0;

Instance::Instance(const Ice::PropertiesPtr &props, const QHash<QString, QString> &parsedOpts, const QHash<QString, QString> &appConfigHash) :
    _props(props), _parsedOpts(parsedOpts), _qtTranslator(NULL), _diameshTranslator(NULL)
{
    _appConfigHash.insert(Diamesh::Constants::APP_CONFIG_NAME, appConfigHash.value(Diamesh::Constants::APP_CONFIG_NAME, "diamesh"));
    _appConfigHash.insert(Diamesh::Constants::APP_CONFIG_ORGANIZATION, appConfigHash.value(Diamesh::Constants::APP_CONFIG_ORGANIZATION, ""));
    QString config = configFileName();
    if (!config.isEmpty()) {
        QSettings settings(config, QSettings::IniFormat);
        settings.beginGroup("Ice");

        foreach (const QString & key, settings.childKeys())
            _props->setProperty(key.toStdString(), settings.value(key).toString().toStdString());

        settings.endGroup();
    }
}

Instance::~Instance()
{
    QCoreApplication::removeTranslator(_qtTranslator);
    QCoreApplication::removeTranslator(_diameshTranslator);
    delete _qtTranslator;
    delete _diameshTranslator;
}

QString Instance::configFileName()
{
    return _parsedOpts.value("config", QString());
}

const QHash<QString, QString> &Instance::parsedOpts()
{
    return _parsedOpts;
}

Ice::PropertiesPtr Instance::props()
{
    return _props->clone();
}

void Instance::handleException()
{
    try {
        throw; // re-throw the current exception
    } catch (const Ice::ConnectionLostException &ex) {
        qDebug("catched ConnectionLost exception");
        emit connectionExceptionCatched();
    } catch (const Ice::Exception &ex) {
        qDebug("catched Ice exception: %s\n%s", ex.ice_name().c_str(), ex.ice_stackTrace().c_str());
    } catch (const std::exception &ex) {
        qDebug("exception catched:\n%s", ex.what());
    } catch (...) {
        qDebug("unknown exception catched");
    }
}

void Instance::loadTranslations()
{
    QString language = applicationLanguage();
    _qtTranslator = new QTranslator();
    _qtTranslator->load("qt_" + language,
                        QLibraryInfo::location(QLibraryInfo::TranslationsPath));
    QCoreApplication::installTranslator(_qtTranslator);

    if (language.startsWith("en_")) //Don't load translator if language is the same as the dev language
        return;
    _diameshTranslator =  new QTranslator();
    QString translationName = QString("diamesh_%1").arg(language);
    qDebug("Loading translations from %s\n", qPrintable(translationName));
    if (!_diameshTranslator->load(translationName, QString(":/i18n"))) {
        qCritical() << tr("Unable to load the translation file");
        return;
    }
    if(_diameshTranslator->isEmpty()) {
        qCritical() << tr("Translator is Empty");
        return;
    }

    QCoreApplication::installTranslator(_diameshTranslator);
}

void Instance::loadExternalTranslations(QString fileName)
{
    QString systemLang = QLocale::system().name().section('_',0,0);
    QString defaultLang = "en"; // should be overwritten by DefaultLang in the XML file

    QDomDocument doc;
    QFile file(fileName);
    if (!file.open(QIODevice::ReadOnly)) {
        qWarning() << "The external translations file" << fileName << "can not be opened";
        return;
    }
    if (!doc.setContent(&file)) {
        file.close();
        return;
    }
    file.close();
    QDomElement root = doc.documentElement();
    // get default lang
    QDomElement defaultLangElement = root.firstChildElement("DefaultLang");
    if (defaultLangElement.isNull())
        qWarning() << "No DefaultLang tag has been defined in" << fileName;
    else
        defaultLang = defaultLangElement.text();
    // get translation entries
    QDomNodeList translationEntryList = root.elementsByTagName("TranslationEntry");
    for (uint i = 0; i < translationEntryList.length(); i++) {
        QDomElement translationEntry = translationEntryList.item(i).toElement();
        if (!translationEntry.hasAttribute("id"))
            qWarning() << "There is no id attribute for the TranslationEntry element on line" << translationEntry.lineNumber() << "in" << fileName;

        QString translationId = translationEntry.attribute("id");

        QDomNodeList translationList = translationEntry.elementsByTagName("Translation");
        QString translation;
        for (uint j = 0; j < translationList.length(); j++) {
            QDomElement translationElement = translationList.item(j).toElement();
            if (translationElement.hasAttribute("lang")) {
                QString lang = translationElement.attribute("lang");
                if (lang == systemLang || lang == defaultLang) {
                    translation = translationElement.text();
                    if (lang == systemLang)
                        break;
                }
            }
        }

        if (!translation.isEmpty())
            _externalTranslations.insert(translationId, translation);
        else
            qWarning() << QString("There is no translation for '%1' using the locale '%2' on line %3 in '%4'").arg(translationId).arg(systemLang).arg(translationEntry.lineNumber()).arg(fileName);
    }
}

// static
QString Instance::extTr(QString translationId)
{
    QString translation;
    if (diameshInstance)
        translation = diameshInstance->_externalTranslations.value(translationId, "");
    return translation;
}

const QHash<QString, QString> &Instance::appConfigHash()
{
    return _appConfigHash;
}

QString Instance::applicationLanguage()
{
    QString language;

    QScopedPointer<QSettings> settings;
    if (!configFileName().isEmpty())
        settings.reset(new QSettings(configFileName(), QSettings::IniFormat));
    else
        settings.reset(new QSettings());
    settings->beginGroup(Diamesh::Constants::SETTINGS_GROUP);
    language = settings->value(Diamesh::Constants::SETTINGS_TRANSLATION_LANGUAGE).toString();
    if (language.isEmpty())
        language = QLocale::system().name();
    return language;
}

Instance *initialize(int &argc, char **argv, const QVector<option> &options_, const QHash<QString, QString> &appConfigHash)
{
    if (diameshInstance) {
        qWarning("Instance::initialize() should only be called once");
        return 0;
    }

    Ice::PropertiesPtr props = Ice::createProperties(argc, argv);

    QString optString;
    QHash<int, QString> optsHash;

    QVector<option> options;
    option opt = { "config", required_argument, NULL, 'c' };
    options.append(opt);
    options += options_;

    foreach (const option &o, options) {
        if (optsHash.contains(o.val))
            qFatal("duplicate short option : %c", o.val);
        if (optsHash.key(o.name) != 0)
            qFatal("duplicate long option : %s", o.name);
        optString.append(QChar(o.val));
        if (o.has_arg)
            optString.append(":");
        optsHash.insert(o.val, o.name);
    }
    // Add a "null" option at the end as required by getopt_long()
    options.append(nullOpt);

    QHash<QString, QString> parsedOpts;
    int option_index = 0;
    int c = 0;
    while ((c = getopt_long(argc, argv, optString.toAscii(), options.constData(), &option_index)) != -1) {

        if (c == '?' || c == ':') {
            // TODO: show usage and exit
            parsedOpts.clear();
            break;
        }

        if (optsHash.contains(c)) {
            QString value = optarg ? optarg : "";
            parsedOpts.insert(optsHash.value(c), value);
        }
        option_index = 0;
    }

    diameshInstance = new Instance(props, parsedOpts, appConfigHash);
    return diameshInstance;
}

Instance *instance()
{
    return diameshInstance;
}

}
