/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include "diamesh/filetransferclient.h"
#include "diamesh/filetransfer.h"
#include "diamesh/filetransferI.h"

#include <QtCore/QtConcurrentRun>
namespace Diamesh
{
/*
This class is a client working with the class FileTransfer
The only necessary call is start()
If you want to display the progression you have to make your slots connected to :
    void progressTransfer(Ice::Long amountTransfered) : Send the current size transferred
    void transferIsFinished() : Notify the end of the transfer, when this signal is emitted, the file has also been written on the disk
    void fileSize(Ice::Long fileSize) : Is emitted when the asynchronous call to FileTransfer for having the size of the original file is complete
*/
FileTransferClient::FileTransferClient(FileTransferPrx file, Ice::ObjectAdapterPtr adapter, QObject *parent):
    QObject(parent)
  , _fileQPrx(file,adapter)
{
    _fileSize = 0;
    _fileNameOriginal = QString();
    connect(this,SIGNAL(transferIsFinished()), this, SLOT(_transferIsFinished()));
//    connect(this,SIGNAL(percentTransfer(int)),this,SLOT(_progress(int))); /*Uncomment for tests*/
}

FileTransferClient::~FileTransferClient()
{
    _fileQPrx = 0;
}

QFuture<void> FileTransferClient::_getFile()
{
    return QtConcurrent::run(this, &FileTransferClient::_getFileThread, _fileDest, _offset, _numberOfBytesToTransfer);
}

QString FileTransferClient::getOriginalFileName()
{
    return _fileNameOriginal;
}

Ice::Long FileTransferClient::getSize()
{
    return _fileSize;
}

QFileInfo FileTransferClient::getFileWritten()
{
    return _fileWritten;
}

void FileTransferClient::start(QString fileDest, Ice::Long offset, Ice::Long numberOfBytesToTransfer)
{
    /*
      start(QString fileDest,Ice::Long offset,Ice::Long numberOfBytesToTransfer) is doing all the asynchronous calls and finally begin
      the transfer when everything is OK
    */
    numberOfBytesToTransfer = (numberOfBytesToTransfer>1048000) ? 1048000 : numberOfBytesToTransfer; /* Ice cannot transfer file bigger than 1MB */
    _fileDest = fileDest;
    _offset = offset;
    _numberOfBytesToTransfer = numberOfBytesToTransfer;
    _askSize();
}

void FileTransferClient::_getFileThread(QString fileDest, Ice::Long offset, Ice::Long numberOfBytesToTransfer)
{
    QFile fileToWrite(fileDest);

    Ice::Long offsetDepart = offset;
    if (fileToWrite.open(QIODevice::Truncate | QIODevice::WriteOnly)) {
        Ice::Long tempFileSize = fileToWrite.size();
        if (_fileSize > 0) {

            Ice::Long percentToFlush = _fileSize/10 ; /*"10" means that we write on the disk every 10% of the copy, for 20% you need to divide by 5*/
            Ice::Long i = 0;
            while (_arrayFile.size() + tempFileSize + offsetDepart<_fileSize) {
                i+=numberOfBytesToTransfer;
                emit percentTransfer(int(((_arrayFile.size() + tempFileSize + offsetDepart)* 100)/_fileSize));
                Diamesh::ByteArray data = _fileQPrx->readFile(offset,numberOfBytesToTransfer);
                if (data.size() != 0) {
                    for (Diamesh::ByteArray::const_iterator it = data.begin() ; it!=data.end() ; it++) {
                        _arrayFile.append(*it);
                        if (i > percentToFlush) {
                            fileToWrite.write(_arrayFile);
                            if (fileToWrite.flush()) {
                                tempFileSize = fileToWrite.size();
                                _arrayFile.resize(0);
                                i = 0;
                            }
                        }
                    }
                }
                offset+=numberOfBytesToTransfer;
            }

            fileToWrite.write(_arrayFile);
            if (fileToWrite.flush()) {
                _fileWritten.setFile(fileToWrite);
                fileToWrite.close();
                emit percentTransfer(100);
                emit transferIsFinished();
            }

        } else {
            qCritical()<<tr("File %1 can not be open").arg(fileDest);
            fileToWrite.remove();
            return;
        }

        _arrayFile.resize(0);
    }
}

void FileTransferClient::_askOriginalFileName()
{
    _fileQPrx->begin_getFileName(this,SLOT(_getOriginalFileNameSlot(::std::string)));
}

void FileTransferClient::_askSize()
{
    _fileQPrx->begin_getSize(this,SLOT(_getFileSizeSlot(::Ice::Long)));
}

void FileTransferClient::_getFileSizeSlot(::Ice::Long size)
{
    _fileSize =size;
    emit fileSize(_fileSize);
    _askOriginalFileName();
}

void FileTransferClient::_progress(int progress)
{
    qDebug()<<progress<<" %";
}

void FileTransferClient::_transferIsFinished()
{
    qDebug()<< tr("File %1 has been transfered to %2").arg(_fileNameOriginal,_fileWritten.absoluteFilePath());
}

void FileTransferClient::_getOriginalFileNameSlot(::std::string name)
{
    _fileNameOriginal = QString::fromStdString(name);
    emit fileName(QString::fromStdString(name));
    _getFile();
}


}//Diamesh
