/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include <Ice/ObjectAdapter.h>

#include <QtCore/QDebug>
#include <QtSql/QSqlRecord>

#include "diamesh/hsetmanagerI.h"
#include "diamesh/sqldatabase.h"
#include "diamesh/constants.h"

#include <stdexcept>

namespace Diamesh
{

HSetManagerI::HSetManagerI(HSetStorage *storage, Ice::ObjectAdapterPtr adapter)
    : _Diamesh_HSetManager()
    , _storage(storage)
    , _adapter(adapter)
{
    _id.name = Constants::HSET_MANAGER_PROXY_NAME;
    _adapter->add(this, _id);
    QObject::connect(_storage, SIGNAL(hSetAdded(qlonglong)), this, SLOT(_hSetAddedSlot(qlonglong)));
    QObject::connect(_storage, SIGNAL(hSetChanged(qlonglong)), this, SLOT(_hSetChangedSlot(qlonglong)));
    QObject::connect(_storage, SIGNAL(hSetRemoved(qlonglong)), this, SLOT(_hSetRemovedSlot(qlonglong)));
    _storage->_init();
}

HSetManagerI::~HSetManagerI()
{
}

HSetManagerPrx HSetManagerI::proxy() const
{
    return HSetManagerPrx::uncheckedCast(_adapter->createProxy(_id));
}

Ice::Long HSetManagerI::addHSet(Ice::Long rank, const std::string &name, const std::string &description, const::Ice::Current &current)
{
    Q_UNUSED(current)
    return _storage->addHSet(rank, QString::fromStdString(name), QString::fromStdString(description));
}

bool HSetManagerI::updateHSet(const HierarchicalSet &updatedHSet, const Ice::Current &current)
{
    Q_UNUSED(current)
    return _storage->updateHSet(updatedHSet);
}

bool HSetManagerI::removeHSet(Ice::Long hSetId, const::Ice::Current &current)
{
    Q_UNUSED(current)
    return _storage->removeHSet(hSetId);
}

HierarchicalSet HSetManagerI::getHSet(Ice::Long hSetId, const::Ice::Current &current)
{
    Q_UNUSED(current)
    return _storage->getHSet(hSetId);
}

HierarchicalSetList HSetManagerI::getHSets(const::Ice::Current &current)
{
    Q_UNUSED(current)
    return _storage->getHSets();
}

Ice::Long HSetManagerI::highestHSetId() const
{
    return _storage->highestHSetId();
}

Ice::Long Diamesh::HSetManagerI::lowestHSetId() const
{
    return _storage->lowestHSetId();
}

QList<Ice::Long> HSetManagerI::getHSetsIdsList()
{
    QList<HierarchicalSet> hSetList = QList<HierarchicalSet>::fromStdList(getHSets());

    QList<qlonglong> idsList;
    foreach (const Diamesh::HierarchicalSet &hSet, hSetList) {
        idsList.append(hSet.id);
    }
    return idsList;
}

void HSetManagerI::_hSetAddedSlot(qlonglong hSetId)
{
    emit hSetAdded(hSetId);
}

void HSetManagerI::_hSetChangedSlot(qlonglong hSetId)
{
    emit hSetChanged(hSetId);
}

void HSetManagerI::_hSetRemovedSlot(qlonglong hSetId)
{
    emit hSetRemoved(hSetId);
}


HSetStorage::HSetStorage(QObject *parent)
    : QObject(parent)
{
}

HSetStorage::~HSetStorage()
{
}

bool HSetStorage::hasHSet(qlonglong hSetId)
{
    QReadLocker rLocker(&_lock);
    return _hSetHash.contains(hSetId);
}

HierarchicalSet HSetStorage::getHSet(qlonglong hSetId)
{
    QReadLocker rLocker(&_lock);
    HierarchicalSet result;
    result.id = -1;
    if (hasHSet(hSetId))
        result = _hSetHash.value(hSetId);
    return result;
}

HierarchicalSetList HSetStorage::getHSets()
{
    QReadLocker rLocker(&_lock);
    return _hSetHash.values().toStdList();
}

qlonglong HSetStorage::addHSet(qlonglong rank, const QString &name, const QString &description)
{
    qlonglong result = -1;
    QWriteLocker wLocker(&_lock);
    QList<qlonglong> modifiedHSetIds;
    if (_preInsertHSet(rank, modifiedHSetIds))  // Make sure that target rank is available
        result = _addHSet(rank, name, description); // Add to storage to generate unique Id

    if (result != -1) {
        // Unique Id is valid
        HierarchicalSet hSet;
        hSet.id = result;
        hSet.rank = rank;
        hSet.name = name.toStdString();
        hSet.description = description.toStdString();
        if (!_insertHSet(hSet)) {
            _removeHSet(result); // TODO: Should check return value
            result = -1;
        }
    }

    if (!modifiedHSetIds.isEmpty()) {
        if (result != -1) {
            // Operation succeeded, we need to update storage for all modified IDs
            emit hSetAdded(result);
            foreach(qlonglong modifiedId, modifiedHSetIds) {
                _updateHSet(_hSetHash.value(modifiedId)); // TODO: Should check return value
                emit hSetChanged(modifiedId);
            }
        } else {
            // Operation failed, we need to revert rank decrease operation
            if (!_decreaseRankRevert(modifiedHSetIds))
                qCritical() << tr("Could not revert Hierarchical set rank decrease operation.");
        }
    }
    return result;
}

bool HSetStorage::updateHSet(const HierarchicalSet &updatedHSet)
{
    bool result = false;
    qlonglong hSetId = updatedHSet.id;
    if (hasHSet(hSetId)) {
        QReadLocker rLocker(&_lock);
        qlonglong targetRank = updatedHSet.rank;
        if (targetRank < 0)
            qWarning() << tr("Trying to set invalid rank to Hierarchical set with Id '%1'.")
                          .arg(QString::number(hSetId));
        else {
            QList<qlonglong> modifiedHSetIds;
            qlonglong sourceRank = _hSetHash.value(hSetId).rank;
            if (sourceRank < targetRank) {
                _hierarchyToSet.remove(sourceRank);
                result = _increaseHSetRankRange(sourceRank, targetRank, modifiedHSetIds); // Make sure that target rank does not leave an empty space
                // TODO: handle updated Ids
            } else if (sourceRank > targetRank) {
                _hierarchyToSet.remove(sourceRank);
                result = _decreaseHsetRankRange(targetRank, sourceRank, modifiedHSetIds); // Make sure that target rank is available
            } else
                result = true;

            if (result) // Rank update successfull
                result = _updateHSet(updatedHSet);

            if (result) { // Storage update successfull
                _hSetHash[hSetId] = updatedHSet;
                _hierarchyToSet[targetRank] = hSetId;
            }

            if (!modifiedHSetIds.isEmpty()) {
                if (result) {
                    emit hSetChanged(hSetId);
                    // Operation succeeded, we need to update storage for all modified IDs
                    foreach(qlonglong modifiedId, modifiedHSetIds) {
                        _updateHSet(_hSetHash.value(modifiedId)); // TODO: Should check return value
                        emit hSetChanged(modifiedId);
                    }
                } else {
                    // Operation failed, we need to revert rank update operation
                    bool revertResult = false;
                    if (sourceRank < targetRank)
                        revertResult = _increaseRankRevert(modifiedHSetIds);
                    if (sourceRank > targetRank)
                        revertResult = _decreaseRankRevert(modifiedHSetIds);
                    if (!revertResult)
                        qCritical() << tr("Could not revert Hierarchical set rank update operation.");
                }
            }
        }
    } else
        qWarning() << tr("Could not locate Hierarchical set with Id '%1'")
                      .arg(QString::number(updatedHSet.id));
    return result;
}

bool HSetStorage::removeHSet(qlonglong hSetId)
{
    bool result = false;
    if (_hSetHash.count() == 1)
        qWarning() << tr("Could not remove last Hierarchical set");
    else if (hasHSet(hSetId)) {
        QWriteLocker wLocker(&_lock);
        qlonglong hSetRank = _hSetHash.value(hSetId).rank;
        if (_removeHSet(hSetId)) {
            _hSetHash.remove(hSetId);
            _hierarchyToSet.remove(hSetRank);
            QList<qlonglong> modifiedHSetIds;
            result = _postRemoveHSet(hSetRank + 1, modifiedHSetIds); // Make sure that target rank does not leave an empty space
            if (!modifiedHSetIds.isEmpty()) {
                if (result) {
                    // Operation succeeded, we need to update storage for all modified IDs
                    emit hSetRemoved(hSetId);
                    foreach(qlonglong modifiedId, modifiedHSetIds) {
                        _updateHSet(_hSetHash.value(modifiedId)); // TODO: Should check return value
                        emit hSetChanged(modifiedId);
                    }
                } else {
                    // Operation failed, we need to revert rank increase operation
                    if (!_increaseRankRevert(modifiedHSetIds))
                        qCritical() << tr("Could not revert Hierarchical set rank increase operation.");
                }
            }
        }
    } else
        qWarning() << tr("Could not locate Hierarchical set with Id '%1'")
                      .arg(QString::number(hSetId));
    return result;
}

qlonglong HSetStorage::highestHSetId() const
{
    QList<qlonglong> rankList = _hierarchyToSet.keys();
    qSort(rankList);
    return _hierarchyToSet.value(rankList.first());
}

qlonglong HSetStorage::lowestHSetId() const
{
    QList<qlonglong> rankList = _hierarchyToSet.keys();
    qSort(rankList);
    return _hierarchyToSet.value(rankList.last());
}

void HSetStorage::_init()
{
    QWriteLocker locker(&_lock);
    if (!_initStorage())
        qFatal("%s", tr("Failed to initialize Hierarchical sets from database.").toUtf8().constData());
    if (_hSetHash.count() == 0)
        qFatal("%s", tr("Could not read any Hierarchical set from database.").toUtf8().constData());
}

bool HSetStorage::_insertHSet(const HierarchicalSet &hSet)
{
    bool result = false;
    qlonglong key = hSet.id;
    if (key == -1)
        qWarning() << tr("Trying to insert invalid Hierarchical set with name '%1'")
                      .arg(QString::fromStdString(hSet.name));
    else if (_hSetHash.contains(key))
        qWarning() << tr("Duplicate Hierarchical set found for Id '%1'")
                      .arg(QString::number(key));
    else if (_hierarchyToSet.contains(hSet.rank))
        qWarning() << tr("Trying to insert Hierarchical set with name '%1' at rank '%2' which is already used.")
                      .arg(QString::fromStdString(hSet.name))
                      .arg(QString::number(hSet.rank));
    else {
        _hSetHash.insert(key, hSet);
        _hierarchyToSet.insert(hSet.rank, key);
        result = true;
    }
    return result;
}

bool HSetStorage::_preInsertHSet(qlonglong rank, QList<qlonglong> &updatedIds)
{
    return _decreaseHsetRankRange(rank, -1, updatedIds);
}

bool HSetStorage::_postRemoveHSet(qlonglong rank, QList<qlonglong> &updatedIds)
{
    return _increaseHSetRankRange(rank, -1, updatedIds);
}

bool HSetStorage::_updateHSetRank(qlonglong hSetId, qlonglong newRank)
{
    bool result = false;
    if (_hSetHash.contains(hSetId)) {
        _hierarchyToSet.remove(_hSetHash.value(hSetId).rank);
        _hSetHash[hSetId].rank = newRank;
        _hierarchyToSet[newRank] = hSetId;
        result = true;
    } else
        qWarning() << tr("Trying to update unknow Hierarchical set with Id '%1'")
                      .arg(QString::number(hSetId));
    return result;
}

bool HSetStorage::_decreaseHsetRankRange(qlonglong startRank, qlonglong endRank, QList<qlonglong> &updatedIds)
{
    bool result = true;
    updatedIds.clear(); // We make sure that we work with an empty list to starts with
    if (_hierarchyToSet.contains(startRank)) {
        result = _decreaseHSetRank(_hierarchyToSet.value(startRank), updatedIds, endRank);
        if (!result)
            if (!_decreaseRankRevert(updatedIds))
                qCritical() << tr("Could not revert Hierarchical set rank decrease operation.");
    }
    return result;
}

bool HSetStorage::_decreaseHSetRank(qlonglong hSetId, QList<qlonglong> &updatedIds, qlonglong rankLimit)
{
    bool result = true;
    if (_hSetHash.contains(hSetId)) {
        qlonglong newRank = _hSetHash.value(hSetId).rank + 1;
        if (rankLimit == -1 || newRank <= rankLimit) {
            if (_hierarchyToSet.contains(newRank))
                result = _decreaseHSetRank(_hierarchyToSet.value(newRank), updatedIds);

            if (result)
                result = _updateHSetRank(hSetId, newRank);

            if (result)
                updatedIds.append(hSetId);
            else
                qWarning() << tr("Could not decrease rank for Hierarchical set with Id '%1'")
                              .arg(QString::number(hSetId));
        }
    } else
        result = false;
    return result;
}

bool HSetStorage::_decreaseRankRevert(QList<qlonglong> &updatedIds)
{
    bool result = true;
    if (!updatedIds.isEmpty()) {
        foreach (qlonglong hSetId, updatedIds)
            result &= _updateHSetRank(hSetId, _hSetHash.value(hSetId).rank - 1);
        updatedIds.clear();
    }
    return result;
}

bool HSetStorage::_increaseHSetRankRange(qlonglong startRank, qlonglong endRank, QList<qlonglong> &updatedIds)
{
    bool result = true;
    updatedIds.clear(); // We make sure that we work with an empty list to starts with
    if (startRank > 0 && _hierarchyToSet.contains(startRank)) {
        result = _increaseHSetRank(_hierarchyToSet.value(startRank), updatedIds, endRank);
        if (!result)
            if (!_increaseRankRevert(updatedIds))
                qCritical() << tr("Could not revert Hierarchical set rank increase operation");
    }
    return result;
}

bool HSetStorage::_increaseHSetRank(qlonglong hSetId, QList<qlonglong> &updatedIds, qlonglong rankLimit)
{
    bool result = false;
    if (_hSetHash.contains(hSetId)) {
        qlonglong newRank = _hSetHash.value(hSetId).rank - 1;
        if (rankLimit != -1 && newRank > rankLimit)
            result = true;
        else if (newRank < 0)
            qWarning() << tr("Could not increase rank of first ranked Hierarchical set");
        else if (_hierarchyToSet.contains(newRank))
            qWarning() << tr("Could not increase rank for Hierarchical set with Id '%1'. Target rank '%2' is already used.")
                          .arg(QString::number(hSetId))
                          .arg(QString::number(newRank));
        else {
            result = _updateHSetRank(hSetId, newRank);
            if (result) {
                updatedIds.append(hSetId);
                qlonglong nextRank = newRank + 1;
                if (_hierarchyToSet.contains(nextRank))
                    result = _increaseHSetRank(_hierarchyToSet.value(nextRank), updatedIds);
            } else
                qWarning() << tr("Could not increase rank for hierarchical set with Id '%1'")
                              .arg(QString::number(hSetId));
        }
    }
    return result;
}

bool HSetStorage::_increaseRankRevert(QList<qlonglong> &updatedIds)
{
    bool result = true;
    if (!updatedIds.isEmpty()) {
        foreach (qlonglong hSetId, updatedIds)
            result &= _updateHSetRank(hSetId, _hSetHash.value(hSetId).rank + 1);
        updatedIds.clear();
    }
    return result;
}


SqlHSetStorage::SqlHSetStorage(SqlDatabase *db, QObject *parent)
    : HSetStorage(parent)
    , _db(db)
{
}

SqlHSetStorage::~SqlHSetStorage()
{
}

bool SqlHSetStorage::_initStorage()
{
    QList<QSqlRecord> results;
    if (_db->executeQuery("SELECT * FROM hsets", QList<QVariant>(), &results) == -1) {
        throw std::runtime_error("SqlHSetStorage: Unable to read Hierarchical sets from database.");
    }
    foreach(const QSqlRecord &r, results) {
        HierarchicalSet hSet;
        hSet.id = r.value(Constants::HSET_ID_FIELD_NAME).toLongLong();
        hSet.name = r.value(Constants::HSET_RANK_FIELD_NAME).toString().toStdString();
        hSet.description = r.value(Constants::HSET_DESC_FIELD_NAME).toString().toStdString();
        hSet.rank = r.value(Constants::HSET_RANK_FIELD_NAME).toLongLong();
        if (!_insertHSet(hSet))
            return false;
    }
    return true;
}

qlonglong SqlHSetStorage::_addHSet(qlonglong rank, const QString &name, const QString &description)
{
    QString query = QString("INSERT INTO hsets (%1, %2, %3) VALUES (?, ?, ?)")
            .arg(Constants::HSET_RANK_FIELD_NAME)
            .arg(Constants::HSET_NAME_FIELD_NAME)
            .arg(Constants::HSET_DESC_FIELD_NAME);
    QList<QVariant> values;
    values.append(rank);
    values.append(name);
    values.append(description);

    qlonglong result;
    if ((result = _db->executeQuery(query, values)) == -1) {
        qCritical() << tr("An error occured while trying to add Hierarchical set with name '%1' into database.")
                       .arg(name);
        return result;
    }
    return result;
}

bool SqlHSetStorage::_updateHSet(const HierarchicalSet &updatedHSet)
{
    QString query = QString("UPDATE hsets SET %1=?, %2=?, %3=? WHERE %4=?")
            .arg(Constants::HSET_RANK_FIELD_NAME)
            .arg(Constants::HSET_NAME_FIELD_NAME)
            .arg(Constants::HSET_DESC_FIELD_NAME)
            .arg(Constants::HSET_ID_FIELD_NAME);
    QList<QVariant> values;
    values.append(QVariant::fromValue(updatedHSet.rank));
    values.append(QString::fromStdString(updatedHSet.name));
    values.append(QString::fromStdString(updatedHSet.description));
    values.append(QVariant::fromValue(updatedHSet.id));
    if (_db->executeQuery(query, values) == -1) {
        qCritical() << tr("An error occured while trying to update Hierarchical set with Id '%1' into database.")
                       .arg(QString::number(updatedHSet.id));
        return false;
    }
    return true;
}

bool SqlHSetStorage::_removeHSet(qlonglong hSetId)
{
    QList<QVariant> values;
    values.append(hSetId);
    QString query = QString("DELETE FROM hsets WHERE %1=?")
            .arg(Constants::HSET_ID_FIELD_NAME);
    if (_db->executeQuery(query, values) == -1) {
        qCritical() << tr("An error occured while trying to delete Hierarchical set with Id '%1' from database.")
                       .arg(QString::number(hSetId));
        return false;
    }
    return true;
}


} // namespace Diamesh
