/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include <diamesh/logger.h>
#include <diamesh/coreapplication.h>

#include <QtCore/QDateTime>
#include <QtCore/QMetaObject>

#include <syslog.h>

namespace Diamesh {

Logger::Logger(LoggingMethod logMethod, QString logFileName, int logLevel)
    : _logFile(NULL)
    , _logMethod(logMethod)
    , _logFileName(logFileName)
    , _logLevel(logLevel)
{
    switch(_logMethod) {
    case Logger::FILE: {
        _initMethodFile();
        break;
    }
    case Logger::SYSTEM: {
        _initMethodSystem();
        break;
    }
    default: {
        _initMethodDefault();
    }
    }
}

Logger::~Logger()
{
    _deleteLogFileHandler();
}

Logger *Logger::create(const QString &logMethod, const QString &logFileName, int logLevel)
{
    LoggingMethod method;
    if (logMethod == QString::fromStdString(Constants::SETTINGS_LOG_SYSTEM))
        method = Logger::SYSTEM;
    else if (logMethod == QString::fromStdString(Constants::SETTINGS_LOG_FILE))
        method = Logger::FILE;
    else
        method = Logger::DEFAULT;
    return new Logger(method, logFileName, logLevel);
}

void Logger::logToFile(const QString &logFileName)
{
    _logFileName = logFileName;
    _logMethod = Logger::FILE;
    _initMethodFile();
}

void Logger::log(QtMsgType type, const QString &msg)
{
    if (type < _logLevel)
        return;

    switch(_logMethod) {
    case Logger::SYSTEM: {
        int priority;
        switch(type) {
        case QtDebugMsg:    priority = LOG_DEBUG; break;
        case QtWarningMsg:  priority = LOG_WARNING; break;
        case QtCriticalMsg: priority = LOG_CRIT; break;
        case QtFatalMsg:    priority = LOG_ALERT; break;
        default:            priority = LOG_NOTICE; break;
        }
        QByteArray mesg = msg.toUtf8();
        openlog(_syslogIdent.constData(), LOG_PID, LOG_USER);
        syslog(priority, "%s", mesg.constData());
        closelog();
        break;
    }
    case Logger::FILE: {
        if (_logFile) {
            QByteArray data = _formatMessage(type, msg);
            QGenericArgument dataArg = Q_ARG(QByteArray, data);
            QMetaObject::invokeMethod(this, "_writeDataToFile", dataArg);
            break;
        } // If _logFile is not set, fallback to default
    }
    default: {
        QByteArray data = _formatMessage(type, msg);
        fprintf(stderr, "%s", data.constData());
    }
    }
}

void Logger::_initMethodDefault()
{
    printf("Logging to stderr.\n");
}

void Logger::_initMethodSystem()
{
    _syslogIdent = CoreApplication::instance()->applicationName().toUtf8();
}

void Logger::_initMethodFile()
{
    if (!_logFileName.isEmpty()) {
        _openLogFileHandler();
        fprintf(stderr, "Logging to %s.\n", qPrintable(_logFileName));
    } else {
        fprintf(stderr, "No log file specified in the configuration file.\n");
        _initMethodDefault();
    }
}

QByteArray Logger::_formatMessage(QtMsgType type, const QString &msg) const
{
    QString dateStr = QDateTime::currentDateTime().toString("yyyy/MM/dd hh:mm:ss.zzz");
    QString levelStr;
    switch(type) {
    case QtDebugMsg:    levelStr = "   Debug"; break;
    case QtWarningMsg:  levelStr = " Warning"; break;
    case QtCriticalMsg: levelStr = "Critical"; break;
    case QtFatalMsg:    levelStr = "   Fatal"; break;
    default:            levelStr = " Unknown"; break;
    }

    QString log("%1 [%2] %3\n");
    return log.arg(dateStr).arg(levelStr).arg(msg).toUtf8();
}

void Logger::_openLogFileHandler()
{
    _deleteLogFileHandler();
    _logFile = new QFile(_logFileName);
    if (!_logFile->open(QIODevice::WriteOnly | QIODevice::Append ))
        fprintf(stderr, "Logger: Could not open log file %s : %s\n", qPrintable(_logFileName), qPrintable(_logFile->errorString()));
}

void Logger::_deleteLogFileHandler()
{
    if (_logFile) {
        _logFile->write("\n\n");
        _logFile->flush();
        _logFile->close();
        delete _logFile;
        _logFile = NULL;
    }
}

void Logger::_writeDataToFile(QByteArray data)
{
    if (!QFile::exists(_logFileName))
        _openLogFileHandler();
    if (_logFile->write(data) != data.size())
        fprintf(stderr, "Could not write log line to log file %s\n", qPrintable(_logFile->fileName()));
    _logFile->flush();
}

} // namespace Diamesh
