/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include <diamesh/masterI.h>

#include <Ice/ObjectAdapter.h>
#include <Ice/Communicator.h>
#include <Ice/Connection.h>
#include <Ice/Initialize.h>
#include <QtIce/utils.h>
#include <QtCore/QVector>
#include <QtCore/QReadWriteLock>
#include <QtCore/QSettings>
#include <QtCore/QtConcurrentRun>
#include <QtCore/QStringList>

#include <diamesh/constants.h>
#include <diamesh/sessionManagerI.h>
#include <diamesh/authmanagerI.h>
#include <diamesh/logger.h>
#include <diamesh/sessionManagerI.h>
#include <diamesh/usermanagerI.h>
#include <diamesh/sqldatabase.h>
#include <diamesh/sqluserstorage.h>
#include <diamesh/diamesh.h>
#include <diamesh/nodeinfoI.h>
#include <diamesh/nodepoller.h>
#include <diamesh/servicedetails.h>
#include <diamesh/aclmanagerI.h>
#include "diamesh/hsetmanagerI.h"
#include <diamesh/accessrightshelper.h>

namespace Diamesh
{

static const char *const masterNameKey = "masterName";

MasterI *MasterI::_instance = 0;

MasterI::MasterI() :
    _authManager(NULL), _sessionManager(NULL), _userManager(NULL),
    _settings(new QSettings(Diamesh::instance()->configFileName(), QSettings::IniFormat, this)),
    _flags(0)
{
    if(_instance)
        qFatal("Only one Master instance allowed !");

    Ice::InitializationData data;
    data.properties = Diamesh::instance()->props();
    _communicator = Ice::initialize(data);
    QtIce::setIceCommunicator(_communicator);

    _adapter = _communicator->createObjectAdapter("Master");

    _id.name = "master";
    _adapter->add(this, _id);

    _settings->beginGroup(Constants::SETTINGS_GROUP);
    _loadAuthorizedNodeList();

    // TODO: add a setting for the serializer
    _db = new SqlDatabase("QSQLITE", _settings->value("defaultDb", "/tmp/diamesh.db").toString());
    _name = _settings->value(masterNameKey, "Master").toString();

    _serviceDetails = new ServiceDetails(_name, _adapter);
    _instance = this;

    QObject::connect(this, SIGNAL(nodeInfoAdded(QString)), this, SLOT(_nodeInfoAddedSlot(QString)));
}

MasterI::~MasterI()
{
    while(_mapPoller.begin() != _mapPoller.end()){
        if(_mapPoller.begin().value()){
            _mapPoller.begin().value()->stop();
            delete _mapPoller.begin().value();
        }
        _mapPoller.erase(_mapPoller.begin());
    }
    delete _authManager;
    delete _userManager;
    delete _aclManager;
}

Q_GLOBAL_STATIC(QReadWriteLock, nodesLock)

bool MasterI::addNode(const NodePrx &node, const std::string &key, const ::Ice::Current &)
{
    QString nodeKey(key.c_str());

    if (!_authorizedNodes.contains(nodeKey)) {
        qCritical() << "A node failed to authenticate with the following id: " << nodeKey;
        return false;
    }

    {
        QReadLocker locker(nodesLock());
        if (_nodeInfoHash.contains(nodeKey)) {
            qCritical() << tr("A node with key %1 is already connected").arg(nodeKey);
            return false;
        }
    }
    
    qDebug("New node : %s", node->ice_toString().c_str());
    bool result = _addNode(node, nodeKey);

    return result;
}


void MasterI::removeNode(const NodePrx &node, const std::string &key, const ::Ice::Current &)
{
    QWriteLocker locker(nodesLock());
    QString nodeKey = QString::fromStdString(key);
    QSharedPointer<NodeInfoI> nodeInfo = _nodeInfoHash.value(nodeKey);
    if (nodeInfo && (nodeInfo->node() == node)) {
        _removeNode(nodeKey);
    }
}


void MasterI::_removeNode(const QString &key)
{
    QSharedPointer<NodeInfoI> nodeInfo = _nodeInfoHash.take(key);
    if (nodeInfo) {
        if(_mapPoller.find(key)!= _mapPoller.end() && _mapPoller[key]){
            _mapPoller[key]->stop();
            _mapPoller[key]->deleteLater();
            _mapPoller.erase(_mapPoller.find(key));
        }        
        qDebug() << tr("Node %1 disconnected").arg(key);
        try {
            nodeInfo->emitNodeRemoved();
        } catch (...) {
            qCritical() << tr("Exception caught while trying to remove node %1").arg(key);
        }
        emit nodeRemoved(key);
    }
}

void MasterI::_initHSetManager()
{
    _hSetManager = new HSetManagerI(new SqlHSetStorage(_db, this), _adapter);
}

void MasterI::_registerACLTargetTypes()
{
    _aclManager->registerACLTargetType(User::ice_staticId().c_str(), Constants::ACL_TARGET_USER);
    _aclManager->registerACLTargetType(Group::ice_staticId().c_str(), Constants::ACL_TARGET_GROUP);
}

void MasterI::_initACLManager()
{
    _aclManager = new ACLManagerI(new SqlACLStorage(_db, this) , _hSetManager, _adapter);
    _registerACLTargetTypes();
    _registerRights();
    _aclManager->_init();
}

void MasterI::_registerRights()
{
    _aclManager->registerAccessRight(Constants::RIGHT_DIAMESH_USERMANAGER_USER_ADD);
    _aclManager->registerAccessRight(Constants::RIGHT_DIAMESH_USERMANAGER_USER_EDIT);
    _aclManager->registerAccessRight(Constants::RIGHT_DIAMESH_USERMANAGER_USER_DELETE);

    _aclManager->registerAccessRight(Constants::RIGHT_DIAMESH_USERMANAGER_GROUP_ADD);
    _aclManager->registerAccessRight(Constants::RIGHT_DIAMESH_USERMANAGER_GROUP_EDIT);
    _aclManager->registerAccessRight(Constants::RIGHT_DIAMESH_USERMANAGER_GROUP_DELETE);
    _aclManager->registerAccessRight(Constants::RIGHT_DIAMESH_USERMANAGER_GROUP_USERS_ADD);
    _aclManager->registerAccessRight(Constants::RIGHT_DIAMESH_USERMANAGER_GROUP_USERS_REMOVE);

    _aclManager->registerAccessRight(Constants::RIGHT_DIAMESH_USER_SESSION);
    _aclManager->registerAccessRight(Constants::RIGHT_DIAMESH_USER_DISCONNECT);
    _aclManager->registerAccessRight(Constants::RIGHT_DIAMESH_USER_PASSWORD);
}

AuthManagerPrx MasterI::getAuthManager(const ::Ice::Current &)
{
    return AuthManagerPrx::uncheckedCast(_adapter->createProxy(_communicator->stringToIdentity("PermissionsVerifier")));
}

SessionManagerPrx MasterI::getSessionManager(const ::Ice::Current &)
{
    return SessionManagerPrx::uncheckedCast(_adapter->createProxy(_communicator->stringToIdentity("SessionManager")));
}

MasterI *MasterI::instance()
{
    return _instance;
}

MasterPrx MasterI::proxy()
{
    return MasterPrx::uncheckedCast(_adapter->createProxy(_id));
}

bool MasterI::start()
{
    if (!_init())
        return false;

    _adapter->activate();
    return true;
}

Ice::ObjectAdapterPtr MasterI::adapter()
{
    return _adapter;
}

Ice::CommunicatorPtr MasterI::communicator()
{
    return _communicator;
}

AuthManagerI *MasterI::authManager()
{
    return _authManager;
}

SessionManagerI *MasterI::sessionManager()
{
    return _sessionManager;
}

Diamesh::ACLManagerI *MasterI::aclManager()
{
    return _aclManager;
}

Diamesh::HSetManagerI *MasterI::hSetManager()
{
    return _hSetManager;
}

void MasterI::setUserManager(UserManagerI* userManager)
{
    _userManager = userManager;
}

void MasterI::setUserManager(SqlUserStorage* sqlUserStorage)
{
    _userManager = new UserManagerI(sqlUserStorage, _adapter);
}

void MasterI::setBaseUserManager()
{
    setUserManager(SqlUserStorage::create(_db));
}

UserManagerI *MasterI::userManager()
{
    return _userManager;
}

SqlDatabase* MasterI::db()
{
    return _db;
}

UserManagerPrx MasterI::getUserManager(const ::Ice::Current&)
{
    return _userManager->proxy();
}

NodeInfoMap MasterI::getNodeInfoMap(const ::Ice::Current &)
{
    NodeInfoMap map;
    QReadLocker locker(nodesLock());
    QHashIterator<QString, QSharedPointer<NodeInfoI> > it(_nodeInfoHash);
    while (it.hasNext()) {
        it.next();
        std::pair<std::string, NodeInfoPrx> pair(it.key().toStdString(), NodeInfoPrx::uncheckedCast(_adapter->createProxy(it.value()->identity())));
        map.insert(pair);
    }
    return map;
}

VersionInfo MasterI::getDiameshVersionInfo(const ::Ice::Current &)
{
    return _serviceDetails->getDiameshVersionInfo();
}

SystemInfo MasterI::getSystemInfo(const ::Ice::Current &)
{
    return _serviceDetails->getSystemInfo();
}

ServiceInfo MasterI::getServiceInfo(const ::Ice::Current &)
{
    return _serviceDetails->getServiceInfo();
}

std::string MasterI::getName(const ::Ice::Current &)
{
    return _name.toStdString();
}

Diamesh::ACLManagerPrx MasterI::getACLManager(const Ice::Current &)
{
    return _aclManager->proxy();
}

HSetManagerPrx MasterI::getHSetManager(const Ice::Current &)
{
    return _hSetManager->proxy();
}

Ice::Long Diamesh::MasterI::getCapabilitiesFlags(const Ice::Current &)
{
    return _flags;
}

QList<QSharedPointer<NodeInfoI> > MasterI::nodeInfoList()
{
    QReadLocker locker(nodesLock());
    return _nodeInfoHash.values();
}

QSharedPointer<NodeInfoI> MasterI::nodeInfo(const QString &key)
{
    QReadLocker locker(nodesLock());
    return _nodeInfoHash.value(key);
}

void MasterI::setDefaultRightsForNewUser(const UserPtr &user, const GroupPtr &group, const UserPtr &currentUser)
{
    std::string uid = QString("u.%1").arg(user->id).toStdString();
    std::string gid = QString("g.%1").arg(group->id).toStdString();
    QStringList rightsListUserUser;
    rightsListUserUser<<Constants::RIGHT_DIAMESH_USERMANAGER_USER_EDIT << Constants::RIGHT_DIAMESH_USER;
    _aclManager->setACL(uid, uid, _aclManager->accessRightsHelper()->rightsFromNames(rightsListUserUser), currentUser->hSetId);

    QStringList rightsListUserGroup;
    rightsListUserGroup << Constants::RIGHT_DIAMESH_USERMANAGER_GROUP_EDIT << Constants::RIGHT_DIAMESH_USERMANAGER_GROUP_USERS_ADD << Constants::RIGHT_DIAMESH_USERMANAGER_GROUP_USERS_REMOVE;
    _aclManager->setACL(gid, uid, _aclManager->accessRightsHelper()->rightsFromNames(rightsListUserGroup), currentUser->hSetId);
}

void MasterI::setDefaultRightsForNewGroup(const GroupPtr &group, const UserPtr &currentUser)
{
    Q_UNUSED(group)
    Q_UNUSED(currentUser)
}


bool MasterI::_init()
{
    if (!_userManager) {
        qCritical() << tr("User manager is not defined. Unable to start");
        return false;
    }

    _initHSetManager();

    _initACLManager();

    _authManager = new AuthManagerI(_settings->fileName(), _userManager);
    _adapter->add(_authManager, _communicator->stringToIdentity("PermissionsVerifier"));

    _sessionManager = new SessionManagerI(_settings->fileName(), _authManager, _userManager);
    _adapter->add(_sessionManager, _communicator->stringToIdentity("SessionManager"));

    return true;
}

QString MasterI::_dbName()
{
    QString dbName = _settings->value("DBName").toString();

    if(dbName.isEmpty())
        qFatal("The database name was not found in the settings.");

    return dbName;
}

bool MasterI::_loadAuthorizedNodeList()
{
    int nbNodes = _settings->beginReadArray("authorizedNodes");
    for (int i = 0; i < nbNodes; ++i) {
        _settings->setArrayIndex(i);

        QString nodeKey = _settings->value("key").toString();
        if (nodeKey.isEmpty()) {
            qCritical() << "The node key is empty.";
            return false;
        }
        if (_authorizedNodes.contains(nodeKey)) {
            qCritical() << "The node key already exists:" << nodeKey;
            return false;
        }
        _authorizedNodes.append(nodeKey);
    }
    _settings->endArray();
    return true;
}


bool MasterI::_addNode(const NodePrx &node, const QString &key)
{
    return _addNodeInfo(new NodeInfoI(key, node));
}

bool MasterI::_addNodeInfo(NodeInfoI *nodeInfo)
{
    Ice::ObjectPrx prx = _adapter->add(nodeInfo, nodeInfo->identity());
    QWriteLocker locker(nodesLock());
    QString nodeKey = nodeInfo->getName().c_str();
    _nodeInfoHash.insert(nodeKey, QSharedPointer<NodeInfoI>(nodeInfo, &NodeInfoI::destroy));
    emit nodeAdded(QString::fromStdString(prx->ice_toString()));
    emit nodeInfoAdded(nodeKey);
    return true;
}

void MasterI::receivedDisconnectSlot(const QString & key){
    QMap<QString, NodePoller * >::iterator itor = _mapPoller.find(key);
    if (itor != _mapPoller.end()) {
        QWriteLocker locker(nodesLock());
        _removeNode(key);
    }
}

void MasterI::_nodeInfoAddedSlot(const QString &nodeKey)
{
    QSharedPointer<NodeInfoI> info = nodeInfo(nodeKey);
    if (info) {
        NodePrx node = info->node()->getPingNodeProxy();
        NodePoller *poller = new NodePoller(node, nodeKey);
        _mapPoller.insert(nodeKey, poller);
        QObject::connect(poller, SIGNAL(nodeDisconnected(const QString &)), this, SLOT(receivedDisconnectSlot(const QString &)));
        poller->execute();
    }

}

} // namespace Diamesh
