/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include <diamesh/nodeI.h>

#include <diamesh/constants.h>
#include <diamesh/coreapplication.h>
#include <diamesh/diamesh.h>
#include <QtCore/QtGlobal>
#include <QtCore/QSettings>
#include <QtCore/QDebug>
#include <QtCore/QFile>

#include <Ice/ObjectAdapter.h>
#include <Ice/Communicator.h>
#include <Ice/Connection.h>
#include <Ice/Initialize.h>
#include <IceUtil/IceUtil.h>
#include <Ice/LocalException.h>
#include <QtIce/utils.h>
#include <diamesh/servicedetails.h>

namespace Diamesh
{

NodeI *NodeI::_instance = 0;

static const char * const PING_ADAPTER = "PingAdapter";

NodeI::NodeI()
    : _master(0)
    , _settings(new QSettings(Diamesh::instance()->configFileName(), QSettings::IniFormat, this))
{
    if (_instance)
        qFatal("Only one node instance is allowed.");

    Ice::InitializationData data;
    data.properties = Diamesh::instance()->props();

    // We want a different thread pool for the ping adapter
    QString prop = QString("%1.ThreadPool.Size").arg(PING_ADAPTER);
    data.properties->setProperty(prop.toStdString(), "1");

    _communicator = Ice::initialize(data);
    QtIce::setIceCommunicator(_communicator);

    _adapter = _communicator->createObjectAdapter("Node");
    _pingAdapter = _communicator->createObjectAdapterWithEndpoints(PING_ADAPTER, "tcp");

    _id.name = IceUtil::generateUUID();

    _nodeProxy = NodePrx::checkedCast(_adapter->add(this, _id));
    _pingNodeProxy = NodePrx::checkedCast(_pingAdapter->add(this, _id));

    _settings->beginGroup(Constants::SETTINGS_GROUP);
    _serviceDetails = new ServiceDetails(_keyFromSettings(), _adapter);

    _instance = this;

    QObject::connect(CoreApplication::instance(), SIGNAL(shutdown()), this, SLOT(_shutdown()));
}

NodeI::~NodeI()
{
    qDebug() << "NodeI::~NodeI()";
    _shutdown();
}

std::string NodeI::getName(const ::Ice::Current &)
{
    return _serviceDetails->getName();
}

VersionInfo NodeI::getDiameshVersionInfo(const ::Ice::Current &)
{
    return _serviceDetails->getDiameshVersionInfo();
}

SystemInfo NodeI::getSystemInfo(const ::Ice::Current &)
{
    return _serviceDetails->getSystemInfo();
}

ServiceInfo NodeI::getServiceInfo(const ::Ice::Current &)
{
    return _serviceDetails->getServiceInfo();
}

NodePrx NodeI::getPingNodeProxy(const Ice::Current &)
{
    return _pingNodeProxy;
}

std::string NodeI::getHost()
{
    return _serviceDetails->getServiceInfo().host;
}

Ice::ObjectAdapterPtr NodeI::adapter()
{
    return _adapter;
}

bool NodeI::connectToMaster()
{
    bool success = false;
    try{
        _master = MasterPrx::checkedCast(_communicator->propertyToProxy("MasterProxy"));
        _pingAdapter->activate();
        success = _master->addNode(_nodeProxy, key().toStdString());
    }
    catch (const Ice::ConnectionRefusedException &ex) {
        qCritical() << "Unable to connect to the master";
        _master = 0;
        return false;
    }
    if(!success) {
        _master = 0;
        qCritical() << "Unable to add node to master";
    }

    return success;
}

NodeI *NodeI::instance()
{
    return _instance;
}

QString NodeI::key()
{
    return QString::fromStdString(getName());
}

void NodeI::setKey(const QString &key)
{
    if (_master != 0) {
        qCritical() << "Illegal operation. Node key can not be changed when connection to master is already established.";
        return;
    }
    _serviceDetails->setName(key);
}

void NodeI::setShortName(const QString &shortName)
{
    if (_master != 0) {
        qCritical() << "Illegal operation. Node short name can not be changed when connection to master is already established.";
        return;
    }
    _serviceDetails->setShortName(shortName);
}

void NodeI::setDescription(const QString &description)
{
    if (_master != 0) {
        qCritical() << "Illegal operation. Node description can not be changed when connection to master is already established.";
        return;
    }
    _serviceDetails->setDescription(description);
}


QString NodeI::_keyFromSettings()
{
    QString value = "";
    if (_settings->contains(Constants::NODE_KEY_SETTING))
        value = _settings->value(Constants::NODE_KEY_SETTING).toString();

    if (value.isEmpty())
        qFatal("The node key identifier was not found in the settings.");

    return value;
}

const MasterPrx &NodeI::master()
{
    return _master;
}

void NodeI::_shutdown()
{
    try {
        _master->removeNode(_nodeProxy, key().toStdString());
        __setNoDelete(true);
        _communicator->destroy();
        __setNoDelete(false);
    } catch (...) { }
}

}
