/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include <diamesh/servicedetails.h>
#include <diamesh/constants.h>
#include <diamesh/version.h>
#include <QtCore/QSettings>
#include <QtCore/QProcess>
#include <QtCore/QFile>
#include <QtCore/QHash>
#include <diamesh/diamesh.h>
#include <Ice/ObjectAdapter.h>
#include <Ice/Properties.h>

namespace Diamesh
{

ServiceDetails::ServiceDetails(const QString &serviceName, Ice::ObjectAdapterPtr adapter)
    : _adapter(adapter)
    , _settings(new QSettings(Diamesh::instance()->configFileName(), QSettings::IniFormat))
    , _serviceUptime(QDateTime::currentDateTime())
    , _versionInformation(Diamesh::versionInfo())
{
    _settings->beginGroup(Constants::SETTINGS_GROUP);
    setName(serviceName);
    setShortName(_shortNameFromSettings());
    setDescription(_descriptionFromSettings());
    _initInformation();
}

ServiceDetails::~ServiceDetails()
{
    _settings->deleteLater();
}

void ServiceDetails::setName(const QString &serviceName)
{
    _serviceInformation.name = serviceName.toStdString();
}

void ServiceDetails::setShortName(const QString &shortName)
{
    _serviceInformation.shortName = shortName.toStdString();
}

void ServiceDetails::setDescription(const QString &description)
{
    _serviceInformation.description = description.toStdString();
}

std::string ServiceDetails::getName()
{
    return _serviceInformation.name;
}

VersionInfo ServiceDetails::getDiameshVersionInfo()
{
    return _versionInformation;
}

SystemInfo ServiceDetails::getSystemInfo()
{
    _refreshDynamicSystemInformation();
    return _systemInformation;
}

ServiceInfo ServiceDetails::getServiceInfo()
{
    _refreshDynamicServiceInformation();
    return _serviceInformation;
}

QByteArray ServiceDetails::_readFile(const QString &filename)
{
    QFile targetFile(filename);
    targetFile.open(QIODevice::ReadOnly);
    QByteArray data = targetFile.readAll();
    targetFile.close();
    return data;
}

QString ServiceDetails::_readCommandOutput(const QString &command)
{
    QString ouput;
    QProcess proc;
    proc.start(command);
    if (proc.waitForFinished(1000) && proc.exitCode() == 0)
        ouput = proc.readAllStandardOutput();
    return ouput;
}

void ServiceDetails::_initInformation()
{
    _initKernelInfo();
    _initProcessorsInfo();
    _initHostInfo();
    _initIceMaxThreadInfo();
    _initSystemUptimeInfo();
    _initMemoryInfo();
    _computeServiceUptimeInfo();
    _initDiskInfo();
}

void ServiceDetails::_initKernelInfo()
{
    QString result = _readCommandOutput("uname -srvm").trimmed();
    if (!result.isEmpty())
        _systemInformation.kernelInfo = result.toStdString();
}

void ServiceDetails::_initProcessorsInfo()
{
    QByteArray procInfoData = _readFile("/proc/cpuinfo");
    QRegExp procRegExp("^.*:\\s+(.*)$");

    QHash<int, std::string> procHash;

    QString modelName;
    foreach (const QByteArray &line , procInfoData.split('\n')) {
        if (line.startsWith("model name") && procRegExp.exactMatch(line))
            modelName = procRegExp.cap(1);
        if (line.startsWith("physical id") && procRegExp.exactMatch(line)) {
            int id = procRegExp.cap(1).toInt();
            if (!procHash.contains(id) && !modelName.isEmpty())
                procHash.insert(id, modelName.toStdString());
        }
    }
    _systemInformation.processors = procHash.values().toStdList();
}

void ServiceDetails::_initHostInfo()
{
    ::Ice::EndpointPtr endpoint = _adapter->getEndpoints().at(0);
    ::Ice::IPEndpointInfoPtr endpointInfo = ::Ice::IPEndpointInfoPtr::dynamicCast(endpoint->getInfo());
    _serviceInformation.host = endpointInfo->host;
}

void ServiceDetails::_initIceMaxThreadInfo()
{
    Ice::PropertiesPtr properties = Diamesh::instance()->props();
    _serviceInformation.maxThread = properties->getProperty("Ice.ThreadPool.Server.SizeMax");
}

void ServiceDetails::_initSystemUptimeInfo()
{
    _systemInformation.uptime = 0;
    _parseSystemUptimeInfo();
}

void ServiceDetails::_initMemoryInfo()
{
    _systemInformation.totalMemory = 0;
    _systemInformation.freeMemory = 0;
    _systemInformation.totalSwap = 0;
    _systemInformation.freeSwap = 0;
    _parseMemoryInfo();
}

void ServiceDetails::_initCpuUsageInfo()
{
    _systemInformation.cpuUsage = 0;
    _parseCpuUsageInfo();
}

void ServiceDetails::_initDiskInfo()
{
    _parseDiskInfo();
}

void ServiceDetails::_refreshDynamicSystemInformation()
{
    QWriteLocker locker(&_systemInformationLock);
    _parseSystemUptimeInfo();
    _parseMemoryInfo();
    _parseCpuUsageInfo();
    _parseDiskInfo();
}

void ServiceDetails::_refreshDynamicServiceInformation()
{
    QWriteLocker locker(&_serviceInformationLock);
    _computeServiceUptimeInfo();
}

void ServiceDetails::_parseSystemUptimeInfo()
{
    QByteArray uptimeInfoData = _readFile("/proc/uptime");
    _systemInformation.uptime = qRound(uptimeInfoData.split(' ').at(0).toDouble());
}

void ServiceDetails::_parseMemoryInfo()
{
    QByteArray meminfoData = _readFile("/proc/meminfo");
    QRegExp memRegExp("^\\w+:\\s+(\\d+)\\s+kB$");
    _systemInformation.freeMemory = 0;
    foreach (const QByteArray &line , meminfoData.split('\n')) {
        if (line.startsWith("MemTotal:") && memRegExp.exactMatch(line))
            _systemInformation.totalMemory = memRegExp.cap(1).toInt();
        else if (line.startsWith("MemFree:") && memRegExp.exactMatch(line))
            _systemInformation.freeMemory += memRegExp.cap(1).toInt();
        else if (line.startsWith("Cached:") && memRegExp.exactMatch(line))
            _systemInformation.freeMemory += memRegExp.cap(1).toInt();
        else if (line.startsWith("SwapTotal:") && memRegExp.exactMatch(line))
            _systemInformation.totalSwap = memRegExp.cap(1).toInt();
        else if (line.startsWith("SwapFree:") && memRegExp.exactMatch(line))
            _systemInformation.freeSwap = memRegExp.cap(1).toInt();
    }
}

void ServiceDetails::_parseCpuUsageInfo()
{
    QString result = _readCommandOutput("ps -eo pcpu --sort -pcpu");
    if (!result.isEmpty()) {
        QStringList psInfos = result.split('\n');
        _systemInformation.cpuUsage = 0.0;
        int processCount = 0;
        foreach (const QString &line, psInfos) {
            if (line == " 0.0")
                break;
            _systemInformation.cpuUsage += line.toDouble();
            processCount++;
        }
        if (processCount > 1)
            _systemInformation.cpuUsage =  _systemInformation.cpuUsage / processCount;
    }
}

void ServiceDetails::_computeServiceUptimeInfo()
{
    _serviceInformation.uptime = _serviceUptime.secsTo(QDateTime::currentDateTime());
}

void ServiceDetails::_parseDiskInfo()
{
    QStringList allowedFs;
    if (_settings->contains(Constants::SETTINGS_ALLOWED_FSTYPE))
        allowedFs = _settings->value(Constants::SETTINGS_ALLOWED_FSTYPE).toStringList();
    else
        allowedFs << "ext2" << "ext3" << "ext4";
    QString command = "df";
    foreach(const QString &fs, allowedFs)
        command += " --type=" + fs;
    QString result = _readCommandOutput(command);
    if (!result.isEmpty()) {
        _systemInformation.disks.clear();
        QStringList resultLines = result.split('\n');
        QRegExp dfRegExp("^(\\S+)\\s+(\\d+)\\s+(\\d+)\\s+(\\d+)\\s+(\\d+)%\\s+(\\S+)$");
        foreach(const QString &line, resultLines) {
            if (dfRegExp.exactMatch(line)) {
                DiskInfo disk;
                disk.filesystem     = dfRegExp.cap(1).toStdString();
                disk.totalSpace     = dfRegExp.cap(2).toLongLong();
                disk.usedSpace      = dfRegExp.cap(3).toLongLong();
                disk.freeSpace      = dfRegExp.cap(4).toLongLong();
                disk.percentageUsed = dfRegExp.cap(5).toDouble();
                disk.mountPoint     = dfRegExp.cap(6).toStdString();
                _systemInformation.disks.push_back(disk);
            }
        }
    }
}

QString ServiceDetails::_shortNameFromSettings()
{
    QString value = "";
    if (_settings->contains(Constants::SETTINGS_SHORTNAME))
        value = _settings->value(Constants::SETTINGS_SHORTNAME).toString();
    return value;
}

QString ServiceDetails::_descriptionFromSettings()
{
    QString value = "";
    if (_settings->contains(Constants::SETTINGS_DESCRIPTION))
        value = _settings->value(Constants::SETTINGS_DESCRIPTION).toString();
    return value;
}

} // namespace Diamesh
