/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include <Ice/Ice.h>
#include <Glacier2/Glacier2.h>

#include <QtCore/QDebug>
#include <QtCore/QDateTime>
#include <QtCore/QSettings>
#include <QtCore/QThread>

#include <diamesh/constants.h>
#include <diamesh/masterI.h>
#include <diamesh/authmanagerI.h>
#include <diamesh/sessionManagerI.h>
#include <diamesh/usermanagerI.h>
#include "diamesh/aclmanagerI.h"

namespace Diamesh
{

/************/
/* SessionI */
/************/

SessionI::SessionI(Ice::Long userId, const std::string &ipAddress, const Glacier2::SessionControlPrx & control) :
        _control(control)
{
    _info.userId = userId;
    _info.ipAddress = ipAddress;
    _info.connectionDateTime = QDateTime::currentDateTime().toMSecsSinceEpoch();
}

SessionI::~SessionI()
{
    qDebug() << "SessionI::~";
}

MasterPrx SessionI::getMaster(const::Ice::Current &)
{
    return MasterI::instance()->proxy();
}

void SessionI::destroy(const::Ice::Current &cur)
{
    qDebug() << "SessionI::destroy";
    MasterI::instance()->sessionManager()->removeSession(_info.userId, QString::fromStdString(cur.id.name));
    cur.adapter->remove(cur.id);
}

SessionInfo SessionI::getInfo(const::Ice::Current &)
{
    return _info;
}

Ice::Long SessionI::userId()
{
    return _info.userId;
}

const Glacier2::SessionControlPrx &SessionI::control()
{
    return _control;
}

/*******************/
/* SessionManagerI */
/*******************/

SessionManagerI::SessionManagerI(const QString &config, AuthManagerI *authManager,
                                 UserManagerI *userManager) :
    _lock(new QReadWriteLock(QReadWriteLock::Recursive)), _authManager(authManager), _userManager(userManager)
{
    QSettings settings(config, QSettings::IniFormat);
    settings.beginGroup(Constants::SETTINGS_GROUP);
    settings.beginGroup(Constants::SESSION_MANAGER_GROUP);

    if (settings.value("disconnectOnAdminChanged", false).toBool())
        QObject::connect(userManager, SIGNAL(userAdminRightsRemoved(Ice::Long)),
                         this, SLOT(disconnectUser(Ice::Long)));

    if (settings.value("disconnectOnPasswordSet", false).toBool())
        QObject::connect(_authManager, SIGNAL(userPasswordSet(Ice::Long)),
                         this, SLOT(disconnectUser(Ice::Long)));

    QObject::connect(userManager, SIGNAL(userDisabled(Ice::Long)),
                     this, SLOT(disconnectUser(Ice::Long)));
    QObject::connect(userManager, SIGNAL(userRemoved(Ice::Long)),
                     this, SLOT(disconnectUser(Ice::Long)));
}

SessionManagerI::~SessionManagerI()
{
    delete _lock;
}

Glacier2::SessionPrx SessionManagerI::create(const::std::string &userName,
                                             const::Glacier2::SessionControlPrx &control,
                                             const::Ice::Current &current)
{
    Ice::Long userId = 0;
    if ((userId = _userManager->getUserIdFromName(userName)) == 0) {
        qWarning() << "user" << userName.c_str() << "doesn't exist";
        throw Glacier2::CannotCreateSessionException("Invalid user");
    }

    ::Ice::Context ctx = current.ctx;
    std::string ctxKey = "_con.remoteAddress";
    if (ctx.find(ctxKey) == ctx.end()) {
        qCritical() << ctxKey.c_str() << "is not defined";
        throw Glacier2::CannotCreateSessionException(ctxKey + " is not defined");
    }
    std::string ipAddress = ctx.at(ctxKey);

    SessionI * sessionI = new SessionI(userId, ipAddress, control);
    Diamesh::SessionPrx session = Diamesh::SessionPrx::uncheckedCast(current.adapter->addWithUUID(sessionI));
    sessionI->_prx = session;
    QString sessionId = session->ice_getIdentity().name.c_str();
    qDebug() << QObject::tr("Session %1 created for user %2").arg(sessionId).arg(userName.c_str());

    QWriteLocker locker(_lock);
    _sessionHash.insert(sessionId, sessionI);
    _userIdSessionsHash.insert(userId, sessionId);

    emit sessionCreated(sessionId);
    return session;
}

Ice::Long SessionManagerI::userFromSessionId(const QString &sessionId)
{
    QReadLocker locker(_lock);
    if (!_sessionHash.contains(sessionId))
        return 0;
    return _sessionHash[sessionId]->userId();
}

UserPtr SessionManagerI::currentUser(const Ice::Current &current)
{
    try {
        if(current.ctx.find("auth") == current.ctx.end()) return NULL;
        QString sessionId(current.ctx.at("auth").c_str());
        Ice::Long userId = userFromSessionId(sessionId);
        return _userManager->getUser(userId);
    } catch (const std::exception & ex) {
        return NULL;
    }
}

bool SessionManagerI::isClientCall(const Ice::Current &current)
{
    const Ice::Context &ctx = current.ctx;
    return current.con && (ctx.find("_con.remoteAddress") != ctx.end());
}

void SessionManagerI::disconnectUser(Ice::Long userId, const::Ice::Current &current)
{
    QReadLocker locker(_lock);

    QString currentSessionId;
    UserPtr user = currentUser(current);
    if (user)
        currentSessionId = current.ctx.at("auth").c_str();
    qDebug() << "disconnectUser" << userId;


    UserPtr targetUser = _userManager->getUser(userId);
    if (user && !targetUser) {
        qWarning() << tr("Trying to disconnect unknown user with Id '%1'.").arg(userId);
        return;
    }

    // If current user is valid, we must check that current user has correct right
    if (user && !MasterI::instance()->aclManager()->testRightForObject(targetUser, user, Constants::RIGHT_DIAMESH_USER_DISCONNECT)) {
        qWarning() << tr("deleteUser: permission denied");
        return;
    }

    QList<QString> sessions = _userIdSessionsHash.values(userId);
    foreach (const QString &sessionId, sessions) {
        SessionI *session = _sessionHash.value(sessionId);
        if (!user || sessionId != currentSessionId)
            session->control()->destroy();
    }
    emit userDisconnected(userId);
}

SessionList SessionManagerI::getSessions(const::Ice::Current &current)
{
    QReadLocker locker(_lock);
    UserPtr user = currentUser(current);
    SessionList list;
    foreach (SessionI *session, _sessionHash) {
        if (MasterI::instance()->aclManager()->testRightForObject(user, user, Constants::RIGHT_DIAMESH_USER_SESSION))
            list.push_back(session->_prx);
    }
    return list;
}

SessionPrx SessionManagerI::getSession(const std::string &sessionId, const::Ice::Current &current)
{
    QReadLocker locker(_lock);
    UserPtr user = currentUser(current);
    SessionI *s = _session(QString::fromStdString(sessionId), user);
    if(s)
        return s->_prx;
    return NULL;
}

SessionI* SessionManagerI::session(const QString &sessionId)
{
    QReadLocker locker(_lock);
    return _session(sessionId, NULL);
}

void SessionManagerI::removeSession(Ice::Long userId, const QString &sessionId)
{
    QWriteLocker locker(_lock);
    _sessionHash.remove(sessionId);
    _userIdSessionsHash.remove(userId, sessionId);
    emit sessionDestroyed(sessionId);
}

SessionI* SessionManagerI::_session(const QString &sessionId, const UserPtr &user)
{
    SessionI *session = _sessionHash.value(sessionId, NULL);
    if (session) {
        if (!user || MasterI::instance()->aclManager()->testRightForObject(user, user, Constants::RIGHT_DIAMESH_USER_SESSION)) {
            return session;
        }
    }
    return NULL;
}

}
