/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include <diamesh/sessionhelper.h>
#include <diamesh/constants.h>
#include <diamesh/sessionManager.h>
#include <diamesh/diamesh.h>

#include <QtCore/QtGlobal>
#include <QtCore/QSettings>
#include <QtCore/QDebug>
#include <QtCore/QStringList>
#include <QtCore/QEventLoop>

#include <Ice/ObjectAdapter.h>
#include <Ice/Communicator.h>
#include <Ice/Connection.h>
#include <Ice/LocalException.h>
#include <IceUtil/IceUtil.h>
#include <Glacier2/Glacier2.h>
#include <QtCore/QThread>
#include <QtCore/QtConcurrentRun>
#include <QtCore/QFutureWatcher>

namespace Diamesh
{

static const char *const historyGroup = "ConnectionHistory";
static const char *const serversKey = "servers";
static const char *const userNamesKey = "usernames";
static const char *const passwordsGroup = "passwords";
static const char *const updateHistoryKey = "updateConnectionHistory";
static const char *const storePasswordKey = "storePassword";

ConnectException::ConnectException(const QString &message)
    : _message(message)
{
}

QString ConnectException::message() const
{
    return _message;
}

SessionPoller::SessionPoller(Glacier2::RouterPrx &router) : _router(router)
{
}

void SessionPoller::timerEvent(QTimerEvent *)
{
    try {
        _router->refreshSession();
    } catch (...) {
        killTimer(_timerId);
        emit disconnected();
    }
}

void SessionPoller::_start()
{
    _timerId = startTimer(1000);
}

SessionHelper::SessionHelper(const Ice::InitializationData &initData) :
    _isConnected(false), _initData(initData), _pollerThread(new QThread(this)), _poller(NULL),
    _adapter(NULL), _communicator(NULL), _master(NULL), _router(NULL), _session(NULL),
    _userId(0), _user(NULL), _alreadyDestroyed(false)
{
}

SessionHelper::~SessionHelper()
{
    if (!_alreadyDestroyed)
        destroy();
}

void SessionHelper::_connect(const QString &userName, const QString &password)
{
    _communicator = Ice::initialize(_initData);
    Glacier2::RouterPrx router = Glacier2::RouterPrx::uncheckedCast(_communicator->getDefaultRouter());

    QEventLoop eventLoop;
    QFutureWatcher<SessionPrx> watcher;
    connect(&watcher, SIGNAL(finished()), &eventLoop, SLOT(quit()));
    QFuture<SessionPrx> future = QtConcurrent::run(this, &SessionHelper::_createSession, router, userName, password/*, error*/);
    watcher.setFuture(future);
    if (watcher.isRunning())
        eventLoop.exec();
    _session = future.result();

    Ice::ImplicitContextPtr context = _communicator->getImplicitContext();
    context->put("auth", _session->ice_getIdentity().name);

    _router = router;
    _isConnected = true;
    _master =  _session->getMaster();
    _userName = userName;
    _userId = _session->getInfo().userId;
    _userManager = UserManagerQPrx(_master->getUserManager(), adapter());
    connect(_userManager.data(), SIGNAL(userChanged(Ice::Long)), this, SLOT(_userChangedSlot(Ice::Long)));
    _user = _userManager->getUser(_userId);
    _poller = new SessionPoller(_router);
    _poller->moveToThread(_pollerThread);
    connect(_pollerThread, SIGNAL(started()), _poller, SLOT(_start()));
    connect(_poller, SIGNAL(disconnected()), this, SLOT(destroy()));
    _pollerThread->start();
    connect(Diamesh::instance(), SIGNAL(connectionExceptionCatched()), this, SLOT(destroy()));
}

SessionPrx SessionHelper::_createSession(Glacier2::RouterPrx router, const QString &userName, const QString &password)
{
    SessionPrx session = 0;
    QString error;
    try {
        session = SessionPrx::checkedCast(router->createSession(userName.toStdString(), password.toStdString()));
    } catch (const Ice::ConnectionRefusedException &ex) {
        error = SessionHelper::tr("Connection refused");
    } catch (const Ice::ConnectFailedException &ex) {
        error = SessionHelper::tr("Connection failed");
    } catch (const Ice::DNSException &ex) {
        error = SessionHelper::tr("Connection failed, DNS exception");
    } catch (const Glacier2::PermissionDeniedException &ex) {
        error = SessionHelper::tr("Permission denied: %1").arg(ex.reason.c_str());
    } catch (const Glacier2::CannotCreateSessionException &ex) {
        error = SessionHelper::tr("Cannot create session: %1").arg(ex.reason.c_str());
    }
    if (!error.isEmpty())
        throw ConnectException(error);
    return session;
}

void SessionHelper::destroy()
{
    _userManager = NULL;

    emit disconnected();
    if (_poller) {
        _poller->deleteLater();
        _poller = NULL;
        _pollerThread->quit();
    }
    _session = NULL;
    try {
        if(_router) {
            _router->destroySession();
            _router = NULL;
        }
    } catch (const Glacier2::SessionNotExistException &) {
        // session already destroyed
    } catch (const IceUtil::NullHandleException &) {
        // NULL _router
    } catch (const Ice::ConnectionRefusedException &) {
        // Server stopped
    } catch (const Ice::ConnectionLostException &) {
    }

    _isConnected = false;

    try {
        _communicator->destroy();
    } catch (...) {
    }
    _communicator = NULL;

    _alreadyDestroyed = true;
}

const Ice::ObjectAdapterPtr &SessionHelper::adapter()
{
    if (!_router)
        throw Glacier2::SessionNotExistException();

    if (!_adapter) {
        _adapter = _communicator->createObjectAdapterWithRouter("", _router);
        _adapter->activate();
    }
    return _adapter;
}

const Ice::CommunicatorPtr &SessionHelper::communicator()
{
    return _communicator;
}

const MasterPrx &SessionHelper::master()
{
    if (!_session)
        throw Glacier2::SessionNotExistException();

    return _master;
}

const SessionPrx &SessionHelper::session()
{
    if (!_session)
        throw Glacier2::SessionNotExistException();

    return _session;
}

bool SessionHelper::isConnected()
{
    return _isConnected;
}

const QString &SessionHelper::userName()
{
    return _userName;
}

const UserPtr &SessionHelper::user()
{
    return _user;
}

void SessionHelper::_userChangedSlot(Ice::Long userId)
{
    if (userId == _userId)
        _userManager->begin_getUser(_userId, this, SLOT(_updateUserSlot(::Diamesh::UserPtr)));
}

void SessionHelper::_updateUserSlot(::Diamesh::UserPtr user)
{
    _user = user;
}

PasswordStorage::PasswordStorage()
{
}

QString PasswordStorage::getPassword(const QString &userName)
{
    QSettings settings;
    settings.beginGroup(historyGroup);
    settings.beginGroup(passwordsGroup);
    return settings.value(userName).toString();
}

void PasswordStorage::savePassword(const QString &userName, const QString &password)
{
    QSettings settings;
    settings.beginGroup(historyGroup);
    settings.beginGroup(passwordsGroup);
    settings.setValue(userName, password);
}

void PasswordStorage::clearPassword(const QString &userName)
{
    QSettings settings;
    settings.beginGroup(historyGroup);
    settings.beginGroup(passwordsGroup);
    settings.remove(userName);
}

SessionHelperFactory::SessionHelperFactory() :
    _routerHost("127.0.0.1"), _port(4063)
{
    QSettings settings;
    if (settings.status() != QSettings::AccessError) {
        settings.beginGroup(historyGroup);
        _recentServers = settings.value(serversKey).toStringList();
        if (!_recentServers.isEmpty()) {
            QStringList lastServer(_recentServers.first().split(':'));
            _routerHost = lastServer.value(0, _routerHost);
            _port = lastServer.value(1, QString::number(_port)).toInt();
        }
        _recentUsernames = settings.value(userNamesKey).toStringList();
        _updateConnectionHistory = settings.value(updateHistoryKey, true).toBool();
        _storePassword = settings.value(storePasswordKey, false).toBool();
    }
}

SessionHelperFactory::~SessionHelperFactory()
{
}


bool SessionHelperFactory::updateConnectionHistory() const
{
    return _updateConnectionHistory;
}

void SessionHelperFactory::setUpdateConnectionHistory(bool value)
{
    _updateConnectionHistory = value;
}

bool SessionHelperFactory::storePassword() const
{
    return _storePassword;
}

void SessionHelperFactory::setStorePassword(bool value)
{
    _storePassword = value;
}

void SessionHelperFactory::setRouterHost(const QString &host)
{
    _routerHost = host;
}

void SessionHelperFactory::setRouterPort(int port)
{
    _port = port;
}

const QStringList &SessionHelperFactory::recentServers() const
{
    return _recentServers;
}

const QStringList &SessionHelperFactory::recentUsernames() const
{
    return _recentUsernames;
}

QString SessionHelperFactory::getPassword(const QString &userName) const
{
    PasswordStorage storage;
    return storage.getPassword(userName);
}

QSharedPointer<SessionHelper> SessionHelperFactory::connect()
{
    QString error;
    return connect(error);
}

QSharedPointer<SessionHelper> SessionHelperFactory::connect(QString &error)
{
    QSettings settings(Diamesh::instance()->configFileName(), QSettings::IniFormat);
    settings.beginGroup(Constants::SETTINGS_GROUP);
    QString userName = settings.value("sessionUserName", "").toString();
    QString password = settings.value("sessionPassword", "").toString();
    if (userName.isEmpty()) {
        userName = _recentUsernames.value(0, QString());
        password = getPassword(userName);
    }
    return connect(userName, password, error);
}

QSharedPointer<SessionHelper> SessionHelperFactory::connect(const QString &userName, const QString &password)
{
    QString error;
    return connect(userName, password, error);
}

QSharedPointer<SessionHelper> SessionHelperFactory::connect(const QString &userName, const QString &password,
                                                            QString &error)
{
    Ice::InitializationData data;
    data.properties = Diamesh::instance()->props();
    data.properties->setProperty("Ice.ImplicitContext", "Shared"); // Needed for the session token
    data.properties->setProperty("Ice.ACM.Client", "0"); // Needed because we user a router

    if (data.properties->getProperty("Ice.Default.Router").empty()) {
        QString routerString = QString("Glacier2/router:tcp -h %1 -p %2").arg(_routerHost).arg(_port);
        data.properties->setProperty("Ice.Default.Router", routerString.toStdString());
    }

    QSharedPointer<SessionHelper> sessionHelper;
    error.clear();
    try {
        QScopedPointer<SessionHelper> session(new SessionHelper(data));
        session->_connect(userName, password);

        QSettings settings;
        settings.beginGroup(historyGroup);
        settings.setValue(updateHistoryKey, _updateConnectionHistory);
        settings.setValue(storePasswordKey, _storePassword);
        if (_updateConnectionHistory) {
            _recentServers.prepend(QString("%1:%2").arg(_routerHost).arg(_port));
            _recentServers.removeDuplicates();
            _recentUsernames.prepend(userName);
            _recentUsernames.removeDuplicates();

            settings.setValue(serversKey, _recentServers);
            settings.setValue(userNamesKey, _recentUsernames);
            if (_storePassword) {
                PasswordStorage storage;
                storage.savePassword(userName, password);
            } else {
                PasswordStorage storage;
                storage.clearPassword(userName);
            }
        } else {
            clearUserNameHistory(userName);
        }

        sessionHelper = QSharedPointer<SessionHelper>(session.take());
    } catch (const Ice::ConnectionRefusedException &ex) {
        error = SessionHelper::tr("Connection refused");
    } catch (const Ice::ConnectFailedException &ex) {
        error = SessionHelper::tr("Connection failed");
    } catch (const Ice::DNSException &ex) {
        error = SessionHelper::tr("Connection failed, DNS exception");
    } catch (const Glacier2::PermissionDeniedException &ex) {
        error = SessionHelper::tr("Permission denied: %1").arg(ex.reason.c_str());
    } catch (const Glacier2::CannotCreateSessionException &ex) {
        error = SessionHelper::tr("Cannot create session: %1").arg(ex.reason.c_str());
    } catch (const ConnectException &ex) {
        error = ex.message();
    }

    if (!error.isEmpty())
        qWarning("%s", qPrintable(SessionHelper::tr("Connection error: %1").arg(error)));

    return sessionHelper;
}

void SessionHelperFactory::clearUserNameHistory(const QString &userName)
{
    QSettings settings;
    settings.beginGroup(historyGroup);
    if (_recentUsernames.contains(userName)) {
        _recentUsernames.removeAll(userName);
        settings.setValue(userNamesKey, _recentUsernames);
        PasswordStorage storage;
        storage.clearPassword(userName);
    }
}

}
