/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include <diamesh/sqldatabase.h>

#include <QtCore/QDebug>
#include <QtCore/QMutex>
#include <QtCore/QThread>
#include <QtCore/QUuid>
#include <QtSql/QSqlError>
#include <QtSql/QSqlQuery>
#include <QtSql/QSqlRecord>
#include <QtCore/QVariant>
#include <QtCore/QFile>

#include <stdexcept>

namespace Diamesh
{

SqlDatabase::SqlDatabase(const QString &type, const QString &dbName, QObject *parent) :
    QObject(parent)
{
    _init(type, dbName);
}

SqlDatabase::SqlDatabase(const QString &type, const QString &dbName, const QString &host,
                         int port, const QString &user, const QString &password, QObject *parent) :
    QObject(parent)
{
    _init(type, dbName, host, port, user, password);
}

SqlDatabase::~SqlDatabase()
{
    _mainDb.close();
}

qlonglong SqlDatabase::executeQuery(const QString &queryStr, const QList<QVariant> &values, QList<QSqlRecord> *results)
{
    qulonglong ret = 0;
    QString connectionName = _getConnectionName();

    {
        QSqlDatabase db = QSqlDatabase::database(connectionName);

        if (db.isOpenError()) {
            qCritical() << "Error opening database:" << db.lastError().text();
            return -1;
        }

        // enable foreign keys support for sqlite databases
        if (db.driverName() == "QSQLITE") {
            QSqlQuery query(db);
            if (!query.exec("pragma foreign_keys = on")) {
                QSqlError error = query.lastError();
                qCritical() << "Error" << error.type() << "executing pragma query:" << error.text();
                return -1;
            }
        }

        qDebug() << "SqlDatabase::executeQuery:" << queryStr << values;

        QSqlQuery query(db);
        query.prepare(queryStr);

        for (int i = 0 ; i < values.size() ; i++)
            query.bindValue(i, values.at(i).toString());

        if (!query.exec()) {
            QSqlError error = query.lastError();
            qCritical() << "Error" << error.type() << "executing query:" << error.text();
            return -1;
        }

        if (results && query.isSelect()) {
            while(query.next()) {
                results->append(query.record());
            }
        }

        QVariant id = query.lastInsertId();
        if (!id.isNull())
            ret = id.toLongLong();
    }

    QMutexLocker locker(&_mutex);
    if (connectionName != _mainDb.connectionName())
        QSqlDatabase::removeDatabase(connectionName);

    return ret;
}

qlonglong SqlDatabase::executeQuery(const QString &queryStr, const QVariant& value, QList<QSqlRecord> *results)
{
    QList<QVariant> values;
    values.append(value);
    return executeQuery(queryStr, values, results);
}

void SqlDatabase::_init(const QString &type, const QString &dbName, const QString &host,
                        int port, const QString &user, const QString &password)
{
    _mainDb = QSqlDatabase::addDatabase(type, QUuid::createUuid().toString());
    if (type == "QSQLITE" && !QFile(dbName).exists()) {
        throw std::runtime_error(tr("The database file does not exist: %1").arg(dbName).toStdString());
    }
    _mainDb.setDatabaseName(dbName);
    _mainDb.setHostName(host);
    if (port != 0)
        _mainDb.setPort(port);
    _mainDb.setUserName(user);
    _mainDb.setPassword(password);
    _mainDb.setConnectOptions("QSQLITE_BUSY_TIMEOUT=15000");
}

QString SqlDatabase::_getConnectionName()
{
    QMutexLocker locker(&_mutex);

    if (QThread::currentThread() == thread()) {
        return _mainDb.connectionName();
    } else {
        QString connectionName = QUuid::createUuid().toString();
        QSqlDatabase::cloneDatabase(_mainDb, connectionName);
        return connectionName;
    }
}

}
