/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include <diamesh/sqluserstorage.h>

#include <diamesh/sqldatabase.h>
#include <QtCore/QtDebug>
#include <QtCore/QDateTime>
#include <QtCore/QStringList>
#include <QtCore/QtDebug>
#include <QtSql/QSqlQuery>
#include <QtSql/QSqlRecord>
#include <stdexcept>

namespace Diamesh
{

SqlUserStorage::SqlUserStorage(SqlDatabase *db) :
    _db(db)
{
}

SqlUserStorage::~SqlUserStorage()
{
}

// static
SqlUserStorage* SqlUserStorage::create(SqlDatabase *db)
{
    SqlUserStorage * storage = new SqlUserStorage(db);
    storage->init();
    return storage;
}

void SqlUserStorage::init()
{
    _loadFromDb();
}

Ice::Long SqlUserStorage::addUser(const ::Diamesh::UserPtr &user, const QHash<QString, QVariant> &additionalFields)
{
    qint64 time = QDateTime::currentMSecsSinceEpoch();

    QString additionalFieldsKeys;
    QString additionalQuestionMarks;
    QStringList keys = additionalFields.keys();
    for (int i = 0 ; i < keys.size() ; i++) {
        additionalFieldsKeys.append(QString(", %1").arg(keys.at(i)));
        additionalQuestionMarks.append(", ?");
    }

    QString query = QString("INSERT INTO users (name, isEnabled, createdOn, modifiedOn, hsetid %1) "
                  "VALUES (?, ?, ?, ?, ? %2)").arg(additionalFieldsKeys).arg(additionalQuestionMarks);

    QVariantList values;
    values.append(user->name.c_str());
    values.append(user->isEnabled);
    values.append(time);
    values.append(time);
    values.append(QVariant::fromValue(user->hSetId));
    values.append(additionalFields.values());

    Ice::Long id;
    if ((id = _db->executeQuery(query, values)) == -1) {
        qCritical() << "Error adding user" << user->name.c_str() << "to database";
        return 0;
    }
    user->id = id;
    user->createdOn = time;
    user->modifiedOn = time;
    QWriteLocker locker(&_usersLock);
    _usersHash.insert(id, user);
    _usersNamesIdsHash.insert(user->name.c_str(), id);

    return id;
}

bool SqlUserStorage::deleteUser(::Ice::Long userId)
{
    QReadLocker locker(&_usersLock);
    if (!_usersHash.contains(userId)) {
        qCritical() << "No user with id" << userId << "exists";
        return false;
    }
    locker.unlock();

    QList<QVariant> values;
    values.append(qulonglong(userId));
    if (_db->executeQuery("DELETE FROM users WHERE id=?", values) == -1) {
        qCritical() << "Error deleting user" << userId << "from database";
        return false;
    }

    _usersLock.lockForWrite();
    UserPtr user = _usersHash.take(userId);
    _usersNamesIdsHash.remove(user->name.c_str());
    _usersLock.unlock();

    // Update groups
    const LongList &groups = user->groupList;
    _groupsLock.lockForWrite();
    for (LongList::const_iterator it = groups.begin() ; it != groups.end() ; ++it)
        _groupsHash.value(*it)->userList.remove(user->id);
    _groupsLock.unlock();

    return true;
}

Diamesh::UserPtr SqlUserStorage::getUser(::Ice::Long userId)
{
    QReadLocker locker(&_usersLock);
    if (!_usersHash.contains(userId)) {
        qCritical() << "No user with id" << userId << "exists";
        return 0;
    }

    return new User(*_usersHash.value(userId));
}

Ice::Long SqlUserStorage::getUserIdFromName(const QString &userName)
{
    QReadLocker locker(&_usersLock);
    return _usersNamesIdsHash.value(userName);
}

bool SqlUserStorage::updateUser(const ::Diamesh::UserPtr &user, const QHash<QString, QVariant> &additionalFields)
{
    QReadLocker readLocker(&_usersLock);
    if (!_usersHash.contains(user->id)) {
        qCritical() << "No user with id" << user->id << "exists";
        return false;
    }
    readLocker.unlock();

    QString additionalFieldsKeys;
    QStringList keys = additionalFields.keys();
    for (int i = 0 ; i < keys.size() ; i++)
        additionalFieldsKeys.append(QString(", %1=?").arg(keys.at(i)));

    qint64 time = QDateTime::currentMSecsSinceEpoch();
    QString query = QString("UPDATE users SET name=?, isEnabled=?, "
                  "modifiedOn=?, hsetid=? %1 WHERE id=?").arg(additionalFieldsKeys);

    QVariantList values;
    values.append(user->name.c_str());
    values.append(user->isEnabled);
    values.append(time);
    values.append(QVariant::fromValue(user->hSetId));
    values.append(additionalFields.values());
    values.append(qulonglong(user->id));

    if (_db->executeQuery(query, values) == -1) {
        qCritical() << "Error updating user" << user->id;
        return false;
    }

    QWriteLocker writeLocker(&_usersLock);
    UserPtr localUser = _usersHash.value(user->id);
    if (localUser->name != user->name) {
        _usersNamesIdsHash.remove(localUser->name.c_str());
        _usersNamesIdsHash.insert(user->name.c_str(), user->id);
    }
    localUser->name = user->name;
    localUser->isEnabled = user->isEnabled;
    localUser->modifiedOn = time;

    return true;
}

Ice::Long SqlUserStorage::addGroup(const ::Diamesh::GroupPtr &group)
{
    qint64 time = QDateTime::currentMSecsSinceEpoch();

    QString query("INSERT INTO groups (name, createdOn, modifiedOn, hsetid) VALUES (?, ?, ?, ?)");
    QList<QVariant> values;
    values.append(group->name.c_str());
    values.append(time);
    values.append(time);
    values.append(QVariant::fromValue(group->hSetId));

    Ice::Long id;
    if ((id = _db->executeQuery(query, values)) == -1) {
        qCritical() << "Error adding group" << group->name.c_str() << "to database";
        return 0;
    }
    group->id = id;
    group->createdOn = time;
    group->modifiedOn = time;
    QWriteLocker locker(&_groupsLock);
    _groupsHash.insert(id, group);

    return id;
}

bool SqlUserStorage::deleteGroup(::Ice::Long groupId)
{
    QReadLocker locker(&_groupsLock);
    if (!_groupsHash.contains(groupId)) {
        qCritical() << "No group with id" << groupId << "exists";
        return false;
    }
    locker.unlock();

    QString query("DELETE FROM groups WHERE id=?");
    QList<QVariant> values;
    values.append(qulonglong(groupId));

    if (_db->executeQuery(query, values) == -1) {
        qCritical() << "Error deleting group" << groupId << "from database";
        return false;
    }

    _groupsLock.lockForWrite();
    GroupPtr group = _groupsHash.take(groupId);
    _groupsLock.unlock();

    // Update users
    const LongList &users = group->userList;
    _usersLock.lockForWrite();
    for (LongList::const_iterator it = users.begin() ; it != users.end() ; ++it)
        _usersHash.value(*it)->groupList.remove(group->id);
    _usersLock.unlock();

    return true;
}

Diamesh::GroupPtr SqlUserStorage::getGroup(::Ice::Long groupId)
{
    QReadLocker locker(&_groupsLock);
    if (!_groupsHash.contains(groupId)) {
        qCritical() << "No group with id" << groupId << "exists";
        return 0;
    }

    return new Group(*_groupsHash.value(groupId));
}

bool SqlUserStorage::updateGroup(const ::Diamesh::GroupPtr &group)
{
    QReadLocker readLocker(&_groupsLock);
    if (!_groupsHash.contains(group->id)) {
        qCritical() << "No group with id" << group->id << "exists";
        return false;
    }
    readLocker.unlock();

    qint64 time = QDateTime::currentMSecsSinceEpoch();

    QString query("UPDATE groups SET name=?, modifiedOn=?, hsetid=? WHERE id=?");
    QList<QVariant> values;
    values.append(group->name.c_str());
    values.append(time);
    values.append(QVariant::fromValue(group->hSetId));
    values.append(qulonglong(group->id));

    if (_db->executeQuery(query, values) == -1) {
        qCritical() << "Error updating group" << group->id;
        return false;
    }

    QWriteLocker writeLocker(&_groupsLock);
    GroupPtr localGroup = _groupsHash.value(group->id);
    localGroup->name = group->name;

    return true;
}

bool SqlUserStorage::addUserToGroup(::Ice::Long userId, ::Ice::Long groupId)
{
    QReadLocker usersReadLocker(&_usersLock);
    if (!_usersHash.contains(userId)) {
        qCritical() << "No user with id" << userId << "exists";
        return false;
    }
    usersReadLocker.unlock();

    QReadLocker groupsReadLocker(&_groupsLock);
    if (!_groupsHash.contains(groupId)) {
        qCritical() << "No group with id" << groupId << "exists";
        return false;
    }
    groupsReadLocker.unlock();

    QString query("INSERT INTO join_groups_users (userId, groupId) VALUES (?, ?)");
    QList<QVariant> values;
    values.append(qulonglong(userId));
    values.append(qulonglong(groupId));

    if (_db->executeQuery(query, values) == -1) {
        qCritical() << "Error adding user" << userId << "to group" << groupId;
        return false;
    }

    QWriteLocker usersWriteLocker(&_usersLock);
    QWriteLocker groupsWriteLocker(&_groupsLock);
    _usersHash.value(userId)->groupList.push_back(groupId);
    _groupsHash.value(groupId)->userList.push_back(userId);

    return true;
}

bool SqlUserStorage::removeUserFromGroup(::Ice::Long userId, ::Ice::Long groupId)
{
    QReadLocker usersReadLocker(&_usersLock);
    if (!_usersHash.contains(userId)) {
        qCritical() << "No user with id" << userId << "exists";
        return false;
    }
    usersReadLocker.unlock();

    QReadLocker groupsReadLocker(&_groupsLock);
    if (!_groupsHash.contains(groupId)) {
        qCritical() << "No group with id" << groupId << "exists";
        return false;
    }
    groupsReadLocker.unlock();

    QString query("DELETE FROM join_groups_users WHERE userId=? and groupId=?");
    QList<QVariant> values;
    values.append(qulonglong(userId));
    values.append(qulonglong(groupId));

    if (_db->executeQuery(query, values) == -1) {
        qCritical() << "Error removing user" << userId << "from group" << groupId;
        return false;
    }

    QWriteLocker usersWriteLocker(&_usersLock);
    QWriteLocker groupsWriteLocker(&_groupsLock);
    _usersHash.value(userId)->groupList.remove(groupId);
    _groupsHash.value(groupId)->userList.remove(userId);

    return true;
}

QList<Ice::Long> SqlUserStorage::getUsers()
{
    QReadLocker locker(&_usersLock);
    return _usersHash.keys();
}

QList<Ice::Long> SqlUserStorage::getGroups()
{
    QReadLocker locker(&_groupsLock);
    return _groupsHash.keys();
}

void SqlUserStorage::_loadFromDb()
{
    QWriteLocker usersLocker(&_usersLock);
    QWriteLocker groupsLocker(&_groupsLock);

    QList<QSqlRecord> results;
    if (_db->executeQuery("SELECT * FROM users", QList<QVariant>(), &results) == -1)
        throw std::runtime_error("SqlUserStorage: Unable to get users from DB");

    foreach(const QSqlRecord &r, results) {
        UserPtr user = new User();
        user->id = r.value("id").toLongLong();
        user->name = r.value("name").toString().toStdString();
        user->isEnabled = r.value("isEnabled").toBool();
        user->createdOn = r.value("createdOn").toLongLong();
        user->modifiedOn = r.value("modifiedOn").toLongLong();
        user->hSetId = r.value("hsetid").toLongLong();
        _usersHash.insert(user->id, user);
        _usersNamesIdsHash.insert(user->name.c_str(), user->id);
    }

    results.clear();
    if (_db->executeQuery("SELECT * FROM groups", QList<QVariant>(), &results) == -1)
        throw std::runtime_error("SqlUserStorage: Unable to get groups from DB");

    foreach(const QSqlRecord &r, results) {
        GroupPtr group = new Group();
        group->id = r.value("id").toLongLong();
        group->name = r.value("name").toString().toStdString();
        group->createdOn = r.value("createdOn").toLongLong();
        group->modifiedOn = r.value("modifiedOn").toLongLong();
        group->hSetId = r.value("hsetid").toLongLong();
        _groupsHash.insert(group->id, group);
    }

    results.clear();
    if (!_db->executeQuery("SELECT * FROM join_groups_users", QList<QVariant>(), &results) == -1)
        throw std::runtime_error("SqlUserStorage: Unable to get join table from DB");

    foreach(const QSqlRecord &r, results) {
        qulonglong groupId = r.value("groupId").toLongLong();
        qulonglong userId = r.value("userId").toLongLong();

        GroupPtr group = _groupsHash.value(groupId);
        UserPtr user = _usersHash.value(userId);

        // ignore unknown groups and users
        if (!group || !user)
            continue;

        group->userList.push_back(userId);
        user->groupList.push_back(groupId);
    }
}

} // namespace Diamesh
