/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include <QtCore/QtDebug>

#include "diamesh/usermanagerI.h"
#include "diamesh/masterI.h"
#include "diamesh/sessionManagerI.h"
#include "diamesh/aclmanagerI.h"
#include "diamesh/hsetmanagerI.h"
#include "diamesh/constants.h"


namespace Diamesh
{

UserManagerI::UserManagerI(UserStorage *storage, Ice::ObjectAdapterPtr adapter, QObject *) :
    _Diamesh_UserManager(), _storage(storage), _adapter(adapter)
{
    _storage->init();
    _id.name = "userManager";
    _adapter->add(this, _id);
}

UserManagerI::~UserManagerI()
{
}

Ice::Long UserManagerI::addUser(const UserPtr &user, const ::Ice::Current &current)
{
    UserPtr currentUser = MasterI::instance()->sessionManager()->currentUser(current);
    if (!MasterI::instance()->aclManager()->testRight(Constants::RIGHT_TARGET_GLOBAL_USERMANAGER, currentUser, Constants::RIGHT_DIAMESH_USERMANAGER_USER_ADD)) {
        qWarning() << tr("addUser: permission denied");
        return 0;
    }

    foreach (Ice::Long groupId, getGroups(current)) {
        if (getGroup(groupId)->name == user->name){
            qCritical() << QObject::tr("There is already a group with the name %1, you cannot create a user with the same name").arg(QString::fromStdString(getGroup(groupId)->name));
            return 0;
        }
    }

    Ice::Long gid = 0;
    if (user->hSetId == -1)
        user->hSetId = MasterI::instance()->hSetManager()->lowestHSetId();
    GroupPtr group = new Group();
    group->name = user->name;
    group->hSetId = user->hSetId;
    if ((gid = _addGroup(group, false, currentUser)) == 0) {
        qWarning() << tr("addUser: unable to add main group for user %1").arg(user->name.c_str());
        return 0;
    }

    Ice::Long id;
    if ((id = _storage->addUser(user)) != 0) {
        emit userAdded(id);
        addUserToGroup(id, gid, current);
    } else {
        deleteGroup(gid, current);
    }
    MasterI::instance()->setDefaultRightsForNewUser(user, getGroup(gid), currentUser);
    return id;
}

LongList UserManagerI::cloneUser(const UserPtr &user, int clonesNumber, const ::Ice::Current &current)
{
    LongList clonedUsers;
    QString baseName = user->name.c_str();
    int nameSuffix = 0;
    for (int i = 0 ; i < clonesNumber ; i++) {
        UserPtr cloneUser = UserPtr::dynamicCast(user->ice_clone());
        cloneUser->id = 0;
        do {
            nameSuffix++;
            cloneUser->name = QString("%1_%2").arg(baseName).arg(nameSuffix).toStdString();
        } while (_userExists(cloneUser->name.c_str()));

        cloneUser->groupList.clear();
        Ice::Long userId = addUser(cloneUser, current);
        if (!userId) {
            qCritical() << tr("Error while cloning user number %1 / %2").arg(i).arg(clonesNumber);
            return clonedUsers;
        }
        clonedUsers.push_back(userId);
    }
    return clonedUsers;
}

bool UserManagerI::deleteUser(::Ice::Long userId, const ::Ice::Current &current)
{
    UserPtr currentUser = MasterI::instance()->sessionManager()->currentUser(current);

    if (currentUser->id == userId) {
        qWarning() << tr("deleteUser: trying to delete current user's account. Forbidden...");
        return false;
    }

    UserPtr user = _storage->getUser(userId);
    if (!user) {
        qWarning() << tr("deleteUser: no user with id %1").arg(userId);
        return false;
    }

    if (!MasterI::instance()->aclManager()->testRightForObject(user, currentUser, Constants::RIGHT_DIAMESH_USERMANAGER_USER_DELETE)) {
        qWarning() << tr("deleteUser: permission denied");
        return false;
    }

    QString userName = QString::fromStdString(user->name);
    if (_storage->deleteUser(userId)) {
        const LongList &groups = user->groupList;
        for (LongList::const_iterator it = groups.begin() ; it != groups.end() ; ++it) {
            if (getGroup(*it)->name == userName.toStdString()) {
                /* We delete the group if there is only this user inside */
                if (getGroup(*it)->userList.size() == 0) {
                    deleteGroup(*it, current);
                    continue;
                }
            }
            emit groupChanged(*it);
        }
        emit _Diamesh_UserManager::userRemoved(userId);
        emit userRemoved(userName);
        return true;
    }

    return false;
}

UserPtr UserManagerI::getUser(::Ice::Long userId, const ::Ice::Current &)
{
    return _storage->getUser(userId);
}

bool UserManagerI::updateUser(const UserPtr &user, const ::Ice::Current &current)
{
    UserPtr currentUser = MasterI::instance()->sessionManager()->currentUser(current);
    UserPtr oldUser = _storage->getUser(user->id);
    if (!oldUser) {
        qWarning() << tr("updateUser: no user with id %1").arg(user->id);
        return false;
    }

    if (!MasterI::instance()->aclManager()->testRightForObject(oldUser, currentUser, Constants::RIGHT_DIAMESH_USERMANAGER_USER_EDIT)) {
        qWarning() << tr("updateUser: permission denied");
        return false;
    }


    bool isEnabledChanged = (oldUser->isEnabled != user->isEnabled);

    if (isEnabledChanged && currentUser->id == user->id) {
        qWarning() << tr("updateUser: trying to disable current user's account. Forbidden...");
        return false;
    }

    if (_storage->updateUser(user)) {
        emit userChanged(user->id);

        if (isEnabledChanged)
            emit userDisabled(user->id);

        if (oldUser->name != user->name)
            emit userRenamed(oldUser->name.c_str(), user->name.c_str());

        return true;
    }

    return false;
}

Ice::Long UserManagerI::addGroup(const GroupPtr &group, const ::Ice::Current &current)
{
    UserPtr currentUser = MasterI::instance()->sessionManager()->currentUser(current);
    if (!MasterI::instance()->aclManager()->testRight(Constants::RIGHT_TARGET_GLOBAL_USERMANAGER, currentUser, Constants::RIGHT_DIAMESH_USERMANAGER_GROUP_ADD)) {
        qWarning() << tr("addGroup: permission denied");
        return 0;
    }
    return _addGroup(group, true, currentUser);
}

bool UserManagerI::deleteGroup(::Ice::Long groupId, const ::Ice::Current &current)
{
    GroupPtr group = _storage->getGroup(groupId);
    if (!group) {
        qWarning() << tr("deleteGroup: no group with id %1").arg(groupId);
        return false;
    }

    UserPtr currentUser = MasterI::instance()->sessionManager()->currentUser(current);
    if (!MasterI::instance()->aclManager()->testRightForObject(group, currentUser, Constants::RIGHT_DIAMESH_USERMANAGER_GROUP_DELETE)) {
        qWarning() << tr("deleteGroup: permission denied");
        return false;
    }

    if (!group->userList.empty()) {
        qWarning() << tr("deleteGroup: the group is not empty");
        return false;
    }

    if (_storage->deleteGroup(groupId)) {
        emit groupRemoved(groupId);
        return true;
    }

    return false;
}

GroupPtr UserManagerI::getGroup(::Ice::Long groupId, const ::Ice::Current &)
{
    return _storage->getGroup(groupId);
}

bool UserManagerI::updateGroup(const GroupPtr &group, const ::Ice::Current &current)
{
    if (!_storage->getGroup(group->id)) {
        qWarning() << tr("updateGroup: no group with id %1").arg(group->id);
        return false;
    }

    UserPtr currentUser = MasterI::instance()->sessionManager()->currentUser(current);
    if (!MasterI::instance()->aclManager()->testRightForObject(group, currentUser, Constants::RIGHT_DIAMESH_USERMANAGER_GROUP_EDIT)) {
        qWarning() << tr("updateGroup: permission denied");
        return false;
    }

    if (_storage->updateGroup(group)) {
        emit groupChanged(group->id);
        return true;
    }

    return false;
}

bool UserManagerI::addUserToGroup(::Ice::Long userId, ::Ice::Long groupId, const::Ice::Current &current)
{
    GroupPtr group = _storage->getGroup(groupId);
    if (!group) {
        qWarning() << tr("addUserToGroup: no group with id %1").arg(groupId);
        return false;
    }

    UserPtr currentUser = MasterI::instance()->sessionManager()->currentUser(current);
    if (!MasterI::instance()->aclManager()->testRightForObject(group, currentUser, Constants::RIGHT_DIAMESH_USERMANAGER_GROUP_USERS_ADD)) {
        qWarning() << tr("addUserToGroup: permission denied");
        return false;
    }

    if (!_storage->getUser(userId)) {
        qWarning() << tr("addUserToGroup: no user with id %1").arg(userId);
        return false;
    }

    if (_storage->addUserToGroup(userId, groupId)) {
        emit userChanged(userId);
        emit groupChanged(groupId);
        return true;
    }

    return false;
}

bool UserManagerI::removeUserFromGroup(::Ice::Long userId, ::Ice::Long groupId, const::Ice::Current &current)
{
    GroupPtr group = _storage->getGroup(groupId);
    if (!group) {
        qWarning() << tr("removeUserFromGroup: no group with id %1").arg(groupId);
        return false;
    }

    UserPtr currentUser = MasterI::instance()->sessionManager()->currentUser(current);
    if (!MasterI::instance()->aclManager()->testRightForObject(group, currentUser, Constants::RIGHT_DIAMESH_USERMANAGER_GROUP_USERS_REMOVE)) {
        qWarning() << tr("removeUserFromGroup: permission denied");
        return false;
    }

    if (!_storage->getUser(userId)) {
        qWarning() << tr("removeUserFromGroup: no user with id %1").arg(userId);
        return false;
    }

    if (_storage->removeUserFromGroup(userId, groupId)) {
        emit userChanged(userId);
        emit groupChanged(groupId);
        return true;
    }

    return false;
}

LongList UserManagerI::getUsers(const ::Ice::Current &)
{
    return _storage->getUsers().toStdList();
}

LongList UserManagerI::getGroups(const ::Ice::Current &)
{
    return _storage->getGroups().toStdList();
}

Ice::Long UserManagerI::getUserIdFromName(const std::string &userName, const::Ice::Current &)
{
    return _storage->getUserIdFromName(userName.c_str());
}

UserManagerPrx UserManagerI::proxy()
{
    return UserManagerPrx::uncheckedCast(_adapter->createProxy(_id));
}

bool UserManagerI::_userExists(const QString &userName)
{
    QList< ::Ice::Long> userIds = _storage->getUsers();
    for (int i = 0 ; i < userIds.size() ; i++) {
        if (_storage->getUser(userIds.at(i))->name.c_str() == userName)
            return true;
    }
    return false;
}

Ice::Long UserManagerI::_addGroup(const GroupPtr &group, bool setDefaultSettings, const UserPtr &currentUser)
{
    group->userList.clear();

    if (setDefaultSettings) { //For the moment, setDefaultSettings is only false when called by addUser
        MasterI::instance()->setDefaultRightsForNewGroup(group, currentUser);
        group->hSetId = currentUser->hSetId;
    }

    Ice::Long id;
    if ((id = _storage->addGroup(group)) != 0)
        emit groupAdded(id);

    return id;
}

} // namespace Diamesh
