/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include <diamesh/utils.h>
#include <QtCore/QDir>
#include <QtCore/QDebug>

#include <errno.h>
#include <unistd.h>

namespace Diamesh
{

QString absolutePath(const QString &path)
{
    if (path.startsWith("/"))
        return path;

    return QString("%1/%2").arg(QDir::current().absolutePath()).arg(path);
}

bool removeDirectory(const QString& path)
{
  QDir dir(path);
  QFileInfoList files = dir.entryInfoList(QDir::AllEntries | QDir::System | QDir::Hidden | QDir::NoDotAndDotDot);

  foreach (const QFileInfo & info, files) {
    if (info.isDir() && !info.isSymLink()) {
      if (!removeDirectory(info.absoluteFilePath())) {
        qWarning() << QObject::tr("Unable to remove directory \"%1\"").arg(info.absoluteFilePath());
        return false;
      }
    } else {
      if (!dir.remove(info.absoluteFilePath())) {
        qWarning() << QObject::tr("Unable to remove file \"%1\"").arg(info.absoluteFilePath());
        return false;
      }
    }
  }
  return dir.rmdir(dir.absolutePath());
}

bool copyDirectory(const QString& fromPath, const QString& toPath)
{
    QDir fromDir(fromPath);
    QFileInfoList files = fromDir.entryInfoList(QDir::AllEntries | QDir::System | QDir::Hidden | QDir::NoDotAndDotDot);

    if (!QDir().mkdir(toPath)) {
        qWarning() << QObject::tr("Unable to create directory \"%1\"").arg(toPath);
        return false;
    }

    foreach (const QFileInfo & info, files) {
        QString childPath(toPath + "/" + info.fileName());
        if (info.isDir() && !info.isSymLink()) {
            if (!copyDirectory(info.absoluteFilePath(), childPath)) {
                qWarning() << QObject::tr("Unable to copy directory \"%1\"").arg(info.absoluteFilePath());
                return false;
            }
        } else {
            if (!QFile().copy(info.absoluteFilePath(), childPath)) {
                qWarning() << QObject::tr("Unable to copy file \"%1\" to \"%2\"").arg(info.absoluteFilePath()).arg(childPath);
                return false;
            }
        }
    }
    return true;
}

void setFlags(int flagsToSet, int &currentFlags, bool enabled)
{
  if (enabled) 
    currentFlags |= flagsToSet;
  else 
    currentFlags &= ~(unsigned int)(flagsToSet);
}

/* Like read(), but restarts after EINTR */
ssize_t saferead(int fd, void *buf, size_t count)
{
    size_t nread = 0;
    while (count > 0) {
        ssize_t r = ::read(fd, buf, count);
        if (r < 0 && errno == EINTR)
            continue;
        if (r < 0)
            return r;
        if (r == 0)
            return nread;
        buf = (char *)buf + r;
        count -= r;
        nread += r;
    }
    return nread;
}

/* Like write(), but restarts after EINTR */
ssize_t safewrite(int fd, const void *buf, size_t count)
{
    size_t nwritten = 0;
    while (count > 0) {
        ssize_t r = ::write(fd, buf, count);

        if (r < 0 && errno == EINTR)
            continue;
        if (r < 0)
            return r;
        if (r == 0)
            return nwritten;
        buf = (const char *)buf + r;
        count -= r;
        nwritten += r;
    }
    return nwritten;
}

};
