/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include <diamesh/connectionwidget.h>

#include <QtCore/QSettings>
#include <QtCore/QVariant>
#include <QtGui/QCompleter>
#include <QtGui/QHBoxLayout>
#include <QtGui/QMessageBox>
#include <QtCore/QDebug>

#include <Glacier2/Router.h>
#include <Ice/Communicator.h>
#include <Ice/Ice.h>

#include "ui_connectionwidget.h"

namespace Diamesh
{

ConnectionWidget::ConnectionWidget(QWidget *parent) :
    QWidget(parent), _ui(new Ui::ConnectionWidget()), _factory(new SessionHelperFactory())
{
    _ui->setupUi(this);

    QStringList recentServers = _factory->recentServers();
    _ui->serverComboBox->addItems(recentServers);

    QStringList recentUsernames = _factory->recentUsernames();
    _ui->usernameComboBox->addItems(recentUsernames);

    _ui->buttonBox->addButton(tr("Connect"), QDialogButtonBox::AcceptRole);
    connect(_ui->buttonBox, SIGNAL(rejected()), this, SIGNAL(rejected()));

    _ui->rememberCheckBox->setChecked(_factory->updateConnectionHistory());
    _ui->storePasswordCheckBox->setChecked(_factory->storePassword());

    connect(_ui->rememberCheckBox, SIGNAL(clicked(bool)), _factory, SLOT(setUpdateConnectionHistory(bool)));
    connect(_ui->storePasswordCheckBox, SIGNAL(clicked(bool)), _factory, SLOT(setStorePassword(bool)));
}

ConnectionWidget::~ConnectionWidget()
{
    delete _ui;
}

void ConnectionWidget::on_buttonBox_accepted()
{
    QStringList serverParts = _ui->serverComboBox->currentText().split(":", QString::SkipEmptyParts);
    if (!serverParts.isEmpty()) {
        _factory->setRouterHost(serverParts.first());
        if (serverParts.size() == 2) {
            int port = serverParts.value(1).toInt();
            if (port)
                _factory->setRouterPort(port);
        }
    }

    emit connecting();
    QString errorMsg;
    QSharedPointer<SessionHelper> sessionHelper = _factory->connect(_ui->usernameComboBox->currentText(),
                                                                    _ui->passwordLineEdit->text(),
                                                                    errorMsg);

    if (!sessionHelper || !errorMsg.isEmpty()) {
        QMessageBox::critical(this, tr("Error"), errorMsg);
        emit connectFailed();
        return;
    }

    emit connected(sessionHelper);
}

void ConnectionWidget::on_usernameComboBox_editTextChanged(const QString &userName)
{
    _ui->passwordLineEdit->setText(_factory->getPassword(userName));
}

void ConnectionWidget::on_rememberCheckBox_toggled(bool checked)
{
    if (!checked) {
        _ui->storePasswordCheckBox->setChecked(checked);
        _factory->setStorePassword(checked);
        _factory->clearUserNameHistory(_ui->usernameComboBox->currentText());
    }
}

ConnectionDialog::ConnectionDialog(QWidget *parent) :
    QDialog(parent)
{
    QHBoxLayout *layout = new QHBoxLayout();
    ConnectionWidget *widget = new ConnectionWidget();
    layout->addWidget(widget);
    setLayout(layout);

    connect(widget, SIGNAL(connecting()), this, SIGNAL(connecting()));
    connect(widget, SIGNAL(connectFailed()), this, SIGNAL(connectFailed()));
    connect(widget, SIGNAL(connected(QSharedPointer<SessionHelper>)), this, SLOT(_accept(QSharedPointer<SessionHelper>)));
    connect(widget, SIGNAL(rejected()), this, SLOT(reject()));
}

ConnectionDialog::~ConnectionDialog()
{
}

QSharedPointer<SessionHelper> ConnectionDialog::sessionHelper()
{
    return _sessionHelper;
}

void ConnectionDialog::_accept(QSharedPointer<SessionHelper> sessionHelper)
{
    _sessionHelper = sessionHelper;
    accept();
}

}
