/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include "tst_sqluserstorage.h"

using namespace Diamesh;

#define NEW_USER(user, idArg, nameArg, isEnabledArg, hSetIdArg) \
    UserPtr user = new User(); \
    user->id = idArg; \
    user->name = nameArg; \
    user->isEnabled = isEnabledArg; \
    user->hSetId = hSetIdArg; \

#define NEW_GROUP(group, idArg, nameArg, hSetIdArg) \
    GroupPtr group = new Group(); \
    group->id = idArg; \
    group->name = nameArg; \
    group->hSetId = hSetIdArg; \

tst_SqlUserStorage::tst_SqlUserStorage() :
        _db(new SqlDatabase("QSQLITE", "test.db")), _storage(SqlUserStorage::create(_db))
{
}

tst_SqlUserStorage::~tst_SqlUserStorage()
{
    delete _storage;
    delete _db;
}

void tst_SqlUserStorage::getUsers()
{
    NEW_USER(admin, 1, "admin", true, 1);

    // get user with valid id
    UserPtr user = _storage->getUser(admin->id);
    QVERIFY(user != 0);
    QCOMPARE(user->id, admin->id);
    QCOMPARE(user->name, admin->name);
    QCOMPARE(user->isEnabled, admin->isEnabled);
    QCOMPARE(user->hSetId, admin->hSetId);

    // get user with invalid id
    user = _storage->getUser(10);
    QVERIFY(user == 0);
}

void tst_SqlUserStorage::addUsers()
{
    NEW_USER(user, 0, "toto", false, 1);
    int usersCount = _storage->getUsers().size();

    // insert new user
    Ice::Long newId = _storage->addUser(user);
    QVERIFY(newId != 0);
    QCOMPARE(_storage->getUsers().size(), usersCount + 1);

    UserPtr newUser = _storage->getUser(newId);
    QVERIFY(newUser != 0);
    QCOMPARE(newUser->name, user->name);
    QCOMPARE(newUser->isEnabled, user->isEnabled);
    QCOMPARE(newUser->hSetId, user->hSetId);

    // insert duplicate user
    newId = _storage->addUser(user);
    QVERIFY(newId == 0);
}

void tst_SqlUserStorage::updateUsers()
{
    // change user name from "toto" to "tata"
    NEW_USER(user, 2, "tata", true, 1);
    QCOMPARE(_storage->updateUser(user), true);

    UserPtr newUser = _storage->getUser(user->id);
    QCOMPARE(newUser->name, user->name);
    QCOMPARE(newUser->isEnabled, user->isEnabled);
    QCOMPARE(newUser->hSetId, user->hSetId);

    // try to change to an already existing name
    user->name = "admin";
    QCOMPARE(_storage->updateUser(user), false);

    // try to update with an invalid id
    user->id = 10;
    user->name = "toto";
    QCOMPARE(_storage->updateUser(user), false);
}

void tst_SqlUserStorage::getGroups()
{
    NEW_GROUP(group1, 1, "group1", 1);

    // get group with valid id
    GroupPtr group = _storage->getGroup(group1->id);
    QVERIFY(group != 0);
    QCOMPARE(group->id, group1->id);
    QCOMPARE(group->name, group1->name);
    QCOMPARE(group->hSetId, group1->hSetId);

    // get group with invalid id
    group = _storage->getGroup(10);
    QVERIFY(group == 0);
}

void tst_SqlUserStorage::addGroups()
{
    NEW_GROUP(group, 0, "group2", 1);
    int groupsCount = _storage->getGroups().size();

    // insert new group
    Ice::Long newId = _storage->addGroup(group);
    QVERIFY(newId != 0);
    QCOMPARE(_storage->getGroups().size(), groupsCount + 1);

    GroupPtr newGroup = _storage->getGroup(newId);
    QVERIFY(newGroup != 0);
    QCOMPARE(newGroup->name, group->name);
    QCOMPARE(newGroup->hSetId, group->hSetId);

    // insert duplicate group
    newId = _storage->addGroup(group);
    QVERIFY(newId == 0);
}

void tst_SqlUserStorage::updateGroups()
{
    // change group name from "group2" to "group3"
    NEW_GROUP(group, 2, "group3", 1);
    QCOMPARE(_storage->updateGroup(group), true);

    GroupPtr newGroup = _storage->getGroup(group->id);
    QCOMPARE(newGroup->name, group->name);
    QCOMPARE(newGroup->hSetId, group->hSetId);

    // try to change to an already existing name
    group->name = "group1";
    QCOMPARE(_storage->updateGroup(group), false);

    // try to update with an invalid id
    group->id = 10;
    group->name = "group2";
    QCOMPARE(_storage->updateGroup(group), false);
}

void tst_SqlUserStorage::changeMembership()
{
    QCOMPARE(_storage->addUserToGroup(1, 1), true);
    QCOMPARE(_storage->addUserToGroup(1, 2), true);
    QCOMPARE(_storage->addUserToGroup(2, 1), true);
    QCOMPARE(_storage->addUserToGroup(2, 2), true);

    QList<Ice::Long> expected = QList<Ice::Long>() << 1 << 2;
    for(int i = 1 ; i <= 2 ; ++i) {
        QList<Ice::Long> groups = QList<Ice::Long>::fromStdList(_storage->getUser(i)->groupList);
        QList<Ice::Long> users = QList<Ice::Long>::fromStdList(_storage->getGroup(i)->userList);
        QCOMPARE(groups, expected);
        QCOMPARE(users, expected);
    }

    QCOMPARE(_storage->removeUserFromGroup(1, 1), true);
    QCOMPARE(_storage->removeUserFromGroup(1, 2), true);
    QCOMPARE(_storage->removeUserFromGroup(2, 1), true);
    QCOMPARE(_storage->removeUserFromGroup(2, 2), true);

    for(int i = 1 ; i <= 2 ; ++i) {
        QVERIFY(_storage->getUser(i)->groupList.empty());
        QVERIFY(_storage->getGroup(i)->userList.empty());
    }
}

void tst_SqlUserStorage::deleteUsersAndGroups()
{
    // Remove all users and groups
    QCOMPARE(_storage->deleteUser(1), true);
    QCOMPARE(_storage->deleteUser(2), true);
    QCOMPARE(_storage->deleteGroup(1), true);
    QCOMPARE(_storage->deleteGroup(2), true);

    QCOMPARE(_storage->getUsers().size(), 0);
    QCOMPARE(_storage->getGroups().size(), 0);

    // Try to remove users and groups with invalid id;
    QCOMPARE(_storage->deleteUser(10), false);
    QCOMPARE(_storage->deleteGroup(10), false);
}

QTEST_MAIN(tst_SqlUserStorage)
