/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include <diamesh/coreapplication.h>
#include <diamesh/sessionhelper.h>
#include <diamesh/authmanager.h>
#include <diamesh/sessionManager.h>
#include <diamesh/usermanager.h>
#include <QtCore/QDebug>
#include <QtCore/QTimer>
#include <QtCore/QEventLoop>
#include <QtTest/QTest>
#include <QtTest/QSignalSpy>
#include <Ice/Communicator.h>
#include <Ice/Ice.h>
#include <Ice/Initialize.h>
#include <diamesh/diamesh.h>
#include <diamesh/proxies_sessionManager.h>

using namespace Diamesh;

class SessionManagerTest : public QObject
{
    Q_OBJECT
private slots:
    void initTestCase();

    void disconnectUser();
    void testSignals();
    void disableUser();
    void deleteUser();

signals:
    void dummySignal(QString);

private:
    SessionHelperFactory sessionFactory;
    QSharedPointer<SessionHelper> sessionHelper;
    QSharedPointer<SessionHelper> fooSessionHelper;
    SessionManagerQPrx sessionManager;
    UserManagerPrx userManager;
    UserPtr fooUser;
};

bool checkSession(const Glacier2::SessionPrx &session)
{
    // Wait a bit for the disconnection to happen
    QEventLoop l;
    QTimer::singleShot(100, &l, SLOT(quit()));
    l.exec();

    try {
        session->ice_ping();
    } catch (const Ice::ConnectionLostException &) {
        return false;
    }

    return true;
}

void SessionManagerTest::initTestCase()
{
    sessionHelper = sessionFactory.connect("admin", "admin");
    if (!sessionHelper)
        QFAIL("Unable to connect as admin user");

    MasterPrx master = sessionHelper->master();
    sessionManager = SessionManagerQPrx(master->getSessionManager(), sessionHelper->adapter());
    userManager = master->getUserManager();

    fooUser = new User();
    fooUser->name = "foo";
    fooUser->isEnabled = true;
    fooUser->hSetId = 1;
    fooUser->id = userManager->addUser(fooUser);
    if (fooUser->id == 0)
        QFAIL("Unable to add user foo");

    AuthManagerPrx permissionsVerifier = master->getAuthManager();
    permissionsVerifier->setPassword("foo", "bar");
}

void SessionManagerTest::disconnectUser()
{
    fooSessionHelper = sessionFactory.connect("foo", "bar");
    SessionPrx fooSession = fooSessionHelper->session();
    fooSession->ice_ping();
    sessionManager->disconnectUser(fooUser->id);
    QVERIFY2(!checkSession(fooSession), "We should have been disconnected");
}

void SessionManagerTest::testSignals()
{
    // dummy connection because connectNotify isn't called with a QSignalSpy
    connect(sessionManager.data(), SIGNAL(sessionCreated(QString)), SIGNAL(dummySignal(QString)));
    QSignalSpy spy(sessionManager.data(), SIGNAL(sessionCreated(QString)));
    fooSessionHelper = sessionFactory.connect("foo", "bar");
    SessionPrx fooSession = fooSessionHelper->session();
    fooSession->ice_ping();

    QTest::qWait(20);
    QCOMPARE(spy.count(), 1);

    // Same as above
    connect(sessionManager.data(), SIGNAL(sessionDestroyed(QString)), SIGNAL(dummySignal(QString)));
    QSignalSpy spy2(sessionManager.data(), SIGNAL(sessionDestroyed(QString)));
    fooSessionHelper->destroy();

    QTest::qWait(20);
    QCOMPARE(spy2.count(), 1);

    QCOMPARE(spy.takeFirst().first(), spy2.takeFirst().first());
}

void SessionManagerTest::disableUser()
{
    fooSessionHelper = sessionFactory.connect("foo", "bar");
    SessionPrx fooSession = fooSessionHelper->session();
    fooUser->isEnabled = false;
    userManager->updateUser(fooUser);
    QVERIFY2(!checkSession(fooSession), "We should have been disconnected");

    fooSessionHelper = sessionFactory.connect("foo", "bar");
    QVERIFY2(!fooSessionHelper, "We shouldn't be able to connect");
}

void SessionManagerTest::deleteUser()
{
    fooUser->isEnabled = true;
    userManager->updateUser(fooUser);
    fooSessionHelper = sessionFactory.connect("foo", "bar");
    SessionPrx fooSession = fooSessionHelper->session();
    fooSession->ice_ping();

    userManager->deleteUser(fooUser->id);
    QVERIFY2(!checkSession(fooSession), "We should have been disconnected");

    fooSessionHelper = sessionFactory.connect("foo", "bar");
    QVERIFY2(!fooSessionHelper, "We shouldn't be able to connect");
}


int main(int argc, char **argv)
{
    CoreApplication *app = CoreApplication::create(argc, argv, NULL);
    Q_UNUSED(app);

    SessionManagerTest test;
    return QTest::qExec(&test);
}

#include "client.moc"
