/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim common headers
#include "common/canimplementmodel.h"
#include "common/initializer.h"

// Generated hynesim headers
#include "domainentry.h"
#include "switchentry.h"
#include "dynamipsrouterentry.h"

// Qt headers
#include <QHash>
#include <QStringList>

namespace Hns {

void _addWarning(QList<QPair<ImplementModelResult, QString> > & errorList, QString warning) { errorList.append(qMakePair(WARNING, warning)); }
void _addError(QList<QPair<ImplementModelResult, QString> > & errorList, QString error) { errorList.append(qMakePair(ERROR, error)); }

// define here the methods which check for each type of entity whether an entry
// can implement a model entry
ImplementModelResult _canImplementModelDomainEntry(const ::Hns::EntityEntryPtr &entityEntryModel, const ::Hns::EntityEntryPtr &entityEntryImplementation, QList<QPair<ImplementModelResult, QString> > &errorList, bool verbose);
ImplementModelResult _canImplementModelSwitchEntry(const ::Hns::EntityEntryPtr &entityEntryModel, const ::Hns::EntityEntryPtr &entityEntryImplementation, QList<QPair<ImplementModelResult, QString> > &errorList, bool verbose);
ImplementModelResult _canImplementModelDynamipsRouterEntry(const ::Hns::EntityEntryPtr &entityEntryModel, const ::Hns::EntityEntryPtr &entityEntryImplementation, QList<QPair<ImplementModelResult, QString> > &errorList, bool verbose);

typedef ImplementModelResult (*CanImplementModel)(const ::Hns::EntityEntryPtr &, const ::Hns::EntityEntryPtr &, QList<QPair<ImplementModelResult, QString> > &errorList, bool verbose);

HNS_STATIC_HASH_INITIALIZER(_canImplementModelHash, CanImplementModelHash, QString, CanImplementModel)
CanImplementModelHash::CanImplementModelHash()
{
    insert(QString::fromStdString(DomainEntry::ice_staticId()), &_canImplementModelDomainEntry);
    insert(QString::fromStdString(SwitchEntry::ice_staticId()), &_canImplementModelSwitchEntry);
    insert(QString::fromStdString(DynamipsRouterEntry::ice_staticId()), &_canImplementModelDynamipsRouterEntry);
}

ImplementModelResult canImplementModel(const ::Hns::EntityEntryPtr &entityEntryModel, const ::Hns::EntityEntryPtr &entityEntryImplementation, QList<QPair<ImplementModelResult, QString> > &errorList, bool verbose)
{
    if (entityEntryModel->ice_id() != entityEntryImplementation->ice_id()) {
        if (!verbose) {
            _addError(errorList, QObject::tr("Model and entry types are incompatible"));
        } else {
            QString implementationType = QString::fromStdString(entityEntryImplementation->ice_id()).split("::").last();
            QString modelType = QString::fromStdString(entityEntryModel->ice_id()).split("::").last();
            _addError(errorList, QObject::tr("<b>Types are incompatible:</b><br/>the model type is <i>%1</i> and the entry type is <i><font color=\"#FF0000\">%2</font></i>").arg(modelType).arg(implementationType));
        }
        return ERROR;
    }
    QString className = QString::fromStdString(entityEntryModel->ice_id());
    if(_canImplementModelHash()->contains(className)){
        return _canImplementModelHash()->value(className)(entityEntryModel, entityEntryImplementation, errorList, verbose);
    }
    return OK;
}

/*******************************************************************************
* Custom method for DomainEntry
*/
ImplementModelResult _canImplementModelDomainEntry(const ::Hns::EntityEntryPtr &entityEntryModel, const ::Hns::EntityEntryPtr &entityEntryImplementation, QList<QPair<ImplementModelResult, QString> > &errorList, bool verbose)
{
    DomainEntryPtr domainEntryModel = DomainEntryPtr::dynamicCast(entityEntryModel);
    DomainEntryPtr domainEntryImplementation = DomainEntryPtr::dynamicCast(entityEntryImplementation);
    ImplementModelResult result = OK;
    // check for warnings first
    if (domainEntryModel->memory != domainEntryImplementation->memory) {
        if (!verbose)
            _addWarning(errorList, QObject::tr("The amount of memory differs"));
        else
            _addWarning(errorList, QObject::tr("<b>The amount of memory differs:</b><br/>the model has <i>%1 MB</i> and the entry has <i><font color=\"#FFA500\">%2 MB</font></i>").arg(domainEntryModel->memory / 1024).arg(QString::number(domainEntryImplementation->memory / 1024)));
        result = WARNING;
    }
    if (domainEntryModel->vcpu != domainEntryImplementation->vcpu) {
        if (!verbose)
            _addWarning(errorList, QObject::tr("The number of CPUs differs"));
        else
            _addWarning(errorList, QObject::tr("<b>The number of CPUs differs:</b><br/>the model has <i>%1 CPU(s)</i> and the entry has <i><font color=\"#FFA500\">%2 CPU(s)</font></i>").arg(domainEntryModel->vcpu).arg(QString::number(domainEntryImplementation->vcpu)));
        result = WARNING;
    }
    // check for errors
    int domainModelPlugs = domainEntryModel->netcards.size();
    if (domainEntryModel->aminterface != -1) domainModelPlugs--;
    int domainImplementationPlugs = domainEntryImplementation->netcards.size();
    if (domainEntryImplementation->aminterface != -1) domainImplementationPlugs--;
    if (domainModelPlugs != domainImplementationPlugs){
        if (!verbose) {
            _addError(errorList, QObject::tr("The number of pluggable interfaces is not matching"));
        } else {
            _addError(errorList,
                      QObject::tr("<b>The number of pluggable interfaces is not matching:</b><br/>the model has <i>%1</i> and the  entry has <i><font color=\"#FF0000\">%2</font></i>")
                        .arg(QObject::tr("%n interface(s)", "", domainModelPlugs))
                        .arg(QObject::tr("%n interface(s)", "", domainImplementationPlugs)));
        }
        result = ERROR;
    }
    return result;
}

/*******************************************************************************
* Custom method for SwitchEntry
*/
ImplementModelResult _canImplementModelSwitchEntry(const ::Hns::EntityEntryPtr &entityEntryModel, const ::Hns::EntityEntryPtr &entityEntryImplementation, QList<QPair<ImplementModelResult, QString> > &errorList, bool verbose)
{
    SwitchEntryPtr switchEntryModel = SwitchEntryPtr::dynamicCast(entityEntryModel);
    SwitchEntryPtr switchEntryImplementation = SwitchEntryPtr::dynamicCast(entityEntryImplementation);
    if (switchEntryModel->ports != switchEntryImplementation->ports) {
        if (!verbose) {
            _addError(errorList, QObject::tr("The number of ports is not matching"));
        } else {
            _addError(errorList,
                      QObject::tr("<b>The number of ports is not matching:</b><br/>the model has <i>%1</i> and the  entry has <i><font color=\"#FF0000\">%2</font></i>")
                        .arg(QObject::tr("%n port(s)", "", switchEntryModel->ports))
                        .arg(QObject::tr("%n port(s)", "", switchEntryImplementation->ports)));
        }
        return ERROR;
    }
    if (switchEntryModel->vlans != switchEntryImplementation->vlans) {
        _addWarning(errorList, QObject::tr("VLAN configuration differs"));
        return WARNING;
    }

    if (switchEntryModel->portMirroring != switchEntryImplementation->portMirroring) {
        _addWarning(errorList, QObject::tr("PortMirroring configuration differs"));
        return WARNING;
    }

    return OK;
}

/*******************************************************************************
* Custom method for DynamipsRouterEntry
*/
ImplementModelResult _canImplementModelDynamipsRouterEntry(const ::Hns::EntityEntryPtr &entityEntryModel, const ::Hns::EntityEntryPtr &entityEntryImplementation, QList<QPair<ImplementModelResult, QString> > &errorList, bool verbose)
{
    DynamipsRouterEntryPtr dynamipsRouterEntryModel = DynamipsRouterEntryPtr::dynamicCast(entityEntryModel);
    DynamipsRouterEntryPtr dynamipsRouterEntryImplementation = DynamipsRouterEntryPtr::dynamicCast(entityEntryImplementation);
    ImplementModelResult result = OK;

    int plugsModel = 0;
    int plugsImplementation = 0;
    ::Hns::ModulePositionMap::iterator itorPlug = dynamipsRouterEntryModel->modulesMap.begin();
    while (itorPlug != dynamipsRouterEntryModel->modulesMap.end()) {
        plugsModel += itorPlug->second.interfaces;
        itorPlug++;
    }
    itorPlug = dynamipsRouterEntryImplementation->modulesMap.begin();
    while (itorPlug != dynamipsRouterEntryImplementation->modulesMap.end()) {
        plugsImplementation += itorPlug->second.interfaces;
        itorPlug++;
    }

    if (plugsModel != plugsImplementation) {
        if (!verbose) {
            _addError(errorList, QObject::tr("The number of pluggable interfaces is not matching"));
        } else {
            _addError(errorList,
                      QObject::tr("<b>The number of pluggable interfaces is not matching:</b><br/>the model has <i>%1</i> and the  entry has <i><font color=\"#FF0000\">%2</font></i>")
                        .arg(QObject::tr("%n interface(s)", "", plugsModel))
                        .arg(QObject::tr("%n interface(s)", "", plugsImplementation)));
        }
        result = ERROR;
    }
    return result;
}

} // namespace Hns
