/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim common headers
#include "common/tracer.h"
#include "common/entity.h"
#include "common/noderesourceinfo.h"
#include "common/entityinfo.h"
#include "common/snapshot.h"

// QtIce headers
#include <QtIce/operators.h>

// diamesh headers
#include <diamesh/types.h>
#include <diamesh/diamesh.h>

// Qt headers
#include <QMetaType>
#include <QCoreApplication>
#include <QTranslator>
#include <QDebug>
#include <QDir>
#include <QObject>

namespace {

QDataStream &operator<<(QDataStream &out, const Hns::Trace &t)
{
    out << QString::fromStdString(t.category)
        << QString::fromStdString(t.content)
        << QString::fromStdString(t.userName)
        << QString::fromStdString(t.userLogin)
        << qulonglong(t.userId)
        << qulonglong(t.date);
    return out;
}

QDataStream &operator>>(QDataStream &in, Hns::Trace &t)
{
    QString category;
    QString content;
    QString userName;
    QString userLogin;
    qulonglong userId;
    qulonglong date;
    in >> category
       >> content
       >> userName
       >> userLogin
       >> userId >> date;
    t.category = category.toStdString();
    t.content = content.toStdString();
    t.userName = userName.toStdString();
    t.userLogin = userLogin.toStdString();
    t.userId = userId;
    t.date = date;
    return in;
}

QDataStream &operator<<(QDataStream &out, const Hns::Plug &p)
{
    out << p.id
        << p.name
        << p.isPlugged;
    return out;
}

QDataStream &operator>>(QDataStream &in, Hns::Plug &p)
{
    Ice::Long id;
    std::string name;
    bool isPlugged;
    in >> id
       >> name
       >> isPlugged;
    p.id = id;
    p.name = name;
    p.isPlugged = isPlugged;
    return in;
}

QDataStream &operator<<(QDataStream &out, const HnsNode::ResourceInfo &ri)
{
    out << ri.currentResources.cpu
        << ri.currentResources.memory
        << ri.maxResources.cpu
        << ri.maxResources.memory
        << ri.definedEntities
        << ri.hybridNetcardsMap;
    return out;
}

QDataStream &operator>>(QDataStream &in, HnsNode::ResourceInfo &ri)
{
    Ice::Long currentResourcesCpu;
    Ice::Long currentResourcesMemory;
    Ice::Long maxResourcesCpu;
    Ice::Long maxResourcesMemory;
    Ice::Int definedEntities;
    Diamesh::StringStringMap hybridNetcardsMap;
    in >> currentResourcesCpu
       >> currentResourcesMemory
       >> maxResourcesCpu
       >> maxResourcesMemory
       >> definedEntities
       >> hybridNetcardsMap;
    ri.currentResources.cpu = currentResourcesCpu;
    ri.currentResources.memory = currentResourcesMemory;
    ri.maxResources.cpu = maxResourcesCpu;
    ri.maxResources.memory = maxResourcesMemory;
    ri.definedEntities = definedEntities;
    ri.hybridNetcardsMap = hybridNetcardsMap;
    return in;
}

QDataStream &operator<<(QDataStream &out, const Hns::EntityInfoData &e)
{
    out << QString::fromStdString(e.shortName)
        << QString::fromStdString(e.longName)
        << QString::fromStdString(e.type)
        << QString::fromStdString(e.subtype);
    return out;
}

QDataStream &operator>>(QDataStream &in, Hns::EntityInfoData &e)
{
    QString shortName;
    QString longName;
    QString type;
    QString subType;
    in >> shortName
       >> longName
       >> type
       >> subType;
    e.shortName = shortName.toStdString();
    e.longName = longName.toStdString();
    e.type = type.toStdString();
    e.subtype = subType.toStdString();
    return in;
}

QDataStream &operator<<(QDataStream &out, const Hns::Snapshot &e)
{
    out << QString::fromStdString(e.creationDateTime)
        << QString::fromStdString(e.description)
        << QString::fromStdString(e.name)
        << QString::fromStdString(e.parent);
    return out;
}

QDataStream &operator>>(QDataStream &in, Hns::Snapshot &e)
{
    QString creationDateTime;
    QString description;
    QString name;
    QString parent;
    in >> creationDateTime
       >> description
       >> name
       >> parent;
    e.creationDateTime = creationDateTime.toStdString();
    e.description = description.toStdString();
    e.name = name.toStdString();
    e.parent = parent.toStdString();

    return in;
}

QDataStream &operator<<(QDataStream &out, const Hns::MemoryDumpInfo &m)
{
    out << QString::fromStdString(m.fileName)
        << QString::fromStdString(m.name)
        << m.dateTime
        << m.size;
    return out;
}

QDataStream &operator>>(QDataStream &in, Hns::MemoryDumpInfo &m)
{
    QString fileName;
    QString name;
    Ice::Long dateTime;
    Ice::Long size;
    in >> fileName
       >> name
       >> dateTime
       >> size;
    m.fileName = fileName.toStdString();
    m.name = name.toStdString();
    m.dateTime = dateTime;
    m.size = size;

    return in;
}

} // Unnamed namespace

namespace Hns {

void loadTranslation(QString filePrefix, bool loadExternal)
{
    QString translationPath = qApp->applicationDirPath() + "/translations";
    if (!QFile::exists(translationPath))
        translationPath = "/usr/share/hynesim/translations";

    QTranslator *translator = new QTranslator();

    QString translationBaseName = translationPath + QDir::separator() + filePrefix;
    QString translationFile = translationBaseName;
    QString language = Diamesh::instance()->applicationLanguage();
    translationFile += language;
    if (!translator->load(translationFile)) {
        if (language.left(2) != "en")
            qCritical() << QObject::tr("The translation file %1 can not be loaded").arg(translationFile);
    }
    else
        QCoreApplication::instance()->installTranslator(translator);
    // load external translations
    if (loadExternal)
        Diamesh::instance()->loadExternalTranslations(translationBaseName + "external.xml");
}

int getNearestMultipleOf(int value, int multiple)
{
    if (multiple == 0)
        return 0;

    int newValue = value;
    int remainder = value % multiple;
    if (remainder != 0) {
        if (remainder < (multiple / 2))
            newValue = value - remainder;
        else
            newValue = value + (multiple - remainder);
    }
    return newValue;
}

class RegisterMetaTypes
{
public:
    RegisterMetaTypes()
    {
        qRegisterMetaType<Hns::Trace>("Hns::Trace");
        qRegisterMetaTypeStreamOperators<Hns::Trace>("Hns::Trace");
        qRegisterMetaType<Hns::Plug>("Hns::Plug");
        qRegisterMetaTypeStreamOperators<Hns::Plug>("Hns::Plug");
        qRegisterMetaType< ::Hns::PlugMap >("::Hns::PlugMap");
        qRegisterMetaTypeStreamOperators< ::Hns::PlugMap >("::Hns::PlugMap");
        qRegisterMetaType<HnsNode::ResourceInfo>("HnsNode::ResourceInfo");
        qRegisterMetaTypeStreamOperators<HnsNode::ResourceInfo>("HnsNode::ResourceInfo");
        qRegisterMetaType<Hns::EntityInfoData>("::Hns::EntityInfoData");
        qRegisterMetaTypeStreamOperators< Hns::EntityInfoData>("::Hns::EntityInfoData");
        qRegisterMetaType<Hns::Snapshot>("::Hns::Snapshot");
        qRegisterMetaTypeStreamOperators<Hns::Snapshot>("::Hns::Snapshot");
        qRegisterMetaType<Hns::MemoryDumpInfo>("::Hns::MemoryDumpInfo");
        qRegisterMetaTypeStreamOperators<Hns::MemoryDumpInfo>("::Hns::MemoryDumpInfo");
    }
} _register;

} // namespace Hns
