/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim common headers
#include "common/jobI.h"

// Qt headers
#include <QDebug>
#include <QEventLoop>
#include <QTimer>
#include <QThread>
#include <QCoreApplication>

namespace Hns {

JobI::JobI()
    : _id(0)
    , _progress(0.0f)
    , _adapter(0)
    , _executionThread(new QThread())
    , _cancelled(false)
    , _running(false)
{
    QObject::connect(this, SIGNAL(terminateExecutionThreadRequested()), SLOT(_terminateExecutionThread()));
}

JobI::~JobI()
{
    delete _executionThread;
}

void JobI::execute(const ::Ice::Current &current)
{
    if (!_canBeExecuted())
        return;
    QWriteLocker locker(&_stateLock);
    Q_UNUSED(current)
    QMetaObject::invokeMethod(this, "_moveToExecutionThread");
    QObject::connect(_executionThread, SIGNAL(started()), this, SLOT(_run()));
    _running = true;
    _executionThread->start();
}

bool JobI::_canBeExecuted()
{
    QReadLocker progressLocker(&_progressLock);
    QReadLocker stateLocker(&_stateLock);
    bool result = true;
    if (_running) {
        result = false;
        qWarning() << tr("Cannot execute an already running job");
    } else if (_cancelled) {
        result = false;
        qWarning() << tr("Cannot execute a cancelled job");
    } else if (_progress >= 1.0) {
        result = false;
        qWarning() << tr("Cannot execute an already finished job");
    }
    return result;
}

void JobI::_moveToExecutionThread()
{
    this->moveToThread(_executionThread);
}

void JobI::_run()
{
    QEventLoop l;
    QTimer::singleShot(3000, &l, SLOT(quit()));
    l.exec();
}

double JobI::getProgress(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QReadLocker locker(&_progressLock);
    return _progress;
}

void JobI::_updateProgress(double progress)
{
    QWriteLocker progressLocker(&_progressLock);
    QWriteLocker stateLocker(&_stateLock);
    if (_progress != progress) {
        _progress = progress;
        qDebug() << tr("progress of job %2 updated to %1").arg(_progress*100).arg(_id);
        emit progressChanged(_id, _progress);
        if (_progress >= 1.0) {
            _running = false;
            emit finished(_id);
            _terminateExecutionThread();
        }
    }
}

void JobI::_setAdapter(Ice::ObjectAdapterPtr adapter)
{
    _adapter = adapter;
}

QThread *JobI::executionThread()
{
    return _executionThread;
}

void JobI::_terminateExecutionThread()
{
    if (thread() != qApp->thread()) {
        moveToThread(qApp->thread());
        _executionThread->quit();
    }
}

bool JobI::cancel(const ::Ice::Current &current)
{
    QWriteLocker locker(&_stateLock);
    Q_UNUSED(current)
    _cancelled = true;
    emit cancelled(_id);
    emit terminateExecutionThreadRequested();
    return true;
}

bool JobI::isRunning(const ::Ice::Current &current)
{
    QReadLocker locker(&_stateLock);
    Q_UNUSED(current)
    return _running && !_cancelled;
}

void JobI::destroy(const Ice::Current &current)
{
    Q_UNUSED(current)
    emit terminateExecutionThreadRequested();
    if (_adapter) {
        emit requestDeletion(_adapter, _identity);
    } else { //requestDeletion calls deleteLater()
        deleteLater();
    }
}

void JobI::setId(int id)
{
    _id = id;
}

int JobI::id()
{
    return _id;
}

void JobI::setIdentity(Ice::Identity identity)
{
    _identity = identity;
}

} // namespace Hns
