#ifndef POOLHANDLER_H
#define POOLHANDLER_H

// Qt headers
#include <QDebug>
#include <QList>
#include <QReadWriteLock>
#include <QSemaphore>

namespace Hns {
/*!
 *\class PoolHandler
 *\brief This class allows you to manage Pools with T* items;
 */
template <class T>
class PoolHandler
{
public:
    PoolHandler(double nmax);
    T getOrCreate();
    void release(T item);
    double getSizeLeft();
    void preDestroy();

protected:
    virtual T create() = 0;
    virtual void closeMethod(T item) = 0;

private:
    double _nmax;
    double _nitemsCreated;
    QList<T> _pool;
    QReadWriteLock _locker;
    QSemaphore _semaphore;

};


template <class T>
PoolHandler<T>::PoolHandler(double nmax):
    _nmax (nmax)
  , _semaphore(nmax)
{
    _nitemsCreated = 0;
}
template <class T>
void PoolHandler<T>::preDestroy()
{
    foreach (T item, _pool) {
        closeMethod(item);
    }
}

/*! \fn template <class T> T *PoolHandler<T>::getOrCreate()
*This method will return an item of the pool if any or it will create a new one
*if we didn't reach the nmax given at the init.
*If it is not possible to return you an item, we return NULL
*/
template <class T>
T PoolHandler<T>::getOrCreate()
{
    T ret = NULL;
    _semaphore.acquire();
    if (getenv("HNS_DEBUG"))
        qDebug() << Q_FUNC_INFO << QObject::tr("Acquire Semaphore, %1 left").arg(_semaphore.available());
    _locker.lockForWrite();
    if (!_pool.isEmpty()) {
        /*If there is some items in the pool*/
        if (getenv("HNS_DEBUG"))
            qDebug() << Q_FUNC_INFO << QObject::tr("Taking item from pool");
        ret = _pool.takeFirst();
    } else {
        /*If there is no items in the pool and we can create some
        *The item is not pushed in the pool, it will when it will be released
        */
        if (getenv("HNS_DEBUG"))
            qDebug() << Q_FUNC_INFO << QObject::tr("Creating an item");
        _nitemsCreated++;
        ret =  create();
    }
    _locker.unlock();
    return ret;
}

/*! \fn template <class T> void PoolHandler<T>::release(T *item)
*This method will put @item into the pool if we we didn't already reached nmax
*given at the init
*/
template <class T>
void PoolHandler<T>::release(T item)
{
    _locker.lockForWrite();
    /*We put the item back in the list
    *We check the number of items because we don't want the list to oversize nmax
    *in the case the item was created outside of the PoolHandler
    */
    if (_pool.size() < _nmax) {
        if (getenv("HNS_DEBUG"))
            qDebug() << Q_FUNC_INFO << QObject::tr("Releasing an item and appending it to the pool");
        _semaphore.release();
        _pool.append(item);
    }
    _locker.unlock();
}

/*!\fn double PoolHandler::getSizeLeft()
*The number of items you can still create
*/
template <class T>
double PoolHandler<T>::getSizeLeft()
{
    _locker.lockForRead();
    return _nmax - _nitemsCreated;
    _locker.unlock();
}

} // namespace Hns
#endif // POOLHANDLER_H
