#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2013 diateam. All rights reserved.
#
# This file is part of hynesim.
#
# Hynesim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Hynesim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Usage : sqlmigration.py -c master.ini --datapath=/usr/share/hynesim/data/scripts/sqlMigration

import imp
import os
import re
import shutil
import sqlite3
import sys
import time

from optparse import OptionParser

from HnsSettings import HnsSettings
from HnsLogger import HnsLogger

# Converts a string with underscores to camel case
def camelize(string):
    return re.sub(r"(?:^|_)(.)", lambda matchobj: matchobj.group(0)[-1].upper(), string)
    
def dir_list_folder(head_dir):
    dirList = []
    result = os.listdir(head_dir)
    result.sort()
    for fn in result:
        dirfile = os.path.join(head_dir, fn)
        if os.path.isdir(dirfile):
            dirList.append(dirfile)
            dirList += dir_list_folder(dirfile)
    return dirList

class SqlMigrationError(Exception):
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return repr(self.value)

class SqlMigration(object):
    def __init__(self):
        self.master_config = "/etc/hynesim/hynesim-master.ini"
        self.data_path = "/usr/share/hynesim/scripts/sqlMigration"
        self.handlers = []
        self.list_db_keys = []
        self.db_name_path_dict = dict()
        self.use_syslog = False
        self.do_check = False

        self.parse_options()

    def create_handlers(self):
        self.parse_master_config()
        directories = dir_list_folder(self.data_path)
        for directory in directories:
            self.handlers.append(SqlMigrationHandler(directory, self.db_name_path_dict))

    def migrate(self):
        self.create_handlers()
        for handler in self.handlers:
            handler.migrate()

    def check(self):
        self.create_handlers()
        for handler in self.handlers:
            if len(handler.migrations) != 0:
                HnsLogger.log_error("There are still migrations to apply")
                exit(1)
        HnsLogger.log_msg("No migration to apply")
        exit(0)

    def parse_options(self):
        parser = OptionParser()
        parser.add_option("-c", "--masterconfig", dest="master_config",
                help = "master configuration file (default is " + self.master_config +")")
        parser.add_option("", "--datapath", dest="data_path",
                help = "SQL migrations data folder (default is " + self.data_path +")")
        parser.add_option("", "--use-syslog", dest="use_syslog", action="store_true", default = False,
                help="Redirect ouput to syslog")
        parser.add_option("", "--check", dest="check", action="store_true", default = False,
                help="Check available migrations")
        try:
            (options, args) = parser.parse_args()
        except:
            exit(2)
            
        HnsLogger.loadLogger("hynesim-master", options.use_syslog)

        if options.master_config is not None:
            self.master_config = os.path.abspath(options.master_config)
        if options.data_path is not None:
            self.data_path = os.path.abspath(options.data_path)

        if not os.path.isfile(self.master_config):
            HnsLogger.log_error("Master configuration file " + self.master_config + " was not found")
            exit(2)
        if not os.path.isdir(self.data_path):
            HnsLogger.log_error("SQL migrations data folder " + self.data_path + " was not found")
            exit(2)

        self.do_check = options.check

    def register_migrations_dir(self, config_key, migrations_dir):
        abs_migrations_dir = os.path.join(self.data_path, migrations_dir)
        if os.path.isdir(abs_migrations_dir):
            self.keys_migrationsdirs_dict[config_key] = abs_migrations_dir
    
    def register_db_key(self, db_name):
        self.list_db_keys.append(db_name)

    def parse_master_config(self):
        settings = HnsSettings(self.master_config)
        master_config_dirname = os.path.dirname(self.master_config)

        try:
            for key in self.list_db_keys:
                if settings.has_key(key):
                    db_path = settings.get(key)
                    if not os.path.isabs(db_path):
                        db_path = os.path.normpath(os.path.join(master_config_dirname, db_path))
                    if not os.path.isfile(db_path):
                        raise SqlMigrationError("The database %s doesn't exist" % db_path)
                    self.db_name_path_dict[key] = db_path
                else:
                    raise SqlMigrationError("Unable to find key %s in master configuration file" % key)

        except SqlMigrationError as error:
            HnsLogger.log_error(error.value)
            exit(1)


class SqlMigrationHandler(object):

    MIGRATION_TABLE = "migration"
    MIGRATION_NAME_COLUMN = "name"

    def __init__(self, migrations_path, db_name_path_dict):
        self.migrations_path = migrations_path
        self.db_name_path_dict = db_name_path_dict

        self.create_table_if_needed()
        self.migrations = self.find_migrations()

    def create_table_if_needed(self):
        diamesh_db_path = self.db_name_path_dict['Diamesh/defaultDb']
        conn = sqlite3.connect(diamesh_db_path)
        if not conn.execute("SELECT name FROM sqlite_master WHERE type = 'table' AND name = '%s'" % SqlMigrationHandler.MIGRATION_TABLE).fetchone():
            HnsLogger.log_msg("Adding migration table to %s database" % diamesh_db_path)
            conn.execute("CREATE TABLE %s (%s TEXT)" % (SqlMigrationHandler.MIGRATION_TABLE, SqlMigrationHandler.MIGRATION_NAME_COLUMN))
            conn.commit()
        conn.close()

    # returns the list of names without extensions of all migration files
    def find_available_migrations(self):
        files = os.listdir(self.migrations_path)
        regex = re.compile(r'^([0-9]{14}_.*)\.py$')
        return [m.group(1) for f in files for m in [regex.search(f)] if m]

    def find_applied_migrations(self):
        diamesh_db_path = self.db_name_path_dict['Diamesh/defaultDb']
        conn = sqlite3.connect(diamesh_db_path)
        versions = [row[0] for row in conn.execute("SELECT %s FROM %s" % (SqlMigrationHandler.MIGRATION_NAME_COLUMN, SqlMigrationHandler.MIGRATION_TABLE))]
        conn.close()
        return versions

    def find_migrations(self):
        available_migrations = self.find_available_migrations()
        applied_migrations = self.find_applied_migrations()
        migrations = list(set(available_migrations) - set(applied_migrations))
        migrations.sort()
        HnsLogger.log_msg("Available : %s" % available_migrations)
        HnsLogger.log_msg("Applied : %s" % applied_migrations)
        HnsLogger.log_msg("Migrations : %s" % migrations)
        return migrations

    def backup_db(self):
        for (key, db_path) in self.db_name_path_dict.items():
            db_dir = os.path.dirname(db_path)
            backup_dir = os.path.join(db_dir, "db_backup")
            if not os.path.isdir(backup_dir):
                os.mkdir(backup_dir)
            backup_basename = "%s_%s" % (time.strftime("%Y%m%d%H%M%S", time.localtime()), os.path.basename(db_path))
            backup_filename = os.path.join(backup_dir, backup_basename)
            shutil.copy(db_path, backup_filename)

    def load_migration_class(self, module_name):
        module_path = os.path.join(self.migrations_path, module_name + ".py")

        try:
            file_obj, path_name, description = imp.find_module(module_name, [self.migrations_path])
            module = imp.load_module(module_name, file_obj, path_name, description)
        except ImportError as import_error:
            HnsLogger.log_msg("Failed to import migration file %s" % module_path)
        finally:
            # Close the imported module's file object
            if file_obj:
                file_obj.close()

        migration = None
        if module:
            try:
                class_name = camelize(module_name.split('_', 1)[1]) # Camelize what is after the date
                class_ = getattr(module, class_name)
                migration = class_()
            except AttributeError as attr_error:
                HnsLogger.log_msg("AttributeError: %s (in file %s)" % (attr_error, module_path))

        return migration

    def migrate(self):
        if len(self.migrations):
            HnsLogger.log_msg("Migrating from %s" % self.migrations_path)
            self.backup_db()
            for module_name in self.migrations:
                migration = self.load_migration_class(module_name)
                if migration:
                    migration.name = module_name
                    migration.db_name_path_dict = self.db_name_path_dict
                    try:
                        migration.migrate()
                    except Exception as error:
                        raise SqlMigrationError("Migration %s failed: %s" % (module_name, str(error)))

class Migration(object):

    def update_migration_table(self):
        diamesh_db_path = self.db_name_path_dict['Diamesh/defaultDb']
        conn = sqlite3.connect(diamesh_db_path)
        conn.execute("INSERT INTO %s(%s) VALUES(?)" % (SqlMigrationHandler.MIGRATION_TABLE, SqlMigrationHandler.MIGRATION_NAME_COLUMN), (self.name,))
        conn.commit()
        conn.close()

    def migrate(self):
        HnsLogger.log_msg("Running migration %s" % self.name)
        self.migrate_impl()
        self.update_migration_table()

if __name__ == '__main__':
    sqlMigration = SqlMigration()

    sqlMigration.register_db_key('Diamesh/defaultDb')
    sqlMigration.register_db_key('Diamesh/authDBName')
    sqlMigration.register_db_key('Hynesim/catalog/dbName')

    if sqlMigration.do_check is True:
        sqlMigration.check()

    try:
        sqlMigration.migrate()
    except SqlMigrationError as error:
        HnsLogger.log_error("Migration error : %s" % error)
