/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "hnvmainwindow.h"
#include "ui_hnvmainwindow.h"
#include "ressources.h"
#include "usersmodel.h"
#include "groupsmodel.h"
#include "sessionsmodel.h"
#include "applicationsettingsdialog.h"
#include "hyneviewsession.h"
#include "connectionstatuswidget.h"
#include "welcomemodewidget.h"
#include "simulationmodewidget.h"
#include "usermanagermodewidget.h"
#include "topologywidget.h"
#include "topologycatalogmodel.h"
#include "entitycatalogmodel.h"
#include "remotemanager.h"
#include "platformmonitormodel.h"
#include "popupstyle.h"
#include "importdialog.h"

// hynesim common headers
#include "common/pluginmanager.h"
#include "common/config-prefix.h"

// hynesim hnsclient headers
#include <libhnsclient/entityentrycatalog.h>
#include <libhnsclient/topologyentrycatalog.h>
#include <libhnsclient/topologymanager.h>
#include <libhnsclient/session.h>
#include <libhnsclient/tracermanager.h>

// diamesh headers
#include <diamesh/connectionwidget.h>
#include <diamesh/diamesh.h>

// Qt headers
#include <QDebug>
#include <QSettings>
#include <QMimeData>
#include <QMessageBox>
#include <QClipboard>
#include <QCloseEvent>

namespace Hnv {

/*******************************************************************************
* HnvMainWindow
*******************************************************************************/
HnvMainWindow *HnvMainWindow::_instance = NULL;

QtMsgHandler HnvMainWindow::_oldMsgHandler = NULL;

HnvMainWindow *HnvMainWindow::create(QWidget *parent)
{
    // Two steps creation in order to make available the main window to children
    // widgets.
    if (!_instance) {
        _instance = new HnvMainWindow(parent);
        _instance->_initWindow();
    }
    return _instance;
}

HnvMainWindow *HnvMainWindow::instance()
{
    return _instance;
}

void HnvMainWindow::messageHandler(QtMsgType type, const char *msg)
{
    if(HnvMainWindow::_oldMsgHandler)
        HnvMainWindow::_oldMsgHandler(type, msg);
    else {
        fprintf(stderr, "%s\n", msg);
        fflush(stderr);
    }
    HnvMainWindow *window = HnvMainWindow::instance();
    if (window)
        window->addLogMessage(LogMessageItem::MessageType(type), msg);
}

HnvMainWindow::~HnvMainWindow()
{
    delete _topologyCatalogModel;
    delete _entityCatalogModel;
    delete _usersModel;
    delete _groupsModel;
    delete ui;
}

void HnvMainWindow::registerModeWidget(ModeWidget *widget, bool isBottomWidget)
{
    connect(widget, SIGNAL(logMessageWidgetClosed()), this, SLOT(_closeLogMessageWidget()));
    ui->modeDisplayWidget->addWidget(widget);
    ui->modeBrowser->registerModeWidget(widget, isBottomWidget);
}

void HnvMainWindow::addFileMenuAction(QList<QAction *> actionList)
{
    _populateMenu(ui->menu_File, actionList);
}

HyneviewSession *HnvMainWindow::session()
{
    return _session;
}

QString HnvMainWindow::generatePassword()
{
    QSettings settings;
    settings.beginGroup(Constants::PASSWORD_SETTINGS_GROUP);
    QString allowedChars = settings.value(Constants::PASSWORD_ALLOWED_CHARACTERS, QVariant(Constants::PASSWORD_ALLOWED_CHARACTERS_LIST)).toString();
    int passwordLength = settings.value(Constants::PASSWORD_LENGTH_KEY, QVariant(Constants::PASSWORD_LENGTH)).toInt();
    int size = allowedChars.length();
    QString password;
    int rndInt;
    // Randomise the seed
    qsrand(QTime::currentTime().msec());
    for(int i=0; i < passwordLength; i++) {
        rndInt = qrand() % size;
        password += allowedChars[rndInt];
    }
    return password;
}

QString HnvMainWindow::aboutMessage(bool displayMasterInfo)
{
    QString message = Diamesh::Instance::extTr("about").arg(qApp->applicationName()).arg(qApp->applicationVersion());

    if (displayMasterInfo && _session) {
        Diamesh::VersionInfo masterInfo = _session->masterVersionInfo();
        QString version = QString::fromStdString(masterInfo.version);
        QString versionDescription = QString::fromStdString(masterInfo.versionDescription);
        QString revision = QString::fromStdString(masterInfo.revision);

        QString appVersion = version;
        if (!versionDescription.isEmpty())
            appVersion.append(QString(" %1").arg(versionDescription));
        if (!revision.isEmpty())
            appVersion.append(QString(" (rev %1)").arg(revision));

        message += tr("<br/><h2>Master information</h2>"
                      "<p><b>Server name:</b> %1</p>"
                      "<p><b>Version:</b> %2</p>")
                .arg(_session->masterName())
                .arg(appVersion);
    }
    return message;
}

HnsClient::EntityEntryCatalog *HnvMainWindow::entityCatalog()
{
    return _session->entityEntryCatalog();
}

HnsClient::TopologyEntryCatalog *HnvMainWindow::topologyCatalog()
{
    return _session->topologyEntryCatalog();
}

HnsClient::TopologyManager *HnvMainWindow::topologyManager()
{
    return _session->topologyManager();
}

EntityCatalogModel *HnvMainWindow::entityCatalogModel()
{
    return _entityCatalogModel;
}

EntityInfoModel *HnvMainWindow::entityInfoModel()
{
    return _entityInfoModel;
}

LogMessageModel *HnvMainWindow::logMessageModel()
{
    return _logMessageModel;
}

TopologyCatalogModel *HnvMainWindow::topologyCatalogModel()
{
    return _topologyCatalogModel;
}

PlatformMonitorModel *HnvMainWindow::platformMonitorModel()
{
    return _platforMonitorModel;
}

UsersModel *HnvMainWindow::usersModel()
{
    return _usersModel;
}

GroupsModel *HnvMainWindow::groupsModel()
{
    return _groupsModel;
}

SessionsModel *HnvMainWindow::sessionsModel()
{
    return _sessionsModel;
}

ApplicationSettingsDialog *HnvMainWindow::applicationSettingsDialog()
{
    if (!_applicationSettings)
        _applicationSettings = new ApplicationSettingsDialog(this);
    return _applicationSettings;
}

Hns::PluginManager *HnvMainWindow::pluginManager()
{
    return _pluginManager;
}
/*******************************************************************************
* HnvMainWindow - Public slots
*/
void HnvMainWindow::addLogMessage(LogMessageItem::MessageType type, const QString & msg, QDateTime dateTime)
{
    emit newLogMessage(type, msg, dateTime);
}

void HnvMainWindow::connectTriggered()
{
    if (!_canTerminateCurrentSession())
        return;
    _disconnectSession(true);
    Diamesh::ConnectionDialog dialog(this);
    dialog.setWindowTitle(tr("hyneview - Login window"));
    connect(&dialog, SIGNAL(connecting()), this, SLOT(_connectStartedSlot()));
    connect(&dialog, SIGNAL(connectFailed()), this, SLOT(_connectEndedSlot()));
    if (dialog.exec() == QDialog::Accepted){
        _connectEndedSlot();
        Diamesh::SessionHelperFactory factory;
        emit enableQuickConnect(factory.storePassword());
        _connectionEstablished(dialog.sessionHelper());
    }
}

void HnvMainWindow::quickConnectTriggered()
{
    if (!_canTerminateCurrentSession())
        return;
    _disconnectSession(true);
    Diamesh::SessionHelperFactory factory;
    QString error;
    _connectStartedSlot();
    QSharedPointer<Diamesh::SessionHelper> helper = factory.connect(error);

    if (helper) {
        _connectionEstablished(helper);
        _connectEndedSlot();
    } else {
        _connectEndedSlot();
        QMessageBox::critical(this, tr("Connection Error"), error);
        qCritical() << error;
    }
}

void HnvMainWindow::remoteProcessStarted(QString title, QString message)
{
    Q_UNUSED(title);
    qDebug() << message;
    _runningRemoteProcess++;
}

void HnvMainWindow::remoteProcessEnded()
{
    _runningRemoteProcess--;
}

void HnvMainWindow::zoomValueChangedSlot(int value)
{
    ui->statusBar->setZoomValue(value);
}

void HnvMainWindow::cursorPositionChangedSlot(int x, int y, bool inScene)
{
    ui->statusBar->displayCursorPosition(x, y, inScene);
}

void HnvMainWindow::clearLabelCoord()
{
    ui->statusBar->clearLabelCoord();
}

void HnvMainWindow::loadingStarted(QString message)
{
    _loadingCount++;
    if (_waitingBox == 0) {
        _waitingBox = new QMessageBox(this);
        _waitingBox->setWindowTitle(qAppName());
        _waitingBox->setIcon(QMessageBox::Information);
        _waitingBox->setStandardButtons(QMessageBox::NoButton);
        _waitingBox->setText(tr("Loading data..."));
        _waitingBox->setModal(true);
        _waitingBox->show();
    }
    _waitingBox->setInformativeText(tr("Loading: %1").arg(message));
}

void HnvMainWindow::loadingEnded(QString message)
{
    if (_waitingBox)
        _waitingBox->setInformativeText(tr("Completed: %1").arg(message));
    _loadingCount--;
    if (_loadingCount == 0) {
        _waitingBox->hide();
        _waitingBox->deleteLater();
        _waitingBox = 0;
    }
}

void HnvMainWindow::entityCatalogLoadingSlot(int transactionCount, bool initialLoading)
{
    if (initialLoading) {
        if (_dataLoading && transactionCount > 0) {
            if (_waitingBox)
                _waitingBox->setInformativeText(tr("%n entities left", "", transactionCount));
            return;
        } else if (transactionCount == 0) {
            _dataLoading = false;
            loadingEnded(tr("Entity catalog"));
        } else {
            _dataLoading = true;
            loadingStarted(tr("Entity catalog"));
        }
    }
}

/*******************************************************************************
* HnvMainWindow - Protected
*/
void HnvMainWindow::_initWindow()
{
    ui->setupUi(this);
    setWindowTitle(tr("hyneview %1 - hynesim GUI").arg(qApp->applicationVersion()));
    setWindowIcon(QIcon(Ressources::themedPath(Constants::ICON_WELCOME)));
    HnvMainWindow::_oldMsgHandler = qInstallMsgHandler(&HnvMainWindow::messageHandler);
    QSettings settings;
    settings.beginGroup(QString(Constants::MAIN_WINDOW_SETTINGS));
    if (settings.contains(Constants::MAIN_WINDOW_GEOMETRY)){
        restoreGeometry(settings.value(Constants::MAIN_WINDOW_GEOMETRY).toByteArray());
        settings.endGroup();
    }

    _pluginManager = new Hns::PluginManager(HYNEVIEW_PLUGINS_DIR, this);
    _pluginManager->buildPluginList();


    addFileMenuAction(ui->modeBrowser->fileMenuActions());

    WelcomeModeWidget * welcomeWidget = new WelcomeModeWidget(this);
    registerModeWidget(welcomeWidget);
    connect(welcomeWidget, SIGNAL(connectTriggered()), this, SLOT(connectTriggered()));
    connect(welcomeWidget, SIGNAL(quickConnectTriggered()), this, SLOT(quickConnectTriggered()));
    connect(this, SIGNAL(sessionConnected(HyneviewSession*)), welcomeWidget, SLOT(connectedSlot(HyneviewSession*)));
    connect(this, SIGNAL(sessionDisconnected()), welcomeWidget, SLOT(disconnectedSlot()));

    RemoteAccess::RemoteManager::create(this);

    // Users & group models
    _usersModel = new UsersModel(this);
    _groupsModel = new GroupsModel(this);
    _sessionsModel = new SessionsModel(this);

    _simulationWidget = SimulationModeWidget::create(this);
    _simulationWidget->initWidget();
    registerModeWidget(_simulationWidget);

    connect(welcomeWidget, SIGNAL(newSessionTriggered()), _simulationWidget->topologyWidget()->getActionNew(), SLOT(trigger()));
    connect(welcomeWidget, SIGNAL(openSessionTriggered()), _simulationWidget->topologyWidget()->getActionOpen(), SLOT(trigger()));

    QList<ModePlugin *> modePluginList = _pluginManager->pluginListByType<ModePlugin>();
    foreach(ModePlugin *modePlugin, modePluginList) {
        ModeWidget * widget = modePlugin->modeWidget(this);
        registerModeWidget(widget);
    }

    UserManagerModeWidget * userManagerWidget = new UserManagerModeWidget(this);
    registerModeWidget(userManagerWidget);

    connect(ui->modeBrowser, SIGNAL(modeChanged(QWidget*)), this, SLOT(_modeChanged(QWidget*)));
    connect(ui->modeBrowser, SIGNAL(connectTriggered()), this, SLOT(connectTriggered()));
    connect(ui->modeBrowser, SIGNAL(quickConnectTriggered()), this, SLOT(quickConnectTriggered()));
    connect(ui->modeBrowser, SIGNAL(disconnectedTriggered()), this, SLOT(_disconnectTriggered()));
    connect(ui->modeBrowser, SIGNAL(sidebarEnabled(bool, bool)), ui->statusBar, SLOT(enableSideBar(bool, bool)));
    connect(ui->modeBrowser, SIGNAL(rightSidebarEnabled(bool, bool)), ui->statusBar, SLOT(enableRightSideBar(bool, bool)));
    connect(this, SIGNAL(enableQuickConnect(bool)), ui->modeBrowser, SLOT(enableQuickConnect(bool)));
    connect(this,SIGNAL(enableQuickConnect(bool)), welcomeWidget, SLOT(enableQuickConnect(bool)));

    connect(ui->statusBar, SIGNAL(toggleSideBar()), this, SLOT(_toggleSideBar()));
    connect(ui->statusBar, SIGNAL(toggleRightSideBar()), this, SLOT(_toggleRightSideBar()));
    connect(ui->statusBar, SIGNAL(toggleLogMessageWidget(bool)), this, SLOT(_toggleLogMessageWidget(bool)));
    connect(ui->statusBar, SIGNAL(zoomValueChanged(int)), this, SIGNAL(zoomValueChanged(int)));

    connect(this, SIGNAL(sessionConnected(HyneviewSession*)), ui->modeBrowser, SLOT(setConnected(HyneviewSession*)));
    connect(this, SIGNAL(sessionDisconnected()), ui->modeBrowser, SLOT(setDisconnected()));

    _populateMenu(ui->menu_View, ui->statusBar->statusBarActions());
    _populateMenu(ui->menu_View, ui->modeBrowser->modeMenuActions());

    ui->modeBrowser->switchToMode(0);
    ui->modeDisplayWidget->setFocus();
    ui->action_Quit->setIcon(QIcon(Ressources::themedPath(Constants::ICON_EXIT)));
    ui->action_Import->setIcon(QIcon(Ressources::themedPath(Constants::ICON_IMPORT)));
    settings.beginGroup(Constants::LOG_MESSAGE_SETTINGS);
    if (settings.contains(Constants::LOG_MESSAGE_IS_DISPLAY)){
        _logMessageDisplayed = settings.value(Constants::LOG_MESSAGE_IS_DISPLAY,QVariant(false)).toBool();
        settings.endGroup();
    } else {
        _logMessageDisplayed = false;
    }
    _toggleLogMessageWidget(_logMessageDisplayed);

    Diamesh::SessionHelperFactory factory;
    QString lastUserName = factory.recentUsernames().value(0, QString());
    if (!lastUserName.isEmpty()) {
        QString lastUserPassword = factory.getPassword(lastUserName);
        emit enableQuickConnect(!lastUserPassword.isEmpty());
    } else
        emit enableQuickConnect(!lastUserName.isEmpty());

    _toggleLocalActions();

    connect(_topologyWidget(), SIGNAL(topologySaveFailed()), this, SLOT(_closeTopologiesFailedSlot()));
}

void HnvMainWindow::closeEvent(QCloseEvent *event)
{
    QSettings settings;
    settings.beginGroup(QString(Constants::MAIN_WINDOW_SETTINGS));
    if (settings.value(Constants::MAIN_WINDOW_SIZE_IS_SAVED, Constants::MAIN_WINDOW_DEFAULT_STATE_SAVE).toBool())
        settings.setValue(QString(Constants::MAIN_WINDOW_GEOMETRY),saveGeometry());
    settings.endGroup();
    if (_canTerminateCurrentSession()){
        _disconnectSession(true);
        ui->modeBrowser->saveSettings();
        qInstallMsgHandler(_oldMsgHandler);
    }
    else
        event->ignore();
}

/*******************************************************************************
* HnvMainWindow - Private slots
*/
void HnvMainWindow::_toggleSideBar()
{
    QWidget * currentWidget = ui->modeDisplayWidget->currentWidget();
    if (!currentWidget)
        return;
    ModeWidget * modeW = qobject_cast<ModeWidget*>(currentWidget);
    if (modeW)
        modeW->toggleSideBar();
}

void HnvMainWindow::_toggleRightSideBar()
{
    QWidget * currentWidget = ui->modeDisplayWidget->currentWidget();
    if (!currentWidget)
        return;
    ModeWidget * modeW = qobject_cast<ModeWidget*>(currentWidget);
    if (modeW)
        modeW->toggleRightSideBar();
}

void HnvMainWindow::_closeLogMessageWidget()
{
    _toggleLogMessageWidget(false);
    ui->statusBar->uncheckMessageAction();
}

void HnvMainWindow::_toggleLogMessageWidget(bool checked)
{
    int i = 0;
    while (i<ui->modeDisplayWidget->count()){
        QWidget *widget = ui->modeDisplayWidget->widget(i);
        if (ModeWidget * modeW = qobject_cast<ModeWidget*>(widget)){
            modeW->toggleLogMessageWidget(checked);
            _logMessageDisplayed = checked;
        }
        i++;
    }
    QSettings settings;
    settings.beginGroup(Constants::LOG_MESSAGE_SETTINGS);
    if (settings.value(Constants::LOG_MESSAGE_STATE_IS_SAVED, Constants::LOG_MESSAGE_DEFAULT_STATE_SAVE).toBool())
        settings.setValue(Constants::LOG_MESSAGE_IS_DISPLAY,_logMessageDisplayed);
    settings.endGroup();
}

void HnvMainWindow::_connectStartedSlot()
{
    loadingStarted(tr("Connecting..."));
}

void HnvMainWindow::_connectEndedSlot()
{
    loadingEnded(tr("Connecting..."));
}

void HnvMainWindow::_disconnectTriggered()
{
    if (_canTerminateCurrentSession())
        _disconnectSession(true);
}

void HnvMainWindow::_sessionDisconnectedSlot()
{
    emit sessionDisconnected();
    if (_usersModel)
        _usersModel->resetModel();

    if (_groupsModel)
        _groupsModel->resetModel();

    if (_sessionsModel)
        _sessionsModel->resetModel();

    if (_entityCatalogModel)
        _entityCatalogModel->resetModel();

    if (_platforMonitorModel)
        _platforMonitorModel->setMonitor(0);

    if (_entityInfoModel)
        _entityInfoModel->setMonitorEntityInfoList(0,0);

    if (_topologyCatalogModel)
        _topologyCatalogModel->resetModel();

    if (!_connectActionRequested && _session != 0) {
        QMessageBox::critical(this, tr("Connection lost"), tr("Connection to server lost."),QMessageBox::Ok,QMessageBox::Ok);
        ui->statusBar->setNetworkStatus(ConnectionStatusWidget::Error, tr("Connection lost"));
    } else
        ui->statusBar->setNetworkStatus(ConnectionStatusWidget::Offline);

    _connectActionRequested = false;

    _runningRemoteProcess = 0;
    _session = 0;
    const QMimeData *mimeData = QApplication::clipboard()->mimeData();
    if (mimeData->hasFormat("hyneview/catalog-entry"))
        QApplication::clipboard()->clear();
    _toggleLocalActions();

}

void HnvMainWindow::_newTraceSlot(QString message, QDateTime messageDate)
{
    HnvMainWindow *window = HnvMainWindow::instance();
    if (window)
        window->addLogMessage(LogMessageItem::HnsTrace, message, messageDate);
}

void HnvMainWindow::on_action_About_triggered()
{
    QString aboutTitle = tr("About %1").arg(qApp->applicationName());
    if (_session) {
        QMessageBox aboutHyneview(QMessageBox::NoIcon, aboutTitle, aboutMessage(true), QMessageBox::Close, this);
        aboutHyneview.setIconPixmap(QPixmap(Ressources::themedPath(Constants::ICON_LOGO)));
        aboutHyneview.exec();
    } else
        QMessageBox::about(this, aboutTitle, aboutMessage());
}

void HnvMainWindow::on_action_Options_triggered()
{
    if(applicationSettingsDialog()->exec() == QDialog::Accepted)
        emit refreshRemoteDisplay();
}


/*******************************************************************************
* HnvMainWindow - Private
*/
HnvMainWindow::HnvMainWindow(QWidget *parent)
    : QMainWindow(parent)
    , ui(new Ui::HnvMainWindow)
    , _session(0)
    , _connectActionRequested(false)
    , _runningRemoteProcess(0)
    , _loadingCount(0)
    , _waitingBox(0)
    , _dataLoading(false)
    , _applicationSettings(0)
    , _topologiesClosed(true)
{
    qRegisterMetaType<LogMessageItem::MessageType>("LogMessageItem::MessageType"); //we have to register this to communicate with other threads
    _topologyCatalogModel = new TopologyCatalogModel(this);
    _logMessageModel = new LogMessageModel(this);
    _entityCatalogModel = new EntityCatalogModel(this);
//    connect(_entityCatalogModel, SIGNAL(loadingCatalog(int,bool)), this, SLOT(entityCatalogLoadingSlot(int,bool)));
    _platforMonitorModel = new PlatformMonitorModel(this);
    _entityInfoModel = new EntityInfoModel(this);
    _applicationSettings = 0;
    connect(this, SIGNAL(newLogMessage(LogMessageItem::MessageType, QString, QDateTime)), _logMessageModel, SLOT(addMessage(LogMessageItem::MessageType, QString, QDateTime)));
}

bool HnvMainWindow::_canTerminateCurrentSession()
{
    _topologiesClosed = true;
    bool canTerminate = false;
    if (!_session)
        canTerminate = true;
    else
        canTerminate = _closeTopologies();
    if (canTerminate && _runningRemoteProcess > 0) {
        QMessageBox box(this);
        box.setStandardButtons(QMessageBox::NoButton);
        box.setText(tr("Saving topologies..."));
        box.setModal(true);
        box.show();
        while (_runningRemoteProcess > 0) {
            qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
        }
        box.hide();
    }
    return canTerminate && _topologiesClosed;
}

bool HnvMainWindow::_closeTopologies()
{
    bool result = true;
    TopologyWidget *topologyWidget = _topologyWidget();
    if (topologyWidget)
        result = topologyWidget->closeAllTopologiesSlot();
    return result;
}

void HnvMainWindow::_clickModeButton(int buttonIndex)
{
    QList<QAbstractButton *> buttons = ui->modeBrowser->modeButtons()->buttons();
    if (buttons.size() <= buttonIndex) {
        qCritical() << tr("Unable to get mode button");
        return;
    }
    buttons.at(buttonIndex)->click();
}

void HnvMainWindow::_connectionEstablished(QSharedPointer<Diamesh::SessionHelper> sessionHelper)
{
    if (_session != 0) {
        _disconnectSession(true);
    }

    loadingStarted(tr("Initialization"));
    _session = HyneviewSession::create(sessionHelper, this);
    if (_session == 0) {
        loadingEnded(tr("Initialization failed"));
        return;
    }
    connect(_session, SIGNAL(disconnected()), this, SLOT(_sessionDisconnectedSlot()));
    ui->statusBar->setNetworkStatus(ConnectionStatusWidget::Online);
    _entityCatalogModel->initModel(_session->entityEntryCatalog());
    _topologyCatalogModel->initModel(_session->topologyAdministrator());
    _platforMonitorModel->setMonitor(_session->platformMonitor());
    if (_entityInfoModel) {
        _entityInfoModel->setMonitorEntityInfoList(_session->platformMonitor(), HnsClient::Session::instance()->topologyManager()->entityInfoList());
    }
    connect(HnsClient::Session::instance()->tracerManager(), SIGNAL(newTrace(QString,QDateTime)), this, SLOT(_newTraceSlot(QString,QDateTime)));
    emit sessionConnected(_session);
    emit hnsClientSessionConnected();
    loadingEnded(tr("Session established"));
    _toggleLocalActions();
    _usersModel->initModel(HnsClient::Session::instance()->userManager());
    _groupsModel->initModel(HnsClient::Session::instance()->userManager());
    _sessionsModel->initModel(HnsClient::Session::instance()->userManager());
    sessionHelper.clear();
    connect(HnsClient::Session::instance()->entityEntryCatalog(), SIGNAL(updateACLFailed(QString)), this, SLOT(_updateACLFailedSlot(QString)));
}

void HnvMainWindow::_disconnectSession(bool userAction)
{
    if (!_session)
        return;
    if (_waitingBox)
        return;
    _connectActionRequested = userAction;
    _platforMonitorModel->setMonitor(0);
    if (_entityInfoModel)
        _entityInfoModel->setMonitorEntityInfoList(0,0);
    _session->disconnectSession();
//    _runningRemoteProcess.clear();
}

TopologyWidget *HnvMainWindow::_topologyWidget()
{
    if (_simulationWidget)
        return _simulationWidget->topologyWidget();
    return NULL;
}

void HnvMainWindow::_populateMenu(QMenu *menu, QList<QAction *> actionList)
{
    if (menu == 0 || actionList.count() == 0)
        return;

    QAction *firstAction = 0;
    if (menu->actions().count())
        firstAction = menu->actions().first();
    menu->insertSeparator(firstAction);
    menu->insertActions(firstAction, actionList);
}

void HnvMainWindow::_toggleLocalActions()
{
    bool connected = false;
    if (_session)
        connected = true;
    ui->action_Import->setEnabled(connected);
}

void HnvMainWindow::_updateACLFailedSlot(QString errorMessage)
{
    QMessageBox::critical(this, QObject::tr("Error"), errorMessage);
}

void HnvMainWindow::on_action_Import_triggered()
{
    ImportDialog importDialog(this);
    importDialog.exec();
}

void HnvMainWindow::_closeTopologiesFailedSlot()
{
    _topologiesClosed = false;
    remoteProcessEnded();
}

void HnvMainWindow::_modeChanged(QWidget *w)
{
    ui->statusBar->enableTopologyActions((w == _simulationWidget));
    ui->modeDisplayWidget->setCurrentWidget(w);
}

void HnvMainWindow::on_actionPopupStyle_triggered()
{
    PopupStyle *popup = new PopupStyle();
    popup->show();
}

} // namespace Hnv
