/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "helpbrowser.h"
#include "helpbrowserwidget.h"

// Qt headers
#include <QTimer>
#include <QHelpEngine>
#include <QWebFrame>
#include <QAction>

namespace Hnv {

/*******************************************************************************
* HelpNetworkReply
*/
HelpNetworkReply::HelpNetworkReply(const QNetworkRequest &request, const QByteArray &fileData, const QString& mimeType)
    : _data(fileData)
    , _origLen(fileData.length())
{
    setRequest(request);
    setOpenMode(QIODevice::ReadOnly);
    setHeader(QNetworkRequest::ContentTypeHeader, mimeType);
    setHeader(QNetworkRequest::ContentLengthHeader, QByteArray::number(_origLen));
    QTimer::singleShot(0, this, SIGNAL(metaDataChanged()));
    QTimer::singleShot(0, this, SIGNAL(readyRead()));
}

void HelpNetworkReply::abort()
{
}

qint64 HelpNetworkReply::bytesAvailable() const
{
    return _data.length() + QNetworkReply::bytesAvailable();
}

qint64 HelpNetworkReply::readData(char *buffer, qint64 maxlen)
{
    qint64 len = qMin(qint64(_data.length()), maxlen);
    if (len) {
        memcpy(buffer, _data.constData(), len);
        _data.remove(0, len);
    }
    if (!_data.length())
        QTimer::singleShot(0, this, SIGNAL(finished()));
    return len;
}

/*******************************************************************************
* HelpNetworkAccessManager
*/
HelpNetworkAccessManager::HelpNetworkAccessManager(QHelpEngine * engine, QObject *parent)
    : QNetworkAccessManager(parent)
    , _engine(engine)
{
}

QNetworkReply *HelpNetworkAccessManager::createRequest(Operation op, const QNetworkRequest &request, QIODevice* outgoingData)
{
    QString url = request.url().toString();
    if (_engine->findFile(url).isValid()) {
        const QString &mimeType = "text/html";
        const QByteArray &data = _engine->fileData(url);
        return new HelpNetworkReply(request, data, mimeType);
    } else {
        return QNetworkAccessManager::createRequest(op, request, outgoingData);
    }
}

/*******************************************************************************
* HelpPage
*/
HelpPage::HelpPage(QObject *parent)
    : QWebPage(parent)
{
    action(QWebPage::DownloadLinkToDisk)->setVisible(false);
    connect(mainFrame(), SIGNAL(titleChanged(QString)), this, SIGNAL(titleChanged(QString)));
}

/*******************************************************************************
* HelpPage - Protected
*/
bool HelpPage::acceptNavigationRequest(QWebFrame * frame, const QNetworkRequest &request, QWebPage::NavigationType type)
{
    return QWebPage::acceptNavigationRequest(frame, request, type);
}

/*******************************************************************************
* HelpBrowser
*/
qlonglong HelpView::_nextViewID = 0;

HelpView::HelpView(HelpBrowserWidget *browser, QHelpEngine *engine, QWidget *parent)
    : QWebView(parent)
    , _viewID(++HelpView::_nextViewID)
    , _browser(browser)
    , _engine(engine)
{
    HelpPage * helpPage = new HelpPage(this);
    helpPage->setNetworkAccessManager(new HelpNetworkAccessManager(_engine, this));
    setPage(helpPage);
    connect(this, SIGNAL(titleChanged(QString)), this, SLOT(_titleChangedSlot(QString)));
}

qlonglong HelpView::viewID() const
{
    return _viewID;
}

bool HelpView::isForwardAvailable() const
{
    return pageAction(QWebPage::Forward)->isEnabled();
}

bool HelpView::isBackwardAvailable() const
{
    return pageAction(QWebPage::Back)->isEnabled();
}

void HelpView::home()
{
    QString homePage = _engine->customValue(QLatin1String("defaultHomepage"), QLatin1String("")).toString();
    if (!homePage.isEmpty()) {
        QUrl urlFound = _engine->findFile(homePage);
        if (urlFound.isValid())
            load(urlFound);
    }
}

void HelpView::nextPage()
{
    if (isForwardAvailable())
        triggerPageAction(QWebPage::Forward);
}

void HelpView::previousPage()
{
    if (isBackwardAvailable())
        triggerPageAction(QWebPage::Back);
}

QWebView * HelpView::createWindow(QWebPage::WebWindowType type)
{
    Q_UNUSED(type);
    return _browser->createEmptyTab();
}

void HelpView::_titleChangedSlot(QString title)
{
    emit viewChanged(_viewID, title);
}

} // namespace Hnv
