/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "helpbrowserwidget.h"
#include "ui_helpbrowserwidget.h"
#include "ressources.h"
#include "helpbrowser.h"

// Qt headers
#include <QHelpEngine>

namespace Hnv {

HelpBrowserWidget::HelpBrowserWidget(QHelpEngine *helpEngine, QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::HelpBrowserWidget)
    , _helpEngine(helpEngine)
{
    _loadedModel= new LoadedHelpViewModel(this, this);
    ui->setupUi(this);
    ui->toolBarWidget->addToolBarAction(ui->action_Home, false);
    ui->toolBarWidget->addToolBarAction(ui->action_Previous_page, false);
    ui->toolBarWidget->addToolBarAction(ui->action_Next_page, false);
    ui->action_Home->setIcon(QIcon(Ressources::themedPath(Constants::ICON_HELP_HOME)));
    ui->action_Previous_page->setIcon(QIcon(Ressources::themedPath(Constants::ICON_HELP_PREVIOUS)));
    ui->action_Next_page->setIcon(QIcon(Ressources::themedPath(Constants::ICON_HELP_NEXT)));
    connect(ui->toolBarWidget, SIGNAL(closeTriggered()), this, SLOT(_closeViewSlot()));
    createEmptyTab();
    _setTitle();
    home();
    connect(ui->action_Home, SIGNAL(triggered()), this, SLOT(home()));
    connect(ui->action_Previous_page, SIGNAL(triggered()), this, SLOT(previousPage()));
    connect(ui->action_Next_page, SIGNAL(triggered()), this, SLOT(nextPage()));
    connect(ui->stackedWidget, SIGNAL(currentChanged(int)), this, SLOT(_currentViewChanged(int)));
}

HelpBrowserWidget::~HelpBrowserWidget()
{
    delete ui;
}

void HelpBrowserWidget::load(QUrl link)
{
    HelpView * view = currentHelpView();
    if (view == 0)
        view = createEmptyTab();
    view->load(link);
}

void HelpBrowserWidget::activateView(qlonglong viewID)
{
    if (_viewHash.contains(viewID))
        ui->stackedWidget->setCurrentWidget(_viewHash.value(viewID));
}

void HelpBrowserWidget::closeView(qlonglong viewID)
{
    if (viewID != 0)
        _closeViewSlot(viewID);
}

bool HelpBrowserWidget::isForwardAvailable() const
{
    HelpView * view = currentHelpView();
    if (view)
        return view->isForwardAvailable();
    return false;
}

bool HelpBrowserWidget::isBackwardAvailable() const
{
    HelpView * view = currentHelpView();
    if (view)
        return view->isBackwardAvailable();
    return false;
}

HelpView * HelpBrowserWidget::currentHelpView() const
{
    HelpView * view = 0;
    QWidget * currentW = ui->stackedWidget->currentWidget();
    if (currentW)
        view = qobject_cast<HelpView *>(currentW);
    return view;
}

qlonglong HelpBrowserWidget::currentViewID() const
{
    HelpView * view = currentHelpView();
    if (view)
        return view->viewID();
    return 0;
}

HelpView * HelpBrowserWidget::viewByID(qlonglong viewID) const
{
    return _viewHash.value(viewID, 0);
}

QList<qlonglong> HelpBrowserWidget::viewIDList() const
{
    return _viewHash.keys();
}

LoadedHelpViewModel * HelpBrowserWidget::loadedViewModel() const
{
    return _loadedModel;
}

void HelpBrowserWidget::home()
{
    HelpView * view = currentHelpView();
    if (view)
        view->home();
}

void HelpBrowserWidget::nextPage()
{
    HelpView * view = currentHelpView();
    if (view)
        view->nextPage();
}

void HelpBrowserWidget::previousPage()
{
    HelpView * view = currentHelpView();
    if (view)
        view->previousPage();
}

HelpView * HelpBrowserWidget::createEmptyTab()
{
    HelpView * view = new HelpView(this, _helpEngine, this);
    connect(view, SIGNAL(viewChanged(qlonglong,QString)), this, SLOT(_viewChangedSlot(qlonglong,QString)));
    qlonglong viewID = view->viewID();
    _viewHash.insert(viewID, view);
    emit helpViewCreated(viewID);
    ui->stackedWidget->addWidget(view);
    ui->stackedWidget->setCurrentWidget(view);
    return view;
}

void HelpBrowserWidget::_setTitle(QString title)
{
    if (title.isEmpty())
        title = tr("hyneview help browser");
    ui->toolBarWidget->setTitle(title);
}

void HelpBrowserWidget::_viewChangedSlot(qlonglong viewID, QString title)
{
    emit helpViewChanged(viewID);
    if (viewID == currentViewID()) {
        _setTitle(title);
        _updateNavigationActions(true);
    }
}

void HelpBrowserWidget::_closeViewSlot(qlonglong viewID)
{
    HelpView * view = 0;
    if (viewID == 0)
        view = currentHelpView();
    else
        view = viewByID(viewID);
    if (view == 0)
        return;
    viewID = view->viewID();
    emit helpViewClosed(viewID);
    _viewHash.remove(viewID);
    ui->stackedWidget->removeWidget(view);
    view->deleteLater();
    if (ui->stackedWidget->count() == 0)
        _setTitle();
}

void HelpBrowserWidget::_currentViewChanged(int index)
{
    bool homeEnabled = false;
    if (index != -1)
        homeEnabled = true;
    _updateNavigationActions(homeEnabled);
}

void HelpBrowserWidget::_updateNavigationActions(bool homeEnabled)
{
    bool nextEnabled = isForwardAvailable();
    bool prevEnabled = isBackwardAvailable();
    ui->action_Home->setEnabled(homeEnabled);
    ui->action_Next_page->setEnabled(nextEnabled);
    ui->action_Previous_page->setEnabled(prevEnabled);
}

/*******************************************************************************
* LoadedHelpBrowserModel
*/
LoadedHelpViewModel::LoadedHelpViewModel(HelpBrowserWidget *browser, QObject *parent)
    : QAbstractItemModel(parent)
    , _browser(browser)
{
    QList<qlonglong> viewList = _browser->viewIDList();
    connect(_browser, SIGNAL(helpViewCreated(qlonglong)), this, SLOT(_helpViewCreatedSlot(qlonglong)));
    connect(_browser, SIGNAL(helpViewChanged(qlonglong)), this, SLOT(_helpViewChangedSlot(qlonglong)));
    connect(_browser, SIGNAL(helpViewClosed(qlonglong)), this, SLOT(_helpViewClosedSlot(qlonglong)));
    foreach(qlonglong newViewID, viewList)
        _helpViewCreatedSlot(newViewID);
}

QModelIndex LoadedHelpViewModel::index(int row, int column, const QModelIndex &parent) const
{
    if (!hasIndex(row, column, parent))
        return QModelIndex();
    if (row < _viewList.size())
        return createIndex(row, column);
    return QModelIndex();
}

QModelIndex LoadedHelpViewModel::parent(const QModelIndex &child) const
{
    Q_UNUSED(child)
    return QModelIndex();
}

int LoadedHelpViewModel::rowCount(const QModelIndex &parent) const
{
    if (parent.isValid())
        return 0;
    return _viewList.size();
}

int LoadedHelpViewModel::columnCount(const QModelIndex &parent) const
{
    Q_UNUSED(parent);
    return 2;
}

QVariant LoadedHelpViewModel::data(const QModelIndex &index, int role) const
{
    if (index.isValid()) {
        int row = index.row();
        if (row < _viewList.size()) {
            HelpView * view = _browser->viewByID(_viewList.at(row));
            if (view) {
                switch(role) {
                case Qt::UserRole: {
                    return view->viewID();
                    break;
                }
                case Qt::DisplayRole : {
                    return view->title();
                    break;
                }
                }
            }
        }
    }
    return QVariant();
}

void LoadedHelpViewModel::clear()
{
    beginResetModel();
    _viewList.clear();
    endResetModel();
}

void LoadedHelpViewModel::_helpViewCreatedSlot(qlonglong viewID)
{
    int row = _viewList.size();
    beginInsertRows(QModelIndex(), row, row);
    _viewList.append(viewID);
    endInsertRows();
}

void LoadedHelpViewModel::_helpViewChangedSlot(qlonglong viewID)
{
    int row = _viewList.indexOf(viewID);
    if (row != -1) {
        QModelIndex idx = index(row, 0);
        emit dataChanged(idx, idx);
    }
}

void LoadedHelpViewModel::_helpViewClosedSlot(qlonglong viewID)
{
    if (_viewList.contains(viewID)) {
        int row = _viewList.indexOf(viewID);
        beginRemoveRows(QModelIndex(), row, row);
        _viewList.removeAll(viewID);
        endRemoveRows();
    }
}

} // namespace Hnv
