/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "helpsearchwidget.h"
#include "helpmodewidget.h"

// Qt headers
#include <QHelpEngine>
#include <QHelpSearchQueryWidget>
#include <QHelpSearchEngine>
#include <QHelpSearchQuery>

namespace Hnv {

/*******************************************************************************
* HelpSearchResultWidget
*/
HelpSearchResultWidget::HelpSearchResultWidget(QWidget *parent)
    : QListWidget(parent)
    , _searchEngine(0)
{
    setFrameShape(QFrame::NoFrame);
    connect(this, SIGNAL(itemActivated(QListWidgetItem*)), this, SLOT(_itemActivatedSlot(QListWidgetItem*)));
}

HelpSearchResultWidget::~HelpSearchResultWidget()
{
}

void HelpSearchResultWidget::setEngine(QHelpSearchEngine *engine)
{
    if (engine)
        _searchEngine = engine;
}

void HelpSearchResultWidget::setResults(int hits)
{
    if (_searchEngine == 0)
        return;
    clear();
    QList<QHelpSearchEngine::SearchHit> resultList = _searchEngine->hits(0, hits);
    foreach(QHelpSearchEngine::SearchHit result, resultList) {
        QListWidgetItem * item = new QListWidgetItem(result.second, this);
        item->setData(Qt::UserRole, result.first);
        addItem(item);
    }
}

void HelpSearchResultWidget::_itemActivatedSlot(QListWidgetItem *item)
{
    if (item) {
        QUrl requestedURL(item->data(Qt::UserRole).toString());
        emit requestShowLink(requestedURL);
    }
}

/*******************************************************************************
* HelpSearchWidget
*/
HelpSearchWidget::HelpSearchWidget(QWidget *parent)
    : SideBarSimpleWidget(Qt::Vertical, parent)
    , _queryWidget(new QHelpSearchQueryWidget(this))
    , _resultWidget(new HelpSearchResultWidget(this))
    , _searchEngine(0)
    , _isSearching(false)
{
    QList<QWidget *> children;
    children.append(_queryWidget);
    children.append(_resultWidget);
    _initChildren(children);

    HelpModeWidget * modeWidget = qobject_cast<HelpModeWidget*>(sideBarParent());
    if (modeWidget) {
        connect(_resultWidget, SIGNAL(requestShowLink(QUrl)), modeWidget, SLOT(activateLink(QUrl)));
        _initSearchEngine(modeWidget->helpEngine()->searchEngine());
    }
}

HelpSearchWidget::~HelpSearchWidget()
{
    delete _queryWidget;
}

QString HelpSearchWidget::title()
{
    return tr("Search");
}

QString HelpSearchWidget::sideBarId()
{
    return "Search";
}

void HelpSearchWidget::_initSearchEngine(QHelpSearchEngine *engine)
{
    if (engine == 0)
        return;
    _searchEngine = engine;
    _resultWidget->setEngine(_searchEngine);
    connect(_queryWidget, SIGNAL(search()), this, SLOT(_searchSlot()));
    connect(_searchEngine, SIGNAL(searchingFinished(int)), this, SLOT(_searchingFinishedSlot(int)));
}

void HelpSearchWidget::_searchSlot()
{
    _currentQueryList = _queryWidget->query();
    _isSearching = true;
    _searchEngine->search(_currentQueryList);
}

void HelpSearchWidget::_searchingFinishedSlot(int hits)
{
    QList<QHelpSearchQuery> searchedQuery = _searchEngine->query();
    bool matchingQuery = false;
    int queryCount = _currentQueryList.count();
    if (searchedQuery.count() == queryCount) {
        matchingQuery = true;
        for(int i = 0; i < queryCount; i++) {
            QHelpSearchQuery engineQuery = searchedQuery.at(i);
            QHelpSearchQuery currentQuery = _currentQueryList.at(i);
            if (engineQuery.fieldName != currentQuery.fieldName || engineQuery.wordList != currentQuery.wordList) {
                matchingQuery = false;
                break;
            }
        }
    }
    if (matchingQuery && _isSearching && _resultWidget) {
        _isSearching = false;
        _resultWidget->setResults(hits);
    }
}

} // namespace Hnv
