/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "sessionlistwidget.h"
#include "hnvmainwindow.h"
#include "hyneviewsession.h"
#include "sessionsmodel.h"
#include "ressources.h"

// diamesh headers
#include <diamesh/diamesh.h>

// Qt headers
#include <QMenu>
#include <QAction>
#include <QSettings>

namespace Hnv {

/*******************************************************************************
* SessionListWidget
*******************************************************************************/
SessionListWidget::SessionListWidget(QWidget *parent)
    : SideBarListWidget(parent)
{
    _actionRemoteDisplay = new QAction(this);
    _actionRemoteDisplay->setText(tr("Remote display"));
    _actionRemoteDisplay->setToolTip(tr("Start remote display client"));
    connect(_actionRemoteDisplay, SIGNAL(triggered()), this, SLOT(_action_RemoteDisplaySlot()));
    _setIconSize(QSize(16,16));
    HnvMainWindow * window = HnvMainWindow::instance();
    if (window)
        setViewModel(window->sessionsModel());
}

SessionListWidget::~SessionListWidget()
{

}

QString SessionListWidget::title()
{
    return tr("Sessions");
}

QString SessionListWidget::sideBarId()
{
    return "Sessions";
}

void SessionListWidget::_indexClicked(const QModelIndex &index)
{
    Q_UNUSED(index)
}

void SessionListWidget::_indexDoubleClicked(const QModelIndex &index)
{
    Q_UNUSED(index)
}

void SessionListWidget::_contextMenuRequested(const QModelIndex &index, const QPoint &position)
{
    if (index.isValid()) {
        QMenu contextMenu;
        contextMenu.addAction(_actionRemoteDisplay);
        contextMenu.exec(_globalPosition(position));
    }
}

void SessionListWidget::_action_RemoteDisplaySlot()
{
    QModelIndex currentIdx = _currentIndex();
    if (currentIdx.isValid())
        _startRemoteDisplay(currentIdx.data(Qt::UserRole).toString(), currentIdx.data().toString());
}

void SessionListWidget::_startRemoteDisplay(QString target, QString title)
{
    if (_processHash.contains(target) && _processHash.value(target)->state() != QProcess::NotRunning) {
        qWarning() << tr("Remote display program is already started for \"%1\".").arg(target);
        return;
    }

    QString myAddress = QString::fromStdString(HyneviewSession::instance()->sessionInformation().ipAddress);

    if (myAddress == target) {
        qWarning() << tr("Remote display forbidden");
        return;
    }

    QStringList arguments;
    QSettings settings(Diamesh::instance()->configFileName(), QSettings::IniFormat, this);
    QStringList vncCommands  = settings.value(Constants::VNC_VIEWER_COMMAND_KEY, Constants::VNC_VIEWER_DEFAULT_COMMAND).toString().split(" ", QString::SkipEmptyParts);
    QString program = vncCommands.takeFirst();
    arguments << target;
    int vncTitleIndex = vncCommands.indexOf("%TITLE%");
    if (vncTitleIndex > 0) {
        arguments << vncCommands.takeAt(vncTitleIndex-1) << title;
        vncCommands.removeAll("%TITLE%");
    }

    qDebug() << tr("Trying to start remote display program for \"%1\" using:%2 %3")
                .arg(title)
                .arg(program)
                .arg(arguments.join(" "));

    QProcess * remoteDisplayProcess = new QProcess(this);
    connect(remoteDisplayProcess, SIGNAL(error(QProcess::ProcessError)), this , SLOT(_remoteDisplayError(QProcess::ProcessError)));
    connect(remoteDisplayProcess, SIGNAL(finished(int,QProcess::ExitStatus)), this, SLOT(_remoteDisplayFinished(int,QProcess::ExitStatus)));
    _processHash.insert(target, remoteDisplayProcess);
    remoteDisplayProcess->start(program, arguments);
}

void SessionListWidget::_remoteDisplayFinished(int exitCode, QProcess::ExitStatus exitStatus)
{
    QProcess * process = qobject_cast<QProcess*>(sender());
    if (process == 0)
        return;

    QString remoteTarget = _processHash.key(process);

    _processHash.remove(remoteTarget);

    QString output = QString(process->readAllStandardOutput()).trimmed();
    QString error = QString(process->readAllStandardError()).trimmed();
    if (!error.isEmpty()) {
        if (!output.isEmpty())
            output.append("\n");
        output.append(error);
    }

    if (exitStatus != QProcess::NormalExit) {
        QString message = tr("Remote display program connected to \"%1\" crashed.").arg(remoteTarget);
        if (!output.isEmpty())
            message.append(QString("\n%1").arg(output));
        qCritical() << message;
    } else if (exitCode != 0) {
        QString message = tr("Remote display program connected to \"%1\" ended.").arg(remoteTarget);
        if (!output.isEmpty())
            message.append(QString("\n%1").arg(output));
        qWarning() << message;
    }
    _destroyRemoteProcess(process);
}

void SessionListWidget::_remoteDisplayError(QProcess::ProcessError error)
{
    QProcess * process = qobject_cast<QProcess*>(sender());
    if (process == 0)
        return;

    QString remoteTarget = _processHash.key(process);

    switch (error) {
    case QProcess::FailedToStart:
        qCritical() << tr("Failed to run remote display program for \"%1\". Either the invoked program is missing, or you may have insufficient permissions to invoke the program.\n%2%3")
                       .arg(remoteTarget)
                       .arg(QString(process->readAllStandardOutput()))
                       .arg(QString(process->readAllStandardError()));
        _processHash.remove(remoteTarget);
        _destroyRemoteProcess(process);
        break;
    case QProcess::Crashed:
        break;
    default:
        qCritical() << tr("Remote display program connected to  \"%1\" reported an error.").arg(remoteTarget);
    }
}

void SessionListWidget::_destroyRemoteProcess(QProcess *process)
{
    if (process != 0) {
        if (process->state() != QProcess::NotRunning)
            process->terminate();
        process->deleteLater();
    }
}

} // namespace Hnv
