// Project headers
#include "rdpworker.h"
#include "display.h"
#include "ressources.h"

// Qt headers
#include <QCoreApplication>
#include <QThread>
#include <QSettings>
#include <QDebug>

// External headers
#include <X11/Xlib.h> // Must be included after Qt

// FreeRDP headers
#include <freerdp/gdi/gdi.h>
#include <freerdp/constants.h>
#include <freerdp/locale/keyboard.h>
#include <freerdp/locale/locale.h>

namespace RemoteAccess {

RdpWorker::RdpWorker(QObject *parent)
    : Worker(parent)
{
    _close = false;
}

RdpWorker::~RdpWorker()
{
    _thread->exit(0);
    _thread->deleteLater();
}

void RdpWorker::run()
{
    _instance = freerdp_new();

    _instance->ContextNew = RdpWorker::contextNew;
    _instance->ContextFree = RdpWorker::contextFree;
    _instance->ContextSize = sizeof(RdpHnsContext);

    freerdp_context_new(_instance); //todo: test return code;

    RdpHnsContext *hnsCtxt = (RdpHnsContext *)_instance->context;
    hnsCtxt->worker = this;

    _instance->PostConnect = RdpWorker::postConnect; //  Callback for post-connect operations. Can be set before calling freerdp_connect() to have it executed after the actual connection has succeeded. Must be set to NULL if not needed.
    _instance->PreConnect = RdpWorker::preConnect; // Callback for pre-connect operations. Can be set before calling freerdp_connect() to have it executed before the actual connection happens. Must be set to NULL if not needed

    /*setting options*/
    //todo: check bmp cache if needed
    rdpSettings *settings = _instance->settings;
    settings->ServerPort = _info.port;
    settings->ServerHostname = strdup(_info.host.c_str());
    settings->SoftwareGdi = 1;
    settings->DesktopResize = false;
    settings->NlaSecurity = false;
    settings->TlsSecurity = false;
    settings->OsMajorType = OSMAJORTYPE_UNIX;
    settings->OsMinorType = OSMINORTYPE_NATIVE_XSERVER;
    settings->OrderSupport[NEG_DRAWNINEGRID_INDEX] = false;
    settings->OrderSupport[NEG_MULTIDSTBLT_INDEX] = false;
    settings->OrderSupport[NEG_MULTIPATBLT_INDEX] = false;
    settings->OrderSupport[NEG_MULTISCRBLT_INDEX] = false;
    settings->OrderSupport[NEG_MULTI_DRAWNINEGRID_INDEX] = false;
    settings->OrderSupport[NEG_MEMBLT_INDEX] = false;
    settings->OrderSupport[NEG_MEM3BLT_INDEX] = false;
    settings->OrderSupport[NEG_SAVEBITMAP_INDEX] = false;
    settings->OrderSupport[NEG_GLYPH_INDEX_INDEX] = false;
    settings->OrderSupport[NEG_FAST_GLYPH_INDEX] = false;
    settings->OrderSupport[NEG_POLYGON_SC_INDEX] = false;
    settings->OrderSupport[NEG_POLYGON_CB_INDEX] = false;
    settings->OrderSupport[NEG_ELLIPSE_SC_INDEX] = false;
    settings->OrderSupport[NEG_ELLIPSE_CB_INDEX] = false;
    settings->OrderSupport[NEG_FAST_INDEX_INDEX] = false;

    QSettings qSettings;
    QString resolution = qSettings.value(Hnv::Constants::REMOTEDISPLAY_RESOLUTION_OPTION).toString();
    if (resolution.isEmpty())
        resolution = "1024x768";
    _setResolution(resolution, settings->DesktopWidth, settings->DesktopHeight);

    /*Assign update callbacks*/
    _instance->update->BeginPaint = RdpWorker::beginPaint;
    _instance->update->EndPaint = RdpWorker::endPaint;

    freerdp_connect(_instance);

    int i;
    int fds;
    int max_fds;
    int rcount;
    int wcount;
    void* rfds[32];
    void* wfds[32];
    fd_set rfds_set;
    fd_set wfds_set;

    memset(rfds, 0, sizeof(rfds));
    memset(wfds, 0, sizeof(wfds));

    while (!freerdp_shall_disconnect(_instance))
    {
        rcount = 0;
        wcount = 0;

        if (freerdp_get_fds(_instance, rfds, &rcount, wfds, &wcount) != true)
        {
            emit connectionError(QObject::tr("Unable to open RDP connection"));
            break;
        }

        max_fds = 0;
        FD_ZERO(&rfds_set);
        FD_ZERO(&wfds_set);

        for (i = 0; i < rcount; i++)
        {
            fds = (int)(long)(rfds[i]);

            if (fds > max_fds)
                max_fds = fds;

            FD_SET(fds, &rfds_set);
        }

        if (max_fds == 0)
            break;
        timeval timeout;
        timeout.tv_sec = 0;
        timeout.tv_usec = 100;
        if (select(max_fds + 1, &rfds_set, &wfds_set, NULL, &timeout) == -1)
        {
            qDebug() << tr("select failed");
        }
        if (freerdp_check_fds(_instance) != true)
        {
            qDebug() << QObject::tr("Failed to check FreeRDP file descriptor");
            break;
        }
        if (_close) {
            break;
        }
        QCoreApplication::processEvents();

    }

    freerdp_disconnect(_instance);
    gdi_free(_instance);
    freerdp_context_free(_instance);
    freerdp_free(_instance);

    emit connectionEnded();
}

void RdpWorker::_setResolution(const QString &resolution, unsigned int &width, unsigned int &height)
{
    width = resolution.split("x").first().toUInt();
    height = resolution.split("x").last().toUInt();
}

int RdpWorker::contextNew(freerdp *instance, rdpContext *context)
{
    Q_UNUSED(instance)
    RdpHnsContext *hnsCtx = (RdpHnsContext *)context;
    hnsCtx->worker = 0;
    return 0;
}

void RdpWorker::contextFree(freerdp *instance, rdpContext *context)
{
    Q_UNUSED(instance)
    RdpHnsContext *hnsCtx = (RdpHnsContext *)context;
    hnsCtx->worker = 0;
}

int RdpWorker::preConnect(rdp_freerdp *instance)
{
    Q_UNUSED(instance)
    freerdp_keyboard_init(0);
    return 1;
}

int RdpWorker::postConnect(rdp_freerdp *instance)
{
    gdi_init(instance, CLRCONV_ALPHA | CLRBUF_32BPP, NULL);

    rdpPointer *pointer = (rdpPointer *)malloc(sizeof(rdpPointer));
    pointer->size = sizeof(rdpPointer);
    pointer->Set = RdpWorker::pointerSet;
    pointer->New = RdpWorker::pointerNew;
    pointer->Free = RdpWorker::pointerFree;
    graphics_register_pointer(instance->context->graphics, pointer);
    free(pointer);
    pointer_cache_register_callbacks(instance->update);
    return 1;
}

void RdpWorker::beginPaint(rdpContext *context)
{
    rdpGdi *gdi = context->gdi;
    gdi->primary->hdc->hwnd->invalid->null = 1;
    gdi->primary->hdc->hwnd->ninvalid = 0;
}

void RdpWorker::endPaint(rdpContext *context)
{
    rdpGdi *gdi = context->gdi;
    int x, y, w, h;

    if (gdi->primary->hdc->hwnd->ninvalid < 1)
        return;

    int ninvalid = gdi->primary->hdc->hwnd->ninvalid;
    HGDI_RGN cinvalid = gdi->primary->hdc->hwnd->cinvalid;

    int widthResolution = context->instance->settings->DesktopWidth;
    int heightResolution = context->instance->settings->DesktopHeight;
    QImage image((uchar *)context->gdi->primary_buffer, widthResolution, heightResolution, QImage::Format_ARGB32);
    for (int i = 0 ; i < ninvalid ; i++) {
        x = cinvalid[i].x;
        y = cinvalid[i].y;
        w = cinvalid[i].w;
        h = cinvalid[i].h;
        RdpHnsContext *hnsCtx = (RdpHnsContext *)context;
        RdpWorker *worker = hnsCtx->worker;
        emit worker->imageChanged(image.copy(x, y, w, h), x, y, w, h);
    }
}

void RdpWorker::pointerSet(rdpContext *context, rdpPointer *pointer)
{
    RdpHnsContext *hnsCtx = (RdpHnsContext *)context;
    RdpWorker *worker = hnsCtx->worker;
    unsigned char *data = (unsigned char *)malloc(pointer->width * pointer->height * 4);
    if ((pointer->andMaskData != 0) && (pointer->xorMaskData != 0)) {
        freerdp_alpha_cursor_convert(data, pointer->xorMaskData, pointer->andMaskData, pointer->width, pointer->height, pointer->xorBpp, context->gdi->clrconv);;
        QImage img((uchar *)data, pointer->width, pointer->height, QImage::Format_ARGB32);
        emit worker->cursorChanged(img.copy(img.rect()), pointer->xPos, pointer->yPos);
    }
    free(data);
}

void RdpWorker::pointerNew(rdpContext *context, rdpPointer *pointer)
{
    Q_UNUSED(context)
    Q_UNUSED(pointer)
}

void RdpWorker::pointerFree(rdpContext *context, rdpPointer *pointer)
{
    Q_UNUSED(context)
    Q_UNUSED(pointer)
}

void RdpWorker::closeConnection()
{
    _close = true;
}

Display *RdpWorker::createDisplay()
{
    ImageDisplay *display = new ImageDisplay();
    connect(this, SIGNAL(imageChanged(QImage, int, int, int, int)), display, SIGNAL(imageChanged(QImage, int, int, int, int)));
    connect(this, SIGNAL(cursorChanged(QImage,int,int)), display, SIGNAL(cursorChanged(QImage,int,int)));
    connect(this, SIGNAL(sizeChanged(QSize)), display, SIGNAL(sizeChanged(QSize)));
    connect(this, SIGNAL(clearImaged()), display, SIGNAL(clearImaged()));
    connect(display, SIGNAL(keyboardEventReceived(RemoteAccess::KeyInfo)), this, SLOT(keyboardEventReceive(RemoteAccess::KeyInfo)));
    connect(display, SIGNAL(mouseEventReceived(RemoteAccess::MouseInfo)), this, SLOT(mouseEventReceive(RemoteAccess::MouseInfo)));
    return display;
}

void RdpWorker::mouseEventReceive(MouseInfo info)
{
    rdpInput *input;
    input = _instance->input;
    int flags;
    if (info.state == Move) {
        flags = PTR_FLAGS_MOVE;
    }
    else if (info.state == Click || info.state == DoubleClick) {
        if (info.button == Left)
            flags = PTR_FLAGS_BUTTON1 | PTR_FLAGS_DOWN;
        else if (info.button == Mid)
            flags = PTR_FLAGS_BUTTON3 | PTR_FLAGS_DOWN;
        else if (info.button == Right)
            flags = PTR_FLAGS_BUTTON2 | PTR_FLAGS_DOWN;
    }
    else if (info.state == Release) {
        if (info.button == Left)
            flags = PTR_FLAGS_BUTTON1;
        else if (info.button == Mid)
            flags = PTR_FLAGS_BUTTON3;
        else if (info.button == Right)
            flags = PTR_FLAGS_BUTTON2;
    }
    else if (info.state == WheelUp)
        flags = PTR_FLAGS_WHEEL;
    else if (info.state == WheelDown)
        flags = PTR_FLAGS_WHEEL | PTR_FLAGS_WHEEL_NEGATIVE;
    input->MouseEvent(input, flags, info.x, info.y);
}

void RdpWorker::keyboardEventReceive(KeyInfo info)
{
    rdpInput *input;
    input = _instance->input;
    DWORD scancode = freerdp_keyboard_get_rdp_scancode_from_x11_keycode(info.nativeVirtualCode);
    if (info.press)
        freerdp_input_send_keyboard_event_ex(input, TRUE, scancode);
    else
        freerdp_input_send_keyboard_event_ex(input, FALSE, scancode);
}

void RdpWorker::openConnection()
{
    _thread = new QThread();
    connect(_thread, SIGNAL(started()), this, SLOT(run()));
    this->moveToThread(_thread);
    _thread->start();
}

} //namespace RemoteAccess
