/******************************************************************************
**
** Copyright (C) 2003-2017 DIATEAM. All rights reserved.
**
** This file is part of Hynesim.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include "spiceusbmanager.h"

#include <QStringList>

namespace RemoteAccess {

SpiceUsbManager::SpiceUsbManager(QObject *parent)
    : UsbManager(parent)
    , _spiceManager(0)
{
}

void SpiceUsbManager::setSpiceUsbDeviceManager(SpiceUsbDeviceManager *manager)
{
    _spiceManager = manager;
    g_signal_connect(_spiceManager, "device-error", G_CALLBACK(SpiceUsbManager::usbDeviceErrorCb), this);
    g_signal_connect(_spiceManager, "device-added", G_CALLBACK(SpiceUsbManager::usbDeviceAddedCb), this);
    g_signal_connect(_spiceManager, "device-removed", G_CALLBACK(SpiceUsbManager::usbDeviceRemovedCb), this);
    resetUsbDevicesList();
}

void SpiceUsbManager::usbDeviceErrorCb(SpiceUsbDeviceManager *manager, SpiceUsbDevice *device, GError *err, void *data)
{
    Q_UNUSED(err)
    usbDeviceRemovedCb(manager, device, data);
}

void SpiceUsbManager::usbDeviceAddedCb(SpiceUsbDeviceManager *manager, SpiceUsbDevice *usbDevice, void *data)
{
    Q_UNUSED(manager)
    Q_UNUSED(usbDevice)
    SpiceUsbManager *me = static_cast<SpiceUsbManager *>(data);
    if (me)
        me->resetUsbDevicesList();
}

void SpiceUsbManager::usbDeviceRemovedCb(SpiceUsbDeviceManager *manager, SpiceUsbDevice *usbDevice, void *data)
{
    Q_UNUSED(manager)
    Q_UNUSED(usbDevice)
    SpiceUsbManager *me = static_cast<SpiceUsbManager *>(data);
    if (me)
        me->resetUsbDevicesList();
}

void SpiceUsbManager::usbConnectAsyncCb(GObject *gobject, GAsyncResult *res, gpointer user_data)
{
    Q_UNUSED(gobject)
    Q_UNUSED(res)
    Q_UNUSED(user_data)
    SpiceUsbManager *me = static_cast<SpiceUsbManager *>(user_data);
    if (me)
        me->resetUsbDevicesList();
}

void SpiceUsbManager::resetUsbDevicesList()
{
    _devices.clear();
    GPtrArray *devices = spice_usb_device_manager_get_devices(_spiceManager);
    if (devices) {
        for (int i = 0, size = devices->len ; i < size ; ++i) {
            SpiceUsbDevice *usbDevice = static_cast<SpiceUsbDevice *>(g_ptr_array_index(devices, i));
            gchar *desc = spice_usb_device_get_description(usbDevice, "%s=%s=%s=%d=%d");
            QByteArray descByteArray(desc);
            UsbDevice device = _createUsbDevice(QString(descByteArray));
            device.connected = spice_usb_device_manager_is_device_connected(_spiceManager, usbDevice);
            _devices.append(device);
            g_free(desc);
        }
    }
    emit devicesReset();
}

void SpiceUsbManager::connectUsbDevice(UsbDevice dev)
{
    GPtrArray *devices = spice_usb_device_manager_get_devices(_spiceManager);
    if (devices) {
        for (int i = 0, size = devices->len ; i < size ; ++i) {
            SpiceUsbDevice *usbDevice = static_cast<SpiceUsbDevice *>(g_ptr_array_index(devices, i));
            gchar *desc = spice_usb_device_get_description(usbDevice, "%s=%s=%s=%d=%d");
            QByteArray descByteArray(desc);
            UsbDevice device = _createUsbDevice(QString(descByteArray));
            device.connected = spice_usb_device_manager_is_device_connected(_spiceManager, usbDevice);
            g_free(desc);
            if (dev == device) {
                spice_usb_device_manager_connect_device_async(_spiceManager, usbDevice, NULL, SpiceUsbManager::usbConnectAsyncCb, this);
                return;
            }
        }
    }
}

void SpiceUsbManager::disconnectUsbDevice(UsbDevice dev)
{
    GPtrArray *devices = spice_usb_device_manager_get_devices(_spiceManager);
    if (devices) {
        for (int i = 0, size = devices->len ; i < size ; ++i) {
            SpiceUsbDevice *usbDevice = static_cast<SpiceUsbDevice *>(g_ptr_array_index(devices, i));
            gchar *desc = spice_usb_device_get_description(usbDevice, "%s=%s=%s=%d=%d");
            QByteArray descByteArray(desc);
            UsbDevice device = _createUsbDevice(QString(descByteArray));
            device.connected = spice_usb_device_manager_is_device_connected(_spiceManager, usbDevice);
            g_free(desc);
            if (dev == device) {
                spice_usb_device_manager_disconnect_device(_spiceManager, usbDevice);
                resetUsbDevicesList();
                return;
            }
        }
    }
}

void SpiceUsbManager::disconnectAllUsbDevices()
{
    GPtrArray *devices = spice_usb_device_manager_get_devices(_spiceManager);
    if (devices) {
        for (int i = 0, size = devices->len ; i < size ; ++i) {
            SpiceUsbDevice *usbDevice = static_cast<SpiceUsbDevice *>(g_ptr_array_index(devices, i));
            bool connected = spice_usb_device_manager_is_device_connected(_spiceManager, usbDevice);
            if (connected) {
                spice_usb_device_manager_disconnect_device(_spiceManager, usbDevice);
                resetUsbDevicesList();
            }
        }
    }
}

bool SpiceUsbManager::hasDeviceConnected()
{
    GPtrArray *devices = spice_usb_device_manager_get_devices(_spiceManager);
    if (devices) {
        for (int i = 0, size = devices->len ; i < size ; ++i) {
            SpiceUsbDevice *usbDevice = static_cast<SpiceUsbDevice *>(g_ptr_array_index(devices, i));
            bool connected = spice_usb_device_manager_is_device_connected(_spiceManager, usbDevice);
            if (connected)
                return true;
        }
    }
    return false;
}

UsbDevice SpiceUsbManager::_createUsbDevice(const QString &desc)
{
    UsbDevice device;
    QStringList list = QString(desc).split("=");
    device.manufacturer = list.takeFirst();
    device.product = list.takeFirst();
    device.descriptor = list.takeFirst();
    device.bus = list.takeFirst();
    device.address = list.takeFirst();
    return device;
}

} //RemoteAccess
