// Project headers
#include "spiceworker.h"
#include "display.h"
#include "ressources.h"

// Spice headers
#include <spice-1/spice/vd_agent.h>

// Qt headers
#include <QDebug>
#include <QSettings>
#include <QX11Info>

// X11 headers
#include <X11/XKBlib.h>

#include "spiceusbmanager.h"

namespace RemoteAccess {

Q_GLOBAL_STATIC(KeyHash, staticKeyHash)

SpiceWorker::SpiceWorker(QObject *parent)
  : Worker(parent)
  , _session(0)
  , _inputChannel(0)
  , _spiceUsbManager(new SpiceUsbManager(this))
  , _resolutionSize(0, 0)
  , _imgdata(0)
  , _agentConnected(false)
  , _buttonMask(0)
  , _mouseMode(0)
  , _mouseLastX(-1)
  , _mouseLastY(-1)
  , _canRedirectUsb(false)
{
    /*function deprecated since GLib 2.36
     *GLib version in wheezy: 2.33
     *in version 2.36 does nothing*/
#if GLIB_CHECK_VERSION(2,36,0) == false
    g_type_init();
#endif
    QSettings settings;
    QString resolution = settings.value(Hnv::Constants::REMOTEDISPLAY_RESOLUTION_OPTION).toString();
    if (resolution.isEmpty())
        resolution = "640x480";
    QStringList resolutionList = resolution.split("x");
    int width = resolutionList.at(0).toInt();
    int height = resolutionList.at(1).toInt();
    _resolutionSize = QSize(width, height);
}

SpiceWorker::~SpiceWorker()
{
    g_object_unref(_session);
}

void SpiceWorker::openConnection()
{
    if (_session)
        return;
    _session = spice_session_new();
    spice_set_session_option(_session);
    QString host = QString::fromStdString(_info.host);
    QString port = QString::number(_info.port);
    QString password = QString::fromStdString(_info.password);
    bool tls = _info.tls;
    /*g_signal_connect
     *1st argument : sender
     *2nd : signal
     *3rd : callback
     *4th : context, here the mainwindow, todo: change it in a worker
     */
    g_signal_connect(_session, "channel-new", G_CALLBACK(SpiceWorker::hns_channel_new), this);
    g_signal_connect(_session, "channel-destroy", G_CALLBACK(SpiceWorker::hns_channel_destroy), this);

    g_object_set(_session, "host", host.toStdString().c_str(), NULL);
    g_object_set(_session, "password", password.toStdString().c_str(), NULL);
    if (tls) {
        QString ca = QString::fromStdString(_info.ca);
        QString certSubject = QString::fromStdString(_info.certSubject);
        g_object_set(_session, "tls-port", port.toStdString().c_str(), NULL);
        GByteArray *caArray = g_byte_array_new();
        g_byte_array_append(caArray, (guint8*)ca.toStdString().c_str(), ca.toStdString().length());
        g_object_set(_session, "ca", caArray, NULL);
        g_byte_array_unref(caArray);
        g_object_set(_session, "cert-subject", certSubject.toStdString().c_str(), NULL);
        gchar allChannels[] = "all";
        gchar *secure_channels[] = { allChannels, NULL };
        g_object_set(_session, "secure-channels", secure_channels, NULL);
    } else {
        g_object_set(_session, "port", port.toStdString().c_str(), NULL);
    }
    g_object_set(_session, "enable_usbredir", TRUE, NULL);

    GError *err = 0;
    SpiceUsbDeviceManager *spiceUsbDeviceMgr = spice_usb_device_manager_get(_session, &err);
    _spiceUsbManager->setSpiceUsbDeviceManager(spiceUsbDeviceMgr);

    spice_session_connect(_session);
}

void SpiceWorker::closeConnection()
{
    /*ending*/
    disconnect_spice();
}

Display *SpiceWorker::createDisplay()
{
    ImageDisplay *display = new ImageDisplay();
    connect(this, SIGNAL(imageChanged(QImage, int, int, int, int)), display, SIGNAL(imageChanged(QImage, int, int, int, int)));
    connect(this, SIGNAL(cursorChanged(QImage,int,int)), display, SIGNAL(cursorChanged(QImage, int, int)));
    connect(this, SIGNAL(sizeChanged(QSize)), display, SIGNAL(sizeChanged(QSize)));
    connect(this, SIGNAL(clipboardDataCleared()), display, SIGNAL(clipboardDataClearedGuestToHost()));
    connect(this, SIGNAL(clipboardDataEvent(QString)), display, SIGNAL(clipboardDataGuestToHost(QString)));
    connect(this, SIGNAL(clipboardSelectionCleared()), display, SIGNAL(clipboardSelectionClearedGuestToHost()));
    connect(this, SIGNAL(clipboardSelectionEvent(QString)), display, SIGNAL(clipboardSelectionGuestToHost(QString)));
    connect(display, SIGNAL(keyboardEventReceived(RemoteAccess::KeyInfo)), this, SLOT(keyboardEventReceive(RemoteAccess::KeyInfo)));
    connect(display, SIGNAL(focusOutEvent()), this, SLOT(releaseAllKeysSlot()));
    connect(display, SIGNAL(mouseEventReceived(RemoteAccess::MouseInfo)), this, SLOT(mouseEventReceive(RemoteAccess::MouseInfo)));
    connect(display, SIGNAL(clipboardDataHostToGuest(QString)), this, SLOT(clipboardDataEventReceive(QString)));
    connect(display, SIGNAL(clipboardSelectionHostToGuest(QString)), this, SLOT(clipboardSelectionEventReceive(QString)));
    return display;
}

UsbManager *SpiceWorker::usbManager()
{
    return _spiceUsbManager;
}

bool SpiceWorker::canRedirectUsb()
{
    return _canRedirectUsb;
}

void SpiceWorker::keyboardEventReceive(RemoteAccess::KeyInfo keyInfo)
{
    // New keyboard event from local client
    if (!_inputChannel)
        return;
    quint32 nativeVirtualCode = keyInfo.nativeVirtualCode;
    if (!staticKeyHash()->contains(nativeVirtualCode)) {
        qDebug() << QObject::tr("Bad hardware keyboard code");
        return;
    }
    int scancode = staticKeyHash()->value(nativeVirtualCode);
    if (keyInfo.press) {
        _pressedKeys.append(scancode);
        spice_inputs_key_press(_inputChannel, scancode);
    } else {
        spice_inputs_key_release(_inputChannel, scancode);
        _pressedKeys.removeAll(scancode);
    }
}

int SpiceWorker::getKeyboardLockState()
{
    int result = 0;
    unsigned state;
    XkbGetIndicatorState(QX11Info::display(), XkbUseCoreKbd, &state);
    if ( state & 0x01 ) // Caps lock is set
        result |= ( 4 << 0 );
    if ( state & 0x02 ) // Num lock is set
        result |= ( 2 << 0 );
    if ( state & 0x04 ) // Scroll lock is set
        result |= ( 1 << 0 );
    return result;
}

void SpiceWorker::mouseEventReceive(RemoteAccess::MouseInfo mouseInfo)
{
    // New mouse event from local client
    if (!_inputChannel)
        return;
    if (mouseInfo.state == Move) {
        handle_mouseMove(mouseInfo.x, mouseInfo.y);
    } else if (mouseInfo.state == WheelUp) {
        handle_mouseWheel(true);
    } else if (mouseInfo.state == WheelDown) {
        handle_mouseWheel(false);
    } else {
        SpiceMouseButtonMask buttonMask;
        SpiceMouseButton button = SPICE_MOUSE_BUTTON_INVALID;
        switch (mouseInfo.button) {
        case Left:
            button = SPICE_MOUSE_BUTTON_LEFT;
            buttonMask = SPICE_MOUSE_BUTTON_MASK_LEFT;
            break;
        case Right:
            button = SPICE_MOUSE_BUTTON_RIGHT;
            buttonMask = SPICE_MOUSE_BUTTON_MASK_RIGHT;
            break;
        case Mid:
            button = SPICE_MOUSE_BUTTON_MIDDLE;
            buttonMask = SPICE_MOUSE_BUTTON_MASK_MIDDLE;
        default:
            break;
        }
        if (mouseInfo.state == Click || mouseInfo.state == DoubleClick) {
            spice_inputs_button_press(_inputChannel, button, 0);
            _buttonMask |= buttonMask;
        } else if (mouseInfo.state == Release) {
            spice_inputs_button_release(_inputChannel, button, 0);
            _buttonMask ^= buttonMask;
        }
    }
}

void SpiceWorker::clipboardDataEventReceive(QString text)
{
    // Local client has new clipboard data
    _handleClipboard(text, VD_AGENT_CLIPBOARD_SELECTION_CLIPBOARD);
}

void SpiceWorker::clipboardSelectionEventReceive(QString text)
{
    // Local client has new clipboard data
    _handleClipboard(text, VD_AGENT_CLIPBOARD_SELECTION_PRIMARY);
}

void SpiceWorker::releaseAllKeysSlot()
{
    foreach(quint32 scanCode, _pressedKeys) {
        spice_inputs_key_release(_inputChannel, scanCode);
    }
    _pressedKeys.clear();
}

void SpiceWorker::hns_channel_new(SpiceSession *session, SpiceChannel *channel, void *data)
{
    if (!session)
        return;
    SpiceWorker *worker = static_cast<SpiceWorker *>(data);
    if (!worker)
        return;
    if (SPICE_IS_MAIN_CHANNEL(channel)) {
        worker->setMainChannel(SPICE_MAIN_CHANNEL(channel));
        g_signal_connect(channel, "channel-event", G_CALLBACK(SpiceWorker::hns_main_channel_event), worker);
        g_signal_connect(channel, "main-agent-update", G_CALLBACK(SpiceWorker::hns_main_agent_update), worker);
        g_signal_connect(channel, "main-clipboard-selection-grab", G_CALLBACK(SpiceWorker::hns_main_clipboard_selection_grab), worker);
        g_signal_connect(channel, "main-clipboard-selection", G_CALLBACK(SpiceWorker::hns_main_clipboard_selection), worker);
        g_signal_connect(channel, "main-clipboard-selection-release", G_CALLBACK(SpiceWorker::hns_main_clipboard_selection_release), worker);
        g_signal_connect(channel, "main-clipboard-selection-request", G_CALLBACK(SpiceWorker::hns_main_clipboard_selection_request), worker);
        g_signal_connect(channel, "main-mouse-update", G_CALLBACK(SpiceWorker::hns_main_mouse_update), worker);
        spice_channel_connect(channel);
    } else if (SPICE_IS_DISPLAY_CHANNEL(channel)) {
        g_signal_connect(channel, "channel-event", G_CALLBACK(SpiceWorker::hns_main_channel_event), worker);
        g_signal_connect(channel, "display-primary-create", G_CALLBACK(SpiceWorker::hns_display_primary_create), worker);
        g_signal_connect(channel, "display-primary-destroy", G_CALLBACK(SpiceWorker::hns_display_primary_destroy), worker);
        g_signal_connect(channel, "display-invalidate", G_CALLBACK(SpiceWorker::hns_display_invalidate), worker);
        spice_channel_connect(channel);
    } if (SPICE_IS_INPUTS_CHANNEL(channel)) {
        worker->setInputChannel(SPICE_INPUTS_CHANNEL(channel));
        g_signal_connect(channel, "channel-event", G_CALLBACK(SpiceWorker::hns_main_channel_event), worker);
        spice_channel_connect(channel);
        spice_inputs_set_key_locks(SPICE_INPUTS_CHANNEL(channel), worker->getKeyboardLockState());
    } else if (SPICE_IS_CURSOR_CHANNEL(channel)) {
        g_signal_connect(channel, "channel-event", G_CALLBACK(SpiceWorker::hns_main_channel_event), worker);
        g_signal_connect(channel, "cursor-set", G_CALLBACK(SpiceWorker::hns_cursor_set), worker);
        g_signal_connect(channel, "cursor-reset", G_CALLBACK(SpiceWorker::hns_cursor_reset), worker);
        spice_channel_connect(channel);
    } else if (SPICE_IS_USBREDIR_CHANNEL(channel)) {
        worker->_canRedirectUsb = true;
        g_signal_connect(channel, "channel-event", G_CALLBACK(SpiceWorker::hns_main_channel_event), worker);
        spice_channel_connect(channel);
        emit worker->canRedirectUsbChanged();
    }
}

void SpiceWorker::hns_channel_destroy(SpiceSession *session, SpiceChannel *channel, void *data)
{
    if (!session)
        return;
    SpiceWorker *worker = static_cast<SpiceWorker *>(data);
    if (!worker)
        return;
    if (SPICE_IS_MAIN_CHANNEL(channel)) {
        g_signal_handlers_disconnect_by_func(channel, (gpointer)G_CALLBACK(SpiceWorker::hns_main_agent_update), worker);
        g_signal_handlers_disconnect_by_func(channel, (gpointer)G_CALLBACK(SpiceWorker::hns_main_clipboard_selection_grab), worker);
        g_signal_handlers_disconnect_by_func(channel, (gpointer)G_CALLBACK(SpiceWorker::hns_main_clipboard_selection), worker);
        g_signal_handlers_disconnect_by_func(channel, (gpointer)G_CALLBACK(SpiceWorker::hns_main_clipboard_selection_release), worker);
        g_signal_handlers_disconnect_by_func(channel, (gpointer)G_CALLBACK(SpiceWorker::hns_main_clipboard_selection_request), worker);
        g_signal_handlers_disconnect_by_func(channel, (gpointer)G_CALLBACK(SpiceWorker::hns_main_mouse_update), worker);
        worker->setMainChannel(0);
    } else if (SPICE_IS_DISPLAY_CHANNEL(channel)) {
        g_signal_handlers_disconnect_by_func(channel, (gpointer)G_CALLBACK(SpiceWorker::hns_display_primary_create), worker);
        g_signal_handlers_disconnect_by_func(channel, (gpointer)G_CALLBACK(SpiceWorker::hns_display_primary_destroy), worker);
        g_signal_handlers_disconnect_by_func(channel, (gpointer)G_CALLBACK(SpiceWorker::hns_display_invalidate), worker);
        worker->setImageData(0);
        // TODO: update display
    } else if (SPICE_IS_INPUTS_CHANNEL(channel)) {
        worker->setInputChannel(0);
    } else if (SPICE_IS_CURSOR_CHANNEL(channel)) {
        g_signal_handlers_disconnect_by_func(channel, (gpointer)G_CALLBACK(SpiceWorker::hns_cursor_set), worker);
        g_signal_handlers_disconnect_by_func(channel, (gpointer)G_CALLBACK(SpiceWorker::hns_cursor_reset), worker);
    }
}

void SpiceWorker::hns_main_channel_event(SpiceChannel *channel, SpiceChannelEvent event, void *data)
{
    if (!channel)
        return;
    SpiceWorker *worker = static_cast<SpiceWorker *>(data);
    if (!worker)
        return;
    switch (event) {
    case SPICE_CHANNEL_OPENED:
        worker->_openedChannels.append(channel);
        break;
    case SPICE_CHANNEL_SWITCHING:
        break;
    case SPICE_CHANNEL_CLOSED:
        /* this event is only sent if the channel was succesfully opened before */
        g_signal_handlers_disconnect_by_func(channel, (gpointer)G_CALLBACK(SpiceWorker::hns_main_channel_event), worker);
        worker->_openedChannels.removeAll(channel);
        if (worker->_openedChannels.isEmpty()) {
            g_signal_handlers_disconnect_by_func(worker->_session, (gpointer)G_CALLBACK(SpiceWorker::hns_channel_new), worker);
            g_signal_handlers_disconnect_by_func(worker->_session, (gpointer)G_CALLBACK(SpiceWorker::hns_channel_destroy), worker);
            /* Follow up
             * RemoteClient::connectionEnd
             * RemoteClient::connectionEnded
             * RemoteManager::connectionEnded
             * TopologyContainer::_connectionEndedSlot
             * TopoplogyContainer::remoteViewClosed -- RemoteView::deleteLater
             * RemoteManager::remoteViewClosedSlot
             * delete RemoteClient
             * Worker::deleteLater
             * delete Display
             */
            worker->disconnect_spice();
            emit worker->connectionEnded();
        }
        break;
    case SPICE_CHANNEL_ERROR_IO:
        /* Follow up
         * RemoteClient::connectionErrorSlot
         * RemoteClient::connectionError
         * RemoteManager::connectionError
         * TopologyContainer::_errorMessageSlot
         */

        // Temporary workaround: we do not want to display a pop-up until server side for LXC domains is handled better
        // emit worker->connectionError("Spice channel input/output error");
        qDebug() << "Spice channel input/output error";
        worker->disconnect_spice();
        break;
    case SPICE_CHANNEL_ERROR_TLS:
    case SPICE_CHANNEL_ERROR_LINK:
    case SPICE_CHANNEL_ERROR_CONNECT:
        emit worker->connectionError("Spice channel failed to connect");
        emit worker->connectionEnded();
        worker->disconnect_spice();
        break;
    case SPICE_CHANNEL_ERROR_AUTH:
        /*should never append in hynesim*/
        emit worker->connectionError("Spice authentication error");
        worker->disconnect_spice();
        break;
    default:
        emit worker->connectionError("Unrecognized spice error");
        worker->disconnect_spice();
        break;
    }
}

void SpiceWorker::hns_main_agent_update(SpiceChannel *channel, void *data)
{
    if (!channel)
        return;
    bool agentConnectedState = false;
    g_object_get(channel, "agent-connected", &agentConnectedState, NULL);
    SpiceWorker *worker = 0;
    worker = static_cast<SpiceWorker *>(data);
    if (!worker)
        return;
    worker->setAgentConnected(agentConnectedState);
}

void SpiceWorker::hns_main_mouse_update(SpiceChannel *channel, void *data)
{
    if (!channel)
        return;
    SpiceWorker *worker = 0;
    worker = static_cast<SpiceWorker *>(data);
    if (!worker)
        return;
    g_object_get(channel, "mouse-mode", &worker->_mouseMode, NULL);
}

void SpiceWorker::hns_display_primary_create(SpiceDisplayChannel *display, int format, int width, int height, int stride, int shmid, void *imgdata, void *data)
{
    Q_UNUSED(format)
    Q_UNUSED(stride)
    Q_UNUSED(shmid)
    /*todo: check image format*/
    if (!display)
        return;
    SpiceWorker *worker = static_cast<SpiceWorker *>(data);
    if (!worker)
        return;

    int new_width = std::max(width, worker->_resolutionSize.width());
    int new_height = std::max(height, worker->_resolutionSize.height());
    if(new_height != height || new_width != width) {
        spice_main_set_display_enabled(worker->_mainChannel, 0, TRUE);
        spice_main_update_display(worker->_mainChannel, 0, 0, 0, new_width, new_height, TRUE);
    }

    worker->setImageData(imgdata);
    worker->setResolution(QSize(width, height));
    emit worker->sizeChanged(QSize(width, height));
    QImage image((uchar *)imgdata, width, height, QImage::Format_RGB32);
    emit worker->imageChanged(image, 0, 0, width, height);
}

void SpiceWorker::hns_display_primary_destroy(SpiceDisplayChannel *display, void *data)
{
    if (!display)
        return;
    SpiceWorker *worker = static_cast<SpiceWorker *>(data);
    if (!worker)
        return;
    worker->setImageData(0);
    worker->setResolution(QSize(0, 0));
    emit worker->sizeChanged(worker->resolution());
}

void SpiceWorker::hns_display_invalidate(SpiceDisplayChannel *display, int x, int y, int w, int h, void *data)
{
    if (!display)
        return;
    /*todo: check image format*/
    SpiceWorker *worker = static_cast<SpiceWorker *>(data);
    if (!worker)
        return;
    if (worker->resolution().isNull())
        return;
    QImage image((uchar *)worker->imageData(), worker->resolution().width(), worker->resolution().height(), QImage::Format_RGB32);
    emit worker->imageChanged(image.copy(x, y, w, h), x, y, w, h);
}

void SpiceWorker::hns_cursor_set(SpiceCursorChannel *channel, int width, int height, int hotX, int hotY, void *rgba, void *data)
{
    if (!channel)
        return;
    /*todo: this does not seem to be called, check*/
    SpiceWorker *worker = static_cast<SpiceWorker *>(data);
    if (!worker)
        return;
    QImage imgCursor((uchar*)rgba, width, height, QImage::Format_ARGB32);
    emit worker->cursorChanged(imgCursor, hotX, hotY);
}

void SpiceWorker::hns_cursor_reset(SpiceCursorChannel *channel, void *data)
{
    if (!channel)
        return;
    SpiceWorker *worker = static_cast<SpiceWorker *>(data);
    if (!worker)
        return;
    QImage img;
    emit worker->cursorChanged(img, 0, 0);
}

bool SpiceWorker::hns_main_clipboard_selection_grab(SpiceMainChannel *mainChannel, uint selection, uint *types, uint ntypes, void *data)
{
    // Guest has new clipboard data available
    Q_UNUSED(data)
    if (selection != VD_AGENT_CLIPBOARD_SELECTION_CLIPBOARD && selection != VD_AGENT_CLIPBOARD_SELECTION_PRIMARY)
        return false;
    uint supportedType = VD_AGENT_CLIPBOARD_NONE;
    uint i;
    for(i = 0; i < ntypes; ++i) {
        if (types[i] == VD_AGENT_CLIPBOARD_UTF8_TEXT) {
            supportedType = types[i];
            break;
        }
    }
    if (supportedType != VD_AGENT_CLIPBOARD_NONE) {
        // Request clibpoard data from the Guest -- Callback: hns_main_clipboard_selection
        spice_main_clipboard_selection_request(mainChannel, selection, supportedType);
        return true;
    }
    return false;
}

void SpiceWorker::hns_main_clipboard_selection(SpiceMainChannel *mainChannel, uint selection, uint type, uchar *data, uint size, void *user_data)
{
    // Guest sent clipboard data
    Q_UNUSED(mainChannel)
    if (type != VD_AGENT_CLIPBOARD_UTF8_TEXT)
        return;
    SpiceWorker *worker = static_cast<SpiceWorker *>(user_data);
    if (!worker)
        return;
    QString guestClipboard = QString::fromUtf8((char *)data, size);
    if (selection == VD_AGENT_CLIPBOARD_SELECTION_CLIPBOARD) {
        /* Follow up
         * SpiceWorker::clipboardDataEvent
         * ImageDisplay::clipboardDataGuestToHost
         * ImageRemoteView::clipboardDataEventSlot
         */
        worker->_clipboardDataString = guestClipboard;
        emit worker->clipboardDataEvent(guestClipboard);
    } else if (selection == VD_AGENT_CLIPBOARD_SELECTION_PRIMARY) {
        /* Follow up
         * SpiceWorker::clipboardDataEvent
         * ImageDisplay::clipboardSelectionGuestToHost
         * ImageRemoteView::clipboardSelectionEventSlot
         */
        worker->_clipboardSelectionString = guestClipboard;
        emit worker->clipboardSelectionEvent(guestClipboard);
    }
}

void SpiceWorker::hns_main_clipboard_selection_release(SpiceMainChannel *mainChannel, uint selection, void *data)
{
    // Guest released the clipboard or has no clipboard data available
    Q_UNUSED(mainChannel)
    SpiceWorker *worker = static_cast<SpiceWorker *>(data);
    if (!worker)
        return;
    if (selection == VD_AGENT_CLIPBOARD_SELECTION_CLIPBOARD)
        emit worker->clipboardDataCleared();
    else if (selection == VD_AGENT_CLIPBOARD_SELECTION_PRIMARY)
        emit worker->clipboardSelectionCleared();
}

bool SpiceWorker::hns_main_clipboard_selection_request(SpiceMainChannel *mainChannel, uint selection, uint types, void *data)
{
    // Guest is requesting local clipboard content
    if (types != VD_AGENT_CLIPBOARD_UTF8_TEXT)
        return false;
    SpiceWorker *worker = static_cast<SpiceWorker *>(data);
    if (!worker)
        return false;
    QString localClipboard;
    if (selection == VD_AGENT_CLIPBOARD_SELECTION_CLIPBOARD)
        localClipboard = worker->clipboardDataString();
    else if (selection == VD_AGENT_CLIPBOARD_SELECTION_PRIMARY)
        localClipboard = worker->clipboardSelectionString();
    if (localClipboard.isEmpty())
        return false;
    QByteArray ba = localClipboard.toUtf8();
    // Sending local clipboard to guest
    spice_main_clipboard_selection_notify(mainChannel, selection, VD_AGENT_CLIPBOARD_UTF8_TEXT, (uchar*)ba.constData(), ba.size());
    return true;
}

void SpiceWorker::disconnect_spice()
{
    _spiceUsbManager->disconnectAllUsbDevices();
    spice_session_disconnect(_session);
}

void SpiceWorker::_handleClipboard(QString text, uint selectionType)
{
    // Local clipboard has new data available, spice needs to grab the guest clipboard
    if (!_agentConnected || !_mainChannel)
        return;
    quint32 types[1];
    types[0] = VD_AGENT_CLIPBOARD_UTF8_TEXT;
    if (selectionType == VD_AGENT_CLIPBOARD_SELECTION_CLIPBOARD) {
        if (_clipboardDataString == text)
            return;
        _clipboardDataString = text;
    } else if (selectionType == VD_AGENT_CLIPBOARD_SELECTION_PRIMARY) {
        if (_clipboardSelectionString == text)
            return;
        _clipboardSelectionString = text;
    } else {
        return;
    }
    if (text.isEmpty())
        spice_main_clipboard_selection_release(_mainChannel, selectionType);
    else
        spice_main_clipboard_selection_grab(_mainChannel, selectionType, types, 1);
}

void SpiceWorker::setImageData(void *imgData)
{
    _imgdata = (uchar *)imgData;
}

void *SpiceWorker::imageData()
{
    return _imgdata;
}

void SpiceWorker::setResolution(QSize size)
{
    _resolutionSize = size;
}

QSize SpiceWorker::resolution()
{
    return _resolutionSize;
}

void SpiceWorker::setInputChannel(SpiceInputsChannel *inputChannel)
{
    _inputChannel = inputChannel;
}

void SpiceWorker::setMainChannel(SpiceMainChannel *mainChannel)
{
    _mainChannel = mainChannel;
}

void SpiceWorker::setAgentConnected(bool connected)
{
    _agentConnected = connected;
}

bool SpiceWorker::agentConnected()
{
    return _agentConnected;
}

QString SpiceWorker::clipboardDataString()
{
    return _clipboardDataString;
}

QString SpiceWorker::clipboardSelectionString()
{
    return _clipboardSelectionString;
}

void SpiceWorker::handle_mouseMove(int xPos, int yPos)
{
    if (!_inputChannel)
        return;
    switch (_mouseMode) {
    case SPICE_MOUSE_MODE_CLIENT:
    {
        spice_inputs_position(_inputChannel, xPos, yPos, 0, _buttonMask);
        break;
    }
    case SPICE_MOUSE_MODE_SERVER:
    {
        int dx = _mouseLastX != -1 ? xPos - _mouseLastX : 0;
        int dy = _mouseLastY != -1 ? yPos - _mouseLastY : 0;
        spice_inputs_motion(_inputChannel, dx, dy, _buttonMask);
        _mouseLastX = xPos;
        _mouseLastY = yPos;
        break;
    }
    default:
        break;
    }
}

void SpiceWorker::handle_mouseWheel(bool wheelUp)
{
    if (!_inputChannel)
        return;
    SpiceMouseButton button = wheelUp ? SPICE_MOUSE_BUTTON_UP : SPICE_MOUSE_BUTTON_DOWN;
    spice_inputs_button_press(_inputChannel, button, 0);
    spice_inputs_button_release(_inputChannel, button, 0);
}

} // namespace RemoteAccess
