/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

extern "C" {
#include <gvnc-1.0/gvnc.h>
}

// Project headers
#include "vncworker.h"
#include "display.h"

// Qt headers
#include <QThread>
#include <QDebug>
#include <QCoreApplication>

namespace RemoteAccess {

VncWorker::VncWorker(QObject *parent)
    : Worker(parent)
    , _connection(0)
    , _baseFrameBuffer(0)
    , _width(0)
    , _height(0)
    , _buttonMask(0)
{
    /*function deprecated since GLib 2.36
     *GLib version in wheezy: 2.33
     *in version 2.36 does nothing*/
#if GLIB_CHECK_VERSION(2,36,0) == false
    g_type_init();
#endif

    _connection = vnc_connection_new();
    g_signal_connect(G_OBJECT(_connection), "vnc-cursor-changed",
                     G_CALLBACK(VncWorker::hns_cursor_changed), this);
    g_signal_connect(G_OBJECT(_connection), "vnc-framebuffer-update",
                     G_CALLBACK(VncWorker::hns_framebuffer_update), this);
    g_signal_connect(G_OBJECT(_connection), "vnc-desktop-resize",
                     G_CALLBACK(VncWorker::hns_desktop_resize), this);
    g_signal_connect(G_OBJECT(_connection), "vnc-auth-choose-type",
                     G_CALLBACK(VncWorker::hns_auth_choose_type), this);
    g_signal_connect(G_OBJECT(_connection), "vnc-initialized",
                     G_CALLBACK(VncWorker::hns_initialized), this);
    g_signal_connect(G_OBJECT(_connection), "vnc-disconnected",
                     G_CALLBACK(VncWorker::hns_disconnected), this);
}

VncWorker::~VncWorker()
{
    g_object_unref(_connection);
    g_object_run_dispose(G_OBJECT(_connection));
}

void VncWorker::openConnection()
{
    QString host = QString::fromStdString(_info.host);
    QString port = QString::number(_info.port);

    if(!vnc_connection_set_credential(_connection, VNC_CONNECTION_CREDENTIAL_PASSWORD, _info.password.c_str())) {
        qDebug() << "VNC connection: unable to set credentials";
    }
    if (!vnc_connection_open_host(_connection, host.toLatin1().constData(), port.toLatin1().constData())) {
        emit connectionError(QObject::tr("Unable to connect"));
        emit connectionEnded();
    }
}

void VncWorker::closeConnection()
{
    vnc_connection_shutdown(_connection);
}

Display* VncWorker::createDisplay()
{
    ImageDisplay *display = new ImageDisplay();
    connect(this, SIGNAL(imageChanged(QImage, int, int, int, int)), display, SIGNAL(imageChanged(QImage, int, int, int, int)));
    connect(this, SIGNAL(cursorChanged(QImage,int,int)), display, SIGNAL(cursorChanged(QImage, int, int)));
    connect(this, SIGNAL(sizeChanged(QSize)), display, SIGNAL(sizeChanged(QSize)));
    connect(this, SIGNAL(clearImaged()), display, SIGNAL(clearImaged()));
    connect(display, SIGNAL(keyboardEventReceived(RemoteAccess::KeyInfo)), this, SLOT(keyboardEventReceive(RemoteAccess::KeyInfo)));
    connect(display, SIGNAL(mouseEventReceived(RemoteAccess::MouseInfo)), this, SLOT(mouseEventReceive(RemoteAccess::MouseInfo)));
    return display;
}

void VncWorker::mouseEventReceive(RemoteAccess::MouseInfo info)
{
    int buttonMask = 0;
    int button = 0;
    if (info.button == Left) {
        button = 1;
    } else if (info.button == Mid) {
        button = 1 << 1;
    } else if (info.button == Right) {
        button = 1 << 2;
    }
    switch (info.state) {
    case Click:
    case DoubleClick:
        buttonMask |= button;
        break;
    case Release:
        buttonMask &= ~button;
        break;
    case WheelUp:
        buttonMask = (1 << 3);
        break;
    case WheelDown:
        buttonMask = (1 << 4);
        break;
    case Move:
        buttonMask = _buttonMask;
    default:
        break;
    }

    _buttonMask = buttonMask;
    vnc_connection_pointer_event(_connection, buttonMask, info.x, info.y);
}

void VncWorker::keyboardEventReceive(KeyInfo info)
{
    int down_flag = info.press ? 1 : 0;
    vnc_connection_key_event(_connection, down_flag, info.nativeVirtualKey, info.nativeVirtualCode);
}

void VncWorker::hns_cursor_changed(VncConnection *connection, VncCursor *cursor, void *data)
{
    Q_UNUSED(connection)
    VncWorker *me = static_cast<VncWorker *>(data);
    if (!me)
        return;

    if (!cursor)
        return;

    int hotX = vnc_cursor_get_hotx(cursor);
    int hotY = vnc_cursor_get_hoty(cursor);
    int width = vnc_cursor_get_width(cursor);
    int height = vnc_cursor_get_height(cursor);
    const guint8 *pixels = vnc_cursor_get_data(cursor);
    QImage image((uchar *)pixels, width, height, QImage::Format_ARGB32);
    emit me->cursorChanged(image, hotX, hotY);
}

void VncWorker::hns_framebuffer_update(VncConnection *connection, int x, int y, int w, int h, void *data)
{
    VncWorker *me = static_cast<VncWorker *>(data);
    if (!me)
        return;

    guint8 *pixels = vnc_framebuffer_get_buffer(VNC_FRAMEBUFFER(me->_baseFrameBuffer));
    QImage image(pixels, me->_width, me->_height, QImage::Format_RGB32);
    emit me->imageChanged(image.copy(x, y, w, h), x, y, w, h);
    vnc_connection_framebuffer_update_request(connection, 1, 0, 0, me->_width, me->_height);
}

void VncWorker::hns_desktop_resize(VncConnection *connection, int width, int height, void *data)
{
    Q_UNUSED(width)
    Q_UNUSED(height)
    VncWorker *me = static_cast<VncWorker *>(data);
    if (!me)
        return;

    me->_initFrameBuffer();
    vnc_connection_framebuffer_update_request(connection, 0, 0, 0, me->_width, me->_height); //todo: check + close connection
}

void VncWorker::hns_auth_choose_type(VncConnection *connection, GValueArray *types, void *data)
{
    Q_UNUSED(types)
    VncWorker *me = static_cast<VncWorker *>(data);
    if (!me)
        return;

    if (!vnc_connection_set_auth_type(connection, VNC_CONNECTION_AUTH_VNC)) {
        emit me->connectionError(QObject::tr("Unable to set the authentification type"));
        me->closeConnection();
    }
}

void VncWorker::hns_initialized(VncConnection *connection, void *data)
{
    VncWorker *me = static_cast<VncWorker *>(data);
    if (!me) {
        qCritical() << "Unable to get me"; //todo
        return;
    }

    me->_initFrameBuffer();

    //set encoding
    int encoding[4];
    encoding[0] = VNC_CONNECTION_ENCODING_CURSOR_POS;
    encoding[1] = VNC_CONNECTION_ENCODING_RICH_CURSOR;
    encoding[2] = VNC_CONNECTION_ENCODING_XCURSOR;
    encoding[3] = VNC_CONNECTION_ENCODING_DESKTOP_RESIZE;
    vnc_connection_set_encodings(connection, 4, encoding);

    //requesting first framebuffer update
    vnc_connection_framebuffer_update_request(connection, 0, 0, 0, me->_width, me->_height); //todo: check + close connection

}

void VncWorker::hns_disconnected(VncConnection *connection, void *data)
{
    Q_UNUSED(connection)
    VncWorker *me = static_cast<VncWorker *>(data);
    if (!me)
        return;

    emit me->connectionEnded();
}

void VncWorker::_initFrameBuffer()
{
    if (_baseFrameBuffer) {
        g_object_unref(_baseFrameBuffer);
        _baseFrameBuffer = 0;
    }
    const VncPixelFormat *pixelFormat = vnc_connection_get_pixel_format(_connection);
    int width = vnc_connection_get_width(_connection);
    int height = vnc_connection_get_height(_connection);

    guint8 *pixels= (guint8 *)malloc(width * height * 4 * sizeof(guint8));
    memset(pixels, 5, width * height * 4);

    VncPixelFormat localFormat;
    localFormat.red_max = 255;
    localFormat.green_max = 255;
    localFormat.blue_max = 255;

    localFormat.red_shift = 16;
    localFormat.green_shift = 8;
    localFormat.blue_shift = 0;

    localFormat.depth = 32;
    localFormat.bits_per_pixel = 32;

    localFormat.byte_order = G_BYTE_ORDER;

    _baseFrameBuffer = vnc_base_framebuffer_new(pixels,
                                               width,
                                               height,
                                               width * 4,
                                               &localFormat,
                                               pixelFormat);

    vnc_connection_set_framebuffer(_connection, VNC_FRAMEBUFFER(_baseFrameBuffer));
    _width = width;
    _height = height;
    emit clearImaged();
    emit sizeChanged(QSize(width, height));
}

} // namespace RemoteAccess
