/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "capturejobmodel.h"
#include "ressources.h"

// hynesim hnsclient headers
#include <libhnsclient/entity.h>

// Qt headers
#include <QMap>
#include <QDateTime>
#include <QIcon>

namespace Hnv {

/*******************************************************************************
* PcapInfoItem
*******************************************************************************/
PcapInfoItem::PcapInfoItem(HnsClient::GenericEntity * entity, Hns::PcapInfo pcapInfo, int jobID, QObject *parent)
    : QObject(parent)
    , _entity(entity)
    , _pcapInfo(pcapInfo)
    , _jobID(jobID)
{
}

PcapInfoItem::~PcapInfoItem()
{
}

void PcapInfoItem::setPcapInfo(Hns::PcapInfo pcapInfo)
{
    _pcapInfo = pcapInfo;
}

Hns::PcapInfo PcapInfoItem::getPcapInfo() const
{
    return _pcapInfo;
}

bool PcapInfoItem::isRunning() const
{
    bool running = false;
    if (_jobID != -1)
        running = true;
    return running;
}

void PcapInfoItem::setJobID(int jobID)
{
    _jobID = jobID;
}

int PcapInfoItem::getJobID() const
{
    return _jobID;
}

QString PcapInfoItem::getPath() const
{
    return QString::fromStdString(_pcapInfo.path);
}

QString PcapInfoItem::getInterfaceName() const
{
    if (_pcapInfo.interfaces == Hns::captureAll)
        return tr("All");
    else
        return QString::fromStdString(_entity->getPlugs().value(_pcapInfo.interfaces).name);
}

QString PcapInfoItem::getCreationDateTime() const
{
    return QDateTime::fromMSecsSinceEpoch(_pcapInfo.startDateTime).toString(Qt::SystemLocaleShortDate);
}

QString PcapInfoItem::getEndDateTime() const
{
    if (_pcapInfo.endDateTime == -1)
        return "-";
    else
        return QDateTime::fromMSecsSinceEpoch(_pcapInfo.endDateTime).toString(Qt::SystemLocaleShortDate);
}

int PcapInfoItem::getTimeLimit() const
{
    return _pcapInfo.timeLimit;
}

int PcapInfoItem::getPacketLimit() const
{
    return _pcapInfo.packetsLimit;
}

QString PcapInfoItem::getFilters() const
{
    return QString::fromStdString(_pcapInfo.filters);
}

int PcapInfoItem::getPacketCount() const
{
    return _pcapInfo.packetsNumber;
}

QString PcapInfoItem::getReadableSize() const
{
    // TODO: Should use a utily class
    double finalSize = _pcapInfo.size;
    if (finalSize == -1)
        return "-";
    QList<QString> sizeUnit;
    sizeUnit.append(tr("B"));
    sizeUnit.append(tr("kB"));
    sizeUnit.append(tr("MB"));
    sizeUnit.append(tr("GB"));
    sizeUnit.append(tr("TB"));
    int unitIndex = 0;
    while (finalSize > 1000 && unitIndex < sizeUnit.count()) {
        finalSize = finalSize / 1000;
        unitIndex++;
    }
    return QString("%1 %2").arg(QString::number(finalSize, 'f', 2)).arg(sizeUnit.at(unitIndex));
}

QVariant PcapInfoItem::data(int column, int role) const
{
    if (role == PcapInfoItem::JobIDRole)
        return QVariant(_jobID);
    else if (role == PcapInfoItem::StatusRole)
        return isRunning();
    else if (role == PcapInfoItem::FilePathRole)
        return getPath();
    else if (role == Qt::DecorationRole && column == COLUMN_INTERFACE) {
        if (isRunning())
            return QIcon(Ressources::themedPath(Constants::GRAPHIC_STARTED));
    } else if (role == Qt::DisplayRole) {
        switch (column) {
        case COLUMN_INTERFACE:
            return getInterfaceName();
        case COLUMN_DATEBEGIN:
            return getCreationDateTime();
        case COLUMN_DATENDED:
            return getEndDateTime();
        case COLUMN_TIMELIMIT:
            return getTimeLimit();
        case COLUMN_PACKETLIMIT:
            return getPacketLimit();
        case COLUMN_FILTERS:
            return getFilters();
        case COLUMN_PACKETCOUNT:
            if (isRunning())
                return tr("-");
            else
                return getPacketCount();
        case COLUMN_SIZE:
            return getReadableSize();
        }
    }
    return QVariant();
}

/*******************************************************************************
* CaptureJobModelLoader
*******************************************************************************/
CaptureJobModelLoader::CaptureJobModelLoader(HnsClient::GenericEntity *entity, QObject *parent)
    : QObject(parent)
    , _entity(entity)
    , _jobID(-1)
{
}

void CaptureJobModelLoader::start(int jobID)
{
    _jobID = jobID;
    _entity->getPcapInfoByJobId(jobID, this, SLOT(_getPcapInfoByJobIDSlot(::Hns::PcapInfo)));
}

void CaptureJobModelLoader::_getPcapInfoByJobIDSlot(Hns::PcapInfo pcapInfo)
{
    emit ended(_jobID, pcapInfo);
    deleteLater();
}

/*******************************************************************************
* CaptureJobModel
*******************************************************************************/
CaptureJobModel::CaptureJobModel(HnsClient::GenericEntity *entity,QObject *parent)
    : QAbstractTableModel(parent)
    , _entity(entity)
{
    connect (_entity, SIGNAL(captureJobUseInterface(Ice::Int,Ice::Int)), this, SLOT(_captureJobStarted(Ice::Int,Ice::Int)));
    connect (_entity, SIGNAL(captureJobReleasedInterface(Ice::Int,Ice::Int)), this, SLOT(_captureJobEnded(Ice::Int,Ice::Int)));
    connect (_entity, SIGNAL(removeCaptureFiles(QString)), this, SLOT(_removeCaptureFile(QString)));
    _entity->getCaptureJobIdPath(this, SLOT(_getCaptureJobMapSlot(::Hns::mapLongString)));
}

CaptureJobModel::~CaptureJobModel()
{
}

int CaptureJobModel::rowCount(const QModelIndex &parent) const
{
    if (parent.isValid()) {
        return 0;
    }
    return _pcapInfoItemsList.count();
}

int CaptureJobModel::columnCount(const QModelIndex &parent) const
{
    if (parent.isValid()) {
        return 0;
    }
    return 8;
}

QVariant CaptureJobModel::data(const QModelIndex &index, int role) const
{
    if (index.isValid()) {
        int row = index.row();
        if (row < _pcapInfoItemsList.size()) {
            return _pcapInfoItemsList[row]->data(index.column(), role);
        }
    }
    return QVariant();
}

QVariant CaptureJobModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if (role == Qt::DisplayRole && orientation == Qt::Horizontal) {
        switch(section) {
        case PcapInfoItem::COLUMN_INTERFACE:
            return tr("Interface");
        case PcapInfoItem::COLUMN_DATEBEGIN:
            return tr("Creation date");
        case PcapInfoItem::COLUMN_DATENDED:
            return tr("End date");
        case PcapInfoItem::COLUMN_TIMELIMIT:
            return tr("Time limit");
        case PcapInfoItem::COLUMN_PACKETLIMIT:
            return tr("Packet limit");
        case PcapInfoItem::COLUMN_FILTERS:
            return tr("Filters");
        case PcapInfoItem::COLUMN_PACKETCOUNT:
            return tr("Packet count");
        case PcapInfoItem::COLUMN_SIZE:
            return tr("Size");
        }
        return QAbstractItemModel::headerData(section, orientation, role);
    }
    return QVariant();
}

QModelIndex CaptureJobModel::index(int row, int column, const QModelIndex &parent) const
{
    if (!hasIndex(row, column, parent))
        return QModelIndex();
    if (row < _pcapInfoItemsList.size()) {
        return createIndex(row, column, _pcapInfoItemsList.at(row));
    }
    return QModelIndex();
}

qlonglong CaptureJobModel::_jobIDByPath(QString path)
{
    qlonglong jobID = -1;
    QHashIterator<qlonglong, QString> iterator(_runningJobsIDPath);
    while(iterator.hasNext()) {
        iterator.next();
        if (iterator.value() == path) {
            jobID = iterator.key();
            break;
        }
    }
    return jobID;
}

void CaptureJobModel::_createPcapInfoItem(::Hns::PcapInfo pcapInfo)
{
    QString path = QString::fromStdString(pcapInfo.path);
    if (_pathToItemHash.contains(path))
        return;
    int pos = _pcapInfoItemsList.count();
    qlonglong jobID = _jobIDByPath(path);
    beginInsertRows(QModelIndex(), pos, pos);
    PcapInfoItem * item = new PcapInfoItem(_entity, pcapInfo, jobID, this);
    _pcapInfoItemsList.append(item);
    _pathToItemHash.insert(path, item);
    if (jobID != -1)
        _jobsToItemHash.insert(jobID, item);
    endInsertRows();
}

void CaptureJobModel::_deletePcapInfoItem(PcapInfoItem *item)
{
    if (item == 0)
        return;
    int row = _pcapInfoItemsList.indexOf(item);
    if (row == -1)
        return;
    QString path = item->getPath();
    beginRemoveRows(QModelIndex(), row, row);
    _pcapInfoItemsList.removeAll(item);
    if (_pathToItemHash.contains(path))
        _pathToItemHash.remove(path);
    qlonglong jobID = _jobIDByPath(path);
    if (jobID != -1) {
        _jobsToItemHash.remove(jobID);
        _runningJobsIDPath.remove(jobID);
    }
    endRemoveRows();
}

void CaptureJobModel::_updatePcapInfoItem(int jobID)
{
    CaptureJobModelLoader *captureJobLoader = new CaptureJobModelLoader(_entity, this);
    connect(captureJobLoader, SIGNAL(ended(int,::Hns::PcapInfo)), this, SLOT(_captureJobLoaderUpdateEnded(int,::Hns::PcapInfo)));
    captureJobLoader->start(jobID);
}

void CaptureJobModel::_captureJobStarted(Ice::Int interfaceID, Ice::Int jobID)
{
    Q_UNUSED(interfaceID)
    CaptureJobModelLoader *captureJobLoader = new CaptureJobModelLoader(_entity, this);
    connect(captureJobLoader, SIGNAL(ended(int,::Hns::PcapInfo)), this, SLOT(_captureJobLoaderEnded(int, ::Hns::PcapInfo)));
    captureJobLoader->start(jobID);
}

void CaptureJobModel::_captureJobEnded(Ice::Int interfaceID, Ice::Int jobID)
{
    Q_UNUSED(interfaceID)

    PcapInfoItem * item = _jobsToItemHash.value(jobID, 0);
    if (item == 0)
        _unknowJobEnded.append(jobID);
    else {
        item->setJobID(-1);
        _runningJobsIDPath.remove(jobID);
        int row = _pcapInfoItemsList.indexOf(item);
        if (row != -1) {
            int lastColumn = columnCount() -1;
            QModelIndex firstIdx = index(row, 0);
            QModelIndex lastIdx = index(row, lastColumn);
            emit dataChanged(firstIdx, lastIdx);
        }
        _updatePcapInfoItem(jobID);
    }
}

void CaptureJobModel::_removeCaptureFile(QString fileName)
{
    QStringList validPath;

    foreach(QString storedPath, _pathToItemHash.keys()) {
        QString filePath = storedPath.split("/").last();
        if (filePath == fileName)
            validPath.append(storedPath);
    }
    foreach(QString path, validPath)
        _deletePcapInfoItem(_pathToItemHash.value(path));
}


void CaptureJobModel::_getCaptureJobMapSlot(::Hns::mapLongString captureJobMap)
{
    QMap<Ice::Long, std::string> runningJobsMap(captureJobMap);
    QMapIterator<Ice::Long, std::string> iterator(runningJobsMap);
    while(iterator.hasNext()) {
        iterator.next();
        _runningJobsIDPath.insert(iterator.key(), QString::fromStdString(iterator.value()));
    }
    _entity->getPcapInfoList(this, SLOT(_getPcapInfoListSlot(::Hns::PcapInfoList)));
}

void CaptureJobModel::_getPcapInfoListSlot(::Hns::PcapInfoList pcapInfoList)
{
    QList< ::Hns::PcapInfo> infoList = QList< ::Hns::PcapInfo>::fromStdList(pcapInfoList);
    foreach(::Hns::PcapInfo pcapInfo, infoList)
        _createPcapInfoItem(pcapInfo);
}

void CaptureJobModel::_captureJobLoaderEnded(int jobID, Hns::PcapInfo pcapInfo)
{
    if (_unknowJobEnded.contains(jobID))
        _unknowJobEnded.removeAll(jobID);
    else
        _runningJobsIDPath.insert(jobID, QString::fromStdString(pcapInfo.path));
    _createPcapInfoItem(pcapInfo);
}

void CaptureJobModel::_captureJobLoaderUpdateEnded(int jobID, Hns::PcapInfo pcapInfo)
{
    PcapInfoItem * item = _jobsToItemHash.value(jobID, 0);
    if (item) {
        item->setPcapInfo(pcapInfo);
        int row = _pcapInfoItemsList.indexOf(item);
        if (row != -1) {
            int lastColumn = columnCount() -1;
            QModelIndex firstIdx = index(row, 0);
            QModelIndex lastIdx = index(row, lastColumn);
            emit dataChanged(firstIdx, lastIdx);
        }
    }
}

} // namespace Hnv
