/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "hnv_catalogitem.h"

// hynesim hnsclient headers
#include <libhnsclient/entry.h>

// Qt headers
#include <QVariant>
#include <QBrush>
#include <QFont>
#include <QIcon>

namespace Hnv {

/*******************************************************************************
* EntryItem
*******************************************************************************/
EntryItem::EntryItem(HnsClient::CatalogItem *entry, QObject *parent)
    : QObject(parent)
    , _entry(entry)
{
}

EntryItem::~EntryItem()
{
}

int EntryItem::columnCount() const
{
    return SIZE_COLUMN;
}

Qt::ItemFlags EntryItem::flags(int column) const
{
    Qt::ItemFlags flags;
    flags |= Qt::ItemIsEnabled | Qt::ItemIsSelectable;
    if (!isLocked() && !hasLinkedClones()) {
        if ( getSyncStatus() != HnsClient::Syncing && isWriteable()) {
            if (column == 0)
                flags |= Qt::ItemIsDragEnabled | Qt::ItemIsDropEnabled;
            flags |= Qt::ItemIsEditable;
        }
    }
    return flags;
}

QVariant EntryItem::data(int column, int role) const
{
    switch(role) {
    case EntryItem::IDRole: {
        return getID();
    }
    case EntryItem::IsReadableRole: {
        return isReadable();
    }
    case EntryItem::IsWriteableRole: {
        return isWriteable();
    }
    case EntryItem::IsExecutableRole: {
        return isExecutable();
    }
    case EntryItem::IsRWXRole: {
        return isRWX();
    }
    case EntryItem::IsLockedRole : {
        return isLocked();
    }
    case EntryItem::IsSavedRole : {
        return isSaved();
    }
    case EntryItem::IsValidRole : {
        return isValid();
    }
    case EntryItem::SyncingStatusRole: {
        return getSyncStatus();
    }
    case EntryItem::IsSyncingRole: {
        return isSyncing();
    }
    case EntryItem::IsModelRole : {
        return isModel();
    }
    case EntryItem::HasLinkedClonesRole : {
        return hasLinkedClones();
    }
    case EntryItem::IsLinkedCloneRole : {
        return isLinkedClone();
    }
    case Qt::ToolTipRole: {
        return getToolTip();
    }
    case Qt::ForegroundRole: {
        if (getSyncStatus() != HnsClient::Synced || !isWriteable())
            return QBrush(Qt::lightGray);
        break;
    }
    case Qt::FontRole: {
        QFont cellFont;
        if (getSyncStatus() != HnsClient::Synced || !isRWX())
            cellFont.setItalic(true);
        return cellFont;
    }
    case Qt::DisplayRole: {
        switch(column) {
        case COLUMN_SHORTNAME:
            return getShortName();
        case COLUMN_LONGNAME:
            return getLongName();
        case COLUMN_DESCRIPTION:
            return getDescription();
        case COLUMN_TAGLIST:
            return getTagList();
        }
    }
    case Qt::EditRole: {
        switch(column) {
        case COLUMN_SHORTNAME:
            return getShortName();
        case COLUMN_LONGNAME:
            return getLongName();
        case COLUMN_DESCRIPTION:
            return getDescription();
        case COLUMN_TAGLIST:
            return getTagList();
        }
    }
    case Qt::DecorationRole: {
        switch(column) {
        case COLUMN_SHORTNAME:
            return QIcon(getPixmap());
        }
    }
    }
    return QVariant();
}

bool EntryItem::setData(int column, const QVariant &value, int role)
{
    if (role == Qt::EditRole) {
        switch(column) {
        case COLUMN_SHORTNAME:
            return catalogItem()->setShortName(value.toString());
        case COLUMN_LONGNAME:
            return catalogItem()->setLongName(value.toString());
        case COLUMN_DESCRIPTION:
            return catalogItem()->setDescription(value.toString());
        case COLUMN_TAGLIST:
            return catalogItem()->setTagList(value.toString());
        }
    }
    return false;
}

HnsClient::CatalogItem *EntryItem::catalogItem() const
{
    return _entry;
}

qlonglong EntryItem::getID() const
{
    return catalogItem()->getID();
}

QString EntryItem::getShortName() const
{
    return catalogItem()->getShortName();
}

QString EntryItem::getLongName() const
{
    return catalogItem()->getLongName();
}

QString EntryItem::getDescription() const
{
    return catalogItem()->getDescription();
}

QString EntryItem::getTagList() const
{
    return catalogItem()->getTagList();
}

bool EntryItem::isReadable() const
{
    return catalogItem()->isReadable();
}

bool EntryItem::isWriteable() const
{
    return catalogItem()->isWriteable();
}

bool EntryItem::isExecutable() const
{
    return catalogItem()->isExecutable();
}

bool EntryItem::isRWX() const
{
    return catalogItem()->isRWX();
}

bool EntryItem::isLocked() const
{
    return catalogItem()->isLocked();
}

bool EntryItem::isSaved() const
{
    return catalogItem()->isSaved();
}

bool EntryItem::isValid() const
{
    return catalogItem()->isValid();
}

HnsClient::SynchronizationStatus EntryItem::getSyncStatus() const
{
    return catalogItem()->getSyncStatus();
}

bool EntryItem::isSyncing() const
{
    return catalogItem()->isSyncing();
}

bool EntryItem::isModel() const
{
    return catalogItem()->isModel();
}

bool EntryItem::hasLinkedClones() const
{
    return catalogItem()->hasLinkedClones();
}

bool EntryItem::isLinkedClone() const
{
    return catalogItem()->isLinkedClone();
}

QString EntryItem::getToolTip() const
{
    QString textLine = getLongName();
    if (textLine == "")
        textLine = getShortName();
    QString toolTip = QObject::tr("<b>Name:&nbsp;</b><i>%1</i>").arg(textLine);
    textLine = getDescription();
    if (textLine != "")
        toolTip += QObject::tr("<br/><b>Description:&nbsp;</b><i>%1</i>").arg(textLine);
    textLine = getTagList();
    if (textLine != "")
        toolTip += QObject::tr("<br/><b>Tags:&nbsp;</b><i>%1</i>").arg(textLine);
    return toolTip;
}

QPixmap EntryItem::getPixmap() const
{
    return QPixmap();
}

} // namespace Hnv
