/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "hyneviewsession.h"
#include "topologyadministrator.h"

// Generated hynesim headers
#include "common/version.h"

// hynesim hnsclient headers
#include <libhnsclient/session.h>
#include <libhnsclient/platformmonitor.h>
#include <libhnsclient/entityinfo.h>
#include <libhnsclient/jobmanager.h>
#include <libhnsclient/catalogimporter.h>
#include <libhnsclient/entityentrycatalog.h>
#include <libhnsclient/topologyentrycatalog.h>
#include <libhnsclient/topologymanager.h>
#include <libhnsclient/usermanager.h>
#include <libhnsclient/aclmanager.h>
#include <libhnsclient/resourcemanager.h>
#include <libhnsclient/macaddressmanager.h>
#include <libhnsclient/dynamipsrouterdb.h>


// Qt headers
#include <QEventLoop>
#include <QStringList>

namespace Hnv {

/*******************************************************************************
* HyneviewSessionLoader
*******************************************************************************/
HyneviewSessionLoader::HyneviewSessionLoader(QObject *parent)
    : QObject(parent)
    , _result(false)
    , _completed(false)
    , _deleteOnCompleted(true)
{
    _parameters.sessionLibClient = 0;
}

HyneviewSessionLoader::~HyneviewSessionLoader()
{
    _parameters.sessionHelper.clear();
}

void HyneviewSessionLoader::load(QSharedPointer<Diamesh::SessionHelper> sessionHelper, bool deleteOnCompleted)
{
    _deleteOnCompleted = deleteOnCompleted;
    _completed = false;
    _parameters.sessionHelper = sessionHelper;
    sessionHelper.clear();
    _result = true;
    _terminate();
}

bool HyneviewSessionLoader::getLoadingResult()
{
    return _result;
}

bool HyneviewSessionLoader::isCompleted()
{
    return _completed;
}

HyneviewSessionParameters HyneviewSessionLoader::getHyneviewSessionParameters()
{
    return _parameters;
}
void HyneviewSessionLoader::_terminate()
{
    _completed = true;
    emit completed(_result, _parameters);

    if (_deleteOnCompleted)
        deleteLater();
}

/*******************************************************************************
* HyneviewSession
*******************************************************************************/
HyneviewSession *HyneviewSession::_singleton = 0;

HyneviewSession::HyneviewSession(HyneviewSessionParameters parameters, QObject *parent)
    : QObject(parent)
    , _parameters(parameters)
{
    _parameters.sessionLibClient = HnsClient::Session::create(_parameters.sessionHelper, this);
    connect(_parameters.sessionLibClient, SIGNAL(disconnected()), this, SLOT(disconnectedSlot()));
    _parameters.topologyAdministrator = new TopologyAdministrator(this);
    _parameters.topologyAdministrator->init(_parameters.sessionLibClient->topologyEntryCatalog(), _parameters.sessionLibClient->topologyManager());
}

HyneviewSession::~HyneviewSession()
{
    if (!_parameters.sessionHelper.isNull() && _parameters.sessionHelper.data()->isConnected())
        _parameters.sessionHelper.data()->disconnect(this);
}

HyneviewSession *HyneviewSession::instance()
{
    return _singleton;
}

HyneviewSession *HyneviewSession::create(QSharedPointer<Diamesh::SessionHelper> sessionHelper, QObject *parent)
{
    if (_singleton == 0) {
        HyneviewSessionLoader * loader = new HyneviewSessionLoader(parent);
        QEventLoop eventLoop;
        QObject::connect(loader, SIGNAL(completed(bool,HyneviewSessionParameters)), &eventLoop, SLOT(quit()));
        loader->load(sessionHelper, false);
        if (!loader->isCompleted())
            eventLoop.exec();
        if (_singleton == 0 && loader->getLoadingResult())
            _singleton = new HyneviewSession(loader->getHyneviewSessionParameters(), parent);
        delete loader;
    }
    sessionHelper.clear();
    return _singleton;
}

bool HyneviewSession::isConnected()
{
    if (_parameters.sessionHelper.isNull())
        return false;
    return _parameters.sessionHelper.data()->isConnected();
}

void HyneviewSession::disconnectSession()
{
    if (isConnected()) {
        _parameters.sessionLibClient->disconnectSession();
        _parameters.sessionHelper.clear();
        if (_parameters.topologyAdministrator)
            delete _parameters.topologyAdministrator;
        _parameters.topologyAdministrator = 0;
    }
}

void HyneviewSession::disconnectedSlot()
{
    emit disconnected();
    _singleton = 0;

    if (_parameters.topologyAdministrator)
        delete _parameters.topologyAdministrator;
    _parameters.topologyAdministrator = 0;
    _parameters.sessionLibClient = 0;
    deleteLater();
}

QString HyneviewSession::userName()
{
    QString userName = "";
    if (!_parameters.sessionHelper.isNull())
        userName =_parameters.sessionHelper.data()->userName();
    return userName;
}

Hns::UserPtr HyneviewSession::user()
{
    Hns::UserPtr user = 0;
    if (!_parameters.sessionHelper.isNull())
        user = Hns::UserPtr::dynamicCast(_parameters.sessionHelper.data()->user());
    return user;
}

QString HyneviewSession::masterName()
{
    return _parameters.sessionLibClient->platformMonitor()->getMasterName();
}

Diamesh::VersionInfo HyneviewSession::masterVersionInfo()
{
    return _parameters.sessionLibClient->platformMonitor()->getMasterInformation()->getVersionInfo();
}

bool HyneviewSession::isAdvancedMode()
{
    return _parameters.sessionLibClient->isAdvancedMode();
}

Diamesh::SessionHelper *HyneviewSession::sessionHelper()
{
    return _parameters.sessionHelper.data();
}

Diamesh::SessionInfo HyneviewSession::sessionInformation()
{
    return _parameters.sessionLibClient->sessionInformation();
}

HnsClient::PlatformMonitor *HyneviewSession::platformMonitor()
{
    return _parameters.sessionLibClient->platformMonitor();
}

HnsClient::EntityInfoList *HyneviewSession::entityInfoList()
{
    return _parameters.sessionLibClient->topologyManager()->entityInfoList();
}

HnsClient::JobManager *HyneviewSession::jobManager()
{
    return _parameters.sessionLibClient->jobManager();
}

QStringList *HyneviewSession::getNetworkCardsVendor()
{
    return _parameters.sessionLibClient->macAddressManager()->getVendorList();
}

TopologyAdministrator *HyneviewSession::topologyAdministrator()
{
    return _parameters.topologyAdministrator;
}

HnsClient::CatalogImporter *HyneviewSession::catalogImporter()
{
    return _parameters.sessionLibClient->catalogImporter();
}

QMap<QString, QString> HyneviewSession::entityCatalogStorageMap()
{
    return _parameters.sessionLibClient->entityCatalogStorageMap();
}

QMap<QString, QString> HyneviewSession::topologyCatalogStorageMap()
{
    return _parameters.sessionLibClient->topologyCatalogStorageMap();
}

void HyneviewSession::getDynamipsRouterData(QMap<QString, Hns::DataDynamipsRouter > & dataDynamipsMap, QMap<QString, Hns::DynamipsRouterModule > & dynamipsRouterModuleMap)
{
    dataDynamipsMap = _parameters.sessionLibClient->dynamipsRouterDB()->getDynamipsRouterMap();
    dynamipsRouterModuleMap = _parameters.sessionLibClient->dynamipsRouterDB()->getDynamipsModuleMap();
}

HnsClient::EntityEntryCatalog *HyneviewSession::entityEntryCatalog()
{
    return _parameters.sessionLibClient->entityEntryCatalog();
}

HnsClient::TopologyEntryCatalog *HyneviewSession::topologyEntryCatalog()
{
    return _parameters.sessionLibClient->topologyEntryCatalog();
}

HnsClient::TopologyManager *HyneviewSession::topologyManager()
{
    return _parameters.sessionLibClient->topologyManager();
}

HnsClient::UserManager *HyneviewSession::userManager()
{
    return _parameters.sessionLibClient->userManager();
}

HnsClient::ACLManager *HyneviewSession::aclManager()
{
    return _parameters.sessionLibClient->aclManager();
}

HnsClient::ResourceManager *HyneviewSession::resourceManager()
{
    return _parameters.sessionLibClient->resourceManager();
}

HnsClient::MacAddressManager *HyneviewSession::macAddressManager()
{
    return _parameters.sessionLibClient->macAddressManager();
}

} // namespace Hnv
