/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "platformmonitormodel.h"
#include "ressources.h"

// hynesim common headers
#include "common/initializer.h"

// Generated hynesim headers
#include "common/entityentry.h"
#include "common/domainentry.h"
#include "common/switchentry.h"
#include "common/hybridnetcardentry.h"
#include "common/wifiapentry.h"
#include "common/dynamipsrouterentry.h"

// hynesim hnsclient headers
#include <libhnsclient/platformmonitor.h>
#include <libhnsclient/entityinfo.h>

// Qt headers
#include <QIcon>
#include <QString>
#include <QSet>
#include <QDebug>

namespace Hnv {

// TODO: Should be using an unique declaration in ressources namespace

HNS_STATIC_HASH_INITIALIZER(_nameHash, NameHash, QString, QString)
NameHash::NameHash()
{
    insert(QString::fromStdString(Hns::DomainEntry::ice_staticId()), QObject::tr("Domain"));
    insert(QString::fromStdString(Hns::SwitchEntry::ice_staticId()), QObject::tr("Switch"));
    insert(QString::fromStdString(Hns::HybridNetcardEntry::ice_staticId()), QObject::tr("Hybrid netcard"));
    insert(QString::fromStdString(Hns::WifiAPEntry::ice_staticId()), QObject::tr("Wi-Fi access point"));
    insert(QString::fromStdString(Hns::DynamipsRouterEntry::ice_staticId()),QObject::tr("Dynamips routers"));
    insert("vbox",QObject::tr("Virtual Box"));
    insert("kvm",QObject::tr("KVM"));
    insert("qemu",QObject::tr("Qemu"));
    insert("vmware",QObject::tr("VMware"));
    insert("openvz",QObject::tr("OpenVZ"));
}

HNS_STATIC_SET_INITIALIZER(_blackListEntity, BlackListEntity, QString)
BlackListEntity::BlackListEntity()
{
    insert(QString::fromStdString(Hns::WireEntry::ice_staticId()));
}

/*******************************************************************************
* MonitorItem
*******************************************************************************/
MonitorItem::MonitorItem(QObject *parent)
    : TreeModelItem(1, parent)
{
}

MonitorItem::~MonitorItem()
{
}

QVariant MonitorItem::data(int column, int role) const
{
    if (column == 0) {
        switch(role) {
        case Qt::DecorationRole:
            return QIcon(_getDecorationRole());
        case Qt::DisplayRole:
            return getName();
        case Qt::ToolTipRole:
            return _getTooltipRole();
        }
    }
    return TreeModelItem::data(column, role);
}

void MonitorItem::_infoChangedSlot()
{
    emit dataChanged();
}

/*******************************************************************************
* MeshMonitorItem
*******************************************************************************/
MeshMonitorItem::MeshMonitorItem(HnsClient::PlatformMonitor *monitor, QObject *parent)
    : MonitorItem(parent)
    , _monitor(monitor)
{
    connect(_monitor->getMasterInformation(), SIGNAL(infoChanged()), this, SLOT(_infoChangedSlot()));
    QHashIterator<QString, HnsClient::NodeInformation*> hashIterator(_monitor->getNodeInformationHash());
    while(hashIterator.hasNext()) {
        hashIterator.next();
        connect(hashIterator.value(), SIGNAL(infoChanged()), this, SLOT(_infoChangedSlot()));
    }
}

MeshMonitorItem::~MeshMonitorItem()
{
}

void MeshMonitorItem::refresh()
{
    _monitor->getMasterInformation()->refresh();
    QHashIterator<QString, HnsClient::NodeInformation*> hashIterator(_monitor->getNodeInformationHash());
    while(hashIterator.hasNext()) {
        hashIterator.next();
        hashIterator.value()->refresh();
    }
}

QString MeshMonitorItem::getName() const
{
    return tr("Mesh summary");
}

QString MeshMonitorItem::getVersion()
{
    HnsClient::ServiceQuery q = &HnsClient::ServiceDetails::getVersionInfoAsString;
    return _buildInfoTable(q, true);
}

Diamesh::VersionInfo MeshMonitorItem::getVersionInfo()
{
    qWarning() << tr("Requested version details is incomplete.");
    return _monitor->getMasterInformation()->getVersionInfo();
}

Diamesh::SystemInfo MeshMonitorItem::getSystemInfo()
{
    qWarning() << tr("Requested system detail is incomplete.");
    return _monitor->getMasterInformation()->getSystemInfo();
}

Diamesh::ServiceInfo MeshMonitorItem::getServiceInfo()
{
    qWarning() << tr("Requested service detail is incomplete.");
    return _monitor->getMasterInformation()->getServiceInfo();
}

QString MeshMonitorItem::getServiceCapabilities()
{
    HnsClient::NodeInformationQuery q = &HnsClient::NodeInformation::getPlatformsAsString;
    return _buildInfoTable(q, true, true);
}

QString MeshMonitorItem::getServiceHost()
{
    HnsClient::ServiceQuery q = &HnsClient::ServiceDetails::getServiceHost;
    return _buildInfoTable(q, true);
}

QString MeshMonitorItem::getSystemKernel()
{
    HnsClient::ServiceQuery q = &HnsClient::ServiceDetails::getKernelInformation;
    return _buildInfoTable(q);
}

QString MeshMonitorItem::getSystemProcessor()
{
    HnsClient::ServiceQuery q = &HnsClient::ServiceDetails::getSystemProcessorsAsString;
    return _buildInfoTable(q);
}

QString MeshMonitorItem::getSystemMemory()
{
    HnsClient::ServiceQuery q = &HnsClient::ServiceDetails::getTotalMemoryAsString;
    return _buildInfoTable(q);
}

QString MeshMonitorItem::getSystemSwap()
{
    HnsClient::ServiceQuery q = &HnsClient::ServiceDetails::getTotalSwapAsString;
    return _buildInfoTable(q);
}

QString MeshMonitorItem::getHnsMaxThread()
{
    HnsClient::ServiceQuery q = &HnsClient::ServiceDetails::getServiceMaxThread;
    return _buildInfoTable(q, true);
}

QString MeshMonitorItem::getHnsMaxProcessors()
{
    HnsClient::NodeInformationQuery q = &HnsClient::NodeInformation::getHnsMaxCpuAsString;
    return _buildInfoTable(q, true, true);
}

QString MeshMonitorItem::getHnsMaxMemory()
{
    HnsClient::NodeInformationQuery q = &HnsClient::NodeInformation::getHnsMaxMemoryAsString;
    return _buildInfoTable(q, true, true);
}

QString MeshMonitorItem::getSystemUptime()
{
    HnsClient::ServiceQuery q = &HnsClient::ServiceDetails::getSystemUptimeAsString;
    return _buildInfoTable(q);
}

QString MeshMonitorItem::getSystemCpuLoad()
{
    HnsClient::ServiceQuery q = &HnsClient::ServiceDetails::getCpuUsageAsString;
    return _buildInfoTable(q);
}

QString MeshMonitorItem::getSystemMemoryLoad()
{
    HnsClient::ServiceQuery q = &HnsClient::ServiceDetails::getMemoryLoad;
    return _buildInfoTable(q);
}

QString MeshMonitorItem::getSystemSwapLoad()
{
    HnsClient::ServiceQuery q = &HnsClient::ServiceDetails::getSwapLoad;
    return _buildInfoTable(q);
}

QString MeshMonitorItem::getDisksStatus()
{
    HnsClient::ServiceQuery q = &HnsClient::ServiceDetails::getDisksStatusAsString;
    return _buildInfoTable(q);
}

QString MeshMonitorItem::getServiceUptime()
{
    HnsClient::ServiceQuery q = &HnsClient::ServiceDetails::getServiceUptimeAsString;
    return _buildInfoTable(q, true);
}

QString MeshMonitorItem::getHnsCpuUsage()
{
    HnsClient::NodeInformationQuery q = &HnsClient::NodeInformation::getHnsCpuUsage;
    return _buildInfoTable(q, true, true);
}

QString MeshMonitorItem::getHnsMemoryUsage()
{
    HnsClient::NodeInformationQuery q = &HnsClient::NodeInformation::getHnsMemoryUsage;
    return _buildInfoTable(q, true, true);
}

QString MeshMonitorItem::getHnsDefinedEntities()
{
    HnsClient::NodeInformationQuery q = &HnsClient::NodeInformation::getHnsDefinedEntitiesAsString;
    return _buildInfoTable(q, true, true);
}

QString MeshMonitorItem::_checkLineValue(QString host, QString name, QString value, QHash<QString, QString> &redudancyHash)
{
    if (redudancyHash.contains(host))
        value = tr("<i>Same server as <b>%1</b></i>").arg(redudancyHash.value(host));
    else
        redudancyHash.insert(host, name);
    return value;
}

QHash<QString, QString> MeshMonitorItem::_buildDataHash(HnsClient::ServiceQuery query, bool ignoreRedudancy, bool skipMaster)
{
    QHash<QString, QString> result;
    QHash<QString, QString> redudancyHash;
    if (!skipMaster) {
        QString masterName = _monitor->getMasterInformation()->getShortName();
        result.insert(masterName, _checkLineValue(_monitor->getMasterInformation()->getServiceHost(), masterName, (_monitor->getMasterInformation()->*query)(), redudancyHash));
    }
    QHashIterator<QString, HnsClient::NodeInformation*> hashIterator(_monitor->getNodeInformationHash());
    while(hashIterator.hasNext()) {
        hashIterator.next();
        HnsClient::NodeInformation * nodeInformation = hashIterator.value();
        QString nodeName = nodeInformation->getShortName();
        QString value = (nodeInformation->*query)();
        if (!ignoreRedudancy)
            value = _checkLineValue(nodeInformation->getServiceHost(), nodeName, value, redudancyHash);
        result.insert(nodeName, value);
    }
    return result;
}

QHash<QString, QString> MeshMonitorItem::_buildDataHash(HnsClient::NodeInformationQuery query, bool ignoreRedudancy)
{
    QHash<QString, QString> result;
    QHash<QString, QString> redudancyHash;
    QHashIterator<QString, HnsClient::NodeInformation*> hashIterator(_monitor->getNodeInformationHash());
    while(hashIterator.hasNext()) {
        hashIterator.next();
        HnsClient::NodeInformation * nodeInformation = hashIterator.value();
        QString nodeName = nodeInformation->getShortName();
        QString value = (nodeInformation->*query)();
        if (!ignoreRedudancy)
            value = _checkLineValue(nodeInformation->getServiceHost(), nodeName, value, redudancyHash);
        result.insert(nodeName, value);
    }
    return result;
}

QString MeshMonitorItem::_buildInfoTableLine(QString label, QString value, QString lineClass)
{
    QString result;
    if (lineClass.isEmpty())
        result = "<tr>";
    else
        result = "<tr class=\"" + lineClass + "\">";
    result += tr("<th width='1%' align='left'>%1:</th><td>%2</td></tr>").arg(label).arg(value);
    return result;
}

QString MeshMonitorItem::_buildInfoTable(QHash<QString, QString> data, bool skipMaster)
{
    QString result = tr("N/A");
    if (data.count() > 0) {
        result = "<table class='infoTable' cellspacing='0' width='100%'>";
        bool first = !skipMaster;
        QHashIterator<QString, QString> dataIterator(data);
        while(dataIterator.hasNext()) {
            dataIterator.next();
            result += _buildInfoTableLine(dataIterator.key(), dataIterator.value(), (first ? "master" : "node"));
            if (first)
                first = false;
        }
        result += "</table>";
    }
    return result;
}

QString MeshMonitorItem::_buildInfoTable(HnsClient::ServiceQuery query, bool ignoreRedudancy, bool skipMaster)
{
    return _buildInfoTable(_buildDataHash(query, ignoreRedudancy, skipMaster), skipMaster);
}

QString MeshMonitorItem::_buildInfoTable(HnsClient::NodeInformationQuery query, bool ignoreRedudancy, bool skipMaster)
{
    return _buildInfoTable(_buildDataHash(query, ignoreRedudancy), skipMaster);
}

QPixmap MeshMonitorItem::_getDecorationRole() const
{
    return QPixmap(Ressources::themedPath(Constants::ICON_MESH));
}

QString MeshMonitorItem::_getTooltipRole() const
{
    // TODO: Custom tooltip
    return tr("Mesh summary");
}

QString MeshMonitorItem::getHnsNetcards()
{
    HnsClient::NodeInformationQuery q = &HnsClient::NodeInformation::getHnsNetcards;
    return _buildInfoTable(q, true, true);
}

/*******************************************************************************
* MasterMonitorItem
*******************************************************************************/
MasterMonitorItem::MasterMonitorItem(HnsClient::MasterInformation *masterInformation, QObject *parent)
    : MonitorItem(parent)
    , _masterInformation(masterInformation)
{
    connect(_masterInformation, SIGNAL(infoChanged()), this, SLOT(_infoChangedSlot()));
}

MasterMonitorItem::~MasterMonitorItem()
{
    _masterInformation = 0;
}

void MasterMonitorItem::refresh()
{
    _masterInformation->refresh();
}

QString MasterMonitorItem::getName() const
{
    return _masterInformation->getShortName();
}

QString MasterMonitorItem::getVersion()
{
    return _masterInformation->getVersionInfoAsString(true);
}

Diamesh::VersionInfo MasterMonitorItem::getVersionInfo()
{
    return _masterInformation->getVersionInfo();
}

Diamesh::SystemInfo MasterMonitorItem::getSystemInfo()
{
    return _masterInformation->getSystemInfo();
}

Diamesh::ServiceInfo MasterMonitorItem::getServiceInfo()
{
    return _masterInformation->getServiceInfo();
}

QString MasterMonitorItem::getServiceCapabilities()
{
    return tr("N/A");
}

QString MasterMonitorItem::getServiceHost()
{
    return _masterInformation->getServiceHost();
}

QString MasterMonitorItem::getSystemKernel()
{
    return _masterInformation->getKernelInformation();
}

QString MasterMonitorItem::getSystemProcessor()
{
    return _masterInformation->getSystemProcessorsAsString();
}

QString MasterMonitorItem::getSystemMemory()
{
    return _masterInformation->getTotalMemoryAsString();
}

QString MasterMonitorItem::getSystemSwap()
{
    return _masterInformation->getTotalSwapAsString();
}

QString MasterMonitorItem::getHnsMaxThread()
{
    return _masterInformation->getServiceMaxThread();
}

QString MasterMonitorItem::getHnsMaxProcessors()
{
    return tr("N/A");
}

QString MasterMonitorItem::getHnsMaxMemory()
{
    return tr("N/A");
}

QString MasterMonitorItem::getSystemUptime()
{
    return _masterInformation->getSystemUptimeAsString();
}

QString MasterMonitorItem::getSystemCpuLoad()
{
    return _masterInformation->getCpuUsageAsString();
}

QString MasterMonitorItem::getSystemMemoryLoad()
{
    return _masterInformation->getMemoryLoad();
}

QString MasterMonitorItem::getSystemSwapLoad()
{
    return _masterInformation->getSwapLoad();
}

QString MasterMonitorItem::getDisksStatus()
{
    return _masterInformation->getDisksStatusAsString();
}

QString MasterMonitorItem::getServiceUptime()
{
    return _masterInformation->getServiceUptimeAsString();
}

QString MasterMonitorItem::getHnsCpuUsage()
{
    return tr("N/A");
}

QString MasterMonitorItem::getHnsMemoryUsage()
{
    return tr("N/A");
}

QString MasterMonitorItem::getHnsDefinedEntities()
{
    return tr("N/A");
}

QString MasterMonitorItem::getHnsNetcards()
{
    return tr("N/A");
}

QPixmap MasterMonitorItem::_getDecorationRole() const
{
    return QPixmap(Ressources::themedPath(Constants::ICON_MASTER));
}

QString MasterMonitorItem::_getTooltipRole() const
{
    // TODO: Custom tooltip
    return _masterInformation->getName();
}

/*******************************************************************************
* NodeMonitorItem
*******************************************************************************/
NodeMonitorItem::NodeMonitorItem(HnsClient::NodeInformation *nodeInformation, QObject *parent)
    : MonitorItem(parent)
    , _nodeInformation(nodeInformation)
{
    connect(_nodeInformation, SIGNAL(infoChanged()), this, SLOT(_infoChangedSlot()));
}

NodeMonitorItem::~NodeMonitorItem()
{
}

void NodeMonitorItem::refresh()
{
    _nodeInformation->refresh();
}

QString NodeMonitorItem::getName() const
{
    return _nodeInformation->getShortName();
}

QString NodeMonitorItem::getVersion()
{
    return _nodeInformation->getVersionInfoAsString(true);
}

Diamesh::VersionInfo NodeMonitorItem::getVersionInfo()
{
    return _nodeInformation->getVersionInfo();
}

Diamesh::SystemInfo NodeMonitorItem::getSystemInfo()
{
    return _nodeInformation->getSystemInfo();
}

Diamesh::ServiceInfo NodeMonitorItem::getServiceInfo()
{
    return _nodeInformation->getServiceInfo();
}

QString NodeMonitorItem::getServiceCapabilities()
{
    return _nodeInformation->getPlatformsAsString();
}

QString NodeMonitorItem::getServiceHost()
{
    return _nodeInformation->getServiceHost();
}

QString NodeMonitorItem::getSystemKernel()
{
    return _nodeInformation->getKernelInformation();
}

QString NodeMonitorItem::getSystemProcessor()
{
    return _nodeInformation->getSystemProcessorsAsString();
}

QString NodeMonitorItem::getSystemMemory()
{
    return _nodeInformation->getTotalMemoryAsString();
}

QString NodeMonitorItem::getSystemSwap()
{
    return _nodeInformation->getTotalSwapAsString();
}

QString NodeMonitorItem::getHnsMaxThread()
{
    return _nodeInformation->getServiceMaxThread();
}

QString NodeMonitorItem::getHnsMaxProcessors()
{
    return _nodeInformation->getHnsMaxCpuAsString();
}

QString NodeMonitorItem::getHnsMaxMemory()
{
    return _nodeInformation->getHnsMaxMemoryAsString();
}

QString NodeMonitorItem::getSystemUptime()
{
    return _nodeInformation->getSystemUptimeAsString();
}

QString NodeMonitorItem::getSystemCpuLoad()
{
    return _nodeInformation->getCpuUsageAsString();
}

QString NodeMonitorItem::getSystemMemoryLoad()
{
    return _nodeInformation->getMemoryLoad();
}

QString NodeMonitorItem::getSystemSwapLoad()
{
    return _nodeInformation->getSwapLoad();
}

QString NodeMonitorItem::getDisksStatus()
{
    return _nodeInformation->getDisksStatusAsString();
}

QString NodeMonitorItem::getServiceUptime()
{
    return _nodeInformation->getServiceUptimeAsString();
}

QString NodeMonitorItem::getHnsCpuUsage()
{
    return _nodeInformation->getHnsCpuUsage();
}

QString NodeMonitorItem::getHnsMemoryUsage()
{
    return _nodeInformation->getHnsMemoryUsage();
}

QString NodeMonitorItem::getHnsDefinedEntities()
{
    return _nodeInformation->getHnsDefinedEntitiesAsString();
}

QString NodeMonitorItem::getHnsNetcards()
{
    return _nodeInformation->getHnsNetcards();
}

QPixmap NodeMonitorItem::_getDecorationRole() const
{
    return QPixmap(Ressources::themedPath(Constants::ICON_NODE));
}

QString NodeMonitorItem::_getTooltipRole() const
{
    QString toolTip = tr("<b>Key:&nbsp;</b><i>%1</i>").arg(_nodeInformation->getName());
    if (!_nodeInformation->getDescription().isEmpty())
        toolTip.append(tr("<br/><b>Desription:&nbsp;</b><i>%1</i>").arg(_nodeInformation->getDescription()));
    return toolTip;
}

/*******************************************************************************
* PlatformMonitorModel
*******************************************************************************/
PlatformMonitorModel::PlatformMonitorModel(QObject *parent)
    : TreeModel(parent)
    , _monitor(0)
    , _meshItem(0)
    , _masterItem(0)
{
    _initRootItem(1);
}

PlatformMonitorModel::~PlatformMonitorModel()
{
    _resetModel();
}

void PlatformMonitorModel::setMonitor(HnsClient::PlatformMonitor *monitor)
{
    if (_monitor != 0)
        _resetModel();
    _monitor = monitor;
    if (_monitor) {
        connect(_monitor, SIGNAL(nodeAdded(QString)), this, SLOT(_nodeAddedSlot(QString)));
        connect(_monitor, SIGNAL(nodeRemoved(QString)), this, SLOT(_nodeRemovedSlot(QString)));
        _populateModel();
    }
}

MonitorItem * PlatformMonitorModel::monitorItemFromIndex(QModelIndex index)
{
    MonitorItem * item = 0;
    if (index.isValid())
        item = static_cast<MonitorItem *>(index.internalPointer());
    return item;
}

MeshMonitorItem * PlatformMonitorModel::meshMonitorItem()
{
    return _meshItem;
}

MasterMonitorItem * PlatformMonitorModel::masterMonitorItem()
{
    return _masterItem;
}

NodeMonitorItem * PlatformMonitorModel::nodeMonitorItem(QString nodeName)
{
    if (_nodeItemHash.contains(nodeName))
        return _nodeItemHash.value(nodeName);
    return 0;
}

void PlatformMonitorModel::_resetModel()
{
    beginResetModel();
    _monitor = 0;
    _meshItem = 0;
    _masterItem = 0;
    _nodeItemHash.clear();
    _rootItem->clear();
    endResetModel();
}

void PlatformMonitorModel::_populateModel()
{
    _meshItem = new MeshMonitorItem(_monitor, this);
    addItem(_rootItem, _meshItem);
    connect(_meshItem, SIGNAL(dataChanged()), this, SLOT(_itemDataChanged()));

    _masterItem = new MasterMonitorItem(_monitor->getMasterInformation(), this);
    addItem(_rootItem, _masterItem);
    connect(_masterItem, SIGNAL(dataChanged()), this, SLOT(_itemDataChanged()));

    QHash<QString, HnsClient::NodeInformation*> nodeInformationHash = _monitor->getNodeInformationHash();

    foreach(HnsClient::NodeInformation * nodeInformation, nodeInformationHash.values())
        _addNodeMonitorItem(nodeInformation);
    emit dataLoaded();
}

void PlatformMonitorModel::_addNodeMonitorItem(HnsClient::NodeInformation *nodeInformation)
{
    if (nodeInformation == 0)
        return;
    NodeMonitorItem * item = new NodeMonitorItem(nodeInformation, this);
    _nodeItemHash.insert(nodeInformation->getName(), item);
    addItem(_rootItem, item);
    connect(item, SIGNAL(dataChanged()), this, SLOT(_itemDataChanged()));
}

void PlatformMonitorModel::_itemDataChanged()
{
    MonitorItem * item = qobject_cast<MonitorItem*>(sender());
    if (item) {
        QModelIndex firstIdx = indexFromNode(item);
        QModelIndex lastIdx = indexFromNode(item, item->columnCount() - 1);
        emit dataChanged(firstIdx, lastIdx);
    }
}

void PlatformMonitorModel::_nodeAddedSlot(QString nodeName)
{
    if (!_nodeItemHash.contains(nodeName))
        _addNodeMonitorItem(_monitor->getNodeInformation(nodeName));
}

void PlatformMonitorModel::_nodeRemovedSlot(QString nodeName)
{
    if (_nodeItemHash.contains(nodeName)) {
        NodeMonitorItem * item = _nodeItemHash.take(nodeName);
        if (item) {
            removeItem(item);
            delete item;
        }
    }
}

/*******************************************************************************
* EntityInfoUpdater
*******************************************************************************/
EntityInfoUpdater::EntityInfoUpdater(qlonglong entityId, Hns::EntityInfoQPrx entityInfo, QObject *parent):
    QObject(parent),
    _entityInfo(entityInfo),
    _entityId(entityId),
    _idxStatus(0)
{
}

void EntityInfoUpdater::init()
{
    _entityInfo->begin_getStatus(this, SLOT(_getStatusSlot(::Hns::Status)));
    connect(_entityInfo.data(), SIGNAL(statusChanged(Ice::Int, Ice::Long)) , this, SLOT(_statusChangedSlot(Ice::Int, Ice::Long)) );
}


EntityInfoUpdater::~EntityInfoUpdater()
{
    _entityInfo = 0;
}

const QString& EntityInfoUpdater::getShortName() const
{
    return _shortName;
}

const QString& EntityInfoUpdater::getLongName() const
{
    return _longName;
}


const QString& EntityInfoUpdater::getType() const
{
    return _type;
}

const QString& EntityInfoUpdater::getSubType() const
{
    return _subType;
}


Hns::Status EntityInfoUpdater::getStatus() const
{
    return _status;
}

void EntityInfoUpdater::_statusChangedSlot(Ice::Int status, Ice::Long cpt)
{
    if (cpt > _idxStatus) {
        _idxStatus = cpt;
        _status = Hns::Status(status);
        emit entityStateChange(status);
    }
}

void EntityInfoUpdater::_entryChangedSlot(::Hns::EntityInfoData data)
{
    _shortName = QString::fromStdString(data.shortName);
    _longName = QString::fromStdString(data.longName);
    _type = QString::fromStdString(data.type);
    _subType = QString::fromStdString(data.subtype);
    emit entityUpdate();
}

void EntityInfoUpdater::_getStatusSlot(::Hns::Status status)
{
    if (_idxStatus == 0) {
        _status = status;
    }
    _entityInfo->begin_infoData(this, SLOT(_getDataSlot(::Hns::EntityInfoData)));
}

void EntityInfoUpdater::_getDataSlot(::Hns::EntityInfoData data)
{
    _shortName = QString::fromStdString(data.shortName);
    _longName = QString::fromStdString(data.longName);
    _type = QString::fromStdString(data.type);
    _subType = QString::fromStdString(data.subtype);
    connect(_entityInfo.data(), SIGNAL(entryChanged(::Hns::EntityInfoData)) , this, SLOT(_entryChangedSlot(::Hns::EntityInfoData)));
    emit completed(_entityId);
}

/*******************************************************************************
* EntityInfoModel
*******************************************************************************/
EntityInfoModel::EntityInfoModel(QObject * parent):
    TreeModel(parent),
    _monitor(0),
    _entityInfoList(0)
{
    _initRootItem(1);
}


EntityInfoModel::~EntityInfoModel()
{
    _resetModel();
}



void EntityInfoModel::setMonitorEntityInfoList(HnsClient::PlatformMonitor *monitor, HnsClient::EntityInfoList *entityInfoList)
{
    if (_entityInfoList || _monitor)
        _resetModel();
    _entityInfoList = entityInfoList;
    _monitor = monitor;
    if (_entityInfoList && _monitor) {
        connect(_monitor, SIGNAL(nodeAdded(QString)), this, SLOT(_nodeAddedSlot(QString)));
        connect(_monitor, SIGNAL(nodeRemoved(QString)), this, SLOT(_nodeRemovedSlot(QString)));
        connect(_entityInfoList, SIGNAL(entityInfoRemoved(qlonglong)), this, SLOT(_entityRemoveSlot(qlonglong)));
        _populateModel();
    }
}


void EntityInfoModel::_nodeAddedSlot(QString nodeName)
{
    if (!_nodeInfoItemMap.contains(nodeName)) {
        _nodeInfoItemMap[nodeName] = new NodeInfoItem(nodeName,_entityInfoList , _monitor,this);
        connect(_nodeInfoItemMap[nodeName], SIGNAL(addEntity(QString,long)), this , SLOT(_entityAddSlot(QString,long)));
        connect(_nodeInfoItemMap[nodeName], SIGNAL(removeEntity(long)), this , SLOT(_entityRemoveByNodeSlot(long)));
        addItem(_rootItem, _nodeInfoItemMap[nodeName]);
        _nodeInfoItemMap[nodeName]->populate();
    }
}

void EntityInfoModel::_nodeRemovedSlot(QString nodeName)
{
    if (_nodeInfoItemMap.contains(nodeName)) {
        disconnect(_nodeInfoItemMap[nodeName]);
        NodeInfoItem * nodeInfoItem = _nodeInfoItemMap.take(nodeName);
        nodeInfoItem->cleanup();
        removeItem(nodeInfoItem);
        nodeInfoItem->deleteLater();
    }
}

void EntityInfoModel::_resetModel()
{
    beginResetModel();
    _monitor = 0;
    _entityInfoList = 0;
    foreach (NodeInfoItem * value, _nodeInfoItemMap) {
        value->cleanup();
        value->deleteLater();
    }
    _entityByNode.clear();
    _nodeInfoItemMap.clear();
    _rootItem->clear();
    endResetModel();
}

void EntityInfoModel::_populateModel()
{
    QHash<QString, HnsClient::NodeInformation*> nodeInformationHash = _monitor->getNodeInformationHash();
    foreach(HnsClient::NodeInformation * nodeInformation, nodeInformationHash.values())
        _nodeAddedSlot(nodeInformation->getName());
    emit dataLoaded();
}

void EntityInfoModel::itemDataChanged()
{
    TreeModelItem * item = qobject_cast<TreeModelItem*>(sender());
    if (item) {
        QModelIndex firstIdx = indexFromNode(item);
        QModelIndex lastIdx = indexFromNode(item, item->columnCount() - 1);
        emit dataChanged(firstIdx, lastIdx);
    }
}

void EntityInfoModel::_entityRemoveSlot(qlonglong entityId) {
    if (_entityByNode.contains(entityId)) {
        QString nameNode = _entityByNode.take(entityId);
        if (_nodeInfoItemMap.contains(nameNode))
            _nodeInfoItemMap[nameNode]->removeEntitySlot(entityId);
    }
}


void EntityInfoModel::_entityAddSlot(QString node, long id) {
    _entityByNode[id] = node;
}

void EntityInfoModel::_entityRemoveByNodeSlot(long id) {
    _entityByNode.take(id);
}


/*******************************************************************************
* EntityInfoItem
*******************************************************************************/
EntityInfoItem::EntityInfoItem(EntityInfoModel * entityInfoModel, HnsClient::EntityInfo *entityInfo,
                               long entityId ,QObject *parent)
    : TreeModelItem(1, parent),
      _entityInfo(entityInfo),
      _entityId(entityId)
{
    connect(entityInfo, SIGNAL(entryChanged()), this, SIGNAL(infoUpdate()));
    connect(entityInfo, SIGNAL(statusChanged()), this, SIGNAL(infoUpdate()));
    connect(this, SIGNAL(infoUpdate()), entityInfoModel , SLOT(itemDataChanged()));
}

EntityInfoItem::~EntityInfoItem()
{
}

QVariant EntityInfoItem::data(int column, int role) const
{
    if (column == 0 && _entityInfo) {
        switch(role) {
        case Qt::DisplayRole:
            return _entityInfo->getShortName();
        case Qt::DecorationRole:
            return QIcon(Ressources::getStatusPixmap(_entityInfo->getStatus()));
        case Qt::ToolTipRole:
            return _entityInfo->getLongName();
        }
    }
    return TreeModelItem::data(column, role);
}

/*******************************************************************************
* NodeInfoModel
*******************************************************************************/
NodeInfoItem::NodeInfoItem(const QString &name, HnsClient::EntityInfoList *entityInfoList, HnsClient::PlatformMonitor *monitor,
                            EntityInfoModel * entityInfoModel, QObject *parent) :
    TreeModelItem(1, parent),
    _nameNode(name),
    _entityInfoList(entityInfoList),
    _monitor(monitor),
    _entityInfoModel(entityInfoModel)
{
    HnsClient::NodeInformation * nodeInformation = _monitor->getNodeInformation(name);
    connect(nodeInformation, SIGNAL(addEntity(long)), this, SLOT(_addEntitySlot(long)));
    connect(nodeInformation, SIGNAL(removeEntity(long)), this, SLOT(removeEntitySlot(long)));
}

NodeInfoItem::~NodeInfoItem()
{
    cleanup();
}

void NodeInfoItem::cleanup()
{
    foreach (TypeInfoItem * value, _typeInfoItemMap) {
        value->cleanup();
        _entityInfoModel->removeItem(value);
        value->deleteLater();
    }
    _typeInfoItemMap.clear();
}

QVariant NodeInfoItem::data(int column, int role) const
{
    if (column == 0) {
        switch(role) {
        case Qt::DisplayRole:
            return _nameNode;
        case Qt::DecorationRole:
            return QIcon(QPixmap(Ressources::themedPath(Constants::ICON_NODE)));
        }
    }
    return TreeModelItem::data(column, role);
}

void NodeInfoItem::_addEntitySlot(long entityId)
{
    HnsClient::EntityInfo *entityInfo = _entityInfoList->getEntityInfo(entityId);
    if (!entityInfo) {
        return;
    }

    if (_blackListEntity()->contains(entityInfo->getType()))
        return;

    if (!_typeInfoItemMap.contains(entityInfo->getType())) {
        _typeInfoItemMap[entityInfo->getType()] = new TypeInfoItem(entityInfo->getType(), _entityInfoModel, this);
        _entityInfoModel->addItem(this, _typeInfoItemMap[entityInfo->getType()]);
    }
    _typeInfoItemMap[entityInfo->getType()]->addEntityInfo(entityId, entityInfo, true);
    _entityByType[entityId] = entityInfo->getType();
    emit addEntity(_nameNode, entityId);
}

void NodeInfoItem::removeEntitySlot(long entityId)
{
    if (_entityByType.contains(entityId)) {
        QString type = _entityByType.take(entityId);
        TypeInfoItem * typeInfoItem = _typeInfoItemMap[type];
        if ( !typeInfoItem->removeEntityInfo(entityId) ) {
            typeInfoItem = _typeInfoItemMap.take(type);
            _entityInfoModel->removeItem(typeInfoItem);
            typeInfoItem->deleteLater();
        }
        emit removeEntity(entityId);
    }
}

void NodeInfoItem::populate()
{
    HnsClient::NodeInformation * nodeInfo = _monitor->getNodeInformation(_nameNode);
    foreach(  long entityId,nodeInfo->getEntityIdList()) {
        _addEntitySlot(entityId);
    }
}

/*******************************************************************************
* TypeInfoItem
*******************************************************************************/
TypeInfoItem::TypeInfoItem(const QString &nameType, EntityInfoModel *entityInfoModel, QObject *parent):
    TreeModelItem(1, parent),
    _nameType(nameType),
    _entityInfoModel(entityInfoModel)
{

}

TypeInfoItem::~TypeInfoItem()
{
    cleanup();
}

void TypeInfoItem::cleanup()
{
    _entityBySubType.clear();
    foreach (EntityInfoItem * value, _entityInfoItemMap) {
        _entityInfoModel->removeItem(value);
        value->deleteLater();
    }
    _entityInfoItemMap.clear();
    foreach (TypeInfoItem * value, _subTypeInfoItemMap) {
        value->cleanup();
        _entityInfoModel->removeItem(value);
        value->deleteLater();
    }
    _subTypeInfoItemMap.clear();
}

QVariant TypeInfoItem::data(int column, int role) const
{
    if (column == 0) {
        switch(role) {
        case Qt::DisplayRole:
            if (_nameHash()->contains(_nameType)) {
                return _nameHash()->value(_nameType);
            }
            return _nameType;
        case Qt::DecorationRole:
            QPixmap pix;
            if (Ressources::hasClassNamePixmap(_nameType))
                pix = Ressources::getClassNamePixmap(_nameType);
            else
                pix = Ressources::getTypeIcon(_nameType);
            return QIcon(pix);
        }
    }
    return TreeModelItem::data(column, role);
}

void TypeInfoItem::addEntityInfo(long entityId, HnsClient::EntityInfo *entityInfo, bool subType)
{
    if (!subType || entityInfo->getSubType().isEmpty()) {
        EntityInfoItem * item = new EntityInfoItem(_entityInfoModel, entityInfo, entityId, this);
        _entityInfoModel->addItem(this, item);
       _entityInfoItemMap[entityId] = item;

    } else {
        if (!_subTypeInfoItemMap.contains(entityInfo->getSubType())) {
            _subTypeInfoItemMap[entityInfo->getSubType()] = new TypeInfoItem(entityInfo->getSubType(), _entityInfoModel , this);
            _entityInfoModel->addItem(this, _subTypeInfoItemMap[entityInfo->getSubType()]);
        }
        _entityBySubType[entityId] = entityInfo->getSubType();
        _subTypeInfoItemMap[entityInfo->getSubType()]->addEntityInfo(entityId,entityInfo,false);
    }
}

bool TypeInfoItem::removeEntityInfo(long entityId)
{
    if (!_entityInfoItemMap.isEmpty()) {
         EntityInfoItem * item = _entityInfoItemMap.take(entityId);
         _entityInfoModel->removeItem(item);
         item->deleteLater();
         return _entityInfoItemMap.size() != 0;
    } else {
        if (_entityBySubType.contains(entityId)) {
            QString subType = _entityBySubType[entityId];
            if (!_subTypeInfoItemMap[subType]->removeEntityInfo(entityId)) {
                subType = _entityBySubType.take(entityId);
                TypeInfoItem * item = _subTypeInfoItemMap.take(subType);
                _entityInfoModel->removeItem(item);
                item->deleteLater();
                return _subTypeInfoItemMap.size() != 0;
            }
        }
    }
    return true;
}

} // namespace Hnv
