/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "entitycatalogmodel.h"
#include "hyneviewsession.h"
#include "ressources.h"

// hynesim hnsclient headers
#include <libhnsclient/entityentrycatalog.h>
#include <libhnsclient/entry.h>

// Qt headers
#include <QMimeData>
#include <QIcon>
#include <QDebug>

namespace Hnv {

/*******************************************************************************
* VirtualFolderItem
*******************************************************************************/
VirtualFolderItem::VirtualFolderItem(QString folderName, int column, QObject *parent)
    : TreeModelItem(column, parent)
    , _folderName(folderName)
{
}

int VirtualFolderItem::columnCount() const
{
    if (_parentItem)
        return _parentItem->columnCount();
    else
        return TreeModelItem::columnCount();
}

QVariant VirtualFolderItem::data(int column, int role) const
{
    if (role == Qt::DecorationRole) {
        return QIcon(Ressources::themedPath(Constants::ICON_DATABASE));
    } else if (column == 0 && ( role == Qt::DisplayRole || role == Qt::EditRole ))
        return _folderName;
    return TreeModelItem::data(column, role);
}

Qt::ItemFlags VirtualFolderItem::flags(int column) const
{
    Qt::ItemFlags flags = TreeModelItem::flags(column);
    flags |= Qt::ItemIsDropEnabled;
    return flags;
}

int VirtualFolderItem::internalType() const
{
    return EntityCatalogModel::VirtualFolder;
}

/*******************************************************************************
* TreeEntryItem
*******************************************************************************/
TreeEntryItem::TreeEntryItem(HnsClient::CatalogItem *entry, QObject *parent)
    : TreeModelItem(SIZE_COLUMN, parent)
    , EntryItem(entry, parent)
{
}

int TreeEntryItem::columnCount() const
{
    return EntryItem::columnCount();
}

QVariant TreeEntryItem::data(int column, int role) const
{
    QVariant result = EntryItem::data(column, role);
    if (result.isValid())
        return result;
    return TreeModelItem::data(column, role);
}

bool TreeEntryItem::setData(int column, const QVariant &value, int role)
{
    return EntryItem::setData(column, value, role);
}

Qt::ItemFlags TreeEntryItem::flags(int column) const
{
    return EntryItem::flags(column);
}

/*******************************************************************************
* FolderItem
*******************************************************************************/
FolderItem::FolderItem(HnsClient::FolderEntry *entry, QObject *parent)
    : TreeEntryItem(entry, parent)
{
}

FolderItem::~FolderItem()
{
}

int FolderItem::internalType() const
{
    return EntityCatalogModel::Folder;
}

QVariant FolderItem::internalSortValue() const
{
    return "100" + data(0, Qt::DisplayRole).toString();
}

QPixmap FolderItem::getPixmap() const
{
    return QPixmap(Ressources::themedPath(Constants::ICON_FOLDER));
}

qlonglong FolderItem::getParentID() const
{
    HnsClient::FolderEntry *entry = qobject_cast<HnsClient::FolderEntry *>(catalogItem());
    if (entry)
        return entry->getParentID();
    return 0;
}

/*******************************************************************************
* EntityItem
*******************************************************************************/
EntityItem::EntityItem(HnsClient::EntityEntry *entry, QObject *parent)
    : TreeEntryItem(entry, parent)
{
}

QVariant EntityItem::data(int column, int role) const
{
    HnsClient::EntityEntry *entry = qobject_cast<HnsClient::EntityEntry *>(catalogItem());
    if (entry) {
        if (role == EntityItem::IsSavedRole) {
            return entry->isSaved();
        } else if (role == EntityItem::HasLinkedClonesRole) {
            return entry->hasLinkedClones();
        } else if (role == EntityItem::IsLinkedCloneRole) {
            return entry->isLinkedClone();
        } else if(role == EntityItem::UuidRole) {
            return entry->getUUID();
        } else if(role == EntityItem::BaseUuidRole) {
            if (entry->isLinkedClone())
                return getBaseUuid();
            return QString();
        } else if (role == EntityItem::IsCloningRole) {
            return entry->isCloning();
        } else if (role == EntityItem::EntityStatusRole) {
            if (entry->isUsedAsEntity())
                return entry->getEntityStatus();
            else
                return QVariant();
        } else if (role == EntityItem::StatusRole) {
            return entry->getStatus();
        } else if (role == Qt::DisplayRole && column == 0 && entry->isCloning()) {
            int cloneProgress = entry->getCloneProgress() * 100.0;
            QString display = QString("%1 (%2 %)")
                    .arg(TreeEntryItem::data(column, role).toString())
                    .arg(QString::number(cloneProgress));
            return display;
        } else if (role == Qt::DisplayRole && column == 0 && entry->isLocked() && entry->lockType() == Hns::LOCK_TYPE_EXPORT) {
            QString display = QObject::tr("%1 (Exporting ...)")
                    .arg(TreeEntryItem::data(column, role).toString());
            return display;
        } else if (role == Qt::ToolTipRole) {
            if (entry->getStatus() == Hns::ENTITY_STATUS_UUID_CONFLICT)
                return TreeEntryItem::data(column,role).toString().append(QObject::tr("\n<br/><b>Problem:&nbsp;</b><i>There is an UUID conflict</i>"));
            else if (entry->getStatus() == Hns::ENTITY_STATUS_PROVISIONING) {
                if (entry->isLocked())
                    return TreeEntryItem::data(column,role).toString().append(QObject::tr("\n<br/><i>The entry is currently in provisioning</i>"));
                else
                    return TreeEntryItem::data(column,role).toString().append(QObject::tr("\n<br/><b>Problem:&nbsp;</b><i>The entry encountered errors during provisioning</i>"));
            }
        }
    }
    return TreeEntryItem::data(column, role);
}

int EntityItem::internalType() const
{
    return EntityCatalogModel::Entity;
}

QVariant EntityItem::internalSortValue() const
{
    return "200" + data(0, Qt::DisplayRole).toString();
}

QPixmap EntityItem::getPixmap() const
{
    return Ressources::getEntryPixmap(catalogItem());
}

Qt::ItemFlags EntityItem::flags(int column) const
{
    Qt::ItemFlags flagsEntityItem = TreeEntryItem::flags(column);
    QVariant dataEntityItem =  data(column, EntityItem::StatusRole);
    if (dataEntityItem.isValid() && dataEntityItem.toInt()) {
        int dataInt = dataEntityItem.toInt();
        if (dataInt == ::Hns::ENTITY_STATUS_UUID_CONFLICT || dataInt == Hns::ENTITY_STATUS_PROVISIONING) {
            if (column == 0) {
                flagsEntityItem &= ~Qt::ItemIsDragEnabled;
                flagsEntityItem &= ~Qt::ItemIsDropEnabled;
                flagsEntityItem &= ~Qt::ItemIsEditable;
            }
        }
    }
    return flagsEntityItem;
}

QString EntityItem::getUuid() const
{
    HnsClient::EntityEntry *entry = qobject_cast<HnsClient::EntityEntry *>(catalogItem());
    if (entry)
        return entry->getUUID();
    else
        return QString();
}

QString EntityItem::getBaseUuid() const
{
    HnsClient::DomainEntry *domainEntry = qobject_cast<HnsClient::DomainEntry *>(catalogItem());
    if (domainEntry)
        return domainEntry->getBaseUuid();
    return QString();
}

/*******************************************************************************
* EntityCatalogModel
*******************************************************************************/
EntityCatalogModel::EntityCatalogModel(QObject *parent)
    : TreeModel(parent)
{
    // Tree model initialization with specified columns number
    _initRootItem(EntryItem::SIZE_COLUMN);
    _catalogRootNode = 0;
}

EntityCatalogModel::~EntityCatalogModel()
{
    _resetModel();
}

void EntityCatalogModel::initModel(HnsClient::EntityEntryCatalog *catalog)
{
    if (catalog && _catalog != catalog) {
        _catalog = catalog;
        _populateCatalog();
        connect(_catalog, SIGNAL(folderAdded(HnsClient::FolderEntry*)), this, SLOT(_folderAddedSlot(HnsClient::FolderEntry*)));
        connect(_catalog, SIGNAL(folderUpdated(HnsClient::FolderEntry*)), this, SLOT(_folderUpdatedSlot(HnsClient::FolderEntry*)));
        connect(_catalog, SIGNAL(folderRemoved(qlonglong)), this, SLOT(_folderRemovedSlot(qlonglong)));
        connect(_catalog, SIGNAL(entityAdded(HnsClient::EntityEntry*)), this, SLOT(_entityAddedSlot(HnsClient::EntityEntry*)));
        connect(_catalog, SIGNAL(entityUpdated(HnsClient::EntityEntry*)), this, SLOT(_entityUpdatedSlot(HnsClient::EntityEntry*)));
        connect(_catalog, SIGNAL(entityRemoved(qlonglong)), this, SLOT(_entityRemovedSlot(qlonglong)));
        //    connect(_catalog, SIGNAL(connected()), this, SLOT(_connectedSlot()));
        //    connect(_catalog, SIGNAL(reseted()), this, SLOT(_resetedSlot()));
        //    connect(_catalog, SIGNAL(loadingCatalog(int,bool)), this, SIGNAL(loadingCatalog(int,bool)));
        connect(_catalog, SIGNAL(focusOnFolder(qlonglong)), this, SLOT(_focusOnFolderSlot(qlonglong)));
        connect(_catalog, SIGNAL(focusOnEntry(qlonglong,qlonglong)), this, SLOT(_focusOnEntrySlot(qlonglong,qlonglong)));
        connect(_catalog, SIGNAL(folderSyncingStateChanged(qlonglong,HnsClient::SynchronizationStatus)), this, SLOT(_folderSyncingStateChangedSlot(qlonglong,HnsClient::SynchronizationStatus)));
        connect(_catalog, SIGNAL(entitySyncingStateChanged(qlonglong,HnsClient::SynchronizationStatus)), this, SLOT(_entitySyncingStateChangedSlot(qlonglong,HnsClient::SynchronizationStatus)));
        emit catalogLoaded();
    }
}

void EntityCatalogModel::resetModel()
{
    _resetModel();
}

int EntityCatalogModel::getItemInternalType(QModelIndex targetIndex) const
{
    int result = -1;
    if (targetIndex.isValid()) {
        QVariant targetTypeData = targetIndex.data(TreeModelItem::InternalTypeRole);
        if (targetTypeData.isValid()) {
            result = targetTypeData.toInt();
        }
    }
    return result;
}

FolderItem * EntityCatalogModel::folderByID(qlonglong folderID)
{
    if (_folderHash.contains(folderID))
        return _folderHash[folderID];
    return 0;
}

Qt::DropActions EntityCatalogModel::supportedDropActions() const
{
    return Qt::CopyAction | Qt::MoveAction;
}

QStringList EntityCatalogModel::mimeTypes() const
{
    QStringList mimeTypes;
    mimeTypes << "hyneview/catalog-category"
              << "hyneview/catalog-entry"
              << "hyneview/catalog-category-selection"
              << "hyneview/catalog-entry-selection"
              << "hyneview/catalog-mixed-selection";
    return mimeTypes;
}

bool EntityCatalogModel::canHandleMimeData(const QMimeData *data, QString &datatype)
{
    QStringList supportedTypes = mimeTypes();
    bool dataIsSupported = false;
    foreach(QString sourceDataType, supportedTypes) {
        if (data->hasFormat(sourceDataType)) {
            dataIsSupported = true;
            datatype = sourceDataType;
            break;
        }
    }
    return dataIsSupported;
}

bool EntityCatalogModel::canHandleMimeData(const QMimeData *data)
{
    QString datatype;
    return canHandleMimeData(data, datatype);
}

QMimeData *EntityCatalogModel::mimeData(const QModelIndexList &indexes) const
{
    QMimeData *mimeData = new QMimeData();
    QString mimeType = "";
    int validItems = 0;
    QByteArray encodedData;
    QDataStream stream(&encodedData, QIODevice::WriteOnly);

    foreach(QModelIndex index, indexes) {
        if (!index.isValid())
            continue;
        int itemType = getItemInternalType(index);
        if (itemType == -1)
            continue;
        validItems++;
        QString itemMimeType;
        if (itemType == EntityCatalogModel::Folder)
            itemMimeType = "hyneview/catalog-category";
        else if (itemType == EntityCatalogModel::Entity)
            itemMimeType = "hyneview/catalog-entry";
        if (mimeType == "")
            mimeType = itemMimeType;
        else if (mimeType != itemMimeType) {
            mimeType = "hyneview/catalog-mixed-selection";
        }
        stream << itemType;
        stream << index.data(EntryItem::IDRole).toLongLong();
        stream << index.parent().data(EntryItem::IDRole).toLongLong();
        stream << index.data(TreeModelItem::InternalIDRole).toInt();
    }
    if (validItems > 1 && mimeType != "hyneview/catalog-mixed-selection") {
        if (mimeType == "hyneview/catalog-category")
            mimeType = "hyneview/catalog-category-selection";
        else if (mimeType == "hyneview/catalog-entry")
            mimeType = "hyneview/catalog-entry-selection";
        else
            mimeType = "hyneview/catalog-mixed-selection";
    }
    mimeData->setData(mimeType, encodedData);
    return mimeData;
}

bool EntityCatalogModel::dropMimeData(const QMimeData *data, Qt::DropAction action, int row, int column, const QModelIndex &parent)
{
    Q_UNUSED(row)
    if (action == Qt::IgnoreAction)
        return true;

    if (column > 0 )
        return false;
    if (action == Qt::MoveAction)
        return handleMimeData(data, parent, EntityCatalogModel::MimeDataMove);
    return false;
}

bool EntityCatalogModel::handleMimeData(const QMimeData *data, const QModelIndex &parent, CatalogMimeDataAction action)
{
    QString dataType = "";
    bool dataIsSupported = canHandleMimeData(data, dataType);

    if (!dataIsSupported)
        return false;

    int parentType = getItemInternalType(parent);
    qlonglong parentID = -1;

    if (parentType == EntityCatalogModel::VirtualFolder)
        parentID = 0;
    else if (parentType == EntityCatalogModel::Folder)
        parentID = parent.data(EntryItem::IDRole).toLongLong();

    if (parentID == -1)
        return false;

    if (parentID != 0 && !canUpdateIndex(parent)) {
        qDebug() << tr("Operation forbidden: '%1' is read-only").arg(parent.data().toString());
        return false;
    }

    QByteArray itemData = data->data(dataType);
    QDataStream dataStream(&itemData, QIODevice::ReadOnly);
    QHash<qlonglong, qlonglong> entitiesSourcePosition;
    while (!dataStream.atEnd()) {
        int itemType;
        qlonglong sourceID;
        qlonglong sourceParentID;
        int itemInternalID;
        dataStream >> itemType >> sourceID >> sourceParentID >> itemInternalID;

        if (itemType == EntityCatalogModel::Folder) {
            if (action == EntityCatalogModel::MimeDataMove)
                _requestMoveFolder(sourceID, parentID);
        } else if (itemType == EntityCatalogModel::Entity) {
            entitiesSourcePosition.insert(sourceID, sourceParentID);
        }
    }
    if (entitiesSourcePosition.size()) {
        if (action == EntityCatalogModel::MimeDataMove)
            _requestEntityMoveInto(entitiesSourcePosition, parentID);
    }
    return true;
}

bool EntityCatalogModel::canUpdateIndex(QModelIndex index)
{
    Q_UNUSED(index)
    return true;
}

bool EntityCatalogModel::canUpdateItem(TreeEntryItem *item)
{
    Q_UNUSED(item)
    return true;
}

QList<EntityItem *> EntityCatalogModel::getEntityList()
{
    return _entityList;
}

bool EntityCatalogModel::setData(const QModelIndex &index, const QVariant &value, int role)
{
    int indexType = getItemInternalType(index);
    if (indexType == -1 || indexType == EntityCatalogModel::VirtualFolder)
        return false;

    if (!canUpdateIndex(index)) {
        qDebug() << tr("Operation forbidden: '%1' is not available for update.").arg(index.data().toString());
        return false;
    }

    qlonglong itemID = index.data(EntryItem::IDRole).toLongLong();
    if (indexType == EntityCatalogModel::Folder) {
        HnsClient::FolderEntry * sourceEntry = _catalog->folderById(itemID);
        if (sourceEntry) {
            HnsClient::FolderEntry * updatedEntry = sourceEntry->duplicateFolder(this);
            FolderItem * updatedItem = new FolderItem(updatedEntry, this);
            if (updatedItem->setData(index.column(), value, role)) {
                _catalog->updateFolder(updatedEntry);
                delete updatedItem;
                return true;
            }
            delete updatedItem;
            delete updatedEntry;
        } else {
            qWarning() << tr("Could not update '%1'. Folder is invalid.").arg(index.data().toString());
        }
    } else if (indexType == EntityCatalogModel::Entity) {
        HnsClient::EntityEntry * sourceEntry = _catalog->entityEntryById(itemID);
        if (sourceEntry) {
            HnsClient::Entry *entryClone = sourceEntry->duplicateEntry(this);
            HnsClient::EntityEntry *updatedEntry = qobject_cast<HnsClient::EntityEntry *>(entryClone);
            EntityItem * updatedItem = new EntityItem(updatedEntry, this);
            if (updatedItem->setData(index.column(), value, role)) {
                _catalog->updateEntityEntry(updatedEntry);
                delete updatedItem;
                return true;
            }
            delete updatedItem;
            delete updatedEntry;
        } else {
            tr("Could not update '%1'. Entity is invalid.").arg(index.data().toString());
        }
    }
    return false;
}

void EntityCatalogModel::_resetModel()
{
    beginResetModel();
    _rootItem->clear();
   _catalogRootNode = 0;
   _entityHash.clear();
   _folderHash.clear();
   endResetModel();
}

/*******************************************************************************
  Tree management
  *****************************************************************************/
void EntityCatalogModel::_attachItemToParent(TreeEntryItem *item, qlonglong parentID)
{
    TreeModelItem *parentItem = 0;
    if (parentID == 0)
        parentItem = _catalogRootNode;
    else
        parentItem = folderByID(parentID);
    if (parentItem != 0)
        addItem(parentItem, item);
}

void EntityCatalogModel::_populateCatalog()
{
    _catalogRootNode = new VirtualFolderItem(tr("%1@%2").arg(HyneviewSession::instance()->userName()).arg(HyneviewSession::instance()->masterName()), EntryItem::SIZE_COLUMN, this);
    addItem(_rootItem, _catalogRootNode);

    QList<HnsClient::FolderEntry *> folders = _catalog->getFolderChildren();
    foreach(HnsClient::FolderEntry *folder, folders)
        _createFolderNode(folder, false);

    QList<HnsClient::EntityEntry *> entities = _catalog->getEntities().values();
    foreach(HnsClient::EntityEntry *entity, entities)
        _createEntityNode(entity, entity->getParentID());
}

void EntityCatalogModel::_itemChanged(TreeEntryItem *item)
{
    QModelIndex firstIdx = indexFromNode(item);
    QModelIndex lastIdx = indexFromNode(item, item->columnCount() - 1);
    emit dataChanged(firstIdx, lastIdx);
}

void EntityCatalogModel::_createFolderNode(HnsClient::FolderEntry *folder, bool addEntityChildren)
{
    if (!folder)
        return;
    qlonglong folderID = folder->getID();
    if (_folderHash.contains(folderID)) {
        qDebug() << tr("Folder already in cache");
        return;
    }
    qlonglong parentID = folder->getParentID();
    if (parentID == 0 || _folderHash.contains(parentID)) {
        FolderItem * item = new FolderItem(folder, this);
        _folderHash.insert(folderID, item);
        _attachItemToParent(item, folder->getParentID());
        foreach(HnsClient::FolderEntry *subFolder, _catalog->getFolderChildren(folderID))
            _createFolderNode(subFolder, addEntityChildren);
        if (addEntityChildren) {
            foreach(HnsClient::EntityEntry *entity, _catalog->getEntityChildren(folderID))
                _createEntityNode(entity, folderID);
        }
        if (_folderItemAdded.contains(folderID)) {
            _folderItemAdded.removeAll(folderID);
            emit focusOnItem(indexFromNode(_folderHash[folderID]));
        }
    }
}

void EntityCatalogModel::_updateFolderNode(HnsClient::FolderEntry *folder)
{
    qlonglong folderID = folder->getID();
    if (_folderHash.contains(folderID)) {
        TreeModelItem *currentParentItem = _folderHash[folderID]->parentItem();
        qlonglong currentParentID = -1;
        if (currentParentItem)
            currentParentID = currentParentItem->data(0, EntryItem::IDRole).toLongLong();
        qlonglong targetParentID = folder->getParentID();
        if (currentParentID != targetParentID) {
            if (currentParentID != -1)
                removeItem(_folderHash.value(folderID));
            _attachItemToParent(_folderHash.value(folderID), targetParentID);
        }
        _itemChanged(_folderHash.value(folderID));
    } else
        _createFolderNode(folder);
}

void EntityCatalogModel::_removeFolderNode(qlonglong folderID)
{
    FolderItem *item = _folderHash.take(folderID);
    if (item) {
        removeItem(item);
        QList<qlonglong> folderChildren;
        QHash<qlonglong, QList<qlonglong> > entityChildren;
        _enumerateChildrenItems(item, folderChildren, entityChildren);

        foreach(qlonglong childFolderID, folderChildren)
            _folderHash.remove(childFolderID);
        QMutableHashIterator<qlonglong, QList<qlonglong> > iterator(entityChildren);
        while(iterator.hasNext()) {
            iterator.next();
            qlonglong entityID = iterator.key();
            if (_entityHash.contains(entityID))
                _entityHash.remove(entityID);
        }
        delete(item);
    }
}

int EntityCatalogModel::_enumerateChildrenItems(TreeModelItem *folder, QList<qlonglong> &folderChildrenIDs, QHash<qlonglong, QList<qlonglong> > &entityChildrenIDs)
{
    int childrenCount = 0;
    if (folder != 0) {
        int directChildrenCount = folder->childCount();
        childrenCount = directChildrenCount;
        qlonglong folderID = folder->data(0, EntryItem::IDRole).toLongLong();
        for (int i = 0; i < directChildrenCount; i++) {
            TreeModelItem *child = folder->child(i);
            if (child) {
                QVariant childTypeData = child->data(0, TreeModelItem::InternalTypeRole);
                if (childTypeData.isValid()) {
                    qlonglong childID = child->data(0, EntryItem::IDRole).toLongLong();
                    int childType = childTypeData.toInt();
                    if (childType == EntityCatalogModel::Folder) {
                        folderChildrenIDs.append(childID);
                        childrenCount += _enumerateChildrenItems(child, folderChildrenIDs, entityChildrenIDs);
                    } else if (childType == EntityCatalogModel::Entity)
                        entityChildrenIDs[childID].append(folderID);
                }
            }
        }
    }
    return childrenCount;
}

void EntityCatalogModel::_createEntityNode(HnsClient::EntityEntry *entity, qlonglong parentID)
{
    if (!entity)
        return;
    qlonglong entityID = entity->getID();
    if (_entityHash.contains(entityID)) {
        qDebug() << tr("Entity item already in cache.");
        return;
    }
    if (parentID == 0 || _folderHash.contains(parentID)) {
        EntityItem * item = new EntityItem(entity, this);
        _entityHash.insert(entityID, QPair<qlonglong, EntityItem *>(parentID, item));
        _entityList.append(item);
        _attachItemToParent(item, parentID);
    }
}

void EntityCatalogModel::_updateEntityNode(HnsClient::EntityEntry *entity)
{
    qlonglong entityID = entity->getID();
    qlonglong newParentId = entity->getParentID();

    if (_entityHash.contains(entityID)) {
        if (newParentId != _entityHash.value(entityID).first) {
            EntityItem *item = _entityHash.value(entityID).second;
            removeItem(item);
            delete item;
            _entityHash.remove(entityID);
            _createEntityNode(entity, newParentId);
        } else {
            _itemChanged(_entityHash.value(entityID).second);
        }
    }
}

void EntityCatalogModel::_folderAddedSlot(HnsClient::FolderEntry *folder)
{
    _createFolderNode(folder);
}

void EntityCatalogModel::_folderUpdatedSlot(HnsClient::FolderEntry *folder)
{
    if (!folder)
        return;
    if (_folderHash.contains(folder->getID()))
        _updateFolderNode(folder);
    else
        _createFolderNode(folder);
}

void EntityCatalogModel::_folderRemovedSlot(qlonglong folderID)
{
    if (_folderHash.contains(folderID))
        _removeFolderNode(folderID);
    else
        qDebug() << tr("Folder not in cache, can not delete.");

}

void EntityCatalogModel::_entityAddedSlot(HnsClient::EntityEntry *entity)
{
    _createEntityNode(entity, entity->getParentID());
}

void EntityCatalogModel::_entityUpdatedSlot(HnsClient::EntityEntry *entity)
{
    if (!entity)
        return;
    qlonglong entityID = entity->getID();
    if (_entityHash.contains(entityID))
        _updateEntityNode(entity);
    else{
        qDebug() << tr("Entity not in cache");
        _createEntityNode(entity, entity->getParentID());
    }
}

void EntityCatalogModel::_entityRemovedSlot(qlonglong entityID)
{
    if (_entityHash.contains(entityID)) {
        EntityItem *item = _entityHash.value(entityID).second;
        removeItem(item);
        delete item;
        _entityHash.remove(entityID);
    }
}

void EntityCatalogModel::_resetedSlot()
{
    _resetModel();
}

void EntityCatalogModel::_connectedSlot()
{
    _populateCatalog();
}

void EntityCatalogModel::_focusOnFolderSlot(qlonglong folderID)
{
    if (_folderHash.contains(folderID))
        emit focusOnItem(indexFromNode(_folderHash[folderID]));
    else
        _folderItemAdded.append(folderID);
}

void EntityCatalogModel::_focusOnEntrySlot(qlonglong entryID, qlonglong parentID)
{
    if (_entityHash.contains(entryID) && parentID != -1)
        emit focusOnItem(indexFromNode(_entityHash.value(entryID).second));
}

void EntityCatalogModel::_folderSyncingStateChangedSlot(qlonglong folderID, HnsClient::SynchronizationStatus status)
{
    if (status == HnsClient::Syncing && _folderHash.contains(folderID)) {
        QModelIndex folderIndex = indexFromNode(_folderHash.value(folderID));
        emit beginTransaction(QPersistentModelIndex(folderIndex));
    }
}

void EntityCatalogModel::_entitySyncingStateChangedSlot(qlonglong entityID, HnsClient::SynchronizationStatus status)
{
    if (status == HnsClient::Syncing && _entityHash.contains(entityID)) {
        QModelIndex entityIndex = indexFromNode(_entityHash.value(entityID).second);
        emit beginTransaction(QPersistentModelIndex(entityIndex));
    }
}

void EntityCatalogModel::_requestMoveFolder(qlonglong folderID, qlonglong newParentID)
{
    _catalog->moveFolder(folderID, newParentID);
}

void EntityCatalogModel::_requestEntityMoveInto(QHash<qlonglong, qlonglong> entitiesSourcePosition, qlonglong newParentID)
{
    if (newParentID != 0 && !_folderHash.contains(newParentID)) {
        qDebug() << tr("Destination folder not in cache. Could not move entity.");
        return;
    }

    QHashIterator<qlonglong, qlonglong> iterator(entitiesSourcePosition);
    while (iterator.hasNext()) {
        iterator.next();
        qlonglong entityId = iterator.key();
        HnsClient::EntityEntry *entry = _catalog->entityEntryById(entityId);
        if (!entry) {
            qDebug() << tr("Entity not in cache, could not move");
            continue;
        }
        HnsClient::EntityEntry *updatedEntry = qobject_cast<HnsClient::EntityEntry *>(entry->duplicateEntry(this));
        updatedEntry->setParentID(newParentID);
        _catalog->updateEntityEntry(updatedEntry, newParentID);
    }
}

/*******************************************************************************
* LinkedClonesFilterProxyModel
*******************************************************************************/
LinkedClonesFilterProxyModel::LinkedClonesFilterProxyModel(QObject *parent)
    : QSortFilterProxyModel(parent)
{
}

LinkedClonesFilterProxyModel::~LinkedClonesFilterProxyModel()
{
}

void LinkedClonesFilterProxyModel::setUuid(const QString &uuid)
{
    _uuid = uuid;
    invalidateFilter();
}

bool LinkedClonesFilterProxyModel::filterAcceptsRow(int source_row, const QModelIndex &source_parent) const
{

    QModelIndex rowIndex = sourceModel()->index(source_row, 0, source_parent);
    return _itemIsValid(rowIndex);
}

bool LinkedClonesFilterProxyModel::_itemIsValid(QModelIndex rowIndex) const
{
    bool result = false;
    QModelIndex index = rowIndex.sibling(rowIndex.row(), filterKeyColumn());
    if (index.isValid()) {
        if (_uuid.isEmpty()) {
            result = true;
        } else {
            result = (index.data(EntityItem::UuidRole) == _uuid)|| (index.data((EntityItem::BaseUuidRole)) == _uuid);
        }
    }
    if (!result)
        result = _hasValidChild(rowIndex);
    return result;
}

bool LinkedClonesFilterProxyModel::_hasValidChild(QModelIndex parent) const
{
    int childCount = sourceModel()->rowCount(parent);
    for (int row = 0 ; row < childCount ; ++row) {
        if (_itemIsValid(parent.child(row, 0)))
            return true;
    }
    return false;
}

} // namespace Hnv
