/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef ENTITYCATALOGMODEL_H
#define ENTITYCATALOGMODEL_H

// Project headers
#include "treemodel.h"
#include "treemodelitem.h"
#include "hnv_catalogitem.h"

// Qt project headers
#include <QSortFilterProxyModel>

namespace HnsClient {
class EntityEntry;
class EntityEntryCatalog;
class CatalogItem;
class FolderEntry;
} // namespace HnsClient

namespace Hnv {

/*******************************************************************************
* VirtualFolderItem
*******************************************************************************/
class VirtualFolderItem : public TreeModelItem
{
public:
    explicit VirtualFolderItem(QString folderName, int column = 1, QObject *parent = 0);
    virtual int columnCount() const;
    virtual QVariant data(int column, int role = Qt::DisplayRole) const;
    virtual Qt::ItemFlags flags(int column) const;
    virtual int internalType() const;
private:
    QString _folderName;
};

/*******************************************************************************
* TreeEntryItem
*******************************************************************************/
class TreeEntryItem : public TreeModelItem, public EntryItem
{
public:
    explicit TreeEntryItem(HnsClient::CatalogItem *entry, QObject *parent = 0);
    virtual int columnCount() const;
    virtual QVariant data(int column = 0, int role = Qt::DisplayRole) const;
    virtual bool setData(int column, const QVariant &value, int role);
    virtual Qt::ItemFlags flags(int column) const;
};

/*******************************************************************************
 * FolderItem
*******************************************************************************/
class FolderItem : public TreeEntryItem
{
public:
    explicit FolderItem(HnsClient::FolderEntry *entry, QObject *parent = 0);
    ~FolderItem();
    virtual int internalType() const;
    virtual QVariant internalSortValue() const;
    virtual QPixmap getPixmap() const;
    virtual qlonglong getParentID() const;
};

/*******************************************************************************
* EntityItem
*******************************************************************************/
class EntityItem : public TreeEntryItem
{
public:
    enum EntityEntryRole {
        StatusRole = EntryItem::IDRole + 100,
        EntityStatusRole,
        IsCloningRole,
        IsSavedRole,
        UuidRole,
        BaseUuidRole
    };
    explicit EntityItem(HnsClient::EntityEntry *entry, QObject *parent = 0);
    virtual QVariant data(int column = 0, int role = Qt::DisplayRole) const;
    virtual int internalType() const;
    virtual QVariant internalSortValue() const;
    virtual QPixmap getPixmap() const;
    virtual Qt::ItemFlags flags(int column) const;
    virtual QString getUuid() const;
    virtual QString getBaseUuid() const;
};

/*******************************************************************************
* EntityCatalogModel
*******************************************************************************/
class EntityCatalogModel : public TreeModel
{
    Q_OBJECT
public:
    enum EntryType {
        VirtualFolder = 1,
        Folder,
        Entity
    };
    enum CatalogMimeDataAction {
        MimeDataMove,
    };

    explicit EntityCatalogModel(QObject *parent = 0);
    ~EntityCatalogModel();

    void initModel(HnsClient::EntityEntryCatalog *catalog);
    void resetModel();

    int getItemInternalType(QModelIndex targetIndex) const;
    FolderItem *folderByID(qlonglong folderID);
    Qt::DropActions supportedDropActions() const;
    QStringList mimeTypes() const;
    bool canHandleMimeData(const QMimeData *data, QString &datatype);
    bool canHandleMimeData(const QMimeData *data);
    QMimeData *mimeData(const QModelIndexList &indexes) const;
    bool dropMimeData(const QMimeData *data, Qt::DropAction action, int row, int column, const QModelIndex &parent);

    bool handleMimeData(const QMimeData *data, const QModelIndex &parent, CatalogMimeDataAction action = EntityCatalogModel::MimeDataMove);

    bool setData(const QModelIndex &index, const QVariant &value, int role);

    bool canUpdateIndex(QModelIndex index);
    bool canUpdateItem(TreeEntryItem *item);
    QList<EntityItem *> getEntityList();

signals:
    void catalogLoaded();
    void focusOnItem(QModelIndex index);
    void beginTransaction(QPersistentModelIndex index);

private slots:
    void _folderAddedSlot(HnsClient::FolderEntry *folder);
    void _folderUpdatedSlot(HnsClient::FolderEntry *folder);
    void _folderRemovedSlot(qlonglong folderID);
    void _entityAddedSlot(HnsClient::EntityEntry *entity);
    void _entityUpdatedSlot(HnsClient::EntityEntry *entity);
    void _entityRemovedSlot(qlonglong entityID);
    void _resetedSlot();
    void _connectedSlot();
    void _focusOnFolderSlot(qlonglong folderID);
    void _focusOnEntrySlot(qlonglong entryID, qlonglong parentID);
    void _folderSyncingStateChangedSlot(qlonglong folderID, HnsClient::SynchronizationStatus status);
    void _entitySyncingStateChangedSlot(qlonglong entityID, HnsClient::SynchronizationStatus status);
private:
    void _resetModel();
    void _populateCatalog();
    void _itemChanged(TreeEntryItem *item);
    void _createFolderNode(HnsClient::FolderEntry *folder, bool addEntityChildren = true);
    void _updateFolderNode(HnsClient::FolderEntry *folder);
    void _removeFolderNode(qlonglong folderID);
    void _createEntityNode(HnsClient::EntityEntry *entity, qlonglong parentID);
    void _updateEntityNode(HnsClient::EntityEntry *entity);
    void _attachItemToParent(TreeEntryItem *item, qlonglong parentID);
    void _requestMoveFolder(qlonglong folderID, qlonglong newParentID);
    void _requestEntityMoveInto(QHash<qlonglong, qlonglong> entitiesSourcePosition, qlonglong newParentID);
    int _enumerateChildrenItems(TreeModelItem *folder, QList<qlonglong> &folderChildrenIDs, QHash<qlonglong, QList<qlonglong> > &entityChildrenIDs);

    HnsClient::EntityEntryCatalog * _catalog;
    TreeModelItem * _catalogRootNode;
    QHash<qlonglong, FolderItem *> _folderHash;
    QHash<qlonglong, QPair<qlonglong, EntityItem *> > _entityHash; // EntityEntryID - ParentID - EntityItem
    QList<EntityItem *> _entityList;
    QList<qlonglong> _folderItemAdded;
};

/*******************************************************************************
* LinkedClonesFilterProxyModel
*******************************************************************************/
class LinkedClonesFilterProxyModel : public QSortFilterProxyModel
{
    Q_OBJECT
public:
    LinkedClonesFilterProxyModel(QObject *parent = 0);
    ~LinkedClonesFilterProxyModel();
    void setUuid(const QString &uuid);

protected:
    virtual bool filterAcceptsRow(int source_row, const QModelIndex &source_parent) const;

private:
    bool _itemIsValid(QModelIndex rowIndex) const;
    bool _hasValidChild(QModelIndex parent) const;

    QString _uuid;
};

} // namespace Hnv
#endif // ENTITYCATALOGMODEL_H
