/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "remotemanager.h"
#include "remoteviewfactory.h"
#include "remoteview.h"
#include "remoteclient.h"
#include "hnvmainwindow.h"

// hynesim hnsclient headers
#include <libhnsclient/entity.h>

// Qt headers
#include <QTimer>
#include <QSignalMapper>

namespace RemoteAccess {

RemoteManager * RemoteManager::_instance = NULL;

void RemoteManager::create(QObject *parent)
{
    if (!_instance)
        _instance = new RemoteManager(parent);
}

RemoteManager * RemoteManager::instance()
{
    return _instance;
}

RemoteManager::~RemoteManager()
{
    _timer->disconnect();
    delete _remoteViewFactory;
    qDeleteAll(_clientHash);
}

UsbManager *RemoteManager::usbManager(int id)
{
    if (_clientHash.contains(id))
        return _clientHash.value(id)->usbManager();
    return 0;
}

int RemoteManager::usbDeviceConnected(UsbDevice device, int entityId)
{
    QHashIterator<int, RemoteClient *> it(_clientHash);
    while (it.hasNext()) {
        it.next();
        if (it.key() == entityId)
            continue;
        UsbManager *usbManager = it.value()->usbManager();
        if (usbManager->deviceConnected(device))
            return it.key();
    }
    return -1;
}

bool RemoteManager::hasUsbDeviceConnected(int id)
{
    if (_clientHash.contains(id))
        return _clientHash.value(id)->usbManager()->hasDeviceConnected();
    return false;
}

void RemoteManager::remoteViewClosedSlot(int id)
{
    if (_clientHash.contains(id)) {
        RemoteClient * client = _clientHash.take(id);
        delete client;
        if (_nameHash.contains(id))
            _nameHash.remove(id);
    }
}

void RemoteManager::newConnection(HnsClient::GenericEntity *entity)
{
    if (entity->isStarted() || entity->getStatus() == Hns::PAUSED) {
        if (!_nameHash.contains(entity->getId())) {
            _nameHash.insert(entity->getId(), entity->getShortName());
            entity->getRemoteDisplayInfo(this, SLOT(_getRemoteDisplayInfo(int,Hns::RemoteDisplayInfo)));
        }
        else {
            emit needToChangeIndex(entity->getId());
        }
    }
}

bool RemoteManager::openConnection(int id, Hns::RemoteDisplayInfo info)
{
    //return true if successful, false otherwise
    //first check if we can handle such a connection
    if (!_canHandle(info.type)) {
        qDebug() << tr("Cannot handle this type of connection");
        return false;
    }

    //then wait if the connection manager can connect to the machine
    if (!_createClient(id, info)) {
        qDebug() << tr("Cannot open connection");
        return false;
    }
    emit connectionOpened(id);
    return true;
}

void RemoteManager::closeConnection(int id)
{
    if (_clientHash.contains(id)) {
        RemoteClient *client = _clientHash.value(id);
        client->closeConnection();
        if (_nameHash.contains(id))
            _nameHash.remove(id);
    }
}

bool RemoteManager::updateShortname(int id, const QString &shortname)
{
    if (_nameHash.contains(id)) {
        _nameHash.insert(id, shortname);
        emit shortnameChanged(id, shortname);
        return true;
    }
    return false;
}

RemoteView *RemoteManager::createRemoteView(int id)
{
    RemoteClient *client = _clientHash.value(id);
    RemoteView *view = _remoteViewFactory->createRemoteView(client->display());
    return view;
}

QImage RemoteManager::screenshot(int id)
{
    if (_clientHash.contains(id)) {
        RemoteClient *client = _clientHash.value(id);
        return client->getScreenShot();
    }
    return QImage();
}

QString RemoteManager::getShortNameById(int id)
{
    if (_nameHash.contains(id))
        return _nameHash.value(id);
    return QString();
}

Hns::RemoteDisplayInfo RemoteManager::remoteDisplayInfo(int id)
{
    RemoteClient *client = _clientHash.value(id);
    return client->info();
}

bool RemoteManager::canRedirectUsb(int id)
{
    if (!_clientHash.contains(id))
        return false;
    if (_clientHash.value(id)->info().type != Hns::SpiceViewer)
        return false;
    return _clientHash.value(id)->canRedirectUsb();
}

RemoteManager::RemoteManager(QObject *parent)
    : QObject(parent)
    , _usbDeviceSignalMapper(new QSignalMapper(this))
    , _canRedirectUsbSignalMapper(new QSignalMapper(this))
{
    _timer = new QTimer(this);
    _timer->setInterval(1000);
    connect(_timer, SIGNAL(timeout()), this, SLOT(_timerOut()), Qt::QueuedConnection);
    _timer->start();
    _remoteViewFactory = new RemoteViewFactory();
    QList<RemoteAccess::RemoteWorkerPlugin*> list = Hnv::HnvMainWindow::instance()->pluginManager()->pluginListByType<RemoteAccess::RemoteWorkerPlugin>();
    foreach(RemoteWorkerPlugin * plugin, list) {
        _pluginHash.insert(plugin->type(), plugin);
    }
    connect(Hnv::HnvMainWindow::instance(), SIGNAL(sessionDisconnected()), this, SLOT(_closeAllConnections()));
    connect(_usbDeviceSignalMapper, SIGNAL(mapped(int)), this, SLOT(_usbDevicesResetSlot(int)));
    connect(_canRedirectUsbSignalMapper, SIGNAL(mapped(int)), this, SLOT(_canRedirectUsbChangedSlot(int)));
}

bool RemoteManager::_canHandle(Hns::RemoteDisplayType type)
{
    if (_pluginHash.contains(type)) {
        return true;
    }
    return false;
}

bool RemoteManager::_createClient(int id, Hns::RemoteDisplayInfo info)
{
    RemoteClient *client = new RemoteClient(info);
    connect(client, SIGNAL(connectionEnded(int)), this, SLOT(_connectionEndedSlot(int)));
    connect(client, SIGNAL(connectionError(int,QString)), this, SIGNAL(connectionError(int,QString)));
    client->setId(id);
    client->openConnection();
    _clientHash.insert(id, client);
    UsbManager *usbManager = client->usbManager();
    if (usbManager) {
        connect(usbManager, SIGNAL(devicesReset()), _usbDeviceSignalMapper, SLOT(map()));
        _usbDeviceSignalMapper->setMapping(usbManager, id);
    }
    connect(client, SIGNAL(canRedirectUsbChanged()), _canRedirectUsbSignalMapper, SLOT(map()));
    _canRedirectUsbSignalMapper->setMapping(client, id);
    return true;
}

Worker *RemoteManager::_createWorker(Hns::RemoteDisplayInfo info)
{
    RemoteWorkerPlugin *plugin = _pluginHash.value(info.type);
    Worker *worker = plugin->worker();
    return worker;
}

void RemoteManager::_getRemoteDisplayInfo(int id, Hns::RemoteDisplayInfo info)
{
    openConnection(id, info);
}

void RemoteManager::_timerOut()
{
    foreach (RemoteClient *client, _clientHash.values()) {
        emit screenshotChanged(client->id(), client->getScreenShot());
    }
}

void RemoteManager::_closeAllConnections()
{
    QHashIterator<int, RemoteClient*> iterator(_clientHash);
    while (iterator.hasNext()) {
        iterator.next();
        closeConnection(iterator.key());
    }
}

void RemoteManager::_connectionEndedSlot(int id)
{
    emit connectionEnded(id);
    QMetaObject::invokeMethod(this, "remoteViewClosedSlot", Qt::QueuedConnection, Q_ARG(int, id));
}

void RemoteManager::_usbDevicesResetSlot(int id)
{
    UsbManager *usbManager = _clientHash.value(id)->usbManager();
    emit usbDeviceReset(id, usbManager->hasDeviceConnected());
}

void RemoteManager::_canRedirectUsbChangedSlot(int id)
{
    emit canRedirectUsbChanged(id);
}

} // namespace RemoteAccess
