/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "remoteview.h"
#include "hnvmainwindow.h"
#include "display.h"
#include "ressources.h"

// Qt headers
#include <QSettings>
#include <QPainter>
#include <QApplication>
#include <QClipboard>
#include <QHBoxLayout>
#include <QMouseEvent>
#include <QKeyEvent>

namespace RemoteAccess {

/*******************************************************************************
* RemoteView
*******************************************************************************/
RemoteView::RemoteView(QWidget *parent)
    : QWidget(parent)
{
}

void RemoteView::invalidate()
{
}

/*******************************************************************************
* WidgetRemoteView
*******************************************************************************/
WidgetRemoteView::WidgetRemoteView(QWidget *parent)
    : RemoteView(parent)
    , _widget(0)
{
    setLayout(new QHBoxLayout());
    setAttribute(Qt::WA_NativeWindow);
    layout()->setMargin(0);
}

void WidgetRemoteView::setDisplay(Display *display)
{
    WidgetDisplay *wd = qobject_cast<WidgetDisplay *>(display);
    _widget = wd->widget();
    if (_widget) {
        layout()->addWidget(_widget);
        resize(_widget->size());
    }
}

void WidgetRemoteView::invalidate()
{
    layout()->removeWidget(_widget);
}

/*******************************************************************************
* ImageRemoteView
*******************************************************************************/
ImageRemoteView::ImageRemoteView(QWidget *parent)
    : RemoteView(parent)
{
    QSettings settings;
    QString resolution = settings.value(Hnv::Constants::REMOTEDISPLAY_RESOLUTION_OPTION).toString();
    if (resolution.isEmpty())
        resolution = "640x480";
    QStringList resolutionList = resolution.split("x");
    int width = resolutionList.at(0).toInt();
    int height = resolutionList.at(1).toInt();
    _image = QPixmap(QSize(width, height));
    _image.fill(Qt::black);
    resize(_image.size());
    qRegisterMetaType<RemoteAccess::KeyInfo>("RemoteAccess::KeyInfo");
    qRegisterMetaType<RemoteAccess::MouseInfo>("RemoteAccess::MouseInfo");
    setAttribute(Qt::WA_OpaquePaintEvent);
    setMouseTracking(true);
    setFocusPolicy(Qt::WheelFocus);
    setFocus();
    QClipboard *clipboard = QApplication::clipboard();
    connect(clipboard, SIGNAL(dataChanged()), this, SLOT(_clipboardDataChangedSlot()));
    connect(clipboard, SIGNAL(selectionChanged()), this, SLOT(_clipboardSelectionChangedSlot()));
}

void ImageRemoteView::setDisplay(Display *display)
{
    ImageDisplay *id = qobject_cast<ImageDisplay *>(display);
    connect(this, SIGNAL(keyEvent(RemoteAccess::KeyInfo)), id, SIGNAL(keyboardEventReceived(RemoteAccess::KeyInfo)));
    connect(this, SIGNAL(mouseEvent(RemoteAccess::MouseInfo)), id, SIGNAL(mouseEventReceived(RemoteAccess::MouseInfo)));
    connect(this, SIGNAL(clipboardDataEvent(QString)), id, SIGNAL(clipboardDataHostToGuest(QString)));
    connect(this, SIGNAL(clipboardSelectionEvent(QString)), id, SIGNAL(clipboardSelectionHostToGuest(QString)));
    connect(this, SIGNAL(focusOutEvent()), id, SIGNAL(focusOutEvent()));
    connect(id, SIGNAL(clearImaged()), this, SLOT(clearImage()));
    connect(id, SIGNAL(cursorChanged(QImage,int,int)), this, SLOT(cursorReceive(QImage,int,int)));
    connect(id, SIGNAL(imageChanged(QImage,int,int,int,int)), this, SLOT(imageReceive(QImage,int,int,int,int)));
    connect(id, SIGNAL(sizeChanged(QSize)), this, SLOT(sizeChange(QSize)));
    connect(id, SIGNAL(clipboardDataGuestToHost(QString)), this, SLOT(clipboardDataEventSlot(QString)));
    connect(id, SIGNAL(clipboardSelectionGuestToHost(QString)), this, SLOT(clipboardSelectionEventSlot(QString)));
    connect(id, SIGNAL(clipboardDataClearedGuestToHost()), this, SLOT(clipboardDataClearedSlot()));
    connect(id, SIGNAL(clipboardSelectionClearedGuestToHost()), this, SLOT(clipboardSelectionClearedSlot()));
}

void ImageRemoteView::paintEvent(QPaintEvent *event)
{
    event->accept();
    QPainter painter(this);
    painter.drawPixmap(0, 0, _image.width(), _image.height(), _image);
    QWidget::paintEvent(event);
}

void ImageRemoteView::mouseMoveEvent(QMouseEvent *event)
{
    MouseInfo info;
    info.x = event->x();
    info.y = event->y();
    if (event->button() == Qt::LeftButton)
        info.button = Left;
    else if (event->button() == Qt::MidButton)
        info.button = Mid;
    else if (event->button() == Qt::RightButton)
        info.button = Right;
    else
        info.button = None;
    info.state = Move;
    emit mouseEvent(info);
}

void ImageRemoteView::mousePressEvent(QMouseEvent *event)
{
    MouseInfo info;
    info.x = event->x();
    info.y = event->y();
    if (event->button() == Qt::LeftButton)
        info.button = Left;
    else if (event->button() == Qt::MidButton)
        info.button = Mid;
    else if (event->button() == Qt::RightButton)
        info.button = Right;
    info.state = Click;
    emit mouseEvent(info);
}

void ImageRemoteView::mouseReleaseEvent(QMouseEvent *event)
{
    MouseInfo info;
    info.x = event->x();
    info.y = event->y();
    if (event->button() == Qt::LeftButton)
        info.button = Left;
    else if (event->button() == Qt::MidButton)
        info.button = Mid;
    else if (event->button() == Qt::RightButton)
        info.button = Right;
    info.state = Release;
    emit mouseEvent(info);
}

void ImageRemoteView::mouseDoubleClickEvent(QMouseEvent *event)
{
    MouseInfo info;
    info.x = event->x();
    info.y = event->y();
    if (event->button() == Qt::LeftButton)
        info.button = Left;
    else if (event->button() == Qt::MidButton)
        info.button = Mid;
    else if (event->button() == Qt::RightButton)
        info.button = Right;
    info.state = DoubleClick;
    emit mouseEvent(info);
}

void ImageRemoteView::keyPressEvent(QKeyEvent *event)
{
    if (event->isAutoRepeat() && (event->type() == QEvent::KeyRelease))
        return; //bug from libvnc
    KeyInfo info;
    info.nativeVirtualKey = event->nativeVirtualKey();
    info.nativeVirtualCode = event->nativeScanCode();
    info.press = (event->type() == QEvent::KeyPress);
    event->accept();
    emit keyEvent(info);
}

void ImageRemoteView::keyReleaseEvent(QKeyEvent *event)
{
    if (event->isAutoRepeat() && (event->type() == QEvent::KeyRelease))
        return; //bug from libvnc
    KeyInfo info;
    info.nativeVirtualKey = event->nativeVirtualKey();
    info.nativeVirtualCode = event->nativeScanCode();
    info.press = (event->type() == QEvent::KeyPress);
    event->accept();
    emit keyEvent(info);
}

void ImageRemoteView::wheelEvent(QWheelEvent *event)
{
    MouseInfo info;
    info.button = None;
    info.x = event->x();
    info.y = event->y();
    if (event->delta() < 0)
        info.state = WheelDown;
    else
        info.state = WheelUp;
    emit mouseEvent(info);
}

void ImageRemoteView::imageReceive(QImage image, int x, int y, int w, int h)
{
    QPainter painter(&_image);
    painter.drawImage(QRect(x, y, w, h), image);
    repaint(x, y, w, h);
}

void ImageRemoteView::cursorReceive(/*QCursor cursor*/QImage image, int hotX, int hotY)
{
    if (image.isNull()) {
        QCursor cursor(Qt::BlankCursor);
        setCursor(cursor);
    }
    else {
        QCursor cursor(QPixmap::fromImage(image), hotX, hotY);
        setCursor(cursor);
    }
}

void ImageRemoteView::clearImage()
{
    _image.fill(Qt::black);
    update();
}

void ImageRemoteView::connectionEnd()
{
    close();
}

void ImageRemoteView::clipboardDataEventSlot(QString text)
{
    QApplication::clipboard()->setText(text);
}

void ImageRemoteView::clipboardSelectionEventSlot(QString text)
{
    QApplication::clipboard()->setText(text, QClipboard::Selection);
}

void ImageRemoteView::clipboardDataClearedSlot()
{
    QApplication::clipboard()->clear();
}

void ImageRemoteView::clipboardSelectionClearedSlot()
{
    QApplication::clipboard()->clear(QClipboard::Selection);
}

void ImageRemoteView::sizeChange(QSize size)
{
    _image = QPixmap(size);
    resize(size);
}

bool ImageRemoteView::event(QEvent *event)
{
    if (event->type() == QEvent::KeyPress) {
        QKeyEvent *ke = static_cast<QKeyEvent *>(event);
        if (ke->key() == Qt::Key_Tab) {
            keyPressEvent(ke);
            return true;
        }
    }
    else if (event->type() == QEvent::KeyRelease) {
        QKeyEvent *ke = static_cast<QKeyEvent *>(event);
        if (ke->key() == Qt::Key_Tab) {
            keyReleaseEvent(ke);
            return true;
        }
    }
    return QWidget::event(event);
}


bool ImageRemoteView::eventFilter(QObject *object, QEvent *event)
{
    if (event->type() == QEvent::ShortcutOverride) {
        event->accept();
        return true;
    }
    return QObject::eventFilter(object, event);
}

void ImageRemoteView::focusInEvent(QFocusEvent *event)
{
    Q_UNUSED(event)
    Hnv::HnvMainWindow::instance()->installEventFilter(this);
}

void ImageRemoteView::focusOutEvent(QFocusEvent *event)
{
    Q_UNUSED(event)
    emit focusOutEvent();
    Hnv::HnvMainWindow::instance()->removeEventFilter(this);
}

void ImageRemoteView::enterEvent(QEvent *event)
{
    Q_UNUSED(event)
    grabKeyboard();
}

void ImageRemoteView::leaveEvent(QEvent *event)
{
    Q_UNUSED(event)
    releaseKeyboard();
}

void ImageRemoteView::_clipboardDataChangedSlot()
{
    emit clipboardDataEvent(QApplication::clipboard()->text(QClipboard::Clipboard));
}

void ImageRemoteView::_clipboardSelectionChangedSlot()
{
    emit clipboardSelectionEvent(QApplication::clipboard()->text(QClipboard::Selection));
}

} // namespace RemoteAccess
