/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "topologycatalogmodel.h"
#include "ressources.h"
#include "topologyadministrator.h"

// hynesim hnsclient headers
#include <libhnsclient/topology.h>
#include <libhnsclient/entry.h>
#include <libhnsclient/topologyentrycatalog.h>
#include <libhnsclient/topologymanager.h>

// Qt headers
#include <QBuffer>
#include <QMimeData>
#include <QIcon>
#include <QPainter>
#include <QPixmapCache>
#include <QDebug>

namespace Hnv {

/*******************************************************************************
* TopologyItem
*******************************************************************************/
TopologyItem::TopologyItem(HnsClient::TopologyEntry *entry, HnsClient::Topology *topology, QObject *parent)
    : EntryItem(entry, parent)
    , _topology(0)
{
    setTopology(topology);
}

TopologyItem::~TopologyItem()
{
}

void TopologyItem::setTopology(HnsClient::Topology *topology)
{
    disconnect(this, SLOT(addConnectedUser(qlonglong)));
    disconnect(this, SLOT(removeConnectedUser(qlonglong)));
    disconnect(this, SLOT(topologyChangedSlot()));
    _topology = topology;
    if (_topology) {
        connect(_topology, SIGNAL(connectedUserAdded(qlonglong)), this, SLOT(addConnectedUser(qlonglong)));
        connect(_topology, SIGNAL(connectedUserRemoved(qlonglong)), this, SLOT(removeConnectedUser(qlonglong)));
        connect(_topology, SIGNAL(topologyChanged()), this, SLOT(topologyChangedSlot()));
    }
    HnsClient::TopologyEntry *topologyEntry = qobject_cast<HnsClient::TopologyEntry*>(_entry);
    if (topologyEntry)
        emit updated(topologyEntry->getUUID());
}

QVariant TopologyItem::data(int column, int role) const
{
    HnsClient::TopologyEntry *entry = topologyEntry();
    if (entry) {
        switch(role) {
        case TopologyItem::StatusRole: {
            return entry->getStatus();
        }
        case TopologyItem::IsNewRole: {
            return isNew();
        }
        case TopologyItem::UUIDRole: {
            return entry->getUUID();
        }
        case TopologyItem::IsLoadedRole: {
            return isLoaded();
        }
        case TopologyItem::IsModifiedRole: {
            return isModified();
        }
        case TopologyItem::ConnectedUsersRole: {
            return connectedUsersCount();
        }
        case TopologyItem::IsModelRole: {
            return entry->isModel();
        }
        case TopologyItem::ScreenShotRole: {
            QPixmap pixmap = entry->getScreenshot();
            if(pixmap.isNull())
                return data(column,Qt::ToolTipRole);
            QByteArray ba;
            QBuffer buffer(&ba);
            buffer.open(QIODevice::WriteOnly);
            pixmap.toImage().save(&buffer, "PNG");
            return QString("<img src=\"data:image/png;base64,%1\">").arg(QString(buffer.data().toBase64()));
        }
        case Qt::ToolTipRole: {
            if (entry->getStatus() == ::Hns::TOPOLOGY_CONTAINS_ENTITY_WITH_UUID_CONFLICT) {
                QString toolTipParent = EntryItem::getToolTip();
                return toolTipParent.append(tr("\n <b>Problem:</b> Topology contains entity(ies) with UUID conflicts"));
            } else if (entry->getStatus() == ::Hns::TOPOLOGY_UUID_CONFLICT) {
                QString toolTipParent = EntryItem::getToolTip();
                return toolTipParent.append(tr("\n <b>Problem:</b> Topology has an UUID conflict"));
            } else if (entry->getStatus() == ::Hns::TOPOLOGY_LOADING_ERROR) {
                QString toolTipParent = EntryItem::getToolTip();
                return toolTipParent.append(tr("\n <b>Problem:</b> Topology has an error to load"));
            } else if (entry->getStatus() == ::Hns::TOPOLOGY_BUSY_EXPORTING) {
                QString toolTipParent = EntryItem::getToolTip();
                return toolTipParent.append("\n\n <b>Busy</b>");
            } else {
                return EntryItem::getToolTip();
            }
        }
        case Qt::DecorationRole: {
            if (column == 0) {
                switch (entry->getStatus()) {
                case Hns::TOPOLOGY_CONTAINS_ENTITY_WITH_UUID_CONFLICT:
                case Hns::TOPOLOGY_UUID_CONFLICT:
                case Hns::TOPOLOGY_LOADING_ERROR:
                    return QIcon(QPixmap(Ressources::themedPath(Constants::TOPOLOGY_WITH_PROBLEM)));
                    break;
                case Hns::TOPOLOGY_BUSY_EXPORTING: {
                    QPixmap pic = _topologyPreview();
                    _drawPreviewStatusIcon(pic, Constants::ICON_PERM_MISSING);
                    _drawPreviewBorder(pic);
                    QPixmapCache::clear();
                    return QIcon(pic);
                    break;
                }
                case Hns::TOPOLOGY_STATUS_OK: {
                    QPixmap pic = _topologyPreview();
                    if (connectedUsersCount() != 0)
                        _drawPreviewStatusIcon(pic, Constants::ICON_GROUP);
                    else if (entry->isPersistent() && isLoaded())
                        _drawPreviewStatusIcon(pic, Constants::ICON_GENERATE_MAC_ADRESS);
                    _drawPreviewBorder(pic);
                    QPixmapCache::clear();
                    return QIcon(pic);
                    break;
                }
                }
            }
        }
        case Qt::DisplayRole: {
            if (column == COLUMN_SHORTNAME && isModified()) {
                return QString("%1 *").arg(entry->getShortName());
            }
            else if(column == COLUMN_MODEL && entry->isModel()){
                return QString("\u2713");
            }
        }
        }
    }
    return EntryItem::data(column, role);
}

HnsClient::TopologyEntry *TopologyItem::topologyEntry() const
{
    return dynamic_cast<HnsClient::TopologyEntry *>(catalogItem());
}

HnsClient::Topology *TopologyItem::topology()
{
    return _topology;
}

Qt::ItemFlags TopologyItem::flags(QModelIndex index) const
{
    Qt::ItemFlags flagsTopologyItem = EntryItem::flags(index.column());
    if (data(index.column(), TopologyItem::IsModelRole).toBool()) {
        flagsTopologyItem |= Qt::ItemIsDragEnabled | Qt::ItemIsDropEnabled;
    }
    QVariant dataTopologyItem = data(index.column(), TopologyItem::StatusRole);
    if (dataTopologyItem.isValid() && dataTopologyItem.toInt()) {
        int dataInt = dataTopologyItem.toInt();
        if (dataInt != ::Hns::TOPOLOGY_STATUS_OK) {
            if (index.column() >= 0) {
                flagsTopologyItem &= ~Qt::ItemIsEditable;
            }
        }
    }
    return flagsTopologyItem;
}

bool TopologyItem::isNew() const
{
    return topologyEntry()->isNew();
}

bool TopologyItem::isLoaded() const
{
    if (_topology)
        return true;
    return false;
}

bool TopologyItem::isModified() const
{
    if (_topology)
        return _topology->isModified();
    return false;
}

QList<qlonglong> TopologyItem::connectedUsers() const
{
    QList<qlonglong> result;
    if (isLoaded())
        result = _topology->connectedUsers();
    return result;
}

int TopologyItem::connectedUsersCount() const
{
    return connectedUsers().count();
}

void TopologyItem::addConnectedUser(qlonglong id)
{
    Q_UNUSED(id)
    HnsClient::TopologyEntry *topologyEntry = qobject_cast<HnsClient::TopologyEntry*>(_entry);
    if (topologyEntry)
        emit updated(topologyEntry->getUUID());
}

void TopologyItem::removeConnectedUser(qlonglong id)
{
    Q_UNUSED(id)
    HnsClient::TopologyEntry *topologyEntry = qobject_cast<HnsClient::TopologyEntry*>(_entry);
    if (topologyEntry)
        emit updated(topologyEntry->getUUID());
}

void TopologyItem::topologyChangedSlot()
{
    HnsClient::TopologyEntry *topologyEntry = qobject_cast<HnsClient::TopologyEntry*>(_entry);
    if (topologyEntry) {
        emit updated(topologyEntry->getUUID());
    }
}

QPixmap TopologyItem::_topologyPreview() const
{
    QPixmap result;
    HnsClient::TopologyEntry *entry = topologyEntry();
    if (entry)
        result = QPixmap(entry->getScreenshot());

    if(result.isNull())
        result = QPixmap(Ressources::themedPath(Constants::DEFAULT_PREVIEW));

    if (entry && entry->isModel())
        result = HnsClient::TopologyEntry::getScreenshotModel(result);

    return result;
}

void TopologyItem::_drawPreviewBorder(QPixmap &preview) const
{
    QPainter painter;
    painter.begin(&preview);
    painter.setPen(Qt::black);
    int penW = painter.pen().width();
    if (penW == 0 && painter.pen().isCosmetic())
        penW = 1;
    QRect borderRect = preview.rect();
    borderRect.setWidth(preview.rect().width() - penW);
    borderRect.setHeight(preview.rect().height() - penW);
    painter.drawRect(borderRect);
    painter.end();
}

void TopologyItem::_drawPreviewStatusIcon(QPixmap &preview, const QString &iconPath) const
{
    QPainter painter;
    painter.begin(&preview);
    painter.drawPixmap(preview.width() - Constants::PREVIEW_STATUS_ICON_SIZE,   // X coordinate in preview coordinate system
                       0,                                                       // Y coordinate in preview coordinate system
                       Constants::PREVIEW_STATUS_ICON_SIZE,                     // icon width
                       Constants::PREVIEW_STATUS_ICON_SIZE,                     // icon height
                       QPixmap(Ressources::themedPath(iconPath)));
    painter.end();
}

/*******************************************************************************
* TopologyCatalogModel
*******************************************************************************/
TopologyCatalogModel::TopologyCatalogModel(QObject *parent)
    : QAbstractTableModel(parent)
    , _administrator(0)
{
}

TopologyCatalogModel::~TopologyCatalogModel()
{
    _resetData();
}

void TopologyCatalogModel::initModel(TopologyAdministrator *administrator)
{
    _administrator = administrator;
    _populateCatalog();
    connect(_administrator->catalog(), SIGNAL(topologyAdded(HnsClient::TopologyEntry*)), this, SLOT(_topologyEntryAddedSlot(HnsClient::TopologyEntry*)));
    connect(_administrator->catalog(), SIGNAL(topologyUpdated(HnsClient::TopologyEntry*)), this, SLOT(_topologyEntryUpdatedSlot(HnsClient::TopologyEntry*)));
    connect(_administrator->catalog(), SIGNAL(topologyRemoved(qlonglong)), this, SLOT(_topologyEntryRemovedSlot(qlonglong)));
    connect(_administrator->manager(), SIGNAL(topologyLoaded(QString)), this, SLOT(_topologyLoadedSlot(QString)));
    connect(_administrator->manager(), SIGNAL(topologyUnloaded(QString,bool)), this, SLOT(_topologyUnloadedSlot(QString)));
    connect(_administrator->manager(), SIGNAL(topologyAdded(HnsClient::Topology*)), this, SLOT(_topologyAddedSlot(HnsClient::Topology*)));
    connect(_administrator->manager(), SIGNAL(topologySavedAs(QString,QString)), this, SLOT(_topologySavedAsSlot(QString, QString)));
}

void TopologyCatalogModel::resetModel()
{
    _resetData();
    _administrator = 0;
}

int TopologyCatalogModel::rowCount(const QModelIndex &parent) const
{
    if (parent.isValid())
        return 0;
    return _topologyList.count();
}

int TopologyCatalogModel::columnCount(const QModelIndex &parent) const
{
    Q_UNUSED(parent)
    return EntryItem::SIZE_COLUMN;
}

QVariant TopologyCatalogModel::data(const QModelIndex &index, int role) const
{
    if (index.isValid()) {
        int row = index.row();
        if (row < _topologyList.size()) {
            return _topologyList[row]->data(index.column(), role);
        }
    }
    return QVariant();
}

bool TopologyCatalogModel::setData(const QModelIndex &index, const QVariant &value, int role)
{
    if (index.isValid()) {
        int row = index.row();
        if (row < _topologyList.size()) {
            QString uuid = index.data(TopologyItem::UUIDRole).toString();
            HnsClient::TopologyEntry *updatedEntry = _administrator->duplicateEntryByUUID(uuid, this);
            if (updatedEntry) {
                TopologyItem *updatedItem = new TopologyItem(updatedEntry, _administrator->topologyByUUID(uuid), this);
                connect(updatedItem, SIGNAL(updated(QString)), this, SLOT(_topologyItemUpdatedSlot(QString)));
                if (updatedItem->setData(index.column(), value, role)) {
                    _administrator->updateEntry(updatedEntry);
                    delete updatedItem;
                    return true;
                }
                delete updatedItem;
                delete updatedEntry;
            }
        }
    }
    return QAbstractTableModel::setData(index,value,role);
}

QMimeData *TopologyCatalogModel::mimeData(const QModelIndexList &indexes) const
{
    QMimeData *mimeData = new QMimeData();
    QByteArray encodedData;
    QDataStream stream(&encodedData, QIODevice::WriteOnly);
    foreach(QModelIndex index, indexes) {
        if (!index.isValid())
            continue;
        stream << index.data(EntryItem::IDRole).toLongLong();
    }
    mimeData->setData("hyneview/topology-entry", encodedData);
    return mimeData;
}

QVariant TopologyCatalogModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if (role == Qt::DisplayRole && orientation == Qt::Horizontal) {
        switch(section) {
        case EntryItem::COLUMN_SHORTNAME:
            return tr("Short Name");
        case EntryItem::COLUMN_LONGNAME:
            return tr("Name");
        case EntryItem::COLUMN_DESCRIPTION:
            return tr("Description");
        case EntryItem::COLUMN_TAGLIST:
            return tr("Tags");
        case EntryItem::COLUMN_MODEL:
            return tr("Model");
        }
    }
    return QAbstractItemModel::headerData(section, orientation, role);
}

QModelIndex TopologyCatalogModel::index(int row, int column, const QModelIndex &parent) const
{
    if (!hasIndex(row, column, parent))
        return QModelIndex();
    if (row < _topologyList.size()) {
        return createIndex(row, column, _topologyList.at(row));
    }
    return QModelIndex();
}

Qt::ItemFlags TopologyCatalogModel::flags(const QModelIndex &index) const
{
    TopologyItem *item = static_cast<TopologyItem *>(index.internalPointer());
    if (item)
        return item->flags(index);
    else
        return Qt::NoItemFlags;
}

QList<TopologyItem *> TopologyCatalogModel::topologyList()
{
    return _topologyList;
}

TopologyItem *TopologyCatalogModel::itemById(qlonglong entryId)
{
    return _topologyHash.value(entryId, 0);
}

/*******************************************************************************
* TopologyCatalogModel - Private slots
*/
void TopologyCatalogModel::_topologyEntryAddedSlot(HnsClient::TopologyEntry *entry)
{
    _createTopologyItem(entry);
}

void TopologyCatalogModel::_topologyEntryUpdatedSlot(HnsClient::TopologyEntry *entry)
{
    if (!entry)
        return;
    qlonglong entryId = entry->getID();
    if (!_topologyHash.contains(entryId)) {
        qDebug() << tr("Topology not in cache, can not update.");
        return;
    }
    TopologyItem *item = _topologyHash.value(entryId);
    _itemUpdated(item);
}

void TopologyCatalogModel::_topologyEntryRemovedSlot(qlonglong topologyEntryID)
{
    if (!_topologyHash.contains(topologyEntryID)) {
        qDebug() << tr("Topology not in cache, can not delete.");
        return;
    }
    TopologyItem *item = _topologyHash.take(topologyEntryID);
    int row = _topologyList.indexOf(item);
    if (row == -1) {
        qDebug() << tr("Topology item not in cache, can not delete.");
        return;
    }
    beginRemoveRows(QModelIndex(), row, row);
    _topologyList.removeAt(row);
    endRemoveRows();
    delete item;
}

void TopologyCatalogModel::_topologyLoadedSlot(QString topologyUUID)
{
    TopologyItem *item = _itemByUUID(topologyUUID);
    if (item) {
        item->setTopology(_administrator->manager()->topologyByUUID(topologyUUID));
        _itemUpdated(item);
    }
}

void TopologyCatalogModel::_topologyUnloadedSlot(QString topologyUUID)
{
    TopologyItem *item = _itemByUUID(topologyUUID);
    if (item) {
        item->setTopology(0);
        _itemUpdated(item);
    }
}

void TopologyCatalogModel::_topologyItemUpdatedSlot(QString uuid)
{
    _itemUpdated(_itemByUUID(uuid));
}

void TopologyCatalogModel::_topologyAddedSlot(HnsClient::Topology *topology)
{
    TopologyItem *item = _topologyHash.value(topology->topologyEntry()->getID());
    if (!item)
        return;
    item->setTopology(topology);
    _itemUpdated(item);
}

void TopologyCatalogModel::_topologySavedAsSlot(QString oldUuid, QString newUuid)
{
    _topologyUnloadedSlot(oldUuid);
    _topologyLoadedSlot(newUuid);
}

/*******************************************************************************
* TopologyCatalogModel - Private
*/
void TopologyCatalogModel::_resetData()
{
    beginResetModel();
    qDeleteAll(_topologyList);
    _topologyList.clear();
    _topologyHash.clear();
    endResetModel();
}

void TopologyCatalogModel::_populateCatalog()
{
    foreach(HnsClient::TopologyEntry *entry, _administrator->getCatalogEntryList())
        _createTopologyItem(entry);
}

void TopologyCatalogModel::_createTopologyItem(HnsClient::TopologyEntry *entry)
{
    if (!entry)
        return;
    qlonglong entryId = entry->getID();
    if (_topologyHash.contains(entryId)) {
        qDebug() << tr("Topology already in cache.");
        return;
    }
    int pos = _topologyList.count();
    beginInsertRows(QModelIndex(), pos, pos);
    TopologyItem *item = new TopologyItem(entry, _administrator->topologyByUUID(entry->getUUID()), this);
    connect(item, SIGNAL(updated(QString)), this, SLOT(_topologyItemUpdatedSlot(QString)));
    _topologyList.append(item);
    _topologyHash.insert(entryId, item);
    endInsertRows();
}

TopologyItem *TopologyCatalogModel::_itemByUUID(QString uuid)
{
    TopologyItem *result = 0;
    foreach(TopologyItem *item, _topologyHash.values()) {
        if (item->topologyEntry()->getUUID() == uuid) {
            result = item;
            break;
        }
    }
    return result;
}

void TopologyCatalogModel::_itemUpdated(TopologyItem *item)
{
    if (item) {
        int row = _topologyList.indexOf(item);
        QModelIndex firstIdx = index(row, 0);
        QModelIndex lastIdx = index(row, item->columnCount() -1);
        emit dataChanged(firstIdx, lastIdx);
    }
}

} // namespace Hnv
