/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "flatproxymodel.h"

namespace Hnv {

/*******************************************************************************
* FlatProxyModel
*******************************************************************************/
FlatProxyModel::FlatProxyModel(QObject *parent)
    : QAbstractProxyModel(parent)
{
    _modelRole = Qt::DisplayRole;
}

FlatProxyModel::FlatProxyModel(QAbstractItemModel *model, QObject *parent)
    : QAbstractProxyModel(parent)
{
    _modelRole = Qt::DisplayRole;
   setSourceModel(model);
}

int FlatProxyModel::columnCount(const QModelIndex &) const
{
    return 1;
}

int FlatProxyModel::rowCount(const QModelIndex &) const
{
    return _sourceIndexList.size();
}

QVariant FlatProxyModel::data(const QModelIndex &index, int role) const
{
    if (sourceModel() == 0)
        return QVariant();
    if (role != _modelRole)
        return QVariant();
    QModelIndex sourceIndex = mapToSource(index);
    if (sourceIndex.isValid())
        return sourceIndex.data(_modelRole);
    return QVariant();
}

QModelIndex FlatProxyModel::index(int row, int column, const QModelIndex &parent) const
{
    if (hasIndex(row, column, parent))
        return createIndex(row, column);
    return QModelIndex();
}

QModelIndex FlatProxyModel::parent(const QModelIndex &) const
{
    return QModelIndex();
}

QModelIndex FlatProxyModel::mapFromSource(const QModelIndex &sourceIndex) const
{
    int row = _sourceIndexList.indexOf(sourceIndex);
    if (row != -1)
        return index(row, 0);
    return QModelIndex();
}

QModelIndex FlatProxyModel::mapToSource(const QModelIndex &proxyIndex) const
{
    if (proxyIndex.isValid()) {
        int row = proxyIndex.row();
        if (row < _sourceIndexList.size())
            return _sourceIndexList.at(row);
    }
    return QModelIndex();
}

void FlatProxyModel::setSourceModel(QAbstractItemModel *model)
{
    if (sourceModel() == model)
        return;
    if (sourceModel() != 0) {
        sourceModel()->disconnect(this);
        _sourceIndexList.clear();
    }
    QAbstractProxyModel::setSourceModel(model);
    _buildModelMap();

    connect(sourceModel(), SIGNAL(dataChanged(QModelIndex,QModelIndex)), this, SLOT(sourceDataChanged(QModelIndex,QModelIndex)));
    connect(sourceModel(), SIGNAL(rowsInserted(QModelIndex,int, int)), this, SLOT(sourceRowsInserted(QModelIndex,int,int)));
    connect(sourceModel(), SIGNAL(rowsRemoved(QModelIndex, int, int)), this, SLOT(sourceRowsRemoved(QModelIndex,int,int)));
    connect(sourceModel(), SIGNAL(modelReset()), this, SLOT(sourceReseted()));
    connect(sourceModel(), SIGNAL(destroyed()), this, SLOT(sourceDestroyed()));
}

Qt::ItemFlags FlatProxyModel::flags(const QModelIndex &index) const
{
    QModelIndex sourceIndex = mapToSource(index);
    if (sourceIndex.isValid())
        return sourceIndex.flags();
    return Qt::NoItemFlags;
}

void FlatProxyModel::setRole(int role) {
    _modelRole = role;
}

/*******************************************************************************
* FlatProxyModel - Public slots
*/
void FlatProxyModel::sourceDataChanged(QModelIndex, QModelIndex)
{
    // TODO: Requires optimization (should only update data which have been changed)
    _buildModelMap();
}

void FlatProxyModel::sourceRowsInserted(const QModelIndex &parent, int start, int end)
{
    QModelIndex previousIndex = parent;
    int position = 0;
    if (start > 0)
        previousIndex = sourceModel()->index(start -1, 0, parent);
    if (previousIndex.isValid()) {
        int previousIndexPosition = _sourceIndexList.indexOf(previousIndex);
        if (previousIndexPosition != -1)
            position = previousIndexPosition + 1;
    }
    for(int i = start; i <= end; i++) {
        QModelIndex newRow = sourceModel()->index(i, 0, parent);
        beginInsertRows(QModelIndex(), position, position);
        _sourceIndexList.insert(position, newRow);
        endInsertRows();
        position++;
    }
}

void FlatProxyModel::sourceRowsRemoved(const QModelIndex &parent, int start, int end)
{
    QModelIndex previousIndex = parent;
    int position = 0;
    if (start > 0)
        previousIndex = sourceModel()->index(start -1, 0, parent);
    if (previousIndex.isValid()) {
        int previousIndexPosition = _sourceIndexList.indexOf(previousIndex);
        if (previousIndexPosition != -1)
            position = previousIndexPosition +1;
    }
    position = position + end - start;
    for(int i = end; i >= start; i--) {
        beginRemoveRows(QModelIndex(), position, position);
        _sourceIndexList.removeAt(position);
        endRemoveRows();
        position--;
    }
}

void FlatProxyModel::sourceReseted()
{
    _buildModelMap();
}

void FlatProxyModel::sourceDestroyed()
{
    sourceModel()->disconnect(this);
}

/*******************************************************************************
* FlatProxyModel - Private
*/
void FlatProxyModel::_buildModelMap()
{
    beginResetModel();
    int position = 0;
    _sourceIndexList.clear();
    _buildModelMapBranch(position, QModelIndex());
    endResetModel();
}

void FlatProxyModel::_buildModelMapBranch(int &position, const QModelIndex &parent)
{
    int childcount = sourceModel()->rowCount(parent);
    for (int i=0; i < childcount; i++) {
        QModelIndex childIndex = sourceModel()->index(i, 0, parent);
        _sourceIndexList.append(childIndex);
        _buildModelMapBranch(position, childIndex);
    }
}

} // namespace Hnv
