/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "multicolumnstoflatproxymodel.h"

// Qt headers
#include <QStringList>

namespace Hnv {

/*******************************************************************************
* MultiColumnsToFlatProxyModel
*******************************************************************************/
MultiColumnsToFlatProxyModel::MultiColumnsToFlatProxyModel(QObject *parent)
    : QAbstractProxyModel(parent)
{
    _modelRole = Qt::DisplayRole;
}

MultiColumnsToFlatProxyModel::MultiColumnsToFlatProxyModel(QAbstractItemModel *model, QHash<int, QString> columns, QObject *parent)
    : QAbstractProxyModel(parent)
{
    _modelRole = Qt::DisplayRole;
    _sourceColumns = columns;
    setSourceModel(model);
}

int MultiColumnsToFlatProxyModel::columnCount(const QModelIndex &) const
{
    return 1;
}

int MultiColumnsToFlatProxyModel::rowCount(const QModelIndex &) const
{
    return _sourceIndexList.size();
}

QVariant MultiColumnsToFlatProxyModel::data(const QModelIndex &index, int role) const
{
    if (sourceModel() == 0)
        return QVariant();
    if (role != _modelRole)
        return QVariant();

    int row = index.row();
    if (row < _sourceIndexList.size()) {
        ProxyMapper mapper = _sourceIndexList.at(row);
        QModelIndex sourceIndex = mapper.sourceIndex;
        int wordPos = mapper.wordIndex;
        if (wordPos == -1)
            return sourceIndex.data(_modelRole).toString();
        else {
            QString separator = _sourceColumns.value(sourceIndex.column());
            QStringList splittedData = sourceIndex.data(_modelRole).toString().split(separator, QString::SkipEmptyParts);
            if (wordPos <= splittedData.count())
                return splittedData.at(wordPos).trimmed();
        }
    }
    return QVariant();
}

QModelIndex MultiColumnsToFlatProxyModel::index(int row, int column, const QModelIndex &parent) const
{
    if (hasIndex(row, column, parent))
        return createIndex(row, column);
    return QModelIndex();
}

QModelIndex MultiColumnsToFlatProxyModel::parent(const QModelIndex &) const
{
    return QModelIndex();
}

QModelIndex MultiColumnsToFlatProxyModel::mapFromSource(const QModelIndex &sourceIndex) const
{
    if (_sourceIndexHash.contains(sourceIndex)) {
        int row = _sourceIndexHash.value(sourceIndex);
        if (row < _sourceIndexList.size()) {
            return index(row, 0);
        }
    }
    return QModelIndex();
}

QModelIndex MultiColumnsToFlatProxyModel::mapToSource(const QModelIndex &proxyIndex) const
{
    if (proxyIndex.isValid()) {
        int row = proxyIndex.row();
        if (row < _sourceIndexList.size())
            return _sourceIndexList.at(row).sourceIndex;
    }
    return QModelIndex();
}

void MultiColumnsToFlatProxyModel::setSourceModel(QAbstractItemModel *model)
{
    if (sourceModel() == model)
        return;
    if (sourceModel() != 0) {
        sourceModel()->disconnect(this);
        _sourceIndexList.clear();
    }
    QAbstractProxyModel::setSourceModel(model);
    _buildModelMap();

    connect(sourceModel(), SIGNAL(dataChanged(QModelIndex,QModelIndex)), this, SLOT(sourceDataChanged(QModelIndex,QModelIndex)));
    connect(sourceModel(), SIGNAL(rowsInserted(QModelIndex,int, int)), this, SLOT(sourceRowsInserted(QModelIndex,int,int)));
    connect(sourceModel(), SIGNAL(rowsRemoved(QModelIndex, int, int)), this, SLOT(sourceRowsRemoved(QModelIndex,int,int)));
    connect(sourceModel(), SIGNAL(modelReset()), this, SLOT(sourceReseted()));
}

Qt::ItemFlags MultiColumnsToFlatProxyModel::flags(const QModelIndex &index) const
{
    QModelIndex sourceIndex = mapToSource(index);
    if (sourceIndex.isValid())
        return sourceIndex.flags();
    return Qt::NoItemFlags;
}

void MultiColumnsToFlatProxyModel::setRole(int role) {
    _modelRole = role;
}

/*******************************************************************************
* MultiColumnsToFlatProxyModel - Public slots
*/
void MultiColumnsToFlatProxyModel::sourceDataChanged(QModelIndex, QModelIndex)
{
    // TODO: Requires optimization (should only update data which have been changed)
    _buildModelMap();
}

void MultiColumnsToFlatProxyModel::sourceRowsInserted(const QModelIndex &parent, int start, int end)
{
    Q_UNUSED(parent);
    Q_UNUSED(start);
    Q_UNUSED(end);
    // TODO: Requires optimization (should only update data which have been changed)
    _buildModelMap();
    //    QModelIndex previousIndex = parent;
    //    int position = 0;
    //    if (start > 0)
    //        previousIndex = sourceModel()->index(start -1, 0, parent);
    //    if (previousIndex.isValid()) {
    //        int previousIndexPosition = _sourceIndexList.indexOf(previousIndex);
    //        if (previousIndexPosition != -1)
    //            position = previousIndexPosition + 1;
    //    }
    //    for(int i = start; i <= end; i++) {
    //        QModelIndex newRow = sourceModel()->index(i, 0, parent);
    //        beginInsertRows(QModelIndex(), position, position);
    //        _sourceIndexList.insert(position, newRow);
    //        endInsertRows();
    //        position++;
    //    }
}

void MultiColumnsToFlatProxyModel::sourceRowsRemoved(const QModelIndex &parent, int start, int end)
{
    Q_UNUSED(parent);
    Q_UNUSED(start);
    Q_UNUSED(end);
    // TODO: Requires optimization (should only update data which have been changed)
    _buildModelMap();
    //    QModelIndex previousIndex = parent;
    //    int position = 0;
    //    if (start > 0)
    //        previousIndex = sourceModel()->index(start -1, 0, parent);
    //    if (previousIndex.isValid()) {
    //        int previousIndexPosition = _sourceIndexList.indexOf(previousIndex);
    //        if (previousIndexPosition != -1)
    //            position = previousIndexPosition +1;
    //    }
    //    position = position + end - start;
    //    for(int i = end; i >= start; i--) {
    //        beginRemoveRows(QModelIndex(), position, position);
    //        _sourceIndexList.removeAt(position);
    //        endRemoveRows();
    //        position--;
    //    }
}

void MultiColumnsToFlatProxyModel::sourceReseted()
{
    _buildModelMap();
}

/*******************************************************************************
* MultiColumnsToFlatProxyModel - Private
*/
void MultiColumnsToFlatProxyModel::_buildModelMap()
{
    beginResetModel();
    int position = 0;
    _sourceIndexList.clear();
    _buildModelMapBranch(position, QModelIndex());
    endResetModel();
}

void MultiColumnsToFlatProxyModel::_buildModelMapBranch(int &position, const QModelIndex &parent)
{
    int childcount = sourceModel()->rowCount(parent);
    for (int i=0; i < childcount; i++) {
        QModelIndex branchRoot = sourceModel()->index(i, 0, parent);
        QMutableHashIterator<int, QString> iterator(_sourceColumns);
        while(iterator.hasNext()) {
            iterator.next();
            int column = iterator.key();
            QString separator = iterator.value();
            QModelIndex targetIdx = sourceModel()->index(i, column, parent);
            if (targetIdx.isValid()) {
                _sourceIndexHash.insert(targetIdx, position);
                if (separator == "") {
                    ProxyMapper mapper;
                    mapper.sourceIndex = targetIdx;
                    mapper.wordIndex = -1;
                    _sourceIndexList.append(mapper);
                    position++;
                } else {
                    QStringList splittedData = targetIdx.data(_modelRole).toString().split(separator, QString::SkipEmptyParts);
                    int splittedCount = splittedData.count();
                    if (splittedCount == 0) {
                        ProxyMapper mapper;
                        mapper.sourceIndex = targetIdx;
                        mapper.wordIndex = -1;
                        _sourceIndexList.append(mapper);
                        position++;
                    } else {
                        for(int j=0; j < splittedCount; j++) {
                            ProxyMapper mapper;
                            mapper.sourceIndex = targetIdx;
                            mapper.wordIndex = j;
                            _sourceIndexList.append(mapper);
                            position++;
                        }
                    }
                }
            }
        }
        _buildModelMapBranch(position, branchRoot);
    }
}

} // namespace Hnv
