/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "treemodel.h"
#include "treemodelitem.h"

namespace Hnv {

/*******************************************************************************
* TreeModel
*******************************************************************************/
TreeModel::TreeModel(QObject *parent)
    : QAbstractItemModel(parent)
    , _rootItem(0)
{
}

TreeModel::~TreeModel()
{
    if (_rootItem)
        delete _rootItem;
}

QModelIndex TreeModel::index(int row, int column, const QModelIndex &parent) const
{
    TreeModelItem * parentItem = itemFromIndex(parent);
    if (parentItem == 0 || row < 0 || column < 0 || row >= parentItem->childCount() || column >= parentItem->columnCount())
        return QModelIndex();
    TreeModelItem * childItem = parentItem->child(row);
    return createIndex(row, column, childItem);
}

QModelIndex TreeModel::parent(const QModelIndex &child) const
{
    TreeModelItem *childItem = itemFromIndex(child);
    if (childItem == 0)
        return QModelIndex();
    return indexFromNode(childItem->parentItem());
}

QModelIndex TreeModel::indexFromNode(TreeModelItem *item, int column) const
{
    if (item == 0 || item == _rootItem || column >= item->columnCount())
        return QModelIndex();
    return createIndex(item->row(), column, item);
}

TreeModelItem * TreeModel::itemFromIndex(const QModelIndex &index) const
{
    if (!index.isValid())
        return _rootItem;
    return static_cast<TreeModelItem *>(index.internalPointer());
}

int TreeModel::rowCount(const QModelIndex &parent) const
{
    if (parent.column() > 0)
        return 0;
    TreeModelItem * parentItem = itemFromIndex(parent);

    if (parentItem == 0)
        return 0;
    return parentItem->childCount();
}

int TreeModel::columnCount(const QModelIndex &parent) const
{
    TreeModelItem * parentItem = itemFromIndex(parent);
    if (parentItem == 0)
        return 0;
    return parentItem->columnCount();
}

Qt::ItemFlags TreeModel::flags(const QModelIndex &index) const
{
    TreeModelItem * item = itemFromIndex(index);
    if (item == 0)
        return Qt::NoItemFlags;
    return item->flags(index.column());
}

QVariant TreeModel::data(const QModelIndex &index, int role) const
{
    TreeModelItem * item = itemFromIndex(index);
    if (item == 0)
        return QVariant();
    return item->data(index.column(), role);
}

bool TreeModel::setData(const QModelIndex &index, const QVariant &value, int role)
{
    TreeModelItem * item = itemFromIndex(index);
    if (item == 0)
        return false;
    if (item->setData(index.column(), value, role)) {
        emit dataChanged(index, index);
        return true;
    }
    return false;
}

void TreeModel::addItem(TreeModelItem *parentItem, TreeModelItem *child)
{
    QModelIndex parentIdx = indexFromNode(parentItem);
    int pos = parentItem->childCount();
    beginInsertRows(parentIdx, pos, pos);
    parentItem->appendChild(child);
    endInsertRows();
}

TreeModelItem * TreeModel::removeItem(TreeModelItem *item)
{
    TreeModelItem *parentItem = item->parentItem();
    QModelIndex parentIdx = indexFromNode(parentItem);
    int row = item->row();
    beginRemoveRows(parentIdx, row, row);
    parentItem->removeChild(row);
    endRemoveRows();
    return item;
}

void TreeModel::_itemChangedSlot(TreeModelItem *item, int column)
{
    QModelIndex idx = indexFromNode(item, column);
    emit dataChanged(idx, idx);
}

bool TreeModel::_isIndexValid(const QModelIndex &index) const
{
    return (index.model() == this && index.row() >= 0 && index.column() >= 0);
}

void TreeModel::_initRootItem(int columns)
{
    if (_rootItem)
        delete _rootItem;
    _rootItem = new TreeModelItem(columns, this);
    connect(_rootItem, SIGNAL(itemChanged(TreeModelItem *,int)), this, SLOT(_itemChangedSlot(TreeModelItem*,int)));
}

} // namespace Hnv
