/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "treetotableproxymodel.h"

namespace Hnv {

/*******************************************************************************
* TreeToTableProxyModel - Private
*******************************************************************************/
TreeToTableProxyModel::TreeToTableProxyModel(QObject *parent) :
    QAbstractProxyModel(parent)
{
    _modelRole = Qt::DisplayRole;
}

TreeToTableProxyModel::TreeToTableProxyModel(QAbstractItemModel *model, QObject *parent) :
        QAbstractProxyModel(parent)
{
    _modelRole = Qt::DisplayRole;
   setSourceModel(model);
}

int TreeToTableProxyModel::columnCount(const QModelIndex &) const
{
    return sourceModel()->columnCount();
}

int TreeToTableProxyModel::rowCount(const QModelIndex &) const
{
    return _sourceIndexList.size();
}

QVariant TreeToTableProxyModel::data(const QModelIndex &index, int role) const
{
    if (sourceModel() == 0)
        return QVariant();
    if (role != _modelRole)
        return QVariant();
    int row = index.row();
    int col = index.column();
    if (row < _sourceIndexList.size() && col < _sourceIndexList.at(row).size())
        return _sourceIndexList.at(row).at(col).data(_modelRole);
    return QVariant();
}

QModelIndex TreeToTableProxyModel::index(int row, int column, const QModelIndex &parent) const
{
    if (hasIndex(row, column, parent))
        return createIndex(row, column);
    return QModelIndex();
}

QModelIndex TreeToTableProxyModel::parent(const QModelIndex &) const
{
    return QModelIndex();
}

QModelIndex TreeToTableProxyModel::mapFromSource(const QModelIndex &sourceIndex) const
{
    if (_sourceIndexHash.contains(sourceIndex)) {
        int row = _sourceIndexHash.value(sourceIndex);
        if (row < _sourceIndexList.size()) {
            int col = _sourceIndexList.at(row).indexOf(sourceIndex);
            if (col != -1)
                return index(row, col);
        }
    }
    return QModelIndex();
}

QModelIndex TreeToTableProxyModel::mapToSource(const QModelIndex &proxyIndex) const
{
    if (proxyIndex.isValid()) {
        int row = proxyIndex.row();
        int col = proxyIndex.column();
        if (row < _sourceIndexList.size() && col < _sourceIndexList.at(row).size())
            return _sourceIndexList.at(row).at(col);
    }
    return QModelIndex();
}

void TreeToTableProxyModel::setSourceModel(QAbstractItemModel *model)
{
    if (sourceModel() == model)
        return;
    if (sourceModel() != 0) {
        sourceModel()->disconnect(this);
        _sourceIndexList.clear();
    }
    QAbstractProxyModel::setSourceModel(model);
    _buildModelMap();

    connect(sourceModel(), SIGNAL(dataChanged(QModelIndex,QModelIndex)), this, SLOT(sourceDataChanged(QModelIndex,QModelIndex)));
    connect(sourceModel(), SIGNAL(rowsInserted(QModelIndex,int, int)), this, SLOT(sourceRowsInserted(QModelIndex,int,int)));
    connect(sourceModel(), SIGNAL(rowsRemoved(QModelIndex, int, int)), this, SLOT(sourceRowsRemoved(QModelIndex,int,int)));
    connect(sourceModel(), SIGNAL(modelReset()), this, SLOT(sourceReseted()));
    connect(sourceModel(), SIGNAL(destroyed()), this, SLOT(sourceDestroyed()));
}

Qt::ItemFlags TreeToTableProxyModel::flags(const QModelIndex &index) const
{
    QModelIndex sourceIndex = mapToSource(index);
    if (sourceIndex.isValid())
        return sourceIndex.flags();
    return Qt::NoItemFlags;
}

void TreeToTableProxyModel::setRole(int role) {
    _modelRole = role;
}

/*******************************************************************************
* TreeToTableProxyModel - Public slots
*/
void TreeToTableProxyModel::sourceDataChanged(QModelIndex, QModelIndex)
{
    // TODO: Requires optimization (should only update data which have been changed)
    _buildModelMap();
}

void TreeToTableProxyModel::sourceRowsInserted(const QModelIndex &parent, int start, int end)
{
    Q_UNUSED(parent);
    Q_UNUSED(start);
    Q_UNUSED(end);
    // TODO: Requires optimization (should only update data which have been changed)
    _buildModelMap();

//    QModelIndex previousIndex = parent;
//    int position = 0;
//    if (start > 0)
//        previousIndex = sourceModel()->index(start -1, 0, parent);
//    if (previousIndex.isValid()) {
//        if (_sourceIndexHash.contains(previousIndex)) {
//            position = _sourceIndexList.value(previousIndex) + 1;
//        }
//    }
//    for(int i = start; i <= end; i++) {
//        QModelIndex newRow = sourceModel()->index(i, 0, parent);
//        beginInsertRows(QModelIndex(), position, position);
//        _sourceIndexList.insert(position, newRow);
//        endInsertRows();
//        position++;
//    }
//    Recalculate saved position
}

void TreeToTableProxyModel::sourceRowsRemoved(const QModelIndex &parent, int start, int end)
{
    Q_UNUSED(parent);
    Q_UNUSED(start);
    Q_UNUSED(end);
    // TODO: Requires optimization (should only update data which have been changed)
    _buildModelMap();

//    QModelIndex previousIndex = parent;
//    int position = 0;
//    if (start > 0)
//        previousIndex = sourceModel()->index(start -1, 0, parent);
//    if (previousIndex.isValid()) {
//        if (_sourceIndexHash.contains(previousIndex)) {
//            position = _sourceIndexList.value(previousIndex) + 1;
//        }
//    }
//    position = position + end - start;
//    for(int i = end; i >= start; i--) {
//        beginRemoveRows(QModelIndex(), position, position);
//        _sourceIndexList.removeAt(position);
//        endRemoveRows();
//        position--;
//    }
//    Recalculate saved position
}

void TreeToTableProxyModel::sourceReseted()
{
    _buildModelMap();
}

void TreeToTableProxyModel::sourceDestroyed()
{
    sourceModel()->disconnect(this);
}

/*******************************************************************************
* TreeToTableProxyModel - Private
*/
void TreeToTableProxyModel::_buildModelMap()
{
    beginResetModel();
    int position = 0;
    _sourceIndexList.clear();
    _buildModelMapBranch(position, QModelIndex());
    endResetModel();
}

void TreeToTableProxyModel::_buildModelMapBranch(int &position, const QModelIndex &parent)
{
    int childcount = sourceModel()->rowCount(parent);
    for (int i=0; i < childcount; i++) {
        QList<QModelIndex> rowIndexes;
        int colCount = columnCount(parent);
        for(int j=0; j < colCount; j++) {
            QModelIndex childIndex = sourceModel()->index(i, j, parent);
            rowIndexes.append(childIndex);
            _sourceIndexHash.insert(childIndex, position);
        }
        _sourceIndexList.append(rowIndexes);
        position++;
        _buildModelMapBranch(position, rowIndexes.at(0));
    }
}

} // namespace Hnv
