/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "groupsmodel.h"

// hynesim hnsclient headers
#include <libhnsclient/utility.h>
#include <libhnsclient/usermanager.h>

// Qt headers
#include <QDateTime>
#include <QStringList>

namespace Hnv {

/*******************************************************************************
* GroupsModel
*******************************************************************************/
GroupsModel::GroupsModel(QObject *parent)
    : QAbstractTableModel(parent)
    , _manager(0)
{
    _manager = 0;
    _positionList.clear();
}

void GroupsModel::initModel(HnsClient::UserManager * manager)
{
    _manager = manager;
    connect(_manager, SIGNAL(groupAdded(qlonglong)), this, SLOT(_groupAdded(qlonglong)));
    connect(_manager, SIGNAL(groupChanged(qlonglong)), this, SLOT(_groupChanged(qlonglong)));
    connect(_manager, SIGNAL(groupRemoved(qlonglong)), this, SLOT(_groupDeleted(qlonglong)));
    _buildData();
}

void GroupsModel::resetModel()
{
    _manager = 0;
    beginResetModel();
    _positionList.clear();
    endResetModel();
}

int GroupsModel::rowCount(const QModelIndex &parent) const
{
    if (!parent.isValid())
        return _positionList.size();
    else
        return 0;
}
int GroupsModel::columnCount(const QModelIndex &parent) const
{
    if (!parent.isValid())
        return COLUMN_SIZE;
    else
        return 0;
}

QVariant GroupsModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid() || !_manager)
        return QVariant();

    if (role == Qt::TextAlignmentRole) {
        return int(Qt::AlignLeft | Qt::AlignVCenter);
    } else if (role == Qt::DisplayRole || role == Qt::EditRole || role == Qt::UserRole) {
        int row = index.row();
        int column = index.column();
        GroupItem item = _groupHash.value(_positionList.at(row));
        if (role == Qt::UserRole)
            return item.id;
        else if (column == COLUMN_ID)
            return QString::number(item.id);
        else if (column == COLUMN_NAME)
            return item.name;
        else if (column == COLUMN_CREATED)
            return QDateTime::fromMSecsSinceEpoch(item.createdOn).toString(Qt::SystemLocaleShortDate);
        else if (column == COLUMN_MODIFIED)
            return QDateTime::fromMSecsSinceEpoch(item.modifiedOn).toString(Qt::SystemLocaleShortDate);
        else if (column == COLUMN_GROUPS) {
            QStringList usersName = _manager->getUsernamesFromIdList(item.users);
            return usersName.join(", ");
        }
    }
    return QVariant();
}

QVariant GroupsModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    Q_UNUSED(orientation)
    if (role == Qt::DisplayRole) {
        if (section == COLUMN_ID)
            return tr("ID");
        else if (section == COLUMN_NAME)
            return tr("Name");
        else if (section == COLUMN_CREATED)
            return tr("Created");
        else if (section == COLUMN_MODIFIED)
            return tr("Modified");
        else if (section == COLUMN_GROUPS)
            return tr("Users");
    }
    return QVariant();
}

int GroupsModel::groupIdByIndex(int index)
{
    if (index < _positionList.size())
        return _positionList.at(index);
    return -1;
}

/*******************************************************************************
* GroupsModel - Private slots
*/
void GroupsModel::_buildData()
{
    beginResetModel();
    _positionList.clear();
    for(int i=0; i < _manager->groupsCount(); i++){
        Diamesh::GroupPtr group = _manager->getGroupByIndex(i);
        _positionList.append(group->id);
        _groupHash.insert(group->id, _createItem(group));
    }
    endResetModel();
}

void GroupsModel::_groupAdded(qlonglong groupID)
{
    int pos = _positionList.size();
    beginInsertRows(QModelIndex(), pos, pos);
    _positionList.append(groupID);
    _groupHash.insert(groupID, _createItem(_manager->getGroupById(groupID)));
    endInsertRows();
}

void GroupsModel::_groupChanged(qlonglong groupID)
{
    int row = _positionList.indexOf(groupID);
    if (row != -1) {
        QModelIndex firstIdx = index(row, 0);
        QModelIndex lastIdx = index(row, COLUMN_SIZE - 1);
        _groupHash.insert(groupID, _createItem(_manager->getGroupById(groupID)));
        emit dataChanged(firstIdx, lastIdx);
    }
}

void GroupsModel::_groupDeleted(qlonglong deletedGroupID)
{
    int row = _positionList.indexOf(deletedGroupID);
    if (row != -1) {
        beginRemoveRows(QModelIndex(), row, row);
        _positionList.removeAt(row);
        _groupHash.remove(deletedGroupID);
        endRemoveRows();
    }
}

GroupsModel::GroupItem GroupsModel::_createItem(Diamesh::GroupPtr groupPtr)
{
    GroupItem item;
    item.id = groupPtr->id;
    item.name = QString::fromStdString(groupPtr->name);
    item.createdOn = groupPtr->createdOn;
    item.modifiedOn = groupPtr->modifiedOn;
    item.users = HnsClient::Utility::qLongListFromLongList(groupPtr->userList);
    return item;
}

/*******************************************************************************
* GroupsProxyModel
*******************************************************************************/
GroupsProxyModel::GroupsProxyModel(QObject *parent)
    : QSortFilterProxyModel(parent)
{
}

void GroupsProxyModel::setAllowedGroupList(QList<qlonglong> allowedGroupList)
{
    _allowedGroupList = allowedGroupList;
}

bool GroupsProxyModel::filterAcceptsRow(int source_row, const QModelIndex &source_parent) const
{
    qlonglong groupID = sourceModel()->index(source_row, 0, source_parent).data(Qt::UserRole).toLongLong();
    if (_allowedGroupList.contains(groupID))
        return true;
    return false;
}

} // namespace Hnv
