/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "sessionsmodel.h"
#include "ressources.h"

// hynesim hnsclient headers
#include <libhnsclient/usermanager.h>

// Qt headers
#include <QDateTime>
#include <QIcon>

namespace Hnv {

SessionsModel::SessionsModel(QObject *parent)
    : QAbstractTableModel(parent)
    , _manager(0)
{
}

void SessionsModel::initModel(HnsClient::UserManager *manager)
{
    _manager = manager;
    connect(_manager, SIGNAL(sessionAdded(QString)), this, SLOT(_sessionAdded(QString)));
    connect(_manager, SIGNAL(sessionRemoved(QString)), this, SLOT(_sessionRemoved(QString)));
    _refreshData();
}

void SessionsModel::resetModel()
{
    _manager = 0;
    beginResetModel();
    _sessionIDList.clear();
    endResetModel();
}

int SessionsModel::rowCount(const QModelIndex &parent) const
{
    if (!parent.isValid())
        return _sessionIDList.count();
    else
        return 0;
}

int SessionsModel::columnCount(const QModelIndex &parent) const
{
    if (!parent.isValid())
        return 1;
    else
        return 0;
}

QVariant SessionsModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid() || index.row() >= _sessionIDList.count() || !_manager)
        return QVariant();
    QString sessionID = _sessionIDList.at(index.row());
    Diamesh::SessionInfo information = _manager->getSession(sessionID);
    if (information.userId == -1)
        return QVariant();
    Hns::UserPtr user = _manager->getUserById(information.userId);
    switch (index.column()) {
    case 0:
    {
        switch (role) {
        case Qt::DisplayRole:
        {
            return QString("%1@%2")
                    .arg(QString::fromStdString(user->name))
                    .arg(QString::fromStdString(information.ipAddress));
            break;
        }
        case Qt::ToolTipRole:
        {
            QString tooltip = tr("<b>Login:&nbsp;</b><i>%1</i>").arg(QString::fromStdString(user->name));
            QString firstName = QString::fromStdString(user->firstname);
            QString lastName = QString::fromStdString(user->lastname);
            QString name = "";
            if (!firstName.isEmpty() && !lastName.isEmpty())
                name = QString("%1&nbsp;%2").arg(firstName).arg(lastName);
            else if (!firstName.isEmpty())
                name = firstName;
            else if (!lastName.isEmpty())
                name = lastName;
            if (!name.isEmpty())
                tooltip += tr("<br/><b>Name:&nbsp;</b><i>%1</i>").arg(name);
//            if (user->isAdmin)
//                tooltip += tr("<br/>Administrator:&nbsp;Yes");
            tooltip += tr("<br/>Connected&nbsp;from&nbsp;%1<br/>since&nbsp;%2")
                    .arg(QString::fromStdString(information.ipAddress))
                    .arg(QDateTime::fromMSecsSinceEpoch(information.connectionDateTime).toString(Qt::SystemLocaleShortDate));
            return tooltip;
            break;
        }
        case Qt::DecorationRole:
        {
            QString iconName = Constants::ICON_PREFIX_USER;
            // TODO:TestRight ?
//            iconName = (user->isAdmin ? Constants::ICON_PREFIX_ADMIN : Constants::ICON_PREFIX_USER);
            iconName += Constants::ICON_SUFFIX_CONNECTED;
            return QIcon(Ressources::themedPath(iconName));
            break;
        }
        case Qt::UserRole :
        {
            return QString::fromStdString(information.ipAddress);
        }
        } // switch (role)
    } // case 0:
    } // switch (index.column())
    return QVariant();
}

QVariant SessionsModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    return QAbstractTableModel::headerData(section, orientation, role);
}

void SessionsModel::_sessionAdded(QString sessionID)
{
    int pos = _sessionIDList.size();
    beginInsertRows(QModelIndex(), pos, pos);
    _sessionIDList.append(sessionID);
    endInsertRows();
}

void SessionsModel::_sessionRemoved(QString sessionID)
{
    int pos = _sessionIDList.indexOf(sessionID);
    if (pos != -1) {
        beginRemoveRows(QModelIndex(), pos, pos);
        _sessionIDList.removeAt(pos);
        endRemoveRows();
    }
}

void SessionsModel::_refreshData()
{
    beginResetModel();
    _sessionIDList.clear();
    _sessionIDList = _manager->getSessionList().keys();
    endResetModel();
}

} // namespace Hnv
