/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "usersmodel.h"
#include "ressources.h"

// hynesim hnsclient headers
#include <libhnsclient/utility.h>
#include <libhnsclient/usermanager.h>

// Qt headers
#include <QDateTime>
#include <QIcon>

namespace Hnv {

/*******************************************************************************
* UsersFilterProxyModel
*******************************************************************************/
UsersFilterProxyModel::UsersFilterProxyModel(QObject *parent)
    : QSortFilterProxyModel(parent)
    , _acceptEnabled(true)
    , _acceptDisabled(true)
    , _acceptOnline(true)
    , _acceptOffline(true)
{
}

void UsersFilterProxyModel::acceptEnabled(bool accept)
{
    _acceptEnabled = accept;
    invalidateFilter();
}

void UsersFilterProxyModel::acceptDisabled(bool accept)
{
    _acceptDisabled = accept;
    invalidateFilter();
}

void UsersFilterProxyModel::acceptOnline(bool accept)
{
    _acceptOnline = accept;
    invalidateFilter();
}

void UsersFilterProxyModel::acceptOffline(bool accept)
{
    _acceptOffline = accept;
    invalidateFilter();
}

/*******************************************************************************
* UsersFilterProxyModel - Protected
*/
bool UsersFilterProxyModel::filterAcceptsRow(int source_row, const QModelIndex &source_parent) const
{
    QModelIndex rowIndex = sourceModel()->index(source_row, 0, source_parent);
    bool isEnabled = rowIndex.data(UsersModel::ENABLED_ROLE).toBool();
    bool isOnline = rowIndex.data(UsersModel::CONNECTED_ROLE).toBool();
    bool rowAccepted = true;

    if ((isEnabled && !_acceptEnabled)
            || (!isEnabled && !_acceptDisabled)
            || (isOnline && !_acceptOnline)
            || (!isOnline && !_acceptOffline)
            )
        rowAccepted = false;
    return rowAccepted;
}

/*******************************************************************************
* UsersModel
*******************************************************************************/
UsersModel::UsersModel(QObject *parent)
    : QAbstractTableModel(parent)
{
    _manager = 0;
    _positionList.clear();
}

void UsersModel::initModel(HnsClient::UserManager *manager)
{
    _manager = manager;
    connect(_manager, SIGNAL(userAdded(qlonglong)), this, SLOT(_userAdded(qlonglong)));
    connect(_manager, SIGNAL(userRemoved(qlonglong)), this, SLOT(_userDeleted(qlonglong)));
    connect(_manager, SIGNAL(userChanged(qlonglong)), this, SLOT(_userChanged(qlonglong)));
    _buildData();
}

void UsersModel::resetModel()
{
    _manager = 0;
    beginResetModel();
    _positionList.clear();
    endResetModel();
}

int UsersModel::rowCount(const QModelIndex &parent) const
{
    if (!parent.isValid())
        return _positionList.size();
    else
        return 0;
}

int UsersModel::columnCount(const QModelIndex &parent) const
{
    if (!parent.isValid())
        return COLUMN_SIZE;
    else
        return 0;
}

QVariant UsersModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid() || !_manager)
        return QVariant();
    qlonglong userId = _positionList.at(index.row());
    UserItem item = _userHash.value(userId);
    if (role == USER_ID_ROLE)
        return qlonglong(userId);
    else if (role == ENABLED_ROLE)
        return item.isEnabled;
    else if (role == CONNECTED_ROLE)
        return _manager->isUserConnected(userId);
    else if (role == Qt::TextAlignmentRole) {
        return int(Qt::AlignLeft | Qt::AlignVCenter);
    } else if (role == Qt::DisplayRole || role == Qt::EditRole) {

        switch(index.column()) {
        case COLUMN_ID:
            return qlonglong(userId);
        case COLUMN_USERNAME:
            return item.name;
        case COLUMN_FIRSTNAME:
            return item.firstName;
        case COLUMN_LASTNAME:
            return item.lastName;
        case COLUMN_CREATED:
            return QDateTime::fromMSecsSinceEpoch(item.createdOn).toString(Qt::SystemLocaleShortDate);
        case COLUMN_LASTMODIFIED:
            return QDateTime::fromMSecsSinceEpoch(item.modifiedOn).toString(Qt::SystemLocaleShortDate);
        case COLUMN_GROUPS:
            return _manager->getGroupsNamesFromIdList(item.groupList).join(", ");
        }
    } else if (role == Qt::DecorationRole && index.column() == COLUMN_USERNAME) {
        QString iconName = Constants::ICON_PREFIX_USER;
        // TODO:TestRight ??
//        iconName += (userPtr->isAdmin ? Constants::ICON_PREFIX_ADMIN : Constants::ICON_PREFIX_USER);
        if (!item.isEnabled)
            iconName += Constants::ICON_SUFFIX_DISABLED;
        else {
            if (_manager->isUserConnected(userId))
                iconName += Constants::ICON_SUFFIX_CONNECTED;
            else
                iconName += Constants::ICON_SUFFIX_ENABLED;
        }
        return QIcon(Ressources::themedPath(iconName));
    } else if (role == Qt::ToolTipRole) {
        QString tooltip = tr("<b>Login:&nbsp;</b><i>%1</i>").arg(item.name);
        QString firstName = item.firstName;
        QString lastName = item.lastName;
        QString name = "";
        if (!firstName.isEmpty() && !lastName.isEmpty())
            name = QString("%1&nbsp;%2").arg(firstName).arg(lastName);
        else if (!firstName.isEmpty())
            name = firstName;
        else if (!lastName.isEmpty())
            name = lastName;
        if (!name.isEmpty())
            tooltip += tr("<br/><b>Name:&nbsp;</b><i>%1</i>").arg(name);
        QString enabled = (item.isEnabled ? tr("Yes") : tr("No"));
        tooltip += tr("<br/>Enabled:&nbsp;%1").arg(enabled);
//        QString admin = (userPtr->isAdmin ? tr("Yes") : tr("No"));
//        tooltip += tr("<br/>Administrator:&nbsp;%1").arg(admin);
        QHash<QString, Diamesh::SessionInfo> sessionList = _manager->getUserSessionList(userId);
        if (sessionList.size()) {
            tooltip += tr("<br/>Current&nbsp;session(s):");
            QMutableHashIterator<QString, Diamesh::SessionInfo> iterator(sessionList);
            while(iterator.hasNext()) {
                iterator.next();
                Diamesh::SessionInfo info = iterator.value();
                tooltip += tr("<br/>Connected&nbsp;from&nbsp;%1&nbsp;since&nbsp;%2&nbsp;(%3)")
                        .arg(QString::fromStdString(info.ipAddress))
                        .arg(QDateTime::fromMSecsSinceEpoch(info.connectionDateTime).toString(Qt::SystemLocaleShortDate))
                        .arg(iterator.key());
            }

        }
        return tooltip;
    }
    return QVariant();
}

QVariant UsersModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if (role == Qt::DisplayRole && orientation == Qt::Horizontal) {
        switch (section) {
        case COLUMN_ID:
            return tr("ID");
        case COLUMN_USERNAME:
            return tr("Username");
        case COLUMN_FIRSTNAME:
            return tr("First name");
        case COLUMN_LASTNAME:
            return tr("Last name");
        case COLUMN_CREATED:
            return tr("Created");
        case COLUMN_LASTMODIFIED:
            return tr("Last modified");
        case COLUMN_GROUPS:
            return tr("Groups");
        }
    }
    return QAbstractTableModel::headerData(section, orientation, role);
}

/*******************************************************************************
* UsersModel - Private slots
*/
void UsersModel::_buildData()
{
    beginResetModel();
    _positionList.clear();
    for(int i=0; i < _manager->usersCount(); i++){
        Hns::UserPtr user = _manager->getUserByIndex(i);
        _positionList.append(user->id);
        _userHash.insert(user->id, _createItem(user));
    }
    endResetModel();
}

void UsersModel::_userAdded(qlonglong newUserID)
{
    int pos = _positionList.size();
    beginInsertRows(QModelIndex(), pos, pos);
    _positionList.append(newUserID);
    _userHash.insert(newUserID, _createItem(_manager->getUserById(newUserID)));
    endInsertRows();
}

void UsersModel::_userChanged(qlonglong userID)
{
    int row = _positionList.indexOf(userID);
    if (row != -1) {
        QModelIndex firstIdx = index(row, 0);
        QModelIndex lastIdx = index(row, COLUMN_SIZE - 1);
        _userHash.insert(userID, _createItem(_manager->getUserById(userID)));
        emit dataChanged(firstIdx, lastIdx);
    }
}

void UsersModel::_userDeleted(qlonglong deletedUserID)
{
    int row = _positionList.indexOf(deletedUserID);
    if (row != -1) {
        beginRemoveRows(QModelIndex(), row, row);
        _positionList.removeAt(row);
        _userHash.remove(deletedUserID);
        endRemoveRows();
    }
}

UsersModel::UserItem UsersModel::_createItem(Hns::UserPtr user)
{
    UserItem item;
    item.id = user->id;
    item.isEnabled = user->isEnabled;
    item.name = QString::fromStdString(user->name);
    item.firstName = QString::fromStdString(user->firstname);
    item.lastName = QString::fromStdString(user->lastname);
    item.createdOn = user->createdOn;
    item.modifiedOn = user->modifiedOn;
    item.groupList = HnsClient::Utility::qLongListFromLongList(user->groupList);
    return item;
}

} // namespace Hnv
