// Project headers
#include "entityaction.h"

// Qt headers
#include <QAction>

namespace Hnv {

/*******************************************************************************
* EntityActionHandler
*******************************************************************************/
EntityActionHandler::EntityActionHandler(QString uniqueID, QIcon icon, QString text, QString tooltip, QKeySequence shortcut, QObject *parent)
    : QObject(parent)
    , _actionUniqueID(uniqueID)
    , _actionIcon(icon)
    , _actionText(text)
    , _actionToolTip(tooltip)
    , _actionShortCut(shortcut)
    , _enableAtStart(false)
    , _alwaysDisable(false)
{
}

EntityActionHandler::~EntityActionHandler()
{
    emit removeFromList(this);
}

QString EntityActionHandler::uniqueID()
{
    return _actionUniqueID;
}

QAction *EntityActionHandler::action()
{
    QAction * result = new QAction(_actionIcon, _actionText, this);
    result->setToolTip(_actionToolTip);
    result->setShortcut(_actionShortCut);
    result->setEnabled(!_alwaysDisable);
    return result;
}


void EntityActionHandler::setEnableAtStart(bool enable){
    _enableAtStart = enable;
}

bool EntityActionHandler::isEnableAtStart(){
    return _enableAtStart;
}

void EntityActionHandler::setAlwaysDisable(bool disable)
{
    _alwaysDisable = disable;
}


bool EntityActionHandler::isAlwaysDisable()
{
    return _alwaysDisable;
}

/*******************************************************************************
* EntityActionDispatcher
*******************************************************************************/
QList<EntityActionDispatcher *> EntityActionDispatcher::parseHandlerList(QList<QList<EntityActionHandler *> > handlerList)
{
    QMap<QString, EntityActionDispatcher*> dispatcherMap;
    QList<EntityActionHandler *> garbageHandlerList;
    QList<EntityActionDispatcher *> garbargeDispatcherList;
    bool firstLoop = true;
    bool clearAll = false;

    foreach(const QList<EntityActionHandler *> &entityHandlerList, handlerList) {
        if (entityHandlerList.count() == 0) {
            clearAll = true;
            break;
        }

        QStringList handlerIDs;
        foreach(EntityActionHandler *handler, entityHandlerList) {
            QString handlerID = handler->uniqueID();
            handlerIDs.append(handlerID);
            // Add new dispatcher based on handler ID
            if (firstLoop && !dispatcherMap.contains(handlerID))
                dispatcherMap.insert(handlerID, new EntityActionDispatcher());

            // Knowed handler ID are registered
            if (dispatcherMap.contains(handlerID)) {
                dispatcherMap[handlerID]->registerHandler(handler);
                if (dispatcherMap[handlerID]->isHandlerDisable(handlerID) && !handler->isAlwaysDisable()) {
                    dispatcherMap[handlerID]->enableHandler(handlerID);
                }
            }
            else
                garbageHandlerList.append(handler);
        }

        QStringList dispatcherKeys = dispatcherMap.keys();
        foreach(const QString &commonHandlerID, dispatcherKeys) {
            if (!handlerIDs.contains(commonHandlerID)) {
                // Clear handlers which are not common to every entities
                garbargeDispatcherList.append(dispatcherMap.take(commonHandlerID));
            }
        }
        firstLoop = false;
    }

    // Destroy un-needed handler and dispatcher
    if (clearAll) {
        dispatcherMap.clear();
        garbageHandlerList.clear();
        garbargeDispatcherList.clear();
        foreach(QList<EntityActionHandler *> entityHandlerList, handlerList) {
            qDeleteAll(entityHandlerList);
            entityHandlerList.clear();
        }
        handlerList.clear();
    } else {
        qDeleteAll(garbageHandlerList);
        garbageHandlerList.clear();
        qDeleteAll(garbargeDispatcherList);
        garbargeDispatcherList.clear();
    }
    return dispatcherMap.values();
}

EntityActionDispatcher::EntityActionDispatcher(QObject *parent)
    : QObject(parent)
    , _actionUniqueID("")
{
}

EntityActionDispatcher::~EntityActionDispatcher()
{
    foreach(EntityActionHandler * entityAction, _handlerList) {
        disconnect(entityAction);
        entityAction->deleteLater();
    }
}

bool EntityActionDispatcher::registerHandler(EntityActionHandler *handler)
{
    QString handlerID = handler->uniqueID();
    if (_actionUniqueID.isEmpty())
        _actionUniqueID = handlerID;
    else if (_actionUniqueID != handlerID)
        return false;
    connect(handler, SIGNAL(removeFromList(EntityActionHandler *)), this, SLOT(_removeHandlerListSlot(EntityActionHandler *)), Qt::DirectConnection);
    _handlerList.append(handler);
    return true;
}

QAction *EntityActionDispatcher::action(bool startIsActivate)
{
    if (_handlerList.isEmpty())
        return 0;
    QAction * result = _handlerList.first()->action();
    if(!startIsActivate && _handlerList.first()->isEnableAtStart()) result->setEnabled(false);
    connect(result, SIGNAL(destroyed()), this, SLOT(_actionDestroyedSlot()));
    foreach(EntityActionHandler *handler, _handlerList)
        connect(result, SIGNAL(triggered()), handler, SIGNAL(actionTriggered()));
    return result;
}

void EntityActionDispatcher::enableHandler(QString handlerID)
{
    foreach(EntityActionHandler *handler, _handlerList) {
        if (handler->uniqueID() == handlerID) {
            handler->setAlwaysDisable(false);
            break;
        }
    }
}

bool EntityActionDispatcher::isHandlerDisable(QString handlerID)
{
    foreach(EntityActionHandler *handler, _handlerList) {
        if (handler->uniqueID() == handlerID) {
            return handler->isAlwaysDisable();
        }
    }
    return false;
}

void EntityActionDispatcher::_actionDestroyedSlot()
{
    deleteLater();
}

void EntityActionDispatcher::_removeHandlerListSlot(EntityActionHandler *actionHandler)
{
    _handlerList.removeOne(actionHandler);
}

} // namespace Hnv
