/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "entitygraphics.h"
#include "ressources.h"
#include "hyneviewsession.h"
#include "entitycatalogmodel.h"
#include "topologyview.h"
#include "topologyscene.h"
#include "entityaction.h"
#include "dynamipsrouterloadidle.h"
#include "wifiapconnectedpeerswidget.h"
#include "remotemanager.h"
#include "topologyviewarea.h"
#include "hnv_entryproperties.h"
#include "hnvwidgets.h"

// hynesim common headers
#include "common/initializer.h"

// Generated hynesim headers
#include "common/entityentry.h"
#include "common/domainentry.h"
#include "common/switchentry.h"
#include "common/hybridnetcardentry.h"
#include "common/wifiapentry.h"
#include "common/dynamipsrouterentry.h"
#include "common/topologygateentry.h"

// hynesim hnsclient headers
#include <libhnsclient/entity.h>
#include <libhnsclient/entry.h>
#include <libhnsclient/session.h>
#include <libhnsclient/platformmonitor.h>
#include <libhnsclient/resourcemanager.h>
#include <libhnsclient/entityentrycatalog.h>
#include <libhnsclient/topology.h>

// Qt headers
#include <qmath.h>
#include <QDebug>
#include <QRectF>
#include <QSettings>
#include <QApplication>
#include <QBrush>
#include <QPainter>
#include <QStyleOptionGraphicsItem>
#include <QGraphicsSceneDragDropEvent>
#include <QVector2D>
#include <QSvgRenderer>
#include <QGraphicsSvgItem>
#include <QMimeData>

namespace Hnv {
HNS_STATIC_HASH_INITIALIZER(_entityGraphicObjectHash, EntityGraphicObjectHash, QString, QMetaObject)
EntityGraphicObjectHash::EntityGraphicObjectHash()
{
    insert(QString::fromStdString(Hns::DomainEntry::ice_staticId()), DomainEntityGraphics::staticMetaObject);
    insert(QString::fromStdString(Hns::SwitchEntry::ice_staticId()), SwitchEntityGraphics::staticMetaObject);
    insert(QString::fromStdString(Hns::HybridNetcardEntry::ice_staticId()), HybridNetcardEntityGraphics::staticMetaObject);
    insert(QString::fromStdString(Hns::WifiAPEntry::ice_staticId()), WifiAPEntityGraphics::staticMetaObject);
    insert(QString::fromStdString(Hns::DynamipsRouterEntry::ice_staticId()), DynamipsRouterEntityGraphics::staticMetaObject);
    insert(QString::fromStdString(Hns::TopologyGateEntry::ice_staticId()), TopologyBridgeEntityGraphics::staticMetaObject);
}

/*******************************************************************************
* GraphicsItem
*******************************************************************************/
GraphicsItem::GraphicsItem(QObject *parent)
    : QObject(parent)
{
}

//inspired by http://www.w3.org/TR/AERT#color-contrast
int GraphicsItem::_brightness(QColor c)
{
    return (int)sqrt(
          c.red() * c.red() * /*.241*/ .299 +
          c.green() * c.green() * /*.691*/ .587 +
          c.blue() * c.blue() * /*.068*/ .114);
}

/*******************************************************************************
* WireGraphics
*******************************************************************************/
WireGraphics *WireGraphics::create(HnsClient::WireEntity *entity, QPointF from, QPointF to, TopologyScene *scene, TopologyViewArea *area, QGraphicsItem *parent)
{
    WireGraphics *graphic = 0;
    if (entity) {
        QString iceId = entity->iceId();
        if (iceId == QString::fromStdString(::Hns::WireEntry::ice_staticId())) {
            graphic = new WireGraphics(entity, from, to, scene, area, parent);
            if (graphic)
                graphic->initGraphics();
        }
    }
    return graphic;
}

void WireGraphics::tryToConnect(QPointF from, QPointF to, TopologyView *view)
{
    view->emitRequestWireEntityItem(from, to);
}

WireGraphics::WireGraphics(HnsClient::WireEntity *wire, QPointF from, QPointF to, TopologyScene *scene, TopologyViewArea *area, QGraphicsItem *parent)
    : QAbstractGraphicsShapeItem(parent)
    , _wire(wire)
    , _scene(scene)
    , _from(0)
    , _to(0)
    , _numberAngles(-1)
    , _area(area)
    , _nearToEntity(false)
    , _nearFromEntity(false)
{
    connect(_wire, SIGNAL(destroyed()), this, SLOT(_wireDestroyedSlot()));
    setFlags(ItemIsSelectable);
    QRect fromRect(_fromOrigin.x() - 10, _fromOrigin.y() - 10, 20, 20);
    QRect toRect(_toOrigin.x() - 10, _toOrigin.y() - 10, 20, 20);
    _fromPlugGraphic = new WirePlugGraphic(_color, fromRect, this);
    connect(_fromPlugGraphic, SIGNAL(movingRequested(QPoint,bool)), this, SLOT(_fromPlugGraphicRequestedSlot(QPoint,bool)));
    _toPlugGraphic = new WirePlugGraphic(_color, toRect, this);
    connect(_toPlugGraphic, SIGNAL(movingRequested(QPoint,bool)), this, SLOT(_toPlugGraphicRequestedSlot(QPoint,bool)));
    setAcceptsHoverEvents(true);
    _setLayout(_wire->getLayout());
    setPen(QPen(QColor(_color), 5, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
    setZValue(-1);

    EntityGraphics::Direction fromDirection;
    EntityGraphics::Direction toDirection;
    _getDirections(fromDirection, toDirection);

    update();

    if (!_to && !_from)
        _draw(from, to);

    connect(_wire, SIGNAL(layoutChanged()), this, SLOT(_layoutChangedSlot()));
}

WireGraphics::~WireGraphics()
{
    if (_from)
        _from->removeWire(this);
    if (_to)
        _to->removeWire(this);
}

void WireGraphics::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *)
{
    painter->save();
    qreal penWidth = 5;

    QTransform worldTransform = painter->worldTransform();
    int levelOfDetail = option->levelOfDetailFromTransform(worldTransform);
    if (penWidth * levelOfDetail < 1)
        penWidth = 0;
    if (_nearToEntity || _nearFromEntity) {
        if (penWidth == 0)
            penWidth = 10;
        else
            penWidth *= 2;
    }

    if (_pointList.size() > 1) {
        QPen customPen(QColor(_color), penWidth, Qt::SolidLine, Qt::RoundCap);

        if (isSelected())
            customPen.setStyle(Qt::DashLine);
        else if (_fromPlugGraphic->isVisible() || _toPlugGraphic->isVisible()) {
            QVector<qreal> dashes;
            dashes << 1 << 4;
            customPen.setDashPattern(dashes);
        }
        if (_fromPlugGraphic->isVisible() || _toPlugGraphic->isVisible()) {
            QBrush penBrush = customPen.brush();
            penBrush.setStyle(Qt::Dense4Pattern);
            customPen.setBrush(penBrush);
        }

        painter->setPen(customPen);

        QPointF p1 = _pointList.at(0);
        for(int i = 1; i < _pointList.size(); i++) {
            QPointF p2 = _pointList.at(i);
            painter->drawLine(p1, p2);
            p1 = p2;
        }
    }
    painter->restore();
}

QRectF WireGraphics::boundingRect() const
{
    return _boundingRect;
}

QPainterPath WireGraphics::shape() const
{
    return _shapePath;
}

void WireGraphics::refresh()
{
}

qlonglong WireGraphics::getEntityID() const
{
    return _wire->getId();
}

void WireGraphics::initGraphics()
{
    _scene->addItem(this);
}

QPointF WireGraphics::getOtherEndPoint(EntityGraphics *thisEntity)
{
    if(thisEntity == _from) {
        if (_to)
            return mapFromItem(_to, _to->boundingRect().center());
        else
            return _toOrigin;
    } else {
        if (_from)
            return mapFromItem(_from, _from->boundingRect().center());
        else
            return _fromOrigin;
    }
    return QPointF();
}

void WireGraphics::updatePosition()
{
    _checkDirections();
}

void WireGraphics::setOrigin(EntityGraphics *source, QPointF origin)
{
    if (source == _from && _fromOrigin != origin) {
        _fromOrigin = origin;
        _calculatePath();
    }
    else if (source == _to && _toOrigin != origin) {
        _toOrigin = origin;
        _calculatePath();
    }
}

void WireGraphics::setFromEntityGraphics(EntityGraphics *source)
{
    _fromPlugGraphic->setVisible(false);
    _from = source;
    EntityGraphics::Direction fromDirection;
    EntityGraphics::Direction toDirection;
    _getDirections(fromDirection, toDirection);
    _from->addWire(this, fromDirection);
}

void WireGraphics::setToEntityGraphics(EntityGraphics *source)
{
    _toPlugGraphic->setVisible(false);
    _to = source;
    EntityGraphics::Direction fromDirection;
    EntityGraphics::Direction toDirection;
    _getDirections(fromDirection, toDirection);
    _to->addWire(this, toDirection);
}

Hns::WireLayoutPtr WireGraphics::layout()
{
    return _wire->getLayout();
}

void WireGraphics::updateLayout(const Hns::WireLayoutPtr &newLayout)
{
    if (newLayout->id
        && (newLayout->id != layout()->id
        || newLayout->labelPolicyState != layout()->labelPolicyState
        || newLayout->colorCode != layout()->colorCode
        || newLayout->x1 != layout()->x1
        || newLayout->x2 != layout()->x2
        || newLayout->y1 != layout()->y1
        || newLayout->y2 != layout()->y2
        || newLayout->numberAngles != layout()->numberAngles)) {
        _setLayout(newLayout);
        emit layoutChanged(newLayout);
        _checkDirections();
        _calculatePath();
        update();
    }
}

QList<EntityActionHandler *> WireGraphics::customActions(bool)
{
    QList<EntityActionHandler *> handlerList;
    QString uniqueID = staticMetaObject.className();

    int angles = layout()->numberAngles;
    handlerList.append(
                new EntityActionHandler(
                    uniqueID + QString::number(WireGraphics::toWire0),
                    QIcon(Ressources::themedPath(Constants::CHANGE_TO_0_ANGLE)),
                    tr("Change to 0 angle"), tr("Change to 0 angle"), QKeySequence(), this));
    if (angles == 0)
        handlerList.last()->setAlwaysDisable(true);
    connect(handlerList.last(), SIGNAL(actionTriggered()), this, SLOT(_changeTo0Angle()));

    handlerList.append(
                new EntityActionHandler(
                    uniqueID + QString::number(WireGraphics::toWire1),
                    QIcon(Ressources::themedPath(Constants::CHANGE_TO_1_ANGLE)),
                    tr("Change to 1 angle"), tr("Change to 1 angle"), QKeySequence(), this));
    if (angles == 1)
        handlerList.last()->setAlwaysDisable(true);
    connect(handlerList.last(), SIGNAL(actionTriggered()), this, SLOT(_changeTo1Angle()));

    handlerList.append(
                new EntityActionHandler(
                    uniqueID + QString::number(WireGraphics::toWire2),
                    QIcon(Ressources::themedPath(Constants::CHANGE_TO_2_ANGLES)),
                    tr("Change to 2 angles"), tr("Change to 2 angles"), QKeySequence(), this));
    if (angles == 2)
        handlerList.last()->setAlwaysDisable(true);
    connect(handlerList.last(), SIGNAL(actionTriggered()), this, SLOT(_changeTo2Angles()));

    if (_from) {
        QString name = tr("Unplug from %1").arg(_from->entity()->getShortName());
        handlerList.append(
                    new EntityActionHandler(
                        uniqueID + "/" + QString::number(WireGraphics::unplugTo) + QString::number(getEntityID()),
                        QIcon(),
                        name, name, QKeySequence(), this));
        connect(handlerList.last(), SIGNAL(actionTriggered()), this, SLOT(_unplugFrom()));
    }
    if (_to) {
        QString name = tr("Unplug from %1").arg(_to->entity()->getShortName());
        handlerList.append(
                    new EntityActionHandler(
                        uniqueID + "/" + QString::number(WireGraphics::unplugFrom) + QString::number(getEntityID()),
                        QIcon(),
                        name, name, QKeySequence(), this));
        connect(handlerList.last(), SIGNAL(actionTriggered()), this, SLOT(_unplugTo()));
    }
    handlerList.append(
                new EntityActionHandler(
                    uniqueID + QString::number(WireGraphics::addToCatalog),
                    QIcon(),
                    tr("Add to catalog"), tr("Add to catalog"), QKeySequence(), this));
    connect(handlerList.last(), SIGNAL(actionTriggered()), this, SLOT(_addToCatalog()));

    return handlerList;
}

void WireGraphics::displayProperties(QWidget *parent)
{
    HnsClient::EntityEntry *updatedEntry = qobject_cast<HnsClient::EntityEntry *>(_wire->getEntry()->duplicateEntry(this));
    if (updatedEntry) {
        EntryProperties popup(updatedEntry, parent, false, (_wire->getStatus() != Hns::UNDEFINED && _wire->getStatus() != Hns::UNIMPLEMENTED), false, false);
        HnvDialogHandler *popupHandler = new HnvDialogHandler(_wire, &popup, parent);
        popupHandler->setSourceDestroyedMessage(tr("\"%1\" was deleted. Operation aborted.").arg(_wire->getShortName()));
        if (popup.exec() == QDialog::Accepted)
            _wire->updateEntry(updatedEntry);
    }
}

QPointF WireGraphics::fromPoint() const
{
    return _fromOrigin;
}

QPointF WireGraphics::toPoint() const
{
    return _toOrigin;
}

void WireGraphics::setColor(const QString &color)
{
    _color = color;
    _toPlugGraphic->setColor(_color);
    _fromPlugGraphic->setColor(_color);
}

EntityGraphics *WireGraphics::to()
{
    return _to;
}

EntityGraphics *WireGraphics::from()
{
    return _from;
}

void WireGraphics::checkIsLayoutChanged()
{
    QPointF p1(layout()->x1, layout()->y1);
    QPointF p2(layout()->x2, layout()->y2);
    if (p1 != _fromOrigin || p2 != _toOrigin) {
        Hns::WireLayoutPtr myLayout = Hns::WireLayoutPtr::dynamicCast(layout()->ice_clone());
        myLayout->x1 = _fromOrigin.x();
        myLayout->y1 = _fromOrigin.y();
        myLayout->x2 = _toOrigin.x();
        myLayout->y2 = _toOrigin.y();
        updateLayout(myLayout);
    }
}

void WireGraphics::unplugEntityGraphics(EntityGraphics *entityGraphics)
{
    if (entityGraphics == _from) {
        _from->removeWire(this);
        _from = 0;
        _fromPlugGraphic->setVisible(true);
    }
    else if (entityGraphics == _to) {
        _to->removeWire(this);
        _toPlugGraphic->setVisible(true);
        _to = 0;
    }
}

void WireGraphics::setNearToEntity(bool nearToEntity)
{
    if (_nearToEntity != nearToEntity) {
        _nearToEntity = nearToEntity;
        _toPlugGraphic->setCloseToEntity(nearToEntity);
    }
}

void WireGraphics::setNearFromEntity(bool nearFromEntity)
{
    if (_nearFromEntity != nearFromEntity) {
        _nearFromEntity = nearFromEntity;
        _fromPlugGraphic->setCloseToEntity(nearFromEntity);
    }
}

void WireGraphics::mouseDoubleClickEvent (QGraphicsSceneMouseEvent * event )
{
    QAbstractGraphicsShapeItem::mouseDoubleClickEvent(event);
    displayProperties(scene()->views().at(0));
}

void WireGraphics::_changeAngles(int angles)
{
    Hns::WireLayoutPtr myLayout = Hns::WireLayoutPtr::dynamicCast(layout()->ice_clone());
    if (myLayout->numberAngles != angles) {
        myLayout->numberAngles = angles;
        updateLayout(myLayout);
    }
}

void WireGraphics::_wireDestroyedSlot()
{
    delete this;
}

void WireGraphics::_layoutChangedSlot()
{
    _setLayout(_wire->getLayout());
    _checkDirections();
    _calculatePath();
    update();
}

void WireGraphics::_changeTo0Angle()
{
    _changeAngles(0);
}

void WireGraphics::_changeTo1Angle()
{
    _changeAngles(1);
}

void WireGraphics::_changeTo2Angles()
{
    _changeAngles(2);
}

void WireGraphics::_fromDestroyed()
{
    _from = 0;
}

void WireGraphics::_toDestroyed()
{
    _to = 0;
}

void WireGraphics::_unplugTo()
{
    emit unplugWireRequested(getEntityID(), 1);
}

void WireGraphics::_unplugFrom()
{
    emit unplugWireRequested(getEntityID(), 0);
}

void WireGraphics::_addToCatalog()
{
    emit addToCatalogRequested(getEntityID());
}

void WireGraphics::_fromPlugGraphicRequestedSlot(QPoint point, bool mouseReleased)
{
    Hns::WireLayoutPtr myLayout = Hns::WireLayoutPtr::dynamicCast(layout()->ice_clone());
    myLayout->x1 = point.x();
    myLayout->y1 = point.y();
    _plugGraphicMoving(myLayout, point, mouseReleased, 0);
}

void WireGraphics::_toPlugGraphicRequestedSlot(QPoint point, bool mouseReleased)
{
    Hns::WireLayoutPtr myLayout = Hns::WireLayoutPtr::dynamicCast(layout()->ice_clone());
    myLayout->x2 = point.x();
    myLayout->y2 = point.y();
    _plugGraphicMoving(myLayout, point, mouseReleased, 1);
}

void WireGraphics::_calculatePath()
{
    qreal fromX = _fromOrigin.x();
    qreal fromY = _fromOrigin.y();
    qreal toX = _toOrigin.x();
    qreal toY = _toOrigin.y();
    qreal deltaX = fromX-toX;
    qreal deltaY = fromY-toY;

    QList<QPointF> pointList;
    pointList.append(_fromOrigin);
    switch(_numberAngles) {
    case 0:
        break;
    case 1: {
        bool reverseX = false;
        qreal midPointX;
        qreal midPointY;
        if (_midPointX)
            midPointY = qMin(fromY, toY);
        else {
            midPointY = qMax(fromY, toY);
            reverseX = true;
        }
        if ((deltaX < 0) ^ (deltaY < 0))
            reverseX = !reverseX;
        if (reverseX)
            midPointX = qMin(fromX, toX);
        else
            midPointX = qMax(fromX, toX);
        pointList.append(QPointF(midPointX, midPointY));
        break;
    }
    case 2: {
        if (_midPointX) {
            pointList.append(QPointF(_midPointX, fromY));
            pointList.append(QPointF(_midPointX, toY));
        } else {
            pointList.append(QPointF(fromX, _midPointY));
            pointList.append(QPointF(toX, _midPointY));
        }
        break;
    }
    }
    pointList.append(_toOrigin);
    if (pointList == _pointList)
        return;
    _pointList.clear();
    _pointList = pointList;

    QPainterPath shapePath;
    QPointF p1 = _pointList.at(0);
    for(int i = 1; i < _pointList.size(); i++) {
        QPointF p2 = _pointList.at(i);
        QVector2D normalVector(-(p2.y() - p1.y()), (p2.x() - p1.x()));
        normalVector.normalize();
        normalVector *= 3;
        QLineF line1(p1, p2);
        QLineF line2(p1, p2);
        line1.translate(normalVector.x(), normalVector.y());
        line2.translate(-normalVector.x(), -normalVector.y());
        shapePath.moveTo(line1.p1());
        shapePath.lineTo(line1.p2());
        shapePath.lineTo(line2.p2());
        shapePath.lineTo(line2.p1());
        shapePath.lineTo(line1.p1());
        p1 = p2;
    }
    _shapePath = shapePath;

    QRectF boundingRect = _shapePath.boundingRect();
    if (boundingRect != _boundingRect) {
        prepareGeometryChange();
        _boundingRect = boundingRect;
    }
}

void WireGraphics::_getDirections(GraphicsItem::Direction &fromDirection, GraphicsItem::Direction &toDirection)
{
    _checkDirections(fromDirection, toDirection, false);
}

bool WireGraphics::_checkDirections()
{
    EntityGraphics::Direction fromDirection;
    EntityGraphics::Direction toDirection;
    return _checkDirections(fromDirection, toDirection, true);
}

bool WireGraphics::_checkDirections(GraphicsItem::Direction &fromDirection, GraphicsItem::Direction &toDirection, bool updateEntities)
{
    QPointF fromCenter;
    QPointF toCenter;
    if (_from)
        fromCenter = mapFromItem(_from, _from->boundingRect().center());
    else
        fromCenter = QPointF(_wire->getLayout()->x1, _wire->getLayout()->y1);

    if(_to)
        toCenter = mapFromItem(_to, _to->boundingRect().center());
    else
        toCenter = QPointF(_wire->getLayout()->x2, _wire->getLayout()->y2);

    qreal fromX = fromCenter.x();
    qreal fromY = fromCenter.y();
    qreal toX = toCenter.x();
    qreal toY = toCenter.y();
    _midPointX = 0;
    _midPointY = 0;

    qreal deltaX = fromX-toX;
    qreal deltaY = fromY-toY;
    qreal absDeltaX = abs(deltaX);
    qreal absDeltaY = abs(deltaY);

    if (absDeltaX >= absDeltaY) {
        fromDirection = _getDirection(fromX, fromY, toX, toY, true);
        toDirection = _getDirection(fromX, fromY, toX, toY, true, false);
        _midPointX = (fromX + toX)/2;
    } else {
        fromDirection = _getDirection(fromX, fromY, toX, toY, false);
        toDirection = _getDirection(fromX, fromY, toX, toY, false, false);
        _midPointY = (fromY + toY)/2;
    }

    if (!updateEntities)
        return false;
    bool result = true;
    if (_from)
        result &= _from->setWireDirection(this, fromDirection);
    if (_to)
        result &= _to->setWireDirection(this, toDirection);
    return result;
}

void WireGraphics::_setLayout(const Hns::WireLayoutPtr &layout)
{
    _numberAngles = layout->numberAngles;
    _color = QString::fromStdString(layout->colorCode);
    _fromOrigin = QPointF(layout->x1, layout->y1);
    _toOrigin = QPointF(layout->x2, layout->y2);
    _fromPlugGraphic->setColor(_color);
    _fromPlugGraphic->setPos(_fromOrigin);
    _toPlugGraphic->setColor(_color);
    _toPlugGraphic->setPos(_toOrigin);
}

void WireGraphics::_draw(const QPointF &from, const QPointF &to)
{
    if (to.isNull() || from.isNull()) {
        qCritical() << tr("Can not draw a wire without two points");
        return;
    }

    _fromOrigin = from;
    _toOrigin = to;
    _calculatePath();
}

GraphicsItem::Direction WireGraphics::_getHorizontalDirection(qreal x1, qreal x2, bool isSource)
{
    if (!isSource) {
        qreal tempX = x1;
        x1 = x2;
        x2 = tempX;
    }
    if (x1 < x2)
        return EntityGraphics::East;
    else
        return EntityGraphics::West;
}

GraphicsItem::Direction WireGraphics::_getVerticalDirection(qreal y1, qreal y2, bool isSource)
{
    if (!isSource) {
        qreal tempY = y1;
        y1 = y2;
        y2 = tempY;
    }
    if (y1 < y2)
        return EntityGraphics::South;
    else
        return EntityGraphics::North;
}

GraphicsItem::Direction WireGraphics::_getDirection(qreal x1, qreal y1, qreal x2, qreal y2, bool horizontal, bool isSource)
{
    if (_numberAngles == 1) {
        if (y2 < y1)
            horizontal = !horizontal;
        if (!isSource)
            horizontal = !horizontal;
    }
    if (horizontal)
        return _getHorizontalDirection(x1, x2, isSource);
    else
        return _getVerticalDirection(y1, y2, isSource);
}

void WireGraphics::_plugGraphicMoving(const Hns::WireLayoutPtr &layout, QPoint point, bool mouseReleased, int wirePort)
{
    if (_area->insertMode())
        return;
    if (mouseReleased) {
        updateLayout(layout);
        foreach (EntityGraphics *entityGraphics, _area->getEntitiesGraphicsByPosition(point))
            entityGraphics->setIsHighlighted(EntityGraphics::None);

        if (_area->getEntityByPosition(point) != 0) {
            EntityGraphics *entity = _area->getEntityGraphicsByPosition(point);
            if (entity->entity()->getAvailablePlugs().isEmpty())
                return;
            if ((_to && _to == entity) || (_from && _from == entity))
                return;
            emit plugWireToEntityRequested(getEntityID(), wirePort, entity->getEntityID());
        }
    }
    else {
        updateLayout(layout);
        foreach (EntityGraphics *entity, _area->entityGraphics()) {
            if (!entity)
                continue;
            if ((_to && _to == entity) || (_from && _from == entity))
                continue;
            QRectF rect(entity->boundingRect());
            rect.moveTopLeft(entity->pos());
            if (rect.contains(point)) {
                if (entity->entity()->getAvailablePlugs().isEmpty())
                    entity->setIsHighlighted(EntityGraphics::NOK);
                else
                    entity->setIsHighlighted(EntityGraphics::OK);
            }
            else
                entity->setIsHighlighted(EntityGraphics::None);
        }
    }
}

/*******************************************************************************
* WirePlugGraphic
*******************************************************************************/
WirePlugGraphic::WirePlugGraphic(const QColor &color, const QRectF &rect, WireGraphics *parentWire)
    : QObject(parentWire)
    , QGraphicsEllipseItem(rect, parentWire)
    , _color(color)
{
    setFlags(ItemIsSelectable);
    setPen(QPen(_color));
    setBrush(_color);
}

WirePlugGraphic::~WirePlugGraphic()
{
}

void WirePlugGraphic::setColor(const QColor &color)
{
    _color = color;
    setPen(QPen(_color));
    setBrush(color);
}

void WirePlugGraphic::setCloseToEntity(bool closeTo)
{
    QRectF rect(this->rect());
    int interval = 10;
    if (!closeTo)
        interval = -interval;

    rect.setLeft(rect.left() - interval);
    rect.setRight(rect.right() + interval);
    rect.setTop(rect.top() - interval);
    rect.setBottom(rect.bottom() + interval);

    setRect(rect);
}

void WirePlugGraphic::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
{
    QStyleOptionGraphicsItem myOption(*option);
    myOption.state &= ~QStyle::State_Selected;
    QGraphicsEllipseItem::paint(painter, &myOption, widget);
}

void WirePlugGraphic::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{
    if (event->buttons() == Qt::LeftButton) {
        QPoint point = EntityGraphics::validateItemPosition(mapToScene(event->pos()), boundingRect(), scene()).toPoint();
        emit movingRequested(point, false);
    }
    QGraphicsItem::mouseMoveEvent(event);
}

void WirePlugGraphic::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
{
    QPoint point = EntityGraphics::validateItemPosition(mapToScene(event->pos()), boundingRect(), scene()).toPoint();
    emit movingRequested(point, true);
    event->accept();
}

/*******************************************************************************
* EntityGrapics
*******************************************************************************/
EntityGraphics *EntityGraphics::create(HnsClient::Entity *entity, TopologyScene *scene, TopologyViewArea *area, QGraphicsItem *parent)
{
    EntityGraphics *graphic = 0;
    if (entity) {
        QString iceId = entity->iceId();
        if (_entityGraphicObjectHash()->contains(iceId)) {
            QMetaObject metaObject = _entityGraphicObjectHash()->value(iceId);
            QGenericArgument argEntity = Q_ARG(HnsClient::Entity *, entity);
            QGenericArgument argScene = Q_ARG(Hnv::TopologyScene *, scene);
            QGenericArgument argArea = Q_ARG(Hnv::TopologyViewArea *, area);
            QGenericArgument argParent = Q_ARG(QGraphicsItem *, parent);
            QObject *obj = metaObject.newInstance(argEntity, argScene, argArea, argParent);
            if (obj) {
                graphic = qobject_cast<EntityGraphics *>(obj);
                if (graphic)
                    graphic->initGraphics();
            }
        }
    }
    return graphic;
}

EntityGraphics::EntityGraphics(HnsClient::Entity *entity, TopologyScene *scene, TopologyViewArea *area, QGraphicsItem *parent)
    : QGraphicsRectItem(parent)
    , _entity(entity)
    , _scene(scene)
    , _area(area)
    , _svgItem(NULL)
    , _shortNameTextItem(NULL)
    , _nodeNameTextItem(NULL)
    , _statusIcon(NULL)
    , _captureIcon(NULL)
    , _immutableIcon(NULL)
    , _usbIcon(NULL)
    , _statusIconDisplayed(true)
    , _satID(0)
    , _isHighlighted(None)
    , _errorItem(NULL)
    , _isGray(false)
{
    connect(_entity, SIGNAL(destroyed()), this, SLOT(_entityDestroyedSlot()));
    setFlags(ItemIsSelectable| ItemIsMovable | QGraphicsItem::ItemSendsGeometryChanges);
    setAcceptsHoverEvents(true);
    _previousLayout = layout();
    setToolTip(Ressources::getToolTip(_entity));
    _setStatus(_entity->getStatus());
    setAcceptDrops(true);
    Ressources::registerSvgRenderer(Ressources::themedPath(Constants::GRAPHIC_INFO_SATELLITE_ACTIVE));
    Ressources::registerSvgRenderer(Ressources::themedPath(Constants::GRAPHIC_INFO_SATELLITE_INACTIVE));

    if (_entity->isModel())
        _isGray = true;
    connect(_entity, SIGNAL(entryChanged(HnsClient::GenericEntity*)), this, SLOT(_entryChangedSlot(HnsClient::GenericEntity *)));
    connect(_entity, SIGNAL(layoutChanged()), this, SLOT(_layoutChangedSlot()));
    connect(_entity, SIGNAL(statusChanged(HnsClient::GenericEntity*)), this, SLOT(_statusChangedSlot(HnsClient::GenericEntity *)));
    connect(_entity, SIGNAL(refreshed(HnsClient::GenericEntity *)), this, SLOT(_refreshedSlot()));
    connect(_entity, SIGNAL(captureJobUseInterface(Ice::Int,Ice::Int)), this, SLOT(_updateCaptureIcon()));
    connect(_entity, SIGNAL(captureJobReleasedInterface(Ice::Int,Ice::Int)), this, SLOT(_updateCaptureIcon()));
}

EntityGraphics::~EntityGraphics()
{
    clearSatellites();
    if (_errorItem) {
        delete _errorItem;
        _errorItem = NULL;
    }
    foreach (QList<WireGraphics*> wireList, _wireHash) {
        foreach (WireGraphics *wireGraphics, wireList)
            wireGraphics->unplugEntityGraphics(this);
    }
}

QPainterPath EntityGraphics::shape() const
{
    QPainterPath path;
    path.addRect(boundingRect());
    return path;
}

QRectF EntityGraphics::boundingRect() const
{
    QRectF result;
    QList<QGraphicsItem *> children;
    if (_statusIcon)
        children.append(_statusIcon);
    if (_captureIcon)
        children.append(_captureIcon);
    if (_immutableIcon)
        children.append(_immutableIcon);
    if (_usbIcon)
        children.append(_usbIcon);
    if (_svgItem)
        children.append(_svgItem);
    if (_shortNameTextItem)
        children.append(_shortNameTextItem);
    if (_nodeNameTextItem)
        children.append(_nodeNameTextItem);
    foreach(QGraphicsItem *child, children) {
        qreal childLeft = child->x();
        qreal childTop = child->y();
        QRectF childRect = child->boundingRect();
        qreal childRight = childLeft + childRect.width();
        qreal childBottom = childTop + childRect.height();
        if (childLeft < result.x())
            result.setX(childLeft);
        if (childTop < result.y())
            result.setY(childTop);
        if (childRight > result.right())
            result.setRight(childRight);
        if (childBottom > result.bottom())
            result.setBottom(childBottom);
    }
    return result;
}

void EntityGraphics::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
{
    QStyleOptionGraphicsItem *paintOption = new QStyleOptionGraphicsItem(*option);
    if (paintOption->state & QStyle::State_Selected) {
        paintOption->state &= !QStyle::State_Selected;
    }

    if (_isHighlighted != None) {
        QBrush hBrush;
        if(_isHighlighted == OK)
            hBrush = QBrush(QColor(154,205,50,100), Qt::SolidPattern);
        else{
            if(_isHighlighted == NOK)
                hBrush = QBrush(QColor(255, 0, 0, 150), Qt::SolidPattern);
            else
                hBrush = QBrush(QColor(255, 165, 0, 100), Qt::SolidPattern);
        }
        painter->translate(0, 0);
        painter->fillPath(shape(), hBrush);
        QPen hPen(Qt::DashLine);
        if(_isHighlighted == OK)
            hPen.setColor(QColor(154,205,50, 255));
        else{
            if(_isHighlighted == NOK)
                hPen.setColor(QColor(255, 0, 0, 255));
            else
                hPen.setColor(QColor(255, 165, 0, 255));
        }
        hPen.setJoinStyle(Qt::RoundJoin);
        painter->translate(0, 0);
        painter->strokePath(shape(), hPen);
    }

    QGraphicsRectItem::paint(painter, paintOption, widget);
    if (option->state & QStyle::State_Selected) {
        QPen outlinePen(Qt::DashLine);
        outlinePen.setColor(Qt::darkGray);
        painter->translate(_svgItem->pos());
        painter->strokePath(_svgItem->shape(), outlinePen);
    }
}

HnsClient::Entity *EntityGraphics::entity()
{
    return _entity;
}

qlonglong EntityGraphics::getEntityID() const
{
    return _entity->getId();
}

void EntityGraphics::refresh()
{
    setToolTip(Ressources::getToolTip(_entity));
    _updateShortname();
    _updateNodeName();
}

void EntityGraphics::checkIsLayoutChanged()
{
    Hns::EntityLayoutPtr newLayout = layout();
    if (newLayout != _previousLayout) {
        _previousLayout = newLayout;
        emit layoutChanged(newLayout);
    }
}

void EntityGraphics::setColorLabel(QColor color)
{
    if (color.isValid()) {
        if (_brightness(color) < 130) {
            _labelColor = Qt::white;
        } else {
            _labelColor = Qt::black;
        }
        if (_shortNameTextItem)
            _shortNameTextItem->setDefaultTextColor(_labelColor);
        if (_nodeNameTextItem)
            _nodeNameTextItem->setDefaultTextColor(_labelColor);
    }
}

qlonglong EntityGraphics::addSatellite(QGraphicsSvgItem *satelliteItem)
{
    qlonglong itemID = _satID++;
    satelliteItem->setParentItem(0);
    satelliteItem->setZValue(100);
    _satelliteItems.insert(itemID, satelliteItem);
    _drawSatellites();
    return itemID;
}

void EntityGraphics::removeSatellite(qlonglong itemID)
{
    if (_satelliteItems.contains(itemID)) {
        QGraphicsSvgItem *satellite = _satelliteItems.take(itemID);
        delete satellite;
    }
}

void EntityGraphics::clearSatellites()
{
    qDeleteAll(_satelliteItems);
    _satelliteItems.clear();
}

QGraphicsSvgItem *EntityGraphics::getSatellite(qlonglong itemID)
{
    QGraphicsSvgItem *satellite = 0;
    if (_satelliteItems.contains(itemID))
        satellite = _satelliteItems.value(itemID);
    return satellite;
}

void EntityGraphics::enableBlackEffect(bool enable)
{
    if (enable != _isGray) {
        _isGray = enable;
        _setCurrentSvg();
    }
}

void EntityGraphics::initGraphics()
{
    _initSvgFileNames();
    _registerCustomGraphic();
    _setCurrentSvg();
    _updateShortname();
    _scene->addItem(this);
    _updateScenePosition(_entity->getLayout()->x, _entity->getLayout()->y, false);
}

void EntityGraphics::displayProperties(QWidget *parent)
{
    HnsClient::EntityEntry *updatedEntry = qobject_cast<HnsClient::EntityEntry *>(_entity->getEntry()->duplicateEntry(this));
    if (updatedEntry) {
        EntryProperties popup(updatedEntry, parent, false, (_entity->getStatus() != Hns::UNDEFINED && _entity->getStatus() != Hns::UNIMPLEMENTED), false, false);
        HnvDialogHandler *popupHandler = new HnvDialogHandler(_entity, &popup, parent);
        popupHandler->setSourceDestroyedMessage(tr("\"%1\" was deleted. Operation aborted.").arg(_entity->getShortName()));
        if (popup.exec() == QDialog::Accepted) {
            foreach (WireGraphics *wire, wires()) {
                if (wire && !_temporaryWires.contains(wire->getEntityID()))
                    _temporaryWires.append(wire->getEntityID());
            }
            _entity->updateEntry(updatedEntry);
        }
    }
}

void EntityGraphics::addWire(WireGraphics *wire, GraphicsItem::Direction direction)
{
    _wireHash[direction].append(wire);
    _setWiresOrigin(direction);
}

void EntityGraphics::removeWire(WireGraphics * wire)
{
    foreach(GraphicsItem::Direction direction, _wireHash.keys()) {
        int index = _wireHash[direction].indexOf(wire);
        if (index != -1) {
            _wireHash[direction].removeAt(index);
            _setWiresOrigin(direction);
            return;
        }
    }
}

QList<WireGraphics*> EntityGraphics::wires(GraphicsItem::Direction direction)
{
    QList<WireGraphics*> wires;
    if (direction == GraphicsItem::Undefined)
        foreach (QList<WireGraphics*> wireList, _wireHash)
            foreach (WireGraphics* wire, wireList)
                wires.append(wire);
    else
        wires = _wireHash[direction];
    return wires;
}

bool EntityGraphics::setWireDirection(WireGraphics* wire, GraphicsItem::Direction newDirection)
{
    foreach(GraphicsItem::Direction direction, _wireHash.keys()) {
        if (_wireHash[direction].contains(wire)) {
            if(newDirection != direction) {
                _wireHash[newDirection].append(wire);
                _wireHash[direction].removeAll(wire);
                _setWiresOrigin(direction);
                _setWiresOrigin(newDirection);
                return true;
            } else {
                _setWiresOrigin(direction);
            }
        }
    }
    return false;
}

void EntityGraphics::redrawWires(GraphicsItem::Direction direction, WireGraphics *excludedWire)
{
    foreach(WireGraphics *wire, wires(direction))
        if (wire != excludedWire)
            wire->update();
}

QList<EntityActionHandler *> EntityGraphics::customActions(bool)
{
    return QList<EntityActionHandler *>();
}

void Hnv::EntityGraphics::applyLabelVisibility()
{
    switch (_entity->getLayout()->labelPolicyState) {
    case Hns::LabelPolicyDefault:
        _showLabels(_labelsDisplayedByDefault);
        break;
    case Hns::LabelPolicyHide:
        _showLabels(false);
        break;
    case Hns::LabelPolicyShow:
        _showLabels(true);
        break;
    }
}

void EntityGraphics::handleNearWire(bool stopMoving)
{
    QRectF rect(pos().x(), pos().y(), boundingRect().width(), boundingRect().height());
    foreach (WireGraphics *wire, _area->wireGraphics()) {
        bool nearTo = false;
        bool nearFrom = false;
        QList<Hns::TopologyLink> links = _area->topology()->getLinksByWireId(wire->getEntityID());
        Hns::TopologyLink toLink;
        Hns::TopologyLink fromLink;
        foreach (const Hns::TopologyLink &link, links) {
            int wirePort = QString::fromStdString(link.wireSide).split(':').last().toInt();
            if (wirePort == 0)
                fromLink = link;
            else if (wirePort == 1)
                toLink = link;
        }
        if (rect.contains(wire->toPoint()) && !wire->to() && QString::fromStdString(fromLink.entitySide).split(':').first().toLongLong() != getEntityID()) {
            nearTo = true;
        }
        else if (rect.contains(wire->fromPoint()) && !wire->from() && QString::fromStdString(toLink.entitySide).split(':').first().toLongLong() != getEntityID()) {
            nearFrom = true;
        }
        nearTo &= (!_entity->getAvailablePlugs().isEmpty() & !stopMoving);
        nearFrom &= (!_entity->getAvailablePlugs().isEmpty() & !stopMoving);
        wire->setNearToEntity(nearTo);
        wire->setNearFromEntity(nearFrom);
    }
}

/*******************************************************************************
* EntityGrapics - Public slots
*/

void Hnv::EntityGraphics::setLabelsDisplayedByDefault(bool labelsDisplayedByDefault)
{
    _labelsDisplayedByDefault = labelsDisplayedByDefault;
    if (_entity->getLayout()->labelPolicyState == Hns::LabelPolicyDefault)
        _showLabels(_labelsDisplayedByDefault);
}

void EntityGraphics::setStatusIconVisibility(bool visible)
{
    if (_statusIconDisplayed != visible) {
        _statusIconDisplayed = visible;
        _statusIcon->setVisible(_statusIconDisplayed);
        if (_nodeNameTextItem)
            _nodeNameTextItem->setVisible(_statusIconDisplayed);
        if (_captureIcon)
            _captureIcon->setVisible(_statusIconDisplayed);
        if (_immutableIcon)
            _immutableIcon->setVisible(_statusIconDisplayed);
        if (_usbIcon)
            _usbIcon->setVisible(_statusIconDisplayed);
    }
}

/*******************************************************************************
* EntityGrapics - Protected slots
*/
void EntityGraphics::_repaintNeeded()
{
    _scene->update(this->x(), this->y(), this->boundingRect().width(), this->boundingRect().height());
}

void EntityGraphics::_layoutChangedSlot()
{
    updateLayout(_entity->getLayout());
}

void EntityGraphics::_entryChangedSlot(HnsClient::GenericEntity *entity)
{
    QString customGraphic = _entity->getEntry()->customGraphic();
    if (customGraphic != _customGraphicName) {
        _registerCustomGraphic();
        _setCurrentSvg();
    }
    enableBlackEffect(entity->isModel());
    _updateImmutableIcon();
}

void EntityGraphics::_statusChangedSlot(HnsClient::GenericEntity *entity)
{
    _setStatus(entity->getStatus());
}

void EntityGraphics::_refreshedSlot()
{
    refresh();
}

void EntityGraphics::_updateCaptureIcon()
{
    bool captureJobRunning = _entity->getCaptureJobHash().size() ? true : false;
    QGraphicsSvgItem *captureItem = NULL;
    if (captureJobRunning) {
        captureItem = new QGraphicsSvgItem(Ressources::themedPath(Constants::GRAPHIC_CAPTURE), this);
        captureItem->setX(_svgItem->boundingRect().width());
    }
    if (_captureIcon) {
        delete _captureIcon;
    }
    _captureIcon = captureItem;
    if (_captureIcon) {
        _captureIcon->setVisible(_statusIconDisplayed);
    }
}

void EntityGraphics::_entityDestroyedSlot()
{
    delete this;
}

/*******************************************************************************
* EntityGrapics - Protected
*/
void EntityGraphics::dragEnterEvent(QGraphicsSceneDragDropEvent *event)
{
    if (!_entity->isModel()) return;
    const QMimeData *data = event->mimeData();
    QString mimeType;
    if(TopologyView::canHandleMimeData(data,mimeType ) ){
        if (mimeType != "hyneview/catalog-entry") {
            event->ignore();
            return;
        }
        QByteArray itemData = data->data(mimeType);
        QDataStream dataStream(&itemData, QIODevice::ReadOnly);

        int itemType;
        qlonglong sourceID;
        qlonglong sourceParentID;
        int itemInternalID;
        dataStream >> itemType >> sourceID >> sourceParentID >> itemInternalID;
        if(dataStream.atEnd() &&  itemType ==  EntityCatalogModel::Entity) {
            HnsClient::EntityEntry *entry = HnsClient::Session::instance()->entityEntryCatalog()->entityEntryById(sourceID);
            QList<QPair<Hns::ImplementModelResult, QString> > errorList;
            if(entry != NULL && !entry->isModel()){
                enableBlackEffect(false);
                Hns::ImplementModelResult result =  Hns::canImplementModel(_entity->getEntry()->entityEntry(), entry->entityEntry(), errorList, false);
                if (result == Hns::OK)
                    _isHighlighted = OK;
                else {
                    if (_errorItem)
                        delete _errorItem;
                    _errorItem = new ImplementErrorItem(errorList);
                    scene()->addItem(_errorItem);
                    ZoomingView  * zoomview = qobject_cast<ZoomingView *> (scene()->parent());
                    if (zoomview)
                        _errorItem->setScale((double)1/zoomview->currentZoomValue()*100);
                    _errorItem->setPos(pos() + boundingRect().center() - _errorItem->mapToParent(_errorItem->boundingRect().center()));
                    if (result == Hns::ERROR)
                        _isHighlighted = NOK;
                    else
                        _isHighlighted = WARNING;
                }
            }
        }
    }
}

void EntityGraphics::dragLeaveEvent(QGraphicsSceneDragDropEvent *event)
{
    event->accept();
    if (_entity->isModel()) {
        _isHighlighted = None;
        enableBlackEffect();
    }
    if (_errorItem) {
        delete _errorItem;
        _errorItem = NULL;
    }
}

void EntityGraphics::dropEvent(QGraphicsSceneDragDropEvent *event)
{
    Q_UNUSED(event)
    if (_entity->isModel()) {
        enableBlackEffect(false);
        _isHighlighted = None;
    }
    if (_errorItem) {
        delete _errorItem;
        _errorItem = NULL;
    }
}

void EntityGraphics::mouseDoubleClickEvent(QGraphicsSceneMouseEvent *)
{
//    TODO: Need to create an option which allow different behaviour on double click
//    QWidget *parent = 0;
//    QList<QGraphicsView *> views = _scene->views();
//    if (views.count() > 0)
//        parent = views.first();
//    _entity->displayProperties(parent);
    RemoteAccess::RemoteManager::instance()->newConnection(_entity);
}

QVariant EntityGraphics::itemChange(GraphicsItemChange change, const QVariant &value)
{
    switch (change) {
    case ItemPositionChange: {
        QPointF newPos = value.toPointF();
        newPos.setX(qRound(newPos.x()/10.0)*10);
        newPos.setY(qRound(newPos.y()/10.0)*10);
        _drawSatellites();
        return _validatePosition(newPos);
    }
    case ItemPositionHasChanged: {
        foreach(Direction dir, _wireHash.keys()) {
            foreach (WireGraphics * wire, _wireHash[dir]) {
                wire->updatePosition();
            }
            _setWiresOrigin(dir);
            redrawWires(dir);
            _drawSatellites();
        }
        break;
    }
    case ItemSelectedChange: {
        break;
    }
    default:
        break;
    }
    return QGraphicsRectItem::itemChange(change, value);
}

void EntityGraphics::_setCurrentSvg()
{
    if (!_customGraphicName.isEmpty())
        _setCurrentSvgFileName(_customGraphicName);
    else if (_isGray)
        _setCurrentSvgFileName(_svgFileNameGray);
    else
        _setCurrentSvgFileName(_svgFileName);
}

void EntityGraphics::_registerCustomGraphic()
{
    _customGraphicName = _entity->getEntry()->customGraphic();
    if (!_customGraphicName.isEmpty()) {
        QByteArray data = HyneviewSession::instance()->resourceManager()->getResource(_customGraphicName);
        if(data.isEmpty()) {
            _customGraphicName = "";
            return;
        }
        QSvgRenderer *renderer = new QSvgRenderer(data);
        if (!Ressources::registerSvgRenderer(_customGraphicName, renderer))
            renderer->deleteLater();
    }
}

void EntityGraphics::_initSvgFileNames()
{
    _setSvgFileNames(Ressources::themedPath(Constants::GRAPHIC_UNKNOWN_ENTITY), Ressources::themedPath(Constants::GRAPHIC_UNKNOWN_ENTITY));
}

void EntityGraphics::_setSvgFileNames(const QString &normal, const QString &gray)
{
    _svgFileName = normal;
    _svgFileNameGray = gray;
    Ressources::registerSvgRenderer(_svgFileName);
    Ressources::registerSvgRenderer(_svgFileNameGray);
}

void EntityGraphics::_setCurrentSvgFileName(const QString &fileName)
{
    QSvgRenderer *renderer = Ressources::getRenderer(fileName);
    if (!renderer)
        return;
    if (!_svgItem)
        _svgItem = new QGraphicsSvgItem(this);
    _svgItem->setSharedRenderer(renderer);
    _svgItem->setCachingEnabled(true);
    connect(renderer, SIGNAL(repaintNeeded()), this, SLOT(_repaintNeeded()));
    if (_statusIcon)
        _svgItem->setPos(_statusIcon->boundingRect().center());
    _updateScenePosition(pos().toPoint()); // used to validate position based on new SVG
    _updateNodeName();
    _updateCaptureIcon();
    _updateImmutableIcon();
}

//QString EntityGraphics::_getToolTip()
//{
//    QString toolTip = tr("<b>Name:&nbsp;</b><i>%1</i>").arg(entity()->getShortName());
//    toolTip += tr("<br/><b>Available plugs:&nbsp;</b><i>%1/%2</i>")
//            .arg(QString::number(entity()->getAvailablePlugs().count()))
//            .arg(QString::number(entity()->getPlugs().count()));
//    QString nodeName = entity()->getNodeName();
//    if (!nodeName.isEmpty()) {
//        HnsClient::NodeInformation *nodeInfo = HyneviewSession::instance()->platformMonitor()->getNodeInformation(nodeName);
//        if (nodeInfo)
//            nodeName = nodeInfo->getShortName();
//        toolTip += tr("<br/><b>Node:&nbsp;</b><i>%1</i>").arg(nodeName);
//    }
//    return toolTip;
//}

void EntityGraphics::_setStatus(Hns::Status status)
{
    QGraphicsSvgItem *statusItem = Ressources::getStatusSVGItem(status, this);
    if (statusItem) {
        if (_svgItem)
            statusItem->setZValue(_svgItem->zValue() - 1);
        if (_statusIcon)
            delete _statusIcon;
        _statusIcon = statusItem;
        _statusIcon->setVisible(_statusIconDisplayed);
        if (_nodeNameTextItem)
            _nodeNameTextItem->setVisible(_statusIconDisplayed);
    }
    if (status == Hns::UNIMPLEMENTED)
        enableBlackEffect();
    _updateNodeName();
}

void EntityGraphics::_updateScenePosition(QPoint scenePosition, bool propagate)
{
    setPos(_validatePosition(QPointF(scenePosition)).toPoint());
    if (propagate)
        checkIsLayoutChanged();
    else
        _previousLayout = layout();
}

void EntityGraphics::_updateScenePosition(int posX, int posY, bool propagate)
{
    _updateScenePosition(QPoint(posX, posY), propagate);
}

void EntityGraphics::_updateShortname()
{
    QPointF textPos = pos();
    QString shortName = _entity->getShortName();

    RemoteAccess::RemoteManager::instance()->updateShortname(_entity->getId(), shortName);

    QSettings settings;
    Qt::TextElideMode elideMode;
    settings.beginGroup(Constants::TOPOLOGY_SETTINGS);
    switch (settings.value(Constants::ELIDE_MODE_KEY,0).toInt()) {
    case 0:
        elideMode = Qt::ElideLeft;
        break;
    case 1:
        elideMode = Qt::ElideRight;
        break;
    case 2:
        elideMode = Qt::ElideMiddle;
        break;
    case 3:
        elideMode = Qt::ElideNone;
        break;
    default:
        elideMode = Qt::ElideLeft;
        break;
    }
    settings.endGroup();
    QFontMetrics fontMetrics(_scene->font());
    shortName = fontMetrics.elidedText(shortName, elideMode, _svgItem->boundingRect().width() * 1.5);
    if (!_shortNameTextItem) {
        _shortNameTextItem = new QGraphicsTextItem(this);
        _shortNameTextItem->setDefaultTextColor(_labelColor);
    }
    if(_shortNameTextItem)
        _shortNameTextItem->setHtml(shortName);
    if (_svgItem) {
        _shortNameTextItem->setZValue(_svgItem->zValue() + 2);
        textPos = _svgItem->pos() + QPointF((_svgItem->boundingRect().width() / 2) - (_shortNameTextItem->boundingRect().width() / 2), _svgItem->boundingRect().height());
    }
    _shortNameTextItem->setPos(textPos);
}

void EntityGraphics::_updateNodeName()
{
    QPointF textPos = pos();
    QString nodeName = _entity->getNodeName();
    if(nodeName.isEmpty()){
        if (_nodeNameTextItem) {
            _nodeNameTextItem->deleteLater();
            _nodeNameTextItem = NULL;
        }
        return;
    }

    if (!_nodeNameTextItem) {
        _nodeNameTextItem = new QGraphicsTextItem(this);
        _nodeNameTextItem->setDefaultTextColor(_labelColor);
    }
    QFont font = _nodeNameTextItem->font();
    font.setItalic(true);
    font.setPixelSize(10);
    _nodeNameTextItem->setFont(font);

    QString displayName = nodeName;
    HnsClient::NodeInformation * nodeInfo = HyneviewSession::instance()->platformMonitor()->getNodeInformation(nodeName);
    if (nodeInfo)
        displayName = nodeInfo->getShortName();

    _nodeNameTextItem->setHtml(displayName);
    if (_statusIcon && _svgItem) {
        _nodeNameTextItem->setZValue(_svgItem->zValue()+1);
        textPos.setX(_statusIcon->boundingRect().right()+1);
        textPos.setY(((_statusIcon->boundingRect().top()*3)+_statusIcon->boundingRect().center().y())/4);
    }
    _nodeNameTextItem->setPos(textPos);
    _nodeNameTextItem->setVisible(_statusIconDisplayed);
}

void EntityGraphics::_updateImmutableIcon()
{
    HnsClient::DomainEntry *domainEntry = qobject_cast<HnsClient::DomainEntry *>(_entity->getEntry());
    if (!domainEntry)
        return;
    bool isImmutable = domainEntry->getImmutable();

    QGraphicsSvgItem *immutableItem = NULL;
    if (isImmutable) {
        immutableItem = new QGraphicsSvgItem(Ressources::themedPath(Constants::GRAPHIC_IMMUTABLE), this);
        immutableItem->setX(_svgItem->boundingRect().width() - immutableItem->boundingRect().width());
    }
    if (_immutableIcon) {
        delete _immutableIcon;
    }
    _immutableIcon = immutableItem;
    if (_immutableIcon) {
        _immutableIcon->setVisible(_statusIconDisplayed);
    }
}


Hns::EntityLayoutPtr EntityGraphics::layout()
{
    Hns::EntityLayoutPtr layout = new Hns::EntityLayout();
    layout->id = _entity->getId();
    layout->x = scenePos().x();
    layout->y = scenePos().y();
    layout->labelPolicyState = _entity->getLayout()->labelPolicyState;
    return layout;
}

void EntityGraphics::updateLayout(Hns::EntityLayoutPtr layout)
{
    if (layout->id != _entity->getId())
        return;
    QPointF layoutPos(layout->x, layout->y);
    if (layoutPos != scenePos()) {
        setPos(layoutPos);
    }
    applyLabelVisibility();
}

QPointF EntityGraphics::_validatePosition(QPointF position)
{
    return validateItemPosition(position, boundingRect(), scene());
}

QPointF EntityGraphics::validateItemPosition(QPointF position, QRectF itemRect, QGraphicsScene *scene)
{
    if (scene) {
        QRectF sceneRect = scene->sceneRect();
        qreal minX = sceneRect.x() - itemRect.x();
        qreal minY = sceneRect.y() - itemRect.y();
        qreal maxX = sceneRect.x() + sceneRect.width() - itemRect.width();
        qreal maxY = sceneRect.y() + sceneRect.height() - itemRect.height();
        if (position.x() > maxX)
            position.setX(maxX);
        if (position.y() > maxY)
            position.setY(maxY);
        if (position.x() < minX)
            position.setX(minX);
        if (position.y() < minY)
            position.setY(minY);
    }
    return position;
}

void EntityGraphics::setIsHighlighted(EntityGraphics::Highlighted isHighlighted)
{
    _isHighlighted = isHighlighted;
}

GraphicsItem::Direction EntityGraphics::getDirection(WireGraphics *wire)
{
    QHashIterator<Direction, QList<WireGraphics *> > hashIterator(_wireHash);
    while (hashIterator.hasNext()) {
        hashIterator.next();
        QList<WireGraphics *> list = hashIterator.value();
        if (list.contains(wire))
            return hashIterator.key();
    }
    return GraphicsItem::Undefined;
}

bool EntityGraphics::handleTemporaryWires(qlonglong wireId)
{
    if (!_temporaryWires.contains(wireId))
        return false;
    _temporaryWires.removeOne(wireId);
    return true;
}

void EntityGraphics::showUsbIcon(bool visible)
{
    QGraphicsSvgItem *usbItem = 0;
    if (visible) {
        usbItem = new QGraphicsSvgItem(Ressources::themedPath((Constants::GRAPHIC_USB)), this);
        usbItem->setX(_svgItem->boundingRect().width());
        usbItem->setY(_svgItem->boundingRect().height());
    }
    if (_usbIcon)
        delete _usbIcon;
    _usbIcon = usbItem;
    if (_usbIcon)
        _usbIcon->setVisible(_statusIconDisplayed);
}

void EntityGraphics::_drawSatellites()
{
    if (!_svgItem || _satelliteItems.isEmpty())
        return;

    QRectF rect = _svgItem->boundingRect();
    QRectF satelliteRect = _satelliteItems.value(_satelliteItems.keys().first())->boundingRect();

    QPointF offset = _svgItem->pos() + rect.center();
    // vector offset to use the center of items as the pos, not the top left corner
    offset -= satelliteRect.center();
    // radius offset beetween each circle
    int radiusOffset = qMax(satelliteRect.width(), satelliteRect.height()) + 25;// TODO: need to use settings/constants
    // base radius
    int radius = (qMax(rect.width(), rect.height()) / 2) + radiusOffset;
    // starting angle
    double origAngle = 3 * M_PI / 2;

    QList<QGraphicsSvgItem *> satelliteItems = _satelliteItems.values();

    while(!satelliteItems.isEmpty()) {
        QList<QPointF> validPosition;
        double angleIncrement = 50.0 / radius;      // equiv: 2*pi / (perimeter / 50.0);
        int maxCount = (2 * M_PI / angleIncrement); // calculate maximum item for this circle
        angleIncrement = (2 * M_PI / maxCount);     // calculate more accurate angle increment for this circle

        for(int posIndex=0; posIndex < maxCount; posIndex++) {
            double itemAngle = (origAngle + posIndex * angleIncrement);
            QPointF itemPos(radius * qCos(itemAngle), radius * qSin(itemAngle));
            itemPos += offset;
            QPointF scenePos = mapToScene(itemPos);
            if (validateItemPosition(scenePos, satelliteRect, scene()) == scenePos)
                validPosition.append(itemPos);
        }

        // If there is no valid position on this circle, no more satellite can be displayed
        if (validPosition.count() == 0)
            break;

        QMutableListIterator<QGraphicsSvgItem *> iterator(satelliteItems);
        iterator.toBack();
        bool invert = false;
        while (iterator.hasPrevious() && !validPosition.isEmpty()) {
            iterator.previous();
            QPointF targetPos = (invert ? validPosition.takeLast() : validPosition.takeFirst());
            iterator.value()->setPos(targetPos + pos());
            iterator.remove();
            invert = !invert;
        }
        radius += radiusOffset;
    }
}

void EntityGraphics::_setWiresOrigin(GraphicsItem::Direction direction)
{
    QSettings settings;
    settings.beginGroup(Constants::WIRES_SETTINGS);
    int originMode = settings.value(Constants::WIRE_ORIGIN_MODE, QVariant(0)).toInt();
    // 0=center, 1=automated spacing based on boundingRect, 2=fixed size spacing

    _sortWiresInDirection(direction);
    QPointF origin = boundingRect().center();

    if (_svgItem)
        origin = _svgItem->pos() + _svgItem->boundingRect().center();
    qreal space = 0;
    qreal offset = 0;
    int nbWires = _wireHash[direction].size();

    switch(originMode) {
    case 1: {
        if (direction == EntityGraphics::North || direction == EntityGraphics::South)
            space = boundingRect().width()/(nbWires+1);
        else
            space = boundingRect().height()/(nbWires+1);
        break;
    }
    case 2: {
            space = Constants::WIRE_SPACING;
        do {
            int delta = (space * nbWires) / 2;
            if (direction == EntityGraphics::North || direction == EntityGraphics::South)
                offset = origin.x() - delta;
            else
                offset = origin.y() - delta;
            space--;
        } while (offset < 0 && space > 0);
        break;
    }
    }
    for(int index = 0 ; index < nbWires ; index++) {
        if (originMode != 0) {
            if (direction == EntityGraphics::North || direction == EntityGraphics::South)
                origin.setX(offset + (index + 1) * space);
            else
                origin.setY(offset + (index + 1) * space);
        }
        _wireHash[direction][index]->setOrigin(this, mapToScene(origin));
    }
}

void EntityGraphics::_sortWiresInDirection(GraphicsItem::Direction direction)
{
    typedef QMap<qreal, QList<WireGraphics*> > realGraphicsMap;
    QMap <qreal, realGraphicsMap > wireMap;
    foreach (WireGraphics* wire, _wireHash[direction]) {
        QPointF otherEnd = wire->getOtherEndPoint(this);
        qreal mainKey(0);
        qreal secondKey(0);
        switch (direction) {
        case North:
            if (x() < otherEnd.x())
                mainKey = otherEnd.y();
            else
                mainKey = -otherEnd.y();
            secondKey = otherEnd.x();
            break;
        case South:
            if (x() < otherEnd.x())
                mainKey = scene()->height()*2 - otherEnd.y();
            else
                mainKey = otherEnd.y();
            secondKey = otherEnd.x();
            break;
        case East:
            if (y() < otherEnd.y())
                mainKey = scene()->width()*2 - otherEnd.x();
            else
                mainKey = otherEnd.x();
            secondKey = otherEnd.y();
            break;
        case West:
            if (y() < otherEnd.y())
                mainKey = otherEnd.x();
            else
                mainKey = -otherEnd.x();
            secondKey = otherEnd.y();
            break;
        default:
            break;
        }
        wireMap[mainKey][secondKey].append(wire);
    }

    QList<WireGraphics*> wireList;
    foreach (realGraphicsMap map, wireMap)
        foreach (QList<WireGraphics*> wl, map)
            wireList.append(wl);
    _wireHash[direction] = wireList;
}

void EntityGraphics::_showLabels(bool show)
{
    _shortNameTextItem->setVisible(show);
}

/*******************************************************************************
* DomainEntityGraphics
*******************************************************************************/
DomainEntityGraphics::DomainEntityGraphics(HnsClient::Entity *entity, TopologyScene *scene, TopologyViewArea *area, QGraphicsItem *parent)
    : EntityGraphics(entity, scene, area, parent)
{
}

void DomainEntityGraphics::_initSvgFileNames()
{
    _setSvgFileNames(Ressources::themedPath(Constants::GRAPHIC_DESKTOP), Ressources::themedPath(Constants::GRAPHIC_DESKTOP_GRAY));
}

/*******************************************************************************
* SwitchEntityGraphics
*******************************************************************************/
SwitchEntityGraphics::SwitchEntityGraphics(HnsClient::Entity *entity, TopologyScene *scene, TopologyViewArea *area, QGraphicsItem *parent)
    : EntityGraphics(entity, scene, area, parent)
{
}

void SwitchEntityGraphics::_initSvgFileNames()
{
    _setSvgFileNames(Ressources::themedPath(Constants::GRAPHIC_SWITCH), Ressources::themedPath(Constants::GRAPHIC_SWITCH_GRAY));
}


/*******************************************************************************
* HybridEntityGraphics
*******************************************************************************/
HybridNetcardEntityGraphics::HybridNetcardEntityGraphics(HnsClient::Entity *entity, TopologyScene *scene, TopologyViewArea *area, QGraphicsItem *parent)
    : EntityGraphics(entity, scene, area, parent)
{
    HnsClient::HybridNetcardEntity *hybrid = qobject_cast<HnsClient::HybridNetcardEntity *>(_entity);
    if (hybrid) {
        connect(hybrid, SIGNAL(macAddressAdded(QString)), this, SLOT(_macAddressAddedSlot(QString)));
        connect(hybrid, SIGNAL(macAddressRemoved(QString)), this, SLOT(_macAddressRemovedSlot(QString)));
        connect(hybrid, SIGNAL(macAddressChanged(QString,bool)), this, SLOT(_macAddressChangedSlot(QString,bool)));
        QMapIterator<QString, bool> iterator(hybrid->getMacAddresses());
        while(iterator.hasNext()) {
            iterator.next();
            _addSatellite(iterator.key(), iterator.value());
        }
    }
}

void HybridNetcardEntityGraphics::_macAddressAddedSlot(QString macAddress)
{
    _addSatellite(macAddress, true);
}

void HybridNetcardEntityGraphics::_macAddressRemovedSlot(QString macAddress)
{
    if (_satelliteMap.contains(macAddress))
        removeSatellite(_satelliteMap.take(macAddress));
}

void HybridNetcardEntityGraphics::_macAddressChangedSlot(QString macAddress, bool active)
{
    if (_satelliteMap.contains(macAddress))
        _updateSatellite(_satelliteMap.value(macAddress), active);
    else
        _addSatellite(macAddress, active);
}

void HybridNetcardEntityGraphics::_initSvgFileNames()
{
    _setSvgFileNames(Ressources::themedPath(Constants::GRAPHIC_HYBRID), Ressources::themedPath(Constants::GRAPHIC_HYBRID_GRAY));
}

void HybridNetcardEntityGraphics::_setSatelliteRenderer(QGraphicsSvgItem *item, bool active)
{
    if (active)
        item->setSharedRenderer(Ressources::getRenderer(Ressources::themedPath(Constants::GRAPHIC_INFO_SATELLITE_ACTIVE)));
    else
        item->setSharedRenderer(Ressources::getRenderer(Ressources::themedPath(Constants::GRAPHIC_INFO_SATELLITE_INACTIVE)));
}

void HybridNetcardEntityGraphics::_addSatellite(QString macAddress, bool active)
{
    if (!_satelliteMap.contains(macAddress))
        _satelliteMap.insert(macAddress, _createSatellite(macAddress, active));
}

qlonglong HybridNetcardEntityGraphics::_createSatellite(QString macAddress, bool active)
{
    QGraphicsSvgItem *item = new QGraphicsSvgItem(this);
    _setSatelliteRenderer(item, active);
    item->setToolTip(tr("MAC address: %1").arg(macAddress));
    return addSatellite(item);
}

void HybridNetcardEntityGraphics::_updateSatellite(qlonglong satelliteID, bool active)
{
    if (_satelliteItems.contains(satelliteID))
        _setSatelliteRenderer(_satelliteItems[satelliteID], active);
}


/*******************************************************************************
* WifiAPEntityGraphics
*******************************************************************************/
WifiAPEntityGraphics::WifiAPEntityGraphics(HnsClient::Entity *entity, TopologyScene *scene, TopologyViewArea *area, QGraphicsItem *parent)
    : EntityGraphics(entity, scene, area, parent)
{
    HnsClient::WifiAPEntity *wifi = qobject_cast<HnsClient::WifiAPEntity *>(_entity);
    if (wifi) {
        connect(wifi, SIGNAL(peerJoined(QString)), this, SLOT(_peerJoinedSlot(QString)));
        connect(wifi, SIGNAL(peerLeft(QString)), this, SLOT(_peerLeftSlot(QString)));
        foreach(const QString &peer, wifi->connectedPeers()) {
            _addSatellite(peer);
        }
    }
}

qlonglong WifiAPEntityGraphics::createSatellite(QString macAddress)
{
    QGraphicsSvgItem *item = new QGraphicsSvgItem(this);
    item->setSharedRenderer(Ressources::getRenderer(Ressources::themedPath(Constants::GRAPHIC_INFO_SATELLITE_ACTIVE)));
    item->setToolTip(tr("MAC address: %1").arg(macAddress));
    return addSatellite(item);
}

QList<EntityActionHandler *> WifiAPEntityGraphics::customActions(bool limited)
{
    if(limited)
        return QList<EntityActionHandler *>();
    QList<EntityActionHandler *> handlerList;
    QString uniqueID = staticMetaObject.className();
    uniqueID += WifiAPEntityGraphics::connectedPeersAction;
    handlerList.append(
                new EntityActionHandler(
                    uniqueID,
                    QIcon(Ressources::themedPath(Constants::ICON_CONNECTED_PEERS)),
                    tr("Display Connected Peers"), tr("Display Connected Peers"), QKeySequence(), this));
    connect(handlerList.last(), SIGNAL(actionTriggered()), this, SLOT(_displayConnectedPeersSlots()));
    return handlerList;
}

void WifiAPEntityGraphics::_displayConnectedPeersSlots()
{
    QWidget * parentWidget = 0;
    if (!_scene->views().isEmpty())
        parentWidget = _scene->views().first();
    HnsClient::WifiAPEntity *wifiAPEntity = qobject_cast<HnsClient::WifiAPEntity *>(_entity);
    if (!wifiAPEntity)
        return;
    WifiAPConnectedPeersWidget *popup = new WifiAPConnectedPeersWidget(wifiAPEntity, parentWidget);
    HnvDialogHandler * popupHandler = new HnvDialogHandler(this, popup, parentWidget);
    popupHandler->setSourceDestroyedMessage(tr("\"%1\" was deleted. Operation aborted.").arg(_entity->getShortName()));
    popup->show();
}

void WifiAPEntityGraphics::_peerJoinedSlot(QString peer)
{
    _addSatellite(peer);
}

void WifiAPEntityGraphics::_peerLeftSlot(QString peer)
{
    _removeSatellite(peer);
}

void WifiAPEntityGraphics::_initSvgFileNames()
{
    _setSvgFileNames(Ressources::themedPath(Constants::GRAPHIC_WIFIAP), Ressources::themedPath(Constants::GRAPHIC_WIFIAP_GRAY));
}

void WifiAPEntityGraphics::_addSatellite(QString peer)
{
    if (!_satelliteMap.contains(peer)) {
        _satelliteMap.insert(peer, createSatellite(peer));
    }
}

void WifiAPEntityGraphics::_removeSatellite(QString peer)
{
    if (_satelliteMap.contains(peer))
        removeSatellite(_satelliteMap.take(peer));
}


/*******************************************************************************
* DynamipsRouterEntityGraphics
*******************************************************************************/
DynamipsRouterEntityGraphics::DynamipsRouterEntityGraphics(HnsClient::Entity *entity, TopologyScene *scene, TopologyViewArea *area, QGraphicsItem *parent)
    : EntityGraphics(entity, scene, area, parent)
{
}

QList<EntityActionHandler *> DynamipsRouterEntityGraphics::customActions(bool limited)
{
    if(limited)
        return QList<EntityActionHandler *>();
    QList<EntityActionHandler *> handlerList;
    QString uniqueID = staticMetaObject.className();
    handlerList.append(
                new EntityActionHandler(
                    uniqueID + DynamipsRouterEntityGraphics::generateIdlePcAction,
                    QIcon(Ressources::themedPath(Constants::GRAPHIC_DYNAMIPSROUTER)),
                    tr("Generate idlepc"), tr("Search best value for idle pc"), QKeySequence(), this));
    connect(handlerList.last(), SIGNAL(actionTriggered()), this, SLOT(_generateIdle()));
    handlerList.last()->setEnableAtStart(true);
    return handlerList;
}

void DynamipsRouterEntityGraphics::_generateIdle()
{
    HnsClient::DynamipsRouterEntity *dynamipsEntity = qobject_cast<HnsClient::DynamipsRouterEntity*>(_entity);
    if (!dynamipsEntity)
        return;
    dynamipsEntity->fixIdle(this, SLOT(_generateIdleSlot(::std::string)));
    QWidget * parentWidget = 0;
    if (!_scene->views().isEmpty())
        parentWidget = _scene->views().first();

    _popup = new DynamipsRouterLoadIdle(tr("Wait while searching idlepc") ,parentWidget);
    _popup->setModal(true);
    _popup->show();
}

void DynamipsRouterEntityGraphics::_generateIdleSlot(std::string idleValue)
{
    if(idleValue.empty()){
        _popup->setEnabledErrorMode(tr("Can't find a valid idlepc value"));
        _popup->exec();
    }
    else{
        _popup->hide();
    }
    delete _popup;
    _popup = NULL;
}

void DynamipsRouterEntityGraphics::_initSvgFileNames()
{
    _setSvgFileNames(Ressources::themedPath(Constants::GRAPHIC_DYNAMIPSROUTER), Ressources::themedPath(Constants::GRAPHIC_DYNAMIPSROUTER_GRAY));
}


/*******************************************************************************
* ImplementErrorItem
*******************************************************************************/
ImplementErrorItem::ImplementErrorItem(const QList<QPair<Hns::ImplementModelResult, QString> > & errorList)
    : QAbstractGraphicsShapeItem(0)
    , _errorList(errorList)
    , _fontScaleFactor(1.3)
    , _errorPixmap(QPixmap(Ressources::themedPath(Constants::ICON_ERROR)))
    , _warningPixmap(QPixmap(Ressources::themedPath(Constants::ICON_WARNING)))
    , _iconsHeight(0)
{
    _iconHeight = qMax(_errorPixmap.height(), _warningPixmap.height());
    _iconWidth = qMax(_errorPixmap.width(), _warningPixmap.width());
    _iconsHeight = _errorList.count() * _iconHeight;
}

ImplementErrorItem::~ImplementErrorItem()
{
}

QRectF ImplementErrorItem::boundingRect() const
{
    QFont font = QApplication::font();
    font.setPointSize(double(font.pointSize())*_fontScaleFactor);
    QFontMetrics fm(font);
    int height = qMax(_errorList.size() * fm.lineSpacing(), _iconsHeight);
    int width = 0;
    QListIterator<QPair<Hns::ImplementModelResult, QString> > errorIterator(_errorList);
    while(errorIterator.hasNext()) {
        QPair<Hns::ImplementModelResult, QString> error = errorIterator.next();
        width = qMax(width, fm.width(error.second));
    }
    width += _iconWidth;
    return QRectF(0, 0, width + 4, height + 4);
}

void ImplementErrorItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget)
{
    Q_UNUSED(option)
    Q_UNUSED(widget)
    QRectF rect = boundingRect();

    painter->save();
    painter->setBrush(Qt::white);
    painter->drawRoundedRect(rect, 5, 5);
    painter->restore();

    QFont f = painter->font();
    f.setPointSize(double(f.pointSize())*_fontScaleFactor);
    painter->setFont(f);
    QFontMetrics fm = painter->fontMetrics();
    int y = 0;
    int iconOffset = 1;
    int textOffset = 0;
    int lineHeight = fm.lineSpacing();
    if (_iconHeight > lineHeight) {
        textOffset = (lineHeight - _iconHeight) / 2;
        lineHeight = _iconHeight;
    } else {
        iconOffset = (_iconHeight - lineHeight) / 2;
    }

    QListIterator<QPair<Hns::ImplementModelResult, QString> > errorIterator(_errorList);
    while(errorIterator.hasNext()) {
        QPair<Hns::ImplementModelResult, QString> error = errorIterator.next();
        if (error.first == Hns::ERROR)
            painter->drawPixmap(1, y + iconOffset, _errorPixmap);
        else
            painter->drawPixmap(1, y + iconOffset, _warningPixmap);
        y += lineHeight;
        painter->drawText(2 + _iconWidth, y + textOffset, error.second);
    }
}

/*******************************************************************************
* TopologyBridgeEntityGraphics
*******************************************************************************/
TopologyBridgeEntityGraphics::TopologyBridgeEntityGraphics(HnsClient::Entity *entity, TopologyScene *scene, TopologyViewArea *area, QGraphicsItem *parent)
    : EntityGraphics(entity, scene, area, parent)
{
}

void TopologyBridgeEntityGraphics::_initSvgFileNames()
{
    _setSvgFileNames(Ressources::themedPath(Constants::GRAPHIC_TOPOLOGY_GATE), Ressources::themedPath(Constants::GRAPHIC_TOPOLOGY_GATE_GRAY));
}

} // namespace Hnv
