/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef ENTITYGRAPHICS_H
#define ENTITYGRAPHICS_H

// hynesim common headers
#include "common/canimplementmodel.h"

// Generated hynesim headers
#include "common/layout.h"
#include "common/entity.h"

// Qt headers
#include <QAbstractGraphicsShapeItem>
#include <QGraphicsRectItem>

class QGraphicsSvgItem;
class QGraphicsTextItem;

namespace HnsClient {
class GenericEntity;
class Entity;
class WireEntity;
} // namespace HnsClient

namespace Hnv {
class TopologyScene;
class TopologyView;
class EntityGraphics;
class ImplementErrorItem;
class EntityActionHandler;
class DynamipsRouterLoadIdle;
class TopologyViewArea;
class WirePlugGraphic;

/*******************************************************************************
* GraphicsItem
*******************************************************************************/
class GraphicsItem : public QObject
{
    Q_OBJECT
public:
    enum Direction {
        Undefined = 0,
        North = 1,
        South,
        West,
        East
    };

    explicit GraphicsItem(QObject *parent = 0);
    virtual void refresh() = 0;
    virtual qlonglong getEntityID() const = 0;
    virtual void initGraphics() = 0;
    virtual QList<EntityActionHandler *> customActions(bool limited) = 0;
    virtual void displayProperties(QWidget *parent) = 0;

protected:
    int _brightness(QColor c);
};


/*******************************************************************************
* WireGraphics
*******************************************************************************/
class WireGraphics : public GraphicsItem, public QAbstractGraphicsShapeItem
{
    Q_OBJECT
public:
    static WireGraphics *create(HnsClient::WireEntity *entity, QPointF from, QPointF to, Hnv::TopologyScene *scene, TopologyViewArea *area, QGraphicsItem *parent = 0);
    static void tryToConnect(QPointF from, QPointF to, TopologyView *view); // TODO: find another way to achieve this

    Q_INVOKABLE explicit WireGraphics(HnsClient::WireEntity *wire, QPointF from, QPointF to, Hnv::TopologyScene *scene, TopologyViewArea *area, QGraphicsItem *parent = 0);
    ~WireGraphics();

    enum CustomActions {
        toWire0,
        toWire1,
        toWire2,
        unplugFrom,
        unplugTo,
        addToCatalog
    };

    virtual void paint(QPainter *painter, const QStyleOptionGraphicsItem *, QWidget *);
    virtual QRectF boundingRect() const;
    virtual QPainterPath shape() const;
    virtual void refresh();
    virtual qlonglong getEntityID() const;
    virtual void initGraphics();
    QPointF getOtherEndPoint(EntityGraphics *thisEntity);
    void updatePosition();
    void setOrigin(EntityGraphics *source, QPointF origin);
    void setFromEntityGraphics(EntityGraphics *source);
    void setToEntityGraphics(EntityGraphics *source);
    Hns::WireLayoutPtr layout();
    void updateLayout(const Hns::WireLayoutPtr &layout);
    virtual QList<EntityActionHandler *> customActions(bool limited);
    virtual void displayProperties(QWidget *parent);
    QPointF fromPoint() const;
    QPointF toPoint() const;
    void setColor(const QString &color);
    EntityGraphics *to();
    EntityGraphics *from();
    void checkIsLayoutChanged();
    void unplugEntityGraphics(EntityGraphics *entityGraphics);
    void setNearToEntity(bool nearToEntity);
    void setNearFromEntity(bool nearFromEntity);

signals:
    void layoutChanged(Hns::WireLayoutPtr);
    void plugWireToEntityRequested(qlonglong wireId, int wirePort, qlonglong entityId);
    void unplugWireRequested(qlonglong wireId, int wirePort);
    void addToCatalogRequested(qlonglong wireId);

protected:
    virtual void mouseDoubleClickEvent(QGraphicsSceneMouseEvent *event);
    void _changeAngles(int angles);

protected slots:
    void _wireDestroyedSlot();
    void _layoutChangedSlot();
    void _changeTo0Angle();
    void _changeTo1Angle();
    void _changeTo2Angles();
    void _fromDestroyed();
    void _toDestroyed();
    void _unplugTo();
    void _unplugFrom();
    void _addToCatalog();

private slots:
    void _fromPlugGraphicRequestedSlot(QPoint point, bool mouseReleased);
    void _toPlugGraphicRequestedSlot(QPoint point, bool mouseReleased);

private:
    void _calculatePath();
    void _getDirections(GraphicsItem::Direction &fromDirection, GraphicsItem::Direction &toDirection);
    bool _checkDirections();
    bool _checkDirections(GraphicsItem::Direction &fromDirection, GraphicsItem::Direction &toDirection, bool updateEntities = false);
    void _setLayout(const Hns::WireLayoutPtr &layout);
    void _draw(const QPointF &from, const QPointF &to);
    GraphicsItem::Direction _getHorizontalDirection(qreal x1, qreal x2, bool isSource = true);
    GraphicsItem::Direction _getVerticalDirection(qreal y1, qreal y2, bool isSource = true);
    GraphicsItem::Direction _getDirection(qreal x1, qreal y1, qreal x2, qreal y2, bool horizontal, bool isSource = true);
    void _plugGraphicMoving(const Hns::WireLayoutPtr &layout, QPoint point, bool mouseReleased, int wirePort);

    HnsClient::WireEntity *_wire;
    TopologyScene *_scene;
    EntityGraphics *_from;
    EntityGraphics *_to;
    QRectF _boundingRect;
    QPainterPath _shapePath;
    QList<QPointF> _pointList;
    int _numberAngles;
    QString _color;
    QPointF _fromOrigin;
    QPointF _toOrigin;
    qreal _midPointX;
    qreal _midPointY;
    Hns::WireLayoutPtr _previousLayout;
    TopologyViewArea *_area;
    bool _nearToEntity;
    bool _nearFromEntity;
    WirePlugGraphic *_fromPlugGraphic;
    WirePlugGraphic *_toPlugGraphic;
};


/*******************************************************************************
* WirePlugGraphic
*******************************************************************************/
class WirePlugGraphic : public QObject, public QGraphicsEllipseItem
{
    Q_OBJECT
public:
    WirePlugGraphic(const QColor &color, const QRectF &rect, WireGraphics *parentWire = 0);
    ~WirePlugGraphic();
    void setColor(const QColor &color);
    void setCloseToEntity(bool closeTo);
    void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);
signals:
    void movingRequested(QPoint point, bool mouseReleased);
protected:
    virtual void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
    virtual void mouseReleaseEvent(QGraphicsSceneMouseEvent *event);
private:
    QColor _color;
};

/*******************************************************************************
* EntityGraphics
*******************************************************************************/
class EntityGraphics : public GraphicsItem, public QGraphicsRectItem
{
    Q_OBJECT
public:
    enum Highlighted {
        None,
        OK,
        WARNING,
        NOK
    };

    static EntityGraphics *create(HnsClient::Entity *entity, Hnv::TopologyScene *scene, TopologyViewArea *area, QGraphicsItem *parent = 0);

    Q_INVOKABLE explicit EntityGraphics(HnsClient::Entity *entity, Hnv::TopologyScene *scene, TopologyViewArea *area, QGraphicsItem *parent = 0);
    ~EntityGraphics();

    virtual QPainterPath shape() const;
    virtual QRectF boundingRect() const;
    virtual void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget = 0);
    HnsClient::Entity *entity();
    virtual qlonglong getEntityID() const;
    virtual void refresh();
    void checkIsLayoutChanged();
    void setColorLabel(QColor color);
    virtual qlonglong addSatellite(QGraphicsSvgItem *satelliteItem);
    virtual void removeSatellite(qlonglong itemID);
    virtual void clearSatellites();
    virtual QGraphicsSvgItem *getSatellite(qlonglong itemID);
    virtual void enableBlackEffect(bool enable = true);
    virtual void initGraphics();
    virtual void displayProperties(QWidget *parent);
    void addWire(WireGraphics *wire, GraphicsItem::Direction direction);
    void removeWire(WireGraphics *wire);
    QList<WireGraphics*> wires(GraphicsItem::Direction direction = GraphicsItem::Undefined);
    bool setWireDirection(WireGraphics* wire, GraphicsItem::Direction newDirection);
    void redrawWires(GraphicsItem::Direction direction = GraphicsItem::Undefined, WireGraphics *excludedWire = 0);
    virtual QList<EntityActionHandler *> customActions(bool limited);
    void applyLabelVisibility();
    void handleNearWire(bool stopMoving = false);
    Hns::EntityLayoutPtr layout();
    void updateLayout(Hns::EntityLayoutPtr layout);
    static QPointF validateItemPosition(QPointF position, QRectF itemRect, QGraphicsScene *scene);
    void setIsHighlighted(Highlighted isHighlighted);
    Direction getDirection(WireGraphics *wire);
    bool handleTemporaryWires(qlonglong wireId);
    void showUsbIcon(bool visible);

public slots:
    void setLabelsDisplayedByDefault(bool labelsDisplayedByDefault);
    void setStatusIconVisibility(bool visible = true);

signals:
    void layoutChanged(Hns::EntityLayoutPtr layout);

protected slots:
    void _repaintNeeded();
    void _layoutChangedSlot();
    void _entryChangedSlot(HnsClient::GenericEntity *entity);
    void _statusChangedSlot(HnsClient::GenericEntity *entity);
    void _refreshedSlot();
    void _updateCaptureIcon();
    void _entityDestroyedSlot();

protected:
    virtual void dragEnterEvent(QGraphicsSceneDragDropEvent *event);
    virtual void dragLeaveEvent(QGraphicsSceneDragDropEvent *event);
    virtual void dropEvent(QGraphicsSceneDragDropEvent *event);
    virtual void mouseDoubleClickEvent(QGraphicsSceneMouseEvent *event);
    virtual QVariant itemChange(GraphicsItemChange change, const QVariant &value);

    void _setCurrentSvg();
    void _registerCustomGraphic();
    virtual void _initSvgFileNames();
    virtual void _setSvgFileNames(const QString & normal, const QString & gray);
    virtual void _setCurrentSvgFileName(const QString &fileName);
//    virtual QString _getToolTip();
    virtual void _setStatus(Hns::Status status);
    virtual void _updateScenePosition(QPoint scenePosition, bool propagate = true);
    virtual void _updateScenePosition(int posX, int posY, bool propagate = true);
    virtual void _updateShortname();
    virtual void _updateNodeName();
    void _updateImmutableIcon();

    QPointF _validatePosition(QPointF position);

    void _drawSatellites();
    void _setWiresOrigin(GraphicsItem::Direction direction);
    void _sortWiresInDirection(GraphicsItem::Direction direction);
    void _showLabels(bool show);

    HnsClient::Entity *_entity;
    TopologyScene *_scene;
    TopologyViewArea *_area;
    QHash<Direction, QList<WireGraphics*> > _wireHash;
    Hns::EntityLayoutPtr _previousLayout;
    QGraphicsSvgItem *_svgItem;
    QGraphicsTextItem *_shortNameTextItem;
    QGraphicsTextItem *_nodeNameTextItem;
    QGraphicsSvgItem *_statusIcon;
    QGraphicsSvgItem *_captureIcon;
    QGraphicsSvgItem *_immutableIcon;
    QGraphicsSvgItem *_usbIcon;
    bool _statusIconDisplayed;
    QMap<qlonglong, QGraphicsSvgItem *> _satelliteItems;
    qlonglong _satID;
    Highlighted _isHighlighted;
    ImplementErrorItem * _errorItem;
    QColor _labelColor;
    QString _svgFileName;
    QString _svgFileNameGray;
    bool _isGray;
    QString _customGraphicName;
    bool _labelsDisplayedByDefault;
    QList<qlonglong> _temporaryWires;
};


/*******************************************************************************
* DomainEntityGraphics
*******************************************************************************/
class DomainEntityGraphics : public EntityGraphics
{
    Q_OBJECT
public:
    Q_INVOKABLE explicit DomainEntityGraphics(HnsClient::Entity *entity, Hnv::TopologyScene *scene, Hnv::TopologyViewArea *area, QGraphicsItem *parent = 0);

protected:
    virtual void _initSvgFileNames();
};


/*******************************************************************************
* SwitchEntityGraphics
*******************************************************************************/
class SwitchEntityGraphics : public EntityGraphics
{
    Q_OBJECT
public:
    Q_INVOKABLE explicit SwitchEntityGraphics(HnsClient::Entity *entity, Hnv::TopologyScene *scene, Hnv::TopologyViewArea *area, QGraphicsItem *parent = 0);

protected:
    virtual void _initSvgFileNames();
};


/*******************************************************************************
* HybridNetcardEntityGraphics
*******************************************************************************/
class HybridNetcardEntityGraphics : public EntityGraphics
{
    Q_OBJECT
public:
    Q_INVOKABLE explicit HybridNetcardEntityGraphics(HnsClient::Entity *entity, Hnv::TopologyScene *scene, Hnv::TopologyViewArea *area, QGraphicsItem *parent = 0);

protected slots:
    void _macAddressAddedSlot(QString macAddress);
    void _macAddressRemovedSlot(QString macAddress);
    void _macAddressChangedSlot(QString macAddress, bool active);

protected:
    virtual void _initSvgFileNames();
    void _setSatelliteRenderer(QGraphicsSvgItem *item, bool active);
    void _addSatellite(QString macAddress, bool active);
    qlonglong _createSatellite(QString macAddress, bool active);
    void _updateSatellite(qlonglong satelliteID, bool active);

    QMap<QString, qlonglong> _satelliteMap;
};


/*******************************************************************************
* WifiAPEntityGraphics
*******************************************************************************/
class WifiAPEntityGraphics : public EntityGraphics
{
    Q_OBJECT
public:
    Q_INVOKABLE explicit WifiAPEntityGraphics(HnsClient::Entity *entity, Hnv::TopologyScene *scene, Hnv::TopologyViewArea *area, QGraphicsItem *parent = 0);

    enum CustomActions {
        connectedPeersAction
    };

    qlonglong createSatellite(QString macAddress);
    virtual QList<EntityActionHandler *> customActions(bool limited);

protected slots:
    void _displayConnectedPeersSlots();
    void _peerJoinedSlot(QString peer);
    void _peerLeftSlot(QString peer);

protected:
    virtual void _initSvgFileNames();
    void _addSatellite(QString peer);
    void _removeSatellite(QString peer);

    QMap<QString, qlonglong> _satelliteMap;
};


/*******************************************************************************
* DynamipsRouterEntityGraphics
*******************************************************************************/
class DynamipsRouterEntityGraphics : public EntityGraphics
{
    Q_OBJECT
public:
    Q_INVOKABLE explicit DynamipsRouterEntityGraphics(HnsClient::Entity *entity, Hnv::TopologyScene *scene, Hnv::TopologyViewArea *area, QGraphicsItem *parent = 0);
    virtual QList<EntityActionHandler *> customActions(bool limited);

    enum CustomActions {
        generateIdlePcAction
        //,extractConfigurationAction
    };

protected slots:
    void _generateIdle();
    void _generateIdleSlot(::std::string idleValue);

protected:
    virtual void _initSvgFileNames();

private:
    DynamipsRouterLoadIdle *_popup;
};


/*******************************************************************************
* ImplementErrorItem
*******************************************************************************/
class ImplementErrorItem : public QAbstractGraphicsShapeItem
{
public:
    ImplementErrorItem(const QList<QPair<Hns::ImplementModelResult, QString> > & errorList);
    ~ImplementErrorItem();

    QRectF boundingRect() const;
    void paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget = 0);

private:
    QList<QPair<Hns::ImplementModelResult, QString> > _errorList;
    const double _fontScaleFactor;
    QPixmap _errorPixmap;
    QPixmap _warningPixmap;
    int _iconHeight;
    int _iconWidth;
    int _iconsHeight;
};

/*******************************************************************************
* TopologyBridgeEntityGraphics
*******************************************************************************/
class TopologyBridgeEntityGraphics : public EntityGraphics
{
    Q_OBJECT
public:
    Q_INVOKABLE explicit TopologyBridgeEntityGraphics(HnsClient::Entity *entity, Hnv::TopologyScene *scene, Hnv::TopologyViewArea *area, QGraphicsItem *parent = 0);

protected:
    virtual void _initSvgFileNames();
};

} // namespace Hnv
#endif // ENTITYGRAPHICS_H
