/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "ressources.h"

// hynesim hnsclient headers
#include "libhnsclient/entry.h"
#include "libhnsclient/entity.h"
#include "libhnsclient/platformmonitor.h"
#include "libhnsclient/session.h"
#include "libhnsclient/resourcemanager.h"

// hynesim common headers
#include "common/initializer.h"

// Qt headers
#include <QSettings>
#include <QFile>
#include <QPainter>
#include <QGraphicsItem>
#include <QSvgRenderer>
#include <QGraphicsSvgItem>

namespace Hnv {

HNS_STATIC_HASH_INITIALIZER(_pixmapPathHash, PixmapPathHash, QString, QString)
PixmapPathHash::PixmapPathHash()
{
    insert(QString::fromStdString(Hns::DomainEntry::ice_staticId()), Constants::GRAPHIC_DESKTOP);
    insert(QString::fromStdString(Hns::SwitchEntry::ice_staticId()), Constants::GRAPHIC_SWITCH);
    insert(QString::fromStdString(Hns::HybridNetcardEntry::ice_staticId()), Constants::GRAPHIC_HYBRID);
    insert(QString::fromStdString(Hns::WifiAPEntry::ice_staticId()),Constants::GRAPHIC_WIFIAP);
    insert(QString::fromStdString(Hns::DynamipsRouterEntry::ice_staticId()),Constants::GRAPHIC_DYNAMIPSROUTER);
    insert(QString::fromStdString(Hns::TopologyGateEntry::ice_staticId()), Constants::GRAPHIC_TOPOLOGY_GATE);
    insert(QString::fromStdString(Hns::WireEntry::ice_staticId()), Constants::GRAPHIC_WIRE);
}

HNS_STATIC_HASH_INITIALIZER(_statusPixmapPathHash, StatusPixmapPathHash, Hns::Status, QString)
StatusPixmapPathHash::StatusPixmapPathHash()
{
    insert(Hns::UNDEFINED, Constants::GRAPHIC_UNDEFINE);
    insert(Hns::STOPPED, Constants::GRAPHIC_STOPPED);
    insert(Hns::STARTED, Constants::GRAPHIC_STARTED);
    insert(Hns::BUSY, Constants::GRAPHIC_BUSY);
    insert(Hns::PAUSED, Constants::GRAPHIC_PAUSED);
    insert(Hns::SAVED, Constants::GRAPHIC_SUSPENDED);
    insert(Hns::UNIMPLEMENTED, Constants::GRAPHIC_UNIMPLEMENTED);
}

QHash<QString, QString> *Ressources::entryPixmapPathHash()
{
    return _pixmapPathHash();
}

HNS_STATIC_HASH(_SVGRendererHash, QString, QSvgRenderer *)

QString Ressources::themedPath(const QString &filename)
{
    QSettings settings;
    settings.beginGroup(Constants::SETTINGS_GROUP_GLOBAL);
    QString theme = settings.value(Constants::SETTINGS_KEY_THEME, Constants::SETTINGS_DEFAULT_THEME).toString();
    settings.endGroup();
    if (theme.isEmpty())
        theme = Constants::SETTINGS_DEFAULT_THEME;
    QString path = _formatPath(filename, theme);
    if (!QFile::exists(path))
        path = _formatPath(filename);
    return path;
}

bool Ressources::hasClassNamePixmap(const QString &className)
{
    return entryPixmapPathHash()->contains(className);
}

QPixmap Ressources::getClassNamePixmap(QString className, bool isTemplate, bool isModel, QVector<std::string> parentClasses)
{
    QString pixmapPath = Constants::ICON_ENTRY;
    if (entryPixmapPathHash()->contains(className))
        pixmapPath = entryPixmapPathHash()->value(className);
    else {
        for(int i = parentClasses.size() - 1; i >= 0; i--) {
            QString parentClassName = QString::fromStdString(parentClasses.at(i));
            if (entryPixmapPathHash()->contains(parentClassName)) {
                pixmapPath = entryPixmapPathHash()->value(parentClassName);
                entryPixmapPathHash()->insert(className, pixmapPath);
            }
        }
    }
    return Ressources::_finalizePixmap(themedPath(pixmapPath), isModel, isTemplate);
}

QPixmap Ressources::getEntryPixmap(HnsClient::CatalogItem *catalogItem)
{
    bool isTemplate = false;
    bool isModel = false;
    QString className;
    QVector<std::string> ice_ids;
    HnsClient::EntityEntry *entry = qobject_cast<HnsClient::EntityEntry *>(catalogItem);
    QString custom = entry->customGraphic();
    if (entry) {
        isTemplate = entry->isTemplate();
        isModel = entry->isModel();
        className = QString::fromStdString(entry->entry()->ice_id());
        ice_ids = QVector<std::string>::fromStdVector(entry->entry()->ice_ids());
    }
    if (!custom.isEmpty()) {
        QByteArray data = HnsClient::Session::instance()->resourceManager()->getResource(custom);
        if(data.isEmpty())
            return getClassNamePixmap(className, isTemplate, isModel, ice_ids);
        QPixmap result(QSize(64,64));
        result.fill(Qt::transparent);
        QSvgRenderer renderer(data);
        QPainter svgpainter(&result);
        renderer.render(&svgpainter);
        return Ressources::_finalizePixmap(result, isModel, isTemplate);
    }
    return getClassNamePixmap(className, isTemplate, isModel, ice_ids);
}

QPixmap Ressources::getTypeIcon(const QString &type)
{
    QString iconPath = Ressources::themedPath(Constants::ICON_PREFIX_TYPE) + type + ".png";
    if (!QFile::exists(iconPath))
        iconPath = Ressources::themedPath(Constants::ICON_GENERIC_TYPE);
    return QPixmap(iconPath);
}

QPixmap Ressources::getStatusPixmap(const Hns::Status &status)
{
    QString pixmapPath = _statusPixmapPathHash()->value(status, Constants::GRAPHIC_UNDEFINE);
    return QPixmap(themedPath(pixmapPath));
}

QGraphicsSvgItem *Ressources::getStatusSVGItem(Hns::Status status, QGraphicsItem *parentItem)
{
    QGraphicsSvgItem *item = 0;
    QString filePath = _statusPixmapPathHash()->value(status, QString());
    if (!filePath.isEmpty())
        item = new QGraphicsSvgItem(Ressources::themedPath(filePath), parentItem);
    return item;
}

bool Ressources::registerSvgRenderer(const QString &filename)
{
    bool result = false;
    if (!_SVGRendererHash()->contains(filename)) {
        QSvgRenderer *renderer = new QSvgRenderer(filename);
        _SVGRendererHash()->insert(filename, renderer);
        result = true;
    }
    return result;
}

bool Ressources::registerSvgRenderer(const QString &filename, QSvgRenderer *renderer)
{
    bool result = false;
    if (!_SVGRendererHash()->contains(filename)) {
        _SVGRendererHash()->insert(filename, renderer);
        result = true;
    }
    return result;
}

QSvgRenderer *Ressources::getRenderer(const QString &filename)
{
    return _SVGRendererHash()->value(filename, 0);
}

QString Ressources::getToolTip(HnsClient::GenericEntity *genericEntity)
{
    QString textLine = genericEntity->getShortName();
    QString toolTip = QObject::tr("<b>Name:&nbsp;</b><i>%1</i>").arg(textLine);
    toolTip += QObject::tr("<br/><b>Available plugs:&nbsp;</b><i>%1/%2</i>").arg(QString::number(genericEntity->getAvailablePlugs().count())).arg(QString::number(genericEntity->getPlugs().count()));
    QString nodeName = genericEntity->getNodeName();
    if (!nodeName.isEmpty()) {
        QString displayName = nodeName;
        HnsClient::NodeInformation * nodeInfo = HnsClient::Session::instance()->platformMonitor()->getNodeInformation(nodeName);
        if (nodeInfo)
            displayName = nodeInfo->getShortName();
        toolTip += QObject::tr("<br/><b>Node:&nbsp;</b><i>%1</i>").arg(displayName);
    }

    if (qobject_cast<HnsClient::DomainEntity*>(genericEntity)) {
        HnsClient::DomainEntity* domainEntity = qobject_cast<HnsClient::DomainEntity*>(genericEntity);
        HnsClient::DomainEntry *domainEntry = qobject_cast<HnsClient::DomainEntry*>(domainEntity->getEntry());
        if (domainEntry)
            toolTip += QObject::tr("<br/><b>Type:&nbsp;</b><i>%1</i>").arg(domainEntry->getType());
    }
    else if (qobject_cast<HnsClient::SwitchEntity*>(genericEntity)) {
        HnsClient::SwitchEntity* switchEntity = qobject_cast<HnsClient::SwitchEntity*>(genericEntity);
        HnsClient::SwitchEntry *switchEntry = qobject_cast<HnsClient::SwitchEntry*>(switchEntity->getEntry());
        if (switchEntry) {
            QString mode = QObject::tr("Switch");
            if (switchEntry->getIsHub())
                mode = QObject::tr("Hub");
            toolTip += QObject::tr("<br/><b>Mode:&nbsp;</b><i>%1</i>").arg(mode);
            toolTip += QObject::tr("<br/><b>STP:&nbsp;</b><i>%1</i>").arg((switchEntry->stpEnabled() ? QObject::tr("Enabled") : QObject::tr("Disabled")));
        }
    }
    else if (qobject_cast<HnsClient::WifiAPEntity*>(genericEntity)) {
        HnsClient::WifiAPEntity* wifiEntity = qobject_cast<HnsClient::WifiAPEntity*>(genericEntity);
        toolTip += QObject::tr("<br/><b>SSID:&nbsp;</b><i>%1</i>").arg(wifiEntity->getSSID());
        toolTip += QObject::tr("<br/><b>Connected peers:&nbsp;</b><i>%1</i>").arg(QString::number(wifiEntity->connectedPeers().count()));
    }
    else if (qobject_cast<HnsClient::DynamipsRouterEntity*>(genericEntity)) {
        HnsClient::DynamipsRouterEntity* dynamipsEntity = qobject_cast<HnsClient::DynamipsRouterEntity*>(genericEntity);
        HnsClient::DynamipsRouterEntry* dynampisEntry = qobject_cast<HnsClient::DynamipsRouterEntry*>(dynamipsEntity);
        if (dynampisEntry)
            toolTip += QObject::tr("<br/><b>Chassis:&nbsp;</b><i>%1</i>").arg(dynampisEntry->getChassis());
    }
    return toolTip;
}

QString Ressources::_formatPath(const QString &filename, QString theme)
{
    return QString(":/themes/%1/%2").arg(theme).arg(filename);
}

QPixmap Ressources::_finalizePixmap(QPixmap pixmap, bool isModel, bool isTemplate)
{
    if (isModel) {
        QPainter templatePainter(&pixmap);
        templatePainter.drawPixmap(pixmap.rect(), QPixmap(themedPath(Constants::ICON_MODEL)));
    } else if (isTemplate) {
        QPainter templatePainter(&pixmap);
        templatePainter.drawPixmap(pixmap.rect(), QPixmap(themedPath(Constants::ICON_TEMPLATE)));
    }
    return pixmap;
}

} // namespace Hnv
