/******************************************************************************
**
** Copyright (C) 2003-2015 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "accessrightwidget.h"
#include "ui_accessrightwidget.h"
#include "hyneviewsession.h"

// hynesim common headers
#include "common/rightslist.h"

// hynesim libhnsclient headers
#include <libhnsclient/entry.h>
#include <libhnsclient/aclmanager.h>
#include <libhnsclient/usermanager.h>

namespace Hnv {
/*******************************************************************************
* AccessRightSubjectModel
*/
AccessRightSubjectModel::AccessRightSubjectModel(const Diamesh::ACLList &aclList, QObject *parent)
    : QAbstractListModel(parent)
{
    QStringList aclSubjectList;
    QList<Diamesh::AccessControlList> aclQList = QList<Diamesh::AccessControlList>::fromStdList(aclList);
    foreach(const Diamesh::AccessControlList &acl, aclQList)
        aclSubjectList.append(QString::fromStdString(acl.subject));

    if (!aclSubjectList.contains(QString("o")))
        _subjectList.append(qMakePair(tr("Everyone"), QString("o")));

    HnsClient::UserManager *uManager = HyneviewSession::instance()->userManager();
    for(int i=0; i < uManager->usersCount(); i++) {
        Hns::UserPtr user = uManager->getUserByIndex(i);
        QString subjectKey = QString("u.%1").arg(QString::number(user->id));
        if (!aclSubjectList.contains(subjectKey))
            _subjectList.append(qMakePair(tr("User/%1").arg(QString::fromStdString(user->name)), subjectKey));
    }
    for(int i=0; i < uManager->groupsCount(); i++) {
        Diamesh::GroupPtr group = uManager->getGroupByIndex(i);
        QString subjectKey = QString("g.%1").arg(QString::number(group->id));
        if (!aclSubjectList.contains(subjectKey))
            _subjectList.append(qMakePair(tr("Group/%1").arg(QString::fromStdString(group->name)), subjectKey));
    }
}

int AccessRightSubjectModel::rowCount(const QModelIndex &parent) const
{
    if (!parent.isValid())
        return _subjectList.count();
    else
        return 0;
}

QVariant AccessRightSubjectModel::data(const QModelIndex &index, int role) const
{
    if (index.isValid()) {
        int row = index.row();
        if (row < _subjectList.count()) {
            switch (role) {
            case Qt::EditRole:
            case Qt::DisplayRole:
                return _subjectList.at(row).first;
                break;
            case Qt::UserRole:
                return _subjectList.at(row).second;
                break;
            }
        }
    }
    return QVariant();
}

void AccessRightSubjectModel::removeSubject(const QString &subject)
{
    int row = -1;
    int pos = 0;
    QListIterator<QPair<QString, QString> >iterator(_subjectList);
    while (iterator.hasNext()) {
        QPair<QString, QString > pair = iterator.next();
        if (pair.second == subject) {
            row = pos;
            break;
        }
        pos++;
    }

    if (row != -1) {
        beginRemoveRows(QModelIndex(), row, row);
        _subjectList.removeAt(row);
        endRemoveRows();
    }
}

void AccessRightSubjectModel::addSubject(const QString &readable, const QString &data)
{
    int row = _subjectList.count();
    beginInsertRows(QModelIndex(), row, row);
    _subjectList.append(qMakePair(readable, data));
    endInsertRows();
}

/*******************************************************************************
* AccessRightModel
*/
AccessRightModel::AccessRightModel(HnsClient::CatalogItem *catalogItem, QObject *parent)
    : QAbstractTableModel(parent)
    , _catalogItem(catalogItem)
    , _aclList(QList<Diamesh::AccessControlList>::fromStdList(catalogItem->aclList()))
{
}

int AccessRightModel::rowCount(const QModelIndex &parent) const
{
    if (!parent.isValid())
        return _aclList.size();
    else
        return 0;
}

int AccessRightModel::columnCount(const QModelIndex &parent) const
{
    if(!parent.isValid())
        return MODEL_COLUMN_COUNT;
    return 0;
}

QVariant AccessRightModel::data(const QModelIndex &index, int role) const
{
    if (index.isValid()) {
        int row = index.row();
        if (row < _aclList.count()) {
            if (role == Qt::UserRole)
                return QString::fromStdString(_aclList[row].subject);

            switch(index.column()) {
            case COLUMN_SUBJECT: {
                if (role == Qt::DisplayRole)
                    return HyneviewSession::instance()->aclManager()->readableSubject(QString::fromStdString(_aclList[row].subject));
                break;
            }
            case COLUMN_PERMISSION: {
                if (role == Qt::DisplayRole)
                    return HyneviewSession::instance()->aclManager()->readableRights(_catalogItem->catalogItem(), _aclList[row].rights);
                break;
            }
            }
        }
    }
    return QVariant();
}

QVariant AccessRightModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if (role == Qt::DisplayRole && orientation == Qt::Horizontal) {
        switch (section) {
        case COLUMN_SUBJECT:
            return tr("Name");
        case COLUMN_PERMISSION:
            return tr("Permission Level");
        case COLUMN_CLOSE:
            return QString();
        }
    }
    return QAbstractTableModel::headerData(section, orientation, role);
}

void AccessRightModel::removeShare(const Hns::CatalogItemPtr &catalogItem, const QString &subject)
{
    QString target = HyneviewSession::instance()->aclManager()->helper()->targetFromObject(catalogItem);
    if (!target.isEmpty()) {
        int row = -1;
        for(int i = 0; i < _aclList.size(); i++) {
            if (_aclList.value(i).target == target.toStdString() && _aclList.value(i).subject == subject.toStdString()) {
                row = i;
                break;
            }
        }
        if (row != -1) {
            beginRemoveRows(QModelIndex(), row, row);
            _aclList.removeAt(row);
            endRemoveRows();
        }
    }
}

Diamesh::ACLList AccessRightModel::aclList()
{
    return _aclList.toStdList();
}

/*******************************************************************************
* AccessRightEntryModel
*/
AccessRightEntryModel::AccessRightEntryModel(HnsClient::CatalogItem *catalogItem, QObject *parent)
    : AccessRightModel(catalogItem, parent)
{
}

void AccessRightEntryModel::addShare(const Hns::CatalogItemPtr &catalogItem, const QString &subject)
{
    QStringList rightsNames;
    rightsNames.append(Hns::Rights::ENTRY_SUFFIX_READ);
    rightsNames.append(Hns::Rights::ENTRY_SUFFIX_UPDATE_ENTRY);
    rightsNames.append(Hns::Rights::ENTRY_SUFFIX_USE);
    Diamesh::AccessControlList newAcl = HyneviewSession::instance()->aclManager()->buildACLForCatalogItem(catalogItem, subject, rightsNames);
    int row = _aclList.count();
    beginInsertRows(QModelIndex(), row, row);
    _aclList.append(newAcl);
    endInsertRows();
}

/*******************************************************************************
* AccessRightFolderModel
*/
AccessRightFolderModel::AccessRightFolderModel(HnsClient::CatalogItem *catalogItem, QObject *parent)
    : AccessRightModel(catalogItem, parent)
{
}

void AccessRightFolderModel::addShare(const Hns::CatalogItemPtr &catalogItem, const QString &subject)
{
    QStringList rightsNames;
    rightsNames.append(Hns::Rights::FOLDER_SUFFIX_READ);
    rightsNames.append(Hns::Rights::FOLDER_SUFFIX_UPDATE_FOLDER);
    rightsNames.append(Hns::Rights::FOLDER_SUFFIX_DELETE);
    Diamesh::AccessControlList newAcl = HyneviewSession::instance()->aclManager()->buildACLForCatalogItem(catalogItem, subject, rightsNames);
    int row = _aclList.count();
    beginInsertRows(QModelIndex(), row, row);
    _aclList.append(newAcl);
    endInsertRows();
}

/*******************************************************************************
* AccessRightWidget
*/
AccessRightWidget::AccessRightWidget(QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::AccessRightWidget)
    , _canModify(false)
    , _catalogItem(0)
{
    ui->setupUi(this);
}

AccessRightWidget::~AccessRightWidget()
{
    delete ui;
}

void AccessRightWidget::initialize(HnsClient::CatalogItem *catalogItem)
{
    _catalogItem = catalogItem;
    _canModify = HyneviewSession::instance()->aclManager()->canBuildAclForCatalogItem(_catalogItem->catalogItem(), HyneviewSession::instance()->user());

    if (qobject_cast<HnsClient::Entry *>(catalogItem))
        _rightModel = new AccessRightEntryModel(catalogItem, this);
    else if (qobject_cast<HnsClient::FolderEntry *>(catalogItem))
        _rightModel = new AccessRightFolderModel(catalogItem, this);
    if (_rightModel) {
        ui->accessPoliciesTableView->setModel(_rightModel);
        ui->accessPoliciesTableView->resizeColumnToContents(AccessRightModel::COLUMN_PERMISSION);
        _subjectModel = new AccessRightSubjectModel(catalogItem->aclList(), this);
        ui->subjectComboBox->setModel(_subjectModel);
        ui->subjectComboBox->setModelColumn(0);
        CloseButtonDelegate *delegate = new CloseButtonDelegate(AccessRightModel::COLUMN_CLOSE, this);
        delegate->setEditable(_canModify);
        connect(ui->accessPoliciesTableView, SIGNAL(cellHovered(int, int)), delegate, SLOT(cellHoverTrackingSlot(int,int)));
        ui->accessPoliciesTableView->setHoverTracking(true);
        ui->accessPoliciesTableView->setItemDelegate(delegate);
        ui->accessPoliciesTableView->horizontalHeader()->setStretchLastSection(false);
        ui->accessPoliciesTableView->horizontalHeader()->setResizeMode(AccessRightModel::COLUMN_PERMISSION, QHeaderView::Stretch);
        ui->accessPoliciesTableView->horizontalHeader()->setResizeMode(AccessRightModel::COLUMN_CLOSE, QHeaderView::Fixed);
        ui->accessPoliciesTableView->horizontalHeader()->resizeSection(AccessRightModel::COLUMN_CLOSE, 16); // TODO : should use a constant
        ui->accessPoliciesTableView->viewport()->setAttribute(Qt::WA_Hover);
    }

    _checkSubjectList();
}

Diamesh::ACLList AccessRightWidget::aclList()
{
    return _rightModel->aclList();
}

void AccessRightWidget::on_subjectPushButton_clicked()
{
    int subjectIndex = ui->subjectComboBox->currentIndex();
    if (subjectIndex != -1 && _canModify) {
            QString subject = ui->subjectComboBox->itemData(subjectIndex).toString();
            _subjectModel->removeSubject(subject);
            _rightModel->addShare(_catalogItem->catalogItem(), subject);
            _checkSubjectList();
    }
}

void AccessRightWidget::on_accessPoliciesTableView_clicked(const QModelIndex &index)
{
    if (index.isValid() && _canModify) {
        if (index.column() == AccessRightModel::COLUMN_CLOSE) {
            QString subject = index.data(Qt::UserRole).toString();
            _rightModel->removeShare(_catalogItem->catalogItem(), subject);
            _subjectModel->addSubject(HyneviewSession::instance()->aclManager()->readableSubject(subject), subject);
            _checkSubjectList();
        }
    }
}

void AccessRightWidget::_checkSubjectList()
{
    bool enabled = true;
    if (!_canModify) {
        enabled = false;
        ui->subjectPushButton->setToolTip(tr("You are not allowed to modify access policies for this entry."));
    } else  if (ui->subjectComboBox->count() == 0)
        enabled = false;
    ui->subjectComboBox->setEnabled(enabled);
    ui->subjectPushButton->setEnabled(enabled);
}

} // namespace Hnv
