/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "dynamicfilterwidget.h"
#include "ui_dynamicfilterwidget.h"
#include "filterlineedit.h"
#include "ressources.h"

// Qt headers
#include <QToolButton>
#include <QCheckBox>
#include <QSignalMapper>

namespace Hnv {

DynamicFilterWidget::DynamicFilterWidget(QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::DynamicFilterWidget)
    , _filterMapper(new QSignalMapper(this))
    , _filterStateMapper(new QSignalMapper(this))
    , _removeFilterMapper(new QSignalMapper(this))
{
    ui->setupUi(this);
    ui->filtersCb->setEnabled(false);
    ui->addFilterPb->setEnabled(false);
    _propertyKey = "FilterKey";
    connect(_filterMapper, SIGNAL(mapped(int)), this, SLOT(_filterTextChanged(int)));
    connect(_filterStateMapper, SIGNAL(mapped(int)), this, SLOT(_filterStateChanged(int)));
    connect(_removeFilterMapper, SIGNAL(mapped(int)), this, SLOT(_filterRemoved(int)));
}

DynamicFilterWidget::~DynamicFilterWidget()
{
    delete ui;
}

void DynamicFilterWidget::addFilter(const QString &key, const QString &text)
{
    if (ui->filtersCb->findData(key) == -1) {
        ui->filtersCb->addItem(text, key);
        ui->filtersCb->setEnabled(true);
        ui->addFilterPb->setEnabled(true);
    }
}

void DynamicFilterWidget::_createFilterWidget(const QString &key, const QString &text)
{
    QCheckBox *cb = new QCheckBox(text, this);
    FilterLineEdit *filterLine = new FilterLineEdit(this);
    QToolButton *delButton = new QToolButton(this);
    int newRowIndex = ui->gridLayout->rowCount();
    _filterRows[key].append(newRowIndex);
    cb->setProperty(_propertyKey.toLatin1().constData(), key);
    cb->setChecked(true);
    _filterStateMapper->setMapping(cb, newRowIndex);
    connect(cb, SIGNAL(stateChanged(int)), _filterStateMapper, SLOT(map()));
    filterLine->setPlaceholderText(tr("Search..."));
    _filterMapper->setMapping(filterLine, newRowIndex);
    connect(filterLine, SIGNAL(textChanged(QString)), _filterMapper, SLOT(map()));
    delButton->setIcon(QIcon(Ressources::themedPath(Constants::ICON_DELETE)));
    _removeFilterMapper->setMapping(delButton, newRowIndex);
    connect(delButton, SIGNAL(clicked()), _removeFilterMapper, SLOT(map()));
    ui->gridLayout->addWidget(cb, newRowIndex, DynamicFilterWidget::State);
    ui->gridLayout->addWidget(filterLine, newRowIndex, DynamicFilterWidget::Value, 1, 2);
    ui->gridLayout->addWidget(delButton, newRowIndex, DynamicFilterWidget::Remove);
}

void DynamicFilterWidget::_removeFilterWidget(int row)
{
    QSet<QWidget *> widgets;
    QString key;
    for(int i = 0; i < ui->gridLayout->columnCount(); ++i) {
        QLayoutItem *item = ui->gridLayout->itemAtPosition(row, i);
        if (item && item->widget()) {
            widgets << item->widget();
            if (QCheckBox *cb = qobject_cast<QCheckBox *>(item->widget()))
                key = cb->property(_propertyKey.toLatin1().constData()).toString();
        }
    }
    foreach(QWidget *w, widgets)
        ui->gridLayout->removeWidget(w);
    qDeleteAll(widgets);
    if (!key.isEmpty())
        _filterRows[key].removeAll(row);
}

DynamicFilterWidget::FilterData DynamicFilterWidget::_dataFromRow(int row)
{
    FilterData data;
    if (row > 0 && row < ui->gridLayout->rowCount()) {
        QLayoutItem *item = ui->gridLayout->itemAtPosition(row, DynamicFilterWidget::State);
        QCheckBox *cb = qobject_cast<QCheckBox *>(item->widget());
        data.enabled = cb->isChecked();
        data.key = cb->property(_propertyKey.toLatin1().constData()).toString();
        data.label = cb->text();
        data.value = qobject_cast<Hnv::FilterLineEdit *>(ui->gridLayout->itemAtPosition(row, DynamicFilterWidget::Value)->widget())->text();
    }
    return data;
}

QStringList DynamicFilterWidget::_filtersFromKey(const QString &key)
{
    QStringList result;
    QList<int> rows = _filterRows.value(key);
    foreach(int row, rows) {
        FilterData data = _dataFromRow(row);
        if (_isDataValid(data) && data.enabled)
            result.append(data.value);
    }
    return result;
}

bool DynamicFilterWidget::_isDataValid(const DynamicFilterWidget::FilterData &data)
{
    if (data.key.isEmpty())
        return false;
    return true;
}

void DynamicFilterWidget::_filterTextChanged(int row)
{

    FilterData data = _dataFromRow(row);
    if (_isDataValid(data) && data.enabled)
        emit filterTextChanged(data.key, _filtersFromKey(data.key));
}

void DynamicFilterWidget::_filterStateChanged(int row)
{
    FilterData data = _dataFromRow(row);
    if (_isDataValid(data))
        emit filterTextChanged(data.key, _filtersFromKey(data.key));
}

void DynamicFilterWidget::_filterRemoved(int row)
{
    FilterData data = _dataFromRow(row);
    if (_isDataValid(data)) {
        _removeFilterWidget(row);
        emit filterTextChanged(data.key, _filtersFromKey(data.key));
    }
}

void DynamicFilterWidget::on_addFilterPb_clicked()
{
    int currentIndex = ui->filtersCb->currentIndex();
    if (currentIndex == -1)
        return;
    QString key = ui->filtersCb->itemData(currentIndex).toString();
    QString text = ui->filtersCb->itemText(currentIndex);
    _createFilterWidget(key, text);
}

} // namespace Hnv

