/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "catalogiconsdelegate.h"
#include "ressources.h"
#include "entitycatalogmodel.h"

// Qt headers
#include <QPainter>

namespace Hnv {

/*******************************************************************************
* CatalogIconsDelegate
*/
CatalogIconsDelegate::CatalogIconsDelegate(QObject *parent)
    : QStyledItemDelegate(parent)
{
    _lockedPixmap = QPixmap(Ressources::themedPath(Constants::ICON_PERM_MISSING));
    _syncingFramesPixmap = QPixmap(Ressources::themedPath(Constants::ICON_WORKING));
    _savedUndefinedPixmap = QPixmap(Ressources::themedPath(Constants::GRAPHIC_SUSPENDED_UNDEFINED));
    _unknowStatus = QPixmap(Ressources::themedPath(Constants::ICON_ENTRY));
    _linkedPixmap = QPixmap(Ressources::themedPath(Constants::GRAPHIC_LINKED));
    _syncingTimerID = -1;
    _statusHash.insert(Hns::UNDEFINED, QPixmap(Ressources::themedPath(Constants::GRAPHIC_UNDEFINE)));
    _statusHash.insert(Hns::STOPPED, QPixmap(Ressources::themedPath(Constants::GRAPHIC_STOPPED)));
    _statusHash.insert(Hns::STARTED, QPixmap(Ressources::themedPath(Constants::GRAPHIC_STARTED)));
    _statusHash.insert(Hns::BUSY, QPixmap(Ressources::themedPath(Constants::GRAPHIC_BUSY)));
    _statusHash.insert(Hns::PAUSED, QPixmap(Ressources::themedPath(Constants::GRAPHIC_PAUSED)));
    _statusHash.insert(Hns::SAVED, QPixmap(Ressources::themedPath(Constants::GRAPHIC_SUSPENDED)));
    _statusHash.insert(Hns::UNIMPLEMENTED, QPixmap(Ressources::themedPath(Constants::GRAPHIC_UNIMPLEMENTED)));
}

QSize CatalogIconsDelegate::sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    QSize hint = QStyledItemDelegate::sizeHint(option, index);
    int extraIcons = extraIconsCount(index);
    if (extraIcons > 0)
        hint.setWidth( hint.width() + option.decorationSize.width() * extraIcons );
    return hint;
}

void CatalogIconsDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    int extraIcons = extraIconsCount(index);
    QStyleOptionViewItem updatedOption = option;
    if (extraIcons > 0 ) {
        QVariant data = index.data(EntryItem::IsLockedRole);
        if (data.isValid() && data.toBool()) {
            updatedOption = paintExtraPixmap(_lockedPixmap, painter, updatedOption, index);
            data = index.data(EntityItem::EntityStatusRole);
            if (data.isValid()) {
                Hns::Status currentStatus = Hns::Status(data.toInt());
                QPixmap statusPixmap = _unknowStatus;
                if (_statusHash.contains(currentStatus))
                    statusPixmap = _statusHash.value(currentStatus);
                updatedOption = paintExtraPixmap(statusPixmap, painter, updatedOption, index);
            }
        } else {
            data = index.data(EntityItem::IsSavedRole);
            if (data.isValid() && data.toBool())
                updatedOption = paintExtraPixmap(_savedUndefinedPixmap, painter, updatedOption, index);
        }
        data = index.data(EntityItem::StatusRole);
        if (data.isValid() && data.toInt()) {
            int dataInt = data.toInt();
            if (dataInt == ::Hns::ENTITY_STATUS_UUID_CONFLICT) {
                QPixmap icon = QPixmap((Ressources::themedPath(Constants::ICON_ENTITY_UUID_CONFLICT)));
                updatedOption = paintExtraPixmap(icon,painter,updatedOption,index);
            }
            else if (dataInt == Hns::ENTITY_STATUS_PROVISIONING) {
                QPixmap icon = QPixmap(Ressources::themedPath(Constants::ICON_SETTINGS));
                updatedOption = paintExtraPixmap(icon, painter, updatedOption, index);
            }
        }
        data = index.data(EntryItem::IsSyncingRole);
        if (data.isValid() && data.toBool()) {
            int currentFrame = 1;
            if (_syncingItems.contains(index))
                currentFrame = _syncingItems.value(index);
            updatedOption = paintExtraPixmap(extractFrameFromPixmap(_syncingFramesPixmap,currentFrame,updatedOption.decorationSize.height()), painter, updatedOption, index);
        }
        data = index.data(EntryItem::HasLinkedClonesRole);
        if (data.isValid() && data.toBool()) {
            updatedOption = paintExtraPixmap(_linkedPixmap, painter, updatedOption, index);
        }
    }
    return QStyledItemDelegate::paint(painter,updatedOption,index);
}

QPixmap CatalogIconsDelegate::extractFrameFromPixmap(QPixmap pixmap, int targetFrame, int framePerLine, int frameSize) const
{
    int x = targetFrame;
    int y = 0;
    while(x >= framePerLine) {
        x = x - framePerLine;
        y++;
    }
    x = x * frameSize;
    y = y * frameSize;
    return pixmap.copy(x,y,frameSize,frameSize);
}

QStyleOptionViewItem CatalogIconsDelegate::paintExtraPixmap(QPixmap pixmap, QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    QPixmap extraPixmap = pixmap.scaledToHeight(option.decorationSize.height(), Qt::SmoothTransformation);
    int pixmapWidth = extraPixmap.width();
    int left;
    int top;
    int right;
    int bottom;
    option.rect.getCoords(&left, &top, &right, &bottom);
    QRect extraRect = QRect(right - pixmapWidth, top, pixmapWidth, option.rect.height());
    QPoint origin = QStyle::alignedRect(option.direction, option.decorationAlignment, extraPixmap.size(), extraRect).topLeft();

    if (option.showDecorationSelected && (option.state & QStyle::State_Selected)) {
        QPalette::ColorGroup cg = option.state & QStyle::State_Enabled ? QPalette::Normal : QPalette::Disabled;
        if (cg == QPalette::Normal && !(option.state & QStyle::State_Active))
            cg = QPalette::Inactive;

        painter->fillRect(extraRect, option.palette.brush(cg, QPalette::Highlight));
    } else {
        QVariant value = index.data(Qt::BackgroundRole);
        if (qVariantCanConvert<QBrush>(value)) {
            painter->fillRect(extraRect, qvariant_cast<QBrush>(value));
        }
    }
    painter->drawPixmap(origin, extraPixmap);
    QStyleOptionViewItem returnOption = option;
    returnOption.rect.setWidth(option.rect.width() - extraRect.width());
    return returnOption;
}

/*******************************************************************************
* CatalogIconsDelegate - Public slots
*/
void CatalogIconsDelegate::transactionStarted(QPersistentModelIndex index)
{
    if (!_syncingItems.contains(index))
        _syncingItems.insert(index, 1);

    if (_syncingTimerID == -1) {
        _syncingTimerID = startTimer(100);
    }
}

void CatalogIconsDelegate::stopTimer()
{
    if (_syncingTimerID == -1)
        return;
    killTimer(_syncingTimerID);
    _syncingTimerID = -1;
}

/*******************************************************************************
* CatalogIconsDelegate - Protected
*/
void CatalogIconsDelegate::timerEvent(QTimerEvent *event)
{
    Q_UNUSED(event)
    QMutableHashIterator<QPersistentModelIndex, int> i(_syncingItems);
    while (i.hasNext()) {
        i.next();
        QPersistentModelIndex index = i.key();
        int currentFrame = i.value();
        if (currentFrame == 31)
            currentFrame = 1;
        else
            currentFrame++;
        i.setValue(currentFrame);
        emit(requestUpdate(index));
        bool syncEnded = true;
        QVariant data = index.data(EntryItem::IsSyncingRole);
        if (data.isValid() && data.toBool())
            syncEnded = false;
        if (syncEnded)
            i.remove();
    }

    if (_syncingItems.size() == 0) {
        stopTimer();
    }
}

int CatalogIconsDelegate::extraIconsCount(const QModelIndex &index) const
{
    int extraIcons = 0;
    if (index.isValid() && index.column() == 0) {
        QVariant data = index.data(EntryItem::IsLockedRole);
        if (data.isValid() && data.toBool()) {
            extraIcons++;
            data = index.data(EntityItem::EntityStatusRole);
            if (data.isValid())
                    extraIcons++;
        } else {
            data = index.data(EntityItem::IsSavedRole);
            if (data.isValid() && data.toBool()) {
                extraIcons++;
            }
        }
        data = index.data(EntryItem::IsSyncingRole);
        if (data.isValid() && data.toBool())
            extraIcons++;
        data = index.data(EntityItem::HasLinkedClonesRole);
        if (data.isValid() && data.toInt())
            extraIcons++;
        data = index.data(EntityItem::StatusRole);
        if (data.isValid() && data.toInt()) {
            extraIcons++;
        }
    }
    return extraIcons;
}

} // namespace Hnv
