/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "entitycatalogwidget.h"
#include "ui_entitycatalogwidget.h"
#include "ressources.h"
#include "hyneviewsession.h"
#include "entitycatalogmodel.h"
#include "treesortfilterproxymodel.h"
#include "multicolumnstoflatproxymodel.h"
#include "uniqueproxymodel.h"
#include "catalogiconsdelegate.h"
#include "hnvmainwindow.h"
#include "graphicselectordialog.h"
#include "hnv_entryproperties.h"
#include "entityclonedialog.h"

// hynesim hnsclient headers
#include <libhnsclient/resourcemanager.h>
#include <libhnsclient/catalogimporter.h>
#include <libhnsclient/entityentrycatalog.h>

// hynesim common headers
#include "common/clonejobflags.h"
#include "common/cloneutils.h"

// Qt headers
#include <QMimeData>
#include <QCompleter>
#include <QModelIndex>
#include <QPersistentModelIndex>
#include <QMessageBox>
#include <QMenu>
#include <QDebug>

namespace Hnv {

struct ItemStates {
    int indexType;
    bool isWriteable;
    bool isSyncing;
    bool isLocked;
    bool isSaved;
    bool isValid;
    bool isModel;
    int status;
    bool hasLinkedClones;
    bool isLinkedClone;
};

/*******************************************************************************
* EntityCatalogWidget
*/
EntityCatalogWidget::EntityCatalogWidget(QWidget *parent)
    : SideBarContentWidget(parent)
    , ui(new Ui::EntityCatalogWidget)
{
    ui->setupUi(this);
    ui->toolBar->setCloseable(false);
    ui->toolBar->setBorders(ToolBar::None);
    _session = 0;

    HnvMainWindow *window = HnvMainWindow::instance();
    // ************************************************************************
    // Source model
    _catalogModel = window->entityCatalogModel();
    // ************************************************************************
    // Proxy model used for filtering
    _proxyModel = new TreeSortFilterProxyModel(this);
    _proxyModel->setSortCaseSensitivity(Qt::CaseInsensitive);
    _proxyModel->setSortRole(TreeModelItem::InternalSortRole);
    _proxyModel->sort(0);
    _proxyModel->setDynamicSortFilter(true);
    _proxyModel->setFilterKeyColumn(0);
    QList<int> filterColumns;
    filterColumns << EntryItem::COLUMN_SHORTNAME << EntryItem::COLUMN_TAGLIST;
    _proxyModel->setFilterColumnsList(filterColumns);
    _proxyModel->setFilterCaseSensitivity(Qt::CaseInsensitive);
    _proxyModel->setSourceModel(_catalogModel);

    //LinkedClonesFilterProxyModel
    _linkedClonesFilterProxyModel = new LinkedClonesFilterProxyModel(this);
    _linkedClonesFilterProxyModel->setSourceModel(_proxyModel);
    _linkedClonesFilterProxyModel->setDynamicSortFilter(true);

    // ************************************************************************
    // Delegate used to display multiple icons
    CatalogIconsDelegate *delegate = new CatalogIconsDelegate(this);
    connect(_catalogModel, SIGNAL(beginTransaction(QPersistentModelIndex)), this, SLOT(_beginTransactionSlot(QPersistentModelIndex)));
    connect(_catalogModel, SIGNAL(catalogLoaded()), this, SLOT(catalogLoadedSlot()));
    connect(this, SIGNAL(beginTransaction(QPersistentModelIndex)), delegate, SLOT(transactionStarted(QPersistentModelIndex)));
    connect(delegate, SIGNAL(requestUpdate(QPersistentModelIndex)), this, SLOT(updateTreeViewCell(QPersistentModelIndex)));
    connect(this, SIGNAL(serverDisconnect()), delegate, SLOT(stopTimer()));
    connect(ui->catalogTreeView, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(on_action_Properties_triggered()));

    // ************************************************************************
    // ToolBar
    ui->toolBar->addAction(ui->action_Add_category);
    ui->toolBar->addAction(ui->action_Properties);
    ui->toolBar->addAction(ui->action_Export);
    ui->toolBar->addSeparator();
    ui->toolBar->addAction(ui->action_Clone_Entity);
    ui->toolBar->addAction(ui->action_Remove_from_category);
    ui->toolBar->addSeparator();
    ui->toolBar->addAction(ui->action_Show_linked_domains);
    ui->toolBar->addSeparator();
    ui->toolBar->addAction(ui->action_Delete);
    ui->toolBar->addSeparator();
    ui->toolBar->addAction(ui->action_Expand_All);
    ui->toolBar->addAction(ui->action_Collapse_All);
    ui->action_Add_category->setIcon(QIcon(Ressources::themedPath(Constants::ICON_CREATE_CATEGORY)));
    ui->action_Clone_Entity->setIcon(QIcon(Ressources::themedPath(Constants::ICON_CLONE_ENTITY)));
    ui->action_Show_linked_domains->setIcon(QIcon(Ressources::themedPath(Constants::GRAPHIC_LINKED)));
    ui->action_Collapse_All->setIcon(QIcon(Ressources::themedPath(Constants::ICON_COLLAPSE)));
    ui->action_Remove_from_category->setIcon(QIcon(Ressources::themedPath(Constants::ICON_REMOVE)));
    ui->action_Delete->setIcon(QIcon(Ressources::themedPath(Constants::ICON_DELETE)));
    ui->action_Expand_All->setIcon(QIcon(Ressources::themedPath(Constants::ICON_EXPAND)));
    ui->action_Properties->setIcon(QIcon(Ressources::themedPath(Constants::ICON_PROPERTIES)));
    ui->action_Export->setIcon(QIcon(Ressources::themedPath(Constants::ICON_EXPORT)));

    // ************************************************************************
    // Tree view
    ui->catalogTreeView->setModel(_linkedClonesFilterProxyModel);
    // Hide all columns but the one displaying "shortName" data
    for (int i=1; i < _catalogModel->columnCount(QModelIndex()); i++)
        ui->catalogTreeView->hideColumn(i);
    // Hide root decoration (expand/collapse symbol)
    ui->catalogTreeView->setRootIsDecorated(false);
    // Work around a crash in QSortFilterProxyModel when dynamicSortFilter is set to true
    ui->catalogTreeView->setUniformRowHeights(true);
    // Disable expand/collapse on double-click for "VirtualFolderItem" objects
    connect(ui->catalogTreeView, SIGNAL(collapsed(QModelIndex)), this, SLOT(_collapsedSlot(QModelIndex)));
    connect(ui->catalogTreeView, SIGNAL(expanded(QModelIndex)), this , SLOT(_expandedSlot(QModelIndex)));
    // Multi icon delegate
    ui->catalogTreeView->setItemDelegate(delegate);
    connect( ui->catalogTreeView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)), this, SLOT(_catalogSelectionChanged(QItemSelection,QItemSelection)));
    connect(_catalogModel, SIGNAL(focusOnItem(QModelIndex)), this, SLOT(_focusOnItemSlot(QModelIndex)));
    connect(ui->catalogTreeView, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(_contextMenuRequested(QPoint)));
    connect(ui->catalogTreeView->model(), SIGNAL(dataChanged(QModelIndex,QModelIndex)), this, SLOT(_modelDataChangedSlot(QModelIndex,QModelIndex)));

    // ************************************************************************
    // Completer
    QHash<int, QString> completionColumnsHash;
    completionColumnsHash.insert(0, QString("")); // First column, full data
    completionColumnsHash.insert(6, QString(",")); // Tag list column, with "," separator
    MultiColumnsToFlatProxyModel *completionModel = new MultiColumnsToFlatProxyModel(_catalogModel, completionColumnsHash, this);
    UniqueProxyModel *uniqueModel = new UniqueProxyModel(0, this);
    uniqueModel->setSourceModel(completionModel);
    QCompleter *catalogCompleter = new QCompleter(uniqueModel, this);
    catalogCompleter->setCompletionColumn(0);
    catalogCompleter->setCaseSensitivity(Qt::CaseInsensitive);
    catalogCompleter->setCompletionRole(Qt::DisplayRole);
    ui->catalogFilterEdit->setCompleter(catalogCompleter);

    connect(window, SIGNAL(sessionConnected(HyneviewSession*)), this, SLOT(connected(HyneviewSession*)));
    connect(window, SIGNAL(sessionDisconnected()), this, SLOT(beginDisconnect()));
    // TODO : check if needed
    //connect(this, SIGNAL(loadingTransaction(int)), window, SLOT(entityCatalogLoadingSlot(int)));

    HyneviewSession * session = HyneviewSession::instance();
    if (session)
        connected(session);
    else
        _toggleActions();

    addActions(ui->toolBar->actions());

    ui->catalogTreeView->expandToDepth(1);
}

EntityCatalogWidget::~EntityCatalogWidget()
{
    delete ui;
}

QString EntityCatalogWidget::title()
{
    return tr("Entity Catalog");
}

QString EntityCatalogWidget::sideBarId()
{
    return "Entity Catalog";
}

void EntityCatalogWidget::editItem(QModelIndex index)
{
    if (index.flags() & Qt::ItemIsEnabled) {
        int itemType = getItemInternalType(index);
        qlonglong itemID = index.data(EntryItem::IDRole).toLongLong();
        if (itemType == EntityCatalogModel::Folder) {
            HnsClient::FolderEntry * sourceEntry = _catalog->folderById(itemID);
            if (sourceEntry) {
                HnsClient::FolderEntry *updatedEntry = sourceEntry->duplicateFolder(this);
                EntryProperties popup(updatedEntry, this);
                HnvDialogHandler *popupHandler = new HnvDialogHandler(sourceEntry, &popup, this);
                popupHandler->setSourceDestroyedMessage(tr("\"%1\" was deleted. Operation aborted.").arg(sourceEntry->getShortName()));
                int popupResult = popup.exec();
                if (popupResult == QDialog::Accepted) {
                    _catalog->updateFolder(updatedEntry);
                } else
                    delete updatedEntry;
            }
        } else if(itemType == EntityCatalogModel::Entity) {
            HnsClient::EntityEntry *sourceEntry = _catalog->entityEntryById(itemID);
            if (sourceEntry) {
                HnsClient::Entry *clonedEntry = sourceEntry->duplicateEntry(this);
                HnsClient::EntityEntry *updatedEntry = qobject_cast<HnsClient::EntityEntry *>(clonedEntry);
                bool readOnly = sourceEntry->isLocked() || sourceEntry->hasLinkedClones() || sourceEntry->isSyncing() || (sourceEntry->getStatus() == ::Hns::ENTITY_STATUS_UUID_CONFLICT)
                        || (sourceEntry->getStatus() == Hns::ENTITY_STATUS_PROVISIONING);
                bool editableWidget = !readOnly;
                EntryProperties popup(updatedEntry, this, readOnly, sourceEntry->isUsedAsEntity(), editableWidget);
                HnvDialogHandler * popupHandler = new HnvDialogHandler(sourceEntry, &popup, this);
                popupHandler->setSourceDestroyedMessage(tr("\"%1\" was deleted. Operation aborted.").arg(sourceEntry->getShortName()));
                int popupResult = popup.exec();
                if (popupResult == QDialog::Accepted) {
                    if (sourceEntry->isLocked() || sourceEntry->isSyncing()) {
                        qCritical() << tr("\"%1\" is currently used and can not be updated anymore.").arg(sourceEntry->getShortName());
                        delete updatedEntry;
                    } else
                        _catalog->updateEntityEntry(updatedEntry);
                } else
                    delete updatedEntry;
            }
        }
    }
}

QModelIndex EntityCatalogWidget::selectedItemIndex()
{
    QModelIndex currentIdx = ui->catalogTreeView->selectionModel()->currentIndex();
    if (currentIdx.isValid())
        return currentIdx;
    return QModelIndex();
}

QModelIndexList EntityCatalogWidget::selectedItemIndexes()
{
    return ui->catalogTreeView->selectionModel()->selectedRows();
}

int EntityCatalogWidget::getItemInternalType(QModelIndex targetIndex)
{
    return _catalogModel->getItemInternalType(_proxyModel->mapToSource(_linkedClonesFilterProxyModel->mapToSource(targetIndex)));
}

bool EntityCatalogWidget::canHandleMimeData(const QMimeData *data)
{
    QString datatype;
    return canHandleMimeData(data, datatype);
}

bool EntityCatalogWidget::canHandleMimeData(const QMimeData *data, QString &datatype)
{
    return _catalogModel->canHandleMimeData(data, datatype);
}

/*******************************************************************************
* EntityCatalogWidget - Public slots
*/

void EntityCatalogWidget::catalogLoadedSlot()
{
    ui->catalogTreeView->expandToDepth(1);
}

void EntityCatalogWidget::connected(HyneviewSession *session)
{
    if (_session != 0)
        beginDisconnect();
    _session = session;
    _catalog = _session->entityEntryCatalog();
    _toggleActions();
}

void EntityCatalogWidget::beginDisconnect()
{
    foreach (QObject *o, children()) {
        QDialog *dialog = qobject_cast<QDialog *>(o);
        if (dialog)
            dialog->reject();
    }
    _catalog = 0;
    _session = 0;
    _toggleActions();
    emit serverDisconnect();
}

void EntityCatalogWidget::updateTreeViewCell(QPersistentModelIndex index)
{
    // refresh display for one cell
    ui->catalogTreeView->update(index);
    _toggleActions();
}

/*******************************************************************************
* EntityCatalogWidget - Protected
*/
void EntityCatalogWidget::addFolder(QModelIndex parentIdx)
{
    int parentType = getItemInternalType(parentIdx);
    qlonglong parentID = -1;
    if (parentType == EntityCatalogModel::VirtualFolder)
        parentID = 0;
    else if (parentType == EntityCatalogModel::Folder) {
        parentID = parentIdx.data(EntryItem::IDRole).toLongLong();

    }

    if (parentID != -1) {
        Hns::FolderPtr newFolder = new Hns::Folder();
        newFolder->parentId = parentID;
        newFolder->shortName = tr("New folder").toStdString();
        HnsClient::FolderEntry *entry = new HnsClient::FolderEntry(newFolder);
        EntryProperties popup(entry, this);
        int popupResult = popup.exec();
        if (popupResult == QDialog::Accepted) {
            _catalog->addFolder(entry);
        } else
            delete entry;
    }
}

void EntityCatalogWidget::_toggleActions()
{
    bool canAddCategory = false;
    bool canRead = false;
    bool canEdit = false;
    bool canDelete = false;
    bool canRemoveFromCategory = false;
    bool canClone = false;
    bool canBuildModelFrom = false;
    bool canDiscardSaveState = false;
    bool canExport = false;
    bool hasLinkedClones = false;
    bool singleSelection = true;

    if (_session) {
        // TODO:TestRight
//        bool isAdmin = _session->user()->isAdmin;
        QModelIndexList selectedIndexes = selectedItemIndexes();
        singleSelection = (selectedIndexes.count() == 1);
        if (selectedIndexes.isEmpty()) {
            canAddCategory = true;
        } else {
            foreach(QModelIndex index, selectedIndexes) {
                if (index.flags() & Qt::ItemIsEnabled) {
                    struct ItemStates item;
                    _parseCatalogItem(index, &item);

                    // TODO:TestRight
                    if (!canAddCategory && _canAddCategory(index, &item, true))
                        canAddCategory = true;
                    if (!canRead && _canRead(index, &item))
                        canRead = true;
                    // TODO:TestRight
                    if (!canRemoveFromCategory && _canRemoveFromCategory(index, &item, true))
                        canRemoveFromCategory = true;
                    // TODO:TestRight
                    if (!canEdit && _canEdit(index, &item, true))
                        canEdit = true;
                    // TODO:TestRight
                    if (!canDelete && _canDelete(index, &item, true))
                        canDelete = true;
                    // TODO:TestRight
                    if (!canClone && _canClone(index, &item, true))
                        canClone = true;
                    // TODO:TestRight
                    if (!canBuildModelFrom && _canBuildFromModel(index, &item, true))
                        canBuildModelFrom = true;
                    // TODO:TestRight
                    if (!canDiscardSaveState && _canDiscardSaveState(index, &item, true))
                        canDiscardSaveState = true;
                    // TODO:TestRight
                    if (_canExport(index, &item, true))
                        canExport = true;
                    // TODO:TestRight
                    if (!hasLinkedClones && item.hasLinkedClones)
                        hasLinkedClones = true;

                    if (canAddCategory && canRead && canRemoveFromCategory && canClone && canEdit && canDelete && canBuildModelFrom && canDiscardSaveState && canExport)
                        break;
                }
            }
        }
    } else {
        ui->action_Show_linked_domains->setChecked(false);
    }
    ui->action_Collapse_All->setEnabled(_session);
    ui->action_Expand_All->setEnabled(_session);
    ui->action_Add_category->setEnabled(canAddCategory && singleSelection);
    ui->action_Rename->setEnabled(canEdit && singleSelection);
    ui->action_Remove_from_category->setEnabled(canRemoveFromCategory);
    ui->action_Delete->setEnabled(canDelete && !hasLinkedClones);
    ui->action_Properties->setEnabled(canRead && singleSelection);
    ui->action_Clone_Entity->setEnabled(canRead && canClone);
    ui->action_Show_linked_domains->setEnabled((hasLinkedClones && singleSelection) || ui->action_Show_linked_domains->isChecked());
    ui->action_Build_Model_From->setEnabled(canRead && canBuildModelFrom);
    ui->action_Export->setEnabled(canRead && canExport && singleSelection);
    ui->action_Discard_save_state->setEnabled(canDiscardSaveState);
    ui->action_Set_custom_graphic->setEnabled(canEdit && singleSelection);
}

void EntityCatalogWidget::_parseCatalogItem(QModelIndex index, ItemStates *item)
{
    item->indexType = getItemInternalType(index);
    item->isWriteable = index.data(EntryItem::IsWriteableRole).toBool();
    item->isSyncing = index.data(EntryItem::IsSyncingRole).toBool();
    item->isLocked = index.data(EntryItem::IsLockedRole).toBool();
    item->isSaved = index.data(EntryItem::IsSavedRole).toBool();
    item->isValid = index.data(EntryItem::IsValidRole).toBool();
    item->isModel = index.data(EntryItem::IsModelRole).toBool();
    item->status = index.data(EntityItem::StatusRole).toInt();
    item->hasLinkedClones = index.data(EntityItem::HasLinkedClonesRole).toInt();
    item->isLinkedClone = index.data(EntityItem::IsLinkedCloneRole).toInt();
}

// TODO:TestRight
bool EntityCatalogWidget::_canAddCategory(QModelIndex index, ItemStates *item, bool isAdmin)
{
    return (!index.isValid()
            || item->indexType == EntityCatalogModel::VirtualFolder
            || (item->indexType == EntityCatalogModel::Folder
                && (item->isWriteable || isAdmin)
                && !item->isSyncing));
}

bool EntityCatalogWidget::_canRead(QModelIndex index, ItemStates *item)
{
    return (index.isValid() && item->indexType != EntityCatalogModel::VirtualFolder && !item->isSyncing);
}

// TODO:TestRight
bool EntityCatalogWidget::_canEdit(QModelIndex index, ItemStates *item, bool isAdmin)
{
    return (index.isValid()
            && item->indexType != EntityCatalogModel::VirtualFolder
            && (item->isWriteable || isAdmin)
            && !item->isSyncing
            && !item->isLocked
            && !item->hasLinkedClones
            && item->status == Hns::ENTITY_STATUS_OK);
}

// TODO:TestRight
bool EntityCatalogWidget::_canDelete(QModelIndex index, ItemStates *item, bool isAdmin)
{
    return (index.isValid()
            && item->indexType != EntityCatalogModel::VirtualFolder
            && (item->isWriteable || isAdmin)
            && !item->isSyncing
            && !item->isLocked);
}

// TODO:TestRight
bool EntityCatalogWidget::_canRemoveFromCategory(QModelIndex index, ItemStates *item, bool isAdmin)
{
    bool result = false;
    if (_canEdit(index, item, isAdmin)) {
        if (item->indexType == EntityCatalogModel::Entity
                && getItemInternalType(index.parent()) != EntityCatalogModel::VirtualFolder)
            result = true;
    }
    return result;
}

// TODO:TestRight
bool EntityCatalogWidget::_canClone(QModelIndex index, ItemStates *item, bool isAdmin)
{
    return (index.isValid() && item->indexType == EntityCatalogModel::Entity
            && (item->isWriteable || isAdmin) && !item->isSaved
            && item->isValid && !item->isLocked && !item->isSyncing
            && item->status == Hns::ENTITY_STATUS_OK);
}

// TODO:TestRight
bool EntityCatalogWidget::_canBuildFromModel(QModelIndex index, ItemStates *item, bool isAdmin)
{
    return (index.isValid() && item->indexType == EntityCatalogModel::Entity
            && (item->isWriteable || isAdmin) && !item->isSaved
            && item->isValid && !item->isLocked && !item->isSyncing && !item->isModel
            && item->status == Hns::ENTITY_STATUS_OK);
}

// TODO:TestRight
bool EntityCatalogWidget::_canDiscardSaveState(QModelIndex index, ItemStates *item, bool isAdmin)
{
    return (index.isValid() && item->indexType == EntityCatalogModel::Entity
            && (item->isWriteable || isAdmin) && item->isSaved
            && item->isValid && !item->isLocked && !item->isSyncing && !item->isModel);
}

bool EntityCatalogWidget::_canExport(QModelIndex index, ItemStates *item, bool isAdmin)
{
    return (index.isValid() && item->indexType == EntityCatalogModel::Entity
            && isAdmin && item->isValid
            && !item->isLocked && !item->isSyncing && !item->isModel
            && item->status == Hns::ENTITY_STATUS_OK
            && item->isLinkedClone == false);
}

int EntityCatalogWidget::_parseSelectedItems(QList<qlonglong> &validFolderIDList, QHash<qlonglong, QList<qlonglong> > &validEntityHash, bool orphansAreValid)
{
    QModelIndexList triggerIndexes = selectedItemIndexes();
    QModelIndexList invalidIndexes;
    int validCount = 0;
    // TODO:TestRight
//    bool isAdmin = _session->user()->isAdmin;
    foreach(QModelIndex index, triggerIndexes) {
        struct ItemStates item;
        _parseCatalogItem(index, &item);
        // TODO:TestRight
        if ((item.isWriteable /*|| isAdmin*/) && !item.isSyncing && !item.isLocked) {
            qlonglong itemID = index.data(EntryItem::IDRole).toLongLong();
            if (item.indexType == EntityCatalogModel::Folder)
                validFolderIDList.append(itemID);
            else if (item.indexType == EntityCatalogModel::Entity) {
                qlonglong parentID = index.parent().data(EntryItem::IDRole).toLongLong();
                if (orphansAreValid || parentID != 0) {
                    validEntityHash[itemID].append(parentID);
                    validCount++;
                } else
                    invalidIndexes.append(index);
            } else
                invalidIndexes.append(index);
        }
    }
    validCount += validFolderIDList.count();
    return validCount;
}

/*******************************************************************************
* EntityCatalogWidget - Private slots
*/
void EntityCatalogWidget::_beginTransactionSlot(QPersistentModelIndex sourceIndex)
{
    // emit signal for animation delegate
    QModelIndex proxyIdx = _linkedClonesFilterProxyModel->mapFromSource(_proxyModel->mapFromSource(sourceIndex));
    emit beginTransaction(QPersistentModelIndex(proxyIdx));
}

void EntityCatalogWidget::_focusOnItemSlot(QModelIndex index)
{
    // Expand parent and select added item
    QModelIndex proxyIndex = _linkedClonesFilterProxyModel->mapFromSource(_proxyModel->mapFromSource(index));
    if (proxyIndex.isValid()) {
        ui->catalogTreeView->expand(proxyIndex.parent());
        ui->catalogTreeView->selectionModel()->setCurrentIndex(proxyIndex, QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows);
    }
}

void EntityCatalogWidget::_collapsedSlot(QModelIndex index)
{
    // block collapse for root node
    if (!index.parent().isValid())
        ui->catalogTreeView->expand(index);
    ui->catalogTreeView->setColumnWidth(0, ui->catalogTreeView->viewport()->width());
}

void EntityCatalogWidget::_expandedSlot(QModelIndex)
{
    ui->catalogTreeView->setColumnWidth(0, ui->catalogTreeView->viewport()->width());
}

void EntityCatalogWidget::_catalogSelectionChanged(QItemSelection,QItemSelection)
{
    _toggleActions();
}

void EntityCatalogWidget::_contextMenuRequested(QPoint pos)
{
    QModelIndex targetIdx = ui->catalogTreeView->indexAt(pos);
    if (targetIdx.isValid()) {
        QMenu contextMenu;
        QModelIndexList selected = selectedItemIndexes();
        if (selected.count() > 1) {
            foreach(QModelIndex selIndex, selected) {
                if (getItemInternalType(selIndex) == EntityCatalogModel::Entity) {
                    contextMenu.addAction(ui->action_Clone_Entity);
                    contextMenu.addAction(ui->action_Build_Model_From);
                    contextMenu.addAction(ui->action_Discard_save_state);
                    contextMenu.addAction(ui->action_Remove_from_category);
                    break;
                }
            }
            contextMenu.addAction(ui->action_Delete);
        } else {
            int itemType = getItemInternalType(targetIdx);
            if (itemType == EntityCatalogModel::VirtualFolder || itemType == EntityCatalogModel::Folder ) {
                contextMenu.addAction(ui->action_Add_category);
            }
            if (itemType != EntityCatalogModel::VirtualFolder) {
                if (itemType == EntityCatalogModel::Entity) {
                    contextMenu.addAction(ui->action_Clone_Entity);
                    contextMenu.addAction(ui->action_Export);
                    contextMenu.addAction(ui->action_Build_Model_From);
                    contextMenu.addAction(ui->action_Discard_save_state);
                }
            }
            if (itemType != EntityCatalogModel::VirtualFolder) {
                contextMenu.addAction(ui->action_Rename);
                contextMenu.addSeparator();
                if (itemType == EntityCatalogModel::Entity) {
                    contextMenu.addAction(ui->action_Set_custom_graphic);
                    contextMenu.addAction(ui->action_Remove_from_category);
                }
                contextMenu.addAction(ui->action_Delete);
                contextMenu.addSeparator();
                contextMenu.addAction(ui->action_Properties);
            }
        }
        if (contextMenu.actions().count())
            contextMenu.exec(ui->catalogTreeView->viewport()->mapToGlobal(pos));
    }
}

void EntityCatalogWidget::_modelDataChangedSlot(QModelIndex , QModelIndex )
{
    // TODO : Find another way to invalidate actions on locked/unlocked changes
    _toggleActions();
}

void EntityCatalogWidget::on_catalogFilterEdit_textChanged(QString currentText)
{
    _proxyModel->setFilterFixedString(currentText);
    if (currentText != "")
        ui->catalogTreeView->expandAll();
    else {
        QModelIndex fakeRoot = ui->catalogTreeView->model()->index(0,0, QModelIndex());
        ui->catalogTreeView->expand(fakeRoot );
    }
}

void EntityCatalogWidget::on_action_Add_category_triggered()
{
    QModelIndex triggerIdx = selectedItemIndex();
    if (!triggerIdx.isValid() || ! ui->catalogTreeView->selectionModel()->isSelected(triggerIdx))
        triggerIdx = ui->catalogTreeView->model()->index(0,0, QModelIndex());
    addFolder(triggerIdx);
}

void EntityCatalogWidget::on_action_Rename_triggered()
{
    QModelIndex triggerIdx = selectedItemIndex();
    if (triggerIdx.isValid())
        ui->catalogTreeView->edit(triggerIdx);
}

void EntityCatalogWidget::on_action_Collapse_All_triggered()
{
    ui->catalogTreeView->collapseAll();
    ui->catalogTreeView->expand(QModelIndex (_linkedClonesFilterProxyModel->index(0,0)));
}

void EntityCatalogWidget::on_action_Expand_All_triggered()
{
    ui->catalogTreeView->expandAll();
}

void EntityCatalogWidget::on_action_Properties_triggered()
{
    if(selectedItemIndexes().count() == 1) {
        QModelIndex triggerIdx = selectedItemIndex();
        if (triggerIdx.isValid())
            editItem(triggerIdx);
    }
}

void EntityCatalogWidget::on_action_Clone_Entity_triggered()
{
    QModelIndexList triggerIndexes = selectedItemIndexes();
    QList<qlonglong> entityList;
    foreach(QModelIndex index, triggerIndexes) {
        struct ItemStates item;
        _parseCatalogItem(index, &item);
        if (!item.isSyncing && !item.isLocked && !item.isSaved && item.isValid && item.indexType == EntityCatalogModel::Entity)
            entityList.append(index.data(EntryItem::IDRole).toLongLong());
    }
    bool showPopup = false;
    foreach (qlonglong entityEntryId, entityList) {
        showPopup |= Hns::canHandleLinkedClone(_catalog->entityEntryById(entityEntryId)->entityEntry());
    }
    bool linkedClones = false;
    bool validateAnswer = false;
    if (showPopup) {
        EntityCloneDialog cloneDialog(entityList.size(), this);
        validateAnswer = (cloneDialog.exec() == QDialog::Accepted);
        linkedClones = cloneDialog.isLinkedCloneChecked();
    } else {
        QString message = tr("Do you really want to clone selected entity ?");
        if (entityList.size() > 1)
            message = tr("Do you really want to clone selected entities ?");
        validateAnswer = (QMessageBox::question(this, tr("Confirmation requested"), message, QMessageBox::No | QMessageBox::Yes, QMessageBox::Yes) == QMessageBox::Yes);
    }
    if (validateAnswer) {
        foreach (qlonglong entityEntryId, entityList) {
            _catalog->cloneEntityEntry(entityEntryId, linkedClones ? Hns::LINKED_CLONE : 0);
        }
    }
}

void EntityCatalogWidget::on_action_Delete_triggered()
{
    QList<qlonglong> validFolderIDList;
    QHash<qlonglong, QList<qlonglong> > validEntityHash;
    int validCount = _parseSelectedItems(validFolderIDList, validEntityHash, true);
    if (validCount > 0) {
        QString message = tr("Do you really want to delete selected item ?");
        if (validCount > 1)
            message = tr("Do you really want to delete selected items ?");
        if (validEntityHash.size() > 0)
            message.append(tr("\n\nWARNING: This operation can not be undone.\nAny ressources on disk will be deleted permanently."));
        int question = QMessageBox::warning(this, tr("Confirmation required"), message, QMessageBox::No | QMessageBox::Yes, QMessageBox::No);
        if (question == QMessageBox::Yes) {
            _catalog->deleteItemList(validFolderIDList, validEntityHash);
        }
    } else {
        // TODO: display error message
    }
}

void EntityCatalogWidget::on_action_Remove_from_category_triggered()
{
    QList<qlonglong> validFolderIDList;
    QHash<qlonglong, QList<qlonglong> > validEntityHash;
    int validCount = _parseSelectedItems(validFolderIDList, validEntityHash, false);
    validCount -= validFolderIDList.count();

    if (validCount > 0) {
        QString message = tr("Do you really want to remove selected item ?");
        if (validCount > 1)
            message = tr("Do you really want to remove selected items ?");
        int question = QMessageBox::question(this, tr("Confirmation required"), message, QMessageBox::No | QMessageBox::Yes, QMessageBox::Yes);
        if (question == QMessageBox::Yes) {
            QList<qlonglong> entriesId = validEntityHash.keys();
            foreach (qlonglong entryId, entriesId) {
                HnsClient::Entry *entry = _catalog->entityEntryById(entryId);
                HnsClient::EntityEntry *newEntry = qobject_cast<HnsClient::EntityEntry *>(entry->duplicateEntry(this));
                newEntry->setParentID(0);
                _catalog->updateEntityEntry(newEntry);
            }
        }
    } else {
        // TODO: display error message
    }
}

void EntityCatalogWidget::on_action_Build_Model_From_triggered()
{
    QModelIndexList triggerIndexes = selectedItemIndexes();
    QList<qlonglong> entityList;
    foreach(QModelIndex index, triggerIndexes) {
        struct ItemStates item;
        _parseCatalogItem(index, &item);
        if (!item.isSyncing && !item.isLocked && !item.isSaved && item.isValid && item.indexType == EntityCatalogModel::Entity && !item.isModel) {
            entityList.append(index.data(EntryItem::IDRole).toLongLong());
        }
    }

    if (entityList.size() > 0) {
        QString message = tr("Do you really want to build model from selected entity ?");
        if (entityList.size() > 0)
            message = tr("Do you really want to build models from selected entities ?");
        int question = QMessageBox::question(this, tr("Confirmation required"), message, QMessageBox::No | QMessageBox::Yes, QMessageBox::Yes);
        if (question == QMessageBox::Yes) {
            foreach(qlonglong entityEntryID, entityList)
                _catalog->buildModelFromEntityEntry(entityEntryID);
        }
    }
}

void EntityCatalogWidget::on_action_Discard_save_state_triggered()
{
    QModelIndexList triggerIndexes = selectedItemIndexes();
    QList<qlonglong> entityList;
    foreach(QModelIndex index, triggerIndexes) {
        struct ItemStates item;
        _parseCatalogItem(index, &item);
        if (!item.isSyncing && !item.isLocked && item.isSaved && item.isValid && item.indexType == EntityCatalogModel::Entity && !item.isModel) {
            entityList.append(index.data(EntryItem::IDRole).toLongLong());
        }
    }

    if (entityList.size() > 0) {
        QString message = tr("Do you really want to discard the save state from the selected entity ?");
        if (entityList.size() > 0)
            message = tr("Do you really want to discard the save state from the selected entities ?");
        int question = QMessageBox::question(this, tr("Confirmation required"), message, QMessageBox::No | QMessageBox::Yes, QMessageBox::No);
        if (question == QMessageBox::Yes) {
            foreach(qlonglong entityEntryID, entityList) {
                _catalog->discardSaveState(entityEntryID);
            }
        }
    }
}

void EntityCatalogWidget::on_action_Set_custom_graphic_triggered()
{
    QModelIndex triggerIdx = selectedItemIndex();
    if (triggerIdx.isValid()) {
        int itemType = getItemInternalType(triggerIdx);
        if (itemType == EntityCatalogModel::Entity) {
            GraphicSelectorDialog dialog(HyneviewSession::instance()->resourceManager()->getResources(), this);
            if (dialog.exec() == QDialog::Accepted) {
                qlonglong itemID = triggerIdx.data(EntryItem::IDRole).toLongLong();
                HnsClient::EntityEntry *entry = _catalog->entityEntryById(itemID);
                if (entry)
                    _catalog->setCustomGraphic(entry->getID(), dialog.selectedFileName());
            }
        }
    }
}

void EntityCatalogWidget::on_action_Export_triggered()
{
    QModelIndex triggerIdx = selectedItemIndex();
    if (triggerIdx.isValid()) {
        int itemType = getItemInternalType(triggerIdx);
        if (itemType == EntityCatalogModel::Entity) {
            qlonglong itemID = triggerIdx.data(EntryItem::IDRole).toLongLong();
            HnsClient::EntityEntry *entry = _catalog->entityEntryById(itemID);
            if (entry) {
                QString message = tr("Are you sure you want to export this entity ?\nMake sure that you have enough free disk space available.");
                int clickedButton = QMessageBox::question(this, tr("Confirmation required"), message, QMessageBox::No | QMessageBox::Yes, QMessageBox::Yes);
                if (clickedButton == QMessageBox::Yes)
                    HyneviewSession::instance()->catalogImporter()->exportEntityEntry(entry, this, SLOT(_exportFinished(bool)));
            }
        }
    }
}

void EntityCatalogWidget::on_action_Show_linked_domains_triggered(bool checked)
{
    if (!checked) {
        _linkedClonesFilterProxyModel->setUuid("");
        return;
    }
    QModelIndex triggerIdx = selectedItemIndex();
    if (triggerIdx.isValid()) {
        int itemType = getItemInternalType(triggerIdx);
        if (itemType == EntityCatalogModel::Entity) {
            QString uuid = triggerIdx.data(EntityItem::UuidRole).toString();
            _linkedClonesFilterProxyModel->setUuid(uuid);
        }
    }
}

void EntityCatalogWidget::_exportFinished(bool result)
{
    if (result)
        QMessageBox::information(this, tr("Success"), tr("Export has succeeded."), QMessageBox::Ok);
    else
        QMessageBox::critical(this, tr("Failure"), tr("Export has failed."), QMessageBox::Ok);
}

} // namespace Hnv
