/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "entitymonitorwidget.h"
#include "simulationmodewidget.h"
#include "topologywidget.h"
#include "hyneviewsession.h"

// hynesim hnsclient headers
#include <libhnsclient/platformmonitor.h>
#include <libhnsclient/entity.h>
#include <libhnsclient/utility.h>

// Qt headers
#include <QTreeView>

namespace Hnv {

/*******************************************************************************
* EntityMonitoringNode
*******************************************************************************/
EntityMonitoringNode::EntityMonitoringNode(QString key, QString unit, QObject *parent)
    : TreeModelItem(NODE_COLUMN_COUNT, parent)
    , _key(key)
    , _value(QString())
    , _unit(unit)
{
}

EntityMonitoringNode::~EntityMonitoringNode()
{
}

Qt::ItemFlags EntityMonitoringNode::flags(int column) const
{
    return TreeModelItem::flags(column);
}

QVariant EntityMonitoringNode::data(int column, int role) const
{
    switch (role) {
    case Qt::TextAlignmentRole:
        if (column == COLUMN_KEY)
            return Qt::AlignLeft;
        if (column == COLUMN_VALUE)
            return Qt::AlignLeft;
    case Qt::DisplayRole: {
        if (column == COLUMN_KEY)
            return _key;
        if (column == COLUMN_VALUE) {
            return QString("%1 %2").arg(_value).arg(_unit, -10);
        }
        break;
    }
    default:
        break;
    }
    return TreeModelItem::data(column, role);
}

void EntityMonitoringNode::updateValue(QString value, QString unit)
{
    if (!unit.isEmpty())
        _unit = unit;
    _value = value;
    columnChanged(COLUMN_VALUE);
}


/*******************************************************************************
* EntityMonitoringModel
*******************************************************************************/
EntityMonitoringModel::EntityMonitoringModel(QWidget *parent)
    : TreeModel(parent)
    , _entityId(0)
    , _monitor(0)
{
    _initRootItem(EntityMonitoringNode::NODE_COLUMN_COUNT);
}

EntityMonitoringModel::~EntityMonitoringModel()
{
}

void EntityMonitoringModel::setEntity(HnsClient::GenericEntity *entity)
{
    if (entity && entity->getId() == _entityId)
        return;
    beginResetModel();
    _rootItem->clear();
    _nodeHash.clear();
    if (_monitor)
        disconnect(_monitor, 0, this, 0);
    _monitor = 0;
    _entityId = 0;

    if (entity && entity->isDefined()) {
        HyneviewSession *session = HyneviewSession::instance();
        if (session) {
            HnsClient::NodeInformation *nodeInfo = session->platformMonitor()->getNodeInformation(entity->getNodeName());
            if (nodeInfo) {
                HnsClient::NodeMonitor *monitor = nodeInfo->getNodeMonitor();
                if (monitor) {
                    Hns::EntityMonitoringDataPtr data = monitor->getEntityMonitoringData(entity->getId());
                    Hns::DomainMonitoringDataPtr domainData = Hns::DomainMonitoringDataPtr::dynamicCast(data);
                    if (domainData) {
                        _monitor = monitor;
                        _entityId = entity->getId();
                        connect(_monitor, SIGNAL(entityMonitoringDataUpdated()), this, SLOT(_monitorDataUpdated()));
                        connect(_monitor, SIGNAL(destroyed()), this, SLOT(_monitorDestroyedSlot()));
                        _updateDomainData(domainData);
                    }
                }
            }
        }
    }
    endResetModel();
}

QVariant EntityMonitoringModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if (role == Qt::DisplayRole && orientation == Qt::Horizontal) {
        switch (section) {
        case EntityMonitoringNode::COLUMN_KEY:
            return tr("Property");
            break;
        case EntityMonitoringNode::COLUMN_VALUE:
            return tr("Value");
            break;
        default:
            break;
        }
    }
    return TreeModel::headerData(section, orientation, role);
}

void EntityMonitoringModel::_monitorDataUpdated()
{
    Hns::EntityMonitoringDataPtr data = _monitor->getEntityMonitoringData(_entityId);
    Hns::DomainMonitoringDataPtr domainData = Hns::DomainMonitoringDataPtr::dynamicCast(data);
    if (domainData) {
        _updateDomainData(domainData);
    } else {
        setEntity(0);
    }
}

void EntityMonitoringModel::_updateDomainData(Hns::DomainMonitoringDataPtr domainData)
{
    EntityMonitoringNode *node = 0;
    if (domainData->cpuData.guestCPU != -1) {
        node = _nodeByKey(tr("CPU Usage"), "%");
        node->updateValue(QString::number(domainData->cpuData.guestCPU, 'f', 2));
    }
    if (domainData->cpuData.hostCPU != -1) {
        node = _nodeByKey(tr("Host CPU Usage"), "%");
        node->updateValue(QString::number(domainData->cpuData.hostCPU, 'f', 2));
    }
    if (domainData->memoryData.totalSize != -1) {
        node = _nodeByKey(tr("Allocated Memory"), "B");
        _updateSizeNode(node, domainData->memoryData.totalSize, HnsClient::KibiBytes);
    }
    if (domainData->memoryData.used != -1) {
        node = _nodeByKey(tr("Used Memory"), "B");
        _updateSizeNode(node, domainData->memoryData.used, HnsClient::KibiBytes);
    }
    if (domainData->memoryData.available != -1) {
        node = _nodeByKey(tr("Available Memory"), "B");
        _updateSizeNode(node, domainData->memoryData.available, HnsClient::KibiBytes);
    }

    QMap<std::string, Hns::NetworkMonitoringData> networkDataMap(domainData->networkDataMap);
    QMapIterator<std::string, Hns::NetworkMonitoringData> networkIterator(networkDataMap);
    while(networkIterator.hasNext()) {
        networkIterator.next();
        Hns::NetworkMonitoringData data = networkIterator.value();
        QString deviceName = QString::fromStdString(networkIterator.key());
        node = _nodeByKey(tr("%1: Input rate").arg(deviceName), "B/s");
        _updateSizeNode(node, data.rRate, HnsClient::Bytes, true);
        node = _nodeByKey(tr("%1: Output rate").arg(deviceName), "B/s");
        _updateSizeNode(node, data.tRate, HnsClient::Bytes, true);
        node = _nodeByKey(tr("%1: Received").arg(deviceName), "B");
        _updateSizeNode(node, data.received, HnsClient::Bytes);
        node = _nodeByKey(tr("%1: Transmitted").arg(deviceName), "B");
        _updateSizeNode(node, data.transmitted, HnsClient::Bytes);
    }
    QMap<std::string, Hns::IOMonitoringData> diskDataMap(domainData->ioDataMap);
    QMapIterator<std::string, Hns::IOMonitoringData> diskIterator(diskDataMap);
    while(diskIterator.hasNext()) {
        diskIterator.next();
        Hns::IOMonitoringData data = diskIterator.value();
        QString deviceName = QString::fromStdString(diskIterator.key());
        node = _nodeByKey(tr("%1: Read rate").arg(deviceName), "B/s");
        _updateSizeNode(node, data.rRate, HnsClient::Bytes, true);
        node = _nodeByKey(tr("%1: Write rate").arg(deviceName), "B/s");
        _updateSizeNode(node, data.wRate, HnsClient::Bytes, true);
        node = _nodeByKey(tr("%1: Read total").arg(deviceName), "B");
        _updateSizeNode(node, data.rBytes, HnsClient::Bytes);
        node = _nodeByKey(tr("%1: Write total").arg(deviceName), "B");
        _updateSizeNode(node, data.wBytes, HnsClient::Bytes);
    }
}

EntityMonitoringNode *EntityMonitoringModel::_nodeByKey(QString key, QString unit)
{
    EntityMonitoringNode *node = 0;
    if (!_nodeHash.contains(key)) {
        node = new EntityMonitoringNode(key, unit, this);
        _nodeHash.insert(key, node);
        _rootItem->appendChild(node);
    } else
        node = _nodeHash.value(key);
    return node;
}

void EntityMonitoringModel::_updateSizeNode(EntityMonitoringNode *node, qreal value, HnsClient::BytesUnitEnum unit, bool asRate)
{
    QPair<QString, HnsClient::BytesUnitEnum> sizeValue = HnsClient::Utility::buildReadableSize(value, unit);
    node->updateValue(sizeValue.first, HnsClient::Utility::unitFromIndex(sizeValue.second, asRate));
}

void EntityMonitoringModel::_monitorDestroyedSlot()
{
    _monitor = 0;
}


/*******************************************************************************
* EntityMonitorWidget
*******************************************************************************/
EntityMonitorWidget::EntityMonitorWidget(QWidget *parent)
    : SideBarTreeWidget(parent)
    , _monitoringModel(new EntityMonitoringModel(this))
    , _currentEntity(0)
{
    _treeView->setSelectionBehavior(QAbstractItemView::SelectRows);
    _treeView->header()->setVisible(true);
    QWidget *widget = sideBarParent();
    if (widget) {
        SimulationModeWidget *simulationModeWidget = qobject_cast<SimulationModeWidget *>(widget);
        if (simulationModeWidget) {
            TopologyWidget *topologyWidget = simulationModeWidget->topologyWidget();
            if (topologyWidget) {
                setViewModel(_monitoringModel);
                connect(topologyWidget, SIGNAL(selectionChanged(QList<HnsClient::GenericEntity*>)), this, SLOT(_selectionChangedSlot(QList<HnsClient::GenericEntity*>)));
                connect(topologyWidget, SIGNAL(topologyChanged(HnsClient::Topology*)), this, SLOT(_topologyChangedSlot()));
                _selectionChangedSlot(topologyWidget->getCurrentEntitiesSelected(true));
            }
        }
    }
}

EntityMonitorWidget::~EntityMonitorWidget()
{
    _currentEntity = 0;
    clearModel();
}

QString EntityMonitorWidget::title()
{
    return tr("Monitoring");
}

QString EntityMonitorWidget::sideBarId()
{
    return "Monitoring";
}

void EntityMonitorWidget::_indexClicked(const QModelIndex &index)
{
    if (index.isValid()) {
    }
}

void EntityMonitorWidget::_indexDoubleClicked(const QModelIndex &index)
{
    if (index.isValid()) {
    }
}

void EntityMonitorWidget::_contextMenuRequested(const QModelIndex &index, const QPoint &position)
{
    Q_UNUSED(position)
    if (index.isValid()) {
    }
}

void EntityMonitorWidget::_setEntity(HnsClient::GenericEntity *entity)
{
    if (entity && _currentEntity && entity->getId() == _currentEntity->getId())
        return;

    if (_currentEntity)
        disconnect(_currentEntity, 0, this, 0);
    _currentEntity = entity;
    if (_currentEntity) {
        connect(_currentEntity, SIGNAL(destroyed()), this, SLOT(_currentEntityDestroyed()));
        connect(_currentEntity, SIGNAL(statusChanged(HnsClient::GenericEntity*)), this, SLOT(_currentEntityStatusChanged()));
    }
    _monitoringModel->setEntity(_currentEntity);
}

void EntityMonitorWidget::_selectionChangedSlot(QList<HnsClient::GenericEntity *> selectedEntities)
{
    if (selectedEntities.count() == 1)
        _setEntity(selectedEntities.first());
    else if (selectedEntities.count() == 0)
        _setEntity(0);
    _treeView->resizeColumnToContents(0);
}

void EntityMonitorWidget::_topologyChangedSlot()
{
    SimulationModeWidget *simulationModeWidget = SimulationModeWidget::instance();
    if (simulationModeWidget) {
        TopologyWidget *topologyWidget = simulationModeWidget->topologyWidget();
        if (topologyWidget) {
            _selectionChangedSlot(topologyWidget->getCurrentEntitiesSelected(true));
        }
    }
}

void EntityMonitorWidget::_currentEntityDestroyed()
{
    _setEntity(0);
}

void EntityMonitorWidget::_currentEntityStatusChanged()
{
    if (_currentEntity->isDefined())
        _monitoringModel->setEntity(_currentEntity);
}

} // namespace Hnv
