/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "domainnetworkpage.h"
#include "ui_domainnetworkpage.h"
#include "ressources.h"
#include "domainnetworkcardwidget.h"

// hynesim hnsclient headers
#include <libhnsclient/entry.h>

// Qt headers
#include <QToolButton>
#include <QLabel>

namespace Hnv {

/*******************************************************************************
* DomainNetworkPage
*/
DomainNetworkPage::DomainNetworkPage(HnsClient::CatalogItem *entry,Hns::EntryPropertiesPtr entryProperties, QWidget *parent, bool readOnly, bool defined)
    : EntryPropertiesPage(entry,entryProperties, parent, readOnly , defined)
    , ui(new Ui::DomainNetworkPage)
    , _noValidCards(false)
    , _maxNetworkCards(0)
{
    ui->setupUi(this);
    ui->amComboxInterface->addItem(tr("None"), -1);
    QToolButton * addButton = new QToolButton(this);
    addButton->setIcon(QIcon(Ressources::themedPath(Constants::ICON_CREATE_NETCARD)));
    addButton->setToolTip(tr("Add new network card"));
    ui->tabWidget->setCornerWidget(addButton);
    connect(addButton, SIGNAL(clicked()), this, SLOT(_addNetcard()));
    addButton->setDisabled( (_readOnly || (_defined && (!_canModifiedWhenDefined))));
    _properties = entryProperties;
    _domainProperties = Hns::DomainEntryPropertiesPtr::dynamicCast(_properties);
    _maxNetworkCards = _domainProperties->nbNetworkCards;

    HnsClient::DomainEntry * domain = qobject_cast<HnsClient::DomainEntry *>(_sourceEntry);
    if (domain) {
        QMap<Ice::Long, Hns::Netcard> networkCards = QMap<Ice::Long, Hns::Netcard>(domain->getNetcardMap());
        if (networkCards.size() == 0) {
            _setNoCardsTab();
        }

        QMap<QString, QString> networkCardNamesMap;
        foreach (const Hns::Netcard &card, networkCards) {
            QString formatedPlugName(card.source.c_str());
            QRegExp regExp("([^0-9])([0-9])$");
            formatedPlugName.replace(regExp, "\\100\\2");
            regExp.setPattern("([^0-9])([0-9][0-9])$");
            formatedPlugName.replace(regExp, "\\10\\2");
            networkCardNamesMap[formatedPlugName] = QString::fromStdString(card.source);
        }
        foreach (QString name,networkCardNamesMap){
            foreach (const Hns::Netcard &card, networkCards)
            {
                if (QString::fromStdString(card.source) == name){
                    _initNetcardTab(card);
                }
            }
        }
    } else {
        addButton->setEnabled(false);
    }
    _disableWidgetsWhenNecessary(!isWriteable() || _readOnly || (_defined && !_canModifiedWhenDefined));
}

DomainNetworkPage::~DomainNetworkPage()
{
    delete ui;
}

void DomainNetworkPage::initFocus()
{
}

QToolButton *DomainNetworkPage::getPageButton()
{
    return _generatePageButton(tr("Network"), QIcon(Ressources::themedPath(Constants::ICON_CONNECT)));
}

void DomainNetworkPage::_disableWidgetsWhenNecessary(bool necessary)
{
    ui->amComboxInterface->setDisabled(necessary);
    int i;
    for (i = 0; i != ui->tabWidget->count(); i++) {
        DomainNetworkCardWidget *page = getNetworkCardWidget(i);
        if (page)
            page->setDisabled(necessary);
    }
}

void DomainNetworkPage::_validate(bool &isModified, bool &isInvalid)
{
    HnsClient::ValidationResult result;
    QMap<Ice::Long, Hns::Netcard> networkCardMap;

    int i = 0;
    if (_noValidCards == true) {
        HnsClient::DomainEntry * domain = qobject_cast<HnsClient::DomainEntry *>(_sourceEntry);
        domain->validateActionManagerInterfaceID(-1);
        result = domain->validateNetworkCardMap(networkCardMap, _errorNetworkCardList);
        _computeValidationState(result,isModified,isInvalid);
        return;
    }
    for (i = 0; i != ui->tabWidget->count(); i++) {
        DomainNetworkCardWidget *page = getNetworkCardWidget(i);
        Hns::Netcard card = page->getCardInfo();
        networkCardMap.insert(card.id, card);
    }
    HnsClient::DomainEntry * domain = qobject_cast<HnsClient::DomainEntry *>(_sourceEntry);
    if (domain) {
        result = domain->validateNetworkCardMap(networkCardMap, _errorNetworkCardList);
        _computeValidationState(result, isModified, isInvalid);
        if (isInvalid) {
            for (i = 0; i != ui->tabWidget->count(); i++) {
                DomainNetworkCardWidget *page = getNetworkCardWidget(i);
                page->clearErrors();
                Hns::Netcard card = page->getCardInfo();
                int error = _errorNetworkCardList.value(card.id);
                if (error != 0) {
                    ui->tabWidget->setCurrentIndex(i);
                    page->displayErrors(error);
                }
            }
        }
        result = domain->validateActionManagerInterfaceID(ui->amComboxInterface->itemData(ui->amComboxInterface->currentIndex()).toLongLong());
        _computeValidationState(result, isModified, isInvalid);
    } else {
        return;
    }
}

DomainNetworkCardWidget* DomainNetworkPage::_addNetcardTab(QString label)
{
    if(!isWriteable() || _readOnly || (_defined && !_canModifiedWhenDefined))
        ui->tabWidget->setTabsClosable(false);

    HnsClient::DomainEntry * domain = qobject_cast<HnsClient::DomainEntry *>(_sourceEntry);
    DomainNetworkCardWidget * networkWidget = new DomainNetworkCardWidget(this, domain->getType());
    if (_noValidCards == true) {
        delete ui->tabWidget->currentWidget();
        ui->tabWidget->removeTab(0);
        ui->tabWidget->setTabsClosable(true);
        _noValidCards = false;
    }
    if (_maxNetworkCards > -1) {
        if (ui->tabWidget->count() < _maxNetworkCards) {
            ui->tabWidget->addTab(networkWidget, label);
            if(ui->tabWidget->count()==_maxNetworkCards ){
                ui->tabWidget->cornerWidget()->setEnabled(false);
            }
        }
    } else {
        ui->tabWidget->addTab(networkWidget, label);
    }
    return networkWidget;
    return 0;
}

DomainNetworkCardWidget *DomainNetworkPage::getNetworkCardWidget(int index)
{
    DomainNetworkCardWidget *result = 0;
    result = dynamic_cast<DomainNetworkCardWidget*>(ui->tabWidget->widget(index));
    return result;
}

void DomainNetworkPage::_initNetcardTab(Hns::Netcard card)
{
    DomainNetworkCardWidget *networkWidget = _addNetcardTab(QString::fromStdString(card.source));
    networkWidget->setCardInfo(card, _domainProperties);

    ui->amComboxInterface->addItem(QString::fromStdString(card.source), QVariant::fromValue(card.id));
    HnsClient::DomainEntry *domain = qobject_cast<HnsClient::DomainEntry *>(_sourceEntry);
    if (card.id == domain->getActionManagerInterfaceID())
        ui->amComboxInterface->setCurrentIndex(ui->amComboxInterface->findData(QVariant::fromValue(card.id)));
}

void DomainNetworkPage::_initNetcardTab(Hns::Netcard card, DomainNetworkCardWidget *tab){
    tab->setCardInfo(card, _domainProperties);
    ui->amComboxInterface->addItem(QString::fromStdString(card.source), QVariant::fromValue(card.id));
}

void DomainNetworkPage::on_tabWidget_tabCloseRequested(int index)
{
    int comboItemIndex = ui->amComboxInterface->findText(ui->tabWidget->tabText(index));
    if (comboItemIndex == ui->amComboxInterface->currentIndex())
        ui->amComboxInterface->setCurrentIndex(0);
    ui->amComboxInterface->removeItem(comboItemIndex);
    ui->tabWidget->removeTab(index);
    if (ui->tabWidget->count()<_maxNetworkCards){
        ui->tabWidget->cornerWidget()->setEnabled(true);
    }
    if (ui->tabWidget->count()==0){
        _setNoCardsTab();
    }
}

void DomainNetworkPage::_setNoCardsTab()
{
    ui->tabWidget->setTabsClosable(false);
    QLabel *labelNoNetworkCards = new QLabel(tr("No network cards"), this);
    ui->tabWidget->addTab(labelNoNetworkCards, tr("No cards"));
    _noValidCards = true;
}

DomainNetworkCardWidget * DomainNetworkPage::_addNetcard() {
    Ice::Long netcardId = 0;
    if (!_noValidCards) {
        for (int i = 0; i != ui->tabWidget->count(); i++) {
            DomainNetworkCardWidget *page = getNetworkCardWidget(i);
            QString source = QString::fromStdString(page->getCardInfo().source);
            QRegExp rx("^eth(\\d+)$");
            if (rx.exactMatch(source)) {
                Ice::Long id = rx.cap(1).toLongLong();
                if (id >= netcardId)
                    netcardId = id + 1;
            }
        }
    }
    Hns::Netcard card;
    card.source = QString("eth%1").arg(netcardId).toStdString();
    card.id = netcardId;
    DomainNetworkCardWidget *networkWidget = _addNetcardTab(QString::fromStdString(card.source));
    networkWidget->initNewCard();
    _initNetcardTab(card, networkWidget);
    ui->tabWidget->setCurrentWidget(networkWidget);
    return networkWidget;
}

} // namespace Hnv
