/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "domainpage.h"
#include "ui_domainpage.h"
#include "ressources.h"

// hynesim common headers
#include "common/hns.h"

// Generated hynesim headers
#include "common/domainentryproperties.h"

// hynesim hnsclient headers
#include <libhnsclient/entry.h>

// Qt headers
#include <QToolButton>

namespace Hnv {

QMultipleOfSpinBox::QMultipleOfSpinBox(QWidget *parent) : QSpinBox(parent),
    _multiple(0)
{
}

QMultipleOfSpinBox::~QMultipleOfSpinBox()
{
}

void QMultipleOfSpinBox::setMultiple(int multiple)
{
    _multiple = multiple;
}

void QMultipleOfSpinBox::fixup(QString &str) const
{
    str.setNum(Hns::getNearestMultipleOf(str.toInt(), _multiple));
}

QValidator::State QMultipleOfSpinBox::validate(QString &input, int &pos) const
{
    QValidator::State validatorState = QSpinBox::validate(input, pos);
    if ((validatorState == QValidator::Acceptable) && (_multiple != 0)) {
        if ((input.toInt() % _multiple) != 0) {
            validatorState = QValidator::Intermediate;
        }
    }
    return validatorState;
}

/*******************************************************************************
* DomainPage
*/
DomainPage::DomainPage(HnsClient::CatalogItem *entry,Hns::EntryPropertiesPtr entryProperties, QWidget *parent, bool readOnly, bool defined)
    : EntryPropertiesPage(entry,entryProperties, parent, readOnly, defined)
    , ui(new Ui::DomainPage)
{
    ui->setupUi(this);
    ui->machineTitle->setVisible(false);
    ui->machineLineEdit->setVisible(false);
    ui->kernelTitle->setVisible(false);
    ui->kernelLine->setVisible(false);
    ui->initrdTitle->setVisible(false);
    ui->initrdLine->setVisible(false);
    ui->cmdTitle->setVisible(false);
    ui->cmdLine->setVisible(false);
    ui->immutableTitle->setVisible(false);
    ui->immutableCheckBox->setVisible(false);
    ui->line_2->setVisible(false);
    ui->keymapTitle->setVisible(false);
    ui->keymapCBox->setVisible(false);
    ui->frame->setHidden(true);
    ui->usbRedirLabel->setVisible(false);
    ui->usbRedirSpinBox->setVisible(false);
    ui->usbControllerTypeLabel->setVisible(false);
    ui->usbControllerTypeComboBox->setVisible(false);
    ui->videoLabel->setVisible(false);
    ui->videoCombobox->setVisible(false);

    connect(ui->vcpuSpinBox, SIGNAL(valueChanged(int)), this, SLOT(updateApicValue(int)));

    HnsClient::DomainEntry * domain = qobject_cast<HnsClient::DomainEntry *>(_sourceEntry);
    if (domain) {
        Hns::EntryPropertiesPtr properties = entryProperties;
        Hns::DomainEntryPropertiesPtr domainProperties = Hns::DomainEntryPropertiesPtr::dynamicCast(properties);
        if (domainProperties->hasKeymap) {
            ui->keymapTitle->setVisible(true);
            ui->keymapCBox->setVisible(true);
            foreach (std::string layout, domainProperties->keymapList)
                ui->keymapCBox->addItem(QString::fromStdString(layout));
            ui->keymapCBox->setCurrentIndex(ui->keymapCBox->findText(QString::fromStdString(domain->getGraphics().keymap)));
        }
        foreach (std::string usbControllerType, domainProperties->usbControllerTypesList) {
            if (usbControllerType.empty())
                ui->usbControllerTypeComboBox->addItem(tr("Default"), QString::fromStdString(usbControllerType));
            else
                ui->usbControllerTypeComboBox->addItem(QString::fromStdString(usbControllerType), QString::fromStdString(usbControllerType));
        }
        foreach (std::string videoModelType, domainProperties->videoModelTypesList) {
            if (videoModelType.empty())
                ui->videoCombobox->addItem(tr("Default"), QString::fromStdString(videoModelType));
            else
                ui->videoCombobox->addItem(QString::fromStdString(videoModelType), QString::fromStdString(videoModelType));
        }
        if (domain->getType() == "qemu" || domain->getType() == "kvm") {
            ui->kernelTitle->setVisible(true);
            ui->kernelLine->setVisible(true);
            ui->initrdTitle->setVisible(true);
            ui->initrdLine->setVisible(true);
            ui->cmdTitle->setVisible(true);
            ui->cmdLine->setVisible(true);
            ui->immutableTitle->setVisible(true);
            ui->immutableCheckBox->setVisible(true);
            ui->line_2->setVisible(true);
            QRegExp rx("[^/]*");
            QValidator *validator = new QRegExpValidator(rx, this);
            ui->kernelLine->setText(QString::fromStdString(domain->getOperatingSystem().kernel));
            ui->kernelLine->setValidator(validator);
            ui->initrdLine->setText(QString::fromStdString(domain->getOperatingSystem().initrd));
            ui->initrdLine->setValidator(validator);
            ui->cmdLine->setText(QString::fromStdString(domain->getOperatingSystem().cmdline));
            ui->machineTitle->setVisible(true);
            ui->machineLineEdit->setVisible(true);
            ui->machineLineEdit->setText(QString::fromStdString(domain->getOperatingSystem().machine));
            ui->usbRedirLabel->setVisible(true);
            ui->usbRedirSpinBox->setVisible(true);
            ui->usbControllerTypeLabel->setVisible(true);
            ui->usbControllerTypeComboBox->setVisible(true);
            ui->videoLabel->setVisible(true);
            ui->videoCombobox->setVisible(true);
        }
        foreach (std::string arch, domainProperties->archList) {
            ui->archCBox->addItem(QString::fromStdString(arch));
            if (arch == domain->getOperatingSystem().arch)
                ui->archCBox->setCurrentIndex(ui->archCBox->findText(QString::fromStdString(domain->getOperatingSystem().arch)));
        }
        ui->vcpuHorizontalSlider->setMaximum(domainProperties->nbCpuMax);
        ui->vcpuHorizontalSlider->setTickInterval(ui->vcpuHorizontalSlider->maximum() / 32);
        ui->vcpuSpinBox->setMaximum(domainProperties->nbCpuMax);
        ui->memoryHorizontalSlider->setMaximum(domainProperties->memoryMax / 1024);
        ui->memoryHorizontalSlider->setTickInterval(ui->memoryHorizontalSlider->maximum() / 32);
        _memoryMultiple = domainProperties->memoryMultiple / 1024;
        ui->memorySpinBox->setSingleStep(_memoryMultiple);
        ui->memorySpinBox->setMaximum(domainProperties->memoryMax / 1024);
        ui->memorySpinBox->setMultiple(_memoryMultiple);
        _displayVideo(domain->getVideoModelType());
        _displayType(domain->getType());
        ui->nodeKeyLineEdit->setText(domain->getNodeKey());
        _displayMemory(domain->getMemory());
        _displayVCpu(domain->getVCpu());
        _displayPae(domain->getPae());
        _displayAcpi(domain->getAcpi());
        _displayApic(domain->getApic());
        _displayImmutable(domain->getImmutable());
        _displaySharedFolder(domain->getSharedFolder());
        _displayUsbRedirectionCount(domain->getUsbRedirectionCount());
        _displayUsbControllerType(domain->usbControllerType());
        connect(ui->memoryHorizontalSlider, SIGNAL(valueChanged(int)), this, SLOT(_getMemoryValue(int)));
        connect(ui->memorySpinBox, SIGNAL(valueChanged(int)), this, SLOT(_setMemoryValue(int)));
        connect(ui->vcpuHorizontalSlider, SIGNAL(valueChanged(int)), this, SLOT(_getVcpuValue(int)));
        connect(ui->vcpuSpinBox, SIGNAL(valueChanged(int)), this, SLOT(_setVcpuValue(int)));
        _apicOldValue = domain->getApic();
    } else {
        ui->memoryHorizontalSlider->setEnabled(false);
        ui->memorySpinBox->setEnabled(false);
        ui->vcpuHorizontalSlider->setEnabled(false);
        ui->vcpuSpinBox->setEnabled(false);
    }
    _disableWidgetsWhenNecessary(!isWriteable() || _readOnly || (_defined && !_canModifiedWhenDefined));
}

DomainPage::~DomainPage()
{
    delete ui;
}

void DomainPage::initFocus()
{
}

QToolButton *DomainPage::getPageButton()
{
    return _generatePageButton(tr("System"), QIcon(Ressources::themedPath(Constants::ICON_SYSTEM_SETTINGS)));
}

/*******************************************************************************
* DomainPage - Protected
*/
void DomainPage::_disableWidgetsWhenNecessary(bool necessary)
{
    ui->archCBox->setDisabled(necessary);
    ui->machineLineEdit->setDisabled(necessary);
    ui->kernelLine->setDisabled(necessary);
    ui->initrdLine->setDisabled(necessary);
    ui->cmdLine->setDisabled(necessary);
    ui->memoryHorizontalSlider->setDisabled(necessary);
    ui->memorySpinBox->setDisabled(necessary);
    ui->vcpuHorizontalSlider->setDisabled(necessary);
    ui->vcpuSpinBox->setDisabled(necessary);
    ui->paeCheckBox->setDisabled(necessary);
    ui->acpiCheckBox->setDisabled(necessary);
    ui->apicCheckBox->setDisabled(necessary);
    ui->immutableCheckBox->setDisabled(necessary);
    ui->sharedFolderCheckBox->setDisabled(necessary);
    ui->keymapCBox->setDisabled(necessary);
    ui->nodeKeyLineEdit->setDisabled(necessary);
    ui->usbRedirSpinBox->setDisabled(necessary);
    ui->usbControllerTypeComboBox->setDisabled(necessary);
    ui->videoCombobox->setDisabled(necessary);
}

void DomainPage::_displayType(QString type)
{
    ui->typeLabel->setText(type);
}

void DomainPage::_displayMemory(int memory)
{
    ui->memoryHorizontalSlider->setValue(memory/1024);
    ui->memorySpinBox->setValue(ui->memoryHorizontalSlider->value());
}

void DomainPage::_displayVCpu(int vcpu)
{
    ui->vcpuHorizontalSlider->setValue(vcpu);
    ui->vcpuSpinBox->setValue(ui->vcpuHorizontalSlider->value());
}

void DomainPage::_displayPae(bool pae)
{
    ui->paeCheckBox->setChecked(pae);

}

void DomainPage::_displayAcpi(bool acpi)
{
    ui->acpiCheckBox->setChecked(acpi);
}

void DomainPage::_displayApic(bool apic)
{
    ui->apicCheckBox->setChecked(apic);
}

void DomainPage::_displayImmutable(bool immutable)
{
    ui->immutableCheckBox->setChecked(immutable);
}

void DomainPage::_displaySharedFolder(bool sharedFolder)
{
    ui->sharedFolderCheckBox->setChecked(sharedFolder);
}

void DomainPage::_displayUsbRedirectionCount(int usbRedirectionCount)
{
    ui->usbRedirSpinBox->setValue(usbRedirectionCount);
}

void DomainPage::_displayUsbControllerType(QString usbControllerType)
{
    for (int i = 0 ; i < ui->usbControllerTypeComboBox->count() ; ++i) {
        if (ui->usbControllerTypeComboBox->itemData(i).toString() == usbControllerType) {
            ui->usbControllerTypeComboBox->setCurrentIndex(i);
            break;
        }
    }
}

void DomainPage::_displayVideo(QString videoModelType)
{
    for (int i = 0 ; i < ui->videoCombobox->count() ; ++i) {
        if (ui->videoCombobox->itemData(i).toString() == videoModelType) {
            ui->videoCombobox->setCurrentIndex(i);
            break;
        }
    }
}

void DomainPage::_validate(bool &isModified, bool &isInvalid)
{
    HnsClient::DomainEntry *entry = qobject_cast<HnsClient::DomainEntry *>(_sourceEntry);
    _computeValidationState(entry->validateArch(ui->archCBox->currentText()), isModified, isInvalid);
    _computeValidationState(entry->validateMachine(ui->machineLineEdit->text()), isModified, isInvalid);
    _computeValidationState(entry->validateOsKernel(ui->kernelLine->text()), isModified, isInvalid);
    _computeValidationState(entry->validateOsInitrd(ui->initrdLine->text()), isModified, isInvalid);
    _computeValidationState(entry->validateOsCmdLine(ui->cmdLine->text()), isModified, isInvalid);
    _computeValidationState(entry->validateMemory(ui->memoryHorizontalSlider->value()*1024), isModified, isInvalid);
    _computeValidationState(entry->validateVCpu(ui->vcpuSpinBox->value()), isModified, isInvalid);
    _computeValidationState(entry->validatePae(ui->paeCheckBox->isChecked()), isModified, isInvalid);
    _computeValidationState(entry->validateAcpi(ui->acpiCheckBox->isChecked()), isModified, isInvalid);
    _computeValidationState(entry->validateApic(ui->apicCheckBox->isChecked()), isModified, isInvalid);
    _computeValidationState(entry->validateImmutable(ui->immutableCheckBox->isChecked()), isModified, isInvalid);
    _computeValidationState(entry->validateSharedFolder(ui->sharedFolderCheckBox->isChecked()), isModified, isInvalid);
    _computeValidationState(entry->validateKeymap(ui->keymapCBox->currentText()), isModified, isInvalid);
    _computeValidationState(entry->validateNodeKey(ui->nodeKeyLineEdit->text()), isModified, isInvalid);
    _computeValidationState(entry->validateUsbRedirectionCount(ui->usbRedirSpinBox->value()), isModified, isInvalid);
    _computeValidationState(entry->validateUsbControllerType(ui->usbControllerTypeComboBox->itemData(ui->usbControllerTypeComboBox->currentIndex()).toString()), isModified, isInvalid);
    _computeValidationState(entry->validateVideoModelType(ui->videoCombobox->itemData(ui->videoCombobox->currentIndex()).toString()), isModified, isInvalid);
}

void DomainPage::updateApicValue(int value)
{
    if (value == 1 && _apicOldValue != true)
        _displayApic(_apicOldValue);
    else
        _displayApic(true);
}

/*******************************************************************************
* DomainPage - Private slots
*/

void DomainPage::on_memoryHorizontalSlider_actionTriggered(int action)
{
    if (action == QSlider::SliderMove)
        ui->memoryHorizontalSlider->setSliderPosition(Hns::getNearestMultipleOf(ui->memoryHorizontalSlider->sliderPosition(), _memoryMultiple));
}

void DomainPage::_getMemoryValue(int value)
{
    ui->memorySpinBox->setValue(value);;
}

void DomainPage::_getVcpuValue(int value)
{
    ui->vcpuSpinBox->setValue(value);;
}

void DomainPage::_setMemoryValue(int value)
{
    int min = ui->memoryHorizontalSlider->minimum();
    int max = ui->memoryHorizontalSlider->maximum();
    QString error;
    error = tr("Memory value %1 is incorrect ! Enter value between %2 and %3").arg(value).arg(min).arg(max);

    if (value >= min && value <= max)
    {
        ui->memoryHorizontalSlider->setValue(value);
        ui->errorMemory->setText("");
        if (ui->errorVcpu->text().isEmpty())
            ui->frame->setHidden(true);
    }
    else
    {
        ui->errorMemory->setText(error);
        ui->errorMemory->setStyleSheet("QLabel{color: red}");
        ui->frame->setHidden(false);
    }
}

void DomainPage::_setVcpuValue(int value)
{
    int min = ui->vcpuHorizontalSlider->minimum();
    int max = ui->vcpuHorizontalSlider->maximum();
    QString error;
    error = tr("VCPU value %1 is incorrect ! Enter value between %2 and %3").arg(value).arg(min).arg(max);

    if (value >= min && value <= max)
    {
        ui->vcpuHorizontalSlider->setValue(value);
        ui->errorVcpu->setText("");
        if (ui->errorMemory->text().isEmpty())
            ui->frame->setHidden(true);
    }
    else
    {
        ui->errorVcpu->setText(error);
        ui->errorVcpu->setStyleSheet("QLabel{color: red}");
        ui->frame->setHidden(false);
    }
}

void DomainPage::on_apicCheckBox_clicked(bool checked)
{
    _apicOldValue = checked;
}

void DomainPage::on_memoryHorizontalSlider_valueChanged(int value)
{
    QString warningMessage;

    if (value >= 8192)
        warningMessage = tr("You have assigned a lot of memory to this VM.\n"
                "Please make sure there is enough available memory on your platform.");
    else
        warningMessage.clear();
    emit warningMessageChanged(warningMessage);
}

} // namespace Hnv
