/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "hnv_entryproperties.h"
#include "ui_hnv_entryproperties.h"
#include "hyneviewsession.h"
#include "propertypagefactory.h"
#include "ressources.h"

// Generated hynesim headers
#include "common/entityentryproperties.h"

// hynesim hnsclient headers
#include <libhnsclient/session.h>
#include <libhnsclient/entrypropertiesmanager.h>
#include <libhnsclient/entry.h>

// Qt headers
#include <QPushButton>
#include <QDebug>

// Qt headers
#include <QToolButton>

namespace Hnv {

/*******************************************************************************
* EntryPropertiesPage
*/
EntryPropertiesPage::EntryPropertiesPage(HnsClient::CatalogItem *entry, Hns::EntryPropertiesPtr entryProperties, QWidget *parent, bool readOnly, bool defined, bool editabletemplate)
    : QWidget(parent)
    , _canModifiedWhenDefined(false)
    , _readOnly(readOnly)
    , _defined(defined)
    , _editabletemplate(editabletemplate)
    , _sourceEntry(entry)
{
    _currentUser = HyneviewSession::instance()->user();
    _isWriteable = _sourceEntry->isWriteable();
    HnsClient::Entry * myEntry =   dynamic_cast<HnsClient::Entry *> (_sourceEntry);
    if(myEntry) {
        Hns::EntryPropertiesPtr properties = entryProperties;
        Hns::EntityEntryPropertiesPtr  entityProperties = Hns::EntityEntryPropertiesPtr::dynamicCast(properties);
        if(entityProperties){
            _canModifiedWhenDefined = entityProperties->canModified;
        }
    }
}

QString EntryPropertiesPage::getItemName()
{
    return _sourceEntry->getShortName();
}

bool EntryPropertiesPage::isWriteable()
{
    return _isWriteable;
}

bool EntryPropertiesPage::isNew()
{
    return _sourceEntry->isNew();
}

/*******************************************************************************
* EntryPropertiesPage - Protected
*/
HnsClient::ValidationResult EntryPropertiesPage::validate()
{
    bool isModified = false;
    bool isInvalid = false;

    _validate(isModified, isInvalid);
    if (isModified && isInvalid)
        return HnsClient::Invalid;
    else if (isModified)
        return HnsClient::Valid;
    else
        return HnsClient::Unmodified;
}

void EntryPropertiesPage::init()
{
}

void EntryPropertiesPage::_computeValidationState(int result, bool &isModified, bool &isInvalid)
{
    if (result != HnsClient::Unmodified) {
        isModified = true;
        if (result == HnsClient::Invalid)
            isInvalid = true;
    }
}

QToolButton *EntryPropertiesPage::_generatePageButton(const QString &title, const QIcon &icon, bool enabled)
{
    QToolButton *button = new QToolButton(this);
    button->setText(title);
    button->setIcon(icon);
    button->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
    button->setCheckable(true);
    button->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Maximum);
    button->setEnabled(enabled);
    return button;
}

/*******************************************************************************
* EntryProperties
*/
EntryProperties::EntryProperties(HnsClient::CatalogItem *entry, QWidget *parent, bool readOnly, bool defined, bool editabletemplate , bool displayAccessRight)
    : QDialog(parent)
    , _ui(new Ui::EntryProperties)
    , _sourceEntry(entry)
    , _readOnly(readOnly)
    , _defined(defined)
    , _editabletemplate(editabletemplate)
    , _displayAccessPage(displayAccessRight)
{
    _ui->setupUi(this);
    _ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
    _ui->buttonBox->setContentsMargins(0, 0, 0, 10);

    HnsClient::Entry *myEntry = dynamic_cast<HnsClient::Entry*>(_sourceEntry);
    HnsClient::FolderEntry *folderEntry = qobject_cast<HnsClient::FolderEntry*>(_sourceEntry);
    if (myEntry) {
        HnsClient::Session::instance()->entryPropertiesManager()->getEntryProperties(myEntry->entry(), this, SLOT(_getPropertiesSlot(::Hns::EntryPropertiesPtr)));
    }
    else if (folderEntry) {
        _getPropertiesSlot(Hns::EntryPropertiesPtr());
    }
}

EntryProperties::~EntryProperties()
{
    delete _ui;
}

/*******************************************************************************
* EntryProperties - Protected
*/
void EntryProperties::_initPages()
{
    PropertyPageFactory *factory = new PropertyPageFactory(_sourceEntry, this);
    _propertyPages = factory->getPropertiesPages(this, _entryProperty, _readOnly, _defined, _editabletemplate, _displayAccessPage);
    _firstWidget = _propertyPages.at(0);

    QSize stackSize = _ui->stackedWidget->size();
    QSize pageSize;
    foreach(EntryPropertiesPage * page, _propertyPages) {
        _ui->listWidget->addButton(page->getPageButton());
        _ui->stackedWidget->addWidget(page);
        connect(page, SIGNAL(warningMessageChanged(QString)), this, SLOT(_setWarningMessage(QString)));
        pageSize = page->size();
        if (stackSize.height() < pageSize.height() || stackSize.width() < pageSize.width())
            _ui->stackedWidget->resize(pageSize);
        
        connect(page, SIGNAL(validationRequested()), this, SLOT(on_buttonBox_accepted()));
    }
    foreach(EntryPropertiesPage *page, _propertyPages)
        page->init();

    setWindowTitle(tr("%1 properties").arg(_firstWidget->getItemName()));
    _firstWidget->initFocus();
}

void EntryProperties::_initDialog()
{
    if (_ui->stackedWidget->count() > 0) {
        _ui->stackedWidget->setCurrentIndex(0);
    }
    connect(_ui->listWidget, SIGNAL(currentButtonChanged(int)), _ui->stackedWidget, SLOT(setCurrentIndex(int)));

    bool itemIsWriteable = false;
    foreach(EntryPropertiesPage * page, _propertyPages) {
        if (page->isWriteable()) {
            itemIsWriteable = true;
            break;
        }
    }
    if (!_firstWidget->isNew() && !itemIsWriteable) {
        QPushButton *cancelButton = _ui->buttonBox->button(QDialogButtonBox::Cancel);
        if (cancelButton != 0)
            _ui->buttonBox->removeButton(cancelButton);
    }
}

void EntryProperties::showEvent(QShowEvent *)
{


}

/*******************************************************************************
* EntryProperties - Private slots
*/
void EntryProperties::_adjustSizeToContent(int /*widgetIndex*/)
{
    // TODO: check if needed
//    QSize widgetSize = ui->stackedWidget->widget(widgetIndex)->size();
//    QSize listSize = ui->listWidget->size();
//    QSize finalSize( widgetSize.width() + listSize.width(), qMax(widgetSize.height(), listSize.height()));
//    qDebug() << "Resizing from " << size() << "to" << finalSize;

//    resize(finalSize);
//    adjustSize();
}

void EntryProperties::on_buttonBox_accepted()
{
    HnsClient::ValidationResult state = HnsClient::Unmodified;
    foreach(EntryPropertiesPage * page, _propertyPages) {
        HnsClient::ValidationResult pageState = HnsClient::Unmodified;
        if (page->isNew() || page->isWriteable()) {
            pageState = page->validate();
        }
        if (pageState == HnsClient::Invalid)
            state = pageState;
        else if (pageState == HnsClient::Valid && state != HnsClient::Invalid)
            state = pageState;
    }

    if (state == HnsClient::Invalid) {
        qDebug() << tr("Properties validation failed.");
    } else if (state == HnsClient::Unmodified) {
        //qDebug() << tr("Entry is unmodified.");
        if (_firstWidget->isNew()) {
            //qDebug() << tr("Entry is new, accepting.");
            accept();
        } else {
            //qDebug() << tr("Rejecting.");
            done(2);
        }
    } else if (state == HnsClient::Valid) {
        //qDebug() << tr("Properties validation succeeded, accepting.");
        accept();
    }
}

void EntryProperties::_getPropertiesSlot(::Hns::EntryPropertiesPtr entryProperty)
{
    _ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(true);
    _entryProperty = entryProperty;
    _ui->stackedWidget->removeWidget(_ui->waitingPage);
    delete(_ui->waitingPage);
    _initPages();
    _initDialog();
}

void EntryProperties::_setWarningMessage(QString message)
{
    _ui->warningLabel->setText(message);
    if (!message.isEmpty()) {
        _ui->warningPixmap->setPixmap(Ressources::themedPath(Constants::ICON_DIALOG_WARNING));
    }
    else
        _ui->warningPixmap->clear();
}

} // namespace Hnv
