/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "switchpage.h"
#include "ui_switchpage.h"
#include "ressources.h"

// Generated hynesim headers
#include "common/switchentryproperties.h"

// hynesim hnsclient headers
#include <libhnsclient/entry.h>

// Qt headers
#include <QToolButton>
#include <QMessageBox>

namespace Hnv {

/*******************************************************************************
* SwitchPage
*/
SwitchPage::SwitchPage(HnsClient::CatalogItem *entry, Hns::EntryPropertiesPtr entryProperties, QWidget *parent, bool readOnly, bool defined)
    : EntryPropertiesPage(entry,entryProperties, parent,readOnly, defined )
    , ui(new Ui::SwitchPage)
    , _portMirroringSupported(false)
    , _stpSupported(false)
{
    ui->setupUi(this);
    connect(ui->hubCheckBox, SIGNAL(toggled(bool)), this, SLOT(_hubCheckBoxChangedSlot(bool)));
    connect(ui->portsSpinBox, SIGNAL(valueChanged(int)), this, SIGNAL(portNumberChanged(int)));
    HnsClient::SwitchEntry *switchEntry = qobject_cast<HnsClient::SwitchEntry*>(_sourceEntry);
    if (switchEntry) {
        _displayPorts(switchEntry->getPorts());
        _displayHub(switchEntry->getIsHub());
        _displayStp(switchEntry->getStpEnabled());
    }
    Hns::SwitchEntryPropertiesPtr switchProperties = Hns::SwitchEntryPropertiesPtr::dynamicCast(entryProperties);
    _stpSupported = switchProperties->stpSupported;
    ui->stpCheckBox->setEnabled(_stpSupported && !switchEntry->getIsHub());
    ui->stpLabel->setEnabled(_stpSupported && !switchEntry->getIsHub());
    _portMirroringSupported = switchProperties->portMirroringSupported;
    ui->cbPortMirroring->setEnabled(_portMirroringSupported && !switchEntry->getIsHub());
    ui->portMirroringLabel->setEnabled(_portMirroringSupported && !switchEntry->getIsHub());
    ui->cbPortMirroring->addItem(tr("None"));
    for (int i = 0 ; i < switchEntry->getPorts() ; i++)
        ui->cbPortMirroring->addItem(tr("Port %1").arg(i));
    QMap<std::string, std::string> mapImplementation(switchProperties->implementationsSupported);
    foreach (std::string xmldata, mapImplementation.keys()) {
        ui->cbImplementation->addItem(QString::fromStdString(mapImplementation.value(xmldata)), QString::fromStdString(xmldata));
    }
    if (switchEntry) {
        ui->cbImplementation->setCurrentIndex(ui->cbImplementation->findData(switchEntry->getType()));
        ui->cbPortMirroring->setCurrentIndex(switchEntry->getPortMirroring() + 1);
    }
    connect(ui->cbImplementation, SIGNAL(currentIndexChanged(QString)), this, SLOT(cbTextChanged()));
    connect(ui->portsSpinBox, SIGNAL(valueChanged(int)), this, SLOT(_portSpinBoxValueChangedSlot(int)));

    _disableWidgetsWhenNecessary(!isWriteable() || _readOnly || (_defined && !_canModifiedWhenDefined));
}

SwitchPage::~SwitchPage()
{
    delete ui;
}

void SwitchPage::initFocus()
{
}

QToolButton *SwitchPage::getPageButton()
{
    return _generatePageButton(tr("System"), QIcon(Ressources::themedPath(Constants::ICON_SYSTEM_SETTINGS)));
}

/*******************************************************************************
* SwitchPage - Protected
*/
void SwitchPage::_disableWidgetsWhenNecessary(bool necessary)
{
    ui->portsSpinBox->setDisabled(necessary);
    ui->hubCheckBox->setDisabled(necessary);
    ui->stpCheckBox->setDisabled(necessary || !ui->stpCheckBox->isEnabled());
    ui->cbImplementation->setDisabled(necessary);
    ui->cbPortMirroring->setDisabled((necessary | !ui->cbPortMirroring->isEnabled()) || ui->hubCheckBox->isChecked());
}

void SwitchPage::_displayPorts(int number)
{
    ui->portsSpinBox->setValue(number);
}

void SwitchPage::_displayHub(bool value)
{
    ui->hubCheckBox->setChecked(value);
}

void SwitchPage::_displayStp(bool value)
{
    ui->stpCheckBox->setChecked(value);
}

void SwitchPage::_validate(bool &isModified, bool &isInvalid)
{
    HnsClient::SwitchEntry *entry = qobject_cast<HnsClient::SwitchEntry *>(_sourceEntry);
    _computeValidationState(entry->validatePorts(ui->portsSpinBox->value()), isModified, isInvalid);
    _computeValidationState(entry->validateIsHub(ui->hubCheckBox->isChecked()), isModified, isInvalid);
    _computeValidationState(entry->validateStpEnabled(ui->stpCheckBox->isChecked()), isModified, isInvalid);
    _computeValidationState(entry->validateType(ui->cbImplementation->itemData(ui->cbImplementation->currentIndex()).toString()), isModified, isInvalid);
    _computeValidationState(entry->validatePortMirroring(ui->cbPortMirroring->currentIndex() - 1), isModified, isInvalid);
}

void SwitchPage::cbTextChanged()
{
    HnsClient::SwitchEntry * switchEntry = qobject_cast<HnsClient::SwitchEntry*>(_sourceEntry);
    if (ui->cbImplementation->itemData(ui->cbImplementation->currentIndex()) == switchEntry->getType()) {
        return;
    }
    
    QMessageBox msgBox(QMessageBox::Warning, tr("Validation"), tr("After changing the value of the implementation, you need to validate the properties by clicking on OK, otherwise you can cancel and retrieve the old value"), QMessageBox::Ok|QMessageBox::Discard, this);
    int ret = msgBox.exec();
    switch (ret) {
    case QMessageBox::Ok: {
        emit validationRequested();
        break;
    }
    case QMessageBox::Discard: {
        ui->cbImplementation->setCurrentIndex(ui->cbImplementation->findData(switchEntry->getType()));
        break;
    }
    default: {
        break;
    }
    }
}

void SwitchPage::_hubCheckBoxChangedSlot(bool value)
{
    if (value) {
        //reset to none, this is a hub
        ui->cbPortMirroring->setCurrentIndex(0);
        ui->cbPortMirroring->setDisabled(true);
        ui->portMirroringLabel->setDisabled(true);
        ui->stpCheckBox->setDisabled(true);
        ui->stpLabel->setDisabled(true);
    } else {
        if (_portMirroringSupported) {
            ui->cbPortMirroring->setEnabled(true);
            ui->portMirroringLabel->setEnabled(true);
        }
        if (_stpSupported) {
            ui->stpCheckBox->setEnabled(true);
            ui->stpLabel->setEnabled(true);
        }
    }
}

void SwitchPage::_portSpinBoxValueChangedSlot(int value)
{
    int oldValue = ui->cbPortMirroring->count() -1;
    if (value > oldValue) {
        //need to add items
        for (int i = oldValue ; i < value ; ++i)
            ui->cbPortMirroring->addItem(tr("Port %1").arg(i));
    } else {
        //set current index lesser
        if (ui->cbPortMirroring->currentIndex() >= value)
            ui->cbPortMirroring->setCurrentIndex(value + 1);
        //need to remove items
        for (int i = oldValue ; i > value ; --i)
            ui->cbPortMirroring->removeItem(i);
    }
}

} // namespace Hnv
