/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "vlanmodel.h"
#include "ressources.h"

// Qt headers
#include <QIcon>
#include <QPainter>
#include <QTextDocument>

namespace Hnv {

/*************************VLAN MODEL*******************************/
VlanModel::VlanModel(Hns::VlanMap vlanList, QObject *parent)
    : QAbstractTableModel(parent)
{
    QMap<Ice::Int, Hns::Vlan>map = QMap<Ice::Int, Hns::Vlan>(vlanList);
    QMapIterator<Ice::Int, Hns::Vlan> iterator(map);
    while (iterator.hasNext()) {
        iterator.next();
        Hns::Vlan vlan = iterator.value();
        VlanLocalInformation infoVlan;
        PortLocalInformation infoPort;
        _vlanList.append(vlan.tag);
        infoVlan.ID = vlan.tag;
        infoVlan.name = QString::fromStdString(vlan.name);
        infoVlan.description = QString::fromStdString(vlan.description);
        QMap<Ice::Int, Hns::PortTag> portsMap(vlan.portsMap);
        QMapIterator<Ice::Int, Hns::PortTag> it(portsMap);
        while (it.hasNext()) {
            it.next();
            infoPort.ID = it.key();
            infoPort.tagged = it.value();
            infoVlan.Ports.append(infoPort);
        }
        _vlanPositionHash.insert(vlan.tag, infoVlan);
    }
}

VlanModel::~VlanModel()
{
}

void VlanModel::clear()
{
    beginResetModel();
    _vlanList.clear();
    _vlanPositionHash.clear();
    endResetModel();
}

void VlanModel::addVlan(VlanLocalInformation vlan)
{
    int row = _vlanList.size();
    beginInsertRows(QModelIndex(), row, row);
    _vlanList.append(vlan.ID);
    _vlanPositionHash.insert(vlan.ID, vlan);
    endInsertRows();
}

void VlanModel::removeVlan(int ID)
{
    if (_vlanList.contains(ID)) {
        int row = _vlanList.indexOf(ID);
        foreach (PortLocalInformation port, _vlanPositionHash[ID].Ports) {
            if (!port.tagged)
                _vlanPositionHash[0].Ports.append(port);
        }
        _vlanPositionHash.remove(ID);
        beginRemoveRows(QModelIndex(),row,row);
        _vlanList.removeAt(row);
        endRemoveRows();
        qSort(_vlanPositionHash[0].Ports);
    }
}

int VlanModel::columnCount(const QModelIndex &parent) const
{
    if (!parent.isValid()) {
        return COLUMN_SIZE;
    }
    else return 0;
}

int VlanModel::rowCount(const QModelIndex &parent) const
{
    if (!parent.isValid()) {
        return _vlanList.size();
    }
    else return 0;
}

QVariant VlanModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if (role == Qt::DisplayRole && orientation == Qt::Horizontal) {
        switch (section) {
        case COLUMN_ID:
            return tr("ID");
        case COLUMN_PORTS:
            return tr("Ports");
        case COLUMN_EDIT:
            return "";
        case COLUMN_DELETE:
            return "";
        }
    }
    if (role == Qt::DecorationRole && orientation == Qt::Horizontal && section == COLUMN_DELETE)
        return QIcon(Ressources::themedPath(Constants::ICON_TRASH));
    if (role == Qt::DecorationRole && orientation == Qt::Horizontal && section == COLUMN_EDIT)
        return QIcon(Ressources::themedPath(Constants::ICON_EDIT_VLAN_HEADER));
    return QAbstractTableModel::headerData(section, orientation, role);
}

QVariant VlanModel::data(const QModelIndex &index, int role) const
{
    if (index.isValid()) {
        int row = index.row();
        if (row < _vlanList.size()) {
            VlanLocalInformation vlan = _vlanPositionHash.value(_vlanList.at(row));
            switch (role) {
            case Qt::DisplayRole:
                switch (index.column()) {
                case COLUMN_ID:
                    return vlan.ID;
                case COLUMN_PORTS:
                    return _parsePorts(vlan.Ports);
                case COLUMN_DELETE:
                    return "";
                }
            case Qt::ToolTipRole: {
                QString tooltip;
                if (vlan.name.isEmpty())
                    tooltip = tr("Unamed VLAN (%1)").arg(QString::number(vlan.ID));
                else
                    tooltip = tr("Name: %1").arg(vlan.name);
                if (!vlan.description.isEmpty())
                    tooltip.append(tr("\nDescription: %1").arg(vlan.description));
                return tooltip;
            }
            case Qt::UserRole+1:
                return vlan.description;
            case Qt::UserRole+2:
                return vlan.name;
            case Qt::UserRole+3:
                return _vlanList.at(row);
            case Qt::UserRole+4:
                return vlan.ID;
            }
        }
    }
    return QVariant();
}

bool VlanModel::setData(const QModelIndex &index, const QVariant &value, int role)
{
    if (index.isValid()) {
        int row = index.row();
        if (row < _vlanList.size()) {
            switch (role) {
            case Qt::UserRole+1:
                _vlanPositionHash[_vlanList.at(row)].description = value.toString();
                return true;
            case Qt::UserRole+2:
                _vlanPositionHash[_vlanList.at(row)].name = value.toString();
                return true;
            }
        }
    }
    return true;
}

QString VlanModel::_parsePorts(QList<PortLocalInformation> ports) const
{
    QString str="";
    QString temp;
    if (ports.size() >= 1) {
        if (ports[0].tagged)
            temp = "<b>" + QString::number(ports[0].ID) + "</b>";
        else
            temp = QString::number(ports[0].ID);
    }
    str = temp;
    for (int i = 1 ; i < ports.size() ; i++) {
        if (ports[i].tagged)
            temp = "<b>" + QString::number(ports[i].ID) + "</b>";
        else
            temp = QString::number(ports[i].ID);
        str = str + ", " + temp;
    }
    return str;
}

QList<int > VlanModel::getIDs()
{
    return _vlanList;
}

QMap<Ice::Int, Hns::Vlan> VlanModel::getVlanMap()
{
    QMap<Ice::Int, Hns::Vlan>map;
    foreach (VlanLocalInformation vlanInfo, _vlanPositionHash) {
        Hns::Vlan vlan;
        vlan.tag = vlanInfo.ID;
        vlan.name = vlanInfo.name.toStdString();
        vlan.description = vlanInfo.description.toStdString();
        QMap<Ice::Int, Hns::PortTag>mapPort;
        foreach (PortLocalInformation portInfo, vlanInfo.Ports) {
            if (portInfo.tagged)
                mapPort.insert(portInfo.ID, Hns::Tagged);
            else
                mapPort.insert(portInfo.ID, Hns::Untagged);
        }
        vlan.portsMap = mapPort.toStdMap();
        map.insert(vlan.tag, vlan);
    }
    return map;
}

QHash<int, VlanLocalInformation> VlanModel::getVlans()
{
    return _vlanPositionHash;
}

VlanLocalInformation VlanModel::getVlan(int id)
{
    return _vlanPositionHash[id];
}

void VlanModel::updatePorts(QList<PortItem> oldList, QList<PortItem> newList, int ID)
{
    for (int i = 0 ; i < newList.size() ; i++) {
        if (newList[i].state == STATE_TAGGED && oldList[i].state == STATE_NONE) {
            PortLocalInformation port;
            port.ID = newList[i].ID;
            port.tagged = true;
            _vlanPositionHash[ID].Ports.append(port);
        }
        else if (newList[i].state == STATE_NONE && oldList[i].state == STATE_UNTAGGED) {
            PortLocalInformation port;
            port.ID = newList[i].ID;
            port.tagged = false;
            _vlanPositionHash[ID].Ports.removeOne(port);
            _vlanPositionHash[0].Ports.append(port);
        }
        else if (newList[i].state == STATE_NONE && oldList[i].state == STATE_TAGGED) {
            PortLocalInformation port;
            port.ID = newList[i].ID;
            port.tagged = true;
            _vlanPositionHash[ID].Ports.removeOne(port);
        }
        else if(newList[i].state == STATE_UNTAGGED) {
            PortLocalInformation port;
            port.ID = newList[i].ID;
            port.tagged = false;
            QHashIterator<int, VlanLocalInformation> it(_vlanPositionHash);
            int cut = 0;
            while (it.hasNext()) {
                it.next();
                VlanLocalInformation vlan = it.value();
                for( int j = 0 ; j < vlan.Ports.size() ; j++) {
                    if (vlan.Ports[j] == port) {
                        vlan.Ports.removeOne(port);
                        _vlanPositionHash[it.key()] = vlan;
                        _vlanPositionHash[ID].Ports.append(port);
                        port.tagged = true;
                        if (oldList[i].state == STATE_TAGGED) {
                            _vlanPositionHash[ID].Ports.removeOne(port);
                        }
                        cut = 1;
                        break;
                    }
                }
                if (cut == 1)
                    break;
            }
        }
    }
    qSort(_vlanPositionHash[ID].Ports);
    qSort(_vlanPositionHash[0].Ports);
}

void VlanModel::portNumberChange(int n)
{
    QHashIterator<int, VlanLocalInformation> iterator(_vlanPositionHash);
    //in case we have to remove port
    while (iterator.hasNext()) {
        iterator.next();
        VlanLocalInformation vlan = iterator.value();
        QList<PortLocalInformation> newList;
        for (int i = 0 ; i < vlan.Ports.size() ; i++) {
            if (vlan.Ports[i].ID < n)
                newList.append(vlan.Ports.at(i));
        }
        vlan.Ports = newList;
        _vlanPositionHash.insert(iterator.key(), vlan);
    }
    //in case we have to add port
    for (int i = _portsNb ; i < n ; i++ ) {
        PortLocalInformation port;
        port.ID = i;
        port.tagged = false;
        _vlanPositionHash[0].Ports.append(port);
    }
    _portsNb = n;

}

void VlanModel::setPortsNb(int n)
{
    _portsNb = n;
}

int VlanModel::portsNb()
{
    return _portsNb;
}

/*********************VLANDELEGATE*********************/
DelegateVlan::DelegateVlan(int deleteColumn, int labelColumn, int editColumn, QSize iconSize, QObject *parent)
    : QStyledItemDelegate(parent)
    , _deleteColumn(deleteColumn)
    , _labelColumn(labelColumn)
    , _editColumn(editColumn)
    , _iconSize(iconSize)
{
}

DelegateVlan::~DelegateVlan()
{
}

void DelegateVlan::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    QStyleOptionViewItemV4 options = option;
    initStyleOption(&options, index);
    if (index.column() == _labelColumn) {
        QTextDocument doc;
        doc.setHtml(options.text);
        options.text = "";
        options.widget->style()->drawControl(QStyle::CE_ItemViewItem, &options, painter);
        painter->save();
        painter->translate(options.rect.left(), options.rect.top());
        QRect clip(0, 0, options.rect.width(), options.rect.height());
        doc.drawContents(painter, clip);
        painter->restore();
    } else if (index.column() == _deleteColumn) {
        QPixmap deletePixmap;
        if (index.row() != 0)
            deletePixmap = QPixmap(Ressources::themedPath(Constants::ICON_DELETE));
        else
            deletePixmap = QPixmap(Ressources::themedPath(Constants::ICON_DELETE_DISABLED));
        options.widget->style()->drawControl(QStyle::CE_ItemViewItem, &option, painter);
        QRect targetRect(QPoint(0, 0), _iconSize);
        targetRect.moveCenter(option.rect.center());
        painter->drawPixmap(targetRect, deletePixmap);
    } else if (index.column() == _editColumn) {
        options.widget->style()->drawControl(QStyle::CE_ItemViewItem, &option, painter);
        QRect targetRect(QPoint(0, 0), _iconSize);
        targetRect.moveCenter(option.rect.center());
        painter->drawPixmap(targetRect, QPixmap(Ressources::themedPath(Constants::ICON_EDIT_VLAN)));
    } else
        QStyledItemDelegate::paint(painter,option,index);
}

QSize DelegateVlan::sizeHint ( const QStyleOptionViewItem & option, const QModelIndex & index ) const
{
    if (index.column() == _labelColumn) {
        QStyleOptionViewItemV4 options = option;
        initStyleOption(&options, index);
        QTextDocument doc;
        doc.setHtml(options.text);
        doc.setTextWidth(options.rect.width());
        return QSize(doc.idealWidth(), doc.size().height());
    } else
        return QStyledItemDelegate::sizeHint(option,index);
}

} // namespace Hnv
