// Project headers
#include "vlanpopup.h"
#include "ui_vlanpopup.h"

// Qt headers
#include <QSignalMapper>
#include <QComboBox>

namespace Hnv {

VlanPopup::VlanPopup(VlanLocalInformation vlan, int nbPorts, QWidget *parent) :
    QDialog(parent)
  , ui(new Ui::VlanPopup)
  , _id(vlan.ID)
  , _name(vlan.name)
  , _description(vlan.description)
  , _vlan(vlan)
  , _nbPorts(nbPorts)

{        
    ui->setupUi(this);    
    ui->leID->setText(QString::number(_id));
    ui->leName->setText(_name);
    ui->lineEditDescription->setText(_description);
    _portModel = new PortModel(_nbPorts, _id, this);
    _portModel->init(_vlan);
    _oldList = _portModel->getPortList();
    _portDelegate = new PortDelegate(_portModel->getPortList(), _id, this);
    ui->tableViewPorts->setItemDelegate(_portDelegate);
    ui->tableViewPorts->setModel(_portModel);
    ui->tableViewPorts->horizontalHeader()->setStretchLastSection(true);    
    for(int i = 0; i < ui->tableViewPorts->model()->rowCount(); i++) {
        if (ui->tableViewPorts->model()->hasIndex(i, PortModel::COLUMN_PORT_STATE)) {
            ui->tableViewPorts->openPersistentEditor(ui->tableViewPorts->model()->index(i, PortModel::COLUMN_PORT_STATE));
            _portModel->setData(ui->tableViewPorts->model()->index(i, PortModel::COLUMN_PORT_STATE), QVariant(QString("")), Qt::EditRole);
        }
    }
    connect(_portDelegate, SIGNAL(cbChanged(int,QComboBox*)), _portModel, SLOT(cbChanged(int,QComboBox*)));
}

VlanPopup::~VlanPopup()
{
    delete ui;
    delete _portModel;
    delete _portDelegate;
}


void VlanPopup::on_buttonBox_accepted()
{
    _name = ui->leName->text();
    _description = ui->lineEditDescription->text();
    emit accept();
}

QString VlanPopup::getName()
{
    return _name;
}

QString VlanPopup::getDescription()
{
    return _description;
}

QList<PortItem> VlanPopup::getOldPortList()
{
    return _oldList;
}

QList<PortItem> VlanPopup::getNewPortList()
{
    return _portModel->getPortList();
}


/**********************PORT MODEL**********************************/
PortModel::PortModel(int nbPorts,int idVlan, QObject *parent) : QAbstractTableModel(parent)
{
    _idVlan = idVlan;
    PortItem port;
    for (int i = 0 ; i < nbPorts ; i++) {
        port.ID = i;
        port.state = VlanModel::STATE_NONE;
        _portList.append(port);
    }
}

PortModel::~PortModel()
{
}

void PortModel::init(VlanLocalInformation vlan)
{
    for (int i = 0 ; i < vlan.Ports.size() ; i++) {
        if (vlan.Ports[i].tagged)
            _portList[vlan.Ports[i].ID].state = VlanModel::STATE_TAGGED;
        else
            _portList[vlan.Ports[i].ID].state = VlanModel::STATE_UNTAGGED;
    }
    emit dataChanged(index(0, 0), index(_portList.size(), PortModel::COLUMN_PORT_SIZE));
}

void PortModel::clear()
{
    beginResetModel();
    _portList.clear();
    _portPositionHash.clear();
    endResetModel();
}

void PortModel::addPort(PortItem port)
{
    int row = _portList.size();
    beginInsertRows(QModelIndex(), row, row);
    _portList.append(port);
    _portPositionHash.insert(port.ID,row);
    endInsertRows();
}

void PortModel::removePort(int ID)
{
    if (_portPositionHash.contains(ID)) {
        int row = _portPositionHash.take(ID);
        beginRemoveRows(QModelIndex(),row,row);
        _portList.removeAt(row);
        endRemoveRows();
    }
}

int PortModel::rowCount(const QModelIndex &parent) const
{
    if (!parent.isValid()) {
        return _portList.size();
    }
    else return 0;
}
int PortModel::columnCount(const QModelIndex &parent) const
{
    if (!parent.isValid()) {
        return PortModel::COLUMN_PORT_SIZE;
    }
    else return 0;
}

QVariant PortModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if (role == Qt::DisplayRole && orientation == Qt::Horizontal) {
        switch (section) {
        case PortModel::COLUMN_PORT_ID:
            return tr("Ports");
        case PortModel::COLUMN_PORT_STATE:
            return tr("State");
        }
    }
    return QAbstractTableModel::headerData(section, orientation, role);
}
QVariant PortModel::data(const QModelIndex &index, int role) const
{
    if (index.isValid()) {
        int row = index.row();
        if (row < _portList.size()) {
            switch (role) {
            case Qt::DisplayRole:
                switch (index.column()) {
                case PortModel::COLUMN_PORT_ID:
                    return _portList[row].ID;
                case PortModel::COLUMN_PORT_STATE:
                    if (_portList[row].state == VlanModel::STATE_UNTAGGED)
                        return "Untagged";
                    else if (_portList[row].state == VlanModel::STATE_TAGGED)
                        return "Tagged";
                    else
                        return "None";
                }
            case Qt::EditRole:
                if (_portList[row].state == VlanModel::STATE_UNTAGGED)
                    return "Untagged";
                else if (_portList[row].state == VlanModel::STATE_TAGGED)
                    return "Tagged";
                else
                    return "None";
            case Qt::UserRole+1:
                return _portList[row].ID;
            }
        }
    }
    return QVariant();
}
bool PortModel::setData(const QModelIndex &index, const QVariant &value, int role)
{
    if (index.isValid()) {
        int row = index.row();
        if (row < _portList.size()) {
            switch(role) {
            case Qt::EditRole:
                if (value.toString() == "None") {
                    if(_idVlan != 0)
                        _portList[row].state = VlanModel::STATE_NONE;
                }
                else if (value.toString() == "Untagged") {
                    _portList[row].state = VlanModel::STATE_UNTAGGED;
                }
                else if (value.toString() == "Tagged") {
                        _portList[row].state = VlanModel::STATE_TAGGED;
                }
            }
        }
    }

    return true;
}

QList<PortItem> PortModel::getPortList()
{
    return _portList;
}

void PortModel::cbChanged(int port, QComboBox *cb)
{
    QModelIndex index = this->index(port, PortModel::COLUMN_PORT_STATE);
    QString text = cb->currentText();
    setData(index, text, Qt::EditRole);
    emit dataChanged(index,index);
}

/*****************************PORT DELEGATE *************************/
PortDelegate::PortDelegate(QList<PortItem> portList, int idVlan, QObject *parent)
    : QStyledItemDelegate(parent)
    , _portList(portList)
    , _idVlan(idVlan)
{
    _mapper = new QSignalMapper();
    connect(_mapper, SIGNAL(mapped(int)), this, SLOT(comboBoxChanged(int)));
}

PortDelegate::~PortDelegate()
{
    delete _mapper;
}

QWidget *PortDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    if (index.column() == PortModel::COLUMN_PORT_STATE) {
        QComboBox *cb = new QComboBox(parent);
        connect(cb, SIGNAL(currentIndexChanged(int)), _mapper, SLOT(map()));
        _mapper->setMapping(cb, index.data(Qt::UserRole+1).toInt());
        _statePorts.insert(index.data(Qt::UserRole+1).toInt(), cb);
        if (_idVlan != 0)
            cb->addItem(QString("None"));
        cb->addItem(QString("Untagged"));
        if (_portList[index.row()].state != VlanModel::STATE_UNTAGGED)
            cb->addItem(QString("Tagged"));
        return cb;
    }
    else {
        return QStyledItemDelegate::createEditor(parent, option, index);
    }
}

void PortDelegate::setEditorData(QWidget *editor, const QModelIndex &index) const
{
    if(QComboBox *cb = qobject_cast<QComboBox *>(editor)) {
        QString currentText = index.data(Qt::EditRole).toString();
        int cbIndex = cb->findText(currentText);
        if(cbIndex >= 0)
            cb->setCurrentIndex(cbIndex);
    }
    else {
        QStyledItemDelegate::setEditorData(editor, index);
    }
}

void PortDelegate::updateEditorGeometry(QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    QStyledItemDelegate::updateEditorGeometry(editor, option, index);
    if (qobject_cast<QComboBox *>(editor)) {
        editor->setMaximumSize(option.rect.width() - 4, option.rect.height() - 4);
        editor->move(option.rect.x() + 2, option.rect.y() + 2);
    }
}

void PortDelegate::setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const
{
    if(QComboBox *cb = qobject_cast<QComboBox *>(editor)) {
        model->setData(index, cb->currentText(), Qt::EditRole);
        QString currentText = index.data(Qt::EditRole).toString();
        int cbIndex = cb->findText(currentText);
        if (cbIndex >= 0)
            cb->setCurrentIndex(cbIndex);
    }
    else
        QStyledItemDelegate::setModelData(editor, model, index);
}

void PortDelegate::comboBoxChanged(int idx)
{
    QComboBox *cb = _statePorts[idx];
    emit cbChanged(idx, cb);
}

} // namespace Hnv
