/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "wirepage.h"
#include "ui_wirepage.h"
#include "ressources.h"

// Generated hynesim headers
#include "common/wireentryproperties.h"

// hynesim hnsclient headers
#include <libhnsclient/entry.h>

// Qt headers
#include <QToolButton>
#include <QMessageBox>

namespace Hnv {

/*******************************************************************************
* WirePage
*/

WirePage::WirePage(HnsClient::CatalogItem *entry, Hns::EntryPropertiesPtr entryProperties, QWidget *parent, bool readOnly, bool defined)
    : EntryPropertiesPage(entry,entryProperties, parent, readOnly, defined)
    , ui(new Ui::WirePage)
{
    ui->setupUi(this);
    HnsClient::WireEntry * wire = qobject_cast<HnsClient::WireEntry *>(_sourceEntry);
    if (wire) {
        QString arrow("%1\u2192%2");
        ui->label_LR_Title->setText(arrow.arg(tr("L")).arg(tr("R")));
        ui->label_RL_Title->setText(arrow.arg(tr("R")).arg(tr("L")));
        ui->label_L_legende->setText( wire->getFromEntity());
        ui->label_R_legende->setText( wire->getToEntity());

        _displayLoss(wire->getLossLR() ,wire->getLossRL() );
        _displayDelay(wire->getDelayLR(), wire->getDelayRL());
        _displayDup(wire->getDupLR(), wire->getDupRL());
        _displayBandWidth(wire->getBandwidthLR(),wire->getBandwidthRL() );
        _displayMtu(wire->getMtuLR(),wire->getMtuRL() );

        Hns::WireEntryPropertiesPtr wireProperties = Hns::WireEntryPropertiesPtr::dynamicCast(entryProperties);
        if (wireProperties->networkPropertiesSupported == false) {
            ui->dupSpinBox->setDisabled(true);
            ui->dupSpinBox_RL->setDisabled(true);
            ui->bandwidthSpinBox->setDisabled(true);
            ui->bandwidthSpinBox_RL->setDisabled(true);
            ui->delaySpinBox->setDisabled(true);
            ui->delaySpinBox_RL->setDisabled(true);
            ui->lossSpinBox->setDisabled(true);
            ui->lossSpinBox_RL->setDisabled(true);
            ui->mtuSpinBox->setDisabled(true);
            ui->mtuSpinBox_RL->setDisabled(true);
        }
        QMap<std::string, std::string> mapImplementation(wireProperties->implementationsSupported);
        foreach (std::string xmldata, mapImplementation.keys()) {
            ui->comboBoxImplementation->addItem(QString::fromStdString(mapImplementation.value(xmldata)), QString::fromStdString(xmldata));
        }
        ui->comboBoxImplementation->setCurrentIndex(ui->comboBoxImplementation->findData(wire->getType()));
        connect(ui->comboBoxImplementation, SIGNAL(currentIndexChanged(QString)), this, SLOT(_cbImplementationTextChanged()));
     }

    //since there is only one wire implementation, we hide the following widgets
    ui->comboBoxImplementation->hide();
    ui->labelImplementation->hide();

    _disableWidgetsWhenNecessary(!isWriteable() || _readOnly || (_defined && !_canModifiedWhenDefined));
}

WirePage::~WirePage()
{
    delete ui;
}

void WirePage::initFocus()
{
    ui->delaySpinBox->setFocus();
}

QToolButton *WirePage::getPageButton()
{
    return _generatePageButton(tr("Settings"), QIcon(Ressources::themedPath(Constants::ICON_SYSTEM_SETTINGS)));
}

void WirePage::_validate(bool &isModified, bool &isInvalid)
{
    HnsClient::WireEntry * wire = qobject_cast<HnsClient::WireEntry *>(_sourceEntry);
    _computeValidationState(wire->validateDelayLR(ui->delaySpinBox->value()), isModified, isInvalid);
    _computeValidationState(wire->validateLossLR(ui->lossSpinBox->value()), isModified, isInvalid);
    _computeValidationState(wire->validateDupLR(ui->dupSpinBox->value()), isModified, isInvalid);
    _computeValidationState(wire->validateBandWidthLR(ui->bandwidthSpinBox->value()), isModified, isInvalid);
    _computeValidationState(wire->validateMtuLR(ui->mtuSpinBox->value()), isModified, isInvalid);
    _computeValidationState(wire->validateDelayRL(ui->delaySpinBox_RL->value()), isModified, isInvalid);
    _computeValidationState(wire->validateLossRL(ui->lossSpinBox_RL->value()), isModified, isInvalid);
    _computeValidationState(wire->validateDupRL(ui->dupSpinBox_RL->value()), isModified, isInvalid);
    _computeValidationState(wire->validateBandWidthRL(ui->bandwidthSpinBox_RL->value()), isModified, isInvalid);
    _computeValidationState(wire->validateMtuRL(ui->mtuSpinBox_RL->value()), isModified, isInvalid);
    _computeValidationState(wire->validateType(ui->comboBoxImplementation->itemData(ui->comboBoxImplementation->currentIndex()).toString()), isModified, isInvalid);
}

void WirePage::_cbImplementationTextChanged()
{
    HnsClient::WireEntry * wire = qobject_cast<HnsClient::WireEntry *>(_sourceEntry);
    if (ui->comboBoxImplementation->itemData(ui->comboBoxImplementation->currentIndex()) == wire->getType()) {
        return;
    }

    QMessageBox msgBox(QMessageBox::Warning, tr("Validation"), tr("After changing the value of the implementation, you need to validate the properties by clicking on OK, otherwise you can cancel and retrieve the old value"), QMessageBox::Ok|QMessageBox::Discard, this);
    int ret = msgBox.exec();
    switch (ret) {
    case QMessageBox::Ok: {
        emit validationRequested();
        break;
    }
    case QMessageBox::Discard: {
        ui->comboBoxImplementation->setCurrentIndex(ui->comboBoxImplementation->findData(wire->getType()));
        break;
    }
    default: {
        break;
    }
    }
}

/*******************************************************************************
* WirePage - Protected
*/
void WirePage::_disableWidgetsWhenNecessary(bool necessary)
{
    ui->lossSpinBox->setDisabled(necessary);
    ui->lossSpinBox_RL->setDisabled(necessary);
    ui->delaySpinBox->setDisabled(necessary);
    ui->delaySpinBox_RL->setDisabled(necessary);
    ui->dupSpinBox->setDisabled(necessary);
    ui->dupSpinBox_RL->setDisabled(necessary);
    ui->bandwidthSpinBox->setDisabled(necessary);
    ui->bandwidthSpinBox_RL->setDisabled(necessary);
    ui->mtuSpinBox->setDisabled(necessary);
    ui->mtuSpinBox_RL->setDisabled(necessary);
}

void WirePage::_displayLoss(int lossLR, int lossRL)
{
    ui->lossSpinBox->setValue(lossLR);
    ui->lossSpinBox_RL->setValue(lossRL);
}

void WirePage::_displayDelay(int delayLR, int delayRL)
{
    ui->delaySpinBox->setValue(delayLR);
    ui->delaySpinBox_RL->setValue(delayRL);
}

void WirePage::_displayDup(int dupLR, int dupRL)
{
    ui->dupSpinBox->setValue(dupLR);
    ui->dupSpinBox_RL->setValue(dupRL);
}

void WirePage::_displayBandWidth(int bandwidthLR, int bandwidthRL)
{
    ui->bandwidthSpinBox->setValue(bandwidthLR);
    ui->bandwidthSpinBox_RL->setValue(bandwidthRL);
}

void WirePage::_displayMtu(int mtuLR, int mtuRL)
{
    ui->mtuSpinBox->setValue(mtuLR);
    ui->mtuSpinBox_RL->setValue(mtuRL);
}

} // namespace Hnv
