/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "groupeditdialog.h"
#include "ui_groupeditdialog.h"

// Qt headers
#include <QMessageBox>

namespace Hnv {

/*******************************************************************************
* GroupEditDialog
*/

GroupEditDialog::GroupEditDialog(Diamesh::GroupPtr &group, QHash<qlonglong, QString> groupNameHash, QHash<qlonglong, QString> usersNameHash, QWidget *parent):
    QDialog(parent),
    ui(new Ui::GroupEditDialog),
    _groupData(group),
    _groupNameHash(groupNameHash),
    _usersNameHash(usersNameHash)
{
    ui->setupUi(this);
    _originalName = QString::fromStdString(_groupData->name);
    ui->groupNameLineEdit->setText(_originalName);
    QList<Ice::Long> usersIDList = QList<Ice::Long>::fromStdList(_groupData->userList);
    foreach(Ice::Long key, _usersNameHash.keys()) {
        if (usersIDList.contains(key)) {
            QListWidgetItem *item = new QListWidgetItem(_usersNameHash.value(key),ui->listUsers);
            item->setData(Qt::UserRole, qlonglong(key));
        } else {
            QListWidgetItem *item = new QListWidgetItem(_usersNameHash.value(key),ui->listAvailableUsers);
            item->setData(Qt::UserRole, qlonglong(key));
        }
    }
    QString windowTitle = tr("%1 properties").arg(_originalName);
    if (_groupData->createdOn == 0)
        windowTitle = tr("New group properties");
    setWindowTitle(windowTitle);
}

GroupEditDialog::~GroupEditDialog()
{
    delete ui;
}

/*******************************************************************************
* GroupEditDialog - Private
*/

bool GroupEditDialog::isNotEmpty(QLabel *targetLabel, QString &text)
{
    bool isValid = false;
    text = text.trimmed();
    if (text == "") {
        isValid = false;
    } else {
        isValid = true;
    }
    setLabelValidationStyle(targetLabel, isValid);
    return isValid;
}

void GroupEditDialog::setLabelValidationStyle(QLabel *targetLabel, bool isValid)
{
    if (isValid)
        targetLabel->setStyleSheet("");
    else
        targetLabel->setStyleSheet("QLabel{color:red;}");
}

/*******************************************************************************
* GroupEditDialog - Private slots
*/

void Hnv::GroupEditDialog::on_addSelected_clicked()
{
    QList<QListWidgetItem*>selectedItems = ui->listAvailableUsers->selectedItems();
    foreach(QListWidgetItem *item, selectedItems) {
        ui->listAvailableUsers->takeItem(ui->listAvailableUsers->row(item));
        ui->listUsers->addItem(item);
    }
}

void Hnv::GroupEditDialog::on_allAll_clicked()
{
    int itemCount = ui->listAvailableUsers->count();
    for(int i=0; i < itemCount; i++) {
        QListWidgetItem *item = ui->listAvailableUsers->takeItem(0);
        ui->listUsers->addItem(item);
    }
}

void Hnv::GroupEditDialog::on_removeSelected_clicked()
{
    QList<QListWidgetItem *> selectedItems = ui->listUsers->selectedItems();
    foreach(QListWidgetItem *item, selectedItems) {
        ui->listUsers->takeItem( ui->listUsers->row(item));
        ui->listAvailableUsers->addItem(item);
    }
}

void GroupEditDialog::on_buttonBox_accepted()
{
    QString groupName = ui->groupNameLineEdit->text();
    bool isValid = isNotEmpty(ui->groupNameLabel, groupName);
    if ( _originalName != groupName && _groupNameHash.values().contains(groupName)) {
        isValid = false;
        setLabelValidationStyle(ui->groupNameLabel, isValid);
        QMessageBox::warning(this, tr("Operation failed"),tr("Group '%1' already exists.").arg(groupName));
    }
    if (isValid) {
        Diamesh::LongList usersList;
        int itemCount = ui->listUsers->count();
        for(int i=0; i < itemCount; i++) {
            QListWidgetItem *item = ui->listUsers->item(i);
            usersList.push_back(item->data(Qt::UserRole).toLongLong());
        }
        _groupData->name = groupName.toStdString();
        _groupData->userList = usersList;
        accept();
    }
}

void GroupEditDialog::on_removeAll_clicked()
{
    int itemCount = ui->listUsers->count();
    for(int i=0; i < itemCount; i++) {
        QListWidgetItem *item = ui->listUsers->takeItem(0);
        ui->listAvailableUsers->addItem(item);
    }
}

void GroupEditDialog::on_listAvailableUsers_doubleClicked(const QModelIndex &index)
{
    QListWidgetItem *item = ui->listAvailableUsers->takeItem(index.row());
    ui->listUsers->addItem(item);
}

void GroupEditDialog::on_listUsers_doubleClicked(const QModelIndex &index)
{
    QListWidgetItem *item = ui->listUsers->takeItem(index.row());
    ui->listAvailableUsers->addItem(item);
}

} // namespace Hnv
