/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "groupsmanagerwidget.h"
#include "ui_groupsmanagerwidget.h"
#include "ressources.h"
#include "groupsmodel.h"
#include "groupeditdialog.h"
#include "treesortfilterproxymodel.h"
#include "filterlineedit.h"

// hynesim hnsclient headers
#include <libhnsclient/usermanager.h>

// Qt headers
#include <QMessageBox>
#include <QDebug>
#include <QMenu>

namespace Hnv {

/*******************************************************************************
* GroupsManagerWidget
*/
GroupsManagerWidget::GroupsManagerWidget(QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::GroupsManagerWidget)
    , _accessor(0)
{
    ui->setupUi(this);

    _filterLineEdit = new FilterLineEdit();
    connect(_filterLineEdit, SIGNAL(textChanged(QString)), this, SLOT(_groupFilter_textChanged(QString)));
    _groupsProxyModel = new class TreeSortFilterProxyModel(this);
    QList<int> filterColumns;
    filterColumns << 1 <<4;
    _groupsProxyModel->setFilterColumnsList(filterColumns);
    _groupsProxyModel->setSortCaseSensitivity(Qt::CaseInsensitive);
    _groupsProxyModel->setFilterCaseSensitivity(Qt::CaseInsensitive);
    ui->groupsTableView->setModel(_groupsProxyModel);
    ui->groupsTableView->sortByColumn(1, Qt::AscendingOrder);
    ui->groupsTableView->resizeColumnsToContents();
    connect(ui->groupsTableView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)), this, SLOT(_selectionChanged(QItemSelection,QItemSelection)));
    connect(ui->groupsTableView, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(_customMenuRequested(QPoint)));
    connect(_groupsProxyModel, SIGNAL(rowsInserted(const QModelIndex &, int, int)), this, SLOT(_rowInserted()));
    connect(_groupsProxyModel, SIGNAL(rowsRemoved(const QModelIndex &, int, int)), this, SLOT(_rowInserted()));
    _enableControls(false);
    ui->action_Delete_group->setIcon(QIcon(Ressources::themedPath(Constants::ICON_DELETE)));
    ui->action_Edit_group->setIcon(QIcon(Ressources::themedPath(Constants::ICON_GROUP_EDIT)));
    ui->action_New_group->setIcon(QIcon(Ressources::themedPath(Constants::ICON_GROUP_ADD)));

    ui->groupsToolBar->setCloseable(false);
    ui->groupsToolBar->addAction(ui->action_New_group);
    ui->action_New_group->setEnabled(true);
    ui->groupsToolBar->addAction(ui->action_Edit_group);
    ui->action_Edit_group->setEnabled(false);
    ui->groupsToolBar->addAction(ui->action_Delete_group);
    ui->action_Delete_group->setEnabled(false);
    ui->groupsToolBar->addSeparator();
    QWidget* spacer = new QWidget();
    spacer->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    ui->groupsToolBar->addWidget(spacer);
    ui->groupsToolBar->addWidget(_filterLineEdit);
    _filterLineEdit->setMinimumSize(300, 0);
    _filterLineEdit->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
    _filterLineEdit->setPlaceholderText(tr("Filter these users..."));
    _enableControls(false);
}

GroupsManagerWidget::~GroupsManagerWidget()
{
    delete ui;
}

void GroupsManagerWidget::setAccessor(HnsClient::UserManager *accessor)
{
    foreach (QObject *o, children()) {
        QDialog *dialog = qobject_cast<QDialog *>(o);
        if (dialog)
            dialog->reject();
    }
    _accessor = accessor;
    bool enable = false;
    if (_accessor) {
        connect(_accessor, SIGNAL(groupErrorOccured(QString)), this, SLOT(_displayErrorMessage(QString)));
        enable = true;
    }
    _enableControls(enable);
}

void GroupsManagerWidget::setGroupsModel(GroupsModel *groupsModel)
{
    _groupsModel = groupsModel;
    _groupsProxyModel->setSourceModel(_groupsModel);
    ui->groupsTableView->resizeColumnsToContents();

}

/*******************************************************************************
* GroupsManagerWidget - Protected slots
*/

void GroupsManagerWidget::_selectionChanged(QItemSelection, QItemSelection)
{
    bool oneSelection = false;
    bool oneAndMoreSelection = false;

    int liSelectionCount = ui->groupsTableView->selectionModel()->selectedRows().count();
    if (liSelectionCount == 1) {
        oneSelection = true;
        oneAndMoreSelection = true;
    } else if (liSelectionCount > 1) {
        oneAndMoreSelection = true;
    }
    ui->action_Edit_group->setEnabled(oneSelection);
    ui->action_Delete_group->setEnabled(oneAndMoreSelection);
}

/*******************************************************************************
* GroupsManagerWidget - Private slots
*/

void GroupsManagerWidget::on_action_Delete_group_triggered()
{
    QModelIndexList selectedRowsIndexes = ui->groupsTableView->selectionModel()->selectedRows();
    QList<qlonglong> groupsIDs;
    foreach(QModelIndex index, selectedRowsIndexes) {
        groupsIDs.append(_groupsProxyModel->data(index).toLongLong());
    }
    QMessageBox msgBox(this);
    msgBox.setText(tr("Do you really want to delete theses groups ?"));
    msgBox.setInformativeText(_accessor->getGroupsNamesFromIdList(groupsIDs).join(", "));
    msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
    msgBox.setDefaultButton(QMessageBox::No);
    if (msgBox.exec() == QMessageBox::Yes) {
        foreach(Ice::Long groupID, groupsIDs) {
            _accessor->deleteGroup(groupID, this, SLOT(_successDeleteGroup(qlonglong, QString, bool)));
        }
    }
}

void GroupsManagerWidget::on_action_Edit_group_triggered()
{
    int targetRow = _groupsProxyModel->mapToSource(ui->groupsTableView->currentIndex()).row();
    _editGroup(targetRow);
}

void GroupsManagerWidget::on_action_New_group_triggered()
{
    Diamesh::GroupPtr newGroup = new Diamesh::Group();
    newGroup->id = -1;
    newGroup->createdOn = 0;
    newGroup->modifiedOn = 0;
    GroupEditDialog popup(newGroup, _accessor->getGroupNamesHash(), _accessor->getUserNamesHash(), this);
    if (popup.exec() == QDialog::Accepted) {
        _accessor->addGroup(newGroup, this, SLOT(_successAddGroup(qlonglong,QString,bool)));
    }
}

void GroupsManagerWidget::_enableControls(bool enable)
{
    ui->action_New_group->setEnabled(enable);
    if (!enable) {
        ui->action_Edit_group->setEnabled(enable);
        ui->action_Delete_group->setEnabled(enable);
    }
    _filterLineEdit->setEnabled(enable);
    ui->groupsTableView->setEnabled(enable);
}

void GroupsManagerWidget::_displayErrorMessage(QString message)
{
    QMessageBox::warning(this, tr("Error"), message);
}

void GroupsManagerWidget::_rowInserted()
{
    ui->groupsTableView->resizeColumnToContents(1);
}

void GroupsManagerWidget::_groupFilter_textChanged(QString currentText)
{
    _groupsProxyModel->setFilterFixedString(currentText);
}

void GroupsManagerWidget::on_groupsTableView_doubleClicked(QModelIndex index)
{
    int targetRow = _groupsProxyModel->mapToSource(index).row();
    _editGroup(targetRow);
}

void GroupsManagerWidget::_customMenuRequested(QPoint pos)
{
    QModelIndex targetIdx = ui->groupsTableView->indexAt(pos);
    if (targetIdx.isValid()) {
        QMenu contextMenu;
        contextMenu.addAction(ui->action_Edit_group);
        contextMenu.addAction(ui->action_Delete_group);

        if (contextMenu.actions().count())
            contextMenu.exec(ui->groupsTableView->viewport()->mapToGlobal(pos));
    }
}

void GroupsManagerWidget::_successDeleteGroup(qlonglong id, QString name, bool ok)
{
    Q_UNUSED(id)
    if (!ok)
        QMessageBox::critical(this, "Error", QString("Unable to delete " + name));
}

void GroupsManagerWidget::_successAddGroup(qlonglong id, QString name, bool ok)
{
    Q_UNUSED(id)
    if (!ok)
        QMessageBox::critical(this, "Error", QString("Unable to add " + name));
}

void GroupsManagerWidget::_successUpdateGroup(qlonglong id, QString name, bool ok)
{
    Q_UNUSED(id)
    if (!ok)
        QMessageBox::critical(this, "Error", QString("Unable to update " + name));
}

/*******************************************************************************
* GroupsManagerWidget - Private
*/

void GroupsManagerWidget::_editGroup(int targetRow)
{
    int groupId = _groupsModel->groupIdByIndex(targetRow);
    if (groupId == -1)
        return;
    Diamesh::GroupPtr selectedGroupPtr = Diamesh::GroupPtr::dynamicCast(_accessor->getGroupById(groupId)->ice_clone());
    if (!selectedGroupPtr) {
        qCritical() << tr("Unable to get group");
        return;
    }
    GroupEditDialog popup(selectedGroupPtr, _accessor->getGroupNamesHash(), _accessor->getUserNamesHash(), this);
    if (popup.exec() == QDialog::Accepted) {
        _accessor->updateGroup(selectedGroupPtr, this, SLOT(_successUpdateGroup(qlonglong,QString,bool)));
    }
}

} // namespace Hnv
