/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "hnvwidgets.h"
#include "ressources.h"

// Qt headers
#include <QStyleOption>
#include <QPainter>
#include <QAction>
#include <QMessageBox>
#include <QVBoxLayout>
#include <QToolButton>
#include <QFocusEvent>
#include <QAbstractButton>

namespace Hnv {

/*******************************************************************************
* StyledWidget
*/
StyledWidget::StyledWidget(QWidget *parent)
    : QWidget(parent)
{
}

/*******************************************************************************
* StyledWidget - Protected
*/
void StyledWidget::paintEvent(QPaintEvent *)
{
    QStyleOption opt;
    opt.init(this);
    QPainter p(this);
    style()->drawPrimitive(QStyle::PE_Widget, &opt, &p, this);
}

/*******************************************************************************
* TableView
*/
TableView::TableView(QWidget *parent)
    : QTableView(parent)
    , _visibleColumns(0)
{
    setHorizontalHeader(new Hnv::HeaderView(this));
    horizontalHeader()->setClickable(true);
    connect(horizontalHeader(), SIGNAL(sectionCountChanged(int,int)), this, SLOT(_initContextMenu()));
    connect(horizontalHeader(), SIGNAL(sectionResized(int,int,int)), this, SLOT(_headerSectionResized(int,int,int)));
}

TableView::~TableView()
{
    _clearActionList();
}

int TableView::_lastColumnDraw(){
    for(int idx = model()->columnCount() - 1; idx >= 0 ; idx-- ){
        if(!isColumnHidden(idx)){
            return idx;
        }
    }
    return -1;
}

/*******************************************************************************
* TableView - Public slots
*/
void TableView::hideColumn(int column)
{
    setColumnHidden(column, true);
}

void TableView::showColumn(int column)
{
    setColumnHidden(column, false);
}

void TableView::setColumnHidden(int column, bool hide)
{
    bool isHidden = isColumnHidden(column);
    if (hide != isHidden && _visibleColumns > 1) {
        QTableView::setColumnHidden(column, hide);
        if(!hide){
            resizeColumnToContents(column);
            if(column == _lastColumnDraw()){
                resizeColumnToContents(column-1);
            }
        }
    }
}

void TableView::mouseMoveEvent(QMouseEvent *event)
{
    if (_hoverTracking) {
        QModelIndex index = indexAt(event->pos());
        if (index.isValid())
            emit cellHovered(index.row(), index.column());
    }
    QTableView::mouseMoveEvent(event);
}

/*******************************************************************************
* TableView - Private slots
*/
void TableView::_headerSectionResized(int section, int oldSize, int newSize)
{
    if (oldSize == 0 && newSize > 0) {
        _visibleColumns++;
        _generatedActionList.at(section)->setChecked(true);
    } else if (oldSize > 0 && newSize == 0) {
        _visibleColumns--;
        _generatedActionList.at(section)->setChecked(false);
    }
}

void TableView::_actionToggled(bool checked)
{
    QAction *action = qobject_cast<QAction*>(sender());
    if (!action)
        return;
    int pos = _generatedActionList.indexOf(action);
    if (pos != -1 && _visibleColumns > 1)
        setColumnHidden(pos, !checked);
    else
        action->setChecked(!checked);
}

void TableView::_initContextMenu()
{
    _clearActionList();
    QHeaderView * hHeader = horizontalHeader();
    _visibleColumns = hHeader->count();
    if (!_visibleColumns)
        return;
    QList<QAction *> generatedActionListTmp;
    for (int i=0; i < _visibleColumns; i++) {
        QString columnTitle = hHeader->model()->headerData(i, Qt::Horizontal).toString();
        if(columnTitle.isEmpty()){
            _generatedActionList.append(NULL);
            continue;
        }
        QAction * action = new QAction(hHeader);
        action->setCheckable(true);
        action->setText(columnTitle);
        action->setToolTip(columnTitle);
        action->setChecked(!isColumnHidden(i));
        connect(action, SIGNAL(triggered(bool)), this, SLOT(_actionToggled(bool)));
        _generatedActionList.append(action);
        generatedActionListTmp.append(action);
    }
    hHeader->addActions(generatedActionListTmp);
    horizontalHeader()->setContextMenuPolicy(Qt::ActionsContextMenu);
}

/*******************************************************************************
* TableView - Private
*/
void TableView::_clearActionList()
{
    if (_generatedActionList.count()) {
        foreach(QAction *action, _generatedActionList)
        {
            if(action != NULL)
                horizontalHeader()->removeAction(action);
        }
        qDeleteAll(_generatedActionList);
        _generatedActionList.clear();
    }
}


void TableView::disabledDoubleClickHeader()
{
    qobject_cast<Hnv::HeaderView *>(horizontalHeader())->disabledDoubleClick();
}

void TableView::setHoverTracking(bool enable)
{
    _hoverTracking = enable;
    setMouseTracking(enable);
}

HeaderView::HeaderView(QWidget *parent):
    QHeaderView(Qt::Horizontal, parent)
  ,_disableDoubleClick(false)
{

}

void HeaderView::mouseDoubleClickEvent(QMouseEvent *e)
{
    if (!_disableDoubleClick)
        QHeaderView::mouseDoubleClickEvent(e);
}

void HeaderView::disabledDoubleClick()
{
    _disableDoubleClick = true;
}


/*******************************************************************************
* RoundedFrame
*/
RoundedFrame::RoundedFrame(QWidget *titleWidget, QWidget *contentWidget, QWidget *parent)
    : QWidget(parent)
    , _titleWidget(titleWidget)
    , _contentWidget(contentWidget)
{
    QVBoxLayout * layout = new QVBoxLayout(this);
    layout->setSpacing(0);
    if (_titleWidget && _contentWidget) {
        _titleWidget->setObjectName("titleWidget");
        _contentWidget->setObjectName("contentWidget");
        layout->addWidget(_titleWidget);
        layout->addWidget(_contentWidget);
    }
}

RoundedFrame::~RoundedFrame()
{
    if (_titleWidget)
        delete _titleWidget;
    if (_contentWidget)
        delete _contentWidget;
}


/*******************************************************************************
* HnvDialogHandler
*/
HnvDialogHandler::HnvDialogHandler(QObject *sourceObject, QDialog *dialog, QWidget *parent)
    : QObject(parent)
    , _parentWidget(parent)
    , _dialog(dialog)
    , _displayMessageOnDestroy(true)
    , _sourceDestroyedMessage(tr("Source was deleted. Operation aborted."))
    , _sourceDestroyedSlotCalled(false)
{
    if (sourceObject)
        connect(sourceObject, SIGNAL(destroyed()), this, SLOT(_sourceDestroyedSlot()),Qt::QueuedConnection);
    if (_dialog) {
        connect(_dialog, SIGNAL(finished(int)), this, SLOT(_dialogClosed()));
        connect(_dialog, SIGNAL(destroyed()), this, SLOT(_dialogClosed()));
    }
}

HnvDialogHandler::~HnvDialogHandler()
{
}

void HnvDialogHandler::displayMessageOnDestroy(bool display)
{
    _displayMessageOnDestroy = display;
}

void HnvDialogHandler::setSourceDestroyedMessage(QString message)
{
    _sourceDestroyedMessage = message;
}

void HnvDialogHandler::addSourceObject(QObject *sourceObject)
{
    connect(sourceObject, SIGNAL(destroyed()), this, SLOT(_sourceDestroyedSlot()));
}

/*******************************************************************************
* HnvDialogHandler - Protected slots
*/
void HnvDialogHandler::_dialogClosed()
{
    _dialog = 0;
    disconnect();
    deleteLater();
}

void HnvDialogHandler::_sourceDestroyedSlot()
{
    if (_sourceDestroyedSlotCalled)
        return;
    _sourceDestroyedSlotCalled = true;
    disconnect();
    if (_dialog) {
        if (_displayMessageOnDestroy)
            QMessageBox::warning(_dialog, tr("Warning"), _sourceDestroyedMessage);
        _dialog->reject();
    }
    deleteLater();
}

/*******************************************************************************
* CloseButtonDelegate
*/
CloseButtonDelegate::CloseButtonDelegate(int targetColumn, QObject *parent)
    : QStyledItemDelegate(parent)
    , _targetColumn(targetColumn)
    , _hoveredRow(-1)
    , _isEditable(true)
    , _excludeRole(-1)
{
}

void CloseButtonDelegate::setEditable(bool isEditable)
{
    _isEditable = isEditable;
}

void CloseButtonDelegate::setExcludeRole(int role)
{
    _excludeRole = role;
}

void CloseButtonDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
{

    QStyledItemDelegate::paint(painter, option, index);
    if (index.column() == _targetColumn
            && _isEditable
            && (option.state & QStyle::State_MouseOver || index.row() == _hoveredRow )
            && ( _excludeRole == -1 || !index.data(_excludeRole).isValid() || !index.data(_excludeRole).toBool())
            ) {
        const QIcon icon(
                (option.state & QStyle::State_Selected) ?
                                       Ressources::themedPath(Constants::ICON_CLOSE_BUTTON) : Ressources::themedPath(Constants::ICON_DARK_CLOSE_BUTTON));
        QRect iconRect(option.rect.right() - option.rect.height(),
                       option.rect.top(),
                       option.rect.height(),
                       option.rect.height());
        icon.paint(painter, iconRect, Qt::AlignRight | Qt::AlignVCenter);
    }
}

void CloseButtonDelegate::cellHoverTrackingSlot(int row, int column)
{
    Q_UNUSED(column)
    _hoveredRow = row;
}

/*******************************************************************************
* ButtonLineEdit
*/
ButtonLineEdit::ButtonLineEdit(QPixmap pixmapButton, int sizeIcone ,QWidget *parent)
    : QLineEdit(parent)
{
    _genericButton = new QToolButton(this);
    _genericButton->setIconSize(QSize(sizeIcone,sizeIcone));
    _genericButton->setIcon(QIcon(pixmapButton));
    _genericButton->setCursor(Qt::ArrowCursor);
    _genericButton->setStyleSheet("QToolButton { border: none; padding: 0px; }");
    _genericButton->hide();
    connect(_genericButton, SIGNAL(clicked()), this, SIGNAL(buttonClicked()));
    int frameWidth = style()->pixelMetric(QStyle::PM_DefaultFrameWidth);
    QMargins margins = textMargins();
    margins.setRight(_genericButton->sizeHint().width() + frameWidth + 1);
    setTextMargins(margins);
    QSize msz = minimumSizeHint();
    setMinimumSize(qMax(msz.width(), _genericButton->sizeHint().height() + frameWidth * 2 + 2),
                   qMax(msz.height(), _genericButton->sizeHint().height() + frameWidth * 2 + 2));
}

void ButtonLineEdit::setButtonTooltip(const QString & tooltip)
{
    _genericButton->setToolTip(tooltip);
}


/*******************************************************************************
* ButtonLineEdit - Protected
*/
void ButtonLineEdit::resizeEvent(QResizeEvent *)
{
    QSize sz = _genericButton->sizeHint();
    int frameWidth = style()->pixelMetric(QStyle::PM_DefaultFrameWidth);
    _genericButton->move(rect().right() - frameWidth - sz.width() - 1,
                      (rect().bottom() - 1 - sz.height())/2);
}

void ButtonLineEdit::setButtonVisible(bool visible)
{
    _genericButton->setVisible(visible);
}


/*******************************************************************************
* WarningLineEdit
*/
WarningLineEdit::WarningLineEdit(QWidget *parent)
    : ButtonLineEdit(Ressources::themedPath(Constants::ICON_WARNING), 13 ,parent)
    , _warningOnFocus(false)
{
}

bool WarningLineEdit::warningOnFocus() const
{
    return _warningOnFocus;
}

void WarningLineEdit::setWarningOnFocus(bool enable)
{
    _warningOnFocus = enable;
}

void WarningLineEdit::focusInEvent(QFocusEvent *event)
{
    if (_warningOnFocus && event->gotFocus())
        setButtonVisible(true);
    QLineEdit::focusInEvent(event);
}


void WarningLineEdit::focusOutEvent(QFocusEvent *event)
{
    if (_warningOnFocus && event->lostFocus())
        setButtonVisible(false);
    QLineEdit::focusOutEvent(event);
}


/*******************************************************************************
* ActionButtonLinker
*/

ActionButtonLinker::ActionButtonLinker(QAction *action, QAbstractButton *button, QObject *parent)
    : QObject(parent)
    , _sourceAction(action)
    , _button(button)
{
    _createLink();
}

void ActionButtonLinker::setAction(QAction *action)
{
    _destroyLink();
    _sourceAction = action;
    if (_sourceAction)
        _createLink();
    else
        _clearButton();
}

void ActionButtonLinker::setButton(QAbstractButton *button)
{
    _destroyLink();
    _button = button;
    _createLink();
}

void ActionButtonLinker::_destroyLink()
{
    if (_sourceAction && _button) {
        _sourceAction->disconnect(_button);
        _button->disconnect(_sourceAction);
    }
}

void ActionButtonLinker::_createLink()
{
    if (_sourceAction == 0 || _button == 0)
        return;
    connect(_sourceAction, SIGNAL(destroyed()), this, SLOT(_actionDestroyed()));
    connect(_sourceAction, SIGNAL(changed()), this, SLOT(_actionChanged()));
    connect(_button, SIGNAL(destroyed()), this, SLOT(_buttonDestroyed()));
    connect(_button, SIGNAL(toggled(bool)), this, SLOT(_buttonToggled(bool)));
    connect(_button, SIGNAL(clicked(bool)), _sourceAction, SLOT(setChecked(bool)));
    connect(_button, SIGNAL(clicked()), _sourceAction, SLOT(trigger()));
    _updateButton();
}

void ActionButtonLinker::_actionChanged()
{
    _updateButton();
}

void ActionButtonLinker::_actionDestroyed()
{
    setAction(0);
}

void ActionButtonLinker::_buttonDestroyed()
{
    setButton(0);
}

void ActionButtonLinker::_buttonToggled(bool checked)
{
    if (_sourceAction)
        _sourceAction->setChecked(checked);
}

void ActionButtonLinker::_updateButton()
{
    if (_sourceAction == 0 || _button == 0)
        return;
    _button->setText(_sourceAction->text());
    _button->setIcon(_sourceAction->icon());
    _button->setToolTip(_sourceAction->toolTip());
    _button->setWhatsThis(_sourceAction->whatsThis());
    _button->setVisible(_sourceAction->isVisible());
    _button->setEnabled(_sourceAction->isEnabled());
    _button->setCheckable(_sourceAction->isCheckable());
    _button->setChecked(_sourceAction->isChecked());
}

void ActionButtonLinker::_clearButton()
{
    _button->setText(0);
    _button->setIcon(QIcon());
    _button->setToolTip(0);
    _button->setWhatsThis(0);
    _button->setVisible(false);
    _button->setEnabled(false);
    _button->setCheckable(false);
    _button->setChecked(false);
}

/*******************************************************************************
* ButtonListWidget
*/

ButtonListWidget::ButtonListWidget(QWidget *parent)
    : StyledWidget(parent)
{
}

ButtonListWidget::~ButtonListWidget()
{
}

} // namespace Hnv
