/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNVWIDGETS_H
#define HNVWIDGETS_H

// Qt headers
#include <QWidget>
#include <QTableView>
#include <QStyledItemDelegate>
#include <QHeaderView>
#include <QLineEdit>

class QAbstractButton;
class QToolButton;

namespace Hnv {

/*******************************************************************************
* StyledWidget
*/
class StyledWidget : public QWidget
{
    Q_OBJECT
public:
    explicit StyledWidget(QWidget *parent = 0);

protected:
    virtual void paintEvent(QPaintEvent *);

};

/*******************************************************************************
* TableView
*/
class TableView : public QTableView
{
    Q_OBJECT
public:
    explicit TableView(QWidget *parent = 0);
    ~TableView();
    void disabledDoubleClickHeader();
    void setHoverTracking(bool enable);

signals:
    void cellHovered(int row, int column);

public slots:
    virtual void hideColumn(int column);
    virtual void showColumn(int column);
    void setColumnHidden(int column, bool hide);

protected:
    void mouseMoveEvent(QMouseEvent *event);

private slots:
    void _headerSectionResized(int section, int oldSize, int newSize);
    void _actionToggled(bool checked);
    void _initContextMenu();
    int _lastColumnDraw();

private:
    void _clearActionList();
    QList<QAction *> _generatedActionList;
    int _visibleColumns;
    bool _hoverTracking;
};


class HeaderView : public QHeaderView
{
    Q_OBJECT
  public:
    HeaderView (QWidget * parent = 0);
    void disabledDoubleClick();

protected:
    virtual void mouseDoubleClickEvent(QMouseEvent *e);
    bool _disableDoubleClick;
};


/*******************************************************************************
* RoundedFrame
*/
class RoundedFrame : public QWidget
{
    Q_OBJECT
public:
    explicit RoundedFrame(QWidget * titleWidget, QWidget * contentWidget, QWidget * parent = 0);
    ~RoundedFrame();

protected:
    QWidget * _titleWidget;
    QWidget * _contentWidget;
};

/*******************************************************************************
* HnvDialogHandler
*/
class HnvDialogHandler: public QObject
{
    Q_OBJECT
public:
    explicit HnvDialogHandler(QObject *sourceObject, QDialog *dialog, QWidget *parent = 0);
    ~HnvDialogHandler();
    void displayMessageOnDestroy(bool display = true);
    void setSourceDestroyedMessage(QString message);
    void addSourceObject(QObject *sourceObject);

protected slots:
    void _sourceDestroyedSlot();
    void _dialogClosed();

private:
    QWidget * _parentWidget;
    QDialog * _dialog;
    bool _displayMessageOnDestroy;
    QString _sourceDestroyedMessage;
    bool _sourceDestroyedSlotCalled;
};

/*******************************************************************************
* CloseButtonDelegate
*/
class CloseButtonDelegate : public QStyledItemDelegate
{
    Q_OBJECT
public:
    explicit CloseButtonDelegate(int targetColumn, QObject *parent = 0);
    void setEditable(bool isEditable = true);
    void setExcludeRole(int role);
    void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const;

public slots:
    void cellHoverTrackingSlot(int row, int column);

private:
    int _targetColumn;
    int _hoveredRow;
    bool _isEditable;
    int _excludeRole;
};


/*******************************************************************************
* ButtonLineEdit
*/

class ButtonLineEdit : public QLineEdit
{
    Q_OBJECT
public:
    explicit ButtonLineEdit(QPixmap pixmapButton, int sizeIcon , QWidget *parent = 0);
    void setButtonTooltip(const QString & tooltip);

protected:
    void resizeEvent(QResizeEvent *);

public slots:
    void setButtonVisible(bool);

signals:
    void buttonClicked();

private:
    QToolButton *_genericButton;
    int _sizeIcon;

};


/*******************************************************************************
* WarningLineEdit
*/
class WarningLineEdit : public ButtonLineEdit
{
    Q_OBJECT
public:
    explicit WarningLineEdit(QWidget *parent = 0);
    bool warningOnFocus() const;
    void setWarningOnFocus(bool enable = true);

protected:
    virtual void focusInEvent(QFocusEvent *e);
    virtual void focusOutEvent(QFocusEvent *e);

    bool _warningOnFocus;

};

/*******************************************************************************
* ActionButtonLinker
*/

class ActionButtonLinker : public QObject
{
    Q_OBJECT
public:
    explicit ActionButtonLinker(QAction * action, QAbstractButton * button, QObject * parent = 0);
    void setAction(QAction * action);
    void setButton(QAbstractButton * button);
protected slots:
    void _actionDestroyed();
    void _actionChanged();
    void _buttonDestroyed();
    void _buttonToggled(bool checked);
protected:
    void _destroyLink();
    void _createLink();
    void _updateButton();
    void _clearButton();

    QAction * _sourceAction;
    QAbstractButton * _button;
};

/*******************************************************************************
* ButtonListWidget
*/

class ButtonListWidget : public StyledWidget
{
    Q_OBJECT
public:
    explicit ButtonListWidget(QWidget *parent = 0);
    ~ButtonListWidget();
};

} // namespace Hnv
#endif // HNVWIDGETS_H
