/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "hyneviewtoolbar.h"
#include "ressources.h"

// Qt headers
#include <QAbstractItemModel>
#include <QLabel>
#include <QComboBox>
#include <QAction>
#include <QPair>

namespace Hnv {

/*****************************************************************************
 *  ToolBar : Base toolbar for hyneview
 */
ToolBar::ToolBar(QWidget *parent, bool closeable, int leftMargin)
    : QToolBar(parent)
    , _closeAction(0)
    , _borders(Bottom)
    , _leftMargin(leftMargin)
{
    setMaximumHeight(26);
    setMinimumHeight(26); // TODO : Should use a constant
    setIconSize(QSize(Constants::TOOLBAR_ICON_SIZE_WIDTH, Constants::TOOLBAR_ICON_SIZE_HEIGHT));
    _leftMarginAction = insertSpacer(0, _leftMargin);
    setCloseable(closeable);
}

void ToolBar::setCloseable(bool closeable)
{
    if (!closeable && _closeAction) {
        delete _closeAction;
    } else if (closeable && _closeAction == 0) {
        _closeAction = new QAction(tr("Close"), this);
        _closeAction->setToolTip(tr("Close"));
        _closeAction->setIcon(QIcon(Ressources::themedPath(Constants::ICON_CLOSE_BUTTON)));
        connect(_closeAction, SIGNAL(triggered()), this, SIGNAL(closeTriggered()));
        addAction(_closeAction);
    }
}

QAction * ToolBar::insertSpacer(QAction *before, int size)
{
    QWidget *spacer = new QWidget(this);
    spacer->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
    if (size == -1)
        size = QWIDGETSIZE_MAX;
    spacer->setMaximumWidth(size);
    spacer->setMinimumWidth(size);
    if (before)
        return insertWidget(before, spacer);
    else
        return addWidget(spacer);
}

QAction * ToolBar::closeAction()
{
    return _closeAction;
}

void ToolBar::setBorders(ToolBar::Borders borders)
{
    if (_borders != borders) {
        _borders = borders;
        update();
    }
}

ToolBar::Borders ToolBar::borders() const
{
    return _borders;
}

void ToolBar::setLeftMargin(int margin)
{
    if (_leftMargin != margin) {
        _leftMargin = margin;
        widgetForAction(_leftMarginAction)->setMinimumWidth(_leftMargin);
        widgetForAction(_leftMarginAction)->setMaximumWidth(_leftMargin);
        update();
    }
}

int ToolBar::leftMargin() const
{
    return _leftMargin;
}


/*****************************************************************************
 *  TitleToolBar : Toolbar with static title
 */
TitleToolBar::TitleToolBar(QWidget *parent, bool closeable)
    : ToolBar(parent, closeable, 2)
{
    setMaximumHeight(26);
    setMinimumHeight(26); // TODO : Should use a constant
    _titleLabel = new QLabel(this);
    if (_closeAction) {
        _titleAction = insertWidget(_closeAction, _titleLabel);
        _spacerAction = insertSpacer(_closeAction);
    } else {
        _titleAction = addWidget(_titleLabel);
        _spacerAction = insertSpacer();
    }
}

void TitleToolBar::setTitle(QString title)
{
    _titleLabel->setText(title);
}

QString TitleToolBar::title()
{
    return _titleLabel->text();
}

void TitleToolBar::addToolBarAction(QAction *action, bool afterTitle)
{
    if (afterTitle)
        insertAction(_spacerAction, action);
    else
        insertAction(_titleAction, action);
}

/*****************************************************************************
 *  SelectorToolBar : Toolbar with combo as selector
 */
SelectorToolBar::SelectorToolBar(const QStringList &itemList, QWidget *parent, bool closeable)
    : ToolBar(parent, closeable)
{
    setMaximumHeight(26);
    setMinimumHeight(26); // TODO : Should use a constant
    _comboBox = new QComboBox(this);
    _comboBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
    connect(_comboBox, SIGNAL(currentIndexChanged(QString)), this, SIGNAL(currentIndexChanged(QString)));
    connect(_comboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(_currentIndexChanged(int)));
    if (actions().count())
        _comboBoxAction = insertWidget(actions().at(0), _comboBox);
    else
        _comboBoxAction = addWidget(_comboBox);
    setItemList(itemList);
}

void SelectorToolBar::setItemList(const QStringList &itemList)
{
    _comboBox->clear();
    if (itemList.count())
        foreach(QString itemTitle, itemList)
            _comboBox->addItem(itemTitle, QVariant(itemTitle));
}

void SelectorToolBar::addItem(const QString &itemText, QVariant itemData, bool setActive)
{
    _comboBox->addItem(itemText, itemData);
    if (setActive)
        _comboBox->setCurrentIndex(_comboBox->findData(itemData));
}

void SelectorToolBar::updateItemText(const QString &text, QVariant itemData)
{
    int index = _comboBox->findData(itemData);
    if (index != -1)
        _comboBox->setItemText(index, text);
}

void SelectorToolBar::removeItem(QVariant itemData)
{
    int index = _comboBox->findData(itemData);
    if (index != -1)
        _comboBox->removeItem(index);
}

QAction * SelectorToolBar::comboAction()
{
    return _comboBoxAction;
}

/*******************************************************************************
* SelectorToolBar - Public slots
*/
void SelectorToolBar::setSelectedItem(QVariant itemData)
{
    _comboBox->setCurrentIndex(_comboBox->findData(itemData));
}

void SelectorToolBar::setSelectedItem(int index)
{
    _comboBox->setCurrentIndex(index);
}

/*******************************************************************************
* SelectorToolBar - Protected slots
*/
void SelectorToolBar::_currentIndexChanged(int index)
{
    if (index != -1)
        emit currentIndexChanged(_comboBox->itemData(index));
//    else
//        qDebug() << tr("Combo box selection changed to null %1").arg(_comboBox->count());
}


/*****************************************************************************
 *  SideBarToolBar : Toolbar with split button
 */
SideBarToolBar::SideBarToolBar(QList<QPair<QString,QString> > itemList, QWidget *parent, bool closeable)
    : SelectorToolBar(QStringList(), parent, closeable)
{
    setMaximumHeight(26);
    setMinimumHeight(26); // TODO : Should use a constant
    setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
    QAction *btnSplit = new QAction(tr("Split"), this);
    btnSplit->setToolTip(tr("Split side bar"));
    btnSplit->setIcon(QIcon(Ressources::themedPath(Constants::ICON_SPLIT_BUTTON)));
    connect(btnSplit, SIGNAL(triggered()), this, SIGNAL(splitTriggered()));
    insertAction(actions().last(), btnSplit);
    QListIterator<QPair<QString,QString> > iterator(itemList);
    while(iterator.hasNext()) {
        QPair<QString,QString> item = iterator.next();
        addItem(item.first, item.second);
    }
}

/*****************************************************************************
 *  TopologyToolBar : Topology Toolbar with topology list, objects list
 */
TopologyToolBar::TopologyToolBar(QWidget *parent, const QStringList &topologyList, bool closeable)
    : SelectorToolBar(topologyList, parent, closeable)
{
    setMaximumHeight(26);
    setMinimumHeight(26); // TODO : Should use a constant
    setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum);
    _objectsComboBox = new QComboBox(this);
    _objectsComboBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
    connect(_objectsComboBox, SIGNAL(currentIndexChanged(QString)), this, SIGNAL(currentIndexChanged(QString)));
    insertWidget(actions().last(), _objectsComboBox);
    connect(_comboBox, SIGNAL(currentIndexChanged(QString)), this, SLOT(_topologyIndexChangedSlot(QString)));
    connect(_objectsComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(_currentObjectIndexChangedSlot(int)));
    _closeAction->setEnabled(false);
    setEnabled(false);
}

void TopologyToolBar::initModel(QAbstractItemModel *model)
{
    _comboBox->setModel(model);
    _comboBox->setModelColumn(0);
}

void TopologyToolBar::addEntity(qlonglong entityId, const QString &shortName)
{
    _objectsComboBox->addItem(shortName, entityId);
}

void TopologyToolBar::removeEntity(int entityId)
{
    _objectsComboBox->removeItem(_objectsComboBox->findData(entityId));
}

void TopologyToolBar::updateEntity(int entityId, const QString &shortName)
{
    int index = _objectsComboBox->findData(entityId);
    if (index != -1)
        _objectsComboBox->setItemText(index, shortName);
}

void TopologyToolBar::resetObjectList(const QHash<qlonglong, QString> &objectList)
{
    _objectsComboBox->clear();
    QHashIterator<qlonglong, QString> it(objectList);
    while (it.hasNext()) {
        it.next();
        _objectsComboBox->addItem(it.value(), it.key());
    }
}

void TopologyToolBar::setSelectedEntity(qlonglong entityID)
{
    _objectsComboBox->setCurrentIndex(_objectsComboBox->findData(entityID));
}

/*******************************************************************************
* TopologyToolBar - Private slots
*/
void TopologyToolBar::_currentObjectIndexChangedSlot(int index)
{
    if (index != -1) {
        qlonglong entityId = _objectsComboBox->itemData(index).toLongLong();
        emit currentObjectIndexChanged(entityId);
    }
}

void TopologyToolBar::_topologyIndexChangedSlot(const QString &text)
{
    int index = _comboBox->findText(text);
    _closeAction->setEnabled((index != -1));
}

} // namespace Hnv
