/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "importarchivemodel.h"
#include "ressources.h"
#include "hnvutility.h"

// Qt headers
#include <QFont>
#include <QIcon>
#include <QBrush>

namespace Hnv {

/*******************************************************************************
* ImportableTableModel
*******************************************************************************/
ImportableTableModel::ImportableTableModel(const QList<Hns::ImportableArchivePtr> &dataList, QObject *parent)
    : QAbstractTableModel(parent)
    , _dataList(dataList)
{
}

ImportableTableModel::~ImportableTableModel()
{

}

int ImportableTableModel::rowCount(const QModelIndex &parent) const
{
    if (!parent.isValid())
        return _dataList.size();
    return 0;
}

int ImportableTableModel::columnCount(const QModelIndex &parent) const
{
    if (!parent.isValid())
        return MODEL_COLUMN_COUNT;
    return 0;
}

QVariant ImportableTableModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if (role == Qt::DisplayRole && orientation == Qt::Horizontal) {
        switch(section) {
        case COLUMN_NAME:
            return tr("Name");
        case COLUMN_PATH:
            return tr("Path");
        case COLUMN_ENTITY_COUNT:
            return tr("Entities");
        case COLUMN_TOPOLOGY_COUNT:
            return tr("Topologies");
        case COLUMN_IMPORTABLE_SIZE:
            return tr("Size");
        }
    }
    return QAbstractItemModel::headerData(section, orientation, role);
}

QVariant ImportableTableModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid() || index.row() >= _dataList.count())
        return QVariant();

    Hns::ImportableArchivePtr importable = _dataList.value(index.row());
    if (role == ImportableKeyRole)
        return QString::fromStdString(importable->path);
    else if (role == Qt::DisplayRole) {
        switch(index.column()) {
        case COLUMN_NAME:
            return QString::fromStdString(importable->archive->name);
        case COLUMN_PATH:
            return QString::fromStdString(importable->path);
        case COLUMN_ENTITY_COUNT:
            return QString::number(importable->archive->entities.size());
        case COLUMN_TOPOLOGY_COUNT:
            return QString::number(importable->archive->topologies.size());
        case COLUMN_IMPORTABLE_SIZE:
            return Utility::makeSizeReadable(importable->size);
        }
    }
    return QVariant();
}


/*******************************************************************************
* ArchiveItem
*******************************************************************************/
ArchiveItem::ArchiveItem(QObject *parent)
    : TreeModelItem(MODEL_COLUMN_COUNT, parent)
{
}

QVariant ArchiveItem::data(int column, int role) const
{
    if (column == COLUMN_SHORTNAME) {
        switch(role) {
        case Qt::DisplayRole:
            return getShortname();
        case Qt::DecorationRole:
            return getDecoration();
        case Qt::ToolTipRole:
            return getTooltip(); //TODO: use Ressources::getToolTip function
        }
    }
    return QVariant();
}

QString ArchiveItem::getTooltip() const
{
    QString name = getLongname();
    if (name.isEmpty())
        name = getShortname();
    QString toolTip = tr("<b>Name:&nbsp;</b><i>%1</i>").arg(name);
    QString description = getDescription();
    if (!description.isEmpty())
        toolTip += QString("<br/><br/>%1").arg(description);
    return toolTip;
}

/*******************************************************************************
* ArchiveVirtualFolderItem
*******************************************************************************/
ArchiveVirtualFolderItem::ArchiveVirtualFolderItem(const QString &folderName, QObject *parent)
    : ArchiveItem(parent)
    , _folderName(folderName)
{
}

QVariant ArchiveVirtualFolderItem::data(int column, int role) const
{
    if (column == COLUMN_SHORTNAME) {
        switch(role) {
        case Qt::FontRole: {
            QFont font;
            font.setBold(true);
            return font;
        }
        case Qt::BackgroundRole: {
            QBrush brush(Qt::lightGray, Qt::SolidPattern);
            return brush;
        }
        }
    }
    return ArchiveItem::data(column, role);
}

QString ArchiveVirtualFolderItem::getShortname() const
{
    return _folderName;
}

QString ArchiveVirtualFolderItem::getLongname() const
{
    return QString();
}

QString ArchiveVirtualFolderItem::getDescription() const
{
    return QString();
}

QIcon ArchiveVirtualFolderItem::getDecoration() const
{
    return QIcon(QPixmap(Ressources::themedPath(Constants::ICON_DATABASE)));
}

QString ArchiveVirtualFolderItem::getTooltip() const
{
    return getShortname();
}

/*******************************************************************************
* ArchiveFolderDescriptorItem
*******************************************************************************/
ArchiveFolderDescriptorItem::ArchiveFolderDescriptorItem(const Hns::FolderDescriptor &descriptor, QObject *parent)
    : ArchiveItem(parent)
    , _descriptor(descriptor)
{
}

QString ArchiveFolderDescriptorItem::getShortname() const
{
    return QString::fromStdString(_descriptor.shortName);
}

QString ArchiveFolderDescriptorItem::getLongname() const
{
    return QString::fromStdString(_descriptor.longName);
}

QString ArchiveFolderDescriptorItem::getDescription() const
{
    return QString::fromStdString(_descriptor.description);
}

QIcon ArchiveFolderDescriptorItem::getDecoration() const
{
    return QIcon(QPixmap(Ressources::themedPath(Constants::ICON_FOLDER)));
}

Hns::FolderDescriptor ArchiveFolderDescriptorItem::getFolderDescriptor() const
{
    return _descriptor;
}

/*******************************************************************************
* ArchiveEntityDescriptorItem
*******************************************************************************/
ArchiveEntityDescriptorItem::ArchiveEntityDescriptorItem(const Hns::EntityDescriptor descriptor, QObject *parent)
    : ArchiveItem(parent)
    , _descriptor(descriptor)
{
}

QString ArchiveEntityDescriptorItem::getShortname() const
{
    return QString::fromStdString(_descriptor.shortName);
}

QString ArchiveEntityDescriptorItem::getLongname() const
{
    return QString::fromStdString(_descriptor.longName);
}

QString ArchiveEntityDescriptorItem::getDescription() const
{
    return QString::fromStdString(_descriptor.description);
}

QIcon ArchiveEntityDescriptorItem::getDecoration() const
{
    return QIcon(Ressources::getClassNamePixmap(QString::fromStdString(_descriptor.type)));
}

/*******************************************************************************
* ArchiveTopologyDescriptorItem
*******************************************************************************/
ArchiveTopologyDescriptorItem::ArchiveTopologyDescriptorItem(const Hns::TopologyDescriptor descriptor, QObject *parent)
    : ArchiveItem(parent)
    , _descriptor(descriptor)
{
}

QString ArchiveTopologyDescriptorItem::getShortname() const
{
    return QString::fromStdString(_descriptor.shortName);
}

QString ArchiveTopologyDescriptorItem::getLongname() const
{
    return QString::fromStdString(_descriptor.longName);
}

QString ArchiveTopologyDescriptorItem::getDescription() const
{
    return QString::fromStdString(_descriptor.description);
}

QIcon ArchiveTopologyDescriptorItem::getDecoration() const
{
    return QIcon(QPixmap(Ressources::themedPath(Constants::DEFAULT_PREVIEW)));
}

/*******************************************************************************
* ArchiveTreeModel
*******************************************************************************/
ArchiveTreeModel::ArchiveTreeModel(QObject *parent)
    : TreeModel(parent)
    , _archiveRootItem(0)
    , _entitiesRootItem(0)
    , _topologiesRootItem(0)
{
    _initRootItem(ArchiveItem::MODEL_COLUMN_COUNT);
}

ArchiveTreeModel::~ArchiveTreeModel()
{
    _resetModel();
}

void ArchiveTreeModel::setArchiveData(const Hns::ArchiveDescriptorPtr &descriptor)
{
    _resetModel();
//    _archiveRootItem = _createVirtualFolder(QString::fromStdString(descriptor.name));
    _populateModel(descriptor);
}

void ArchiveTreeModel::setArchiveData(const Hns::ImportableArchiveList &list)
{
    _resetModel();
    foreach (const Hns::ImportableArchivePtr &archive, list) {
        _populateModel(archive->archive);
    }
}

void ArchiveTreeModel::_resetModel()
{
    beginResetModel();
    _rootItem->clear();
    _archiveRootItem = 0;
    _entitiesRootItem = 0;
    _topologiesRootItem = 0;
    _folderIDToItemHash.clear();
    endResetModel();
}

void ArchiveTreeModel::_populateModel(const Hns::ArchiveDescriptorPtr &descriptor)
{
    QList<Hns::FolderDescriptor> folderList = QList<Hns::FolderDescriptor>::fromStdList(descriptor->folders);
    QList<Hns::EntityDescriptor> entityList = QList<Hns::EntityDescriptor>::fromStdList(descriptor->entities);
    QList<Hns::TopologyDescriptor> topologyList = QList<Hns::TopologyDescriptor>::fromStdList(descriptor->topologies);
    ArchiveVirtualFolderItem *rootItem = _createVirtualFolder(descriptor->name.c_str());
    if (folderList.count() > 0 || entityList.count() > 0) {
//        _entitiesRootItem = _createVirtualFolder(tr("Entities"), _archiveRootItem);
        _entitiesRootItem = _createVirtualFolder(tr("Entities"), rootItem);
        _handleFolderDescriptorList(folderList);
        _handleEntityDescriptorList(entityList);
    }

    if (topologyList.count() > 0) {
        _topologiesRootItem = _createVirtualFolder(tr("Topologies"), rootItem);
        _handleTopologyDescriptorList(topologyList);
    }

}

ArchiveVirtualFolderItem * ArchiveTreeModel::_createVirtualFolder(const QString &folderName, TreeModelItem *parentItem)
{
    if (parentItem == 0)
        parentItem = _rootItem;
    ArchiveVirtualFolderItem * item = new ArchiveVirtualFolderItem(folderName, this);
    addItem(parentItem, item);
    return item;
}

ArchiveItem * ArchiveTreeModel::_getParentItemByFolderID(const qlonglong &parentID)
{
    ArchiveItem * parentItem = 0;
    if (parentID == 0)
        parentItem = _entitiesRootItem;
    else
        parentItem = _folderIDToItemHash.value(parentID, 0);
    return parentItem;
}

bool ArchiveTreeModel::_addToParentByID(const qlonglong &parentID, ArchiveItem *item)
{
    bool result = false;
    ArchiveItem * parentItem = _getParentItemByFolderID(parentID);
    if (parentItem) {
        addItem(parentItem, item);
        result = true;
    }
    return result;
}

void ArchiveTreeModel::_createArchiveEntityItem(ArchiveItem *parentItem, const Hns::EntityDescriptor &descriptor)
{
    if (parentItem == 0)
        return;
    ArchiveEntityDescriptorItem * item = new ArchiveEntityDescriptorItem(descriptor, this);
    addItem(parentItem, item);
}

void ArchiveTreeModel::_handleFolderDescriptorList(const QList<Hns::FolderDescriptor> &folderList)
{
    QList<ArchiveFolderDescriptorItem*> orphanFolders;
    foreach(const Hns::FolderDescriptor &folder, folderList) {
        qlonglong folderID = folder.id;
        // Already added
        if (_folderIDToItemHash.contains(folderID)) {
            continue;
        }
        qlonglong parentID = folder.parentId;
        // Make sure that parent ID is valid
        if (parentID == folder.id)
            parentID = 0;
        ArchiveFolderDescriptorItem * item = new ArchiveFolderDescriptorItem(folder, this);
        _folderIDToItemHash.insert(folder.id, item);
        if (!_addToParentByID(parentID, item))
            orphanFolders.append(item);
    }
    QMutableListIterator<ArchiveFolderDescriptorItem *> iterator(orphanFolders);
    while(iterator.hasNext()) {
        iterator.next();
        ArchiveFolderDescriptorItem * item = iterator.value();
        qlonglong parentID = item->getFolderDescriptor().parentId;
        if (_addToParentByID(parentID, item)) {
            iterator.remove();
            iterator.toFront();
        }
    }
}

void ArchiveTreeModel::_handleEntityDescriptorList(const QList<Hns::EntityDescriptor> &entityList)
{
    foreach(const Hns::EntityDescriptor &entity, entityList) {
        ArchiveItem *parentItem = _getParentItemByFolderID(entity.parentId);
        if (parentItem) {
            _createArchiveEntityItem(parentItem, entity);
        } else
            _createArchiveEntityItem(_entitiesRootItem, entity);
    }
}

void ArchiveTreeModel::_handleTopologyDescriptorList(const QList<Hns::TopologyDescriptor> &topologyList)
{
    foreach(const Hns::TopologyDescriptor &topology, topologyList) {
        ArchiveTopologyDescriptorItem * item = new ArchiveTopologyDescriptorItem(topology, this);
        addItem(_topologiesRootItem, item);
    }
}

} // namespace Hnv
