/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNV_IMPORTABLEMODEL_H
#define HNV_IMPORTABLEMODEL_H

// Project headers
#include "treemodel.h"
#include "treemodelitem.h"

// Generated hynesim headers
#include "common/importable.h"
#include "common/archivedescriptor.h"
#include "common/catalogimporter.h"

// Qt headers
#include <QAbstractTableModel>

namespace Hnv {

/*******************************************************************************
* ImportableTableModel
*******************************************************************************/
class ImportableTableModel : public QAbstractTableModel
{
    Q_OBJECT
public:
    enum {
        COLUMN_NAME,
        COLUMN_PATH,
        COLUMN_ENTITY_COUNT,
        COLUMN_TOPOLOGY_COUNT,
        COLUMN_IMPORTABLE_SIZE,
        MODEL_COLUMN_COUNT          // Must be last of enum
    };

    enum {
        ImportableKeyRole = Qt::UserRole + 1
    };

    explicit ImportableTableModel(const QList<Hns::ImportableArchivePtr> &dataList, QObject *parent = 0);
    ~ImportableTableModel();

    int rowCount(const QModelIndex &parent) const;
    int columnCount(const QModelIndex &parent) const;
    QVariant headerData(int section, Qt::Orientation orientation, int role) const;
    QVariant data(const QModelIndex &index, int role) const;

private:
    QList<Hns::ImportableArchivePtr> _dataList;
};

/*******************************************************************************
* ArchiveItem
*******************************************************************************/
class ArchiveItem : public TreeModelItem
{
    Q_OBJECT
public:
    enum {
        COLUMN_SHORTNAME,
        MODEL_COLUMN_COUNT          // Must be last of enum
    };
    explicit ArchiveItem(QObject * parent = 0);
    virtual QVariant data(int column, int role) const;
    virtual QString getShortname() const = 0;
    virtual QString getLongname() const = 0;
    virtual QString getDescription() const = 0;
    virtual QIcon getDecoration() const = 0;
    virtual QString getTooltip() const;
};


/*******************************************************************************
* ArchiveVirtualFolderItem
*******************************************************************************/
class ArchiveVirtualFolderItem : public ArchiveItem
{
    Q_OBJECT
public:
    explicit ArchiveVirtualFolderItem(const QString &folderName, QObject * parent = 0);
    virtual QVariant data(int column, int role) const;
    virtual QString getShortname() const;
    virtual QString getLongname() const;
    virtual QString getDescription() const;
    virtual QIcon getDecoration() const;
    virtual QString getTooltip() const;
private:
    QString _folderName;
};

/*******************************************************************************
* ArchiveFolderDescriptorItem
*******************************************************************************/
class ArchiveFolderDescriptorItem : public ArchiveItem
{
    Q_OBJECT
public:
    explicit ArchiveFolderDescriptorItem(const Hns::FolderDescriptor &descriptor, QObject * parent = 0);
    virtual QString getShortname() const;
    virtual QString getLongname() const;
    virtual QString getDescription() const;
    virtual QIcon getDecoration() const;
    Hns::FolderDescriptor getFolderDescriptor() const;
private:
    Hns::FolderDescriptor _descriptor;
};

/*******************************************************************************
* ArchiveEntityDescriptorItem
*******************************************************************************/
class ArchiveEntityDescriptorItem : public ArchiveItem
{
    Q_OBJECT
public:
    explicit ArchiveEntityDescriptorItem(const Hns::EntityDescriptor descriptor, QObject * parent = 0);
    virtual QString getShortname() const;
    virtual QString getLongname() const;
    virtual QString getDescription() const;
    virtual QIcon getDecoration() const;
private:
    Hns::EntityDescriptor _descriptor;
};

/*******************************************************************************
* ArchiveTopologyDescriptorItem
*******************************************************************************/
class ArchiveTopologyDescriptorItem : public ArchiveItem
{
    Q_OBJECT
public:
    explicit ArchiveTopologyDescriptorItem(const Hns::TopologyDescriptor descriptor, QObject * parent = 0);
    virtual QString getShortname() const;
    virtual QString getLongname() const;
    virtual QString getDescription() const;
    virtual QIcon getDecoration() const;
private:
    Hns::TopologyDescriptor _descriptor;
};

/*******************************************************************************
* ArchiveTreeModel
*******************************************************************************/
class ArchiveTreeModel : public TreeModel
{
    Q_OBJECT
public:
    explicit ArchiveTreeModel(QObject * parent = 0);
    ~ArchiveTreeModel();
    void setArchiveData(const Hns::ArchiveDescriptorPtr &descriptor);
    void setArchiveData(const Hns::ImportableArchiveList &list);

protected:
    void _resetModel();
    void _populateModel(const Hns::ArchiveDescriptorPtr &descriptor);
    ArchiveVirtualFolderItem * _createVirtualFolder(const QString &folderName, TreeModelItem * parentItem = 0);
    void _createArchiveEntityItem(ArchiveItem * parentItem, const Hns::EntityDescriptor &descriptor);
    ArchiveItem * _getParentItemByFolderID(const qlonglong &parentID);
    bool _addToParentByID(const qlonglong &parentID, ArchiveItem * item);
    void _handleFolderDescriptorList(const QList<Hns::FolderDescriptor> &folderList);
    void _handleEntityDescriptorList(const QList<Hns::EntityDescriptor> &entityList);
    void _handleTopologyDescriptorList(const QList<Hns::TopologyDescriptor> &topologyList);

private:
    ArchiveVirtualFolderItem * _archiveRootItem;
    ArchiveVirtualFolderItem * _entitiesRootItem;
    ArchiveVirtualFolderItem * _topologiesRootItem;
    QHash<qlonglong, ArchiveFolderDescriptorItem *> _folderIDToItemHash;
};

} // namespace Hnv

#endif // HNV_IMPORTABLEMODEL_H
