// Project headers
#include "importdialog.h"
#include "ui_importdialog.h"
#include "hyneviewsession.h"
#include "importentrymodel.h"
#include "importarchivemodel.h"
#include "customrightdialog.h"

// hynesim hnsclient headers
#include <libhnsclient/entry.h>
#include <libhnsclient/catalogimporter.h>

// Qt headers
#include <QMessageBox>
#include <QSortFilterProxyModel>
#include <QModelIndex>

namespace Hnv {

ImportDialog::ImportDialog(QWidget *parent)
    : QDialog(parent)
    , ui(new Ui::ImportDialog)
    , _filterProxyModel(0)
    , _importEntryModel(0)
    , _catalogImporter(0)
    , _archiveTreeModel(0)
    , _numberOfImport(0)
{
    ui->setupUi(this);
    setWindowTitle(tr("Import"));

    ui->progressBar->setVisible(false);
    ui->importPushButton->setEnabled(false);
    ui->topologyNameLineEdit->setEnabled(false);
    ui->entitiesTableView->setEnabled(false);
    ui->entityPathComboBox->setEnabled(false);
    ui->topologiesTreeView->setEnabled(false);
    ui->customRightsButton->setEnabled(false);

    _filterProxyModel = new QSortFilterProxyModel(this);
    _importEntryModel = new ImportEntryModel(QMap<QString, HnsClient::EntityEntry *>(), this);
    _filterProxyModel->setSourceModel(_importEntryModel);
    _filterProxyModel->setFilterKeyColumn(0);
    ui->entitiesTableView->setModel(_filterProxyModel);
    ui->entitiesTableView->horizontalHeader()->setStretchLastSection(true);
    connect(ui->entitiesTableView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)), this, SLOT(_selectionChangedSlot()));

    _archiveTreeModel = new ArchiveTreeModel(this);
    ui->topologiesTreeView->setModel(_archiveTreeModel);
    connect(ui->topologiesTreeView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)), this, SLOT(_selectionChangedSlot()));
    ui->topologiesTreeView->header()->hide();

    HyneviewSession * session = HyneviewSession::instance();
    _catalogImporter = session->catalogImporter();
    QMap<QString, QString> entityCatalogStorageMap = session->entityCatalogStorageMap();
    QMapIterator<QString, QString> entityIterator(entityCatalogStorageMap);
    while(entityIterator.hasNext()) {
        entityIterator.next();
        ui->entityPathComboBox->addItem(entityIterator.value(), entityIterator.key());
    }

    connect(session->catalogImporter(), SIGNAL(progressUpdated(int)), this, SLOT(_progressUpdatedSlot(int)));
    session->catalogImporter()->getImportableEntityEntry(this, SLOT(_getImportableEntityEntrySlot(QMap<QString,HnsClient::EntityEntry*>)));
    QMap<QString, QString> topologyCatalogStorageMap = session->topologyCatalogStorageMap();
    QMapIterator<QString, QString> topologyIterator(topologyCatalogStorageMap);
    while(topologyIterator.hasNext()) {
        topologyIterator.next();
        ui->topologyPathComboBox->addItem(topologyIterator.value(), topologyIterator.key());
    }
    ui->tabWidget->setCurrentWidget(ui->tabWidgetEntityPage);
    ui->topologyNameLineEdit->setEnabled(false);
    ui->topologyPathComboBox->setEnabled(false);
    _catalogImporter->getArchiveList(this, SLOT(_getArchiveListSlot(Hns::ImportableArchiveList)));

    connect(session, SIGNAL(disconnected()), this, SLOT(_sessionDisconnected()));
}

ImportDialog::~ImportDialog()
{
    delete ui;
}

void ImportDialog::_getImportableEntityEntrySlot(QMap<QString, HnsClient::EntityEntry *> importableMap)
{
    ui->entitiesTableView->setEnabled(true);
    ui->entityPathComboBox->setEnabled(true);
    _importEntryModel->setImportableMap(importableMap);
    ui->entitiesTableView->resizeColumnsToContents();
    _mapEntitiesImport = importableMap;
    on_tabWidget_currentChanged(0);
}

void ImportDialog::_getArchiveListSlot(Hns::ImportableArchiveList archiveList)
{
    ui->topologiesTreeView->setEnabled(true);
    ui->topologyNameLineEdit->setEnabled(true);
    ui->topologyPathComboBox->setEnabled(true);
    _importableList = QList<Hns::ImportableArchivePtr>::fromStdList(archiveList);
    for (int i = 0, size = _importableList.size() ; i < size ; ++i) {
        Hns::ImportOptionsList optionList;
        Hns::ImportOptions opt;
        opt.newName = _importableList.at(i)->archive->topologies.front().shortName + "_0";
        optionList.push_back(opt);
        _optionsList.insert(i, optionList);
    }
    _archiveTreeModel->setArchiveData(archiveList);
    ui->topologiesTreeView->resizeColumnToContents(0);
    on_tabWidget_currentChanged(0);
}

void ImportDialog::_selectionChangedSlot()
{
    int value = 1;
    ui->importPushButton->setEnabled(false);
    if(ui->topologiesTreeView->selectionModel()->hasSelection() && ui->tabWidget->currentWidget() == ui->tabWidgetTopologyPage) {
        ui->customRightsButton->setEnabled(true);
        QModelIndexList indexList = ui->topologiesTreeView->selectionModel()->selectedIndexes();
        if (indexList.count() == 1) {
            QModelIndex index = indexList.first();
            while (index.parent().isValid()) {
                /* Get the root item for index */
                index = index.parent();
            }
            int row = index.row();
            if (row < _importableList.count()) {
                Hns::ImportableArchivePtr archive = Hns::ImportableArchivePtr::dynamicCast(_importableList.at(row));
                if (archive) {
                    ui->topologyNameLineEdit->setText(QString::fromStdString(archive->archive->topologies.front().shortName));
                    ui->topologyNameLineEdit->setEnabled(true);
                    value = _optionsList.value(row).size();
                    ui->importCountSpinBox->setValue(value);
                    ui->importPushButton->setEnabled(true);
                }
            }
        }
    }

    if (value != 1)
        ui->topologyNameLineEdit->setEnabled(false);
    else
        ui->topologyNameLineEdit->setEnabled(true);

    if(ui->entitiesTableView->selectionModel()->hasSelection() && ui->tabWidget->currentWidget() == ui->tabWidgetEntityPage) {
        ui->importPushButton->setEnabled(true);
        ui->topologyNameLineEdit->setEnabled(false);
    }
}

void ImportDialog::on_importPushButton_clicked()
{
    ui->progressBar->setVisible(true);
    ui->importPushButton->setEnabled(false);
    ui->topologyNameLineEdit->setEnabled(false);
    ui->buttonBox->setEnabled(false);
    if(ui->entitiesTableView->selectionModel()->hasSelection() && ui->tabWidget->currentWidget() == ui->tabWidgetEntityPage) {
        HyneviewSession * session = HyneviewSession::instance();
        if (session == 0)
            return;

        QModelIndexList selectedRows = ui->entitiesTableView->selectionModel()->selectedRows();
        _numberOfImport = selectedRows.size();
        QString folderPath = ui->entityPathComboBox->itemData(ui->entityPathComboBox->currentIndex()).toString();
        foreach(QModelIndex modelIdx, selectedRows) {
            QVariant folderName = modelIdx.data(Qt::UserRole);
            if (folderName.isValid()) {
                QString strFolder = folderName.toString();
                session->catalogImporter()->importEntityEntry(strFolder, folderPath, this, SLOT(_importFolderSlot(bool)));
            }
        }
    }
    if(ui->topologiesTreeView->selectionModel()->hasSelection() && ui->tabWidget->currentWidget() == ui->tabWidgetTopologyPage) {
        QModelIndexList indexList = ui->topologiesTreeView->selectionModel()->selectedIndexes();
        if (indexList.count() == 1) {
            QModelIndex index = indexList.first();
            while (index.parent().isValid()) {
                /* Get the root item for index */
                index = index.parent();
            }
            int row = index.row();
            if (row < _importableList.count()) {
                Hns::ImportableArchivePtr archive = Hns::ImportableArchivePtr::dynamicCast(_importableList.at(row));
                Hns::ImportOptionsList optionsList = _optionsList.value(row);
                int optionsListSize = optionsList.size();
                if (optionsListSize == 1) {
                    Hns::ImportOptions opt = optionsList.front();
                    opt.newName = ui->topologyNameLineEdit->text().toStdString();
                    optionsList.pop_front();
                    optionsList.push_back(opt);
                    ui->progressBar->setMinimum(0);
                    ui->progressBar->setMaximum(0);
                } else {
                    ui->progressBar->setTextVisible(true);
                    ui->progressBar->setProperty("format", tr("Importing topology %v of %m"));
                    ui->progressBar->setMinimum(0);
                    ui->progressBar->setMaximum(optionsListSize);
                    ui->progressBar->setValue(1);
                }
                if (archive) {
                    _catalogImporter->importArchive(ui->topologyPathComboBox->itemData(ui->topologyPathComboBox->currentIndex()).toString(),
                                                    ui->entityPathComboBox->itemData(ui->entityPathComboBox->currentIndex()).toString(),
                                                    archive, optionsList, this, SLOT(_slotImportArchiveFinished(bool)));
                }
            }
        }
    }
}

void ImportDialog::_importFolderSlot(bool success)
{
    if (--_numberOfImport == 0)
        _slotImportArchiveFinished(success);
}

void ImportDialog::_slotImportArchiveFinished(bool success)
{
    ui->topologiesTreeView->setEnabled(false);
    ui->topologyNameLineEdit->setEnabled(false);
    ui->topologyPathComboBox->setEnabled(false);
    ui->entitiesTableView->setEnabled(false);
    ui->entityPathComboBox->setEnabled(false);
    if (_catalogImporter) {
        _catalogImporter->getArchiveList(this, SLOT(_getArchiveListSlot(Hns::ImportableArchiveList)));
        _catalogImporter->getImportableEntityEntry(this, SLOT(_getImportableEntityEntrySlot(QMap<QString,HnsClient::EntityEntry*>)));
    }
    ui->progressBar->setMinimum(0);
    ui->progressBar->setMaximum(0);
    ui->progressBar->setVisible(false);
    ui->progressBar->setProperty("format", "");
    ui->buttonBox->setEnabled(true);
    if (success)
        QMessageBox::information(this, tr("Success"), tr("Import finished"), QMessageBox::Ok);
    else
        QMessageBox::warning(this, tr("Error"), tr("Error during importing the archive"), QMessageBox::Ok);
    ui->buttonBox->setEnabled(true);
}

void ImportDialog::_sessionDisconnected()
{
    _catalogImporter = 0;
    reject();
}

void ImportDialog::on_tabWidget_currentChanged(int index)
{
    Q_UNUSED(index)
    if (ui->tabWidget->currentWidget() == ui->tabWidgetEntityPage) {
        ui->entityPathComboBox->setEnabled(true);
        ui->topologyNameLineEdit->setEnabled(false);
        ui->topologyPathComboBox->setEnabled(false);
        ui->importCountSpinBox->setEnabled(false);
        ui->customRightsButton->setEnabled(false);
        ui->topologiesTreeView->clearSelection();
    } else if (ui->tabWidget->currentWidget() == ui->tabWidgetTopologyPage) {
        ui->entityPathComboBox->setEnabled(true);
        if (ui->importCountSpinBox->value() > 1)
            ui->topologyNameLineEdit->setEnabled(false);
        else
            ui->topologyNameLineEdit->setEnabled(true);
        ui->topologyPathComboBox->setEnabled(true);
        ui->importCountSpinBox->setEnabled(true);
        if (ui->topologiesTreeView->selectionModel()->hasSelection())
            ui->customRightsButton->setEnabled(true);
        else
            ui->customRightsButton->setEnabled(false);
        ui->entitiesTableView->clearSelection();
    }
}
void ImportDialog::on_topologiesTreeView_expanded(const QModelIndex &index)
{
    if (!index.parent().isValid()) { //Case we are on root items
        TreeModelItem *item = _archiveTreeModel->itemFromIndex(index);
        for (int i = 0; i < item->childCount(); i++) {
            ui->topologiesTreeView->setExpanded(_archiveTreeModel->indexFromNode(item).child(i,0), true);
        }
    }
}

void ImportDialog::on_importCountSpinBox_valueChanged(int value)
{
    if (value != 1)
        ui->topologyNameLineEdit->setEnabled(false);
    else
        ui->topologyNameLineEdit->setEnabled(true);

    QModelIndex index = ui->topologiesTreeView->currentIndex();
    while (index.parent().isValid()) {
        /* Get the root item for index */
        index = index.parent();
    }

    int selectedRow = index.row();
    Hns::ImportOptionsList options = _optionsList.value(selectedRow);

    int optionSize = options.size();
    if (optionSize == value) {
        return;
    } else if (optionSize < value) { //add elements
        int numberElements = value - optionSize;
        for (int i = 0 ; i < numberElements ; ++i) {
            Hns::ImportOptions opt;
            opt.newName = (ui->topologyNameLineEdit->text() + QString("_%1").arg(optionSize +i)).toStdString();
            opt.flags = 0;
            options.push_back(opt);
        }
    } else { //remove elements
        int numberElements = optionSize - value;
        for (int i = 0 ; i < numberElements ; ++i)
            options.pop_back();
    }
    _optionsList.insert(selectedRow, options);
}

void ImportDialog::on_customRightsButton_clicked()
{
    QModelIndex index = ui->topologiesTreeView->currentIndex();
    while (index.parent().isValid()) {
        /* Get the root item for index */
        index = index.parent();
    }

    int selectedRow = index.row();

    Hns::ImportOptionsList options = _optionsList.value(selectedRow);
    Hns::ImportOptionsList newOptions;
    int counter = 0;
    for (Hns::ImportOptionsList::iterator it = options.begin() ; it != options.end() ; ++it) {
        Hns::ImportOptions opt = *it;
        opt.newName = (ui->topologyNameLineEdit->text() + QString("_%1").arg(counter++)).toStdString();
        newOptions.push_back(opt);
    }
    _optionsList.insert(selectedRow, newOptions);

    CustomRightDialog *dialog = new CustomRightDialog(_optionsList.value(selectedRow), this);
    if (dialog->exec() == QDialog::Accepted)
        _optionsList.insert(selectedRow, dialog->options());
    delete dialog;
}

void ImportDialog::_progressUpdatedSlot(int value)
{
    ui->progressBar->setValue(value + 1);
}

} // namespace Hnv
