/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "capturejobdialog.h"
#include "ui_capturejobdialog.h"
#include "capturejobmodel.h"

// Generated hynesim headers
#include "common/capturejoboptions.h"

// hynesim hnsclient headers
#include <libhnsclient/entity.h>
#include <libhnsclient/filetransferclient.h>

// Qt headers
#include <QMessageBox>
#include <QMenu>
#include <QSortFilterProxyModel>
#include <QProcess>
#include <QDebug>

namespace Hnv {

CaptureJobDialog::CaptureJobDialog(HnsClient::GenericEntity *entity,QWidget *parent)
    : QDialog(parent)
    , ui(new Ui::CaptureJobDialog)
    , _sourceValid(true)
    , _entity(entity)
    , _entityShortName(_entity->getShortName())
{
    ui->setupUi(this);
    ui->widget->setEntity(_entity);
    this->setWindowTitle(tr("%1: capture job management").arg(_entityShortName));
    connect(_entity, SIGNAL(destroyed()), this, SLOT(_entityDestroyedSlot()), Qt::QueuedConnection);
    connect(_entity, SIGNAL(statusChanged(HnsClient::GenericEntity *)), this, SLOT(_entityStatusChangedSlot()), Qt::QueuedConnection);
    connect(_entity, SIGNAL(availableInterfacesForCaptureChanged(QList<int>)), this, SLOT(_updateActionsSlot()));
    CaptureJobModel * model = new CaptureJobModel(_entity, this);
    _proxyModel = new QSortFilterProxyModel(this);
    _proxyModel->setSourceModel(model);
    _proxyModel->setDynamicSortFilter(true);
    ui->tableView->setModel(_proxyModel);
    ui->tableView->hideColumn(PcapInfoItem::COLUMN_TIMELIMIT);
    ui->tableView->hideColumn(PcapInfoItem::COLUMN_PACKETLIMIT);
    ui->tableView->hideColumn(PcapInfoItem::COLUMN_FILTERS);
    connect(ui->tableView->selectionModel()
        , SIGNAL(selectionChanged(const QItemSelection &, const QItemSelection &))
        , this
        , SLOT(tableViewModelSelectionChanged(const QItemSelection &, const QItemSelection &)));
    connect(model, SIGNAL(dataChanged(QModelIndex,QModelIndex)), this, SLOT(_updateActionsSlot()));
    ActionButtonLinker * linker = new ActionButtonLinker(ui->actionStop, ui->pbStopCapture, this);
    linker = new ActionButtonLinker(ui->actionDownload, ui->pbDownloadFile, this);
    linker = new ActionButtonLinker(ui->actionDelete, ui->pbDeleteFile, this);
    linker = new ActionButtonLinker(ui->actionReuse_settings, ui->pbReuse, this);
    Q_UNUSED(linker)
    _updateActionsSlot();
    addAction(ui->actionDelete);
}

CaptureJobDialog::~CaptureJobDialog()
{
    delete ui;
}

void CaptureJobDialog::_updateActionsSlot()
{
    bool isValid = false;
    bool isRunning = false;
    bool canDelete = false;
    bool canReuse = false;
    QModelIndexList indexList = ui->tableView->selectionModel()->selectedRows();
    if (!indexList.isEmpty()) {
        foreach(const QModelIndex &index, indexList) {
            if (isValid && isRunning)
                break;
            if (index.isValid()) {
                QVariant filePathData = index.data(PcapInfoItem::FilePathRole);
                if (filePathData.isValid())
                    isValid = true;
                if (index.data(PcapInfoItem::StatusRole).toBool()) {
                    isRunning = true;
                    canDelete = false;
                }
            }

        }
        if (indexList.count() == 1) {
            QList<int> list = _entity->getAvailableInterfacesForCapture();
            if (list.size() > 0)
                canReuse = true;
        }
        if (isValid && !isRunning)
            canDelete = true;
    }
    ui->actionStop->setEnabled(isRunning);
    ui->actionDownload->setEnabled(isValid);
    ui->actionDelete->setEnabled(canDelete);
    ui->actionReuse_settings->setEnabled(canReuse);
}

void CaptureJobDialog::_stopRunningCapture(QModelIndexList indexList)
{
    foreach(const QModelIndex &index, indexList) {
        if (index.isValid()) {
            if (index.data(PcapInfoItem::StatusRole).toBool()) {
                QVariant jobIDData = index.data(PcapInfoItem::JobIDRole);
                if (jobIDData.isValid()) {
                    int jobID = jobIDData.toInt();
                    if (jobID != -1) {
                        _entity->cancelCaptureJob(jobID);
                    }
                }
            }
        }
    }
}

void CaptureJobDialog::_downloadFiles(QModelIndexList indexList)
{
    QStringList files;
    foreach(const QModelIndex &index, indexList) {
        if (index.isValid()) {
            QVariant filePathData = index.data(PcapInfoItem::FilePathRole);
            if (filePathData.isValid()) {
                QString filePath = filePathData.toString().split("/").last();
                if (!filePath.isEmpty()) {
                    if (_fileTransfertClientHash.contains(filePath))
                        QMessageBox::information(this, tr("Please wait..."), tr("The file '%1' is already being downloaded.\nPlease wait...").arg(filePath), QMessageBox::Ok, QMessageBox::Ok);
                    else
                        files.append(filePath);
                }
            }
        }
    }
    if (!files.isEmpty())
        _entity->downloadCaptureFile(files, this, SLOT(_entityDownloadFileSlot(HnsClient::FileTransferClient*)));
}

void CaptureJobDialog::_deleteJobs(QModelIndexList indexList)
{
    bool isRunning = false;
    QStringList files;
    foreach(const QModelIndex &index, indexList) {
        if (index.isValid()) {
            QVariant filePathData = index.data(PcapInfoItem::FilePathRole);
            QVariant isRunningData = index.data(PcapInfoItem::StatusRole);
            if (filePathData.isValid()) {
                if (isRunningData.isValid() && isRunningData.toBool())
                    isRunning = true;
                QString filePath = filePathData.toString().split("/").last();
                if (!filePath.isEmpty())
                    files.append(filePath);
            }

        }
    }
    if (!files.isEmpty()) {
        QString message;
        if (files.count() > 1)
            message = tr("Do you really want to delete the selected capture jobs ?");
        else
            message = tr("Do you really want to delete the selected capture job ?");
        if (isRunning)
            message.append(tr("\nRunning capture jobs will not be deleted."));
        message.append(tr("\n\nWARNING:This operation can not be undone.\nWould you like to continue anyway ?"));

        QMessageBox msgBox;
        msgBox.setIcon(QMessageBox::Warning);
        msgBox.setWindowTitle(tr("Confirmation required"));
        msgBox.setText(message);
        msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
        msgBox.setDefaultButton(QMessageBox::No);
        HnvDialogHandler * popupHandler = new HnvDialogHandler(_entity, &msgBox, this);
        popupHandler->displayMessageOnDestroy(false);
        if (msgBox.exec() == QMessageBox::Yes)
            _entity->eraseFile(files);
    }
}

void CaptureJobDialog::_entityDestroyedSlot()
{
    if (_sourceValid) {
        _sourceValid = false;
        QMessageBox::warning(this, tr("Warning"), tr("\"%1\" was deleted. Operation aborted.").arg(_entityShortName));
        reject();
    }
}

void CaptureJobDialog::_entityStatusChangedSlot()
{
    if (_sourceValid) {
        Hns::Status status = _entity->getStatus();
        if (status == Hns::UNDEFINED) {
            _sourceValid = false;
            QMessageBox::warning(this, tr("Warning"), tr("\"%1\" was undefined. Operation aborted.").arg(_entityShortName));
            reject();
        }
    }
}

void CaptureJobDialog::tableViewModelSelectionChanged(const QItemSelection &currentIndex, const QItemSelection &previousIndex)
{
    Q_UNUSED(currentIndex);
    Q_UNUSED(previousIndex);
    _updateActionsSlot();
}

void CaptureJobDialog::on_tableView_doubleClicked(const QModelIndex &index)
{
    QModelIndexList indexList;
    indexList.append(index);
    _downloadFiles(indexList);
}

void CaptureJobDialog::on_tableView_customContextMenuRequested(const QPoint &pos)
{
    QMenu menu(this);
    menu.addAction(ui->actionStop);
    menu.addAction(ui->actionDownload);
    menu.addAction(ui->actionReuse_settings);
    menu.addSeparator();
    menu.addAction(ui->actionDelete);
    menu.exec(ui->tableView->mapToGlobal(pos));
}

void CaptureJobDialog::on_actionStop_triggered()
{
    _stopRunningCapture(ui->tableView->selectionModel()->selectedRows());
}

void CaptureJobDialog::on_actionDownload_triggered()
{
    _downloadFiles(ui->tableView->selectionModel()->selectedRows());
}

void CaptureJobDialog::on_actionDelete_triggered()
{
    _deleteJobs(ui->tableView->selectionModel()->selectedRows());
}

void CaptureJobDialog::on_actionReuse_settings_triggered()
{
    QModelIndex index = ui->tableView->currentIndex();
    PcapInfoItem * item = 0;
    if (index.isValid())
        item = static_cast<PcapInfoItem *>(_proxyModel->mapToSource(index).internalPointer());
    if (item)
        ui->widget->setCaptureSettings(item->getPcapInfo());
}

void CaptureJobDialog::_entityDownloadFileSlot(HnsClient::FileTransferClient *fileTransfertClient)
{
    _fileTransfertClientHash.insert(fileTransfertClient->remoteFileName(), fileTransfertClient);
    if (!fileTransfertClient->canDownload())
        return;
    connect(fileTransfertClient, SIGNAL(fileSize(QString,qlonglong)), this, SLOT(_fileSizeSlot(QString, qlonglong)));
    if (fileTransfertClient->localFileExists()) {
        QMessageBox msgBox;
        msgBox.setIcon(QMessageBox::Warning);
        msgBox.setWindowTitle(tr("File already exists"));
        msgBox.setText(tr("The file actually exists, do you really want to overwrite ?"));
        msgBox.setInformativeText(fileTransfertClient->localFileAbsolutePath());
        msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
        msgBox.setDefaultButton(QMessageBox::No);
        HnvDialogHandler * popupHandler = new HnvDialogHandler(_entity, &msgBox, NULL);
        popupHandler->displayMessageOnDestroy(false);
        if (msgBox.exec() == QMessageBox::Yes) {
            if (!fileTransfertClient->startDownload(true)) {
                _fileTransfertClientHash.remove(fileTransfertClient->remoteFileName());
                fileTransfertClient->deleteLater();
            }
            else {
                connect(fileTransfertClient, SIGNAL(transferFinished(QString, QString)), this, SLOT(_transferFinishedSlot(QString, QString)));
            }
        }
        else {
            _fileTransfertClientHash.remove(fileTransfertClient->remoteFileName());
            fileTransfertClient->deleteLater();
        }
    }
    else {
        if (!fileTransfertClient->startDownload()) {
            _fileTransfertClientHash.remove(fileTransfertClient->remoteFileName());
            fileTransfertClient->deleteLater();
        }
        else {
            connect(fileTransfertClient, SIGNAL(transferFinished(QString, QString)), this, SLOT(_transferFinishedSlot(QString, QString)));
        }
    }
}

void CaptureJobDialog::_fileSizeSlot(QString remoteFileName, qlonglong size)
{
    if (!_fileTransfertClientHash.contains(remoteFileName))
        return;
    if (size <= 0 ){
        QMessageBox msgBox;
        msgBox.setIcon(QMessageBox::Warning);
        msgBox.setWindowTitle(tr("File is empty"));
        msgBox.setText(tr("The file is actually empty"));
        msgBox.setStandardButtons(QMessageBox::Ok);
        HnvDialogHandler * popupHandler = new HnvDialogHandler(_entity, &msgBox, NULL);
        popupHandler->displayMessageOnDestroy(false);
        msgBox.exec();
        HnsClient::FileTransferClient* fileTransferClient = _fileTransfertClientHash.take(remoteFileName);
        fileTransferClient->deleteLater();
    }
}

void CaptureJobDialog::_transferFinishedSlot(QString remoteFileName, QString path)
{
    if (_fileTransfertClientHash.contains(remoteFileName)) {
        HnsClient::FileTransferClient *fileTransfertClient = _fileTransfertClientHash.take(remoteFileName);
        fileTransfertClient->deleteLater();
    }
    QProcess processWireShark;
    QStringList args;
    args.append("-r");
    args.append(path);
    if (!processWireShark.startDetached("wireshark",args)){
        qCritical()<<(tr("An error occurs while executing wireshark"));
    }
}

} // namespace Hnv
