/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "listnodedelegate.h"
#include "hyneviewsession.h"
#include "loadbalancerdialog.h"

// hynesim hnsclient headers
#include <libhnsclient/platformmonitor.h>

// Qt headers
#include <QComboBox>
#include <QCheckBox>

namespace Hnv {

ListNodeDelegate::ListNodeDelegate(int comboColumn, int checkColumn , QObject *parent)
    : QStyledItemDelegate(parent)
    , _comboColumn(comboColumn)
    , _checkColumn(checkColumn)
{
}

ListNodeDelegate::~ListNodeDelegate()
{
    clear();
}

void ListNodeDelegate::clear()
{
    qDeleteAll(_hashSignalDelegate);
    _hashSignalDelegate.clear();
}

QWidget *ListNodeDelegate::createEditor(QWidget *parent,
        const QStyleOptionViewItem &option,
        const QModelIndex &index) const
{
    if (index.column() == _comboColumn) {
        QComboBox *comboBox = new QComboBox(parent);
        bool defined = index.data(Qt::UserRole+3).toBool();

        QString nodeName = index.data(Qt::UserRole+1).toString();
        if (defined) {
            comboBox->addItem(nodeName);
            comboBox->setCurrentIndex(0);
            comboBox->setEnabled(false);
        } else {
            comboBox->addItem(tr("Not defined"), NOT_DEFINED);
            comboBox->addItem(tr("Auto"), AUTO);
            QStringList list = index.data(Qt::UserRole+2).toStringList();
            if (list.isEmpty()) {
                comboBox->setCurrentIndex(0);
                comboBox->setEnabled(false);
            } else {
                foreach(QString node, list) {
                    QString comboText = node;
                    HnsClient::NodeInformation * nodeInfo = HyneviewSession::instance()->platformMonitor()->getNodeInformation(node);
                    if (nodeInfo)
                        comboText = nodeInfo->getShortName();
                    comboBox->addItem(comboText, node);
                }
                comboBox->setCurrentIndex(1);
            }
        }
        qlonglong id = index.data(Qt::UserRole+4).toLongLong();
        if (_hashSignalDelegate.find(id) == _hashSignalDelegate.end())
            _hashSignalDelegate[id] = new ComboboxToNode(id, qobject_cast<LoadBalancerDialog *>(this->parent()), comboBox);
        connect(comboBox, SIGNAL(currentIndexChanged(int)), _hashSignalDelegate[id], SLOT(comboboxChangeSlot(int)));
        connect(comboBox, SIGNAL(destroyed(QObject*)), this, SLOT(_comboboxDestroyedSlot(QObject*)));
        _comboBoxHash.insert(id, comboBox);
        return comboBox;
    } else if(index.column() == _checkColumn) {
        QCheckBox *checkbox = new QCheckBox(parent);
        if(index.data(Qt::UserRole+5).toInt() == NOTHING)
            checkbox->setEnabled(false);
        if (index.data(Qt::UserRole+5).toInt() == CHECK)
            checkbox->setCheckState(Qt::Checked);
        else
            checkbox->setCheckState(Qt::Unchecked);

        qlonglong id = index.data(Qt::UserRole+4).toLongLong();
        if (_hashSignalDelegate.find(id) == _hashSignalDelegate.end())
            _hashSignalDelegate[id] = new ComboboxToNode(id, qobject_cast<LoadBalancerDialog *>(this->parent()), NULL);

        connect(checkbox, SIGNAL(toggled(bool)),_hashSignalDelegate[id] , SLOT(checkboxChangeSlot(bool)));
        connect(checkbox, SIGNAL(destroyed(QObject*)), this, SLOT(_checkBoxDestroyedSlot(QObject*)));
        _checkBoxHash.insert(id, checkbox);
        return checkbox;
    } else {
        return QStyledItemDelegate::createEditor(parent, option, index);
    }
}

void ListNodeDelegate::setEditorData(QWidget *editor,
                                  const QModelIndex &index) const
{
    if (index.column() == _comboColumn) {
        QString node = index.model()->data(index, Qt::UserRole + 6).toString();
        QComboBox *comboBox = qobject_cast<QComboBox *>(editor);
        int idxText = comboBox->findData(node);
        comboBox->setCurrentIndex(idxText);
    } else if(index.column() == _checkColumn) {
        QCheckBox *checkbox = qobject_cast<QCheckBox *>(editor);
        if(checkbox == NULL) return QStyledItemDelegate::setEditorData(editor, index);
        if(index.model()->data(index , Qt::UserRole+5).toInt() == CHECK)
            checkbox->setCheckState(Qt::Checked);
        else
            checkbox->setCheckState(Qt::Unchecked);

    } else {
        QStyledItemDelegate::setEditorData(editor, index);
    }
}

void ListNodeDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    QStyledItemDelegate::paint(painter, option, index);
}

void ListNodeDelegate::setModelData(QWidget *editor,
                                 QAbstractItemModel *model,
                                 const QModelIndex &index) const
{
    if (index.column() == _comboColumn) {
        QComboBox *comboBox = qobject_cast<QComboBox *>(editor);
        QString node = comboBox->itemData(comboBox->currentIndex()).toString();
        model->setData(index, node);
    } else if(index.column() == _checkColumn) {
        QCheckBox *checkbox = qobject_cast<QCheckBox *>(editor);
        if(checkbox->checkState() == Qt::Checked)
            model->setData(index, CHECK);
        else
            model->setData(index, UNCHECK);
    }
    else {
        QStyledItemDelegate::setModelData(editor, model, index);
    }
}

void ListNodeDelegate::updateEditorGeometry(QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    QStyledItemDelegate::updateEditorGeometry(editor, option, index);
    if (qobject_cast<QComboBox *>(editor)) {
        QRect editorRect = option.rect.adjusted(1, 2, -1, -2);
        editor->setGeometry(editorRect);
    } else if (qobject_cast<QCheckBox *>(editor)) {
        QRect cbRect(QPoint(0, 0), editor->minimumSizeHint());
        cbRect.moveCenter(option.rect.center());
        editor->setGeometry(cbRect);
    }
}

void ListNodeDelegate::_comboboxDestroyedSlot(QObject *comboBox)
{
    QComboBox *cb = qobject_cast<QComboBox *>(comboBox);
    if (!cb)
        return;
    qlonglong id = _comboBoxHash.key(cb);
    _comboBoxHash.remove(id);
    _hashSignalDelegate.remove(id);
}

void ListNodeDelegate::_checkBoxDestroyedSlot(QObject *checkBox)
{
    QCheckBox *cb = qobject_cast<QCheckBox *>(checkBox);
    if (!cb)
        return;
    qlonglong id = _checkBoxHash.key(cb);
    _checkBoxHash.remove(id);
}

void ListNodeDelegate::changeEntity(qlonglong entityId, QModelIndex index)
{
    QComboBox *comboBox = _comboBoxHash.value(entityId);
    QCheckBox *checkBox = _checkBoxHash.value(entityId);
    if (!comboBox || !checkBox)
        return;
    QStringList nodesAvailable = index.data(Qt::UserRole + 2).toStringList();
    bool isDefined = index.data(Qt::UserRole + 3).toBool();
    if (isDefined) {
        comboBox->setEnabled(false);
        checkBox->setEnabled(false);
        checkBox->setCheckState(Qt::Unchecked);
        comboBox->setCurrentIndex(-1);
        return;
    }
    foreach (const QString &node, nodesAvailable) { //add node
        if (comboBox->findData(node) == -1) {
            HnsClient::NodeInformation *nodeInfo = HyneviewSession::instance()->platformMonitor()->getNodeInformation(node);
            if (nodeInfo) {
                comboBox->addItem(nodeInfo->getShortName(), node);
                if (!checkBox->isEnabled()) {
                    checkBox->setEnabled(true);
                }
            }
        }
    }
    int comboBoxCount = comboBox->count(); //remove node
    for (int i = 0 ; i < comboBoxCount ; i++) {
        QString key = comboBox->itemData(i).toString();
        if (key != AUTO && key != NOT_DEFINED && !nodesAvailable.contains(key)) {
            comboBox->removeItem(i);
            if (nodesAvailable.isEmpty()) {
                comboBox->setEnabled(false);
                checkBox->setEnabled(false);
                checkBox->setCheckState(Qt::Unchecked);
            }
            else {
                comboBox->setCurrentIndex(comboBox->findData(AUTO));
            }
        }
    }
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ComboboxToNode~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
ComboboxToNode::ComboboxToNode(qlonglong idEntity , LoadBalancerDialog *loadBalancerDialog, QComboBox * comboBox)
    : QObject()
    , _idEntity(idEntity)
    , _comboBox(comboBox)
{
    connect(this, SIGNAL(comboboxChange(qlonglong,const QString &)),loadBalancerDialog , SLOT(nodeChangeToComboboxSlot(qlonglong,const QString &)));
    connect(this, SIGNAL(checkboxChange(qlonglong,int)),loadBalancerDialog , SLOT(nodeChangeToCheckSlot(qlonglong,int)));
}

ComboboxToNode::~ComboboxToNode()
{
}


void ComboboxToNode::comboboxChangeSlot(int index)
{
    QVariant itemData = _comboBox->itemData(index);
    if (!itemData.isValid()) {
        return;
    }
    QString newNode = itemData.toString();
    if (newNode.isEmpty())
        return;

    if (newNode == NOT_DEFINED)
        _comboBox->setEnabled(false);
    else if (!_comboBox->isEnabled())
        _comboBox->setEnabled(true);
    emit comboboxChange(_idEntity, newNode);
}

void ComboboxToNode::checkboxChangeSlot(bool value)
{
    emit checkboxChange(_idEntity, (value)?CHECK:UNCHECK);
}

} // namespace Hnv
