/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "loadbalancerdialog.h"
#include "ui_loadbalancerdialog.h"
#include "hyneviewsession.h"
#include "noderesourcewidget.h"
#include "loadbalancermodel.h"
#include "listnodedelegate.h"
#include "ressources.h"

// hynesim common headers
#include "common/resourceutils.h"

// hynesim hnsclient headers
#include <libhnsclient/entity.h>
#include <libhnsclient/topology.h>
#include <libhnsclient/platformmonitor.h>
#include <libhnsclient/entry.h>

// Qt headers
#include <QMessageBox>
#include <QSortFilterProxyModel>

#define WARNING_FULL_CHARGE tr("Cpu and Memory overflow")
#define WARNING_CPU_CHARGE tr("Cpu overflow")
#define WARNING_MEMORY_CHARGE tr("Memory overflow")

namespace Hnv {

//==========================LoadBalancerDialog========================================

LoadBalancerDialog::LoadBalancerDialog(const QList<HnsClient::GenericEntity *> & listEntitiesSelected  , HnsClient::Topology *topology ,QWidget *parent)
    : QDialog(parent, Qt::WindowCloseButtonHint | Qt::WindowMaximizeButtonHint)
    , _ui(new Ui::LoadBalancerDialog)
    , _topology(topology)
    , _loadBalancerModel(new LoadBalancerModel(this))
    , _needToResizeLastColumn(true)
    , _platformMonitor(HyneviewSession::instance()->platformMonitor())
    , _delegate(new ListNodeDelegate(COLUMN_COMBOBOX, COLUMN_CHECK,this))
    , _canCompute(false)
    , _noNodeLabel(0)
{
    _ui->setupUi(this);
    _ui->buttonReset->setIcon(QIcon(Ressources::themedPath(Constants::ICON_GENERATE_MAC_ADRESS)));
    _ui->buttonCompute->setIcon(QIcon(Ressources::themedPath(Constants::ICON_PERM_EXEC)));

    QSortFilterProxyModel * filterProxyModel = new QSortFilterProxyModel(this);
    filterProxyModel->setSourceModel(_loadBalancerModel);
    filterProxyModel->setSortRole(Qt::UserRole+6);
    filterProxyModel->setFilterKeyColumn(COLUMN_CHECK);

    _ui->tableView->setModel(filterProxyModel);
    _ui->tableView->hideColumn(COLUMN_CPU);
    _ui->tableView->hideColumn(COLUMN_MEMORY);
    _ui->tableView->sortByColumn(COLUMN_CHECK, Qt::AscendingOrder);
    if (!_needToResizeLastColumn) {
        _ui->tableView->resizeColumnsToContents();
    } else {
        _ui->tableView->resizeColumnToContents(COLUMN_CHECK);
        _ui->tableView->resizeColumnToContents(COLUMN_SHORT_NAME);
        _ui->tableView->resizeColumnToContents(COLUMN_COMBOBOX);
        _ui->tableView->resizeColumnToContents(COLUMN_CPU);
        _ui->tableView->resizeColumnToContents(COLUMN_MEMORY);
    }

    _ui->tableView->disabledDoubleClickHeader();
    _ui->tableView->horizontalHeader()->setContextMenuPolicy(Qt::NoContextMenu);
    _ui->tableView->setItemDelegate(_delegate);
    connect(_ui->tableView->model(), SIGNAL(modelReset()), this, SLOT(_modelResetSlot()));
    connect(_ui->tableView->model(), SIGNAL(rowsInserted(QModelIndex,int,int)), this, SLOT(_rowsInsertedSlot(QModelIndex,int,int)));

    _ui->tableView->addAction(_ui->actionSelectAll);
    _ui->tableView->addAction(_ui->actionUnselect_All);
    _ui->tableView->addAction(_ui->actionInverse_Selection);
    _ui->tableView->setContextMenuPolicy(Qt::ActionsContextMenu);

    _nodeVerticalSpacer = new QSpacerItem(2, 4, QSizePolicy::Minimum, QSizePolicy::Expanding);

    _noNodeLabel = new QLabel(tr("No node is registered with the master"),this);
    _noNodeLabel->setAlignment(Qt::AlignHCenter);
    _noNodeLabel->setStyleSheet("QLabel{color:red;}");
    _noNodeLabel->hide();

    _fillCombobox();
    _initData(listEntitiesSelected);

    connect(_platformMonitor, SIGNAL(nodeAdded(QString)), this, SLOT(_nodeAddedSlot(QString)));
    connect(_platformMonitor, SIGNAL(nodeRemoved(QString)), this, SLOT(_nodeRemovedSlot(QString)));
}

LoadBalancerDialog::~LoadBalancerDialog()
{
    _cleanAll();
    delete _loadBalancerModel;
    delete _ui;
}

void LoadBalancerDialog::nodeChangeToComboboxSlot(qlonglong id, const QString & newNodeName)
{
    QString oldNodeName = _entityInformationHash[id].nodeName;
    _entityInformationHash[id].nodeName = newNodeName;
    if (oldNodeName == newNodeName)
        return;

    if (newNodeName == AUTO) {
        _canCompute = true;
    } else {
        _canCompute = false;
        foreach(const EntityLocalInformation &entity, _entityInformationHash) {
            if (entity.nodeName == AUTO) {
                _canCompute = true;
                break;
            }
        }
    }

    int row = _loadBalancerModel->getRowFromID(id);
    if (row == -1)
        return;
    QModelIndex warningIndex = _loadBalancerModel->index(row, COLUMN_WARNING);
    QModelIndex checkboxIndex = _loadBalancerModel->index(row , COLUMN_CHECK);

    if (oldNodeName != AUTO && oldNodeName != NOT_DEFINED && !oldNodeName.isEmpty()) {
        _loadBalancerModel->setData(warningIndex, QVariant(QString("")), Qt::EditRole);
        _nodeWidgetHash[oldNodeName]->removePrevisionResource(_entityInformationHash[id].cpu, _entityInformationHash[id].memory);
    }

    if (newNodeName != AUTO && newNodeName != NOT_DEFINED) {
        if (_nodeWarningHash.contains(newNodeName)) {
            _loadBalancerModel->setData(warningIndex ,QVariant(_nodeWarningHash.value(newNodeName)), Qt::EditRole);
        }
        _nodeWidgetHash[_entityInformationHash[id].nodeName]->addPrevisionResource(_entityInformationHash[id].cpu, _entityInformationHash[id].memory);
     }

    if (_entityInformationHash[id].isSelected == CHECK && newNodeName == NOT_DEFINED) {
        _entityInformationHash[id].isSelected = UNCHECK;
        _loadBalancerModel->setData(checkboxIndex, QVariant(UNCHECK), Qt::EditRole);
    } else if (_entityInformationHash[id].isSelected == UNCHECK) {
        _entityInformationHash[id].isSelected = CHECK;
        _loadBalancerModel->setData(checkboxIndex, QVariant(CHECK), Qt::EditRole);
    }
    _ui->buttonCompute->setEnabled(_canCompute);
}

void LoadBalancerDialog::nodeChangeToCheckSlot(qlonglong id, int selected)
{
    if(_entityInformationHash[id].isDefined)
        return;
    if(_entityInformationHash[id].isSelected == selected)
        return;
    _entityInformationHash[id].isSelected = selected;
    if (selected == CHECK) {
        _canCompute = true;
        _entityInformationHash[id].nodeName = AUTO;
    } else if (selected == UNCHECK) {
        _canCompute = false;
        if (_entityInformationHash[id].nodeName != AUTO) {
            if (_nodeWidgetHash.contains(_entityInformationHash[id].nodeName))
                _nodeWidgetHash[_entityInformationHash[id].nodeName]->removePrevisionResource(_entityInformationHash[id].cpu, _entityInformationHash[id].memory);
        }
        _entityInformationHash[id].nodeName = NOT_DEFINED;
        foreach(const EntityLocalInformation &entity, _entityInformationHash) {
            if (entity.isSelected == CHECK && entity.nodeName == AUTO) {
                _canCompute = true;
                break;
            }
        }
    }

    int row = _loadBalancerModel->getRowFromID(id);
    if(row == -1)
        return;
    QModelIndex myIndex = _loadBalancerModel->index(row , COLUMN_COMBOBOX);
    _loadBalancerModel->setData(myIndex ,QVariant(_entityInformationHash[id].nodeName), Qt::EditRole);

    if (_entityInformationHash[id].nodeName == NOT_DEFINED && !_entityInformationHash[id].listAvailableNodeName.isEmpty()) {
        myIndex = _loadBalancerModel->index(row , COLUMN_CHECK);
        _loadBalancerModel->setData(myIndex, QVariant(UNCHECK), Qt::EditRole);
    } else {
        myIndex = _loadBalancerModel->index(row, COLUMN_CHECK);
        _loadBalancerModel->setData(myIndex, QVariant(CHECK), Qt::EditRole);
    }
    _ui->buttonCompute->setEnabled(_canCompute);
}

void LoadBalancerDialog::_modelResetSlot()
{
    for(int i = 0; i < _ui->tableView->model()->rowCount(); i++) {
        if (_ui->tableView->model()->hasIndex(i, COLUMN_COMBOBOX))
            _ui->tableView->openPersistentEditor(_ui->tableView->model()->index(i, COLUMN_COMBOBOX));
        if (_ui->tableView->model()->hasIndex(i , COLUMN_CHECK) )
            _ui->tableView->openPersistentEditor(_ui->tableView->model()->index(i,COLUMN_CHECK));
    }
}

void LoadBalancerDialog::_rowsInsertedSlot(QModelIndex parent, int start, int end)
{
    for(int i = start; i < end; i++) {
        if (_ui->tableView->model()->hasIndex(i, COLUMN_COMBOBOX, parent))
            _ui->tableView->openPersistentEditor(_ui->tableView->model()->index(i, COLUMN_COMBOBOX, parent));
        if (_ui->tableView->model()->hasIndex(i , COLUMN_CHECK, parent))
            _ui->tableView->openPersistentEditor(_ui->tableView->model()->index(i,COLUMN_CHECK, parent));
    }
}

void LoadBalancerDialog::_overloadedSlot(const QString & nodeName ,int value)
{
    if ((value & FULL_CHARGE) == FULL_CHARGE) {
        if (_nodeWarningHash.value(nodeName) == WARNING_FULL_CHARGE)
            return;
        _nodeWarningHash[nodeName] = WARNING_FULL_CHARGE;
    } else if (value & CPU_CHARGE) {
        if (_nodeWarningHash.value(nodeName) == WARNING_CPU_CHARGE)
            return;
        _nodeWarningHash[nodeName] = WARNING_CPU_CHARGE;
    } else if (value & MEMORY_CHARGE) {
        if (_nodeWarningHash.value(nodeName) == WARNING_MEMORY_CHARGE)
            return;
        _nodeWarningHash[nodeName] = WARNING_MEMORY_CHARGE;
    } else {
        if (_nodeWarningHash.value(nodeName).isEmpty())
            return;
        _nodeWarningHash[nodeName].clear();
    }

    foreach(EntityLocalInformation entity, _entityInformationHash) {
        if (entity.nodeName == nodeName && entity.isSelected == CHECK) {
            int row = _loadBalancerModel->getRowFromID(entity.id);
            if (row == -1)
                continue;
            QModelIndex index = _loadBalancerModel->index(row, COLUMN_WARNING);
            _loadBalancerModel->setData(index, _nodeWarningHash[nodeName]);
        }
    }
}

void LoadBalancerDialog::_computeDefineInfosSlot(::Hns::LongDefineInfoMap longDefineInfoMap)
{
    QMap< ::Ice::Long, Hns::DefineInfo> defineMap = QMap< ::Ice::Long, Hns::DefineInfo>(longDefineInfoMap);
    foreach(const Hns::DefineInfo defineInformation, defineMap) {
        int row = _loadBalancerModel->getRowFromID(defineInformation.entityId);
        if (row == -1)
            continue;
        QModelIndex index = _loadBalancerModel->index(row, COLUMN_COMBOBOX);
        _loadBalancerModel->setData(index, QString::fromStdString(defineInformation.nodeId));
    }
    _enableInterface(true, false);
}

void LoadBalancerDialog::_nodeAddedSlot(QString nodeKey)
{
    HnsClient::NodeInformation *nodeInformation = _platformMonitor->getNodeInformation(nodeKey);
    _addNodeWidget(nodeKey, nodeInformation);
    _enableInterface();
}

void LoadBalancerDialog::_nodeRemovedSlot(QString nodeKey)
{
    _removeNodeWidget(nodeKey);
}

void LoadBalancerDialog::_entityRefreshedSlot(HnsClient::GenericEntity *genericEntity)
{
    qlonglong entityId = genericEntity->getId();
    if (_entityInformationHash.contains(entityId)) {
        QString nodeName = _entityInformationHash[entityId].nodeName;
        EntityLocalInformation localInfo = _generateEntityLocalInformation(genericEntity);
        if (!localInfo.isDefined)
            localInfo.nodeName = nodeName;
        localInfo.isSelected = _entityInformationHash[entityId].isSelected;
        _entityInformationHash.insert(entityId, localInfo);
        _loadBalancerModel->changeEntity(entityId, localInfo);
        int row = _loadBalancerModel->getRowFromID(entityId);
        QModelIndex idx = _loadBalancerModel->index(row, COLUMN_COMBOBOX);
        _delegate->changeEntity(entityId, idx);
    }
}

void LoadBalancerDialog::on_buttonBox_accepted()
{
    if (_isDataValid())
        _defineEntity();
}

void LoadBalancerDialog::on_buttonCompute_clicked()
{
    if (_isDataValid())
        _compute();
}

void LoadBalancerDialog::on_buttonReset_clicked()
{
    foreach(EntityLocalInformation entity, _entityInformationHash) {
        if (!entity.isDefined
                && entity.nodeName != AUTO
                && entity.nodeName != NOT_DEFINED
                && entity.isSelected == CHECK) {
            int row = _loadBalancerModel->getRowFromID(entity.id);
            if(row == -1)
                continue;
            QModelIndex index = _loadBalancerModel->index(row , COLUMN_COMBOBOX);
            _loadBalancerModel->setData(index ,QVariant(AUTO), Qt::EditRole);
        }
    }
    _isDataValid();
}

void LoadBalancerDialog::on_actionSelectAll_triggered()
{
    foreach (EntityLocalInformation entity, _entityInformationHash) {
        if (entity.isSelected == UNCHECK) {
            int row = _loadBalancerModel->getRowFromID(entity.id);
            if (row == -1)
                continue;
            QModelIndex index = _loadBalancerModel->index(row , COLUMN_CHECK);
            _loadBalancerModel->setData(index, QVariant(CHECK), Qt::EditRole);
        }
    }
}

void LoadBalancerDialog::on_actionUnselect_All_triggered()
{
    foreach (EntityLocalInformation entity, _entityInformationHash) {
        if (entity.isSelected == CHECK) {
            int row = _loadBalancerModel->getRowFromID(entity.id);
            if (row == -1)
                continue;
            QModelIndex index = _loadBalancerModel->index(row , COLUMN_CHECK);
            _loadBalancerModel->setData(index, QVariant(UNCHECK), Qt::EditRole);
        }
    }
}

void LoadBalancerDialog::on_actionInverse_Selection_triggered()
{
    foreach (EntityLocalInformation entity, _entityInformationHash) {
        int row = _loadBalancerModel->getRowFromID(entity.id);
        if (row == -1)
            continue;
        QModelIndex index = _loadBalancerModel->index(row , COLUMN_CHECK);
        if(entity.isSelected == CHECK)
            _loadBalancerModel->setData(index ,QVariant(UNCHECK), Qt::EditRole);
        else if(entity.isSelected == UNCHECK)
            _loadBalancerModel->setData(index ,QVariant(CHECK), Qt::EditRole);
    }
}

void LoadBalancerDialog::_resetData()
{
    _cleanAll();
    _initData();
}

void LoadBalancerDialog::_initData(const QList<HnsClient::GenericEntity *> & listEntitiesSelected)
{
    _initEntityList(listEntitiesSelected);
    _loadBalancerModel->setEntityList(_entityInformationHash.values());
    if (_entityInformationHash.empty()) {
        _ui->actionSelectAll->setEnabled(false);
        _ui->actionUnselect_All->setEnabled(false);
        _ui->actionInverse_Selection->setEnabled(false);
    }
    _initNodeWidgets();
}

void LoadBalancerDialog::_cleanAll()
{
    if (_delegate)
        _delegate->clear();
    _loadBalancerModel->clear();
    _entityInformationHash.clear();
    foreach(NodeResourceWidget * myNodeResourWidget, _nodeWidgetHash) {
        _ui->scrollAreaWidgetContents->layout()->removeWidget(myNodeResourWidget);
        myNodeResourWidget->deleteLater();
    }
    qDeleteAll(_nodeWidgetHash);
    _nodeWidgetHash.clear();
    _ui->scrollAreaWidgetContents->layout()->removeItem(_ui->scrollAreaWidgetContents->layout()->itemAt(0)); // Vertical spacer
    if (_noNodeLabel) {
        _noNodeLabel->deleteLater();
        _noNodeLabel = 0;
    }
    _nodeWarningHash.clear();
}

QHash<qlonglong, EntityLocalInformation> LoadBalancerDialog::_getEntitiesFromTopology(bool completeInformation)
{
    QHash<qlonglong, EntityLocalInformation> result;
    QHash<qlonglong, HnsClient::GenericEntity*> listEntities = _topology->getEntities(false);
    foreach(HnsClient::GenericEntity * entity, listEntities) {
        if(entity->isModel())
            continue;
        connect(entity, SIGNAL(refreshed(HnsClient::GenericEntity*)), this, SLOT(_entityRefreshedSlot(HnsClient::GenericEntity*)));
        EntityLocalInformation entityInformation = _generateEntityLocalInformation(entity, completeInformation);
        if (entityInformation.isSelected == CHECK)
            _canCompute = true;
        result.insert(entity->getId(), entityInformation);
    }
    return result;
}

void LoadBalancerDialog::_initEntityList(const QList<HnsClient::GenericEntity *> & listEntitiesSelected)
{
    QList<qlonglong> selectedEntitiesID;
    foreach(HnsClient::GenericEntity * entity, listEntitiesSelected)
        selectedEntitiesID.append(entity->getId());

    _entityInformationHash = _getEntitiesFromTopology();

    if (!selectedEntitiesID.isEmpty()) {
        foreach(EntityLocalInformation entity, _entityInformationHash) {
            if (!selectedEntitiesID.contains(entity.id) && _entityInformationHash[entity.id].nodeName == AUTO) {
                _entityInformationHash[entity.id].isSelected = UNCHECK;
                _entityInformationHash[entity.id].nodeName = NOT_DEFINED;
            }
        }
    }
}

void LoadBalancerDialog::_initNodeWidgets()
{
    QHash<QString, HnsClient::NodeInformation*> nodeInfoHash = _platformMonitor->getNodeInformationHash();

    _ui->scrollAreaWidgetContents->layout()->addItem(_nodeVerticalSpacer);
    if (!_noNodeLabel) {
        _noNodeLabel = new QLabel(tr("No node is registered with the master"),this);
        _noNodeLabel->setAlignment(Qt::AlignHCenter);
        _noNodeLabel->setStyleSheet("QLabel{color:red;}");
        _noNodeLabel->hide();
    }
    _ui->scrollAreaWidgetContents->layout()->addWidget(_noNodeLabel);
    if (nodeInfoHash.isEmpty())
        _noRessourceWidget();
    else {
        QList<QPair<QString, int> > listNodeOverloaded;
        QHashIterator<QString, HnsClient::NodeInformation*> iterator(nodeInfoHash);
        while (iterator.hasNext()) {
            iterator.next();
            QString nodeKey = iterator.key();
            HnsClient::NodeInformation * nodeInfo = iterator.value();
            _addNodeWidget(nodeKey, nodeInfo);

            int startOverload = NO_CHARGE;
            if(nodeInfo->getHnsCurrentCpu() > nodeInfo->getHnsMaxCpu())
                startOverload |= CPU_CHARGE;
            if (nodeInfo->getHnsCurrentMemory() > nodeInfo->getHnsMaxMemory())
                startOverload |= MEMORY_CHARGE;
            listNodeOverloaded.append( QPair<QString, int>(nodeKey, startOverload));
        }
         for(int i = 0 ; i < listNodeOverloaded.size() ; i++){
             _overloadedSlot(listNodeOverloaded[i].first, listNodeOverloaded[i].second);
         }
         _enableInterface();
    }
}

void LoadBalancerDialog::_noRessourceWidget()
{
    _noNodeLabel->setVisible(true);
    _enableInterface(false, false);
}

void LoadBalancerDialog::_fillCombobox()
{
    QMap<int , Hns::BalancingBehaviorInfo>::const_iterator itor = _topology->getMapBehavior().begin();
    while(itor != _topology->getMapBehavior().end()) {
        _ui->comboBox->addItem(QString::fromStdString(itor.value().name));
        itor++;
    }
}

QString LoadBalancerDialog::_validateNodeData()
{
    QHash<QString, HnsClient::NodeInformation*> nodeInfoHash = _platformMonitor->getNodeInformationHash();
    QString error = "";
    QHashIterator<QString, NodeResourceWidget *> iterator(_nodeWidgetHash);
    while(iterator.hasNext()) {
        iterator.next();
        QString nodeKey = iterator.key();
        NodeResourceWidget * widget = iterator.value();
        if (!nodeInfoHash.contains(nodeKey)) {
            error = tr("Node \"%1\" does not exist").arg(widget->nodeName());
            break;
        } else {
            HnsClient::NodeInformation * nodeInfo = nodeInfoHash.value(nodeKey);
            if (nodeInfo->getHnsCurrentCpu() != widget->initialCpu()
                    || nodeInfo->getHnsCurrentResources().memory != widget->inititalMemory()) {
                error = tr("Node \"%1\" state changed.").arg(widget->nodeName());
                break;
            }
        }
    }
    return error;
}

QString LoadBalancerDialog::_validateEntityData()
{
    QString error = "";
    //test if entity always ok and register node to connect
    QHash<qlonglong, EntityLocalInformation> hashVerificationEntity = _getEntitiesFromTopology(false);

    foreach(const EntityLocalInformation &entity, _entityInformationHash) {
        if ((entity.isSelected == CHECK) && entity.nodeName != NOT_DEFINED) {
            if (!hashVerificationEntity.contains(entity.id)) {
                error = tr("Entity \"%1\" does not exist").arg(entity.shortName);
                break;
            }
            if (hashVerificationEntity[entity.id].cpu != entity.cpu
                    || hashVerificationEntity[entity.id].memory != entity.memory) {
                error = tr("Properties of \"%1\" have changed").arg(entity.shortName);
                break;
            }
            if (hashVerificationEntity[entity.id].isDefined != entity.isDefined) {
                if(hashVerificationEntity[entity.id].isDefined)
                    error = tr("\"%1\" is already defined").arg(entity.shortName);
                else
                    error = tr("\"%1\" is not defined").arg(entity.shortName);
                break;
            }
        }
        if (!hashVerificationEntity.contains(entity.id)) {
            error = tr("Entity \"%1\" does not exist").arg(entity.shortName);
            break;
        }
        if (entity.isDefined != _topology->getEntityById(entity.id)->isDefined()) {
            error = tr("Status of \"%1\" have changed").arg(entity.shortName);
            break;
        }
    }
    return error;
}

void LoadBalancerDialog::_displayError(QString error)
{
    error.append(tr("<br/>The load balancer will reset."));
    QMessageBox(QMessageBox::Warning, tr("Warning"), error, QMessageBox::Ok, this).exec();
    _resetData();
}

bool LoadBalancerDialog::_isDataValid()
{
    QString error = _validateEntityData();
    if (!error.isEmpty()) {
        _displayError(error);
        return false;
    }

    error = _validateNodeData();
    if (!error.isEmpty()) {
        _displayError(error);
        return false;
    }
    _enableInterface();
    return true;
}

void LoadBalancerDialog::_compute()
{
    QMap<Ice::Long, Hns::DefineInfo> defineInfos;
    foreach(const EntityLocalInformation &entity, _entityInformationHash) {
        if (!entity.isDefined
                && !entity.listAvailableNodeName.empty()
                && (entity.isSelected == CHECK)) {
            Hns::DefineInfo defineInfo;
            defineInfo.isDefined = true;
            defineInfo.entityId = entity.id;
            if (entity.nodeName != AUTO)
                defineInfo.nodeId = entity.nodeName.toStdString();
            defineInfos.insert(entity.id, defineInfo);
        }
    }
    _topology->computeDefineInfos(defineInfos, _topology->getMapBehavior()[_ui->comboBox->currentIndex()].behavior, this, SLOT(_computeDefineInfosSlot(::Hns::LongDefineInfoMap)));
    _enableInterface(false, true);
}

void LoadBalancerDialog::_enableInterface(bool enable, bool runningTransaction)
{
    _ui->progressBar->setVisible(runningTransaction);
    _ui->tableView->setEnabled(enable);
    _ui->comboBox->setEnabled(enable);
    _ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(enable);
    _ui->buttonReset->setEnabled(enable);
    _ui->buttonCompute->setEnabled(enable && _canCompute);
}

void LoadBalancerDialog::_defineEntity()
{
    QHash<qlonglong, HnsClient::GenericEntity*> hashEntities = _topology->getEntities(false);
    foreach(EntityLocalInformation entityInformation, _entityInformationHash) {
        if (!entityInformation.isDefined) {
            if (entityInformation.nodeName != AUTO
                    && entityInformation.nodeName != NOT_DEFINED
                    && (entityInformation.isSelected == CHECK)) {
                if (hashEntities.contains(entityInformation.id))
                    hashEntities[entityInformation.id]->define(entityInformation.nodeName);
            }
        }
    }
    accept();
}

void LoadBalancerDialog::_addNodeWidget(QString nodeKey, HnsClient::NodeInformation *nodeInformation)
{
    if (_noNodeLabel->isVisible())
        _noNodeLabel->hide();

    if (_nodeWidgetHash.contains(nodeKey)) {
        NodeResourceWidget *widget = _nodeWidgetHash.take(nodeKey);
        widget->deleteLater();
    }
    NodeResourceWidget *nodeResourceWidget = new NodeResourceWidget(nodeInformation, this);
    _nodeWidgetHash.insert(nodeKey, nodeResourceWidget);

    QVBoxLayout *vboxLayout = qobject_cast<QVBoxLayout *>(_ui->scrollAreaWidgetContents->layout());
    vboxLayout->insertWidget(vboxLayout->count() - 2, nodeResourceWidget);
    connect(_nodeWidgetHash[nodeKey], SIGNAL(overloaded(const QString &, int)) , this , SLOT(_overloadedSlot(const QString &, int)));
}

void LoadBalancerDialog::_removeNodeWidget(QString nodeKey)
{
    if (!_nodeWidgetHash.contains(nodeKey))
        return;
    NodeResourceWidget *nodeResourceWidget = _nodeWidgetHash.take(nodeKey);
    _ui->scrollAreaWidgetContents->layout()->removeWidget(nodeResourceWidget);
    nodeResourceWidget->disconnect();
    nodeResourceWidget->deleteLater();
    if (_ui->scrollAreaWidgetContents->layout()->count() <= 2) {//spacer + noNodeLabel
        _noNodeLabel->setVisible(true);
        _enableInterface(false);
    }
}

EntityLocalInformation LoadBalancerDialog::_generateEntityLocalInformation(HnsClient::GenericEntity *genericEntity, bool completeInformation)
{
    ::Hns::Resource resource =  ::Hns::getResourceFromEntry(genericEntity->getEntry()->entityEntry());
    EntityLocalInformation entityInformation;
    entityInformation.id = genericEntity->getId();
    entityInformation.shortName = genericEntity->getShortName();
    entityInformation.toolTip = Ressources::getToolTip(genericEntity);
    entityInformation.cpu = resource.cpu;
    entityInformation.memory = resource.memory;
    entityInformation.listAvailableNodeName = genericEntity->getNodeList();
    entityInformation.isDefined = genericEntity->isDefined();

    if (completeInformation) {
        entityInformation.isSelected = CHECK;
        entityInformation.nodeName = AUTO;
        entityInformation.warning = "";
        entityInformation.pixmap = Ressources::getClassNamePixmap(genericEntity->iceId());
        if(genericEntity->isDefined()) {
            entityInformation.isSelected = NOTHING;
            entityInformation.nodeName = genericEntity->getNodeName();
        } else if(entityInformation.listAvailableNodeName.isEmpty()) {
            entityInformation.isSelected = NOTHING;
            entityInformation.nodeName = NOT_DEFINED;
            entityInformation.warning = QString(tr("No node available"));
            _needToResizeLastColumn = false;
        }
    }
    return entityInformation;
}

} // namespace Hnv
