/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "loadbalancermodel.h"
#include "ressources.h"
#include "loadbalancerdialog.h"
#include "listnodedelegate.h"

// Qt headers
#include <QIcon>

namespace Hnv {

LoadBalancerModel::LoadBalancerModel(QObject *parent)
    : QAbstractTableModel(parent)
{
}

LoadBalancerModel::~LoadBalancerModel()
{
}

void LoadBalancerModel::clear()
{
    beginResetModel();
    _entityList.clear();
    _entityPositionHash.clear();
    endResetModel();
}

void LoadBalancerModel::setEntityList(QList<EntityLocalInformation> entityList)
{
    beginResetModel();
    _entityList.clear();
    _entityPositionHash.clear();
    _entityList = entityList;
    int row = 0;
    foreach(EntityLocalInformation entity, _entityList)
        _entityPositionHash.insert(entity.id, row++);
    endResetModel();
}

void LoadBalancerModel::addEntity(EntityLocalInformation entity)
{
    int row = _entityList.size();
    beginInsertRows(QModelIndex(), row, row);
    _entityList.append(entity);
    _entityPositionHash.insert(entity.id, row);
    endInsertRows();
}

void LoadBalancerModel::removeEntity(qlonglong entityID)
{
    if (_entityPositionHash.contains(entityID)) {
        int row  = _entityPositionHash.take(entityID);
        beginRemoveRows(QModelIndex(), row, row);
        _entityList.removeAt(row);
        endRemoveRows();
    }
}

void LoadBalancerModel::changeEntity(qlonglong entityID, EntityLocalInformation localInfo)
{
    if (_entityPositionHash.contains(entityID)) {
        int row = _entityPositionHash.value(entityID);
        QModelIndex firstIdx = index(row, 0);
        QModelIndex lastIdx = index(row, COLUMN_SIZE - 1);
        _entityList.replace(row, localInfo);
        emit dataChanged(firstIdx, lastIdx);
    }
}

int LoadBalancerModel::columnCount(const QModelIndex & parent) const
{
    if (!parent.isValid()) {
        return COLUMN_SIZE;
    }
    else return 0;
}

int LoadBalancerModel::rowCount(const QModelIndex & parent) const
{
    if (!parent.isValid()) {
        return _entityList.size();
    }
    else return 0;
}

QVariant LoadBalancerModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if (role == Qt::DisplayRole && orientation == Qt::Horizontal) {
        switch (section) {
        case COLUMN_SHORT_NAME:
            return tr("Short Name");
        case COLUMN_COMBOBOX:
            return tr("Node");
        case COLUMN_CPU:
            return tr("CPU");
        case COLUMN_MEMORY:
            return tr("Memory");
        case COLUMN_CHECK:
            return "";
        case COLUMN_WARNING:
            return tr("Information");
        }
    }
    return QAbstractTableModel::headerData(section, orientation, role);
}

QVariant LoadBalancerModel::data(const QModelIndex &index, int role) const
{
    if (index.isValid()) {
        int row = index.row();
        if (row < _entityList.size()) {
            EntityLocalInformation entity = _entityList.at(row);
            switch(role) {
            case Qt::DecorationRole: {
                switch(index.column()) {
                case COLUMN_SHORT_NAME: {
                    return QIcon(entity.pixmap);
                }
                case COLUMN_WARNING: {
                    if (entity.isDefined)
                        return QIcon(Ressources::themedPath(Constants::ICON_CHECK));
                    if (entity.listAvailableNodeName.empty())
                        return QIcon(Ressources::themedPath(Constants::ICON_ERROR));
                    if (!entity.warning.isEmpty())
                        return(QIcon(Ressources::themedPath(Constants::ICON_WARNING)));
                }
                    break;
                }
            }
            case Qt::ToolTipRole: {
                return entity.toolTip;
            }
            case Qt::UserRole: {
                return entity.shortName;
            }
            case Qt::UserRole+1: {
                if(index.column() == COLUMN_COMBOBOX)
                    return entity.nodeName;
                break;
            }
            case Qt::UserRole+2: {
                return entity.listAvailableNodeName;
            }
            case Qt::UserRole+3: {
                return entity.isDefined;
            }
            case Qt::UserRole+4: {
                return entity.id;
            }
            case Qt::UserRole+5: {
                return entity.isSelected;
            }
            case Qt::DisplayRole:
            case Qt::UserRole+6: {
                switch (index.column()) {
                case COLUMN_SHORT_NAME:
                    return entity.shortName;
                case COLUMN_CPU:
                    return QString::number(entity.cpu);
                case COLUMN_MEMORY:
                    return QString::number(entity.memory);
                case COLUMN_COMBOBOX: {
                    if(role == Qt::UserRole + 6){
                        if (!entity.nodeName.isEmpty())
                            return entity.nodeName;
                        else
                            return tr("Not defined");
                    }
                    return QVariant();
                }
                case COLUMN_CHECK: {
                    // Utilisation: pour un tri
                    if (role == Qt::UserRole + 6){
                        if (entity.isSelected == CHECK)
                            return "a";
                        else if(entity.isSelected == UNCHECK)
                            return "b";
                        else
                            return "c";
                    }
                }
                case COLUMN_WARNING: {
                    if (entity.isDefined)
                        return tr("Define on: %1").arg(entity.nodeName);
                    return entity.warning;
                }
                }
            }
            }
        }
    }
    return QVariant();
}

Qt::ItemFlags LoadBalancerModel::flags(const QModelIndex &index) const
{
    Qt::ItemFlags flags = QAbstractItemModel::flags(index);

    if (index.isValid()) {
        int row = index.row();
        if (row < _entityList.size()) {
            EntityLocalInformation entity = _entityList.at(row);
            if (entity.isSelected == NOTHING || entity.isSelected == UNCHECK) {
                switch(index.column()) {
                case COLUMN_WARNING:
                    flags = Qt::ItemIsEnabled;
                    break;
                default:
                    flags = Qt::NoItemFlags;
                }
            }
            if (entity.isSelected == UNCHECK) {
                switch(index.column()) {
                case COLUMN_CHECK:
                    flags = Qt::ItemIsEditable /*| Qt::ItemIsSelectable*/ | Qt::ItemIsEnabled;
                }
            }
            else if(entity.isSelected == CHECK) {
                switch (index.column()) {
                case COLUMN_COMBOBOX:
                case COLUMN_CHECK:
                    flags = Qt::ItemIsSelectable | Qt::ItemIsEditable | Qt::ItemIsEnabled;
                    break;
                default:
                    flags = Qt::ItemIsEnabled | Qt::ItemIsSelectable;
                }
            }
        }
    }
    return flags;
}

bool LoadBalancerModel::setData(const QModelIndex &idx, const QVariant &value, int role)
{
    if (idx.isValid()) {
        int row = idx.row();
        if (row < _entityList.size()) {
            if (role == Qt::EditRole) {
                switch(idx.column()) {
                case COLUMN_COMBOBOX: {
                    _entityList[row].nodeName = value.toString();
                    emit dataChanged(idx, idx);
                    return true;
                }
                case COLUMN_CHECK: {
                    if (_entityList[row].isSelected != value.toInt()) {
                        _entityList[row].isSelected = value.toInt();
                        _entityList[row].warning = "";
                        QModelIndex firstIdx = index(idx.row(), COLUMN_CHECK);
                        QModelIndex lastIdx = index(idx.row(),COLUMN_SIZE-1);
                        emit dataChanged(firstIdx, lastIdx);
                        return true;
                    }
                    break;
                }
                case COLUMN_WARNING: {
                    _entityList[row].warning = value.toString();
                    emit dataChanged(idx, idx);
                    return true;
                }
                }
            }
        }
    }
    return false;
}

int LoadBalancerModel::getRowFromID(qlonglong entityID)
{
    return _entityPositionHash.value(entityID, -1);
}

} // namespace Hnv
