/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "logmessagewidget.h"
#include "ui_logmessagewidget.h"
#include "hnvmainwindow.h"
#include "ressources.h"

// Qt headers
#include <QTimer>
#include <QSortFilterProxyModel>
#include <QScrollBar>

namespace Hnv {

/*******************************************************************************
* LogMessageItem
*/
LogMessageItem::LogMessageItem(LogMessageItem::MessageType type, QString msg, QDateTime dateTime)
    : _type(type)
    , _message(msg)
    , _dateTime(dateTime)
{
}

LogMessageItem::MessageType LogMessageItem::type()
{
    return _type;
}

QString LogMessageItem::message()
{
    return _message;
}

QDateTime LogMessageItem::date()
{
    return _dateTime;
}

/*******************************************************************************
* LogMessageModel
*/
LogMessageModel::LogMessageModel(QObject *parent)
    : QAbstractListModel(parent)
{
    _messageTypeStringHash.insert(LogMessageItem::Debug, tr("Debug"));
    _messageTypeStringHash.insert(LogMessageItem::Warning, tr("Warning"));
    _messageTypeStringHash.insert(LogMessageItem::Critical, tr("Error"));
    _messageTypeStringHash.insert(LogMessageItem::Fatal, tr("Fatal"));
    _messageTypeStringHash.insert(LogMessageItem::HnsTrace, tr("Master info"));
}

int LogMessageModel::rowCount(const QModelIndex &parent) const
{
    if (!parent.isValid())
        return _messageList.size();
    else
        return 0;
}

QVariant LogMessageModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid())
        return QVariant();
    int row = index.row();
    if (row >= _messageList.count())
        return QVariant();
    LogMessageItem item = _messageList.at(row);
    switch(role) {
    case LogMessageItem::MessageTypeRole:
        return item.type();
    case LogMessageItem::MessageDateRole:
        return item.date();
    case LogMessageItem::MessageSortRole:
        return row;
    case Qt::DisplayRole: {
        QString messageType = _messageTypeStringHash.value(item.type());
        return QString("[%1] %2 - %3").arg(item.date().time().toString(), messageType, item.message());
        break;
    }
    case Qt::ForegroundRole:
        QBrush defaultBrush;
        switch(item.type()) {
        case LogMessageItem::Debug:
            defaultBrush.setColor(Qt::gray);
            break;
        case LogMessageItem::Warning:
            defaultBrush.setColor(Qt::darkGreen);
            break;
        case LogMessageItem::Critical:
            defaultBrush.setColor(Qt::red);
            break;
        case LogMessageItem::Fatal:
            defaultBrush.setColor(Qt::darkRed);
            break;
        case LogMessageItem::HnsTrace:
            defaultBrush.setColor(Qt::darkBlue);
            break;
        }
        return defaultBrush;
        break;
    }
    return QVariant();
}

void LogMessageModel::clear()
{
    beginResetModel();
    _messageList.clear();
    endResetModel();
}

void LogMessageModel::addMessage(LogMessageItem::MessageType type, QString msg, QDateTime dateTime)
{
    int pos = _messageList.size();
    beginInsertRows(QModelIndex(), pos, pos);
    LogMessageItem message(type, msg, dateTime);
    _messageList.append(message);
    endInsertRows();
}

/*******************************************************************************
* LogMessageWidget
*/
LogMessageWidget::LogMessageWidget(QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::LogMessageWidget)
    , _waitToScroll(false)
{
    ui->setupUi(this);
    _proxyModel = new QSortFilterProxyModel(this);
    _proxyModel->setSortRole(LogMessageItem::MessageSortRole);
    _proxyModel->setFilterRole(LogMessageItem::MessageTypeRole);
    _proxyModel->setDynamicSortFilter(true);
    _proxyModel->setSourceModel(HnvMainWindow::instance()->logMessageModel());
    ui->messageListView->setModel(_proxyModel);
    connect(_proxyModel, SIGNAL(rowsInserted(const QModelIndex&, int, int)), this, SLOT(_triggerScrollToBottom()));
    ui->toolBarWidget->setTitle(windowTitle());
    ui->toolBarWidget->addToolBarAction(ui->action_Debug);
    ui->toolBarWidget->addToolBarAction(ui->action_Warning);
    ui->toolBarWidget->addToolBarAction(ui->action_Error);
    ui->toolBarWidget->addToolBarAction(ui->action_Trace);
    ui->toolBarWidget->addToolBarAction(ui->action_Clear);
    connect(ui->toolBarWidget, SIGNAL(closeTriggered()), this, SIGNAL(closeTriggered()));
    ui->messageListView->scrollToBottom();
    _setFilter();
    connect(ui->action_Debug, SIGNAL(triggered()), this, SLOT(_setFilter()));
    connect(ui->action_Warning, SIGNAL(triggered()), this, SLOT(_setFilter()));
    connect(ui->action_Error, SIGNAL(triggered()), this, SLOT(_setFilter()));
    connect(ui->action_Trace, SIGNAL(triggered()), this, SLOT(_setFilter()));
    ui->action_Clear->setIcon(QIcon(Ressources::themedPath(Constants::ICON_CLEAR_MESSAGES)));
    ui->action_Debug->setIcon(QIcon(Ressources::themedPath(Constants::ICON_DEBUG)));
    ui->action_Error->setIcon(QIcon(Ressources::themedPath(Constants::ICON_ERROR)));
    ui->action_Trace->setIcon(QIcon(Ressources::themedPath(Constants::ICON_TRACE)));
    ui->action_Warning->setIcon(QIcon(Ressources::themedPath(Constants::ICON_WARNING)));
    ui->toolBarWidget->setBorders(ToolBar::Bottom);
}

LogMessageWidget::~LogMessageWidget()
{
    delete ui;
}

/*******************************************************************************
* LogMessageWidget - Private slots
*/
void LogMessageWidget::_setFilter()
{
    QString regExp;
    if (!ui->action_Debug->isChecked())
        regExp += QString::number(LogMessageItem::Debug);
    if (!ui->action_Warning->isChecked())
        regExp += QString::number(LogMessageItem::Warning);
    if (!ui->action_Error->isChecked())
        regExp += QString::number(LogMessageItem::Critical);
    if (!ui->action_Trace->isChecked())
        regExp += QString::number(LogMessageItem::HnsTrace);

    if (regExp != "")
        regExp = QString("[^%1]").arg(regExp);
    _proxyModel->setFilterRegExp(regExp);
}

void LogMessageWidget::_triggerScrollToBottom()
{
    QScrollBar *scrollBar = ui->messageListView->verticalScrollBar();
    // Check if this widget is visible because there is currently one LogMessageWidget for each mode
    if (!_waitToScroll && isVisible() && (scrollBar->value() == scrollBar->maximum())) {
        _waitToScroll = true;
        // Don't move the scrollbar immediately to let the view update itself
        QTimer::singleShot(50, this, SLOT(_scrollToBottom()));
    }
}

void LogMessageWidget::_scrollToBottom()
{
    ui->messageListView->scrollToBottom();
    _waitToScroll = false;
}

void LogMessageWidget::on_action_Clear_triggered()
{
    HnvMainWindow::instance()->logMessageModel()->clear();
}

void LogMessageWidget::showEvent(QShowEvent *)
{
    _scrollToBottom();
}

} // namespace Hnv
