#include "memorydumpdialog.h"
#include "ui_memorydumpdialog.h"

#include "hyneviewsession.h"

#include "libhnsclient/entity.h"
#include "libhnsclient/jobmanager.h"
#include "libhnsclient/utility.h"
#include "libhnsclient/filetransferclient.h"

#include <QDateTime>
#include <QSortFilterProxyModel>
#include <QFileDialog>
#include <QMessageBox>

namespace Hnv {

/*******************************************************************************
 * MemoryDumpModel
 *******************************************************************************/
MemoryDumpModel::MemoryDumpModel(HnsClient::DomainEntity *domain, QObject *parent)
    : QAbstractTableModel(parent)
    , _domain(domain)
{
    _dumps = _domain->memoryDumpInfos();
    connect(_domain, SIGNAL(memoryDumpAdded(::Hns::MemoryDumpInfo)), this, SLOT(_memoryDumpAddedSlot(::Hns::MemoryDumpInfo)));
    connect(_domain, SIGNAL(memoryDumpDeleted(QString)), this, SLOT(_memoryDumpRemovedSlot(QString)));
}

MemoryDumpModel::~MemoryDumpModel()
{
}

int MemoryDumpModel::rowCount(const QModelIndex &parent) const
{
    if (!parent.isValid())
        return _dumps.size();
    return 0;
}

int MemoryDumpModel::columnCount(const QModelIndex &parent) const
{
    if (!parent.isValid())
        return COLUMN_COUNT;
    return 0;
}

QVariant MemoryDumpModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if (role == Qt::DisplayRole) {
        if (section == COLUMN_NAME)
            return tr("Name");
        else if (section == COLUMN_DATE)
            return tr("Date");
        else if (section == COLUMN_SIZE)
            return tr("Size");
    }
    return QAbstractTableModel::headerData(section, orientation, role);
}

QVariant MemoryDumpModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid() || index.row() >= _dumps.size())
        return QVariant();
    Hns::MemoryDumpInfo dumpInfo = _dumps.at(index.row());
    if (role == Qt::DisplayRole) {
        switch(index.column()) {
        case COLUMN_NAME:
            return QString::fromStdString(dumpInfo.name);
        case COLUMN_DATE:
            return QDateTime::fromMSecsSinceEpoch(dumpInfo.dateTime).toString();
        case COLUMN_SIZE:
            return HnsClient::Utility::makeSizeReadable(dumpInfo.size, HnsClient::Bytes);
        }
    } else if (role == FileNameRole) {
        return QString::fromStdString(dumpInfo.fileName);
    } else if (role == NameRole) {
        return QString::fromStdString(dumpInfo.name);
    }
    return QVariant();
}

void MemoryDumpModel::_memoryDumpAddedSlot(Hns::MemoryDumpInfo info)
{
    int pos = _dumps.size();
    beginInsertRows(QModelIndex(), pos, pos);
    _dumps.append(info);
    endInsertRows();
}

void MemoryDumpModel::_memoryDumpRemovedSlot(QString fileName)
{
    int size = _dumps.size();
    for (int i = 0 ; i < size ; ++i) {
        Hns::MemoryDumpInfo info = _dumps.at(i);
        if (info.fileName == fileName.toStdString()) {
            beginRemoveRows(QModelIndex(), i, i);
            _dumps.removeAt(i);
            endRemoveRows();
            return;
        }
    }
}

/*******************************************************************************
 * MemoryDumpDialog
 *******************************************************************************/
MemoryDumpDialog::MemoryDumpDialog(HnsClient::GenericEntity *genericEntity, QWidget *parent)
    : QDialog(parent)
    , ui(new Ui::MemoryDumpDialog)
    , _job(0)
    , _client(0)
{
    Q_UNUSED(genericEntity)
    ui->setupUi(this);
    ui->newDumpProgressBar->setVisible(false);
    ui->deleteProgressBar->setVisible(false);
    ui->downloadProgressBar->setVisible(false);

    ui->dumpHandlerWidget->setEnabled(false);

    _domain = qobject_cast<HnsClient::DomainEntity *>(genericEntity);
    if (!_domain)
        return;
    _domainShortNameChangedSlot();
    connect(_domain, SIGNAL(shortNameChanged()), this, SLOT(_domainShortNameChangedSlot()));
    _domainStatusChangedSlot();
    connect(_domain, SIGNAL(statusChanged(HnsClient::GenericEntity*)), this, SLOT(_domainStatusChangedSlot()));
    MemoryDumpModel *model = new MemoryDumpModel(_domain, this);
    QSortFilterProxyModel *proxyModel = new QSortFilterProxyModel(this);
    proxyModel->setSourceModel(model);
    ui->memoryDumpTableView->setModel(proxyModel);
    ui->memoryDumpTableView->resizeColumnsToContents();
    connect(ui->memoryDumpTableView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)), this, SLOT(_selectionChangedSlot()));
}

MemoryDumpDialog::~MemoryDumpDialog()
{
    delete ui;
}

void MemoryDumpDialog::on_buttonBox_rejected()
{
    reject();
}

void MemoryDumpDialog::_domainShortNameChangedSlot()
{
    ui->entityNameLabel->setText(tr("Memory dumps for %1").arg(_domain->getShortName()));
}

void MemoryDumpDialog::_domainStatusChangedSlot()
{
    Hns::Status status = _domain->getStatus();
    if (status == Hns::STARTED)
        ui->newDumpWidget->setEnabled(true);
    else
        ui->newDumpWidget->setEnabled(false);
    on_memoryDumpNameLineEdit_textChanged(ui->memoryDumpNameLineEdit->text());
}

void MemoryDumpDialog::_selectionChangedSlot()
{
    QModelIndexList indexList = ui->memoryDumpTableView->selectionModel()->selectedRows();
    ui->dumpHandlerWidget->setEnabled(!indexList.isEmpty());
}

void MemoryDumpDialog::on_downloadPushButton_clicked()
{
    QModelIndexList indexList = ui->memoryDumpTableView->selectionModel()->selectedRows();
    if (indexList.size() != 1)
        return;
    QModelIndex index = indexList.at(0);
    if (!index.isValid())
        return;
    QString remoteFileName = index.data(MemoryDumpModel::FileNameRole).toString();

    QFileDialog fileDialog(this, tr("Select destination directory"), QDir::currentPath());
    fileDialog.setFileMode(QFileDialog::Directory);
    fileDialog.setOption(QFileDialog::ShowDirsOnly);
    if (fileDialog.exec() == QDialog::Rejected)
        return;

    QString destDirPath = fileDialog.selectedFiles().value(0);
    QDir destDir(destDirPath);
    if (!destDir.exists()) {
        QMessageBox::warning(this, tr("Error"), tr("Destination directory doesn't exist"), QMessageBox::Ok);
        return;
    }

    QString localFileName = destDirPath + "/" + remoteFileName;
    _domain->downloadMemoryDump(remoteFileName, localFileName, this, SLOT(_getFileTransferClient(HnsClient::FileTransferClient*)));
    ui->newDumpWidget->setEnabled(false);
    ui->memoryDumpTableView->setEnabled(false);
    ui->downloadPushButton->setEnabled(false);
    ui->deletePushButton->setEnabled(false);
}

void MemoryDumpDialog::on_deletePushButton_clicked()
{
    QModelIndexList indexList = ui->memoryDumpTableView->selectionModel()->selectedRows();
    if (indexList.size() != 1)
        return;
    QModelIndex index = indexList.at(0);
    if (!index.isValid())
        return;
    ui->dumpHandlerWidget->setVisible(false);
    ui->deleteProgressBar->setVisible(true);
    _domain->deleteMemoryDump(index.data(MemoryDumpModel::FileNameRole).toString(), this, SLOT(_deleteMemoryDumpSlot()));
}

void MemoryDumpDialog::on_createPushButton_clicked()
{
    ui->newDumpProgressBar->setVisible(true);
    ui->newDumpWidget->setVisible(false);
    _domain->createMemoryDumpJob(ui->memoryDumpNameLineEdit->text(), this, SLOT(_createMemoryDumpJobSlot(int)));
}

void MemoryDumpDialog::on_memoryDumpNameLineEdit_textChanged(const QString &arg1)
{
    ui->createPushButton->setEnabled(!arg1.isEmpty());
}

void MemoryDumpDialog::_createMemoryDumpJobSlot(int jobId)
{
    if (jobId < 0) {// something went wrong
        QMessageBox::critical(this, tr("Error"), tr("Memory dump failed"), QMessageBox::Ok);
        ui->newDumpProgressBar->setVisible(false);
        ui->newDumpWidget->setVisible(true);
        return;
    }
    HyneviewSession::instance()->jobManager()->getJob(jobId, this, SLOT(_getJobSlot(HnsClient::Job*)));
}

void MemoryDumpDialog::_deleteMemoryDumpSlot()
{
    ui->dumpHandlerWidget->setVisible(true);
    ui->deleteProgressBar->setVisible(false);
}

void MemoryDumpDialog::_getJobSlot(HnsClient::Job *job)
{
    _job = job;
    double progress = job->getProgress();
    if (progress != 1.0 && job->isRunning())
        connect(job, SIGNAL(finished(int)), this, SLOT(_jobFinishedSlot()));
    else
        _jobFinishedSlot();
}

void MemoryDumpDialog::_getFileTransferClient(HnsClient::FileTransferClient *client)
{
    _client = client;
    connect(client, SIGNAL(transferProgression(int)), ui->downloadProgressBar, SLOT(setValue(int)));
    connect(client, SIGNAL(transferFinished(QString,QString)), this, SLOT(_transferClientFinishedSlot(QString,QString)));
    ui->downloadProgressBar->setVisible(true);
    _client->startDownload(true);
}

void MemoryDumpDialog::_transferClientFinishedSlot(QString remoteFileName, QString localFileName)
{
    Q_UNUSED(remoteFileName)
    Q_UNUSED(localFileName)
    ui->downloadProgressBar->disconnect(_client);
    ui->downloadProgressBar->setValue(0);
    ui->downloadProgressBar->setVisible(false);
    QMessageBox::information(this, tr("Success"), tr("File successfully downloaded"), QMessageBox::Ok);
    _client->deleteLater();
    _client = 0;
    _domainStatusChangedSlot();
    ui->memoryDumpTableView->setEnabled(true);
    ui->downloadPushButton->setEnabled(true);
    ui->deletePushButton->setEnabled(true);
}

void MemoryDumpDialog::_jobFinishedSlot()
{
    ui->newDumpProgressBar->setVisible(false);
    ui->newDumpWidget->setVisible(true);
    _job->deleteLater();
    _job = 0;
}

} // namespace Hnv
