/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "modebrowserwidget.h"
#include "ui_modebrowserwidget.h"
#include "logmessagewidget.h"
#include "accountdialog.h"
#include "ressources.h"
//#include "hnvmainwindow.h"
#include "sidebar.h"

// Qt headers
#include <QButtonGroup>
#include <QSettings>

namespace Hnv {

/*****************************************************************************
 *  ModeButton: QToolButton with pre-defined properties
*******************************************************************************/
ModeButton::ModeButton(ModeWidget *modeWidget)
    : QToolButton(modeWidget)
    , _modeWidget(modeWidget)
{
    setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
    setCheckable(true);
    setAutoRaise(true);

    QSettings settings;
    settings.beginGroup(Constants::MAIN_WINDOW_SETTINGS);
    switch (settings.value(Constants::MODEBROWSERBAR_KEY
        , Constants::MODEBROWSERBAR_DEFAULT_STATE).toInt()) {
    case Constants::ModeBrowserFull:
        setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
        setIconSize(QSize(Constants::MODE_BUTTON_ICON_SIZE_WIDTH
            , Constants::MODE_BUTTON_ICON_SIZE_HEIGHT));
        break;
    case Constants::ModeBrowserIcon:
        setToolButtonStyle(Qt::ToolButtonIconOnly);
        setIconSize(QSize(Constants::MODE_BUTTON_ICON_SIZE_WIDTH
            , Constants::MODE_BUTTON_ICON_SIZE_HEIGHT));
        break;
    case Constants::ModeBrowserMini:
        setToolButtonStyle(Qt::ToolButtonIconOnly);
        setIconSize(QSize(Constants::BROSWER_TOOLBUTTON_ICON_SIZE_WIDTH
            , Constants::BROSWER_TOOLBUTTON_ICON_SIZE_HEIGHT));
        break;
    }
    settings.endGroup();
}

ModeWidget * ModeButton::modeWidget()
{
    return _modeWidget;
}

/*****************************************************************************
 *  BrowserToolButton: QToolButton with pre-defined properties
*******************************************************************************/
BrowserToolButton::BrowserToolButton(QWidget *parent)
    : QToolButton(parent)
{
    setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
    setIconSize(QSize(Constants::BROSWER_TOOLBUTTON_ICON_SIZE_WIDTH, Constants::BROSWER_TOOLBUTTON_ICON_SIZE_HEIGHT));
    setAutoRaise(true);
}

/*****************************************************************************
 *  ModeDisplayArea: Splittable area which display main widget and log message
*******************************************************************************/
ModeDisplayArea::ModeDisplayArea(QWidget *displayWidget, QWidget *parent)
    : QSplitter(Qt::Vertical, parent)
    , _displayWidget(displayWidget)
    , _logMessageWidget(0)
{
    setHandleWidth(1);
    setChildrenCollapsible(false);
    setContentsMargins(0, 0, 0, 0);
    addWidget(_displayWidget);
    int idx = indexOf(_displayWidget);
    setStretchFactor(idx, 1);
}

ModeDisplayArea::~ModeDisplayArea()
{
    if (_logMessageWidget)
        delete _logMessageWidget;
}

QWidget * ModeDisplayArea::displayWidget()
{
    return _displayWidget;
}

/*******************************************************************************
* ModeDisplayArea - Public slots
*/
void ModeDisplayArea::toggleLogMessageWidget(bool display)
{
    if (!display) {
        if (_logMessageWidget) {
            _logMessageWidget->hide();
        }
    } else {
        if (_logMessageWidget == 0) {
            _logMessageWidget = new LogMessageWidget(this);
            addWidget(_logMessageWidget);
            connect(_logMessageWidget, SIGNAL(closeTriggered()), this, SIGNAL(logMessageWidgetClosed()));
        } else {
            _logMessageWidget->show();
        }
    }
}

/*****************************************************************************
 *  ModeWidget: Base class to be used as main window widget pages
 */
ModeWidget::ModeWidget(QWidget *parent)
    : QSplitter(Qt::Horizontal, parent)
    , _modeButton(0)
    , _modeAction(0)
    , _sideBar(0)
    , _rightSideBar(0)
    , _displayArea(0)
{
    setHandleWidth(1);
    setChildrenCollapsible(false);
    setContentsMargins(0, 0, 0, 0);
}

ModeWidget::~ModeWidget()
{
    if (_displayArea)
        delete _displayArea;
    if (_sideBar)
        delete _sideBar;
    if (_rightSideBar)
        delete _rightSideBar;
}

void ModeWidget::initWidget()
{
}

ModeButton * ModeWidget::getModeButton()
{
    if (!_modeButton)
        _initModeButton();
    return _modeButton;
}

void ModeWidget::_initDisplayArea(QWidget *displayWidget)
{
    if (_displayArea) {
        delete _displayArea;
        _displayArea = 0;
    }
    _displayArea = new ModeDisplayArea(displayWidget, this);
    connect(_displayArea, SIGNAL(logMessageWidgetClosed()), this, SIGNAL(logMessageWidgetClosed()));
    addWidget(_displayArea);
    int idx = indexOf(_displayArea);
    setStretchFactor(idx, 1);
}

void ModeWidget::_initSideBar(SideBar *sideBar)
{
    if (sideBar) {
        if (_sideBar)
            delete _sideBar;
        _sideBar = sideBar;
        connect(_sideBar, SIGNAL(hidden()), this, SIGNAL(sideBarHidden()));
        insertWidget(0, _sideBar);
    }
}

void ModeWidget::_initRightSideBar(SideBar *sideBar)
{
    if (sideBar) {
        if (_rightSideBar)
            delete _rightSideBar;
        _rightSideBar = sideBar;
        connect(_rightSideBar, SIGNAL(hidden()), this, SIGNAL(rightSideBarHidden()));
    }
}

void ModeWidget::setModeEnabled(bool enabled)
{
    _modeButton->setEnabled(enabled);
    _modeButton->setVisible(enabled);
    if (_modeButton->defaultAction()) {
        _modeButton->defaultAction()->setEnabled(enabled);
        _modeButton->defaultAction()->setVisible(enabled);
    }
    setEnabled(enabled);
    if (enabled == false)
        emit hidden();
}

ModeDisplayArea * ModeWidget::displayArea()
{
    return _displayArea;
}

QWidget * ModeWidget::displayWidget()
{
    if (_displayArea)
        return _displayArea->displayWidget();
    return 0;
}

void ModeWidget::restoreDefaultSettings()
{
    if (_sideBar)
        _sideBar->restoreDefaultSettings();
    if (_rightSideBar)
        _rightSideBar->restoreDefaultSettings();
}

void ModeWidget::saveSettings()
{
    if (_sideBar)
        _sideBar->saveSettings();
    if (_rightSideBar)
        _rightSideBar->saveSettings();
}

bool ModeWidget::isSideBarVisible()
{
    if (_sideBar)
        return _sideBar->isVisible();
    return false;
}

bool ModeWidget::isRightSideBarVisible()
{
    if (_rightSideBar)
        return _rightSideBar->isVisible();
    return false;
}

/*******************************************************************************
* ModeWidget - Public slots
*/
void ModeWidget::toggleSideBar()
{
    if (_sideBar)
        _sideBar->toggleDisplay();
}

void ModeWidget::toggleRightSideBar()
{
    if (_rightSideBar)
        _rightSideBar->toggleDisplay();
}

void ModeWidget::toggleLogMessageWidget(bool checked)
{
    if (_displayArea)
        _displayArea->toggleLogMessageWidget(checked);
}

void ModeWidget::_initModeButton()
{
    if (!_modeAction)
        _initModeAction();
    _modeAction->setCheckable(true);
    _modeButton = new ModeButton(this);
    _modeButton->setDefaultAction(_modeAction);
    _modeButton->setEnabled(_modeAction->isEnabled());
    _modeButton->setVisible(_modeAction->isVisible());
    connect(_modeAction, SIGNAL(triggered()), _modeButton, SIGNAL(clicked()));
}

/*****************************************************************************
 *  ModeBrowserWidget: Insert buttons in browser bar and connect to pages
*******************************************************************************/
ModeBrowserWidget::ModeBrowserWidget(QWidget *parent)
    : ButtonListWidget(parent)
    , ui(new Ui::ModeBrowserWidget)
    , _actionQuickConnectEnable(false)
{
    ui->setupUi(this);
    _mModeButtons = new QButtonGroup(this);
    _mModeButtons->setExclusive(true);
    _actionGroup = new QActionGroup(this);
    _actionGroup->setExclusive(true);
    ui->accountButton->setDefaultAction(ui->action_Account);
    ui->connectButton->setDefaultAction(ui->action_Connect);
    ui->quickConnectButton->setDefaultAction(ui->action_Quick_connect);
    ui->disconnectButton->setDefaultAction(ui->action_Disconnect);
    connect(ui->action_Connect, SIGNAL(triggered()), this, SIGNAL(connectTriggered()));
    connect(ui->action_Quick_connect, SIGNAL(triggered()), this, SIGNAL(quickConnectTriggered()));
    connect(ui->action_Disconnect, SIGNAL(triggered()), this, SIGNAL(disconnectedTriggered()));
    setDisconnected();
    ui->action_Account->setIcon(QIcon(Ressources::themedPath(Constants::ICON_PREFERENCE_SETTINGS)));
    ui->action_Connect->setIcon(QIcon(Ressources::themedPath((Constants::ICON_CONNECT))));
    ui->action_Disconnect->setIcon(QIcon(Ressources::themedPath(Constants::ICON_DISCONNECT)));
    ui->action_Quick_connect->setIcon(QIcon(Ressources::themedPath(Constants::ICON_QUICK_CONNECT)));
}

ModeBrowserWidget::~ModeBrowserWidget()
{
    delete ui;
}

void ModeBrowserWidget::registerModeWidget(ModeWidget *modeWidget, bool isBottomWidget)
{
    ModeButton * modeButton = modeWidget->getModeButton();
    connect(modeButton, SIGNAL(clicked()), this, SLOT(modeButtonTriggered()));
    connect(modeWidget, SIGNAL(hidden()), this, SLOT(_modeHidden()));
    connect(modeWidget, SIGNAL(sideBarHidden()), this, SLOT(_sideBarHiddenSlot()));
    connect(modeWidget, SIGNAL(rightSideBarHidden()), this, SLOT(_rightSideBarHiddenSlot()));
    _modeActionlist.append(modeButton->defaultAction());
    modeButton->defaultAction()->setShortcut(QKeySequence(tr("Ctrl+%1").arg(_modeActionlist.size())));
    _mModeButtons->addButton(modeButton);
    _actionGroup->addAction(modeButton->defaultAction());
    int pos = _mTopButtons.count();
    if (isBottomWidget) {
        pos++; // After spacer
        _mBottomButtons.append(modeButton);
    } else
        _mTopButtons.append(modeButton);
    ui->verticalLayout->insertWidget(pos, modeButton);
}

QList<QAction *> ModeBrowserWidget::fileMenuActions()
{
    QList<QAction *> list;
    list.append(ui->action_Connect);
    list.append(ui->action_Quick_connect);
    list.append(ui->action_Disconnect);
    return list;
}

QList<QAction *> ModeBrowserWidget::modeMenuActions()
{
    return _modeActionlist;
}

void ModeBrowserWidget::switchToMode(int index)
{
    if (index < 0)
        return;
    ModeButton *targetButton = 0;
    int topListCount = _mTopButtons.size();
    if (index < topListCount)
        targetButton = _mTopButtons.at(index);
    else if (index < topListCount + _mBottomButtons.size())
        targetButton = _mBottomButtons.at(index - topListCount);
    if (targetButton)
        targetButton->defaultAction()->trigger();
}

QButtonGroup * ModeBrowserWidget::modeButtons()
{
    return _mModeButtons;
}

void ModeBrowserWidget::restoreDefaultSettings()
{
    foreach(ModeButton * button, _mBottomButtons)
        button->modeWidget()->restoreDefaultSettings();
    foreach(ModeButton * button, _mTopButtons)
        button->modeWidget()->restoreDefaultSettings();
}

void ModeBrowserWidget::saveSettings()
{
    foreach(ModeButton * button, _mBottomButtons)
        button->modeWidget()->saveSettings();
    foreach(ModeButton * button, _mTopButtons)
        button->modeWidget()->saveSettings();
}

/*******************************************************************************
* ModeBrowserWidget - Public slots
*/
void ModeBrowserWidget::modeButtonTriggered()
{
    ModeButton *button = qobject_cast<ModeButton*>(sender());
    if (!button)
        return;
    emit modeChanged(button->modeWidget());
    emit sidebarEnabled(button->modeWidget()->hasSideBar(), button->modeWidget()->isSideBarVisible());
    emit rightSidebarEnabled(button->modeWidget()->hasRightSideBar(), button->modeWidget()->isRightSideBarVisible());
}

void ModeBrowserWidget::setConnected(HyneviewSession *)
{
    ui->action_Disconnect->setEnabled(true);
    ui->action_Account->setEnabled(true);
    ui->action_Connect->setEnabled(false);
    ui->connectButton->setEnabled(false);
    ui->action_Quick_connect->setEnabled(false);
    ui->quickConnectButton->setEnabled(false);

}

void ModeBrowserWidget::setDisconnected()
{
    ui->action_Disconnect->setEnabled(false);
    ui->action_Account->setEnabled(false);
    ui->connectButton->setEnabled(true);
    ui->action_Connect->setEnabled(true);
    if(_actionQuickConnectEnable)
    {
        ui->action_Quick_connect->setEnabled(true);
        ui->quickConnectButton->setEnabled(true);
    }
}

/*******************************************************************************
* ModeBrowserWidget - Private slots
*/
void ModeBrowserWidget::_modeHidden()
{
    ModeWidget *widget = qobject_cast<ModeWidget*>(sender());
    if (!widget)
        return;
    ModeButton *button = widget->getModeButton();
    if (button->isChecked())
        switchToMode(0);
}

void ModeBrowserWidget::on_action_Account_triggered()
{
    AccountDialog popup(this);
    popup.exec();
}

void ModeBrowserWidget::_sideBarHiddenSlot()
{
    ModeWidget *modeWidget = qobject_cast<ModeWidget*>(sender());
    if (!modeWidget)
        return;
    modeWidget->toggleSideBar();
    emit sidebarEnabled(modeWidget->hasSideBar(), false);
}

void ModeBrowserWidget::_rightSideBarHiddenSlot()
{
    ModeWidget *modeWidget = qobject_cast<ModeWidget*>(sender());
    if (!modeWidget)
        return;
    modeWidget->toggleRightSideBar();
    emit rightSidebarEnabled(modeWidget->hasRightSideBar(), false);
}

void ModeBrowserWidget::enableQuickConnect(bool enable){
    ui->quickConnectButton->setEnabled(enable);
    ui->action_Quick_connect->setEnabled(enable);
    _actionQuickConnectEnable = enable;
}

} // namespace Hnv
