/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "overlayexpandablewidget.h"
#include "ui_overlayexpandablewidget.h"
#include "ressources.h"

// Qt headers
#include <QToolButton>
#include <QResizeEvent>

namespace Hnv {

/*******************************************************************************
* OverlayExpandableWidget
*******************************************************************************/
OverlayExpandableWidget::OverlayExpandableWidget(QString title, QWidget *parent)
    : OverlayWidget(parent)
    , ui(new Ui::OverlayExpandableWidget)
    , _centralWidget(NULL)
    , _centralIsVisible(true)
{
    _centralWidget = 0;
    ui->setupUi(this);
    _initTitleBar(title);
}

OverlayExpandableWidget::OverlayExpandableWidget(QString title, Qt::Corner corner, QWidget *parent)
    : OverlayWidget(corner, parent)
    , ui(new Ui::OverlayExpandableWidget)
    , _centralWidget(NULL)
    , _centralIsVisible(true)
{
    ui->setupUi(this);
    _initTitleBar(title);
}

OverlayExpandableWidget::~OverlayExpandableWidget()
{
    delete ui;
}

void OverlayExpandableWidget::setAttachCorner(Qt::Corner corner)
{
    if (_attachCorner == corner)
        return;
    _setTitleBarPosition(corner);
    switch(corner) {
    case Qt::TopLeftCorner:
        setContentsMargins(0, 0, 1, 1);
        break;
    case Qt::TopRightCorner:
        setContentsMargins(1, 0, 0, 1);
        break;
    case Qt::BottomLeftCorner:
        setContentsMargins(0, 1, 1, 0);
        break;
    case Qt::BottomRightCorner:
        setContentsMargins(1, 1, 0, 0);
    }
    OverlayWidget::setAttachCorner(corner);
}

void OverlayExpandableWidget::setCentralWidget(QWidget *widget)
{
    if (_centralWidget == widget)
        return;
    if (_centralWidget != 0) {
        _centralWidget->removeEventFilter(this);
        ui->verticalLayout->removeWidget(_centralWidget);
    }
    _centralWidget = widget;
    if (_centralWidget) {
        _centralWidget->setParent(this);
        _centralIsVisible = true;
        if (_attachCorner == Qt::TopLeftCorner || _attachCorner == Qt::TopRightCorner)
            ui->verticalLayout->insertWidget(0, _centralWidget);
        else
            ui->verticalLayout->addWidget(_centralWidget);
        _updateSize();
        _centralWidget->installEventFilter(this);
    }
}

QWidget * OverlayExpandableWidget::centralWidget()
{
    return _centralWidget;
}

/*******************************************************************************
* OverlayExpandableWidget - Protected
*/
bool OverlayExpandableWidget::eventFilter(QObject *obj, QEvent *event)
{
    if (obj == _centralWidget && event->type() == QEvent::Resize) {
        _updateSize();
    }
    return OverlayWidget::eventFilter(obj, event);
}

void OverlayExpandableWidget::resizeEvent(QResizeEvent *event)
{
    OverlayWidget::resizeEvent(event);
}

void OverlayExpandableWidget::_setTitleBarPosition(Qt::Corner attachCorner)
{
    ui->verticalLayout->removeWidget(ui->titleBarFrame);
    if (attachCorner == Qt::TopLeftCorner || attachCorner == Qt::TopRightCorner)
        ui->verticalLayout->addWidget(ui->titleBarFrame);
    else
        ui->verticalLayout->insertWidget(0, ui->titleBarFrame);
}

/*******************************************************************************
* OverlayExpandableWidget - Private slots
*/
void OverlayExpandableWidget::_toggleButtonClicked(bool checked)
{
    if (_centralWidget != 0) {
        _centralWidget->setHidden(checked);
        _centralIsVisible = !checked;
        _updateSize();
    }
}

void OverlayExpandableWidget::_initTitleBar(QString title)
{
    ui->titleLabel->setText(title);
    ui->titleLabel->setForegroundRole(QPalette::Light);
    QIcon icon(Ressources::themedPath(Constants::ICON_COLLAPSE));
    icon.addFile(Ressources::themedPath(Constants::ICON_EXPAND),QSize(),QIcon::Normal, QIcon::On);
    ui->toggleButton->setIcon(icon);
    connect(ui->toggleButton, SIGNAL(clicked(bool)), this, SLOT(_toggleButtonClicked(bool)));
    int maxH = qMax(ui->titleLabel->sizeHint().height(), ui->toggleButton->sizeHint().height());
    int frameWidth = ui->titleLabel->sizeHint().width() + ui->toggleButton->sizeHint().width();
    ui->titleBarFrame->resize(frameWidth, maxH);
    _setTitleBarPosition(_attachCorner);
}

void OverlayExpandableWidget::_updateSize()
{
    QSize centralSize = _centralWidget->sizeHint();
    int targetWidth = centralSize.width()/* + frameWidth * 2*/;
    int targetHeight = ui->titleBarFrame->height()/* + frameWidth * 2*/;
    int titlePos = 0;
    if (_centralIsVisible) {
        targetHeight += centralSize.height();
        titlePos += centralSize.height();
    }
    ui->titleLabel->setMinimumWidth(qMax(_centralWidget->width() - ui->toggleButton->width(), 0));
    ui->titleBarFrame->adjustSize();
    setGeometry(0,0, targetWidth, targetHeight);
    _attachToParent();
}

} // namespace Hnv
