/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "sidebar.h"
#include "hyneviewtoolbar.h"
#include "ressources.h"

// Qt headers
#include <QEvent>
#include <QSettings>
#include <QPair>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QPainter>
#include <QStyleOption>
#include <QTreeView>
#include <QListView>
#include <QHeaderView>
#include <QKeyEvent>

namespace Hnv {

/*******************************************************************************
* SideBarItem
*******************************************************************************/
SideBarItem::SideBarItem(QString id, QString title, QMetaObject metaObject, int stretchFactor)
    : _widgetID(id)
    , _widgetTitle(title)
    , _metaObject(metaObject)
    , _stretchFactor(stretchFactor)
{
}

QString SideBarItem::itemID()
{
    return _widgetID;
}

QString SideBarItem::itemTitle()
{
    return _widgetTitle;
}

QMetaObject SideBarItem::itemMetaObject()
{
    return _metaObject;
}

QWidget * SideBarItem::createWidget(QWidget *parent)
{
    QGenericArgument parentArg = Q_ARG(QWidget *, parent);
    QObject * obj = _metaObject.newInstance(parentArg);
    QWidget * widget = qobject_cast<QWidget *>(obj);
    if(widget)
        return widget;
    return 0;
}

int SideBarItem::stretchFactor()
{
    return _stretchFactor;
}

/*******************************************************************************
* SideBar
*******************************************************************************/
SideBar::SideBar(QString sideBarID, QString defaultWidgetID, QList<SideBarItem *> itemList, QWidget *parent)
    : QSplitter(Qt::Vertical, parent)
    , _sideBarID(sideBarID)
    , _defaultWidgetID(defaultWidgetID)
    , _itemList(itemList)
{
    setHandleWidth(1);
    setChildrenCollapsible(false);
    setContentsMargins(0, 0, 0, 0);
    loadSettings();
    if (_itemList.count()) {
        foreach(SideBarItem *item, _itemList)
            _itemIdsHash.insert(item->itemID(), item->itemTitle());
        _initSideBar();
    }
}

SideBar::~SideBar()
{
    qDeleteAll(_widgetList);
    _widgetList.clear();
    qDeleteAll(_itemList);
    _itemList.clear();
}
QHash<QString, QString> SideBar::itemIdsMap()
{
    return _itemIdsHash;
}

QList<SideBarItem *> SideBar::itemList()
{
    return _itemList;
}

QWidget *SideBar::getItemWidget(QString widgetId, int &stretchFactor, SideBarWidget *parent)
{
    if (_itemIdsHash.contains(widgetId)) {
        foreach(SideBarItem * item, _itemList) {
            if (item->itemID() == widgetId) {
                stretchFactor = item->stretchFactor();
                return item->createWidget(parent);
            }
        }
    }
    return 0;
}

void SideBar::toggleDisplay()
{
    int widgetCount = _widgetList.count();
    if (isVisible())
        hide();
    else {
        if (!widgetCount)
            _initSideBar();
        show();
    }
}

void SideBar::displayWidget(QString widgetId)
{
    if (_itemIdsHash.contains(widgetId))
        _insertSideBarWidget(_widgetList.count(), widgetId);
}

void SideBar::loadSettings()
{
    QSettings settings;
    settings.beginGroup(QString(Constants::SIDEBAR_SETTINGS) + _sideBarID);
    if (settings.contains(QString(Constants::SIDEBAR_IS_VISIBLE))){
        this->setVisible(settings.value(Constants::SIDEBAR_IS_VISIBLE,true).toBool());
    } else {
        this->setVisible(true);
    }
    settings.beginGroup(QString(Constants::OPENED_WIDGETS));
    if (settings.contains(_sideBarID)) {
        _openedWidgets = settings.value(_sideBarID).toStringList();
    } else {
        if (!_defaultWidgetID.isEmpty())
            _openedWidgets.append(_defaultWidgetID);
    }
    if (_openedWidgets.size() == 0 ){
        this->setVisible(false);
    }
    settings.endGroup();
    settings.endGroup();
}

void SideBar::saveSettings()
{
    _openedWidgets.clear();
    QSettings settings;
    settings.beginGroup(QString(Constants::SIDEBAR_SETTINGS) + _sideBarID);
    if (settings.value(Constants::SIDEBAR_STATE_IS_SAVED, Constants::SIDEBAR_DEFAULT_STATE_SAVE).toBool()) {
        settings.setValue(Constants::SIDEBAR_IS_VISIBLE,!this->isHidden());
        if (_widgetList.size() != 0) {
            foreach (SideBarWidget *widget,_widgetList) {
                _openedWidgets.append(widget->widgetId());
            }
        } else {
            _openedWidgets = QStringList();
        }
        settings.beginGroup(QString(Constants::OPENED_WIDGETS));
        settings.setValue(_sideBarID, _openedWidgets);
        settings.endGroup();
    }
    settings.endGroup();
}

void SideBar::restoreDefaultSettings()
{
}

/*******************************************************************************
* SideBar - Protected
*******************************************************************************/
void SideBar::paintEvent(QPaintEvent *)
{
    QStyleOption opt;
    opt.init(this);
    QPainter p(this);
    style()->drawPrimitive(QStyle::PE_Widget, &opt, &p, this);
}

void SideBar::_initSideBar()
{
    if (_openedWidgets.size()){
        foreach(QString saveWidgetID, _openedWidgets)
            displayWidget(saveWidgetID);
    } else {
        _insertSideBarWidget(0, _itemList.at(0)->itemID());
    }
}

/*******************************************************************************
* SideBar - Private slots
*******************************************************************************/
void SideBar::_splitSideBar()
{
    SideBarWidget *widget = qobject_cast<SideBarWidget*>(sender());
    if (!widget)
        return;
    int pos = indexOf(widget);
    _insertSideBarWidget(pos + 1, widget->widgetId());
}

void SideBar::_closeSideBarWidget()
{
    SideBarWidget *widget = qobject_cast<SideBarWidget*>(sender());
    if (!widget)
        return;
    _widgetList.removeAll(widget);
    delete widget;
    if (!_widgetList.count())
        emit hidden();
}

/*******************************************************************************
* SideBar - Private
*******************************************************************************/
void SideBar::_insertSideBarWidget(int position, QString widgetId)
{
    SideBarWidget *widget = _createWidget(widgetId);
    insertWidget(position, widget);
    setStretchFactor(position, widget->stretchFactor());
}

SideBarWidget * SideBar::_createWidget(QString widgetId)
{
    SideBarWidget *widget = new SideBarWidget(this, widgetId, this);
    _widgetList.append(widget);
    connect(widget, SIGNAL(splitWidget()), this, SLOT(_splitSideBar()));
    connect(widget, SIGNAL(closeWidget()), this, SLOT(_closeSideBarWidget()));
    return widget;
}

/*******************************************************************************
 * SideBarWidget
*******************************************************************************/
SideBarWidget::SideBarWidget(SideBar *sideBar, QString widgetId, QWidget *parent)
    : StyledWidget(parent)
    , _sideBar(sideBar)
    , _itemWidget(0)
    , _stretchFactor(0)
{
    _widgetId = "";
    QList<QPair<QString,QString> > listPairs;
    QHash<QString,QString> map = _sideBar->itemIdsMap();
    QList<SideBarItem *> itemList = _sideBar->itemList();
    int i = 0;
    while (i < itemList.size()){
        if (map.contains(itemList.value(i)->itemID())){
            listPairs.append(QPair<QString,QString> (map.value(itemList.value(i)->itemID()),itemList.value(i)->itemID()));
        }
        i++;
    }

    _toolBar = new SideBarToolBar(listPairs, this);
    QVBoxLayout *vLayout = new QVBoxLayout();
    vLayout->setContentsMargins(0, 0, 0, 0);
    vLayout->setSpacing(0);
    setLayout(vLayout);
    vLayout->addWidget(_toolBar);
    setCurrentWidget(QVariant(widgetId));
    connect(_toolBar, SIGNAL(closeTriggered()), this, SIGNAL(closeWidget()));
    connect(_toolBar, SIGNAL(splitTriggered()), this, SIGNAL(splitWidget()));
    connect(_toolBar, SIGNAL(currentIndexChanged(QVariant)), this, SLOT(setCurrentWidget(QVariant)));
}

SideBarWidget::~SideBarWidget()
{

}

void SideBarWidget::removeCurrentWidget()
{
    if (!_itemWidget)
        return;
    _itemWidget->hide();
    layout()->removeWidget(_itemWidget);
    delete _itemWidget;
    _itemWidget = 0;
}

QString SideBarWidget::widgetId()
{
    return _widgetId;
}

int SideBarWidget::stretchFactor()
{
    return _stretchFactor;
}

SideBar * SideBarWidget::parentSideBar()
{
    return _sideBar;
}

QWidget * SideBarWidget::sideBarParent()
{
    QWidget * widget = 0;
    if (_sideBar)
        widget = _sideBar->parentWidget();
    return widget;
}

/*******************************************************************************
* SideBarWidget - Public slots
*******************************************************************************/
void SideBarWidget::setCurrentWidget(QVariant widgetIdv)
{
    QString widgetId = widgetIdv.toString();
    int strechFactor = 0;
    QWidget *w = _sideBar->getItemWidget(widgetId, strechFactor, this);
    if (!w)
        return;
    _stretchFactor = strechFactor;
    _widgetId = widgetId;
    _toolBar->setSelectedItem(_widgetId);
    removeCurrentWidget();
    _itemWidget = w;
    layout()->addWidget(_itemWidget);
    _itemWidget->installEventFilter(this);
}

/*******************************************************************************
* SideBarWidget - Protected
*******************************************************************************/
bool SideBarWidget::eventFilter(QObject *obj, QEvent *event)
{
    if (obj != _itemWidget || event->type() != QEvent::Resize)
        return false;
//    adjustSize();
    return false;
}

/*******************************************************************************
* SideBarContentWidget
*******************************************************************************/
SideBarContentWidget::SideBarContentWidget(QWidget *parent)
    : StyledWidget(parent)
{
}

SideBarWidget * SideBarContentWidget::sideBarWidget()
{
    SideBarWidget * sbWidget = 0;
    if (parent())
        sbWidget = qobject_cast<SideBarWidget *>(parent());
    return sbWidget;
}

SideBar * SideBarContentWidget::parentSideBar()
{
    SideBarWidget * sbWidget = sideBarWidget();
    if (sbWidget)
        return sbWidget->parentSideBar();
    return 0;
}

QWidget * SideBarContentWidget::sideBarParent()
{
    SideBar * sideBar = parentSideBar();
    if (sideBar)
        return sideBar->parentWidget();
    return 0;
}

/*******************************************************************************
* SideBarSimpleWidget
*******************************************************************************/
SideBarSimpleWidget::SideBarSimpleWidget(Qt::Orientation layoutOrientation, QWidget *parent)
    : SideBarContentWidget(parent)
{
    _initLayout(layoutOrientation);
}

void SideBarSimpleWidget::_initChildren(QList<QWidget *> children)
{
    foreach(QWidget * child, children)
        layout()->addWidget(child);
}

void SideBarSimpleWidget::_initLayout(Qt::Orientation layoutOrientation)
{
    QLayout * widgetLayout = 0;
    if (layoutOrientation == Qt::Horizontal)
        widgetLayout = new QHBoxLayout(this);
    else
        widgetLayout = new QVBoxLayout(this);
    widgetLayout->setSpacing(0);
    widgetLayout->setContentsMargins(0,0,0,0);
    setLayout(widgetLayout);
}

/*******************************************************************************
* SideBarTreeWidget
*******************************************************************************/
SideBarTreeWidget::SideBarTreeWidget(QWidget *parent)
    : SideBarSimpleWidget(Qt::Vertical, parent)
    , _fullyCollapsable(true)
    , _treeView(new QTreeView(this))
{
    QList<QWidget *> children;
    _treeView->setFrameShape(QFrame::NoFrame);
    _treeView->setContextMenuPolicy(Qt::CustomContextMenu);
    _treeView->setRootIsDecorated(false);
    _treeView->header()->setVisible(false);
    children.append(_treeView);
    _initChildren(children);

    connect(_treeView, SIGNAL(clicked(QModelIndex)), this, SLOT(_viewClickedSlot(QModelIndex)));
    connect(_treeView, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(_viewDoubleClickedSlot(QModelIndex)));
    connect(_treeView, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(_viewContextMenuRequestedSlot(QPoint)));
    connect(_treeView, SIGNAL(collapsed(QModelIndex)), this, SLOT(_collapsedSlot(QModelIndex)));
    connect(_treeView, SIGNAL(expanded(QModelIndex)), this, SLOT(_expandedSlot(QModelIndex)));
    _treeView->installEventFilter(this);
}

SideBarTreeWidget::~SideBarTreeWidget()
{
    delete _treeView;
}

void SideBarTreeWidget::setViewModel(QAbstractItemModel *model)
{
    QAbstractItemModel *currentModel = _treeView->model();
    if (currentModel)
        disconnect(currentModel);
    _treeView->setModel(model);
    if (model)
        connect(model, SIGNAL(modelReset()), this, SLOT(_modelResetSlot()));
}

void SideBarTreeWidget::clearModel()
{
    QAbstractItemModel *currentModel = _treeView->model();
    if (currentModel)
        disconnect(currentModel);
    _treeView->setModel(0);
}

void SideBarTreeWidget::setFullyCollapsable(bool collapsable)
{
    _fullyCollapsable = collapsable;
    if (!_fullyCollapsable)
        _treeView->expandToDepth(1);
}

void SideBarTreeWidget::focusOnIndex(QModelIndex index, bool selectIndex)
{
    // Expand parent and select added item
    if (index.isValid() && index.parent().isValid())
        _treeView->expand(index.parent());
    if (selectIndex)
        _treeView->selectionModel()->setCurrentIndex(index, QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows);
}

/*******************************************************************************
* SideBarTreeWidget - Protected slots
*/
void SideBarTreeWidget::_viewClickedSlot(const QModelIndex &index)
{
    _indexClicked(index);
}

void SideBarTreeWidget::_viewDoubleClickedSlot(const QModelIndex &index)
{
    _indexDoubleClicked(index);
}

void SideBarTreeWidget::_viewContextMenuRequestedSlot(QPoint position)
{
    _contextMenuRequested(_treeView->indexAt(position), position);
}

void SideBarTreeWidget::_collapsedSlot(QModelIndex index)
{
    // block collapse for root node
    if (!_fullyCollapsable && !index.parent().isValid())
        _treeView->expand(index);
    _treeView->resizeColumnToContents(0);
}

void SideBarTreeWidget::_expandedSlot(QModelIndex)
{
    _treeView->resizeColumnToContents(0);
}

void SideBarTreeWidget::_modelResetSlot()
{
    if (!_fullyCollapsable)
        _treeView->expandToDepth(1);
}

/*******************************************************************************
* SideBarTreeWidget - Protected
*/
bool SideBarTreeWidget::eventFilter(QObject *obj, QEvent *event)
{
    if (obj == _treeView && event->type() == QEvent::KeyRelease) {
        _refreshData((QKeyEvent *)event);
    }
    return SideBarSimpleWidget::eventFilter(obj, event);
}

void SideBarTreeWidget::_refreshData(QKeyEvent * event)
{
    switch (event->key()) {
    case Qt::Key_Home:
    case Qt::Key_End:
    case Qt::Key_Left:
    case Qt::Key_Up:
    case Qt::Key_Right:
    case Qt::Key_Down:
    case Qt::Key_PageUp:
    case Qt::Key_PageDown: {
        emit  _viewClickedSlot(_currentIndex());
        break;
    }
    }
}

QTreeView * SideBarTreeWidget::_view() const
{
    return _treeView;
}

QModelIndex SideBarTreeWidget::_currentIndex() const
{
    return _treeView->selectionModel()->currentIndex();
}

QPoint SideBarTreeWidget::_globalPosition(QPoint viewportPosition) const
{
    return _treeView->viewport()->mapToGlobal(viewportPosition);
}

void SideBarTreeWidget::_setIconSize(QSize size)
{
    _treeView->setIconSize(size);
}

void SideBarTreeWidget::_initCloseDelegate(int column)
{
    CloseButtonDelegate * delegate = new CloseButtonDelegate(column, this);
    _treeView->setItemDelegate(delegate);
    if (column == _treeView->model()->columnCount() - 1) {
        _treeView->header()->setStretchLastSection(false);
        _treeView->header()->setResizeMode(column - 1, QHeaderView::Stretch);
    }
    _treeView->header()->setResizeMode(column, QHeaderView::Fixed);
    _treeView->header()->resizeSection(column, 16); // TODO : should use a constant
    _treeView->viewport()->setAttribute(Qt::WA_Hover);
}

/*******************************************************************************
* SideBarListWidget
*******************************************************************************/
SideBarListWidget::SideBarListWidget(QWidget *parent)
    : SideBarSimpleWidget(Qt::Vertical, parent)
    , _listView(new QListView(this))
{
    QList<QWidget *> children;
    _listView->setContextMenuPolicy(Qt::CustomContextMenu);
    _listView->setFrameShape(QFrame::NoFrame);
    children.append(_listView);
    _initChildren(children);

    connect(_listView, SIGNAL(clicked(QModelIndex)), this, SLOT(_viewClickedSlot(QModelIndex)));
    connect(_listView, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(_viewDoubleClickedSlot(QModelIndex)));
    connect(_listView, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(_viewContextMenuRequestedSlot(QPoint)));
    connect(_listView, SIGNAL(activated(QModelIndex)), this, SLOT(_viewActivatedSlot(QModelIndex)));
    _listView->installEventFilter(this);
}

SideBarListWidget::~SideBarListWidget()
{
    delete _listView;
}

void SideBarListWidget::setViewModel(QAbstractItemModel *model)
{
    _listView->setModel(model);
}

void SideBarListWidget::clearModel()
{
    _listView->setModel(0);
}

/*******************************************************************************
* SideBarListWidget - Protected slots
*/
void SideBarListWidget::_activateViewIndexSlot(const QModelIndex &index)
{
    if (index.isValid() && index.model() == _listView->model()) {
        _listView->selectionModel()->setCurrentIndex(index, QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows);
        _viewClickedSlot(index);
    }
}

void SideBarListWidget::_viewClickedSlot(const QModelIndex &index)
{
    _indexClicked(index);
}

void SideBarListWidget::_viewDoubleClickedSlot(const QModelIndex &index)
{
    _indexDoubleClicked(index);
}

void SideBarListWidget::_viewContextMenuRequestedSlot(QPoint position)
{
    _contextMenuRequested(_listView->indexAt(position), position);
}

void SideBarListWidget::_viewActivatedSlot(const QModelIndex &index)
{
    _indexActivated(index);
}

/*******************************************************************************
* SideBarListWidget - Protected
*/
bool SideBarListWidget::eventFilter(QObject *obj, QEvent *event)
{
    if (obj == _listView && event->type() == QEvent::KeyRelease) {
        _refreshData((QKeyEvent *)event);
    }
    return SideBarSimpleWidget::eventFilter(obj, event);
}

void SideBarListWidget::_refreshData(QKeyEvent * event)
{
    switch (event->key()) {
    case Qt::Key_Home:
    case Qt::Key_End:
    case Qt::Key_Left:
    case Qt::Key_Up:
    case Qt::Key_Right:
    case Qt::Key_Down:
    case Qt::Key_PageUp:
    case Qt::Key_PageDown: {
        emit  _viewClickedSlot(_currentIndex());
        break;
    }
    }
}

QListView * SideBarListWidget::_view() const
{
    return _listView;
}

QModelIndex SideBarListWidget::_currentIndex() const
{
    return _listView->selectionModel()->currentIndex();
}

QPoint SideBarListWidget::_globalPosition(QPoint viewportPosition) const
{
    return _listView->viewport()->mapToGlobal(viewportPosition);
}

void SideBarListWidget::_setIconSize(QSize size)
{
    _listView->setIconSize(size);
}

void SideBarListWidget::_indexActivated(const QModelIndex &index)
{
    _indexClicked(index);
}

} // namespace Hnv
