/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNV_SIDEBAR_H
#define HNV_SIDEBAR_H

// Project headers
#include "hnvwidgets.h"

// Qt headers
#include <QSplitter>

class QTreeView;
class QListView;

namespace Hnv {

class SideBarToolBar;
class SideBarWidget;
class HyneviewSession;

/*******************************************************************************
* SideBarItem
*******************************************************************************/
class SideBarItem
{
public:
    explicit SideBarItem(QString id, QString title, QMetaObject metaObject, int stretchFactor = 0);
    QString itemID();
    QString itemTitle();
    QMetaObject itemMetaObject();
    QWidget * createWidget(QWidget *parent);
    int stretchFactor();

private:
    QString _widgetID;
    QString _widgetTitle;
    QMetaObject _metaObject;
    int _stretchFactor;
};

/*******************************************************************************
* SideBar
*******************************************************************************/
class SideBar : public QSplitter
{
    Q_OBJECT
public:
    explicit SideBar(QString sideBarID, QString defaultWidgetID, QList<SideBarItem *> itemList, QWidget *parent = 0);
    virtual ~SideBar();
    QHash<QString, QString> itemIdsMap();
    QList<SideBarItem *> itemList();
    QWidget * getItemWidget(QString widgetId, int &stretchFactor, SideBarWidget *parent);
    void toggleDisplay();
    void displayWidget(QString widgetId);
    void loadSettings();
    void saveSettings();
    void restoreDefaultSettings();

signals:
    void hidden();

protected:
    virtual void paintEvent(QPaintEvent *);
    void _initSideBar();

private slots:
    void _splitSideBar();
    void _closeSideBarWidget();

private:
    void _insertSideBarWidget(int position, QString widgetId = "");

    QString _sideBarID;
    QString _defaultWidgetID;
    SideBarWidget * _createWidget(QString widgetId);
    QList<SideBarWidget *> _widgetList;
    QList<SideBarItem *> _itemList;
    QHash<QString, QString> _itemIdsHash;
    QStringList _openedWidgets;
};

/*******************************************************************************
* SideBarWidget
*******************************************************************************/
class SideBarWidget : public StyledWidget
{
    Q_OBJECT
public:
    explicit SideBarWidget(SideBar *sideBar, QString widgetId = "", QWidget *parent = 0);
    virtual ~SideBarWidget();
    void removeCurrentWidget();
    QString widgetId();
    int stretchFactor();

    SideBar * parentSideBar();
    QWidget * sideBarParent();

public slots:
    void setCurrentWidget(QVariant widgetId);

signals:
    void splitWidget();
    void closeWidget();

protected:
    bool eventFilter(QObject *obj, QEvent *event);

private:
    SideBar * _sideBar;
    QWidget * _itemWidget;
    SideBarToolBar * _toolBar;
    QString _widgetId;
    int _stretchFactor;
};

/*******************************************************************************
* SideBarContentWidget
*******************************************************************************/
class SideBarContentWidget : public StyledWidget
{
    Q_OBJECT
public:
    explicit SideBarContentWidget(QWidget * parent = 0);
    SideBarWidget * sideBarWidget();
    SideBar * parentSideBar();
    QWidget * sideBarParent();
};

/*******************************************************************************
* SideBarSimpleWidget
*******************************************************************************/
class SideBarSimpleWidget : public SideBarContentWidget
{
    Q_OBJECT
public:
    explicit SideBarSimpleWidget(Qt::Orientation layoutOrientation, QWidget * parent = 0);
protected:
    virtual void _initChildren(QList<QWidget *> children);

private:
    void _initLayout(Qt::Orientation layoutOrientation);
};

/*******************************************************************************
* SideBarTreeWidget
*******************************************************************************/
class SideBarTreeWidget : public SideBarSimpleWidget
{
    Q_OBJECT
public:
    explicit SideBarTreeWidget(QWidget *parent = 0);
    ~SideBarTreeWidget();

    void setViewModel(QAbstractItemModel * model);
    void clearModel();
    void setFullyCollapsable(bool collapsable);

public slots:
    void focusOnIndex(QModelIndex index, bool selectIndex = false);

protected slots:
    void _viewClickedSlot(const QModelIndex &index);
    void _viewDoubleClickedSlot(const QModelIndex &index);
    void _viewContextMenuRequestedSlot(QPoint position);
    virtual void _collapsedSlot(QModelIndex index);
    virtual void _expandedSlot(QModelIndex index);
    void _modelResetSlot();

protected:
    bool eventFilter(QObject *obj, QEvent *event);
    QTreeView * _view() const;
    QModelIndex _currentIndex() const;
    QPoint _globalPosition(QPoint viewportPosition) const;
    void _setIconSize(QSize size);
    void _initCloseDelegate(int column);
    virtual void _indexClicked(const QModelIndex &index) = 0;
    virtual void _indexDoubleClicked(const QModelIndex &index) = 0;
    virtual void _contextMenuRequested(const QModelIndex &index, const QPoint &position) = 0;
    void _refreshData(QKeyEvent * event);
    bool _fullyCollapsable;

    QTreeView * _treeView;
};

/*******************************************************************************
* SideBarListWidget
*******************************************************************************/
class SideBarListWidget : public SideBarSimpleWidget
{
    Q_OBJECT
public:
    explicit SideBarListWidget(QWidget *parent = 0);
    ~SideBarListWidget();

    void setViewModel(QAbstractItemModel * model);
    void clearModel();

protected slots:
    void _activateViewIndexSlot(const QModelIndex &index);
    void _viewClickedSlot(const QModelIndex &index);
    void _viewDoubleClickedSlot(const QModelIndex &index);
    void _viewContextMenuRequestedSlot(QPoint position);
    void _viewActivatedSlot(const QModelIndex &index);

protected:
    bool eventFilter(QObject *obj, QEvent *event);
    QListView * _view() const;
    QModelIndex _currentIndex() const;
    QPoint _globalPosition(QPoint viewportPosition) const;
    void _setIconSize(QSize size);
    virtual void _indexClicked(const QModelIndex &index) = 0;
    virtual void _indexDoubleClicked(const QModelIndex &index) = 0;
    virtual void _contextMenuRequested(const QModelIndex &index, const QPoint &position) = 0;
    virtual void _indexActivated(const QModelIndex &index);
    void _refreshData(QKeyEvent * event);

    QListView * _listView;
};

} // namespace Hnv

#endif // HNV_SIDEBAR_H
