/******************************************************************************
**
** Copyright (C) 2003-2013 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "snapshotmanager.h"
#include "ui_snapshotmanager.h"
#include "snapshottreemodel.h"
#include "snapshottreemodelitem.h"
#include "snapshotcreationdialog.h"
#include "filterlineedit.h"
#include "hnvwidgets.h"

// Qt headers
#include <QDebug>
#include <QMessageBox>

namespace Hnv {

SnapshotManager::SnapshotManager(HnsClient::DomainEntity *entity, QWidget *parent)
    : QDialog(parent)
    , ui(new Ui::SnapshotManager)
    , _entity(entity)
{
    ui->setupUi(this);

    setWindowTitle(tr("%1 snapshot").arg(_entity->getShortName()));

    ui->toolBar->setCloseable(false);
    ui->toolBar->addAction(ui->actionCreate_snapshot);
    ui->actionCreate_snapshot->setEnabled(true);
    ui->toolBar->addAction(ui->actionRevert_to_snapshot);
    ui->actionRevert_to_snapshot->setEnabled(false);
    ui->toolBar->addAction(ui->actionDelete_snapshot);
    ui->actionDelete_snapshot->setEnabled(false);
    ui->toolBar->addAction(ui->actionDetails);

    QWidget *spacer = new QWidget(this);
    spacer->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
    ui->toolBar->addWidget(spacer);

//    QToolButton *filterButton = new QToolButton(this);
//    filterButton->setAutoRaise(true);
//    filterButton->setIcon(QIcon(Ressources::themedPath(Constants::ICON_FILTER)));
//    filterButton->setPopupMode(QToolButton::InstantPopup);
//    ui->toolBar->addWidget(filterButton);

    FilterLineEdit *filterLineEdit = new FilterLineEdit(this); //TODO: connect this object
    filterLineEdit->setMinimumSize(300, 0);
    filterLineEdit->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
    ui->toolBar->addWidget(filterLineEdit);
    ui->details->setVisible(ui->actionDetails->isChecked());
    ui->progressBar->setHidden(true);

    connect(ui->actionDetails, SIGNAL(toggled(bool)), this, SLOT(_detailsIsVisible(bool)));
    connect(ui->treeView, SIGNAL(collapsed(QModelIndex)), this, SLOT(_resizeColumns()));
    connect(ui->treeView, SIGNAL(expanded(QModelIndex)), this , SLOT(_resizeColumns()));

    _model = new SnapshotTreeModel(SnapshotTreeModelItem::COLUMN_COUNT, this);
    ui->treeView->setModel(_model);
    _populateModel();

    connect(ui->treeView->selectionModel(), SIGNAL(selectionChanged(QItemSelection, QItemSelection)), this, SLOT(_selectedSnapshotChangedSlot()));
    connect(_entity, SIGNAL(currentSnapshotChanged(::Hns::Snapshot)), this, SLOT(_currentSnapshotChangedSlot(::Hns::Snapshot)));
}

SnapshotManager::~SnapshotManager()
{
    delete ui;
}

void SnapshotManager::_populateModel()
{
    QString currentSnapshotName = _entity->currentSnapshotName();
    foreach(Hns::Snapshot snapshot, _entity->snapshotList()) {
        SnapshotTreeModelItem *item = new SnapshotTreeModelItem(snapshot, false, SnapshotTreeModelItem::COLUMN_COUNT);
        _items.insert(QString::fromStdString(snapshot.name), item);
    }
    foreach(Hns::Snapshot snapshot, _entity->snapshotList()) {
        SnapshotTreeModelItem *item = _items.value(QString::fromStdString(snapshot.name));
        _model->add(_items.value(QString::fromStdString(snapshot.parent), 0), item);
        if (QString::fromStdString(snapshot.name) == currentSnapshotName) {
            _model->setCurrentSnapshot(item);
            _updateIndex(_model->indexFromNode(item));
            _updateButtons(_model->indexFromNode(item));
        }
    }
}

void SnapshotManager::on_actionCreate_snapshot_triggered()
{
    ui->warningLabel->setText("");
    ui->warningLabel->setVisible(false);
    SnapshotCreationDialog popup(_entity, this, SLOT(_snapshotCreatedSlot(bool,QString)));
    HnvDialogHandler *popupHandler = new HnvDialogHandler(_entity, &popup, this);
    popupHandler->displayMessageOnDestroy(false);
    connect(&popup, SIGNAL(dialogRejected(bool)), this, SLOT(_enableActions(bool)));
    popup.exec();
}

void SnapshotManager::on_actionRevert_to_snapshot_triggered()
{
    ui->warningLabel->setText("");
    ui->warningLabel->setVisible(false);
    _enableActions(false);
    QModelIndex index = ui->treeView->currentIndex();
    SnapshotTreeModelItem *item = qobject_cast<SnapshotTreeModelItem *>(_model->itemFromIndex(index));
    if (item) {
        if (QString::fromStdString(item->getSnapshot().name) == _entity->currentSnapshotName()) {
            qCritical() << tr("This is already the actual snapshot, you can't revert it to itself");
            _enableActions(true);
            return;
        }
        _entity->revertSnapshot(QString::fromStdString(item->getSnapshot().name), this, SLOT(_snapshotRevertedSlot(bool,QString)));
    }
}

void SnapshotManager::on_actionDelete_snapshot_triggered()
{
    ui->warningLabel->setText("");
    ui->warningLabel->setVisible(false);
    _enableActions(false);
    QModelIndex index = ui->treeView->currentIndex();
    SnapshotTreeModelItem *item = qobject_cast<SnapshotTreeModelItem *>(_model->itemFromIndex(index));
    if (item) {
        /*Workaround
         *We should not block the "Delete when there is children" in the GUI,
         *but treat the error sent by the node
        */
        if (item->childCount() >= 1) {
            ui->warningLabel->setText(tr("Operation forbidden: Can not delete a snapshot with children."));
            ui->warningLabel->setStyleSheet("QLabel{color:red;}");
            ui->warningLabel->setVisible(true);
            qCritical() << tr("Operation forbidden: Can not delete a snapshot with children.");
            _enableActions(true);
            return;
        }
        _entity->deleteSnapshot(QString::fromStdString(item->getSnapshot().name),this, SLOT(_snapshotDeletedSlot(bool,QString)));
    }
}

void SnapshotManager::_detailsIsVisible(bool visible)
{
    ui->details->setVisible(visible);
    SnapshotTreeModelItem *item = qobject_cast<SnapshotTreeModelItem *>(_model->itemFromIndex(ui->treeView->selectionModel()->currentIndex()));
    if (item) {
        ui->details->setName(item->getSnapshot().name.c_str());
        ui->details->setDescription(item->getSnapshot().description.c_str());
    }
}

void SnapshotManager::_selectedSnapshotChangedSlot()
{
    QModelIndex index = ui->treeView->selectionModel()->currentIndex();
    if (index.isValid()) {
        SnapshotTreeModelItem *item = qobject_cast<SnapshotTreeModelItem *>(_model->itemFromIndex(index));
        if (ui->details->isVisible()) {
            ui->details->setName(item->data(SnapshotTreeModelItem::COLUMN_NAME, Qt::DisplayRole).toString());
            ui->details->setDescription(item->data(SnapshotTreeModelItem::COLUMN_DESCRIPTION, Qt::DisplayRole).toString());
        }
        _updateButtons(index);
    }
}

void SnapshotManager::_currentSnapshotChangedSlot(::Hns::Snapshot snapshot)
{
    QString snapshotName = QString::fromStdString(snapshot.name);
    if (!snapshotName.isEmpty()) {
        SnapshotTreeModelItem *currentSnapshot = _items.value(snapshotName, 0);
        if (!currentSnapshot) {
            currentSnapshot = new SnapshotTreeModelItem(snapshot, false, SnapshotTreeModelItem::COLUMN_COUNT);
            SnapshotTreeModelItem *parent = _items.value(QString::fromStdString(snapshot.parent), 0);
            _model->add(parent, currentSnapshot);
            _items.insert(snapshotName, currentSnapshot);
        }
        if (currentSnapshot != _model->getCurrentSnapshot())
            _model->setCurrentSnapshot(currentSnapshot);
        QModelIndex newIndex = _model->indexFromNode(currentSnapshot);
        _updateIndex(newIndex);
        _updateButtons(newIndex);
    } else {
        _model->setCurrentSnapshot(NULL);
        _updateButtons(ui->treeView->selectionModel()->currentIndex());
    }
    _enableActions(true);
}

void SnapshotManager::_updateIndex(QModelIndex index)
{
    ui->treeView->setCurrentIndex(index);
    ui->treeView->selectionModel()->setCurrentIndex(index, QItemSelectionModel::Rows);
    _resizeColumns();
}

void SnapshotManager::_updateButtons(QModelIndex index)
{
    if (index.isValid()) {
        SnapshotTreeModelItem *item = qobject_cast<SnapshotTreeModelItem *>(_model->itemFromIndex(index));
        ui->actionDelete_snapshot->setEnabled(true);
        if (item == _model->getCurrentSnapshot()) {
            ui->actionCreate_snapshot->setEnabled(true);
            ui->actionRevert_to_snapshot->setEnabled(false);
        } else {
            ui->actionCreate_snapshot->setEnabled(false);
            ui->actionRevert_to_snapshot->setEnabled(true);
        }
    } else {
        ui->actionCreate_snapshot->setEnabled(true);
        ui->actionRevert_to_snapshot->setEnabled(false);
        ui->actionDelete_snapshot->setEnabled(false);
    }
    if (_model->getCurrentSnapshot() == NULL) {
        ui->actionCreate_snapshot->setEnabled(true);
    }
    ui->treeView->update(index);
}

void SnapshotManager::_enableActions(bool enabled)
{
    ui->progressBar->setHidden(enabled);
    this->setEnabled(enabled);
}

void SnapshotManager::_resizeColumns()
{
    ui->treeView->resizeColumnToContents(SnapshotTreeModelItem::COLUMN_NAME);
    ui->treeView->resizeColumnToContents(SnapshotTreeModelItem::COLUMN_CURRENT_DATE);
}

void SnapshotManager::_snapshotDeletedSlot(bool result, const QString &snapshotName)
{
    if (!result) {
        _enableActions(true); //don't break the previous behaviour
        QMessageBox::critical(this, tr("Error"), tr("Unable to delete snapshot %1").arg(snapshotName), QMessageBox::Ok, QMessageBox::Ok);
        return;
    }
    SnapshotTreeModelItem * item = _items.value(snapshotName);
    _model->removeItem(item);
    _items.remove(QString::fromStdString(item->getSnapshot().name));
}

void SnapshotManager::_snapshotCreatedSlot(bool result, const QString &snapshotName)
{
    if (!result) {
        QMessageBox::critical(this, tr("Error"), tr("Unable to create snapshot %1").arg(snapshotName), QMessageBox::Ok, QMessageBox::Ok);
        _enableActions(true); //don't break the previous behaviour
    }
}

void SnapshotManager::_snapshotRevertedSlot(bool result, const QString &snapshotName)
{
    if (!result) {
        QMessageBox::critical(this, tr("Error"), tr("Unable to revert to snapshot %1").arg(snapshotName), QMessageBox::Ok, QMessageBox::Ok);
        _enableActions(true); //don't break the previous behaviour
    }
}

} // namespace Hnv
