/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of HYNESIM.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "entitiesmodel.h"
#include "topologyadministrator.h"

// hynesim hnsclient headers
#include <libhnsclient/entry.h>
#include <libhnsclient/topologyentrycatalog.h>

// Qt headers
#include <QApplication>
#include <QMouseEvent>

namespace Hnv {

EntitiesModel::EntitiesModel(TopologyAdministrator *administrator, QObject *parent)
    : QAbstractTableModel(parent)
    , _administrator(administrator)
    , _id(-1)
{
}

EntitiesModel::~EntitiesModel()
{
}

int EntitiesModel::rowCount(const QModelIndex &parent) const
{
    if (!parent.isValid())
        return _valuesToShow.size();
    return 0;
}
int EntitiesModel::columnCount(const QModelIndex &parent) const
{
    if (!parent.isValid())
        return COLUMN_SIZE;
    return 0;
}

QVariant EntitiesModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if (role == Qt::DisplayRole && orientation == Qt::Horizontal) {
        switch (section) {
        case COLUMN_NAME:
            return tr("Name");
        case COLUMN_CHECKBOX:
            return tr("Compress");
        }
    }
    return QAbstractTableModel::headerData(section, orientation, role);
}

QVariant EntitiesModel::data(const QModelIndex &index, int role) const
{
    if (index.isValid()) {
        int row = index.row();
        if (row < _valuesToShow.size()) {
            EntitiesLocalInformation entity = _valuesToShow.at(row);
            switch (role) {
            case Qt::DisplayRole:
                switch (index.column()) {
                case COLUMN_NAME:
                    return entity.name;
                case COLUMN_CHECKBOX:
                    return entity.state;
                }
            case Qt::ToolTipRole:
                return QString(entity.name);
            }
        }
    }
    return QVariant();
}

bool EntitiesModel::setData(const QModelIndex &index, const QVariant &value, int role)
{
    if (index.isValid()) {
        int row = index.row();
        if (row < _valuesToShow.size()) {
            switch (role) {
            case Qt::EditRole:
                _valuesToShow[row].state = value.toBool();
                emit dataChanged(index, index);
                return true;
            }
        }
    }
    return false;
}

void EntitiesModel::topologyChange(int id)
{
    if (id == _id)
        return;
    _id = id;
    HnsClient::TopologyEntry *topologyEntry = _administrator->catalog()->topologyEntryByID(id);
    _populate(topologyEntry);
}

void EntitiesModel::_populate(HnsClient::TopologyEntry *topologyEntry, bool update)
{
    beginResetModel();
    _valuesToShow.clear();
    if (topologyEntry) {
        QHash<QString, bool> previousState;
        if (update) {
            foreach(const EntitiesLocalInformation &info, _valuesToShow)
                previousState.insert(info.uuid, info.state);
        }
        QMap<Ice::Int, Hns::EntityEntryPtr> map = QMap<Ice::Int, Hns::EntityEntryPtr>(topologyEntry->topologyEntry()->entitiesMap);
        QMapIterator<Ice::Int, Hns::EntityEntryPtr> iterator(map);
        while (iterator.hasNext()) {
            iterator.next();
            Hns::EntityEntryPtr entityEntry = iterator.value();
            EntitiesLocalInformation entityInformation;
            entityInformation.name = QString::fromStdString(entityEntry->shortName);
            entityInformation.uuid = QString::fromStdString(entityEntry->uuid);
            bool state = false;
            if (update)
                state = previousState.value(entityInformation.uuid, false);
            entityInformation.state = state;
            _valuesToShow.append(entityInformation);
        }
    }
    endResetModel();
}

void EntitiesModel::_refresh(HnsClient::TopologyEntry *topologyEntry)
{
    _populate(topologyEntry, true);
}

void EntitiesModel::_topologyEntryUpdatedSlot(HnsClient::TopologyEntry *entry)
{
    if (entry->getID() == _id)
        _refresh(entry);
}

bool EntitiesModel::getStateByUuid(QString uuid)
{
    EntitiesLocalInformation info;
    info.uuid = uuid;
    foreach (EntitiesLocalInformation information, _valuesToShow) {
        if (info.uuid == information.uuid)
            return information.state;
    }
    return false;
}


/************************************DELEGATE*************************************/



static QRect CheckBoxRect(const QStyleOptionViewItem &view_item_style_options) {
    QStyleOptionButton check_box_style_option;
    QRect check_box_rect = QApplication::style()->subElementRect(QStyle::SE_CheckBoxIndicator, &check_box_style_option);
    QPoint check_box_point(view_item_style_options.rect.x() +
                           view_item_style_options.rect.width() / 2 -
                           check_box_rect.width() / 2,
                           view_item_style_options.rect.y() +
                           view_item_style_options.rect.height() / 2 -
                           check_box_rect.height() / 2);
    return QRect(check_box_point, check_box_rect.size());
}

CheckBoxDelegate::CheckBoxDelegate(QObject *parent)
    : QStyledItemDelegate(parent)
{

}

CheckBoxDelegate::~CheckBoxDelegate()
{

}

void CheckBoxDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    if (index.column() != 1) {
        QStyledItemDelegate::paint(painter, option, index);
        return;
    }
    bool checked = index.model()->data(index, Qt::DisplayRole).toBool();
    QStyleOptionButton check_box_style_option;
    check_box_style_option.state |= QStyle::State_Enabled;
    if (checked)
        check_box_style_option.state |= QStyle::State_On;

    else
        check_box_style_option.state |= QStyle::State_Off;

    check_box_style_option.rect = CheckBoxRect(option);

    QApplication::style()->drawControl(QStyle::CE_CheckBox, &check_box_style_option,painter);
}

// This is essentially copied from QStyledItemEditor, except that we
// have to determine our own "hot zone" for the mouse click.
bool CheckBoxDelegate::editorEvent(QEvent *event, QAbstractItemModel *model,const QStyleOptionViewItem &option,const QModelIndex &index)
{
    if ((event->type() == QEvent::MouseButtonRelease) || (event->type() == QEvent::MouseButtonDblClick)) {
        QMouseEvent *mouse_event = static_cast<QMouseEvent*>(event);
        if (mouse_event->button() != Qt::LeftButton || !CheckBoxRect(option).contains(mouse_event->pos()))
            return false;

        if (event->type() == QEvent::MouseButtonDblClick)
            return true;
    }
    else if (event->type() == QEvent::KeyPress) {
        if (static_cast<QKeyEvent*>(event)->key() != Qt::Key_Space && static_cast<QKeyEvent*>(event)->key() != Qt::Key_Select)
            return false;
    }
    else
        return false;

    bool checked = index.model()->data(index, Qt::DisplayRole).toBool();
    return model->setData(index, !checked, Qt::EditRole);
}

} // namespace Hnv
