/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "topologydialog.h"
#include "ui_topologydialog.h"
#include "topologyexportwidget.h"
#include "hyneviewsession.h"
#include "ressources.h"
#include "topologycatalogmodel.h"
#include "multicolumnstoflatproxymodel.h"
#include "treesortfilterproxymodel.h"
#include "uniqueproxymodel.h"
#include "topologyadministrator.h"
#include "hnv_entryproperties.h"

// hynesim hnsclient headers
#include <libhnsclient/entry.h>
#include <libhnsclient/usermanager.h>
#include <libhnsclient/topologyentrycatalog.h>

// Qt headers
#include <QCompleter>
#include <QMenu>
#include <QActionGroup>
#include <QMessageBox>
#include <QPushButton>
#include <QMouseEvent>
#include <QSortFilterProxyModel>


namespace Hnv {

/*******************************************************************************
* TopologyTableView
*******************************************************************************/
TopologyListView::TopologyListView(QWidget *parent)
    : QListView(parent)
{
}

TopologyListView::~TopologyListView()
{
}

void TopologyListView::mousePressEvent(QMouseEvent *event)
{
    QListView::mousePressEvent(event);
    QModelIndex index = indexAt(event->pos());
    if (!index.isValid())
        clearSelection();
}

/*******************************************************************************
* TopologyDialog
*******************************************************************************/
TopologyDialog::TopologyDialog(TopologyAdministrator *administrator, TopologyCatalogModel *model, QWidget *parent)
    : QDialog(parent)
    , _ui(new Ui::TopologyDialog)
    , _administrator(administrator)
    , _sourceModel(model)
{
    _ui->setupUi(this);

    _proxyModel = new TreeSortFilterProxyModel(this);
    _proxyModel->setSortCaseSensitivity(Qt::CaseInsensitive);
    _proxyModel->sort(0);
    _proxyModel->setDynamicSortFilter(true);
    _proxyModel->setFilterKeyColumn(0);
    QList<int> filterColumns;
    filterColumns << 0 << 6;
    _proxyModel->setFilterColumnsList(filterColumns);

    _proxyModel->setFilterCaseSensitivity(Qt::CaseInsensitive);
    _proxyModel->setSourceModel(_sourceModel);

    _proxyFilterModel = new QSortFilterProxyModel(this);
    _proxyFilterModel->setSourceModel(_proxyModel);
    _proxyFilterModel->setFilterRole(TopologyItem::IsModelRole);
    _proxyFilterModel->setDynamicSortFilter(true);
    _proxyFilterModel->setFilterKeyColumn(0);
    _proxyFilterModel->setFilterCaseSensitivity(Qt::CaseInsensitive);

    _ui->listView->setModel(_proxyFilterModel);
    _ui->listView->setCurrentIndex(_proxyFilterModel->index(0, 0));
    _ui->listView->setResizeMode(QListView::Adjust);

    QHash<int, QString> completionColumnsHash;
    completionColumnsHash.insert(0, QString("")); // First column, full data
    completionColumnsHash.insert(6, QString(",")); // Tag list column, with "," separator
    MultiColumnsToFlatProxyModel *completionModel = new MultiColumnsToFlatProxyModel(_sourceModel, completionColumnsHash, this);
    UniqueProxyModel *uniqueModel = new UniqueProxyModel(0, this);
    uniqueModel->setSourceModel(completionModel);
    QCompleter * catalogCompleter = new QCompleter(uniqueModel, this);
    catalogCompleter->setCompletionColumn(0);
    catalogCompleter->setCaseSensitivity(Qt::CaseInsensitive);
    catalogCompleter->setCompletionRole(Qt::DisplayRole);
    _ui->catalogFilterEdit->setCompleter(catalogCompleter);

    connect(_proxyModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)), this, SLOT(_sourceDataChanged(QModelIndex,QModelIndex)));
    connect(_ui->listView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)), this, SLOT(_listViewSelectionChangedSlot(QItemSelection,QItemSelection)));

    _ui->propertiesButton->setDefaultAction(_ui->action_Properties);
    _ui->deleteButton->setDefaultAction(_ui->action_Delete);
    _ui->unloadButton->setDefaultAction(_ui->action_Unload);
    _ui->toolButtonExport->setDefaultAction(_ui->actionSwitchExportDetails);

    _ui->action_Delete->setIcon(QIcon(Ressources::themedPath(Constants::ICON_DELETE)));
    _ui->action_Properties->setIcon(QIcon(Ressources::themedPath(Constants::ICON_PROPERTIES)));
    _ui->action_Unload->setIcon(QIcon(Ressources::themedPath(Constants::ICON_UNLOAD_TOPOLOGY)));
    _ui->action_Fix->setIcon(QIcon(Ressources::themedPath(Constants::ICON_FIX_TOPOLOGY)));
    _ui->actionSwitchExportDetails->setIcon(QIcon(Ressources::themedPath(Constants::ICON_EXPORT)));

    _ui->unloadButton->setVisible(true);
    _ui->action_Unload->setVisible(true);
    setFocus();

    TopologyItem *item = _getSelectedItem();
    _exportWidget = new TopologyExportWidget(_administrator, this);
    _ui->stackedWidgetDetailsExport->addWidget(_exportWidget);

    _updateCurrentItem(item);

    _ui->listView->setViewMode(QListView::IconMode);
    _ui->listView->setFlow(QListView::LeftToRight);
    _ui->listView->setGridSize(QSize(Constants::WIDTH_LIST_ICON_PREVIEW * 1.1,Constants::HEIGHT_LIST_ICON_PREVIEW * 1.2));
    _ui->listView->setIconSize(QSize(Constants::WIDTH_LIST_ICON_PREVIEW,Constants::HEIGHT_LIST_ICON_PREVIEW));
    _ui->listView->setWrapping(true);
    _ui->listView->setWordWrap(true);
    _ui->listView->addAction(_ui->action_Properties);
    _ui->listView->addAction(_ui->action_Unload);
    _ui->listView->addAction(_ui->action_Delete);
    _ui->listView->addAction(_ui->action_Fix);
    _ui->listView->addAction(_ui->actionSwitchExportDetails);

    QPair<QString, QString > pairModel;
    pairModel.first = tr("All");
    pairModel.second = "true|false";
    _filterModelList.append(pairModel);
    _ui->comboBox->addItem(pairModel.first);
    pairModel.first = tr("Topology");
    pairModel.second = "false";
    _filterModelList.append(pairModel);
    _ui->comboBox->addItem(pairModel.first);
    pairModel.first = tr("Model");
    pairModel.second = "true";
    _filterModelList.append(pairModel);
    _ui->comboBox->addItem(pairModel.first);

    QObject::connect(_administrator->catalog(), SIGNAL(updateACLFailed(QString)), this, SLOT(_updateACLFailedSlot(QString)));
}

TopologyDialog::~TopologyDialog()
{
    delete _ui;
    delete _proxyModel;
    delete _proxyFilterModel;
}

QList<HnsClient::TopologyEntry *> TopologyDialog::selectedEntries()
{
    return _selectedEntries;
}

/*******************************************************************************
* TopologyDialog - Private slots
*/
void TopologyDialog::on_action_Delete_triggered()
{
    QList<TopologyItem *> items = _getSelectedItems();
    if (items.isEmpty())
        return;

    HyneviewSession *session = HyneviewSession::instance();
    if (!session)
        return;

    QString message;
    if (items.size() == 1)
        message = tr("Are you sure that you want to delete this topology ?");
    else
        message = tr("Are you sure that you want to delete these topologies ?");
    QStringList connectedUsers;
    foreach (TopologyItem *item, items) {
        if (item->isLoaded()) {
            message = tr("One of the selected topology is currently used, are you sure you want to delete it ?");
            connectedUsers << tr("%1: %2").arg(item->getShortName()).arg(session->userManager()->getUsernamesFromIdList(item->connectedUsers()).join(","));
        }
    }
    QMessageBox msgBox(
                QMessageBox::Warning,
                tr("Delete Topology"),
                message,
                QMessageBox::Yes | QMessageBox::No,
                this);
    HnvDialogHandler * popupHandler = new HnvDialogHandler(0, &msgBox, this);
    foreach (TopologyItem *item, items)
        popupHandler->addSourceObject(item);

    popupHandler->setSourceDestroyedMessage(tr("One of the selected topology was deleted. Operation aborted."));
    msgBox.setDefaultButton(QMessageBox::No);

    if (!connectedUsers.isEmpty())
        msgBox.setDetailedText(tr("Connected users: \n %1").arg(connectedUsers.join("\n")));

    if (msgBox.exec() == QMessageBox::Yes) {
        foreach (TopologyItem *item, items)
            _administrator->deleteTopology(item->topologyEntry()->getUUID());
    }
}

void TopologyDialog::on_action_Properties_triggered()
{
    TopologyItem *item = _getSelectedItem();
    if (!item)
        return;
    HnsClient::TopologyEntry *updatedEntry = _administrator->duplicateEntryByUUID(item->topologyEntry()->getUUID(), this);
    if (updatedEntry) {
        bool readOnly = updatedEntry->isNew() || updatedEntry->getStatus() != Hns::TOPOLOGY_STATUS_OK;
        EntryProperties popup(updatedEntry, this, readOnly);
        HnvDialogHandler * popupHandler = new HnvDialogHandler(item, &popup, this);
        popupHandler->setSourceDestroyedMessage(tr("\"%1\" was deleted. Operation aborted.").arg(item->getShortName()));
        if (popup.exec() == QDialog::Accepted) {
            _administrator->updateEntry(updatedEntry);
        }
        updatedEntry->deleteLater();
    }
}

void TopologyDialog::on_action_Unload_triggered()
{
    QList<TopologyItem *> items = _getSelectedItems();
    if (items.isEmpty())
        return;

    HyneviewSession *session = HyneviewSession::instance();
    if (!session)
        return;

    QString message;
    if (items.size() == 1)
        message = tr("This topology is currently used, are you sure you want to unload it ?");
    else
        message = tr("These topologies are currently used, are you sure you want to unload them ?");

    QStringList connectedUsers;
    foreach (TopologyItem *item, items) {
        if (item->isNew())
            connectedUsers << tr("%1 (will be deleted): %2").arg(item->getShortName()).arg(session->userManager()->getUsernamesFromIdList(item->connectedUsers()).join(","));
        else
            connectedUsers << tr("%1: %2").arg(item->getShortName()).arg(session->userManager()->getUsernamesFromIdList(item->connectedUsers()).join(","));
    }

    QMessageBox msgBox(
                QMessageBox::Warning,
                tr("Unload Topology"),
                message,
                QMessageBox::Yes | QMessageBox::No,
                this);
    HnvDialogHandler *popupHandler = new HnvDialogHandler(0, &msgBox, this);
    foreach (TopologyItem *item, items)
        popupHandler->addSourceObject(item);
    popupHandler->setSourceDestroyedMessage(tr("One of the selected topology was deleted. Operation aborted."));
    msgBox.setDetailedText(tr("Connected users: \n %1").arg(connectedUsers.join("\n")));
    msgBox.setDefaultButton(QMessageBox::No);
    if (msgBox.exec() == QMessageBox::Yes) {
        foreach (TopologyItem *item, items)
            _administrator->unloadTopology(item->topologyEntry()->getUUID());
    }
}

void TopologyDialog::on_action_Fix_triggered()
{
    TopologyItem *item = _getSelectedItem();
    if (!item)
        return;
    HnsClient::TopologyEntry *entry = item->topologyEntry();
    QMessageBox msgBox(
                QMessageBox::Warning,
                tr("Fix Topology"),
                tr("Do you really want to fix this topology? \n\nWARNING: This operation can not be undone.\nAny missing entities will be removed from the topology permanently."),
                QMessageBox::Yes | QMessageBox::No,
                this);
    HnvDialogHandler * popupHandler = new HnvDialogHandler(item, &msgBox, this);
    popupHandler->setSourceDestroyedMessage(tr("\"%1\" has already been fixed. Operation aborted.").arg(item->getShortName()));
    msgBox.setDefaultButton(QMessageBox::No);
    if (msgBox.exec() == QMessageBox::Yes)
        _administrator->fixTopology(entry);
}

void TopologyDialog::on_catalogFilterEdit_textChanged(QString currentText)
{
    _proxyModel->setFilterFixedString(currentText);
}

void TopologyDialog::on_buttonBox_accepted()
{
    QList<TopologyItem *> items = _getSelectedItems();
    foreach (TopologyItem *item, items) {
        if (item->topologyEntry()->getStatus() == ::Hns::TOPOLOGY_STATUS_OK)
            _selectedEntries << item->topologyEntry();
    }
}

void TopologyDialog::on_listView_doubleClicked(QModelIndex index)
{
    TopologyItem *item = _getItemFromProxyIndex(index);
    if (item && item->topologyEntry()->getStatus() == ::Hns::TOPOLOGY_STATUS_OK) {
        _selectedEntries << item->topologyEntry();
        accept();
    }
}

void TopologyDialog::_listViewSelectionChangedSlot(const QItemSelection &selected, const QItemSelection &deselected)
{
    Q_UNUSED(selected)
    Q_UNUSED(deselected)
    int selectionSize = _ui->listView->selectionModel()->selectedIndexes().size();
    if (selectionSize == 0)
        _updateCurrentItem(0);
    else if (selectionSize > 1)
        _updateCurrentItems(_getSelectedItems());
    else
        _updateCurrentItem(_getSelectedItem());
}

void TopologyDialog::_sourceDataChanged(QModelIndex topLeft, QModelIndex bottomRight)
{
    Q_UNUSED(topLeft)
    Q_UNUSED(bottomRight)
    _updateCurrentItem(_getSelectedItem());
}

/*******************************************************************************
* TopologyDialog - Private
*/
void TopologyDialog::_updateCurrentItem(TopologyItem *item)
{
    bool canOpen        = false;
    bool canDelete      = false;
    bool canUnload      = false;
    bool canFix         = false;
    bool canExport      = false;
    qlonglong itemId    = 0;

    int topologyStatus = -1;

    if (item == 0) {
        _ui->detailsWidget->setDefaultsDetails();
        _ui->action_Properties->setEnabled(false);
    } else {
        itemId = item->getID();
        _ui->detailsWidget->setItem(item->topologyEntry(), item->topology());
        topologyStatus = item->topologyEntry()->getStatus();
        canOpen = topologyStatus == ::Hns::TOPOLOGY_STATUS_OK;
        canDelete = item->isWriteable() && (topologyStatus != Hns::TOPOLOGY_BUSY_EXPORTING);
        canUnload = item->isLoaded();
        // TODO: Check if every status of following list is valid to be fixed
        canFix = (topologyStatus == Hns::TOPOLOGY_LOADING_ERROR ||
                  topologyStatus == Hns::TOPOLOGY_CONTAINS_ENTITY_WITH_UUID_CONFLICT ||
                  topologyStatus == Hns::TOPOLOGY_UUID_CONFLICT);
        canExport = (topologyStatus == Hns::TOPOLOGY_STATUS_OK) && (!item->isLoaded());
        _ui->action_Properties->setEnabled(true);
    }
    _exportWidget->setTopology(itemId);
    _ui->buttonBox->button(QDialogButtonBox::Open)->setEnabled(canOpen);
    _ui->action_Delete->setEnabled(canDelete);
    _ui->action_Unload->setEnabled(canUnload);
    _ui->action_Fix->setEnabled(canFix);

    bool busyExporting = (topologyStatus == Hns::TOPOLOGY_BUSY_EXPORTING);
    _ui->actionSwitchExportDetails->setEnabled(canExport);

    if (busyExporting) {
        _ui->actionSwitchExportDetails->setIcon(QIcon(Ressources::themedPath(Constants::ICON_SHOW_TOPOLOGY_DETAILS)));
        _ui->stackedWidgetDetailsExport->setCurrentWidget(_exportWidget);
        _ui->actionSwitchExportDetails->setToolTip(tr("Details"));
        _ui->actionSwitchExportDetails->setText(tr("Details"));
    } else {
        _ui->actionSwitchExportDetails->setIcon(QIcon(Ressources::themedPath(Constants::ICON_EXPORT)));
        _ui->stackedWidgetDetailsExport->setCurrentWidget(_ui->stackedPageDetails);
        _ui->actionSwitchExportDetails->setToolTip(tr("Export"));
        _ui->actionSwitchExportDetails->setText(tr("Export"));
    }
}

void TopologyDialog::_updateCurrentItems(QList<TopologyItem *> items)
{
    bool canOpen = true;
    bool canDelete = true;
    bool canUnload = true;

    _ui->detailsWidget->setDefaultsDetails();
    _ui->action_Properties->setEnabled(false);
    _ui->action_Fix->setEnabled(false);
    _ui->actionSwitchExportDetails->setEnabled(false);

    foreach (TopologyItem *item, items) {
        if (item->topologyEntry()->getStatus() != ::Hns::TOPOLOGY_STATUS_OK)
            canOpen &= false;
        if (item->topologyEntry()->getStatus() == ::Hns::TOPOLOGY_BUSY_EXPORTING)
            canDelete &= false;
        if (!item->isLoaded())
            canUnload &= false;
    }
    _ui->buttonBox->button(QDialogButtonBox::Open)->setEnabled(canOpen);
    _ui->action_Delete->setEnabled(canDelete);
    _ui->action_Unload->setEnabled(canUnload);
}

TopologyItem * TopologyDialog::_getSelectedItem()
{
    QModelIndexList indexList = _ui->listView->selectionModel()->selectedIndexes();
    if (!indexList.isEmpty())
        return _getItemFromProxyIndex(indexList.first());
    return 0;
}

QList<TopologyItem *> TopologyDialog::_getSelectedItems()
{
    QList<TopologyItem *> ret;
    QModelIndexList indexList = _ui->listView->selectionModel()->selectedIndexes();
    foreach (QModelIndex index, indexList)
        ret.append(_getItemFromProxyIndex(index));
    return ret;
}

TopologyItem * TopologyDialog::_getItemFromProxyIndex(QModelIndex proxyIndex)
{
    if (proxyIndex.isValid())
        return _getItemFromIndex(_proxyModel->mapToSource(_proxyFilterModel->mapToSource(proxyIndex)));
    return 0;
}

TopologyItem * TopologyDialog::_getItemFromIndex(QModelIndex index)
{
    if (index.isValid()) {
        TopologyItem * item = static_cast<TopologyItem *>(index.internalPointer());
        if (item)
            return item;
    }
    return 0;
}

void TopologyDialog::on_comboBox_currentIndexChanged(const QString & ccbValue)
{
    for(int i = 0 ; i < _filterModelList.size() ; i++){
        if(_filterModelList.at(i).first == ccbValue){
            _proxyFilterModel->setFilterRegExp(QRegExp(_filterModelList.at(i).second));
            break;
        }
    }
}

void TopologyDialog::_updateACLFailedSlot(QString errorMessage)
{
    QMessageBox::critical(this, QObject::tr("Error"), errorMessage);
}

void TopologyDialog::on_actionSwitchExportDetails_triggered()
{
    if (_ui->stackedWidgetDetailsExport->currentWidget() == _ui->stackedPageDetails) {
        _ui->actionSwitchExportDetails->setIcon(QIcon(Ressources::themedPath(Constants::ICON_SHOW_TOPOLOGY_DETAILS)));
        _ui->stackedWidgetDetailsExport->setCurrentWidget(_exportWidget);
        _ui->actionSwitchExportDetails->setToolTip(tr("Details"));
        _ui->actionSwitchExportDetails->setText(tr("Details"));
    } else if (_ui->stackedWidgetDetailsExport->currentWidget() == _exportWidget) {
        _ui->actionSwitchExportDetails->setIcon(QIcon(Ressources::themedPath(Constants::ICON_EXPORT)));
        _ui->stackedWidgetDetailsExport->setCurrentWidget(_ui->stackedPageDetails);
        _ui->actionSwitchExportDetails->setToolTip(tr("Export"));
        _ui->actionSwitchExportDetails->setText(tr("Export"));
    }
}
} // namespace Hnv
