/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of Hynesim.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "topologyexportwidget.h"
#include "ui_topologyexportwidget.h"
#include "topologycatalogmodel.h"
#include "entitiesmodel.h"
#include "hyneviewsession.h"
#include "topologyadministrator.h"

// hynesim hnsclient headers
#include <libhnsclient/topologyentrycatalog.h>
#include <libhnsclient/entry.h>
#include <libhnsclient/topology.h>
#include <libhnsclient/entityentrycatalog.h>
#include <libhnsclient/catalogimporter.h>
#include <libhnsclient/session.h>

// Qt headers
#include <QMessageBox>
#include <QDateTime>
#include <QDebug>

namespace Hnv {

TopologyExportWidget::TopologyExportWidget(TopologyAdministrator *administrator, QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::TopologyExportWidget)
    , _administrator(administrator)
    , _entitiesModel(new EntitiesModel(_administrator, this))
    , _currentId(-1)
    , _delegate(new CheckBoxDelegate(this))
    , _compressAllButtonState(true)
{
    ui->setupUi(this);
    ui->entitiesTableView->setModel(_entitiesModel);
    ui->entitiesTableView->horizontalHeader()->setResizeMode(0, QHeaderView::Stretch);
    ui->entitiesTableView->horizontalHeader()->setResizeMode(1, QHeaderView::ResizeToContents);
    ui->entitiesTableView->verticalHeader()->hide();
    ui->entitiesTableView->setItemDelegate(_delegate);
    _setControls(0);
    connect(_administrator->catalog(), SIGNAL(topologyUpdated(HnsClient::TopologyEntry*)), this, SLOT(_topologyEntryUpdatedSlot(HnsClient::TopologyEntry*)));
}

TopologyExportWidget::~TopologyExportWidget()
{
    delete ui;
}

void TopologyExportWidget::setTopology(qlonglong id)
{
    if (id == _currentId)
        return;

    _currentId = _setControls(_administrator->catalog()->topologyEntryByID(id));
    _entitiesModel->topologyChange(_currentId);
}

qlonglong TopologyExportWidget::_setControls(HnsClient::TopologyEntry *entry)
{
    QString shortName   = tr("N/A");
    bool controlEnabled = false;
    bool isRunning      = false;
    qlonglong resultId = 0;
    if (entry) {
        QString entryShortName = entry->getShortName();
        if (!entryShortName.isEmpty()) {
            shortName = entryShortName;
            resultId = entry->getID();
            if (entry->getStatus() == Hns::TOPOLOGY_BUSY_EXPORTING)
                isRunning = true;
            else {
                controlEnabled = true;
            }
        }
    }
    ui->newNameLineEdit->setText(shortName);
    ui->exportPushButton->setEnabled(controlEnabled);
    ui->newNameLineEdit->setEnabled(controlEnabled);
    ui->createTarArchiveCheckBox->setEnabled(controlEnabled);
    ui->progressBar->setVisible(isRunning);
    ui->compressAllPushButton->setText(tr("Compress all"));
    _compressAllButtonState = true;
    return resultId;
}

void TopologyExportWidget::on_exportPushButton_clicked()
{
    HnsClient::TopologyEntry *topologyEntry = _administrator->catalog()->topologyEntryByID(_currentId);
    if (!topologyEntry) {
        return;
    }

    HnsClient::EntityEntryCatalog *entityCatalog = HyneviewSession::instance()->entityEntryCatalog();
    if (!entityCatalog) {
        qWarning() << tr("Unable to access Entity Catalog. Operation aborted.");
        return;
    }

    HnsClient::Topology *loadedTopology = _administrator->topologyByEntryId(_currentId);
    if (loadedTopology) {
        QMessageBox::critical(this, tr("Exporting Topology"), tr("Selected topology is currently used. Failed to export..."));
        return;
    }


    bool entitiesAvailable = true;
    QMap<Ice::Int, Hns::EntityEntryPtr> entityEntryMap(topologyEntry->topologyEntry()->entitiesMap);
    foreach(const Hns::EntityEntryPtr &entry, entityEntryMap) {
        qlonglong entryId = entry->id;
        if (entryId == 0) // entity was created with a template and is not stored in catalog
            continue;
        HnsClient::EntityEntry *localEntry = entityCatalog->entityEntryById(entryId);
        if (!localEntry) {
            entitiesAvailable = false;
            qWarning() << tr("Entry with Id %1 seems missing. Aborting...").arg(QString::number(entryId));
        } else if (localEntry->isLocked()) {
            entitiesAvailable = false;
            qWarning() << tr("Entry with Id %1 is locked. Aborting...").arg(QString::number(entryId));
        }
    }
    if (!entitiesAvailable) {
        QMessageBox::critical(this, tr("Exporting Topology"), tr("Somes entities are not available at this time. Failed to export..."));
        return;
    }

    HyneviewSession::instance()->catalogImporter()->exportTopology(_createArchiveDescriptor(topologyEntry), ui->createTarArchiveCheckBox->isChecked(), this, SLOT(_exportEnded(bool)));
}

void TopologyExportWidget::_exportEnded(bool success)
{
    Q_UNUSED(success)
}

void TopologyExportWidget::_topologyEntryUpdatedSlot(HnsClient::TopologyEntry *entry)
{
    if (entry->getID() == _currentId) {
        _setControls(entry);
    }
}

Hns::ArchiveDescriptorPtr TopologyExportWidget::_createArchiveDescriptor(HnsClient::TopologyEntry *topologyEntry)
{
    QDateTime date;
    Hns::ArchiveDescriptorPtr archiveDescriptor = new Hns::ArchiveDescriptor();
    archiveDescriptor->creationDateTime = QDateTime::currentMSecsSinceEpoch();
    archiveDescriptor->name = QString("Topology-%1-%2-%3")
            .arg(topologyEntry->getUUID())
            .arg(topologyEntry->getShortName())
            .arg(date.currentDateTime().toString("yyyyMMdd-hhmmss"))
            .toStdString();

    QList<std::string> customResources;
    //entities
    Hns::EntityDescriptorList entities;
    QMap<Ice::Int, Hns::EntityEntryPtr> entitiesMap = QMap<Ice::Int, Hns::EntityEntryPtr>(topologyEntry->topologyEntry()->entitiesMap);
    QList<Hns::EntityEntryPtr> entitiesList(entitiesMap.values());

    HnsClient::EntityEntryCatalog *catalog = HnsClient::Session::instance()->entityEntryCatalog();
    QList<Hns::EntityEntryPtr> baseEntitiesList;

    foreach (const Hns::EntityEntryPtr &entityEntry, entitiesMap) {
        Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(entityEntry);

        if (domainEntry && !domainEntry->baseUuid.empty()) {
            HnsClient::EntityEntry *baseEntityEntry = catalog->entityEntryByUUID(QString::fromStdString(domainEntry->baseUuid));
            archiveDescriptor->baseEntities[domainEntry->uuid] = domainEntry->baseUuid;
            if (!baseEntitiesList.contains(baseEntityEntry->entityEntry()))
                baseEntitiesList.append(baseEntityEntry->entityEntry());
        }
    }

    entitiesList.append(baseEntitiesList);

    foreach (const Hns::EntityEntryPtr &entityEntry, entitiesList) {
        if (!entityEntry->hasResources && (entityEntry->path == "")) {
            /*Entry with no resources and no path is an entry template saved
             *in the topology.
             *No need to export its folder
             **/
            qDebug() << tr("The entry %1 is a template without resources, no need to export").arg(QString::fromStdString(entityEntry->uuid));
            continue;
        }
        Hns::EntityDescriptor entityDescriptor;
        entityDescriptor.type = entityEntry->ice_id();
        entityDescriptor.shortName = entityEntry->shortName;
        entityDescriptor.longName = entityEntry->longName;
        entityDescriptor.description = entityEntry->description;
        entityDescriptor.path = entityEntry->path;
        entityDescriptor.uuid = entityEntry->uuid;
        entityDescriptor.parentId = entityEntry->parentId;
        entityDescriptor.customGraphic = entityEntry->customGraphic;

        if (!entityDescriptor.customGraphic.empty() && !customResources.contains(entityDescriptor.customGraphic))
            customResources.append(entityDescriptor.customGraphic);

        Hns::EntityEntryPtr stateEntry = entityEntry;

        foreach (Hns::EntityEntryPtr child, entitiesMap) {
            Hns::DomainEntryPtr childEntry = Hns::DomainEntryPtr::dynamicCast(child);
            if (childEntry && childEntry->baseUuid == entityEntry->uuid) {
                stateEntry = childEntry;
                break;
            }
        }

        if (_entitiesModel->getStateByUuid(QString::fromStdString(stateEntry->uuid))) {
            entityDescriptor.fileType = Hns::FileType::COMPRESSED_FILE;
            QString fileLocation = QString::fromStdString(entityEntry->uuid) + ".tar.gz";
            entityDescriptor.fileLocation = fileLocation.toStdString();
        }
        else
            entityDescriptor.fileType = Hns::FileType::PLAIN_FOLDER;
        entityDescriptor.tags = entityEntry->tagList;
        entities.push_back(entityDescriptor);
    }
    archiveDescriptor->customResources = customResources.toStdList();

    //topology
    Hns::TopologyDescriptorList topologies;
    Hns::TopologyDescriptor topologyDescriptor;
    topologyDescriptor.shortName = ui->newNameLineEdit->text().toStdString();
    topologyDescriptor.longName = topologyEntry->topologyEntry()->longName;
    topologyDescriptor.description = topologyEntry->topologyEntry()->description;
    topologyDescriptor.path = topologyEntry->topologyEntry()->path;
    topologyDescriptor.uuid = topologyEntry->topologyEntry()->uuid;
    topologyDescriptor.tags = topologyEntry->topologyEntry()->tagList;
    //dependencies
    foreach (Hns::EntityEntryPtr entityEntry, entitiesList) {
        topologyDescriptor.dependencies.push_back(entityEntry->uuid);
    }
    topologies.push_back(topologyDescriptor);
    archiveDescriptor->topologies = topologies;

    //folders
    QList <HnsClient::FolderEntry *> folderList;
    Hns::FolderDescriptorList folders;

    for (Hns::EntityDescriptorList::iterator it = entities.begin() ; it != entities.end() ; ++it) {
        Hns::EntityDescriptor entityDescriptor = *it;
        if (entityDescriptor.parentId != 0) {
            HnsClient::FolderEntry *folder = catalog->folderById(entityDescriptor.parentId);
            if (!folder) {
                it->parentId = 0;
                continue;
            }
            if (!folderList.contains(folder))
                folderList.append(folder);
            while (folder->getParentID() != 0) {
                folder = catalog->folderById(folder->getParentID());
                if (!folderList.contains(folder))
                    folderList.append(folder);
            }
        }
    }
    foreach (HnsClient::FolderEntry *folder, folderList) {
        Hns::FolderDescriptor folderDescriptor;
        folderDescriptor.shortName = folder->getShortName().toStdString();
        folderDescriptor.longName = folder->getLongName().toStdString();
        folderDescriptor.description = folder->getDescription().toStdString();
        foreach (QString str, folder->getTagList().split(", ")) {
            folderDescriptor.tags.push_back(str.toStdString());
        }
        folderDescriptor.id = folder->getID();
        folderDescriptor.parentId = folder->getParentID();
        folders.push_back(folderDescriptor);
    }
    archiveDescriptor->entities = entities;
    archiveDescriptor->folders = folders;
    return archiveDescriptor;
}

void TopologyExportWidget::on_compressAllPushButton_clicked()
{
    for (int i = 0 ; i < _entitiesModel->rowCount(); ++i) {
        QModelIndex index = _entitiesModel->index(i, EntitiesModel::COLUMN_CHECKBOX);
        _entitiesModel->setData(index, _compressAllButtonState, Qt::EditRole);
    }
    _compressAllButtonState = !_compressAllButtonState;
    if (_compressAllButtonState)
        ui->compressAllPushButton->setText(tr("Compress all"));
    else
        ui->compressAllPushButton->setText(tr("Uncompress all"));
}

} // namespace Hnv
