/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "topologylistwidget.h"
#include "simulationmodewidget.h"
#include "topologywidget.h"

// Qt headers
#include <QMenu>
#include <QTreeView>

namespace Hnv {

/*******************************************************************************
* TopologyListWidget
*******************************************************************************/
TopologyListWidget::TopologyListWidget(QWidget *parent)
    : SideBarTreeWidget(parent)
    , _topologyWidget(0)
{
    _actionProperties = new QAction(this);
    _actionProperties->setText(tr("Properties"));
    _actionProperties->setToolTip(tr("Display topology properties"));
    connect(_actionProperties, SIGNAL(triggered()), this, SLOT(_actionPropertiesTriggeredSlot()));

    _actionCloseTopology = new QAction(this);
    _actionCloseTopology->setText(tr("Close topology"));
    _actionCloseTopology->setToolTip(tr("Close current topology"));
    _actionCloseTopology->setShortcut(tr("Ctrl+W"));
    connect(_actionCloseTopology, SIGNAL(triggered()), this, SLOT(_actionCloseTopologyTriggeredSlot()));

    _actionCloseAllTopologies = new QAction(this);
    _actionCloseAllTopologies->setText(tr("Close all topology"));
    _actionCloseAllTopologies->setToolTip(tr("Close all topology"));
    _actionCloseAllTopologies->setShortcut(tr("Ctrl+Shift+W"));
    connect(_actionCloseAllTopologies, SIGNAL(triggered()), this, SLOT(_actionCloseAllTopologiesTriggeredSlot()));

    _actionCloseAllTopologiesExpect = new QAction(this);
    _actionCloseAllTopologiesExpect->setText(tr("Close all topology except this one"));
    _actionCloseAllTopologiesExpect->setToolTip(tr("Close all topology except current topology"));
    connect(_actionCloseAllTopologiesExpect, SIGNAL(triggered()), this, SLOT(_actionCloseAllTopologiesExpectTriggeredSlot()));

    QWidget * widget = sideBarParent();

    if (widget)
        _simulationModeWidget = dynamic_cast<SimulationModeWidget*>(widget);
    if (_simulationModeWidget) {
        _topologyWidget = _simulationModeWidget->topologyWidget();
        if (_topologyWidget) {
            setViewModel(_topologyWidget->loadedTopologyModel());
            connect(_topologyWidget, SIGNAL(topologyActivated(QVariant)), this, SLOT(_topologyActivatedSlot(QVariant)));
        }
    }
    _initCloseDelegate(1);
}

TopologyListWidget::~TopologyListWidget()
{
    clearModel();
}

QString TopologyListWidget::title()
{
    return tr("Open Topologies");
}

QString TopologyListWidget::sideBarId()
{
    return "Open Topologies";
}

/*******************************************************************************
* TopologyListWidget - Protected slots
*/
void TopologyListWidget::_indexClicked(const QModelIndex &index)
{
    if (index.isValid()) {
        if (index.column() == 0)
            _activateTopology(index);
        else if (index.column() == 1)
            _closeTopology(index);
    }
}

void TopologyListWidget::_indexDoubleClicked(const QModelIndex &index)
{
    if (index.isValid()) {
        QString topologyUUID = _topologyUUIDFromIndex(index);
        _activateTopology(topologyUUID);
        _displayTopologyProperties(topologyUUID);
    }
}

void TopologyListWidget::_contextMenuRequested(const QModelIndex &index, const QPoint &position)
{
    if (index.isValid()) {
        QMenu contextMenu;
        contextMenu.addAction(_actionCloseTopology);
        contextMenu.addAction(_actionCloseAllTopologiesExpect);
        contextMenu.addAction(_actionCloseAllTopologies);
        contextMenu.addSeparator();
        contextMenu.addAction(_actionProperties);
        contextMenu.exec(_globalPosition(position));
    }
}

/*******************************************************************************
* TopologyListWidget - Private slots
*/
void TopologyListWidget::_actionPropertiesTriggeredSlot()
{
    _displayTopologyProperties(_currentIndex());
}

void TopologyListWidget::_actionCloseTopologyTriggeredSlot()
{
    _closeTopology(_currentIndex());
}

void TopologyListWidget::_actionCloseAllTopologiesTriggeredSlot()
{
    _closeAllTopologies();
}

void TopologyListWidget::_actionCloseAllTopologiesExpectTriggeredSlot()
{
    QModelIndex currentIdx = _currentIndex();
    if (currentIdx.isValid())
        _closeAllTopologies(_topologyUUIDFromIndex(currentIdx));
}

void TopologyListWidget::_topologyActivatedSlot(QVariant topologyUUID)
{
    QTreeView * treeView = _view();
    QModelIndexList matchList = treeView->model()->match(treeView->model()->index(0,0), Qt::UserRole, topologyUUID);
    if (!matchList.isEmpty())
        treeView->selectionModel()->setCurrentIndex(matchList.first(), QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows);
}

/*******************************************************************************
* TopologyListWidget - Private
*/
QString TopologyListWidget::_topologyUUIDFromIndex(QModelIndex index)
{
    return index.data(Qt::UserRole).toString();
}

void TopologyListWidget::_activateTopology(QString topologyUUID)
{
    if (_topologyWidget)
        _topologyWidget->setCurrentTopology(topologyUUID);
}

void TopologyListWidget::_activateTopology(QModelIndex index)
{
    _activateTopology(_topologyUUIDFromIndex(index));
}

void TopologyListWidget::_displayTopologyProperties(QModelIndex index)
{
    _displayTopologyProperties(_topologyUUIDFromIndex(index));
}

void TopologyListWidget::_displayTopologyProperties(QString topologyUUID)
{
    _topologyWidget->displayTopologyProperties(topologyUUID);
}

void TopologyListWidget::_closeTopology(QModelIndex index)
{
    _closeTopology(_topologyUUIDFromIndex(index));
}

void TopologyListWidget::_closeTopology(QString topologyUUID)
{
    _topologyWidget->closeTopologySlot(topologyUUID);
}

void TopologyListWidget::_closeAllTopologies(QString excludedTopologyUUID)
{
    _topologyWidget->closeAllTopologiesSlot(excludedTopologyUUID);
}

} // namespace Hnv
