/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "topologyscene.h"
#include "hyneviewsession.h"
#include "topologyview.h"
#include "entitygraphics.h"
#include "ressources.h"
#include "hnvmainwindow.h"

// Qt headers
#include <QGraphicsLineItem>
#include <QGraphicsSvgItem>
#include <QFontMetrics>
#include <QSettings>
#include <QGraphicsBlurEffect>

// hynesim hnsclient headers
#include <libhnsclient/jobmanager.h>
#include <libhnsclient/entityentrycatalog.h>
#include <libhnsclient/entry.h>

namespace Hnv {

/*******************************************************************************
* TopologyScene
*******************************************************************************/
TopologyScene::TopologyScene(qreal x, qreal y, qreal width, qreal height, TopologyView *view)
    : QGraphicsScene(x, y, width, height, view)
    , _currentLine(NULL)
    , _view(view)
    , _minimumWireLength(50)
{
}

void TopologyScene::startWireLine(QPointF scenePos)
{
    if (_currentLine) {
        removeItem(_currentLine);
        delete _currentLine;
    }
    _currentLine = new QGraphicsLineItem(QLineF(scenePos, scenePos));
    _currentLine->setPen(QPen(_view->getWireColor(), 2));
    _currentLine->setZValue(Constants::DEFAULT_TOPOLOGY_SIZE);
    addItem(_currentLine);
}

void TopologyScene::extendWireLineTo(QPointF scenePos)
{
    if (_currentLine) {
        QLineF line = _currentLine->line();
        line.setP2(scenePos);
        _currentLine->setLine(line);
        if (_currentLine->line().length() < _minimumWireLength) {
            if (!_currentLine->graphicsEffect()) {
                QGraphicsBlurEffect *effect = new QGraphicsBlurEffect();
                _currentLine->setGraphicsEffect(effect);
            }
        }
        else {
            if (_currentLine->graphicsEffect())
                _currentLine->setGraphicsEffect(0);
        }
    }
}

void TopologyScene::endWireLine()
{
    if (_currentLine) {
        EntityGraphics *startGraphicsItem = NULL;
        EntityGraphics *endGraphicsItem = NULL;
        QList<QGraphicsItem *> startItems = items(_currentLine->line().p1());
        if (startItems.count() && startItems.first() == _currentLine)
            startItems.removeFirst();
        QList<QGraphicsItem *> endItems = items(_currentLine->line().p2());
        if (endItems.count() && endItems.first() == _currentLine)
            endItems.removeFirst();
        if (startItems.count() > 0) {
            QGraphicsItem * startItem = startItems.first()->parentItem();
            if (startItem)
                startItem->setFlag(QGraphicsItem::ItemIsMovable);
            else
                startItems.first()->setFlag(QGraphicsItem::ItemIsMovable);
        }
        if (startItems.count() > 0) {
            QGraphicsItem * startItem = startItems.first()->parentItem();
            if (!startItem)
                startItem = startItems.first();

            startGraphicsItem = dynamic_cast<EntityGraphics *>(startItem);
        }
        if (endItems.count() > 0) {
            QGraphicsItem * endItem = endItems.first()->parentItem();
            if (!endItem)
                endItem = endItems.first();
            endGraphicsItem = dynamic_cast<EntityGraphics *>(endItem);
        }
        if (startGraphicsItem && endGraphicsItem) {
            if (startGraphicsItem == endGraphicsItem) {
                qCritical() << tr("Plugging one entity to itself is not possible");
            }
            else {
                WireGraphics::tryToConnect( _currentLine->line().p1(),  _currentLine->line().p2(), _view);
            }
        }
        else {
            if (_currentLine->line().length() >= _minimumWireLength && sceneRect().contains(_currentLine->line().p2()))
                WireGraphics::tryToConnect( _currentLine->line().p1(),  _currentLine->line().p2(), _view);
        }

        removeItem(_currentLine);
        delete _currentLine;
        _currentLine = NULL;
    }
}

QGraphicsLineItem * TopologyScene::currentLine()
{
    return _currentLine;
}

void TopologyScene::displayCloneGraphic(qlonglong entryID, int cloneJobID, QPoint position)
{
    if (cloneJobID == 0)
        qCritical() << tr("Could not load clone job with ID '%1', for entity '%2'").arg(cloneJobID).arg(entryID);
    else {
        _clonePosition.insert(cloneJobID, QPair<QPoint, qlonglong>(position, entryID));
        HyneviewSession::instance()->jobManager()->getJob(cloneJobID, this, SLOT(_getJobSlot(HnsClient::Job *)));
    }
}

void TopologyScene::_getJobSlot(HnsClient::Job *job)
{
    int jobId = job->getId();
    QPoint position = _clonePosition.value(jobId).first;
    qlonglong entityEntryId = _clonePosition.value(jobId).second;
    _clonePosition.remove(jobId);
    if (job->getProgress() != 1 && job->isRunning() && !_cloneGraphics.contains(jobId)) {
        QSettings settings;
        Qt::TextElideMode elideMode;
        settings.beginGroup(Constants::TOPOLOGY_SETTINGS);
        switch (settings.value(Constants::ELIDE_MODE_KEY,0).toInt()) {
        case 0:
            elideMode = Qt::ElideLeft;
            break;
        case 1:
            elideMode = Qt::ElideRight;
            break;
        case 2:
            elideMode = Qt::ElideMiddle;
            break;
        case 3:
            elideMode = Qt::ElideNone;
            break;
        default:
            elideMode = Qt::ElideLeft;
            break;
        }
        settings.endGroup();
        QGraphicsSvgItem * item = new QGraphicsSvgItem(Ressources::themedPath(Constants::GRAPHIC_BUSY));
        _cloneGraphics.insert(jobId, item);
        _cloneJobs.insert(jobId, job);
        addItem(item);
        item->setPos(position);

        HnsClient::EntityEntryCatalog *catalog = HnvMainWindow::instance()->entityCatalog();
        QFontMetrics fontMetrics(font());
        QGraphicsTextItem *shortname = new QGraphicsTextItem(fontMetrics.elidedText(catalog->entityEntryById(entityEntryId)->getShortName(), elideMode,(item->boundingRect().width()*3.0)));
        _cloneText.insert(jobId,shortname);
        QPoint textPosition;
        textPosition.setX(position.x()-(item->boundingRect().width()/2.0));
        textPosition.setY(position.y()+item->boundingRect().height()+5);
        addItem(shortname);
        shortname->setPos(textPosition);
        connect(job, SIGNAL(progressUpdated(int,double)), this, SLOT(_cloneJobProgressChangedSlot(int,double)));
        connect(job, SIGNAL(finished(int)), this, SLOT(_cloneJobCancelledSlot(int)));
    } else
        job->deleteLater();
}

void TopologyScene::_cloneJobCancelledSlot(int jobId)
{
    if (_cloneJobs.contains(jobId)) {
        HnsClient::Job *job = _cloneJobs.take(jobId);
        delete job;
    }
    if (_cloneGraphics.contains(jobId)) {
        QGraphicsSvgItem *item = _cloneGraphics.take(jobId);
        delete item;
    }
    if (_cloneText.contains(jobId)) {
        QGraphicsTextItem * item = _cloneText.take(jobId);
        delete item;
    }
}

void TopologyScene::_cloneJobProgressChangedSlot(int jobId, double progress)
{
    if (progress == 1.0)
        _cloneJobCancelledSlot(jobId);
}
// TODO : requires cleaning
//void TopologyScene::updateSceneRect(QList<QRectF>)
//{
//    qreal margin = 100;
//    QRectF rect = itemsBoundingRect();
//    QRectF sr = _view->sceneRect();
//    if (!sr.contains(rect)) {
//        QRectF marginRect = rect;
//        int rectX1 = rect.x();
//        int rectX2 = rectX1 + rect.width();
//        int rectY1 = rect.y();
//        int rectY2 = rectY1 + rect.height();
//        if (rectX1 < sr.x())
//            marginRect.setX(rectX1 - margin);
//        if (rectY1 < sr.y())
//            marginRect.setY(rectY1 - margin);
//        if (rectX2 > sr.x() + sr.width())
//            marginRect.setWidth(marginRect.width() + margin);
//        if (rectY2 > sr.y() + sr.height())
//            marginRect.setHeight(marginRect.height() + margin);
//        if (marginRect != rect)
//            setSceneRect(marginRect);
//    }
//}

} // namespace Hnv
