/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "topologyview.h"
#include "entitygraphics.h"
#include "topologyscene.h"
#include "ressources.h"
#include "entitycatalogmodel.h"

// hynesim hnsclient headers
#include <libhnsclient/entity.h>

// Qt headers
#include <QSettings>
#include <qmath.h>
#include <QApplication>
#include <QDropEvent>
#include <QDragEnterEvent>
#include <QDragMoveEvent>
#include <QMimeData>
#include <QScrollBar>

namespace Hnv {

/*******************************************************************************
* TopologyView
*/
TopologyView::TopologyView(QWidget *parent)
    : ZoomingView(parent)
    , _mode(SelectMode)
    , _isWriteable(false)
    , _isModel(false)
    , _mouseMoved(false)
    , _wireColor(Qt::black)
{
    _initView();
}

TopologyView::TopologyView(QGraphicsScene *scene, QWidget *parent)
    : ZoomingView(scene, parent)
    , _mode(SelectMode)
    , _isWriteable(false)
    , _isModel(false)
    , _wireColor(Qt::black)
{
    _initView();
}

void TopologyView::setSceneRect(qreal x, qreal y, qreal w, qreal h, bool centerView)
{
    scene()->setSceneRect(x, y, w, h);
    if (centerView)
        setCenter(sceneRect().center());
}

QStringList TopologyView::supportedMimeTypes()
{
    QStringList mimeTypes;
    mimeTypes
//            << "hyneview/catalog-category"
            << "hyneview/catalog-entry"
//            << "hyneview/catalog-category-selection"
            << "hyneview/catalog-entry-selection"
            << "hyneview/topology-entry"
            << "hyneview/catalog-mixed-selection";

    return mimeTypes;
}

bool TopologyView::canHandleMimeData(const QMimeData *data)
{
    QString datatype;
    return canHandleMimeData(data, datatype);
}

bool TopologyView::canHandleMimeData(const QMimeData *data, QString &datatype)
{
    QStringList supportedTypes = supportedMimeTypes();
    bool dataIsSupported = false;
   // if (_isWriteable) {
        foreach(QString sourceDataType, supportedTypes) {
            if (data->hasFormat(sourceDataType)) {
                dataIsSupported = true;
                datatype = sourceDataType;
                break;
            }
   //     }
    }
    return dataIsSupported;
}

TopologyScene * TopologyView::topologyScene()
{
    return dynamic_cast<TopologyScene*>(scene());
}

void TopologyView::emitRequestWireEntityItem(QPointF from, QPointF to)
{
    Q_UNUSED(from)
    Q_UNUSED(to)
    emit requestWireEntityItem(from, to);
}

void TopologyView::matchZoomToItems()
{
    while (!mapToScene(frameRect()).boundingRect().contains(scene()->itemsBoundingRect())) {
        int oldZoom = currentZoomFactor();
        zoomOut();
        if(currentZoomFactor() == oldZoom)
            break;
    }
}

void TopologyView::matchZoomToRect(QRectF targetRect)
{
    while (!mapToScene(frameRect()).boundingRect().contains(targetRect)) {
        int oldZoom = currentZoomFactor();
        zoomOut();
        if(currentZoomFactor() == oldZoom) {
            centerOn(targetRect.center());
            break;
        }
    }
}

TopologyView::InsertMode TopologyView::insertMode()
{
    return _mode;
}

void TopologyView::setInsertMode(InsertMode insertMode)
{
    _mode = insertMode;
}

void TopologyView::setIsWriteable(bool isWriteable)
{
    _isWriteable = isWriteable;
}

void TopologyView::setIsModel(bool isModel)
{
    _isModel = isModel;
}

QColor TopologyView::getWireColor()
{
    return _wireColor;
}

void TopologyView::setWireColor(QColor color)
{
    _wireColor = color;
}

/*******************************************************************************
* TopologyView - Protected
*/
void TopologyView::dropEvent(QDropEvent *event)
{
    QApplication::setOverrideCursor(QCursor(Qt::ArrowCursor));
    if(!_isModel){
        ZoomingView::dropEvent(event);
        if(!event->isAccepted() && !items(event->pos()).isEmpty())
            return;
    }
    QString datatype;
    const QMimeData *data = event->mimeData();
    if (canHandleMimeData(event->mimeData(), datatype) && _isWriteable) {
        event->acceptProposedAction();
        setFocus();
        QByteArray itemData = data->data(datatype);
        QDataStream dataStream(&itemData, QIODevice::ReadOnly);
        if (datatype == "hyneview/topology-entry") {
            while (!dataStream.atEnd()) {
                qlonglong topologyId;
                dataStream >> topologyId;
                QPoint pos = mapToScene(event->pos()).toPoint();
                emit requestTopologyImport(topologyId, pos);
            }
        } else {
            QList<qlonglong> sourceIdList;
            QList<EntityGraphics *> overlappedEntityList;
            QPoint pos;
            EntityGraphics *overlappedEntity = NULL;
            qlonglong sourceID;

            while (!dataStream.atEnd()) {
                int itemType;
                qlonglong sourceParentID;
                int itemInternalID;
                dataStream >> itemType >> sourceID >> sourceParentID >> itemInternalID;
                if (itemType == EntityCatalogModel::Folder) {
                    // "TODO:Manage folder drop";
                } else if (itemType == EntityCatalogModel::Entity) {
                    pos=mapToScene(event->pos()).toPoint();
                    if (!_isModel) {
                        QGraphicsItem * targetItem = itemAt(event->pos());
                        if (targetItem) {
                            overlappedEntity = dynamic_cast<EntityGraphics *>(targetItem->topLevelItem());
                        }
                    }
                    sourceIdList.append(sourceID);
                    overlappedEntityList.append(overlappedEntity);
                }
            }

            QHash<qlonglong,QPoint>  toDraw = _calculatePosition(sourceIdList, pos);
            for (int i=0;i<toDraw.keys().size();i++){
                emit requestEntityCreation(sourceIdList.at(i), toDraw.value(sourceIdList.at(i)), overlappedEntityList.at(i));
            }
        }
    }
    QApplication::restoreOverrideCursor();
    scene()->clearSelection();
}

QPoint TopologyView::_validateItemPosition(QPoint position, QRect itemRect)
{
    if (scene()) {
        QRectF sceneRect = scene()->sceneRect();
        qreal minX = sceneRect.x();
        qreal minY = sceneRect.y();
        qreal maxX = minX + sceneRect.width() - itemRect.width();
        qreal maxY = minY + sceneRect.height() - itemRect.height();
        if (position.x() > maxX)
            position.setX(maxX);
        if (position.y() > maxY)
            position.setY(maxY);
        if (position.x() < minX)
            position.setX(minX);
        if (position.y() < minY)
            position.setY(minY);
    }
    return position;
}

QHash<qlonglong,QPoint> TopologyView::_calculatePosition(QList<qlonglong> listItemsID, QPoint initialPosition)
{
    QSettings settings;
    settings.beginGroup(Constants::TOPOLOGY_SETTINGS);
    double angleDeg = settings.value(Constants::ANGLE_FOR_DROPING_ENTITIES_KEY,QVariant(60.0)).toDouble();
    double angleRad = angleDeg* M_PI / 180.0;
    int radiusOriginal = settings.value(Constants::RADIUS_FOR_DROPING_ENTITIES_KEY,QVariant(256)).toInt();
    settings.endGroup();


    int radius = radiusOriginal;
    int modulo = 360/angleDeg;
    QHash<qlonglong,QPoint> returnHash;
    QRect boundingRect(initialPosition.x()-128,initialPosition.y()-128,256,256); //TODO: Change this numbers in function of the real BoundigRect of the entities
    returnHash.insert(listItemsID.takeFirst(),initialPosition);

    int numberItemPerCircle = 0;
    for (int i=0;i<listItemsID.size();i++){
        if ((numberItemPerCircle)%(modulo)==0 && (numberItemPerCircle)!=0){
            radius+=radiusOriginal;
            angleRad = angleRad/2.0;
            modulo = modulo*2;
            numberItemPerCircle = 0;
        }
        QPoint position = _validateItemPosition(QPoint(initialPosition.x()+radius*qCos(i*angleRad), initialPosition.y()+radius*qSin(i*angleRad)), boundingRect);
        returnHash.insert(listItemsID.at(i),position);
        numberItemPerCircle++;
    }
    return returnHash;

}

void TopologyView::dragEnterEvent(QDragEnterEvent *event)
{
    ZoomingView::dragEnterEvent(event);
    if(!event->isAccepted() && !items(event->pos()).isEmpty())
        return;

    if (canHandleMimeData(event->mimeData()) && _isWriteable)
        event->accept();
}

void TopologyView::dragMoveEvent(QDragMoveEvent *event)
{
    if(!_isModel && !event->mimeData()->hasFormat("hyneview/topology-entry"))
        ZoomingView::dragMoveEvent(event);
    if(!event->isAccepted() && !items(event->pos()).isEmpty()){
        return;
    }
    if (canHandleMimeData(event->mimeData())
            && scene()
            && sceneRect().contains(mapToScene(event->pos()))
            && _isWriteable) {
        event->accept();
    } else
        event->ignore();    
}

/**
  * Handles when the mouse button is pressed
  */
void TopologyView::mousePressEvent(QMouseEvent* event)
{
    bool ctrlPress = ((event->modifiers() & Qt::ControlModifier) == Qt::ControlModifier );
    _itemSelected.clear();
    _currentEntityGraphics.second = NULL;
    _currentEntityGraphics.first = 0;
    _mouseMoved = false;
    QGraphicsItem * targetItem = itemAt(event->pos());
    if (event->button() == Qt::RightButton) {
        if (targetItem) {
            if (!targetItem->topLevelItem()->isSelected()) {
                scene()->clearSelection();
                targetItem->topLevelItem()->setSelected(true);
            }
            event->accept();
            return;
        }
        ZoomingView::mousePressEvent(event);
    } else if (event->button() == Qt::LeftButton) {
        ZoomingView::mousePressEvent(event);
        if (_mode && sceneRect().contains(mapToScene(event->pos())))
            topologyScene()->startWireLine(mapToScene(event->pos()));
        if (targetItem) {
            EntityGraphics * entityGraphics = dynamic_cast<EntityGraphics *>(targetItem->topLevelItem());
            if (entityGraphics) {
                if (!_mode) {
                    _currentEntityGraphics.second = entityGraphics;
                    _currentEntityGraphics.first = entityGraphics->getEntityID();
                    QPointF pos = matrix().map(_currentEntityGraphics.second->pos());
                    _translationPosition.setX(pos.x() - event->x() - horizontalScrollBar()->value());
                    _translationPosition.setY(pos.y() - event->y() - verticalScrollBar()->value());
                    foreach(QGraphicsItem* item, scene()->selectedItems()) {
                        EntityGraphics *entity = qgraphicsitem_cast<EntityGraphics *>(item->topLevelItem());
                        if (entity == 0)
                            continue;
                        if ((item->flags() & QGraphicsItem::ItemIsMovable) == QGraphicsItem::ItemIsMovable) {
                            //calcul distance between QGraphicsItem and the currentEntityGraphics
                                _itemSelected.insert(entity->getEntityID(), (qMakePair(entity, entity->pos() - entityGraphics->pos())));
                                if (ctrlPress)
                                    _selectedItemsWithCtrl.insert(entity->getEntityID(), (qMakePair(entity, entity->pos() - entityGraphics->pos())));
                        }
                    }

                    if(ctrlPress) {
                        if (_currentEntityGraphics.second->isSelected()) {
                            _itemSelected.clear();
                            _unselectedItemWithCtrl = qMakePair(_currentEntityGraphics.first, _currentEntityGraphics.second);
                        } else {
                            _itemSelected.insert(_currentEntityGraphics.first, qMakePair(_currentEntityGraphics.second, QPointF()));
                            _selectedItemsWithCtrl.insert(_currentEntityGraphics.first, qMakePair(_currentEntityGraphics.second, QPointF()));
                        }
                    }
                }
            }
        } else if (!ctrlPress) {
            _selectedItemsWithCtrl.clear();
        }

    } else {
        ZoomingView::mousePressEvent(event);
    }

    if (ctrlPress) {
        _mouseMovedWithCtrl = false;
        QMapIterator<qlonglong, QPair<EntityGraphics *, QPointF> > iterator(_selectedItemsWithCtrl);
        while (iterator.hasNext()) {
            iterator.next();
            iterator.value().first->topLevelItem()->setSelected(true);
        }
    }
}

/**
*Handles the mouse move event
*/
void TopologyView::mouseMoveEvent(QMouseEvent* event)
{
    bool ctrlPress = ((event->modifiers() & Qt::ControlModifier) == Qt::ControlModifier );
    if (_mode && topologyScene()->currentLine()) {
        topologyScene()->extendWireLineTo(mapToScene(event->pos()));
        event->accept();
        _updateCoordonnate(event);
    } else {
        if (event->buttons() == Qt::LeftButton && _currentEntityGraphics.second) {
            if (ctrlPress && !_mouseMoved) {
                _currentEntityGraphics.second->setSelected(!_currentEntityGraphics.second->isSelected());
            }
            _mouseMoved = true;

            QMap<qlonglong, QPair< EntityGraphics*, QPointF> > selectedItems;
            if (!ctrlPress)
                selectedItems = _itemSelected;
            else
                selectedItems = _selectedItemsWithCtrl;
            if (selectedItems.isEmpty())
                return;
            qreal xOffset=event->x() + horizontalScrollBar()->value() +_translationPosition.x();
            qreal yOffset=event->y() + verticalScrollBar()->value() + _translationPosition.y();
            bool invertible;
            QMatrix matrixe = matrix().inverted(&invertible);
            if (invertible) {
                QPoint newPos = (matrixe.map(QPointF(xOffset,yOffset)).toPoint() /10.0)*10 ;
                ensureVisible(QRectF(newPos.x(),newPos.y(), _currentEntityGraphics.second->boundingRect().width(),_currentEntityGraphics.second->boundingRect().height() ),0,0);
                QMapIterator<qlonglong, QPair<EntityGraphics *, QPointF> > iterator(selectedItems);
                while(iterator.hasNext()) {
                    iterator.next();
                    iterator.value().first->setPos(newPos + iterator.value().second);
                }
                if (selectedItems.size() == 1) {
                    EntityGraphics *entityGraphics = selectedItems.values().first().first;
                    entityGraphics->handleNearWire();
                }
            }
            _updateCoordonnate(event);
        } else {
            ZoomingView::mouseMoveEvent(event);
        }
    }

    if (ctrlPress) {
        _mouseMovedWithCtrl = true;
        QMapIterator<qlonglong, QPair<EntityGraphics *, QPointF> > iterator(_selectedItemsWithCtrl);
        while (iterator.hasNext()) {
            iterator.next();
            iterator.value().first->topLevelItem()->setSelected(true);
        }
    }
}

/**
  * Handles when the mouse button is released
  */
void TopologyView::mouseReleaseEvent(QMouseEvent* event)
{
    bool ctrlPress = ((event->modifiers() & Qt::ControlModifier) == Qt::ControlModifier );
    _itemSelected.clear();
    _currentEntityGraphics.second = NULL;
    _currentEntityGraphics.first = 0;
    _mouseMoved = false;
    QList<QGraphicsItem *>selectedItemList = scene()->selectedItems();
    if (selectedItemList.size()) {
        foreach(QGraphicsItem *item, selectedItemList) {
            EntityGraphics * entityGraphics = dynamic_cast<EntityGraphics *>(item);
            if (entityGraphics) {
                entityGraphics->checkIsLayoutChanged();
                entityGraphics->redrawWires();
                foreach (WireGraphics *wire, entityGraphics->wires()) {
                    wire->checkIsLayoutChanged();
                }
            }
        }
    }
    if (selectedItemList.size() == 1) {
        EntityGraphics *entityGraphics = dynamic_cast<EntityGraphics *>(selectedItemList.at(0));
        if (entityGraphics) {
            entityGraphics->handleNearWire(true);
            if (!entityGraphics->entity()->getAvailablePlugs().isEmpty())
                emit wirePluggingRequested(entityGraphics->getEntityID());
        }
    }
    if (event->button() == Qt::LeftButton) {
        if (_mode)
            topologyScene()->endWireLine();
    }
    else if (event->button() == Qt::RightButton && _panningOrigin.isNull()) {
        if (selectedItemList.size())
            emit contextMenuRequested(event->globalPos());
    }
    ZoomingView::mouseReleaseEvent(event);

    if (ctrlPress) {
        foreach (QGraphicsItem * item, selectedItemList) {
            EntityGraphics * graphicItem = dynamic_cast<EntityGraphics *>(item);
            if (graphicItem)
                _selectedItemsWithCtrl.insert(graphicItem->getEntityID(), qMakePair(graphicItem, QPointF()));
        }
        QMapIterator<qlonglong, QPair<EntityGraphics *, QPointF> > iterator(_selectedItemsWithCtrl);
        while (iterator.hasNext()) {
            iterator.next();
            iterator.value().first->topLevelItem()->setSelected(true);
        }
        if (!_mouseMovedWithCtrl) {
            if (_unselectedItemWithCtrl.first != 0) {
                _unselectedItemWithCtrl.second->topLevelItem()->setSelected(false);
                if (_selectedItemsWithCtrl.contains(_unselectedItemWithCtrl.first))
                    _selectedItemsWithCtrl.remove(_unselectedItemWithCtrl.first);
                _unselectedItemWithCtrl.first = 0;
            }
        }
    }
}

void TopologyView::wheelEvent(QWheelEvent *event)
{
    if(!_currentEntityGraphics.second) {
        ZoomingView::wheelEvent(event);
    }
}

void TopologyView::drawBackground(QPainter *painter, const QRectF &rect)
{
    QBrush brush = backgroundBrush();
    if (_isModel) {
        QImage image(Ressources::themedPath(Constants::GRAPHIC_MODEL));
        QPainter backgroundPainter(&image);
        backgroundPainter.setCompositionMode(QPainter::CompositionMode_Multiply);
        backgroundPainter.fillRect(image.rect(), brush.color());
        //brush.setTexture(QBitmap::fromImage(image, Qt::MonoOnly));
        brush.setTexture(QPixmap::fromImage(image));
        brush.setTransform(transform().inverted());
    }
    painter->fillRect(rect, Qt::white);
    painter->fillRect(rect, brush);

    ZoomingView::drawBackground(painter, rect);
}

void TopologyView::_initView()
{
    ZoomingView::_initView();
    setAcceptDrops(true);
    TopologyScene * sc = new TopologyScene(0, 0, 500, 500, this);
    setScene(sc);
    setCenter(sc->sceneRect().center());
    setTransformationAnchor(AnchorUnderMouse);
    setDragMode(QGraphicsView::RubberBandDrag);
    setViewportUpdateMode(FullViewportUpdate);
    setCacheMode(QGraphicsView::CacheBackground);
    setOptimizationFlag(QGraphicsView::DontSavePainterState, true);
    setFrameStyle(0);
    setAttribute(Qt::WA_TranslucentBackground, false);
    showOutline(true);
}


bool TopologyView::isModel(){
    return _isModel;
}

void TopologyView::_checkDeleted(qlonglong ID)
{
    if (_currentEntityGraphics.first == ID) {
        _currentEntityGraphics.second = NULL;
        _currentEntityGraphics.first = 0;
        _itemSelected.clear();
    }
    _itemSelected.remove(ID);
    _selectedItemsWithCtrl.remove(ID);
}

} // namespace Hnv
