/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "topologyviewarea.h"
#include "ui_topologyviewarea.h"
#include "overlayexpandablewidget.h"
#include "topologyscene.h"
#include "entitypluggingdialog.h"
#include "wireplugtoentitydialog.h"
#include "entitygraphics.h"
#include "remotemanager.h"
#include "hyneviewsession.h"
#include "wireplugselector.h"
#include "zoompreview.h"
#include "loadbalancerdialog.h"
#include "capturejobdialog.h"
#include "snapshotmanager.h"
#include "snapshotcreationdialog.h"
#include "usbmanagerdialog.h"
#include "entityaction.h"
#include "ressources.h"
#include "wireaddtocatalogdialog.h"
#include "memorydumpdialog.h"

// hynesim common headers
#include "common/canimplementmodel.h"

// hynesim hnsclient headers
#include <libhnsclient/session.h>
#include <libhnsclient/entityentrycatalog.h>
#include <libhnsclient/entity.h>
#include <libhnsclient/topology.h>
#include <libhnsclient/resourcemanager.h>
#include <libhnsclient/utility.h>
#include <libhnsclient/entry.h>
#include <libhnsclient/platformmonitor.h>

// Qt headers
#include <qmath.h>
#include <QSize>
#include <QDir>
#include <QBuffer>
#include <QList>
#include <QSettings>
#include <QDateTime>
#include <QDebug>
#include <qmath.h>
#include <QPrinter>
#include <QPrintDialog>
#include <QPainter>
#include <QRectF>
#include <QGraphicsScene>
#include <QImage>
#include <QBrush>
#include <QMessageBox>
#include <QPushButton>

namespace Hnv {

/*******************************************************************************
* NodeContentItem
*******************************************************************************/
NodeContentItem::NodeContentItem(QString nodeName, int column, QObject *parent)
    : TreeModelItem(column, parent)
    , _nodeName(nodeName)
{
}

int NodeContentItem::columnCount() const
{
    if (_parentItem)
        return _parentItem->columnCount();
    else
        return TreeModelItem::columnCount();
}


QVariant NodeContentItem::data(int column, int role) const
{
    if (role == Qt::DecorationRole)
        return QIcon(Ressources::themedPath(Constants::ICON_NODE));
    else if (role == Qt::UserRole)
        return _nodeName;
    else if (column == 0 && (role == Qt::DisplayRole || role == Qt::EditRole)) {
        QString result = _nodeName;
        if (result.isEmpty())
            result = tr("Undefined");
        result.append(QString("(%1)").arg(QString::number(childCount())));
        return result;
    }
    return TreeModelItem::data(column, role);
}

int NodeContentItem::internalType() const
{
    return TopologyContentModel::NodeItem;
}

/*******************************************************************************
* EntityContentItem
*******************************************************************************/
EntityContentItem::EntityContentItem(HnsClient::GenericEntity *entity, QObject *parent)
    : TreeModelItem(2, parent)
{
    _entityItem.entity = entity;
    _entityItem.className = entity->iceId();
    _entityItem.id = entity->getId();
    _entityItem.shortName = entity->getShortName();
    _entityItem.ice_ids = QVector<std::string>::fromStdVector(entity->getHnsEntry()->ice_ids());
    QString className = _entityItem.className;
    QVector<std::string> ice_ids = _entityItem.ice_ids;
    _pixmap = Ressources::getClassNamePixmap(className, false, false, ice_ids);
    connect(_entityItem.entity, SIGNAL(destroyed()), this, SLOT(_entityDestroyed()));
    connect(_entityItem.entity, SIGNAL(shortNameChanged()), this, SLOT(_entityShortNameChanged()));
}

int EntityContentItem::columnCount() const
{
    if (_parentItem)
        return _parentItem->columnCount();
    else
        return TreeModelItem::columnCount();
}

QVariant EntityContentItem::data(int column, int role) const
{
    if (role == Qt::UserRole)
        return _entityItem.id;
    else if (column == 0) {
        if (role == Qt::DecorationRole)
            return QIcon(_pixmap);
        else if (role == Qt::DisplayRole || role == Qt::EditRole) {
            return _entityItem.shortName;
        }
    }
    return TreeModelItem::data(column, role);
}

int EntityContentItem::internalType() const
{
    return TopologyContentModel::EntityItem;
}

HnsClient::GenericEntity *EntityContentItem::getEntity()
{
    return _entityItem.entity;
}

void EntityContentItem::_entityDestroyed()
{
    _entityItem.entity = 0;

}

void EntityContentItem::_entityShortNameChanged()
{
    _entityItem.shortName = _entityItem.entity->getShortName();
}

/*******************************************************************************
* TopologyContentModel
*******************************************************************************/
TopologyContentModel::TopologyContentModel(TopologyViewArea *sourceArea, QObject *parent)
    : TreeModel(parent)
    , _sourceArea(sourceArea)
{
    // Tree model initialization with specified columns number
    _initRootItem(1);
    connect(_sourceArea, SIGNAL(loadingEnded(qlonglong)), this, SLOT(_loadingEndedSlot()));
    connect(_sourceArea, SIGNAL(entityCreated(HnsClient::GenericEntity*)), this, SLOT(_entityAddedSlot(HnsClient::GenericEntity*)));
    connect(_sourceArea, SIGNAL(entityChanged(HnsClient::GenericEntity*)), this, SLOT(_entityChangedSlot(HnsClient::GenericEntity*)));
    connect(_sourceArea, SIGNAL(entityRemoved(qlonglong)), this, SLOT(_entityRemovedSlot(qlonglong)));
    _populateModel();
}

 TopologyContentModel::~TopologyContentModel()
 {
     beginResetModel();
     _entityHash.clear();
     _nodeHash.clear();
     _rootItem->clear();
     endResetModel();
 }

 HnsClient::GenericEntity *TopologyContentModel::getEntityFromId(qlonglong entityID)
 {
     return _entityHash[entityID]->getEntity();
 }

void TopologyContentModel::_loadingEndedSlot()
{
    _populateModel();
}

void TopologyContentModel::_entityAddedSlot(HnsClient::GenericEntity *entity)
{
    _addEntity(entity);
}

void TopologyContentModel::_entityChangedSlot(HnsClient::GenericEntity *entity)
{
    qlonglong entityID = entity->getId();
    if (_entityHash.contains(entityID)) {
        EntityContentItem *item = _entityHash.value(entityID);
        QString previousNode = item->parentItem()->data(0, Qt::UserRole).toString();
        QString currentNode = entity->getNodeName();
        if (previousNode == currentNode)
            _itemChanged(item);
        else {
            removeItem(item);
            _attachEntityItem(currentNode, item);
            _checkNodeChildren(previousNode);
        }
    } else
        _addEntity(entity);
}

void TopologyContentModel::_entityRemovedSlot(qlonglong entityID)
{
    EntityContentItem *item = _entityHash.take(entityID);
    if (item) {
        QString previousNode = item->parentItem()->data(0, Qt::UserRole).toString();
        removeItem(item);
        delete(item);
        _checkNodeChildren(previousNode);
    }
}

void TopologyContentModel::_populateModel()
{
    QHash<qlonglong, HnsClient::GenericEntity*> entities = _sourceArea->getEntities();
    foreach(HnsClient::GenericEntity *entity, entities)
        _addEntity(entity);
}

void TopologyContentModel::_itemChanged(TreeModelItem *item)
{
    QModelIndex firstIdx = indexFromNode(item);
    QModelIndex lastIdx = indexFromNode(item, item->columnCount() - 1);
    emit dataChanged(firstIdx, lastIdx);
}

void TopologyContentModel::_addNode(QString nodeName)
{
    if (!_nodeHash.contains(nodeName)) {
        NodeContentItem *newItem = new NodeContentItem(nodeName, 1, this);
        _nodeHash.insert(nodeName, newItem);
        addItem(_rootItem, newItem);
    }
}

void TopologyContentModel::_attachEntityItem(QString nodeName, EntityContentItem *item)
{
    if (!_nodeHash.contains(nodeName))
        _addNode(nodeName);
    NodeContentItem *parentItem = _nodeHash.value(nodeName);
    if (parentItem) {
        addItem(parentItem, item);
        emit focusOnItem(indexFromNode(item), false);
    }
}

void TopologyContentModel::_checkNodeChildren(QString nodeName)
{
    if (_nodeHash.contains(nodeName)) {
        NodeContentItem *item = _nodeHash.value(nodeName);
        if (item->childCount() == 0) {
            _nodeHash.remove(nodeName);
            removeItem(item);
            delete(item);
        }
    }
}

void TopologyContentModel::_addEntity(HnsClient::GenericEntity *entity)
{
    qlonglong entityID = entity->getId();
    if (!_entityHash.contains(entityID)) {
        EntityContentItem *newItem = new EntityContentItem(entity, this);
        _entityHash.insert(entityID, newItem);
        _attachEntityItem(entity->getNodeName(), newItem);
    } else
        qWarning() << tr("Could not add entity to content model. ID already used.");
}

void TopologyContentModel::_focusItem(TreeModelItem *item)
{
    QModelIndex itemIdx = indexFromNode(item);
    emit focusOnItem(itemIdx);
}


/*******************************************************************************
* TopologyViewArea
*******************************************************************************/
TopologyViewArea::TopologyViewArea(HnsClient::Topology *topology, QWidget *parent)
    : QWidget(parent)
    , _ui(new Ui::TopologyViewArea)
    , _topology(topology)
    , _statusIconDisplayed(true)
    , _contentModel(0)
{
    _ui->setupUi(this);
    _contentModel = new TopologyContentModel(this, this);

    _updateLayout(topology->layout(), true);

    connect(_topology, SIGNAL(contentLoaded()), this, SLOT(_contentLoadedSlot()));
    connect(_topology, SIGNAL(topologyChanged()), this, SLOT(_topologyChangedSlot()));
    connect(_topology, SIGNAL(topologyLayoutChanged()), this, SLOT(_topologyLayoutChangedSlot()));
    connect(_topology, SIGNAL(topologyLayoutOptionsChanged()), this, SLOT(_topologyLayoutOptionsChangedSlot()));
    connect(_topology, SIGNAL(entityCreated(HnsClient::Entity*)), this, SLOT(_entityCreatedSlot(HnsClient::Entity*)));
    connect(_topology, SIGNAL(entityEntryChanged(HnsClient::Entity*)), this, SLOT(_entityEntryChangedSlot(HnsClient::Entity*)));
    connect(_topology, SIGNAL(entityStatusChanged(HnsClient::Entity*)), this, SLOT(_entityStatusChangedSlot(HnsClient::Entity*)));
    connect(_topology, SIGNAL(entityRefreshed(HnsClient::Entity*)), this, SLOT(_entityRefreshedSlot(HnsClient::Entity*)));
    connect(_topology, SIGNAL(entityRemoved(qlonglong)), this, SIGNAL(entityRemoved(qlonglong)));
    connect(_topology, SIGNAL(entityRemoved(qlonglong)), this, SLOT(_entityRemovedSlot(qlonglong)));
    connect(_topology, SIGNAL(entityRemoved(qlonglong)), _ui->graphicsView, SLOT(_checkDeleted(qlonglong)), Qt::DirectConnection);
    connect(_topology, SIGNAL(entityPlugged(qlonglong,qlonglong)), this, SLOT(_entityPluggedSlot(qlonglong,qlonglong)));
    connect(_topology, SIGNAL(entityUnplugged(qlonglong,qlonglong,int)), this, SLOT(_entityUnpluggedSlot(qlonglong,qlonglong,int)));
    connect(_topology, SIGNAL(wireCreated(HnsClient::WireEntity*)), this, SLOT(wireCreatedSlot(HnsClient::WireEntity*)));
    connect(_topology, SIGNAL(wireRemoved(qlonglong)), this, SLOT(_wireRemovedSlot(qlonglong)));

    _overlayDetails = new OverlayExpandableWidget(tr("Overview"), _ui->graphicsView);
    _overlayDetails->setAttachCorner(Qt::TopRightCorner);
    ZoomPreviewView * control = new ZoomPreviewView(this);
    control->setLinkedView(_ui->graphicsView);
    _overlayDetails->setCentralWidget(control);

    connect(_ui->graphicsView, SIGNAL(cursorPositionChanged(int,int,bool)), this, SIGNAL(cursorPositionChanged(int,int,bool)));
    connect(_ui->graphicsView, SIGNAL(zoomValueChanged(int)), this, SIGNAL(zoomValueChanged(int)));

    connect(_ui->graphicsView, SIGNAL(contextMenuRequested(QPointF)), this, SIGNAL(contextMenuRequested(QPointF)));
    connect(_ui->graphicsView, SIGNAL(requestEntityCreation(qlonglong, QPoint, EntityGraphics *)), this, SLOT(_requestEntitySlot(qlonglong, QPoint, EntityGraphics *)));
    connect(_ui->graphicsView, SIGNAL(requestTopologyImport(qlonglong, QPoint)), this, SLOT(_requestTopologyImportSlot(qlonglong, QPoint)));
    connect(_ui->graphicsView, SIGNAL(requestWireEntityItem(QPointF , QPointF )), this, SLOT(_requestWireEntitySlot(QPointF , QPointF )));
    connect(_ui->graphicsView, SIGNAL(wirePluggingRequested(qlonglong)), this, SLOT(_wirePluggingRequestedSlot(qlonglong)));

    connect(RemoteAccess::RemoteManager::instance(), SIGNAL(usbDeviceReset(int,bool)), this, SLOT(_usbDevicesResetSlot(int,bool)));

    _ui->graphicsView->setIsWriteable(_topology->topologyEntry()->isWriteable());
    _ui->graphicsView->setIsModel(_topology->topologyEntry()->isModel());
    topology->loadContent();
}

TopologyViewArea::~TopologyViewArea()
{
    delete _contentModel;
    _pendingEntityCreation.clear();
    delete _ui;
    delete _overlayDetails;
}

void TopologyViewArea::destroyArea()
{
    foreach (QObject *o, children()) {
        QDialog *dialog = qobject_cast<QDialog *>(o);
        if (dialog)
            dialog->reject();
    }
    _topology->unloadContent();
    _pendingEntityCreation.clear();
    deleteLater();
}

HnsClient::Topology *TopologyViewArea::topology()
{
    return _topology;
}

HnsClient::TopologyEntry * TopologyViewArea::topologyEntry()
{
    if (_topology->topologyEntry())
        return _topology->topologyEntry();
    return 0;
}

TopologyView * TopologyViewArea::topologyView()
{
    return _ui->graphicsView;
}

QGraphicsScene * TopologyViewArea::scene()
{
    return _ui->graphicsView->topologyScene();
}

QHash<qlonglong, HnsClient::GenericEntity*> TopologyViewArea::getEntities(bool includeWires)
{
    return _topology->getEntities(includeWires);
}

QList<HnsClient::GenericEntity *> TopologyViewArea::getSelectedEntities(bool includeWires)
{
    QList<HnsClient::WireEntity*> wires;
    QList<HnsClient::GenericEntity*> entities;
    getSelectedEntities(entities, wires, includeWires);
    return entities;
}

void TopologyViewArea::getSelectedEntities(QList<HnsClient::GenericEntity *> &entities, QList<HnsClient::WireEntity *> &wires, bool wireAsEntity)
{
    QList<QGraphicsItem*> graphicsItemList = scene()->selectedItems();
    QHash<qlonglong, HnsClient::GenericEntity*> entityHash = _topology->getEntities();
    QHash<qlonglong, HnsClient::GenericEntity*> wireHash = _topology->getWires();
    foreach(QGraphicsItem* item, graphicsItemList) {
        GraphicsItem* graphics = dynamic_cast<GraphicsItem*>(item);
        if (graphics) {
            qlonglong entityID = graphics->getEntityID();
            if (entityHash.contains(entityID))
                entities.append(entityHash.value(entityID));
            else if (wireHash.contains(entityID)) {
                if (wireAsEntity)
                    entities.append(wireHash.value(entityID));
                HnsClient::WireEntity * wire = dynamic_cast<HnsClient::WireEntity *>(wireHash.value(entityID));
                if (wire)
                    wires.append(wire);
            }
        }
    }
}

bool TopologyViewArea::isEntitySelected(qlonglong entityID)
{
    QList<QGraphicsItem*> graphicsItemList = scene()->selectedItems();
    foreach(QGraphicsItem* item, graphicsItemList) {
        GraphicsItem* graphics = dynamic_cast<GraphicsItem*>(item);
        if (graphics) {
            qlonglong selectedID = graphics->getEntityID();
            if (entityID == selectedID)
                return true;
        }
    }
    return false;
}

TopologyContentModel * TopologyViewArea::getContentModel()
{
    return _contentModel;
}

HnsClient::Entity *TopologyViewArea::getEntityByPosition(QPointF position)
{
    QList<QGraphicsItem *> items = scene()->items(position);
    QMutableListIterator<QGraphicsItem *> it(items);
    while (it.hasNext()) {
        it.next();
        QGraphicsItem *item = it.value();
        EntityGraphics *graphics = dynamic_cast<EntityGraphics *>(item);
        if (!graphics)
            it.remove();
        else
            return qobject_cast<HnsClient::Entity *>(graphics->entity());
    }
    return 0;
}

EntityGraphics *TopologyViewArea::getEntityGraphicsByPosition(QPointF position)
{
    QList<QGraphicsItem *> items = scene()->items(position);
    QMutableListIterator<QGraphicsItem *> it(items);
    while (it.hasNext()) {
        it.next();
        QGraphicsItem *item = it.value();
        EntityGraphics *graphics = dynamic_cast<EntityGraphics *>(item);
        if (!graphics)
            it.remove();
        else
            return graphics;
    }
    return 0;
}

QList<EntityGraphics *> TopologyViewArea::getEntitiesGraphicsByPosition(QPointF position)
{
    QList<QGraphicsItem *> items = scene()->items(position);
    QList<EntityGraphics *> ret;
    foreach (QGraphicsItem *item, items) {
        EntityGraphics *entity = dynamic_cast<EntityGraphics *>(item);
        if (entity)
            ret.append(entity);
    }
    return ret;
}

QHash<qlonglong, WireGraphics *> TopologyViewArea::wireGraphics() const
{
    return _wireGraphicsHash;
}

QHash<qlonglong, EntityGraphics *> TopologyViewArea::entityGraphics() const
{
    return _entityGraphicsHash;
}

void TopologyViewArea::setZoomValue(int value)
{
    _ui->graphicsView->setZoomValue(value);
}

void TopologyViewArea::setInsertMode(TopologyView::InsertMode insertMode)
{
    _ui->graphicsView->setInsertMode(insertMode);
}

void TopologyViewArea::setWireColor(QColor color)
{
    _ui->graphicsView->setWireColor(color);
}

void TopologyViewArea::setSelectedEntity(qlonglong entityID)
{
    if (entityID == -1)
        return;
    else if (entityID == 0) {
        scene()->clearSelection();
        return;
    }
    QList<HnsClient::GenericEntity *> entities = getSelectedEntities();
    if (entities.size() == 1 && entities.first()->getId() == entityID) {
        return;
    }
    if (_entityGraphicsHash.contains(entityID)) {
        scene()->clearSelection();
        _entityGraphicsHash.value(entityID)->setSelected(true);
        centerOnEntity(_entityGraphicsHash.value(entityID));
    }
    else
        qCritical() << tr("Entity %1 was not found. Unable to select it.").arg(entityID);
}

void TopologyViewArea::setSelectedEntities(QList<qlonglong> entityIDList)
{
    if (entityIDList.isEmpty()) {
        scene()->clearSelection();
        return;
    }

    QHash<qlonglong, HnsClient::GenericEntity*> entityHash = _topology->getEntities();
    QList<qlonglong> validIDs;
    foreach(qlonglong entityID, entityIDList) {
        if (entityHash.contains(entityID))
            validIDs.append(entityID);
    }
    if (validIDs.isEmpty())
        return;

    scene()->clearSelection();

    qreal left = 0;
    qreal right = 0;
    qreal top = 0;
    qreal bottom = 0;
    bool firstGraphics = true;

    foreach(qlonglong entityID, validIDs) {
        _entityGraphicsHash.value(entityID)->setSelected(true);
        EntityGraphics *entity = dynamic_cast<EntityGraphics *>(_entityGraphicsHash.value(entityID));
        if (entity) {
            QRectF graphicsRect = entity->mapRectToScene(entity->boundingRect());
            qreal entityLeft = graphicsRect.left();
            qreal entityRight = graphicsRect.right();
            qreal entityTop = graphicsRect.top();
            qreal entityBottom = graphicsRect.bottom();

            if (firstGraphics) {
                left = entityLeft;
                right = entityRight;
                top = entityTop;
                bottom = entityBottom;
                firstGraphics = false;
            } else {
                left = qMin(left, entityLeft);
                right = qMax(right, entityRight);
                top = qMin(top, entityTop);
                bottom = qMax(bottom, entityBottom);
            }
        }
    }
    QRectF targetRect = QRectF(QPointF(left, top), QPointF(right, bottom));
    _ui->graphicsView->matchZoomToRect(targetRect);
}

void TopologyViewArea::centerOnEntity(qlonglong entityID)
{
    if (_entityGraphicsHash.contains(entityID))
        centerOnEntity(_entityGraphicsHash.value(entityID));
}

void TopologyViewArea::centerOnEntity(EntityGraphics * entity)
{
    if (entity)
        _ui->graphicsView->centerOn(entity);
}

TopologyView::InsertMode TopologyViewArea::insertMode()
{
    return _ui->graphicsView->insertMode();
}

void TopologyViewArea::defineSelected(QString nodeName)
{
    foreach(HnsClient::GenericEntity *entity, getSelectedEntities(false))
        entity->define(nodeName);
}

void TopologyViewArea::startSelected()
{
    foreach(HnsClient::GenericEntity *entity, getSelectedEntities(false))
        entity->start();
}

void TopologyViewArea::suspendSelected()
{
    foreach(HnsClient::GenericEntity *entity, getSelectedEntities(false))
        entity->suspend();
}

void TopologyViewArea::saveSelected()
{
    foreach(HnsClient::GenericEntity *entity, getSelectedEntities(false))
        entity->save();
}

void TopologyViewArea::undefineSelected()
{
    foreach(HnsClient::GenericEntity *entity, getSelectedEntities(false))
        entity->undefine();
}

void TopologyViewArea::unImplementSelected()
{
    foreach(HnsClient::GenericEntity *entity, getSelectedEntities())
        entity->unImplement();
}

void TopologyViewArea::stopSelected()
{
    foreach(HnsClient::GenericEntity *entity, getSelectedEntities(false))
        entity->stop();
}

void TopologyViewArea::remoteDisplaySelected()
{
    foreach(HnsClient::GenericEntity *entity, getSelectedEntities(false)) {
        RemoteAccess::RemoteManager::instance()->newConnection(entity);
    }
}

void TopologyViewArea::displayPropertiesById(qlonglong itemId)
{
    GraphicsItem *graphic = 0;
    if (_entityGraphicsHash.contains(itemId))
        graphic = _entityGraphicsHash.value(itemId);
    else if (_wireGraphicsHash.contains(itemId))
        graphic = _wireGraphicsHash.value(itemId);
    if (graphic)
        graphic->displayProperties(this);
}

void TopologyViewArea::displayPropertiesSelected()
{
    QList<QGraphicsItem *> graphicsItemList = scene()->selectedItems();
    if (graphicsItemList.size() == 1) {
        QGraphicsItem *item = graphicsItemList.at(0);
        GraphicsItem *graphics = dynamic_cast<GraphicsItem *>(item);
        if (graphics) {
            graphics->displayProperties(this);
        }
    }
}

void TopologyViewArea::deleteSelected()
{
    QList<HnsClient::WireEntity *> wires;
    QList<HnsClient::GenericEntity *> entities;
    getSelectedEntities(entities, wires);
    int startedEntities = 0;
    foreach (HnsClient::GenericEntity *entity, entities) {
        if (entity->isStarted())
            ++startedEntities;
    }
    bool deletionConfirmed = true;
    if (startedEntities > 0) {
        QString message;
        if (startedEntities == 1)
            message = tr("The selected entity is started, are you sure to delete it ?");
        else
            message = tr("Some selected entities are started, are you sure to delete them ?");
        int result = QMessageBox::question(this, tr("Entity deletion"), message, QMessageBox::Yes | QMessageBox::No, QMessageBox::Yes);
        switch(result) {
        case QMessageBox::Yes:
            break;
        case QMessageBox::No:
        default:
            deletionConfirmed = false;
            break;
        }
    }
    if (!deletionConfirmed)
        return;
    foreach(HnsClient::WireEntity *wire, wires)
        wire->destroy();
    // TODO : might need to be wait for wire to be destroyed before proceeding
    foreach(HnsClient::GenericEntity *entity, entities)
        entity->destroy();
}


void TopologyViewArea::deleteItemsByIDs(QList<qlonglong> itemList)
{
    foreach(HnsClient::GenericEntity *entity, _topology->getEntities(true)) {
        if (itemList.contains(entity->getId()))
            entity->destroy();
    }
}

void TopologyViewArea::_displayPrintDialog(QGraphicsScene *scene, QRect sourceRect, QList<QGraphicsItem *> itemsToHide)
{
    QPrinter printer;
    printer.setResolution(Constants::DEFAULT_IMPRESSION_RESOLUTION);
    if (sourceRect.width() > sourceRect.height())
        printer.setOrientation(QPrinter::Landscape);
    else
        printer.setOrientation(QPrinter::Portrait);
    QPrintDialog printDialog(&printer, this);
    HnvDialogHandler *popupHandler = new HnvDialogHandler(topology(), &printDialog, this);
    popupHandler->setSourceDestroyedMessage(tr("\"%1\" was deleted. Operation aborted.").arg(topologyEntry()->getShortName()));
    if(printDialog.exec()){
        QPainterPath selectedArea = scene->selectionArea();
        scene->clearSelection();
        foreach (QGraphicsItem* i, itemsToHide)
            i->hide();
        QPainter painter;
        painter.begin(&printer);
        QRectF outputRect;
        QRectF inputRect(sourceRect);
        scene->render(&painter, outputRect, inputRect);
        painter.end();
        foreach (QGraphicsItem* i, itemsToHide)
            i->show();
        scene->setSelectionArea(selectedArea);
    }
}

void TopologyViewArea::_displaySnapshotDialog(QGraphicsScene *scene, QRect sourceRect, QList<QGraphicsItem *> itemsToHide)
{
    QHash<QString,QString> hashFormats;
    hashFormats.insert("JPG","JPG");
    hashFormats.insert("GIF","GIF");
    hashFormats.insert("BMP","BMP");
    hashFormats.insert("JPEG","JPEG");
    hashFormats.insert("PNG","PNG");
    hashFormats.insert("PBM","PBM");
    hashFormats.insert("PGM","PGM");
    hashFormats.insert("PPM","PPM");
    hashFormats.insert("TIFF","TIFF");
    hashFormats.insert("XBM","XBM");
    hashFormats.insert("XPM","XPM");

    QDateTime dateTime;
    dateTime = dateTime.currentDateTime();
    QString fileName = dateTime.toString("yyyyMMdd-hhmmss")+QString(".");

    QSettings settings;
    settings.beginGroup(Constants::SNAPSHOT_SETTINGS);
    QList<QString> childrenKeys = settings.childKeys();
    QString fileFormat = settings.value(Constants::SNAPSHOT_FILE_FORMAT,QVariant(QString("jpg"))).toString().toUpper();
    if (!hashFormats.keys().contains(fileFormat) || !childrenKeys.contains(Constants::SNAPSHOT_FILE_FORMAT)) {
        fileFormat = "JPG";
    }
    QDir locationFolder(settings.value(Constants::SNAPSHOT_FOLDER).toString());
    if (!locationFolder.exists() ||locationFolder.path()=="."|| !childrenKeys.contains("snapshotFolder")) {
        QString path = QDir::homePath()+QDir::separator()+"Hyneview"+QDir::separator()+"Images";
        locationFolder.mkpath(path);
        locationFolder.setPath(path);
    }
    bool originalSize = true;
    int snapshotWidth = -1;
    int snapshotHeight = -1;

    if (childrenKeys.contains(QString(Constants::SNAPSHOT_ORIGINAL_SIZE))) {
        originalSize = settings.value(Constants::SNAPSHOT_ORIGINAL_SIZE,QVariant(true)).toBool();
    }
    if (originalSize ==false) {
        if (childrenKeys.contains(QString(Constants::SNAPSHOT_WIDTH)) && childrenKeys.contains(QString("snapshotHeight"))) {
            snapshotWidth = settings.value(Constants::SNAPSHOT_WIDTH,QVariant(-1)).toInt();
            snapshotHeight = settings.value(Constants::SNAPSHOT_HEIGHT,QVariant(-1)).toInt();
            if (snapshotWidth == 0 || snapshotHeight == 0) {
                originalSize = true;
            }
        }
        else {
            originalSize = true;
        }
    }
    int snapshotQuality = settings.value(Constants::SNAPSHOT_QUALITY).toInt();
    if (snapshotQuality == 0) {
        snapshotQuality = 65;
    }
    snapshotQuality = (snapshotQuality<1) ? 1 : snapshotQuality;
    snapshotQuality = (snapshotQuality>100) ? 100 : snapshotQuality;
    settings.endGroup();

    QSize sizeTransformation(snapshotWidth,snapshotHeight);
    QPixmap pixmap;
    if (originalSize == true){
        pixmap = _generatingSnapshot(sourceRect.width(),sourceRect.height(),scene,sourceRect, itemsToHide);
    } else {
        pixmap = _generatingSnapshot(sizeTransformation.width(),sizeTransformation.height(),scene,sourceRect, itemsToHide);
    }
    QImage image = pixmap.toImage();
    bool succeed = image.save(QDir::toNativeSeparators(locationFolder.absolutePath()+QDir::separator())+fileName+fileFormat,fileFormat.toStdString().c_str(),snapshotQuality);
    if (succeed){
        qDebug()<<tr("Snapshot saved to %1").arg(QDir::toNativeSeparators(locationFolder.absolutePath()+QDir::separator())+fileName+fileFormat);
    } else {
        qDebug()<<tr("Could not save the snapshot");
    }
}
QPixmap TopologyViewArea::_generatingSnapshot(int width,int height, QGraphicsScene *scene, QRect sourceRect, QList<QGraphicsItem *> itemsToHide, bool centerCapture)
{
    QList<QGraphicsItem *> selectedItems = scene->selectedItems();
    scene->clearSelection();
    foreach (QGraphicsItem* i, itemsToHide)
        i->hide();
    QPixmap pixmap(width,height);
    QPainter painter;
    if (!pixmap.isNull()){
        painter.begin(&pixmap);
        pixmap.fill(topologyView()->backgroundBrush().color());
        QRectF outputRect;
        QRectF inputRect(sourceRect);
        if (pixmap.rect().contains(sourceRect.width(), sourceRect.height())){
            /*No resizing if the source is fully contained in the destination
            (i.e : The source is "inferior" to the destination*/
            outputRect.setWidth(inputRect.width());
            outputRect.setHeight(inputRect.height());
        }
        if (centerCapture) {
            qreal wRatio = qreal(width) / qreal(sourceRect.width());
            qreal hWratio = qreal(height) / qreal(sourceRect.height());
            qreal realRatio = qMin(wRatio, hWratio);
            qreal deltaX = (width - (sourceRect.width() * realRatio)) / 2;
            qreal deltaY = (height - (sourceRect.height() * realRatio)) / 2;
            painter.translate(deltaX, deltaY);
        }
        scene->render(&painter, outputRect, inputRect);
        painter.end();
    }
    foreach (QGraphicsItem* i, itemsToHide)
        i->show();
    foreach (QGraphicsItem *item, selectedItems)
        item->setSelected(true);
    return pixmap;
}

Hns::ByteArray TopologyViewArea::createPreviewSnapshot()
{
    int width = Constants::WIDTH_PREVIEW;
    int height = Constants::HEIGHT_PREVIEW;
    QList<QGraphicsItem*> itemsToHide;
    QPixmap pixmap = _generatingSnapshot(width, height,topologyView()->scene(), _getRectTopology(), itemsToHide, true);
    QByteArray byteArray;
    QBuffer buffer(&byteArray);
    buffer.open(QIODevice::WriteOnly);
    QImage image = pixmap.toImage();
    image.save(&buffer,Constants::FORMAT_PREVIEW,Constants::QUALITY_PREVIEW);
    const char * data = byteArray.constData();
    return Hns::ByteArray(data, data + byteArray.size());
}

QRect TopologyViewArea::_getRectTopology()
{
    return topologyView()->scene()->sceneRect().toRect();
}

QRect TopologyViewArea::_getRectSelection()
{
    return topologyView()->scene()->selectionArea().boundingRect().toRect();
}

QRect TopologyViewArea::_getRectView()
{
    return topologyView()->mapToScene(topologyView()->viewport()->rect()).boundingRect().toRect();
}

EntityGraphics *TopologyViewArea::_createEntityGraphics(HnsClient::Entity *entity)
{
    EntityGraphics *graphic = EntityGraphics::create(entity, _ui->graphicsView->topologyScene(), this, 0);
    if (graphic) {
        graphic->setLabelsDisplayedByDefault(_labelsDisplayed);
        _entityGraphicsHash.insert(entity->getId(), graphic);
        connect(graphic, SIGNAL(layoutChanged(Hns::EntityLayoutPtr)), this, SLOT(_entityGraphicLayoutChangedSlot(Hns::EntityLayoutPtr)));
        connect(this, SIGNAL(labelsDisplayedChanged(bool)), graphic, SLOT(setLabelsDisplayedByDefault(bool)));
        connect(this, SIGNAL(statusIconVisibilityChanged(bool)), graphic, SLOT(setStatusIconVisibility(bool)));
        graphic->setStatusIconVisibility(_statusIconDisplayed);
        graphic->applyLabelVisibility();
        graphic->setColorLabel(QString::fromStdString(_topology->layout().layoutOptions.colorCode));
    }
    return graphic;
}

void TopologyViewArea::snapshotTopology()
{
    QGraphicsScene *scene = topologyView()->scene();
    _displaySnapshotDialog(scene, _getRectTopology(), {});
}

void TopologyViewArea::snapshotSelection()
{
    QGraphicsScene *scene = topologyView()->scene();
    QList<QGraphicsItem *> items = scene->items();
    QList<QGraphicsItem *> selectedItems = scene->selectedItems();
    QList<QGraphicsItem *> itemsToHide;
    foreach (QGraphicsItem* i, items) {
        if (selectedItems.contains(i))
            continue;
        if ((i->parentItem()) && (selectedItems.contains(i->parentItem())))
            continue;
        if (i->isVisible()) {
            itemsToHide.append(i);
        }
    }
    QRect rectTopology = _getRectTopology();
    int minX = rectTopology.width();
    int minY = rectTopology.height();
    int maxX = 0;
    int maxY = 0;
    foreach (QGraphicsItem* item, selectedItems) {
        QRect itemSceneRect = item->sceneBoundingRect().toRect();
        minX = qMin(qMin(itemSceneRect.x(), itemSceneRect.x() + itemSceneRect.width()), minX);
        minY = qMin(qMin(itemSceneRect.y(), itemSceneRect.y() + itemSceneRect.height()), minY);
        maxX = qMax(qMax(itemSceneRect.x(), itemSceneRect.x() + itemSceneRect.width()), maxX);
        maxY = qMax(qMax(itemSceneRect.y(), itemSceneRect.y() + itemSceneRect.height()), maxY);
    }
    _displaySnapshotDialog(scene, QRect(QPoint(minX, minY), QPoint(maxX, maxY)), itemsToHide);
}

void TopologyViewArea::snapshotView()
{
    QGraphicsScene *scene = topologyView()->scene();
    _displaySnapshotDialog(scene, _getRectView(), {});
}

void TopologyViewArea::printTopology()
{
    QGraphicsScene *scene = topologyView()->scene();
    _displayPrintDialog(scene, _getRectTopology(), {});
}

void TopologyViewArea::printSelection()
{
    QGraphicsScene *scene = topologyView()->scene();
    QList<QGraphicsItem *> items = scene->items();
    QList<QGraphicsItem *> selectedItems = scene->selectedItems();
    QList<QGraphicsItem *> itemsToHide;
    foreach (QGraphicsItem* i, items) {
        if (selectedItems.contains(i))
            continue;
        if ((i->parentItem()) && (selectedItems.contains(i->parentItem())))
            continue;
        if (i->isVisible()) {
            itemsToHide.append(i);
        }
    }
    QRect rectTopology = _getRectTopology();
    int minX = rectTopology.width();
    int minY = rectTopology.height();
    int maxX = 0;
    int maxY = 0;
    foreach (QGraphicsItem* item, selectedItems) {
        QRect itemSceneRect = item->sceneBoundingRect().toRect();
        minX = qMin(qMin(itemSceneRect.x(), itemSceneRect.x() + itemSceneRect.width()), minX);
        minY = qMin(qMin(itemSceneRect.y(), itemSceneRect.y() + itemSceneRect.height()), minY);
        maxX = qMax(qMax(itemSceneRect.x(), itemSceneRect.x() + itemSceneRect.width()), maxX);
        maxY = qMax(qMax(itemSceneRect.y(), itemSceneRect.y() + itemSceneRect.height()), maxY);
    }
    _displayPrintDialog(scene, QRect(QPoint(minX, minY), QPoint(maxX, maxY)), itemsToHide);
}

void TopologyViewArea::printView()
{
    QGraphicsScene *scene = topologyView()->scene();
    _displayPrintDialog(scene, _getRectView(), {});
}

void TopologyViewArea::toggleEntityLabel(bool displayLabels)
{
    _labelsDisplayed = displayLabels;
    emit labelsDisplayedChanged(_labelsDisplayed);
}

void TopologyViewArea::triggerForceDisplayLabel(Hns::LabelPolicy labelPolicy)
{
    QList<HnsClient::GenericEntity *> entities = getSelectedEntities(false);
    foreach (HnsClient::GenericEntity *genericEntity, entities) {
        HnsClient::Entity *entity = qobject_cast<HnsClient::Entity *>(genericEntity);
        if (!entity)
            continue;
        Hns::EntityLayoutPtr entityLayout = entity->getLayout();
        if (entityLayout->labelPolicyState != labelPolicy) {
            entityLayout->labelPolicyState = labelPolicy;
            topology()->updateEntityLayout(entityLayout);
        }
    }
}

void TopologyViewArea::createSnapshot()
{
    if (getSelectedEntities().count() != 1){
        qDebug()<<tr("Not supported yet. Please select one entity");
        return;
    }
    HnsClient::DomainEntity *domainEntity= qobject_cast<HnsClient::DomainEntity *>(getSelectedEntities().first());
    if (domainEntity) {
        SnapshotCreationDialog popup(domainEntity, this, SLOT(_snapshotCreatedSlot(bool,QString)));
        HnvDialogHandler *popupHandler = new HnvDialogHandler(domainEntity, &popup, this);
        popupHandler->setSourceDestroyedMessage(tr("\"%1\" was deleted.").arg(domainEntity->getShortName()));
        popup.exec();
    }
}

void TopologyViewArea::setLabelsDisplayed(bool labelsDisplayed)
{
    _labelsDisplayed = labelsDisplayed;
}

void TopologyViewArea::setStatusIconVisibility(bool visible)
{
    if (_statusIconDisplayed != visible) {
        _statusIconDisplayed = visible;
        emit statusIconVisibilityChanged(_statusIconDisplayed);
    }
}

void TopologyViewArea::displayCaptureJob()
{
    if (getSelectedEntities().count() != 1){
        qDebug()<<tr("Not supported yet. Please select one entity");
        return;
    }
    HnsClient::GenericEntity *entity = getSelectedEntities().first();
    CaptureJobDialog popup(entity, this);
    HnvDialogHandler *popupHandler = new HnvDialogHandler(entity, &popup, this);
    popupHandler->setSourceDestroyedMessage(tr("\"%1\" was deleted.").arg(entity->getShortName()));
    popup.exec();
}

void TopologyViewArea::displaySnapshotManager()
{
    HnsClient::DomainEntity *domainEntity= qobject_cast<HnsClient::DomainEntity *>(getSelectedEntities().first());
    if (domainEntity) {
        SnapshotManager popup(domainEntity, this);
        HnvDialogHandler *popupHandler = new HnvDialogHandler(domainEntity, &popup, this);
        popupHandler->setSourceDestroyedMessage(tr("\"%1\" was deleted.").arg(domainEntity->getShortName()));
        popup.exec();
    }
}

void TopologyViewArea::displayUsbManager()
{
    HnsClient::DomainEntity *domainEntity= qobject_cast<HnsClient::DomainEntity *>(getSelectedEntities().first());
    if (domainEntity) {
        UsbManagerDialog popup(domainEntity, this);
        HnvDialogHandler *popupHandler = new HnvDialogHandler(domainEntity, &popup, this);
        popupHandler->setSourceDestroyedMessage(tr("\"%1\" was deleted.").arg(domainEntity->getShortName()));
        popup.exec();
    }
}

void TopologyViewArea::hideUsbIcon(int entityId)
{
    if (!_entityGraphicsHash.contains(entityId))
        return;
    _entityGraphicsHash.value(entityId)->showUsbIcon(false);
}

void TopologyViewArea::displayMemoryDumpDialog()
{
    if (getSelectedEntities().count() != 1)
        return;
    HnsClient::GenericEntity *entity = getSelectedEntities().first();
    MemoryDumpDialog popup(entity, this);
    HnvDialogHandler *popupHandler = new HnvDialogHandler(entity, &popup, this);
    popupHandler->setSourceDestroyedMessage(tr("\"%1\" was deleted.").arg(entity->getShortName()));
    popup.exec();
}

void TopologyViewArea::selectAll()
{
    QPainterPath path;
    path.addRect(scene()->sceneRect());
    scene()->setSelectionArea(path);
}

void TopologyViewArea::loadBalancerSelected()
{
    LoadBalancerDialog loadBalancerpopup(getSelectedEntities(false), _topology, this);
    HnvDialogHandler *popupHandler = new HnvDialogHandler(_topology, &loadBalancerpopup, this);
    popupHandler->setSourceDestroyedMessage(tr("\"%1\" was deleted. Operation aborted.").arg(_topology->topologyEntry()->getShortName()));
    loadBalancerpopup.exec();
}

void TopologyViewArea::loadBalanceAuto(Hns::BalancingBehavior behavior)
{
    QMap<Ice::Long, Hns::DefineInfo> defineInfos;
    QList<HnsClient::GenericEntity *>  listEntity = getSelectedEntities(false);
    foreach(HnsClient::GenericEntity * entity, listEntity){
        if(entity->isDefined() || entity->getNodeList().isEmpty()) continue;
        Hns::DefineInfo defineInfo;
        defineInfo.isDefined = true;
        qlonglong entityId = entity->getId();
        defineInfo.entityId = entityId;
         defineInfos.insert(entityId , defineInfo);
    }
    topology()->computeDefineInfos(defineInfos, behavior, this, SLOT(_computeAndDefineInfosSlot(::Hns::LongDefineInfoMap)));
}


/*******************************************************************************
* TopologyViewArea - Public slots
*/
void TopologyViewArea::toggleGrid(bool toggle)
{
    _ui->graphicsView->toggleGrid(toggle);
}

void TopologyViewArea::setDetailOverlayCorner(Qt::Corner corner)
{
    _overlayDetails->setAttachCorner(corner);
}

void TopologyViewArea::hideDetailOverlay(bool checked)
{
    _overlayDetails->setHidden(checked);
}

/*******************************************************************************
* TopologyViewArea - Private slots
*/
void TopologyViewArea::_topologyChangedSlot()
{
    _ui->graphicsView->setIsWriteable(_topology->topologyEntry()->isWriteable());
    emit topologyChanged(_topology);
}

void TopologyViewArea::_topologyLayoutChangedSlot()
{
    _updateLayout(_topology->layout());
}

void TopologyViewArea::_topologyLayoutOptionsChangedSlot()
{
    _updateLayout(_topology->layout());
    QList<QGraphicsItem*> graphicsItemList = scene()->items();
    QHash<qlonglong, HnsClient::GenericEntity*> entityHash = _topology->getEntities();

    foreach(QGraphicsItem* item, graphicsItemList) {
        GraphicsItem* entity = dynamic_cast<GraphicsItem*>(item);
        if (entity){
            qlonglong entityID = entity->getEntityID();
            if (entityHash.contains(entityID)){
                EntityGraphics* entityGraphic = dynamic_cast<EntityGraphics*>(entity);
                if (entityGraphic){
                    entityGraphic->setColorLabel(QString::fromStdString(_topology->layout().layoutOptions.colorCode));
                }
            }
        }
    }
}

void TopologyViewArea::_entityCreatedSlot(HnsClient::Entity *entity)
{
    qlonglong entryID = entity->getEntryId();
    EntityGraphics *graphic = _entityGraphicsHash.value(entity->getId());
    if (graphic)
        return;
    graphic = _createEntityGraphics(entity);
    if (graphic)
        graphic->setSelected(true);

    if (_pendingEntityCreation.contains(entryID))
        _pendingEntityCreation.removeAll(entryID);

    emit entityCreated(entity);
}

void TopologyViewArea::_entityEntryChangedSlot(HnsClient::Entity *entity)
{
    emit entityChanged(entity);
}

void TopologyViewArea::_entityStatusChangedSlot(HnsClient::Entity *entity)
{
    emit entityChanged(entity);
    if(entity->getStatus() == Hns::Status::UNDEFINED)
        emit entityStatusUndefined(entity->getId());
}

void TopologyViewArea::_entityRefreshedSlot(HnsClient::Entity *entity)
{
    emit entityChanged(entity);
}

void TopologyViewArea::_entityPluggedSlot(qlonglong entityId, qlonglong wireId)
{
    WireGraphics *wireGraphic = _wireGraphicsHash.value(wireId);
    EntityGraphics *entityGraphic = _entityGraphicsHash.value(entityId);
    if (!wireGraphic || !entityGraphic)
        return;
    //to or from ?
    QList<Hns::TopologyLink> links = topology()->getLinksByWireId(wireId);
    foreach (Hns::TopologyLink link, links) {
        qlonglong id = QString::fromStdString(link.entitySide).split(':').first().toLongLong();
        if (id != entityId)
            continue;
        int wirePort = QString::fromStdString(link.wireSide).split(':').last().toInt();
        if (wirePort == 0)
            wireGraphic->setFromEntityGraphics(entityGraphic);
        else if (wirePort == 1)
            wireGraphic->setToEntityGraphics(entityGraphic);
    }
}

void TopologyViewArea::_entityUnpluggedSlot(qlonglong entityId, qlonglong wireId, int wirePort)
{
    if (!_entityGraphicsHash.contains(entityId) || !_wireGraphicsHash.contains(wireId))
        return;
    _wireGraphicsHash[wireId]->unplugEntityGraphics(_entityGraphicsHash.value(entityId));
    if (_entityGraphicsHash[entityId]->handleTemporaryWires(wireId)) {
        Hns::TopologyLink link;
        link.entitySide = QString("%1:-1").arg(entityId).toStdString();
        link.wireSide = QString("%1:%2").arg(wireId).arg(wirePort).toStdString();
        _wireUnpluggingSlot(link);
    }
}

void TopologyViewArea::_entityRemovedSlot(qlonglong entityId)
{
    if (_entityGraphicsHash.contains(entityId))
        _entityGraphicsHash.remove(entityId);
}

void TopologyViewArea::_wireRemovedSlot(qlonglong wireId)
{
    if (_wireGraphicsHash.contains(wireId))
        scene()->removeItem(_wireGraphicsHash.take(wireId));
}

void TopologyViewArea::_entityGraphicLayoutChangedSlot(Hns::EntityLayoutPtr layout)
{
    _topology->updateEntityLayout(layout);
}

void TopologyViewArea::_wireGraphicLayoutChangedSlot(Hns::WireLayoutPtr layout)
{
    _topology->updateWireLayout(layout);
}

void TopologyViewArea::_requestEntitySlot(const qlonglong &entryID, const QPoint &scenePosition, EntityGraphics *overlappedEntity)
{
    bool clone = false;

    HnsClient::EntityEntry *entry = HnsClient::Session::instance()->entityEntryCatalog()->entityEntryById(entryID);
    if (entry) {
        if (overlappedEntity) {
            HnsClient::Entity *entity = overlappedEntity->entity();
            if (entity && entity->isModel()) {
                if (!entry->isModel()) {
                    QList<QPair<Hns::ImplementModelResult, QString> > errorList;
                    Hns::ImplementModelResult result =  Hns::canImplementModel(entity->getHnsEntry(), entry->entityEntry(), errorList, true);
                    bool implement = true;
                    if (result == Hns::ERROR)
                        implement = false;
                    if (!implement) {
                        overlappedEntity->enableBlackEffect();
                        return;
                    }
                } else {
                    overlappedEntity->enableBlackEffect();
                }
            }
        }

        if (entry->isTemplate() && entry->hasResources() && !_topology->topologyEntry()->isModel()) {
            QMessageBox msgBox(this);
            msgBox.setWindowTitle(tr("Entity creation"));
            msgBox.setText(tr("This entity is a template. Would you like to clone it ?"));
            QPushButton *cloneTemplateButton = msgBox.addButton(tr("Clone template"), QMessageBox::YesRole);
            QPushButton *editTemplateButton = msgBox.addButton(tr("Edit template"), QMessageBox::NoRole);
            QPushButton *cancelButton = msgBox.addButton(QMessageBox::Cancel);
            msgBox.setDefaultButton(cloneTemplateButton);
            msgBox.setEscapeButton(cancelButton);
            msgBox.exec();
            QAbstractButton *clickedButton = msgBox.clickedButton();
            if (clickedButton == cloneTemplateButton) {
                clone = true;
            } else if (clickedButton == editTemplateButton) {
                // Nothing to do
            } else if (clickedButton == cancelButton) {
                if(overlappedEntity){
                    HnsClient::Entity *entity = overlappedEntity->entity();
                    if(entity && entity->isModel())
                      overlappedEntity->enableBlackEffect();
                }
                return;
            }
        }
    }
    if (clone && !entry->isTemplate()) {
        qCritical() << tr("Entity '%1' is not a template anymore. Unable to start clone operation.").arg(entry->getShortName());
    } else if (entry->isLocked()) {
        qCritical() << tr("Entity '%1' is locked. Unable to use this entity.").arg(entry->getShortName());
    } else {
        Hns::TopologyLinkList linkList;
        Hns::LayoutPtr layout;
        //TODO: layoutptr factory
        if (qobject_cast<HnsClient::WireEntry *>(entry)) {
            layout = new Hns::WireLayout();
            Hns::WireLayoutPtr wireLayout = Hns::WireLayoutPtr::dynamicCast(layout);
            wireLayout->x1 = qMax(0, scenePosition.x() - 100);
            wireLayout->y1 = scenePosition.y();
            wireLayout->x2 = qMin(_getRectTopology().width(), scenePosition.x() + 100);
            wireLayout->y2 = scenePosition.y();

            Hns::TopologyLink fromLink;
            fromLink.wireSide = "-1:0";
            linkList.push_back(fromLink);
            Hns::TopologyLink toLink;
            toLink.wireSide = "-1:1";
            linkList.push_back(toLink);
        }
        else {
            layout = new Hns::EntityLayout();
            Hns::EntityLayoutPtr entityLayout = Hns::EntityLayoutPtr::dynamicCast(layout);
            entityLayout->x = scenePosition.x();
            entityLayout->y = scenePosition.y();
        }

        if (!topologyEntry()->isModel()
                && !entry->isModel()
                && overlappedEntity != NULL
                && overlappedEntity->entity()->isModel()) {
            if (clone) {
                //TODO: change signature of createEntityFromClonedEntry
                _topology->cloneEntity(entryID, Hns::EntityLayoutPtr::dynamicCast(layout), overlappedEntity->entity()->getId(), this, SLOT(_entityCloneStartedSlot(int,qlonglong,int,int)));
            } else {
                _pendingEntityCreation.append(entryID);
                _topology->implementEntity(overlappedEntity->entity(), entryID);
            }
        } else {
            if (clone) {
                //TODO: change signature of createEntityFromClonedEntryId
                _topology->cloneEntity(entryID, Hns::EntityLayoutPtr::dynamicCast(layout), 0, this, SLOT(_entityCloneStartedSlot(int,qlonglong,int,int)));
            }
            else {
                _pendingEntityCreation.append(entryID);
                _topology->createEntity(entryID, layout, linkList);
            }
        }
    }
}

void TopologyViewArea::_requestTopologyImportSlot(const qlonglong &topologyId, const QPoint &scenePosition)
{
    Hns::EntityLayoutPtr layout = new Hns::EntityLayout();
    layout->x = scenePosition.x();
    layout->y = scenePosition.y();
    _topology->importTopology(topologyId, layout, this, SLOT(_topologyImportedSlot(::Hns::IdImportedList)));
}

void TopologyViewArea::_topologyImportedSlot(::Hns::IdImportedList result)
{
    setSelectedEntities(HnsClient::Utility::qLongListFromVectorList(result));
}

void TopologyViewArea::_contentLoadedSlot()
{
    emit loadingEnded(_topology->topologyId());
    foreach(HnsClient::GenericEntity *entity, _topology->getEntities()) {
        _createEntityGraphics(qobject_cast<HnsClient::Entity *>(entity));
    }
    foreach(HnsClient::GenericEntity *entity, _topology->getWires()) {
        HnsClient::WireEntity *wire = qobject_cast<HnsClient::WireEntity *>(entity);
        if (wire) {
            QPoint from(wire->getLayout()->x1, wire->getLayout()->y1);
            QPoint to(wire->getLayout()->x2, wire->getLayout()->y2);
            WireGraphics *graphic = WireGraphics::create(wire, from, to,_ui->graphicsView->topologyScene(),this, 0);
            if (graphic) {
                connect(graphic, SIGNAL(layoutChanged(Hns::WireLayoutPtr)), this, SLOT(_wireGraphicLayoutChangedSlot(Hns::WireLayoutPtr)));
                connect(graphic, SIGNAL(plugWireToEntityRequested(qlonglong,int,qlonglong)), this, SLOT(_wirePlugToEntityRequestedSlot(qlonglong,int,qlonglong)));
                connect(graphic, SIGNAL(unplugWireRequested(qlonglong,int)), this, SLOT(_wireUnplugRequestedSlot(qlonglong,int)));
                connect(graphic, SIGNAL(addToCatalogRequested(qlonglong)), this, SLOT(_addWireToCatalogRequestedSlot(qlonglong)));
                _wireGraphicsHash.insert(wire->getId(), graphic);
            }
            QList<Hns::TopologyLink> links = _topology->getLinksByWireId(entity->getId());
            foreach (Hns::TopologyLink link, links) {
                QString entitySide = QString::fromStdString(link.entitySide);
                QString wireSide = QString::fromStdString(link.wireSide);
                if (entitySide.isEmpty())
                    continue;
                qlonglong entityId = entitySide.split(':').first().toLongLong();
                qlonglong portWireId = wireSide.split(':').last().toInt();
                if (portWireId == 0) {
                    graphic->setFromEntityGraphics(_entityGraphicsHash.value(entityId));
                }
                else if (portWireId == 1) {
                    graphic->setToEntityGraphics(_entityGraphicsHash.value(entityId));
                }
            }
        }
    }
}

void TopologyViewArea::wireCreatedSlot(HnsClient::WireEntity *entity)
{
    QPointF fromPoint(entity->getLayout()->x1, entity->getLayout()->y1);
    QPointF toPoint(entity->getLayout()->x2, entity->getLayout()->y2);
    WireGraphics *graphic = _wireGraphicsHash.value(entity->getId());
    if (graphic)
        return;
    graphic = WireGraphics::create(entity, fromPoint, toPoint, _ui->graphicsView->topologyScene(),this, 0);
    if (graphic) {
        foreach (Hns::TopologyLink link, _topology->getLinksByWireId(entity->getId())) {
            QString entitySide = QString::fromStdString(link.entitySide);
            QString wireSide = QString::fromStdString(link.wireSide);
            if (entitySide.isEmpty())
                continue;
            qlonglong entityId = entitySide.split(':').first().toLongLong();
            qlonglong portWireId = wireSide.split(':').last().toInt();
            if (portWireId == 0) {
                graphic->setFromEntityGraphics(_entityGraphicsHash.value(entityId));
            }
            else if (portWireId == 1) {
                graphic->setToEntityGraphics(_entityGraphicsHash.value(entityId));
            }
        }

        connect(graphic, SIGNAL(layoutChanged(Hns::WireLayoutPtr)), this, SLOT(_wireGraphicLayoutChangedSlot(Hns::WireLayoutPtr)));
        connect(graphic, SIGNAL(plugWireToEntityRequested(qlonglong,int,qlonglong)), this, SLOT(_wirePlugToEntityRequestedSlot(qlonglong,int,qlonglong)));
        connect(graphic, SIGNAL(unplugWireRequested(qlonglong,int)), this, SLOT(_wireUnplugRequestedSlot(qlonglong,int)));
        connect(graphic, SIGNAL(addToCatalogRequested(qlonglong)), this, SLOT(_addWireToCatalogRequestedSlot(qlonglong)));
        _wireGraphicsHash.insert(entity->getId(), graphic);
        graphic->setSelected(true);
    }
}

void TopologyViewArea::_wirePluggingRequestedSlot(qlonglong entityId)
{
    QMap<qlonglong, QString> origWireColorMap;
    QMap<qlonglong, QString> tempWireColorMap;
    QMap<qlonglong, int> isToMap;
    EntityGraphics *gr = _entityGraphicsHash.value(entityId);

    QRectF rect(gr->pos().x(), gr->pos().y(), gr->boundingRect().width(), gr->boundingRect().height());
    foreach (WireGraphics *wire, _wireGraphicsHash) {
        bool ok = false;
        QList<Hns::TopologyLink> links = _topology->getLinksByWireId(wire->getEntityID());
        Hns::TopologyLink toLink;
        Hns::TopologyLink fromLink;
        foreach (const Hns::TopologyLink &link, links) {
            int wirePort = QString::fromStdString(link.wireSide).split(':').last().toInt();
            if (wirePort == 0)
                fromLink = link;
            else if (wirePort == 1)
                toLink = link;
        }

        if (rect.contains(wire->toPoint())) {
            ok = true;
            if (wire->to() || QString::fromStdString(fromLink.entitySide).split(':').first().toLongLong() == entityId)
                ok = false;
            else
                isToMap.insert(wire->getEntityID(), 1);
        }
        else if(rect.contains(wire->fromPoint())) {
            ok = true;
            if (wire->from() || QString::fromStdString(toLink.entitySide).split(':').first().toLongLong() == entityId)
                ok = false;
            else
                isToMap.insert(wire->getEntityID(), 0);
        }
        if (ok)
            origWireColorMap.insert(wire->getEntityID(), QString::fromStdString(wire->layout()->colorCode));
    }

    if (origWireColorMap.isEmpty())
        return;
    /*todo: change this algorithm because colors are not pretty*/
    QString topologyColor = QString::fromStdString(_topology->layout().layoutOptions.colorCode);
    int interval = 360 / origWireColorMap.size() + 1;
    QColor color;
    color.setHsv(QColor(topologyColor).hue(), 100, 50);
    QMapIterator<qlonglong, QString> mapIterator(origWireColorMap);
    while (mapIterator.hasNext()) {
        mapIterator.next();
        color.setHsv(color.hue() + interval, 100, 100);
        tempWireColorMap[mapIterator.key()] = color.name();
        _wireGraphicsHash[mapIterator.key()]->setColor(color.name());
    }
    QPixmap screenshot = _generatingSnapshot(200, 200, scene(), QRect(gr->pos().x() - 100, gr->pos().y() - 100, 300, 300), {}, true); /*todo: mettre des constantes et bien cadrer l'image*/
    QMap<qlonglong, Hns::Plug> plugMap;
    EntityPluggingDialog dialog(gr->entity(), tempWireColorMap, screenshot, this);
    HnvDialogHandler *popupHandler = new HnvDialogHandler(gr->entity(), &dialog, this);
    popupHandler->setSourceDestroyedMessage(tr("\"%1\" was deleted. Operation aborted.").arg(gr->entity()->getShortName()));
    if (dialog.exec() == QDialog::Accepted) {
        plugMap = dialog.plugMap();
        QMapIterator<qlonglong, Hns::Plug> iterator(plugMap);
        while (iterator.hasNext()) {
            iterator.next();
            Hns::Plug plug = iterator.value();
            topology()->plugEntity(entityId, plug.id, iterator.key(), isToMap.value(iterator.key()), this, SLOT(_entityPluggingSlot(Hns::TopologyLink)));
        }
    }

    mapIterator.toFront();
    while (mapIterator.hasNext()) {
        mapIterator.next();
        _wireGraphicsHash[mapIterator.key()]->setColor(mapIterator.value());
    }
    if (!dialog.isTrullyRejected()) //means the entity is destroyed
        return;

    //setLayout
    QList<qlonglong> wiresToPlug = plugMap.keys();
    QList<qlonglong> allWires = origWireColorMap.keys();
    QList<qlonglong> wiresToMove;
    foreach (qlonglong id, allWires) {
        if (!wiresToPlug.contains(id)) {
            wiresToMove.append(id);
        }
    }
    QPointF centerPoint = gr->boundingRect().center() + gr->pos();
    foreach (qlonglong id, wiresToMove) {
        int x = 0;
        int y = 0;
        int interval = 10;
        WireGraphics *wire = _wireGraphicsHash.value(id);
        Hns::WireLayoutPtr wireLayout = Hns::WireLayoutPtr::dynamicCast(wire->layout()->ice_clone());
        if (!wire)
            return;
        if (!isToMap.value(id)) { //x1,y1
            if (wire->fromPoint().x() < centerPoint.x())
                x = gr->x() - interval;
            else
                x = gr->x() + gr->boundingRect().width() + interval;
            if (wire->fromPoint().y() < centerPoint.y())
                y = gr->y() - interval;
            else
                y = gr->y() + gr->boundingRect().height() + interval;
            wireLayout->x1 = qMax(10, qMin(x, _getRectTopology().width() - 10));
            wireLayout->y1 = qMax(10, qMin(y, _getRectTopology().height() - 10));
        }
        else { //x2,y2
            if (wire->toPoint().x() < centerPoint.x())
                x = gr->x() - interval;
            else
                x = gr->x() + gr->boundingRect().width() + interval;
            if (wire->toPoint().y() < centerPoint.y())
                y = gr->y() - interval;
            else
                y = gr->y() + gr->boundingRect().height() + interval;
            wireLayout->x2 = qMax(10, qMin(x, _getRectTopology().width() - 10));
            wireLayout->y2 = qMax(10, qMin(y, _getRectTopology().height() - 10));
        }
        wire->updateLayout(wireLayout);
    }
}

void TopologyViewArea::_wirePlugToEntityRequestedSlot(qlonglong wireId, int wirePort, qlonglong entityId)
{
    WirePlugToEntityDialog dialog(_topology->getEntityById(entityId), this);
    HnvDialogHandler *popupHandler = new HnvDialogHandler(_topology->getWires().value(wireId), &dialog, this);
    popupHandler->setSourceDestroyedMessage(tr("\"%1\" was deleted. Operation aborted.").arg(_topology->getWires().value(wireId)->getShortName()));
    if (dialog.exec() == QDialog::Accepted) {
        topology()->plugEntity(entityId, dialog.entityPort(), wireId, wirePort, this, SLOT(_entityPluggingSlot(Hns::TopologyLink)));
    }
    else {
        if (!dialog.isTrullyRejected())
            return;
        WireGraphics *wire = _wireGraphicsHash.value(wireId);
        EntityGraphics *entity = _entityGraphicsHash.value(entityId);
        if (!wire || !entity)
            return;
        Hns::WireLayoutPtr wireLayout = Hns::WireLayoutPtr::dynamicCast(wire->layout()->ice_clone());
        QLineF line(wireLayout->x1, wireLayout->y1, wireLayout->x2, wireLayout->y2);
        double angleDeg = line.angle();
        double angleRad = angleDeg * M_PI / 180.0;

        qreal radius = qMax(entity->boundingRect().height(), entity->boundingRect().width()) + 5;
        if (wirePort == 0) {
            wireLayout->x1 = entity->pos().x() + entity->boundingRect().center().x() + radius * qCos(angleRad);
            wireLayout->y1 = entity->pos().y() + entity->boundingRect().center().y() - radius * qSin(angleRad);
        } else {
            wireLayout->x2 = entity->pos().x() + entity->boundingRect().center().x() - radius * qCos(angleRad);
            wireLayout->y2 = entity->pos().y() + entity->boundingRect().center().y() + radius * qSin(angleRad);
        }
        wire->updateLayout(wireLayout);
    }
}

void TopologyViewArea::_entityPluggingSlot(Hns::TopologyLink link)
{
    qlonglong entityId = QString::fromStdString(link.entitySide).split(':').first().toLongLong(); //refactor
    EntityGraphics *gr = _entityGraphicsHash.value(entityId);
    int wireSide = QString::fromStdString(link.wireSide).split(':').last().toInt(); //refactor
    qlonglong wireId = QString::fromStdString(link.wireSide).split(':').first().toLongLong(); //refactor
    WireGraphics *wireGraphic = wireGraphics().value(wireId);
    Hns::WireLayoutPtr wireLayout = Hns::WireLayoutPtr::dynamicCast(wireGraphic->layout()->ice_clone());
    if (wireSide == 0) {
        //p1
        wireLayout->x1 = gr->layout()->x;
        wireLayout->y1 = gr->layout()->y;
    }
    else {
        //p2
        wireLayout->x2 = gr->layout()->x;
        wireLayout->y2 = gr->layout()->y;
    }
    wireGraphic->updateLayout(wireLayout);
}

void TopologyViewArea::_wireUnplugRequestedSlot(qlonglong wireId, int wirePort)
{
    QString wireSide = QString("%1:%2").arg(wireId).arg(wirePort);
    QList<Hns::TopologyLink> links = topology()->getLinksByWireId(wireId);
    if (links.isEmpty())
        return;
    foreach (const Hns::TopologyLink &link, links) {
        if (link.wireSide == wireSide.toStdString()) {
            qlonglong entityId = QString::fromStdString(link.entitySide).split(':').first().toLongLong();
            int entityPort  = QString::fromStdString(link.entitySide).split(':').last().toInt();
            topology()->unplugEntity(entityId, entityPort, wireId, wirePort, this, SLOT(_wireUnpluggingSlot(Hns::TopologyLink)));
        }
    }
}

void TopologyViewArea::_wireUnpluggingSlot(Hns::TopologyLink link)
{
    //TODO: changer le layout
    qlonglong wireId = QString::fromStdString(link.wireSide).split(':').first().toLongLong();
    qlonglong wirePort = QString::fromStdString(link.wireSide).split(':').last().toInt();
    qlonglong entityId = QString::fromStdString(link.entitySide).split(':').first().toLongLong();
    WireGraphics *wireGraphic = wireGraphics().value(wireId);
    EntityGraphics *entityGraphic = entityGraphics().value(entityId); //old entity
    if (!wireGraphic || !entityGraphic)
        return;
    int margin = 10;
    int dx = 0;
    int dy = 0;
    GraphicsItem::Direction direction = entityGraphic->getDirection(wireGraphic);
    switch (direction) {
    case GraphicsItem::North:
        dy = -entityGraphic->boundingRect().height() / 2 - margin;
        break;
    case GraphicsItem::South:
        dy = entityGraphic->boundingRect().height() / 2 + margin;
        break;
    case GraphicsItem::West:
        dx = -entityGraphic->boundingRect().width() / 2 - margin;
        break;
    case GraphicsItem::East:
        dx = entityGraphic->boundingRect().width() / 2 + margin;
        break;
    default:
        return;
    }

    Hns::WireLayoutPtr myLayout = Hns::WireLayoutPtr::dynamicCast(wireGraphic->layout()->ice_clone());
    if (wirePort == 0) {
        myLayout->x1 += dx;
        myLayout->y1 += dy;
    }
    else if (wirePort == 1) {
        myLayout->x2 += dx;
        myLayout->y2 += dy;
    }
    wireGraphic->updateLayout(myLayout);
}

void TopologyViewArea::_addWireToCatalogRequestedSlot(qlonglong wireId)
{
    HnsClient::Session::instance()->entityEntryCatalog()->getUnusedUUID(wireId, this, SLOT(_getUnusedUUIDForWireSlot(QString,qlonglong)));
}

void TopologyViewArea::_getUnusedUUIDForWireSlot(QString uuid, qlonglong entityId)
{
    HnsClient::GenericEntity *entity = _topology->getWires().value(entityId);
    if (!entity)
        return;

    HnsClient::WireEntry *entry = qobject_cast<HnsClient::WireEntry *>(entity->getEntry()->duplicateEntry());
    entry->entityEntry()->uuid = uuid.toStdString();
    entry->entityEntry()->isTemplate = true;
    WireAddToCatalogDialog dialog(entry, this);
    if(dialog.exec() == QDialog::Accepted){
        HnsClient::Session::instance()->entityEntryCatalog()->addEntry(entry->entityEntry());
    }

}

void TopologyViewArea::_snapshotCreatedSlot(bool result, const QString &snapshotName)
{
    if (result) {
        QMessageBox::information(this, tr("Success"), tr("Snapshot %1 successfully created").arg(snapshotName), QMessageBox::Ok, QMessageBox::Ok);
    } else {
        QMessageBox::critical(this, tr("Error"), tr("Unable to create snapshot %1").arg(snapshotName), QMessageBox::Ok, QMessageBox::Ok);
    }
}

void TopologyViewArea::_usbDevicesResetSlot(int id, bool deviceConnected)
{
    if (!_entityGraphicsHash.contains(id))
        return;
    _entityGraphicsHash.value(id)->showUsbIcon(deviceConnected);
}

void TopologyViewArea::_requestWireEntitySlot(QPointF from, QPointF to)
{
    Hns::WireLayoutPtr layout = new Hns::WireLayout();
    switch(_ui->graphicsView->insertMode()) {
    case TopologyView::InsertWireMode:
        layout->numberAngles = 0;
        break;
    case TopologyView::InsertWire1AngleMode:
        layout->numberAngles = 1;
        break;
    case TopologyView::InsertWire2AnglesMode:
        layout->numberAngles = 2;
        break;
    default:
        layout->numberAngles = 0;
    }
    QString fromInfo;
    QString toInfo;
    int fromPort = 0;
    int toPort = 0;
    HnsClient::Entity *fromEntity = getEntityByPosition(from);
    HnsClient::Entity *toEntity = getEntityByPosition(to);
    layout->colorCode = _ui->graphicsView->getWireColor().name().toStdString();
    layout->x1 = from.x();
    layout->y1 = from.y();
    layout->x2 = to.x();
    layout->y2 = to.y();
    if (fromEntity) {
        if (fromEntity->getAvailablePlugs().isEmpty())
            fromEntity = 0;
        else
            fromPort = fromEntity->getAvailablePlugs().begin().key();
    }
    if (toEntity) {
        if (toEntity->getAvailablePlugs().isEmpty())
            toEntity = 0;
        else
            toPort = toEntity->getAvailablePlugs().begin().key();
    }
    if(fromEntity || toEntity) {
        WirePlugSelector popup(fromEntity, toEntity, fromPort, toPort, this);
        HnvDialogHandler *popupHandler;
        if (fromEntity) {
            popupHandler = new HnvDialogHandler(fromEntity, &popup, this);
            popupHandler->setSourceDestroyedMessage(tr("\"%1\" was deleted. Operation aborted.").arg(fromEntity->getShortName()));
        }
        else { //toEntity
            popupHandler = new HnvDialogHandler(toEntity, &popup, this);
            popupHandler->setSourceDestroyedMessage(tr("\"%1\" was deleted. Operation aborted.").arg(toEntity->getShortName()));
        }
        if (popup.exec() == QDialog::Accepted) {
            if (fromEntity)
                fromInfo = QString("%1:%2").arg(QString::number(fromEntity->getId())).arg(QString::number(fromPort));
            if (toEntity)
                toInfo = QString("%1:%2").arg(QString::number(toEntity->getId())).arg(QString::number(toPort));
            topology()->addWire(layout, fromInfo, toInfo);
        }
        return;
    }
    topology()->addWire(layout, fromInfo, toInfo);
}

void TopologyViewArea::_entityCreationFailedSlot(qlonglong entryID)
{
    qCritical() << tr("Failed to create entity using entry with ID %1").arg(QString::number(entryID));
    _pendingEntityCreation.removeAll(entryID);
}

void TopologyViewArea::_entityCloneStartedSlot(int cloneJobID, qlonglong entryID, int posX, int posY)
{
    if (cloneJobID == 0)
        _entityCreationFailedSlot(entryID);
    else
        _ui->graphicsView->topologyScene()->displayCloneGraphic(entryID, cloneJobID, QPoint(posX, posY));
}

void TopologyViewArea::_wireCreationFailedSlot(HnsClient::GenericEntity *from, int fromPort, HnsClient::GenericEntity *to, int toPort)
{
    qCritical() << tr("Failed to create wire from %1:%2 to %3:%4")
                   .arg(QString::number(from->getId()))
                   .arg(QString::number(fromPort))
                   .arg(QString::number(to->getId()))
                   .arg(QString::number(toPort));
}

TopologyViewArea::selectionStatus TopologyViewArea::getSelectionStatus()
{
    QList<HnsClient::WireEntity *> wires;
    QList<HnsClient::GenericEntity *> entities;
    getSelectedEntities(entities, wires);

    selectionStatus result;
    result.canDefine = false;
    result.canStart = false;
    result.canPause = false;
    result.canSave = false;
    result.canStop = false;
    result.canUndefine = false;
    result.canRemote = false;
    result.canRedirectUsb = false;
    result.canEditProperties = false;
    result.canDelete = false;
    result.canCapture = false;
    result.canUnImplement = false;
    result.canCreateSnapshot = false;
    result.canDeleteSnapshot = false;
    result.canRevertSnapshot = false;
    result.canDumpMemory = false;
    result.nodeMap = QMap<QString, QPair<bool, bool> >();
    result.lastSelected = 0;

    if (wires.count() && topologyEntry()->isWriteable()) {
        foreach(HnsClient::WireEntity * wire, wires){
            if (wire->userCanWrite()){
                result.canDelete = true;
            }
        }
        result.lastSelected = -1;
    }
    if (entities.count()) {
        int flags = getCapabilitiesFlags();
        foreach(HnsClient::GenericEntity * entity, entities){
            if (flags & Hns::MEMORY_DUMP_FLAG)
                result.canDumpMemory = true;
            Hns::Status status = entity->getStatus();
            if (entity->userCanExec() && topologyEntry()->isExecutable()) {
                QList<Hns::Status> allowedStatus = entity->getAllowedStatus();
                switch (status) {
                case Hns::UNDEFINED:
                    if (allowedStatus.contains(Hns::STOPPED))
                        result.canDefine = true;
                    if(allowedStatus.contains(Hns::UNIMPLEMENTED))
                        result.canUnImplement = true;
                    break;
                case Hns::STOPPED:
                    if (allowedStatus.contains(Hns::STARTED))
                        result.canStart = true;
                    result.canCapture = true;
                    result.canUndefine = true;
                    if (flags & Hns::SNAPSHOT_FLAG) {
                        result.canCreateSnapshot = true;
                        result.canRevertSnapshot = true;
                        result.canDeleteSnapshot = true;
                    }
                    break;
                case Hns::STARTED:
                    if (allowedStatus.contains(Hns::STOPPED))
                        result.canStop = true;
                    if (allowedStatus.contains(Hns::PAUSED))
                        result.canPause = true;
                    if (allowedStatus.contains(Hns::SAVED))
                        result.canSave = true;
                    if (flags & Hns::REMOTE_DISPLAY) {
                        result.canRemote = true;
                        result.canRedirectUsb = RemoteAccess::RemoteManager::instance()->canRedirectUsb(entity->getId());
                    }
                    if (flags & Hns::NET_CAPTURE)
                        result.canCapture = true;
                    break;
                case Hns::BUSY:
                case Hns::UNIMPLEMENTED:
                    break;
                case Hns::PAUSED:
                    if (allowedStatus.contains(Hns::STARTED))
                        result.canStart = true;
                    if (flags & Hns::REMOTE_DISPLAY) {
                        result.canRemote = true;
                        result.canRedirectUsb = RemoteAccess::RemoteManager::instance()->canRedirectUsb(entity->getId());
                    }
                    if (flags & Hns::NET_CAPTURE)
                        result.canCapture = true;
                    break;
                case Hns::SAVED:
                    if (allowedStatus.contains(Hns::STARTED))
                        result.canStart = true;
                    if (flags & Hns::NET_CAPTURE)
                        result.canCapture = true;
                    result.canUndefine = true;
                    break;
                }
            }
            if (topologyEntry()->isWriteable() && entity->userCanWrite() && status != Hns::BUSY)
                result.canDelete = true;
        }
        if (wires.count() == 0 && entities.count() == 1)
            result.lastSelected = entities.last()->getId();
        else
            result.lastSelected = -1;
    }

    if ((wires.count() + entities.count()) == 1)
        result.canEditProperties = true;
    else
        result.canCapture = false;

    if (result.canDefine)
        result.nodeMap = getSelectionNodeMap(entities);

    return result;
}

QList<EntityActionDispatcher *> TopologyViewArea::getSelectionCustomActions(bool limited)
{
    QList<QList<EntityActionHandler *> > handlerList;
    QList<QGraphicsItem *> graphicsItemList = scene()->selectedItems();
    foreach (QGraphicsItem *item, graphicsItemList) {
        GraphicsItem *graphics = dynamic_cast<GraphicsItem *>(item);
        if (graphics) {
            handlerList.append(graphics->customActions(limited));
        }
    }
    return EntityActionDispatcher::parseHandlerList(handlerList);
}

int TopologyViewArea::getCapabilitiesFlags()
{
    QList<HnsClient::WireEntity *> wires;
    QList<HnsClient::GenericEntity *> entities;
    getSelectedEntities(entities, wires, false);
    if (!entities.isEmpty()){
        unsigned int flags = 0; // initialisation
        foreach (HnsClient::GenericEntity *entity,entities){
            flags |= entity->getFlags();
        }
        return flags;
    } else {
        return 0;
    }
}

bool TopologyViewArea::getIsImplementedFromModel()
{
    QList<HnsClient::GenericEntity *> entities = getSelectedEntities(false);
    if (!entities.isEmpty()) {
        bool isImplemented = true;
        foreach (HnsClient::GenericEntity *entity, entities) {
            isImplemented &= entity->isImplementedFromModel();
        }
        return isImplemented;
    }
    return false;
}

QMap<QString, QPair<bool, bool> > TopologyViewArea::getSelectionNodeMap(QList<HnsClient::GenericEntity *> entities)
{
    QMap<QString, QPair<bool, bool> > nodeMap;
    QStringList nodeNames = HyneviewSession::instance()->platformMonitor()->getNodeNames();
    foreach(const QString &nodeName, nodeNames) {
        bool canDefineOne = false;
        bool canDefineAll = true;
        foreach(HnsClient::GenericEntity *entity, entities) {
            if (!entity->isDefined()) {
                if (entity->getNodeList().contains(nodeName))
                    canDefineOne = true;
                else
                    canDefineAll &= false;
            }
            if (canDefineOne && !canDefineAll)
                break;
        }
        nodeMap.insert(nodeName, qMakePair(canDefineOne, canDefineAll));
    }
    return nodeMap;
}

void TopologyViewArea::setCustomGraphic(QString selectedFileName)
{
    QList<HnsClient::GenericEntity *> entities = getSelectedEntities();
    if (entities.size() == 1) {
        HnsClient::EntityEntry *updatedEntry = qobject_cast<HnsClient::EntityEntry *>(entities.first()->getEntry()->duplicateEntry());
        if (updatedEntry) {
            QString fileKey = HyneviewSession::instance()->resourceManager()->uploadResource(selectedFileName);
            updatedEntry->entityEntry()->customGraphic = fileKey.toStdString();
            entities.first()->updateEntry(updatedEntry);
        }
    }
}

void TopologyViewArea::_computeAndDefineInfosSlot(::Hns::LongDefineInfoMap longDefineInfoMap)
{
    QMap<Ice::Long, Hns::DefineInfo> map = QMap<Ice::Long, Hns::DefineInfo>(longDefineInfoMap);
    QMapIterator<Ice::Long, Hns::DefineInfo> iterator(map);
    QHash<qlonglong, HnsClient::GenericEntity *> entityList = getEntities(false);
    while (iterator.hasNext()) {
        iterator.next();
        qlonglong id = iterator.key();
        Hns::DefineInfo defineInfo = iterator.value();
        if (entityList.contains(id))
            entityList.value(id)->define(QString::fromStdString(defineInfo.nodeId));
    }
}

/*******************************************************************************
* TopologyViewArea - Private
*/
void TopologyViewArea::_updateLayout(Hns::TopologyLayout layout, bool centerView)
{
    _ui->graphicsView->setSceneRect(0, 0, layout.layoutOptions.width, layout.layoutOptions.height, centerView);
    QString colorCode = QString::fromStdString(layout.layoutOptions.colorCode);
    if (QColor::isValidColor(colorCode)) {
        QBrush backgroundBrush = _ui->graphicsView->backgroundBrush();
        backgroundBrush.setStyle(Qt::SolidPattern);
        QColor backgroundColor(colorCode);
        backgroundBrush.setColor(backgroundColor);
        _ui->graphicsView->setBackgroundBrush(backgroundBrush);
    }
}

// TODO : requires cleaning
//in comment in the older version
//void TopologyViewArea::refreshOverlayWidget(qlonglong entryId)
//{
//    qDebug() << "refreshOverlayWidget();" << entryId;
////    QList<EntityGraphics*> graphicsList = getSelectedEntityGraphics();
////    if (graphicsList.size() == 1 && (!entryId || graphicsList.first()->entryId() == entryId)) {
////        Hns::EntityEntryPtr entry = graphicsList.first()->entity()->getEntry();
////        EntityCatalogEntityItem item(entry, HyneviewSession::getInstance()->userManager());
////        displayItemDetails(&item);
////    }
//}

//void TopologyViewArea::displayItemDetails(EntityCatalogItem *cItem)
//{
//    _detailsWidget->updateDisplay(cItem);
//}

//void TopologyViewArea::_displayBestFit()
//{
////    _ui->graphicsView->topologyScene()->updateSceneRect();
//    _ui->graphicsView->matchZoomToItems();
//}


}// namespace Hnv
