/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef TOPOLOGYVIEWAREA_H
#define TOPOLOGYVIEWAREA_H

// Project headers
#include "treemodel.h"
#include "treemodelitem.h"
#include "topologyview.h"

// hynesim common headers
#include "common/types.h"
#include "common/topology.h"

// Qt headers
#include <QHash>
#include <QColor>

class QGraphicsScene;
class QRect;
class QWidget;

namespace Ui {
class TopologyViewArea;
} // namespace Ui

namespace HnsClient {
class GenericEntity;
class Entity;
class WireEntity;
class Topology;
class TopologyEntry;
} // namespace HnsClient

namespace Hnv {
class GraphicsItem;
class EntityGraphics;
class WireGraphics;
class EntityActionDispatcher;
class TopologyViewArea;
class OverlayExpandableWidget;

/*******************************************************************************
* NodeContentItem
*******************************************************************************/
class NodeContentItem : public TreeModelItem
{
public:
    explicit NodeContentItem(QString nodeName, int column = 1, QObject *parent = 0);
    virtual int columnCount() const;
    virtual QVariant data(int column, int role = Qt::DisplayRole) const;
    virtual int internalType() const;

protected:
    QString _nodeName;
};

/*******************************************************************************
* EntityContentItem
*******************************************************************************/
struct EntityItemStruct {
    HnsClient::GenericEntity *entity;
    QString className;
    qlonglong id;
    QString shortName;
    QVector<std::string> ice_ids;
};

class EntityContentItem : public TreeModelItem
{
    Q_OBJECT
public:
    explicit EntityContentItem(HnsClient::GenericEntity *entity, QObject *parent = 0);
    virtual int columnCount() const;
    virtual QVariant data(int column, int role = Qt::DisplayRole) const;
    virtual int internalType() const;
    HnsClient::GenericEntity *getEntity();

protected slots:
    void _entityDestroyed();
    void _entityShortNameChanged();

protected:
    QPixmap _pixmap;
    EntityItemStruct _entityItem;
};

/*******************************************************************************
* TopologyContentModel
*******************************************************************************/
class TopologyContentModel : public TreeModel
{
    Q_OBJECT
public:
    enum TopologyContentType {
        NodeItem = 1,
        EntityItem
    };

    explicit TopologyContentModel(TopologyViewArea *sourceArea, QObject *parent = 0);
    ~TopologyContentModel();
    HnsClient::GenericEntity *getEntityFromId(qlonglong entityID);

signals:
    void focusOnItem(QModelIndex index, bool selectIndex = false);

protected slots:
    void _loadingEndedSlot();
    void _entityAddedSlot(HnsClient::GenericEntity *entity);
    void _entityChangedSlot(HnsClient::GenericEntity *entity);
    void _entityRemovedSlot(qlonglong entityID);

protected:
    void _populateModel();
    void _itemChanged(TreeModelItem *item);
    void _addNode(QString nodeName);
    void _attachEntityItem(QString nodeName, EntityContentItem *item);
    void _checkNodeChildren(QString nodeName);
    void _addEntity(HnsClient::GenericEntity *entity);
    void _focusItem(TreeModelItem *item);

private:
    TopologyViewArea * _sourceArea;
    QHash<QString, NodeContentItem *> _nodeHash;
    QHash<qlonglong, EntityContentItem *> _entityHash;
};


/*******************************************************************************
* TopologyViewArea
*******************************************************************************/
class TopologyViewArea : public QWidget
{
    Q_OBJECT

public:
    struct selectionStatus {
        bool canDefine;
        bool canStart;
        bool canPause;
        bool canSave;
        bool canStop;
        bool canUndefine;
        bool canRemote;
        bool canRedirectUsb;
        bool canEditProperties;
        bool canDelete;
        bool canCapture;
        bool canUnImplement;
        bool canCreateSnapshot;
        bool canRevertSnapshot;
        bool canDeleteSnapshot;
        bool canDumpMemory;
        QMap<QString, QPair<bool, bool> > nodeMap;
        qlonglong lastSelected;
    };

    explicit TopologyViewArea(HnsClient::Topology *topology, QWidget *parent = 0);
    ~TopologyViewArea();

    void destroyArea();

    HnsClient::Topology *topology();
    HnsClient::TopologyEntry *topologyEntry();
    TopologyView  *topologyView();
    QGraphicsScene *scene();
    QHash<qlonglong, HnsClient::GenericEntity*> getEntities(bool includeWires = false);
    QList<HnsClient::GenericEntity *> getSelectedEntities(bool includeWires = true);
    void getSelectedEntities(QList<HnsClient::GenericEntity *> &entities, QList<HnsClient::WireEntity *> &wires, bool wireAsEntity = false);
    bool isEntitySelected(qlonglong entityID);
    TopologyContentModel *getContentModel();
    HnsClient::Entity *getEntityByPosition(QPointF position);

    void setZoomValue(int value);
    void setInsertMode(TopologyView::InsertMode insertMode);
    void setWireColor(QColor color);
    void setSelectedEntity(qlonglong entityID);
    void setSelectedEntities(QList<qlonglong> entityIDList);
    void centerOnEntity(qlonglong entityID);
    void centerOnEntity(EntityGraphics *entity);
    TopologyView::InsertMode insertMode();

    void defineSelected(QString nodeName);
    void startSelected();
    void suspendSelected();
    void saveSelected();
    void undefineSelected();
    void unImplementSelected();
    void stopSelected();
    void remoteDisplaySelected();
    void displayPropertiesById(qlonglong itemId);
    void displayPropertiesSelected();
    void deleteSelected();
    void deleteItemsByIDs(QList<qlonglong> itemList);
    void printTopology();
    void printSelection();
    void printView();
    void snapshotTopology();
    void snapshotSelection();
    void snapshotView();
    void toggleEntityLabel(bool checked);
    void setLabelsDisplayed(bool labelsDisplayed);
    void setStatusIconVisibility(bool visible);
    void displayCaptureJob();
    void displaySnapshotManager();
    void displayUsbManager();
    void hideUsbIcon(int entityId);
    void displayMemoryDumpDialog();
    void selectAll();
    void loadBalancerSelected();
    Hns::ByteArray createPreviewSnapshot();
    void loadBalanceAuto(Hns::BalancingBehavior behavior);
    void triggerForceDisplayLabel(Hns::LabelPolicy labelPolicy);
    void createSnapshot();

//    void displayItemDetails(EntityCatalogItem *cItem);
    TopologyViewArea::selectionStatus getSelectionStatus();
    QList<EntityActionDispatcher*> getSelectionCustomActions(bool limited = false);
    bool getIsImplementedFromModel();
    int getCapabilitiesFlags();

    QMap<QString, QPair<bool, bool> > getSelectionNodeMap(QList<HnsClient::GenericEntity*> entities);
//    void refreshOverlayWidget(qlonglong entryId = 0);
    void setCustomGraphic(QString selectedFileName);

    EntityGraphics *getEntityGraphicsByPosition(QPointF position);
    QList<EntityGraphics *> getEntitiesGraphicsByPosition(QPointF position);

    QHash<qlonglong, WireGraphics *> wireGraphics() const;
    QHash<qlonglong, EntityGraphics *> entityGraphics() const;
signals:
    void contextMenuRequested(QPointF screenPos);
    void cursorPositionChanged(int x, int y, bool inScene);
    void zoomValueChanged(int value);
    void topologyChanged(HnsClient::Topology *topology);
    void entityCreated(HnsClient::GenericEntity *entity);
    void entityChanged(HnsClient::GenericEntity *entity);
    void entityStatusUndefined(qlonglong entityID);
    void entityRemoved(qlonglong entityID);
    void loadingEnded(qlonglong topologyId);
    void labelsDisplayedChanged(bool labelsDisplayed);
    void statusIconVisibilityChanged(bool visible);
    void remoteDisplayOpened();

public slots:
    void toggleGrid(bool toggle);
    void setDetailOverlayCorner(Qt::Corner corner);
    void hideDetailOverlay(bool checked);

private slots:
    // Slots connected to Topology
    void _topologyChangedSlot();
    void _topologyLayoutChangedSlot();
    void _topologyLayoutOptionsChangedSlot();
    void _entityCreatedSlot(HnsClient::Entity *entity);
    void _entityEntryChangedSlot(HnsClient::Entity *entity);
    void _entityStatusChangedSlot(HnsClient::Entity *entity);
    void _entityRefreshedSlot(HnsClient::Entity *entity);
    void _entityPluggedSlot(qlonglong entityId, qlonglong wireId);
    void _entityUnpluggedSlot(qlonglong entityId, qlonglong wireId, int wirePort);
    void _entityRemovedSlot(qlonglong entityId);
    void _wireRemovedSlot(qlonglong wireId);

    // Slots conneted to graphics item
    void _entityGraphicLayoutChangedSlot(Hns::EntityLayoutPtr layout);
    void _wireGraphicLayoutChangedSlot(Hns::WireLayoutPtr layout);

    // Slots connected to TopologyView
    void _requestEntitySlot(const qlonglong &entryID, const QPoint &scenePosition, EntityGraphics *overlappedEntity);
    void _requestTopologyImportSlot(const qlonglong &topologyId, const QPoint &scenePosition);
    void _requestWireEntitySlot(QPointF from, QPointF to);

    // Slots connected to factory
    void _entityCreationFailedSlot(qlonglong entryID);
    void _entityCloneStartedSlot(int cloneJobID, qlonglong entryID, int posX, int posY);
    void _wireCreationFailedSlot(HnsClient::GenericEntity* from, int fromPort, HnsClient::GenericEntity* to, int toPort);

    void _computeAndDefineInfosSlot(::Hns::LongDefineInfoMap longDefineInfoMap);
    void _topologyImportedSlot(::Hns::IdImportedList result);

    void _contentLoadedSlot();

    void wireCreatedSlot(HnsClient::WireEntity *);

    void _wirePluggingRequestedSlot(qlonglong entityId);
    void _wirePlugToEntityRequestedSlot(qlonglong wireId, int wirePort, qlonglong entityId);

    void _entityPluggingSlot(Hns::TopologyLink link);
    void _wireUnplugRequestedSlot(qlonglong wireId, int wirePort);
    void _wireUnpluggingSlot(Hns::TopologyLink link);
    void _addWireToCatalogRequestedSlot(qlonglong wireId);
    void _getUnusedUUIDForWireSlot(QString uuid, qlonglong entityId);

    void _snapshotCreatedSlot(bool result, const QString &snapshotName);

    // Slots connected to the remote manager
    void _usbDevicesResetSlot(int id, bool deviceConnected);

private:
    void _displayPrintDialog(QGraphicsScene *scene, QRect sourceRect, QList<QGraphicsItem *> itemsToHide);
    void _displaySnapshotDialog(QGraphicsScene *scene, QRect sourceRect, QList<QGraphicsItem *> itemsToHide);
    QPixmap _generatingSnapshot(int width,int height,QGraphicsScene *scene,QRect sourceRect, QList<QGraphicsItem *> itemsToHide, bool centerCapture = false);
    QRect _getRectTopology();
    QRect _getRectSelection();
    QRect _getRectView();
    EntityGraphics *_createEntityGraphics(HnsClient::Entity *entity);
//    void _displayBestFit();
    void _updateLayout(Hns::TopologyLayout layout, bool centerView = false);
    Ui::TopologyViewArea *_ui;
    HnsClient::Topology * _topology;
    OverlayExpandableWidget *_overlayDetails;
    QList<qlonglong> _pendingEntityCreation;
    bool _labelsDisplayed;
    bool _statusIconDisplayed;
    TopologyContentModel *_contentModel;
    QHash<qlonglong, EntityGraphics *> _entityGraphicsHash;
    QHash<qlonglong, WireGraphics *> _wireGraphicsHash;
};

} // namespace Hnv
#endif // TOPOLOGYVIEWAREA_H
