// Project headers
#include "topologycontainer.h"
#include "ui_topologycontainer.h"
#include "ressources.h"
#include "topologyviewarea.h"
#include "topologyscreenshotdeclarativeview.h"
#include "remotemanager.h"
#include "remoteview.h"
#include "hnvmainwindow.h"

// hynesim hnsclient headers
#include <libhnsclient/topology.h>

// Qt headers
#include <QSettings>
#include <QScrollArea>
#include <QStackedWidget>
#include <QMenu>
#include <QVBoxLayout>
#include <QMessageBox>
#include <QMouseEvent>
#include <QDrag>
#include <QMimeData>
#include <QWheelEvent>
#include <QDragEnterEvent>
#include <QDropEvent>

namespace Hnv {

/*******************************************************************************
* TopologyContainer
*******************************************************************************/
TopologyContainer::TopologyContainer(TopologyViewArea *topologyViewArea, QWidget *parent) :
    QWidget(parent),
    ui(new Ui::TopologyContainer)
  , _topologyViewArea(topologyViewArea)
  , _windowCount(0)
{
    ui->setupUi(this);
    ui->tabBar->setId(-1);
    ui->stackedWidget->addWidget(_topologyViewArea);
    ui->tabBar->addTab(QObject::tr("Topology"));
    ui->tabBar->addView(0, true);

    _screenshotModel = new RemoteAccess::ScreenshotModel(_topologyViewArea->topology()->getUuid(), this);
    _screenshotView = new RemoteAccess::TopologyScreenshotDeclarativeView(_screenshotModel, this);
    _screenshotView->resize(_topologyViewArea->size());

    ui->stackedWidget->addWidget(_screenshotView);
    ui->tabBar->addTab(QObject::tr("Mosaic"));
    ui->tabBar->addView(1, true);

    ui->tabBar->setTabsClosable(true);
    ui->tabBar->tabButton(0, QTabBar::RightSide)->hide();
    ui->tabBar->tabButton(1, QTabBar::RightSide)->hide();

    setFocusPolicy(Qt::StrongFocus);

    connect(RemoteAccess::RemoteManager::instance(), SIGNAL(screenshotChanged(int,QImage)), _screenshotModel, SLOT(screenshotChange(int,QImage)));
    connect(RemoteAccess::RemoteManager::instance(), SIGNAL(connectionOpened(int)), this, SLOT(_connectionOpenedSlot(int)));
    connect(RemoteAccess::RemoteManager::instance(), SIGNAL(connectionEnded(int)), this, SLOT(_connectionEndedSlot(int)));
    connect(RemoteAccess::RemoteManager::instance(), SIGNAL(needToChangeIndex(int)), this, SLOT(_switchTabSlot(int)));
    connect(RemoteAccess::RemoteManager::instance(), SIGNAL(connectionError(int,QString)), this, SLOT(_errorMessageSlot(int,QString)));
    connect(RemoteAccess::RemoteManager::instance(), SIGNAL(shortnameChanged(int, QString)), this, SLOT(_shortnameChangedSlot(int, QString)));
    connect(_screenshotView, SIGNAL(screenshotClicked(int)), this, SLOT(_switchTabSlot(int)));
    connect(this, SIGNAL(remoteViewClosed(int)), RemoteAccess::RemoteManager::instance(), SLOT(remoteViewClosedSlot(int)));
    connect(ui->tabBar, SIGNAL(currentChanged(int)), this, SLOT(_currentIndexChangedSlot(int)));
    connect(ui->tabBar, SIGNAL(tabCloseRequested(int)), this, SLOT(_tabClosedRequestedSlot(int)));
    connect(ui->tabBar, SIGNAL(detachTabRequested(int)), this, SLOT(_detachTab(int)));
    connect(ui->tabBar, SIGNAL(retachTabRequested(int)), this, SLOT(_retachTab(int)));
    connect(ui->tabBar, SIGNAL(focusOnSeparatedWindowRequested(int)), this, SLOT(_focusOnSeparateWindowRequestedSlot(int)));
    connect(topologyViewArea, SIGNAL(entityRemoved(qlonglong)), this, SLOT(_entityDestroyedSlot(qlonglong)));
    connect(topologyViewArea, SIGNAL(entityStatusUndefined(qlonglong)), this, SLOT(_entityStatusUndefinedSlot(qlonglong)));
    connect(this, SIGNAL(remoteDisplayOpened()), topologyViewArea, SIGNAL(remoteDisplayOpened()));
}

TopologyContainer::~TopologyContainer()
{
    QHashIterator<int, RemoteAccess::RemoteView *> iterator(_viewHash);
    while (iterator.hasNext()) {
        iterator.next();
        int id = iterator.key();
        if (_isTabHash.value(id))
            RemoteAccess::RemoteManager::instance()->closeConnection(id);
    }

    foreach (RemoteControlSeparateWindow *separateWindow, _separateWindowHash) {
        if (separateWindow)
            separateWindow->close();
    }

    delete ui;
}

TopologyViewArea *TopologyContainer::topologyViewArea()
{
    return _topologyViewArea;
}

bool TopologyContainer::eventFilter(QObject *object, QEvent *event)
{
    if (event->type() == QEvent::ShortcutOverride && ui->stackedWidget->currentIndex() != 0) {
        event->accept();
        return true;
    }
    return QObject::eventFilter(object, event);
}

void TopologyContainer::focusInEvent(QFocusEvent *event)
{
    Q_UNUSED(event)
    Hnv::HnvMainWindow::instance()->installEventFilter(this);
}

void TopologyContainer::focusOutEvent(QFocusEvent *event)
{
    Q_UNUSED(event)
    Hnv::HnvMainWindow::instance()->removeEventFilter(this);
}

void TopologyContainer::_connectionOpenedSlot(int id)
{
    if (!_topologyViewArea->getEntities().contains(id))
        return;
    _screenshotModel->addView(id, RemoteAccess::RemoteManager::instance()->remoteDisplayInfo(id));
    QSettings settings;
    RemoteAccess::RemoteView *view = RemoteAccess::RemoteManager::instance()->createRemoteView(id);
    if ((Qt::CheckState)(settings.value(Constants::DETACHED_TAB_OPTION).toInt()) == Qt::Checked) {
        RemoteControlSeparateWindow *window = new RemoteControlSeparateWindow(_windowCount, view, id, this);
        _viewHash.insert(id, view);
        _separateWindowHash.insert(id, window);
        _windowHash.insert(_windowCount, window);
        _windowCount++;
        ui->tabBar->addTab(RemoteAccess::RemoteManager::instance()->getShortNameById(id));
        ui->tabBar->addView(ui->tabBar->count() - 1, false);
        ui->stackedWidget->addWidget(new QScrollArea(this));

        _isTabHash.insert(id, false);
        _idHash.insert(id, ui->tabBar->count() - 1);
        connect(window, SIGNAL(quit(int)), this, SLOT(_separateWindowQuit(int)));
        connect(window, SIGNAL(newViewRequested(int,int,int)), this, SLOT(_newViewRequestedSlot(int,int,int)));
        connect(window, SIGNAL(detachTabRequested(int)), this, SLOT(_detachTabWindowSlot(int)));
        connect(window, SIGNAL(retachTabRequested(int)), this, SLOT(_retachTabWindowSlot(int)));
        window->show();
    }
    else {
        QScrollArea *scrollArea = new QScrollArea(this);
        scrollArea->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
        scrollArea->viewport()->setBackgroundRole(QPalette::Text);
        scrollArea->setWidget(view);
        _viewHash.insert(id, view);
        connect(scrollArea->widget(), SIGNAL(destroyed()), scrollArea, SLOT(deleteLater()));
        ui->stackedWidget->addWidget(scrollArea);
        ui->tabBar->addTab(RemoteAccess::RemoteManager::instance()->getShortNameById(id));
        ui->tabBar->addView(ui->tabBar->count() - 1, true);
        _idHash.insert(id, ui->tabBar->count() - 1);
        _isTabHash.insert(id, true);
        _switchTabSlot(id);
    }
    emit remoteDisplayOpened();
}

void TopologyContainer::_connectionEndedSlot(int id)
{
    if (!_topologyViewArea->getEntities().contains(id) && !_temporaryViewsList.contains(id))
        return;
    _topologyViewArea->hideUsbIcon(id);
    if (_temporaryViewsList.contains(id))
        _temporaryViewsList.removeOne(id);
    _screenshotModel->removeView(id);
    int index = _idHash.value(id);
    emit remoteViewClosed(id);
    if (!_isTabHash.value(id)) {
        RemoteControlSeparateWindow *window = _separateWindowHash.take(id);
        if (window) {
            window->closeView(id);
            if (window->numberOfView() == 0)
                window->backToTab();
        }
    }
    _idHash.remove(id);
    _isTabHash.remove(id);
    QHash<int, int> hash;
    QHashIterator<int, int> iterator(_idHash);
    while (iterator.hasNext()) {
        iterator.next();
        int key = iterator.key(); //id
        int value = iterator.value(); //index
        if (value > index)
            hash.insert(key, value - 1);
        else if (value < index) {
            hash.insert(key, value);
        }
    }
    _idHash = hash;
    ui->tabBar->removeTab(index);
    ui->tabBar->removeView(index);
    ui->stackedWidget->removeWidget(ui->stackedWidget->widget(index));
    if (_viewHash.contains(id)) {
        RemoteAccess::RemoteView *view = _viewHash.value(id);
        view->invalidate();
    }
    if (_idHash.isEmpty()) {
        _switchTabSlot(-1);
        return;
    }
    int currentIndex = ui->tabBar->currentIndex();
    if (currentIndex < 2) { //check if we are on topology tab or mosaic view
        return;
    }
    int sizeTab = ui->tabBar->count();
    while (currentIndex != sizeTab) { //go to the right
        int viewId = _idHash.key(currentIndex, -1);
        if (viewId != -1 && _isTabHash.value(viewId)) {
            _switchTabSlot(viewId);
            return;
        }
        ++currentIndex;
    }
    currentIndex = ui->tabBar->currentIndex();//reset, go to the left
    while (currentIndex > 1) { //until mosaic view
        int viewId = _idHash.key(currentIndex, -1);
        if (viewId != -1 && _isTabHash.value(viewId)) {
            _switchTabSlot(viewId);
            return;
        }
        --currentIndex;
    }
    _switchTabSlot(-1);
}

void TopologyContainer::_tabClosedRequestedSlot(int index)
{
    int id = _idHash.key(index);
    RemoteAccess::RemoteManager::instance()->closeConnection(id);
}

void TopologyContainer::_separateWindowQuit(int id)
{
    if (!_windowHash.contains(id))
        return;
    RemoteControlSeparateWindow *window = _windowHash.take(id);
    QHash<int, QWidget*> hash = window->widgets();
    QHashIterator<int, QWidget*> iterator(hash);
    while (iterator.hasNext()) {
        iterator.next();
        int idView = iterator.key();
        _viewHash.remove(id);
        RemoteAccess::RemoteManager::instance()->closeConnection(idView);
    }
}

void TopologyContainer::_detachTab(int index)
{
    int id = _idHash.key(index);
    _isTabHash.insert(id, false);
    RemoteControlSeparateWindow *window = new RemoteControlSeparateWindow(_windowCount, _viewHash.value(id), id, 0);
    if (_separateWindowHash.contains(id)) {
        RemoteControlSeparateWindow *oldWindow = _separateWindowHash.value(id);
        oldWindow->closeView(id);
    }
    _separateWindowHash.insert(id, window);
    _windowHash.insert(_windowCount, window);
    _windowCount++;
    ui->tabBar->addView(index, false);
    connect(window, SIGNAL(quit(int)), this, SLOT(_separateWindowQuit(int)));
    connect(window, SIGNAL(newViewRequested(int,int,int)), this, SLOT(_newViewRequestedSlot(int,int,int)));
    connect(window, SIGNAL(detachTabRequested(int)), this, SLOT(_detachTabWindowSlot(int)));
    connect(window, SIGNAL(retachTabRequested(int)), this, SLOT(_retachTabWindowSlot(int)));
    _switchTabSlot(-1);
    window->show();
}

void TopologyContainer::_retachTab(int index)
{
    int id = _idHash.key(index);
    RemoteControlSeparateWindow *window = _separateWindowHash.take(id);
    ui->tabBar->addView(index, true);
    QScrollArea *scrollArea = new QScrollArea(this);
    scrollArea->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
    scrollArea->viewport()->setBackgroundRole(QPalette::Text);
    scrollArea->setWidget(window->widget(id));
    connect(scrollArea->widget(), SIGNAL(destroyed()), scrollArea, SLOT(deleteLater()));
    QWidget *widgetToDelete = ui->stackedWidget->widget(index);

    ui->stackedWidget->removeWidget(ui->stackedWidget->widget(index));
    ui->stackedWidget->insertWidget(index, scrollArea);
    if (widgetToDelete)
        widgetToDelete->deleteLater();

    window->closeView(id);
    _isTabHash.insert(id, true);
    _switchTabSlot(id);
}

void TopologyContainer::_focusOnSeparateWindowRequestedSlot(int index)
{
    int id = _idHash.key(index);
    RemoteControlSeparateWindow *window = _separateWindowHash.value(id);
    if (window) {
        if (window->isMinimized()) {
            window->showNormal();
        }
        window->activateWindow();
        window->raiseById(id);
    }
}

void TopologyContainer::_switchTabSlot(int id)
{
    if (!_topologyViewArea->getEntities().contains(id) && id != -1)
        return;
    if (id == -1) { //back to topologyviewarea
        ui->stackedWidget->setCurrentIndex(0);
        ui->tabBar->setCurrentIndex(0);
    }
    else if (_isTabHash.value(id)) {
        ui->stackedWidget->setCurrentIndex(_idHash.value(id));
        ui->tabBar->setCurrentIndex(_idHash.value(id));
    }
    else {
        _focusOnSeparateWindowRequestedSlot(_idHash.value(id));
    }
}

void TopologyContainer::_newViewRequestedSlot(int index, int idSender, int idReceiver)
{
    int idView;
    if (idSender == -1)
        idView = _idHash.key(index, -1);
    else {
        RemoteControlSeparateWindow *window = _windowHash.value(idSender);
        if (window) {
            idView = window->idFromIndex(index);
        }
        else {
            idView = -1;
        }
    }
    if (idView == -1)
        return;
    if (_separateWindowHash.contains(idView) && _separateWindowHash.value(idView) == _windowHash.value(idReceiver))
        return;
    RemoteControlSeparateWindow *windowReceiver = _windowHash.value(idReceiver);
    if (windowReceiver) {
        windowReceiver->addView(_viewHash.value(idView), idView);
        if (!_isTabHash.value(idView)) {
            RemoteControlSeparateWindow *windowSender = _separateWindowHash.value(idView);
            windowSender->closeView(idView);
        }
        _separateWindowHash.insert(idView, windowReceiver);
        ui->tabBar->addView(_idHash.value(idView), false);
        _switchTabSlot(-1);
        _isTabHash.insert(idView, false);
    }
}

void TopologyContainer::_detachTabWindowSlot(int id)
{
    int index = _idHash.value(id);
    _detachTab(index);
}

void TopologyContainer::_retachTabWindowSlot(int id)
{
    int index = _idHash.value(id);
    _retachTab(index);
}

void TopologyContainer::_currentIndexChangedSlot(int index)
{
    ui->stackedWidget->setCurrentIndex(index);
    QScrollArea *scrollArea = qobject_cast<QScrollArea*>(ui->stackedWidget->widget(index));
    if (scrollArea) {
        RemoteAccess::RemoteView *view = qobject_cast<RemoteAccess::RemoteView*>(scrollArea->widget());
        if (view)
            view->setFocus();
        else { //scrollArea is not destroyed yet
            QScrollArea *sc = qobject_cast<QScrollArea*>(ui->stackedWidget->widget(index + 1));
            if (sc)
                sc->widget()->setFocus();
        }
    }
}

void TopologyContainer::_entityDestroyedSlot(qlonglong entityId)
{
    if (_idHash.contains(entityId))
        _temporaryViewsList.append(entityId);
}

void TopologyContainer::_entityStatusUndefinedSlot(qlonglong entityId)
{
    RemoteAccess::RemoteManager::instance()->closeConnection(entityId);
}

void TopologyContainer::_errorMessageSlot(int id, QString message)
{
    QString text = RemoteAccess::RemoteManager::instance()->getShortNameById(id) + ": " + message;
    QMessageBox::critical(this, QObject::tr("Error"), text);
}

void TopologyContainer::_shortnameChangedSlot(int id, const QString &shortname)
{
    if (_idHash.contains(id)) {
        int index = _idHash.value(id);
        ui->tabBar->setTabText(index, shortname);
        if (_separateWindowHash.contains(id)) {
            RemoteControlSeparateWindow* window = _separateWindowHash.value(id);
            window->updateShortname(id, shortname);
        }
    }
}

/*******************************************************************************
* TopologyContainerTabBar
*******************************************************************************/
TopologyContainerTabBar::TopologyContainerTabBar(bool isMain, QWidget *parent)
    : QTabBar(parent)
    , _isMain(isMain)
    , _wheelIndex(0)
    , _selectedColor(Qt::white)
{
    setShape(QTabBar::RoundedSouth);
    setExpanding(false);
    _menu = new QMenu(this);
    _actionDetachTab = _menu->addAction(QObject::tr("Detach Tab"));
    _actionRetachTab = _menu->addAction(QObject::tr("Retach Tab"));
    _actionDetachTab->setEnabled(false);
    _actionDetachTab->setVisible(false);
    _actionRetachTab->setEnabled(false);
    _actionRetachTab->setVisible(false);
    connect(_actionDetachTab, SIGNAL(triggered()), this, SLOT(_actionDetachTabTriggeredSlot()));
    connect(_actionRetachTab, SIGNAL(triggered()), this, SLOT(_actionRetachTabTriggeredSlot()));
    connect(this, SIGNAL(currentChanged(int)), SLOT(_indexChangedSlot(int)));
}

TopologyContainerTabBar::~TopologyContainerTabBar()
{
}

void TopologyContainerTabBar::addView(int index, bool isTab)
{
    _isTabHash.insert(index, isTab);
}

void TopologyContainerTabBar::removeView(int index)
{
    QHash<int, bool> hash;
    QHashIterator<int, bool> iterator(_isTabHash);
    while (iterator.hasNext()) {
        iterator.next();
        int key = iterator.key();
        bool value = iterator.value();
        if (key > index)
            hash.insert(key - 1, value);
        else if (key < index)
            hash.insert(key, value);
    }
    _isTabHash = hash;
}

void TopologyContainerTabBar::setId(int id)
{
    _id = id;
}

void TopologyContainerTabBar::mousePressEvent(QMouseEvent *event)
{
    _indexSelected = _selectedIndex(event->pos());
    if (event->button() == Qt::RightButton) {
        int n = _isMain ? 1 : -1;
        if (_indexSelected > n) {
            _actionDetachTab->setVisible(true);
            _actionRetachTab->setVisible(true);
            if (_isMain) {
                if (_isTabHash.value(_indexSelected)) {
                    _actionDetachTab->setEnabled(true);
                    _actionRetachTab->setEnabled(false);
                }
                else {
                    _actionDetachTab->setEnabled(false);
                    _actionRetachTab->setEnabled(true);
                }
            }
            else {
                _actionRetachTab->setEnabled(true);
                if (count() == 1)
                    _actionDetachTab->setEnabled(false);
                else
                    _actionDetachTab->setEnabled(true);
            }
        }
        else {
            _actionDetachTab->setVisible(false);
            _actionRetachTab->setVisible(false);
        }
        _menu->move(mapToGlobal(event->pos()));
        _menu->exec();
        _actionDetachTab->setVisible(false);
        _actionRetachTab->setVisible(false);
    }
    else if (event->button() == Qt::LeftButton) {
        _dragStartPosition = event->pos();
        if (_isTabHash.value(_indexSelected)) {
            QTabBar::mousePressEvent(event);
        }
        else {
            _indexChangedSlot(_indexSelected);
            emit focusOnSeparatedWindowRequested(_indexSelected);
        }
    }
    else {
        QTabBar::mousePressEvent(event);
    }
}

void TopologyContainerTabBar::mouseMoveEvent(QMouseEvent *event)
{
    if (event->buttons() & Qt::LeftButton) {
        if ((event->pos() - _dragStartPosition).manhattanLength() > QApplication::startDragDistance()) {
            QDrag *drag = new QDrag(this);
            QMimeData *mimeData = new QMimeData();
            QByteArray encodedData;
            QDataStream stream(&encodedData, QIODevice::WriteOnly);
            stream << _indexSelected;
            stream << _id;
            mimeData->setData("hyneview/remotedisplay", encodedData);
            drag->setMimeData(mimeData);
            drag->exec();
        }
    }
    QTabBar::mouseMoveEvent(event);
}

void TopologyContainerTabBar::wheelEvent(QWheelEvent *event)
{
    int offset = event->delta() > 0 ? -1 : 1;
    _wheelIndex += offset;
    if (_wheelIndex < 0 || _wheelIndex >= count()) {
        _wheelIndex -= offset;
        return;
    }
    setTabTextColor(_wheelIndex - offset, QColor());
    if (!_isTabHash.value(_wheelIndex, true)) {
        setTabTextColor(_wheelIndex, _selectedColor);
        emit focusOnSeparatedWindowRequested(_wheelIndex);
    }
    else {
        setCurrentIndex(_wheelIndex);
        _indexChangedSlot(_wheelIndex);
    }
}

void TopologyContainerTabBar::_indexChangedSlot(int index)
{
    setTabTextColor(_wheelIndex, QColor());
    _wheelIndex = index;
    setTabTextColor(_wheelIndex, _selectedColor);
}

int TopologyContainerTabBar::_selectedIndex(const QPoint &pos)
{
    const int tabCount = count();
    for (int i = 0 ; i < tabCount ; i++) {
        if (tabRect(i).contains(pos))
            return i;
    }
    return -1;
}

void TopologyContainerTabBar::_actionDetachTabTriggeredSlot()
{
    emit detachTabRequested(_indexSelected);
}

void TopologyContainerTabBar::_actionRetachTabTriggeredSlot()
{
    emit retachTabRequested(_indexSelected);
}

/*******************************************************************************
* RemoteControlSeparateWindow
*******************************************************************************/
RemoteControlSeparateWindow::RemoteControlSeparateWindow(int id, QWidget *widgetToDisplay, int idView, QWidget*parent)
    : QDialog(parent, Qt::Window)
    , _id(id)
{
    setWindowTitle(QObject::tr("Remote Control Manager"));
    setBackgroundRole(QPalette::Text);
    setWindowIcon(QIcon(Ressources::themedPath(Constants::ICON_WELCOME)));
    setAcceptDrops(true);

    _layout = new QVBoxLayout(this);
    _layout->setMargin(0);
    _layout->setSpacing(0);

    _viewHash.insert(idView, widgetToDisplay);

    _tabBar = new TopologyContainerTabBar(false, this);
    _tabBar->setId(_id);
    _tabBar->addTab(RemoteAccess::RemoteManager::instance()->getShortNameById(idView));
    _tabBar->addView(0, true);
    _layout->addWidget(_tabBar);
    _tabBar->setTabsClosable(true);

    QScrollArea *scrollArea = new QScrollArea(this);
    scrollArea->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
    scrollArea->viewport()->setBackgroundRole(QPalette::Text);

    scrollArea->setWidget(widgetToDisplay);

    _stackedWidget = new QStackedWidget(this);
    _stackedWidget->addWidget(scrollArea);

    _layout->addWidget(_stackedWidget);

    resize(widgetToDisplay->width() + 5, widgetToDisplay->size().height() + _tabBar->size().height());

    _idHash.insert(idView, _stackedWidget->count() - 1);
    connect(this, SIGNAL(rejected()), SLOT(_dialogDestroy()));
    connect(_tabBar, SIGNAL(currentChanged(int)), _stackedWidget, SLOT(setCurrentIndex(int)));
    connect(_tabBar, SIGNAL(tabCloseRequested(int)), this, SLOT(_tabCloseRequestedSlot(int)));
    connect(_tabBar, SIGNAL(detachTabRequested(int)), this, SLOT(_detachTabRequestedSlot(int)));
    connect(_tabBar, SIGNAL(retachTabRequested(int)), this, SLOT(_retachTabRequestedSlot(int)));
}

RemoteControlSeparateWindow::~RemoteControlSeparateWindow()
{
}

QWidget *RemoteControlSeparateWindow::widget(int id)
{
    return _viewHash.value(id);
}

QHash<int, QWidget *> RemoteControlSeparateWindow::widgets()
{
    return _viewHash;
}

void RemoteControlSeparateWindow::backToTab()
{
    disconnect();
    close();
}

void RemoteControlSeparateWindow::addView(QWidget *widget, int id)
{
    _viewHash.insert(id, widget);
    QScrollArea *scrollArea = new QScrollArea(this);
    scrollArea->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
    scrollArea->viewport()->setBackgroundRole(QPalette::Text);
    scrollArea->setWidget(widget);
    _stackedWidget->addWidget(scrollArea);
    _idHash.insert(id, _stackedWidget->count() - 1);
    _tabBar->addTab(RemoteAccess::RemoteManager::instance()->getShortNameById(id));
    _tabBar->addView(_idHash.value(id), true);
    _stackedWidget->setCurrentWidget(scrollArea);
    _tabBar->setCurrentIndex(_idHash.value(id));
}

int RemoteControlSeparateWindow::numberOfView()
{
    return _viewHash.size();
}

void RemoteControlSeparateWindow::closeView(int id)
{
    if (!_idHash.contains(id))
        return;
    _tabBar->removeTab(_idHash.value(id));
    _tabBar->removeView(_idHash.value(id));
    QWidget *widgetToDestroy = _stackedWidget->widget(_idHash.value(id));
    _stackedWidget->removeWidget(widgetToDestroy);
    if (widgetToDestroy) {
        widgetToDestroy->deleteLater();
    }
    int index = _idHash.value(id);
    _idHash.remove(id);
    _viewHash.remove(id);
    QHash<int, int> hash;
    QHashIterator<int, int> iterator(_idHash);
    while (iterator.hasNext()) {
        iterator.next();
        int key = iterator.key(); //id
        int value = iterator.value(); //index
        if (value > index)
            hash.insert(key, value - 1);
        else if (value < index) {
            hash.insert(key, value);
        }
    }
    _idHash = hash;
    if (_idHash.isEmpty())
        backToTab();
}

int RemoteControlSeparateWindow::idFromIndex(int index)
{
    return _idHash.key(index, -1);
}

void RemoteControlSeparateWindow::raiseById(int id)
{
    if (_idHash.contains(id)) {
        raise();
        int index = _idHash.value(id);
        _stackedWidget->setCurrentIndex(index);
        _tabBar->setCurrentIndex(index);
    }
}

void RemoteControlSeparateWindow::updateShortname(int id, const QString &shortname)
{
    if (_idHash.contains(id)) {
        int index = _idHash.value(id);
        _tabBar->setTabText(index, shortname);
    }
}

void RemoteControlSeparateWindow::_dialogDestroy()
{
    emit quit(_id);
}

void RemoteControlSeparateWindow::_tabCloseRequestedSlot(int index)
{
    int id = _idHash.key(index);
    RemoteAccess::RemoteManager::instance()->closeConnection(id);
}

void RemoteControlSeparateWindow::_detachTabRequestedSlot(int index)
{
    int id = _idHash.key(index);
    emit detachTabRequested(id);
    _idHash.remove(id);
    _viewHash.remove(id);
    _tabBar->removeTab(index);
    foreach (int value, _idHash) {
        if (value > index)
            _idHash[_idHash.key(value)]--;
    }
}

void RemoteControlSeparateWindow::_retachTabRequestedSlot(int index)
{
    int id = _idHash.key(index);
    emit retachTabRequested(id);
}

void RemoteControlSeparateWindow::dragEnterEvent(QDragEnterEvent *event)
{
    if (event->mimeData()->hasFormat("hyneview/remotedisplay"))
        event->acceptProposedAction();
}

void RemoteControlSeparateWindow::dropEvent(QDropEvent *event)
{
    QDataStream stream(event->mimeData()->data("hyneview/remotedisplay"));
    int index;
    int idSender;
    stream >> index;
    stream >> idSender;
    event->acceptProposedAction();
    emit newViewRequested(index, idSender, _id);
}

} //namespace Hnv
