/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "topologycontentwidget.h"
#include "simulationmodewidget.h"
#include "topologywidget.h"
#include "treemodelitem.h"
#include "topologyviewarea.h"

// Qt headers
#include <QTreeView>
#include <QMenu>
#include <QWidget>

#include <libhnsclient/entity.h>

namespace Hnv {

TopologyContentWidget::TopologyContentWidget(QWidget *parent)
    : SideBarTreeWidget(parent)
    , _topologyWidget(0)
    , _menu(new QMenu(this))
    , _actionDelete(new QAction(this))
    , _actionProperties(new QAction(this))
    , _topologyContentModel(0)
{
    _treeView->setSelectionMode(QAbstractItemView::ExtendedSelection);
    QWidget *widget = sideBarParent();
    if (widget)
        _simulationModeWidget = dynamic_cast<SimulationModeWidget *>(widget);
    if (_simulationModeWidget) {
        _topologyWidget = _simulationModeWidget->topologyWidget();
        if (_topologyWidget) {
            _setContentModel(_topologyWidget->topologyContentModel());
            connect(_topologyWidget, SIGNAL(topologyActivated(QVariant)), this, SLOT(_topologyActivatedSlot(QVariant)));
            connect(_topologyWidget, SIGNAL(selectionChanged(const QList<HnsClient::GenericEntity *>)), this, SLOT(_entitySelectionChanged(const QList<HnsClient::GenericEntity *>)));
        }
    }

    _actionDelete->setText(tr("Delete"));
    _menu->addAction(_actionDelete);
    connect(_actionDelete, SIGNAL(triggered()), this, SLOT(_actionDeleteTriggeredSlot()));

    _actionProperties->setText(tr("Properties"));
    _menu->addAction(_actionProperties);
    connect(_actionProperties, SIGNAL(triggered()), this, SLOT(_actionPropertiesTriggeredSlot()));
}

TopologyContentWidget::~TopologyContentWidget()
{
    clearModel();
}

QString TopologyContentWidget::title()
{
    return tr("Topology content");
}

QString TopologyContentWidget::sideBarId()
{
    return "Topology content";
}

/*******************************************************************************
* TopologyListWidget - Protected slots
*/
void TopologyContentWidget::_topologyActivatedSlot(QVariant topologyUUID)
{
    Q_UNUSED(topologyUUID)
    _setContentModel(_topologyWidget->topologyContentModel());
}

void TopologyContentWidget::_entitySelectionChanged(const QList<HnsClient::GenericEntity *> entityList)
{
    _treeView->selectionModel()->clearSelection();
    if (!entityList.isEmpty()) {
        QModelIndexList matchList;
        foreach (const HnsClient::GenericEntity *genericEntity, entityList) {
            const HnsClient::Entity *entity = qobject_cast<const HnsClient::Entity *>(genericEntity);
            if (entity) {
                matchList.append(_treeView->model()->match(
                                     _treeView->model()->index(0,0),
                                     Qt::UserRole,
                                     QVariant(entity->getId()),
                                     1,
                                     Qt::MatchRecursive));
            }
        }
        if (!matchList.isEmpty()) {
            foreach(const QModelIndex index, matchList)
                _treeView->selectionModel()->setCurrentIndex(index, QItemSelectionModel::Select | QItemSelectionModel::Rows); // TODO optimize it
        }
    }
}

void TopologyContentWidget::_actionPropertiesTriggeredSlot()
{
    QModelIndex index = _treeView->selectionModel()->selectedRows().first();
    if (index.isValid()) {
        QVariant targetTypeData = index.data(TreeModelItem::InternalTypeRole);
        if (targetTypeData.isValid()) {
            int result = targetTypeData.toInt();
            if (result == TopologyContentModel::EntityItem) {
                _topologyWidget->displayEntityPropertiesById(index.data(Qt::UserRole).toLongLong());
            }
        }
    }
}

void TopologyContentWidget::_actionDeleteTriggeredSlot()
{
    QModelIndexList indexList = _treeView->selectionModel()->selectedRows();
    if (!indexList.isEmpty()) {
        QList<qlonglong> itemList;
        foreach (const QModelIndex index, indexList) {
            if (index.isValid()) {
                QVariant targetTypeData = index.data(TreeModelItem::InternalTypeRole);
                if (targetTypeData.isValid()) {
                    int result = targetTypeData.toInt();
                    if (result == TopologyContentModel::EntityItem) {
                        itemList.append(index.data(Qt::UserRole).toLongLong());
                    }
                }
            }
        }
        _topologyWidget->deleteItemsByIDs(itemList);
    }
}

void TopologyContentWidget::_indexClicked(const QModelIndex &index)
{
    Q_UNUSED(index)
    QModelIndexList selectedRows = _treeView->selectionModel()->selectedRows();
    QList<qlonglong> selectedIDs;
    if (!selectedRows.isEmpty()) {
        bool onlyEntityItemSelected = true;
        foreach(QModelIndex index, selectedRows) {
            QVariant targetTypeData = index.data(TreeModelItem::InternalTypeRole);
            if (targetTypeData.isValid()) {
                int result = targetTypeData.toInt();
                if (result == TopologyContentModel::EntityItem)
                    selectedIDs.append(index.data(Qt::UserRole).toLongLong());
                else
                    onlyEntityItemSelected = false;
            }
        }
        if (onlyEntityItemSelected) {
            if (selectedRows.count() <= 1)
                _actionProperties->setEnabled(true);
            else
                _actionProperties->setEnabled(false);
        }
        _topologyWidget->setEntitySelection(selectedIDs);
    }
}

void TopologyContentWidget::_indexDoubleClicked(const QModelIndex &index)
{
    Q_UNUSED(index)
}

void TopologyContentWidget::_contextMenuRequested(const QModelIndex &index, const QPoint &position)
{
    Q_UNUSED(position)
    if (index.isValid()) {
        QVariant targetTypeData = index.data(TreeModelItem::InternalTypeRole);
        int result = targetTypeData.toInt();
        if (result == TopologyContentModel::EntityItem) {
            _menu->exec(QCursor::pos());
        }
    }
}

void TopologyContentWidget::_setContentModel(TopologyContentModel *contentModel)
{
    _topologyContentModel = contentModel;
    setViewModel(_topologyContentModel);
    if (_topologyContentModel)
        connect(_topologyContentModel, SIGNAL(focusOnItem(QModelIndex,bool)), this, SLOT(focusOnIndex(QModelIndex,bool)));
}

} // namespace Hnv
