// Project headers
#include "topologyscreenshotdeclarativeview.h"
#include "remotemanager.h"
#include "ressources.h"

// Qt headers
#include <QResizeEvent>
#include <QGraphicsObject>

#include <QDeclarativeContext>
#include <QDeclarativeEngine>

namespace RemoteAccess {

/*******************************************************************************
* ScreenshotModelProvider
*******************************************************************************/
ScreenshotModelProvider::ScreenshotModelProvider()
    : QDeclarativeImageProvider(QDeclarativeImageProvider::Image)
{
}

ScreenshotModelProvider::~ScreenshotModelProvider()
{
}

QImage ScreenshotModelProvider::requestImage(const QString &id, QSize *size, const QSize &requestedSize)
{
    if (_img.isNull()) {
        QString realId = id.left(id.indexOf("/"));
        int width = 32;
        int height = 32;
        if (size)
            *size = QSize(width, height);
        QPixmap pixmap(requestedSize.width() > 0 ? requestedSize.width() : width,
                       requestedSize.height() > 0 ? requestedSize.height() : height);
        pixmap.fill(Qt::red);
        QPainter p(&pixmap);
        p.drawText(0, 10, realId);
        p.end();
        return pixmap.toImage();
    }
    return _img;
}

void ScreenshotModelProvider::setImage(QImage img)
{
    _img = img;
}

/*******************************************************************************
* ScreenshotModel
*******************************************************************************/
enum RolesModel {
    RoleInformation = Qt::UserRole + 1,
    RoleImage = Qt::UserRole + 2,
    RoleId = Qt::UserRole + 3
};

ScreenshotModel::ScreenshotModel(QString topologyUuid, QObject *parent)
    : QAbstractTableModel(parent)
    , _topologyUuid(topologyUuid)
{
    QHash<int, QByteArray> roles;
    roles[RoleInformation] = "roleInformation";
    roles[RoleImage] = "roleImage";
    roles[RoleId] = "roleId";
    setRoleNames(roles);
    _provider = new ScreenshotModelProvider();
}

ScreenshotModel::~ScreenshotModel()
{
}

void ScreenshotModel::clear()
{
    beginResetModel();
    _idList.clear();
    _dataHash.clear();
    endResetModel();
}

void ScreenshotModel::addView(int id, Hns::RemoteDisplayInfo info)
{
    int row = _idList.size();
    beginInsertRows(QModelIndex(), row, row);
    _idList.append(id);
    _dataHash.insert(id, info);
    endInsertRows();
}

int ScreenshotModel::rowCount(const QModelIndex &parent) const
{
    if (!parent.isValid())
        return _idList.size();
    else
        return 0;
}

int ScreenshotModel::columnCount(const QModelIndex &parent) const
{
    if (!parent.isValid())
        return COLUMN_SIZE;
    else
        return 0;
}

QVariant ScreenshotModel::data(const QModelIndex &index, int role) const
{
    if (index.isValid()) {
            int row = index.row();
            if (row < _idList.size()) {
                Hns::RemoteDisplayInfo info = _dataHash.value(_idList.at(row));
                if (role == Qt::DisplayRole)
                    return QString(QString::fromStdString(info.host) + ":" + QString::number(info.port) + " id:" + QString::number(_idList.at(row)));
                else if (role == Qt::DecorationRole) {
                    if (!_screenshotHash.value(_idList.at(row)).isNull()) {
                        return _screenshotHash.value(_idList.at(row)).scaled(200, 200);
                    }
                    //TODO: else return black image ?
                }
                else if (role == RoleInformation)
                    return QString(RemoteManager::instance()->getShortNameById(_idList.at(row)));
                else if (role == RoleImage) {
                    QImage image;
                    if (!_screenshotHash.value(_idList.at(row)).isNull()) {
                        image = _screenshotHash.value(_idList.at(row));
                    }
                    else { //first time
                        image = QImage(200, 200, QImage::Format_ARGB32);
                        image.fill(QColor(Qt::black).rgb());
                    }
                    _provider->setImage(image);
                    return QVariant(QString("image://modelProvider/row%1/%2/%3").arg(_topologyUuid).arg(index.row()).arg(QDateTime::currentDateTime().toString("dd-MM-yyyy:hh.mm.ss")));
                }
                else if (role == RoleId) {
                    return _idList.at(row);
                }
            }
        }
        return QVariant();
}

bool ScreenshotModel::setData(const QModelIndex &index, const QVariant &value, int role)
{
    if (index.isValid()) {
            int row = index.row();
            if (row < _idList.size()) {
                if (role == Qt::DecorationRole) {/* à tester pour RoleNumberTwo */
                    _screenshotHash[_idList.at(row)] = value.value<QImage>();
                    emit dataChanged(index, index);
                    return true;
                }
            }
        }
        return true;
}

ScreenshotModelProvider *ScreenshotModel::screenshotModelProvider()
{
    return _provider;
}

void ScreenshotModel::screenshotChange(int id, QImage image)
{
    if (_idList.contains(id))
        setData(index(_idList.indexOf(id), 0), image, Qt::DecorationRole);
}

void ScreenshotModel::removeView(int id)
{
    if (_dataHash.contains(id) && _idList.contains(id)) {
            int row = _idList.indexOf(id);
            beginRemoveRows(QModelIndex(), row, row);
            _idList.removeAt(row);
            _dataHash.remove(id);
            _screenshotHash.remove(id);
            endRemoveRows();
        }
}

/*******************************************************************************
* MainAnchor
*******************************************************************************/
MainAnchor::MainAnchor(QObject *parent)
    :QObject(parent)
{
    //set default values
    _color = Qt::black;
    _height = 10;
    _width = 10;
}

QColor MainAnchor::color()
{
    return _color;
}

void MainAnchor::setColor(QColor color)
{
    _color = color;
    emit colorChanged();
}

int MainAnchor::width()
{
    return _width;
}

void MainAnchor::setWidth(int width)
{
    _width = width;
    emit widthChanged();
}

int MainAnchor::height()
{
    return _height;
}

void MainAnchor::setHeight(int height)
{
    _height = height;
    emit heightChanged();
}

/*******************************************************************************
* TopologyScreenshotDeclarativeView
*******************************************************************************/
TopologyScreenshotDeclarativeView::TopologyScreenshotDeclarativeView(ScreenshotModel * model, QWidget *parent) :
    QDeclarativeView(parent)
  , _model(model)
{
    _anchor = new MainAnchor(this);
    _anchor->setColor(QColor("#969696"));
    this->rootContext()->setContextProperty("anchor", _anchor);
    this->rootContext()->setContextProperty("screenshotModel", _model);
    this->engine()->addImageProvider("modelProvider", _model->screenshotModelProvider());
    setSource(QUrl(Hnv::Constants::TOPOLOGY_QML));

    QGraphicsObject *item = rootObject();
    connect(item, SIGNAL(qmlSignalClicked(int)), this, SIGNAL(screenshotClicked(int)));
}

void TopologyScreenshotDeclarativeView::resizeEvent(QResizeEvent *event)
{
    _anchor->setWidth(event->size().width());
    _anchor->setHeight(event->size().height());
    setSceneRect(0, 0, event->size().width(), event->size().height());
}

} // namespace RemoteAccess
