/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "topologysizedialog.h"
#include "ui_topologysizedialog.h"
#include "ressources.h"

// Qt headers
#include <QHashIterator>
#include <QColorDialog>
#include <QSettings>
#include <QMessageBox>

namespace Hnv {

/*******************************************************************************
* TopologySizeDialog
*******************************************************************************/
TopologySizeDialog::TopologySizeDialog(bool isNew, QWidget *parent)
    : QDialog(parent)
    , ui(new Ui::TopologySizeDialog)
    , _miniPixels(300)
    , _maxiPixels(10000)
    , _defaultSizePixels(_miniPixels)
    , _defaultSizeUnit(QString("px"))
    , _defaultColor(Qt::white)
    , _mininalWidth(_miniPixels)
    , _minimalHeight(_miniPixels)
{
    ui->setupUi(this);

    ui->labelHeightValue->setMinimumSize(ui->labelHeightValue->sizeHint());
    ui->labelWidthValue->setMinimumSize(ui->labelWidthValue->sizeHint());
    QSettings settings;
    settings.beginGroup(Constants::TOPOLOGY_SETTINGS);
    _iniFormat = settings.value(Constants::TOPOLOGY_DEFAULT_FORMAT).toString();
    QString settingsColor = "#"+settings.value(Constants::TOPOLOGY_DEFAULT_BACKGROUND_COLOR_KEY, Constants::TOPOLOGY_DEFAULT_BACKGROUND_COLOR).toString();
    if (QColor::isValidColor(settingsColor))
         _topologyColor.setNamedColor(settingsColor);
    else
        _topologyColor = _defaultColor;
    _setColorPreview(_topologyColor.name());

    settingsColor = "#"+settings.value(Constants::TOPOLOGY_DEFAULT_MODEL_BACKGROUND_COLOR_KEY, Constants::TOPOLOGY_DEFAULT_MODEL_BACKGROUND_COLOR).toString();
    if (QColor::isValidColor(settingsColor))
        _modelColor.setNamedColor(settingsColor);
    else
        _modelColor = _defaultColor;

    _iniSizeUnit = settings.value(Constants::TOPOLOGY_DEFAULT_SIZE_UNIT).toString();
    QList<QString> enabledFormats = settings.value(Constants::TOPOLOGY_ENABLED_FORMAT).toStringList();
    settings.endGroup();
    QStringList groups = settings.childGroups();
    foreach (QString s,groups){
        if (!(s.startsWith(QString("TopologyFormats-")))){
            groups.removeOne(s);
        }
    }
    QStringList keys;
    formatTemplate model;
    foreach (QString s,groups){
        settings.beginGroup(s);
        keys = settings.childKeys();
        model.id = settings.value(Constants::TOPOLOGY_ID_KEY).toString();
        model.description = settings.value(Constants::TOPOLOGY_DESCRIPTION).toString();
        model.width = settings.value(Constants::TOPOLOGY_WIDTH).toInt();
        model.width = (model.width<300) ? 300 : model.width;
        model.height = settings.value(Constants::TOPOLOGY_HEIGHT).toInt();
        model.height = (model.height<300) ? 300 : model.height;
        if (enabledFormats.contains(model.id)){
            _hashModels.insert(model.id,model);
        }
        settings.endGroup();
    }
    foreach (formatTemplate model,_hashModels.values()){
        ui->cbFormat->addItem(model.description,QVariant(model.id));
    }
    ui->cbFormat->addItem(QString(tr("Custom format")),QVariant(QString()));

    /** Default configuration if there is no ini file or if the ini file contains errors**/
    if (!(_hashModels.contains(_iniFormat))){
        _iniFormat = "";
    }
    if (_iniFormat == "") {
        setWidth(_defaultSizePixels);
        setHeight(_defaultSizePixels);
    }
    ui->cbFormat->setCurrentIndex(ui->cbFormat->findData(_iniFormat));

    if(_iniSizeUnit=="") {
        _iniSizeUnit = _defaultSizeUnit;
    }
    _hashUnits.insert(QString("mm"),tr("Millimeters"));
    _hashUnits.insert(QString("px"),tr("Pixels"));
    _hashUnits.insert(QString("in"),tr("Inches"));
    QHashIterator<QString,QString> it(_hashUnits);
    while(it.hasNext()){
        it.next();
        ui->cbUnit->addItem(it.value(),QVariant(it.key()));
    }
    if (!(_hashUnits.contains(_iniSizeUnit))){
        _iniSizeUnit = _defaultSizeUnit;
    }
    ui->tbColor->setIcon(QIcon(Ressources::themedPath(Constants::ICON_COLORIZE)));
    ui->tbLandscape->setIcon(QIcon(Ressources::themedPath(Constants::ICON_LANDSCAPE)));
    ui->tbPortrait->setIcon(QIcon(Ressources::themedPath(Constants::ICON_PORTRAIT)));

    ui->labelResolutionValue->setText(QString::number(Constants::DEFAULT_IMPRESSION_RESOLUTION));

    ui->cbUnit->setCurrentIndex(ui->cbUnit->findData(_iniSizeUnit));

    if (isNew)
        setWindowTitle(tr("New"));
    else {
        ui->typeModel->setDisabled(true);
        ui->typeTopology->setDisabled(true);
    }
}

TopologySizeDialog::~TopologySizeDialog()
{
    delete ui;
}

int TopologySizeDialog::getWidth()
{
    return _width;
}

int TopologySizeDialog::getHeight()
{
    return _height;
}

bool TopologySizeDialog::isModel()
{
    return ui->typeModel->isChecked();
}

void TopologySizeDialog::setWidth(int value)
{
    _width = value;
    _checkButtons();
    _isExistingFormat();
    _update();
}

void TopologySizeDialog::setHeight(int value)
{
    _height = value;
    _checkButtons();
    _isExistingFormat();
    _update();
}

void TopologySizeDialog::setSize(int w, int h)
{
    _width = w;
    _height = h;
    _checkButtons();
    _isExistingFormat();
    _update();
}

QColor TopologySizeDialog::getColor()
{
    if (isModel())
        return _modelColor;
    else
        return _topologyColor;
}

void TopologySizeDialog::setColor(QColor value)
{
    if (isModel())
        _modelColor = value;
    else
        _topologyColor = value;
    _setColorPreview(value.name());
}

void TopologySizeDialog::setMinimalSize(int minW, int minH)
{
    _mininalWidth = qMax(_miniPixels, minW);
    _minimalHeight = qMax(_miniPixels, minH);
}

void TopologySizeDialog::setIsModel(bool isModel)
{
    ui->typeModel->setChecked(isModel);
    if (isModel)
        _setColorPreview(_modelColor.name());
    else
        _setColorPreview(_topologyColor.name());
}
/*******************************************************************************
* TopologySizeDialog - Private functions
*/
double TopologySizeDialog::_pixelToInch(int n, double resolution)
{
    return n/resolution;
}

double TopologySizeDialog::_pixelToMillimeter(int n, double resolution)
{
    return _pixelToInch(n,resolution) * Constants::INCH_IN_MILLIMETER;
}

double TopologySizeDialog::_millimeterToInch(double mmValue)
{
    return mmValue / Constants::INCH_IN_MILLIMETER;
}

int TopologySizeDialog::_millimeterToPixel(double n, double resolution)
{
    double returnValue  = (n*resolution)/Constants::INCH_IN_MILLIMETER;
    return int(returnValue);
}

double TopologySizeDialog::_inchToMillimeter(double n)
{
    return n * Constants::INCH_IN_MILLIMETER;
}

int TopologySizeDialog::_inchToPixel(double n, double resolution)
{
    double result= n * resolution;
    return (int)result;
}

void TopologySizeDialog::_update()
{
    ui->labelHeightValue->setText(QString::number(_height));
    ui->labelWidthValue->setText(QString::number(_width));
    QString unit = _getUnit();
    ui->sbWidth->setValue(_unitValue(_width,unit));
    ui->sbHeight->setValue(_unitValue(_height,unit));
}

void TopologySizeDialog::_checkButtons()
{
    double width = ui->sbWidth->value();
    double height = ui->sbHeight->value();
    bool notIdentical = width != height;
    bool isPortrait = width < height;

    ui->tbLandscape->setEnabled(notIdentical);
    ui->tbPortrait->setEnabled(notIdentical);
    ui->tbLandscape->setChecked(notIdentical && !isPortrait);
    ui->tbPortrait->setChecked(notIdentical && isPortrait);
}

double TopologySizeDialog::_unitValue(int value, QString unit)
{
    if (unit=="mm"){
        return _pixelToMillimeter(value);
    } else if (unit == "px"){
        return value;
    } else if (unit == "in"){
        return _pixelToInch(value);
    }
    return 0.0;
}

int TopologySizeDialog::_pixelValue(double value, QString unit)
{
    if (unit=="mm"){
        return _millimeterToPixel(value);
    } else if (unit == "px"){
        return value;
    } else if (unit == "in"){
        return _inchToPixel(value);
    }
    return 0;
}

void TopologySizeDialog::_updateLabelValue(double value, QString unit, QString type)
{
    int valueToDisplay = _pixelValue(value,unit);
    if (type=="w"){
        ui->labelWidthValue->setText(QString::number(valueToDisplay));
    } else if (type=="h"){
        ui->labelHeightValue->setText(QString::number(valueToDisplay));
    }
}

void TopologySizeDialog::_swapSbValue()
{
    double temp = ui->sbHeight->value();
    ui->sbHeight->setValue(ui->sbWidth->value());
    ui->sbWidth->setValue(temp);
}

QString TopologySizeDialog::_getUnit()
{
    return ui->cbUnit->itemData(ui->cbUnit->currentIndex()).toString();
}

void TopologySizeDialog::_isExistingFormat()
{
    bool check = false;
    foreach (formatTemplate model,_hashModels.values()){
        if ((model.height)==_height && (model.width)==_width){
            check = true;
            ui->cbFormat->setCurrentIndex(ui->cbFormat->findData(model.id));
        }
    }
    if (!check){
        ui->cbFormat->setCurrentIndex(ui->cbFormat->findData(""));
    }
}
/*******************************************************************************
* TopologySizeDialog - Private slots
*/
void TopologySizeDialog::on_cbFormat_currentIndexChanged(int index)
{
    if(ui->cbFormat->itemData(index) == ""){
        return;
    }
    formatTemplate format = _hashModels.value(ui->cbFormat->itemData(index).toString());
    _height = format.height;
    _width = format.width;
    _checkButtons();
    _update();
}

void TopologySizeDialog::on_cbUnit_currentIndexChanged(int index)
{
    Q_UNUSED(index);
    QString unit = _getUnit();
    double maxi = _unitValue(_maxiPixels,unit);
    double mini = _unitValue(_miniPixels,unit);
    _changeDecimals();
    ui->sbHeight->setMaximum(maxi);
    ui->sbHeight->setMinimum(mini);
    ui->sbWidth->setMaximum(maxi);;
    ui->sbWidth->setMinimum(mini);
    _update();
}

void TopologySizeDialog::_changeDecimals()
{
    QString unit = _getUnit();
    if (unit =="px"){
        ui->sbHeight->setDecimals(0);
        ui->sbWidth->setDecimals(0);
    } else {
        ui->sbHeight->setDecimals(2);
        ui->sbWidth->setDecimals(2);
    }
}

void TopologySizeDialog::_setColorPreview(const QString &name)
{
    ui->labelColor->setStyleSheet("QLabel{background:"+ name+ ";}");
}

void TopologySizeDialog::on_sbWidth_editingFinished()
{
    ui->cbFormat->setCurrentIndex(ui->cbFormat->findData(""));
    QString unit = _getUnit();
    _width=_pixelValue(ui->sbWidth->value(),unit);
    _checkButtons();
    _update();
}

void TopologySizeDialog::on_sbHeight_editingFinished()
{
    ui->cbFormat->setCurrentIndex(ui->cbFormat->findData(""));
    QString unit = _getUnit();
    _height = _pixelValue(ui->sbHeight->value(),unit);
    _checkButtons();
    _update();
}

void TopologySizeDialog::on_sbWidth_valueChanged(double arg1)
{
    Q_UNUSED(arg1)
    QString unit = _getUnit();
    _updateLabelValue(ui->sbWidth->value(),unit,"w");
    _checkButtons();
}

void TopologySizeDialog::on_sbHeight_valueChanged(double arg1)
{
    Q_UNUSED(arg1)
    QString unit = _getUnit();
    _updateLabelValue(ui->sbHeight->value(),unit,"h");
    _checkButtons();
}

void TopologySizeDialog::on_tbLandscape_clicked(bool checked)
{
    Q_UNUSED(checked);
    _handleButtons();
}

void TopologySizeDialog::on_tbPortrait_clicked(bool checked)
{
    Q_UNUSED(checked);
    _handleButtons();
}

void TopologySizeDialog::_handleButtons()
{
    _swapSbValue();
    ui->cbFormat->setCurrentIndex(ui->cbFormat->findData(""));
    QString unit = _getUnit();
    _height = _pixelValue(ui->sbHeight->value(),unit);
    _width = _pixelValue(ui->sbWidth->value(),unit);
}

void TopologySizeDialog::on_tbColor_clicked()
{
    QColor returnColor = QColorDialog::getColor(_topologyColor,this,tr("Color"));
    if (returnColor.isValid()) {
        if (isModel())
            _modelColor = returnColor;
        else
            _topologyColor = returnColor;
        _setColorPreview(returnColor.name());
    }
}

void TopologySizeDialog::on_typeTopology_toggled(bool checked)
{
    if (checked)
        _setColorPreview(_topologyColor.name());
    else
        _setColorPreview(_modelColor.name());
}

void TopologySizeDialog::on_buttonBox_accepted()
{
    if (_width < _mininalWidth || _height < _minimalHeight)
        if (QMessageBox::Ok != QMessageBox::question(
                    this,
                    tr("Confirmation required"),
                    tr("The requested size is too small to contains your current topology. It will be resized to the minimal allowed size. Would you like to continue?"),
                    QMessageBox::Ok, QMessageBox::Cancel))
            return;
    accept();
}

} // namespace Hnv
