/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "topologywidget.h"
#include "ui_topologywidget.h"
#include "hyneviewsession.h"
#include "ressources.h"
#include "topologyviewarea.h"
#include "topologyview.h"
#include "topologysavedialog.h"
#include "hnv_entryproperties.h"
#include "simulationmodewidget.h"
#include "topologydialog.h"
#include "hnvmainwindow.h"
#include "topologycatalogmodel.h"
#include "topologysizedialog.h"
#include "topologysavealldialog.h"
#include "entityaction.h"
#include "topologyadministrator.h"
#include "topologycontainer.h"
#include "graphicselectordialog.h"
#include "remotemanager.h"

// hynesim hnsclient headers
#include <libhnsclient/entity.h>
#include <libhnsclient/entry.h>
#include <libhnsclient/topology.h>
#include <libhnsclient/topologymanager.h>
#include <libhnsclient/platformmonitor.h>
#include <libhnsclient/resourcemanager.h>

// Qt headers
#include <QSettings>
#include <QMessageBox>
#include <QSignalMapper>
#include <QDebug>
#include <QMenu>
#include <QActionGroup>
#include <QColorDialog>

// C++ std headers
#include <malloc.h>

namespace Hnv {

/*******************************************************************************
 *  LoadedTopologyModel
 ******************************************************************************/
LoadedTopologyModel::LoadedTopologyModel(QObject *parent)
    : QAbstractItemModel(parent)
{
}

QModelIndex LoadedTopologyModel::index(int row, int column, const QModelIndex &parent) const
{
    if (!hasIndex(row, column, parent))
        return QModelIndex();
    if (row < _topologyList.size())
        return createIndex(row, column);
    return QModelIndex();
}

QModelIndex LoadedTopologyModel::parent(const QModelIndex &child) const
{
    Q_UNUSED(child);
    return QModelIndex();
}

int LoadedTopologyModel::rowCount(const QModelIndex &parent) const
{
    if (parent.isValid())
        return 0;
    return _topologyList.size();
}

int LoadedTopologyModel::columnCount(const QModelIndex &parent) const
{
    Q_UNUSED(parent);
    return 2;
}

QVariant LoadedTopologyModel::data(const QModelIndex &index, int role) const
{
    if (index.isValid()) {
        int row = index.row();
        if (row < _topologyList.size()) {
            HnsClient::Topology *topology = _topologyList.at(row);
            if (topology) {
                if (index.column() == 0 && role == Qt::DisplayRole) {
                    QString name = topology->topologyEntry()->getShortName();
                    if (topology->isModified())
                        name = QString("%1 *").arg(name);
                    return name;
                } else if (role == Qt::UserRole) {
                    return topology->topologyEntry()->getUUID();
                }
            }
        }
    }
    return QVariant();
}

void LoadedTopologyModel::clear()
{
    beginResetModel();
    _topologyList.clear();
    endResetModel();
}
/*******************************************************************************
* LoadedTopologyModel - Public slots
*/
void LoadedTopologyModel::topologyOpenedSlot(HnsClient::Topology *topology)
{
    int row = _topologyList.size();
    beginInsertRows(QModelIndex(), row, row);
    _topologyList.append(topology);
    endInsertRows();
}

void LoadedTopologyModel::topologyChangedSlot(HnsClient::Topology *topology)
{
    int row = _topologyList.indexOf(topology);
    if (row != -1) {
        QModelIndex idx = index(row, 0);
        emit dataChanged(idx, idx);
    }
}

void LoadedTopologyModel::topologyClosedSlot(QString topologyUUID)
{
    HnsClient::Topology *targetTopology = 0;
    foreach(HnsClient::Topology *topology, _topologyList) {
        if (topology->getUuid() == topologyUUID) {
            targetTopology = topology;
            break;
        }
    }
    if (targetTopology) {
        int row = _topologyList.indexOf(targetTopology);
        beginRemoveRows(QModelIndex(), row, row);
        _topologyList.removeAll(targetTopology);
        endRemoveRows();
    }
}

/*******************************************************************************
 *  TopologyWidget
 ******************************************************************************/
TopologyWidget::TopologyWidget(QWidget *parent)
    : QWidget(parent)
    , _ui(new Ui::TopologyWidget)
{
    _ui->setupUi(this);
    QSettings settings;
    settings.beginGroup(Constants::TOPOLOGY_SETTINGS);
    _wireColor = QColor("#"+settings.value(Constants::WIRE_DEFAULT_COLOR, "000000").toString());
    _gridEnabled = settings.value(Constants::TOPOLOGY_DEFAULT_DISPLAY_GRID, "false").toBool();
    _ui->action_Toggle_Status_Icons->setChecked(settings.value(Constants::TOPOLOGY_STATUS_ICON, "true").toBool());
    settings.endGroup();

    _drawScreenshotMapper = new QSignalMapper(this);
    connect(_drawScreenshotMapper, SIGNAL(mapped(QString)), this, SLOT(_drawScreenShotSlot(QString)));

    _defineButton = new QToolButton(this);
    _defineButton->setDefaultAction(_ui->action_Define_entity);
    _defineButton->setPopupMode(QToolButton::MenuButtonPopup);
    addAction(_ui->action_Define_entity);

    _forceDisplayLabel = new QToolButton(this);
    _forceDisplayLabel->setPopupMode(QToolButton::InstantPopup);
    _forceDisplayLabel->setText(tr("Display label policy"));
    QMenu *displayLabelMenu = new QMenu(tr("Display label policy"),this);
    displayLabelMenu->setIcon(QIcon(Ressources::themedPath(Constants::ICON_TOGGLE_LABEL)));
    QActionGroup *actionGroup = new QActionGroup(displayLabelMenu);
    actionGroup->setExclusive(true);
    actionGroup->addAction(_ui->action_Force_to_Default);
    _labelPolicyActionsHash.insert(Hns::LabelPolicyDefault, _ui->action_Force_to_Default);
    actionGroup->addAction(_ui->action_Force_to_Show);
    _labelPolicyActionsHash.insert(Hns::LabelPolicyShow, _ui->action_Force_to_Show);
    actionGroup->addAction(_ui->action_Force_to_Hide);
    _labelPolicyActionsHash.insert(Hns::LabelPolicyHide, _ui->action_Force_to_Hide);
    displayLabelMenu->addActions(actionGroup->actions());
    _forceDisplayLabel->setMenu(displayLabelMenu);
    _forceDisplayLabel->hide();

    _printButton = new QToolButton(this);
    _printButton->setDefaultAction((_ui->action_Print_Topology));
    _printButton->setPopupMode(QToolButton::InstantPopup);
    QMenu *printMenu = new QMenu(_ui->action_Print_Topology->text(),this);
    printMenu->addAction(_ui->action_Print_Topology);
    printMenu->addAction(_ui->action_Print_Selection);
    printMenu->addAction(_ui->action_Print_View);
    _printButton->setMenu(printMenu);

    _snapshotButton = new QToolButton(this);
    _snapshotButton->setDefaultAction((_ui->action_Snapshot_Topology));
    _snapshotButton->setPopupMode(QToolButton::InstantPopup);
    QMenu *snapshotMenu = new QMenu(_ui->action_Snapshot_Topology->text(),this);
    snapshotMenu->addAction(_ui->action_Snapshot_Topology);
    snapshotMenu->addAction(_ui->action_Snapshot_Selection);
    snapshotMenu->addAction(_ui->action_Snapshot_View);
    _snapshotButton->setMenu(snapshotMenu);
    _ui->action_Toggle_Entity_Label->setChecked(true);
    _defineMapper = new QSignalMapper(this);
    connect(_defineMapper, SIGNAL(mapped(QString)), this, SLOT(_defineMappingTriggered(QString)));
    _defineBehaviorMapper = new QSignalMapper(this);
    connect(_defineBehaviorMapper, SIGNAL(mapped(int)), this, SLOT(_defineBehaviorMappingTriggered(int)));

    _loadedModel = new LoadedTopologyModel(this);
    connect(this, SIGNAL(topologyOpened(HnsClient::Topology*)), _loadedModel, SLOT(topologyOpenedSlot(HnsClient::Topology*)));
    connect(this, SIGNAL(topologyChanged(HnsClient::Topology*)), _loadedModel, SLOT(topologyChangedSlot(HnsClient::Topology*)));
    connect(this, SIGNAL(topologyClosed(QString)), _loadedModel, SLOT(topologyClosedSlot(QString)));

    _drawWireButton = new QToolButton(this);
    _drawWireButton->setPopupMode(QToolButton::MenuButtonPopup);
    QMenu *drawWireMenu = new QMenu(_ui->action_Draw_wire->text(), this);
    drawWireMenu->addAction(_ui->action_Draw_wire);
    drawWireMenu->addAction(_ui->action_Draw_wire_1_angle);
    drawWireMenu->addAction(_ui->action_Draw_wire_2angles);
    _drawWireButton->setMenu(drawWireMenu);
    _drawWireButton->setDefaultAction(_ui->action_Draw_wire);

    _ui->topologyToolBar->initModel(_loadedModel);
    _ui->topologyToolBar->insertAction(_ui->topologyToolBar->comboAction(), _ui->action_Create_topology);
    _ui->topologyToolBar->insertAction(_ui->topologyToolBar->comboAction(), _ui->action_Open_topology);
    _ui->topologyToolBar->insertAction(_ui->topologyToolBar->comboAction(), _ui->action_Save_topology);
    _ui->topologyToolBar->insertAction(_ui->topologyToolBar->comboAction(), _ui->action_Save_topology_as);
    _ui->topologyToolBar->insertAction(_ui->topologyToolBar->comboAction(), _ui->action_Build_Topology_From);
    _ui->topologyToolBar->insertAction(_ui->topologyToolBar->comboAction(), _ui->action_Build_Model_From);
    _ui->topologyToolBar->insertSeparator(_ui->topologyToolBar->comboAction());
    _ui->topologyToolBar->insertAction(_ui->topologyToolBar->comboAction(), _ui->action_Topology_properties);
    _ui->topologyToolBar->insertSeparator(_ui->topologyToolBar->comboAction());
    _ui->topologyToolBar->insertWidget(_ui->topologyToolBar->comboAction(),_printButton);
    _ui->topologyToolBar->insertWidget(_ui->topologyToolBar->comboAction(),_snapshotButton);
    _ui->topologyToolBar->insertSeparator(_ui->topologyToolBar->comboAction());
    _ui->topologyToolBar->insertAction(_ui->topologyToolBar->closeAction(),_ui->action_Resize_Scene);
    _ui->topologyToolBar->insertAction(_ui->topologyToolBar->closeAction(), _ui->action_Display_grid);
    _ui->topologyToolBar->insertSeparator(_ui->topologyToolBar->closeAction());
    _ui->topologyToolBar->insertAction(_ui->topologyToolBar->closeAction(),_ui->action_Toggle_Entity_Label);
    _ui->topologyToolBar->insertAction(_ui->topologyToolBar->closeAction(), _ui->action_Toggle_Status_Icons);
    _ui->topologyToolBar->insertAction(_ui->topologyToolBar->closeAction(), _ui->action_Select_and_move);
    _ui->topologyToolBar->insertWidget(_ui->topologyToolBar->closeAction(), _drawWireButton);
    _ui->topologyToolBar->insertAction(_ui->topologyToolBar->closeAction(), _ui->action_Current_wire_color);

    addAction(_ui->action_Close_topology);
    addAction(_ui->action_Close_all_topologies);
    addAction(_ui->action_Select_all);

    _ui->action_Create_topology->setEnabled(false);
    _ui->action_Open_topology->setEnabled(false);
    _ui->action_Close_topology->setEnabled(false);
    _ui->action_Close_all_topologies->setEnabled(false);

    _ui->action_Save_topology->setEnabled(false);
    _ui->action_Save_topology_as->setEnabled(false);
    _ui->action_Build_Topology_From->setEnabled(false);
    _ui->action_Build_Model_From->setEnabled(false);
    _ui->action_Topology_properties->setEnabled(false);
    _ui->action_Resize_Scene->setEnabled(false);
    _ui->action_Select_and_move->setEnabled(false);
    _ui->action_Draw_wire->setEnabled(false);
    _ui->action_Draw_wire_1_angle->setEnabled(false);
    _ui->action_Draw_wire_2angles->setEnabled(false);
    _ui->action_Current_wire_color->setEnabled(false);
    _ui->action_Print_Selection->setEnabled(false);
    _ui->action_Print_Topology->setEnabled(false);
    _ui->action_Print_View->setEnabled(false);
    _ui->action_Snapshot_Selection->setEnabled(false);
    _ui->action_Snapshot_Topology->setEnabled(false);
    _ui->action_Snapshot_View->setEnabled(false);
    QActionGroup *modeGroup = new QActionGroup(this);
    modeGroup->addAction(_ui->action_Select_and_move);
    modeGroup->addAction(_ui->action_Draw_wire);
    modeGroup->addAction(_ui->action_Draw_wire_1_angle);
    modeGroup->addAction(_ui->action_Draw_wire_2angles);

    _ui->entityToolBar->setCloseable(false);

    _hideWhenModelList.append(_ui->entityToolBar->addWidget(_defineButton));
    _ui->action_Define_entity->setEnabled(false);


    _hideWhenModelList.append(_ui->entityToolBar->addSeparator());

    _hideWhenModelList.append(_ui->action_Start_entity);
    _ui->entityToolBar->addAction(_ui->action_Start_entity);
    _ui->action_Start_entity->setEnabled(false);

    _hideWhenModelList.append(_ui->action_Pause_entity);
    _ui->entityToolBar->addAction(_ui->action_Pause_entity);
    _ui->action_Pause_entity->setEnabled(false);

    _hideWhenModelList.append(_ui->action_Save_entity);
    _ui->entityToolBar->addAction(_ui->action_Save_entity);
    _ui->action_Save_entity->setEnabled(false);

    _hideWhenModelList.append(_ui->action_Stop_entity);
    _ui->entityToolBar->addAction(_ui->action_Stop_entity);
    _ui->action_Stop_entity->setEnabled(false);

    _hideWhenModelList.append(_ui->action_Undefine_entity);
    _ui->entityToolBar->addAction(_ui->action_Undefine_entity);
    _ui->action_Undefine_entity->setEnabled(false);

    _hideWhenModelList.append(_ui->action_Remote_display);
    _ui->entityToolBar->addAction(_ui->action_Remote_display);
    _ui->action_Remote_display->setEnabled(false);

    _hideWhenModelList.append(_ui->action_Usb_redirection);
    _ui->entityToolBar->addAction(_ui->action_Usb_redirection);
    _ui->action_Usb_redirection->setEnabled(false);

    _hideWhenModelList.append(_ui->entityToolBar->addSeparator());

    _ui->entityToolBar->addAction(_ui->action_Properties);
    _ui->action_Properties->setEnabled(false);

    _ui->entityToolBar->addAction(_ui->action_Delete_selected_items);
    _ui->action_Delete_selected_items->setEnabled(false);

    _hideWhenModelList.append(_ui->entityToolBar->addSeparator());

    _hideWhenModelList.append(_ui->action_Load_Balancer);
    _ui->entityToolBar->addAction(_ui->action_Load_Balancer);
    _ui->action_Load_Balancer->setEnabled(false);

    _hideWhenModelList.append(_ui->entityToolBar->addSeparator());

    _ui->action_Create_snapshot->setEnabled(false);
    _ui->entityToolBar->addAction(_ui->actionSnapshot_manager);
    _ui->actionSnapshot_manager->setEnabled(false);
    _ui->entityToolBar->addAction(_ui->action_display_memory_dump);
    _ui->action_display_memory_dump->setEnabled(false);

    connect(_ui->topologyToolBar, SIGNAL(currentIndexChanged(QVariant)), this, SLOT(_setCurrentTopologySlot(QVariant)));
    connect(_ui->topologyToolBar, SIGNAL(closeTriggered()), this, SLOT(closeTopologySlot()));
    connect(_ui->topologyToolBar, SIGNAL(currentObjectIndexChanged(qlonglong)), this, SLOT(_setSelectedEntitiesSlot(qlonglong)));
    connect(this, SIGNAL(topologyActivated(QVariant)), _ui->topologyToolBar, SLOT(setSelectedItem(QVariant)));
    connect(modeGroup, SIGNAL(triggered(QAction*)), this, SLOT(_insertModeSelectedSlot(QAction*)));

    HnvMainWindow *window = HnvMainWindow::instance();
    connect(window, SIGNAL(sessionConnected(HyneviewSession*)), this, SLOT(_connectedSlot(HyneviewSession*)));
    connect(window, SIGNAL(sessionDisconnected()), this, SLOT(_disconnectedSlot()));
    connect(this, SIGNAL(cursorPositionChanged(int,int,bool)), window, SLOT(cursorPositionChangedSlot(int,int,bool)));
    connect(this, SIGNAL(zoomValueChanged(int)), window, SLOT(zoomValueChangedSlot(int)));
    connect(window, SIGNAL(zoomValueChanged(int)), this, SLOT(_zoomValueChangedSlot(int)));
    connect (this,SIGNAL(clearLabelCoord()),window,SLOT(clearLabelCoord()));
    connect(drawWireMenu, SIGNAL(triggered(QAction*)), _drawWireButton, SLOT(setDefaultAction(QAction*)));

    QList<QAction *> menuAction;
    menuAction.append(_ui->action_Close_topology);
    menuAction.append(_ui->action_Close_all_topologies);
    window->addFileMenuAction(menuAction);
    menuAction.clear();
    menuAction.append(_ui->action_Create_topology);
    menuAction.append(_ui->action_Open_topology);
    menuAction.append(_ui->action_Save_topology);
    menuAction.append(_ui->action_Save_topology_as);
    menuAction.append(_ui->action_Build_Topology_From);
    menuAction.append(_ui->action_Build_Model_From);
    window->addFileMenuAction(menuAction);

    connect(_ui->stackedWidget, SIGNAL(currentChanged(int)), this, SLOT(_currentTopologyChangedSlot(int)));
    addActions(_ui->entityToolBar->actions());

    _ui->action_Create_topology->setIcon(QIcon(Ressources::themedPath(Constants::ICON_NEW)));
    _ui->action_Print_Topology->setIcon(QIcon(Ressources::themedPath(Constants::ICON_PRINT_TOPOLOGY)));
    _ui->action_Print_View->setIcon(QIcon(Ressources::themedPath(Constants::ICON_PRINT_VIEW)));
    _ui->action_Print_Selection->setIcon(QIcon(Ressources::themedPath(Constants::ICON_PRINT_SELECTION)));
    _ui->action_Snapshot_Topology->setIcon(QIcon(Ressources::themedPath(Constants::ICON_SNAPSHOT_TOPOLOGY)));
    _ui->action_Snapshot_View->setIcon(QIcon(Ressources::themedPath(Constants::ICON_SNAPSHOT_VIEW)));
    _ui->action_Snapshot_Selection->setIcon(QIcon(Ressources::themedPath(Constants::ICON_SNAPSHOT_SELECTION)));
    _ui->action_Define_entity->setIcon(QIcon(Ressources::themedPath(Constants::ICON_DEFINE)));
    _ui->action_Undefine_entity->setIcon(QIcon(Ressources::themedPath(Constants::ICON_UNDEFINE)));
    _ui->action_Load_Balancer->setIcon(QIcon(Ressources::themedPath(Constants::ICON_LOAD_BALANCING)));
    _ui->action_Delete_selected_items->setIcon(QIcon(Ressources::themedPath(Constants::ICON_DELETE)));
    _ui->action_Display_grid->setIcon(QIcon(Ressources::themedPath(Constants::ICON_DISPLAY_GRID)));
    _ui->action_Toggle_Entity_Label->setIcon(QIcon(Ressources::themedPath(Constants::ICON_TOGGLE_LABEL)));
    _ui->action_Toggle_Status_Icons->setIcon(QIcon(Ressources::themedPath(Constants::GRAPHIC_CREATED)));
    _ui->action_Draw_wire->setIcon(QIcon(Ressources::themedPath(Constants::ICON_WIRE)));
    _ui->action_Draw_wire_1_angle->setIcon(QIcon(Ressources::themedPath(Constants::ICON_WIRE_1_ANGLE)));
    _ui->action_Draw_wire_2angles->setIcon(QIcon(Ressources::themedPath(Constants::ICON_WIRE_2_ANGLE)));
    _ui->action_Open_topology->setIcon(QIcon(Ressources::themedPath(Constants::ICON_OPEN)));
    _ui->action_Pause_entity->setIcon(QIcon(Ressources::themedPath(Constants::ICON_PAUSE)));
    _ui->action_Save_entity->setIcon(QIcon(Ressources::themedPath(Constants::ICON_SAVE)));
    _ui->action_Properties->setIcon(QIcon(Ressources::themedPath(Constants::ICON_PROPERTIES)));
    _ui->action_Remote_display->setIcon(QIcon(Ressources::themedPath(Constants::ICON_REMOTE_DISPLAY)));
    _ui->action_Usb_redirection->setIcon(QIcon(Ressources::themedPath(Constants::ICON_USB_REDIRECTION)));
    _ui->action_Save_topology->setIcon(QIcon(Ressources::themedPath(Constants::ICON_SAVE)));
    _ui->action_Save_topology_as->setIcon(QIcon(Ressources::themedPath(Constants::ICON_SAVE_AS)));
    _ui->action_Build_Topology_From->setIcon(QIcon(Ressources::themedPath(Constants::ICON_BUILD_TOPOLOGY)));
    _ui->action_Build_Model_From->setIcon(QIcon(Ressources::themedPath(Constants::ICON_BUILD_MODEL)));
    _ui->action_Select_and_move->setIcon(QIcon(Ressources::themedPath(Constants::ICON_SELECTOR)));
    _ui->action_Start_entity->setIcon(QIcon(Ressources::themedPath(Constants::ICON_START)));
    _ui->action_Stop_entity->setIcon(QIcon(Ressources::themedPath(Constants::ICON_STOP)));
    _ui->action_Unimplement_entity->setIcon(QIcon(Ressources::themedPath(Constants::GRAPHIC_UNIMPLEMENTED)));
    _ui->action_Topology_properties->setIcon(QIcon(Ressources::themedPath(Constants::ICON_PROPERTIES)));
    _ui->action_Resize_Scene->setIcon(QIcon(Ressources::themedPath(Constants::ICON_RESIZE_SCENE)));
    _ui->action_Display_Capture_Job->setIcon(QIcon(Ressources::themedPath(Constants::ICON_REMOTE_DISPLAY)));
    _ui->action_Display_Capture_Job->setIcon(QIcon(Ressources::themedPath(Constants::ICON_CAPTURE_JOB)));
    _ui->action_Current_wire_color->setIcon(QIcon(Ressources::themedPath(Constants::ICON_COLORIZE)));
    _ui->action_Change_wire_color->setIcon(QIcon(Ressources::themedPath(Constants::ICON_COLORIZE)));

    _ui->action_Create_snapshot->setIcon(QIcon(Ressources::themedPath(Constants::ICON_CREATE_SNAPSHOT)));
    _ui->actionSnapshot_manager->setIcon(QIcon(Ressources::themedPath(Constants::ICON_CREATE_SNAPSHOT)));
    _ui->action_display_memory_dump->setIcon(QIcon(Ressources::themedPath(Constants::ICON_MEMORY_DUMP)));

    _ui->action_Select_all->setEnabled(true);

    _ui->action_Display_grid->setChecked(_gridEnabled);

    connect(RemoteAccess::RemoteManager::instance(), SIGNAL(canRedirectUsbChanged(int)), this, SLOT(_canRedirectUsbChangedSlot(int)));
}

TopologyWidget::~TopologyWidget()
{
    delete _drawScreenshotMapper;
    _disconnectedSlot();
    delete _loadedModel;
    delete _ui;
}

LoadedTopologyModel *TopologyWidget::loadedTopologyModel()
{
    return _loadedModel;
}

TopologyContentModel *TopologyWidget::topologyContentModel()
{
    TopologyContentModel *model = 0;
    TopologyViewArea *targetArea = _getArea();
    if (targetArea)
        model = targetArea->getContentModel();
    return model;
}

void TopologyWidget::setCurrentTopology(QString topologyUUID)
{
    if (_areaHash.contains(topologyUUID)) {
        // Disconnect current view
        QWidget *currentContainer = _ui->stackedWidget->currentWidget();
        if (currentContainer) {
            TopologyContainer *container = qobject_cast<TopologyContainer *>(currentContainer);
            QWidget *currentView = container->topologyViewArea();
            if (currentView) {
                disconnect(currentView, 0, this, 0);
                TopologyViewArea *currentTopologyView = qobject_cast<TopologyViewArea *>(currentView);
                if (currentTopologyView && currentTopologyView->scene())
                    disconnect(currentTopologyView->scene(), 0, this, 0);
            }
        }

        TopologyViewArea *targetArea = _areaHash.value(topologyUUID);
        connect(targetArea, SIGNAL(contextMenuRequested(QPointF)), this, SLOT(_contextMenuRequestedSlot(QPointF)));
        connect(targetArea, SIGNAL(cursorPositionChanged(int,int,bool)), this, SIGNAL(cursorPositionChanged(int,int,bool)));
        connect(targetArea, SIGNAL(zoomValueChanged(int)), this, SIGNAL(zoomValueChanged(int)));
        connect(targetArea, SIGNAL(entityCreated(HnsClient::GenericEntity*)), this, SLOT(_entityCreatedSlot(HnsClient::GenericEntity *)));
        connect(targetArea, SIGNAL(entityChanged(HnsClient::GenericEntity *)), this, SLOT(_entityChangedSlot(HnsClient::GenericEntity *)));
        connect(targetArea, SIGNAL(entityRemoved(qlonglong)), this, SLOT(_entityRemovedSlot(qlonglong)));
        connect(targetArea->scene(), SIGNAL(selectionChanged()), this , SLOT(_entitySelectionChangedSlot()));
        connect(targetArea, SIGNAL(loadingEnded(qlonglong)), this, SLOT(_topologyAreaLoadedSlot(qlonglong)));
        connect(targetArea, SIGNAL(topologyChanged(HnsClient::Topology*)), this, SIGNAL(topologyChanged(HnsClient::Topology*)));
        connect(targetArea, SIGNAL(remoteDisplayOpened()), this, SLOT(_entitySelectionChangedSlot()));
        _ui->stackedWidget->setCurrentWidget(_containerHash.value(targetArea));

        foreach(QAction *mutableAction, _hideWhenModelList) {
            bool visible = !targetArea->topologyEntry()->isModel();
            mutableAction->setVisible(visible);
        }
    }
}

void TopologyWidget::displayTopologyProperties(QString topologyUUID)
{
    _displayTopologyProperties(_getArea(topologyUUID));
}

void TopologyWidget::displayTopologyProperties(TopologyItem *item)
{
    if (!item)
        return;
    _displayTopologyProperties(item->topologyEntry(), item);
}

void TopologyWidget::createTopology()
{
    TopologyAdministrator *administrator = HyneviewSession::instance()->topologyAdministrator();
    TopologySizeDialog popup(true, this);
    if (popup.exec() == QDialog::Accepted) {
        HnvMainWindow::instance()->remoteProcessStarted(tr("Creating new session"), tr("Please wait while creating new session."));
        administrator->createTopology(
                    popup.getWidth(),
                    popup.getHeight(),
                    popup.getColor().name(),
                    popup.isModel(),
                    this,
                    SLOT(_createTopologySlot(HnsClient::Topology *)));
    }
}

void TopologyWidget::displayTopologyManager()
{
    TopologyAdministrator *administrator = HyneviewSession::instance()->topologyAdministrator();
    TopologyDialog popup(administrator, HnvMainWindow::instance()->topologyCatalogModel(), this);
    if (popup.exec()) {
        QList<HnsClient::TopologyEntry *> selectedEntries = popup.selectedEntries();
        foreach (HnsClient::TopologyEntry * selectedEntry, selectedEntries) {
            if (selectedEntry) {
                QString topologyUUID = selectedEntry->getUUID();
                if (_areaHash.contains(topologyUUID)) {
                    _activateSimulationMode();
                    setCurrentTopology(topologyUUID);
                } else {
                    HnvMainWindow::instance()->remoteProcessStarted(
                                tr("Loading session %1").arg(selectedEntry->getShortName()),
                                tr("Please wait while loading session %1.").arg(selectedEntry->getShortName()));
                    administrator->loadTopology(topologyUUID, this, SLOT(_loadTopologySlot(HnsClient::Topology *)));
                }
            }
        }
    }
}

TopologyView * TopologyWidget::getCurrentView()
{
    TopologyView * currentView = 0;
    TopologyViewArea *targetArea = _getArea();
    if (targetArea)
        currentView = targetArea->topologyView();
    return currentView;
}

QAction * TopologyWidget::getActionNew()
{
    return _ui->action_Create_topology;
}

QAction * TopologyWidget::getActionOpen()
{
    return _ui->action_Open_topology;
}

void TopologyWidget::setEntitySelection(QList<qlonglong> entityIDList)
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea) {
        int listCount = entityIDList.count();
        switch(listCount) {
        case 0: {
            targetArea->setSelectedEntity(0);
            break;
        }
        case 1: {
            targetArea->setSelectedEntity(entityIDList.first());
            targetArea->centerOnEntity(entityIDList.first());
            break;
        }
        default:
            targetArea->setSelectedEntities(entityIDList);
        }
    }
}

void TopologyWidget::centerOnEntity(qlonglong entityID)
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea)
        targetArea->centerOnEntity(entityID);
}

void TopologyWidget::deleteItemsByIDs(QList<qlonglong> itemList)
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea)
        targetArea->deleteItemsByIDs(itemList);
}

QList<HnsClient::GenericEntity *> TopologyWidget::getCurrentEntitiesSelected(bool includeWires)
{
    QList<HnsClient::GenericEntity *> listEntity;
    TopologyViewArea * targetArea = _getArea();
    if (targetArea)
        listEntity = targetArea->getSelectedEntities(includeWires);
    return listEntity;
}

void TopologyWidget::refreshAllEntities()
{
    for (int count=0;count<_ui->stackedWidget->count();count++){
        TopologyViewArea *topologyViewArea = qobject_cast<TopologyContainer *>(_ui->stackedWidget->widget(count))->topologyViewArea();
        QHash<qlonglong, HnsClient::GenericEntity*> listEntities = topologyViewArea->getEntities();
        foreach (HnsClient::GenericEntity* entity,listEntities.values()){
            entity->refresh();
        }
    }
}

void TopologyWidget::displayEntityPropertiesById(qlonglong entityId)
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea)
        targetArea->displayPropertiesById(entityId);
}
/*******************************************************************************
* TopologyWidget - Public slots
*/
void TopologyWidget::setDetailOverlayCorner(Qt::Corner corner)
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea)
        targetArea->setDetailOverlayCorner(corner);
}

void TopologyWidget::hideDetailOverlay(bool checked)
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea)
        targetArea->hideDetailOverlay(checked);
}

bool TopologyWidget::closeTopologySlot(QString topologyUUID, bool forceClose)
{
    return _closeTopology(_getArea(topologyUUID), forceClose);
}

bool TopologyWidget::closeAllTopologiesSlot(QString excludedTopologyUUID)
{
    bool result = true;
    QHash<QString, TopologyViewArea*> areaHash = _areaHash;
    QMutableHashIterator<QString, TopologyViewArea*> iterator(areaHash);
    while(iterator.hasNext()) {
        iterator.next();
        if (iterator.key() == excludedTopologyUUID)
            iterator.remove();
        else {
            if (!iterator.value()->topology()->isModified()) {
                result &= _closeTopology(iterator.value());
                iterator.remove();
            }
        }
    }
    QList<QString> uuidToSave;
    if (areaHash.size() > 0) {
        TopologySaveAllDialog saveDialog(areaHash, this);
        QList<HnvDialogHandler *> topologyHandlerList;
        foreach(TopologyViewArea *area, areaHash.values()) {
            HnvDialogHandler *saveHandler = new HnvDialogHandler(area->topology(), &saveDialog, this);
            saveHandler->setSourceDestroyedMessage(tr("\"%1\" was deleted. Operation aborted.").arg(area->topologyEntry()->getShortName()));
            topologyHandlerList.append(saveHandler);
        }
        if (saveDialog.exec() == QDialog::Accepted) {
            uuidToSave = saveDialog.getTopologyUUIDToSave();
            qDeleteAll(topologyHandlerList);
        } else {
            qDeleteAll(topologyHandlerList);
            return false;
        }

        foreach (TopologyViewArea *area, areaHash) {
            if (uuidToSave.contains(area->topologyEntry()->getUUID())) {
                result &= _saveAndCloseTopology(area);
            } else {
                _destroyTopology(area, true);
            }
        }
    }
    return result;
}

/*******************************************************************************
* TopologyWidget - Privates slots
*/
void TopologyWidget::on_action_Create_topology_triggered()
{
    createTopology();
}

void TopologyWidget::on_action_Open_topology_triggered()
{
    displayTopologyManager();
}

void TopologyWidget::on_action_Close_topology_triggered()
{
    _activateSimulationMode();
    closeTopologySlot();
}

void TopologyWidget::on_action_Close_all_topologies_triggered()
{
    closeAllTopologiesSlot();
}

void TopologyWidget::on_action_Save_topology_triggered()
{
    _activateSimulationMode();
    TopologyViewArea *targetArea = _getArea();
    if (!targetArea)
        qCritical() << tr("Unable to retrieve topology area.");
    else
        _saveTopology(targetArea);
}

void TopologyWidget::on_action_Save_topology_as_triggered()
{
    _activateSimulationMode();
    TopologyViewArea *targetArea = _getArea();
    if (!targetArea)
        qCritical() << tr("Unable to retrieve topology area.");
    else
        _saveTopologyAs(targetArea);
}

void TopologyWidget::on_action_Topology_properties_triggered()
{
    TopologyViewArea *targetArea = _getArea();
    if (!targetArea) {
        qCritical() << tr("Unable to retrieve topology area.");
    } else {
        _displayTopologyProperties(targetArea);
    }
}

void TopologyWidget::on_action_Display_grid_toggled(bool checked)
{
    _toggleGrid(checked);
}

void TopologyWidget::on_action_Delete_selected_items_triggered()
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea)
        targetArea->deleteSelected();
}

void TopologyWidget::on_action_Properties_triggered()
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea)
        targetArea->displayPropertiesSelected();
}

void TopologyWidget::on_action_Pause_entity_triggered()
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea)
        targetArea->suspendSelected();
}

void TopologyWidget::on_action_Save_entity_triggered()
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea)
        targetArea->saveSelected();
}

void TopologyWidget::on_action_Start_entity_triggered()
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea)
        targetArea->startSelected();
}

void TopologyWidget::on_action_Stop_entity_triggered()
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea)
        targetArea->stopSelected();
}

void TopologyWidget::on_action_Undefine_entity_triggered()
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea)
        targetArea->undefineSelected();
}

void TopologyWidget::on_action_Unimplement_entity_triggered()
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea)
        targetArea->unImplementSelected();
}

void TopologyWidget::on_action_Remote_display_triggered()
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea)
        targetArea->remoteDisplaySelected();
}

void TopologyWidget::on_action_Usb_redirection_triggered()
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea){
        targetArea->displayUsbManager();
    }
}

void TopologyWidget::on_action_Resize_Scene_triggered()
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea) {
        Hns::TopologyLayout layout = targetArea->topology()->layout();
        TopologySizeDialog popup(false, this);
        HnvDialogHandler * propertiesHandler = new HnvDialogHandler(targetArea->topology(), &popup, this);
        propertiesHandler->setSourceDestroyedMessage(tr("\"%1\" was deleted. Operation aborted.").arg(targetArea->topology()->topologyEntry()->getShortName()));
        popup.setSize(layout.layoutOptions.width,layout.layoutOptions.height);
        QString colorCode = QString::fromStdString(layout.layoutOptions.colorCode);
        if (QColor::isValidColor(colorCode))
            popup.setColor(QColor(colorCode));
        QRect boudingRect = targetArea->scene()->itemsBoundingRect().toRect();
        int minW = boudingRect.x() + boudingRect.width();
        int minH = boudingRect.y() + boudingRect.height();
        popup.setMinimalSize(minW, minH);
        popup.setIsModel(targetArea->topologyEntry()->isModel());
        if (popup.exec() == QDialog::Accepted) {
            targetArea->topology()->setMinimalSize(minW, minH);
            targetArea->topology()->setLayoutInformation(popup.getWidth(), popup.getHeight(), popup.getColor().name());
        }
    }
}

void TopologyWidget::on_action_Print_Topology_triggered()
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea)
        targetArea->printTopology();
}

void TopologyWidget::on_action_Print_Selection_triggered()
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea){
        targetArea->printSelection();
    }
}

void TopologyWidget::on_action_Print_View_triggered()
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea){
        targetArea->printView();
    }
}

void TopologyWidget::on_action_Snapshot_Topology_triggered()
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea){
        targetArea->snapshotTopology();
    }
}

void TopologyWidget::on_action_Snapshot_View_triggered()
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea){
        targetArea->snapshotView();
    }
}

void TopologyWidget::on_action_Snapshot_Selection_triggered()
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea){
        targetArea->snapshotSelection();
    }
}

void TopologyWidget::on_action_Toggle_Entity_Label_triggered(bool checked)
{
    foreach (TopologyViewArea *targetArea,_areaHash.values()){
        targetArea->toggleEntityLabel(checked);
    }
}

void TopologyWidget::on_action_Force_to_Default_triggered()
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea)
        targetArea->triggerForceDisplayLabel(Hns::LabelPolicyDefault);
}

void TopologyWidget::on_action_Force_to_Hide_triggered()
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea)
        targetArea->triggerForceDisplayLabel(Hns::LabelPolicyHide);
}

void TopologyWidget::on_action_Force_to_Show_triggered()
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea)
        targetArea->triggerForceDisplayLabel(Hns::LabelPolicyShow);
}

void TopologyWidget::on_action_Custom_Graphic_triggered()
{
    GraphicSelectorDialog dialog(HyneviewSession::instance()->resourceManager()->getResources(), this);
    HnvDialogHandler *popupHandler = new HnvDialogHandler(_getArea(), &dialog, this);
    popupHandler->setSourceDestroyedMessage(tr("\"%1\" was deleted. Operation aborted.").arg(_getArea()->topologyEntry()->getShortName()));
    if (dialog.exec() == QDialog::Accepted) {
        TopologyViewArea *targetArea = _getArea();
        if (targetArea)
            targetArea->setCustomGraphic(dialog.selectedFileName());
    }
}

void TopologyWidget::on_action_Display_Capture_Job_triggered()
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea){
        targetArea->displayCaptureJob();
    }
}

void TopologyWidget::on_action_Select_all_triggered()
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea){
        targetArea->selectAll();
    }
}

void TopologyWidget::on_action_Load_Balancer_triggered()
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea)
        targetArea->loadBalancerSelected();
}

void TopologyWidget::on_action_Build_Topology_From_triggered()
{
    _switchTopologyType();
}

void TopologyWidget::on_action_Build_Model_From_triggered()
{
    _switchTopologyType();
}

void TopologyWidget::on_action_Create_snapshot_triggered()
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea)
        targetArea->createSnapshot();
}

void TopologyWidget::on_actionSnapshot_manager_triggered()
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea){
        targetArea->displaySnapshotManager();
    }
}

void TopologyWidget::on_action_Current_wire_color_triggered()
{
    QColor returnColor = QColorDialog::getColor(_wireColor, this, tr("Color"));
    if (returnColor.isValid()) {
        _wireColor = returnColor;
        foreach (TopologyViewArea *area, _areaHash)
            area->setWireColor(_wireColor);
    }
}

void TopologyWidget::on_action_Change_wire_color_triggered()
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea){
        QColor returnColor = QColorDialog::getColor(_wireColor, this, tr("Color"));
        if (returnColor.isValid()) {
            QList<HnsClient::GenericEntity *> entities;
            QList<HnsClient::WireEntity *> wires;
            targetArea->getSelectedEntities(entities, wires);
            if (wires.count()) {
                foreach (HnsClient::WireEntity *wire, wires) {
                    Hns::WireLayoutPtr layout = Hns::WireLayoutPtr::dynamicCast(wire->getLayout()->ice_clone());
                    layout->colorCode = returnColor.name().toStdString();
                    targetArea->topology()->updateWireLayout(layout);
                }
            }
        }
    }
}

void TopologyWidget::on_action_Toggle_Status_Icons_toggled(bool checked)
{
    foreach (TopologyViewArea *targetArea,_areaHash.values()){
        targetArea->setStatusIconVisibility(checked);
    }
    QSettings settings;
    settings.beginGroup(Constants::TOPOLOGY_SETTINGS);
    settings.setValue(Constants::TOPOLOGY_STATUS_ICON, checked);
    settings.endGroup();
}

void TopologyWidget::_contextMenuRequestedSlot(QPointF screenPos)
{
    TopologyViewArea * targetArea = _getArea();
    if (!targetArea)
        return;
    TopologyViewArea::selectionStatus status = targetArea->getSelectionStatus();
    QList<HnsClient::GenericEntity *> entities;
    QList<HnsClient::WireEntity *> wires;
    targetArea->getSelectedEntities(entities, wires);
    //Reconstruction of the define menu, in case a node has been turned off

    _ui->topologyToolBar->setSelectedEntity(status.lastSelected);

    status.canDefine = _populateDefineMenu();
    //end of reconstruction
    QMenu menu(this);
    if (!targetArea->topologyEntry()->isModel()) {
        if (_defineButton->menu())
            menu.addMenu(_defineButton->menu());
        else
            menu.addAction(_ui->action_Define_entity);
        menu.addAction(_ui->action_Start_entity);
        menu.addAction(_ui->action_Pause_entity);
        menu.addAction(_ui->action_Save_entity);
        menu.addAction(_ui->action_Stop_entity);
        menu.addAction(_ui->action_Undefine_entity);
        if(targetArea->getIsImplementedFromModel()){
            menu.addAction(_ui->action_Unimplement_entity);
        }
        int flags = targetArea->getCapabilitiesFlags();
        if (flags & Hns::NET_CAPTURE){
            menu.addAction(_ui->action_Display_Capture_Job);
        }
        if (flags & Hns::REMOTE_DISPLAY){
            menu.addAction(_ui->action_Remote_display);
            menu.addAction(_ui->action_Usb_redirection);
        }
        if (flags & Hns::SNAPSHOT_FLAG) {
            menu.addAction(_ui->action_Create_snapshot);
            menu.addAction(_ui->actionSnapshot_manager);
        }
        if (flags & Hns::MEMORY_DUMP_FLAG) {
            menu.addAction(_ui->action_display_memory_dump);
        }
        menu.addSeparator();
        if (wires.count()) {
            menu.addAction(_ui->action_Change_wire_color);
        }
        QList<EntityActionDispatcher *> dispatcherList = targetArea->getSelectionCustomActions();
        if (dispatcherList.count()) {
            foreach(EntityActionDispatcher * dispatcher, dispatcherList) {
                QAction * customAction = dispatcher->action(_ui->action_Stop_entity->isEnabled());
                customAction->setParent(&menu);
                menu.addAction(customAction);
            }
        }
        menu.addSeparator();
    } else {
        QList<EntityActionDispatcher *> dispatcherList = targetArea->getSelectionCustomActions(true);
        if (dispatcherList.count()) {
            foreach(EntityActionDispatcher * dispatcher, dispatcherList) {
                QAction * customAction = dispatcher->action(true);
                customAction->setParent(&menu);
                menu.addAction(customAction);
            }
            menu.addSeparator();
        }
    }
    menu.addAction(_ui->action_Delete_selected_items);
    menu.addSeparator();

    _forceDisplayLabel->menu()->setEnabled(false);
    foreach (QAction *action, _labelPolicyActionsHash)
        action->setChecked(false);

    if (!entities.isEmpty()) {
        bool enableMenu = false;
        bool validPolicy = true;
        Hns::LabelPolicy labelPolicy;
        foreach (HnsClient::GenericEntity *genericEntity, entities) {
            HnsClient::Entity *entity = qobject_cast<HnsClient::Entity *>(genericEntity);
            if (!entity)
                continue;
            if (!enableMenu) {
                enableMenu = true;
                labelPolicy = entity->getLayout()->labelPolicyState;
            } else {
                if (labelPolicy != entity->getLayout()->labelPolicyState)
                    validPolicy = false;
            }
        }
        if (enableMenu) {
            _forceDisplayLabel->menu()->setEnabled(true);
            if (validPolicy)
                _labelPolicyActionsHash.value(labelPolicy)->setChecked(true);
        }
    }
    menu.addMenu(_forceDisplayLabel->menu());
    if (wires.count() == 0)
        menu.addAction(_ui->action_Custom_Graphic);
    menu.addAction(_ui->action_Properties);
    menu.exec(screenPos.toPoint());
}

void TopologyWidget::_insertModeSelectedSlot(QAction *sourceAction)
{
    TopologyView::InsertMode insertMode = TopologyView::SelectMode;
    if (sourceAction == _ui->action_Draw_wire)
        insertMode = TopologyView::InsertWireMode;
    else if (sourceAction == _ui->action_Draw_wire_1_angle)
        insertMode = TopologyView::InsertWire1AngleMode;
    else if (sourceAction == _ui->action_Draw_wire_2angles)
        insertMode = TopologyView::InsertWire2AnglesMode;
    foreach(TopologyViewArea* area, _areaHash)
        area->setInsertMode(insertMode);
}

void TopologyWidget::_currentTopologyChangedSlot(int index)
{
    Q_UNUSED(index);
    _refreshViewArea();
}

void TopologyWidget::_setCurrentTopologySlot(QVariant uuidData)
{
    if (uuidData.isValid())
        setCurrentTopology(uuidData.toString());
}

void TopologyWidget::_setSelectedEntitiesSlot(qlonglong entityId)
{
    TopologyViewArea *targetArea = _getArea();
    if(targetArea)
        targetArea->setSelectedEntity(entityId);
}

void TopologyWidget::_zoomValueChangedSlot(int value)
{
    TopologyViewArea * targetArea = _getArea();
    if (targetArea)
        targetArea->setZoomValue(value);
}

void TopologyWidget::_connectedSlot(HyneviewSession *session)
{
    connect(session->topologyManager(), SIGNAL(topologyUnloaded(QString,bool)), this, SLOT(_topologyUnloadedSlot(QString,bool)));
    connect(session->topologyManager(), SIGNAL(topologySavedAs(QString,QString)), this, SLOT(_topologySavedAsByTopologyManagerSlot(QString,QString)));
    _ui->topologyToolBar->setEnabled(true);
    _ui->action_Create_topology->setEnabled(true);
    _ui->action_Open_topology->setEnabled(true);
}

void TopologyWidget::_disconnectedSlot()
{
    foreach (QObject *o, children()) {
        QDialog *dialog = qobject_cast<QDialog *>(o);
        if (dialog)
            dialog->reject();
    }
    _ui->action_Create_topology->setEnabled(false);
    _ui->action_Open_topology->setEnabled(false);
    _ui->action_Close_topology->setEnabled(false);
    _ui->action_Close_all_topologies->setEnabled(false);
    _destroyAllTopologies();
    _loadedModel->clear();
    qDeleteAll(_areaHash);
    _areaHash.clear();
    _ui->topologyToolBar->setEnabled(false);
}

void TopologyWidget::_topologyAreaLoadedSlot(qlonglong topologyID)
{
    Q_UNUSED(topologyID);
    _refreshViewArea();
}

void TopologyWidget::_topologyUnloadedSlot(QString uuid, bool isDeleted)
{
    TopologyViewArea *area = _getArea(uuid);
    if (!area)
        return;
    QString topologyName = area->topologyEntry()->getShortName();
    _destroyTopology(area);

    QString message;
    if (isDeleted)
        message = tr("Topology \"%1\" has been deleted.").arg(topologyName);
    else
        message = tr("Topology \"%1\" has been unloaded.").arg(topologyName);
    QMessageBox::information(this, tr("Topology close"), message, QMessageBox::Ok, QMessageBox::Ok);
    emit topologyClosed(uuid);
}

void TopologyWidget::_entityCreatedSlot(HnsClient::GenericEntity *entity)
{
    _ui->topologyToolBar->addEntity(entity->getId(), entity->getShortName());
    if (_getArea()->isEntitySelected(entity->getId()))
        _entitySelectionChangedSlot();
}

void TopologyWidget::_entityChangedSlot(HnsClient::GenericEntity *entity)
{
    _ui->topologyToolBar->updateEntity(entity->getId(), entity->getShortName());
    if (_getArea()->isEntitySelected(entity->getId()))
        _entitySelectionChangedSlot();
}

void TopologyWidget::_entityRemovedSlot(qlonglong entityID)
{
    _ui->topologyToolBar->removeEntity(entityID);
}

void TopologyWidget::_loadTopologySlot(HnsClient::Topology *topology)
{
    _handleTopology(topology, tr("Loading failure..."), tr("Failed to load topology. Please check that all entities are available for use."));
}

void TopologyWidget::_loadTopologySwitchedSlot(HnsClient::Topology *topology)
{
    if (topology) {
        connect(topology, SIGNAL(contentLoaded()), _drawScreenshotMapper, SLOT(map()));
        _drawScreenshotMapper->setMapping(topology, topology->getUuid());
    }
    _loadTopologySlot(topology);
}

void TopologyWidget::_createTopologySlot(HnsClient::Topology *topology)
{
    _handleTopology(topology, tr("Creation failure..."), tr("Failed to create new topology."));
}

void TopologyWidget::_topologySavedSlot(QString topologyUUID, bool result, QString exceptionMessage)
{
    _handleSaveResult(topologyUUID, result, exceptionMessage);
}

void TopologyWidget::_topologySavedAndClosedSlot(QString topologyUUID, bool result, QString exceptionMessage)
{
    _topologySavedSlot(topologyUUID, result, exceptionMessage);
    if (result)
        _destroyTopology(topologyUUID, true);
}

void TopologyWidget::_topologySavedAsSlot(QString topologyUUID, QString newUUID, bool result, QString exceptionMessage)
{
    HnvMainWindow::instance()->remoteProcessEnded();
    if (!result) {
        QString message = tr("Could not save topology with UUID \"%1\"").arg(newUUID);
        if (!exceptionMessage.isEmpty())
            message.append("\n%1").arg(exceptionMessage);
        QMessageBox::warning(this, tr("Error"), message, QMessageBox::Ok, QMessageBox::Ok);
    } else {
        TopologyViewArea *area = _areaHash.take(topologyUUID);
        if (area)
            _areaHash.insert(newUUID, area);
    }
}

void TopologyWidget::_switchTopologyTypeSlot(qlonglong entryId)
{
    HnvMainWindow::instance()->remoteProcessEnded();
    if (entryId == -1) {
        QMessageBox::warning(this, tr("Error"), tr("Could not switch topology type"), QMessageBox::Ok, QMessageBox::Ok);
    } else {
        TopologyAdministrator *administrator = HyneviewSession::instance()->topologyAdministrator();
        HnvMainWindow::instance()->remoteProcessStarted(
                    tr("Loading session"),
                    tr("Please wait while loading session."));
        administrator->loadTopology(entryId, this, SLOT(_loadTopologySwitchedSlot(HnsClient::Topology *)));
    }
}

void TopologyWidget::_drawScreenShotSlot(QString uuid)
{
    _drawScreenshotMapper->disconnect(_getArea(uuid)->topology());
    _setScreenshot(uuid);
}

void TopologyWidget::_topologySavedAsByTopologyManagerSlot(QString oldUuid, QString newUuid)
{
    if (_areaHash.contains(oldUuid)) {
        _areaHash.insert(newUuid, _areaHash.take(oldUuid));
    }
}

void TopologyWidget::_entitySelectionChangedSlot()
{
    TopologyViewArea * targetArea = _getArea();
    if (!targetArea)
        return;
    TopologyViewArea::selectionStatus status = targetArea->getSelectionStatus();
    emit selectionChanged(targetArea->getSelectedEntities());

    _ui->topologyToolBar->setSelectedEntity(status.lastSelected);

    status.canDefine = _populateDefineMenu();

    _ui->action_Define_entity->setEnabled(status.canDefine);
    _ui->action_Start_entity->setEnabled(status.canStart);
    _ui->action_Pause_entity->setEnabled(status.canPause);
    _ui->action_Save_entity->setEnabled(status.canSave);
    _ui->action_Stop_entity->setEnabled(status.canStop);
    _ui->action_Undefine_entity->setEnabled(status.canUndefine);
    _ui->action_Remote_display->setEnabled(status.canRemote);
    _ui->action_Properties->setEnabled(status.canEditProperties);
    _ui->action_Delete_selected_items->setEnabled(status.canDelete);
    _ui->action_Display_Capture_Job->setEnabled(status.canCapture);
    _ui->action_Unimplement_entity->setEnabled(status.canUnImplement);
    QList<HnsClient::GenericEntity *> entities;
    QList<HnsClient::WireEntity *> wires;
    targetArea->getSelectedEntities(entities, wires);
    bool oneEntity = wires.count() == 0 && entities.count() == 1;
    _ui->action_Custom_Graphic->setEnabled(oneEntity);
    _ui->action_Create_snapshot->setEnabled(oneEntity && status.canCreateSnapshot);
    _ui->actionSnapshot_manager->setEnabled(oneEntity && status.canCreateSnapshot);
    _ui->action_Usb_redirection->setEnabled(oneEntity && status.canRedirectUsb);
    _ui->action_display_memory_dump->setEnabled(oneEntity && status.canDumpMemory);
    //    _refreshOverlayWidgetSlot();
}

void TopologyWidget::_defineMappingTriggered(const QString &nodeName)
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea)
        targetArea->defineSelected(nodeName);
}


void TopologyWidget::_defineBehaviorMappingTriggered(int idx)
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea)
        targetArea->loadBalanceAuto(targetArea->topology()->getMapBehavior()[idx].behavior);
}

/*******************************************************************************
* TopologyWidget - Private
*/
void TopologyWidget::_handleTopology(HnsClient::Topology *topology, QString errorTitle, QString errorMessage)
{
    _activateSimulationMode();
    HnvMainWindow::instance()->remoteProcessEnded();
    if (topology) {
        _initViewArea(topology);
    } else {
        QMessageBox::warning(this, errorTitle, errorMessage, QMessageBox::Ok, QMessageBox::Ok);
    }
}

void TopologyWidget::_displayTopologyProperties(TopologyViewArea *targetArea)
{
    if (!targetArea)
        return;
    _displayTopologyProperties(targetArea->topologyEntry(), targetArea->topology());
}

void TopologyWidget::_displayTopologyProperties(HnsClient::TopologyEntry *entry, QObject *source)
{
    HnsClient::TopologyEntry *updatedEntry = qobject_cast<HnsClient::TopologyEntry *>(entry->duplicateEntry(this));
    if (updatedEntry) {
        bool readOnly = updatedEntry->isNew() || updatedEntry->getStatus() != Hns::TOPOLOGY_STATUS_OK;
        EntryProperties popup(updatedEntry, this, readOnly);
        HnvDialogHandler *popupHandler = new HnvDialogHandler(source, &popup, this);
        popupHandler->setSourceDestroyedMessage(tr("\"%1\" was deleted. Operation aborted.").arg(entry->getShortName()));
        if (popup.exec() == QDialog::Accepted)
            HyneviewSession::instance()->topologyAdministrator()->updateEntry(updatedEntry);
    }
}

bool TopologyWidget::_closeTopology(TopologyViewArea *targetArea, bool forceToClose)
{
    if (!targetArea)
        return true;
    if (!forceToClose) {
        HnsClient::TopologyEntry *entry = targetArea->topologyEntry();
        HnsClient::Topology *topology = targetArea->topology();
        if (topology->isModified() && entry->isWriteable()) {
            QMessageBox msgBox(this);
            HnvDialogHandler *propertiesHandler = new HnvDialogHandler(topology, &msgBox, this);
            propertiesHandler->setSourceDestroyedMessage(tr("\"%1\" was deleted. Operation aborted.").arg(entry->getShortName()));
            msgBox.setWindowTitle(tr("Changes might be lost..."));
            msgBox.setText(tr("The topology '%1' has been modified.").arg(entry->getShortName()));
            msgBox.setInformativeText(tr("Do you want to save your changes?"));
            msgBox.setStandardButtons(QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel);
            msgBox.setDefaultButton(QMessageBox::Save);
            int ret = msgBox.exec();
            switch (ret) {
            case QMessageBox::Save:
                // Save was clicked
                _saveAndCloseTopology(targetArea);
                return true;
                break;
            case QMessageBox::Discard:
                // Don't Save was clicked
                break;
            case QMessageBox::Cancel:
                // Cancel was clicked
                return false;
                break;
            default:
                // Only when topology is remotely destroyed while waiting for answer
                return false;
            }
        }
    }
    _destroyTopology(targetArea, true);
    return true;
}

void TopologyWidget::_destroyAllTopologies()
{
    foreach(TopologyViewArea *targetArea, _areaHash.values())
        _destroyTopology(targetArea);
}

void TopologyWidget::_destroyTopology(QString topologyUUID, bool requestClose)
{
    _destroyTopology(_getArea(topologyUUID), requestClose);
}

void TopologyWidget::_destroyTopology(TopologyViewArea *targetArea, bool requestClose)
{
    if (targetArea != 0 && targetArea->topology() != 0) {
        QString topologyUUID = targetArea->topologyEntry()->getUUID();
        emit topologyClosed(topologyUUID);
        _ui->stackedWidget->removeWidget(_containerHash.value(targetArea));
        TopologyContainer *topologyContainer = _containerHash.take(targetArea);
        topologyContainer->deleteLater();
        _areaHash.remove(topologyUUID);
        targetArea->destroyArea();
        if (_areaHash.count() == 0) {
            _ui->action_Close_topology->setEnabled(false);
            _ui->action_Close_all_topologies->setEnabled(false);
        }
        // TODO : à remplacer par un controle et pas une désactivation systématique
        _defineButton->setEnabled(false);
        _ui->action_Define_entity->setEnabled(false);
        _ui->action_Start_entity->setEnabled(false);
        _ui->action_Pause_entity->setEnabled(false);
        _ui->action_Save_entity->setEnabled(false);
        _ui->action_Stop_entity->setEnabled(false);
        _ui->action_Remote_display->setEnabled(false);
        _ui->action_Usb_redirection->setEnabled(false);
        _ui->action_Properties->setEnabled(false);
        _ui->action_Delete_selected_items->setEnabled(false);
        _ui->action_Display_Capture_Job->setEnabled(false);
        _ui->action_Undefine_entity->setEnabled(false);
        _ui->action_Unimplement_entity->setEnabled(false);
        if (requestClose)
            HyneviewSession::instance()->topologyAdministrator()->closeTopology(topologyUUID);
//        malloc_trim(0);
    }
}

void TopologyWidget::_saveTopology(TopologyViewArea *targetArea)
{
    if (targetArea)
        _saveTopology(targetArea->topology(), this, SLOT(_topologySavedSlot(QString, bool, QString)));
}

bool TopologyWidget::_saveAndCloseTopology(TopologyViewArea *targetArea)
{
    if (targetArea)
        return _saveTopology(targetArea->topology(), this, SLOT(_topologySavedAndClosedSlot(QString, bool, QString)));
    return false;
}

bool TopologyWidget::_saveTopologyAs(TopologyViewArea *targetArea)
{
    if (targetArea) {
        HnsClient::Topology *topology = targetArea->topology();
        if (!topology->topologyEntry()->isWriteable()) {
            qDebug() << tr("Insuficient permissions");
            return false;
        }
        if (topology->topologyEntry()->isNew())
            return _saveTopology(topology, this, SLOT(_topologySavedSlot(QString, bool, QString)));
        HnsClient::TopologyEntry *updatedEntry = _displaySaveDialog(topology);
        if (updatedEntry) {
            Hns::ByteArray screenshotData = targetArea->createPreviewSnapshot();
            HnvMainWindow::instance()->remoteProcessStarted(tr("Saving session %1").arg(updatedEntry->getShortName()), tr("Please wait while saving session %1.").arg(updatedEntry->getShortName()));
            updatedEntry->entry()->hSetId = HyneviewSession::instance()->user()->hSetId;
            return HyneviewSession::instance()->topologyAdministrator()->saveTopologyAs(topology, updatedEntry, this, SLOT(_topologySavedAsSlot(QString, QString, bool, QString)), screenshotData);
        }
    }
    return false;
}

bool TopologyWidget::_saveTopology(HnsClient::Topology *topology, QObject *receiver, const char *slot)
{
    QString shortName = topology->topologyEntry()->getShortName();
    if (!topology->topologyEntry()->isWriteable()) {
        qDebug() << tr("Insuficient permissions");
        return false;
    }
    HnsClient::TopologyEntry *updatedEntry = 0;
    if (topology->topologyEntry()->isNew()) {
        updatedEntry = _displaySaveDialog(topology);
        if (!updatedEntry)
            return false;
        else
            shortName = updatedEntry->getShortName();
    }
    Hns::ByteArray screenshotData = _getArea(topology->getUuid())->createPreviewSnapshot();
    HnvMainWindow::instance()->remoteProcessStarted(tr("Saving session %1").arg(shortName), tr("Please wait while saving session %1.").arg(shortName));
    return HyneviewSession::instance()->topologyAdministrator()->saveTopology(topology, updatedEntry, receiver, slot, screenshotData);
}

void TopologyWidget::_switchTopologyType(HnsClient::Topology *topology) {

    if (topology == 0) {
        TopologyViewArea *targetArea = _getArea();
        if (!targetArea)
            qCritical() << tr("Unable to retrieve topology area.");
        else if (!targetArea->topologyEntry()->isWriteable())
            qDebug() << tr("Insuficient permissions");
        else
            topology = targetArea->topology();
    }

    if (topology) {
        QString defaultBackgroundColorKey = Constants::TOPOLOGY_DEFAULT_BACKGROUND_COLOR_KEY;
        QString defaultBackgroundColorValue = Constants::TOPOLOGY_DEFAULT_BACKGROUND_COLOR;
        if (!topology->topologyEntry()->isModel()) {
            defaultBackgroundColorKey = Constants::TOPOLOGY_DEFAULT_MODEL_BACKGROUND_COLOR_KEY;
            defaultBackgroundColorValue = Constants::TOPOLOGY_DEFAULT_MODEL_BACKGROUND_COLOR;
        }
        QSettings settings;
        settings.beginGroup(Constants::TOPOLOGY_SETTINGS);
        QString newBackgroundColor = "#" + settings.value(defaultBackgroundColorKey, defaultBackgroundColorValue).toString();
        HnvMainWindow::instance()->remoteProcessStarted(tr("Switching topology type of '%1'").arg(topology->topologyEntry()->getShortName()), tr("Please wait while switching topology type of '%1'.").arg(topology->topologyEntry()->getShortName()));
        HyneviewSession::instance()->topologyAdministrator()->switchTopologyType(topology->getUuid(), newBackgroundColor, this, SLOT(_switchTopologyTypeSlot(qlonglong)));
    }
}

HnsClient::TopologyEntry *TopologyWidget::_displaySaveDialog(HnsClient::Topology *topology)
{
    if (topology) {
        HnsClient::TopologyEntry *updatedEntry = dynamic_cast<HnsClient::TopologyEntry*>(topology->topologyEntry()->duplicateEntry(this));
        TopologySaveDialog popup(updatedEntry, this);
        HnvDialogHandler *propertiesHandler = new HnvDialogHandler(topology, &popup, this);
        propertiesHandler->setSourceDestroyedMessage(tr("\"%1\" was deleted. Operation aborted.").arg(topology->topologyEntry()->getShortName()));
        if (popup.exec() == QDialog::Accepted) {
            return updatedEntry;
        } else {
            delete updatedEntry;
        }
    }
    return 0;
}

void TopologyWidget::_setScreenshot(QString uuid)
{
    /*Excuting the snapshot for preview*/
    Hns::ByteArray screenshotData = _getArea(uuid)->createPreviewSnapshot();
    HyneviewSession::instance()->topologyAdministrator()->setScreenshot(uuid, screenshotData);
}

void TopologyWidget::_handleSaveResult(QString uuid, bool result, QString exceptionMessage)
{
    if (result) {
        HnvMainWindow::instance()->remoteProcessEnded();
    } else {
        QString message = tr("Could not save topology with UUID \"%1\"").arg(uuid);
        if (!exceptionMessage.isEmpty())
            message.append(QString("\n%1").arg(exceptionMessage));
        QMessageBox::warning(this, tr("Error"), message, QMessageBox::Ok, QMessageBox::Ok);
        emit topologySaveFailed();
    }
}

void TopologyWidget::_activateSimulationMode()
{
    ModeButton *simulationModeButton = SimulationModeWidget::instance()->getModeButton();
    if (!simulationModeButton->isChecked())
        simulationModeButton->click();
}

void TopologyWidget::_toggleGrid(bool toggle)
{
    _gridEnabled = toggle;
    foreach(TopologyViewArea *area, _areaHash)
        area->toggleGrid(_gridEnabled);
}

TopologyViewArea *TopologyWidget::_getArea(QString topologyUUID)
{
    TopologyViewArea *targetArea = 0;
    if (topologyUUID.isEmpty()) {
        QWidget *widget = _ui->stackedWidget->currentWidget();
        if (widget) {
            TopologyContainer *topologyContainer = qobject_cast<TopologyContainer *>(widget);
            targetArea = topologyContainer->topologyViewArea();
        }
    } else if (_areaHash.contains(topologyUUID)) {
        targetArea = _areaHash.value(topologyUUID);
    }
    return targetArea;
}

void TopologyWidget::_initViewArea(HnsClient::Topology *topology)
{
    TopologyViewArea *newArea = new TopologyViewArea(topology, this);
    QString topologyUUID = topology->getUuid();
    _areaHash.insert(topologyUUID, newArea);
    newArea->toggleGrid(_gridEnabled);

    TopologyContainer *topologyContainer = new TopologyContainer(newArea, this);
    _ui->stackedWidget->addWidget(topologyContainer);
    _containerHash.insert(newArea, topologyContainer);
    emit topologyOpened(topology);
    setCurrentTopology(topologyUUID);
    if (_ui->action_Draw_wire->isChecked())
        newArea->setInsertMode(TopologyView::InsertWireMode);
    else if (_ui->action_Draw_wire_1_angle->isChecked())
        newArea->setInsertMode(TopologyView::InsertWire1AngleMode);
    else if (_ui->action_Draw_wire_2angles->isChecked())
        newArea->setInsertMode(TopologyView::InsertWire2AnglesMode);
    else
        newArea->setInsertMode(TopologyView::SelectMode);
    newArea->setLabelsDisplayed(_ui->action_Toggle_Entity_Label->isChecked());
    newArea->setStatusIconVisibility(_ui->action_Toggle_Status_Icons->isChecked());
    newArea->setWireColor(_wireColor);
    _ui->action_Close_topology->setEnabled(true);
    _ui->action_Close_all_topologies->setEnabled(true);
}

void TopologyWidget::_refreshViewArea()
{
    QHash<qlonglong, QString> objectList;
    bool enableControls = false;
    TopologyViewArea *targetArea = _getArea();
    QList<HnsClient::GenericEntity*> newSelectionList;
    if (targetArea) {
        emit zoomValueChanged(targetArea->topologyView()->currentZoomValue());
        enableControls = true;
        objectList.insert(0, tr("<Empty selection>"));
        QHash<qlonglong, HnsClient::GenericEntity *> entities = targetArea->getEntities();
        foreach(HnsClient::GenericEntity *entity, entities)
            objectList.insert(entity->getId(), entity->getShortName());
        objectList.insert(-1, tr("<Custom selection>"));
        emit topologyActivated(targetArea->topology()->getUuid());
        newSelectionList = targetArea->getSelectedEntities();
    } else {
        emit topologyActivated(QString());
        emit zoomValueChanged(-1);
        emit clearLabelCoord();
    }
    emit selectionChanged(newSelectionList);
    _defineButton->setEnabled(false);
    _ui->action_Save_topology->setEnabled(enableControls);
    _ui->action_Save_topology_as->setEnabled(enableControls);
    _ui->action_Build_Topology_From->setEnabled(enableControls && targetArea->topologyEntry()->isModel());
    _ui->action_Build_Model_From->setEnabled(enableControls && !targetArea->topologyEntry()->isModel());
    _ui->action_Topology_properties->setEnabled(enableControls);
    _ui->action_Print_Topology->setEnabled(enableControls);
    _ui->action_Print_Selection->setEnabled(enableControls);
    _ui->action_Print_View->setEnabled(enableControls);
    _ui->action_Snapshot_Topology->setEnabled(enableControls);
    _ui->action_Snapshot_Selection->setEnabled(enableControls);
    _ui->action_Snapshot_View->setEnabled(enableControls);
    _ui->action_Resize_Scene->setEnabled(enableControls);
    _ui->action_Display_grid->setEnabled(enableControls);
    _ui->action_Select_and_move->setEnabled(enableControls);
    _ui->action_Draw_wire->setEnabled(enableControls);
    _ui->action_Draw_wire_1_angle->setEnabled(enableControls);
    _ui->action_Draw_wire_2angles->setEnabled(enableControls);
    _ui->action_Current_wire_color->setEnabled(enableControls);
    _ui->action_Load_Balancer->setEnabled(enableControls);
    _ui->topologyToolBar->closeAction()->setEnabled(enableControls);
    _ui->topologyToolBar->resetObjectList(objectList);
}

bool TopologyWidget::_populateDefineMenu()
{
    _disconnectDefineAction();
    TopologyViewArea *targetArea = _getArea();
    if (!targetArea) {
        _deleteDefineSubMenu();
        return false;
    }
    TopologyViewArea::selectionStatus status = targetArea->getSelectionStatus();
    bool canDefine = false;

    QStringList validNodeNames;
    QMapIterator<QString, QPair<bool, bool> > iterator(status.nodeMap);
    while (iterator.hasNext()) {
        iterator.next();
        if (iterator.value().first) {
            // Node can define at least one entity
            canDefine = true;
            if (iterator.value().second)
                validNodeNames.append(iterator.key());
        }
    }

    if (canDefine) {
        if (status.nodeMap.count() == 1) {
            // Delete sub menu if needed
            _deleteDefineSubMenu();
            QAction *action = _defineButton->defaultAction();
            QString nodeName;
            nodeName = status.nodeMap.keys().at(0);
            if (qobject_cast<QAction *>(_defineMapper->mapping(nodeName)) != action) {
                connect(action, SIGNAL(triggered()), _defineMapper, SLOT(map()), Qt::UniqueConnection);
                _defineMapper->setMapping(action, nodeName);
            }
        } else {
            // Create sub menu if needed
            QMenu *defineMenu = _defineButton->menu();
            if (!defineMenu)
                defineMenu = new QMenu(_ui->action_Define_entity->text(), this);
            else
                defineMenu->clear();
            QAction *defineAction = 0;
            QAction *defaultAction = 0;
            if (validNodeNames.count() > 0 ) {
                foreach(const QString &validNodeName, validNodeNames) {
                    QString actionText = validNodeName;
                    HnsClient::NodeInformation *nodeInfo = HyneviewSession::instance()->platformMonitor()->getNodeInformation(validNodeName);
                    if (nodeInfo)
                        actionText = nodeInfo->getShortName();
                    defineAction = new QAction(QIcon(Ressources::themedPath(Constants::ICON_SERVER)), actionText, defineMenu);
                    defineAction->setEnabled(true);
                    defineMenu->addAction(defineAction);
                    connect(defineAction, SIGNAL(triggered()), _defineMapper, SLOT(map()));
                    _defineMapper->setMapping(defineAction, validNodeName);
                }
            }
            QMapIterator<int, Hns::BalancingBehaviorInfo> itor(targetArea->topology()->getMapBehavior());
            while(itor.hasNext()){
                itor.next();
                defineAction = new QAction(QIcon(Ressources::themedPath(Constants::ICON_LOAD_BALANCING)) , QString::fromStdString(itor.value().name), defineMenu);
                if (defaultAction == 0)
                    defaultAction = defineAction;
                defineMenu->addAction(defineAction);
                connect(defineAction, SIGNAL(triggered()), _defineBehaviorMapper, SLOT(map()));
                _defineBehaviorMapper->setMapping(defineAction, itor.key());
            }
            if (defaultAction == 0)
                defaultAction = defineMenu->actions().first();
            connect(_defineButton->defaultAction(), SIGNAL(triggered(bool)), defaultAction, SLOT(trigger()));
            _defineButton->setPopupMode(QToolButton::MenuButtonPopup);
            _defineButton->setMenu(defineMenu);
        }
    } else
        _deleteDefineSubMenu(); // Delete sub menu if needed
    return canDefine;
}

void TopologyWidget::_deleteDefineSubMenu()
{
    // Delete sub menu if needed
    QMenu *menuToDelete = _defineButton->menu();
    _defineButton->setMenu(0);
    _defineButton->setPopupMode(QToolButton::MenuButtonPopup);
    if (menuToDelete)
        menuToDelete->deleteLater();
}

void TopologyWidget::_disconnectDefineAction()
{
    QAction *action = _defineButton->defaultAction();
    _defineMapper->removeMappings(action);
    action->disconnect();
}

void TopologyWidget::on_action_display_memory_dump_triggered()
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea){
        targetArea->displayMemoryDumpDialog();
    }
}

void TopologyWidget::_canRedirectUsbChangedSlot(int id)
{
    TopologyViewArea *targetArea = _getArea();
    if (targetArea && targetArea->isEntitySelected(id))
        _entitySelectionChangedSlot();
}

//void TopologyWidget::_refreshOverlayWidgetSlot()
//{
//    TopologyViewArea *area = getArea();
//    if(area)
//        area->refreshOverlayWidget();
//}

} // namespace Hnv

