/******************************************************************************
**
** Copyright (C) 2003-2017 DIATEAM. All rights reserved.
**
** This file is part of Hynesim.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include "usbmanagerdialog.h"
#include "remotemanager.h"
#include "usbmanager.h"
#include "ui_usbmanagerdialog.h"

#include <QTableWidgetItem>
#include <QAbstractButton>
#include <QDialogButtonBox>

namespace Hnv {
/*******************************************************************************
 * UsbManager
 ******************************************************************************/
UsbManagerDialog::UsbManagerDialog(HnsClient::DomainEntity *entity, QWidget *parent) :
    QDialog(parent),
    ui(new Ui::UsbManagerDialog),
    _entity(entity)
{
    ui->setupUi(this);
    ui->usbTableWidget->setColumnCount(COLUMN_COUNT);
    QStringList labels = QStringList()
            << tr("redirect")
            << tr("Manufacturer")
            << tr("Product")
            << tr("Descriptor")
            << tr("Bus")
            << tr("Address")
            << tr("Connected")
            << tr("Connected by another session");
    ui->usbTableWidget->setHorizontalHeaderLabels(labels);

    _usbManager = RemoteAccess::RemoteManager::instance()->usbManager(_entity->getId());
    connect(_usbManager, SIGNAL(devicesReset()), this, SLOT(refresh()));
    refresh();
}

UsbManagerDialog::~UsbManagerDialog()
{
    delete ui;
}

void UsbManagerDialog::refresh()
{
    _alreadyConnectedDevices.clear();
    ui->usbTableWidget->clear();
    ui->usbTableWidget->setColumnCount(COLUMN_COUNT);
    QStringList labels = QStringList()
            << tr("redirect")
            << tr("Manufacturer")
            << tr("Product")
            << tr("Descriptor")
            << tr("Bus")
            << tr("Address")
            << tr("Connected")
            << tr("Connected by another session");
    ui->usbTableWidget->setHorizontalHeaderLabels(labels);
    ui->usbTableWidget->setRowCount(_usbManager->deviceSize());
    for (int i = 0, size = _usbManager->deviceSize() ; i < size; ++i) {
        RemoteAccess::UsbDevice device = _usbManager->device(i);
        ui->usbTableWidget->setItem(i, COLUMN_MANUFACTURER, new QTableWidgetItem(device.manufacturer));
        ui->usbTableWidget->setItem(i, COLUMN_PRODUCT, new QTableWidgetItem(device.product));
        ui->usbTableWidget->setItem(i, COLUMN_DESCRIPTOR, new QTableWidgetItem(device.descriptor));
        ui->usbTableWidget->setItem(i, COLUMN_BUS, new QTableWidgetItem(device.bus));
        ui->usbTableWidget->setItem(i, COLUMN_ADDRESS, new QTableWidgetItem(device.address));
        ui->usbTableWidget->setItem(i, COLUMN_CONNECTED, new QTableWidgetItem(device.connected ? tr("Yes") : tr("No")));

        QTableWidgetItem *checkBoxitem = new QTableWidgetItem();
        int alreadyConnectedId = RemoteAccess::RemoteManager::instance()->usbDeviceConnected(device, _entity->getId());
        if (alreadyConnectedId < 0) {
            checkBoxitem->setFlags(Qt::ItemIsEnabled | Qt::ItemIsUserCheckable);
            ui->usbTableWidget->setItem(i, COLUMN_ANOTHER_SESSION, new QTableWidgetItem(tr("No")));
        } else {
            checkBoxitem->setFlags(Qt::ItemIsEnabled);
            ui->usbTableWidget->setItem(i, COLUMN_ANOTHER_SESSION, new QTableWidgetItem(tr("Yes, by %1").arg(RemoteAccess::RemoteManager::instance()->getShortNameById(alreadyConnectedId))));
        }
        if (device.connected) {
            checkBoxitem->setCheckState(Qt::Checked);
        } else {
            checkBoxitem->setCheckState(Qt::Unchecked);
        }
        ui->usbTableWidget->setItem(i, COLUMN_REDIRECT, checkBoxitem);
        if (device.connected) {
            for (int j = 0 ; j < COLUMN_COUNT ; ++j)
                ui->usbTableWidget->item(i, j)->setBackgroundColor(QColor(Qt::darkGreen));
        }
        if (alreadyConnectedId >= 0) {
            for (int j = 0 ; j < COLUMN_COUNT ; ++j)
                ui->usbTableWidget->item(i, j)->setBackgroundColor(QColor(Qt::darkGray));
        }
        if (device.connected)
            _alreadyConnectedDevices << i;
    }
    ui->usbTableWidget->resizeColumnsToContents();
    ui->usbTableWidget->horizontalHeader()->setStretchLastSection(true);
    ui->usbRedirectionCountLabel->setText(tr("%1 usb device(s) can be redirected").arg(_entity->usbRedirectionCount() - _alreadyConnectedDevices.size()));
}

void UsbManagerDialog::apply()
{
    for (int row = 0, rowCount = ui->usbTableWidget->rowCount() ; row < rowCount ; ++row) {
        QTableWidgetItem *checkBoxItem = ui->usbTableWidget->item(row, COLUMN_REDIRECT);
        Qt::CheckState state = checkBoxItem->checkState();
        if (state == Qt::Checked && !_alreadyConnectedDevices.contains(row)) {
            _usbManager->connectUsbDevice(_usbManager->device(row));
        } else if (state == Qt::Unchecked && _alreadyConnectedDevices.contains(row)) {
            _usbManager->disconnectUsbDevice(_usbManager->device(row));
        }
    }
    refresh();
}

void UsbManagerDialog::on_buttonBox_clicked(QAbstractButton *button)
{
    if (ui->buttonBox->buttonRole(button) == QDialogButtonBox::ApplyRole) {
        apply();
    } else if (ui->buttonBox->buttonRole(button) == QDialogButtonBox::RejectRole) {
        reject();
    }
}

} // namespace Hnv
