/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "usersmanagerwidget.h"
#include "ui_usersmanagerwidget.h"
#include "usersmodel.h"
#include "usereditdialog.h"
#include "userclonedialog.h"
#include "treesortfilterproxymodel.h"
#include "filterlineedit.h"
#include "ressources.h"

// hynesim hnsclient headers
#include "libhnsclient/usermanager.h"

// Qt headers
#include <QMessageBox>
#include <QDebug>
#include <QMenu>
#include <QToolButton>
#include <QDialog>

namespace Hnv {

/*******************************************************************************
* UsersManagerWidget
*******************************************************************************/
UsersManagerWidget::UsersManagerWidget(QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::UsersManagerWidget)
    , _accessor(0)
    , _usersModel(0)
{
    ui->setupUi(this);

    _filterButton = new QToolButton();
    _filterLineEdit = new FilterLineEdit();
    connect(_filterLineEdit, SIGNAL(textChanged(QString)), this, SLOT(_filterLineEdit_textChanged(QString)));
    _usersFilterProxyModel = new UsersFilterProxyModel(this);
    _usersFilterProxyModel->setDynamicSortFilter(true);
    _usersProxyModel = new TreeSortFilterProxyModel(this);
    QList<int> filterColumns;
    filterColumns << 1 << 2 << 3 << 8;
    _usersProxyModel->setFilterColumnsList(filterColumns);
    _usersProxyModel->setDynamicSortFilter(true);
    _usersProxyModel->setSortCaseSensitivity(Qt::CaseInsensitive);
    _usersProxyModel->setFilterCaseSensitivity(Qt::CaseInsensitive);
    _usersProxyModel->setSourceModel(_usersFilterProxyModel);
    ui->usersTableView->setModel(_usersProxyModel);
    ui->usersTableView->sortByColumn(1, Qt::AscendingOrder);
    ui->usersTableView->resizeColumnsToContents();
    connect(ui->usersTableView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)), this, SLOT(selectionChanged(QItemSelection,QItemSelection)));
    connect(_usersProxyModel, SIGNAL(rowsInserted(const QModelIndex &, int, int)), this, SLOT(_rowInserted()));
    connect(_usersProxyModel, SIGNAL(rowsRemoved(const QModelIndex &, int, int)), this, SLOT (_rowInserted()));
    _enableControls(false);
    ui->action_New_user->setIcon(QIcon(Ressources::themedPath(Constants::ICON_USER_ADD)));
    ui->action_Clone_user->setIcon(QIcon(Ressources::themedPath(Constants::ICON_USER_CLONE)));
    ui->action_Edit_user->setIcon(QIcon(Ressources::themedPath(Constants::ICON_USER_EDIT)));
    ui->action_Delete_user->setIcon(QIcon(Ressources::themedPath(Constants::ICON_DELETE)));
    connect(ui->usersTableView, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(_contextMenuRequested(QPoint)));
    _filterButton->setIcon(QIcon(Ressources::themedPath(Constants::ICON_FILTER)));

    QMenu *viewMenu = new QMenu(this);
    viewMenu->addAction(ui->action_Online);
    viewMenu->addAction(ui->action_Offline);
    viewMenu->addSeparator();
    viewMenu->addAction(ui->action_Enabled);
    viewMenu->addAction(ui->action_Disabled);
    viewMenu->addSeparator();
    viewMenu->addAction(ui->action_All);
    _filterButton->setMenu(viewMenu);

    ui->usersToolBar->setCloseable(false);
    ui->usersToolBar->addAction(ui->action_New_user);
    ui->action_New_user->setEnabled(true);
    ui->usersToolBar->addAction(ui->action_Clone_user);
    ui->action_Clone_user->setEnabled(false);
    ui->usersToolBar->addAction(ui->action_Edit_user);
    ui->action_Edit_user->setEnabled(false);
    ui->usersToolBar->addAction(ui->action_Delete_user);
    ui->action_Delete_user->setEnabled(false);
    ui->usersToolBar->addSeparator();
    ui->usersToolBar->addWidget(_filterButton);
    QWidget* spacer = new QWidget();
    spacer->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    ui->usersToolBar->addWidget(spacer);
    ui->usersToolBar->addWidget(_filterLineEdit);
    _filterLineEdit->setMinimumSize(300, 0);
    _filterLineEdit->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
    _filterLineEdit->setPlaceholderText(tr("Filter these users..."));
    _enableControls(false);
}

UsersManagerWidget::~UsersManagerWidget()
{
    delete _usersFilterProxyModel;
    delete _usersProxyModel;
    delete ui;
}

void UsersManagerWidget::setAccessor(HnsClient::UserManager *accessor)
{
    foreach (QObject *o, children()) {
        QDialog *dialog = qobject_cast<QDialog *>(o);
        if (dialog)
            dialog->reject();
    }
    _accessor = accessor;
    bool enable = false;
    if (_accessor) {
        connect(_accessor, SIGNAL(userErrorOccured(QString)), this, SLOT(_displayErrorMessage(QString)));
        enable = true;
    }
    _enableControls(enable);
}

void UsersManagerWidget::setUsersModel(UsersModel *usersModel)
{
    _usersModel = usersModel;
    _usersFilterProxyModel->setSourceModel(_usersModel);
    ui->usersTableView->hideColumn(0);
    ui->usersTableView->hideColumn(4);
    ui->usersTableView->hideColumn(5);
    ui->usersTableView->hideColumn(6);
    ui->usersTableView->hideColumn(7);
    ui->usersTableView->resizeColumnsToContents();
}
/*******************************************************************************
* UsersManagerWidget - Protected slots
*/
void UsersManagerWidget::selectionChanged(QItemSelection, QItemSelection)
{
    bool oneSelection = false;
    bool oneAndMoreSelection = false;
    bool allowDisconnect = false;

    QModelIndexList selectedRows = ui->usersTableView->selectionModel()->selectedRows();
    foreach(QModelIndex row, selectedRows) {
        QVariant isConnected = row.data(Qt::UserRole + 3);
        if (isConnected.isValid() && isConnected.toBool()) {
            allowDisconnect = true;
            break;
        }
    }

    int liSelectionCount = selectedRows.count();
    if (liSelectionCount == 1) {
        oneSelection = true;
        oneAndMoreSelection = true;
    } else if (liSelectionCount > 1) {
        oneAndMoreSelection = true;
    }
    ui->action_Clone_user->setEnabled(oneSelection);
    ui->action_Edit_user->setEnabled(oneSelection);
    ui->action_Delete_user->setEnabled(oneAndMoreSelection);
    ui->action_Disconnect_user->setEnabled(allowDisconnect);
}

void UsersManagerWidget::_contextMenuRequested(QPoint pos)
{
    QModelIndex targetIdx = ui->usersTableView->indexAt(pos);
    if (targetIdx.isValid()) {
        QMenu contextMenu;
        contextMenu.addAction(ui->action_Clone_user);
        contextMenu.addAction(ui->action_Edit_user);
        contextMenu.addSeparator();
        contextMenu.addAction(ui->action_Disconnect_user);
        contextMenu.addSeparator();
        contextMenu.addAction(ui->action_Delete_user);

        if (contextMenu.actions().count())
            contextMenu.exec(ui->usersTableView->viewport()->mapToGlobal(pos));
    }
}
/*******************************************************************************
* UsersManagerWidget - Private slots
*/
void UsersManagerWidget::on_action_Offline_toggled(bool checked)
{
    _usersFilterProxyModel->acceptOffline(checked);
}

void UsersManagerWidget::on_action_Disabled_toggled(bool checked)
{
    _usersFilterProxyModel->acceptDisabled(checked);
}

void UsersManagerWidget::on_action_Enabled_toggled(bool checked)
{
    _usersFilterProxyModel->acceptEnabled(checked);
}

void UsersManagerWidget::on_action_Online_toggled(bool checked)
{
    _usersFilterProxyModel->acceptOnline(checked);
}

void UsersManagerWidget::on_action_All_triggered()
{
    ui->action_Online->setChecked(true);
    ui->action_Offline->setChecked(true);
    ui->action_Enabled->setChecked(true);
    ui->action_Disabled->setChecked(true);
}

void UsersManagerWidget::on_action_Delete_user_triggered()
{
    QModelIndexList selectedRowsIndexes = ui->usersTableView->selectionModel()->selectedRows();
    QList<qlonglong> usersIDs;
    QList<qlonglong> connectedUsersIDs;
    foreach(QModelIndex index, selectedRowsIndexes) {
        qlonglong userID = index.data().toLongLong();
        usersIDs.append(userID);
        if (index.data(Qt::UserRole + 3).toBool())
            connectedUsersIDs.append(userID);
    }
    QMessageBox msgBox(this);
    msgBox.setText(tr("Do you really want to delete theses users ?"));
    QString infoText = _accessor->getUsernamesFromIdList(usersIDs).join(", ");
    if (connectedUsersIDs.count() > 0) {
        infoText = QString("%1\n\n%2").arg(infoText).arg(tr("WARNING: Some users are currently connected and will be forcefully disconnected"));
        msgBox.setDetailedText(tr("Connected users: %1").arg(_accessor->getUsernamesFromIdList(connectedUsersIDs).join(", ")));
    }
    msgBox.setInformativeText(infoText);
    msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
    msgBox.setDefaultButton(QMessageBox::No);
    if (msgBox.exec() == QMessageBox::Yes) {
        foreach(Ice::Long userID, usersIDs) {
            _accessor->deleteUser(userID, this, SLOT(_successDeleteUser(qlonglong, QString, bool)));
        }
    }
}

void UsersManagerWidget::on_action_Edit_user_triggered()
{
    _editUser(ui->usersTableView->currentIndex());
}

void UsersManagerWidget::on_action_Clone_user_triggered()
{
    int cloneCount;
    QString password;
    int targetRow = _usersProxyModel->mapToSource(ui->usersTableView->currentIndex()).row();
    Hns::UserPtr sourceUserPtr = Hns::UserPtr::dynamicCast(_accessor->getUserByIndex(targetRow)->ice_clone());
    if (sourceUserPtr) {
        UserCloneDialog popup(QString::fromStdString(sourceUserPtr->name), cloneCount, password, this);
        if (popup.exec() == QDialog::Accepted) {
            sourceUserPtr->hSetId = -1;
            _accessor->cloneUser(sourceUserPtr, password, cloneCount, this, SLOT(_successCloneUser(qlonglong,QString,bool)));
        }
    }
}

void UsersManagerWidget::on_action_New_user_triggered()
{
    Hns::UserPtr newUser = new Hns::User();
    newUser->id = -1;
    newUser->createdOn = 0;
    newUser->modifiedOn = 0;
    QString password;
    UserEditDialog popup(newUser, password, _accessor->getGroupNamesHash(), this);
    //TODO, Temporary, we need to manage hsetids
    newUser->hSetId = -1;
    if (popup.exec() == QDialog::Accepted) {
        _accessor->addUser(newUser, password, this, SLOT(_successAddUser(qlonglong, QString, bool)));
    }
}

void UsersManagerWidget::on_action_Disconnect_user_triggered()
{
    QModelIndexList selectedRowsIndexes = ui->usersTableView->selectionModel()->selectedRows();
    QList<qlonglong> usersIDs;
    foreach(QModelIndex index, selectedRowsIndexes) {
        QVariant isConnected = index.data(Qt::UserRole+3);
        if (isConnected.isValid() && isConnected.toBool())
            usersIDs.append(index.data().toLongLong());
    }
    if (usersIDs.count() > 0) {
        QMessageBox msgBox;
        msgBox.setText(tr("Do you really want to disconnect theses users ?"));
        msgBox.setInformativeText(_accessor->getUsernamesFromIdList(usersIDs).join(", "));
        msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
        msgBox.setDefaultButton(QMessageBox::No);
        if (msgBox.exec() == QMessageBox::Yes) {
            foreach(qlonglong userId, usersIDs) {
                _accessor->disconnectUser(userId);
            }
        }
    }
}

void UsersManagerWidget::_enableControls(bool enable)
{
    ui->action_New_user->setEnabled(enable);
    if (!enable) {

        ui->action_Edit_user->setEnabled(enable);
        ui->action_Clone_user->setEnabled(enable);
        ui->action_Delete_user->setEnabled(enable);
        ui->action_Disconnect_user->setEnabled(enable);
    }
    _filterLineEdit->setEnabled(enable);
    ui->usersTableView->setEnabled(enable);
    _filterButton->setEnabled(enable);
}

void UsersManagerWidget::_displayErrorMessage(QString message)
{
    QMessageBox::warning(this, tr("Error"), message);
}

void UsersManagerWidget::_rowInserted()
{
    ui->usersTableView->resizeColumnToContents(1);
}

void UsersManagerWidget::_filterLineEdit_textChanged(QString currentText)
{
    _usersProxyModel->setFilterFixedString(currentText);
}

void UsersManagerWidget::on_usersTableView_doubleClicked(QModelIndex index)
{
    _editUser(index);
}

void UsersManagerWidget::_successDeleteUser(qlonglong id, QString userName, bool ok)
{
    Q_UNUSED(id)
    if (!ok)
        QMessageBox::critical(this, "Unable to delete user", QString("Unable to delete " + userName));
}

void UsersManagerWidget::_successAddUser(qlonglong id, QString name, bool ok)
{
    Q_UNUSED(id)
    if (!ok)
        QMessageBox::critical(this, "Unable to add user", QString("Unable to add " + name));
}

void UsersManagerWidget::_successCloneUser(qlonglong id, QString name , bool ok)
{
    Q_UNUSED(id)
    if (!ok)
        QMessageBox::critical(this, "Unable to clone user", QString("Unable to clone " + name));
}

void UsersManagerWidget::_successUpdateUser(qlonglong id, QString name , bool ok)
{
    Q_UNUSED(id)
    if (!ok)
        QMessageBox::critical(this, "Unable to update user", QString("Unable to update " + name));
}
/*******************************************************************************
* UsersManagerWidget - Private
*/
void UsersManagerWidget::_editUser(QModelIndex triggerIndex)
{
    QModelIndex rowIdx = ui->usersTableView->model()->index(triggerIndex.row(), 0);
    Hns::UserPtr selectedUserPtr = Hns::UserPtr::dynamicCast(_accessor->getUserById(rowIdx.data().toLongLong())->ice_clone());
    if (!selectedUserPtr) {
        qCritical() << tr("Unable to get user");
        return;
    }
    QString password;
    UserEditDialog popup(selectedUserPtr, password, _accessor->getGroupNamesHash(), this);
    if (popup.exec() == QDialog::Accepted) {
        _accessor->updateUser(selectedUserPtr, password, this, SLOT(_successUpdateUser(qlonglong,QString,bool)));
    }
}

} // namespace Hnv
