/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "zoomingview.h"

// Qt headers
#include <QWheelEvent>

// C++ std headers
#include <math.h>

namespace Hnv {

/*******************************************************************************
* ZoomingView
*******************************************************************************/
ZoomingView::ZoomingView(QWidget * parent)
    : QGraphicsView(parent)
    , _currentCenterPoint(QPoint(0,0))
{
    _initView();
}

ZoomingView::ZoomingView(QGraphicsScene *scene, QWidget *parent)
    : QGraphicsView(scene, parent)
    , _currentCenterPoint(QPoint(0,0))
{
    _initView();
}

void ZoomingView::toggleGrid(bool toggle)
{
    _displayGrid = toggle;
    resetCachedContent();
    update();
}

void ZoomingView::setCenter(const QPointF &centerPoint)
{
    if (scene() == 0)
        return;
    QRectF visibleArea = mapToScene(rect()).boundingRect();
    //Get the scene area
    QRectF sceneBounds = sceneRect();
    double boundX = visibleArea.width() / 2.0;
    double boundY = visibleArea.height() / 2.0;
    double boundWidth = sceneBounds.width() - 2.0 * boundX;
    double boundHeight = sceneBounds.height() - 2.0 * boundY;
    //The max boundary that the centerPoint can be to
    QRectF centerBounds(sceneBounds.x() + boundX, sceneBounds.y() + boundY, boundWidth, boundHeight);

    if(centerBounds.contains(centerPoint)) {
        //We are within the bounds
        _currentCenterPoint = centerPoint;
    } else {
        //We need to clamp or use the center of the screen
        if(visibleArea.contains(sceneBounds)) {
            //Use the center of scene ie. we can see the whole scene
            _currentCenterPoint = sceneBounds.center();
        } else {
            _currentCenterPoint = centerPoint;
            //We need to clamp the center. The centerPoint is too large
            if(centerPoint.x() > centerBounds.x() + centerBounds.width()) {
                _currentCenterPoint.setX(centerBounds.x() + centerBounds.width());
            } else if(centerPoint.x() < centerBounds.x()) {
                _currentCenterPoint.setX(centerBounds.x());
            }

            if(centerPoint.y() > centerBounds.y() + centerBounds.height()) {
                _currentCenterPoint.setY(centerBounds.y() + centerBounds.height());
            } else if(centerPoint.y() < centerBounds.y()) {
                _currentCenterPoint.setY(centerBounds.y());
            }
        }
    }
    //Update the scrollbars
    centerOn(_currentCenterPoint);
}

void ZoomingView::moveCenter(const QPointF &delta)
{
    setCenter(_currentCenterPoint + delta);
}

void ZoomingView::setScene(QGraphicsScene *newScene)
{
    QGraphicsView::setScene(newScene);
}

void ZoomingView::showOutline(bool show)
{
    if (scene() == 0)
        return;
    if (show != _showOutline) {
        _showOutline = show;
        update();
    }
}

void ZoomingView::setZoomInLimit(int limit)
{
    _zoomInLimit = limit;
}

void ZoomingView::setZoomOutLimit(int limit)
{
    _zoomOutLimit = limit;
}

double ZoomingView::currentZoomFactor()
{
    return _zoomFactor;
}

int ZoomingView::currentZoomValue()
{
    return exp2(_zoomFactor) * 100;
}

//void ZoomingView::toggleOpenGL(const bool pbEnabled)
//{
//  setupViewport(pbEnabled ? new QGLWidget(QGLFormat(QGL::SampleBuffers)) : new QWidget);
//}

/*******************************************************************************
* ZoomingView - Public slots
*/
void ZoomingView::zoomOut(QPointF centerPos)
{
    if (_zoomFactor > _zoomOutLimit)
        _zoom(_zoomFactor - 1, centerPos);
}

void ZoomingView::zoomIn(QPointF centerPos)
{
    if (_zoomFactor < _zoomInLimit)
        _zoom(_zoomFactor + 1, centerPos);
}

void ZoomingView::setZoomValue(double zoomValue)
{
    double newZoomFactor = log2(zoomValue / 100);
    if (newZoomFactor != _zoomFactor && newZoomFactor >= _zoomOutLimit && newZoomFactor <= _zoomInLimit)
        _zoom(newZoomFactor);
}

/*******************************************************************************
* ZoomingView - Protected
*/
void ZoomingView::_initView()
{
    setResizeAnchor( QGraphicsView::AnchorUnderMouse );
    setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform);
    setMouseTracking(true);
    _showOutline = false;
    _zoomFactor = 0;
    _zoomOutLimit = -2;
    _zoomInLimit = 2;
    _displayGrid = false;
}

void ZoomingView::_zoom(double zoomFactor, QPointF centerPos)
{
    double steps = fabs(_zoomFactor - zoomFactor);
    if (steps == 0)
        return;

    double scaleFactor = 0;
    if (_zoomFactor > zoomFactor)
        scaleFactor = 0.5;
    else if (_zoomFactor < zoomFactor)
        scaleFactor = 2;
    double scaleBy = pow(scaleFactor, steps);

    QPointF currentCenter = mapToScene(rect().center());
    QPointF sceneDelta = currentCenter - centerPos;
    QPointF scaledDelta = sceneDelta / scaleBy;

    scale(scaleBy, scaleBy);

    if (transformationAnchor() == QGraphicsView::AnchorUnderMouse && !centerPos.isNull())
        setCenter(centerPos + scaledDelta);
    _zoomFactor = zoomFactor;
    emit zoomValueChanged(currentZoomValue());
}

void ZoomingView::wheelEvent (QWheelEvent *event) {
    int delta = event->delta() ;
    if (delta> 0)
        zoomIn(mapToScene(event->pos()));
    else
        zoomOut(mapToScene(event->pos()));
    event->accept();
}

/**
  * Handles when the mouse button is pressed
  */
void ZoomingView::mousePressEvent(QMouseEvent* event)
{
    if (scene() == 0)
        return;
    if (event->button() == Qt::RightButton) {
        if (!itemAt(event->pos())) {
            _panningOrigin = event->pos();
            setCursor(Qt::ClosedHandCursor);
            event->accept();
            return;
        }
    }
    QGraphicsView::mousePressEvent(event);
}

/**
  * Handles when the mouse button is released
  */
void ZoomingView::mouseReleaseEvent(QMouseEvent* event)
{
    if (scene() == 0)
        return;
    setCursor(Qt::ArrowCursor);
    _panningOrigin = QPoint();
    QGraphicsView::mouseReleaseEvent(event);
}

/**
*Handles the mouse move event
*/
void ZoomingView::mouseMoveEvent(QMouseEvent* event)
{
    if (scene() == 0)
        return;
    QGraphicsView::mouseMoveEvent(event);
    _updateCoordonnate(event);

}

void ZoomingView::drawBackground(QPainter *painter, const QRectF &rect)
{
    Q_UNUSED(rect);
    //QGraphicsView::drawBackground(painter, rect);
    if (!scene())
        return;
    if (_showOutline) {
        QRect outlineRect = sceneRect().toRect();
        painter->drawRect(outlineRect);
    }

    if (_displayGrid) {
        int step = 100;
        QRect intSceneRect = sceneRect().toRect();
//        QRect intVisualRect = rect.toRect();

//        int sceneX1 = intSceneRect.x();
//        int visualX1 = intVisualRect.x();
//        int left = sceneX1;
//        if (sceneX1 <= visualX1)
//            left += (((visualX1 - sceneX1) / step) + 1) * step;

//        int sceneX2 = sceneX1 + intSceneRect.width();
//        int visualX2 = visualX1 + intVisualRect.width();
//        int right = sceneX2;
//        if (sceneX2 >= visualX2)
//            right -= (((sceneX2 - visualX2) / step) + 1) * step;

//        int sceneY1 = intSceneRect.y();
//        int visualY1 = intVisualRect.y();
//        int top = sceneY1;
//        if (sceneY1 < visualY1)
//            top += (((visualY1 - sceneY1) / step) + 1) *step;

//        int sceneY2 = sceneY1 + intSceneRect.height();
//        int visualY2 = visualY1 + intVisualRect.height();
//        int bottom = sceneY2;
//        if (sceneY2 >= visualY2)
//            bottom -= (((sceneY2 - visualY2) / step) +1) *step;
        int left = intSceneRect.topLeft().x();
        int right = intSceneRect.topRight().x();
        int top = intSceneRect.topLeft().y();
        int bottom = intSceneRect.bottomLeft().y();

        if (left < right && top < bottom) {
            for(int x = left; x <= right; x = x + step)
                for(int y = top; y <= bottom; y = y + step) {
                    painter->drawPoint(x, y);
                }
        }
    }
}

void ZoomingView::paintEvent(QPaintEvent *event)
{
    emit viewPainted();
    QGraphicsView::paintEvent(event);
}

void ZoomingView::_updateCoordonnate(QMouseEvent *event)
{
    QPointF mappedPosition = mapToScene(event->pos());
    bool inScene = sceneRect().contains(mappedPosition);
    emit cursorPositionChanged(mappedPosition.x(), mappedPosition.y(), inScene);

    if (!_panningOrigin.isNull()) {
        QPointF delta = mapToScene(_panningOrigin) - mappedPosition;
        _panningOrigin = event->pos();
        moveCenter(delta);
    }
}

} // namespace Hnv
