/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "zoompreview.h"
#include "simulationmodewidget.h"
#include "topologywidget.h"
#include "topologyview.h"
#include "zoomingview.h"

// Qt headers
#include <QWidget>
#include <QPainter>
#include <QMouseEvent>

namespace Hnv {

/*******************************************************************************
* ZoomPreviewView
*******************************************************************************/
ZoomPreviewView::ZoomPreviewView(QWidget *parent)
    : QGraphicsView(parent)
    , _linkedView(0)
    , _areaSize(125)
    , _expandToMax(false)
{
    setInteractive(false);
    setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    setViewportUpdateMode(FullViewportUpdate);
    setFrameShape(QFrame::NoFrame);
}

void ZoomPreviewView::setLinkedView(ZoomingView *linkedView)
{
    if (linkedView == _linkedView)
        return;

    if (_linkedView != 0) {
        _linkedView->disconnect(this);
        if (_linkedView->scene())
            _linkedView->scene()->disconnect(this);
    }

    QGraphicsScene * newScene = 0;
    if (linkedView != 0)
        newScene = linkedView->scene();

    _linkedView = linkedView;
    setScene(newScene);

    if (_linkedView)
        connect(linkedView, SIGNAL(viewPainted()), this, SLOT(_linkedViewRefreshedSlot()));

    if (newScene) {
        connect(newScene, SIGNAL(sceneRectChanged(QRectF)), this, SLOT(_sceneRectChangedSlot(QRectF)));
        _updateSize(newScene->sceneRect());
    }
}

void ZoomPreviewView::setAreaSize(int value, bool expandToMax)
{
    _areaSize = value;
    _expandToMax = expandToMax;
    if (_linkedView)
        _updateSize(_linkedView->sceneRect());
}

/*******************************************************************************
* ZoomPreviewView - Protected slots
*/
void ZoomPreviewView::_sceneRectChangedSlot(QRectF scRect)
{
    _updateSize(scRect);
}

void ZoomPreviewView::_linkedViewRefreshedSlot()
{
    _outlineLinkedArea();
}

/*******************************************************************************
* ZoomPreviewView - Protected
*/
void ZoomPreviewView::drawForeground(QPainter *painter, const QRectF &rect)
{
    QGraphicsView::drawForeground(painter, rect);
    // TODO: Should use a setting or compute color code from background color
    QColor outlineColor(252, 201, 1); // fcc901
    painter->setPen(outlineColor);
    painter->drawRect(_linkedRect);
    outlineColor.setAlpha(80);
    QBrush outlineBrush(outlineColor);
    painter->fillRect(_linkedRect, outlineBrush);
}

void ZoomPreviewView::_outlineLinkedArea()
{
    if (_linkedView) {
        QRectF targetRect = _linkedView->mapToScene(_linkedView->viewport()->geometry()).boundingRect();
        if (targetRect.x() < 0)
            targetRect.setX(0);
        if (targetRect.y() < 0)
            targetRect.setY(0);
        if ((targetRect.x() + targetRect.width()) > sceneRect().width())
            targetRect.setWidth(sceneRect().width() - targetRect.x());
        if (targetRect.y() + targetRect.height() > sceneRect().height())
            targetRect.setHeight(sceneRect().height() - targetRect.y());
        if (targetRect != _linkedRect) {
            _linkedRect = targetRect;
            viewport()->update();
        }
    }
}

void ZoomPreviewView::_updateSize(QRectF sceneRect)
{
    qreal hFactor =  _areaSize / sceneRect.width();
    qreal vFactor = _areaSize / sceneRect.height();
    resetMatrix();

    qreal scaleFactor = 0;

    if (_expandToMax)
        scaleFactor = qMax(hFactor, vFactor);
    else
        scaleFactor = qMin(hFactor, vFactor);
    scale(scaleFactor, scaleFactor);

    resize(sizeHint());
    _outlineLinkedArea();
}

void ZoomPreviewView::mousePressEvent(QMouseEvent *event)
{
    if (scene() == 0)
        return;
    if (_linkedView && event->button() == Qt::LeftButton)
        _linkedView->setCenter(mapToScene(event->pos()));
    QGraphicsView::mousePressEvent(event);
}

void ZoomPreviewView::mouseMoveEvent(QMouseEvent *event)
{
    if (scene() == 0)
        return;
    if (_linkedView && event->buttons() == Qt::LeftButton)
        _linkedView->setCenter(mapToScene(event->pos()));
    QGraphicsView::mouseMoveEvent(event);
}

/*******************************************************************************
* ZoomPreviewSideBarWidget
*******************************************************************************/
ZoomPreviewSideBarWidget::ZoomPreviewSideBarWidget(QWidget *parent)
    : SideBarSimpleWidget(Qt::Vertical, parent)
    , _previewWidget(new ZoomPreviewView(this))
{
    QList<QWidget *> children;
    children.append(_previewWidget);
    _initChildren(children);
    _previewWidget->setAreaSize(width(), true);
    QWidget * widget = sideBarParent();
    if (widget)
        _simulationModeWidget = dynamic_cast<SimulationModeWidget*>(widget);
    if (_simulationModeWidget) {
        _topologyWidget = _simulationModeWidget->topologyWidget();
        if (_topologyWidget) {
            connect(_topologyWidget, SIGNAL(topologyActivated(QVariant)), this, SLOT(_topologyActivatedSlot(QVariant)));
            setLinkedView(_topologyWidget->getCurrentView());
        }
    }
}

ZoomPreviewSideBarWidget::~ZoomPreviewSideBarWidget()
{
    delete _previewWidget;
}

QString ZoomPreviewSideBarWidget::title()
{
    return tr("Topology preview");
}

QString ZoomPreviewSideBarWidget::sideBarId()
{
    return "Topology preview";
}

void ZoomPreviewSideBarWidget::setLinkedView(ZoomingView *linkedView)
{
    _previewWidget->setLinkedView(linkedView);
}

/*******************************************************************************
* ZoomPreviewSideBarWidget - Protected
*/
void ZoomPreviewSideBarWidget::resizeEvent(QResizeEvent *event)
{
    Q_UNUSED(event);
    _previewWidget->setAreaSize(width(), true);
}

/*******************************************************************************
* ZoomPreviewSideBarWidget - Private slots
*/
void ZoomPreviewSideBarWidget::_topologyActivatedSlot(QVariant topologyUUID)
{
    Q_UNUSED(topologyUUID)
    setLinkedView(_topologyWidget->getCurrentView());
}

} // namespace Hnv
