/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim hnsclient headers
#include "libhnsclient/aclmanager.h"
#include "libhnsclient/private/aclmanager_p.h"
#include "libhnsclient/session.h"
#include "libhnsclient/usermanager.h"
#include "libhnsclient/hsetmanager.h"

// hynesim common headers
#include "common/rightslist.h"

// Qt headers
#include <QStringList>

namespace HnsClient {

/*******************************************************************************
* ACLManagerPrivate
*******************************************************************************/
ACLManagerPrivate::ACLManagerPrivate()
{
    aclManagerQPrx = 0;
    remoteCallCount = 0;
    helper = 0;
    entryTargetToPrefix.insert(Hns::Rights::ACL_TARGET_ENTITYENTRY, Hns::Rights::ENTITY_ENTRY_PREFIX);
    entryTargetToPrefix.insert(Hns::Rights::ACL_TARGET_TOPOLOGYENTRY, Hns::Rights::TOPOLOGY_ENTRY_PREFIX);
    entryTargetToPrefix.insert(Hns::Rights::ACL_TARGET_FOLDER, Hns::Rights::FOLDER_PREFIX);
}

ACLManagerPrivate::~ACLManagerPrivate()
{
    aclManagerQPrx = 0;
}


/*******************************************************************************
* GetACLsForTagetCookie
*******************************************************************************/
GetACLsForTagetCookie::GetACLsForTagetCookie(QObject *receiver, const char *slot)
    : QObject()
{
    connect(this, SIGNAL(completed(Diamesh::ACLList)), receiver, slot);
}

GetACLsForTagetCookie::~GetACLsForTagetCookie()
{
}

void GetACLsForTagetCookie::emitCompleted(const Diamesh::ACLList &aclList)
{
    emit completed(aclList);
    deleteLater();
}


/*******************************************************************************
* ACLManager
*******************************************************************************/
ACLManager::~ACLManager()
{
    delete d_ptr;
}
/*! \fn helper() const
 *\brief TODO
 */
Diamesh::AccessRightsHelper *ACLManager::helper() const
{
    Q_D(const ACLManager);
    return d->helper;
}

/*! \fn getSubjectType(const QString &subject) const
 *\brief TODO
 */
Diamesh::AccessRightsHelper::SubjectType ACLManager::getSubjectType(const QString &subject) const
{
    Q_D(const ACLManager);
    QPair<Diamesh::AccessRightsHelper::SubjectType, qlonglong> details = d->helper->subjectDetails(subject);
    return details.first;
}

/*! \fn readableSubject(const QString &subject) const
 *\brief TODO
 */
QString ACLManager::readableSubject(const QString &subject) const
{
    Q_D(const ACLManager);
    QPair<Diamesh::AccessRightsHelper::SubjectType, qlonglong> details = d->helper->subjectDetails(subject);
    switch(details.first) {
    case Diamesh::AccessRightsHelper::USER_SUBJECT: {
        Hns::UserPtr user = _userFromSubject(details.second);
        if (user)
            return tr("User/%1").arg(QString::fromStdString(user->name));
        break;
    }
    case Diamesh::AccessRightsHelper::GROUP_SUBJECT: {
        Diamesh::GroupPtr group = _groupFromSubject(details.second);
        if (group)
            return tr("Group/%1").arg(QString::fromStdString(group->name));
        break;
    }
    case Diamesh::AccessRightsHelper::HSET_SUBJECT: {
        Diamesh::HierarchicalSet hSet = _hSetFromSubject(details.second);
        if (hSet.id != -1)
            return tr("HSet/%1").arg(QString::fromStdString(hSet.name));
        break;
    }
    case Diamesh::AccessRightsHelper::OTHER_SUBJECT: {
        return tr("Everyone");
        break;
    }
    case Diamesh::AccessRightsHelper::INVALID_SUBJECT: {
        break;
    }
    }
    return tr("ERROR:Invalid");
}

/*! \fn readableRights(const Hns::CatalogItemPtr &catalogItem, const Diamesh::LongList &rights) const
 *\brief TODO
 *\param &entry
 *\param &rights
 */
QString ACLManager::readableRights(const Hns::CatalogItemPtr &catalogItem, const Diamesh::LongList &rights) const
{
    Q_D(const ACLManager);
    QString result = tr("Custom access");
    QString prefix = _prefixFromCatalogItem(catalogItem);
    QStringList rightsNames;
    rightsNames.append(prefix);
    if (d->helper->rightsMatchesRightsNames(rights, rightsNames))
        result = tr("Manager");
    else {
        rightsNames.clear();
        if (Hns::EntryPtr::dynamicCast(catalogItem)) {
            rightsNames.append(QString("%1%2").arg(prefix).arg(Hns::Rights::ENTRY_SUFFIX_READ));
            rightsNames.append(QString("%1%2").arg(prefix).arg(Hns::Rights::ENTRY_SUFFIX_UPDATE_ENTRY));
            rightsNames.append(QString("%1%2").arg(prefix).arg(Hns::Rights::ENTRY_SUFFIX_USE));
        } else if (Hns::FolderPtr::dynamicCast(catalogItem)) {
            rightsNames.append(QString("%1%2").arg(prefix).arg(Hns::Rights::FOLDER_SUFFIX_READ));
            rightsNames.append(QString("%1%2").arg(prefix).arg(Hns::Rights::FOLDER_SUFFIX_DELETE));
            rightsNames.append(QString("%1%2").arg(prefix).arg(Hns::Rights::FOLDER_SUFFIX_UPDATE_FOLDER));
        }
        if (d->helper->rightsMatchesRightsNames(rights, rightsNames))
            result = tr("Shared");
    }
    return result;
}

/*! \fn canBuildAclForEntry(const Hns::EntryPtr &target, const Hns::UserPtr &user)
 *\brief TODO
 *\param &target    Target entry
 *\param &user      User trying to build an ACL
 */
bool ACLManager::canBuildAclForCatalogItem(const Hns::CatalogItemPtr &target, const Hns::UserPtr &user)
{
    bool result = true;
    HSetManager *manager = Session::instance()->hSetManager();
    if (manager->getHSetById(target->hSetId).rank < manager->getHSetById(user->hSetId).rank)
        result = false;
    return result;
}

/*! \fn buildACLForEntry(const Hns::EntryPtr &target, const QString &subject, const QStringList &rights) const
 *\brief TODO
 *\param &target
 *\param &subject
 *\param &rights
 */
Diamesh::AccessControlList ACLManager::buildACLForCatalogItem(const Hns::CatalogItemPtr &target, const QString &subject, const QStringList &rights) const
{
    QStringList rightList;
    QString prefix = _prefixFromCatalogItem(target);
    if (prefix.isEmpty())
        rightList = rights;
    else
        foreach(const QString right, rights)
            rightList.append(prefix + right);
    return buildACL(target, subject, rightList);
}

/*! \fn buildACL(const Diamesh::ACLTargetPtr &target, const QString &subject, const QStringList &rights) const
 *\brief TODO
 *\param &target
 *\param &subject
 *\param &rights
 */
Diamesh::AccessControlList ACLManager::buildACL(const Diamesh::ACLTargetPtr &target, const QString &subject, const QStringList &rights) const
{
    Q_D(const ACLManager);
    Diamesh::AccessControlList acl;
    acl.target = d->helper->targetFromObject(target).toStdString();
    acl.subject = subject.toStdString();
    acl.hSetId = Session::instance()->user()->hSetId;
    acl.rights = d->helper->rightsFromNames(rights);
    return acl;
}

/*! \fn userFromSubject(const QString &subject) const
 *\brief TODO
 *\param &subject
 */
Hns::UserPtr ACLManager::userFromSubject(const QString &subject) const
{
    Q_D(const ACLManager);
    QPair<Diamesh::AccessRightsHelper::SubjectType, qlonglong> details = d->helper->subjectDetails(subject);
    if (details.first == Diamesh::AccessRightsHelper::USER_SUBJECT) {
        return _userFromSubject(details.second);
    }
    return 0;
}

/*! \fn groupFromSubject(const QString &subject) const
 *\brief TODO
 *\param &subject
 */
Diamesh::GroupPtr ACLManager::groupFromSubject(const QString &subject) const
{
    Q_D(const ACLManager);
    QPair<Diamesh::AccessRightsHelper::SubjectType, qlonglong> details = d->helper->subjectDetails(subject);
    if (details.first == Diamesh::AccessRightsHelper::GROUP_SUBJECT) {
        return _groupFromSubject(details.second);
    }
    return 0;
}

/*! \fn hSetFromSubject(const QString &subject) const
 *\brief TODO
 *\param &subject
 */
Diamesh::HierarchicalSet ACLManager::hSetFromSubject(const QString &subject) const
{
    Q_D(const ACLManager);
    QPair<Diamesh::AccessRightsHelper::SubjectType, qlonglong> details = d->helper->subjectDetails(subject);
    if (details.first == Diamesh::AccessRightsHelper::HSET_SUBJECT) {
        return _hSetFromSubject(details.second);
    }
    return HSetManager::invalidHSet();
}

/*! \fn ACLManager::getACLsForTarget(const Diamesh::ACLTargetPtr &target, QObject *receiver, const char *slot)
 *\brief Create a cookie object with receiver and slot parameters and give it to begin_importArchive with eStorage, tStorage and archive parameters.
 *\param target
 *\param receiver object connected to internal signal
 *\param slot Return slot connected to signal "completed()"
 */
void ACLManager::getACLsForTarget(const Diamesh::ACLTargetPtr &target, QObject *receiver, const char *slot)
{
    Q_D(ACLManager);
    GetACLsForTagetCookie *cookie = new GetACLsForTagetCookie(receiver, slot);
    d->aclManagerQPrx->begin_getACLsForTarget(target, this, SLOT(_getACLsForTargetSlot(::Diamesh::ACLList, QObject *)), 0, 0, cookie);
}

ACLManager::ACLManager(QObject *parent)
    : QObject(parent)
    , d_ptr(new ACLManagerPrivate)
{
}

void ACLManager::_load(Diamesh::ACLManagerQPrx qprx)
{
    Q_D(ACLManager);
    d->aclManagerQPrx = qprx;
    d->remoteCallCount = 2;
    d->aclManagerQPrx->begin_getRegisteredRights(this, SLOT(_loadGetRegisteredRightsSlot(::Diamesh::AccessRightList)));
    d->aclManagerQPrx->begin_getTargetTypes(this, SLOT(_loadGetTargetTypesSlot(::Diamesh::StringStringMap)));
}

void ACLManager::_callEnded()
{
    Q_D(ACLManager);
    d->remoteCallCount--;
    _checkLoading();
}

void ACLManager::_checkLoading()
{
    Q_D(ACLManager);
    if (d->remoteCallCount == 0)
        _init();
}

void ACLManager::_init()
{
    Q_D(ACLManager);
    connect(d->aclManagerQPrx.data(), SIGNAL(aclAdded(QString, QString)), this, SLOT(_aclAddedSlot(QString, QString)));
    connect(d->aclManagerQPrx.data(), SIGNAL(aclChanged(QString, QString)), this, SLOT(_aclChangedSlot(QString, QString)));
    connect(d->aclManagerQPrx.data(), SIGNAL(aclRemoved(QString, QString)), this, SLOT(_aclRemovedSlot(QString, QString)));
    emit loaded();
}

void ACLManager::_loadAccessRightsHelper()
{
    Q_D(ACLManager);
    if (d->helper)
        return;
    if (!d->rightList.empty() && !d->typeMap.empty())
        d->helper = new Diamesh::AccessRightsHelper(d->rightList, d->typeMap, this);
}

QString ACLManager::_prefixFromCatalogItem(const Hns::CatalogItemPtr &target) const
{
    Q_D(const ACLManager);
    QPair<QString, QString> details = d->helper->targetDetailsFromObject(target);
    return d->entryTargetToPrefix.value(details.first);
}

void ACLManager::_loadGetRegisteredRightsSlot(Diamesh::AccessRightList rightList)
{
    Q_D(ACLManager);
    d->rightList = rightList;
    _loadAccessRightsHelper();
    _callEnded();
}

void ACLManager::_loadGetTargetTypesSlot(Diamesh::StringStringMap typeMap)
{
    Q_D(ACLManager);
    d->typeMap = typeMap;
    _loadAccessRightsHelper();
    _callEnded();
}

void ACLManager::_getACLsForTargetSlot(Diamesh::ACLList aclList, QObject *obj)
{
    GetACLsForTagetCookie *cookie = qobject_cast<GetACLsForTagetCookie *>(obj);
    if (cookie)
        cookie->emitCompleted(aclList);
}

void ACLManager::_aclAddedSlot(QString target, QString subject)
{
    Q_UNUSED(target)
    Q_UNUSED(subject)
}

void ACLManager::_aclChangedSlot(QString target, QString subject)
{
    Q_UNUSED(target)
    Q_UNUSED(subject)
}

void ACLManager::_aclRemovedSlot(QString target, QString subject)
{
    Q_UNUSED(target)
    Q_UNUSED(subject)
}

Hns::UserPtr ACLManager::_userFromSubject(qlonglong userId) const
{
    return Session::instance()->userManager()->getUserById(userId);
}

Diamesh::GroupPtr ACLManager::_groupFromSubject(qlonglong groupId) const
{
    return Session::instance()->userManager()->getGroupById(groupId);
}

Diamesh::HierarchicalSet ACLManager::_hSetFromSubject(qlonglong hSetId) const
{
    return Session::instance()->hSetManager()->getHSetById(hSetId);
}

} // namespace HnsClient
