/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim hnsclient headers
#include "libhnsclient/catalogimporter.h"
#include "libhnsclient/private/catalogimporter_p.h"
#include "libhnsclient/session.h"

namespace HnsClient {

/*******************************************************************************
* CatalogImporterPrivate
*******************************************************************************/
CatalogImporterPrivate::CatalogImporterPrivate()
{
    catalogImporterQPrx = 0;
}

CatalogImporterPrivate::~CatalogImporterPrivate()
{
    catalogImporterQPrx = 0;
}

/*******************************************************************************
* ShowEntityEntryImportCookie
*******************************************************************************/
ShowEntityEntryImportCookie::ShowEntityEntryImportCookie(QObject *receiver, const char *slot)
    : QObject()
{
    connect(this, SIGNAL(completed(QMap<QString,HnsClient::EntityEntry*>)), receiver, slot);
}

ShowEntityEntryImportCookie::~ShowEntityEntryImportCookie()
{
}

void ShowEntityEntryImportCookie::emitCompleted(QMap<QString, EntityEntry *> importableMap)
{
    emit completed(importableMap);
    deleteLater();
}

/*******************************************************************************
* ImportFolderCookie
*******************************************************************************/
ImportFolderCookie::ImportFolderCookie(QObject *receiver, const char *slot)
    : QObject()
{
    connect(this, SIGNAL(completed(bool)), receiver, slot);
}

ImportFolderCookie::~ImportFolderCookie()
{
}

void ImportFolderCookie::emitCompleted(bool success)
{
    emit completed(success);
    deleteLater();
}

/*******************************************************************************
* ImportArchiveCookie
*******************************************************************************/
ImportArchiveCookie::ImportArchiveCookie(QObject *receiver, const char *slot)
    : QObject()
{
    connect(this, SIGNAL(completed(bool,QString)), receiver, slot);
}

ImportArchiveCookie::~ImportArchiveCookie()
{
}

void ImportArchiveCookie::emitCompleted(bool success, const QString &message)
{
    emit completed(success, message);
    deleteLater();
}


/*******************************************************************************
* ExportTopologyCookie
*******************************************************************************/
ExportTopologyCookie::ExportTopologyCookie(QObject *receiver, const char *slot)
    : QObject()
{
    connect(this, SIGNAL(completed(bool)), receiver, slot);
}

ExportTopologyCookie::~ExportTopologyCookie()
{
}

void ExportTopologyCookie::emitCompleted(bool result)
{
    emit completed(result);
    deleteLater();
}


/*******************************************************************************
* ExportEntityEntryCookie
*******************************************************************************/
ExportEntityEntryCookie::ExportEntityEntryCookie(qlonglong entryId, QObject *receiver, const char *slot)
    : QObject(),
      _entryId(entryId)
{
    connect(this, SIGNAL(completed(bool)), receiver, slot);
}

ExportEntityEntryCookie::~ExportEntityEntryCookie()
{
}

void ExportEntityEntryCookie::emitCompleted(bool result)
{
    emit completed(result);
    deleteLater();
}

/*******************************************************************************
* GetArchiveListCookie
*******************************************************************************/
GetArchiveListCookie::GetArchiveListCookie(QObject *receiver, const char *slot)
    : QObject()
{
    connect(this, SIGNAL(completed(Hns::ImportableArchiveList)), receiver, slot);
}

GetArchiveListCookie::~GetArchiveListCookie()
{
}

void GetArchiveListCookie::emitCompleted(Hns::ImportableArchiveList list)
{
    emit completed(list);
    deleteLater();
}


/*******************************************************************************
* CatalogImporter
*******************************************************************************/
CatalogImporter::CatalogImporter(QObject *parent)
    : QObject(parent)
    , d_ptr(new CatalogImporterPrivate)
{
}

CatalogImporter::~CatalogImporter()
{
    delete d_ptr;
}

void CatalogImporter::getImportableEntityEntry(QObject *receiver, const char *slot)
{
    Q_D(CatalogImporter);
    ShowEntityEntryImportCookie *cookie = new ShowEntityEntryImportCookie(receiver, slot);
    d->catalogImporterQPrx->begin_showEntityEntry(this, SLOT(_showEntityEntryImportSlot(::Hns::EntityEntryToImportMap,QObject*)), 0, 0, cookie);
}

/*! \fn importEntityEntry(QString entityEntryPath, QString entityStorage, QObject *receiver, const char *slot)
 *\brief Create a cookie object with receiver and slot parameters and give it to begin_importFolder with entityEntryPath and entityStorage on parameters.
 *\param entityEntryPath entity entry path
 *\param entityStorage entity storage
 *\param receiver object connected to internal signal
 *\param slot Return slot connected to signal "completed(qlonglong entityEntryId)", where "entityEntryId" is ID of entity entry
 */
void CatalogImporter::importEntityEntry(QString entityEntryPath, QString entityStorage, QObject *receiver, const char *slot)
{
    Q_D(CatalogImporter);
    ImportFolderCookie *cookie = new ImportFolderCookie(receiver, slot);
    Hns::ImportableEntryPtr importable = new Hns::ImportableEntry();
    importable->path = entityEntryPath.toStdString();
    d->catalogImporterQPrx->begin_importData(importable, entityStorage.toStdString(), std::string(), Hns::ImportOptionsList(), this, SLOT(_importFolderSlot(bool,QObject*)), 0, 0, cookie);
}


/*! \fn importArchive(const QString &eStorage, const QString &tStorage, const Hns::ImportableArchivePtr &archive, QObject *receiver, const char *slot)
 *\brief Create a cookie object with receiver and slot parameters and give it to begin_importArchive with eStorage, tStorage and archive parameters.
 *\param eStorage entity storage
 *\param tStorage topology storage
 *\param archive importable archive pointer
 *\param receiver object connected to internal signal
 *\param slot Return slot connected to signal "completed()"
 */
void CatalogImporter::importArchive(const QString &tStorage, const QString &eStorage, const Hns::ImportableArchivePtr &archive, const Hns::ImportOptionsList &optionsList, QObject *receiver, const char *slot)
{
    Q_D(CatalogImporter);
    ImportArchiveCookie *cookie = new ImportArchiveCookie(receiver, slot);
    //TODO: add a failure slot
    d->catalogImporterQPrx->begin_importData(archive, eStorage.toStdString(), tStorage.toStdString(), optionsList, this, SLOT(_importArchiveSlot(bool, QObject*)), 0, 0, cookie) ;
}

/*! \fn exportTopology(Hns::ArchiveDescriptorPtr archiveDescriptor, bool isArchive, QObject *receiver, const char *slot)
 *\brief Create a cookie object with receiver and slot parameters and give it to begin_exportTopology with eStorage, tStorage and isArchive parameters.
 *\param isArchive a boolean which specify if exported file will be an archive or a directory
 *\param receiver object connected to internal signal
 *\param slot Return slot connected to signal "completed(bool result)", where "result" is true when archive is successfully imported
 */
void CatalogImporter::exportTopology(Hns::ArchiveDescriptorPtr archiveDescriptor, bool isArchive, QObject *receiver, const char *slot)
{
    Q_D(CatalogImporter);
    ExportTopologyCookie *cookie = new ExportTopologyCookie(receiver, slot);

    d->catalogImporterQPrx->begin_exportData(archiveDescriptor, isArchive, this, SLOT(_exportTopologySlot(bool, QObject *)), 0, 0, cookie);
}

/*! \fn getArchiveList(QObject *receiver, const char *slot)
 *\brief Create a cookie object with receiver and slot parameters and give it to begin_getArchiveList.
 *\param receiver object connected to internal signal
 *\param slot Return slot connected to signal "completed(Hns::ImportableArchiveList list)", where list is the list of Importable Archive
 */
void CatalogImporter::getArchiveList(QObject *receiver, const char *slot)
{
    Q_D(CatalogImporter);
    GetArchiveListCookie *cookie = new GetArchiveListCookie(receiver, slot);
    d->catalogImporterQPrx->begin_getArchiveList(this, SLOT(_getArchiveListSlot(::Hns::ImportableArchiveList, QObject *)), 0, 0, cookie);
}

void CatalogImporter::exportEntityEntry(HnsClient::EntityEntry *entry, QObject *receiver, const char *slot)
{
    Q_D(CatalogImporter);
    ExportEntityEntryCookie *cookie = new ExportEntityEntryCookie(entry->getID(), receiver, slot);
//    entry->setSyncStatus(Syncing);
    d->catalogImporterQPrx->begin_exportData(_createEntityEntryDescriptor(entry), false, this, SLOT(_exportEntityEntrySlot(bool,QObject*)), 0, 0, cookie);
}

void CatalogImporter::_load(Hns::CatalogImporterQPrx catalogImporterQPrx)
{
    Q_D(CatalogImporter);
    d->catalogImporterQPrx = catalogImporterQPrx;
    connect(d->catalogImporterQPrx.data(), SIGNAL(progressUpdated(int)), this, SIGNAL(progressUpdated(int)));
    emit loaded();
}

void CatalogImporter::_importArchiveSlot(bool success, QObject *obj)
{
    ImportArchiveCookie *cookie = qobject_cast<ImportArchiveCookie *>(obj);
    if (cookie)
        cookie->emitCompleted(success);
}

void CatalogImporter::_exportTopologySlot(bool result, QObject *obj)
{
    qDebug() << tr("Export finished.") + (result ? tr("Success") : tr("Failed"));
    ExportTopologyCookie *cookie = qobject_cast<ExportTopologyCookie *>(obj);
    if (cookie)
        cookie->emitCompleted(result);
}

void CatalogImporter::_getArchiveListSlot(::Hns::ImportableArchiveList list, QObject *obj)
{
    GetArchiveListCookie *cookie = qobject_cast<GetArchiveListCookie *>(obj);
    if (cookie)
        cookie->emitCompleted(list);
}

void CatalogImporter::_showEntityEntryImportSlot(Hns::EntityEntryToImportMap importMap, QObject *obj)
{
    ShowEntityEntryImportCookie *cookie = qobject_cast<ShowEntityEntryImportCookie *>(obj);
    if (cookie) {
        QMap<std::string, Hns::EntityEntryPtr> importQMap(importMap);
        QMapIterator<std::string, Hns::EntityEntryPtr> iterator(importQMap);
        QMap<QString, HnsClient::EntityEntry *> importableMap;
        while(iterator.hasNext()) {
            iterator.next();
            importableMap.insert(QString::fromStdString(iterator.key()), EntryFactory::createEntityEntry(iterator.value(), this));
        }
        cookie->emitCompleted(importableMap);
    }
}

void CatalogImporter::_importFolderSlot(bool success, QObject *obj)
{
    ImportFolderCookie *cookie = qobject_cast<ImportFolderCookie *>(obj);
    if (cookie) {
        cookie->emitCompleted(success);
    }
}

void CatalogImporter::_exportEntityEntrySlot(bool result, QObject *obj)
{
    ExportEntityEntryCookie *cookie = qobject_cast<ExportEntityEntryCookie *>(obj);
    if (cookie) {
        cookie->emitCompleted(result);
    }
}


Hns::EntityEntryDescriptorPtr CatalogImporter::_createEntityEntryDescriptor(HnsClient::EntityEntry *entry)
{
    Hns::EntityEntryDescriptorPtr descriptor = new Hns::EntityEntryDescriptor();
    Hns::EntityDescriptor entityDescriptor;
    entityDescriptor.type = entry->entityEntry()->ice_id();
    entityDescriptor.shortName = entry->entityEntry()->shortName;
    entityDescriptor.longName = entry->entityEntry()->longName;
    entityDescriptor.description = entry->entityEntry()->description;
    entityDescriptor.path = entry->entityEntry()->path;
    entityDescriptor.uuid = entry->entityEntry()->uuid;
    entityDescriptor.fileType = Hns::FileType::PLAIN_FOLDER;
    entityDescriptor.tags = entry->entityEntry()->tagList;

    descriptor->entity = entityDescriptor;
    return descriptor;
}

} // namespace HnsClient
