/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNSCLIENT_ENTITY_H
#define HNSCLIENT_ENTITY_H

// Generated hynesim headers
#include "common/proxies_entity.h"
#include "common/proxies_wire.h"
#include "common/snapshot.h"
#include "common/master.h"
#include "common/layout.h"

// Qt headers
#include <QObject>
#include <QString>
#include <QMap>
#include <QVariant>

namespace HnsClient {

typedef QHash<QString, QString> StringStringHash;

class EntityEntry;
class DomainEntry;
class SwitchEntry;
class WireEntry;
class GenericEntityPrivate;
class EntityPrivate;
class WireEntityPrivate;
class DomainEntityPrivate;
class SwitchEntityPrivate;
class HybridNetcardEntityPrivate;
class WifiAPEntityPrivate;
class DynamipsRouterEntityPrivate;
class Topology;
class TopologyGateEntityPrivate;

/*******************************************************************************
* GenericEntity
*******************************************************************************/
class GenericEntity : public QObject
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(GenericEntity)
    Q_PROPERTY(QString shortName READ getShortName WRITE setShortName NOTIFY shortNameChanged)
    Q_PROPERTY(QString longName READ longName WRITE setLongName NOTIFY longNameChanged)
    Q_PROPERTY(QString description READ description WRITE setDescription NOTIFY descriptionChanged)
    Q_PROPERTY(QString tags READ tags WRITE setTags NOTIFY tagsChanged)
public:
    ~GenericEntity();

    static QString propertyDescription(const QString &propertyName);
    QMap<QString, QVariant> propertyInformation(const QString &propertyName) const;
    // Local methods, access to data stored in cache
    qlonglong getId() const;
    EntityEntry *getEntry() const;
    Hns::EntityEntryPtr getHnsEntry() const;
    QString iceId() const;
    qlonglong getEntryId() const;
    QString getShortName() const;
    QString longName() const;
    QString description() const;
    QString tags() const;

    QList<Hns::Status> getAllowedStatus() const;
    Hns::Status getStatus() const;

    bool isDefined() const;
    bool isStarted() const;
    bool isModel() const;

    QStringList getNodeList() const;
    QString getNodeName() const;
    QHash<int, int> getCaptureJobHash() const;
    QMap<int, Hns::Plug> getPlugs() const;
    QMap<int, Hns::Plug> getAvailablePlugs() const;
    QList<int> getAvailableInterfacesForCapture() const;
    int getFlags() const;

    bool isImplementedFromModel() const;
    void registerTracer();
    void unregisterTracer();
    bool userCanRead() const;
    bool userCanWrite() const;
    bool userCanExec() const;
    void addNode(QString nodeName);
    void removeNode(QString nodeName);

    // Method based on remote call
    virtual void define(QString nodeName);
    virtual void start();
    virtual void suspend();
    virtual void stop();
    virtual void save();
    virtual void undefine();
    virtual void destroy();
    virtual void unImplement();
    virtual void createCaptureJob(Hns::CaptureJobOptions options);
    virtual void cancelCaptureJob(Ice::Int jobId);
    void updateEntry(EntityEntry *updatedEntry);
    void setShortName(const QString &shortName);
    void setLongName(const QString &value);
    void setDescription(const QString &value);
    void setTags(const QString &value);

    void getPcapInfoByJobId(int jobID, QObject *receiver, const char * slot);
    void getCaptureJobIdPath(QObject *receiver, const char * slot);
    void getPcapInfoList(QObject *receiver, const char * slot);

    virtual void downloadCaptureFile(QStringList sourceList, QObject *receiver, const char *slot);
    virtual void eraseFile(QStringList sourceList);
    void refreshNodeList();
    void refresh();
    void getRemoteDisplayInfo(QObject *receiver, const char * slot);

signals:
    void loaded();
    void refreshed(HnsClient::GenericEntity *entity);
    void entryChanged(HnsClient::GenericEntity *entity);
    void statusChanged(HnsClient::GenericEntity *entity);
    void captureJobUseInterface(Ice::Int,Ice::Int);
    void captureJobReleasedInterface(Ice::Int,Ice::Int);
    void captureFilesReceived(::Diamesh::StringList);
    void availableInterfacesForCaptureChanged(QList<int>);
    void removeCaptureFiles(QString);
    void currentSnapshotChanged(::Hns::Snapshot);

    void shortNameChanged();
    void longNameChanged();
    void descriptionChanged();
    void tagsChanged();

protected slots:
    void _loadIdSlot(::Ice::Long entityId);
    void _loadEntrySlot(::Hns::EntityEntryPtr entry);
    void _loadStatusSlot(::Hns::Status currentStatus);
    void _loadAllowedStatusesSlot(::Hns::StatusList allowedStatus);
    void _loadPlugsSlot(::Hns::PlugMap plugMap);
    void _loadAvailablePlugsForCaptureSlot(::Diamesh::IntList interfaceList);
    void _loadFlagsSlot(::Ice::Int flags);
    void _loadIsImplementFromModelSlot(bool isImplement);
    void _loadTracerSlot(::Hns::TracerPrx tracerProxy);
    void _loadEntryNodeInfoMapSlot(::Hns::NodeInfoMap nodeMap);
    void _loadNodeNameSlot(::std::string nodeName);
    void _loadCaptureJobHashSlot(::Hns::IntIntMap map);
    // Slots received after a remote call
    void _defineSlot(bool result);
    void _undefineSlot(bool result);
    void _startSlot(bool result);
    void _resumeSlot(bool result);
    void _restoreSlot(bool result);
    void _suspendSlot(bool result);
    void _stopSlot(bool result);
    void _saveSlot(bool result);
    void _removeSlot(bool result);
    void _unImplementSlot(bool result);
    void _downloadCaptureFileSlot(::Diamesh::FileTransferPrx transferProxy, QObject *obj);
    void _getNodeNameSlot(::std::string nodeName);
    void _getEntrySlot(::Hns::EntityEntryPtr entry);
    void _getEntryNodeInfoMap(::Hns::NodeInfoMap nodeMap);
    void _getRemoteDisplayInfoSlot(::Hns::RemoteDisplayInfo remoteDisplayInfo, QObject *obj);
    // Slots connected to remote signals
    void _capturableInterfacesListChangedSlot(::Diamesh::IntList interfaceList);
    void _captureJobFileDeleteSlot(QString fileName);
    void _entryChangedSlot();
    void _implementFromModelChangedSlot(bool isImplemented);
    void _interfaceUsedSlot(Ice::Int interfaceId, Ice::Int jobId);
    void _interfaceReleasedSlot(Ice::Int interfaceId, Ice::Int jobId);
    void _plugListChangedSlot(::Hns::PlugMap plugMap);
    void _plugReleasedSlot(Ice::Long plugId);
    void _plugUsedSlot(Ice::Long plugId);
    void _statusChangedSlot(Ice::Int status, Ice::Long index);
    void _allowedStatusChangedSlot();
    void _createCaptureJobSlot(::Ice::Int);
    void _cancelCaptureJobSlot(bool);

protected:
    explicit GenericEntity(GenericEntityPrivate *d, QObject *parent = 0);
    virtual void _load(Hns::EntityQPrx entityQtProxy, Topology *topology);
    void _callEnded();
    void _checkLoading();
    virtual void _init();
    virtual void _loadEntryData(Hns::EntityEntryPtr entry);
    virtual bool _handleStatusData(Hns::Status status, QObject *receiver, const char *slot);
    void _handleAvailablePlugsForCaptureData(Diamesh::IntList interfaceList);
    void _handleNodeNameData(std::string nodeName);
    void _handleEntryNodeInfoMapData(Hns::NodeInfoMap nodeMap);
    void _handleIsImplementFromModelData(bool isImplement);
    void _setPlugUsed(int plugId, bool used);
    void _handlePlugsData(Hns::PlugMap plugMap);
    EntityEntry *_currentEntry() const;
    EntityEntry *_localEntry();
    void _resetLocalEntry();
    virtual void _fillPropertyInformation();

    GenericEntityPrivate * const d_ptr;

private:
    friend class Topology;
};


/*******************************************************************************
* WireEntity
*******************************************************************************/
class WireEntity : public GenericEntity
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(WireEntity)
    Q_ENUMS(VisualLayout)
    Q_PROPERTY(QString wireType READ wireType WRITE setWireType NOTIFY wireTypeChanged)
    Q_PROPERTY(VisualLayout visualLayout READ visualLayout NOTIFY visualLayoutChanged)
    Q_PROPERTY(int lossLR READ lossLR WRITE setLossLR)
    Q_PROPERTY(int delayLR READ delayLR WRITE setDelayLR)
    Q_PROPERTY(int dupLR READ dupLR WRITE setDupLR)
    Q_PROPERTY(int bandwidthLR READ bandwidthLR WRITE setBandwidthLR)
    Q_PROPERTY(int mtuLR READ mtuLR WRITE setMtuLR)
    Q_PROPERTY(int lossRL READ lossRL WRITE setLossRL)
    Q_PROPERTY(int delayRL READ delayRL WRITE setDelayRL)
    Q_PROPERTY(int dupRL READ dupRL WRITE setDupRL)
    Q_PROPERTY(int bandwidthRL READ bandwidthRL WRITE setBandwidthRL)
    Q_PROPERTY(int mtuRL READ mtuRL WRITE setMtuRL)

public:
    enum VisualLayout {
        BASIC
        , ONE_ANGLE
        , TWO_ANGLE
    };
    Q_INVOKABLE explicit WireEntity(HnsClient::Topology *parent = 0);
    ~WireEntity();

    virtual void destroy();
    void setLayout(Hns::WireLayoutPtr layout);
    Hns::WireLayoutPtr getLayout() const;
    qlonglong fromEntityId() const;
    qlonglong toEntityId() const;
    int fromEntityPlugId() const;
    int toEntityPlugId() const;
    GenericEntity *fromEntity() const;
    GenericEntity *toEntity() const;

    QString wireType() const;
    VisualLayout visualLayout() const;
    WireEntry *wireEntry() const;

    void setWireType(const QString &value);

    int lossLR() const;
    int delayLR() const;
    int dupLR() const;
    int bandwidthLR() const;
    int mtuLR() const;

    int lossRL() const;
    int delayRL() const;
    int dupRL() const;
    int bandwidthRL() const;
    int mtuRL() const;

    void setLossLR(int value);
    void setDelayLR(int value);
    void setDupLR(int value);
    void setBandwidthLR(int value);
    void setMtuLR(int value);

    void setLossRL(int value);
    void setDelayRL(int value);
    void setDupRL(int value);
    void setBandwidthRL(int value);
    void setMtuRL(int value);

signals:
    void layoutChanged();
    void wireTypeChanged();
    void visualLayoutChanged();

protected:
    bool _loadWire(Hns::WireQPrx wireQtProxy, QString from, QString to, Topology *topology);
    bool _loadWire(Hns::WireQPrx wireQtProxy, Topology *topology);
    bool _tryLoading(Topology *topology);
    void _splitWireEndInfo(QString endInfo, qlonglong &endId, int &endPort);
    virtual void _init();
    virtual void _loadEntryData(Hns::EntityEntryPtr entry);
    WireEntry *_currentWireEntry() const;
    WireEntry *_localWireEntry();
    virtual void _fillPropertyInformation();

private slots:
    void _toEntityEntryChangedSlot(HnsClient::GenericEntity*);
    void _fromEntityEntryChangedSlot(HnsClient::GenericEntity*);
private:
    friend class Topology;
};


/*******************************************************************************
* Entity
*******************************************************************************/
class Entity : public GenericEntity
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(Entity)
public:
    explicit Entity(EntityPrivate *d, QObject *parent = 0);
    ~Entity();

    Hns::EntityLayoutPtr getLayout() const;

signals:
    void layoutChanged();

protected:
    void _setLayout(Hns::EntityLayoutPtr layout);

private:
    friend class Topology;
};


/*******************************************************************************
* DomainEntity
*******************************************************************************/
class DomainEntity : public Entity
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(DomainEntity)
    Q_PROPERTY(QString domainType READ domainType)
    Q_PROPERTY(int memory READ memory)
    Q_PROPERTY(int vCpu READ vCpu)
public:
    Q_INVOKABLE explicit DomainEntity(QObject *parent = 0);
    ~DomainEntity();
    virtual void createCaptureJob(Hns::CaptureJobOptions options);
    virtual void cancelCaptureJob(Ice::Int jobId);

    virtual void createSnapshot(Hns::Snapshot snapshot, QObject *receiver, const char *slot);
    virtual void revertSnapshot(QString snapshotName, QObject *receiver, const char *slot);
    virtual void deleteSnapshot(const QString &snapshotName, QObject *receiver, const char *slot);
    virtual QString currentSnapshotName();
    virtual QStringList snapshotNameList();
    virtual QList<Hns::Snapshot> snapshotList();
    virtual void snapshotbyName(QString name, QObject *receiver, const char * slot);
    virtual int usbRedirectionCount();

    QList<Hns::MemoryDumpInfo> memoryDumpInfos();
    void createMemoryDumpJob(const QString &name, QObject *receiver, const char *slot);
    void deleteMemoryDump(QString fileName, QObject *receiver, const char *slot);
    void downloadMemoryDump(const QString &remoteFileName, const QString &localFilename, QObject *receiver, const char *slot);

    QString domainType() const;
    int memory() const;
    int vCpu() const;

signals:
    void memoryDumpAdded(::Hns::MemoryDumpInfo);
    void memoryDumpDeleted(QString);

protected slots:
    void _createSnapshotSlot(bool result, QObject *obj);
    void _revertSnapshotSlot(bool result, QObject *obj);
    void _deleteSnapshotSlot(bool result, QObject *obj);

    void _createMemoryDumpJobSlot(::Ice::Int jobId, QObject *obj);
    void _deleteMemoryDumpSlot(QObject *obj);
    void _getMemoryDumpInfoListSlot(::Hns::MemoryDumpInfoList list);
    void _memoryDumpAddedSlot(::Hns::MemoryDumpInfo info);
    void _memoryDumpRemovedSlot(QString fileName);
    void _downloadMemoryDumpSlot(::Diamesh::FileTransferPrx fileTransferPrx, QObject *obj);

protected:
    virtual void _load(Hns::EntityQPrx entityQtProxy, Topology *topology);
    virtual void _init();
    DomainEntry * _currentDomainEntry() const;
};


/*******************************************************************************
* SwitchEntity
*******************************************************************************/
class SwitchEntity : public Entity
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(SwitchEntity)
    Q_PROPERTY(QString switchType READ switchType WRITE setSwitchType)
    Q_PROPERTY(int portCount READ portCount WRITE setPortCount)
    Q_PROPERTY(bool isHub READ isHub WRITE setIsHub USER true)
//    Q_PROPERTY(bool stpEnabled READ stpEnabled WRITE setStpEnabled)

public:
    Q_INVOKABLE explicit SwitchEntity(QObject *parent = 0);
    ~SwitchEntity();
    virtual void createCaptureJob(Hns::CaptureJobOptions options);
    virtual void cancelCaptureJob(Ice::Int jobId);

    QString switchType() const;
    int portCount() const;
    bool isHub() const;
    bool stpEnabled() const;
    int portMirroring() const;

    void setSwitchType(const QString &value);
    void setPortCount(int value);
    void setIsHub(bool value);
    void setStpEnabled(bool value);
    void setPortMirroring(int value);
    SwitchEntry *switchEntry() const;


protected:
    SwitchEntry *_currentSwitchEntry() const;
    SwitchEntry *_localSwitchEntry();
    virtual void _fillPropertyInformation();
};


/*******************************************************************************
* HybridNetcardEntity
*******************************************************************************/
class HybridNetcardEntity : public Entity
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(HybridNetcardEntity)
public:
    Q_INVOKABLE explicit HybridNetcardEntity(QObject *parent = 0);
    ~HybridNetcardEntity();
    virtual void createCaptureJob(Hns::CaptureJobOptions options);
    virtual void cancelCaptureJob(Ice::Int jobId);
    QMap<QString, bool> getMacAddresses() const;

signals:
    void macAddressAdded(QString macAddress);
    void macAddressRemoved(QString macAddress);
    void macAddressChanged(QString macAddress, bool active);

protected slots:
    void _loadGetMacAddressesSlot(::Hns::MacAddressMap addressesMap);
    void _newMacAddrSlot(QString macAddress);
    void _macAddrGoneSlot(QString macAddress);
    void _macBecameActiveSlot(QString macAddress);
    void _macBecameInactiveSlot(QString macAddress);

protected:
    virtual void _load(Hns::EntityQPrx entityQtProxy, Topology *topology);
    virtual void _init();
    virtual bool _handleStatusData(Hns::Status status, QObject *receiver, const char *slot);
};


/*******************************************************************************
* WifiAPEntity
*******************************************************************************/
class WifiAPEntity : public Entity
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(WifiAPEntity)
public:
    Q_INVOKABLE explicit WifiAPEntity(QObject *parent = 0);
    ~WifiAPEntity();
    virtual void createCaptureJob(Hns::CaptureJobOptions options);
    virtual void cancelCaptureJob(Ice::Int jobId);
    QStringList connectedPeers() const;
    QString getSSID() const;

signals:
    void peerJoined(QString peer);
    void peerLeft(QString peer);

protected slots:
    void _loadConnectedPeersSlot(::Diamesh::StringList connectedPeerList);
    void _peerJoinedSlot(QString peer);
    void _peerLeftSlot(QString peer);

protected:
    virtual void _load(Hns::EntityQPrx entityQtProxy, Topology *topology);
    virtual void _init();
    virtual bool _handleStatusData(Hns::Status status, QObject *receiver, const char *slot);
};


/*******************************************************************************
* DynamipsRouterEntity
*******************************************************************************/
class DynamipsRouterEntity : public Entity
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(DynamipsRouterEntity)
public:
    Q_INVOKABLE explicit DynamipsRouterEntity(QObject *parent = 0);
    ~DynamipsRouterEntity();
    virtual void createCaptureJob(Hns::CaptureJobOptions options);
    virtual void cancelCaptureJob(Ice::Int jobId);
    void fixIdle(QObject *receiver, const char *slot);
    void extractConfiguration(QObject *object, const char *slot);
    void pushConfiguration(const QString &configuration);

signals:
    void pushConfigurationSignal();
protected slots:
};

/*******************************************************************************
* TopologyGateEntity
*******************************************************************************/
class TopologyGateEntity : public Entity
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(TopologyGateEntity)

public:
    Q_INVOKABLE explicit TopologyGateEntity(QObject *parent = 0);
    ~TopologyGateEntity();

    virtual void createCaptureJob(Hns::CaptureJobOptions options);
    virtual void cancelCaptureJob(Ice::Int jobId);
};

} // namespace HnsClient

#endif //HNSCLIENT_ENTITY_H
