/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim hnsclient headers
#include "libhnsclient/entityinfo.h"
#include "libhnsclient/private/entityinfo_p.h"

// Qt headers
#include <QDebug>

namespace HnsClient {

/*******************************************************************************
* EntityInfoPrivate
*******************************************************************************/
EntityInfoPrivate::EntityInfoPrivate()
{
    infoQtProxy = 0;
    entityId = 0;
    entryId = 0;
    remoteCallCount = 0;
    statusIndex = -1;
}

EntityInfoPrivate::~EntityInfoPrivate()
{
    infoQtProxy = 0;
}


/*******************************************************************************
* EntityInfoListPrivate
*******************************************************************************/
EntityInfoListPrivate::EntityInfoListPrivate()
{
}

EntityInfoListPrivate::~EntityInfoListPrivate()
{
    qDeleteAll(infoList);
    infoList.clear();
}


/*******************************************************************************
* EntityInfo
*******************************************************************************/
EntityInfo::EntityInfo(QObject *parent)
    : QObject(parent)
    , d_ptr(new EntityInfoPrivate)
{
}

EntityInfo::~EntityInfo()
{
    delete d_ptr;
}

/*! \fn getStatus();
 *\brief TODO
 */
Hns::Status EntityInfo::getStatus() const
{
    Q_D(const EntityInfo);
    return d->status;
}

/*! \fn entityId()
 *\brief TODO
 */
qlonglong EntityInfo::entityId() const
{
    Q_D(const EntityInfo);
    return d->entityId;
}

/*! \fn entryId()
 *\brief TODO
 */
qlonglong EntityInfo::entryId() const
{
    Q_D(const EntityInfo);
    return d->entryId;
}

/*! \fn infoData()
 *\brief TODO
 */
Hns::EntityInfoData EntityInfo::infoData() const
{
    Q_D(const EntityInfo);
    return d->data;
}

/*! \fn getShortName()
 *\brief TODO
 */
QString EntityInfo::getShortName() const
{
    Q_D(const EntityInfo);
    return QString::fromStdString(d->data.shortName);
}

/*! \fn getLongName()
 *\brief TODO
 */
QString EntityInfo::getLongName() const
{
    Q_D(const EntityInfo);
    return QString::fromStdString(d->data.longName);
}

/*! \fn getType()
 *\brief TODO
 */
QString EntityInfo::getType() const
{
    Q_D(const EntityInfo);
    return QString::fromStdString(d->data.type);
}

/*! \fn getSubType()
 *\brief TODO
 */
QString EntityInfo::getSubType() const
{
    Q_D(const EntityInfo);
    return QString::fromStdString(d->data.subtype);
}

void EntityInfo::_load(qlonglong entityId, Hns::EntityInfoQPrx qprx)
{
    Q_D(EntityInfo);
    d->infoQtProxy = qprx;
    d->entityId = entityId;
    d->remoteCallCount = 3;
    d->infoQtProxy->begin_entryId(this, SLOT(_loadEntryIdSlot(::Ice::Long)));
    d->infoQtProxy->begin_getStatus(this, SLOT(_loadGetStatusSlot(::Hns::Status)));
    d->infoQtProxy->begin_infoData(this, SLOT(_loadInfoDataSlot(::Hns::EntityInfoData)));
}

void EntityInfo::_callEnded()
{
    Q_D(EntityInfo);
    d->remoteCallCount--;
    _checkLoading();
}

void EntityInfo::_checkLoading()
{
    Q_D(EntityInfo);
    if (d->remoteCallCount == 0)
        _init();
}

void EntityInfo::_init()
{
    Q_D(EntityInfo);
    connect(d->infoQtProxy.data(), SIGNAL(statusChanged(Ice::Int, Ice::Long)), this, SLOT(_statusChangedSlot(Ice::Int,Ice::Long)));
    connect(d->infoQtProxy.data(), SIGNAL(entryChanged(::Hns::EntityInfoData)), this, SLOT(_entryChangedSlot(::Hns::EntityInfoData)));
    emit loaded();
}

void EntityInfo::_loadGetStatusSlot(Hns::Status status)
{
    Q_D(EntityInfo);
    d->status = status;
    _callEnded();
}

void EntityInfo::_loadEntryIdSlot(Ice::Long entryId)
{
    Q_D(EntityInfo);
    d->entryId = entryId;
    _callEnded();
}

void EntityInfo::_loadInfoDataSlot(Hns::EntityInfoData data)
{
    Q_D(EntityInfo);
    d->data = data;
    _callEnded();
}

void EntityInfo::_statusChangedSlot(Ice::Int status, Ice::Long statusIndex)
{
    Q_D(EntityInfo);
    if (statusIndex > d->statusIndex) {
        d->status = Hns::Status(status);
        d->statusIndex = statusIndex;
        emit statusChanged();
    }
}

void EntityInfo::_entryChangedSlot(Hns::EntityInfoData data)
{
    Q_D(EntityInfo);
    if (d->data != data) {
        d->data = data;
        emit entryChanged();
    }
}


/*******************************************************************************
* EntityInfoList
*******************************************************************************/
EntityInfoList::EntityInfoList(QObject *parent)
    : QObject(parent)
    , d_ptr(new EntityInfoListPrivate)
{
}

EntityInfoList::~EntityInfoList()
{
    delete d_ptr;
}

EntityInfo *EntityInfoList::getEntityInfo(qlonglong entityId)
{
    Q_D(EntityInfoList);
    EntityInfo *result = 0;
    foreach(EntityInfo *info, d->infoList)
        if (info->entityId() == entityId)
            result = info;
    return result;
}

bool EntityInfoList::hasEntityInfo(qlonglong entityId)
{
    if (getEntityInfo(entityId) != 0)
        return true;
    return false;
}

void EntityInfoList::_addEntityInfo(EntityInfo *info)
{
    Q_D(EntityInfoList);
    qlonglong entityId = info->entityId();
    EntityInfo *result = getEntityInfo(entityId);
    if (result)
        qWarning() << tr("EntityInfo for entity with Id %1 already in cache").arg(QString::number(entityId));
    else {
        d->infoList.append(info);
        emit entityInfoAdded(entityId);
    }
}

void EntityInfoList::_removeEntityInfo(qlonglong entityId)
{
    Q_D(EntityInfoList);
    EntityInfo *result = getEntityInfo(entityId);
    if (result) {
        d->infoList.removeAll(result);
        result->deleteLater();
        emit entityInfoRemoved(entityId);
    }
}

} // namespace HnsClient
