/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim hnsclient headers
#include "libhnsclient/entry.h"
#include "libhnsclient/private/entry_p.h"
#include "libhnsclient/private/privatesessionaccessor.h"
#include "libhnsclient/session.h"
#include "libhnsclient/entityentrycatalog.h"
#include "libhnsclient/entrypropertiesmanager.h"
#include "libhnsclient/macaddressmanager.h"

// hynesim common headers
#include "common/initializer.h"

// Qt headers
#include <QHash>
#include <QDateTime>
#include <QEventLoop>
#include <QPainter>

namespace HnsClient {

HNS_STATIC_HASH_INITIALIZER(hashAuthTypeString, HashAuthTypeString, ::Hns::AuthTypeEnum, QString)
HashAuthTypeString::HashAuthTypeString()
{
    insert(::Hns::NONE,"NONE");
    insert(::Hns::WEP,"WEP");
    insert(::Hns::WPA1,"WPA1");
    insert(::Hns::WPA2,"WPA2");
    insert(::Hns::WPA12,"WPA12");
}

HNS_STATIC_HASH_INITIALIZER(_entryObjectHash, EntryObjectHash, QString, QMetaObject)
EntryObjectHash::EntryObjectHash()
{
    insert(QString::fromStdString(Hns::DomainEntry::ice_staticId()), DomainEntry::staticMetaObject);
    insert(QString::fromStdString(Hns::SwitchEntry::ice_staticId()), SwitchEntry::staticMetaObject);
    insert(QString::fromStdString(Hns::HybridNetcardEntry::ice_staticId()), HybridNetcardEntry::staticMetaObject);
    insert(QString::fromStdString(Hns::WifiAPEntry::ice_staticId()), WifiAPEntry::staticMetaObject);
    insert(QString::fromStdString(Hns::DynamipsRouterEntry::ice_staticId()), DynamipsRouterEntry::staticMetaObject);
    insert(QString::fromStdString(Hns::WireEntry::ice_staticId()), WireEntry::staticMetaObject);
    insert(QString::fromStdString(Hns::TopologyGateEntry::ice_staticId()), TopologyGateEntry::staticMetaObject);
}

/*******************************************************************************
* CatalogItemPrivate
*******************************************************************************/
CatalogItemPrivate::CatalogItemPrivate()
    : catalogItem(0)
    , syncStatus(Synced)
{
}

CatalogItemPrivate::CatalogItemPrivate(const CatalogItemPrivate &d)
    : syncStatus(d.syncStatus)
    , aclList(d.aclList)
{
}

CatalogItemPrivate::~CatalogItemPrivate()
{
    catalogItem = 0;
}


/*******************************************************************************
* FolderEntryPrivate
*******************************************************************************/
FolderEntryPrivate::FolderEntryPrivate()
    : CatalogItemPrivate()
{
}

FolderEntryPrivate::FolderEntryPrivate(const FolderEntryPrivate &d, const Hns::FolderPtr &folder)
    : CatalogItemPrivate(d)
{
    catalogItem = folder;
}

FolderEntryPrivate::~FolderEntryPrivate()
{
}


/*******************************************************************************
* EntryPrivate
*******************************************************************************/
EntryPrivate::EntryPrivate()
    : CatalogItemPrivate()
    , isLocked(false)
    , lockType(::Hns::LOCK_TYPE_NOLOCK)
    , entryProperties(0)
{
}

EntryPrivate::EntryPrivate(const EntryPrivate &d, const Hns::EntryPtr &clonedEntry, const Hns::EntryPropertiesPtr &clonedProperties)
    : CatalogItemPrivate(d)
    , isLocked(d.isLocked)
    , entryProperties(clonedProperties)
{
    catalogItem = clonedEntry;
}

EntryPrivate::~EntryPrivate()
{
    entryProperties = 0;
}


/*******************************************************************************
* EntityEntryInfoUpdaterPrivate
*******************************************************************************/
EntityEntryInfoUpdaterPrivate::EntityEntryInfoUpdaterPrivate()
{
}

EntityEntryInfoUpdaterPrivate::~EntityEntryInfoUpdaterPrivate()
{
    entityInfo = 0;
    topologyManager = 0;
}


/*******************************************************************************
* EntityEntryPrivate
*******************************************************************************/
EntityEntryPrivate::EntityEntryPrivate()
    : EntryPrivate()
    , infoUpdater(0)
    , entityStatus(-1)
    , cloneProgress(0.0)
    , isCloning(false)
{
}

EntityEntryPrivate::EntityEntryPrivate(const EntityEntryPrivate &d, const Hns::EntityEntryPtr &clonedEntry, const Hns::EntityEntryPropertiesPtr &clonedProperties)
    : EntryPrivate(d, clonedEntry, clonedProperties)
    , infoUpdater(0)
    , entityStatus(d.entityStatus)
    , cloneProgress(d.cloneProgress)
    , isCloning(d.isCloning)
{
}

EntityEntryPrivate::~EntityEntryPrivate()
{
    if (infoUpdater) {
        delete infoUpdater;
    }
    infoUpdater = 0;
}


/*******************************************************************************
* WireEntryPrivate
*******************************************************************************/
WireEntryPrivate::WireEntryPrivate()
    : EntityEntryPrivate()
    , strFrom("")
    , strTo("")
{
}

WireEntryPrivate::WireEntryPrivate(const WireEntryPrivate &d, const Hns::WireEntryPtr &clonedEntry, const Hns::WireEntryPropertiesPtr &clonedProperties)
    : EntityEntryPrivate(d, clonedEntry, clonedProperties)
    , strFrom(d.strFrom)
    , strTo(d.strTo)
    , supportedImplementation(d.supportedImplementation)
{
}

WireEntryPrivate::~WireEntryPrivate()
{
}


/*******************************************************************************
* DomainEntryPrivate
*******************************************************************************/
DomainEntryPrivate::DomainEntryPrivate()
    : EntityEntryPrivate()
{
}

DomainEntryPrivate::DomainEntryPrivate(const DomainEntryPrivate &d, const Hns::DomainEntryPtr &clonedEntry, const Hns::DomainEntryPropertiesPtr &clonedProperties)
    : EntityEntryPrivate(d, clonedEntry, clonedProperties)
    , macAdressIsUsed(d.macAdressIsUsed)
{
}

DomainEntryPrivate::~DomainEntryPrivate()
{
}


/*******************************************************************************
* SwitchEntryPrivate
*******************************************************************************/
SwitchEntryPrivate::SwitchEntryPrivate()
    : EntityEntryPrivate()
{
}

SwitchEntryPrivate::SwitchEntryPrivate(const SwitchEntryPrivate &d, const Hns::SwitchEntryPtr &clonedEntry, const Hns::SwitchEntryPropertiesPtr &clonedProperties)
    : EntityEntryPrivate(d, clonedEntry, clonedProperties)
    , supportedImplementation(d.supportedImplementation)
{
}

SwitchEntryPrivate::~SwitchEntryPrivate()
{
}


/*******************************************************************************
* HybridNetcardEntryPrivate
*******************************************************************************/
HybridNetcardEntryPrivate::HybridNetcardEntryPrivate()
    : EntityEntryPrivate()
{
}

HybridNetcardEntryPrivate::HybridNetcardEntryPrivate(const HybridNetcardEntryPrivate &d, const Hns::HybridNetcardEntryPtr &clonedEntry, const Hns::EntityEntryPropertiesPtr &clonedProperties)
    : EntityEntryPrivate(d, clonedEntry, clonedProperties)
{
}

HybridNetcardEntryPrivate::~HybridNetcardEntryPrivate()
{
}


/*******************************************************************************
* WifiAPEntryPrivate
*******************************************************************************/
WifiAPEntryPrivate::WifiAPEntryPrivate()
    : EntityEntryPrivate()
{
}

WifiAPEntryPrivate::WifiAPEntryPrivate(const WifiAPEntryPrivate &d, const Hns::WifiAPEntryPtr &clonedEntry, const Hns::EntityEntryPropertiesPtr &clonedProperties)
    : EntityEntryPrivate(d, clonedEntry, clonedProperties)
{
}

WifiAPEntryPrivate::~WifiAPEntryPrivate()
{
}


/*******************************************************************************
* DynamipsRouterEntryPrivate
*******************************************************************************/
DynamipsRouterEntryPrivate::DynamipsRouterEntryPrivate()
    : EntityEntryPrivate()
{
}

DynamipsRouterEntryPrivate::DynamipsRouterEntryPrivate(const DynamipsRouterEntryPrivate &d, const Hns::DynamipsRouterEntryPtr &clonedEntry, const Hns::EntityEntryPropertiesPtr &clonedProperties)
    : EntityEntryPrivate(d, clonedEntry, clonedProperties)
{
}

DynamipsRouterEntryPrivate::~DynamipsRouterEntryPrivate()
{
}


/*******************************************************************************
* TopologyGateEntryPrivate
*******************************************************************************/
TopologyGateEntryPrivate::TopologyGateEntryPrivate()
    : EntityEntryPrivate()
{
}

TopologyGateEntryPrivate::TopologyGateEntryPrivate(const TopologyGateEntryPrivate &d, const Hns::TopologyGateEntryPtr &clonedEntry, const Hns::EntityEntryPropertiesPtr &clonedProperties)
    : EntityEntryPrivate(d, clonedEntry, clonedProperties)
{
}

TopologyGateEntryPrivate::~TopologyGateEntryPrivate()
{
}


/*******************************************************************************
* TopologyEntryPrivate
*******************************************************************************/
TopologyEntryPrivate::TopologyEntryPrivate()
    : EntryPrivate()
{
}

TopologyEntryPrivate::TopologyEntryPrivate(const TopologyEntryPrivate &d, const Hns::TopologyEntryPtr &clonedEntry, const Hns::EntryPropertiesPtr &clonedProperties)
    : EntryPrivate(d, clonedEntry, clonedProperties)
    , screenshot(d.screenshot)
{
}

TopologyEntryPrivate::~TopologyEntryPrivate()
{
}


/*******************************************************************************
* CatalogItem
*******************************************************************************/
CatalogItem::CatalogItem(CatalogItemPrivate *d, QObject* parent)
    : QObject(parent)
    , d_ptr(d)
{
}

CatalogItem::~CatalogItem()
{
    delete (d_ptr);
}

qlonglong CatalogItem::getID() const
{
    Q_D(const CatalogItem);
    return d->catalogItem->id;
}

QString CatalogItem::getShortName() const
{
    Q_D(const CatalogItem);
    return QString::fromStdString(d->catalogItem->shortName);
}

/*! \fn setShortName(const QString &shortName)
 *\brief TODO
 *\param shortName
 */
bool CatalogItem::setShortName(const QString &shortName)
{
    if (validateShortName(shortName) == Valid)
        return true;
    return false;
}

ValidationResult CatalogItem::validateShortName(const QString &shortName)
{
    Q_D(CatalogItem);
    if (shortName == QString::fromStdString(d->catalogItem->shortName))
        return Unmodified;
    if (shortName.trimmed() != "") {
        d->catalogItem->shortName = shortName.toStdString();
        return Valid;
    }
    return Invalid;
}

QString CatalogItem::getLongName() const
{
    Q_D(const CatalogItem);
    return QString::fromStdString(d->catalogItem->longName);
}

/*! \fn setLongName(const QString &longName)
 *\brief TODO
 *\param longName
 */
bool CatalogItem::setLongName(const QString &longName)
{
    if (validateLongName(longName) == Valid)
        return true;
    return false;
}

ValidationResult CatalogItem::validateLongName(const QString &longName)
{
    Q_D(CatalogItem);
    if (longName == QString::fromStdString(d->catalogItem->longName))
        return Unmodified;
    d->catalogItem->longName = longName.toStdString();
    return Valid;
}

QString CatalogItem::getDescription() const
{
    Q_D(const CatalogItem);
    return QString::fromStdString(d->catalogItem->description);
}

/*! \fn setDescription(const QString &description)
 *\brief TODO
 *\param description
 */
bool CatalogItem::setDescription(const QString &description)
{
    if (validateDescription(description) == Valid)
        return true;
    return false;
}

ValidationResult CatalogItem::validateDescription(const QString &description)
{
    Q_D(CatalogItem);
    if (description == QString::fromStdString(d->catalogItem->description))
        return Unmodified;
    d->catalogItem->description = description.toStdString();
    return Valid;
}

QString CatalogItem::getCreationDate() const
{
    Q_D(const CatalogItem);
    return QDateTime::fromMSecsSinceEpoch(d->catalogItem->creationDateTime).toString(Qt::SystemLocaleShortDate);
}

QString CatalogItem::getModificationDate() const
{
    Q_D(const CatalogItem);
    return QDateTime::fromMSecsSinceEpoch(d->catalogItem->lastModificationDateTime).toString(Qt::SystemLocaleShortDate);
}

QString CatalogItem::getTagList() const
{
    Q_D(const CatalogItem);
    QList<std::string> sourceTagQList = QList<std::string>::fromStdList(d->catalogItem->tagList);
    QStringList tagList;
    foreach(std::string tag, sourceTagQList)
        tagList.append(QString::fromStdString(tag));
    return tagList.join(", ");
}

/*! \fn setTagList(const QString &tagListString)
 *\brief TODO
 *\param tagListString
 */
bool CatalogItem::setTagList(const QString &tagListString)
{
    if (validateTagList(tagListString) == Valid)
        return true;
    return false;
}

ValidationResult CatalogItem::validateTagList(const QString &tagListString)
{
    Q_D(CatalogItem);
    QStringList sourceTagList = tagListString.split(",", QString::SkipEmptyParts);
    QList<std::string> tagList;
    foreach(QString tag, sourceTagList)  {
        tagList.append(tag.trimmed().toStdString());
    }
    QList<std::string> original = QList<std::string>::fromStdList(d->catalogItem->tagList);
    if (tagList == original)
        return Unmodified;
    d->catalogItem->tagList = tagList.toStdList();
    return Valid;
}

ValidationResult CatalogItem::validateACLList(const Diamesh::ACLList &aclList)
{
    Q_D(CatalogItem);
    if (aclList.size() != d->aclList.size()) {
        d->aclList = aclList;
        return Valid;
    } else {
        QList<Diamesh::AccessControlList> sourceList = QList<Diamesh::AccessControlList>::fromStdList(d->aclList);
        QList<Diamesh::AccessControlList> updateList = QList<Diamesh::AccessControlList>::fromStdList(aclList);
        foreach (const Diamesh::AccessControlList &source, sourceList) {
            bool found = false;
            foreach (const Diamesh::AccessControlList &update, updateList) {
                if (update == source) {
                    found = true;
                    break;
                }
            }
            if (!found) {
                d->aclList = aclList;
                return Valid;
            }
        }
    }
    return Unmodified;
}

Diamesh::ACLList CatalogItem::aclList() const
{
    Q_D(const CatalogItem);
    return d->aclList;
}

/*! \fn getParentID() const
 *\brief TODO
 */
qlonglong CatalogItem::getParentID()
{
    Q_D(CatalogItem);
    return d->catalogItem->parentId;
}

void CatalogItem::setParentID(qlonglong parentId)
{
    Q_D(CatalogItem);
    d->catalogItem->parentId = parentId;
}

/*! \fn isReadable() const
 *\brief TODO
 */
bool CatalogItem::isReadable() const
{
    return true;
}

/*! \fn isWriteable() const
 *\brief TODO
 */
bool CatalogItem::isWriteable() const
{
    return true;
}

/*! \fn isExecutable() const
 *\brief TODO
 */
bool CatalogItem::isExecutable() const
{
    return true;
}

/*! \fn isRWX() const
 *\brief TODO
 */
bool CatalogItem::isRWX() const
{
    return isReadable() && isWriteable() && isExecutable();
}

/*! \fn getSyncStatus() const
 *\brief TODO
 */
SynchronizationStatus CatalogItem::getSyncStatus() const
{
    Q_D(const CatalogItem);
    return d->syncStatus;
}

/*! \fn setSyncStatus(SynchronizationStatus status)
 *\brief TODO
 *\param status
 */
void CatalogItem::setSyncStatus(SynchronizationStatus status)
{
    Q_D(CatalogItem);
    d->syncStatus = status;
}

/*! \fn isSyncing() const
 *\brief TODO
 */
bool CatalogItem::isSyncing() const
{
    Q_D(const CatalogItem);
    return (d->syncStatus == Syncing);
}

void CatalogItem::setSourceCatalogItem(Hns::CatalogItemPtr catalogItem)
{
    Q_D(CatalogItem);
    d->catalogItem = catalogItem;
}

Hns::CatalogItemPtr CatalogItem::catalogItem()
{
    Q_D(CatalogItem);
    return d->catalogItem;
}

void CatalogItem::_updateACLList(Diamesh::ACLList aclList)
{
    Q_D(CatalogItem);
    d->aclList = aclList;
}

/*******************************************************************************
* FolderEntry
*******************************************************************************/
FolderEntry::FolderEntry(Hns::FolderPtr folderEntry, QObject *parent)
    : CatalogItem(new FolderEntryPrivate, parent)
{
    Q_D(FolderEntry);
    d->catalogItem = folderEntry;
}

FolderEntry::~FolderEntry()
{
}

/*! \fn isLocked() const
 *\brief TODO
 */
bool FolderEntry::isLocked() const
{
    return false;
}

/*! \fn isSaved() const
 *\brief TODO
 */
bool FolderEntry::isSaved() const
{
    return false;
}

/*! \fn isValid() const
 *\brief TODO
 */
bool FolderEntry::isValid() const
{
    return true;
}

/*! \fn isModel() const
 *\brief TODO
 */
bool FolderEntry::isModel() const
{
    return false;
}

/*! \fn hasLinkedClones() const
 *\brief TODO
 */
bool FolderEntry::hasLinkedClones() const
{
    return false;
}

/*! \fn isLinkedClone() const
 *\brief TODO
 */
bool FolderEntry::isLinkedClone() const
{
    return false;
}

/*! \fn isNew() const
 *\brief TODO
 */
bool FolderEntry::isNew() const
{
    Q_D(const FolderEntry);
    return (d->catalogItem->id == 0);
}

/*! \fn folderEntry()
 *\brief TODO
 */
Hns::FolderPtr FolderEntry::folderEntry()
{
    Q_D(FolderEntry);
    return Hns::FolderPtr::dynamicCast(d->catalogItem);
}

FolderEntry *FolderEntry::duplicateFolder(QObject *parentObject)
{
    Q_D(FolderEntry);
    if (!parentObject)
        parentObject = parent();
    return new FolderEntry(new FolderEntryPrivate(*d, Hns::FolderPtr::dynamicCast(d->catalogItem->ice_clone())), parentObject);
}

FolderEntry::FolderEntry(FolderEntryPrivate *d_ptr, QObject *parent)
    : CatalogItem(d_ptr, parent)
{
}

/*******************************************************************************
* Entry
*******************************************************************************/
Entry::Entry(EntryPrivate *d_ptr, Hns::EntryPtr entry, QObject *parent)
    : CatalogItem(d_ptr, parent)
{
    Q_D(Entry);
    d->catalogItem = entry;
    d->isLocked = false;
    refreshProperties();
}

Entry::Entry(EntryPrivate *d_ptr, QObject *parent)
    : CatalogItem(d_ptr, parent)
{
}

Entry::~Entry()
{
}

/*! \fn isLocked() const
 *\brief TODO
 */
bool Entry::isLocked() const
{
    Q_D(const Entry);
    return d->isLocked;
}

Hns::LockType Entry::lockType() const
{
    Q_D(const Entry);
    return d->lockType;
}

/*! \fn isSaved() const
 *\brief TODO
 */
bool Entry::isSaved() const
{
    return false;
}

/*! \fn isValid() const
 *\brief TODO
 */
bool Entry::isValid() const
{
    Q_D(const Entry);
    if (d->catalogItem->ice_id() == Hns::EntityEntry::ice_staticId())
        return false;
    return true;
}

/*! \fn isNew() const
 *\brief TODO
 */
bool Entry::isNew() const
{
    return getID() == 0;
}


/*! \fn isModel() const
 *\brief TODO
 */
bool Entry::isModel() const
{
    return false;
}

/*! \fn hasLinkedClones() const
 *\brief TODO
 */
bool Entry::hasLinkedClones() const
{
    return false;
}

/*! \fn isLinkedClone() const
 *\brief TODO
 */
bool Entry::isLinkedClone() const
{
    return false;
}

/*! \fn setLocked(bool locked)
 *\brief TODO
 *\param locked
 */
void Entry::setLocked(bool locked)
{
    Q_D(Entry);
    d->isLocked = locked;
}

void Entry::setLockType(Hns::LockType lockType)
{
    Q_D(Entry);
    d->lockType = lockType;
}

/*! \fn getUUID() const
 *\brief TODO
 */
QString Entry::getUUID() const
{
    return QString::fromStdString(entry()->uuid);
}

/*! \fn getPath() const
 *\brief TODO
 */
QString Entry::getPath() const
{
    return QString::fromStdString(entry()->path);
}

/*! \fn validatePath(const QString &path)
 *\brief TODO
 *\param path
 */
ValidationResult Entry::validatePath(const QString &path)
{
    if (path == QString::fromStdString(entry()->path))
        return Unmodified;
    entry()->path = path.toStdString();
    return Valid;
}

/*! \fn entry() const
 *\brief TODO
 */
Hns::EntryPtr Entry::entry() const
{
    Q_D(const Entry);
    return Hns::EntryPtr::dynamicCast(d->catalogItem);
}

void Entry::refreshProperties()
{
    PrivateSessionAccessor::instance()->session()->entryPropertiesManager()->getEntryProperties(entry(), this, SLOT(_getPropertiesSlot(::Hns::EntryPropertiesPtr)));
}

Hns::EntryPropertiesPtr Entry::entryProperties() const
{
    Q_D(const Entry);
    return d->entryProperties;
}

void Entry::_postGetProperties()
{
}

void Entry::_getPropertiesSlot(Hns::EntryPropertiesPtr properties)
{
    Q_D(Entry);
    d->entryProperties = properties;
    _postGetProperties();
}


/*******************************************************************************
* EntityEntryInfoUpdater
*******************************************************************************/
EntityEntryInfoUpdater::EntityEntryInfoUpdater(EntityEntry *entry, Hns::TopologyManagerQPrx topologyManager, QObject *parent)
    : QObject(parent)
    , d_ptr(new EntityEntryInfoUpdaterPrivate)
{
    Q_D(EntityEntryInfoUpdater);
    d->entry = entry;
    d->entityInfo = 0;
    d->topologyManager = topologyManager;
    d->cptStatus = 0;
    connect(d->topologyManager.data(), SIGNAL(requestEntityInfo(Ice::Long)), this ,SLOT(_requestGetEntityInfoSlot(Ice::Long)));
    d->topologyManager->begin_requestGetEntityInfo(d->entry->getID());
}

EntityEntryInfoUpdater::~EntityEntryInfoUpdater()
{
    delete (d_ptr);
}
/*******************************************************************************
* EntityEntryInfoUpdater - Private slots
*/
void EntityEntryInfoUpdater::_requestGetEntityInfoSlot(Ice::Long entityId)
{
    Q_D(EntityEntryInfoUpdater);
    if(entityId == d->entry->getID()){
        disconnect(d->topologyManager.data(), SIGNAL(requestEntityInfo(Ice::Long)));
        d->topologyManager->begin_getEntityInfo(d->entry->getID(), this, SLOT(_getEntityInfoSlot(::Hns::EntityInfoPrx)));
    }
}

void EntityEntryInfoUpdater::_getEntityInfoSlot(::Hns::EntityInfoPrx entityInfo)
{
    Q_D(EntityEntryInfoUpdater);
    if (entityInfo) {
        d->entityInfo = Hns::EntityInfoQPrx(entityInfo, PrivateSessionAccessor::instance()->adapter());
        d->entityInfo->begin_getStatus(this, SLOT(_getStatusSlot(::Hns::Status)));
        connect(d->entityInfo.data(), SIGNAL(statusChanged(Ice::Int, Ice::Long)), this, SLOT(_statusChangedSlot(Ice::Int,Ice::Long)));
    }
}

void EntityEntryInfoUpdater::_statusChangedSlot(Ice::Int status, Ice::Long cpt)
{
    Q_D(EntityEntryInfoUpdater);
    if(cpt > d->cptStatus){
        d->cptStatus = cpt;
        _updateEntry(Hns::Status(status));
    }
}

void EntityEntryInfoUpdater::_getStatusSlot(::Hns::Status status)
{
    Q_D(EntityEntryInfoUpdater);
    if(d->cptStatus == 0)
        _updateEntry(status);
}
/*******************************************************************************
* EntityEntryInfoUpdater - Private
*/
void EntityEntryInfoUpdater::_updateEntry(Hns::Status status)
{
    Q_D(EntityEntryInfoUpdater);
    d->entry->setEntityStatus(status);
    emit statusChanged(d->entry->getID());
}


/*******************************************************************************
* EntityEntry
*******************************************************************************/
EntityEntry::EntityEntry(EntityEntryPrivate *d_ptr, QObject *parent)
    : Entry(d_ptr, parent)
{
}

EntityEntry::EntityEntry(EntityEntryPrivate *d_ptr, Hns::EntityEntryPtr entityEntry, QObject *parent)
    : Entry(d_ptr, entityEntry, parent)
{
}

EntityEntry::EntityEntry(Hns::EntityEntryPtr entityEntry, QObject *parent) //protected
    : Entry(new EntityEntryPrivate, entityEntry, parent)
{
}

EntityEntry::~EntityEntry()
{
}

/*! \fn entityEntry() const
 *\brief TODO
 */
Hns::EntityEntryPtr EntityEntry::entityEntry() const
{
    Q_D(const EntityEntry);
    return Hns::EntityEntryPtr::dynamicCast(d->catalogItem);
}

/*! \fn getStatus() const
 *\brief TODO
 */
int EntityEntry::getStatus() const
{
    return entityEntry()->status;
}

/*! \fn Entry *duplicateEntry() const
 *\brief TODO
 */
Entry *EntityEntry::duplicateEntry(QObject *parentObject) const
{
    Q_D(const EntityEntry);
    if (!parentObject)
        parentObject = parent();
    Hns::EntityEntryPropertiesPtr properties = 0;
    if (d->entryProperties)
        properties = Hns::EntityEntryPropertiesPtr::dynamicCast(d->entryProperties->ice_clone());
    return new EntityEntry(new EntityEntryPrivate(*d, Hns::EntityEntryPtr::dynamicCast(d->catalogItem->ice_clone()), properties), parentObject);
}

/*! \fn setInfoUpdater(EntityEntryInfoUpdater *infoUpdater)
 *\brief TODO
 *\param infoUpdater
 */
void EntityEntry::setInfoUpdater(EntityEntryInfoUpdater *infoUpdater)
{
    Q_D(EntityEntry);
    if (d->infoUpdater) {
        delete d->infoUpdater;
        d->infoUpdater = 0;
    }
    if (infoUpdater == 0)
        d->entityStatus = -1;
    d->infoUpdater = infoUpdater;
}

/*! \fn setEntityStatus(Hns::Status status)
 *\brief TODO
 *\param status
 */
void EntityEntry::setEntityStatus(Hns::Status status)
{
    Q_D(EntityEntry);
    d->entityStatus = int(status);
}

/*! \fn getEntityStatus()
 *\brief TODO
 */
Hns::Status EntityEntry::getEntityStatus()
{
    Q_D(EntityEntry);
    return Hns::Status(d->entityStatus);
}

/*! \fn isUsedAsEntity()
 *\brief TODO
 */
bool EntityEntry::isUsedAsEntity()
{
    Q_D(EntityEntry);
    if (d->entityStatus != -1)
        return true;
    return false;
}

/*! \fn setIsCloning(bool isCloning)
 *\brief TODO
 *\param isCloning
 */
void EntityEntry::setIsCloning(bool isCloning)
{
    Q_D(EntityEntry);
    d->isCloning = isCloning;
}

/*! \fn setCloneProgress(double value)
 *\brief TODO
 *\param value
 */
void EntityEntry::setCloneProgress(double value)
{
    Q_D(EntityEntry);
    d->cloneProgress = value;
}

/*! \fn getCloneProgress() const
 *\brief TODO
 */
double EntityEntry::getCloneProgress() const
{
    Q_D(const EntityEntry);
    return d->cloneProgress;
}

/*! \fn isCloning() const
 *\brief TODO
 */
bool EntityEntry::isCloning() const
{
    Q_D(const EntityEntry);
    return (d->isCloning);
}

/*! \fn isTemplate() const
 *\brief TODO
 */
bool EntityEntry::isTemplate() const
{
    return entityEntry()->isTemplate;
}

/*! \fn isModel() const
 *\brief TODO
 */
bool EntityEntry::isModel() const
{
    return entityEntry()->isModel;
}

/*! \fn hasLinkedClonesqq() const
 *\brief TODO
 */
bool EntityEntry::hasLinkedClones() const
{
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(entityEntry());
    if (domainEntry && !domainEntry->diffUuids.empty())
        return true;
    return false;
}

/*! \fn isLinkedClone() const
 *\brief TODO
 */
bool EntityEntry::isLinkedClone() const
{
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(entityEntry());
    if (domainEntry && !domainEntry->baseUuid.empty())
        return true;
    return false;
}

/*! \fn isSaved() const
 *\brief TODO
 */
bool EntityEntry::isSaved() const
{
    return entityEntry()->saved;
}

/*! \fn hasResources() const
 *\brief TODO
 */
bool EntityEntry::hasResources() const
{
    return entityEntry()->hasResources;
}
/*! \fn customGraphic() const
 *\brief TODO
 */
QString EntityEntry::customGraphic() const
{
    return QString::fromStdString(entityEntry()->customGraphic);
}

/*! \fn validateTemplate(bool value)
 *\brief TODO
 *\param value
 */
ValidationResult EntityEntry::validateTemplate(bool value)
{
    if (entityEntry()->isTemplate == value)
        return Unmodified;
    entityEntry()->isTemplate = value;
    return Valid;
}

Hns::EntityEntryPropertiesPtr EntityEntry::entityEntryProperties() const
{
    Q_D(const EntityEntry);
    return Hns::EntityEntryPropertiesPtr::dynamicCast(d->entryProperties);
}


/*******************************************************************************
* WireEntry
*******************************************************************************/
WireEntry::WireEntry(Hns::EntityEntryPtr wireEntry, QObject *parent)
    : EntityEntry(new WireEntryPrivate, wireEntry, parent)
{
}

WireEntry::WireEntry(WireEntryPrivate *d_ptr, QObject *parent)
    : EntityEntry(d_ptr, parent)
{
}

WireEntry::~WireEntry()
{
}

Hns::WireEntryPtr WireEntry::wireEntry() const
{
    Q_D(const WireEntry);
    return Hns::WireEntryPtr::dynamicCast(d->catalogItem);
}

/*! \fn getLossLR() const
 *\brief TODO
 */
int WireEntry::getLossLR() const
{
    return wireEntry()->propertyLR.loss;
}

/*! \fn validateLossLR(int value)
 *\brief TODO
 *\param value
 */
ValidationResult WireEntry::validateLossLR(int value)
{
    Hns::WireEntryPtr entry = wireEntry();
    if (entry->propertyLR.loss == value)
        return Unmodified;
    entry->propertyLR.loss = value;
    return Valid;
}

/*! \fn getDelayLR() const
 *\brief TODO
 */
int WireEntry::getDelayLR() const
{
    return wireEntry()->propertyLR.delay;
}

/*! \fn validateDelayLR(int value)
 *\brief TODO
 *\param value
 */
ValidationResult WireEntry::validateDelayLR(int value)
{
    Hns::WireEntryPtr entry = wireEntry();
    if (entry->propertyLR.delay == value)
        return Unmodified;
    entry->propertyLR.delay = value;
    return Valid;
}

/*! \fn getDupLR() const
 *\brief TODO
 */
int WireEntry::getDupLR() const
{
    return wireEntry()->propertyLR.dup;
}

/*! \fn validateDupLR(int value)
 *\brief TODO
 *\param value
 */
ValidationResult WireEntry::validateDupLR(int value)
{
    Hns::WireEntryPtr entry = wireEntry();
    if (entry->propertyLR.dup == value)
        return Unmodified;
    entry->propertyLR.dup = value;
    return Valid;
}

/*! \fn getBandwidthLR() const
 *\brief TODO
 */
int WireEntry::getBandwidthLR() const
{
    return wireEntry()->propertyLR.bandwidth;
}

/*! \fn validateBandWidthLR(int value)
 *\brief TODO
 *\param value
 */
ValidationResult WireEntry::validateBandWidthLR(int value)
{
    Hns::WireEntryPtr entry = wireEntry();
    if (entry->propertyLR.bandwidth == value)
        return Unmodified;
    entry->propertyLR.bandwidth = value;
    return Valid;
}

/*! \fn getMtuLR() const
 *\brief TODO
 */
int WireEntry::getMtuLR() const
{
    return wireEntry()->propertyLR.mtu;
}

/*! \fn validateMtuLR(int value)
 *\brief TODO
 *\param value
 */
ValidationResult WireEntry::validateMtuLR(int value)
{
    Hns::WireEntryPtr entry = wireEntry();
    if (entry->propertyLR.mtu == value)
        return Unmodified;
    entry->propertyLR.mtu = value;
    return Valid;
}

/*! \fn getLossRL() const
 *\brief TODO
 */
int WireEntry::getLossRL() const
{
    return wireEntry()->propertyRL.loss;
}

/*! \fn validateLossRL(int value)
 *\brief TODO
 *\param value
 */
ValidationResult WireEntry::validateLossRL(int value)
{
    Hns::WireEntryPtr entry = wireEntry();
    if (entry->propertyRL.loss == value)
        return Unmodified;
    entry->propertyRL.loss = value;
    return Valid;
}

/*! \fn getDelayRL() const
 *\brief TODO
 */
int WireEntry::getDelayRL() const
{
    return wireEntry()->propertyRL.delay;
}

/*! \fn validateDelayRL(int value)
 *\brief TODO
 *\param value
 */
ValidationResult WireEntry::validateDelayRL(int value)
{
    Hns::WireEntryPtr entry = wireEntry();
    if (entry->propertyRL.delay == value)
        return Unmodified;
    entry->propertyRL.delay = value;
    return Valid;
}

/*! \fn getDupRL() const
 *\brief TODO
 */
int WireEntry::getDupRL() const
{
    return wireEntry()->propertyRL.dup;
}

/*! \fn validateDupRL(int value)
 *\brief TODO
 *\param value
 */
ValidationResult WireEntry::validateDupRL(int value)
{
    Hns::WireEntryPtr entry = wireEntry();
    if (entry->propertyRL.dup == value)
        return Unmodified;
    entry->propertyRL.dup = value;
    return Valid;
}

/*! \fn getBandwidthRL() const
 *\brief TODO
 */
int WireEntry::getBandwidthRL() const
{
    return wireEntry()->propertyRL.bandwidth;
}

/*! \fn validateBandWidthRL(int value)
 *\brief TODO
 *\param value
 */
ValidationResult WireEntry::validateBandWidthRL(int value)
{
    Hns::WireEntryPtr entry = wireEntry();
    if (entry->propertyRL.bandwidth == value)
        return Unmodified;
    entry->propertyRL.bandwidth = value;
    return Valid;
}

/*! \fn getMtuRL() const
 *\brief TODO
 */
int WireEntry::getMtuRL() const
{
    return wireEntry()->propertyRL.mtu;
}

/*! \fn validateMtuRL(int value)
 *\brief TODO
 *\param value
 */
ValidationResult WireEntry::validateMtuRL(int value)
{
    Hns::WireEntryPtr entry = wireEntry();
    if (entry->propertyRL.mtu == value)
        return Unmodified;
    entry->propertyRL.mtu = value;
    return Valid;
}

/*! \fn *duplicateEntry() const
 *\brief TODO
 */
Entry *WireEntry::duplicateEntry(QObject *parentObject) const
{
    Q_D(const WireEntry);
    if (!parentObject)
        parentObject = parent();
    Hns::WireEntryPropertiesPtr properties = 0;
    if (d->entryProperties)
        properties = Hns::WireEntryPropertiesPtr::dynamicCast(d->entryProperties->ice_clone());
    return new WireEntry(new WireEntryPrivate(*d, Hns::WireEntryPtr::dynamicCast(d->catalogItem->ice_clone()), properties), parentObject);
}

/*! \fn getFromEntity() const
 *\brief TODO
 */
QString WireEntry::getFromEntity() const
{
    Q_D(const WireEntry);
    return d->strFrom;
}

/*! \fn setFromEntity(const QString &fromEntity)
 *\brief TODO
 *\param fromEntity
 */
void WireEntry::setFromEntity(const QString &fromEntity)
{
    Q_D(WireEntry);
    d->strFrom = fromEntity;
}

/*! \fn getToEntity() const
 *\brief TODO
 */
QString WireEntry::getToEntity() const
{
    Q_D(const WireEntry);
    return d->strTo;
}

/*! \fn setToEntity(const QString &toEntity)
 *\brief TODO
 *\param toEntity
 */
void WireEntry::setToEntity(const QString &toEntity)
{
    Q_D(WireEntry);
    d->strTo = toEntity;
}

QString WireEntry::getType() const
{
    return QString::fromStdString(wireEntry()->type);
}

bool WireEntry::setType(const QString &type)
{
    if (validateType(type) == Valid)
        return true;
    return false;
}

ValidationResult WireEntry::validateType(const QString &type)
{
    Hns::WireEntryPtr entry = wireEntry();
    if (entry->type == type.toStdString())
        return Unmodified;
    entry->type = type.toStdString();
    return Valid;
}

bool WireEntry::setLossLR(int value)
{
    if (validateLossLR(value) == Valid)
        return true;
    return false;
}

bool WireEntry::setDelayLR(int value)
{
    if (validateDelayLR(value) == Valid)
        return true;
    return false;
}

bool WireEntry::setDupLR(int value)
{
    if (validateDupLR(value) == Valid)
        return true;
    return false;
}

bool WireEntry::setBandwidthLR(int value)
{
    if (validateBandWidthLR(value) == Valid)
        return true;
    return false;
}

bool WireEntry::setMtuLR(int value)
{
    if (validateMtuLR(value) == Valid)
        return true;
    return false;
}

bool WireEntry::setLossRL(int value)
{
    if (validateLossRL(value) == Valid)
        return true;
    return false;
}

bool WireEntry::setDelayRL(int value)
{
    if (validateDelayRL(value) == Valid)
        return true;
    return false;
}

bool WireEntry::setDupRL(int value)
{
    if (validateDupRL(value) == Valid)
        return true;
    return false;
}

bool WireEntry::setBandwidthRL(int value)
{
    if (validateBandWidthRL(value) == Valid)
        return true;
    return false;
}

bool WireEntry::setMtuRL(int value)
{
    if (validateMtuRL(value) == Valid)
        return true;
    return false;
}

Hns::WireEntryPropertiesPtr WireEntry::wireEntryProperties() const
{
    Q_D(const WireEntry);
    return Hns::WireEntryPropertiesPtr::dynamicCast(d->entryProperties);
}

QMap<QString, QString> WireEntry::supportedImplementation() const
{
    Q_D(const WireEntry);
    return d->supportedImplementation;
}

void WireEntry::_postGetProperties()
{
    Q_D(WireEntry);
    QMap<std::string, std::string> implementationMap(wireEntryProperties()->implementationsSupported);
    QMapIterator<std::string, std::string> iterator(implementationMap);
    while(iterator.hasNext()) {
        iterator.next();
        d->supportedImplementation.insert(QString::fromStdString(iterator.key()), QString::fromStdString(iterator.value()));
    }
}


/*******************************************************************************
* DomainEntry
*******************************************************************************/
DomainEntry::DomainEntry(Hns::EntityEntryPtr domainEntry, QObject *parent)
    : EntityEntry(new DomainEntryPrivate, domainEntry, parent)
{
    Q_D(DomainEntry);
    d->macAdressIsUsed = true;
}

DomainEntry::DomainEntry(DomainEntryPrivate *d_ptr, QObject *parent)
    : EntityEntry(d_ptr, parent)
{
}

Hns::DomainEntryPtr DomainEntry::domainEntry() const
{
    Q_D(const DomainEntry);
    return Hns::DomainEntryPtr::dynamicCast(d->catalogItem);
}

/*! \fn getType() const
 *\brief TODO
 */
QString DomainEntry::getType() const
{
    return QString::fromStdString(domainEntry()->type);
}

/*! \fn validateType(const QString &value)
 *\brief TODO
 *\param value
 */
ValidationResult DomainEntry::validateType(const QString &value)
{
    Hns::DomainEntryPtr entry = domainEntry();
    if (entry->type == value.toStdString())
        return Unmodified;
    entry->type = value.toStdString();
    return Valid;
}

/*! \fn getOperatingSystem() const
 *\brief TODO
 */
Hns::OperatingSystem DomainEntry::getOperatingSystem() const
{
    return domainEntry()->os;
}

/*! \fn validateArch(const QString &arch)
 *\brief TODO
 *\param arch
 */
ValidationResult DomainEntry::validateArch(const QString &arch)
{
    Hns::DomainEntryPtr entry = domainEntry();
    if (entry->os.arch == arch.toStdString())
        return Unmodified;
    entry->os.arch = arch.toStdString();
    return Valid;
}

/*! \fn validateMachine(const QString &machine)
 *\brief TODO
 *\param machine
 */
ValidationResult DomainEntry::validateMachine(const QString &machine)
{
    Hns::DomainEntryPtr entry = domainEntry();
    if (entry->os.machine == machine.toStdString())
        return Unmodified;
    entry->os.machine = machine.toStdString();
    return Valid;
}

/*! \fn validateOsKernel(const QString &kernel)
 *\brief TODO
 *\param kernel
 */
ValidationResult DomainEntry::validateOsKernel(const QString &kernel)
{
    if (kernel.contains("/"))
        return Invalid;
    Hns::DomainEntryPtr entry = domainEntry();
    if (entry->os.kernel == kernel.toStdString())
        return Unmodified;
    entry->os.kernel = kernel.toStdString();
    return Valid;
}

/*! \fn validateOsInitrd(const QString &initrd)
 *\brief TODO
 *\param initrd
 */
ValidationResult DomainEntry::validateOsInitrd(const QString &initrd)
{
    if (initrd.contains("/"))
        return Invalid;
    Hns::DomainEntryPtr entry = domainEntry();
    if (entry->os.initrd == initrd.toStdString())
        return Unmodified;
    entry->os.initrd = initrd.toStdString();
    return Valid;
}

/*! \fn validateOsCmdLine(const QString &cmdLine)
 *\brief TODO
 *\param cmdLine
 */
ValidationResult DomainEntry::validateOsCmdLine(const QString &cmdLine)
{
    Hns::DomainEntryPtr entry = domainEntry();
    if (entry->os.cmdline == cmdLine.toStdString())
        return Unmodified;
    entry->os.cmdline = cmdLine.toStdString();
    return Valid;
}

/*! \fn getMemory() const
 *\brief TODO
 */
int DomainEntry::getMemory() const
{
    return domainEntry()->memory;
}

/*! \fn validateMemory(int value)
 *\brief TODO
 *\param value
 */
ValidationResult DomainEntry::validateMemory(int value)
{
    Hns::DomainEntryPtr entry = domainEntry();
    if (entry->memory == value)
        return Unmodified;
    entry->memory = value;
    return Valid;
}

/*! \fn getVCpu() const
 *\brief TODO
 */
int DomainEntry::getVCpu() const
{
    return domainEntry()->vcpu;
}

/*! \fn validateVCpu(int value)
 *\brief TODO
 *\param value
 */
ValidationResult DomainEntry::validateVCpu(int value)
{
    Hns::DomainEntryPtr entry = domainEntry();
    if (entry->vcpu == value)
        return Unmodified;
    entry->vcpu = value;
    return Valid;
}

/*! \fn getPae() const
 *\brief TODO
 */
bool DomainEntry::getPae() const
{
    return domainEntry()->pae;
}

/*! \fn validatePae(bool value)
 *\brief TODO
 *\param value
 */
ValidationResult DomainEntry::validatePae(bool value)
{
    Hns::DomainEntryPtr entry = domainEntry();
    if (entry->pae == value)
        return Unmodified;
    entry->pae = value;
    return Valid;
}

/*! \fn getAcpi() const
 *\brief TODO
 */
bool DomainEntry::getAcpi() const
{
    return domainEntry()->acpi;
}

/*! \fn validateAcpi(bool value)
 *\brief TODO
 *\param value
 */
ValidationResult DomainEntry::validateAcpi(bool value)
{
    Hns::DomainEntryPtr entry = domainEntry();
    if (entry->acpi == value)
        return Unmodified;
    entry->acpi = value;
    return Valid;
}

/*! \fn getApic() const
 *\brief TODO
 */
bool DomainEntry::getApic() const
{
    return domainEntry()->apic;
}

/*! \fn validateApic(bool value)
 *\brief TODO
 *\param value
 */
ValidationResult DomainEntry::validateApic(bool value)
{
    Hns::DomainEntryPtr entry = domainEntry();
    if (entry->apic == value)
        return Unmodified;
    entry->apic = value;
    return Valid;
}

/*! \fn getImmutable() const
 *\brief return immutable domain entry attribute
 */
bool DomainEntry::getImmutable() const
{
    return domainEntry()->immutable;
}

/*! \fn validateImmutable(bool value)
 *\brief Set domain entry attribute.
 *\param value
 */
ValidationResult DomainEntry::validateImmutable(bool value)
{
    Hns::DomainEntryPtr entry = domainEntry();
    if (entry->immutable == value)
        return Unmodified;
    entry->immutable = value;
    return Valid;
}

/*! \fn getDisks() const
 *\brief TODO
 */
QList<Hns::Disk> HnsClient::DomainEntry::getDisks() const
{
    QList<Hns::Disk> disksList = QList<Hns::Disk>::fromStdList(domainEntry()->disks);
    return disksList;
}

/*! \fn validateDisks(QList<Hns::Disk> diskList, QHash<QString, int> &error)
 *\brief TODO
 *\param diskList
 *\param error
 */
ValidationResult DomainEntry::validateDisks(QList<Hns::Disk> diskList, QHash<QString, int> &error)
{
    error.clear();
    QList<Hns::Disk> savedList = QList<Hns::Disk>(getDisks());
    if (savedList == diskList) {
        return HnsClient::Unmodified;
    }
    else {
        for (int i = 0; i<diskList.count(); i++) {
            Hns::Disk disk = diskList.at(i);
            QString source = QString::fromStdString(disk.source);
            QString target(QString::fromStdString(disk.target));
            if (target.count() != 3) {
                error[source] = HnsClient::deviceIsInvalid;
                return HnsClient::Invalid;
            }
        }
        domainEntry()->disks = diskList.toStdList();
        return HnsClient::Valid;
    }
    return HnsClient::Invalid;
}

/*! \fn getNetcardMap() const
 *\brief TODO
 */
Hns::NetcardMap DomainEntry::getNetcardMap() const
{
    return domainEntry()->netcards;
}

void DomainEntry::_isUsedSlot(bool used)
{
    Q_D(DomainEntry);
    d->macAdressIsUsed = used;
    emit callCompleted();
    _isLoopNeedToBeLaunch = false;
}

/*! \fn validateNetworkCardMap(QMap<Ice::Long, Hns::Netcard> networkCardMap, QHash<Ice::Long, int>&errorHash)
 *\brief TODO
 *\param networkCardMap
 *\param errorHash
 */
ValidationResult DomainEntry::validateNetworkCardMap(QMap<Ice::Long, Hns::Netcard> networkCardMap, QHash<Ice::Long, int> &error)
{
    Q_D(DomainEntry);
    error.clear();
    QMap<Ice::Long, Hns::Netcard> savedMap = QMap<Ice::Long, Hns::Netcard>(getNetcardMap());
    QStringList networkCardNames;
    QRegExp validPattern("(?:[0-9a-fA-F]{2}:){5}[0-9a-fA-F]{2}");  //RegExp d'une adresse Mac
    QRegExp namePattern("eth[0-9][0-9]?[0-9]?[0-9]?");  //RegExp du nom d'une carte réseau
    MacAddressManager *macAddressManager = PrivateSessionAccessor::instance()->session()->macAddressManager();
    if (savedMap == networkCardMap) {
        return HnsClient::Unmodified;
    } else {
        for (QMap<Ice::Long, Hns::Netcard>::const_iterator it = networkCardMap.constBegin(); it != networkCardMap.constEnd(); ++it) {
            Ice::Long id = it.key();
            Hns::Netcard card = it.value();
            if (!networkCardNames.contains(QString::fromStdString(card.source))){
                networkCardNames.append(QString::fromStdString(card.source));
            } else {
                error[id] = HnsClient::nameIsUsed;
                return HnsClient::Invalid;
            }
            if (!namePattern.exactMatch(QString::fromStdString(card.source))){
                error[id] = HnsClient::nameIsInvalid;
                return HnsClient::Invalid;
            }
            if (!(validPattern.exactMatch(QString::fromStdString(card.macAddress)))){
                error[id]= HnsClient::macAdressError;
                return HnsClient::Invalid;
            }
            macAddressManager->isUsed(QString::fromStdString(card.macAddress), this, SLOT(_isUsedSlot(bool)));
            QEventLoop eventLoop;
            _isLoopNeedToBeLaunch = true;
            QObject::connect(this, SIGNAL(callCompleted()), &eventLoop, SLOT(quit()));
            if (_isLoopNeedToBeLaunch)
                eventLoop.exec();
            if (d->macAdressIsUsed) {
                error[id] = HnsClient::macAdressIsUsed;
                return HnsClient::Invalid;
            }
        }
        domainEntry()->netcards = networkCardMap.toStdMap();
        return HnsClient::Valid;
    }
    return HnsClient::Invalid;
}

/*! \fn getActionManagerInterfaceID() const
 *\brief TODO
 */
qlonglong DomainEntry::getActionManagerInterfaceID() const
{
    return domainEntry()->aminterface;
}

/*! \fn validateActionManagerInterfaceID(qlonglong value)
 *\brief TODO
 *\param value
 */
ValidationResult DomainEntry::validateActionManagerInterfaceID(qlonglong value)
{
    Hns::DomainEntryPtr entry = domainEntry();
    if (entry->aminterface == value)
        return Unmodified;
    QMap<Ice::Long, Hns::Netcard> networkCards = QMap<Ice::Long, Hns::Netcard>(entry->netcards);
    if (value != -1 && !networkCards.keys().contains(value))
        return Invalid;
    entry->aminterface = value;
    QList<std::string> restrictionList = QList<std::string>::fromStdList(entry->restrictionList);
    if (value == -1 && restrictionList.contains("actionManager"))
        restrictionList.removeAll("actionManager");
    else if (!restrictionList.contains("actionManager"))
        restrictionList.append("actionManager");

    entry->restrictionList = restrictionList.toStdList();
    return Valid;
}

/*! \fn getSharedFolder() const
 *\brief TODO
 */
bool DomainEntry::getSharedFolder() const
{
    return domainEntry()->sharedFolder;
}

/*! \fn getUsbRedirectionNumber() const
 *\brief TODO
 */
int DomainEntry::getUsbRedirectionCount() const
{
    return domainEntry()->usbRedirectionCount;
}

/*! \fn validateSharedFolder(bool value)
 *\brief TODO
 *\param value
 */
ValidationResult DomainEntry::validateSharedFolder(bool value)
{
    Hns::DomainEntryPtr entry = domainEntry();
    if (entry->sharedFolder == value)
        return Unmodified;
    entry->sharedFolder = value;
    return Valid;
}

/*! \fn getGraphics() const
 *\brief TODO
 */
Hns::Graphics DomainEntry::getGraphics() const
{
    return domainEntry()->graphic;
}

/*! \fn validateKeymap(QString keymap)
 *\brief TODO
 *\param keymap
 */
ValidationResult DomainEntry::validateKeymap(QString keymap)
{
    Hns::DomainEntryPtr entry = domainEntry();
    std::string newKeymap = keymap.toStdString();
    if (entry->graphic.keymap == newKeymap)
        return Unmodified;
    entry->graphic.keymap = newKeymap;
    return Valid;
}

/*! \fn *duplicateEntry()
 *\brief TODO
 */
Entry *DomainEntry::duplicateEntry(QObject *parentObject) const
{
    Q_D(const DomainEntry);
    if (!parentObject)
        parentObject = parent();
    Hns::DomainEntryPropertiesPtr properties = 0;
    if (d->entryProperties)
        properties = Hns::DomainEntryPropertiesPtr::dynamicCast(d->entryProperties->ice_clone());
    return new DomainEntry(new DomainEntryPrivate(*d, Hns::DomainEntryPtr::dynamicCast(d->catalogItem->ice_clone()), properties), parentObject);
}

/*! \fn getSnapshotList() const
 *\brief TODO
 */
Hns::SnapshotList DomainEntry::getSnapshotList() const
{
    return domainEntry()->snapshots;
}

/*! \fn getCurrentSnapshotName() const
 *\brief TODO
 */
QString DomainEntry::getCurrentSnapshotName() const
{
    return QString::fromStdString(domainEntry()->currentSnapshotName);
}

QString DomainEntry::getNodeKey() const
{
    return QString::fromStdString(domainEntry()->nodeKey);
}

ValidationResult DomainEntry::validateNodeKey(QString nodeKey)
{
    if (QString::fromStdString(domainEntry()->nodeKey) == nodeKey)
        return Unmodified;
    domainEntry()->nodeKey = nodeKey.toStdString();
    return Valid;
}
QString DomainEntry::getBaseUuid() const
{
    return QString::fromStdString(domainEntry()->baseUuid);
}

QStringList DomainEntry::getDiffUuids() const
{
    QStringList list;
    foreach(const std::string &s, domainEntry()->diffUuids)
        list.append(QString::fromStdString(s));
    return list;
}

/*! \fn validateUsbRedirectionNumber(int value)
 *\brief TODO
 *\param value
 */
ValidationResult DomainEntry::validateUsbRedirectionCount(int value)
{
    Hns::DomainEntryPtr entry = domainEntry();
    if (entry->usbRedirectionCount == value)
        return Unmodified;
    entry->usbRedirectionCount = value;
    return Valid;
}

QString DomainEntry::usbControllerType() const
{
    return QString::fromStdString(domainEntry()->usbControllerType);
}

ValidationResult DomainEntry::validateUsbControllerType(const QString &controllerType)
{
    Hns::DomainEntryPtr entry = domainEntry();
    if (entry->usbControllerType == controllerType.toStdString())
        return Unmodified;
    entry->usbControllerType = controllerType.toStdString();
    return Valid;
}

QString DomainEntry::getVideoModelType() const
{
    return QString::fromStdString(domainEntry()->vid.videoModelType);
}

ValidationResult DomainEntry::validateVideoModelType(QString videoModelType)
{
    if (getVideoModelType() == videoModelType)
        return Unmodified;
    domainEntry()->vid.videoModelType = videoModelType.toStdString();
    return Valid;
}

/*******************************************************************************
* SwitchEntry
*******************************************************************************/
SwitchEntry::SwitchEntry(Hns::EntityEntryPtr switchEntry, QObject *parent)
    : EntityEntry(new SwitchEntryPrivate, switchEntry, parent)
{
}

SwitchEntry::SwitchEntry(SwitchEntryPrivate *d_ptr, QObject *parent)
    : EntityEntry(d_ptr, parent)
{
}

Hns::SwitchEntryPtr SwitchEntry::switchEntry() const
{
    Q_D(const SwitchEntry);
    return Hns::SwitchEntryPtr::dynamicCast(d->catalogItem);
}

/*! \fn getPorts() const
 *\brief TODO
 */
int SwitchEntry::getPorts() const
{
    return switchEntry()->ports;
}

/*! \fn validatePorts(int value)
 *\brief TODO
 *\param value
 */
ValidationResult SwitchEntry::validatePorts(int value)
{
    Hns::SwitchEntryPtr entry = switchEntry();
    if (entry->ports == value)
        return Unmodified;
    entry->ports = value;
    return Valid;
}

/*! \fn getIsHub() const
 *\brief TODO
 */
bool SwitchEntry::getIsHub() const
{
    return switchEntry()->isHub;
}

/*! \fn getVlan()
 *\brief TODO
 */
Hns::VlanMap SwitchEntry::getVlan()
{
    return switchEntry()->vlans;
}

/*! \fn validateIsHub(bool value)
 *\brief TODO
 *\param value
 */
ValidationResult SwitchEntry::validateIsHub(bool value)
{
    Hns::SwitchEntryPtr entry = switchEntry();
    if (entry->isHub == value)
        return Unmodified;
    entry->isHub = value;
    return Valid;
}

/*! \fn getStpEnabled() const
 *\brief TODO
 */
bool SwitchEntry::getStpEnabled() const
{
    return switchEntry()->stpEnabled;
}

/*! \fn validateVlan(QMap<Ice::Int, Hns::Vlan> vlanMap)
 *\brief TODO
 *\param vlanMap
 */
ValidationResult SwitchEntry::validateVlan(QMap<Ice::Int, Hns::Vlan> vlanMap)
{
    if (vlanMap.toStdMap() == switchEntry()->vlans)
        return Unmodified;

    QList<int> listPorts;
    QMapIterator<Ice::Int, Hns::Vlan> iterator(vlanMap);
    while (iterator.hasNext()) {
        iterator.next();
        Hns::Vlan vlan = iterator.value();
        QMap<Ice::Int, Hns::PortTag> portsMap(vlan.portsMap);
        QMapIterator<Ice::Int, Hns::PortTag> it(portsMap);
        while (it.hasNext()) {
            it.next();
            if ( !(listPorts.contains(it.key())) && it.value() == Hns::Untagged )
                listPorts.append(it.key());
            else if (listPorts.contains(it.key()) && it.value() == Hns::Untagged)
                return Invalid;
        }
    }
    if (listPorts.size() >= switchEntry()->ports) {
        switchEntry()->vlans = vlanMap.toStdMap();
        return Valid;
    }
    else
        return Invalid;
}

/*! \fn validateStpEnabled(bool value)
 *\brief TODO
 *\param value
 */
ValidationResult SwitchEntry::validateStpEnabled(bool value)
{
    Hns::SwitchEntryPtr entry = switchEntry();
    if (entry->stpEnabled == value)
        return Unmodified;
    entry->stpEnabled = value;
    return Valid;
}

/*! \fn stpEnabled()
 *\brief TODO
 */
bool SwitchEntry::stpEnabled()
{
    return switchEntry()->stpEnabled;
}

/*! \fn getPortMirroring() const
 *\brief TODO
 */
int SwitchEntry::getPortMirroring() const
{
    return switchEntry()->portMirroring;
}

/*! \fn validatePortMirroring(int value)
 *\brief TODO
 *\param value
 */
ValidationResult SwitchEntry::validatePortMirroring(int value)
{
    Hns::SwitchEntryPtr entry = switchEntry();
    if (entry->portMirroring == value)
        return Unmodified;
    entry->portMirroring = value;
    return Valid;
}

/*! \fn duplicateEntry()
 *\brief TODO
 */
Entry *SwitchEntry::duplicateEntry(QObject *parentObject) const
{
    Q_D(const SwitchEntry);
    if (!parentObject)
        parentObject = parent();
    Hns::SwitchEntryPropertiesPtr properties = 0;
    if (d->entryProperties)
        properties = Hns::SwitchEntryPropertiesPtr::dynamicCast(d->entryProperties->ice_clone());
    return new SwitchEntry(new SwitchEntryPrivate(*d, Hns::SwitchEntryPtr::dynamicCast(d->catalogItem->ice_clone()), properties), parentObject);
}

QString SwitchEntry::getType() const
{
    return QString::fromStdString(switchEntry()->type);
}

ValidationResult SwitchEntry::validateType(QString type)
{
    Hns::SwitchEntryPtr entry = switchEntry();
    if (entry->type == type.toStdString())
        return Unmodified;
    entry->type = type.toStdString();
    return Valid;
}

bool SwitchEntry::setSwitchType(const QString &value)
{
    if (validateType(value) == Valid)
        return true;
    return false;
}

bool SwitchEntry::setPortCount(int value)
{
    if (validatePorts(value) == Valid)
        return true;
    return false;
}

bool SwitchEntry::setIsHub(bool value)
{
    if (validateIsHub(value) == Valid)
        return true;
    return false;
}

bool SwitchEntry::setStpEnabled(bool value)
{
    if (validateStpEnabled(value) == Valid)
        return true;
    return false;
}

bool SwitchEntry::setPortMirroring(int value)
{
    if (validatePortMirroring(value) == Valid)
        return true;
    return false;
}


Hns::SwitchEntryPropertiesPtr SwitchEntry::switchEntryProperties() const
{
    Q_D(const SwitchEntry);
    return Hns::SwitchEntryPropertiesPtr::dynamicCast(d->entryProperties);
}

QMap<QString, QString> SwitchEntry::supportedImplementation() const
{
    Q_D(const SwitchEntry);
    return d->supportedImplementation;
}

void SwitchEntry::_postGetProperties()
{
    Q_D(SwitchEntry);
    QMap<std::string, std::string> implementationMap(switchEntryProperties()->implementationsSupported);
    QMapIterator<std::string, std::string> iterator(implementationMap);
    while(iterator.hasNext()) {
        iterator.next();
        d->supportedImplementation.insert(QString::fromStdString(iterator.key()), QString::fromStdString(iterator.value()));
    }
}


/*******************************************************************************
* HybridNetcardEntry
*******************************************************************************/
HybridNetcardEntry::HybridNetcardEntry(Hns::EntityEntryPtr hybridNetcardEntry, QObject *parent)
    : EntityEntry(new HybridNetcardEntryPrivate, hybridNetcardEntry, parent)
{
}

HybridNetcardEntry::HybridNetcardEntry(HybridNetcardEntryPrivate *d_ptr, QObject *parent)
    : EntityEntry(d_ptr, parent)
{
}

Hns::HybridNetcardEntryPtr HybridNetcardEntry::hybridNetcardEntry() const
{
    Q_D(const HybridNetcardEntry);
    return Hns::HybridNetcardEntryPtr::dynamicCast(d->catalogItem);
}

/*! \fn getNodeName() const
 *\brief TODO
 */
QString HybridNetcardEntry::getNodeName() const
{
    return QString::fromStdString(hybridNetcardEntry()->nodeName);
}

/*! \fn validateNodeName(const QString &value)
 *\brief TODO
 *\param value
 */
ValidationResult HybridNetcardEntry::validateNodeName(const QString &value)
{
    Hns::HybridNetcardEntryPtr entry = hybridNetcardEntry();
    if (entry->nodeName == value.toStdString())
        return Unmodified;
    entry->nodeName = value.toStdString();
    return Valid;
}

/*! \fn getPlugIndex() const
 *\brief TODO
 */
QString HybridNetcardEntry::getPlugIndex() const
{
    return QString::fromStdString(hybridNetcardEntry()->plugIndex);
}

/*! \fn validatePlugIndex(const QString &value)
 *\brief TODO
 *\param value
 */
ValidationResult HybridNetcardEntry::validatePlugIndex(const QString &value)
{
    Hns::HybridNetcardEntryPtr entry = hybridNetcardEntry();
    if (entry->plugIndex == value.toStdString())
        return Unmodified;
    entry->plugIndex = value.toStdString();
    return Valid;
}

/*! \fn *duplicateEntry() const
 *\brief TODO
 */
Entry *HybridNetcardEntry::duplicateEntry(QObject *parentObject) const
{
    Q_D(const HybridNetcardEntry);
    if (!parentObject)
        parentObject = parent();
    Hns::EntityEntryPropertiesPtr properties = 0;
    if (d->entryProperties)
        properties = Hns::EntityEntryPropertiesPtr::dynamicCast(d->entryProperties->ice_clone());
    return new HybridNetcardEntry(new HybridNetcardEntryPrivate(*d, Hns::HybridNetcardEntryPtr::dynamicCast(d->catalogItem->ice_clone()), properties), parentObject);
}


/*******************************************************************************
* WifiAPEntry
*******************************************************************************/
WifiAPEntry::WifiAPEntry(Hns::EntityEntryPtr wifiApEntry, QObject *parent)
    : EntityEntry(new WifiAPEntryPrivate, wifiApEntry, parent)
{
}

WifiAPEntry::WifiAPEntry(WifiAPEntryPrivate *d_ptr, QObject *parent)
    : EntityEntry(d_ptr, parent)
{
}

Hns::WifiAPEntryPtr WifiAPEntry::wifiApEntry() const
{
    Q_D(const WifiAPEntry);
    return Hns::WifiAPEntryPtr::dynamicCast(d->catalogItem);
}

/*! \fn getSsid() const
 *\brief TODO
 */
QString WifiAPEntry::getSsid() const
{
    return QString::fromStdString(wifiApEntry()->ssid);
}

/*! \fn validateSsid(const QString &value)
 *\brief TODO
 *\param value
 */
ValidationResult WifiAPEntry::validateSsid(const QString &value)
{
    QString ssid = QString::fromStdString(wifiApEntry()->ssid);
    if (ssid == value){
        return Unmodified;
    }
    Hns::WifiAPEntryPtr entry = wifiApEntry();

    QRegExp begin("^(!|#|;)"); //SSID should not begin with !, # or;
    QRegExp forbidden("(/|\\]|\\+|\"|\\t)"); //Ssid can not be composed to +, ], /, \" or <TAB>
    if (value.contains(begin)){
        return Invalid;
    }
    if (value.contains(forbidden)){
        return Invalid;
    }
    if (value.size()>32){
        return Invalid;
    }
    entry->ssid = value.toStdString();
    return Valid;
}

/*! \fn getAuthentificationType() const
 *\brief TODO
 */
QString WifiAPEntry::getAuthentificationType() const
{
    return hashAuthTypeString()->value(wifiApEntry()->authType);
}

/*! \fn validateAuthentificationType(const QString &value)
 *\brief TODO
 *\param value
 */
ValidationResult WifiAPEntry::validateAuthentificationType(const QString &value)
{
    Hns::WifiAPEntryPtr entry = wifiApEntry();
    if (hashAuthTypeString()->key(value)== entry->authType){
        return Unmodified;
    }
    if (hashAuthTypeString()->values().contains(value)){
        entry->authType = hashAuthTypeString()->key(value);
        if(entry->authType == Hns::NONE)
            entry->key ="";
        return Valid;
    } else {
        return Invalid;
    }
}

/*! \fn getPassword()
 *\brief TODO
 */
QString WifiAPEntry::getPassword()
{
    return QString::fromStdString(wifiApEntry()->key);
}

/*! \fn validatePassword(const QString &value,bool isHexa)
 *\brief TODO
 *\param value
 *\param isHexa
 */
ValidationResult WifiAPEntry::validatePassword(const QString &value,bool isHexa)
{
    Hns::WifiAPEntryPtr entry = wifiApEntry();
    if (entry->key == value.toStdString()){
        bool result = checkPassphrase(value,getAuthentificationType(),isHexa);
        if(result)
            return Unmodified;
    }
    bool result = checkPassphrase(value,getAuthentificationType(),isHexa);
    if (result){
        entry->key = value.toStdString();
        return Valid;
    }
    return Invalid;
}

/*! \fn getIsHidden()
 *\brief TODO
 */
bool WifiAPEntry::getIsHidden()
{
    return wifiApEntry()->hidden;
}

/*! \fn validateHidden(bool value)
 *\brief TODO
 *\param value
 */
ValidationResult WifiAPEntry::validateHidden(bool value)
{
    Hns::WifiAPEntryPtr entry = wifiApEntry();
    if (value == entry->hidden){
        return Unmodified;
    }
    entry->hidden = value;
    return Valid;
}

/*! \fn getMacAdress()
 *\brief TODO
 */
QString WifiAPEntry::getMacAdress()
{
    return QString::fromStdString(wifiApEntry()->macAdress);
}

/*! \fn validateMacAddress(const QString &value)
 *\brief TODO
 *\param value
 */
ValidationResult WifiAPEntry::validateMacAddress(const QString &value)
{
    Hns::WifiAPEntryPtr entry = wifiApEntry();
    if (value == QString::fromStdString(entry->macAdress)){
        return Unmodified;
    }
//    TODO: Rechercher l'utilité de ce contrôle
//    if (value.startsWith("0")){
//        return HnsClient::Invalid;    //Il ne faut pas que l'adresse Mac commence par un 0 pour un wifiAP;
//    }
    QRegExp validPattern("(?:[0-9a-fA-F]{2}:){5}[0-9a-fA-F]{2}"); //RegExp d'une adresse Mac
    if (!validPattern.exactMatch(value)){
        return Invalid;
    } else {
        entry->macAdress = value.toStdString();
        return Valid;
    }
}

/*! \fn *duplicateEntry() const
 *\brief TODO
 */
Entry *WifiAPEntry::duplicateEntry(QObject *parentObject) const
{
    Q_D(const WifiAPEntry);
    if (!parentObject)
        parentObject = parent();
    Hns::EntityEntryPropertiesPtr properties = 0;
    if (d->entryProperties)
        properties = Hns::EntityEntryPropertiesPtr::dynamicCast(d->entryProperties->ice_clone());
    return new WifiAPEntry(new WifiAPEntryPrivate(*d, Hns::WifiAPEntryPtr::dynamicCast(d->catalogItem->ice_clone()), properties), parentObject);
}

/*! \fn checkPassphrase(QString passphrase,QString authTypeWanted,bool isHexa)
 *\brief TODO
 *\param passPhrase
 *\param authTypeWanted
 *\param isHexa
 */
bool WifiAPEntry::checkPassphrase(QString passphrase,QString authTypeWanted,bool isHexa)
{
    if (authTypeWanted == "WEP"){
        if (isHexa){
            if (passphrase.size() == Constants::WEP_SIZE_HEXA_40_BITS || passphrase.size() == Constants::WEP_SIZE_HEXA_128_BITS || passphrase.size() == Constants::WEP_SIZE_HEXA_152_BITS){
                return isHex(passphrase,passphrase.size());
            } else {
                return false;
            }
        } else {
            if (passphrase.size() == Constants::WEP_SIZE_DEC_40_BITS || passphrase.size() == Constants::WEP_SIZE_DEC_128_BITS || passphrase.size() == Constants::WEP_SIZE_DEC_152_BITS){
                return true;
            } else {
                return false;
            }
        }
    } else if (authTypeWanted == "WPA1" || authTypeWanted == "WPA2" || authTypeWanted == "WPA12"){
        if ((passphrase.size() >= Constants::WPA_SIZE_MIN && passphrase.size() <= Constants::WPA_SIZE_MAX) && !isHexa){
            return true;
        } else if ((passphrase.size() == Constants::WPA_SIZE_HEXA) && isHexa){
            return isHex(passphrase,passphrase.size());
        } else {
            return false;
        }
    }
    return false;
}

/*! \fn isHex(QString value,int amountCharacters)
 *\brief TODO
 *\param amountCharacters
 */
bool WifiAPEntry::isHex(QString value,int amountCharacters)
{
   QString hexa("^(\\d|[a-f]|[A-F])");
   QRegExp regExp = QRegExp(hexa.append("{%1}$").arg(amountCharacters));
   if (regExp.exactMatch(value)){
       return true;
   } else {
       return false;
   }
   return false;
}

/*! \fn getChannel() const
 *\brief TODO
 */
int WifiAPEntry::getChannel() const
{
    return wifiApEntry()->channel;
}

/*! \fn validateChannel(int value)
 *\brief TODO
 *\param value
 */
ValidationResult WifiAPEntry::validateChannel(int value)
{
    Hns::WifiAPEntryPtr entry = wifiApEntry();
    if (value == entry->channel) {
        return Unmodified;
    }
    else if (value > 0 && value < 12 ) {
        entry->channel = value;
        return Valid;
    }
    else
        return Invalid;
}


/*******************************************************************************
* DynamipsRouterEntry
*******************************************************************************/
DynamipsRouterEntry::DynamipsRouterEntry(Hns::EntityEntryPtr dynamipsRouterEntry, QObject *parent)
    : EntityEntry(new DynamipsRouterEntryPrivate, dynamipsRouterEntry, parent){
}

DynamipsRouterEntry::DynamipsRouterEntry(DynamipsRouterEntryPrivate *d_ptr, QObject *parent)
    : EntityEntry(d_ptr, parent)
{
}

/*! \fn dynamipsRouterEntry() const
 *\brief TODO
 */
Hns::DynamipsRouterEntryPtr DynamipsRouterEntry::dynamipsRouterEntry() const
{
    Q_D(const DynamipsRouterEntry);
    return Hns::DynamipsRouterEntryPtr::dynamicCast(d->catalogItem);
}

/*! \fn *duplicateEntry() const
 *\brief TODO
 */
Entry *DynamipsRouterEntry::duplicateEntry(QObject *parentObject) const
{
    Q_D(const DynamipsRouterEntry);
    if (!parentObject)
        parentObject = parent();
    Hns::EntityEntryPropertiesPtr properties = 0;
    if (d->entryProperties)
        properties = Hns::EntityEntryPropertiesPtr::dynamicCast(d->entryProperties->ice_clone());
    return new DynamipsRouterEntry(new DynamipsRouterEntryPrivate(*d, Hns::DynamipsRouterEntryPtr::dynamicCast(d->catalogItem->ice_clone()), properties), parentObject);
}

/*! \fn validateMacAdress(const QString &value)
 *\brief TODO
 *\param value
 */
ValidationResult DynamipsRouterEntry::validateMacAdress(const QString &value)
{
    Q_D(const DynamipsRouterEntry);
    Hns::DynamipsRouterEntryPtr entry = dynamipsRouterEntry();
    if(value == QString::fromStdString(entry->macAddress)){
        return Unmodified;
    }
    MacAddressManager *macAddressManager = PrivateSessionAccessor::instance()->session()->macAddressManager();
    QRegExp validPattern("(?:[0-9a-fA-F]{2}:){5}[0-9a-fA-F]{2}"); //RegExp d'une adresse Mac

    macAddressManager->isUsed(value, this, SLOT(_isUsedSlot(bool)));
    QEventLoop eventLoop;
    _isLoopNeedToBeLaunch = true;
    QObject::connect(this, SIGNAL(callCompleted()), &eventLoop, SLOT(quit()));
    if (_isLoopNeedToBeLaunch)
        eventLoop.exec();
    if (!validPattern.exactMatch(value) || d->macAddressIsUsed ){
        return HnsClient::Invalid;
    } else {
        entry->macAddress = value.toStdString();
        return Valid;
    }
    return Invalid;
}

/*! \fn validateConfiguration(const QString &value)
 *\brief TODO
 *\param value
 */
ValidationResult DynamipsRouterEntry::validateConfiguration(const QString & value)
{
    Hns::DynamipsRouterEntryPtr entry = dynamipsRouterEntry();
    std::string valueStr = value.toStdString();
    if (entry->configuration == valueStr)
        return Unmodified;
    entry->configuration = valueStr;
    return Valid;
}

/*! \fn validateApplyOnBot(bool value)
 *\brief TODO
 *\param value
 */
ValidationResult DynamipsRouterEntry::validateApplyOnBot(bool value)
{
    Hns::DynamipsRouterEntryPtr entry = dynamipsRouterEntry();

    if (entry->applyOnBoot == value)
        return Unmodified;
    entry->applyOnBoot = value;
    return Valid;
}

/*! \fn validateAutoExtract(bool value)
 *\brief TODO
 *\param value
 */
ValidationResult DynamipsRouterEntry::validateAutoExtract(bool value)
{
    Hns::DynamipsRouterEntryPtr entry = dynamipsRouterEntry();
    if (entry->autoExtract == value)
        return Unmodified;
    entry->autoExtract = value;
    return Valid;
}

/*! \fn validateModules(const ::Hns::ModulePositionMap &positionMap)
 *\brief TODO
 *\param positionMap
 */
ValidationResult DynamipsRouterEntry::validateModules(const ::Hns::ModulePositionMap &positionMap)
{
    Hns::DynamipsRouterEntryPtr entry = dynamipsRouterEntry();
    bool modified = (positionMap.size() != entry->modulesMap.size());
    ::Hns::ModulePositionMap::const_iterator itorBase = entry->modulesMap.begin();
    ::Hns::ModulePositionMap::const_iterator itorNew = positionMap.begin();
    while(itorBase != entry->modulesMap.end() && !modified) {
        modified = itorBase->first != itorNew->first || itorBase->second.name != itorNew->second.name;
        itorBase++;
        itorNew++;
    }
    if(!modified)
        return Unmodified;
    entry->modulesMap = positionMap;
    return Valid;
}

/*! \fn validateChassis(const QString &chassis)
 *\brief TODO
 *\param chassis
 */
ValidationResult DynamipsRouterEntry::validateChassis(const QString & chassis)
{
    Hns::DynamipsRouterEntryPtr entry = dynamipsRouterEntry();
    ::std::string chassisStr = chassis.toStdString();
    if (entry->chassis == chassisStr)
        return Unmodified;
    entry->chassis = chassisStr;
    return Valid;
}

/*! \fn validatePlatform(const QString &router)
 *\brief TODO
 *\param router
 */
ValidationResult DynamipsRouterEntry::validatePlatform(const QString & router)
{
    Hns::DynamipsRouterEntryPtr entry = dynamipsRouterEntry();
    ::std::string routerStr = router.toStdString();
    if (entry->router == routerStr)
        return Unmodified;
    entry->router = routerStr;
    return Valid;
}

/*! \fn validateNvRam(int nvram)
 *\brief TODO
 *\param nvram
 */
ValidationResult DynamipsRouterEntry::validateNvRam(int nvram)
{
    Hns::DynamipsRouterEntryPtr entry = dynamipsRouterEntry();
    if (entry->nvram == nvram)
        return Unmodified;
    entry->nvram = nvram;
    return Valid;
}

/*! \fn validateNeedSetChassis(int needSetChassis)
 *\brief TODO
 *\param needSetChassis
 */
ValidationResult DynamipsRouterEntry::validateNeedSetChassis(int needSetChassis)
{
    Hns::DynamipsRouterEntryPtr entry = dynamipsRouterEntry();
    if (entry->needSetChassis == needSetChassis)
        return Unmodified;
    entry->needSetChassis = needSetChassis;
    return Valid;
}

/*! \fn validateMemory(int memory)
 *\brief TODO
 *\param memory
 */
ValidationResult DynamipsRouterEntry::validateMemory(int memory)
{
    Hns::DynamipsRouterEntryPtr entry = dynamipsRouterEntry();
    if (entry->ram == memory)
        return Unmodified;
    entry->ram = memory;
    return Valid;
}

/*! \fn validateImage(const QString &image)
 *\brief TODO
 *\param image
 */
ValidationResult DynamipsRouterEntry::validateImage(const QString & image)
{
    Hns::DynamipsRouterEntryPtr entry = dynamipsRouterEntry();
    ::std::string imageStr = image.toStdString();
    if (entry->imageDisk== imageStr)
        return Unmodified;
    entry->imageDisk = imageStr;
    return Valid;
}

/*! \fn validateIdle(const QString &idle)
 *\brief TODO
 *\param idle
 */
ValidationResult DynamipsRouterEntry::validateIdle(const QString & idle)
{
    Hns::DynamipsRouterEntryPtr entry = dynamipsRouterEntry();
    std::string idleStr = idle.toStdString();
    if(entry->idlePc == idleStr){
        return Unmodified;
    }
    QRegExp validPattern("(0x[0-9a-f]{8})");
    if (!validPattern.exactMatch(idle) && !idle.isEmpty() ){
        return Invalid;
    } else {
        entry->idlePc = idleStr;
        return Valid;
    }
}

/*! \fn getChassis() const
 *\brief TODO
 */
QString DynamipsRouterEntry::getChassis() const
{
    return QString::fromStdString(dynamipsRouterEntry()->chassis);
}

/*! \fn getRouter() const
 *\brief TODO
 */
QString DynamipsRouterEntry::getRouter() const
{
    return QString::fromStdString(dynamipsRouterEntry()->router);
}

/*! \fn getMacAdress() const
 *\brief TODO
 */
QString DynamipsRouterEntry::getMacAdress() const
{
    return QString::fromStdString(dynamipsRouterEntry()->macAddress);
}

/*! \fn getIdlePc() const
 *\brief TODO
 */
QString DynamipsRouterEntry::getIdlePc() const
{
    return QString::fromStdString(dynamipsRouterEntry()->idlePc);
}

/*! \fn getConfiguration() const
 *\brief TODO
 */
QString DynamipsRouterEntry::getConfiguration() const
{
    return QString::fromStdString(dynamipsRouterEntry()->configuration);
}

/*! \fn getAutoExtract() const
 *\brief TODO
 */
bool DynamipsRouterEntry::getAutoExtract() const
{
    return dynamipsRouterEntry()->autoExtract;
}

/*! \fn getApplyOnBoot() const
 *\brief TODO
 */
bool DynamipsRouterEntry::getApplyOnBoot() const
{
    return dynamipsRouterEntry()->applyOnBoot;
}

/*! \fn getModulesMap() const
 *\brief TODO
 */
Hns::ModulePositionMap DynamipsRouterEntry::getModulesMap() const
{
    return dynamipsRouterEntry()->modulesMap;
}

/*! \fn getMemory() const
 *\brief TODO
 */
int DynamipsRouterEntry::getMemory() const
{
    return dynamipsRouterEntry()->ram;
}

/*! \fn getImage() const
 *\brief TODO
 */
QString DynamipsRouterEntry::getImage() const
{
    return QString::fromStdString(dynamipsRouterEntry()->imageDisk);
}

void DynamipsRouterEntry::_isUsedSlot(bool used)
{
    Q_D(DynamipsRouterEntry);
    d->macAddressIsUsed = used;
    emit callCompleted();
    _isLoopNeedToBeLaunch = false;
}


/*******************************************************************************
* TopologyEntry
*******************************************************************************/
TopologyEntry::TopologyEntry(Hns::TopologyEntryPtr topologyEntry, QPixmap screenshot, QObject *parent)
    : Entry(new TopologyEntryPrivate, topologyEntry, parent)
{
    Q_D(TopologyEntry);
    d->screenshot = screenshot;
}

TopologyEntry::TopologyEntry(TopologyEntryPrivate *d_ptr, QObject *parent)
    : Entry(d_ptr, parent)
{
}

TopologyEntry::~TopologyEntry()
{
}

/*! \fn topologyEntry() const
 *\brief TODO
 */
Hns::TopologyEntryPtr TopologyEntry::topologyEntry() const
{
    Q_D(const TopologyEntry);
    return Hns::TopologyEntryPtr::dynamicCast(d->catalogItem);
}

/*! \fn getStatus() const
 *\brief TODO
 */
int TopologyEntry::getStatus() const
{
    return topologyEntry()->status;
}

/*! \fn isNew() const
 *\brief TODO
 */
bool TopologyEntry::isNew() const
{
    return getPath() == "";
}

/*! \fn isModel() const
 *\brief TODO
 */
bool TopologyEntry::isModel() const
{
    return topologyEntry()->isModel;
}

bool TopologyEntry::isPersistent() const
{
    return topologyEntry()->isPersistent;
}

/*! \fn *duplicateEntry() const
 *\brief TODO
 */
Entry *TopologyEntry::duplicateEntry(QObject *parentObject) const
{
    Q_D(const TopologyEntry);
    if (!parentObject)
        parentObject = parent();
    Hns::EntryPropertiesPtr properties = 0;
    if (d->entryProperties)
        properties = Hns::EntryPropertiesPtr::dynamicCast(d->entryProperties->ice_clone());
    return new TopologyEntry(new TopologyEntryPrivate(*d, Hns::TopologyEntryPtr::dynamicCast(d->catalogItem->ice_clone()), properties), parentObject);
}

ValidationResult TopologyEntry::validateIsPersistent(bool value)
{
    Hns::TopologyEntryPtr entry = topologyEntry();
    if (entry->isPersistent == value)
        return Unmodified;
    entry->isPersistent = value;
    return Valid;
}

/*! \fn setScreenshot(QPixmap scr)
 *\brief TODO
 *\param src
 */
void TopologyEntry::setScreenshot(QPixmap scr)
{
    Q_D(TopologyEntry);
    d->screenshot = scr;
}

/*! \fn getScreenshot() const
 *\brief TODO
 */
QPixmap TopologyEntry::getScreenshot() const
{
    Q_D(const TopologyEntry);
    return d->screenshot;
}

/*! \fn getScreenshotModel(const QPixmap &oldPixmap)
 *\brief TODO
 *\param oldPixmap
 */
QPixmap TopologyEntry::getScreenshotModel(const QPixmap &oldPixmap)
{
    if(oldPixmap.isNull()) return oldPixmap;
    QPainter paint;
    QPixmap pixmap = oldPixmap;
    paint.begin(&pixmap);
    paint.save();
    // TODO: Mettre en place l'utilisation de coordonnées relatives
    paint.rotate(-45);
    paint.fillRect(-38,19,80,20 , QColor(Qt::gray));
    paint.setPen(Qt::white);
    paint.setFont(QFont("Arial",10));
    paint.drawText(-23, 20 ,50,17, Qt::AlignLeft| Qt::AlignBottom, "MODEL");
    paint.fillRect(21,216,80,20 , QColor(Qt::gray));
    paint.drawText(-23+ (21+38), 20+ (216-19) ,50,17, Qt::AlignLeft| Qt::AlignBottom, "MODEL");
    paint.restore();
    paint.end();
    return pixmap;
}

/*******************************************************************************
* TopologyGateEntry
*******************************************************************************/
TopologyGateEntry::TopologyGateEntry(Hns::EntityEntryPtr topologyGateEntry, QObject *parent)
    : EntityEntry(new TopologyGateEntryPrivate, topologyGateEntry, parent)
{
}

TopologyGateEntry::TopologyGateEntry(TopologyGateEntryPrivate *d_ptr, QObject *parent)
    : EntityEntry(d_ptr, parent)
{
}

Hns::TopologyGateEntryPtr TopologyGateEntry::topologyGateEntry() const
{
    Q_D(const TopologyGateEntry);
    return Hns::TopologyGateEntryPtr::dynamicCast(d->catalogItem);
}

QString TopologyGateEntry::getNetworkName() const
{
    return QString::fromStdString(topologyGateEntry()->networkName);
}

ValidationResult TopologyGateEntry::validateNetworkName(const QString &value)
{
    Hns::TopologyGateEntryPtr entry = topologyGateEntry();
    if (entry->networkName == value.toStdString())
        return Unmodified;
    entry->networkName = value.toStdString();
    return Valid;
}

Entry *TopologyGateEntry::duplicateEntry(QObject *parentObject) const
{
    Q_D(const TopologyGateEntry);
    if (!parentObject)
        parentObject = parent();
    Hns::EntityEntryPropertiesPtr properties = 0;
    if (d->entryProperties)
        properties = Hns::EntityEntryPropertiesPtr::dynamicCast(d->entryProperties->ice_clone());
    return new TopologyGateEntry(new TopologyGateEntryPrivate(*d, Hns::TopologyGateEntryPtr::dynamicCast(d->catalogItem->ice_clone()), properties), parentObject);
}

/*******************************************************************************
* EntityEntryFactory
*******************************************************************************/
EntityEntry *EntryFactory::createEntityEntry(Hns::EntityEntryPtr entityEntry, QObject *parent)
{
    EntityEntry *entityEntryObj = 0;
    QString entryIceId = QString::fromStdString(entityEntry->ice_id());
    if (_entryObjectHash()->contains(entryIceId)) {
        QMetaObject metaObject = _entryObjectHash()->value(entryIceId);
        QGenericArgument entryArg = Q_ARG(Hns::EntityEntryPtr, entityEntry);
        QGenericArgument parentArg = Q_ARG(QObject *, parent);
        QObject *object = metaObject.newInstance(entryArg, parentArg);
        if (object)
            entityEntryObj = qobject_cast<EntityEntry *>(object);
    }
    if (!entityEntryObj)
        entityEntryObj = new EntityEntry(entityEntry, parent);
    qlonglong entryId = entityEntryObj->getID();
    entityEntryObj->setLocked(PrivateSessionAccessor::instance()->session()->entityEntryCatalog()->isEntryLocked(entryId));
    if (entityEntryObj->isLocked())
        entityEntryObj->setLockType(PrivateSessionAccessor::instance()->session()->entityEntryCatalog()->lockType(entryId));
    return entityEntryObj;
}

} // namespace HnsClient
