/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNSCLIENT_ENTRY_H
#define HNSCLIENT_ENTRY_H

// hynesim hnsclient headers
#include "libhnsclient/common.h"

// Generated hynesim headers
#include "common/folder.h"
#include "common/entityentry.h"
#include "common/wireentry.h"
#include "common/domainentry.h"
#include "common/switchentry.h"
#include "common/hybridnetcardentry.h"
#include "common/topologyentry.h"
#include "common/wifiapentry.h"
#include "common/dynamipsrouterentry.h"
#include "common/entryproperties.h"
#include "common/entityentryproperties.h"
#include "common/wireentryproperties.h"
#include "common/domainentryproperties.h"
#include "common/switchentryproperties.h"
#include "common/proxies_entityinfo.h"
#include "common/proxies_topologymanager.h"
#include "common/topologygateentry.h"
#include "common/catalog.h"

// Qt headers
#include <QObject>
#include <QPixmap>

namespace HnsClient {

class CatalogItemPrivate;
class FolderEntryPrivate;
class EntryPrivate;
class EntityEntryInfoUpdaterPrivate;
class EntityEntryPrivate;
class WireEntryPrivate;
class DomainEntryPrivate;
class SwitchEntryPrivate;
class HybridNetcardEntryPrivate;
class WifiAPEntryPrivate;
class DynamipsRouterEntryPrivate;
class EntityEntry;
class EntityEntryCatalog;
class TopologyEntryPrivate;
class TopologyGateEntryPrivate;

/*******************************************************************************
* CatalogItem
*******************************************************************************/
class CatalogItem : public QObject
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(CatalogItem)
public:
    explicit CatalogItem(CatalogItemPrivate *d, QObject* parent = 0);
    ~CatalogItem();

    qlonglong getID() const;
    QString getShortName() const;
    bool setShortName(const QString &shortName);
    ValidationResult validateShortName(const QString &shortName);
    QString getLongName() const;
    bool setLongName(const QString &longName);
    ValidationResult validateLongName(const QString &longName);
    QString getDescription() const;
    bool setDescription(const QString &description);
    ValidationResult validateDescription(const QString &description);
    virtual QString getCreationDate() const;
    virtual QString getModificationDate() const;
    QString getTagList() const;
    bool setTagList(const QString &tagListString);
    ValidationResult validateTagList(const QString &tagListString);
    ValidationResult validateACLList(const Diamesh::ACLList &aclList);
    Diamesh::ACLList aclList() const;
    qlonglong getParentID();
    void setParentID(qlonglong parentId);
    bool isReadable() const;
    bool isWriteable() const;
    bool isExecutable() const;
    bool isRWX() const;
    SynchronizationStatus getSyncStatus() const;
    void setSyncStatus(SynchronizationStatus status);
    bool isSyncing() const;
    virtual bool isLocked() const = 0;
    virtual bool isSaved() const = 0;
    virtual bool isValid() const = 0;
    virtual bool isNew() const = 0;
    virtual bool isModel() const = 0;
    virtual bool hasLinkedClones() const = 0;
    virtual bool isLinkedClone() const = 0;

    void setSourceCatalogItem(Hns::CatalogItemPtr catalogItem);
    Hns::CatalogItemPtr catalogItem();

protected:
    void _updateACLList(Diamesh::ACLList aclList);

    CatalogItemPrivate * const d_ptr;
};


/*******************************************************************************
* FolderEntry
*******************************************************************************/
class FolderEntry : public CatalogItem
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(FolderEntry)
public:
    explicit FolderEntry(Hns::FolderPtr folderEntry, QObject *parent = 0);

    virtual ~FolderEntry();
    virtual bool isLocked() const;
    virtual bool isSaved() const;
    virtual bool isValid() const;
    virtual bool isNew() const;
    virtual bool isModel() const;
    virtual bool hasLinkedClones() const;
    virtual bool isLinkedClone() const;

    Hns::FolderPtr folderEntry();

    FolderEntry *duplicateFolder(QObject *parentObject = 0);

private:
    FolderEntry(FolderEntryPrivate *d_ptr, QObject *parent = 0);
    friend class EntityEntryCatalog;
};


/*******************************************************************************
* Entry
*******************************************************************************/
class Entry : public CatalogItem
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(Entry)
public:
    explicit Entry(EntryPrivate *d_ptr, Hns::EntryPtr entry, QObject *parent = 0);
    ~Entry();
    virtual bool isLocked() const;
    ::Hns::LockType lockType() const;
    virtual bool isSaved() const;
    virtual bool isValid() const;
    virtual bool isModel() const;
    virtual bool hasLinkedClones() const;
    virtual bool isLinkedClone() const;
    virtual bool isNew() const;
    virtual void setLocked(bool locked);
    void setLockType(::Hns::LockType lockType);
    virtual QString getUUID() const;
    virtual QString getPath() const;
    virtual ValidationResult validatePath(const QString &path);
    Hns::EntryPtr entry() const;
    virtual Entry *duplicateEntry(QObject *parentObject = 0) const = 0;
    void refreshProperties();
    Hns::EntryPropertiesPtr entryProperties() const;

protected:
    Entry(EntryPrivate *d_ptr, QObject *parent = 0);
    virtual void _postGetProperties();

protected slots:
    void _getPropertiesSlot(::Hns::EntryPropertiesPtr properties);
};


/*******************************************************************************
* EntityEntryInfoUpdater
*******************************************************************************/
class EntityEntryInfoUpdater : public QObject
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(EntityEntryInfoUpdater)
public:
    explicit EntityEntryInfoUpdater(EntityEntry *entry, Hns::TopologyManagerQPrx topologyManager, QObject *parent = 0);
    ~EntityEntryInfoUpdater();

signals:
    void statusChanged(qlonglong entryID);

private slots:
    void _getEntityInfoSlot(::Hns::EntityInfoPrx entityInfo);
    void _statusChangedSlot(Ice::Int status, Ice::Long);
    void _getStatusSlot(::Hns::Status status);
    void _requestGetEntityInfoSlot(Ice::Long);

private:
    void _updateEntry(Hns::Status status);

    EntityEntryInfoUpdaterPrivate *const d_ptr;
};


/*******************************************************************************
* EntityEntry
*******************************************************************************/
class EntityEntry : public Entry
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(EntityEntry)
public:
    ~EntityEntry();
    Hns::EntityEntryPtr entityEntry() const;
    int getStatus() const;
    virtual Entry *duplicateEntry(QObject *parentObject = 0) const;
    void setInfoUpdater(EntityEntryInfoUpdater *infoUpdater);
    void setEntityStatus(Hns::Status status);
    Hns::Status getEntityStatus();
    bool isUsedAsEntity();
    void setIsCloning(bool isCloning);
    void setCloneProgress(double value);
    double getCloneProgress() const;
    bool isCloning() const;
    bool isTemplate() const;
    virtual bool isModel() const;
    bool isSaved() const;
    bool hasLinkedClones() const;
    bool isLinkedClone() const;
    bool hasResources() const;
    QString customGraphic() const;
    ValidationResult validateTemplate(bool value);
    Hns::EntityEntryPropertiesPtr entityEntryProperties() const;

protected:
    explicit EntityEntry(Hns::EntityEntryPtr entityEntry, QObject *parent = 0);
    EntityEntry(EntityEntryPrivate *d_ptr, QObject *parent = 0);
    EntityEntry(EntityEntryPrivate *d_ptr, Hns::EntityEntryPtr entityEntry, QObject *parent = 0);

private:
    friend class EntityEntryCatalog;
    friend class EntryFactory;
};


/*******************************************************************************
* WireEntry
*******************************************************************************/
class WireEntry : public EntityEntry
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(WireEntry)
public:
    Q_INVOKABLE explicit WireEntry(Hns::EntityEntryPtr wireEntry, QObject *parent = 0);
    ~WireEntry();
    Hns::WireEntryPtr wireEntry() const;
    virtual int getLossLR() const;
    virtual ValidationResult validateLossLR(int value);
    virtual int getDelayLR() const;
    virtual ValidationResult validateDelayLR(int value);
    virtual int getDupLR() const;
    virtual ValidationResult validateDupLR(int value);
    virtual int getBandwidthLR() const;
    virtual ValidationResult validateBandWidthLR(int value);
    virtual int getMtuLR() const;
    virtual ValidationResult validateMtuLR(int value);
    virtual int getLossRL() const;
    virtual ValidationResult validateLossRL(int value);
    virtual int getDelayRL() const;
    virtual ValidationResult validateDelayRL(int value);
    virtual int getDupRL() const;
    virtual ValidationResult validateDupRL(int value);
    virtual int getBandwidthRL() const;
    virtual ValidationResult validateBandWidthRL(int value);
    virtual int getMtuRL() const;
    virtual ValidationResult validateMtuRL(int value);
    virtual Entry *duplicateEntry(QObject *parentObject = 0) const;
    QString getFromEntity() const;
    void setFromEntity(const QString &fromEntity);
    QString getToEntity() const;
    void setToEntity(const QString &toEntity);
    virtual QString getType() const;
    bool setType(const QString &type);
    virtual ValidationResult validateType(const QString &type);

    bool setLossLR(int value);
    bool setDelayLR(int value);
    bool setDupLR(int value);
    bool setBandwidthLR(int value);
    bool setMtuLR(int value);

    bool setLossRL(int value);
    bool setDelayRL(int value);
    bool setDupRL(int value);
    bool setBandwidthRL(int value);
    bool setMtuRL(int value);

    Hns::WireEntryPropertiesPtr wireEntryProperties() const;
    QMap<QString, QString> supportedImplementation() const;

protected:
    virtual void _postGetProperties();
    WireEntry(WireEntryPrivate *d_ptr, QObject *parent = 0);
};


/*******************************************************************************
* DomainEntry
*******************************************************************************/
class DomainEntry : public EntityEntry
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(DomainEntry)
public:
    Q_INVOKABLE explicit DomainEntry(Hns::EntityEntryPtr domainEntry, QObject *parent = 0);
    Hns::DomainEntryPtr domainEntry() const;
    virtual QString getType() const;
    virtual ValidationResult validateType(const QString &value);
    virtual ValidationResult validateArch(const QString &arch);
    virtual ValidationResult validateMachine(const QString &machine);
    virtual Hns::OperatingSystem getOperatingSystem() const;
    virtual ValidationResult validateOsKernel(const QString &kernel);
    virtual ValidationResult validateOsInitrd(const QString &initrd);
    virtual ValidationResult validateOsCmdLine(const QString &cmdLine);
    virtual int getMemory() const;
    virtual ValidationResult validateMemory(int value);
    virtual int getVCpu() const;
    virtual ValidationResult validateVCpu(int value);
    virtual bool getPae() const;
    virtual ValidationResult validatePae(bool value);
    virtual bool getAcpi() const;
    virtual ValidationResult validateAcpi(bool value);
    virtual bool getApic() const;
    virtual ValidationResult validateApic(bool value);
    virtual bool getImmutable() const;
    virtual ValidationResult validateImmutable(bool value);
    virtual QList<Hns::Disk> getDisks() const;
    virtual ValidationResult validateDisks(QList<Hns::Disk> diskList, QHash<QString, int> &error);
    virtual Hns::NetcardMap getNetcardMap() const;
    virtual ValidationResult validateNetworkCardMap(QMap<Ice::Long, Hns::Netcard> networkCardMap, QHash<Ice::Long, int>&errorHash);
    virtual qlonglong getActionManagerInterfaceID() const;
    virtual ValidationResult validateActionManagerInterfaceID(qlonglong value);
    virtual bool getSharedFolder() const;
    virtual int getUsbRedirectionCount() const;
    virtual ValidationResult validateSharedFolder(bool value);
    virtual Hns::Graphics getGraphics() const;
    virtual ValidationResult validateKeymap(QString keymap);
    virtual Entry *duplicateEntry(QObject *parentObject = 0) const;
    virtual Hns::SnapshotList getSnapshotList() const;
    virtual QString getCurrentSnapshotName() const;
    virtual QString getNodeKey() const;
    virtual ValidationResult validateNodeKey(QString nodeKey);
    virtual QString getBaseUuid() const;
    virtual QStringList getDiffUuids() const;
    virtual ValidationResult validateUsbRedirectionCount(int value);
    QString usbControllerType() const;
    ValidationResult validateUsbControllerType(const QString &controllerType);
    virtual QString getVideoModelType() const;
    virtual ValidationResult validateVideoModelType(QString videoModelType);

signals:
    void callCompleted();

private slots:
    void _isUsedSlot(bool used);

protected:
    DomainEntry(DomainEntryPrivate *d_ptr, QObject *parent = 0);

private:
    bool _isLoopNeedToBeLaunch;
};


/*******************************************************************************
* SwitchEntry
*******************************************************************************/
class SwitchEntry : public EntityEntry
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(SwitchEntry)
public:
    Q_INVOKABLE explicit SwitchEntry(Hns::EntityEntryPtr switchEntry, QObject *parent = 0);
    Hns::SwitchEntryPtr switchEntry() const;
    virtual int getPorts() const;
    virtual ValidationResult validatePorts(int value);
    virtual bool getIsHub() const;
    virtual ValidationResult validateIsHub(bool value);
    virtual Hns::VlanMap getVlan();
    virtual ValidationResult validateVlan(QMap<Ice::Int, Hns::Vlan> vlanMap);
    virtual bool getStpEnabled() const;
    virtual ValidationResult validateStpEnabled(bool value);
    virtual bool stpEnabled();
    virtual int getPortMirroring() const;
    virtual ValidationResult validatePortMirroring(int value);
    virtual Entry *duplicateEntry(QObject *parentObject = 0) const;
    virtual QString getType() const;
    virtual ValidationResult validateType(QString type);

    bool setSwitchType(const QString &value);
    bool setPortCount(int value);
    bool setIsHub(bool value);
    bool setStpEnabled(bool value);
    bool setPortMirroring(int value);

    Hns::SwitchEntryPropertiesPtr switchEntryProperties() const;
    QMap<QString, QString> supportedImplementation() const;

protected:
    virtual void _postGetProperties();
    SwitchEntry(SwitchEntryPrivate *d_ptr, QObject *parent = 0);
};


/*******************************************************************************
* HybridNetcardEntry
*******************************************************************************/
class HybridNetcardEntry : public EntityEntry
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(HybridNetcardEntry)
public:
    Q_INVOKABLE explicit HybridNetcardEntry(Hns::EntityEntryPtr hybridNetcardEntry, QObject *parent = 0);
    Hns::HybridNetcardEntryPtr hybridNetcardEntry() const;
    virtual QString getNodeName() const;
    virtual ValidationResult validateNodeName(const QString &value);
    virtual QString getPlugIndex() const;
    virtual ValidationResult validatePlugIndex(const QString &value);
    virtual Entry *duplicateEntry(QObject *parentObject = 0) const;

protected:
    HybridNetcardEntry(HybridNetcardEntryPrivate *d_ptr, QObject *parent = 0);
};


/*******************************************************************************
* WifiAPEntry
*******************************************************************************/
class WifiAPEntry : public EntityEntry
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(WifiAPEntry)
public:
    Q_INVOKABLE explicit WifiAPEntry(Hns::EntityEntryPtr wifiApEntry, QObject *parent = 0);
    Hns::WifiAPEntryPtr wifiApEntry() const;
    virtual QString getSsid() const;
    virtual ValidationResult validateSsid(const QString &value);
    virtual QString getAuthentificationType() const;
    virtual ValidationResult validateAuthentificationType(const QString &value);
    virtual QString getPassword();
    virtual ValidationResult validatePassword(const QString &value,bool isHexa);
    virtual bool getIsHidden();
    virtual ValidationResult validateHidden(bool value);
    virtual QString getMacAdress();
    virtual ValidationResult validateMacAddress(const QString &value);
    bool checkPassphrase(QString passphrase,QString authTypeWanted,bool isHexa);
    virtual Entry *duplicateEntry(QObject *parentObject = 0) const;
    bool isHex(QString value,int amountCharacters);
    virtual int getChannel() const;
    virtual ValidationResult validateChannel(int value);

protected:
    WifiAPEntry(WifiAPEntryPrivate *d_ptr, QObject *parent = 0);
};


/*******************************************************************************
* DynamipsRouterEntry
*******************************************************************************/
class DynamipsRouterEntry : public EntityEntry
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(DynamipsRouterEntry)
public:
    Q_INVOKABLE explicit DynamipsRouterEntry(Hns::EntityEntryPtr dynamipsRouterEntry, QObject *parent = 0);
    Hns::DynamipsRouterEntryPtr dynamipsRouterEntry() const;
    Entry *duplicateEntry(QObject *parentObject = 0) const;
    ValidationResult validateMacAdress(const QString &value);
    ValidationResult validateConfiguration(const QString &value);
    ValidationResult validateApplyOnBot(bool value);
    ValidationResult validateAutoExtract(bool value);
    ValidationResult validateModules(const ::Hns::ModulePositionMap &positionMap);
    ValidationResult validateChassis(const QString &chassis);
    ValidationResult validatePlatform(const QString &router);
    ValidationResult validateNvRam(int nvram);
    ValidationResult validateNeedSetChassis(int needSetChassis);
    ValidationResult validateMemory(int memory);
    ValidationResult validateImage(const QString &image);
    ValidationResult validateIdle(const QString &idle);
    QString getChassis() const;
    QString getRouter() const;
    QString getMacAdress() const;
    QString getIdlePc() const;
    QString getConfiguration() const;
    bool getAutoExtract() const;
    bool getApplyOnBoot() const;
    Hns::ModulePositionMap getModulesMap() const;
    int getMemory() const;
    QString getImage() const;

signals:
    void callCompleted();

protected:
    DynamipsRouterEntry(DynamipsRouterEntryPrivate *d_ptr, QObject *parent = 0);

private slots:
    void _isUsedSlot(bool used);

private:
    bool _isLoopNeedToBeLaunch;
};


/*******************************************************************************
* TopologyEntry
*******************************************************************************/
class TopologyEntry : public Entry
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(TopologyEntry)
public:
    explicit TopologyEntry(Hns::TopologyEntryPtr topologyEntry, QPixmap screenshot, QObject *parent = 0);
    ~TopologyEntry();
    Hns::TopologyEntryPtr topologyEntry() const;
    int getStatus() const;
    virtual bool isNew() const;
    virtual bool isModel() const;
    bool isPersistent() const;
    virtual Entry *duplicateEntry(QObject *parentObject = 0) const;
    ValidationResult validateIsPersistent(bool value);
    void setScreenshot(QPixmap scr);
    QPixmap getScreenshot() const;
    static QPixmap getScreenshotModel(const QPixmap &oldPixmap);

protected:
    TopologyEntry(TopologyEntryPrivate *d_ptr, QObject *parent = 0);
    friend class TopologyEntryCatalog;
    friend class Topology;

};

/*******************************************************************************
* TopologyGateEntry
*******************************************************************************/
class TopologyGateEntry : public EntityEntry
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(TopologyGateEntry)
public:
    Q_INVOKABLE explicit TopologyGateEntry(Hns::EntityEntryPtr topologyGateEntry, QObject *parent = 0);
    Hns::TopologyGateEntryPtr topologyGateEntry() const;
    virtual QString getNetworkName() const;
    virtual ValidationResult validateNetworkName(const QString &value);
    virtual Entry *duplicateEntry(QObject *parentObject = 0) const;

protected:
    TopologyGateEntry(TopologyGateEntryPrivate *d_ptr, QObject *parent = 0);
};

/*******************************************************************************
* EntityEntryFactory
*******************************************************************************/
class EntryFactory : public QObject
{
    Q_OBJECT
public:
    static EntityEntry *createEntityEntry(Hns::EntityEntryPtr entityEntry, QObject *parent);
};

} // namespace HnsClient
#endif // HNSCLIENT_ENTRY_H
