/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim hnsclient headers
#include "libhnsclient/jobmanager.h"
#include "libhnsclient/private/jobmanager_p.h"
#include "libhnsclient/private/privatesessionaccessor.h"

// ZeroC Ice headers
#include <Ice/ObjectAdapter.h>

namespace HnsClient {

/*******************************************************************************
* JobPrivate
*******************************************************************************/
JobPrivate::JobPrivate()
{
    jobQPrx = 0;
    progress = 0.0f;
}

JobPrivate::~JobPrivate()
{
    jobQPrx = 0;
}


/*******************************************************************************
* JobManagerPrivate
*******************************************************************************/
JobManagerPrivate::JobManagerPrivate()
{
    jobManagerQPrx = 0;
}

JobManagerPrivate::~JobManagerPrivate()
{
    jobManagerQPrx = 0;
}


/*******************************************************************************
* Job
*******************************************************************************/
Job::Job(QObject *parent)
    : QObject(parent)
    , d_ptr(new JobPrivate)
{
}

Job::~Job()
{
    delete d_ptr;
}

/*! \fn getId() const
 *\brief TODO
 */
int Job::getId() const
{
    Q_D(const Job);
    return d->jobId;
}

/*! \fn isRunning() const
 *\brief TODO
 */
bool Job::isRunning() const
{
    Q_D(const Job);
    return d->isRunning;
}

/*! \fn getErrorMessage() const
 *\brief TODO
 */
QString Job::getErrorMessage() const
{
    Q_D(const Job);
    return d->errorMessage;
}

/*! \fn getProgress() const
 *\brief TODO
 */
double Job::getProgress() const
{
    Q_D(const Job);
    return d->progress;
}

void Job::_load(Hns::JobQPrx jobQPrx, int jobId)
{
    Q_D(Job);
    d->jobQPrx = jobQPrx;
    d->jobId = jobId;
    d->remoteCallCount = 2;
    connect(d->jobQPrx.data(), SIGNAL(progressChanged(Ice::Int, Ice::Double)), this, SLOT(_progressChangedSlot(Ice::Int,Ice::Double)));
    connect(d->jobQPrx.data(), SIGNAL(finished(Ice::Int)), this, SLOT(_finished(Ice::Int)));
    connect(d->jobQPrx.data(), SIGNAL(cancelled(Ice::Int)), this, SLOT(_cancelled(Ice::Int)));
    d->jobQPrx->begin_isRunning(this, SLOT(_loadIsRunningSlot(bool)));
    d->jobQPrx->begin_getProgress(this, SLOT(_loadProgressSlot(::Ice::Double)));
}

void Job::_callEnded()
{
    Q_D(Job);
    d->remoteCallCount--;
    _checkLoading();}

void Job::_checkLoading()
{
    Q_D(Job);
    if (d->remoteCallCount == 0)
        _init();
}

void Job::_init()
{
    emit loaded();
}

void Job::_progressChangedSlot(Ice::Int jobId, Ice::Double progress)
{
    Q_D(Job);
    if (progress >= d->progress) {
        d->progress = progress;
        emit progressUpdated(jobId, d->progress);
    }
}

void Job::_finished(Ice::Int jobId)
{
    Q_D(Job);
    d->isRunning = false;
    emit finished(jobId);
}

void Job::_cancelled(Ice::Int jobId)
{
    Q_D(Job);
    d->isRunning = false;
    emit finished(jobId);
}

void Job::_loadIsRunningSlot(bool isRunning)
{
    Q_D(Job);
    d->isRunning = isRunning;
    _callEnded();
}

void Job::_loadProgressSlot(Ice::Double progress)
{
    Q_D(Job);
    if (progress >= d->progress)
        d->progress = progress;
    _callEnded();
}

/*******************************************************************************
* GetJobCookie
*******************************************************************************/
GetJobCookie::GetJobCookie(int jobId, QObject *receiver, const char *slot, QObject *parent)
    : QObject(parent)
    , _jobId(jobId)
    , _job(0)
{
    connect(this, SIGNAL(completed(HnsClient::Job*)), receiver, slot);
}

GetJobCookie::~GetJobCookie()
{
}

void GetJobCookie::setJobQPrx(Hns::JobQPrx qprx)
{
    if (qprx == 0)
        _loadedSlot();
    else {
        _job = new Job(parent());
        connect(_job, SIGNAL(loaded()), this, SLOT(_loadedSlot()));
        _job->_load(qprx, _jobId);
    }
}

void GetJobCookie::_loadedSlot()
{
    emit completed(_job);
    deleteLater();
}

/*******************************************************************************
* JobManager
*******************************************************************************/
JobManager::JobManager(QObject * parent)
    : QObject(parent)
    , d_ptr(new JobManagerPrivate)
{
}

JobManager::~JobManager()
{
    delete d_ptr;
}

/*! \fn getJob(int jobId, QObject *receiver, const char *slot);
 *\brief Create a cookie object with jobId, receiver and slot parameters and give it to begin_getJob.
 *\param jobId jobId
 *\param receiver object connected to internal signal
 *\param slot Return slot connected to signal "completed(HnsClient::Job *)"
 */
void JobManager::getJob(int jobId, QObject *receiver, const char *slot)
{
    Q_D(JobManager);
    GetJobCookie *cookie = new GetJobCookie(jobId, receiver, slot, this);
    d->jobManagerQPrx->begin_getJob(jobId, this, SLOT(_getJobSlot(::Hns::JobPrx, QObject *)), 0, 0, cookie);
}

void JobManager::_load(Hns::JobManagerQPrx jobManagerQPrx)
{
    Q_D(JobManager);
    d->jobManagerQPrx = jobManagerQPrx;
    emit loaded();
}

void JobManager::_getJobSlot(::Hns::JobPrx jobPrx, QObject *obj)
{
    GetJobCookie *cookie = qobject_cast<GetJobCookie *>(obj);
    if (cookie) {
        Hns::JobQPrx qprx = Hns::JobQPrx(jobPrx, PrivateSessionAccessor::instance()->adapter());
        cookie->setJobQPrx(qprx);
    }
}

} // namespace HnsClient
