/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim hnsclient headers
#include "libhnsclient/macaddressmanager.h"
#include "libhnsclient/private/macaddressmanager_p.h"

namespace HnsClient {

/*******************************************************************************
* MacAdressManagerPrivate
*******************************************************************************/
MacAddressManagerPrivate::MacAddressManagerPrivate()
{
    macAdressManagerQPrx = 0;
    networkCardsVendorList = 0;
}

MacAddressManagerPrivate::~MacAddressManagerPrivate()
{
    delete networkCardsVendorList;
    networkCardsVendorList = 0;
    macAdressManagerQPrx = 0;
}


/*******************************************************************************
* GenerateRandomAddressCookie
*******************************************************************************/
GenerateRandomAddressCookie::GenerateRandomAddressCookie(QObject *receiver, const char *slot)
    : QObject()
{
    connect(this, SIGNAL(completed(QString)), receiver, slot);
}

GenerateRandomAddressCookie::~GenerateRandomAddressCookie()
{
}

void GenerateRandomAddressCookie::emitCompleted(QString result)
{
    emit completed(result);
    deleteLater();
}


/*******************************************************************************
* GetVendorFromPrefixCookie
*******************************************************************************/
GetVendorFromPrefixCookie::GetVendorFromPrefixCookie(QObject *receiver, const char *slot)
    : QObject()
{
    connect(this, SIGNAL(completed(QString)), receiver, slot);
}

GetVendorFromPrefixCookie::~GetVendorFromPrefixCookie()
{
}

void GetVendorFromPrefixCookie::emitCompleted(QString result)
{
    emit completed(result);
}


/*******************************************************************************
* IsUsedCookie
*******************************************************************************/
IsUsedCookie::IsUsedCookie(QObject *receiver, const char *slot)
{
    connect(this, SIGNAL(completed(bool)), receiver, slot);
}

IsUsedCookie::~IsUsedCookie()
{
}

void IsUsedCookie::emitCompleted(bool result)
{
    emit completed(result);
}


/*******************************************************************************
* MacAdressManager
*******************************************************************************/
MacAddressManager::MacAddressManager(QObject *parent)
    : QObject(parent)
    , d_ptr(new MacAddressManagerPrivate)
{
}

MacAddressManager::~MacAddressManager()
{
    delete d_ptr;
}

/*! \fn generateRandomAddress(const QString &vendor, QObject *receiver, const char *slot);
 *\brief Create a cookie object with receiver and slot parameters and give it to begin_generateRandomAddress with vendor on parameter.
 *\param vendor vendor name
 *\param receiver object connected to internal signal
 *\param slot Return slot connected to signal "completed(QString result)", where "result" is a new mac address
 */
void MacAddressManager::generateRandomAddress(const QString &vendor, QObject *receiver, const char *slot)
{
    Q_D(MacAddressManager);
    GenerateRandomAddressCookie *cookie = new GenerateRandomAddressCookie(receiver, slot);
    d->macAdressManagerQPrx->begin_generateRandomAddress(vendor.toStdString(), this, SLOT(_generateRandomAddressSlot(::std::string, QObject*)), 0, 0, cookie);
}

/*! \fn getVendorFromPrefix(const QString &macPrefix, QObject *receiver, const char *slot)
 *\brief Create a cookie object with receiver and slot parameters and give it to begin_getVendorFromPrefix with macPrefix on parameter.
 *\param macPrefix prefix mac address of vendor
 *\param receiver object connected to internal signal
 *\param slot Return slot connected to signal "completed(QString result)", where "result" is vendor name
 */
void MacAddressManager::getVendorFromPrefix(const QString &macPrefix, QObject *receiver, const char *slot)
{
    Q_D(MacAddressManager);
    GetVendorFromPrefixCookie *cookie = new GetVendorFromPrefixCookie(receiver, slot);
    d->macAdressManagerQPrx->begin_getVendorFromPrefix(macPrefix.toStdString(), this, SLOT(_getVendorFromPrefixSlot(::std::string,QObject*)), 0, 0, cookie);
}

/*! \fn isUsed(const QString &macAddress, QObject *receiver, const char *slot)
 *\brief Create a cookie object with receiver and slot parameters and give it to begin_isUsed with mac address on parameter.
 *\param macAddress mac address
 *\param receiver object connected to internal signal
 *\param slot Return slot connected to signal "completed(bool result)", where "result" is true if mac address is used
 */
void MacAddressManager::isUsed(const QString &macAddress, QObject *receiver, const char *slot)
{
    Q_D(MacAddressManager);
    IsUsedCookie *cookie = new IsUsedCookie(receiver, slot);
    d->macAdressManagerQPrx->begin_isUsed(macAddress.toStdString(), this, SLOT(_isUsedSlot(bool,QObject*)), 0, 0, cookie);
}

/*! QStringList *MacAddressManager::getVendorList()
 *\brief TODO
 */
QStringList *MacAddressManager::getVendorList()
{
    Q_D(MacAddressManager);
    return d->networkCardsVendorList;
}

void MacAddressManager::_load(Hns::MacAddressManagerQPrx qprx)
{
    Q_D(MacAddressManager);
    d->macAdressManagerQPrx = qprx;
    d->macAdressManagerQPrx->begin_getVendorList(this, SLOT(_loadVendorListSlot(::Diamesh::StringList)));
}

void MacAddressManager::_loadVendorListSlot(Diamesh::StringList vendorList)
{
    Q_D(MacAddressManager);
    d->networkCardsVendorList = new QStringList();
    d->networkCardsVendorList->append(QString());
    foreach(std::string const &vendor, vendorList)
        d->networkCardsVendorList->append(QString::fromStdString(vendor));
    emit loaded();
}

void MacAddressManager::_generateRandomAddressSlot(::std::string address, QObject *obj)
{
    GenerateRandomAddressCookie *cookie = qobject_cast<GenerateRandomAddressCookie *>(obj);
    if (cookie)
        cookie->emitCompleted(QString::fromStdString(address));
}

void MacAddressManager::_getVendorFromPrefixSlot(::std::string vendor, QObject *obj)
{
    GetVendorFromPrefixCookie *cookie = qobject_cast<GetVendorFromPrefixCookie *>(obj);
    if (cookie)
        cookie->emitCompleted(QString::fromStdString(vendor));
}

void MacAddressManager::_isUsedSlot(bool isUsed, QObject *obj)
{
    IsUsedCookie *cookie = qobject_cast<IsUsedCookie *>(obj);
    if (cookie)
        cookie->emitCompleted(isUsed);
}

} // namespace HnsClient
