/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNSCLIENT_PLATFORMMONITOR_H
#define HNSCLIENT_PLATFORMMONITOR_H

// Generated hynesim headers
#include "common/proxies_master.h"
#include "common/proxies_nodeinfo.h"
#include "common/proxies_tracer.h"
#include "common/proxies_nodemonitor.h"

// Qt headers
#include <QObject>
#include <QString>
#include <QMap>
#include <QHash>
#include <QSharedPointer>

namespace HnsClient {

class ServiceDetails;
class NodeInformation;
class NodeMonitorPrivate;
class ServiceDetailsPrivate;
class NodeInformationPrivate;
class MasterInformationPrivate;
class PlatformMonitorPrivate;
class NodeInformationLoader;

typedef QString (ServiceDetails::*ServiceQuery)();
typedef QString (NodeInformation::*NodeInformationQuery)();

/*******************************************************************************
* NodeMonitor
*******************************************************************************/
class NodeMonitor : public QObject
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(NodeMonitor)
public:
    ~NodeMonitor();
    QMap<Ice::Long, Hns::EntityMonitoringDataPtr> getEntityMonitoringDataMap();
    Hns::EntityMonitoringDataPtr getEntityMonitoringData(qlonglong entityId);

signals:
    void entityMonitoringDataUpdated();
    void loaded();

protected slots:
    void _loadEntityMonitorDataMapSlot(::Hns::EntityMonitorDataMap dataMap);
    void _entityDataChangedSlot(Hns::EntityMonitorDataMap dataMap);

protected:
    NodeMonitor(QObject *parent = 0);
    void _load(Hns::NodeMonitorQPrx nodeMonitorQPrx);
    void _callEnded();
    void _checkLoading();
    void _init();

    NodeMonitorPrivate * const d_ptr;
    friend class NodeInformation;
};


/*******************************************************************************
* ServiceDetails
*******************************************************************************/
class ServiceDetails : public QObject
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(ServiceDetails)
public:
    ~ServiceDetails();

    virtual void refresh();

    virtual void refreshSystemInfo() = 0;

    virtual void refreshServiceInfo() = 0;

    Diamesh::VersionInfo getVersionInfo();

    Diamesh::SystemInfo getSystemInfo();

    Diamesh::ServiceInfo getServiceInfo();

    QString getVersionInfoAsString(bool includeDate = false);

    QString getVersionInfoAsString();

    QString getVersion();

    QString getVersionDescription();

    QString getRevision();

    QString getBuildDate();

    QString getBuildTime();

    QString getKernelInformation();

    QStringList getSystemProcessors();

    QString getSystemProcessorsAsString();

    int getSystemUptime() const;

    QString getSystemUptimeAsString();

    double getCpuUsage() const;

    QString getCpuUsageAsString();

    qlonglong getFreeMemory() const;

    QString getFreeMemoryAsString();

    qlonglong getTotalMemory() const;

    QString getTotalMemoryAsString();

    QString getMemoryLoad();

    qlonglong getFreeSwap() const;

    QString getFreeSwapAsString();

    qlonglong getTotalSwap() const;

    QString getTotalSwapAsString();

    QString getSwapLoad();

    QString getDisksStatusAsString();

    QString getName() const;

    QString getShortName();

    QString getDescription();

    QString getServiceHost();

    QString getServiceMaxThread();

    int getServiceUptime() const;

    QString getServiceUptimeAsString();

signals:
    void infoChanged();

protected slots:
    void _remoteMethodFailedSlot(QSharedPointer<Ice::Exception> exception);
    void _getSystemInfoSlot(::Diamesh::SystemInfo systemInformation);
    void _getServiceInfoSlot(::Diamesh::ServiceInfo serviceInformation);

protected:
    explicit ServiceDetails(ServiceDetailsPrivate *d, QObject *parent = 0);
    ServiceDetailsPrivate * const d_ptr;
};


/*******************************************************************************
* NodeInformation
*******************************************************************************/
class NodeInformation : public ServiceDetails
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(NodeInformation)
public:
    ~NodeInformation();
    virtual void refresh();
    QMap<QString, Hns::PlatformInfo> getPlatforms();
    QString getPlatformsAsString();
    HnsNode::ResourceInfo getHnsResourceInformation();
    Hns::Resource getHnsMaxResources();
    Hns::Resource getHnsCurrentResources();
    qlonglong getHnsMaxCpu();
    QString getHnsMaxCpuAsString();
    qlonglong getHnsCurrentCpu();
    QString getHnsCpuUsage();
    qlonglong getHnsMaxMemory();
    QString getHnsMaxMemoryAsString();
    qlonglong getHnsCurrentMemory();
    QString getHnsCurrentMemoryAsString();
    QString getHnsMemoryUsage();
    int getHnsDefinedEntities();
    QString getHnsDefinedEntitiesAsString();
    void refreshSystemInfo();
    void refreshServiceInfo();
    void refreshResourceInfo();
    const QSet<qlonglong> &getEntityIdList() const;

    QString getHnsNetcards();
    NodeMonitor *getNodeMonitor();

signals:
    void addEntity(long);
    void removeEntity(long);
    void loaded();

protected:
    explicit NodeInformation(QObject *parent = 0);
    QString _translatePlatform(QString platform);
    void _updateResourceInfo(HnsNode::ResourceInfo resourceInformation);
    void _load(Hns::NodeInfoQPrx nodeInfoQtPrx);
    void _callEnded();
    void _checkLoading();
    void _init();

protected slots:
    void _getResourceInfoSlot(::HnsNode::ResourceInfo resourceInformation);
    void _resourceInfoChangedSlot(HnsNode::ResourceInfo resourceInformation);
    void _addEntity(long);
    void _removeEntity(long);

    void _loadNameSlot(::std::string name);
    void _loadVersionInfoSlot(::Diamesh::VersionInfo versionInformation);
    void _loadSystemInfoSlot(::Diamesh::SystemInfo systemInformation);
    void _loadServiceInfoSlot(::Diamesh::ServiceInfo serviceInformation);
    void _loadResourceInfoSlot(::HnsNode::ResourceInfo resourceInformation);
    void _loadPlatformsSlot(::Hns::PlatformInfoMap platformInformationMap);
    void _loadEntityDefineIdsSlot(::Diamesh::LongList entitiesIdList);
    void _loadNodeMonitorPrxSlot(::Hns::NodeMonitorPrx nodeMonitorPrx);
    void _loadNodeMonitorSlot();

private:
    friend class NodeInformationLoader;
};


/*******************************************************************************
* MasterInformation
*******************************************************************************/
class MasterInformation : public ServiceDetails
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(MasterInformation)
public:
    ~MasterInformation();
    void refreshSystemInfo();
    void refreshServiceInfo();
    Hns::TracerQPrx getQtTracerProxy();

signals:
    void loaded();

protected:
    explicit MasterInformation(QObject *parent = 0);
    void _load(Hns::MasterQPrx masterQtPrx);
    void _callEnded();
    void _checkLoading();
    void _init();

protected slots:
    void _loadNameSlot(::std::string name);
    void _loadVersionInfoSlot(::Diamesh::VersionInfo versionInformation);
    void _loadSystemInfoSlot(::Diamesh::SystemInfo systemInformation);
    void _loadServiceInfoSlot(::Diamesh::ServiceInfo serviceInformation);
    void _loadTracerSlot(::Hns::TracerPrx tracerPrx);

private:
    friend class MasterInformationLoader;
};


/*******************************************************************************
* PlatformMonitor
*******************************************************************************/
class PlatformMonitor : public QObject
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(PlatformMonitor)
public:
    ~PlatformMonitor();
    QString getMasterName() const;
    MasterInformation *getMasterInformation();
    QHash<QString, NodeInformation *> getNodeInformationHash() const;
    QStringList getNodeNames() const;
    NodeInformation *getNodeInformation(QString nodeName) const;

signals:
    void nodeAdded(QString nodeName);
    void nodeRemoved(QString nodeName);
    void refreshClassName(QString className);
    void loaded();

protected:
    explicit PlatformMonitor(QObject *parent = 0);
    void _load(Hns::MasterQPrx masterQtPrx);
    void _callEnded();
    void _checkLoading();
    void _init();
    NodeInformationLoader *_createNodeInfoLoader();
    void _clearNodeInfoLoader(NodeInformationLoader *nodeLoader);

protected slots:
    void _loadMasterInformationSlot(MasterInformation *masterInformation);
    void _loadNodeInfoMapSlot(::Diamesh::NodeInfoMap nodeInformationMap);
    void _loadNodeInformationSlot(NodeInformation *nodeInformation);
    void _nodeAddedSlot(QString nodeInfoProxyString);
    void _nodeRemovedSlot(const QString &nodeName);
    void _nodeInformationLoaderSlot(NodeInformation *nodeInformation);

private:
    friend class Session;
    PlatformMonitorPrivate * const d_ptr;
};

} // namespace HnsClient

#endif //HNSCLIENT_PLATFORMMONITOR_H
