/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim hnsclient headers
#include "libhnsclient/private/resourcemanager_p.h"
#include "libhnsclient/resourcemanager.h"
#include "libhnsclient/utility.h"

// Qt headers
#include <QFile>
#include <QFileInfo>
#include <QDomDocument>
#include <QDebug>

namespace HnsClient {

GetResourceCookie::GetResourceCookie(const QString &key, QObject *parent)
    : QObject(parent)
    , _key(key)
{
}

GetResourceCookie::~GetResourceCookie()
{
}

QString GetResourceCookie::getKey() const
{
    return _key;
}

ResourceManagerPrivate::ResourceManagerPrivate()
{
    managerQtProxy = 0;
    remoteCallCount = 0;
}

ResourceManagerPrivate::~ResourceManagerPrivate()
{
    managerQtProxy = 0;
}

ResourceManager::ResourceManager(QObject *parent)
    : QObject(parent)
    , d_ptr(new ResourceManagerPrivate)
{
}

ResourceManager::~ResourceManager()
{
    delete d_ptr;
}

/*! \fn QHash<QString, QByteArray> ResourceManager::getResources() const
 *\brief TODO
 */
QHash<QString, QByteArray> ResourceManager::getResources() const
{
    Q_D(const ResourceManager);
    return d->resourceHash;
}

/*! \fn QString ResourceManager::uploadResource(const QString &filePath, bool force)
 *\brief TODO
 */
QString ResourceManager::uploadResource(const QString &filePath, bool force)
{
    QString fileKey;
    Q_D(ResourceManager);
    if (!filePath.isEmpty()) {
        if (d->resourceHash.contains(filePath))
            fileKey = filePath;
        else
            fileKey = _uploadResource(filePath, force);
    }
    return fileKey;
}

/*! \fn QString ResourceManager::uploadResource(const QString &filePath, bool force)
 *\brief TODO
 */
QByteArray ResourceManager::getResource(QString key) const
{
    Q_D(const ResourceManager);
    return d->resourceHash.value(key, QByteArray());
}

void ResourceManager::deleteResource(const QString &key)
{
    Q_D(ResourceManager);
    GetResourceCookie *cookie = new GetResourceCookie(key);
    d->managerQtProxy->begin_deleteResource(key.toStdString(), this, SLOT(_deleteResourceSlot(bool,QObject*)), 0, 0, cookie);
}

void ResourceManager::_load(Hns::CustomResourceManagerQPrx managerQtPrx)
{
    Q_D(ResourceManager);
    d->managerQtProxy = managerQtPrx;
    d->remoteCallCount = 1;
    d->managerQtProxy->begin_getResourceKeys(this, SLOT(_loadGetResourceKeysSlot(::Diamesh::StringList)));
}

void ResourceManager::_callEnded()
{
    Q_D(ResourceManager);
    d->remoteCallCount--;
    _checkLoading();
}

void ResourceManager::_checkLoading()
{
    Q_D(ResourceManager);
    if (d->remoteCallCount == 0)
        _init();
}

void ResourceManager::_init()
{
    Q_D(ResourceManager);
    connect(d->managerQtProxy.data(), SIGNAL(resourceAdded(QString)), this, SLOT(_resourceAddedSlot(QString)));
    connect(d->managerQtProxy.data(), SIGNAL(resourceDeleted(QString)), this, SLOT(_resourceDeletedSlot(QString)));
    emit loaded();
}

QString ResourceManager::_uploadResource(const QString &filePath, bool force)
{
    Q_D(ResourceManager);
    QString fileName;
    QFile resourceFile(filePath);
    if (resourceFile.exists()) {
        QFileInfo fInfo(resourceFile);
        fileName = fInfo.fileName();
        if (!d->resourceHash.contains(fileName) || force) {
            resourceFile.open(QFile::ReadOnly | QFile::Text);
            // Hack for SVGs files
            QDomDocument domDocument;
            domDocument.setContent(&resourceFile);
            QDomElement root = domDocument.documentElement();
            root.setAttribute("width", 128);
            root.setAttribute("height", 128);
            QByteArray fileData = domDocument.toByteArray();
            const char *constData = fileData.constData();
            Hns::ByteArray hnsData(constData, constData + fileData.size());
            d->managerQtProxy->begin_uploadResource(fileName.toStdString(), hnsData);
            d->resourceHash.insert(fileName, fileData);
            resourceFile.close();
        }
    }
    return fileName;
}

void ResourceManager::_handleGetResource(Hns::ByteArray data, QObject *obj)
{
    Q_D(ResourceManager);
    GetResourceCookie *cookie = qobject_cast<GetResourceCookie*>(obj);
    if (cookie) {
        QByteArray resourceByteArray;
        for(Hns::ByteArray::const_iterator it = data.begin(); it != data.end(); ++it)
            resourceByteArray.append(*it);
        d->resourceHash[cookie->getKey()] = resourceByteArray;
    }
}

void ResourceManager::_loadGetResourceKeysSlot(::Diamesh::StringList resourceKeys)
{
    Q_D(ResourceManager);
    QStringList keys = Utility::qStringListFromStringList(resourceKeys);
    d->remoteCallCount += keys.count();
    foreach(const QString &key, keys)
        d->managerQtProxy->begin_getResource(key.toStdString(), this, SLOT(_loadGetResourceSlot(::Hns::ByteArray, QObject *)), this, SLOT(_loadGetResourceFailedSlot(QSharedPointer<Ice::Exception>, QObject *)), new GetResourceCookie(key, this));
    _callEnded();
}

void ResourceManager::_loadGetResourceSlot(::Hns::ByteArray data, QObject *obj)
{
    _handleGetResource(data, obj);
    _callEnded();
}

void ResourceManager::_getResourceSlot(Hns::ByteArray data, QObject *obj)
{
    _handleGetResource(data, obj);
}

void ResourceManager::_loadGetResourceFailedSlot(QSharedPointer<Ice::Exception> exception, QObject *obj)
{
    GetResourceCookie *cookie = qobject_cast<GetResourceCookie*>(obj);
    if (exception && cookie) {
        qWarning() << tr("Failed to load resource %1").arg(cookie->getKey());
        _callEnded();
        delete cookie;
    }
}

void ResourceManager::_getResourceFailedSlot(QSharedPointer<Ice::Exception> exception, QObject *obj)
{
    GetResourceCookie *cookie = qobject_cast<GetResourceCookie*>(obj);
    if (exception && cookie) {
        qWarning() << tr("Failed to load resource %1").arg(cookie->getKey());
        delete cookie;
    }
}

void ResourceManager::_resourceAddedSlot(QString key)
{
    Q_D(ResourceManager);
    d->managerQtProxy->begin_getResource(key.toStdString(), this, SLOT(_getResourceSlot(::Hns::ByteArray, QObject *)), this, SLOT(_getResourceFailedSlot(QSharedPointer<Ice::Exception>, QObject *)), new GetResourceCookie(key, this));
}

void ResourceManager::_resourceDeletedSlot(QString key)
{
    Q_D(ResourceManager);
    d->resourceHash.remove(key);
}

void ResourceManager::_deleteResourceSlot(bool result, QObject *obj)
{
    GetResourceCookie *cookie = qobject_cast<GetResourceCookie*>(obj);
    if (!cookie)
        return;
    if (!result) {
        qCritical() << tr("Unable to delete resource %1").arg(cookie->getKey());
    }
    delete cookie;
}

} // namespace HnsClient
