/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim hnsclient headers
#include "libhnsclient/topology.h"
#include "libhnsclient/private/topology_p.h"
#include "libhnsclient/session.h"
#include "libhnsclient/private/privatesessionaccessor.h"
#include "libhnsclient/topologyentrycatalog.h"
#include "libhnsclient/topologymanager.h"
#include "libhnsclient/entry.h"
#include "libhnsclient/entity.h"
#include "libhnsclient/utility.h"
#include "libhnsclient/tracermanager.h"
#include "libhnsclient/platformmonitor.h"

// hynesim common headers
#include "common/initializer.h"

// Generated hynesim headers
#include "common/proxies_entity.h"
#include "common/proxies_wire.h"
#include "common/wireentry.h"
#include "common/domainentry.h"
#include "common/switchentry.h"
#include "common/hybridnetcardentry.h"
#include "common/wifiapentry.h"
#include "common/dynamipsrouterentry.h"
#include "common/topologygateentry.h"

namespace HnsClient {

HNS_STATIC_HASH_INITIALIZER(_entityObjectHash, EntityObjectHash, QString, QMetaObject)
EntityObjectHash::EntityObjectHash()
{
    insert(QString::fromStdString(Hns::WireEntry::ice_staticId()), WireEntity::staticMetaObject);
    insert(QString::fromStdString(Hns::DomainEntry::ice_staticId()), DomainEntity::staticMetaObject);
    insert(QString::fromStdString(Hns::SwitchEntry::ice_staticId()), SwitchEntity::staticMetaObject);
    insert(QString::fromStdString(Hns::HybridNetcardEntry::ice_staticId()), HybridNetcardEntity::staticMetaObject);
    insert(QString::fromStdString(Hns::WifiAPEntry::ice_staticId()), WifiAPEntity::staticMetaObject);
    insert(QString::fromStdString(Hns::DynamipsRouterEntry::ice_staticId()), DynamipsRouterEntity::staticMetaObject);
    insert(QString::fromStdString(Hns::TopologyGateEntry::ice_staticId()), TopologyGateEntity::staticMetaObject);
}

/*******************************************************************************
* GetScreenshotCookie
*******************************************************************************/
GetScreenshotCookie::GetScreenshotCookie(Hns::TopologyEntryPtr entry)
    : QObject()
    , _entry(entry)
{
}

GetScreenshotCookie::~GetScreenshotCookie()
{
}

Hns::TopologyEntryPtr GetScreenshotCookie::getPtr()
{
    return _entry;
}


/*******************************************************************************
* GetEntityLayoutCookie
*******************************************************************************/
GetEntityLayoutCookie::GetEntityLayoutCookie(Entity *entity)
    : QObject()
    , _entity(entity)
{
}

GetEntityLayoutCookie::~GetEntityLayoutCookie()
{
}

Entity *GetEntityLayoutCookie::entity()
{
    return _entity;
}


/*******************************************************************************
* GetWireLayoutCookie
*******************************************************************************/
GetWireLayoutCookie::GetWireLayoutCookie(WireEntity *entity)
    : QObject()
    , _entity(entity)
{
}

GetWireLayoutCookie::~GetWireLayoutCookie()
{
}

WireEntity *GetWireLayoutCookie::entity()
{
    return _entity;
}


/*******************************************************************************
* RefreshEntryCookie
*******************************************************************************/
RefreshEntryCookie::RefreshEntryCookie(QObject *receiver, const char *slot)
    : QObject()
{
    connect(this, SIGNAL(refreshed(QString)), receiver, slot);
}

RefreshEntryCookie::~RefreshEntryCookie()
{
}

void RefreshEntryCookie::emitRefreshed(QString uuid)
{
    emit refreshed(uuid);
    deleteLater();
}


/*******************************************************************************
* CanBuildWithNodeCookie
*******************************************************************************/
CanBuildWithNodeCookie::CanBuildWithNodeCookie(QString nodeName)
    : QObject()
    , _nodeName(nodeName)
{
}

CanBuildWithNodeCookie::~CanBuildWithNodeCookie()
{
}

QString CanBuildWithNodeCookie::nodeName()
{
    return _nodeName;
}


/*******************************************************************************
* EntityCookie
*******************************************************************************/
EntityCookie::EntityCookie(qlonglong entityId)
    : QObject()
    , _entityId(entityId)
{
}

EntityCookie::~EntityCookie()
{
}

qlonglong EntityCookie::entityId()
{
    return _entityId;
}


/*******************************************************************************
* CloneEntityFromEntryIdCookie
*******************************************************************************/
CloneEntityFromEntryIdCookie::CloneEntityFromEntryIdCookie(qlonglong entryId, Hns::EntityLayoutPtr layout, QObject *receiver, const char *slot)
    : QObject()
    , _entryId(entryId)
    , _layout(layout)
{
    connect(this, SIGNAL(completed(int,qlonglong,int,int)), receiver, slot);
}

CloneEntityFromEntryIdCookie::~CloneEntityFromEntryIdCookie()
{
}

void CloneEntityFromEntryIdCookie::emitCompleted(int jobId)
{
    emit completed(jobId, _entryId, _layout->x, _layout->y);
    deleteLater();
}


/*******************************************************************************
* UpdateTopologyACLCookie
*******************************************************************************/
UpdateTopologyACLCookie::UpdateTopologyACLCookie(Hns::TopologyEntryPtr entry)
    : QObject()
    , _entry(entry)
{
}

UpdateTopologyACLCookie::~UpdateTopologyACLCookie()
{
}

QString UpdateTopologyACLCookie::getEntryName() const
{
    return QString::fromStdString(_entry->shortName);
}

qlonglong UpdateTopologyACLCookie::getID() const
{
    return _entry->id;
}

Hns::TopologyEntryPtr UpdateTopologyACLCookie::topologyEntry()
{
    return _entry;
}


/*******************************************************************************
* PlugEntityCookie
*******************************************************************************/
PlugEntityCookie::PlugEntityCookie(Hns::TopologyLink link, QObject *receiver, const char *slot)
    : QObject()
    , _link(link)
{
    connect(this, SIGNAL(completed(Hns::TopologyLink)), receiver, slot);
}

PlugEntityCookie::~PlugEntityCookie()
{
}

void PlugEntityCookie::emitCompleted(bool result)
{
    if (result)
        emit completed(_link);
    deleteLater();
}

/*******************************************************************************
* TopologyPrivate
*******************************************************************************/
TopologyPrivate::TopologyPrivate()
{
    topologyQtProxy = 0;
    tracerQtProxy = 0;
    topologyId = 0;
    entry = 0;
    isModified = false;
    contentLoaded = false;

    entitiesLoadingCount = 0;
    wiresLoadingCount = 0;
    entityMapper = new QSignalMapper;
    wireMapper = new QSignalMapper;

    loaded = false;
    wireLoaded = false;
    minHeight = 0;
    minWidth = 0;
}

TopologyPrivate::~TopologyPrivate()
{
    qDeleteAll(wireHash);
    qDeleteAll(entityHash);
    delete entry;
    topologyQtProxy = 0;
    tracerQtProxy = 0;
    delete entityMapper;
    delete wireMapper;
}


/*******************************************************************************
* Topology
*******************************************************************************/
Topology::Topology(QObject *parent)
    : QObject(parent)
    , d_ptr(new TopologyPrivate)
{
}

Topology::~Topology()
{
    delete d_ptr;
}

/*! \fn topologyId()
 *\brief TODO
 */
qlonglong Topology::topologyId()
{
    Q_D(Topology);
    return d->topologyId;
}

/*! \fn *topologyEntry()
 *\brief TODO
 */
TopologyEntry *Topology::topologyEntry()
{
    Q_D(Topology);
    return d->entry;
}

/*! \fn isModified()
 *\brief TODO
 */
bool Topology::isModified()
{
    Q_D(Topology);
    return d->isModified;
}

/*! \fn isContentLoaded()
 *\brief TODO
 */
bool Topology::isContentLoaded()
{
    Q_D(Topology);
    return d->contentLoaded;
}

/*! \fn layout()
 *\brief TODO
 */
Hns::TopologyLayout Topology::layout()
{
    Q_D(Topology);
    return d->layout;
}

/*! \fn getUuid()
 *\brief TODO
 */
QString Topology::getUuid()
{
    Q_D(Topology);
    return d->entry->getUUID();
}

/*! \fn connectedUsers() const
 *\brief TODO
 */
QList<qlonglong> Topology::connectedUsers() const
{
    Q_D(const Topology);
    return d->connectedUsers;
}

/*! \fn getMapBehavior() const
 *\brief TODO
 */
const QMap<int, Hns::BalancingBehaviorInfo> &Topology::getMapBehavior() const
{
    Q_D(const Topology);
    return d->balancingBehaviorMap;
}

/*! \fn loadContent()
 *\brief TODO
 */
void Topology::loadContent()
{
    Q_D(Topology);
    d->entityMapper->disconnect();
    d->wireMapper->disconnect();
    connect(d->entityMapper, SIGNAL(mapped(QObject*)), this, SLOT(_loadEntityLoadedSlot(QObject*)));
    connect(d->wireMapper, SIGNAL(mapped(QObject*)), this, SLOT(_loadWireLoadedSlot(QObject*)));
    d->remoteCallCount = 1;
    d->topologyQtProxy->begin_getEntities(this, SLOT(_loadGetEntitiesSlot(::Hns::LongEntityMap)));
    PrivateSessionAccessor::instance()->session()->tracerManager()->registerTopologyTracer(this, d->tracerQtProxy);
}

/*! \fn unloadContent()
 *\brief TODO
 */
void Topology::unloadContent()
{
    Q_D(Topology);
    PrivateSessionAccessor::instance()->session()->tracerManager()->unregisterTopologyTracer(this, d->tracerQtProxy);
    foreach (GenericEntity *entity, getEntities()) {
        if (entity)
            delete entity;
    }
    foreach (GenericEntity *wireEntity, getWires()) {
        if (wireEntity)
            delete wireEntity;
    }
    d->wireHash.clear();
    d->remoteCallCount = 0;
    d->wireLoaded = false;
    d->entityHash.clear();
    d->linksHash.clear();

    d->topologyQtProxy.data()->disconnect(SIGNAL(entityAdded(Ice::Long,Ice::Long)));
    d->topologyQtProxy.data()->disconnect(SIGNAL(entityRemoved(Ice::Long)));
    d->topologyQtProxy.data()->disconnect(SIGNAL(entityLayoutChanged(Hns::EntityLayoutPtr)));
    d->topologyQtProxy.data()->disconnect(SIGNAL(entityPlugged(QString, QString)));
    d->topologyQtProxy.data()->disconnect(SIGNAL(entityUnplugged(QString, QString)));
    d->topologyQtProxy.data()->disconnect(SIGNAL(wireAdded(Ice::Long,Ice::Long)));
    d->topologyQtProxy.data()->disconnect(SIGNAL(wireRemoved(Ice::Long)));
    d->topologyQtProxy.data()->disconnect(SIGNAL(wireLayoutChanged(Hns::WireLayoutPtr)));
}

/*! \fn getEntityById(qlonglong entityId)
 *\brief TODO
 *\param entityId
 */
GenericEntity *Topology::getEntityById(qlonglong entityId)
{
    Q_D(Topology);
    return d->entityHash.value(entityId, 0);
}

/*! \fn getEntities(bool includeWires = false)
 *\brief TODO
 *\param includeWires
 */
QHash<qlonglong, GenericEntity *> Topology::getEntities(bool includeWires)
{
    Q_D(Topology);
    QHash<qlonglong, GenericEntity*> result = d->entityHash;
    if (includeWires)
        result.unite(getWires());
    return result;
}

/*! \fn getWires()
 *\brief TODO
 */
QHash<qlonglong, GenericEntity *> Topology::getWires()
{
    Q_D(Topology);
    return d->wireHash;
}

QMultiHash<qlonglong, Hns::TopologyLink> Topology::getLinks()
{
    Q_D(Topology);
    return d->linksHash;
}

QList<Hns::TopologyLink> Topology::getLinksByWireId(qlonglong wireId)
{
    Q_D(Topology);
    return d->linksHash.values(wireId);
}

/*! \fn setConnectedUsers(Diamesh::LongList)
 *\brief TODO
 *\param
 */
void Topology::setConnectedUsers(Diamesh::LongList list)
{
    Q_D(Topology);
    d->connectedUsers = Utility::qLongListFromLongList(list);
}

/*! \fn addConnectedUser(qlonglong id)
 *\brief TODO
 *\param id
 */
void Topology::addConnectedUser(qlonglong id)
{
    Q_D(Topology);
    d->connectedUsers.append(id);
    emit connectedUserAdded(id);
}

/*! \fn removeConnectedUser(qlonglong id)
 *\brief TODO
 *\param id
 */
void Topology::removeConnectedUser(qlonglong id)
{
    Q_D(Topology);
    if (d->connectedUsers.contains(id)) {
        d->connectedUsers.removeOne(id);
        emit connectedUserRemoved(id);
    }
}

/*! \fn updateEntry(TopologyEntry *entry)
 *\brief TODO
 *\param entry
 */
void Topology::updateEntry(TopologyEntry *entry)
{
    Q_D(Topology);
    if (d->entry->aclList() != entry->aclList()) {
        UpdateTopologyACLCookie *cookie = new UpdateTopologyACLCookie(entry->topologyEntry());
        d->topologyQtProxy->begin_setACLs(entry->aclList(), this, SLOT(_setACLsSlot(bool, QObject*)), 0, 0, cookie);
    } else
        _updateEntry(entry, 0, 0, 0);
}

/*! \fn setSize(int width, int height)
 *\brief TODO
 *\param width
 *\param height
 */
void Topology::setSize(int width, int height)
{
    Q_D(Topology);
    setLayoutInformation(width, height, QString::fromStdString(d->layout.layoutOptions.colorCode));
}

/*! \fn setBackgroundColor(QString colorCode)
 *\brief TODO
 *\param colorCode
 */
void Topology::setBackgroundColor(QString colorCode)
{
    Q_D(Topology);
    setLayoutInformation(d->layout.layoutOptions.width, d->layout.layoutOptions.height, colorCode);
}

/*! \fn setLayoutInformation(int width, int height, QString colorCode)
 *\brief TODO
 *\param width
 *\param height
 *\param colorCode
 */
void Topology::setLayoutInformation(int width, int height, QString colorCode)
{
    Q_D(Topology);

    if (width < d->minWidth)
        width = d->minWidth;
    if (height < d->minHeight)
        height = d->minHeight;

    if (width == d->layout.layoutOptions.width && height == d->layout.layoutOptions.height && colorCode.toStdString() == d->layout.layoutOptions.colorCode)
        return;

    Hns::TopologyLayoutOptions options = d->layout.layoutOptions;
    options.width = width;
    options.height = height;
    options.colorCode = colorCode.toStdString();
    d->topologyQtProxy->begin_setLayoutOptions(options);
}

/*! \fn setMinimalSize(int width, int height)
 *\brief TODO
 *\param width
 *\param height
 */
void Topology::setMinimalSize(int width, int height)
{
    Q_D(Topology);
    d->minHeight = height;
    d->minWidth = width;
}

/*! \fn updateEntityLayout(const Hns::EntityLayoutPtr &layout)
 *\brief TODO
 *\param layout
 */
void Topology::updateEntityLayout(const Hns::EntityLayoutPtr &layout)
{
    Q_D(Topology);
    d->topologyQtProxy->begin_updateEntityLayout(layout);
}

/*! \fn updateWireLayout(const Hns::WireLayoutPtr &layout)
 *\brief TODO
 *\param layout
 */
void Topology::updateWireLayout(const Hns::WireLayoutPtr &layout)
{
    Q_D(Topology);
    d->topologyQtProxy->begin_updateWireLayout(layout);
}

/*! \fn removeEntity(qlonglong entityId)
 *\brief TODO
 *\param entityId
 */
void Topology::removeEntity(qlonglong entityId)
{
    Q_D(Topology);
    EntityCookie *cookie = new EntityCookie(entityId);
    d->topologyQtProxy->begin_removeEntity(entityId, this, SLOT(_removeEntitySlot(bool, QObject *)), 0, 0, cookie);
}

/*! \fn unImplementEntity(qlonglong entityId)
 *\brief TODO
 *\param entityId
 */
void Topology::unImplementEntity(qlonglong entityId)
{
    Q_D(Topology);
    EntityCookie *cookie = new EntityCookie(entityId);
    d->topologyQtProxy->begin_unImplementModelEntity(entityId, this, SLOT(_unImplementModelEntitySlot(bool, QObject*)), 0, 0, cookie);
}

/*! \fn removeWire(qlonglong wireId)
 *\brief TODO
 *\param wireId
 */
void Topology::removeWire(qlonglong wireId)
{
    Q_D(Topology);
    EntityCookie *cookie = new EntityCookie(wireId);
    d->topologyQtProxy->begin_removeEntity(wireId, this, SLOT(_removeWireSlot(bool,QObject*)), 0, 0, cookie);
}

/*! \fn addWire(Hns::WireLayoutPtr layout, QString fromInfo, QString toInfo)
 *\brief TODO
 *\param layout
 *\param fromInfo
 *\param toInfo
 */
void Topology::addWire(Hns::WireLayoutPtr layout, QString fromInfo, QString toInfo)
{
    Q_D(Topology);
    Hns::WireEntryPtr wireEntry = new Hns::WireEntry();
    Hns::TopologyLink fromLink;
    fromLink.entitySide=fromInfo.toStdString();
    fromLink.wireSide="-1:0";
    Hns::TopologyLink toLink;
    toLink.entitySide=toInfo.toStdString();
    toLink.wireSide="-1:1";
    QList<Hns::TopologyLink> links;
    links << fromLink << toLink;
    d->topologyQtProxy->begin_createEntityFromEntry(wireEntry, layout,links.toStdList(), this, SLOT(_addWireSlot(::Hns::EntityTopologyLinkStruct)));
}

/*! \fn createEntity(qlonglong entryId, Hns::EntityLayoutPtr layout)
 *\brief TODO
 *\param entryID
 *\param layout
 */
void Topology::createEntity(qlonglong entryId, Hns::LayoutPtr layout, Hns::TopologyLinkList list)
{
    Q_D(Topology);
    d->topologyQtProxy->begin_createEntityFromEntryId(entryId, layout, list, this, SLOT(_entityPrxCreatedSlot(::Hns::EntityTopologyLinkStruct)));
}

/*! \fn cloneEntity(qlonglong entryId, Hns::EntityLayoutPtr layout, qlonglong modelEntityId, QObject *receiver, const char *slot)
 *\brief TODO
 *\param entryId
 *\param layout
 *\param modelEntityID
 *\param receiver
 *\param slot
 */
void Topology::cloneEntity(qlonglong entryId, Hns::EntityLayoutPtr layout, qlonglong modelEntityId, QObject *receiver, const char *slot)
{
    Q_D(Topology);
    CloneEntityFromEntryIdCookie *cookie = new CloneEntityFromEntryIdCookie(entryId, layout, receiver, slot);
    d->topologyQtProxy->begin_createEntityFromClonedEntryId(entryId, layout, modelEntityId, this, SLOT(_cloneStartedSlot(::Ice::Int, QObject *)), 0, 0, cookie);
}

/*! \fn implementEntity(HnsClient::Entity *modelEntity, qlonglong entryId)
 *\brief TODO
 *\param modelEntity
 *\param entryId
 */
void Topology::implementEntity(Entity *modelEntity, qlonglong entryId)
{
    Q_D(Topology);
    d->topologyQtProxy->begin_implementModelEntity(modelEntity->getId(), entryId, this, SLOT(_entityImplementedSlot(::Hns::EntityPrx)));
}

/*! \fn importTopology(qlonglong topologyId, Hns::EntityLayoutPtr layout, QObject *receiver, const char *slot)
 *\brief TODO
 *\param topologyId
 *\param layout
 *\param receiver
 *\param slot
 */
void Topology::importTopology(qlonglong topologyId, Hns::EntityLayoutPtr layout, QObject *receiver, const char *slot)
{
    Q_D(Topology);
    d->topologyQtProxy->begin_importTopology(topologyId, layout, receiver, slot);
}

/*! \fn computeDefineInfos(QMap<Ice::Long, Hns::DefineInfo> map, Hns::BalancingBehavior behavior, QObject *receiver, const char *slot)
 *\brief TODO
 *\param map
 *\param beahvior
 *\param receiver
 *\param slot
 */
void Topology::computeDefineInfos(QMap<Ice::Long, Hns::DefineInfo> map, Hns::BalancingBehavior behavior, QObject *receiver, const char *slot)
{
    Q_D(Topology);
    d->topologyQtProxy->begin_computeDefineInfos(map.toStdMap(), behavior, receiver, slot);
}

void Topology::plugEntity(qlonglong entityId, int entityPortId, qlonglong wireId, int wirePortId, QObject *receiver, const char *slot)
{
    Q_D(Topology);
    Hns::TopologyLink link;
    QString entitySide = QString("%1:%2").arg(QString::number(entityId)).arg(QString::number(entityPortId));
    QString wireSide = QString("%1:%2").arg(QString::number(wireId)).arg(QString::number(wirePortId));
    link.entitySide = entitySide.toStdString();
    link.wireSide = wireSide.toStdString();
    PlugEntityCookie *cookie = new PlugEntityCookie(link, receiver, slot);
    d->topologyQtProxy->begin_plugEntity(entityId, entityPortId, wireId, wirePortId, this, SLOT(_plugEntitySlot(bool,QObject*)), 0, 0, cookie);
}

void Topology::unplugEntity(qlonglong entityId, int entityPortId, qlonglong wireId, int wirePortId, QObject *receiver, const char *slot)
{
    Q_D(Topology);
    Hns::TopologyLink link;
    QString entitySide = QString("%1:%2").arg(QString::number(entityId)).arg(QString::number(entityPortId));
    QString wireSide = QString("%1:%2").arg(QString::number(wireId)).arg(QString::number(wirePortId));
    link.entitySide = entitySide.toStdString();
    link.wireSide = wireSide.toStdString();
    PlugEntityCookie *cookie = new PlugEntityCookie(link, receiver, slot);
    d->topologyQtProxy->begin_unplugEntity(entityId, entityPortId, wireId, wirePortId, this, SLOT(_plugEntitySlot(bool,QObject*)), 0, 0, cookie);
}

bool Topology::_isLoaded() const
{
    Q_D(const Topology);
    return d->loaded;
}

void Topology::_load(Hns::TopologyQPrx qprx)
{
    Q_D(Topology);
    if (d->topologyQtProxy)
        return;
    d->topologyQtProxy = qprx;
    d->remoteCallCount = 6;
    d->topologyQtProxy->begin_getId(this, SLOT(_loadGetIdSlot(::Ice::Long)));
    d->topologyQtProxy->begin_getLayout(this, SLOT(_loadGetLayoutSlot(::Hns::TopologyLayout)));
    d->topologyQtProxy->begin_getEntry(this, SLOT(_loadGetEntrySlot(::Hns::TopologyEntryPtr)));
    d->topologyQtProxy->begin_getTracer(this, SLOT(_loadGetTracerSlot(::Hns::TracerPrx)));
    d->topologyQtProxy->begin_isModified(this, SLOT(_loadGetIsModifiedSlot(bool)));
    d->topologyQtProxy->begin_getBehaviorMap(this, SLOT(_loadGetBehaviorMapSlot(::Hns::BalancingBehaviorMap)));
}

void Topology::_callEnded(bool loadingContent)
{
    Q_D(Topology);
    d->remoteCallCount--;
    _checkLoading(loadingContent);
}

void Topology::_checkLoading(bool loadingContent)
{
    Q_D(Topology);
    if (d->remoteCallCount == 0)
        _init(loadingContent);
}

void Topology::_init(bool loadingContent)
{
    Q_D(Topology);
    if (loadingContent) {
        if (d->wireLoaded) {
            d->entityMapper->disconnect();
            d->wireMapper->disconnect();

            connect(d->entityMapper, SIGNAL(mapped(QObject*)), this, SLOT(_getEntityLoadedSlot(QObject*)));
            connect(d->wireMapper, SIGNAL(mapped(QObject*)), this, SLOT(_getWireLoadedSlot(QObject*)));
            connect(d->topologyQtProxy.data(), SIGNAL(entityAdded(Ice::Long,Ice::Long)), this, SLOT(_entityAddedSlot(Ice::Long,Ice::Long)));
            connect(d->topologyQtProxy.data(), SIGNAL(entityRemoved(Ice::Long)), this, SLOT(_entityRemovedSlot(Ice::Long)));
            connect(d->topologyQtProxy.data(), SIGNAL(entityLayoutChanged(Hns::EntityLayoutPtr)), this, SLOT(_entityLayoutChangedSlot(Hns::EntityLayoutPtr)));
            connect(d->topologyQtProxy.data(), SIGNAL(entityPlugged(QString, QString)), this, SLOT(_entityPluggedSlot(QString, QString)));
            connect(d->topologyQtProxy.data(), SIGNAL(entityUnplugged(QString, QString)), this, SLOT(_entityUnpluggedSlot(QString,QString)));
            connect(d->topologyQtProxy.data(), SIGNAL(wireAdded(Ice::Long,Ice::Long)), this, SLOT(_wireAddedSlot(Ice::Long,Ice::Long)));
            connect(d->topologyQtProxy.data(), SIGNAL(wireRemoved(Ice::Long)), this, SLOT(_wireRemovedSlot(Ice::Long)));
            connect(d->topologyQtProxy.data(), SIGNAL(wireLayoutChanged(Hns::WireLayoutPtr)), this, SLOT(_wireLayoutChangedSlot(Hns::WireLayoutPtr)));
            PlatformMonitor *monitor = PrivateSessionAccessor::instance()->session()->platformMonitor();
            connect(monitor, SIGNAL(nodeAdded(QString)), this, SLOT(_nodeAddedSlot(QString)));
            connect(monitor, SIGNAL(nodeRemoved(QString)), this, SLOT(_nodeRemovedSlot(QString)));
            connect(monitor, SIGNAL(refreshClassName(QString)), this, SLOT(_refreshClassNameSlot(QString)));
            d->contentLoaded = true;
            emit contentLoaded();
        }
        else {
            d->wireLoaded = true;
            d->remoteCallCount++;
            d->topologyQtProxy->begin_getTopologyLinks(this, SLOT(_loadGetTopologyLinksSlot(::Hns::LongLinkListMap)));
        }
    } else {
        d->loaded = true;
        connect(d->topologyQtProxy.data(), SIGNAL(modified(QString, bool)), this, SLOT(_modifiedSlot(QString,bool)));
        connect(d->topologyQtProxy.data(), SIGNAL(entryModified(QString)), this, SLOT(_entryModifiedSlot(QString)));
        connect(d->topologyQtProxy.data(), SIGNAL(layoutChanged()), this, SLOT(_layoutChangedSlot()));
        connect(d->topologyQtProxy.data(), SIGNAL(layoutOptionsChanged()), this, SLOT(_layoutOptionsChangedSlot()));
        connect(d->topologyQtProxy.data(), SIGNAL(aclsChanged(Diamesh::ACLList)), this, SLOT(_aclsChangedSlot(Diamesh::ACLList)));
        emit loaded();
    }
}

void Topology::_handleEntityProxy(Hns::EntityPrx prx, bool countCalls)
{
    Q_D(Topology);
    Hns::EntityQPrx entityQtProxy(prx, PrivateSessionAccessor::instance()->adapter());
    QString entityIceID = QString::fromStdString(entityQtProxy->getEntry()->ice_id());
    QMetaObject metaObject;

    if (_entityObjectHash()->contains(entityIceID))
        metaObject = _entityObjectHash()->value(entityIceID);
    else {
        metaObject = Entity::staticMetaObject;
        qWarning() << tr("Entity is unrecognized (type:'%1').").arg(entityIceID);
    }
    QGenericArgument parentArg = Q_ARG(QObject *, this);
    QObject *object = metaObject.newInstance(parentArg);
    if (object) {
        Entity *entity = qobject_cast<Entity *>(object);
        connect(entity, SIGNAL(loaded()), d->entityMapper, SLOT(map()));
        d->entityMapper->setMapping(entity, entity);
        if (countCalls)
            d->remoteCallCount++;
        entity->_load(entityQtProxy, this);
    }
    if (countCalls)
        _callEnded();
}

void Topology::_handleTopologyLink(Hns::TopologyLink link)
{
    Q_D(Topology);
    Hns::WireQPrx wireQtProxy(link.wireProxy, PrivateSessionAccessor::instance()->adapter());
    WireEntity *entity = new WireEntity(this);
    connect(entity, SIGNAL(loaded()), d->wireMapper, SLOT(map()));
    d->wireMapper->setMapping(entity, entity);
    entity->_loadWire(wireQtProxy, this);
}

Entity *Topology::_handleEntityData(Entity *entity, Hns::EntityLayoutPtr layout)
{
    Q_D(Topology);
    Entity *result = 0;
    if (entity) {
        result = entity;
        entity->_setLayout(layout);
        d->entityHash.insert(entity->getId(), entity);
        connect(entity, SIGNAL(entryChanged(HnsClient::GenericEntity*)), this, SLOT(_entityEntryChangedSlot(HnsClient::GenericEntity*)));
        connect(entity, SIGNAL(statusChanged(HnsClient::GenericEntity*)), this, SLOT(_entityStatusChangedSlot(HnsClient::GenericEntity*)));
        connect(entity, SIGNAL(refreshed(HnsClient::GenericEntity*)), this, SLOT(_entityRefreshedSlot(HnsClient::GenericEntity*)));
        foreach(qlonglong pendingWireId, d->pendingWires) {
            _handlePendingWires(pendingWireId);
        }
    }
    return result;
}

void Topology::_updateEntry(TopologyEntry *entry, QObject *receiver, const char *slot, QObject *cookie)
{
    _updateEntry(entry->topologyEntry(), receiver, slot, cookie);
}

void Topology::_updateEntry(Hns::TopologyEntryPtr entry, QObject *receiver, const char *slot, QObject *cookie)
{
    Q_D(Topology);
    d->topologyQtProxy->begin_updateEntry(entry, receiver, slot, 0, 0, cookie);
}

void Topology::_refreshEntry(QObject *receiver, const char *slot)
{
    Q_D(Topology);
    RefreshEntryCookie *cookie = new RefreshEntryCookie(receiver, slot);
    d->topologyQtProxy->begin_getEntry(this, SLOT(_refreshEntrySlot(::Hns::TopologyEntryPtr, QObject *)), 0, 0, cookie);
}

QString Topology::_getMetaClassHnv(const QString &metaClassHns)
{
    QString result;
    if (_entityObjectHash()->contains(metaClassHns))
        result = _entityObjectHash()->value(metaClassHns).className();
    return result;
}

void Topology::_loadGetIdSlot(Ice::Long topologyId)
{
    Q_D(Topology);
    d->topologyId = topologyId;
    PrivateSessionAccessor::instance()->session()->topologyManager()->getTopologyUsers(d->topologyId, this, SLOT(_loadGetTopologyUsersSlot(::Diamesh::LongList)));
}

void Topology::_loadGetLayoutSlot(::Hns::TopologyLayout layout)
{
    Q_D(Topology);
    d->layout = layout;
    _callEnded();
}

void Topology::_loadGetEntrySlot(::Hns::TopologyEntryPtr entry)
{
    GetScreenshotCookie *cookie = new GetScreenshotCookie(entry);
    PrivateSessionAccessor::instance()->session()->topologyEntryCatalog()->getEntryScreenshot(entry->id, this, SLOT(_loadGetScreenshotSlot(::Hns::ByteArray,QObject*)), cookie);
}

void Topology::_loadGetScreenshotSlot(::Hns::ByteArray screenshotData, QObject *obj)
{
    Q_D(Topology);
    GetScreenshotCookie *cookie = qobject_cast<GetScreenshotCookie *>(obj);
    if (cookie) {
        QPixmap screenshot = Utility::pixmapFromByteArray(screenshotData);
        d->entry = new TopologyEntry(cookie->getPtr(), screenshot, this);
        d->topologyQtProxy->begin_getACLs(this, SLOT(_loadGetACLsSlot(::Diamesh::ACLList)));
    } else
        _callEnded();
}

void Topology::_loadGetTracerSlot(::Hns::TracerPrx tracerPrx)
{
    Q_D(Topology);
    Hns::TracerQPrx qprx = Hns::TracerQPrx(tracerPrx, PrivateSessionAccessor::instance()->adapter());
    d->tracerQtProxy = qprx;
    _callEnded();
}

void Topology::_loadGetTopologyUsersSlot(Diamesh::LongList list)
{
    setConnectedUsers(list);
    _callEnded();
}

void Topology::_loadGetIsModifiedSlot(bool isModified)
{
    Q_D(Topology);
    d->isModified = isModified;
    _callEnded();
}

void Topology::_loadGetBehaviorMapSlot(Hns::BalancingBehaviorMap balancingBehaviorMap)
{
    Q_D(Topology);
    d->balancingBehaviorMap = QMap<int, Hns::BalancingBehaviorInfo>(balancingBehaviorMap);
    _callEnded();
}

void Topology::_loadGetACLsSlot(::Diamesh::ACLList aclList)
{
    Q_D(Topology);
    d->entry->_updateACLList(aclList);
    _callEnded();
}

void Topology::_loadGetEntitiesSlot(::Hns::LongEntityMap entities)
{
    Q_D(Topology);
    QMap< ::Ice::Long, Hns::EntityPrx> entitiesMap = QMap< ::Ice::Long, Hns::EntityPrx>(entities);
    d->remoteCallCount += entitiesMap.count();
    foreach(const Hns::EntityPrx &prx, entitiesMap) {
        _handleEntityProxy(prx, true);
    }
    _callEnded(true);
}

void Topology::_loadEntityLoadedSlot(QObject *object)
{
    Q_D(Topology);
    Entity *entity = qobject_cast<Entity *>(object);
    if (entity) {
        GetEntityLayoutCookie *cookie = new GetEntityLayoutCookie(entity);
        d->remoteCallCount++;
        d->topologyQtProxy->begin_getEntityLayout(entity->getId(), this, SLOT(_loadGetEntityLayoutSlot(::Hns::EntityLayoutPtr,QObject*)), 0, 0, cookie);
    }
    _callEnded(true);
}

void Topology::_loadGetEntityLayoutSlot(::Hns::EntityLayoutPtr layout, QObject *obj)
{
    GetEntityLayoutCookie *cookie = qobject_cast<GetEntityLayoutCookie *>(obj);
    if (cookie) {
        _handleEntityData(cookie->entity(), layout);
        delete cookie;
    }
    _callEnded(true);
}

void Topology::_loadWireLoadedSlot(QObject *object)
{
    Q_D(Topology);
    WireEntity *entity = qobject_cast<WireEntity *>(object);
    if (entity) {
        GetWireLayoutCookie *cookie = new GetWireLayoutCookie(entity);
        d->remoteCallCount++;
        d->topologyQtProxy->begin_getWireLayout(entity->getId(), this, SLOT(_loadGetWireLayoutSlot(::Hns::WireLayoutPtr,QObject*)), 0, 0, cookie);
    }
    _callEnded(true);
}

void Topology::_loadGetWireLayoutSlot(Hns::WireLayoutPtr layout, QObject *obj)
{
    Q_D(Topology);
    GetWireLayoutCookie *cookie = qobject_cast<GetWireLayoutCookie *>(obj);
    if (cookie) {
        WireEntity *entity = cookie->entity();
        entity->setLayout(layout);
        delete cookie;
        d->wireHash.insert(entity->getId(), entity);
    }
    _callEnded(true);
}

void Topology::_getEntrySlot(Hns::TopologyEntryPtr entry)
{
    Q_D(Topology);
    if (entry) {
        d->entry->setSourceCatalogItem(entry);
        emit topologyChanged();
    }
}

void Topology::_refreshEntrySlot(Hns::TopologyEntryPtr entry, QObject *obj)
{
    RefreshEntryCookie *cookie = qobject_cast<RefreshEntryCookie *>(obj);
    if (cookie) {
        _getEntrySlot(entry);
        cookie->emitRefreshed(getUuid());
    }
}
void Topology::_getLayoutSlot(Hns::TopologyLayout layout)
{
    Q_D(Topology);
    if (d->layout != layout) {
        d->layout = layout;
        emit topologyLayoutChanged();
    }
}

void Topology::_getLayoutOptionsSlot(Hns::TopologyLayoutOptions options)
{
    Q_D(Topology);
    if (d->layout.layoutOptions != options) {
        d->layout.layoutOptions = options;
        emit topologyLayoutOptionsChanged();
    }
}

void Topology::_getEntitySlot(Hns::EntityPrx prx)
{
    _handleEntityProxy(prx);
}

void Topology::_getEntityLoadedSlot(QObject *object)
{
    Q_D(Topology);
    Entity *entity = qobject_cast<Entity *>(object);
    if (entity) {
        GetEntityLayoutCookie *cookie = new GetEntityLayoutCookie(entity);
        d->topologyQtProxy->begin_getEntityLayout(entity->getId(), this, SLOT(_getEntityLayoutSlot(::Hns::EntityLayoutPtr,QObject*)), 0, 0, cookie);
    }
}

void Topology::_getEntityLayoutSlot(::Hns::EntityLayoutPtr layout, QObject *obj)
{
    GetEntityLayoutCookie *cookie = qobject_cast<GetEntityLayoutCookie *>(obj);
    if (cookie) {
        Entity *entity = _handleEntityData(cookie->entity(), layout);
        delete cookie;
        if (entity)
            emit entityCreated(entity);
    }
}

void Topology::_getLinkSlot(Hns::TopologyLink link)
{
    _handleTopologyLink(link);
}

void Topology::_getWireLoadedSlot(QObject *object)
{
    Q_D(Topology);
    WireEntity *entity = qobject_cast<WireEntity *>(object);
    if (entity) {
        GetWireLayoutCookie *cookie = new GetWireLayoutCookie(entity);
        d->topologyQtProxy->begin_getWireLayout(entity->getId(), this, SLOT(_getWireLayoutSlot(::Hns::WireLayoutPtr,QObject*)), 0, 0, cookie);
    }
}

void Topology::_getWireLayoutSlot(::Hns::WireLayoutPtr layout, QObject *obj)
{
    Q_D(Topology);
    GetWireLayoutCookie *cookie = qobject_cast<GetWireLayoutCookie *>(obj);
    if (cookie) {
        WireEntity *entity = cookie->entity();
        entity->setLayout(layout);
        delete cookie;
        d->wireHash.insert(entity->getId(), entity);
        emit wireCreated(entity);
    }
}

void Topology::_canBuildWithNodeSlot(Diamesh::LongList entityDefinableList, QObject *obj)
{
    Q_D(Topology);
    CanBuildWithNodeCookie *cookie = qobject_cast<CanBuildWithNodeCookie *>(obj);
    if (cookie) {
        foreach(Ice::Long entityId, QList<Ice::Long>::fromStdList(entityDefinableList)) {
            GenericEntity *entity = d->entityHash.value(entityId, 0);
            if (entity)
                entity->addNode(cookie->nodeName());
        }
        cookie->deleteLater();
    }
}

void Topology::_entityEntryChangedSlot(GenericEntity *generic)
{
    Q_D(Topology);
    Entity *entity = qobject_cast<Entity *>(generic);
    if (entity) {
        if (d->entityHash.contains(entity->getId())) {
            QHashIterator<qlonglong, Hns::TopologyLink> it(d->linksHash);
            while (it.hasNext()) {
                it.next();
                qlonglong entityId = QString::fromStdString(it.value().entitySide).split(":").first().toLongLong();
                int entityPortId = QString::fromStdString(it.value().entitySide).split(":").last().toInt();
                if (entityId == entity->getId()) {
                    qlonglong wireId = QString::fromStdString(it.value().wireSide).split(":").first().toLongLong();
                    int wirePort = QString::fromStdString(it.value().wireSide).split(":").last().toInt();
                    if (d->wireHash.contains(wireId)) {
                        WireEntry *wireEntry = qobject_cast<WireEntry *>(d->wireHash.value(wireId)->getEntry());
                        if (wirePort == 0)
                            wireEntry->setFromEntity(entity->getShortName() + ":" + QString::number(entityPortId));
                        else
                            wireEntry->setToEntity(entity->getShortName() + ":" + QString::number(entityPortId));
                    }
                }
            }
        }
        emit entityEntryChanged(entity);
    }
}

void Topology::_entityStatusChangedSlot(GenericEntity *generic)
{
    Entity *entity = qobject_cast<Entity *>(generic);
    if (entity)
        emit entityStatusChanged(entity);
}

void Topology::_entityRefreshedSlot(HnsClient::GenericEntity *generic)
{
    Entity *entity = qobject_cast<Entity *>(generic);
    if (entity)
        emit entityRefreshed(entity);
}


void Topology::_removeEntitySlot(bool result, QObject *obj)
{
    EntityCookie *cookie = qobject_cast<EntityCookie *>(obj);
    if (cookie) {
        if (!result)
            qCritical() << tr("Unable to remove entity %1").arg(cookie->entityId());
    }
}

void Topology::_unImplementModelEntitySlot(bool result, QObject *obj)
{
    EntityCookie *cookie = qobject_cast<EntityCookie *>(obj);
    if (cookie) {
        if (!result)
            qCritical() << tr("Unable to unimplement entity %1").arg(cookie->entityId());
    }
}

void Topology::_removeWireSlot(bool result, QObject *obj)
{
    EntityCookie *cookie = qobject_cast<EntityCookie *>(obj);
    if (cookie) {
        if (!result)
            qCritical() << tr("Unable to remove wire %1").arg(cookie->entityId());
    }
}

void Topology::_addWireSlot(Hns::EntityTopologyLinkStruct prx)
{
    if (!prx.e)
        qDebug() << tr("Failed to create wire");
}

void Topology::_setACLsSlot(bool result, QObject *obj)
{
    UpdateTopologyACLCookie *cookie = qobject_cast<UpdateTopologyACLCookie *>(obj);
    if (cookie) {
        if (!result)
            qCritical() << tr("Unable to update '%1' access control policies.").arg(cookie->getEntryName());
        else
            _updateEntry(cookie->topologyEntry(), 0, 0, 0);
        cookie->deleteLater();
    }
}

void Topology::_plugEntitySlot(bool result, QObject *obj)
{
    PlugEntityCookie *cookie = qobject_cast<PlugEntityCookie *>(obj);
    if (cookie)
        cookie->emitCompleted(result);
}

void Topology::_loadGetTopologyLinksSlot(::Hns::LongLinkListMap map)
{
    Q_D(Topology);
    QMap<Ice::Long, Hns::TopologyLinkList> qmap(map);
    QMapIterator<Ice::Long, Hns::TopologyLinkList> iterator(qmap);
    while (iterator.hasNext()) {
        iterator.next();
        QList<Hns::TopologyLink> links = QList<Hns::TopologyLink>::fromStdList(iterator.value());
        foreach (Hns::TopologyLink link, links) {
            d->linksHash.insertMulti(iterator.key(), link);
        }
    }
    QList<qlonglong> keys = d->linksHash.uniqueKeys();
    d->remoteCallCount += keys.size();
    foreach (qlonglong key, keys)
        _handleTopologyLink(d->linksHash.value(key));
    _callEnded(true);
}

void Topology::_handlePendingWires(qlonglong wireId)
{
    Q_D(Topology);
    QList<qlonglong> keys = d->linksHash.uniqueKeys();
    QList<Hns::TopologyLink> links;
    foreach (qlonglong id, keys){
        if(id == wireId){
            links = d->linksHash.values(id);
            break;
        }
    }
    bool isLoadable = true;
    foreach (Hns::TopologyLink link, links) {
        if (!QString::fromStdString(link.entitySide).isEmpty()) {
            qlonglong entityId = QString::fromStdString(link.entitySide).split(':').first().toLongLong();
            if(!d->entityHash.contains(entityId))
                isLoadable = false;
        }
    }
    if(isLoadable){
        _handleTopologyLink(links.first());
        d->pendingWires.removeOne(wireId);
    }
}

void Topology::_getTopologyLinks(::Hns::TopologyLinkList list)
{
    Q_D(Topology);
    //TODO: maybe use a cookie
    QList<Hns::TopologyLink> links = QList<Hns::TopologyLink>::fromStdList(list);
    qlonglong wireId = QString::fromStdString(links.first().wireSide).split(':').first().toLongLong();
    bool isLoadable = true;
    foreach (Hns::TopologyLink link, links) {
        d->linksHash.insertMulti(wireId, link);
        if (!QString::fromStdString(link.entitySide).isEmpty()) {
            qlonglong entityId = QString::fromStdString(link.entitySide).split(':').first().toLongLong();
            if(!d->entityHash.contains(entityId))
                isLoadable = false;
        }
    }
    if(isLoadable)
        _handleTopologyLink(links.first());
    else
        d->pendingWires.append(wireId);
}

void Topology::_modifiedSlot(QString uuid, bool isModified)
{
    Q_UNUSED(uuid)
    Q_D(Topology);
    if (d->isModified != isModified) {
        d->isModified = isModified;
        emit topologyChanged();
    }
}

void Topology::_entryModifiedSlot(QString uuid)
{
    Q_UNUSED(uuid)
    Q_D(Topology);
    d->topologyQtProxy->begin_getEntry(this, SLOT(_getEntrySlot(::Hns::TopologyEntryPtr)));
}

void Topology::_layoutChangedSlot()
{
    Q_D(Topology);
    d->topologyQtProxy->begin_getLayout(this, SLOT(_getLayoutSlot(::Hns::TopologyLayout)));
}

void Topology::_layoutOptionsChangedSlot()
{
    Q_D(Topology);
    d->topologyQtProxy->begin_getLayoutOptions(this, SLOT(_getLayoutOptionsSlot(::Hns::TopologyLayoutOptions)));
}

void Topology::_aclsChangedSlot(Diamesh::ACLList aclList)
{
    Q_D(Topology);
    d->entry->_updateACLList(aclList);
}

void Topology::_entityAddedSlot(Ice::Long topologyId, Ice::Long entityId)
{
    Q_UNUSED(topologyId)
    Q_D(Topology);
    d->topologyQtProxy->begin_getEntity(entityId, this, SLOT(_getEntitySlot(::Hns::EntityPrx)));
}

void Topology::_entityRemovedSlot(Ice::Long entityId)
{
    Q_D(Topology);
    if (d->entityHash.contains(entityId)) {
        GenericEntity *entity = d->entityHash.take(entityId);
        if (entity) {
            delete entity;
            emit entityRemoved(entityId);
        }
    }
}

void Topology::_entityLayoutChangedSlot(Hns::EntityLayoutPtr layout)
{
    Q_D(Topology);
    GenericEntity *generic = d->entityHash.value(layout->id, 0);
    if (generic) {
        Entity *entity = qobject_cast<Entity *>(generic);
        if (entity)
            entity->_setLayout(layout);
    } else
        qCritical() << tr("Entity %1 was not found. Unable to update layout").arg(QString::number(layout->id));
}

void Topology::_entityPluggedSlot(QString wireSide, QString entitySide)
{
    Q_D(Topology);
    qlonglong wireId = wireSide.split(":").first().toLongLong();
    int portId = wireSide.split(":").last().toInt();
    qlonglong entityId = entitySide.split(":").first().toLongLong();
    int entityPortId = entitySide.split(":").last().toInt();
    QList<Hns::TopologyLink> links = d->linksHash.values(wireId);
    foreach (Hns::TopologyLink oldLink, links) {
        if (portId == QString::fromStdString(oldLink.wireSide).split(":").last().toInt()) {
            Hns::TopologyLink newLink = oldLink;
            newLink.entitySide = entitySide.toStdString();
            d->linksHash.remove(wireId, oldLink);
            d->linksHash.insertMulti(wireId, newLink);
        }
    }
    WireEntry *wireEntry = qobject_cast<WireEntry *>(d->wireHash[wireId]->getEntry());
    if (wireEntry) {
        if (portId == 0)
            wireEntry->setFromEntity(d->entityHash[entityId]->getShortName() + ":" + QString::number(entityPortId));
        else
            wireEntry->setToEntity(d->entityHash[entityId]->getShortName() + ":" + QString::number(entityPortId));
    }

    emit entityPlugged(entityId, wireId);
}

void Topology::_entityUnpluggedSlot(QString wireSide, QString entitySide)
{
    Q_D(Topology);
    qlonglong entityId = entitySide.split(":").first().toLongLong();
    qlonglong wireId = wireSide.split(":").first().toLongLong();
    int portId = wireSide.split(":").last().toInt();
    QList<Hns::TopologyLink> links = d->linksHash.values(wireId);
    foreach (Hns::TopologyLink oldLink, links) {
        if (portId == QString::fromStdString(oldLink.wireSide).split(":").last().toInt()) {
            Hns::TopologyLink newLink = oldLink;
            newLink.entitySide = "";
            d->linksHash.remove(wireId, oldLink);
            d->linksHash.insertMulti(wireId, newLink);
        }
    }

    if (d->wireHash.contains(wireId)) {
        WireEntry *wireEntry = qobject_cast<WireEntry *>(d->wireHash[wireId]->getEntry());
        if (wireEntry) {
            if (portId == 0)
                wireEntry->setFromEntity("");
            else
                wireEntry->setToEntity("");
        }
    }

    emit entityUnplugged(entityId, wireId, portId);
}

void Topology::_wireAddedSlot(Ice::Long topologyId, Ice::Long wireId)
{
    Q_UNUSED(topologyId)
    Q_D(Topology);
    d->topologyQtProxy->begin_getTopologyLink(wireId, this, SLOT(_getTopologyLinks(::Hns::TopologyLinkList)));
}

void Topology::_wireRemovedSlot(Ice::Long wireId)
{
    Q_D(Topology);
    if (d->wireHash.contains(wireId)) {
        GenericEntity *wire = d->wireHash.take(wireId);
        d->linksHash.remove(wireId);
        emit wireRemoved(wireId);
        delete wire;
    }
}

void Topology::_wireLayoutChangedSlot(Hns::WireLayoutPtr layout)
{
    Q_D(Topology);
    GenericEntity *generic = d->wireHash.value(layout->id, 0);
    if (generic) {
        WireEntity *wire = qobject_cast<WireEntity *>(generic);
        if (wire)
            wire->setLayout(layout);
    }
}

void Topology::_nodeAddedSlot(QString nodeName)
{
    Q_D(Topology);
    CanBuildWithNodeCookie *cookie = new CanBuildWithNodeCookie(nodeName);
    d->topologyQtProxy->begin_canBuildWithNode(nodeName.toStdString(), this, SLOT(_canBuildWithNodeSlot(::Diamesh::LongList,QObject*)), 0, 0, cookie);
}

void Topology::_nodeRemovedSlot(QString nodeName)
{
    Q_D(Topology);
    foreach(GenericEntity *generic, d->entityHash.values())
        generic->removeNode(nodeName);
}

void Topology::_refreshClassNameSlot(QString className)
{
    Q_D(Topology);
    QString localClassName = _getMetaClassHnv(className);
    foreach(GenericEntity *generic, d->entityHash.values()) {
        if (generic->metaObject()->className() == localClassName)
            generic->refreshNodeList();
    }
}

void Topology::_cloneStartedSlot(Ice::Int cloneJobId, QObject *obj)
{
    CloneEntityFromEntryIdCookie *cookie = qobject_cast<CloneEntityFromEntryIdCookie *>(obj);
    if (cookie) {
        cookie->emitCompleted(cloneJobId);
    }
}

void Topology::_entityPrxCreatedSlot(Hns::EntityTopologyLinkStruct entityPrx)
{
    if (!entityPrx.e)
        qDebug() << tr("Failed to create entity");
}

void Topology::_entityImplementedSlot(Hns::EntityPrx entityPrx)
{
    Q_UNUSED(entityPrx)
}

} // namespace HnsClient
