/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim hnsclient headers
#include "libhnsclient/topologymanager.h"
#include "libhnsclient/private/topologymanager_p.h"
#include "libhnsclient/session.h"
#include "libhnsclient/private/privatesessionaccessor.h"
#include "libhnsclient/topologyentrycatalog.h"
#include "libhnsclient/topology.h"
#include "libhnsclient/entry.h"
#include "libhnsclient/entityinfo.h"

// ZeroC Ice headers
#include <Ice/Communicator.h>

// Qt headers
#include <QDebug>

namespace HnsClient {

/*******************************************************************************
* TopologyCookie
*******************************************************************************/
TopologyCookie::TopologyCookie(QString topologyUUID, QString topologyShortName, qlonglong entryId, QObject *receiver, const char *slot)
    : QObject()
    , _topologyUUID(topologyUUID)
    , _topologyShortName(topologyShortName)
    , _entryId(entryId)
{
    if (receiver)
        connect(this, SIGNAL(completed(QString,QString,bool)), receiver, slot);
}

TopologyCookie::~TopologyCookie()
{
}

void TopologyCookie::emitCompleted(bool result)
{
    emit completed(_topologyUUID, _topologyShortName, result);
    deleteLater();
}

qlonglong TopologyCookie::entryId()
{
    return _entryId;
}

QString TopologyCookie::shortName()
{
    return _topologyShortName;
}


/*******************************************************************************
* LoadTopologyCookie
*******************************************************************************/
LoadTopologyCookie::LoadTopologyCookie(QObject *receiver, const char *slot)
    : QObject()
    , _topology(0)
{
    connect(this, SIGNAL(completed(HnsClient::Topology*)), receiver, slot);
}

LoadTopologyCookie::~LoadTopologyCookie()
{
}

void LoadTopologyCookie::setTopology(Topology *topology)
{
    _topology = topology;
}

Topology *LoadTopologyCookie::topology()
{
    return _topology;
}

void LoadTopologyCookie::postLoading(Topology *topology)
{
    _topology = topology;
    emitCompleted(_topology);
}

void LoadTopologyCookie::emitCompleted(Topology *topology)
{
    emit completed(topology);
    deleteLater();
}

/*******************************************************************************
* SaveTopologyCookie
*******************************************************************************/
SaveTopologyCookie::SaveTopologyCookie(qlonglong topologyId, QString topologyUUID, QObject *receiver, const char *slot)
    : QObject()
    , _topologyId(topologyId)
    , _topologyUUID(topologyUUID)
{
    connect(this, SIGNAL(completed(QString,bool, QString)), receiver, slot);
}

SaveTopologyCookie::~SaveTopologyCookie()
{
}

qlonglong SaveTopologyCookie::topologyId()
{
    return _topologyId;
}

QString SaveTopologyCookie::topologyUUID()
{
    return _topologyUUID;
}

void SaveTopologyCookie::emitCompleted(bool result, QString exceptionMessage)
{
    emit completed(_topologyUUID, result, exceptionMessage);
    deleteLater();
}


/*******************************************************************************
* SaveAsTopologyCookie
*******************************************************************************/
SaveAsTopologyCookie::SaveAsTopologyCookie(HnsClient::Topology *topology, QObject *receiver, const char *slot)
    : QObject()
    , _topology(topology)
{
    if (_topology)
        _topologyUUID = _topology->getUuid();
    connect(this, SIGNAL(completed(QString,QString,bool,QString)), receiver, slot);
}

SaveAsTopologyCookie::~SaveAsTopologyCookie()
{
}

QString SaveAsTopologyCookie::topologyUUID()
{
    return _topologyUUID;
}

void SaveAsTopologyCookie::setResult(bool result, QString exceptionMessage)
{
    if (!result) {
        emit completed(_topologyUUID, _topologyUUID, result, exceptionMessage);
        deleteLater();
    }
}

Topology *SaveAsTopologyCookie::topology()
{
    return _topology;
}

void SaveAsTopologyCookie::finish()
{
    emit completed(_topologyUUID, _newTopologyUUID, true, QString());
    deleteLater();
}

void SaveAsTopologyCookie::entryRefreshedSlot(QString uuid)
{
    _newTopologyUUID = uuid;
    emit updateHash(_topology, uuid, this);
}


/*******************************************************************************
* SwitchTopologyTypeCookie
*******************************************************************************/
SwitchTopologyTypeCookie::SwitchTopologyTypeCookie(QObject *receiver, const char *slot)
    : QObject()
{
    connect(this, SIGNAL(completed(qlonglong)), receiver, slot);
}

void SwitchTopologyTypeCookie::emitCompleted(qlonglong newEntryId)
{
    emit completed(newEntryId);
    deleteLater();
}


/*******************************************************************************
* TopologyManagerPrivate
*******************************************************************************/
TopologyManagerPrivate::TopologyManagerPrivate()
{
    managerQtProxy = 0;
    remoteCallCount = 0;
    entifyInfoMapper = new QSignalMapper();
    topologyMapper = new QSignalMapper();
}

TopologyManagerPrivate::~TopologyManagerPrivate()
{
    qDeleteAll(topologyHash);
    topologyHash.clear();
    loadingProxiesHash.clear();
    loadingCookieHash.clear();
    managerQtProxy = 0;
    delete entityInfoList;
    delete entifyInfoMapper;
    delete topologyMapper;
}


/*******************************************************************************
* TopologyManager
*******************************************************************************/
TopologyManager::TopologyManager(QObject *parent)
    : QObject(parent)
    , d_ptr(new TopologyManagerPrivate)
{
}

TopologyManager::~TopologyManager()
{
    delete d_ptr;
}

/*! \fn *entityInfoList()
 *\brief TODO
 */
EntityInfoList *TopologyManager::entityInfoList()
{
    Q_D(TopologyManager);
    return d->entityInfoList;
}

/*! \fn *topologyByUUID(QString uuid)
 *\brief TODO
 *\param uuid
 */
Topology *TopologyManager::topologyByUUID(QString uuid)
{
    Q_D(TopologyManager);
    return d->topologyHash.value(uuid, 0);
}

/*! \fn *topologyById(qlonglong id)
 *\brief TODO
 *\param id
 */
Topology *TopologyManager::topologyById(qlonglong id)
{
    Q_D(TopologyManager);
    foreach (Topology *topology, d->topologyHash) {
        if (topology->topologyId() == id)
            return topology;
    }
    return 0;
}

/*! \fn *topologyByEntryId(qlonglong entryId)
 *\brief TODO
 *\param entryId
 */
Topology *TopologyManager::topologyByEntryId(qlonglong entryId)
{
    Q_D(TopologyManager);
    foreach (Topology *topology, d->topologyHash) {
        if (topology->topologyEntry()->getID() == entryId)
            return topology;
    }
    return 0;
}

/*! \fn createTopology(int width, int height, QString colorName, bool isModel, QObject *receiver, const char *slot)
 *\brief Create a cookie object with width, height, colorName,receiver and slot on parameters and give it to begin_createTopology.
 *\param width topology's width
 *\param height topology's height
 *\param colorName topology's color
 *\param isModel true if topology is a model topology
 *\param receiver object connected to internal signal
 *\param slot Return slot connected to signal "completed(HnsClient::Topology*)"
 */
void TopologyManager::createTopology(int width, int height, QString colorName, bool isModel, QObject *receiver, const char *slot)
{
    Q_D(TopologyManager);
    LoadTopologyCookie *cookie = new LoadTopologyCookie(receiver, slot);
    Hns::TopologyLayoutOptions options;
    options.width = width;
    options.height = height;
    options.colorCode = colorName.toStdString();
    d->managerQtProxy->begin_createTopology(isModel, options, this, SLOT(_createTopologySlot(::Hns::TopologyPrx, QObject*)), 0, 0, cookie);
}

/*! \fn loadTopology(qlonglong topologyEntryId, QObject *receiver, const char *slot)
 *\brief Create a cookie object with receiver and slot on parameters and give it to begin_loadTopology.
 *\param topologyEntryId
 *\param receiver object connected to internal signal
 *\param slot Return slot connected to signal "completed(HnsClient::Topology*)"
 */
void TopologyManager::loadTopology(qlonglong topologyEntryId, QObject *receiver, const char *slot)
{
    LoadTopologyCookie *cookie = new LoadTopologyCookie(receiver, slot);
    Topology *topology = topologyByEntryId(topologyEntryId);
    if (topology)
        cookie->setTopology(topology);
    _loadTopology(topologyEntryId, cookie);
}

/*! \fn loadTopologyByUUID(QString topologyUUID, QObject *receiver, const char *slot)
 *\brief Call loadTopology(qlonglong topologyEntryId, QObject *receiver, const char *slot).
 *\param topologyEntryUUID
 *\param receiver object connected to internal signal
 *\param slot Return slot connected to signal "completed(HnsClient::Topology*)"
 */
void TopologyManager::loadTopologyByUUID(QString topologyUUID, QObject *receiver, const char *slot)
{
    LoadTopologyCookie *cookie = new LoadTopologyCookie(receiver, slot);
    Topology *topology = topologyByUUID(topologyUUID);
    if (topology)
        cookie->setTopology(topology);
    TopologyEntry *entry = PrivateSessionAccessor::instance()->session()->topologyEntryCatalog()->topologyEntryByUUID(topologyUUID);
    if (entry)
        _loadTopology(entry->getID(), cookie);
    else
        cookie->emitCompleted(0);
}

/*! \fn unloadTopology(Topology *topology, QObject *receiver, const char *slot)
 *\brief Create a cookie object and give it to begin_unloadTopology.
 *\param topology
 *\param receiver object connected to internal signal
 *\param slot Return slot connected to signal "completed(QString topologyUUID, QString topologyShortName, bool result)"
 */
void TopologyManager::unloadTopology(Topology *topology, QObject *receiver, const char *slot)
{
    Q_D(TopologyManager);
    if (!topology)
        return;
    TopologyEntry *entry = topology->topologyEntry();
    qlonglong entryId = entry->getID();
    TopologyCookie *cookie = new TopologyCookie(topology->getUuid(), entry->getShortName(), entryId, receiver, slot);
    PrivateSessionAccessor::instance()->session()->topologyEntryCatalog()->_setTopologySyncing(entryId, Syncing);
    d->managerQtProxy->begin_unloadTopology(topology->topologyId(), this, SLOT(_unloadTopologySlot(bool, QObject*)), 0, 0, cookie);
}

/*! \fn unloadTopologyByUUID(QString uuid, QObject *receiver, const char *slot)
 *\brief Call unloadTopology(Topology *topology, QObject *receiver, const char *slot)
 *\param uuid topology's uuid
 *\param receiver object connected to internal signal
 *\param slot Return slot connected to signal "completed(QString topologyUUID, QString topologyShortName, bool result)"
 */
void TopologyManager::unloadTopologyByUUID(QString uuid, QObject *receiver, const char *slot)
{
    Topology *topology = topologyByUUID(uuid);
    if (!topology)
        qDebug() << tr("Topology is not in cache. Could not request unload.");
    else
        unloadTopology(topology, receiver, slot);
}

/*! \fn deleteTopology(Topology *topology, QObject *receiver, const char *slot)
 *\brief Create a cookie object and give it to begin_deleteTopology.
 *\param topology
 *\param receiver object connected to internal signal
 *\param slot Return slot connected to signal "completed(QString topologyUUID, QString topologyShortName, bool result)"
 */
void TopologyManager::deleteTopology(Topology *topology, QObject *receiver, const char *slot)
{
    TopologyEntry *entry = 0;
    if (topology)
        entry = topology->topologyEntry();
    deleteTopologyByEntry(entry, receiver, slot);
}

/*! \fn deleteTopologyByUUID(QString uuid, QObject *receiver, const char *slot)
 *\brief Call deleteTopology(Topology *topology, QObject *receiver, const char *slot)
 *\param uuid topology's uuid
 *\param receiver object connected to internal signal
 *\param slot Return slot connected to signal "completed(QString topologyUUID, QString topologyShortName, bool result)"
 */
void TopologyManager::deleteTopologyByUUID(QString uuid, QObject *receiver, const char *slot)
{
    deleteTopologyByEntry(PrivateSessionAccessor::instance()->session()->topologyEntryCatalog()->topologyEntryByUUID(uuid), receiver, slot);
}

/*! \fn deleteTopologyByEntry(TopologyEntry *entry, QObject *receiver, const char *slot)
 *\brief TODO
 *\param entry
 *\param receiver
 *\param slot
 */
void TopologyManager::deleteTopologyByEntry(TopologyEntry *entry, QObject *receiver, const char *slot)
{
    Q_D(TopologyManager);
    if (!entry) {
        qDebug() << tr("Topology is not in cache. Could not delete.");
        return;
    }
    qlonglong entryId = entry->getID();
    TopologyCookie *cookie = new TopologyCookie(entry->getUUID(), entry->getShortName(), entryId, receiver, slot);
    PrivateSessionAccessor::instance()->session()->topologyEntryCatalog()->_setTopologySyncing(entryId, Syncing);
    d->managerQtProxy->begin_deleteTopology(entryId, this, SLOT(_deleteTopologySlot(bool, QObject*)), 0, 0, cookie);
}

/*! \fn deleteTopologyByEntryId(qlonglong entryId, QObject *receiver, const char *slot)
 *\brief TODO
 *\param entryId
 *\param receiver
 *\param slot
 */
void TopologyManager::deleteTopologyByEntryId(qlonglong entryId, QObject *receiver, const char *slot)
{
    deleteTopologyByEntry(PrivateSessionAccessor::instance()->session()->topologyEntryCatalog()->topologyEntryByID(entryId), receiver, slot);
}

/*! \fn switchTopologyType(QString topologyUUID, QString backgroundColorName, QObject *receiver, const char *slot)
 *\brief TODO
 *\param topologyUUID
 *\param backgroundColorName
 *\param receiver
 *\param slot
 */
void TopologyManager::switchTopologyType(QString topologyUUID, QString backgroundColorName, QObject *receiver, const char *slot)
{
    Q_D(TopologyManager);
    Topology *topology = topologyByUUID(topologyUUID);
    if (!topology) {
        qDebug() << tr("Topology is not in cache. Could not switch type.");
        return;
    }
    SwitchTopologyTypeCookie *cookie = new SwitchTopologyTypeCookie(receiver, slot);
    d->managerQtProxy->begin_switchTopologyType(topology->topologyId(), backgroundColorName.toStdString(), this, SLOT(_switchTopologyTypeSlot(::Ice::Long, QObject *)), 0, 0, cookie);
}

/*! \fn closeTopology(Topology *topology)
 *\brief TODO
 *\param topology
 */
void TopologyManager::closeTopology(Topology *topology)
{
    Q_D(TopologyManager);
    if (!topology)
        return;
    d->managerQtProxy->begin_closeTopology(topology->topologyId());
}

/*! \fn closeTopologyByUUID(QString uuid)
 *\brief TODO
 *\param uuid
 */
void TopologyManager::closeTopologyByUUID(QString uuid)
{
    closeTopology(topologyByUUID(uuid));
}

/*! \fn getTopologyUsers(qlonglong topologyId, QObject *receiver, const char *slot)
 *\brief TODO
 *\param topologyId
 *\param receiver
 *\param slot
 */
void TopologyManager::getTopologyUsers(qlonglong topologyId, QObject *receiver, const char *slot)
{
    Q_D(TopologyManager);
    d->managerQtProxy->begin_getTopologyUsers(topologyId, receiver, slot);
}

/*! \fn saveTopology(Topology *topology, TopologyEntry *updatedEntry, QObject *receiver, const char *slot, Hns::ByteArray screenshotData = Hns::ByteArray())
 *\brief TODO
 *\param topology
 *\param updatedEntry
 *\param receiver
 *\param slot
 *\param screenshotData
 */
void TopologyManager::saveTopology(Topology *topology, TopologyEntry *updatedEntry, QObject *receiver, const char *slot, Hns::ByteArray screenshotData)
{
    Q_D(TopologyManager);
    SaveTopologyCookie *cookie = new SaveTopologyCookie(topology->topologyId(), topology->getUuid(), receiver, slot);
    if (!screenshotData.empty())
        d->screenshotCookieHash.insert(cookie, screenshotData);
    _saveTopology(cookie, updatedEntry);
}

/*! \fn saveTopologyAs(Topology *topology, TopologyEntry *updatedEntry, QObject *receiver, const char *slot, Hns::ByteArray screenshotData = Hns::ByteArray())
 *\brief TODO
 *\param topology
 *\param UpdatedEntry
 *\param receiver
 *\param slot
 *\param screenshotData
 */
void TopologyManager::saveTopologyAs(Topology *topology, TopologyEntry *updatedEntry, QObject *receiver, const char *slot, Hns::ByteArray screenshotData)
{
    Q_D(TopologyManager);
    SaveAsTopologyCookie *cookie = new SaveAsTopologyCookie(topology, receiver, slot);
    if (!screenshotData.empty())
        d->screenshotCookieHash.insert(cookie, screenshotData);
    connect(cookie, SIGNAL(updateHash(HnsClient::Topology*,QString, SaveAsTopologyCookie*)), this, SLOT(_updateHashSlot(HnsClient::Topology*,QString, SaveAsTopologyCookie *)));
    d->managerQtProxy->begin_saveTopologyAs(topology->topologyId(), updatedEntry->topologyEntry(), this, SLOT(_saveTopologyAsSlot(bool, QObject*)), this, SLOT(_saveTopologyAsFailedSlot(QSharedPointer<Ice::Exception>,QObject*)), cookie);
    delete updatedEntry;
}

void TopologyManager::_load(Hns::TopologyManagerQPrx qprx)
{
    Q_D(TopologyManager);
    d->managerQtProxy = qprx;
    d->remoteCallCount = 2;
    d->entityInfoList = new EntityInfoList(this);
    d->managerQtProxy->begin_getListEntityInfo(this, SLOT(_loadGetEntityListInfoSlot(::Hns::EntityInfoMap)));
    d->managerQtProxy->begin_getTopologies(this, SLOT(_loadGetTopologiesSlot(::Diamesh::LongList)));
}

void TopologyManager::_callEnded()
{
    Q_D(TopologyManager);
    d->remoteCallCount--;
    _checkLoading();
}

void TopologyManager::_checkLoading()
{
    Q_D(TopologyManager);
    if (d->remoteCallCount == 0)
        _init();
}

void TopologyManager::_init()
{
    Q_D(TopologyManager);
    d->entifyInfoMapper->disconnect();
    d->topologyMapper->disconnect();
    connect(d->entifyInfoMapper, SIGNAL(mapped(QObject*)), this, SLOT(_entityInfoLoadedSlot(QObject*)));
    connect(d->topologyMapper, SIGNAL(mapped(QObject*)), this, SLOT(_topologyObjectLoadedSlot(QObject*)));
    connect(d->managerQtProxy.data(), SIGNAL(addEntityInfo(Ice::Long, QString)) , this, SLOT(_addEntityInfoSlot(Ice::Long, QString)));
    connect(d->managerQtProxy.data(), SIGNAL(removeEntityInfo(Ice::Long)) , this, SLOT(_removeEntityInfoSlot(Ice::Long)));
    connect(d->managerQtProxy.data(), SIGNAL(topologyLoaded(QString, Ice::Long)), this, SLOT(_topologyLoadedSlot(QString, Ice::Long)));
    connect(d->managerQtProxy.data(), SIGNAL(topologyUnloaded(QString, Ice::Long, bool)), this, SLOT(_topologyUnloadedSlot(QString, Ice::Long, bool)));
    connect(d->managerQtProxy.data(), SIGNAL(topologyModified(QString, bool)), this, SLOT(_topologyModifiedSlot(QString,bool)));
    connect(d->managerQtProxy.data(), SIGNAL(userOpenedTopology(Ice::Long, Ice::Long)), this, SLOT(_userOpenedTopologySlot(Ice::Long,Ice::Long)));
    connect(d->managerQtProxy.data(), SIGNAL(userClosedTopology(Ice::Long, Ice::Long)), this, SLOT(_userClosedTopologySlot(Ice::Long,Ice::Long)));
    connect(d->managerQtProxy.data(), SIGNAL(topologySavedAs(QString, QString)), this, SLOT(_topologySavedAsSlot(QString, QString)));
    emit loaded();
}

void TopologyManager::_handleEntityInfo(QObject *object)
{
    Q_D(TopologyManager);
    EntityInfo *info = qobject_cast<EntityInfo *>(object);
    if (info)
        d->entityInfoList->_addEntityInfo(info);
}

void TopologyManager::_handleTopologyPrx(Hns::TopologyPrx topologyPrx, LoadTopologyCookie *cookie)
{
    Q_D(TopologyManager);
    Topology *topology = new Topology(this);
    d->loadingProxiesHash.insert(topologyPrx, topology);
    if (cookie)
        d->loadingCookieHash.insert(topology, cookie);
    d->topologyMapper->setMapping(topology, topology);
    connect(topology, SIGNAL(loaded()), d->topologyMapper, SLOT(map()));
    topology->_load(Hns::TopologyQPrx(topologyPrx, PrivateSessionAccessor::instance()->adapter()));
}

Topology *TopologyManager::_handleTopology(QObject *object)
{
    Topology *topology = qobject_cast<Topology *>(object);
    return _handleTopology(topology);
}

Topology *TopologyManager::_handleTopology(Topology *topology)
{
    Q_D(TopologyManager);
    Topology *result = 0;
    if (topology) {
        Hns::TopologyPrx prx = d->loadingProxiesHash.key(topology, 0);
        if (prx)
            d->loadingProxiesHash.remove(prx);
        QString uuid = topology->getUuid();
        result = d->topologyHash.value(uuid, 0);
        if (result) {
            delete topology;
        } else {
            result = topology;
            d->topologyHash.insert(uuid, result);
            emit topologyAdded(result);
        }
    }
    return result;
}

void TopologyManager::_saveTopology(SaveTopologyCookie *cookie, TopologyEntry *updatedEntry)
{
    Q_D(TopologyManager);
    if (updatedEntry)
        d->managerQtProxy->begin_saveNewTopology(cookie->topologyId(), updatedEntry->topologyEntry(), this, SLOT(_saveTopologySlot(bool,QObject*)), this, SLOT(_savetopologyFailedSlot(QSharedPointer<Ice::Exception>,QObject*)), cookie);
    else
        d->managerQtProxy->begin_saveTopology(cookie->topologyId(), this, SLOT(_saveTopologySlot(bool,QObject*)), this, SLOT(_savetopologyFailedSlot(QSharedPointer<Ice::Exception>,QObject*)), cookie);
}

void TopologyManager::_loadTopology(qlonglong entryId, LoadTopologyCookie *cookie)
{
    Q_D(TopologyManager);
    d->managerQtProxy->begin_loadTopology(entryId, this, SLOT(_loadTopologySlot(::Hns::TopologyPrx, QObject*)), 0, 0, cookie);
}

void TopologyManager::_handleScreenshotDataCookie(QObject *obj, qlonglong entryId, bool result)
{
    Q_D(TopologyManager);
    Hns::ByteArray data = _getScreenshotDataCookie(obj);
    if (result) {
        if (!data.empty()) {
            TopologyEntryCatalog *catalog = Session::instance()->topologyEntryCatalog();
            TopologyEntry *catalogEntry = catalog->topologyEntryByID(entryId);
            if (catalogEntry)
                catalog->setScreenshot(entryId, data);
            else
                d->screenshotEntryHash.insert(entryId, data);
        }
    }
}

Hns::ByteArray TopologyManager::_getScreenshotDataCookie(QObject *obj)
{
    Q_D(TopologyManager);
    return d->screenshotCookieHash.take(obj);
}

void TopologyManager::_loadGetEntityListInfoSlot(Hns::EntityInfoMap entityInfoMap)
{
    Q_D(TopologyManager);
    QMap<Ice::Long, Hns::EntityInfoPrx> prxMap(entityInfoMap);
    QMapIterator<Ice::Long, Hns::EntityInfoPrx> iterator(prxMap);
    d->remoteCallCount += prxMap.count();
    connect(d->entifyInfoMapper, SIGNAL(mapped(QObject*)), this, SLOT(_loadEntityInfoLoadedSlot(QObject*)));
    while(iterator.hasNext()) {
        iterator.next();
        EntityInfo *info = new EntityInfo(this);
        d->entifyInfoMapper->setMapping(info, info);
        connect(info, SIGNAL(loaded()), d->entifyInfoMapper, SLOT(map()));
        info->_load(iterator.key(), Hns::EntityInfoQPrx(iterator.value(), PrivateSessionAccessor::instance()->adapter()));
    }
    _callEnded();
}

void TopologyManager::_loadEntityInfoLoadedSlot(QObject *object)
{
    _handleEntityInfo(object);
    _callEnded();
}

void TopologyManager::_loadGetTopologiesSlot(Diamesh::LongList topologyIdList)
{
    Q_D(TopologyManager);
    QList<Ice::Long> topologies = QList<Ice::Long>::fromStdList(topologyIdList);
    d->remoteCallCount += topologies.count();
    connect(d->topologyMapper, SIGNAL(mapped(QObject*)), this, SLOT(_loadTopologyObjectLoadedSlot(QObject*)));
    foreach(Ice::Long topologyId, topologies) {
        d->managerQtProxy->begin_getTopologyById(topologyId, this, SLOT(_loadGetTopologyByIdSlot(::Hns::TopologyPrx)));
    }
    _callEnded();
}

void TopologyManager::_loadGetTopologyByIdSlot(Hns::TopologyPrx topologyPrx)
{
    Q_D(TopologyManager);
    if (topologyPrx) {
        d->remoteCallCount++;
        _handleTopologyPrx(topologyPrx);
    }
    _callEnded();
}

void TopologyManager::_loadTopologyObjectLoadedSlot(QObject *object)
{
    _handleTopology(object);
    _callEnded();
}

void TopologyManager::_addEntityInfoSlot(Ice::Long entityId, QString entityInfoPrxStr)
{
    Q_D(TopologyManager);
    if (!d->entityInfoList->hasEntityInfo(entityId)) {
        const Ice::CommunicatorPtr &communicator = PrivateSessionAccessor::instance()->communicator();
        Hns::EntityInfoPrx prx = Hns::EntityInfoPrx::uncheckedCast(communicator->stringToProxy(entityInfoPrxStr.toStdString()));
        EntityInfo *info = new EntityInfo(this);
        d->entifyInfoMapper->setMapping(info, info);
        connect(info, SIGNAL(loaded()), d->entifyInfoMapper, SLOT(map()));
        info->_load(entityId, Hns::EntityInfoQPrx(prx, PrivateSessionAccessor::instance()->adapter()));
    }
}

void TopologyManager::_removeEntityInfoSlot(Ice::Long entityId)
{
    Q_D(TopologyManager);
    d->entityInfoList->_removeEntityInfo(entityId);
}

void TopologyManager::_topologyLoadedSlot(QString topologyUUID, Ice::Long topologyId)
{
    Q_D(TopologyManager);
    if (!d->topologyHash.contains(topologyUUID))
        d->managerQtProxy->begin_getTopologyById(topologyId, this, SLOT(_getTopologyByIdSlot(::Hns::TopologyPrx)));
}

void TopologyManager::_topologyUnloadedSlot(QString topologyUUID, Ice::Long topologyId, bool isDeleted)
{
    Q_UNUSED(topologyId)
    Q_D(TopologyManager);
    emit topologyUnloaded(topologyUUID, isDeleted);
    Topology *topology = d->topologyHash.take(topologyUUID);
    if (topology) {
        delete topology;
    }
}

void TopologyManager::_topologyModifiedSlot(QString topologyUUID, bool isModified)
{
    emit topologyModified(topologyUUID, isModified);
}

void TopologyManager::_topologySavedAsSlot(QString oldUUID, QString newUUID)
{
    Q_D(TopologyManager);
    Topology *topology = d->topologyHash.take(oldUUID);
    if (topology) {
        d->topologyHash.insert(newUUID, topology);
        emit topologyAdded(topology);
    }
    emit topologySavedAs(oldUUID, newUUID);
}

void TopologyManager::_userOpenedTopologySlot(Ice::Long userId, Ice::Long topologyId)
{
    Topology *topology = topologyById(topologyId);
    if (!topology)
        return;
    topology->addConnectedUser(userId);
}

void TopologyManager::_userClosedTopologySlot(Ice::Long userId, Ice::Long topologyId)
{
    Topology *topology = topologyById(topologyId);
    if (!topology)
        return;
    topology->removeConnectedUser(userId);
}

void TopologyManager::_getTopologyByIdSlot(Hns::TopologyPrx topologyPrx)
{
    Q_D(TopologyManager);
    if (d->loadingProxiesHash.contains(topologyPrx))
        qDebug() << tr("Topology is already being loaded. Please wait...");
    else {
        _handleTopologyPrx(topologyPrx);
    }
}

void TopologyManager::_createTopologySlot(Hns::TopologyPrx topologyPrx, QObject *obj)
{
    Q_D(TopologyManager);
    LoadTopologyCookie *cookie = qobject_cast<LoadTopologyCookie *>(obj);
    if (cookie) {
        if (topologyPrx) {
            Topology *topology = d->loadingProxiesHash.value(topologyPrx, 0);
            if (topology) {
                d->loadingCookieHash.insert(topology, cookie);
            } else
                _handleTopologyPrx(topologyPrx, cookie);
        } else
            cookie->emitCompleted(0);
    }
}

void TopologyManager::_loadTopologySlot(Hns::TopologyPrx topologyPrx, QObject *obj)
{
    Q_D(TopologyManager);
    LoadTopologyCookie *cookie = qobject_cast<LoadTopologyCookie *>(obj);
    if (cookie) {
        if (topologyPrx) {
            Topology *cachedTopology = cookie->topology();
            if (cachedTopology)
                cookie->emitCompleted(cachedTopology);
            else {
                Topology *topology = d->loadingProxiesHash.value(topologyPrx, 0);
                if (topology) {
                    d->loadingCookieHash.insert(topology, cookie);
                } else
                    _handleTopologyPrx(topologyPrx, cookie);
            }
        } else
            cookie->emitCompleted(0);
    }
}

void TopologyManager::_unloadTopologySlot(bool result, QObject *obj)
{
    TopologyCookie *cookie = qobject_cast<TopologyCookie *>(obj);
    if (cookie) {
        SynchronizationStatus entryStatus = Synced;
        if (!result)
            qCritical() << tr("Could not unload topology '%1'").arg(cookie->shortName());
        PrivateSessionAccessor::instance()->session()->topologyEntryCatalog()->_setTopologySyncing(cookie->entryId(), entryStatus);
        cookie->emitCompleted(result);
    }
}

void TopologyManager::_deleteTopologySlot(bool result, QObject *obj)
{
    TopologyCookie *cookie = qobject_cast<TopologyCookie *>(obj);
    if (cookie) {
        SynchronizationStatus entryStatus = Synced;
        if (!result)
            qCritical() << tr("Could not delete topology '%1'").arg(cookie->shortName());
        PrivateSessionAccessor::instance()->session()->topologyEntryCatalog()->_setTopologySyncing(cookie->entryId(), entryStatus);
        cookie->emitCompleted(result);
    }
}

void TopologyManager::_saveTopologySlot(bool result, QObject *obj)
{
    SaveTopologyCookie *cookie = qobject_cast<SaveTopologyCookie *>(obj);
    if (cookie) {
        Topology *topology = topologyById(cookie->topologyId());
        if (topology)
            _handleScreenshotDataCookie(cookie, topology->topologyEntry()->getID(), result);
        cookie->emitCompleted(result);
    }
}

void TopologyManager::_savetopologyFailedSlot(QSharedPointer<Ice::Exception> exception, QObject *obj)
{
    SaveTopologyCookie *cookie = qobject_cast<SaveTopologyCookie *>(obj);
    if (cookie) {
        std::string exceptionMessage = qSharedPointerDynamicCast<Hns::SaveError>(exception)->message;
        cookie->emitCompleted(false, QString::fromStdString(exceptionMessage));
    }
}

void TopologyManager::_saveTopologyAsSlot(bool result, QObject *obj)
{
    SaveAsTopologyCookie *cookie = qobject_cast<SaveAsTopologyCookie *>(obj);
    if (cookie) {
        if (!result)
            _getScreenshotDataCookie(cookie);
        cookie->setResult(result);
        if (result)
            cookie->topology()->_refreshEntry(cookie, SLOT(entryRefreshedSlot(QString)));
    }
}

void TopologyManager::_saveTopologyAsFailedSlot(QSharedPointer<Ice::Exception> exception, QObject *obj)
{
    SaveTopologyCookie *cookie = qobject_cast<SaveTopologyCookie *>(obj);
    if (cookie) {
        std::string exceptionMessage = qSharedPointerDynamicCast<Hns::SaveError>(exception)->message;
        cookie->emitCompleted(false, QString::fromStdString(exceptionMessage));
    }
}

void TopologyManager::_switchTopologyTypeSlot(Ice::Long newEntryId, QObject *obj)
{
    SwitchTopologyTypeCookie *cookie = qobject_cast<SwitchTopologyTypeCookie *>(obj);
    if (cookie)
        cookie->emitCompleted(newEntryId);
}

void TopologyManager::_entityInfoLoadedSlot(QObject *object)
{
    _handleEntityInfo(object);
}

void TopologyManager::_topologyObjectLoadedSlot(QObject *object)
{
    Q_D(TopologyManager);
    Topology *topology = _handleTopology(object);
    if (topology) {
        LoadTopologyCookie *cookie = d->loadingCookieHash.take(topology);
        if (cookie) {
            connect(cookie, SIGNAL(completed(HnsClient::Topology*)), this, SLOT(_loadCookieCompletedSlot(HnsClient::Topology *)));
            cookie->postLoading(topology);
        } else
            emit topologyLoaded(topology->getUuid());
    }
}

void TopologyManager::_loadCookieCompletedSlot(Topology *topology)
{
    emit topologyLoaded(topology->getUuid());
}

void TopologyManager::_updateHashSlot(Topology *topology, QString newUUID, SaveAsTopologyCookie *cookie)
{
    Q_D(TopologyManager);
    QString key = d->topologyHash.key(topology, QString());
    if (!key.isEmpty())
        d->topologyHash.insert(newUUID, d->topologyHash.take(key));
    _handleScreenshotDataCookie(cookie, topology->topologyEntry()->getID(), true);
    cookie->finish();
}

void TopologyManager::_topologyEntryAddedSlot(TopologyEntry *entry)
{
    Q_D(TopologyManager);
    qlonglong entryId = entry->getID();
    if (d->screenshotEntryHash.contains(entryId)) {
        TopologyEntryCatalog *catalog = Session::instance()->topologyEntryCatalog();
        TopologyEntry *catalogEntry = catalog->topologyEntryByID(entryId);
        if (catalogEntry) {
            Hns::ByteArray data = d->screenshotEntryHash.take(entryId);
            catalog->setScreenshot(entryId, data);
        }
    }
}

} // namespace HnsClient
