/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim hnsclient headers
#include "libhnsclient/usermanager.h"
#include "libhnsclient/private/usermanager_p.h"
#include "libhnsclient/private/common_p.h"

// Qt headers
#include <QEventLoop>
#include <QStringList>
#include <QDebug>

namespace HnsClient {

/*******************************************************************************
* Callback_ProxyAuthManager_setPassword
*******************************************************************************/
Callback_ProxyAuthManager_setPassword::Callback_ProxyAuthManager_setPassword()
    : QObject()
    , ::IceUtil::Shared()
{
}

Callback_ProxyAuthManager_setPassword::~Callback_ProxyAuthManager_setPassword()
{
}

void Callback_ProxyAuthManager_setPassword::_success(bool result)
{
    emit succeeded(result);
}

void Callback_ProxyAuthManager_setPassword::_failure(const Ice::Exception &e)
{
    emit failed(QSharedPointer<Ice::Exception>(e.ice_clone()));
}


/*******************************************************************************
* UserManagerPrivate
*******************************************************************************/
UserManagerPrivate::UserManagerPrivate()
{
    _resetData();
}

UserManagerPrivate::~UserManagerPrivate()
{
    _resetData();
}

void UserManagerPrivate::_resetData()
{
    usersHash.clear();
    groupsHash.clear();
    sessionInfoHash.clear();
    remoteCallCount = 0;
    userManagerQtPrx = 0;
    authManagerPrx = 0;
    sessionManagerQPrx = 0;
}


/*******************************************************************************
* SessionGetInfoCookie
*******************************************************************************/
SessionGetInfoCookie::SessionGetInfoCookie()
    : QObject()
    , _completed(false)
{
}

SessionGetInfoCookie::~SessionGetInfoCookie()
{
}

bool SessionGetInfoCookie::isCompleted()
{
    return _completed;
}

Diamesh::SessionInfo SessionGetInfoCookie::information()
{
    return _information;
}

void SessionGetInfoCookie::callGetInfo(Diamesh::SessionPrx proxy)
{
    Callback_ProxySession_getInfoPtr callback = new Callback_ProxySession_getInfo;
    callback->moveToThread(this->thread());
    connect(callback.get(), SIGNAL(succeeded(::Diamesh::SessionInfo)), this, SLOT(_getInfoSlot(::Diamesh::SessionInfo)));
    connect(callback.get(), SIGNAL(failed(QSharedPointer<Ice::Exception>)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
    Diamesh::Callback_Session_getInfoPtr cb = Diamesh::newCallback_Session_getInfo(callback, &Callback_ProxySession_getInfo::_success, &Callback_ProxySession_getInfo::_failure);
    proxy->begin_getInfo(cb);
}

void SessionGetInfoCookie::_getInfoSlot(Diamesh::SessionInfo information)
{
    _completed = true;
    _information = information;
    emit completed();
}

void SessionGetInfoCookie::_remoteMethodFailedSlot(QSharedPointer<Ice::Exception> exception)
{
    qCritical() << tr("Catched Ice exception: %1").arg(exception->ice_name().c_str());
}


/*******************************************************************************
* UserManagerCookie
*******************************************************************************/
UserManagerCookie::UserManagerCookie()
{
}

UserManagerCookie::~UserManagerCookie()
{
}

void UserManagerCookie::_callSetPassword(Diamesh::AuthManagerPrx authPrx, std::string login, std::string pwd)
{
    Callback_ProxyAuthManager_setPasswordPtr callback = new Callback_ProxyAuthManager_setPassword;
    callback->moveToThread(this->thread());
    connect(callback.get(), SIGNAL(succeeded(bool)), this, SLOT(_setPasswordSlot(bool)));
    connect(callback.get(), SIGNAL(failed(QSharedPointer<Ice::Exception>)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
    Diamesh::Callback_AuthManager_setPasswordPtr cb = Diamesh::newCallback_AuthManager_setPassword(callback, &Callback_ProxyAuthManager_setPassword::_success, &Callback_ProxyAuthManager_setPassword::_failure);
    authPrx->begin_setPassword(login, pwd, cb);
}

void UserManagerCookie::_remoteMethodFailedSlot(QSharedPointer<Ice::Exception> exception)
{
    qCritical() << tr("Catched Ice exception: %1").arg(exception->ice_name().c_str());
}

void UserManagerCookie::_setPasswordSlot(bool result)
{
    _handleSetPasswordResult(result);
}


/*******************************************************************************
* AddUserCookie
*******************************************************************************/
AddUserCookie::AddUserCookie(Hns::UserPtr user, QString password, QObject *receiver, const char *slot)
    : UserManagerCookie()
    , _sourceUser(user)
    , _pwd(password)
    , _result(false)
{
    connect(this, SIGNAL(completed(qlonglong,QString,bool)), receiver, slot);
    _groupIdList = QList<Ice::Long>::fromStdList(_sourceUser->groupList);
    _remoteCalls = _groupIdList.count();
    if (_pwd != "")
        _remoteCalls++;
}

AddUserCookie::~AddUserCookie()
{
}

void AddUserCookie::emitCompleted()
{
    emit completed(_sourceUser->id, QString::fromStdString(_sourceUser->name), _result);
    deleteLater();
}

void AddUserCookie::setUserId(qlonglong userId)
{
    _sourceUser->id = userId;
    _result = true;
}

QList<Ice::Long> AddUserCookie::groupIdList()
{
    return _groupIdList;
}

void AddUserCookie::setPassword(Diamesh::AuthManagerPrx authPrx)
{
    if (_pwd != "")
        _callSetPassword(authPrx, _sourceUser->name, _pwd.toStdString());
}

void AddUserCookie::addUserToGroupSlot(bool result)
{
    _remoteCallEnded(result);
}

void AddUserCookie::_handleSetPasswordResult(bool result)
{
    _remoteCallEnded(result);
}

void AddUserCookie::_remoteCallEnded(bool result)
{
    _remoteCalls--;
    _result &= result;
    if (_remoteCalls == 0)
        emitCompleted();
}


/*******************************************************************************
* CloneUserCookie
*******************************************************************************/
CloneUserCookie::CloneUserCookie(Hns::UserPtr user, QString password, QObject *receiver, const char *slot)
    : UserManagerCookie()
    , _sourceUser(user)
    , _pwd(password)
    , _result(false)
    , _remoteCalls(0)
{
    connect(this, SIGNAL(completed(qlonglong,QString,bool)), receiver, slot);
}

CloneUserCookie::~CloneUserCookie()
{
}

void CloneUserCookie::emitCompleted(bool result)
{
    emit completed(_sourceUser->id, userName(), result);
    deleteLater();
}

void CloneUserCookie::setCloneCount(int count)
{
    if (count == 0)
        emitCompleted(false);
    else {
        _result = true;
        _remoteCalls = count;
    }
}

void CloneUserCookie::setClonePassword(Diamesh::AuthManagerPrx authPrx, std::string cloneName)
{
    if (_pwd != "")
        _callSetPassword(authPrx, cloneName, _pwd.toStdString());
    else
        _remoteCallEnded(true);
}

void CloneUserCookie::fetchCloneFailed()
{
    _remoteCallEnded(false);
}

QString CloneUserCookie::userName()
{
    return QString::fromStdString(_sourceUser->name);
}

void CloneUserCookie::_handleSetPasswordResult(bool result)
{
    _remoteCallEnded(result);
}

void CloneUserCookie::_remoteCallEnded(bool result)
{
    _remoteCalls--;
    _result &= result;
    if (_remoteCalls == 0)
        emitCompleted(_result);
}


/*******************************************************************************
* UpdateUserCookie
*******************************************************************************/
UpdateUserCookie::UpdateUserCookie(Hns::UserPtr user, QString password, Diamesh::LongList originalGroupList, QObject *receiver, const char *slot)
    : UserManagerCookie()
    , _sourceUser(user)
    , _pwd(password)
    , _result(false)
    , _remoteCalls(0)
{
    if (receiver)
        connect(this, SIGNAL(completed(qlonglong,QString,bool)), receiver, slot);

    _groupsToAdd = QList<Ice::Long>::fromStdList(user->groupList);
    foreach(Ice::Long groupId, originalGroupList) {
        // User is already in this group, no need to add again
        if (_groupsToAdd.contains(groupId))
            _groupsToAdd.removeAll(groupId);
        else {
            _remoteCalls++;
            _groupsToRemove.append(groupId);
        }
    }
    _remoteCalls += _groupsToAdd.count();
    if (_pwd != "")
        _remoteCalls++;
}

UpdateUserCookie::~UpdateUserCookie()
{
}

void UpdateUserCookie::emitCompleted(bool result)
{
    emit completed(_sourceUser->id, QString::fromStdString(_sourceUser->name), result);
    deleteLater();
}

QList<Ice::Long> UpdateUserCookie::groupToAddIdList()
{
    return _groupsToAdd;
}

QList<Ice::Long> UpdateUserCookie::groupToRemoveIdList()
{
    return _groupsToRemove;
}

void UpdateUserCookie::setPassword(Diamesh::AuthManagerPrx authPrx)
{
    if (_pwd != "")
        _callSetPassword(authPrx, _sourceUser->name, _pwd.toStdString());
}

qlonglong UpdateUserCookie::getUserId()
{
    _result = true;
    return _sourceUser->id;
}

void UpdateUserCookie::addUserToGroup(bool result)
{
    _remoteCallEnded(result);
}

void UpdateUserCookie::removeUserFromGroup(bool result)
{
    _remoteCallEnded(result);
}

void UpdateUserCookie::_handleSetPasswordResult(bool result)
{
    _remoteCallEnded(result);
}

void UpdateUserCookie::_remoteCallEnded(bool result)
{
    _remoteCalls--;
    _result &= result;
    if (_remoteCalls == 0)
        emitCompleted(_result);
}


/*******************************************************************************
* DeleteUserCookie
*******************************************************************************/
DeleteUserCookie::DeleteUserCookie(qlonglong userId, QString userName, QObject *receiver, const char *slot)
    : QObject()
    , _userId(userId)
    , _userName(userName)
{
    connect(this, SIGNAL(completed(qlonglong, QString, bool)), receiver, slot);
}

DeleteUserCookie::~DeleteUserCookie()
{
}

void DeleteUserCookie::emitCompleted(bool result)
{
    emit completed(_userId, _userName, result);
    deleteLater();
}


/*******************************************************************************
* AddGroupCookie
*******************************************************************************/
AddGroupCookie::AddGroupCookie(Diamesh::GroupPtr group, QObject *receiver, const char *slot)
    : QObject()
    , _sourceGroup(group)
    , _result(false)
{
    connect(this, SIGNAL(completed(qlonglong,QString,bool)), receiver, slot);
    _userIdList = QList<Ice::Long>::fromStdList(_sourceGroup->userList);
    _remoteCalls = _userIdList.count();
}

AddGroupCookie::~AddGroupCookie()
{
}

void AddGroupCookie::emitCompleted()
{
    emit completed(_sourceGroup->id, QString::fromStdString(_sourceGroup->name), _result);
    deleteLater();
}

void AddGroupCookie::setGroupId(qlonglong groupId)
{
    _sourceGroup->id = groupId;
    _result = true;
}

QList<Ice::Long> AddGroupCookie::userIdList()
{
    return _userIdList;
}

void AddGroupCookie::addUserToGroupSlot(bool result)
{
    _remoteCallEnded(result);
}

void AddGroupCookie::_remoteCallEnded(bool result)
{
    _remoteCalls--;
    _result &= result;
    if (_remoteCalls == 0)
        emitCompleted();
}


/*******************************************************************************
* UpdateGroupCookie
*******************************************************************************/
UpdateGroupCookie::UpdateGroupCookie(Diamesh::GroupPtr group, Diamesh::LongList originalUserList, QObject *receiver, const char *slot)
    : QObject()
    , _sourceGroup(group)
    , _result(false)
    , _remoteCalls(0)
{
    connect(this, SIGNAL(completed(qlonglong,QString,bool)), receiver, slot);
    _usersToAdd = QList<Ice::Long>::fromStdList(group->userList);
    foreach(Ice::Long userId, originalUserList) {
        // User is already in this group, no need to add again
        if (_usersToAdd.contains(userId))
            _usersToAdd.removeAll(userId);
        else {
            _remoteCalls++;
            _usersToRemove.append(userId);
        }
    }
    _remoteCalls += _usersToAdd.count();
}

UpdateGroupCookie::~UpdateGroupCookie()
{
}

void UpdateGroupCookie::emitCompleted(bool result)
{
    emit completed(_sourceGroup->id, QString::fromStdString(_sourceGroup->name), result);
    deleteLater();
}

QList<Ice::Long> UpdateGroupCookie::userToAddIdList()
{
    return _usersToAdd;
}

QList<Ice::Long> UpdateGroupCookie::userToRemoveIdList()
{
    return _usersToRemove;
}

qlonglong UpdateGroupCookie::getGroupId()
{
    _result = true;
    return _sourceGroup->id;
}

void UpdateGroupCookie::addUserToGroup(bool result)
{
    _remoteCallEnded(result);
}

void UpdateGroupCookie::removeUserFromGroup(bool result)
{
    _remoteCallEnded(result);
}

void UpdateGroupCookie::_remoteCallEnded(bool result)
{
    _remoteCalls--;
    _result &= result;
    if (_remoteCalls == 0)
        emitCompleted(_result);
}


/*******************************************************************************
* DeleteGroupCookie
*******************************************************************************/
DeleteGroupCookie::DeleteGroupCookie(qlonglong groupId, QString groupName, QObject *receiver, const char *slot)
    : QObject()
    , _groupId(groupId)
    , _groupName(groupName)
{
    connect(this, SIGNAL(completed(qlonglong, QString, bool)), receiver, slot);
}

DeleteGroupCookie::~DeleteGroupCookie()
{
}

void DeleteGroupCookie::emitCompleted(bool result)
{
    emit completed(_groupId, _groupName, result);
    deleteLater();
}


/*******************************************************************************
* UserManager
*******************************************************************************/
UserManager::UserManager(QObject *parent)
    : QObject(parent)
    , d_ptr(new UserManagerPrivate)
{
}

UserManager::~UserManager()
{
    delete d_ptr;
}

/*! \fn usersCount() const
 *\brief Local method. Returns users count
 */
int UserManager::usersCount() const
{
    Q_D(const UserManager);
    return d->usersHash.count();
}

/*! \fn groupsCount() const
 *\brief Local method. Returns group count
 */
int UserManager::groupsCount() const
{
    Q_D(const UserManager);
    return d->groupsHash.count();
}

/*! \fn userExists(qlonglong userId) const
 *\brief Local method. Returns true if user with Id exists in local cache
 *\param userId
 */
bool UserManager::userExists(qlonglong userId) const
{
    Q_D(const UserManager);
    return d->usersHash.contains(userId);
}

/*! \fn groupExists(qlonglong groupId) const
 *\brief Local method. Returns true if group with Id exists in local cache
 *\param groupId
 */
bool UserManager::groupExists(qlonglong groupId) const
{
    Q_D(const UserManager);
    return d->groupsHash.contains(groupId);
}

/*! \fn getUserById(qlonglong userId) const
 *\brief Local method. Return the user with specified Id. Returns 0 if the user is not found
 *\param userId
 */
Hns::UserPtr UserManager::getUserById(qlonglong userId) const
{
    Q_D(const UserManager);
    return d->usersHash.value(userId, 0);
}

/*! \fn getUserByIndex(int index)
 *\brief TODO
 *\param index
 */
Hns::UserPtr UserManager::getUserByIndex(int index) const
{
    Q_D(const UserManager);
    QList<qlonglong> keys = d->usersHash.keys();
    if (index < keys.size()) {
        return d->usersHash.value(keys.at(index));
    }
    return 0;
}

/*! \fn getUserIndex(qlonglong userId) const
 *\brief TODO
 *\param index
 */
int UserManager::getUserIndex(qlonglong userId) const
{
    Q_D(const UserManager);
    if (userExists(userId)) {
        QList<qlonglong> keys = d->usersHash.keys();
        return keys.indexOf(userId);
    }
    return -1;
}

/*! \fn getGroupById(qlonglong groupId)
 *\brief TODO
 *\param groupId
 */
Diamesh::GroupPtr UserManager::getGroupById(qlonglong groupId) const
{
    Q_D(const UserManager);
    return d->groupsHash.value(groupId, 0);
}

/*! \fn getGroupByIndex(int index) const
 *\brief TODO
 *\param index
 */
Diamesh::GroupPtr UserManager::getGroupByIndex(int index) const
{
    Q_D(const UserManager);
    QList<qlonglong> keys = d->groupsHash.keys();
    if (index < keys.size()) {
        return d->groupsHash.value(keys.at(index));
    }
    return 0;
}

/*! \fn getGroupIndex(qlonglong groupId) const
 *\brief TODO
 *\param groupId
 */
int UserManager::getGroupIndex(qlonglong groupId) const
{
    Q_D(const UserManager);
    if (groupExists(groupId)) {
        QList<qlonglong> keys = d->groupsHash.keys();
        return keys.indexOf(groupId);
    }
    return -1;
}

/*! \fn getUserNameById(qlonglong userId)
 *\brief TODO
 *\param userId
 */
QString UserManager::getUserNameById(qlonglong userId)
{
    Q_D(UserManager);
    QString userName;
    if (userExists(userId))
        userName = QString::fromStdString(d->usersHash.value(userId)->name);
    return userName;
}

/*! \fn getUsernamesFromIdList(QList<qlonglong> usersIdList)
 *\brief TODO
 *\param usersIdList
 */
QStringList UserManager::getUsernamesFromIdList(QList<qlonglong> usersIdList)
{
    QStringList userNamesList;
    foreach(qlonglong userId, usersIdList) {
        QString userName = getUserNameById(userId);
        if (!userName.isNull())
            userNamesList.append(userName);
    }
    return userNamesList;
}

/*! \fn getGroupNameById(qlonglong groupId)
 *\brief TODO
 *\param groupId
 */
QString UserManager::getGroupNameById(qlonglong groupId)
{
    Q_D(UserManager);
    QString groupName;
    if (groupExists(groupId))
        groupName = QString::fromStdString(d->groupsHash.value(groupId)->name);
    return groupName;
}

/*! \fn getGroupsNamesFromIdList(QList<qlonglong> groupsIdList)
 *\brief TODO
 *\param groupsIdList
 */
QStringList UserManager::getGroupsNamesFromIdList(QList<qlonglong> groupsIdList)
{
    QStringList groupsNamesList;
    foreach(qlonglong groupId, groupsIdList) {
        QString groupName = getGroupNameById(groupId);
        if (!groupName.isNull())
            groupsNamesList.append(groupName);
    }
    return groupsNamesList;
}

/*! \fn getUserNamesHash()
 *\brief TODO
 */
QHash<qlonglong, QString> UserManager::getUserNamesHash()
{
    Q_D(UserManager);
    QHash<qlonglong, QString> userNamesHash;
    QHashIterator<qlonglong, Hns::UserPtr> iterator(d->usersHash);
    while(iterator.hasNext()) {
        iterator.next();
        userNamesHash.insert(iterator.key(), QString::fromStdString(iterator.value()->name));
    }
    return userNamesHash;
}

/*! \fn getGroupNamesHash()
 *\brief TODO
 */
QHash<qlonglong, QString> UserManager::getGroupNamesHash()
{
    Q_D(UserManager);
    QHash<qlonglong, QString> groupNamesHash;
    QHashIterator<qlonglong, Diamesh::GroupPtr> iterator(d->groupsHash);
    while(iterator.hasNext()) {
        iterator.next();
        groupNamesHash.insert(iterator.key(), QString::fromStdString(iterator.value()->name));
    }
    return groupNamesHash;
}

/*! \fn isUserConnected(qlonglong userId)
 *\brief TODO
 *\param userId
 */
bool UserManager::isUserConnected(qlonglong userID)
{
    Q_D(UserManager);
    bool userFound = false;
    QHashIterator<QString, Diamesh::SessionInfo> iterator(d->sessionInfoHash);
    while(!userFound && iterator.hasNext()) {
        iterator.next();
        if (iterator.value().userId == userID) {
            userFound = true;
        }
    }
    return userFound;
}

/*! \fn getUserSessionList(qlonglong userId)
 *\brief TODO
 *\param userId
 */
QHash<QString, Diamesh::SessionInfo> UserManager::getUserSessionList(qlonglong userID)
{
    Q_D(UserManager);
    QHash<QString, Diamesh::SessionInfo> sessionList;
    QHashIterator<QString, Diamesh::SessionInfo> iterator(d->sessionInfoHash);
    while(iterator.hasNext()) {
        iterator.next();
        if (iterator.value().userId == userID)
            sessionList.insert(iterator.key(), iterator.value());
    }
    return sessionList;
}

/*! \fn getSessionList()
 *\brief TODO
 */
QHash<QString, Diamesh::SessionInfo> UserManager::getSessionList()
{
    Q_D(UserManager);
    return d->sessionInfoHash;
}

/*! \fn getSession(QString sessionId)
 *\brief TODO
 *\param sessionId
 */
Diamesh::SessionInfo UserManager::getSession(QString sessionId)
{
    Q_D(UserManager);
    Diamesh::SessionInfo info;
    if (d->sessionInfoHash.contains(sessionId))
        info = d->sessionInfoHash.value(sessionId);
    else
        info.userId = -1;
    return info;
}

/*! \fn addUser(Hns::UserPtr user, QString password, QObject *receiver, const char *slot);
 *\brief Create a cookie object with user, password, receiver and slot parameters and give it to begin_addUser.
 *\param user Point on user object
 *\param password user's password
 *\param receiver object connected to internal signal
 *\param slot Return slot connected to signal "completed()"
 */
void UserManager::addUser(Hns::UserPtr user, QString password, QObject *receiver, const char *slot)
{
    Q_D(UserManager);
    AddUserCookie *cookie = new AddUserCookie(user, password, receiver, slot);
    connect(cookie, SIGNAL(completed(qlonglong,QString,bool)), this, SLOT(_addUserCookieCompleted(qlonglong,QString,bool)));
    d->userManagerQtPrx->begin_addUser(user, this, SLOT(_addUserSlot(::Ice::Long, QObject *)), 0, 0, cookie);
}

/*! \fn cloneUser(Hns::UserPtr user, QString password, int count, QObject *receiver, const char *slot);
 *\brief Create a cookie object with user, password, receiver and slot parameters and give it to begin_cloneUser.
 *\param user Point on user object
 *\param password user's password
 *\param receiver object connected to internal signal
 *\param slot Return slot connected to signal "completed(qlonglong userId, QString userName, bool result)"
 */
void UserManager::cloneUser(Hns::UserPtr user, QString password, int count, QObject *receiver, const char *slot)
{
    Q_D(UserManager);
    CloneUserCookie *cookie = new CloneUserCookie(user, password, receiver, slot);
    connect(cookie, SIGNAL(completed(qlonglong,QString,bool)), this, SLOT(_cloneUserCookieCompleted(qlonglong,QString,bool)));
    d->userManagerQtPrx->begin_cloneUser(user, count, this, SLOT(_cloneUserSlot(::Diamesh::LongList, QObject *)), 0, 0, cookie);
}

/*! \fn updateUser(Hns::UserPtr user, QString password, QObject *receiver, const char *slot)
 *\brief Create a cookie object with user, password, originalGroupList, receiver and slot parameters and give it to begin_updateUser.
 *\param user Point on user object
 *\param password user's password
 *\param receiver object connected to internal signal
 *\param slot Return slot connected to signal "completed(qlonglong userId, QString userName, bool result)"
 */
void UserManager::updateUser(Hns::UserPtr user, QString password, QObject *receiver, const char *slot)
{
    Q_D(UserManager);
    Ice::Long userId = user->id;
    Hns::UserPtr localUser = getUserById(userId);
    if (localUser) {
        Diamesh::LongList originalGroupList = localUser->groupList;
        UpdateUserCookie *cookie = new UpdateUserCookie(user, password, originalGroupList, receiver, slot);
        connect(cookie, SIGNAL(completed(qlonglong,QString,bool)), this, SLOT(_updateUserCookieCompleted(qlonglong,QString,bool)));
        d->userManagerQtPrx->begin_updateUser(user, this, SLOT(_updateUserSlot(bool, QObject *)), 0, 0, cookie);
    }
}

/*! \fn deleteUser(qlonglong userId, QObject *receiver, const char *slot)
 *\brief Create a cookie object with userId, userName, receiver and slot parameters and give it to begin_deleteUser.
 *\param userId userId
 *\param receiver object connected to internal signal
 *\param slot Return slot connected to signal "completed(qlonglong userId, QString userName, bool result)"
 */
void UserManager::deleteUser(qlonglong userId, QObject *receiver, const char *slot)
{
    Q_D(UserManager);
    QString userName  = getUserNameById(userId);
    if (!userName.isEmpty()) {
        DeleteUserCookie *cookie = new DeleteUserCookie(userId, userName, receiver, slot);
        connect(cookie, SIGNAL(completed(qlonglong,QString,bool)), this, SLOT(_deleteUserCookieCompleted(qlonglong,QString,bool)));
        d->userManagerQtPrx->begin_deleteUser(userId, this, SLOT(_deleteUserSlot(bool, QObject *)), 0, 0, cookie);
    }
}

/*! \fn  addGroup(Diamesh::GroupPtr group, QObject *receiver, const char *slot)
 *\brief Create a cookie object with group, userName, receiver and slot parameters and give it to begin_addGroup.
 *\param group Point on group
 *\param receiver object connected to internal signal
 *\param slot Return slot connected to signal "completed(qlonglong groupId, QString groupName, bool result)"
 */
void UserManager::addGroup(Diamesh::GroupPtr group, QObject *receiver, const char *slot)
{
    Q_D(UserManager);
    AddGroupCookie *cookie = new AddGroupCookie(group, receiver, slot);
    connect(cookie, SIGNAL(completed(qlonglong,QString,bool)), this, SLOT(_addGroupCookieCompleted(qlonglong,QString,bool)));
    d->userManagerQtPrx->begin_addGroup(group, this, SLOT(_addGroupSlot(::Ice::Long,QObject*)), 0, 0, cookie);
}

/*! \fn updateGroup(Diamesh::GroupPtr group, QObject *receiver, const char *slot)
 *\brief Create a cookie object with group, originalUserList, receiver and slot parameters and give it to begin_cloneUser.
 *\param user Point on user object
 *\param receiver object connected to internal signal
 *\param slot Return slot connected to signal "completed(qlonglong groupId, QString groupName, bool result)"
 */
void UserManager::updateGroup(Diamesh::GroupPtr group, QObject *receiver, const char *slot)
{
    Q_D(UserManager);
    Ice::Long groupId = group->id;
    Diamesh::GroupPtr localGroup = getGroupById(groupId);
    if (localGroup) {
        Diamesh::LongList originalUserList = localGroup->userList;
        UpdateGroupCookie *cookie = new UpdateGroupCookie(group, originalUserList, receiver, slot);
        connect(cookie, SIGNAL(completed(qlonglong,QString,bool)), this, SLOT(_updateGroupCookieCompleted(qlonglong,QString,bool)));
        d->userManagerQtPrx->begin_updateGroup(group, this, SLOT(_updateGroupSlot(bool,QObject*)), 0, 0, cookie);
    }
}

/*! \fn deleteGroup(qlonglong groupId, QObject *receiver, const char *slot)
 *\brief Create a cookie object with groupId, groupName, receiver and slot parameters and give it to begin_deleteGroup.
 *\param groupId
 *\param receiver object connected to internal signal
 *\param slot Return slot connected to signal "completed(qlonglong groupId, QString groupName, bool result)"
 */
void UserManager::deleteGroup(qlonglong groupId, QObject *receiver, const char *slot)
{
    Q_D(UserManager);
    QString groupName = getGroupNameById(groupId);
    if (!groupName.isEmpty()) {
        DeleteGroupCookie *cookie = new DeleteGroupCookie(groupId, groupName, receiver, slot);
        connect(cookie, SIGNAL(completed(qlonglong,QString,bool)), this, SLOT(_deleteGroupCookieCompleted(qlonglong,QString,bool)));
        d->userManagerQtPrx->begin_deleteGroup(groupId, this, SLOT(_deleteGroupSlot(bool,QObject*)), 0, 0, cookie);
    }
}

/*! \fn disconnectUser(qlonglong userId, QObject *receiver = 0, const char *slot = 0)
 *\brief TODO
 *\param userId
 *\param receiver
 *\param slot
 */
void UserManager::disconnectUser(qlonglong userId, QObject *receiver, const char *slot)
{
    Q_D(UserManager);
    d->sessionManagerQPrx->begin_disconnectUser(userId, receiver, slot);
}

void UserManager::_load(Diamesh::UserManagerQPrx userManagerQtPrx, Diamesh::AuthManagerPrx authManagerPrx, Diamesh::SessionManagerQPrx sessionManagerQPrx)
{
    Q_D(UserManager);
    d->userManagerQtPrx = userManagerQtPrx;
    d->authManagerPrx = authManagerPrx;
    d->sessionManagerQPrx = sessionManagerQPrx;
    d->remoteCallCount = 3;
    d->sessionManagerQPrx->begin_getSessions(this, SLOT(_loadSessionListSlot(::Diamesh::SessionList)));
    d->userManagerQtPrx->begin_getUsers(this, SLOT(_loadUsersSlot(::Diamesh::LongList)));
    d->userManagerQtPrx->begin_getGroups(this, SLOT(_loadGroupsSlot(::Diamesh::LongList)));
}

void UserManager::_callEnded()
{
    Q_D(UserManager);
    d->remoteCallCount--;
    _checkLoading();
}

void UserManager::_checkLoading()
{
    Q_D(UserManager);
    if (d->remoteCallCount == 0)
        _init();
}

void UserManager::_init()
{
    Q_D(UserManager);
    connect(d->sessionManagerQPrx.data(), SIGNAL(sessionCreated(QString)), this, SLOT(_sessionCreatedSlot(QString)));
    connect(d->sessionManagerQPrx.data(), SIGNAL(sessionDestroyed(QString)), this, SLOT(_sessionDestroyedSlot(QString)));
    connect(d->sessionManagerQPrx.data(), SIGNAL(userDisconnected(Ice::Long)), this, SLOT(_userDisconnectedSlot(Ice::Long)));

    connect(d->userManagerQtPrx.data(), SIGNAL(userAdded(Ice::Long)), this, SLOT(_userAddedSlot(Ice::Long)));
    connect(d->userManagerQtPrx.data(), SIGNAL(userChanged(Ice::Long)), this, SLOT(_userChangedSlot(Ice::Long)));
    connect(d->userManagerQtPrx.data(), SIGNAL(userRemoved(Ice::Long)), this, SLOT(_userRemovedSlot(Ice::Long)));

    connect(d->userManagerQtPrx.data(), SIGNAL(groupAdded(Ice::Long)), this, SLOT(_groupAddedSlot(Ice::Long)));
    connect(d->userManagerQtPrx.data(), SIGNAL(groupChanged(Ice::Long)), this, SLOT(_groupChangedSlot(Ice::Long)));
    connect(d->userManagerQtPrx.data(), SIGNAL(groupRemoved(Ice::Long)), this, SLOT(_groupRemovedSlot(Ice::Long)));

    emit loaded();
}

void UserManager::_handleSessionProxy(Diamesh::SessionPrx sessionPrx)
{
    Q_D(UserManager);
    if (sessionPrx) {
        QString sessionId = QString::fromStdString(sessionPrx->ice_getIdentity().name); // ice_getIdentity is a local method
        if (d->sessionInfoHash.contains(sessionId)) {
            qDebug() << tr("Session %1 is already registered.").arg(sessionId);
        } else {
            SessionGetInfoCookie *cookie = new SessionGetInfoCookie();
            QEventLoop eventLoop;
            connect(cookie, SIGNAL(completed()), &eventLoop, SLOT(quit()));
            cookie->callGetInfo(sessionPrx);
            if (!cookie->isCompleted())
                eventLoop.exec();
            Diamesh::SessionInfo information = cookie->information();
            d->sessionInfoHash.insert(sessionId, information);
            emit userChanged(information.userId);
            emit sessionAdded(sessionId);
        }
    }
}

void UserManager::_handleUserPtr(Diamesh::UserPtr user)
{
    Q_D(UserManager);
    if (user) {
        qlonglong userId = user->id;
        if (userExists(userId)) {
            qDebug() << tr("User with ID %1 already registered.").arg(QString::number(userId));
            return;
        }
        d->usersHash.insert(userId, Hns::UserPtr::dynamicCast(user));
        emit userAdded(userId);
    } else {
        qDebug() << tr("Could not build user node. User data is null");
    }
}

void UserManager::_handleGroupPtr(Diamesh::GroupPtr group)
{
    Q_D(UserManager);
    if (group) {
        qlonglong groupId = group->id;
        if (groupExists(groupId)) {
            qDebug() << tr("Group with ID %1 already registered.").arg(QString::number(groupId));
            return;
        }
        d->groupsHash.insert(groupId, group);
        emit groupAdded(groupId);
    } else {
        qDebug() << tr("Could not build group node. Group data is null");
    }
}

void UserManager::_emitAddUserError(QString login)
{
    emit userErrorOccured(tr("Could not add user %1. Please contact your administrator.").arg(login));
}

void UserManager::_emitCloneUserError(QString login, qlonglong userId)
{
    emit userErrorOccured(tr("Could not clone user %1 (ID:%2)").arg(login).arg(QString::number(userId)));
}

void UserManager::_emitUpdateUserError(QString login, qlonglong userId)
{
    emit userErrorOccured(tr("Could not update user %1 (ID:%2)").arg(login).arg(QString::number(userId)));
}

void UserManager::_emitDeleteUserError(QString login, qlonglong userId)
{
    emit userErrorOccured(tr("Could not delete user %1 (ID:%2)").arg(login).arg(QString::number(userId)));
}

void UserManager::_emitAddGroupError(QString groupName)
{
    emit groupErrorOccured(tr("Could not add group %1. Please contact your administrator.").arg(groupName));
}

void UserManager::_emitUpdateGroupError(QString groupName, qlonglong groupId)
{
    emit groupErrorOccured(tr("Could not update group %1 (ID:%2)").arg(groupName).arg(QString::number(groupId)));
}

void UserManager::_emitDeleteGroupError(QString groupName, qlonglong groupId)
{
    emit groupErrorOccured(tr("Could not delete group %1 (ID:%2)").arg(groupName).arg(QString::number(groupId)));
}

void UserManager::_loadSessionListSlot(Diamesh::SessionList sessionList)
{
    Q_D(UserManager);
    QList<Diamesh::SessionPrx> sessionQList = QList<Diamesh::SessionPrx>::fromStdList(sessionList);
    d->remoteCallCount += sessionQList.count();
    foreach(Diamesh::SessionPrx sessionPrx, sessionQList)
        _loadSessionSlot(sessionPrx);
    _callEnded();
}

void UserManager::_loadUsersSlot(Diamesh::LongList userIdList)
{
    Q_D(UserManager);
    d->remoteCallCount += userIdList.size();
    foreach(Ice::Long userId, userIdList) {
        d->userManagerQtPrx->begin_getUser(userId, this, SLOT(_loadUserSlot(::Diamesh::UserPtr)));
    }
    _callEnded();
}

void UserManager::_loadGroupsSlot(Diamesh::LongList groupIdList)
{
    Q_D(UserManager);
    d->remoteCallCount += groupIdList.size();
    foreach(Ice::Long groupId, groupIdList) {
        d->userManagerQtPrx->begin_getGroup(groupId, this, SLOT(_loadGroupSlot(::Diamesh::GroupPtr)));
    }
    _callEnded();
}

void UserManager::_loadSessionSlot(Diamesh::SessionPrx sessionPrx)
{
    _handleSessionProxy(sessionPrx);
    _callEnded();
}

void UserManager::_loadUserSlot(Diamesh::UserPtr user)
{
    _handleUserPtr(user);
    _callEnded();
}

void UserManager::_loadGroupSlot(Diamesh::GroupPtr group)
{
    _handleGroupPtr(group);
    _callEnded();
}

void UserManager::_addUserSlot(Ice::Long userId, QObject *obj)
{
    Q_D(UserManager);
    AddUserCookie *cookie = qobject_cast<AddUserCookie *>(obj);
    if (cookie) {
        if (userId == 0)
            cookie->emitCompleted();
        else {
            cookie->setUserId(userId);
            cookie->setPassword(d->authManagerPrx);
            foreach(Ice::Long groupId, cookie->groupIdList()) {
                d->userManagerQtPrx->begin_addUserToGroup(userId, groupId, cookie, SLOT(addUserToGroupSlot(bool)));
            }
        }
    }
}

void UserManager::_cloneUserSlot(Diamesh::LongList clonedUserIdList, QObject *obj)
{
    Q_D(UserManager);
    CloneUserCookie *cookie = qobject_cast<CloneUserCookie *>(obj);
    if (cookie) {
        QList<Ice::Long> cloneIdList = QList<Ice::Long>::fromStdList(clonedUserIdList);
        cookie->setCloneCount(cloneIdList.size());
        foreach(Ice::Long cloneId, cloneIdList)
            d->userManagerQtPrx->begin_getUser(cloneId, this, SLOT(_getClonedUserSlot(::Diamesh::UserPtr, QObject *)), 0, 0, cookie);
    }
}

void UserManager::_getClonedUserSlot(::Diamesh::UserPtr clone, QObject *obj)
{
    Q_D(UserManager);
    CloneUserCookie *cookie = qobject_cast<CloneUserCookie *>(obj);
    if (cookie) {
        if (clone) {
            cookie->setClonePassword(d->authManagerPrx, clone->name);
        } else {
            cookie->fetchCloneFailed();
        }
    }
}

void UserManager::_updateUserSlot(bool result, QObject *obj)
{
    Q_D(UserManager);
    UpdateUserCookie *cookie = qobject_cast<UpdateUserCookie *>(obj);
    if (cookie) {
        if (result) {
            qlonglong userId = cookie->getUserId();
            cookie->setPassword(d->authManagerPrx);
            foreach(Ice::Long groupId, cookie->groupToAddIdList()) {
                d->userManagerQtPrx->begin_addUserToGroup(userId, groupId, cookie, SLOT(addUserToGroupSlot(bool)));
            }
            foreach(Ice::Long groupId, cookie->groupToRemoveIdList()) {
                d->userManagerQtPrx->begin_removeUserFromGroup(userId, groupId, cookie, SLOT(addUserToGroupSlot(bool)));
            }
        } else {
            cookie->emitCompleted(false);
        }
    }
}

void UserManager::_deleteUserSlot(bool result, QObject *obj)
{
    DeleteUserCookie *cookie = qobject_cast<DeleteUserCookie *>(obj);
    if (cookie)
        cookie->emitCompleted(result);
}

void UserManager::_addGroupSlot(Ice::Long groupId, QObject *obj)
{
    Q_D(UserManager);
    AddGroupCookie *cookie = qobject_cast<AddGroupCookie *>(obj);
    if (cookie) {
        if (groupId == 0)
            cookie->emitCompleted();
        else {
            cookie->setGroupId(groupId);
            foreach(Ice::Long userId, cookie->userIdList()) {
                d->userManagerQtPrx->begin_addUserToGroup(userId, groupId, cookie, SLOT(addUserToGroupSlot(bool)));
            }
        }
    }
}

void UserManager::_updateGroupSlot(bool result, QObject *obj)
{
    Q_D(UserManager);
    UpdateGroupCookie *cookie = qobject_cast<UpdateGroupCookie *>(obj);
    if (cookie) {
        if (result) {
            qlonglong groupId = cookie->getGroupId();
            foreach(Ice::Long userId, cookie->userToRemoveIdList()) {
                d->userManagerQtPrx->begin_removeUserFromGroup(userId, groupId, cookie, SLOT(removeUserFromGroup(bool)));
            }
            foreach(Ice::Long userId, cookie->userToAddIdList()) {
                d->userManagerQtPrx->begin_addUserToGroup(userId, groupId, cookie, SLOT(addUserToGroup(bool)));
            }
        } else {
            cookie->emitCompleted(false);
        }
    }
}

void UserManager::_deleteGroupSlot(bool result, QObject *obj)
{
    DeleteGroupCookie *cookie = qobject_cast<DeleteGroupCookie *>(obj);
    if (cookie)
        cookie->emitCompleted(result);
}

void UserManager::_getSessionSlot(Diamesh::SessionPrx sessionPrx)
{
    _handleSessionProxy(sessionPrx);
}

void UserManager::_getUserSlot(Diamesh::UserPtr user)
{
    _handleUserPtr(user);
}

void UserManager::_getUserUpdateSlot(Diamesh::UserPtr user)
{
    Q_D(UserManager);
    if (user) {
        qlonglong userId = user->id;
        d->usersHash[userId] = Hns::UserPtr::dynamicCast(user);
        emit userChanged(userId);
    } else {
        qDebug() << tr("Could not build user node. User data is null");
    }
}

void UserManager::_getGroupSlot(Diamesh::GroupPtr group)
{
    _handleGroupPtr(group);
}

void UserManager::_getGroupUpdateSlot(Diamesh::GroupPtr group)
{
    Q_D(UserManager);
    if (group) {
        qlonglong groupId = group->id;
        d->groupsHash[groupId] = group;
        emit groupChanged(groupId);
    } else {
        qDebug() << tr("Could not build group node. Group data is null");
    }
}

void UserManager::_addUserCookieCompleted(qlonglong userId, QString userName, bool result)
{
    Q_UNUSED(userId)
    if (!result)
        _emitAddUserError(userName);
}

void UserManager::_cloneUserCookieCompleted(qlonglong userId, QString userName, bool result)
{
    if (!result)
        _emitCloneUserError(userName, userId);
}

void UserManager::_updateUserCookieCompleted(qlonglong userId, QString userName, bool result)
{
    if (!result)
        _emitUpdateUserError(userName, userId);
}

void UserManager::_deleteUserCookieCompleted(qlonglong userId, QString userName, bool result)
{
    if (!result)
        _emitDeleteUserError(userName, userId);
}

void UserManager::_addGroupCookieCompleted(qlonglong groupId, QString groupName, bool result)
{
    Q_UNUSED(groupId)
    if (!result)
        _emitAddGroupError(groupName);
}

void UserManager::_updateGroupCookieCompleted(qlonglong groupId, QString groupName, bool result)
{
    if (!result)
        _emitUpdateGroupError(groupName, groupId);
}

void UserManager::_deleteGroupCookieCompleted(qlonglong groupId, QString groupName, bool result)
{
    if (!result)
        _emitDeleteGroupError(groupName, groupId);
}

void UserManager::_sessionCreatedSlot(QString sessionId)
{
    Q_D(UserManager);
    d->sessionManagerQPrx->begin_getSession(sessionId.toStdString(), this, SLOT(_getSessionSlot(::Diamesh::SessionPrx)));
}

void UserManager::_sessionDestroyedSlot(QString sessionId)
{
    Q_D(UserManager);
    if (d->sessionInfoHash.contains(sessionId)) {
        Diamesh::SessionInfo info = d->sessionInfoHash.take(sessionId);
        qlonglong userId = info.userId;
        emit userChanged(userId);
        emit sessionRemoved(sessionId);
    }
}

void UserManager::_userDisconnectedSlot(Ice::Long userId)
{
    Q_D(UserManager);
    QHash<QString, Diamesh::SessionInfo> userSessions = getUserSessionList(userId);
    if (userSessions.count() > 0) {
        foreach(const QString &sessionId, userSessions.keys()){
            if (d->sessionInfoHash.contains(sessionId)) {
                d->sessionInfoHash.remove(sessionId);
                emit sessionRemoved(sessionId);
            }
        }
//        emit userChanged(userId);
    }
}

void UserManager::_userAddedSlot(Ice::Long userId)
{
    Q_D(UserManager);
    d->userManagerQtPrx->begin_getUser(userId, this, SLOT(_getUserSlot(::Diamesh::UserPtr)));
}

void UserManager::_userChangedSlot(Ice::Long userId)
{
    Q_D(UserManager);
    d->userManagerQtPrx->begin_getUser(userId, this, SLOT(_getUserUpdateSlot(::Diamesh::UserPtr)));
}

void UserManager::_userRemovedSlot(Ice::Long userId)
{
    Q_D(UserManager);
    if (userExists(userId)) {
        int index = getUserIndex(userId);
        if (index != -1) {
            d->usersHash.remove(userId);
            emit userRemoved(userId);
        }
    }
}

void UserManager::_groupAddedSlot(Ice::Long groupId)
{
    Q_D(UserManager);
    d->userManagerQtPrx->begin_getGroup(groupId, this, SLOT(_getGroupSlot(::Diamesh::GroupPtr)));
}

void UserManager::_groupChangedSlot(Ice::Long groupId)
{
    Q_D(UserManager);
    d->userManagerQtPrx->begin_getGroup(groupId, this, SLOT(_getGroupUpdateSlot(::Diamesh::GroupPtr)));
}

void UserManager::_groupRemovedSlot(Ice::Long groupId)
{
    Q_D(UserManager);
    if (groupExists(groupId)) {
        int index = getGroupIndex(groupId);
        if (index != -1) {
            d->groupsHash.remove(groupId);
            emit groupRemoved(groupId);
        }
    }
}

} // namespace HnsClient
