/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNSCLIENT_USERMANAGER_H
#define HNSCLIENT_USERMANAGER_H

// Generated hynesim headers
#include "common/usermanager.h"

// diamesh headers
#include <diamesh/proxies_authmanager.h>
#include <diamesh/proxies_sessionManager.h>
#include <diamesh/proxies_usermanager.h>

// Qt headers
#include <QObject>
#include <QHash>

namespace HnsClient {

class UserManagerPrivate;

/*******************************************************************************
* UserManager
*******************************************************************************/
class UserManager : public QObject
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(UserManager)
public:
    ~UserManager();

    // Local methods, access to data stored in cache
    int usersCount() const;
    int groupsCount() const;
    bool userExists(qlonglong userId) const;
    bool groupExists(qlonglong groupId) const;
    Hns::UserPtr getUserById(qlonglong userId) const;
    Hns::UserPtr getUserByIndex(int index) const;
    int getUserIndex(qlonglong userId) const;
    Diamesh::GroupPtr getGroupById(qlonglong groupId) const;
    Diamesh::GroupPtr getGroupByIndex(int index) const;
    int getGroupIndex(qlonglong groupId) const;
    QString getUserNameById(qlonglong userId);
    QStringList getUsernamesFromIdList(QList<qlonglong> usersIdList);
    QString getGroupNameById(qlonglong groupId);
    QStringList getGroupsNamesFromIdList(QList<qlonglong> groupsIdList);
    QHash<qlonglong, QString> getUserNamesHash();
    QHash<qlonglong, QString> getGroupNamesHash();
    bool isUserConnected(qlonglong userId);
    QHash<QString, Diamesh::SessionInfo> getUserSessionList(qlonglong userId);
    QHash<QString, Diamesh::SessionInfo> getSessionList();
    Diamesh::SessionInfo getSession(QString sessionId);

    // Methods based on remote calls
    void addUser(Hns::UserPtr user, QString password, QObject *receiver, const char *slot);
    void cloneUser(Hns::UserPtr user, QString password, int count, QObject *receiver, const char *slot);
    void updateUser(Hns::UserPtr user, QString password, QObject *receiver, const char *slot);
    void deleteUser(qlonglong userId, QObject *receiver, const char *slot);
    void addGroup(Diamesh::GroupPtr group, QObject *receiver, const char *slot);
    void updateGroup(Diamesh::GroupPtr group, QObject *receiver, const char *slot);
    void deleteGroup(qlonglong groupId, QObject *receiver, const char *slot);
    void disconnectUser(qlonglong userId, QObject *receiver = 0, const char *slot = 0);

signals:
    void loaded();

    void sessionAdded(QString sessionId);
    void sessionRemoved(QString sessionId);

    void userAdded(qlonglong userId);
    void userChanged(qlonglong userId);
    void userRemoved(qlonglong userId);

    void groupAdded(qlonglong groupId);
    void groupChanged(qlonglong groupId);
    void groupRemoved(qlonglong groupId);

    void userErrorOccured(QString message);
    void groupErrorOccured(QString message);

protected:
    explicit UserManager(QObject* parent = 0);
    void _load(Diamesh::UserManagerQPrx userManagerQtPrx, Diamesh::AuthManagerPrx authManagerPrx, Diamesh::SessionManagerQPrx sessionManagerQPrx);
    void _callEnded();
    void _checkLoading();
    void _init();

    void _handleSessionProxy(Diamesh::SessionPrx sessionPrx);
    void _handleUserPtr(Diamesh::UserPtr user);
    void _handleGroupPtr(Diamesh::GroupPtr group);

    void _emitAddUserError(QString login);
    void _emitCloneUserError(QString login, qlonglong userId);
    void _emitUpdateUserError(QString login, qlonglong userId);
    void _emitDeleteUserError(QString login, qlonglong userId);

    void _emitAddGroupError(QString groupName);
    void _emitUpdateGroupError(QString groupName, qlonglong groupId);
    void _emitDeleteGroupError(QString groupName, qlonglong groupId);

    UserManagerPrivate * const d_ptr;

protected slots:
    void _loadSessionListSlot(::Diamesh::SessionList sessionList);
    void _loadUsersSlot(::Diamesh::LongList userIdList);
    void _loadGroupsSlot(::Diamesh::LongList groupIdList);
    void _loadSessionSlot(::Diamesh::SessionPrx sessionPrx);
    void _loadUserSlot(::Diamesh::UserPtr user);
    void _loadGroupSlot(::Diamesh::GroupPtr group);

    // Slots received after a remote call
    void _addUserSlot(::Ice::Long userId, QObject *obj);
    void _cloneUserSlot(::Diamesh::LongList clonedUserIdList, QObject *obj);
    void _getClonedUserSlot(::Diamesh::UserPtr clone, QObject *obj);
    void _updateUserSlot(bool result, QObject *obj);
    void _deleteUserSlot(bool result, QObject *obj);

    void _addGroupSlot(::Ice::Long groupId, QObject *obj);
    void _updateGroupSlot(bool result, QObject *obj);
    void _deleteGroupSlot(bool result, QObject *obj);

    void _getSessionSlot(::Diamesh::SessionPrx sessionPrx);
    void _getUserSlot(::Diamesh::UserPtr user);
    void _getUserUpdateSlot(::Diamesh::UserPtr user);
    void _getGroupSlot(::Diamesh::GroupPtr group);
    void _getGroupUpdateSlot(::Diamesh::GroupPtr group);

    // Slots connected to QtIce cookies
    void _addUserCookieCompleted(qlonglong userId, QString userName, bool result);
    void _cloneUserCookieCompleted(qlonglong userId, QString userName, bool result);
    void _updateUserCookieCompleted(qlonglong userId, QString userName, bool result);
    void _deleteUserCookieCompleted(qlonglong userId, QString userName, bool result);
    void _addGroupCookieCompleted(qlonglong groupId, QString groupName, bool result);
    void _updateGroupCookieCompleted(qlonglong groupId, QString groupName, bool result);
    void _deleteGroupCookieCompleted(qlonglong groupId, QString groupName, bool result);

    // Slots connected to remote signals
    void _sessionCreatedSlot(QString sessionId);
    void _sessionDestroyedSlot(QString sessionId);
    void _userDisconnectedSlot(Ice::Long userId);

    void _userAddedSlot(Ice::Long userId);
    void _userChangedSlot(Ice::Long userId);
    void _userRemovedSlot(Ice::Long userId);

    void _groupAddedSlot(Ice::Long groupId);
    void _groupChangedSlot(Ice::Long groupId);
    void _groupRemovedSlot(Ice::Long groupId);

private:
    friend class Session;
};

} // namespace HnsClient

#endif //HNSCLIENT_USERMANAGER_H
