/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim hnsclient headers
#include "libhnsclient/utility.h"

// hynesim common headers
#include "common/initializer.h"

// Qt headers
#include <QObject>

namespace HnsClient {

HNS_STATIC_HASH_INITIALIZER(sizeUnitHash, SizeUnitHash, HnsClient::BytesUnitEnum, QString)
SizeUnitHash::SizeUnitHash()
{
    insert(HnsClient::Bytes, QObject::tr("B"));
    insert(HnsClient::KibiBytes, QObject::tr("KiB"));
    insert(HnsClient::MebiBytes, QObject::tr("MiB"));
    insert(HnsClient::GibiBytes, QObject::tr("GiB"));
    insert(HnsClient::TebiBytes, QObject::tr("TiB"));
}


/*******************************************************************************
* Utility
*******************************************************************************/

/*! \fn unitFromIndex(BytesUnitEnum index)
 *\brief TODO
 *\param index
 *\param asRate
 *\param rate
 */
QString Utility::unitFromIndex(BytesUnitEnum index, bool asRate, QString rate)
{
    QString result = sizeUnitHash()->value(index);
    if (asRate)
        result.append("/").append(rate);
    return result;
}

/*! \fn buildReadableSize(const qlonglong &size, BytesUnitEnum from, int baseUnit = 1000)
 *\brief TODO
 *\param size
 *\param from
 *\param baseUnit
 */
QPair<QString, BytesUnitEnum> Utility::buildReadableSize(const qlonglong &size, BytesUnitEnum from, int baseUnit)
{
    QPair<QString, BytesUnitEnum> result;
    double finalSize = size;
    if (finalSize == -1)
        return result;
    int unitIndex = from;
    while (finalSize >= baseUnit && unitIndex < sizeUnitHash()->count() - 1) {
        finalSize = finalSize / baseUnit;
        ++unitIndex;
    }
    result.first = QString::number(finalSize, 'f', 2);
    result.second = (BytesUnitEnum)unitIndex;
    return result;
}

/*! \fn makeSizeReadable(const qlonglong &size, BytesUnitEnum from, int baseUnit = 1000)
 *\brief TODO
 *\param size
 *\param from
 *\param baseUnit
 */
QString Utility::makeSizeReadable(const qlonglong &size, BytesUnitEnum from, int baseUnit)
{
    QPair<QString, BytesUnitEnum> result = buildReadableSize(size, from, baseUnit);
    if (result.first.isNull())
        return QObject::tr("-");
    else
        return QString("%1 %2").arg(result.first).arg(unitFromIndex(result.second));
}

/*! \fn makeTimeReadable(int mseconds)
 *\brief TODO
 *\param mseconds
 */
QString Utility::makeTimeReadable(int seconds)
{
    int timeLeft = seconds;
    int days = timeLeft / (60 * 60 * 24);
    timeLeft -= days * (60 * 60 * 24);
    int hours = timeLeft / (60 * 60);
    timeLeft -= hours * (60 * 60);
    int minutes = timeLeft / 60;
    timeLeft -= minutes * 60;

    return QObject::tr("%1 day(s), %2:%3:%4")
            .arg(days)
            .arg(hours, 2, 10, QLatin1Char('0'))
            .arg(minutes, 2, 10, QLatin1Char('0'))
            .arg(timeLeft, 2, 10, QLatin1Char('0'));
}

/*! \fn makePercentageReadable(double used, double total)
 *\brief TODO
 *\param used
 *\param total
 */
QString Utility::makePercentageReadable(double used, double total)
{
    return makePercentageReadable((used * 100 ) / total);
}

/*! \fn makePercentageReadable(double value)
 *\brief TODO
 *\param value
 */
QString Utility::makePercentageReadable(double value)
{
    return QString("%1 %").arg(value, 2, 'f', 1);
}

/*! \fn makeMemoryLoadReadable(qlonglong used, qlonglong total)
 *\brief TODO
 *\param used
 *\param total
 */
QString Utility::makeMemoryLoadReadable(qlonglong used, qlonglong total)
{
    return QString("%1 (%2) of %3")
            .arg(makeSizeReadable(used, HnsClient::KibiBytes))
            .arg(makePercentageReadable(used, total))
            .arg(makeSizeReadable(total, HnsClient::KibiBytes));
}

/*! \fn readableStringValue(std::string value, QString defaultValue = QString())
 *\brief TODO
 *\param value
 *\param defaultValue
 */
QString Utility::readableStringValue(std::string value, QString defaultValue)
{
    if (defaultValue.isNull())
        defaultValue = QObject::tr("Default");

    QString result = QString::fromStdString(value);
    if (result.isEmpty())
        result = defaultValue;
    return result;
}

/*! \fn qStringListFromStringList(Diamesh::StringList)
 *\brief TODO
 *\param
 */
QStringList Utility::qStringListFromStringList(Diamesh::StringList list)
{
    QStringList stringList;
    foreach (std::string str, list) {
        stringList.append(QString::fromStdString(str));
    }
    return stringList;
}

/*! \fn qLongListFromLongList(Diamesh::LongList)
 *\brief TODO
 *\param
 */
QList<qlonglong> Utility::qLongListFromLongList(Diamesh::LongList list)
{
    QList<qlonglong> resultList;
    foreach(Ice::Long value, list) {
        resultList.append(value);
    }
    return resultList;
}

/*! \fn qLongListFromVectorList(::Hns::IdImportedList)
 *\brief TODO
 *\param
 */
QList<qlonglong> Utility::qLongListFromVectorList(Hns::IdImportedList list)
{
    QList<qlonglong> resultList;
    foreach(Ice::Long value, list) {
        resultList.append(value);
    }
    return resultList;
}

/*! \fn qLongSetFromLongList(Diamesh::LongList)
 *\brief TODO
 *\param
 */
QSet<qlonglong> Utility::qLongSetFromLongList(Diamesh::LongList list)
{
    QSet<qlonglong> resultSet;
    foreach(Ice::Long value, list) {
        resultSet.insert(value);
    }
    return resultSet;
}

/*! \fn pixmapFromByteArray(Hns::ByteArray data)
 *\brief TODO
 *\param data
 */
QPixmap Utility::pixmapFromByteArray(Hns::ByteArray data)
{
    QByteArray screenshotByteArray;
    QPixmap screenshot;
    for(Hns::ByteArray::const_iterator it = data.begin(); it != data.end(); ++it)
        screenshotByteArray.append(*it);
    screenshot.loadFromData(screenshotByteArray,Constants::FORMAT_PREVIEW);
    return screenshot;
}

} // namespace HnsClient
