/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim server headers
#include "processrunner.h"

// Qt headers
#include <QDebug>
#include <QMetaObject>
#include <QThread>
#include <QEventLoop>
#include <QTimer>
#include <QElapsedTimer>
#include <QCoreApplication>
#include <QPointer>

namespace HnsServer {
/**************************************************
 *                 ProcessWrapper                 *
 **************************************************/
ProcessWrapper::ProcessWrapper(ProcessRunner *processRunner, const QMetaObject *processMetaObj)
    : _processRunner(processRunner)
    , _process(NULL)
    , _errorFound(false)
    , _processMetaObj(processMetaObj)
{
}

void ProcessWrapper::start()
{
    _outputBuffer.clear();
    _errorBuffer.clear();
    _errorFound = false;

    if (_processMetaObj != 0)
        _process = qobject_cast<QProcess *>(_processMetaObj->newInstance(Q_ARG(QObject *, this)));

    if (!_process || !_processMetaObj)
        _process = new QProcess(this);

    connect(_process, SIGNAL(readyReadStandardError()), this, SLOT(_parseError()));
    connect(_process, SIGNAL(readyReadStandardOutput()), this, SLOT(_parseOutput()));

    connect(_process, SIGNAL(error(QProcess::ProcessError)), this, SLOT(_error()));
    connect(_process, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(_parseOutput()));
    connect(_process, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(_finished(int, QProcess::ExitStatus)));

    _process->start(_processRunner->_program, _processRunner->_arguments);
}

void ProcessWrapper::_parseError()
{
    QByteArray data = _process->readAllStandardError();
    if (data.size()) {
        if (getenv("HNS_DEBUG"))
            qCritical() << "program: " << _processRunner->_program << "args:" << _processRunner->_arguments << "output:" << data;
        _errorBuffer.append(data);
        emit _processRunner->errorRead(data);
    }
}

void ProcessWrapper::_parseOutput()
{
    QByteArray data = _process->readAllStandardOutput();
    if (data.size()) {
        if (getenv("HNS_DEBUG"))
            qDebug() << data;
        _outputBuffer.append(data);
        emit _processRunner->outputRead(data);
    }
}

void ProcessWrapper::_error()
{
    _parseOutput();
    _parseError();
    _errorFound = true;
    qCritical() << _process->errorString() << _process->pid() << _processRunner->_program;
    _processRunner->_executionThread->quit();
    _processRunner->_isRunning = false;
}

void ProcessWrapper::_cancel()
{
    if (_process != NULL) {
        QPointer<QProcess> process(_process);
        _process->terminate();
        if (!_process->waitForFinished(3000) && process)
            process->kill();
        _processRunner->_executionThread->quit();
        _processRunner->_isRunning = false;
    }
}

void ProcessWrapper::_finished(int exitCode, QProcess::ExitStatus exitStatus)
{
    if (_process != NULL) {        
        delete _process;
        _process = NULL;
    }    
    emit finished(exitCode, exitStatus);
}

bool ProcessWrapper::errorFound()
{
    return _errorFound;
}

QString ProcessWrapper::getOutputBuffer()
{
    return _outputBuffer;
}

QString ProcessWrapper::getErrorBuffer()
{
    return _errorBuffer;
}

/**************************************************
 *                 ProcessRunner                  *
 **************************************************/
ProcessRunner::ProcessRunner(QObject *parent, const QMetaObject *processMetaObj)
    : QObject(parent)
    , _isRunning(false)
    , _exitCode(-1)
{
    _executionThread = new QThread(this);
    _processWrapper = new ProcessWrapper(this, processMetaObj);
    qRegisterMetaType<QProcess::ExitStatus>("QProcess::ExitStatus");
    _processWrapper->moveToThread(_executionThread);
    QObject::connect(_executionThread, SIGNAL(started()), _processWrapper, SLOT(start()));
    QObject::connect(this, SIGNAL(cancelProcess()), _processWrapper, SLOT(_cancel()));
    QObject::connect(_processWrapper, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(_finished(int, QProcess::ExitStatus)));
}

ProcessRunner::~ProcessRunner()
{
    if (_isRunning) {
        QMetaObject::invokeMethod(_processWrapper, "_cancel", Qt::BlockingQueuedConnection);
        QEventLoop l;
        connect(this, SIGNAL(finished()), &l, SLOT(quit()));
        l.exec();
    }
    delete _processWrapper;
    delete _executionThread;
}

void ProcessRunner::runAsynchronous(const QString &program, const QStringList &arguments)
{
    _program = program;
    _arguments = arguments;
    _start();
}

qint64 ProcessRunner::runDetached(const QString &program, const QStringList &arguments)
{
    _program = program;
    _arguments = arguments;
    return _startDetached();
}

bool ProcessRunner::runSynchronous(const QString &program, const QStringList &arguments, int timeout)
{
    _program = program;
    _arguments = arguments;
    _start();

    QElapsedTimer timer;
    timer.start();
    while (_isRunning == true) {
        QEventLoop l;
        QTimer::singleShot(100, &l, SLOT(quit()));
        l.exec();
        QCoreApplication::processEvents();

        if (timer.isValid() && timer.hasExpired(timeout)) {
            timer.invalidate();
            cancel();
        }
    }

    bool result = (!_processWrapper->errorFound() && _exitCode == 0);
    return result;
}

void ProcessRunner::cancel()
{
    emit cancelProcess();
}

void ProcessRunner::_beforeStart()
{
}

void ProcessRunner::_start()
{    
    _beforeStart();
    _executionThread->start();
    _isRunning = true;
}

qint64 ProcessRunner::_startDetached()
{
    _beforeStart();
    qint64 pid = 0;
    QProcess::startDetached(_program, _arguments, QString(), &pid);
    return pid;
}

void ProcessRunner::_finished(int exitCode, QProcess::ExitStatus exitStatus)
{
    _exitCode = exitCode;
    _executionThread->quit();
    _executionThread->wait();
    Q_UNUSED(exitStatus);
    emit finished();
    _isRunning = false;
}

int ProcessRunner::exitCode()
{
    return _exitCode;
}

QString ProcessRunner::getOutput()
{
    return _processWrapper->getOutputBuffer();
}

QString ProcessRunner::getError()
{
    return _processWrapper->getErrorBuffer();
}

} // namespace HnsServer
