/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "storagemanager.h"

// hynesim common headers
#include "common/constants.h"

// Qt headers
#include <QDebug>
#include <QDir>
#include <QSettings>
#include <QStringList>

namespace HnsServer {

StorageManager::StorageManager(const QString &settingsFileName, QObject *parent)
    : QObject(parent)
    , _settingsFileName(settingsFileName)
{
    _loadStorages();
}

QString StorageManager::getFullPathFromMixed(const QString &storageType,const QString &mixedPath, const QString &uuid)
{
    QReadLocker locker(&_storageLock);
    QString name;
    QString subPath;
    QStringList splitList = mixedPath.split(":", QString::SkipEmptyParts);
    if (splitList.isEmpty()) {
        qWarning() << tr("Mixed path \"%1\" is invalid").arg(mixedPath);
        return "";
    }
    name = splitList.first();
    if (splitList.size()>2) {
        QStringList joinList;
        int i = 1;
        while (i<splitList.count()) {
            joinList.append(splitList.value(i));
            i++;
        }
        subPath = joinList.join(QString(":"));
    } else if (splitList.size() == 2) {
        subPath = splitList.at(1);
    } else if (splitList.size()<2) {
        subPath = QString();
    }
    if (!(_typeStorageHash.value(storageType).contains(name))) {
        qCritical() << tr("Mixed path %1 does not belong to any file storage").arg(mixedPath);
        return "";
    }
    return QDir::cleanPath(QString("%1/%2/%3").arg(_typeStorageHash.value(storageType).value(name)).arg(subPath).arg(uuid));
}

bool StorageManager::_loadStorages()
{
    QWriteLocker locker(&_storageLock);
    QSettings settings(_settingsFileName, QSettings::IniFormat, this);
    settings.beginGroup(Hns::Constants::SETTINGS_HYNESIM_GROUP);

    int nbStorages = settings.beginReadArray(Hns::Constants::SETTINGS_FILE_STORAGE_ARRAY);
    for (int i = 0 ; i < nbStorages ; ++i) {
        settings.setArrayIndex(i);
        if (settings.childKeys().contains(Hns::Constants::SETTINGS_FILE_STORAGE_TYPE)) {
            QString type = settings.value(Hns::Constants::SETTINGS_FILE_STORAGE_TYPE).toString();
            QString storageName = settings.value("name").toString();
            if (storageName.isEmpty()) {
                qCritical() << tr("The storage name is empty.");
                return false;
            }
            QString storagePath = settings.value("path").toString();
            if (storagePath.isEmpty()) {
                qCritical() << tr("The storage path is empty.");
                return false;
            }

            if (_typeStorageHash.contains(type) && _typeStorageHash[type].contains(storageName)) {
                qCritical() << tr("File storage %1 is already defined").arg(storageName);
                return false;
            }

            QDir storageDir(storagePath);
            if (!storageDir.exists()) {
                qCritical() << tr("File storage %1 (%2) does not exist").arg(storagePath).arg(storageName);
                return false;
            }

            if (_typeStorageHash.contains(type)) {
                _typeStorageHash[type][storageName] = storageDir.absolutePath();
            } else {
               QHash<QString,QString>  storageHash;
               storageHash[storageName] = storageDir.absolutePath();
                _typeStorageHash[type]=storageHash;
            }
        }
    }
    settings.endArray();
    settings.endGroup();
    return true;
}

} // namespace HnsServer
