/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "catalogI.h"
#include "catalogdbstorage.h"
#include "catalogfilestorage.h"
#include "masterI.h"
#include "entrywrapper.h"

// hynesim common headers
#include "common/constants.h"
#include "common/rightslist.h"

// Generated hynesim headers
#include "common/entrypropertiesmanager.h"

// diamesh headers
#include <diamesh/accessrightshelper.h>
#include <diamesh/aclmanagerI.h>
#include <diamesh/sessionManagerI.h>
#include <diamesh/hsetmanagerI.h>
#include <diamesh/usermanager.h>

// ZeroC Ice headers
#include <Ice/ObjectAdapter.h>
#include <Ice/Communicator.h>
#include <Ice/Identity.h>

// Qt headers
#include <QDir>
#include <QSettings>
#include <QDebug>
#include <QDateTime>
#include <QUuid>
#include <QReadWriteLock>

namespace Hns {

CatalogI::CatalogI(const Ice::CommunicatorPtr &communicator, Ice::ObjectAdapterPtr adapter, const QString &settingsFileName)
    : _communicator(communicator)
    , _adapter(adapter)
    , _dbStorage(NULL)
    , _settingsFileName(settingsFileName)
{
    QObject::connect(MasterI::instance()->aclManager(), SIGNAL(aclAdded(QString,QString)), this, SLOT(_aclAddedSlot(QString, QString)));
    QObject::connect(MasterI::instance()->aclManager(), SIGNAL(aclChanged(QString,QString)), this, SLOT(_aclChangedSlot(QString, QString)));
    QObject::connect(MasterI::instance()->aclManager(), SIGNAL(aclRemoved(QString,QString)), this, SLOT(_aclRemovedSlot(QString, QString)));
}

CatalogI::~CatalogI()
{
    delete _dbStorage;
    foreach (QString storagePath, _fileStoragesHash.keys()) {
        _removeFileStorage(storagePath);
    }
}

CatalogPrx CatalogI::proxy()
{
  return CatalogPrx::uncheckedCast(_adapter->createProxy(_id));
}

Diamesh::LongList CatalogI::getEntries(const ::Ice::Current &current)
{
    QReadLocker locker(&_entriesLock);
    std::list<Ice::Long> list;

    foreach (CatalogItemWrapper *wrapper, _catalogItemHash) {
        if (!qobject_cast<EntryWrapper *>(wrapper))
            continue;
        if (wrapper->testACL(Rights::ENTRY_SUFFIX_READ, current, false))
            list.push_back(wrapper->catalogItem()->id);
    }
    return list;
}

EntryPtr CatalogI::getEntry(Ice::Long entryId, const ::Ice::Current &current)
{
    QReadLocker locker(&_entriesLock);
    if (!_catalogItemHash.contains(entryId)) {
        qCritical() << "getEntry:" << tr("No entry with the id %1 has been found").arg(entryId);
        return NULL;
    }
    if (!_catalogItemHash.value(entryId)->testACL(Rights::ENTRY_SUFFIX_READ, current))
        return NULL;

    return EntryPtr::dynamicCast(_catalogItemHash.value(entryId)->catalogItem());
}

Ice::Long CatalogI::addEntry(const EntryPtr &entry, const ::Ice::Current &current)
{
    Diamesh::UserPtr currentUser = MasterI::instance()->sessionManager()->currentUser(current);
    return addEntry(entry, currentUser);
}

bool CatalogI::deleteEntry(const EntryPtr &entry, const ::Ice::Current &current)
{
    QWriteLocker locker(&_entriesLock);

    qlonglong entryId = entry->id;

    if (!_catalogItemHash.contains(entryId)) {
        qCritical() << "deleteEntry:" << tr("The entry %1 was not found in the catalog").arg(entryId);
        return false;
    }

    if (entryIsLocked(entryId, current)) {
        qCritical() << "deleteEntry:" << tr("The entry %1 is locked. Unable to delete it.").arg(entryId);
        return false;
    }
    if (!_catalogItemHash.value(entryId)->testACL(Rights::ENTRY_SUFFIX_DELETE, current))
            return false;

    CatalogFileStorage *fileStorage = 0;
    if (entry->path == "")
        fileStorage = getDefaultFileStorage();
    else
        fileStorage = _getEntryFileStorage(entry);

    if (!_catalogItemHash.value(entryId)->remove(_dbStorage, fileStorage))
        return false;

    CatalogItemWrapper *wrapper = _catalogItemHash.take(entryId);

    delete wrapper;

    Diamesh::ACLList acls = MasterI::instance()->aclManager()->getACLsForTarget(entry);
    foreach (Diamesh::AccessControlList acl, acls)
        MasterI::instance()->aclManager()->removeACL(acl.target, acl.subject);

    qDebug() << tr("Entry %1 has been deleted").arg(entryId);

    emit entryRemoved(entryId);
    return true;
}

bool CatalogI::updateEntry(const EntryPtr &entry, const ::Ice::Current &current)
{
    if(entryIsLocked(entry->id)) {
        qCritical() << tr("Entry %1 is locked, unable to update it directly through the catalog").arg(entry->id);
        return false;
    }

    EntryPropertiesManagerPrx propertiesManager = MasterI::instance()->getEntryPropertiesManager();
    if (!propertiesManager) {
        qCritical() << tr("Could not get entry properties manager");
        return false;
    }

    EntryPtr newEntry = NULL;
    if (propertiesManager->canValidateEntry(entry)) {
       newEntry = propertiesManager->validateEntry(entry);
       if (!newEntry) {
           qCritical() << tr("Entry %1 is not valid. Unable to set it as entity entry").arg(entry->uuid.c_str());
           return false;
       }
    } else {
        newEntry = entry;
    }
    QReadLocker locker(&_entriesLock);
    if (!_catalogItemHash.value(entry->id)->testACL(Rights::ENTRY_SUFFIX_UPDATE_ENTRY, current))
        return false;
    locker.unlock();

    return updateEntry(newEntry);
}

bool CatalogI::lockEntry(Ice::Long entryId, LockType lockType)
{
    QWriteLocker locker(&_lockedEntriesLock);

    if (_lockedEntriesHash.contains(entryId)) {
        qCritical() << tr("Entry %1 is already locked, unable to lock it").arg(entryId);
        return false;
    }

    _lockedEntriesHash.insert(entryId, lockType);
    emit entryLocked(entryId, lockType);
    return true;
}

bool CatalogI::unlockEntry(Ice::Long entryId)
{
    QWriteLocker locker(&_lockedEntriesLock);

    if (!_lockedEntriesHash.contains(entryId)) {
        qCritical() << tr("Entry %1 is already unlocked, unable to unlock it").arg(entryId);
        return false;
    }

    _lockedEntriesHash.remove(entryId);
    emit entryUnlocked(entryId);
    return true;
}

bool CatalogI::entryIsLocked(Ice::Long entryId, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QReadLocker locker(&_lockedEntriesLock);
    return _lockedEntriesHash.contains(entryId);
}

LockTypeLongMap CatalogI::getLockedEntries(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QReadLocker locker(&_lockedEntriesLock);
    LockTypeLongMap ret;
    for (QHash<Ice::Long, LockType>::const_iterator it = _lockedEntriesHash.constBegin() ; it != _lockedEntriesHash.constEnd() ; ++it) {
        ret[it.key()] = it.value();
    }
    return ret;
}

LockType CatalogI::getLockType(Ice::Long entryId)
{
    QReadLocker locker(&_lockedEntriesLock);
    return _lockedEntriesHash.value(entryId);
}

Diamesh::StringStringMap CatalogI::getFileStorages(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QReadLocker locker(&_fileStoragesLock);
    Diamesh::StringStringMap map;
    foreach (CatalogFileStorage* storage, _fileStoragesHash) {
        map[storage->storagePath().toStdString()] = storage->storageName().toStdString();
    }
    return map;
}

std::string CatalogI::getFullEntryPathFromMixed(const std::string &mixedPath, const std::string &uuid, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QString uuidDir;
    if (!QString::fromStdString(uuid).isEmpty()){
        uuidDir = "/" + QString::fromStdString(uuid) + "/";
    } else {
        uuidDir = QString();
    }
    QStringList splitList = QString::fromStdString(mixedPath).split(":");
    if (splitList.size() < 2){
        qCritical()<<tr("Unable to get the full path of %1").arg(QString::fromStdString(mixedPath));
        return "";
    } else if (splitList.size() >= 2){
        int i=1;
        QStringList toJoin;
        while (i<splitList.size()){
            toJoin.append(splitList.value(i));
            i++;
        }
        QString endPath = toJoin.join(":");
        QString storageName = splitList.first();
        if (!_fileStoragesHash.contains(storageName)) {
            qCritical() << tr("File storage %1 does not exist").arg(storageName) << "(" << mixedPath.c_str() << ")";
            return "";
        }
        QDir storageDir(_fileStoragesHash.value(storageName)->storagePath());
        return QDir::cleanPath(storageDir.absolutePath() + "/" +endPath+ uuidDir ).toStdString();
    }
    return "";
}

Diamesh::ACLList CatalogI::getACLsForCatalogItem(Ice::Long catalogItemId, const Ice::Current &current)
{
    Q_UNUSED(current);
    return MasterI::instance()->aclManager()->getACLsForTarget(getCatalogItem(catalogItemId));
}

bool CatalogI::setACLsForCatalogItem(Ice::Long catalogItemId, const Diamesh::ACLList &aclList, const Ice::Current &current)
{
    bool result = false;
    QList<Diamesh::AccessControlList> aclQList = QList<Diamesh::AccessControlList>::fromStdList(aclList);
    Diamesh::ACLManagerI * aclManager = MasterI::instance()->aclManager();
    CatalogItemPtr catalogItem = getCatalogItem(catalogItemId);
    if (catalogItem) {
        result = true;
        QList<Diamesh::AccessControlList> sourceAcls = QList<Diamesh::AccessControlList>::fromStdList(aclManager->getACLsForTarget(catalogItem));
        foreach(const Diamesh::AccessControlList &sourceAcl, sourceAcls) {
            bool aclFound = false;
            QMutableListIterator<Diamesh::AccessControlList> iterator(aclQList);
            while (iterator.hasNext()) {
                Diamesh::AccessControlList acl = iterator.next();
                if (acl.subject == sourceAcl.subject && acl.target == sourceAcl.target) {
                    aclFound = true;
                    if (acl.hSetId != sourceAcl.hSetId || acl.rights != sourceAcl.rights)
                        result &= aclManager->setACL(acl.target, acl.subject, acl.rights, acl.hSetId, current);
                    iterator.remove();
                    break;
                }
            }
            if (!aclFound) {
                result &= aclManager->removeACL(sourceAcl.target, sourceAcl.subject, current);
            }
        }
        foreach(const Diamesh::AccessControlList &acl, aclQList) {
            result &= aclManager->setACL(acl.target, acl.subject, acl.rights, acl.hSetId, current);
        }
    }
    if (!result) {
        RightError exception;
        QString errorMessage = QObject::tr("Unable to update right");
        exception.message = errorMessage.toStdString();
        throw exception;
    }
    return result;
}

bool CatalogI::refresh(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QWriteLocker fileStoragesLocker(&_fileStoragesLock);
    QWriteLocker usedUuidLocker(&_usedUuidLock);
    QWriteLocker entriesLocker(&_entriesLock);

    QHash<QString, QList<EntryPtr> > newEntriesHash;
    QList<QString> newEntriesUuidList;
    // Get entries from all file storages
    foreach (CatalogFileStorage *storage, _fileStoragesHash) {
        qDebug() << tr("Updating storage %1").arg(storage->storagePath());
        QList<EntryPtr> newEntries = storage->getEntries();

        foreach (EntryPtr entry, newEntries) {
            QString entryPath = getFullEntryPathFromMixed(entry->path, entry->uuid).c_str();
            QString entryUuid = entry->uuid.c_str();
            if (newEntriesHash.contains(entryPath)) {
                qWarning() << tr("Error while refreshing storage %1: the entry %2 has already been found.").
                        arg(storage->storagePath()).
                        arg(entryPath);
            }
            if (newEntriesUuidList.contains(entryUuid))
                qWarning() << tr("Error while refreshing storage %1: the entry %2 has already been found.").
                        arg(storage->storagePath()).
                        arg(entryUuid);

            newEntriesHash[entryPath].append(entry);
            newEntriesUuidList.append(entryUuid);
            _hashUuidEntry.insert(entryUuid, entry);
        }
    }
    _handleUuidConflicts(_hashUuidEntry);

    // Mark entries that are not in the file storage any more as unavailable
    foreach (CatalogItemWrapper *catalogItemWrapper, _catalogItemHash) {
        if (!qobject_cast<EntryWrapper *>(catalogItemWrapper))
            continue;
        EntryPtr dbEntry = EntryPtr::dynamicCast(catalogItemWrapper->catalogItem());

        qlonglong entryId = dbEntry->id;
        if (dbEntry->path == "") { // the old entry was already not found
            if (_hashUuidEntry.contains(QString::fromStdString(dbEntry->uuid))) // The entity has been moved back in the file storage
                dbEntry->path = _hashUuidEntry.value(QString::fromStdString(dbEntry->uuid))->path;
            else
                continue;
        }
        QString entryPath = getFullEntryPathFromMixed(dbEntry->path, dbEntry->uuid).c_str();
        if (!newEntriesHash.contains(entryPath)) {
            qWarning() << tr("Entry %1 (%2) is not in file storage anymore").arg(entryId).arg(entryPath);
            dbEntry->path = "";
            _markEntryAsInvalid(dbEntry);
            _usedUuidList.removeAll(dbEntry->uuid.c_str());
        } else {
            // If the entries are different, update them
            EntryPtr storageEntry = newEntriesHash[entryPath].first();
            _handleEntryLoading(storageEntry, dbEntry);

            newEntriesHash[entryPath].removeFirst();
            if (newEntriesHash[entryPath].isEmpty())
                newEntriesHash.remove(entryPath);

            CatalogItemWrapper *catalogItemWrapperStorage = CatalogItemWrapperFactory::createWrapper(QString::fromStdString(storageEntry->ice_id()));
            if (catalogItemWrapperStorage) {
                catalogItemWrapperStorage->setCatalogItem(storageEntry);
                catalogItemWrapperStorage->init();
                _catalogItemHash.insert(entryId, catalogItemWrapperStorage);
                catalogItemWrapper->deleteLater();
            }
        }
    }

    // Insert the new entries
    foreach (QList<EntryPtr> entryList, newEntriesHash) {
        foreach (EntryPtr entry, entryList) {
            if (!_addEntry(entry)) {
                qCritical() << tr("Unable to add new entry while refreshing the catalog");
                return false;
            } else {
                qDebug() << "add Entry" << entry->shortName.c_str();
            }
            _usedUuidList.append(entry->uuid.c_str());
        }
    }

    foreach (CatalogItemWrapper *catalogItemWrapper, _catalogItemHash) {
        qlonglong parentId = catalogItemWrapper->catalogItem()->parentId;
        if (parentId != 0 && !_catalogItemHash.contains(parentId)) {
            qWarning() << tr("A catalog item (%1) was found with an invalid parentId (%2), setting its parentId to 0")
                          .arg(catalogItemWrapper->catalogItem()->id)
                          .arg(parentId);
            catalogItemWrapper->catalogItem()->parentId = 0;
        }
    }

    return true;
}

std::string CatalogI::getUnusedUUID(const Ice::Current &current)
{
    Q_UNUSED(current)
    return generateUnusedUUID().toStdString();
}

Ice::Long CatalogI::addEntry(const EntryPtr &entry, const Diamesh::UserPtr &currentUser, int flags)
{
    QWriteLocker locker(&_entriesLock);
    QWriteLocker fsLocker(&_fileStoragesLock);
    return _addEntry(entry, currentUser, flags);
}

bool CatalogI::updateEntry(const EntryPtr &entry)
{
    QReadLocker locker(&_entriesLock);
    return updateEntryNoLock(entry);
}

bool CatalogI::updateEntryNoLock(const EntryPtr &entry)
{
    qlonglong entryId = entry->id;
    if (!_catalogItemHash.contains(entryId)) {
        qCritical() << "updateEntryNoLock:" << tr("The entry %1 was not found in the catalog").arg(entryId);
        return false;
    }

    _updateModificationTime(entry);

    EntryPtr oldEntry = EntryPtr::dynamicCast(_catalogItemHash.value(entryId)->catalogItem());
    /*truc topo à mettre*/
    CatalogFileStorage *fileStorage = nullptr;
    if (oldEntry->path != entry->path || oldEntry->uuid != entry->uuid) {
        if (oldEntry->path == "")
            fileStorage = getDefaultFileStorage();
        else
            fileStorage = _getEntryFileStorage(oldEntry);
    } else {
        fileStorage = _getEntryFileStorage(oldEntry);
    }

    if (!_catalogItemHash.value(entryId)->update(_dbStorage, fileStorage, entry)) {
        qCritical() << tr("Unable to update entry %1").arg(entryId);
        return false;
    }
    emit entryChanged(entryId);
    return true;
}

bool CatalogI::copyEntryResources(const EntryPtr &fromEntry, const EntryPtr &toEntry)
{
    Q_UNUSED(fromEntry);
    Q_UNUSED(toEntry);
    return true;
}

EntryPtr CatalogI::getEntryByUuid(const QString &entryUuid)
{
    QReadLocker locker(&_entriesLock);
    foreach (CatalogItemWrapper *wrapper, _catalogItemHash) {
        if (!qobject_cast<EntryWrapper *>(wrapper))
            continue;
        EntryPtr entry = EntryPtr::dynamicCast(wrapper->catalogItem());
        if (entry->uuid.c_str() == entryUuid)
            return entry;
    }
    return NULL;
}

CatalogFileStorage* CatalogI::getDefaultFileStorage()
{
    if (_fileStoragesHash.size() == 0)
        return NULL;
    else
        return _fileStoragesHash.begin().value();
}

QString CatalogI::generateUnusedUUID()
{
    QWriteLocker locker(&_usedUuidLock);
    QString uuid;
    bool uuidIsUsed = true;

    while (uuidIsUsed) {
        uuidIsUsed = false;
        // remove opening and closing curly braces and make it lower.
        uuid = QUuid::createUuid().toString().mid(1, 36).toLower();
        if (_usedUuidList.contains(uuid))
            uuidIsUsed = true;
    }
    _usedUuidList.append(uuid);
    return uuid;
}

const QString & CatalogI::getSettingName()
{
    return _settingsFileName;
}

const std::string & CatalogI::idName()
{
    return _id.name;
}

CatalogItemPtr CatalogI::getCatalogItem(qlonglong id)
{
    QReadLocker locker(&_entriesLock);
    if (!_catalogItemHash.contains(id)) {
        qCritical() << "getCatalogItem:" << tr("No catalogItem with the id %1 has been found").arg(id);
        return NULL;
    }

    return _catalogItemHash.value(id)->catalogItem();
}

CatalogItemPtr CatalogI::catalogItemByUuid(const QString &uuid)
{
    /*WARNING: this method is not protected by any locker*/
    foreach (CatalogItemWrapper *wrapper, _catalogItemHash) {
        EntryPtr entry = EntryPtr::dynamicCast(wrapper->catalogItem());
        if (entry && QString::fromStdString(entry->uuid) == uuid)
            return entry;
    }
    return 0;
}

CatalogItemWrapper *CatalogI::catalogItem(qlonglong id)
{
    QReadLocker locker(&_entriesLock);
    return _catalogItemHash.value(id);
}

void CatalogI::_aclAddedSlot(QString target, QString subject)
{
    Q_UNUSED(subject)
    _checkACL(target);
}

void CatalogI::_aclChangedSlot(QString target, QString subject)
{
    Q_UNUSED(subject)
    _checkACL(target);
}

void CatalogI::_aclRemovedSlot(QString target, QString subject)
{
    Q_UNUSED(subject)
    _checkACL(target);
}

bool CatalogI::_init()
{
    QList<CatalogItemWrapper *> wrappers = _dbStorage->getCatalogItems(_dbtypes);
    foreach (CatalogItemWrapper *wrapper, wrappers)
        _catalogItemHash.insert(wrapper->catalogItem()->id, wrapper);

    if(!_loadFileStorages())
        return false;

    return true;
}

Ice::Long CatalogI::_addEntry(const EntryPtr &entry, const Diamesh::UserPtr &currentUser, int flags)
{
    if(!entry->creationDateTime)
        entry->creationDateTime = _currentTime();

    _updateModificationTime(entry);

    CatalogFileStorage *fileStorage = 0;
    if (entry->path == "") {
        fileStorage = getDefaultFileStorage();
    } else {
        fileStorage = _getEntryFileStorage(entry);
    }

    CatalogItemWrapper *wrapper = CatalogItemWrapperFactory::createWrapper(QString::fromStdString(entry->ice_id()));
    if (!wrapper->create(_dbStorage, fileStorage, entry, currentUser))
        return 0;
    wrapper->init();

    qlonglong entryId = wrapper->catalogItem()->id;
    _catalogItemHash.insert(entryId, wrapper);

    if (flags & LockEntry)
        lockEntry(entryId);

    emit entryAdded(entryId);
    return entryId;
}

bool CatalogI::_loadFileStorages()
{
    QSettings settings(_settingsFileName, QSettings::IniFormat, this);
    settings.beginGroup(Constants::SETTINGS_HYNESIM_GROUP);

    int nbStorages = settings.beginReadArray(Constants::SETTINGS_FILE_STORAGE_ARRAY);
    for (int i = 0 ; i < nbStorages ; ++i) {
        settings.setArrayIndex(i);
        if (settings.childKeys().contains(Constants::SETTINGS_FILE_STORAGE_TYPE) &&
            settings.value(Constants::SETTINGS_FILE_STORAGE_TYPE) == _storageType) {
            QString storagePath = settings.value(Constants::SETTINGS_FILE_STORAGE_PATH).toString();
            if (storagePath.isEmpty()) {
                qCritical() << tr("The storage path is empty.");
                return false;
            }
            QString storageName = settings.value(Constants::SETTINGS_FILE_STORAGE_NAME).toString();
            if (storageName.isEmpty()) {
                qCritical() << tr("The storage name is empty.");
                return false;
            }
            if (!_addFileStorage(storagePath, storageName)) {
                qCritical() << tr("The file storage %1 could not be added(%2)").arg(storageName).arg(storagePath);
                return false;
            }
        }
    }
    settings.endArray();
    settings.endGroup();

    refresh();
    return true;
}

bool CatalogI::_addFileStorage(const QString &storagePath, const QString &storageName)
{
    qDebug() << tr("Adding file storage %1").arg(storagePath);
    if (_fileStoragesHash.contains(storageName)) {
        qCritical() << tr("Unable to add the file storage %1, as it already exists.").arg(storagePath);
        return false;
    }

    CatalogFileStorage * fileStorage = new CatalogFileStorage(storagePath, storageName, _storageDocType);
    _fileStoragesHash[storageName] = fileStorage;
    return true;
}

CatalogFileStorage* CatalogI::_getFileStorage(const QString &entryMixedPath)
{
    QString name = entryMixedPath.split(":").first();
    if (!_fileStoragesHash.contains(name)) {
        qCritical() << tr("Unable to find a file storage for entry path %1").arg(entryMixedPath) << _fileStoragesHash.keys();
        return NULL;
    }
    return _fileStoragesHash[name];
}

bool CatalogI::_removeFileStorage(const QString &storageName)
{
    qDebug() << tr("Removing file storage %1").arg(storageName);
    if (_fileStoragesHash.contains(storageName)) {
        qCritical() << tr("Unable to remove unexisting file storage %1.").arg(storageName);
        return false;
    }

    delete _fileStoragesHash[storageName];
    _fileStoragesHash.remove(storageName);

    return true;
}

CatalogFileStorage * CatalogI::_getEntryFileStorage(const EntryPtr &entry)
{
    return _getFileStorage(entry->path.c_str());
}

std::list<std::string> CatalogI::_getSubDirList(const QString &dirPath)
{
    std::list<std::string> list;

    QDir dir(dirPath);
    if (!dir.exists()) {
        qCritical() << tr("Directory %1 does not exist, unable to get sub directories").arg(dirPath);
        return list;
    }

    dir.setFilter(QDir::Dirs | QDir::NoDotAndDotDot);
    dir.setSorting(QDir::Name);
    QFileInfoList dirList = dir.entryInfoList();
    for (int i = 0; i < dirList.size(); ++i) {
        QString subDirName = dirList.at(i).baseName();
        if (QUuid(subDirName).isNull()) {

            list.push_back(QString("/%1").arg(subDirName).toStdString());

            QString subDirFullPath = dirList.at(i).absoluteFilePath();
            std::list<std::string> subList = _getSubDirList(subDirFullPath);

            for(std::list<std::string>::iterator it = subList.begin(); it != subList.end(); it++)
                list.push_back(QString("/%1%2").arg(subDirName).arg((*it).c_str()).toStdString());
        }
    }

    return list;
}

qint64 CatalogI::_currentTime()
{
    return QDateTime::currentDateTime().toMSecsSinceEpoch();
}

void CatalogI::_updateModificationTime(const EntryPtr &entry)
{
    entry->lastModificationDateTime = _currentTime();
}

void CatalogI::_handleEntryLoading(EntryPtr storageEntry, EntryPtr dbEntry)
{
    storageEntry->id = dbEntry->id;
    storageEntry->parentId = dbEntry->parentId;
    storageEntry->hSetId = dbEntry->hSetId;
    if (storageEntry->lastModificationDateTime < dbEntry->lastModificationDateTime)
        storageEntry->lastModificationDateTime = dbEntry->lastModificationDateTime;
}

qlonglong CatalogI::_entryIdFromTarget(const QString &target)
{
    qlonglong result = -1;
    QPair<QString, qlonglong> details = MasterI::instance()->aclManager()->helper()->targetDetails(target);
    if (_aclTargetTypes.contains(details.first))
        result = details.second;
    return result;
}

void CatalogI::_checkACL(const QString &target)
{
    qlonglong entryId = _entryIdFromTarget(target);
    if (entryId != -1) {
        /*the call to _hasEntry is made to avoid qcritical messages when the entry has already been deleted*/
        if (!_hasEntry(entryId))
            return;
        EntryPtr entry = getEntry(entryId);
        if (entry)
            emit entryACLsChanged(entryId, getACLsForCatalogItem(entryId));
    }
}

bool CatalogI::_hasEntry(qlonglong entryId)
{
    QReadLocker locker(&_entriesLock);
    if (_catalogItemHash.contains(entryId))
        return true;
    return false;
}

} // namespace Hns
