/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "catalogdbstorage.h"
#include "tagmanagerI.h"
#include "catalogitemwrapper.h"
#include "catalogI.h"
#include "masterI.h"

// hynesim common headers
#include <common/constants.h>
#include <common/topologyentry.h>
#include <common/folder.h>
#include <common/domainentry.h>
#include <common/dynamipsrouterentry.h>
#include <common/hybridnetcardentry.h>
#include <common/switchentry.h>
#include <common/topologygateentry.h>
#include <common/wifiapentry.h>
#include <common/wireentry.h>

// diamesh headers
#include <diamesh/sqldatabase.h>

// Qt headers
#include <QSettings>
#include <QDebug>
#include <QSqlField>

namespace Hns {

CatalogDBStorage::CatalogDBStorage(const QString &settingsFileName, QObject *parent)
    : QObject(parent)
{
    _createCatalogItemHash.insert(CatalogI::EntityType, &CatalogDBStorage::_createEntityEntry);
    _createCatalogItemHash.insert(CatalogI::TopologyType, &CatalogDBStorage::_createTopologyEntry);
    _createCatalogItemHash.insert(CatalogI::FolderType, &CatalogDBStorage::_createFolder);

    _addCatalogItemHash.insert(QString::fromStdString(TopologyEntry::ice_staticId()), &CatalogDBStorage::_addTopologyEntry);
    _addCatalogItemHash.insert(QString::fromStdString(EntityEntry::ice_staticId()), &CatalogDBStorage::_addEntityEntry);
    _addCatalogItemHash.insert(QString::fromStdString(DomainEntry::ice_staticId()), &CatalogDBStorage::_addEntityEntry);
    _addCatalogItemHash.insert(QString::fromStdString(DynamipsRouterEntry::ice_staticId()), &CatalogDBStorage::_addEntityEntry);
    _addCatalogItemHash.insert(QString::fromStdString(HybridNetcardEntry::ice_staticId()), &CatalogDBStorage::_addEntityEntry);
    _addCatalogItemHash.insert(QString::fromStdString(SwitchEntry::ice_staticId()), &CatalogDBStorage::_addEntityEntry);
    _addCatalogItemHash.insert(QString::fromStdString(TopologyGateEntry::ice_staticId()), &CatalogDBStorage::_addEntityEntry);
    _addCatalogItemHash.insert(QString::fromStdString(WifiAPEntry::ice_staticId()), &CatalogDBStorage::_addEntityEntry);
    _addCatalogItemHash.insert(QString::fromStdString(WireEntry::ice_staticId()), &CatalogDBStorage::_addEntityEntry);
    _addCatalogItemHash.insert(QString::fromStdString(Folder::ice_staticId()), &CatalogDBStorage::_addFolder);

    QSettings settings(settingsFileName, QSettings::IniFormat, this);
    settings.beginGroup(Constants::SETTINGS_HYNESIM_GROUP);
    QString dbName(settings.value("catalog/dbName", "/tmp/hnsCatalogI.db").toString());
    _database = new Diamesh::SqlDatabase("QSQLITE", dbName);
    settings.endGroup();

    _tagManager = MasterI::instance()->tagManager();
}

CatalogDBStorage::~CatalogDBStorage()
{
}

qlonglong CatalogDBStorage::addCatalogItem(const CatalogItemPtr &catalogItem)
{
    QString iceId = QString::fromStdString(catalogItem->ice_id());
    if (!_addCatalogItemHash.contains(iceId)) {
        qCritical() << tr("This database cannot add this type of catalog item: %1").arg(iceId);
        return 0;
    }
    addCatalogItemMethod method = _addCatalogItemHash.value(iceId);
    qlonglong ret = (this->*method)(catalogItem);
    return ret;
}

bool CatalogDBStorage::deleteEntry(const EntryPtr &entry)
{
    qulonglong entryId = entry->id;
    if (!entryId) {
        qCritical() << tr("Entry id is not defined, unable to delete it from database");
        return false;
    }
    QString query = QString("DELETE FROM elements WHERE id=?");

    if (_database->executeQuery(query, qlonglong(entryId)) == -1) {
        qCritical() << tr("The entry could not be removed from the catalog");
        return false;
    }

    if (entry->tagList.size() != 0)
        _tagManager->removeTags(entryId);

    return true;
}

bool CatalogDBStorage::deleteFolder(const FolderPtr folder)
{
    qlonglong folderId = folder->id;
    QString query("DELETE FROM elements WHERE id=?");
    if (_database->executeQuery(query, qlonglong(folderId)) < 0) {
        qCritical() << tr("Unable to remove folder %1 from database").arg(folder);
        return false;
    }
    if (folder->tagList.size() != 0)
        _tagManager->removeTags(folderId);
    return true;
}

bool CatalogDBStorage::updateEntry(const EntryPtr &entry, const EntryPtr &oldEntry)
{
    qlonglong entryId = entry->id;
    if (!entryId) {
        qCritical() << tr("Entry id is not defined, unable to update it in the database");
        return false;
    }
    QString query("UPDATE elements SET shortName=?, "
                  "longName=?, "
                  "description=?, "
                  "creationDateTime=?, "
                  "lastModificationDateTime=?, "
                  "path=?, "
                  "uuid=?, "
                  "hsetid=?, "
                  "parentId=? "
                  "WHERE id=?");

    QVariantList values;
    values.append(QString::fromStdString(entry->shortName));
    values.append(QString::fromStdString(entry->longName));
    values.append(QString::fromStdString(entry->description));
    values.append(qlonglong(entry->creationDateTime));
    values.append(qlonglong(entry->lastModificationDateTime));
    values.append(QString::fromStdString(entry->path));
    values.append(QString::fromStdString(entry->uuid));
    if (entry->hSetId == -1)
        values.append(int(oldEntry->hSetId));
    else
        values.append(int(entry->hSetId));
    values.append(qlonglong(entry->parentId));
    values.append(entryId);

    if (_database->executeQuery(query, values) == -1) {
        qCritical() << tr("Error while updating the entry %1 in database").arg(entryId);
        return false;
    }

    _tagManager->updateTags(entryId, entry->tagList);

    return true;
}

bool CatalogDBStorage::updateFolder(const FolderPtr &newFolder, const FolderPtr &oldFolder)
{
    QString query=QString("UPDATE elements SET shortName=?, longName=?, "
                          "description=?, parentId=?, hsetid=? "
                          "WHERE id=?");
    QVariantList values;
    values.append(QString::fromStdString(newFolder->shortName));
    values.append(QString::fromStdString(newFolder->longName));
    values.append(QString::fromStdString(newFolder->description));
    values.append(qlonglong(newFolder->parentId));
    values.append(int(newFolder->hSetId));
    values.append(qlonglong(newFolder->id));

    if (_database->executeQuery(query, values) < 0) {
        qCritical() << tr("Error while updating folder %1 in database").arg(newFolder->id);
        return false;
    }

    _tagManager->updateTags(newFolder->id, newFolder->tagList);
    Q_UNUSED(oldFolder)
    return true;
}

QList<CatalogItemWrapper *> CatalogDBStorage::getCatalogItems(QList<int> types)
{
    QList<CatalogItemWrapper *> ret;
    foreach (int type, types) {
        QList<QSqlRecord> sqlResults;
        if (_database->executeQuery("SELECT * FROM elements WHERE type=?", QVariant(type), &sqlResults) < 0) {
            qCritical() << tr("Error while getting existing database catalog items");
            return ret;
        }
        foreach (const QSqlRecord &rec, sqlResults) {
            if (!_createCatalogItemHash.contains(type)) {
                qCritical() << tr("The database cannot handle this type of catalog item: %1").arg(type);
                continue;
            }
            createCatalogItemMethod method = _createCatalogItemHash.value(type);
            CatalogItemWrapper *wrapper = (this->*method)(rec);
            if (!wrapper) {
                qCritical() << tr("Unable to deserialize this type: %1").arg(type);
                continue;
            }
            ret.append(wrapper);
        }
    }
    return ret;
}

CatalogItemWrapper *CatalogDBStorage::_createEntityEntry(const QSqlRecord &record)
{
    EntityEntryPtr entityEntry = new EntityEntry();
    entityEntry->id = record.value("id").toLongLong();
    entityEntry->uuid = record.value("uuid").toString().toStdString();
    entityEntry->shortName = record.value("shortName").toString().toStdString();
    entityEntry->longName = record.value("longName").toString().toStdString();
    entityEntry->description = record.value("description").toString().toStdString();
    entityEntry->creationDateTime = record.value("creationDateTime").toLongLong();
    entityEntry->lastModificationDateTime = record.value("lastModificationDateTime").toLongLong();
    entityEntry->hSetId = record.value("hsetid").toInt();
    entityEntry->path = record.value("path").toString().toStdString();
    entityEntry->status = record.value("status").toInt();
    entityEntry->parentId = record.value("parentId").toLongLong();
    if (!_parentIdHash.contains(entityEntry->parentId))
        _parentIdHash.insert(entityEntry->parentId, QList<qlonglong>());
    _parentIdHash[entityEntry->parentId].append(entityEntry->id);
    entityEntry->tagList = _tagManager->tags(entityEntry->id);
    CatalogItemWrapper *wrapper = CatalogItemWrapperFactory::createWrapper(QString::fromStdString(entityEntry->ice_id()));
    if (wrapper)
        wrapper->setCatalogItem(entityEntry);
    return wrapper;
}

CatalogItemWrapper *CatalogDBStorage::_createTopologyEntry(const QSqlRecord &record)
{
    TopologyEntryPtr topologyEntry = new TopologyEntry();
    topologyEntry->id = record.value("id").toULongLong();
    topologyEntry->uuid = record.value("uuid").toString().toStdString();
    topologyEntry->shortName = record.value("shortName").toString().toStdString();
    topologyEntry->longName = record.value("longName").toString().toStdString();
    topologyEntry->description = record.value("description").toString().toStdString();
    topologyEntry->creationDateTime = record.value("creationDateTime").toULongLong();
    topologyEntry->lastModificationDateTime = record.value("lastModificationDateTime").toULongLong();
    topologyEntry->path = record.value("path").toString().toStdString();
    topologyEntry->tagList = _tagManager->tags(topologyEntry->id);
    topologyEntry->hSetId = record.value("hsetid").toInt();
    CatalogItemWrapper *wrapper = CatalogItemWrapperFactory::createWrapper(QString::fromStdString(topologyEntry->ice_id()));
    if (wrapper)
        wrapper->setCatalogItem(topologyEntry);
    return wrapper;
}

CatalogItemWrapper *CatalogDBStorage::_createFolder(const QSqlRecord &record)
{
    FolderPtr folder = new Folder();
    folder->id = record.value("id").toULongLong();
    folder->shortName = record.value("shortName").toString().toStdString();
    folder->longName = record.value("longName").toString().toStdString();
    folder->description = record.value("description").toString().toStdString();
    folder->parentId = record.value("parentId").toULongLong();
    folder->tagList = _tagManager->tags(folder->id);
    folder->hSetId = record.value("hsetid").toInt();
    CatalogItemWrapper *wrapper = CatalogItemWrapperFactory::createWrapper(QString::fromStdString(folder->ice_id()));
    if (wrapper)
        wrapper->setCatalogItem(folder);
    return wrapper;
}

qlonglong CatalogDBStorage::_addEntityEntry(const CatalogItemPtr &catalogItem)
{
    EntityEntryPtr entry = EntityEntryPtr::dynamicCast(catalogItem);
    if (!entry) {
        qCritical() << tr("This catalog item is not an entity entry");
        return 0;
    }
    QString query("INSERT INTO elements (uuid, shortName, longName,"
                            "description, creationDateTime, lastModificationDateTime, "
                            "path, type, parentid, hsetid) VALUES(?, ?, ?, ?, "
                            "?, ?, ?, ?, ?, ?)");
    QVariantList values;
    values.append(QString::fromStdString(entry->uuid));
    values.append(QString::fromStdString(entry->shortName));
    values.append(QString::fromStdString(entry->longName));
    values.append(QString::fromStdString(entry->description));
    values.append(qlonglong(entry->creationDateTime));
    values.append(qlonglong(entry->lastModificationDateTime));
    values.append(QString::fromStdString(entry->path));
    values.append(int(CatalogI::EntityType));
    values.append(qlonglong(entry->parentId));
    values.append(int(entry->hSetId));
    qlonglong entryId = _database->executeQuery(query, values);
    if (!entryId)
        qCritical() << tr("The entity entry could not be added to the database");
    _tagManager->addTags(entryId, entry->tagList);
    return entryId;
}

qlonglong CatalogDBStorage::_addTopologyEntry(const CatalogItemPtr &catalogItem)
{
    TopologyEntryPtr entry = TopologyEntryPtr::dynamicCast(catalogItem);
    if (!entry) {
        qCritical() << tr("This catalog item is not a topology entry");
        return 0;
    }
    QString query("INSERT INTO elements (uuid, shortName, longName,"
                            "description, creationDateTime, lastModificationDateTime, "
                            "path, type,hsetid) VALUES(?, ?, ?, ?, "
                            "?, ?, ?, ?,?)");
    QVariantList values;
    values.append(QString::fromStdString(entry->uuid));
    values.append(QString::fromStdString(entry->shortName));
    values.append(QString::fromStdString(entry->longName));
    values.append(QString::fromStdString(entry->description));
    values.append(qlonglong(entry->creationDateTime));
    values.append(qlonglong(entry->lastModificationDateTime));
    values.append(QString::fromStdString(entry->path));
    values.append(int(CatalogI::TopologyType));
    values.append(int(entry->hSetId));
    qlonglong entryId = _database->executeQuery(query, values);
    if (!entryId)
        qCritical() << tr("The topology entry could not be added to the database");
    _tagManager->addTags(entryId, entry->tagList);
    return entryId;
}

qlonglong CatalogDBStorage::_addFolder(const CatalogItemPtr &catalogItem)
{
    FolderPtr folder = FolderPtr::dynamicCast(catalogItem);
    if (!folder) {
        qCritical() << tr("This catalog item is not a folder");
        return 0;
    }
    QString query("INSERT INTO elements (shortName, longName, description, "
                  "parentId, type, hsetid) VALUES(?, ?, ?, ?, ?, ?)");
    QVariantList values;
    values.append(QString::fromStdString(folder->shortName));
    values.append(QString::fromStdString(folder->longName));
    values.append(QString::fromStdString(folder->description));
    values.append(qlonglong(folder->parentId));
    values.append(int(CatalogI::FolderType));
    values.append(int(folder->hSetId));
    qlonglong folderId = _database->executeQuery(query, values);
    if (!folderId)
        qCritical() << tr("The folder could not be added to the database");
    _tagManager->addTags(folderId, folder->tagList);
    return folderId;
}

} // namespace Hns
