/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "catalogfilestorage.h"
#include "entryserializer.h"
#include "layoutserializer.h"

// Generated hynesim headers
#include "common/topologyentry.h"

// diamesh headers
#include <diamesh/utils.h>

// Qt headers
#include <QFile>
#include <QDir>
#include <QDebug>
#include <QUuid>
#include <QDomDocument>

namespace Hns {

CatalogFileStorage::CatalogFileStorage(const QString &storagePath, const QString &storageName, const QString &docType, QObject *parent)
    : QObject(parent)
    , _storagePath(storagePath)
    , _storageName(storageName)
    , _docType(docType)
{
}

qulonglong CatalogFileStorage::addEntry(const ::Hns::EntryPtr &entry)
{
    Q_UNUSED(entry)
    return 0;
}

bool CatalogFileStorage::deleteEntry(const ::Hns::EntryPtr &entry)
{
    QString path = getFullPathFromMixed(QString::fromStdString(entry->path), QString::fromStdString(entry->uuid));
    QDir dir(path);
    if (!QFile::exists(path)) {
        qCritical() << tr("Unable to delete entry, the following path does not exist: %1").arg(path);
        return false;
    }
    if (!Diamesh::removeDirectory(dir.absolutePath())) {
        qCritical() << tr("Unable to remove the following directory: %1").arg(path);
        return false;
    }
    return true;
}

bool CatalogFileStorage::updateEntry(const ::Hns::EntryPtr &newEntry, const ::Hns::EntryPtr &oldEntry)
{
    if (newEntry->path != oldEntry->path && newEntry->path != "" && oldEntry->path != "") {
        qCritical() << tr("Unable to update the entry, paths are different (%1, %2)").arg(newEntry->path.c_str()).arg(oldEntry->path.c_str());
        return false;
    }

    QString fullPath = getFullPathFromMixed(newEntry->path.c_str(), newEntry->uuid.c_str());

    return EntrySerializer::writeEntry(newEntry, _docType, fullPath);
}

QList<EntryPtr> CatalogFileStorage::getEntries()
{
    return _getEntries();
}

QString CatalogFileStorage::getFullPathFromMixed(const QString &mixedPath, const QString &uuid)
{
    QString name;
    QString subPath;
    QStringList splitList = mixedPath.split(":", QString::SkipEmptyParts);

    if (splitList.isEmpty()) {
        qCritical() << tr("Empty mixed path provided.");
        return "";
    }

    name = splitList.first();
    if (splitList.size()>2){
        QStringList joinList;
        int i = 1;
        while (i<splitList.count()){
            joinList.append(splitList.value(i));
            i++;
        }
        subPath = joinList.join(QString(":"));
    } else if (splitList.size() == 2) {
        subPath = splitList.at(1);
    } else if (splitList.size()<2){
        subPath = QString();
    }
    if (_storageName != name) {
        qCritical() << tr("Mixed path %1 does not belong to this file storage (%2)")
                       .arg(mixedPath)
                       .arg(_storageName);
        return "";
    }
    return QDir::cleanPath(QString("%1%2/%3").arg(_storagePath).arg(subPath).arg(uuid));
}

bool CatalogFileStorage::entryFilesAvailable(const EntryPtr &entry)
{
    QString entryPath = getFullPathFromMixed(QString::fromStdString(entry->path), QString::fromStdString(entry->uuid));
    return QFile::exists(entryPath) && QFile::exists(QDir(entryPath).absoluteFilePath("hnsEntryConfig.xml"));
}

bool CatalogFileStorage::createEntryDir(const QString &entryPath)
{
    QDir dir(entryPath);
    QDir entryDir(dir.absolutePath());
    return QFile::exists(entryPath) || entryDir.mkpath(dir.absolutePath());
}

bool CatalogFileStorage::deleteEntryDir(const QString &entryPath)
{
    QDir entryDir(entryPath);
    if (!QFile::exists(entryPath)) {
        qWarning() << tr("The entry directory %1 does not exist, unable to delete it").arg(entryPath);
        return true;
    }
    if (!Diamesh::removeDirectory(entryPath)) {
        qWarning() << tr("Failed to remove entry directory %1").arg(entryPath);
        return false;
    }
    return true;
}

bool CatalogFileStorage::createEntryFiles(const EntryPtr &entry)
{
    if (entry->path == "") {
        qCritical() << tr("The entry path is not defined, unable to create the entry files");
        return false;
    }
    if (entry->uuid == "") {
        qCritical() << tr("The entry UUID is not defined, unable to create the entry files");
        return false;
    }
    if (entryFilesAvailable(entry)) {
        qCritical() << tr("The entry files already exist, unable to create them");
        return false;
    }

    QString dirPath = getFullPathFromMixed(QString::fromStdString(entry->path), "");
    QDir dir(dirPath);
    if (!QFile::exists(dirPath) && !dir.mkpath(dirPath)) {
        qCritical() << tr("The entry directory %1 could not be created in %2").arg(entry->uuid.c_str()).arg(dirPath);
        return false;
    }

    QString fullPath = getFullPathFromMixed(entry->path.c_str(), entry->uuid.c_str());

    if (!EntrySerializer::writeEntry(entry, _docType, fullPath)) {
        qCritical() << tr("Unable to serialize the entry, while creating its files");
        return false;
    }

    return true;
}

QString CatalogFileStorage::storageName()
{
    return _storageName;
}

QString CatalogFileStorage::storagePath()
{
    return _storagePath;
}

TopologyLayout CatalogFileStorage::getTopologyLayout(const EntryPtr &entry)
{
    TopologyLayout layout;
    if (entry->path == "")
        return layout;
    QDomDocument domDocument = _getEntryDomDocument(entry);
    QDomNodeList layoutList = domDocument.elementsByTagName("Layout");
    if (!layoutList.size())
        return layout;
    QDomElement layoutElement = layoutList.item(0).toElement();
    QDomElement layoutSubElement = layoutElement.firstChildElement();
    while (!layoutSubElement.isNull()) {

        if (layoutSubElement.tagName() == "TopologyLayout") {
            layout.layoutOptions.width = layoutSubElement.attribute("width").toInt();
            layout.layoutOptions.height = layoutSubElement.attribute("height").toInt();
            layout.layoutOptions.colorCode = layoutSubElement.attribute("color").toStdString();
        }
        else if (layoutSubElement.tagName() == "EntityLayout") {
            EntityLayoutPtr entityLayout = EntityLayoutSerializer::unserialize(layoutSubElement);
            layout.entityLayouts[entityLayout->id]=entityLayout;
        }
        else if (layoutSubElement.tagName() == "WireLayout") {
            WireLayoutPtr wireLayout = WireLayoutSerializer::unserialize(layoutSubElement);
            layout.wireLayouts[wireLayout->id]=wireLayout;
        }
        layoutSubElement = layoutSubElement.nextSiblingElement();
    }
    double xmlVersion = 0.0;
    QDomElement root = domDocument.documentElement();
    if (root.hasAttribute("version"))
        xmlVersion = root.attribute("version").toDouble();
    if (xmlVersion < 1.1) {
        TopologyEntryPtr topologyEntry = TopologyEntryPtr::dynamicCast(entry);

        QList<TopologyLink> topologyLinkList = QList<TopologyLink>::fromStdList(topologyEntry->topologyLinks);
        foreach (TopologyLink topologyLink, topologyLinkList) {
            qlonglong entityId = QString::fromStdString(topologyLink.entitySide).split(':').first().toLongLong();
            EntityLayoutPtr eLayout = layout.entityLayouts[entityId];
            QStringList wireSideDetails = QString::fromStdString(topologyLink.wireSide).split(":");
            qlonglong wireId = wireSideDetails.first().toLongLong();
            int wireSide = wireSideDetails.at(1).toInt();

            if (wireSide == 0) { // from
                layout.wireLayouts[wireId]->x1 = eLayout->x;
                layout.wireLayouts[wireId]->y1 = eLayout->y;
            } else {
                layout.wireLayouts[wireId]->x2 = eLayout->x;
                layout.wireLayouts[wireId]->y2 = eLayout->y;
            }
        }
    }

    return layout;
}

bool CatalogFileStorage::updateTopologyLayout(const EntryPtr &entry, TopologyLayout layout)
{
    QDomDocument domDocument = _getEntryDomDocument(entry);
    QDomElement root = domDocument.documentElement();

    QDomNodeList topologyNodeList = root.elementsByTagName("Topology");
    if(topologyNodeList.size() != 1) {
        qCritical() << tr("There should be exactly one <Topology> tag in the configuration file");
        return false;
    }
    QDomElement topologyElement = topologyNodeList.item(0).toElement();

    QDomElement layoutElement;
    QDomNodeList layoutNodeList = topologyElement.elementsByTagName("Layout");
    if(layoutNodeList.size()) {
        layoutElement = layoutNodeList.item(0).toElement();
        topologyElement.removeChild(layoutElement);
        layoutElement.clear();
    }

    layoutElement = domDocument.createElement("Layout");
    topologyElement.appendChild(layoutElement);

    QDomElement topologyLayoutElement = domDocument.createElement("TopologyLayout");
    topologyLayoutElement.setAttribute("width", layout.layoutOptions.width);
    topologyLayoutElement.setAttribute("height", layout.layoutOptions.height);
    topologyLayoutElement.setAttribute("color", layout.layoutOptions.colorCode.c_str());
    layoutElement.appendChild(topologyLayoutElement);

    QMap<Ice::Long, EntityLayoutPtr> entityLayoutMap(layout.entityLayouts);
    foreach(EntityLayoutPtr entityLayout, entityLayoutMap) {

        QDomElement element = EntityLayoutSerializer::serialize(&domDocument, entityLayout);
        layoutElement.appendChild(element);
    }

    QMap<Ice::Long, WireLayoutPtr> wireLayoutMap(layout.wireLayouts);
    foreach(WireLayoutPtr wireLayout, wireLayoutMap) {
        QDomElement element = WireLayoutSerializer::serialize(&domDocument, wireLayout);
        layoutElement.appendChild(element);
    }

    QString fileName = _getEntryConfigFileName(entry);
    QFile file(fileName);
    if (!file.open(QIODevice::WriteOnly | QFile::Text)) {
        qCritical() << tr("Unable to open file %1").arg(fileName);
        return false;
    }

    QTextStream out(&file);
    const int xmlIndentSize = 2;
    domDocument.save(out, xmlIndentSize);
    file.close();

    qDebug() << domDocument.toString();

    return true;
}

QList<EntryPtr> CatalogFileStorage::_getEntries(const QString &path)
{
    QList<EntryPtr> list;

    QString dirPath = path.isEmpty() ? _storagePath : path;

    QDir dir(dirPath);
    if (!dir.exists()) {
        qCritical() << tr("Unable to get the file storage entries: directory %1 does not exist.").arg(dirPath);
        return list;
    }

    dir.setFilter(QDir::Dirs | QDir::NoDotAndDotDot);
    dir.setSorting(QDir::Name);
    QFileInfoList dirList = dir.entryInfoList();
    for (int i = 0; i < dirList.size(); ++i) {
        QString dirFullPath = dirList.at(i).absoluteFilePath();
        if (QUuid(dirList.at(i).baseName()).isNull())
            list.append(_getEntries(dirFullPath));
        else {
            EntryPtr entry = EntrySerializer::readEntry(dirFullPath, _docType);
            if (entry) {
                QString path = entry->path.c_str();

                // in case the catalog path is relative, replace the absolute path with the relative path
                if (!_storagePath.startsWith("/")) {
                    int index = path.lastIndexOf(_storagePath);
                    if (index != -1)
                        path.replace(path.mid(0, index+_storagePath.length()), _storagePath);
                }

                path.replace(_storagePath, QString("%1:").arg(_storageName));
                entry->path = path.toStdString();
                list.append(entry);
            }
        }
    }


    return list;
}

bool CatalogFileStorage::_saveEntryConfig(const QDomDocument &doc, const QString &path)
{
    QString fileName = QString("%1/hnsEntryConfig.xml").arg(path);
    QFile file(fileName);
    if (!file.open(QIODevice::WriteOnly | QFile::Text)) {
        qCritical() << tr("Unable to open file %1").arg(fileName);
        return false;
    }
    QTextStream out(&file);
    const int xmlIndentSize = 2;
    doc.save(out, xmlIndentSize);
    file.close();
    return true;
}

QString CatalogFileStorage::_getEntryConfigFileName(const EntryPtr &entry)
{
    QString fullPath = getFullPathFromMixed(QString::fromStdString(entry->path), QString::fromStdString(entry->uuid));
    if (!QFile::exists(fullPath)) {
        qCritical() << Q_FUNC_INFO <<tr("cannot find %1 path").arg(QString::fromStdString(entry->uuid));
        return QString();
    }
    return QString("%1/hnsEntryConfig.xml").arg(fullPath);
}

QDomDocument CatalogFileStorage::_getEntryDomDocument(const EntryPtr &entry)
{
    QString fileName = _getEntryConfigFileName(entry);
    if (fileName.isEmpty()) {
        qCritical() << tr("Entry file was not found. Unable to read layout");
        return QDomDocument();
    }
    QFile file(fileName);
    if (!file.exists()) {
        qCritical() << tr("Entry file %1 was not found. Unable to read layout").arg(fileName);
        return QDomDocument();
    }

    QString errorStr;
    int errorLine;
    int errorColumn;
    QDomDocument domDocument;

    if (!domDocument.setContent(&file, true, &errorStr, &errorLine, &errorColumn))
    {
        qCritical() << tr("Error at line %1, column %2: %3").arg(errorLine).arg(errorColumn).arg(errorStr);
        file.close();
        return QDomDocument();
    }
    file.close();
    return domDocument;
}

} // namespace Hns
