/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "entitycatalogI.h"
#include "catalogfilestorage.h"
#include "catalogdbstorage.h"
#include "catalogitemwrapper.h"
#include "entityentrywrapper.h"
#include "jobmanagerI.h"
#include "masterI.h"
#include "folderwrapper.h"
#include "entityentryrelationshipmapper.h"
#include "clonejobs/entityclonejob.h"

// hynesim common headers
#include "common/rightslist.h"
#include "common/constants.h"

// diamesh headers
#include <diamesh/usermanager.h>
#include <diamesh/sessionManagerI.h>
#include <diamesh/aclmanagerI.h>

// ZeroC Ice headers
#include <Ice/Communicator.h>
#include <Ice/ObjectAdapter.h>
#include <Ice/Current.h>

// Qt headers
#include <QDebug>
#include <QSettings>
#include <QStringList>

namespace Hns {

EntityCatalogI::EntityCatalogI(const Ice::CommunicatorPtr &communicator, Ice::ObjectAdapterPtr adapter, const QString &settingsFileName)
    : _Hns_EntityCatalog(communicator, adapter, settingsFileName)
{
    _id.name = "entityCatalog";
    _storageType = QString::fromStdString("entity");
    _adapter->add(this, _id);
    _aclTargetTypes << Rights::ACL_TARGET_ENTITYENTRY << Rights::ACL_TARGET_FOLDER;

    QSettings settings(_settingsFileName, QSettings::IniFormat, this);
    settings.beginGroup(Constants::SETTINGS_HYNESIM_GROUP);

    _maxCloneJobs = settings.value("catalog/maxCloneJobs", 1).toInt();

    settings.endGroup();

    _dbtypes << CatalogI::EntityType << CatalogI::FolderType;

    _dbStorage = new CatalogDBStorage(_settingsFileName);
    _storageDocType = "HnsEntityConfig";

    EntityEntryRelationshipMapper::init();

    _init();
}

EntityCatalogI::~EntityCatalogI()
{
    EntityEntryRelationshipMapper::close();
}

EntityCatalogPrx EntityCatalogI::proxy()
{
    return EntityCatalogPrx::uncheckedCast(_adapter->createProxy(_id));
}

EntityEntryPtr EntityCatalogI::getEntityEntry(Ice::Long entryId, const ::Ice::Current &current)
{
    EntryPtr entry = getEntry(entryId, current);
    if (!entry) {
        qDebug() << tr("Unable to get the entity entry");
        return NULL;
    }
    return EntityEntryPtr::dynamicCast(entry);
}

Ice::Int EntityCatalogI::cloneEntityEntry(::Ice::Long entryId, ::Ice::Int flags, const ::Ice::Current &current)
{
    int jobId = 0;
    EntryPtr entry = getEntry(entryId, current);
    if (!entry) {
        qDebug() << tr("Unable to get the entity entry");
        return 0;
    }
    if (!_catalogItemHash.value(entryId)->testACL(Rights::ENTRY_SUFFIX_USE_CLONE, current))
        return 0;

    if (entryIsLocked(entryId)) {
        LockType lockType = getLockType(entryId);
        if (lockType == LOCK_TYPE_NORMAL || lockType == LOCK_TYPE_EXPORT) {
            qCritical() << tr("Entry %1 is locked. Unable to clone it").arg(entryId);
            return 0;
        } else if (lockType == LOCK_TYPE_CLONE) {
            jobId = _makeCloneJob(entryId, flags, current);
        } else {
            qCritical() << tr("Unable to clone entry %1").arg(entryId);
            return 0;
        }
    } else {
        /*lock entry*/
        if (!lockEntry(entryId, LOCK_TYPE_CLONE)) {
            qCritical() << tr("The entry %1 could not be locked. It will not be cloned").arg(entryId);
            return 0;
        }
        jobId = _makeCloneJob(entryId, flags, current);
    }

    /*lock*/
    QWriteLocker locker(&_cloneJobsLock);

    /*start the next queued job if possible*/
    if (_cloneJobsHash.size() < _maxCloneJobs)
        _startNextCloneJob();

    return jobId;
}

Ice::Long EntityCatalogI::buildModelFromEntityEntry(::Ice::Long entryId, const::Ice::Current &current)
{
    EntryPtr entry = getEntry(entryId, current);
    if (!entry) {
        qDebug() << tr("Unable to get the entity entry");
        return 0;
    }

    if (!lockEntry(entryId)) {
        qCritical() << tr("The entry %1 could not be locked. No model will be build from it.").arg(entryId);
        return 0;
    }
    QReadLocker locker(&_entriesLock);
    if (!_catalogItemHash.value(entryId)->testACL(Rights::ENTRY_SUFFIX_USE_BUILD_MODEL_FROM, current))
        return 0;
    locker.unlock();

    EntityEntryPtr modelEntry = EntityEntryPtr::dynamicCast(_catalogItemHash.value(entryId)->iceClone());
    modelEntry->id = 0;
    modelEntry->isTemplate = false;
    modelEntry->isModel = true;
    modelEntry->uuid = generateUnusedUUID().toStdString();
    Diamesh::UserPtr currentUser = MasterI::instance()->sessionManager()->currentUser(current);
    if (currentUser)
        modelEntry->hSetId = currentUser->hSetId;

    if (!QString::fromStdString(modelEntry->shortName).endsWith(" model"))
        modelEntry->shortName.append(" model");

    Ice::Long modelEntryID = addEntry(modelEntry, current);
    if (modelEntryID != 0)
        updateEntry(modelEntry, current);        // Nécessaire pour sérialiser sur le disque (TODO:à modifier)

    if (!unlockEntry(entryId))
        qCritical() << tr("The entry %1 could not be unlocked, after being used to build a model.").arg(entryId);

    return modelEntryID;
}

bool EntityCatalogI::discardSaveState(::Ice::Long entryId, const::Ice::Current &current)
{
    EntryPtr entry = getEntry(entryId, current);
    if (!entry) {
        qDebug() << tr("Unable to get the entity entry");
        return false;
    }
    QReadLocker locker(&_entriesLock);
    if (!_catalogItemHash.value(entryId)->testACL(Rights::ENTRY_SUFFIX_USE_DISCARD_SAVE_STATE, current))
        return false;
    locker.unlock();

    EntityEntryPtr entityEntry = EntityEntryPtr::dynamicCast(entry);
    entityEntry->saved = false;
    return updateEntry(entityEntry, current);
}

LongIntMap EntityCatalogI::getCloneJobs(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QReadLocker locker(&_cloneJobsLock);
    LongIntMap map;
    for (QHash< ::Ice::Long, int>::iterator i = _cloneJobsHash.begin(); i != _cloneJobsHash.end(); ++i)
        map[i.key()] = i.value();

    return map;
}

Diamesh::LongList EntityCatalogI::getFolders(::Ice::Long parentId, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QReadLocker locker(&_entriesLock);
    Diamesh::LongList ret;
    foreach (CatalogItemWrapper *wrapper, _catalogItemHash) {
        if (!qobject_cast<FolderWrapper *>(wrapper))
            continue;
        FolderPtr folder = FolderPtr::dynamicCast(wrapper->catalogItem());
        if (folder->parentId == parentId && _testFolderRightRecursiverly(folder->id, current))
            ret.push_back(folder->id);
    }
    return ret;
}

FolderPtr EntityCatalogI::getFolder(::Ice::Long folderId, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QReadLocker locker(&_entriesLock);
    if (_catalogItemHash.contains(folderId)) {
        if (_catalogItemHash.value(folderId)->testACL(Rights::FOLDER_SUFFIX_READ, current)) {
            FolderPtr folder = FolderPtr::dynamicCast(_catalogItemHash.value(folderId)->catalogItem());
            return folder;
        } else {
            return NULL;
        }
    }
    qCritical() << tr("Unable to find folder %1").arg(folderId);
    return NULL;
}

::Ice::Long EntityCatalogI::addFolder(const FolderPtr &folder, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QWriteLocker locker(&_entriesLock);
    Diamesh::UserPtr currentUser = MasterI::instance()->sessionManager()->currentUser(current);

    /*TODO: check parentId*/
    if (folder->creationDateTime)
        folder->creationDateTime = _currentTime();

    folder->lastModificationDateTime = _currentTime();

    CatalogItemWrapper *wrapper = CatalogItemWrapperFactory::createWrapper(QString::fromStdString(folder->ice_id()));
    if (!wrapper->create(_dbStorage, 0, folder, currentUser))
        return 0;

    qlonglong folderId = wrapper->catalogItem()->id;
    _catalogItemHash.insert(folderId, wrapper);

    emit folderAdded(folderId);
    return folderId;
}

bool EntityCatalogI::deleteFolder(::Ice::Long folderId, const ::Ice::Current &current)
{
    QWriteLocker locker(&_entriesLock);
    if (!_catalogItemHash.contains(folderId)) {
        qCritical() << tr("folder %1 not found, unable to delete id").arg(folderId);
        return false;
    }
    if (!_catalogItemHash.value(folderId)->testACL(Rights::FOLDER_SUFFIX_DELETE, current))
        return false;
    if (!_catalogItemHash.value(folderId)->remove(_dbStorage, 0))
        return false;

    CatalogItemWrapper *wrapper = _catalogItemHash.take(folderId);

    foreach (CatalogItemWrapper *w, _catalogItemHash) {
        if (w->catalogItem()->parentId == folderId){
            EntryPtr entry = EntryPtr::dynamicCast(w->catalogItem());
            if (entry) {
                entry->parentId = wrapper->catalogItem()->parentId;
                updateEntryNoLock(entry);
            } else {
                FolderPtr folder = FolderPtr::dynamicCast(w->catalogItem());
                if (folder) {
                    folder->parentId = wrapper->catalogItem()->parentId;
                    if (!_catalogItemHash.value(folder->id)->update(_dbStorage, 0, folder))
                        qCritical() << tr("Unable to update folder %1").arg(folder->id);
                    emit folderChanged(folder->id);
                }
            }
        }
    }

    delete wrapper;

    qDebug() << tr("Folder %1 has been deleted").arg(folderId);

    emit folderRemoved(folderId);
    return true;
}

bool EntityCatalogI::updateFolder(const FolderPtr &folder, const ::Ice::Current &current)
{
    QWriteLocker locker(&_entriesLock);

    if (!_catalogItemHash.contains(folder->id)) {
        qCritical() << tr("Folder %1 not found, unable to update it").arg(folder->id);
        return false;
    }
    if (!_catalogItemHash.value(folder->id)->testACL(Rights::FOLDER_SUFFIX_UPDATE_FOLDER, current))
        return false;
    if (!_catalogItemHash.value(folder->id)->update(_dbStorage, 0, folder))
        return false;

    emit folderChanged(folder->id);
    return true;
}

void EntityCatalogI::_cloneEntityProgressChangedSlot(Ice::Int jobId, Ice::Double progress)
{
    QWriteLocker locker(&_cloneJobsLock);
    if (progress == 1) {
        qDebug() << tr("Job %1 has finished").arg(jobId);
        Ice::Long entryId = _cloneJobsHash.key(jobId);
        if (!entryId) {
            qCritical() << tr("Job %1 was not found in entry hash.").arg(jobId);
        } else {
            bool unlockNeeded = !_isEntryInCloneBatch(entryId);
            _cloneJobsHash.remove(entryId);
            //launch another job
            _startNextCloneJob();
            EntityCloneJob *oldJob = qobject_cast<EntityCloneJob*>(JobManagerI::instance()->job(jobId));
            emit cloneJobSucceeded(oldJob->clonedEntryId(), jobId);
            if (unlockNeeded && !unlockEntry(entryId))
                qCritical() << tr("The entry %1 could not be unlocked, after being cloned").arg(entryId);
        }
    }
}

void EntityCatalogI::_cloneEntityCancelled(Ice::Int jobId)
{
    QWriteLocker locker(&_cloneJobsLock);
    qDebug() << tr("Job %1 was cancelled").arg(jobId);
    Ice::Long entryId = _cloneJobsHash.key(jobId);
    if (!entryId) {
        qCritical() << tr("Job %1 was not found in entry hash.").arg(jobId);
    } else {
        bool unlockNeeded = !_isEntryInCloneBatch(entryId);
        _cloneJobsHash.remove(entryId);
        //launch another job
        _startNextCloneJob();
        if (unlockNeeded && !unlockEntry(entryId))
            qCritical() << tr("The entry %1 could not be unlocked, after being cloned").arg(entryId);
    }
}

void EntityCatalogI::_handleUuidConflicts(const QMultiHash<QString, EntryPtr> &multiHashUuidEntryPtr)
{
    QStringList keys = multiHashUuidEntryPtr.keys();
    foreach (const QString &key, keys) {
        QList<EntryPtr> list = multiHashUuidEntryPtr.values(key);
        if (list.size() > 1) {
            foreach (EntryPtr entry, list)
                entry->status = ENTITY_STATUS_UUID_CONFLICT;
        }
    }
}

void EntityCatalogI::_markEntryAsInvalid(const EntryPtr &entry)
{
    entry->status = ENTITY_STATUS_NOT_IN_FILE_STORAGE;
}

void EntityCatalogI::_checkACL(const QString &target)
{
    qlonglong id = _entryIdFromTarget(target);
    if (id != -1) {
        QReadLocker locker(&_entriesLock);
        if (_catalogItemHash.contains(id)) {
            // Do not use getACLsForCatalogItem() below because it would lock _entriesLock
            // for read again and this could lead to a deadlock.
            Diamesh::ACLList acls;
            CatalogItemPtr catalogItem = _catalogItemHash.value(id)->catalogItem();

            EntryPtr entry = EntryPtr::dynamicCast(catalogItem);
            if (entry) {
                acls = MasterI::instance()->aclManager()->getACLsForTarget(entry);
                emit entryACLsChanged(id, acls);
                return;
            }

            FolderPtr folder = FolderPtr::dynamicCast(catalogItem);
            if (folder) {
                acls = MasterI::instance()->aclManager()->getACLsForTarget(folder);
                emit folderACLsChanged(id, acls);
                return;
            }
        }
    }
}

bool EntityCatalogI::_testFolderRightRecursiverly(qlonglong folderId, const Ice::Current &current)
{
    FolderWrapper *folder = qobject_cast<FolderWrapper *>(_catalogItemHash.value(folderId));
    if (folder) {
        if (folder->testACL(Rights::FOLDER_SUFFIX_READ, current, false)) {
            qlonglong parentId = FolderPtr::dynamicCast(folder->catalogItem())->parentId;
            if (!parentId)
                return true;
            else
                return _testFolderRightRecursiverly(parentId, current);
        }
    }
    return false;
}

int EntityCatalogI::_makeCloneJob(qlonglong entryId, int flags, const Ice::Current &current)
{
    QWriteLocker locker(&_cloneJobsLock);
    EntityEntryPtr entityEntry = EntityEntryPtr::dynamicCast(getEntry(entryId));

    JobManagerI *jobManager = JobManagerI::instance();

    EntityCloneJob *job = new EntityCloneJob(entityEntry, flags, MasterI::instance()->sessionManager()->currentUser(current));
    int jobId = jobManager->addJob(job);

    QObject::connect(job, SIGNAL(progressChanged(Ice::Int, Ice::Double)), this, SLOT(_cloneEntityProgressChangedSlot(Ice::Int, Ice::Double)));
    QObject::connect(job, SIGNAL(cancelled(Ice::Int)), this, SLOT(_cloneEntityCancelled(Ice::Int)));

    _cloneBatch.append(qMakePair(entryId, jobId));

    return jobId;
}

void EntityCatalogI::_startNextCloneJob()
{
    JobI* job = 0;
    QListIterator<QPair<qlonglong, int> > it(_cloneBatch);
    QPair<qlonglong, int> pair;
    while (it.hasNext()) {
        pair = it.next();
        if (_cloneJobsHash.contains(pair.first))
            continue;
        job = JobManagerI::instance()->job(pair.second);
        _cloneBatch.removeOne(pair);
        break;
    }
    if (job) {
        _cloneJobsHash[pair.first] = pair.second;
        job->execute();
        emit cloneJobStarted(pair.first, pair.second);
    }
}

bool EntityCatalogI::_isEntryInCloneBatch(qlonglong entryId)
{
    QListIterator<QPair<qlonglong, int> > it(_cloneBatch);
    QPair<qlonglong, int> pair;
    while (it.hasNext()) {
        pair = it.next();
        if (pair.first == entryId)
            return true;
    }
    return false;
}

} // namespace Hns
