/******************************************************************************
**
** Copyright (C) 2003-2016 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "entityentryrelationshipmapper.h"

// hynesim common headers
#include <common/constants.h>

// diamesh headers
#include <diamesh/diamesh.h>
#include <diamesh/sqldatabase.h>

// Qt project headers
#include <QSettings>
#include <QSqlRecord>
#include <QDebug>
#include <QStringList>
#include <QReadWriteLock>

namespace Hns {

//Diamesh::SqlDatabase and G_GLOBAL_STATIC are thread-safe, so a mutex/locker is not needed
static Diamesh::SqlDatabase* database;
typedef QMultiHash<QString, QString> MultiHashStringString;
Q_GLOBAL_STATIC(MultiHashStringString, relationships)
Q_GLOBAL_STATIC(QReadWriteLock, lock)

bool EntityEntryRelationshipMapper::init()
{
    //opening database
    const QString &config = Diamesh::instance()->configFileName();
    QSettings settings(config, QSettings::IniFormat);
    settings.beginGroup(Constants::SETTINGS_HYNESIM_GROUP);
    const QString &dbName = settings.value("catalog/dbName").toString();
    database = new Diamesh::SqlDatabase("QSQLITE", dbName);
    QList<QSqlRecord> results;
    if (database->executeQuery("select * from entity_relationship", QList<QVariant>(), &results) < 0) {
        qCritical() << QObject::tr("EntityEntryRelationshipMapper: Unable to get entity relationships from DB");
        return false;
    }
    foreach (const QSqlRecord &record, results) {
        if (!record.contains("base_uuid") || !record.contains("diff_uuid")) {
            qWarning() << QObject::tr("Values in relationship database are incomplete");
            continue;
        }
        relationships()->insert(record.value("base_uuid").toString(), record.value("diff_uuid").toString());
    }
    //find duplicates
    QList<QString> listValues = relationships()->values();
    QSet<QString> setValues(QSet<QString>::fromList(listValues));
    if (listValues.size() != setValues.size()) {
        qCritical() << QObject::tr("EntityEntryRelationshipMapper: found duplicate relationships");
        return false;
    }

    return true;
}

void EntityEntryRelationshipMapper::close()
{
    delete database;
}

bool EntityEntryRelationshipMapper::addRelationship(const QString &baseUuid, const QString &diffUuid)
{
    QWriteLocker locker(lock());
    if (relationships()->contains(baseUuid, diffUuid)) {
        qDebug() << QObject::tr("A relationship between base %1 and diff %2 is already registered").arg(baseUuid).arg(diffUuid);
        return true;
    }
    if(database->executeQuery("insert into entity_relationship (base_uuid, diff_uuid) values(?, ?)", QVariantList() << baseUuid << diffUuid) < 0) {
        qCritical() << QObject::tr("Error while inserting relationship in database");
        return false;
    }
    relationships()->insert(baseUuid, diffUuid);
    return true;
}

bool EntityEntryRelationshipMapper::removeRelationship(const QString &baseUuid, const QString &diffUuid)
{
    QWriteLocker locker(lock());
    if (!relationships()->contains(baseUuid, diffUuid)) {
        qDebug() << QObject::tr("There is no relationship between base %1 and diff %2.").arg(baseUuid).arg(diffUuid);
        return true;
    }
    if (database->executeQuery("delete from entity_relationship where base_uuid=? and diff_uuid=?", QVariantList() << baseUuid << diffUuid) < 0) {
        qCritical() << QObject::tr("Error while deleting relationship in database");
        return false;
    }
    relationships()->remove(baseUuid, diffUuid);
    return true;
}

QString EntityEntryRelationshipMapper::getBaseUuid(const QString &diffUuid)
{
    QReadLocker locker(lock());
    return relationships()->key(diffUuid);
}

QStringList EntityEntryRelationshipMapper::getDiffUuids(const QString &baseUuid)
{
    QReadLocker locker(lock());
    return relationships()->values(baseUuid);
}

} // namespace Hns
