#include "entityentrywrapper.h"

#include "catalogfilestorage.h"
#include "entityentryrelationshipmapper.h"
#include "entitycatalogI.h"
#include "masterI.h"

#include <common/entityentry.h>
#include <common/domainentry.h>
#include <common/dynamipsrouterentry.h>
#include <common/hybridnetcardentry.h>
#include <common/switchentry.h>
#include <common/topologygateentry.h>
#include <common/wifiapentry.h>
#include <common/wireentry.h>
#include <common/rightslist.h>

#include <QDebug>
#include <QStringList>

namespace Hns
{

static RegisterCatalogItemWrapper<EntityEntryWrapper> registerEntity(QString::fromStdString(EntityEntry::ice_staticId()));
static RegisterCatalogItemWrapper<DomainEntryWrapper> registerDomain(QString::fromStdString(DomainEntry::ice_staticId()));
static RegisterCatalogItemWrapper<DynamipsEntryWrapper> registerDynamips(QString::fromStdString(DynamipsRouterEntry::ice_staticId()));
static RegisterCatalogItemWrapper<HybridNetcardEntryWrapper> registerHybridNetcard(QString::fromStdString(HybridNetcardEntry::ice_staticId()));
static RegisterCatalogItemWrapper<SwitchEntryWrapper> registerSwitch(QString::fromStdString(SwitchEntry::ice_staticId()));
static RegisterCatalogItemWrapper<TopologyGateEntryWrapper> registerTopologyGate(QString::fromStdString(TopologyGateEntry::ice_staticId()));
static RegisterCatalogItemWrapper<WifiAPEntryWrapper> registerWifi(QString::fromStdString(WifiAPEntry::ice_staticId()));
static RegisterCatalogItemWrapper<WireEntryWrapper> registerWire(QString::fromStdString(WireEntry::ice_staticId()));

EntityEntryWrapper::EntityEntryWrapper(QObject *parent)
    : EntryWrapper(parent)
{
    _prefixForRights = Rights::ENTITY_ENTRY_PREFIX;
}

bool EntityEntryWrapper::create(CatalogDBStorage *db, CatalogFileStorage *fileStorage, CatalogItemPtr newItem, const Diamesh::UserPtr &currentUser)
{
    if (!EntryWrapper::create(db, fileStorage, newItem, currentUser))
        return false;
    if (!fileStorage) {
        qCritical() << tr("Unable to find a file storage");
        return false;
    }
    EntryPtr entry = EntryPtr::dynamicCast(newItem);
    if (entry->path == "") {
        entry->path = QString("%1:").arg(fileStorage->storageName()).toStdString();
    }

    if (!fileStorage->entryFilesAvailable(entry) && !fileStorage->createEntryFiles(entry)) {
        qCritical() << tr("Error while creating the entry files, unable to add the entry");
        return false;
    }

    return true;
}

DomainEntryWrapper::DomainEntryWrapper(QObject *parent)
    : EntityEntryWrapper(parent)
{
}

void DomainEntryWrapper::init()
{
    DomainEntryPtr domainEntry = DomainEntryPtr::dynamicCast(catalogItem());
    QString uuid = QString::fromStdString(domainEntry->uuid);
    QString baseUuid = EntityEntryRelationshipMapper::getBaseUuid(uuid);
    QStringList diffUuids = EntityEntryRelationshipMapper::getDiffUuids(uuid);
    if (!baseUuid.isEmpty())
        domainEntry->baseUuid = baseUuid.toStdString();
    if (!diffUuids.isEmpty()) {
        foreach (const QString &diffUuid, diffUuids) {
            domainEntry->diffUuids.push_back(diffUuid.toStdString());
        }
    }
}

void DomainEntryWrapper::addDomainDiffUuids(QString uuid)
{
    QWriteLocker locker(&_diffUuidsLock);

    DomainEntryPtr domainEntry = DomainEntryPtr::dynamicCast(catalogItem());
    domainEntry->diffUuids.push_back(uuid.toStdString());
}

Ice::ObjectPtr DomainEntryWrapper::iceClone()
{
    DomainEntryPtr domain = DomainEntryPtr::dynamicCast(CatalogItemWrapper::iceClone());
    domain->diffUuids.clear();
    return domain;
}

bool DomainEntryWrapper::update(CatalogDBStorage *db, CatalogFileStorage *fileStorage, CatalogItemPtr newItem)
{
    QReadLocker locker(&_diffUuidsLock);
    /*add and remove relationship*/
    DomainEntryPtr domainEntry = DomainEntryPtr::dynamicCast(newItem);
    QString baseUuid = QString::fromStdString(domainEntry->uuid);
    QStringList relationships = EntityEntryRelationshipMapper::getDiffUuids(baseUuid);
    QStringList newRelationships;
    foreach (const std::string &relationship, domainEntry->diffUuids) {
        QString diffUuid = QString::fromStdString(relationship);
        newRelationships << diffUuid;
        if (!relationships.contains(diffUuid)) {
            if (!EntityEntryRelationshipMapper::addRelationship(baseUuid, diffUuid)) {
                continue;
            }
            relationships.append(diffUuid);
        }
    }
    foreach (const QString &relationship, relationships) {
        if (!newRelationships.contains(relationship)) {
            EntityEntryRelationshipMapper::removeRelationship(baseUuid, relationship);
        }
    }

    /*update entry*/
    return EntityEntryWrapper::update(db, fileStorage, newItem);
}

bool DomainEntryWrapper::remove(CatalogDBStorage *db, CatalogFileStorage *fileStorage)
{
    QReadLocker locker(&_diffUuidsLock);

    DomainEntryPtr domainEntry = DomainEntryPtr::dynamicCast(catalogItem());

    if (domainEntry->diffUuids.size() != 0) {
        qCritical() << tr("Unable to delete domain entry: this domain has still children");
        return false;
    }
    if (!domainEntry->baseUuid.empty()) {
        //update parent
        EntityCatalogI *entityCatalog = MasterI::instance()->entityCatalog();
        CatalogItemPtr baseItem = entityCatalog->catalogItemByUuid(QString::fromStdString(domainEntry->baseUuid));
        if (baseItem) {
            //we don't need to protect this entry because it's a clone.
            DomainEntryPtr baseDomain = DomainEntryPtr::dynamicCast(baseItem->ice_clone());
            baseDomain->diffUuids.remove(domainEntry->uuid);
            entityCatalog->updateEntryNoLock(baseDomain);
        }
    }
    return EntityEntryWrapper::remove(db, fileStorage);
}

DynamipsEntryWrapper::DynamipsEntryWrapper(QObject *parent)
    : EntityEntryWrapper(parent)
{
}

HybridNetcardEntryWrapper::HybridNetcardEntryWrapper(QObject *parent)
    : EntityEntryWrapper(parent)
{
}

SwitchEntryWrapper::SwitchEntryWrapper(QObject *parent)
    : EntityEntryWrapper(parent)
{
}

TopologyGateEntryWrapper::TopologyGateEntryWrapper(QObject *parent)
    : EntityEntryWrapper(parent)
{
}

WifiAPEntryWrapper::WifiAPEntryWrapper(QObject *parent)
    : EntityEntryWrapper(parent)
{
}

WireEntryWrapper::WireEntryWrapper(QObject *parent)
    : EntityEntryWrapper(parent)
{
}

} // namespace Hns
