/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "topologycatalogI.h"
#include "catalogdbstorage.h"
#include "catalogfilestorage.h"
#include "entitycatalogI.h"
#include "masterI.h"
#include "topologyentrywrapper.h"
#include "jobmanagerI.h"
#include "clonejobs/topologyclonejob.h"
#include "catalog/entityentryrelationshipmapper.h"

// hynesim common headers
#include "common/rightslist.h"

// diamesh headers
#include <diamesh/sessionManagerI.h>
#include <diamesh/utils.h>

// Qt headers
#include <QDebug>
#include <QFile>
#include <QDir>
#include <QByteArray>

namespace Hns {

TopologyCatalogI::TopologyCatalogI(const Ice::CommunicatorPtr & communicator, Ice::ObjectAdapterPtr adapter, const QString& settingsFileName)
    : _Hns_TopologyCatalog(communicator, adapter, settingsFileName)
{
    _id.name = "topologyCatalog";
    _storageType = QString::fromStdString("topology");
    _adapter->add(this, _id);
    _aclTargetTypes << Rights::ACL_TARGET_TOPOLOGYENTRY;

    _dbtypes << CatalogI::TopologyType;

    _dbStorage = new CatalogDBStorage(_settingsFileName);
    _storageDocType = "HnsTopologyConfig";

    _init();

    QObject::connect(MasterI::instance()->entityCatalog(), SIGNAL(entryChanged(Ice::Long)), this, SLOT(_entityEntryChangedSlot(Ice::Long)));
}

TopologyCatalogI::~TopologyCatalogI()
{
}

TopologyCatalogPrx TopologyCatalogI::proxy()
{
    return TopologyCatalogPrx::uncheckedCast(_adapter->createProxy(_id));
}

TopologyEntryPtr TopologyCatalogI::getTopologyEntry(Ice::Long entryId, const ::Ice::Current &current)
{
    EntryPtr entry = getEntry(entryId, current);
    if (!entry) {
        qDebug() << tr("Unable to get the topology entry");
        return NULL;
    }
    return TopologyEntryPtr::dynamicCast(entry);
}

bool TopologyCatalogI::fixTopologyEntry(const TopologyEntryPtr &topologyEntry, const ::Ice::Current &current)
{
    // check if all the entities are still existing in the entity catalog
    // (or are templates or models)
    QMap<int, EntityEntryPtr> entitiesMap(topologyEntry->entitiesMap);
    QMap<int, EntityEntryPtr> existingEntitiesMap;
    EntityCatalogI *entityCatalog = MasterI::instance()->entityCatalog();
    QMapIterator<Ice::Int, EntityEntryPtr> entitiesIt(entitiesMap);
    while (entitiesIt.hasNext()) {
        entitiesIt.next();
        Ice::Int entityId = entitiesIt.key();
        EntityEntryPtr entityEntry = entitiesIt.value();
        if ((entityEntry->isModel || entityEntry->id == 0 || entityCatalog->getEntry(entityEntry->id, current))
                && EntityEntryRelationshipMapper::getDiffUuids(entityEntry->uuid.c_str()).empty())
            existingEntitiesMap.insert(entityId, entityEntry);
    }

    topologyEntry->entitiesMap = existingEntitiesMap.toStdMap();

    QList<TopologyLink> topologyLinkList = QList<TopologyLink>::fromStdList(topologyEntry->topologyLinks);
    QList<TopologyLink> newTopologyLinkList;
    foreach (TopologyLink topologyLink, topologyLinkList) {
        if (!QString::fromStdString(topologyLink.entitySide).isEmpty()) {
            int entityId = QString::fromStdString(topologyLink.entitySide).split(':').first().toInt();
            if (!existingEntitiesMap.contains(entityId))
                topologyLink.entitySide.clear();
        }
        newTopologyLinkList.append(topologyLink);
    }
    topologyEntry->topologyLinks = newTopologyLinkList.toStdList();

    /*TODO: when wires in catalog check if the wire still exists*/

    // fix the layout
    TopologyLayout layout = getTopologyLayout(topologyEntry);
    // check the entities in the layout
    QMap<Ice::Long, EntityLayoutPtr> entityLayoutsMap(layout.entityLayouts);
    layout.entityLayouts.clear();
    foreach (EntityLayoutPtr entityLayout, entityLayoutsMap) {
        if (existingEntitiesMap.contains(entityLayout->id))
            layout.entityLayouts[entityLayout->id] = entityLayout;
    }

    // mark the entry as a valid entry
    topologyEntry->status = TOPOLOGY_STATUS_OK;

    bool result = updateEntry(topologyEntry, current);
    if (result)
        result = updateTopologyLayout(topologyEntry, layout);

    return result;
}

Hns::ByteArray TopologyCatalogI::getScreenshot(Ice::Long entryId, const ::Ice::Current &current)
{
    QReadLocker locker(&_screenshotLock);
    EntryPtr entry = getEntry(entryId, current);

    if (!entry) {
        qCritical() << tr("Unable to get the topology screenshot, entry %1 was not found").arg(entryId);
        return Hns::ByteArray();
    }
    if (QString::fromStdString(entry->path).isEmpty())
        return Hns::ByteArray();

    QString screenShotPath=QString("%1/screenshot.png").arg(getFullEntryPathFromMixed(entry->path, entry->uuid).c_str());
    QFile file(screenShotPath);
    if (!file.exists()) {
        qCritical () << tr("Screenshot file %1 does not exist").arg(screenShotPath);
        return Hns::ByteArray();
    }

    if (!file.open(QIODevice::ReadOnly)) {
        qCritical () << tr("Screenshot file %1 could not be opened").arg(screenShotPath);
        return Hns::ByteArray();
    }

    QByteArray array = file.readAll();
    if (array.isEmpty())  {
        qCritical () << tr("No data was read from screenshot file %1").arg(screenShotPath);
        file.close();
        return Hns::ByteArray();
    }

    file.close();
    const char * data = array.constData();
    return Hns::ByteArray(data, data + array.size());
}

bool TopologyCatalogI::setScreenshot(Ice::Long entryId, const Hns::ByteArray &data, const ::Ice::Current &current)
{
    QWriteLocker locker(&_screenshotLock);
    EntryPtr entry = getEntry(entryId, current);
    if (!entry) {
        qCritical() << tr("Unable to set the topology screenshot, entry %1 was not found").arg(entryId);
        return false;
    }

    QString screenShotPath=QString("%1/screenshot.png").arg(getFullEntryPathFromMixed(entry->path, entry->uuid).c_str());
    QFile file(screenShotPath);

    if (!file.open(QIODevice::WriteOnly)) {
        qCritical () << tr("Screenshot file %1 could not be opened").arg(screenShotPath);
        return false;
    }

    QByteArray screenshot;
    for(Hns::ByteArray::const_iterator it = data.begin(); it != data.end(); ++it)
        screenshot.append(*it);
    qint64 length = screenshot.size();
    qint64 lengthWritten = file.write(screenshot);
    if (lengthWritten != length) {
        qCritical() << tr("Error : only %1 on %2 bytes were written when saving topology screenshot in %3").arg(lengthWritten).arg(length).arg(screenShotPath);
        file.close();
        return false;
    }
    file.close();
    emit screenshotUpdated(entryId);
    return true;
}

bool TopologyCatalogI::copyEntryResources(const EntryPtr &fromEntry, const EntryPtr &toEntry, bool createStorage)
{
    if (!CatalogI::copyEntryResources(fromEntry, toEntry))
        return false;

    CatalogFileStorage* fs = NULL;

    QString fromEntryMixedPath;
    if (fromEntry->path == "") {
        // if the topology was never saved
        fs = getDefaultFileStorage();
        fromEntryMixedPath = QString("%1:%2").arg(fs->storageName()).arg("/.tmp/");
    } else {
        fs = _getEntryFileStorage(fromEntry);
        fromEntryMixedPath = QString::fromStdString(fromEntry->path);
    }
    if (!fs) {
        qCritical() << tr("No file storage was found, unable to delete the topolgy entry");
        return false;
    }
    QString fromPath = fs->getFullPathFromMixed(fromEntryMixedPath, QString::fromStdString(fromEntry->uuid));
    if (!QFile::exists(fromPath)) {
        qCritical() << Q_FUNC_INFO <<tr("cannot find %1 path").arg(QString::fromStdString(fromEntry->uuid));
        return false;
    }
    QString toEntryMixedPath;
    if (toEntry->path == "") {
        // if the topology was never saved
        fs = getDefaultFileStorage();
        toEntryMixedPath = QString("%1:%2").arg(fs->storageName()).arg("/.tmp/");
    } else {
        fs = _getEntryFileStorage(toEntry);
        toEntryMixedPath = QString::fromStdString(toEntry->path);
    }
    if (!fs) {
        qCritical() << Q_FUNC_INFO <<tr("No file storage was found, unable to delete the topolgy entry");
        return false;
    }
    QString toPath = fs->getFullPathFromMixed(toEntryMixedPath, QString::fromStdString(toEntry->uuid));
    if (!QFile::exists(toPath)) {
        if (!createStorage) {
            qCritical() << tr("cannot find %1 path").arg(QString::fromStdString(toEntry->uuid));
            return false;
        } else {
            fs->createEntryDir(toPath);
        }
    }
    // copy pcap folder
    QDir pcapDir(fromPath + "/pcap");
    if (pcapDir.exists())
        Diamesh::copyDirectory(fromPath + "/pcap", toPath + "/pcap");

    // copy memory dumps folder
    QDir memoryDumpsFolder(fromPath + "/memorydumps");
    if (memoryDumpsFolder.exists())
        Diamesh::copyDirectory(fromPath + "/memorydumps", toPath + "/memorydumps");

    // delete .tmp folder
    if (fromEntry->path == "") {
        // if the topology was never saved
        if (!getDefaultFileStorage()->deleteEntryDir(fromPath)) {
            qCritical() << tr("Error while deleting the topology directory");
        }
    }

    return true;
}

TopologyLayout TopologyCatalogI::getTopologyLayout(const EntryPtr &entry)
{
    QWriteLocker locker(&_entriesLock);

    CatalogFileStorage * fileStorage = _getEntryFileStorage(entry);
    if (!fileStorage) {
        qCritical() << tr("Unable to get topology layout");
        return TopologyLayout();
    }
    return fileStorage->getTopologyLayout(entry);
}

bool TopologyCatalogI::updateTopologyLayout(const EntryPtr entry, TopologyLayout layout)
{
    QWriteLocker locker(&_entriesLock);

    CatalogFileStorage *fileStorage = _getEntryFileStorage(entry);
    if (!fileStorage) {
        qCritical() << tr("Unable to update topology entry");
        return false;
    }
    if (!fileStorage->updateTopologyLayout(entry, layout)) {
        qCritical() << tr("Unable to update topology %1 layout in catalog filesystem").arg(entry->id);
        return false;
    }
    return true;
}

Ice::Int TopologyCatalogI::cloneTopology(Ice::Long topologyId, Ice::Int flags, const std::string &newName, const Ice::Current &current)
{
    EntryPtr entry = getEntry(topologyId, current);
    if (!entry)
        return 0;
    TopologyEntryPtr topologyEntry = TopologyEntryPtr::dynamicCast(entry);
    if (!topologyEntry) {
        qCritical() << tr("Entry %1 is not a topology entry").arg(topologyId);
        return 0;
    }
    if (!lockEntry(topologyId, LOCK_TYPE_CLONE))
        return 0;

    TopologyCloneJob *job = new TopologyCloneJob(topologyEntry, flags, MasterI::instance()->sessionManager()->currentUser(current), QString::fromStdString(newName));
    QObject::connect(job, SIGNAL(cancelled(Ice::Int)), this, SLOT(_cloneJobCancelledSlot(Ice::Int)));
    QObject::connect(job, SIGNAL(finished(Ice::Int)), this, SLOT(_cloneJobFinishedSlot(Ice::Int)));

    int jobId = JobManagerI::instance()->addJob(job);

    job->execute();

    return jobId;
}

void TopologyCatalogI::_entityEntryChangedSlot(Ice::Long entityEntryId)
{
    QWriteLocker locker(&_entriesLock);
    EntryPtr updatedEntry = MasterI::instance()->entityCatalog()->getEntry(entityEntryId);

    foreach (CatalogItemWrapper *wrapper, _catalogItemHash) {
        if (!qobject_cast<TopologyEntryWrapper *>(wrapper))
            continue;
        TopologyEntryPtr topologyEntry = TopologyEntryPtr::dynamicCast(wrapper->catalogItem());
        if (!topologyEntry) {
            qCritical() << tr("Unable to take entry change into account, entry %1 is not a topology entry").arg(wrapper->catalogItem()->id);
            return;
        }

        for (std::map<int, EntityEntryPtr>::iterator it = topologyEntry->entitiesMap.begin() ;
             it != topologyEntry->entitiesMap.end() ; it++) {
            EntityEntryPtr entityEntry = it->second;
            if (entityEntry->id == entityEntryId)
                it->second = EntityEntryPtr::dynamicCast(updatedEntry);
        }

        for (std::map<int, WireEntryPtr>::iterator it = topologyEntry->wiresMap.begin() ;
             it != topologyEntry->wiresMap.end() ; it++) {
            WireEntryPtr wireEntry = it->second;
            if (wireEntry->id == entityEntryId)
                it->second = WireEntryPtr::dynamicCast(updatedEntry);
        }
    }
}

void TopologyCatalogI::_cloneJobCancelledSlot(Ice::Int jobId)
{
    TopologyCloneJob *job = qobject_cast<TopologyCloneJob *>(JobManagerI::instance()->job(jobId));
    unlockEntry(job->originalEntry()->id);
}

void TopologyCatalogI::_cloneJobFinishedSlot(Ice::Int jobId)
{
    TopologyCloneJob *job = qobject_cast<TopologyCloneJob *>(JobManagerI::instance()->job(jobId));
    unlockEntry(job->originalEntry()->id);
}

void TopologyCatalogI::_handleUuidConflicts(const QMultiHash<QString, EntryPtr> &multiHashUuidEntryPtr)
{
    foreach (QString uuid, multiHashUuidEntryPtr.keys()) {
        if (multiHashUuidEntryPtr.count(uuid) > 1) {
            foreach(EntryPtr topologyToModify, multiHashUuidEntryPtr.values(uuid)) {
                TopologyEntryPtr topologyEntry = TopologyEntryPtr::dynamicCast(topologyToModify);
                topologyEntry->status = TOPOLOGY_UUID_CONFLICT;
            }
        }
    }
}

void TopologyCatalogI::_markEntryAsInvalid(const EntryPtr &entry)
{
    entry->status = TOPOLOGY_LOADING_ERROR;
}

} // namespace Hns
