/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "clonejobdomain.h"
#include "macaddressmanagerI.h"
#include "catalog/entityentryrelationshipmapper.h"
#include "catalog/entitycatalogI.h"
#include "catalog/entityentrywrapper.h"
#include "masterI.h"

// hynesim server headers
#include "libhnsserver/processrunner.h"

// Generated hynesim headers
#include "common/domainentry.h"
#include "common/clonejobflags.h"

// Qt headers
#include <QDebug>
#include <QFileInfo>
#include <QDir>
#include <QTimer>

namespace Hns {

CloneJobDomain::CloneJobDomain(Hns::EntityEntryPtr originalEntry, Hns::EntityEntryPtr clonedEntry, int flags)
    : CloneJobImplementation(originalEntry,clonedEntry, flags)
    , _timer(0)
    , _currentStorage(-1)
{
}

CloneJobDomain::~CloneJobDomain()
{
}

void CloneJobDomain::_run()
{
    _timer = new QTimer(this);
    QObject::connect(_timer, SIGNAL(timeout()), this, SLOT(_timeoutSlot()));
     Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(_clonedEntry);

     QMap<Ice::Long, Hns::Netcard> netcards(domainEntry->netcards);
     QMapIterator<Ice::Long, Hns::Netcard> iterator(netcards);
     QMap<Ice::Long, Hns::Netcard> newNetcards;
     while (iterator.hasNext()) {
         iterator.next();
         Ice::Long id = iterator.key();
         Hns::Netcard netcard = iterator.value();
         netcard.macAddress = MacAddressManagerI::instance()->generateRandomAddress("");
         newNetcards.insert(id, netcard);
     }
     domainEntry->netcards = newNetcards.toStdMap();
     _initStoragePercent();

     _timer->start(1000);
     if (!_copyDisks(domainEntry)) {
         _timer->stop();
         cancel();
         return;
     }
     if (!_copyFileSystems()) {
         _timer->stop();
         cancel();
         return;
     }
     if (!_copyAdditionalFiles()) {
         _timer->stop();
         cancel();
         return;
     }
     _updateDisks();
     if (_flags & Hns::LINKED_CLONE) {
         /*clonedEntry will be updated in DomainEntryWrapper::init()*/
         DomainEntryWrapper *originalEntryWrapper = qobject_cast<DomainEntryWrapper *>(MasterI::instance()->entityCatalog()->catalogItem(_originalEntry->id));
         if (!originalEntryWrapper)
             qCritical() << tr("Unable to update parent entry with id %1").arg(_originalEntry->id);
         else
             originalEntryWrapper->addDomainDiffUuids(QString::fromStdString(domainEntry->uuid));
     } else {
         DomainEntryPtr originalEntry = DomainEntryPtr::dynamicCast(_originalEntry);
         if (!originalEntry->baseUuid.empty()) {
             QString baseUuid = QString::fromStdString(originalEntry->baseUuid);
             EntryPtr baseEntry = MasterI::instance()->entityCatalog()->getEntryByUuid(baseUuid);
             DomainEntryWrapper *originalEntryWrapper = qobject_cast<DomainEntryWrapper *>(MasterI::instance()->entityCatalog()->catalogItem(baseEntry->id));
             if (!originalEntryWrapper)
                 qCritical() << tr("Unable to update parent entry with id %1").arg(baseEntry->id);
             else
                 originalEntryWrapper->addDomainDiffUuids(QString::fromStdString(domainEntry->uuid));
         }
     }
     _timer->stop();
     _updateProgress(1.0);
}

bool CloneJobDomain::_copyDisks(const Hns::DomainEntryPtr &domainEntry)
{
    QList<Hns::Disk> disks = QList<Hns::Disk>::fromStdList(domainEntry->disks);
    for (int i = 0 ; i < disks.size() ; ++i) {
        Hns::Disk disk = disks[i];
        if (disk.device == "disk" && disk.type == "file") {
            QString source = QString::fromStdString(disk.source);
            if (source.isEmpty()) {
                qCritical() << tr("Source disk is empty");
                return false;
            }
            ++_currentStorage;

            if ((_flags & LINKED_CLONE)) {
                if (!_createDiffDisk(source))
                    return false;
            } else {
                if (!_copyDisk(source))
                    return false;
            }


        }
    }
    return true;
}

bool CloneJobDomain::_copyDisk(const QString &disk)
{
    QString source = _fullSourcePath(disk);
    QString destination = _fullDestinationPath(disk);

    QFileInfo destinationFileInfo(destination);
    QDir destinationDirUp(destinationFileInfo.absolutePath());

    if(!destinationDirUp.exists())
        destinationDirUp.mkpath(".");

    HnsServer::ProcessRunner process;
    QStringList arguments = QStringList() << source << destination;
    if (!process.runSynchronous("cp", arguments, -1))
        return false;
    return true;
}

bool CloneJobDomain::_createDiffDisk(const QString &disk)
{
    Q_UNUSED(disk);
    return false;
}

bool CloneJobDomain::_copyFileSystems()
{
    foreach (const QString &fileSystem, fileSystems()) {
        if (fileSystem.isEmpty())
            continue;
        ++_currentStorage;
        if (!_copyFileSystem(fileSystem)) {
            qCritical() << tr("Failed to copy filesystem %1").arg(fileSystem);
            return false;
        }
    }
    return true;
}

bool CloneJobDomain::_copyFileSystem(const QString &fileSystem)
{
    QString source = _fullSourcePath(fileSystem);
    QString destination = _fullDestinationPath(fileSystem);
    QFileInfo destinationFileInfo(destination);
    QDir destinationDir = destinationFileInfo.absoluteDir();
    if (!destinationDir.exists()) {
        QString dirName = destinationDir.dirName();
        destinationDir.cdUp();
        destinationDir.mkpath(dirName);
    }
    HnsServer::ProcessRunner process;
    QStringList args = QStringList() << "-a" << source << destination;
    if (!process.runSynchronous("cp", args, -1))
        return false;
    return true;
}

bool CloneJobDomain::_copyAdditionalFiles()
{
    foreach (const QString &additionalFile, additionalFiles()) {
        if (additionalFile.isEmpty())
            continue;
        QString source = _fullSourcePath(additionalFile);
        QString destination = _fullDestinationPath(additionalFile);
        if (!QFile::copy(source, destination)) {
            qCritical() << tr("Failed to copy file %1 to %2").arg(source).arg(destination);
            return false;
        }
    }
    return true;
}

void CloneJobDomain::_updateDisks()
{
}

QStringList CloneJobDomain::fileSystems() const
{
    return _fileSystems;
}

void CloneJobDomain::addFileSystem(const QString &fileSystem)
{
    _fileSystems.append(fileSystem);
}

QStringList CloneJobDomain::additionalFiles() const
{
    return _additionalFiles;
}

void CloneJobDomain::addAdditionalFile(const QString &additionalFile)
{
    _additionalFiles.append(additionalFile);
}

double CloneJobDomain::getGeneralProgress()
{
    int counter = 0;
    if (_currentStorage == -1)
        return 0.0;
    if (_currentStorage >= _storageInformation.size())
        return 1.0;

    DomainEntryPtr domain = DomainEntryPtr::dynamicCast(_originalEntry);
    QList<Hns::Disk> disks = QList<Hns::Disk>::fromStdList(domain->disks);
    foreach (Hns::Disk disk, disks) {
        if (disk.device == "disk" && disk.type == "file") {
            if (counter == _currentStorage) {
                QString diskPath = _fullDestinationPath(QString::fromStdString(disk.source));
                qlonglong actualSize = QFileInfo(diskPath).size();
                double currentProgress = (double)actualSize / (double)_storageInformation.at(_currentStorage).totalSize;
                double finalProgress = 0;
                for (int i = 0 ; i < _currentStorage ; ++i) {
                    finalProgress += _storageInformation.at(i).percent;
                }
                finalProgress += currentProgress * _storageInformation.at(_currentStorage).percent;
                return finalProgress;
            }
            ++counter;
        }
    }
    foreach (const QString &fileSystem, fileSystems()) {
        if (counter == _currentStorage) {
            QString fspath = _fullDestinationPath(fileSystem);
            int filesWritten = _getDirCount(fspath);
            double currentProgress = (double)filesWritten / (double)_storageInformation.at(_currentStorage).totalSize;
            return currentProgress * _storageInformation.at(_currentStorage).percent;
        }
        ++counter;
    }
    return 0.0;
}

void CloneJobDomain::_timeoutSlot()
{
    double generalProgress = getGeneralProgress();
    if (generalProgress < 1.0)
        _updateProgress(generalProgress);
}

void CloneJobDomain::_initStoragePercent()
{
    QList<qlonglong> sizes;
    QList<int> lengths;
    DomainEntryPtr domain = DomainEntryPtr::dynamicCast(_originalEntry);
    QList<Hns::Disk> disks = QList<Hns::Disk>::fromStdList(domain->disks);
    foreach (Hns::Disk disk, disks) {
        if (disk.device == "disk" && disk.type == "file") {
            QString diskPath = _fullSourcePath(QString::fromStdString(disk.source));
            QFileInfo fileInfo(diskPath);
            sizes.append(fileInfo.size());
            lengths.append(0);
        }
    }
    foreach (const QString &fileSystem, fileSystems()) {
        QString fsPath = _fullSourcePath(fileSystem);
        HnsServer::ProcessRunner process;
        QStringList args = QStringList() << "-s" << fsPath;
        process.runSynchronous("du", args, 10000);
        QString output = process.getOutput();
        QRegExp pattern("^(\\d+)");
        pattern.indexIn(output);
        qlonglong size = pattern.cap(0).toLongLong();
        sizes.append(size);
        int length = _getDirCount(fsPath);
        lengths.append(length);
    }

    qlonglong total = 0;
    foreach (qlonglong size, sizes)
        total += size;

    if (!total)
        return;
    for (int i = 0, size = sizes.size() ; i < size ; ++i) {
        CloneJobDomainSize storageInfo;
        storageInfo.percent = (double)sizes.at(i) / (double)total;
        if (lengths.at(i) > 0)
            storageInfo.totalSize = lengths.at(i);
        else
            storageInfo.totalSize = sizes.at(i);
        _storageInformation.append(storageInfo);
    }
}

} // namespace Hns
