/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "clonejobvbox.h"
#include "scriptrunner.h"
#include "entryserializer.h"

// hynesim server headers
#include "libhnsserver/storagemanager.h"

// Qt headers
#include <QDebug>
#include <QDir>

namespace Hns {

static RegisterCloneImplementation<CloneJobVBox> registerMe(&CloneJobVBox::canClone);

CloneJobVBox::CloneJobVBox(Hns::EntityEntryPtr originalEntry, Hns::EntityEntryPtr clonedEntry, int flags)
    : CloneJobDomain(originalEntry, clonedEntry, flags)
    , _generalProgress(0.0)
{
}

CloneJobVBox::~CloneJobVBox()
{
}

bool CloneJobVBox::canClone(const EntityEntryPtr &entityEntry)
{
    Hns::DomainEntryPtr domain = Hns::DomainEntryPtr::dynamicCast(entityEntry);
    if (domain && domain->type == "vbox") {
        if (domain->snapshots.size() > 0) {
            qWarning() << tr("Cannot clone a vbox domain with snapshots");
            return false;
        }
        return true;
    }
    return false;
}

bool CloneJobVBox::_copyDisks(const Hns::DomainEntryPtr &domainEntry)
{
    QString path = _fullDestinationPath("");
    QDir entryDir(path);
    QDir origEntryDir(_fullSourcePath(""));
    if (!entryDir.mkpath(path)) {
        qCritical() << tr("The entry directory does not exist: %1").arg(path);
        return false;
    }
    QString tmpEntryFilePath = QDir::cleanPath(entryDir.path() + QDir::separator() + ".tmp.xml");
    QString finalEntryFilePath = QDir::cleanPath(entryDir.path() + QDir::separator() + "hnsEntryConfig.xml");
    EntrySerializer serializer;
    serializer.writeEntry(domainEntry, "HnsEntityConfig", entryDir.absolutePath());
    if (!QFile::rename(finalEntryFilePath, tmpEntryFilePath)) {
        /*We move the file generated by the serializer,
        * in order not to work on a file that could be
        * parsed by the master
        */
        qCritical() << tr("Can not rename %1 to %2").arg(finalEntryFilePath, tmpEntryFilePath);
        return false;
    }
    ScriptRunner runner;
    QObject::connect(&runner, SIGNAL(errorRead(QString)), this, SLOT(_readDataError(QString)));
    QObject::connect(&runner, SIGNAL(outputRead(QString)), this, SLOT(_readData(QString)));
    QStringList args = QStringList() << tmpEntryFilePath << origEntryDir.absolutePath();
    if (!runner.runSynchronous("cloneVbox.py", args, -1)){
        qCritical() << tr("Error while running cloneVbox.py with args: '%1'").arg(args.join(" "));
        return false;
    }
    std::string realPath = _clonedEntry->path;

    QFile::rename(tmpEntryFilePath, finalEntryFilePath);
    _clonedEntry = EntityEntryPtr::dynamicCast(serializer.readEntry(entryDir.absolutePath(), "HnsEntityConfig"));
    _clonedEntry->path = realPath;
    return true;
}

double CloneJobVBox::getGeneralProgress()
{
    return _generalProgress;
}

void CloneJobVBox::_readData(QString data)
{
    QRegExp pattern("(\\d+)[%.\\s]*.*");
    if (pattern.exactMatch(data)) {
        _generalProgress = pattern.cap(1).toDouble() / 100.0f;
    }
}

void CloneJobVBox::_readDataError(QString data)
{
    qCritical() << tr("Error while running clone job %1:").arg(id()) << data;
}

} // namespace Hns
