/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "entityclonejob.h"
#include "masterI.h"
#include "catalog/entitycatalogI.h"
#include "clonejobs/clonejobimplementation.h"

// Generated hynesim headers
#include "common/domainentry.h"
#include "common/types.h"
#include "common/clonejobflags.h"
#include "common/cloneutils.h"

// ZeroC Ice headers
#include <Ice/ObjectAdapter.h>

// Qt headers
#include <QDebug>
#include <QSharedPointer>

namespace Hns {

EntityCloneJob::EntityCloneJob(EntityEntryPtr originalEntry, Ice::Int flags, const Diamesh::UserPtr &currentUser)
    : _impl(0)
    , _flags(flags)
    , _clonedEntryId(0)
{
    if ((_flags & LINKED_CLONE) && !canHandleLinkedClone(originalEntry)) {
        qWarning() << tr("A linked clone was requested for entry %1 but it is not supported, doing a full clone instead").arg(originalEntry->id);
        _flags &= ~LINKED_CLONE;
    }
    _originalEntry = originalEntry;
    _currentUser = currentUser;
    _clonedEntry = Hns::EntityEntryPtr::dynamicCast(_originalEntry->ice_clone());
    _clonedEntry->id = 0;
    Hns::DomainEntryPtr clonedDomainEntry = Hns::DomainEntryPtr::dynamicCast(_clonedEntry);
    if (clonedDomainEntry)
        clonedDomainEntry->diffUuids.clear();
    if (_flags & Hns::REMOVE_TEMPLATE)
        _clonedEntry->isTemplate = false;

    if (!QString::fromStdString(_clonedEntry->shortName).endsWith(" cloned"))
        _clonedEntry->shortName.append(" cloned");

    if (currentUser)
        _clonedEntry->hSetId = currentUser->hSetId;

}

EntityCloneJob::~EntityCloneJob()
{
}

void EntityCloneJob::_run()
{
    EntityCatalogI *entityCatalog = MasterI::instance()->entityCatalog();
    _clonedEntry->uuid = entityCatalog->generateUnusedUUID().toStdString();

    if (_originalEntry->hasResources) {
        _impl = CloneJobImplementation::createImplementation(_originalEntry, _clonedEntry, _flags);
        if (!_impl) {
            cancel();
            return;
        }
        QObject::connect(_impl, SIGNAL(progressChanged(Ice::Int,Ice::Double)),
                         this, SLOT(_updateImplementationProgress(Ice::Int,Ice::Double)));
        QObject::connect(_impl, SIGNAL(cancelled(Ice::Int)),
                         this, SLOT(_implementationCancelled(Ice::Int)));
        _impl->execute();
    } else {
        _updateImplementationProgress(0, 1.0);
    }
}

void EntityCloneJob::_updateImplementationProgress(Ice::Int jobID, Ice::Double progress)
{
    Q_UNUSED(jobID)
    if (progress == 1) {
        if (!_impl) {
            EntityCatalogI *entityCatalog = MasterI::instance()->entityCatalog();
            _clonedEntryId = entityCatalog->addEntry(_clonedEntry, _currentUser);
            _updateProgress(1);
        } else {
            _getEntryModified(_impl->getEntityEntry());
        }
        return;
    }
    _updateProgress(progress);
}

void EntityCloneJob::_getEntryModified(::Hns::EntityEntryPtr entry)
{
    _clonedEntry = entry;
    _impl->deleteLater();
    _impl = 0;
    int catalogFlags = 0;
    EntityCatalogI *entityCatalog = MasterI::instance()->entityCatalog();
    if (_flags & LINKED_CLONE) {
        entityCatalog->updateEntry(_originalEntry);
    } else {
        DomainEntryPtr originalEntry = DomainEntryPtr::dynamicCast(_originalEntry);
        if (!originalEntry->baseUuid.empty()) {
            QString baseUuid = QString::fromStdString(originalEntry->baseUuid);
            EntryPtr baseEntry = MasterI::instance()->entityCatalog()->getEntryByUuid(baseUuid);
            entityCatalog->updateEntry(baseEntry);
        }
    }
    _clonedEntryId = entityCatalog->addEntry(_clonedEntry, _currentUser, catalogFlags);
    _updateProgress(1);
}

void EntityCloneJob::_implementationCancelled(Ice::Int id)
{
    Q_UNUSED(id);
    cancel();
}

EntityEntryPtr EntityCloneJob::clonedEntry()
{
    return _clonedEntry;
}

void EntityCloneJob::setClonedEntry(Hns::EntityEntryPtr newEntry)
{
    _clonedEntry = newEntry;
}

Ice::Long EntityCloneJob::clonedEntryId()
{
    return _clonedEntryId;
}

} // namespace Hns
