#include "topologyclonejob.h"
#include "masterI.h"
#include "jobmanagerI.h"
#include "catalog/entitycatalogI.h"
#include "catalog/topologycatalogI.h"
#include "entityclonejob.h"
#include "entryserializer.h"

#include "common/clonejobflags.h"

#include <QDebug>

namespace Hns {

TopologyCloneJob::TopologyCloneJob(TopologyEntryPtr originalEntry, int flags, const Diamesh::UserPtr &currentUser, const QString &newName)
    : JobI()
    , _originalEntry(originalEntry)
    , _currentUser(currentUser)
    , _flags(flags)
    , _remainingEntities(0)
    , _entitiesCount(0)
    , _topologyFolderId(0)
{
    _clonedEntry = Hns::TopologyEntryPtr::dynamicCast(_originalEntry->ice_clone());
    _clonedEntry->id = 0;
    if (!newName.isEmpty())
        _clonedEntry->shortName = newName.toStdString();
    else if (!QString::fromStdString(_originalEntry->shortName).endsWith(" cloned"))
        _clonedEntry->shortName += " cloned";

}

TopologyCloneJob::~TopologyCloneJob()
{
}

const TopologyEntryPtr &TopologyCloneJob::originalEntry()
{
    return _originalEntry;
}

void TopologyCloneJob::_run()
{
    TopologyCatalogI *topologyCatalog = MasterI::instance()->topologyCatalog();
    _clonedEntry->uuid = topologyCatalog->getUnusedUUID();
    FolderPtr folderEntry = new Folder();
    folderEntry->shortName = _clonedEntry->shortName;
    _topologyFolderId = MasterI::instance()->entityCatalog()->addFolder(folderEntry);
    if (!_topologyFolderId)
        qWarning() << tr("Unable to create folder, all cloned entities will be in the catalog root.");

    //initialize counters and maps
    _clonedEntry->entitiesMap.clear();
    _clonedEntry->wiresMap.clear();
    for (std::map<int, EntityEntryPtr>::iterator it = _originalEntry->entitiesMap.begin() ; it != _originalEntry->entitiesMap.end() ; ++it) {
        EntityEntryPtr entityEntryToClone = it->second;
        if (entityEntryToClone->id != 0) {
            ++_remainingEntities;
            ++_entitiesCount;
        }
    }
    for (std::map<int, WireEntryPtr>::iterator it = _originalEntry->wiresMap.begin() ; it != _originalEntry->wiresMap.end() ; ++it) {
        WireEntryPtr wireToClone = it->second;
        if (wireToClone->id != 0) {
            ++_remainingEntities;
            ++_entitiesCount;
        }
    }

    // Keep only CloneJobFlags
    int flags = _flags & 0x0f;

    //clone entities
    for (std::map<int, EntityEntryPtr>::iterator it = _originalEntry->entitiesMap.begin() ; it != _originalEntry->entitiesMap.end() ; ++it) {
        EntityEntryPtr entityEntryToClone = it->second;
        if (entityEntryToClone->id != 0) {
            int jobId = MasterI::instance()->entityCatalog()->cloneEntityEntry(entityEntryToClone->id, flags);
            if (!jobId) {
                qCritical() << tr("Unable to clone entity entry %1. Operation aborted").arg(it->first);
                cancel();
                return;
            }
            _jobOldEntryHash.insert(jobId, it->first);
            QObject::connect(JobManagerI::instance()->job(jobId), SIGNAL(finished(Ice::Int)), this, SLOT(_entityCloneJobFinishedSlot(Ice::Int)));
            QObject::connect(JobManagerI::instance()->job(jobId), SIGNAL(cancelled(Ice::Int)), this, SLOT(_entityCloneJobCancelledSlot(Ice::Int)));
            if (JobManagerI::instance()->job(jobId)->getProgress() >= 1.0)
                _entityCloneJobFinishedSlot(jobId);
        } else {
            EntityEntryPtr newEntityEntry = EntityEntryPtr::dynamicCast(entityEntryToClone->ice_clone());
            _clonedEntry->entitiesMap[it->first] = newEntityEntry;
        }
    }
    //clone wires
    for (std::map<int, WireEntryPtr>::iterator it = _originalEntry->wiresMap.begin() ; it != _originalEntry->wiresMap.end() ; ++it) {
        WireEntryPtr wireToClone = it->second;
        if (wireToClone->id != 0) {
            int jobId = MasterI::instance()->entityCatalog()->cloneEntityEntry(wireToClone->id, flags);
            if (!jobId) {
                qCritical() << tr("Unable to clone wire entry %1. Operation aborted").arg(it->first);
                cancel();
                return;
            }
            _jobOldEntryHash.insert(jobId, it->first);
            QObject::connect(JobManagerI::instance()->job(jobId), SIGNAL(finished(Ice::Int)), this, SLOT(_entityCloneJobFinishedSlot(Ice::Int)));
            QObject::connect(JobManagerI::instance()->job(jobId), SIGNAL(cancelled(Ice::Int)), this, SLOT(_entityCloneJobCancelledSlot(Ice::Int)));
            if (JobManagerI::instance()->job(jobId)->getProgress() >= 1.0)
                _entityCloneJobFinishedSlot(jobId);
        } else {
            WireEntryPtr newWire = WireEntryPtr::dynamicCast(wireToClone->ice_clone());
            _clonedEntry->wiresMap[it->first] = newWire;
        }
    }

    if (_flags & REMOVE_PCAP_FILES) {
        _clonedEntry->pcapInfoMap.clear();
    } else {
        if (!topologyCatalog->copyEntryResources(_originalEntry, _clonedEntry, true)) {
            qWarning() << tr("Unable to copy pcap files");
            _clonedEntry->pcapInfoMap.clear();
        }
    }

    if (!_remainingEntities)
        _addTopologyEntryToCatalog();
}

void TopologyCloneJob::_entityCloneJobFinishedSlot(Ice::Int jobId)
{
    --_remainingEntities;
    _updateProgress(qMin(1.0 - (double)_remainingEntities / _entitiesCount, 0.99));
    qlonglong entityId = _jobOldEntryHash.value(jobId);
    EntityCloneJob *cloneJob = qobject_cast<EntityCloneJob *>(JobManagerI::instance()->job(jobId));
    if (!cloneJob) {
        qCritical() << tr("Unable to get job with id %1").arg(jobId);
        return;
    }
    EntityEntryPtr clonedEntry = cloneJob->clonedEntry();
    clonedEntry->shortName = clonedEntry->shortName + " - " + _clonedEntry->shortName;
    clonedEntry->parentId = _topologyFolderId;
    if (!MasterI::instance()->entityCatalog()->updateEntry(clonedEntry)) {
        qWarning() << tr("Unable to update shortName and parentId of entry %1").arg(clonedEntry->id);
    }
    _clonedEntry->entitiesMap[entityId] = cloneJob->clonedEntry();
    if (!_remainingEntities && isRunning())
        _addTopologyEntryToCatalog();
}

void TopologyCloneJob::_entityCloneJobCancelledSlot(Ice::Int jobId)
{
    qCritical() << tr("Job %1 has been cancelled").arg(jobId);
    cancel();
}

void TopologyCloneJob::_addTopologyEntryToCatalog()
{
    TopologyCatalogI *topologyCatalog = MasterI::instance()->topologyCatalog();

    if (!topologyCatalog->addEntry(_clonedEntry, _currentUser)) {
        cancel();
        return;
    }

    QString fullPath = QString::fromStdString(topologyCatalog->getFullEntryPathFromMixed(_clonedEntry->path, _clonedEntry->uuid));
    if (fullPath.isEmpty()) {
        cancel();
        return;
    }

    if (!EntrySerializer::writeEntry(_clonedEntry, "HnsTopologyConfig", fullPath)) {
        cancel();
        return;
    }
    if (!topologyCatalog->updateTopologyLayout(_clonedEntry, topologyCatalog->getTopologyLayout(_originalEntry))) {
        cancel();
        return;
    }

    if (!topologyCatalog->setScreenshot(_clonedEntry->id, topologyCatalog->getScreenshot(_originalEntry->id)))
        qWarning() << tr("Unable to set topology screenshot");

    _updateProgress(1.0);
}

} // namespace Hns
