/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "customresourcemanagerI.h"
#include "masterI.h"

// hynesim common headers
#include "common/constants.h"

// ZeroC Ice headers
#include <Ice/ObjectAdapter.h>
#include <Ice/Current.h>

// Qt headers
#include <QStringList>
#include <QRegExp>
#include <QSettings>
#include <QFile>
#include <QFileInfo>
#include <QDir>
#include <QDebug>

namespace Hns {

CustomResourceManagerI::CustomResourceManagerI(const QString &settingsFileName)
    : _Hns_CustomResourceManager()
{
    _id.name = Constants::RESOURCE_MANAGER_PROXY_NAME;

    QSettings settings(settingsFileName, QSettings::IniFormat);
    settings.beginGroup(Constants::SETTINGS_HYNESIM_GROUP);
    QString path = settings.value(Constants::SETTINGS_RESOURCEPATH).toString();
    settings.endGroup();

    if (path.isEmpty()) {
        qCritical() << tr("Resource directory is not set in configuration, unable to use custom resources.");
    } else {
        QDir dir(path);
        if (!dir.exists()) {
            qCritical() << tr("Directory %1 does not exist, unable to use custom resources.").arg(path);
            path = QString();
        }
        // Avoid a bug when the path is relative and getResource() is called during an import because the CWD
        // is changed temporarily.
        if (!dir.isAbsolute())
            path = dir.absolutePath();
    }
    _resourceDirectoryPath = path;

    MasterI::instance()->adapter()->add(this, _id);
}

CustomResourceManagerI::~CustomResourceManagerI()
{
}

CustomResourceManagerPrx CustomResourceManagerI::proxy() const
{
    return CustomResourceManagerPrx::uncheckedCast(MasterI::instance()->adapter()->createProxy(_id));
}

Diamesh::StringList CustomResourceManagerI::getResourceKeys(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QReadLocker locker(&_resourcesLock);
    return _resources.toStdList();
}

Hns::ByteArray CustomResourceManagerI::getResource(const std::string &key, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QString resourceName = QString::fromStdString(key);
    if (resourceName.isEmpty())
        return Hns::ByteArray();

    if (_resourceDirectoryPath.isEmpty())
        return Hns::ByteArray();

    QString resourceFilePath = QDir::cleanPath(QString("%1/%2").arg(_resourceDirectoryPath).arg(resourceName));
    QFile resourceFile(resourceFilePath);
    if (!resourceFile.exists()) {
        qCritical () << tr("Resource file %1 does not exist").arg(resourceFilePath);
        return Hns::ByteArray();
    }
    if (!resourceFile.open(QIODevice::ReadOnly)) {
        qCritical () << tr("Resource file %1 could not be opened").arg(resourceFilePath);
        return Hns::ByteArray();
    }

    QByteArray array = resourceFile.readAll();
    if (array.isEmpty())  {
        qCritical () << tr("No data was read from Resource file %1").arg(resourceFilePath);
        resourceFile.close();
        return Hns::ByteArray();
    }
    resourceFile.close();
    const char * data = array.constData();
    return Hns::ByteArray(data, data + array.size());
}

bool CustomResourceManagerI::uploadResource(const std::string &key, const ByteArray &data, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QWriteLocker locker(&_resourcesLock);
    if (_resourceDirectoryPath.isEmpty())
        return false;

    QString graphicPath = QDir::cleanPath(QString("%1/%2").arg(_resourceDirectoryPath).arg(QString::fromStdString(key)));
    QFile file(graphicPath);

    if (!file.open(QIODevice::WriteOnly)) {
        qCritical () << tr("Icon file %1 could not be opened").arg(graphicPath);
        return false;
    }

    QByteArray graphicData;
    for(Hns::ByteArray::const_iterator it = data.begin(); it != data.end(); ++it)
        graphicData.append(*it);
    qint64 length = graphicData.size();
    qint64 lengthWritten = file.write(graphicData);
    if (lengthWritten != length) {
        qCritical() << tr("Error : only %1 on %2 bytes were written when saving custom icon in %3").arg(lengthWritten).arg(length).arg(graphicPath);
        file.close();
        return false;
    }
    file.close();
    _resources.append(key);
    emit resourceAdded(QString::fromStdString(key));
    return true;
}

bool CustomResourceManagerI::deleteResource(const std::string &key, const Ice::Current &current)
{
    Q_UNUSED(current)
    QWriteLocker locker(&_resourcesLock);
    if (_resourceDirectoryPath.isEmpty())
        return false;

    QString graphicPath = QDir::cleanPath(QString("%1/%2").arg(_resourceDirectoryPath).arg(QString::fromStdString(key)));
    QFile file(graphicPath);

    if (!file.remove()) {
        qCritical() << tr("Unable to delete icon file %1").arg(graphicPath);
        return false;
    }
    _resources.removeAll(key);
    emit resourceDeleted(QString::fromStdString(key));
    return true;
}

QString CustomResourceManagerI::getResourcePath(const QString &key)
{
    if (_resourceDirectoryPath.isEmpty()) {
        return "";
    }
    return QDir::cleanPath(QString("%1/%2").arg(_resourceDirectoryPath).arg(key));
}

QString CustomResourceManagerI::generateNewKey(const QString &key)
{
    QReadLocker locker(&_resourcesLock);
    QString newKey = key;
    while (_resources.contains(newKey.toStdString())) {
        QStringList splittedList = newKey.split(".");
        QString extension = splittedList.takeLast();
        QString baseName = splittedList.join(".");
        QRegExp rx("(.*)_(\\d+)$");
        int id = 1;
        if (baseName.contains(rx)) {
            baseName = rx.cap(1);
            id = rx.cap(2).toInt() + 1;
        }
        newKey = QString("%1_%2.%3").arg(baseName).arg(QString::number(id)).arg(extension);
    }
    return newKey;
}

const QString &CustomResourceManagerI::getResourceDirectory()
{
    return _resourceDirectoryPath;
}

void CustomResourceManagerI::addResource(const QString &key)
{
    QWriteLocker locker(&_resourcesLock);
    _resources.append(key.toStdString());
    emit resourceAdded(key);
}

void CustomResourceManagerI::_init()
{
    if (_resourceDirectoryPath.isEmpty())
        return;

    QDir resourceDir(_resourceDirectoryPath);
    resourceDir.setFilter(QDir::Files | QDir::NoSymLinks | QDir::NoDotAndDotDot | QDir::Readable);
    resourceDir.setSorting(QDir::Name);
    QFileInfoList dirList = resourceDir.entryInfoList();
    foreach(const QFileInfo &entryInfo, dirList)
        _resources.append(entryInfo.fileName().toStdString());
}

} // namespace Hns
