/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "domainI.h"
#include "jobmanagerI.h"
#include "memorydumpjob.h"
#include "masterI.h"

// hynesim libhnsserver headers
#include "libhnsserver/storagemanager.h"

// hynesim common headers
#include "common/constants.h"

// Generated hynesim headers
#include "common/entity.h"
#include "common/domainentry.h"

// diamesh headers
#include <diamesh/utils.h>
#include <diamesh/diamesh.h>
#include <diamesh/filetransferI.h>

// Qt headers
#include <QString>
#include <QList>
#include <QDateTime>
#include <QDebug>
#include <QSettings>
#include <QReadWriteLock>
#include <QFile>

namespace Hns {

DomainI::DomainI()
    : _availableForMemoryDump(true)
{
    Diamesh::setFlags(Hns::NET_CAPTURE | Hns::REMOTE_DISPLAY /*| Hns::SNAPSHOT_FLAG*/, _flags);
}

DomainI::~DomainI()
{
}

MemoryDumpInfoList DomainI::getMemoryDumpInfoList(const Ice::Current &current)
{
    Q_UNUSED(current)
    QReadLocker locker(&_memoryDumpLock);
    return _memoryDumpInfoHash.values().toStdList();
}

int DomainI::createMemoryDumpJob(const std::string &name, const Ice::Current &current)
{
    Q_UNUSED(current)
    Q_UNUSED(name)
    QWriteLocker locker(&_memoryDumpLock);
    if (!_availableForMemoryDump || getStatus() != STARTED)
        return -1;
    emit memoryDumpJobStarted();
    _availableForMemoryDump = false;
    HnsNode::DomainPrx domainPrx = HnsNode::DomainPrx::uncheckedCast(_entityProxy);
    MemoryDumpJob *job = new MemoryDumpJob(domainPrx);
    job->setName(QString::fromStdString(name));
    job->setTopologyPath(_topologyPath);

    int jobId = JobManagerI::instance()->addJob(job);

    QObject::connect(job, SIGNAL(progressChanged(Ice::Int,Ice::Double)),this, SLOT(_memoryDumpJobProgressChangedSlot(Ice::Int,Ice::Double)));
    QObject::connect(job, SIGNAL(cancelled(Ice::Int)), this, SLOT(_memoryDumpJobCancelledSlot(Ice::Int)));

    job->execute();

    return jobId;
}

void DomainI::deleteMemoryDump(const std::string &fileName, const Ice::Current &current)
{
    Q_UNUSED(current)
    QWriteLocker locker(&_memoryDumpLock);
    MemoryDumpInfo dumpToRemove;
    QMutableHashIterator<Ice::Int, MemoryDumpInfo> it(_memoryDumpInfoHash);
    while (it.hasNext()) {
        it.next();
        MemoryDumpInfo info = it.value();
        if (info.fileName == fileName) {
            dumpToRemove = info;
            it.remove();
            break;
        }
    }

    HnsServer::StorageManager *storageManager = Hns::MasterI::instance()->storageManager();
    QString path = QString("%1/memorydumps/%2")
            .arg(_topologyPath)
            .arg(QString::fromStdString(dumpToRemove.fileName));
    QString fullPath = storageManager->getFullPathFromMixed("topology", path);
    QFile file(fullPath);
    qDebug() << tr("Deleting %1").arg(fullPath);
    if (!file.remove())
        qCritical() << tr("Unable to delete %1").arg(fullPath);
    emit memoryDumpDeleted(QString::fromStdString(fileName));
}

void DomainI::setMemoryDumpInfoList(const MemoryDumpInfoList &memoryDumpInfoList, const Ice::Current &current)
{
    Q_UNUSED(current)
    QWriteLocker locker(&_memoryDumpLock);
    foreach (const MemoryDumpInfo &info, memoryDumpInfoList)
        _memoryDumpInfoHash.insertMulti(-1, info);
}

Diamesh::FileTransferPrx DomainI::downloadMemoryDump(const std::string &fileName, const Ice::Current &current)
{
    Q_UNUSED(current)
    HnsServer::StorageManager *storageManager = Hns::MasterI::instance()->storageManager();
    QString path = QString("%1/memorydumps/%2")
            .arg(_topologyPath)
            .arg(QString::fromStdString(fileName));
    QString fullPath = storageManager->getFullPathFromMixed("topology", path);
    if (QFile::exists(fullPath)) {
        ::Diamesh::FileTransferI *fileTransfer = new ::Diamesh::FileTransferI(fullPath, MasterI::instance()->adapter());
        return fileTransfer->proxy();
    }
    return NULL;
}

bool DomainI::_createSnapshot(const Snapshot &snapshot)
{
    QWriteLocker locker(&_entityLock);

    if (this->getStatus() != Hns::STOPPED) {
        qCritical() << tr("DomainI::createSnapshot() entity status is not stopped, couldn't create snapshot");
        return false;
    }

    _propagateStatus(Hns::BUSY);

    _checkDestroyed();

    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(_entityEntry);
    if (!domainEntry) {
        qCritical() << tr("DomainI::createSnapshot() null entry");
        _propagateStatus(Hns::STOPPED);
        return false;
    }

    foreach (Hns::Snapshot snap, QList<Hns::Snapshot>::fromStdList(domainEntry->snapshots)) {
        if (snapshot.name == snap.name) {
            qCritical() << tr("DomainI::createSnapshot() Snapshot name %1 already used").arg(QString::fromStdString(snap.name));
            _propagateStatus(Hns::STOPPED);
            return false;
        }
    }

    QString xmlDesc;
    QDateTime creationTime = QDateTime::currentDateTime();
    xmlDesc= QString("<domainsnapshot>"
                     "<name>%1</name>"
                     "<state>shutoff</state>"
                     "<description>%2</description>"
                     "<disks/>"
                     "<creationTime>%3</creationTime>"
                     "</domainsnapshot>").arg(QString::fromStdString(snapshot.name), QString::fromStdString(snapshot.description), QString::number(creationTime.toTime_t()));

    if (!_entityProxy->createSnapshot(xmlDesc.toStdString())) {
        qCritical() << tr("DomainI::createSnapshot() cannot create snapshot %1").arg(QString::fromStdString(snapshot.name));
        _propagateStatus(Hns::STOPPED);
        return false;
    }
    Hns::Snapshot snapshotCreated = _entityProxy->getSnapshot(snapshot.name);
    domainEntry->snapshots.push_back(snapshotCreated);
    std::string currentSnapshotName = snapshot.name;
    domainEntry->currentSnapshotName = currentSnapshotName;
    updateEntry(domainEntry);
    emit currentSnapshotChanged(snapshotCreated);
    _propagateStatus(Hns::STOPPED);
    return true;

}

bool DomainI::_revertSnapshot(const std::string &snapshotName)
{
    QWriteLocker locker(&_entityLock);

    if (this->getStatus() != Hns::STOPPED) {
        qCritical() << tr("DomainI::revertSnapshot() entity status is not stopped, couldn't revert snapshot");
        return false;
    }

    _propagateStatus(Hns::BUSY);

    _checkDestroyed();

    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(_entityEntry);
    if (!domainEntry) {
        qCritical() << tr("DomainI::revertSnapshot() null entry");
        _propagateStatus(Hns::STOPPED);
        return false;
    }
    //Check for the existence of snapshotName in snapshots
    bool contains = false;
    Hns::Snapshot snapshotRevertedTo;
    foreach (Hns::Snapshot snap, QList<Hns::Snapshot>::fromStdList(domainEntry->snapshots)) {
        if (snapshotName == snap.name) {
            contains = true;
            snapshotRevertedTo = snap;
        }
    }
    if (!contains) {
        qCritical() << tr("DomainI::revertSnapshot cannot revert to a non existent snapshot %1").arg(QString::fromStdString(snapshotName));
        _propagateStatus(Hns::STOPPED);
        return false;
    }

    if (!_entityProxy->revertSnapshot(snapshotName)) {
        qCritical() << tr("DomainI::revertSnapshot() cannot revert to snapshot %1").arg(QString::fromStdString(snapshotName));
        _propagateStatus(Hns::STOPPED);
        return false;
    }

    domainEntry->snapshots = Hns::DomainEntryPtr::dynamicCast(_entityProxy->getEntry())->snapshots;
    domainEntry->currentSnapshotName = snapshotName;
    updateEntry(domainEntry);
    emit currentSnapshotChanged(snapshotRevertedTo);
    _propagateStatus(Hns::STOPPED);
    return true;
}

bool DomainI::_deleteSnapshot(const std::string &snapshotName)
{
    QWriteLocker locker(&_entityLock);

    if (this->getStatus() != Hns::STOPPED) {
        qCritical() << tr("DomainI::deleteSnapshot() entity status is not stopped, couldn't delete snapshot");
        return false;
    }

    _propagateStatus(Hns::BUSY);

    _checkDestroyed();

    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(_entityEntry);
    if (!domainEntry) {
        qCritical() << tr("DomainI::deleteSnapshot() null entry");
        _propagateStatus(Hns::STOPPED);
        return false;
    }
    //Check for the existence of snapshotName in snapshots
    bool contains = false;
    foreach (Hns::Snapshot snap, QList<Hns::Snapshot>::fromStdList(domainEntry->snapshots)) {
        if (snapshotName == snap.name) {
            contains = true;
        }
    }
    if (!contains) {
        qCritical() << tr("DomainI::deleteSnapshot cannot delete a non existent snapshot %1").arg(QString::fromStdString(snapshotName));
        _propagateStatus(Hns::STOPPED);
        return false;
    }

    if (!_entityProxy->deleteSnapshot(snapshotName)) {
        qCritical() << tr("DomainI::deleteSnapshot() cannot delete the snapshot %1").arg(QString::fromStdString(snapshotName));
        _propagateStatus(Hns::STOPPED);
        return false;
    }
    QList<Hns::Snapshot> qlistSnapshots = QList<Hns::Snapshot>::fromStdList(domainEntry->snapshots);
    QMutableListIterator<Hns::Snapshot> mutableIt(qlistSnapshots);
    while(mutableIt.hasNext()) {
        mutableIt.next();
        Hns::Snapshot item = mutableIt.value();
        if (item.name == snapshotName) {
            mutableIt.remove();
        } else {
            if (!item.name.empty()) {
                mutableIt.setValue(_entityProxy->getSnapshot(item.name));
            }
        }
    }
    domainEntry->snapshots = qlistSnapshots.toStdList();
    updateEntry(domainEntry);
    if (domainEntry->snapshots.size() > 0) {
        std::string currentSnapshotName = _entityProxy->getCurrentSnapshotName();
        domainEntry->currentSnapshotName = currentSnapshotName;
        updateEntry(domainEntry);
        if (currentSnapshotName.empty())
            emit currentSnapshotChanged(Hns::Snapshot());
        else
            emit currentSnapshotChanged(_entityProxy->getSnapshot(currentSnapshotName));
    } else {
        domainEntry->currentSnapshotName = QString().toStdString();
        updateEntry(domainEntry);
        emit currentSnapshotChanged(Hns::Snapshot());
    }
    _propagateStatus(Hns::STOPPED);
    return true;
}

Snapshot DomainI::_getSnapshot(const std::string &snapshotName)
{
    QReadLocker locker(&_entityLock);

    _checkDestroyed();
    return _entityProxy->getSnapshot(snapshotName);
}

std::string DomainI::_getCurrentSnapshotName()
{
    QReadLocker locker(&_entityLock);

    _checkDestroyed();
    return _entityProxy->getCurrentSnapshotName();
}

QMap<Ice::Long, Hns::Plug> DomainI::_getPlugs()
{
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(_entityEntry);
    QMap<Ice::Long, Hns::Plug> plugMap;
    Hns::NetcardMap netcardMap = domainEntry->netcards;
    for (Hns::NetcardMap::iterator it = netcardMap.begin(); it != netcardMap.end(); ++it) {
        Ice::Long id = (*it).first;
        Hns::Netcard netcard = (*it).second;

        if (id != domainEntry->aminterface) {
            Hns::Plug plug;
            plug.id = id;
            plug.name = netcard.source;
            plugMap.insert(id, plug);
        }
    }
    return plugMap;
}

void DomainI::_initSnapshotFlags()
{
    QString configFileName = Diamesh::instance()->configFileName();
    QSettings settings(configFileName, QSettings::IniFormat);
    settings.beginGroup(Constants::SETTINGS_HYNESIM_GROUP);
    bool vboxSnapshotSupport = settings.value("vboxSnapshotSupport", false).toBool();
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(_entityEntry);
    if (domainEntry->type == "qemu" || domainEntry->type == "kvm" || (domainEntry->type == "vbox" && vboxSnapshotSupport))
        Diamesh::setFlags(Hns::SNAPSHOT_FLAG, _flags);
    if (domainEntry->type == "qemu" || domainEntry->type == "kvm")
        Diamesh::setFlags(Hns::MEMORY_DUMP_FLAG, _flags);
}

void DomainI::_initStatusesList()
{
    QWriteLocker locker(&_statusesLock);
    _Hns_Domain::_initStatusesList();
    _statusesList.append(Hns::PAUSED);
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(_entityEntry);
    if (domainEntry) {
        QString type = QString::fromStdString(domainEntry->type);
        if(type != "lxc" && type != "openvz")
            _statusesList.append(Hns::SAVED);
    }
    emit allowedStatusListChanged();
}

void DomainI::_memoryDumpJobProgressChangedSlot(Ice::Int jobId, Ice::Double progress)
{
    if (progress < 1.0)
        return;
    MemoryDumpJob *job = qobject_cast<MemoryDumpJob *>(JobManagerI::instance()->job(jobId));
    if (!job)
        return;

    QWriteLocker locker(&_memoryDumpLock);
    MemoryDumpInfo dumpInfo = job->memoryDumpInfo();
    _memoryDumpInfoHash.insert(jobId, dumpInfo);
    emit memoryDumpAdded(dumpInfo);
    _availableForMemoryDump = true;
}

void DomainI::_memoryDumpJobCancelledSlot(Ice::Int jobId)
{
    Q_UNUSED(jobId)
    QWriteLocker locker(&_memoryDumpLock);
    _availableForMemoryDump = true;
}

} // namespace Hns
