/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "dynamipsrouterdbI.h"
#include "masterI.h"

// hynesim common headers
#include "common/constants.h"

// diamesh headers
#include <diamesh/diamesh.h>

// ZeroC Ice headers
#include <Ice/ObjectAdapter.h>
#include <Ice/Current.h>
#include <Ice/Identity.h>

// Qt headers
#include <QMutex>
#include <QSettings>
#include <QDir>
#include <QDebug>
#include <QDomDocument>
#include <QDomElement>
#include <QDomNode>
#include <QDomNodeList>

namespace Hns {

DynamipsRouterDBI * DynamipsRouterDBI::_instance = NULL;

DynamipsRouterDBI::DynamipsRouterDBI()
{
    _id.name = "DynamipsRouterDBI";
    MasterI::instance()->adapter()->add(this, _id);
    _fillData();
}

DynamipsRouterDBI::~DynamipsRouterDBI()
{
}

DynamipsRouterDBI * DynamipsRouterDBI::instance()
{
    static QMutex mutex;
    if (!_instance) {
        mutex.lock();
        if (!_instance) {
            _instance = new DynamipsRouterDBI();
        }
        mutex.unlock();
    }
    return _instance;
}

DynamipsRouterDBPrx DynamipsRouterDBI::proxy()
{
   return DynamipsRouterDBPrx::uncheckedCast(MasterI::instance()->adapter()->createProxy(_id));
}

bool DynamipsRouterDBI::isValid(const ::Hns::EntryPtr &entry, QString &error) const
{
    DynamipsRouterEntryPtr dynamipsRouterEntry = DynamipsRouterEntryPtr::dynamicCast(entry);
    QString routerName = QString::fromStdString(dynamipsRouterEntry->router);


    if (!_dataDynamipsRouterMap.contains(routerName)) {
        error = QObject::tr("Unknown router \"%1\"").arg(routerName);
        return false;
    }

    DataDynamipsRouter dataDynamipsRouter = _dataDynamipsRouterMap[routerName];
    QString chassisName = QString::fromStdString(dynamipsRouterEntry->chassis);

    if (dataDynamipsRouter.availableChassis.count(dynamipsRouterEntry->chassis) == 0) {
        error = QObject::tr("Unknown chassis \"%1\" for router \"%2\"").arg(chassisName).arg(routerName);
        return false;
    }

    DynamipsRouterChassis dynamipsChassis = dataDynamipsRouter.availableChassis[dynamipsRouterEntry->chassis];


    //test there is a MB and if it s on the slot 0
    //search if there is a MB
    //found a MB module in chassis list

    bool foundMB = false;
    ModuleMap::iterator itorModuleChassis = dynamipsChassis.availableModules.begin();
    while (itorModuleChassis != dynamipsChassis.availableModules.end() && !foundMB) {
        if (itorModuleChassis->second.isMB)
            foundMB = true;
        itorModuleChassis++;
    }

    //if not found, search a MB module in module list
    ModuleMap::iterator itorModule = dataDynamipsRouter.availableModules.begin();
    while (itorModule != dataDynamipsRouter.availableModules.end() && !foundMB) {
        if (itorModule->second.isMB)
            foundMB = true;
        itorModule++;
    }
    if (foundMB) {
        ModulePositionMap::iterator itorMB = dynamipsRouterEntry->modulesMap.find(0);
        if (itorMB == dynamipsRouterEntry->modulesMap.end()) {
            error = QObject::tr("Router's slot 0 can't be empty");
            return false;
        } else if (!itorMB->second.isMB) {
            error = QObject::tr("Router's module in slot 0 has to be a motherboard (module \"%1\" isn't a motherboard)").arg(QString::fromStdString(itorMB->second.name));
            return false;
        }
    }


    //generalist test for all slot don t test what is populate by db
    ModulePositionMap::iterator itorPosition = dynamipsRouterEntry->modulesMap.begin();
    while (itorPosition != dynamipsRouterEntry->modulesMap.end()) {
        QString moduleName = QString::fromStdString(itorPosition->second.name);
        if (dynamipsChassis.numSlots <= itorPosition->first) {
            error = QObject::tr("Invalid slot \"%1\" of module \"%2\"").arg(itorPosition->first).arg(moduleName);
            return false;
        }
        if (itorPosition->first != 0 && itorPosition->second.isMB) {
            error = QObject::tr("Module \"%1\" in slot \"%2\" is a motherboard and has to be in slot 0").arg(moduleName).arg(itorPosition->first);
            return false;
        }
        if ( dataDynamipsRouter.availableModules.count(itorPosition->second.name) == 0 &&
             dynamipsChassis.availableModules.count(itorPosition->second.name) == 0) {
            error = QObject::tr("Unknown module \"%1\" for router \"%2\" and chassis \"%3\"").arg(moduleName).arg(routerName).arg(chassisName);
            return false;
        }
        itorPosition++;
    }
    return true;
}


void DynamipsRouterDBI::populate(::Hns::EntryPtr &entry) //add the slot if not here
{
    DynamipsRouterEntryPtr dynamipsRouterEntry = DynamipsRouterEntryPtr::dynamicCast(entry);
    QString nameRouter = QString::fromStdString(dynamipsRouterEntry->router);
    if ( _dataDynamipsRouterMap.contains(nameRouter) ) {
        DataDynamipsRouter dataDynamipsRouter = _dataDynamipsRouterMap[nameRouter];
        dynamipsRouterEntry->needSetChassis = dataDynamipsRouter.needSetChassis;
        dynamipsRouterEntry->nvram = dataDynamipsRouter.nvram;
        ModulePositionMap::iterator itorModulePos = dynamipsRouterEntry->modulesMap.begin();
        while ( itorModulePos != dynamipsRouterEntry->modulesMap.end()) {
            QString nameModule = QString::fromStdString(itorModulePos->second.name);
            if (_mapModule.contains(nameModule) ) {
                itorModulePos->second.interfaces = _mapModule[nameModule].interfaces;
                itorModulePos->second.isMB = _mapModule[nameModule].isMB;
            }
            itorModulePos++;
        }
        //add slot 0 if not here
        if (dynamipsRouterEntry->modulesMap.count(0) == 0) {
            //found a MB module in chassis list
            if ( dataDynamipsRouter.availableChassis.find(dynamipsRouterEntry->chassis) != dataDynamipsRouter.availableChassis.end()) {
                DynamipsRouterChassis dynamipsChassis = dataDynamipsRouter.availableChassis[dynamipsRouterEntry->chassis];
                ModuleMap::iterator itorModuleChassis = dynamipsChassis.availableModules.begin();
                while (itorModuleChassis != dynamipsChassis.availableModules.end()) {
                    if (itorModuleChassis->second.isMB) {
                        dynamipsRouterEntry->modulesMap[0] = itorModuleChassis->second;
                        return;
                    }
                    itorModuleChassis++;
                }
            }
            //if not found, search a MB module in module list
            ModuleMap::iterator itorModule = dataDynamipsRouter.availableModules.begin();
            while (itorModule != dataDynamipsRouter.availableModules.end()) {
                if (itorModule->second.isMB) {
                    dynamipsRouterEntry->modulesMap[0] = itorModule->second;
                    break;
                }
                itorModule++;
            }
        }
    }
}

void DynamipsRouterDBI::_fillData()
{
    QSettings settings(Diamesh::instance()->configFileName(), QSettings::IniFormat);
    settings.beginGroup(Constants::SETTINGS_HYNESIM_GROUP);
    QString dataPath = settings.value("dataDirectory", ".").toString();
    settings.endGroup();
    if (!dataPath.endsWith("/"))
        dataPath.append("/");

    QDir dataDir(dataPath);
    if(!dataDir.exists())
    {
        qCritical() << QObject::tr("The directory \"%1\" does not exist.").arg(dataDir.path());
        return;
    }

    QFile dataFile(dataDir.absoluteFilePath("hnsDynamipsDB.xml"));
    if(!dataFile.exists())
    {
        qCritical() << QObject::tr("The file \"%1\" does not exist.").arg(dataFile.fileName());
        return;
    }

    if (!dataFile.open(QIODevice::ReadOnly)) {
        qCritical() << QObject::tr("The file \"%1\" can not be read.").arg(dataFile.fileName());
        return;
    }
    QDomDocument doc;
    QString errorStr;
    int errorLine;
    int errorColumn;
    if (!doc.setContent(&dataFile,true, &errorStr, &errorLine, &errorColumn )) {
        dataFile.close();
        qCritical() << QObject::tr("The file \"%1\" is not a valid xml file.").arg(dataFile.fileName());
        qCritical() << QObject::tr("Error at line %1, column %2: %3").arg(errorLine).arg(errorColumn).arg(errorStr);
        return;
    }
    dataFile.close();

    QDomElement root = doc.documentElement();

    //existing modules
    QDomNodeList moduleRoot = root.elementsByTagName("modulesList");
    if (!moduleRoot.isEmpty()) {
        QDomNodeList listModuleNode = moduleRoot.item(0).toElement().elementsByTagName("module");
        int idxModule = 0;
        int sizeModules = listModuleNode.size();
        while (idxModule != sizeModules) {
            QDomElement moduleElement = listModuleNode.item(idxModule).toElement();
            DynamipsRouterModule dynamipsModule;
            dynamipsModule.name = moduleElement.attribute("name").toStdString();
            dynamipsModule.interfaces = moduleElement.attribute("interfaces").toInt();
            if (moduleElement.hasAttribute("isMB")) {
                dynamipsModule.isMB = (moduleElement.attribute("isMB").toInt() == 1);
            }
            _mapModule[moduleElement.attribute("name")] = dynamipsModule;
            idxModule++;
        }
    }

    //existing chassis
    QMap<QString, DynamipsRouterChassis> mapChassis;
    QDomNodeList chassisRoot = root.elementsByTagName("chassisList");
    if (!chassisRoot.isEmpty()) {
        QDomNodeList listChassisNode = chassisRoot.item(0).toElement().elementsByTagName("chassis");
        int idxChassis = 0;
        int sizeChassis = listChassisNode.size();
        while (idxChassis != sizeChassis) {
            QDomElement chassisElement = listChassisNode.item(idxChassis).toElement();
            DynamipsRouterChassis dynamipsChassis;
            QString nameChassis = chassisElement.attribute("name");
            dynamipsChassis.name = nameChassis.toStdString();
            dynamipsChassis.numSlots = chassisElement.attribute("slots").toInt();
            QDomNodeList availableModulesNode = chassisElement.elementsByTagName("availableModules");
            if (!availableModulesNode.isEmpty()) {
               QDomNodeList listModuleNode = availableModulesNode.item(0).toElement().elementsByTagName("module");
               int idxModule = 0;
               int sizeModules = listModuleNode.size();
               while (idxModule != sizeModules) {
                   QDomElement moduleElement = listModuleNode.item(idxModule).toElement();
                   QString moduleName = moduleElement.attribute("name");
                   if (!_mapModule.contains(moduleName)) {
                       qCritical() << QObject::tr("Module \"%1\" don't exist for \"%2\"").arg(moduleName).arg(nameChassis);
                   } else {
                       dynamipsChassis.availableModules[moduleName.toStdString()] = _mapModule[moduleName];
                   }
                   idxModule++;
               }
            }
            mapChassis[nameChassis] = dynamipsChassis;
            idxChassis++;
        }
    }

    //existing router
    QDomNodeList routerRoot = root.elementsByTagName("routersList");
    if (routerRoot.isEmpty()) {
        qCritical() << QObject::tr("No router registered in database");
    } else {
        QDomNodeList listRouterNode = routerRoot.item(0).toElement().elementsByTagName("router");
        int idxRouter = 0;
        int sizeRouters = listRouterNode.size();
        while (idxRouter != sizeRouters) {
            QDomElement routerElement = listRouterNode.item(idxRouter).toElement();
            DataDynamipsRouter dataDynamipsRouter;
            QString nameRouter = routerElement.attribute("name");
            dataDynamipsRouter.name = nameRouter.toStdString();
            dataDynamipsRouter.needSetChassis = (routerElement.attribute("needSetChassis").toInt() == 1);
            dataDynamipsRouter.nvram = routerElement.attribute("nvram").toInt();

            QDomNodeList availableChassisNode = routerElement.elementsByTagName("availableChassis");
            if (availableChassisNode.isEmpty()) {
                qCritical() << QObject::tr("No chassis available for %1").arg(nameRouter);
            } else {
                QDomNodeList listChassisNode = availableChassisNode.item(0).toElement().elementsByTagName("chassis");
                int idxChassis = 0;
                int sizeChassis = listChassisNode.size();
                while (idxChassis != sizeChassis) {
                    QDomElement chassisElement = listChassisNode.item(idxChassis).toElement();
                    QString nameChassis = chassisElement.attribute("name");
                    if (!mapChassis.contains(nameChassis)) {
                        qCritical() << QObject::tr("Chassis \"%1\" don't exist for \"%2\"").arg(nameChassis).arg(nameRouter);
                    } else {
                        dataDynamipsRouter.availableChassis[nameChassis.toStdString()] = mapChassis[nameChassis];
                    }
                    idxChassis++;
                }
            }
            QDomNodeList availableModuleNode = routerElement.elementsByTagName("availableModules");
            if (!availableModuleNode.isEmpty()) {
                QDomNodeList listModuleNode = availableModuleNode.item(0).toElement().elementsByTagName("module");
                int idxModule = 0;
                int sizeModule = listModuleNode.size();
                while (idxModule != sizeModule) {
                    QDomElement moduleElement = listModuleNode.item(idxModule).toElement();
                    QString nameModule = moduleElement.attribute("name");
                    if (!_mapModule.contains(nameModule)) {
                        qCritical() << QObject::tr("Module \"%1\" don't exist for \"%2\"").arg(nameModule).arg(nameRouter);
                    } else {
                        dataDynamipsRouter.availableModules[nameModule.toStdString()] =  _mapModule[nameModule];
                    }
                    idxModule++;
                }
            }
            _dataDynamipsRouterMap[nameRouter] = dataDynamipsRouter;
            idxRouter++;
        }
    }
}

::Hns::DataDynamipsRouterMap DynamipsRouterDBI::getDataDynamipsMap(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    ::Hns::DataDynamipsRouterMap dataDynamipsRouterMap;
    QMapIterator<QString, Hns::DataDynamipsRouter> itor(_dataDynamipsRouterMap);
    while(itor.hasNext()) {
        itor.next();
        dataDynamipsRouterMap[itor.key().toStdString()] = itor.value();
    }
    return dataDynamipsRouterMap;
}

::Hns::DataDynamipsRouterModuleMap DynamipsRouterDBI::getDynamipsRouterModuleMap(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    ::Hns::DataDynamipsRouterModuleMap map;
    QMapIterator<QString, Hns::DynamipsRouterModule> itor(_mapModule);
    while(itor.hasNext()) {
        itor.next();
        map[itor.key().toStdString()] = itor.value();
    }
    return map;
}

} // namespace Hns
