/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "entityI.h"
#include "masterI.h"
#include "catalog/entitycatalogI.h"
#include "catalog/topologycatalogI.h"
#include "capturejobI.h"
#include "tracerI.h"
#include "entrypropertiesmanagerI.h"
#include "entrypropertiessetter.h"
#include "nodeinfoI.h"
#include "jobmanagerI.h"

// diamesh headers
#include <diamesh/sessionManagerI.h>
#include <diamesh/filetransferI.h>

// Qt headers
#include <QDebug>
#include <QCoreApplication>
#include <QStringList>
#include <QFile>
#include <QAtomicInt>

namespace Hns {

Q_GLOBAL_STATIC_WITH_ARGS(QAtomicInt, _idCounter, (1))

EntityI::EntityI()
    : _entityEntry(NULL)
    , _entityProxy(NULL)
    , _nodeInfo(NULL)
    , _tracer(NULL)
    , _statusesLock(QReadWriteLock::Recursive)
    , _status(Hns::UNDEFINED)
    , _idxCptStatus(0)
    , _idxCptNodeStatus(0)
{
    _id = _idCounter()->fetchAndAddOrdered(1);
    _availableInterfacesForCapture = QList<Ice::Int>();
    _flags = 0;
    _isDestroyed = false;
    _entityEntryModel = 0;
}

EntityI::~EntityI()
{
    _tracer->destroy();
}

EntityEntryPtr EntityI::getEntry(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return _entityEntry;
}

Hns::TracerPrx EntityI::getTracer(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    if (_tracer)
        return _tracer->proxy();
    return NULL;
}

bool EntityI::updateEntry(const::Hns::EntityEntryPtr &entityEntry, const ::Ice::Current &current)
{
    EntryPropertiesManagerI *propertiesManager = MasterI::instance()->entryPropertiesManager();
    if (!propertiesManager) {
        qCritical() << tr("Could not get entry properties manager");
        return false;
    }

    EntityEntryPtr newEntityEntry = NULL;
    QSharedPointer<EntityEntryPropertiesSetter> propertiesSetter;
    if (propertiesManager->canValidateEntry(entityEntry)) {
        propertiesSetter = propertiesManager->propertiesSetter(entityEntry).dynamicCast<EntityEntryPropertiesSetter>();
        if (propertiesSetter.isNull()) {
            qCritical() << tr("Can't get propertiesSetter for %1").arg(entityEntry->uuid.c_str());
            return false;
        }
        newEntityEntry = Hns::EntityEntryPtr::dynamicCast(propertiesSetter->validateEntry(entityEntry));
        if (!newEntityEntry) {
            qCritical() << tr("Entry %1 is not valid. Unable to set it as entity entry").arg(entityEntry->uuid.c_str());
            return false;
        }
    } else {
        newEntityEntry = entityEntry;
    }

    if (isDefined(current)) {
        if (propertiesSetter && !propertiesSetter->validateEntryWhenDefined(_entityEntry, newEntityEntry)) {
            qCritical() << tr("Advanced properties were changed for defined entity %1, unable to update entity entry").arg(_id);
            return false;
        }

        if (!_entityProxy->updateEntry(newEntityEntry)) {
            qCritical() << tr("Failed to update entry on node entity");
            return false;
        }
    }

    if (newEntityEntry->id) {
        EntityCatalogI *catalog = MasterI::instance()->entityCatalog();
        if (!catalog) {
            qCritical() << tr("Unable to get the entity catalog");
            return false;
        }

        if (!catalog->updateEntry(newEntityEntry)) {
            qCritical() << tr("Unable to update the entity entry");
            return false;
        }
    }


    setEntry(newEntityEntry);

    updatePlugs();

    return true;
}

bool EntityI::_beforeDefine(const HnsNode::EntityPrx &entityProxy)
{
    Q_UNUSED(entityProxy)
    return true;
}

bool EntityI::_afterDefine()
{
    return true;
}

bool EntityI::define(const std::string &nodeId, const ::Ice::Current &current)
{
    return _define(nodeId, current);
}

void EntityI::define_effective(const Hns::AMD_Entity_definePtr &amdPtr, const std::string &nodeId, const Ice::Current &current)
{
    if (define(nodeId, current))
        amdPtr->ice_response(true);
    else
        amdPtr->ice_response(false);
}

bool EntityI::start(const ::Ice::Current &current)
{
    return _start(current);
}

bool EntityI::stop(const ::Ice::Current &current)
{
    return _stop(current);
}

bool EntityI::save(const ::Ice::Current &current)
{
    QWriteLocker locker(&_entityLock);

     _checkDestroyed();

    if (!_isExecutable(current)) {
        qCritical() << tr("Unable to save entity");
        return false;
    }

    if (!_entityProxy->save()) {
        qCritical() << tr("Unable to save entry %1").arg(_entityEntry->uuid.c_str());
        return false;
    }
    _entityEntry->saved = true;
    updateEntry(_entityEntry, current);

    _tracer->trace(tr("Entity saved"), current);

    return true;
}

bool EntityI::restore(const ::Ice::Current &current)
{
    QWriteLocker locker(&_entityLock);

    _checkDestroyed();

    if(!_isExecutable(current)) {
        qCritical() << tr("Unable to restore entity");
        return false;
    }

    if (!_entityProxy->restore()) {
        qCritical() << tr("Unable to restore entry %1").arg(_entityEntry->uuid.c_str());
        return false;
    }
    _entityEntry->saved = false;
    updateEntry(_entityEntry, current);

    _tracer->trace(tr("Entity restored"), current);

    return true;
}

bool EntityI::suspend(const ::Ice::Current &current)
{
    QWriteLocker locker(&_entityLock);

    _checkDestroyed();

    if(!_isExecutable(current)) {
        qCritical() << tr("Unable to restore entity");
        return false;
    }

    if (!_entityProxy->suspend()) {
        qCritical() << tr("Unable to suspend entry %1").arg(_entityEntry->uuid.c_str());
        return false;
    }

    _tracer->trace(tr("Entity suspended"), current);

    return true;
}

bool EntityI::resume(const ::Ice::Current &current)
{
    QWriteLocker locker(&_entityLock);

    _checkDestroyed();

    if(!_isExecutable(current)) {
        qCritical() << tr("Unable to resume entity");
        return false;
    }

    if (!_entityProxy->resume()) {
        qCritical() << tr("Unable to resume entry %1").arg(_entityEntry->uuid.c_str());
        return false;
    }

    _tracer->trace(tr("Entity resumed"), current);

    return true;
}

bool EntityI::createSnapshot(const Snapshot &snapshot, const Ice::Current &current)
{
    Q_UNUSED(current)
    return _createSnapshot(snapshot);
}

bool EntityI::revertSnapshot(const std::string &snapshotName, const Ice::Current &current)
{
    Q_UNUSED(current)
    return _revertSnapshot(snapshotName);
}

bool EntityI::deleteSnapshot(const std::string &snapshotName, const Ice::Current &current)
{
    Q_UNUSED(current)
    return _deleteSnapshot(snapshotName);
}

Snapshot EntityI::getSnapshot(const std::string &snapshotName, const Ice::Current &current)
{
    Q_UNUSED(current)
    return _getSnapshot(snapshotName);
}

std::string EntityI::getCurrentSnapshotName(const Ice::Current &current)
{
    Q_UNUSED(current)
    return _getCurrentSnapshotName();
}

bool EntityI::isDefinable(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return true;
}

bool EntityI::isDefined(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return (_entityProxy != 0);
}

bool EntityI::_beforeUndefine()
{
    return true;
}

void EntityI::_afterUndefine()
{
    QWriteLocker locker(&_entityLock);
    _idxCptNodeStatus = 0;
    _propagateStatus(Hns::UNDEFINED);
    _tracer->trace(tr("Entity undefined"));
}

bool EntityI::_define(const std::string &nodeId, const Ice::Current &current)
{
    QWriteLocker locker(&_entityLock);

    _checkDestroyed();

    if (isDefined(current)) {
        qDebug() << tr("Entity is already defined.");
        return false;
    }

    if (!_isExecutable(current)) {
        qCritical() << tr("Unable to define entity");
        return false;
    }

    _propagateStatus(Hns::BUSY);

    QSharedPointer<NodeInfoI> hnsNodeInfo;
    NodePrx nodePrx = NULL;
    if (nodeId == "") {
        hnsNodeInfo = MasterI::instance()->getSuitableNode(_entityEntry);
        if (!hnsNodeInfo) {
            qCritical() << tr("Unable to find a suitable node to define the entity");
            _propagateStatus(Hns::UNDEFINED);
            return false;
        }
    } else {
        QSharedPointer<Diamesh::NodeInfoI> diameshNodeInfo = MasterI::instance()->nodeInfo(nodeId.c_str());
        if (!diameshNodeInfo) {
            qCritical() << tr("No node info was found for node %1. Unable to define the entity").arg(nodeId.c_str());
            _propagateStatus(Hns::UNDEFINED);
            return false;
        }

        hnsNodeInfo = qSharedPointerDynamicCast<NodeInfoI>(diameshNodeInfo);
        if (!hnsNodeInfo) {
            qCritical() << tr("The node info is not a hynesim node info. Unable to define the entity");
            _propagateStatus(Hns::UNDEFINED);
            return false;
        }

        if (!hnsNodeInfo->canBuild(_entityEntry, current)) {
            qCritical() << tr("The node %1 can not build entity %2").
                             arg(hnsNodeInfo->identity().name.c_str()).
                             arg(_entityEntry->uuid.c_str());
            _propagateStatus(Hns::UNDEFINED);
            return false;
        }

    }
    nodePrx = NodePrx::uncheckedCast(hnsNodeInfo->node());

    if (!nodePrx) {
        qCritical() << tr("Unable to get the hynesim node proxy");
        _propagateStatus(Hns::UNDEFINED);
        return false;
    }


    HnsNode::EntityFactoryPrx factory = nodePrx->getEntityFactory();
    if (!factory) {
        qCritical() << tr("Unable to get the node factory");
        _propagateStatus(Hns::UNDEFINED);
        return false;
    }

    HnsNode::EntityPrx entityProxy = factory->createEntity(_entityEntry, _id);

    if (!entityProxy) {
        qCritical() << tr("The factory could not create the entity on the node");
        _propagateStatus(Hns::UNDEFINED);
        return false;
    }

    _entityProxyQtice = HnsNode::EntityQPrx(entityProxy, MasterI::instance()->adapter());

    QString action = "_beforeDefine";
    try {
        if (!_beforeDefine(entityProxy)) {
            factory->destroyEntity(entityProxy);
            entityProxy = NULL;
            _entityProxyQtice = NULL;
            _propagateStatus(Hns::UNDEFINED);
            return false;
        }

        action = "define";
        if (!entityProxy->define()) {
            qCritical() << tr("The factory could not define the entity");
            factory->destroyEntity(entityProxy);
            entityProxy = NULL;
            _entityProxyQtice = NULL;
            _propagateStatus(Hns::UNDEFINED);
            return false;
        }
    } catch(...) {
        qCritical() << tr("The node has raised an exception during the action %1").arg(action);
        entityProxy = NULL;
        _entityProxyQtice = NULL;
        _propagateStatus(Hns::UNDEFINED);
        return false;
    }

    _afterDefine();

    QObject::connect(_entityProxyQtice.data(),SIGNAL(capturableInterfacesListChanged(::Diamesh::IntList)),this,SLOT(_updateCapturableInterfacesList(::Diamesh::IntList)));
    _availableInterfacesForCapture = QList<Ice::Int>::fromStdList(_entityProxyQtice->getAvailablePlugsForCapture());
    emit capturableInterfacesListChanged(getAvailablePlugsForCapture());

    _tracer->trace(tr("Entity defined on node %1").arg(QString::fromStdString(hnsNodeInfo->getName())), current);
    _entityProxy = entityProxy;
    _nodeInfo = QSharedPointer<NodeInfoI>(hnsNodeInfo);
    QObject::connect(_nodeInfo.data(), SIGNAL(nodeRemoved()), this, SLOT(_nodeRemovedSlot()));
    QObject::connect(
                _entityProxyQtice.data(), SIGNAL(statusChanged(Ice::Int, Ice::Long)),
                this, SLOT(_statusChangedSlot(Ice::Int, Ice::Long)));
    _propagateStatus(_entityProxy->getStatus());

    qDebug() << defineDoneMessage();
    return true;
}

bool EntityI::_start(const Ice::Current &current)
{
    QWriteLocker locker(&_entityLock);

    _checkDestroyed();

    if (!isDefined(current)) {
        qDebug() << tr("Entity is not defined. Unable to start it.");
        return false;
    }

    if (!_isExecutable(current)) {
        qCritical() << tr("Unable to start entity");
        return false;
    }

    if (!_entityProxy->start()) {
        qCritical() << tr("Unable to start entity %1").arg(_entityEntry->uuid.c_str());
        return false;
    }

    _tracer->trace(tr("Entity started"), current);

    return true;
}

bool EntityI::_stop(const Ice::Current &current)
{
    QWriteLocker locker(&_entityLock);

     _checkDestroyed();

    if(!_isExecutable(current)) {
        qCritical() << tr("Unable to stop entity");
        return false;
    }

    if (!_entityProxy->stop()) {
        qCritical() << tr("Unable to stop entity %1").arg(_entityEntry->uuid.c_str());
        return false;
    }

    _tracer->trace(tr("Entity stopped"), current);

    return true;
}

bool EntityI::_undefine(const Ice::Current &current)
{
    Q_UNUSED(current)
    QWriteLocker locker(&_entityLock);
    QReadLocker captureLocker(&_captureLock);

   _checkDestroyed();

    if (!isDefined(current))
        return true;

    _propagateStatus(Hns::BUSY);

    // Cancel all capture jobs
    foreach (Ice::Int captureJobId, _captureJobHash)
        cancelCaptureJob(captureJobId, current);

    if (!_beforeUndefine())
        return false;


    if (_entityProxy) {
        HnsNode::EntityPrx prx = _entityProxy;
        // We set _entityProxy to 0 here in case isDefined() is called before we are finished here
        _entityProxy = NULL;
        _entityProxyQtice = NULL;
        if (!_nodeInfo.isNull() && !_nodeInfo->hasNodeBeenRemoved()) {
            EntityUndefineCallback *callback = new EntityUndefineCallback(this, SLOT(_afterUndefine()), _nodeInfo, prx);
            callback->moveToThread(qApp->thread());
            callback->start();
            _nodeInfo.clear();
            return true;
        }
    }
    locker.unlock();
    _afterUndefine();
    return true;
}

bool EntityI::undefine(const ::Ice::Current &current)
{
    return _undefine(current);
}

bool EntityI::destroy(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    _checkDestroyed();

    if (!undefine(current)) {
        qCritical() << tr("The entry %1 could not be undefined.").arg(_entityEntry->id);
        return false;
    }

    // Make sure that the QWriteLocker is destroyed before this entity
    {
        QWriteLocker locker(&_entityLock);

        _checkDestroyed();

        if (_entityEntry->id)
        {
            EntityCatalogI *catalog = MasterI::instance()->entityCatalog();
            if (!catalog->unlockEntry(_entityEntry->id)) {
                qCritical() << tr("The entry %1 could not be unlocked.").arg(_entityEntry->id);
                return false;
            }
        }

        _tracer->trace(tr("Entity destroyed"), current);

        _isDestroyed = true;

        emit requestDeletion(MasterI::instance()->adapter(), _identity);
    }

    return true;
}

void EntityI::_checkDestroyed()
{
    if (_isDestroyed) {
        qCritical() << tr("The entity has already been destroyed");
        throw Ice::FacetNotExistException(__FILE__, __LINE__);
    }
}

bool EntityI::setProxy(HnsNode::EntityPrx entityProxy)
{
    _entityProxy = entityProxy;

    _entityProxyQtice = HnsNode::EntityQPrx(_entityProxy, MasterI::instance()->adapter());

    QObject::connect(
                _entityProxyQtice.data(), SIGNAL(statusChanged(Ice::Int, Ice::Long)),
                this, SLOT(_statusChangedSlot(Ice::Int, Ice::Long)));
    _propagateStatus(_entityProxy->getStatus());

    return true;
}

Ice::Long EntityI::getId(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return _id;
}

::Hns::PlugMap EntityI::getPlugs(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QReadLocker locker(&_plugLock);
    return _plugsMap.toStdMap();
}

::Hns::PlugMap EntityI::getAvailablePlugs(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QReadLocker locker(&_plugLock);
    QMap<Ice::Long, Hns::Plug> plugMap;
    for (QMap<Ice::Long, Hns::Plug>::iterator it = _plugsMap.begin(); it != _plugsMap.end(); ++it) {
        Hns::Plug plug = it.value();

        if (!plug.isPlugged)
            plugMap.insert(it.key(), plug);
    }
    return plugMap.toStdMap();
}

std::string EntityI::getPlugName(::Ice::Long id, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    if (!_plugsMap.contains(id)) {
        qDebug() << tr("Invalid plug port: %1").arg(id);
        return std::string();
    } else {
        return _plugsMap.value(id).name;
    }
}

::Hns::StatusList EntityI::getAllowedStatuses(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QReadLocker locker(&_statusesLock);
    return _statusesList.toStdList();
}

::Hns::PcapInfoList EntityI::getPcapInfoList(const ::Ice::Current &current)
{    
    Q_UNUSED(current)
    QReadLocker locker(&_captureLock);
    QList<PcapInfo> list;
    list = _pcapInfosHash.values();
    return list.toStdList();
}

void EntityI::setPcapInfoList(const ::Hns::PcapInfoList &pcapInfoList, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QWriteLocker locker(&_captureLock);
    foreach (const PcapInfo &info, pcapInfoList)
        _pcapInfosHash.insertMulti(-1, info);
}

void EntityI::setEntry(const ::Hns::EntityEntryPtr &entityEntry)
{
    QString path;
    if (entityEntry->path != "")
        path = QString::fromStdString(MasterI::instance()->entityCatalog()->getFullEntryPathFromMixed(entityEntry->path, entityEntry->uuid));

    if (!path.isEmpty()) {
        if (_tracer) {
            if (_tracer->directory().isEmpty())
                _tracer->setDirectory(path);
        } else {
            _tracer = TracerI::get(path);
        }

    } else if (!_tracer) {
        _tracer = new TracerI();
    }

    if (entityEntry->isModel) {
        _status = Hns::UNIMPLEMENTED;
    } else if (_entityEntry && _entityEntry->isModel && !entityEntry->isModel) {
        _status = Hns::UNDEFINED;
        _entityEntryModel = _entityEntry;
        emit statusChanged(_status, ++ _idxCptStatus);
        emit implementFromModelChanged(true);
    }

    _entityEntry = entityEntry;
    _initSnapshotFlags();
    _initStatusesList();
    emit entryChanged();
}


bool EntityI::unImplement()
{
    QWriteLocker locker(&_entityLock);
    if(_status != Hns::UNDEFINED)
        return false;
    setEntry(_entityEntryModel);
    emit statusChanged(_status, ++_idxCptStatus);
    _entityEntryModel = NULL;
    emit implementFromModelChanged(false);
    return true;
}


bool EntityI::plug(int port)
{
    QWriteLocker locker(&_plugLock);
    if (!_plugsMap.contains(port)) {
        qWarning() << tr("Invalid plug port: %1").arg(port);
        return false;
    }

    if (_plugsMap.value(port).isPlugged) {
        qWarning() << tr ("Plug %1 is already plugged").arg(port);
        return false;
    }

    _plugsMap[port].isPlugged = true;
    emit plugUsed(port);
    return true;
}

void EntityI::unplug(int port)
{
    QWriteLocker locker(&_plugLock);
    if (!_plugsMap.contains(port)) {
        //qWarning() << tr("Invalid plug port: %1").arg(port);
        return;
    }

    if (!_plugsMap.value(port).isPlugged) {
        qWarning() << tr ("Plug %1 is not plugged").arg(port);
        return;
    }

    _plugsMap[port].isPlugged = false;
    emit plugReleased(port);
}

QMap<Ice::Long, Hns::Plug> EntityI::_getPlugs()
{
    return QMap<Ice::Long, Hns::Plug>();
}

void EntityI::initPlugs()
{
    QWriteLocker locker(&_plugLock);
    _plugsMap.clear();
    _plugsMap = _getPlugs();
}

void EntityI::updatePlugs()
{
    QWriteLocker locker(&_plugLock);
    bool modified = false;
    QMap<Ice::Long, Hns::Plug> plugMapAfter = _getPlugs();

    QSet<Ice::Long> plugsBefore = _plugsMap.keys().toSet();
    QSet<Ice::Long> plugsAfter = plugMapAfter.keys().toSet();

//    qDebug() << "plugsBefore" << plugsBefore;
//    qDebug() << "plugsAfter" << plugsAfter;

    QSet<Ice::Long> plugsToRemove = plugsBefore - plugsAfter;
    QSet<Ice::Long> plugsToAdd = plugsAfter - plugsBefore;
    QSet<Ice::Long> intersectPlugs = (plugsBefore | plugsAfter) - plugsToRemove - plugsToAdd;

//    qDebug() << "plugsToRemove" << plugsToRemove;
//    qDebug() << "plugsToAdd" << plugsToAdd;
//    qDebug() << "intersectPlugs" << intersectPlugs;

    if (plugsBefore != plugsAfter) {
        if (plugsToRemove.size() > 0) {
            foreach (Ice::Long id, plugsToRemove) {
                _plugsMap.remove(id);
            }
            emit plugsRemoved();
        }
        if (plugsToAdd.size() > 0) {
            foreach (Ice::Long id, plugsToAdd) {
                Hns::Plug plug = plugMapAfter.value(id);
                _plugsMap.insert(id, plug);
            }
        }
        modified = true;
    }
    if (intersectPlugs.size() != 0){
        if (_plugsMap.values() != plugMapAfter.values()){
            foreach (Ice::Long id,intersectPlugs){
                Hns::Plug plug = plugMapAfter.value(id);
                plug.isPlugged = _plugsMap.value(id).isPlugged;
                _plugsMap.insert(id, plug);
            }
        }
        modified = true;
    }

    if (modified){
        locker.unlock();
        emit plugListChanged(_plugsMap.toStdMap());
    }
}

QHash<Ice::Int, Ice::Int>  EntityI::changePlugsForImplementation()
{
    QHash<Ice::Int, Ice::Int> listPlug2Change;
    QWriteLocker locker(&_plugLock);
    QMap<Ice::Long, Hns::Plug> plugMapAfter = _getPlugs();
    QMap<Ice::Long, Hns::Plug> plugMapBefore = _plugsMap;

    //if plugsBefore != plugsAfter there is a problem
    if (plugMapAfter.size() != plugMapBefore.size()) {
        locker.unlock();
        updatePlugs();
        return listPlug2Change;
    }
    _plugsMap.clear();

    QMap<Ice::Long, Hns::Plug>::iterator itorAfter = plugMapAfter.begin();
    QMap<Ice::Long, Hns::Plug>::iterator itorBefore = plugMapBefore.begin();
    while (itorAfter != plugMapAfter.end()) {
        if (itorAfter.key() != itorBefore.key()) {
            listPlug2Change.insert(itorBefore.key(), itorAfter.key());
        }
        _plugsMap.insert(itorAfter.key(), itorAfter.value());
        _plugsMap[itorAfter.key()].isPlugged = itorBefore.value().isPlugged;
        itorAfter++;
        itorBefore++;
    }
    if (!listPlug2Change.isEmpty()){
        locker.unlock();
        emit plugListChanged(_plugsMap.toStdMap());
    }
    return listPlug2Change;
}


HnsNode::EntityPrx EntityI::_getProxy(EntityI *entity)
{
    return entity->_entityProxy;
}

bool EntityI::_isExecutable(const ::Ice::Current &current)
{
    Diamesh::SessionManagerI *sessionManager = Diamesh::MasterI::instance()->sessionManager();
    if(!sessionManager) {
        qCritical() << tr("Unable to get the session manager");
        return false;
    }

    if (sessionManager->isClientCall(current)) {
        Diamesh::UserPtr user = sessionManager->currentUser(current);
        if(!user._ptr) {
            qCritical() << tr("Unable to get the user information");
            return false;
        }

        // TODO:TestRight
//        if (!user->isAdmin) {
//            qCritical() << "define:" << tr("The current user does not have execution rights on entry %1").arg(_entityEntry->uuid.c_str());
//            return false;
//        }
    }
    return true;
}

Hns::Status EntityI::getStatus(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return _status;
}

bool EntityI::isCaptureValid(CaptureJobOptions jobOptions)
{
    /* If there is one job running on the same interface*/
    return _availableInterfacesForCapture.contains(jobOptions.interfaceList.front());
}

Ice::Int EntityI::createCaptureJob(const ::Hns::CaptureJobOptions &jobOptions, const Ice::Current &current)
{
    Q_UNUSED(current)
    QWriteLocker locker(&_captureLock);
    if (!isDefined())
        return -1;
    Hns::CaptureJobI *captureJob = new CaptureJobI();
    HnsNode::CaptureJobPrx nodeCapJobPrx = _entityProxy->createCaptureJob(_topologyPath.toStdString());
    if (!nodeCapJobPrx) {
        qCritical() << tr("Unable to create the job proxy");
        return -1;
    }

    captureJob->setCaptureJobPrx(nodeCapJobPrx);

    JobManagerI *jobManager = JobManagerI::instance();
    Ice::Int jobId = jobManager->addJob(captureJob);
    QObject::connect(captureJob, SIGNAL(progressChanged(Ice::Int, Ice::Double)),this, SLOT(_captureProgressChangedSlot(Ice::Int, Ice::Double)));

    if (!isCaptureValid(jobOptions)) {
        qCritical()<<tr("Selected interface already in use");
        return -1;
    }

    if (!jobId)
        return -1;

    /* Add jobId to JobManager */
    Hns::CaptureJobPrx captureJobPrx = Hns::CaptureJobPrx::uncheckedCast(jobManager->getJob(jobId));

    if (!captureJobPrx->setOptions(jobOptions))
        return -1;

    if (!startCaptureJob(jobId))
        return -1;

    PcapInfo pcapinfo = nodeCapJobPrx->getPcapInfo();
    _pcapInfosHash.insert(jobId, pcapinfo);
    _captureJobHash.insert(jobOptions.interfaceList.front(),jobId);
    _jobIdPathHash.insert(jobId,pcapinfo.path);
    // Notice the gui
    emit interfaceUsed(jobOptions.interfaceList.front(), jobId);

    return jobId;
}

bool EntityI::startCaptureJob(Ice::Int jobId)
{
    JobManagerI *jobManager = JobManagerI::instance();
    Hns::CaptureJobPrx captureJobPrx = Hns::CaptureJobPrx::uncheckedCast(jobManager->getJob(jobId));

    if (!captureJobPrx){
        qDebug()<<tr("Error : couldn't get job with id \"%1\" from job manager.").arg(jobId);
        return false;
    }
    captureJobPrx->execute();
    emit captureJobStarted();
    return true;
}

bool EntityI::cancelCaptureJob(Ice::Int jobId,const Ice::Current &current)
{
    Q_UNUSED(current)
    JobManagerI *jobManager = JobManagerI::instance();
    Hns::CaptureJobPrx captureJobPrx = Hns::CaptureJobPrx::uncheckedCast(jobManager->getJob(jobId));

    if (!captureJobPrx) {
        qDebug()<<tr("Error : couldn't get job with id \"%1\" from job manager.").arg(jobId);
        return false;
    }

    return captureJobPrx->cancel();
}

bool EntityI::isCapturing()
{
    QReadLocker locker(&_captureLock);
    qDebug() << QString(tr("There are currently %1 capture jobs running")).arg(_captureJobHash.size());

    return (_captureJobHash.size() > 0);
}

::Diamesh::StringList EntityI::getCaptureFiles(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QReadLocker locker(&_captureLock);
    QList<std::string> captureFilesList;
    foreach (const PcapInfo &pcapInfo, _pcapInfosHash) {
        QString qstringPcapInfoPath = QString::fromStdString(pcapInfo.path);
        captureFilesList.append(qstringPcapInfoPath.split("/").last().toStdString());
    }
    ::Diamesh::StringList listToReturn = captureFilesList.toStdList();
    return listToReturn;
}

::Diamesh::FileTransferPrx EntityI::downloadCaptureFile(const std::string &file, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QReadLocker locker(&_captureLock);
    QString fileQString = QString::fromStdString(file);
    QString fileToDownload;
    foreach (const PcapInfo &pcapInfo, _pcapInfosHash) {
        if (QString::fromStdString(pcapInfo.path).endsWith(fileQString)) {
            fileToDownload = QString::fromStdString(pcapInfo.path);
            break;
        }
    }
    QString pathFileToDownload = QString::fromStdString(MasterI::instance()->topologyCatalog()->getFullEntryPathFromMixed(fileToDownload.toStdString()));

    if (QFile::exists(pathFileToDownload)) {
        ::Diamesh::FileTransferI *fileTransferI= new ::Diamesh::FileTransferI(pathFileToDownload,MasterI::instance()->adapter());
        return fileTransferI->proxy();
    } else {
        emit checkPcapFiles();
        return NULL;
    }
}

void EntityI::eraseCaptureFile (const std::string &file,const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QWriteLocker locker(&_captureLock);
    QString fileQString = QString::fromStdString(file);
    QString fileToErase;

    QMutableHashIterator<Ice::Int, PcapInfo> iterator(_pcapInfosHash);
    while (iterator.hasNext()) {
        iterator.next();
        if (QString::fromStdString(iterator.value().path).endsWith(fileQString)){
            fileToErase = QString::fromStdString(iterator.value().path);
            iterator.remove();
            break;
        }
    }
    QString pathFileToErase = QString::fromStdString(MasterI::instance()->topologyCatalog()->getFullEntryPathFromMixed(fileToErase.toStdString()));

    if(QFile::exists(pathFileToErase)) {
        QFile::remove(pathFileToErase);
    }
    emit captureJobFileDelete(fileQString);
}

std::string EntityI::getNodeName(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    if (isDefined())
        return _nodeInfo->getName();
    else
        return "";
}

::Diamesh::IntList EntityI::getAvailablePlugsForCapture(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return _availableInterfacesForCapture.toStdList();
}

::Ice::Int EntityI::getFlags(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return _flags;
}

void EntityI::setTopologyPath(QString topologyPath)
{    
    QWriteLocker locker(&_captureLock);
    foreach (PcapInfo pcapInfo, _pcapInfosHash) {
        int id = _pcapInfosHash.key(pcapInfo);
        QString pcapInfoPath = QString::fromStdString(pcapInfo.path);
        pcapInfo.path = pcapInfoPath.replace(_topologyPath, topologyPath).toStdString();
        _pcapInfosHash.insert(id, pcapInfo);
    }
    _topologyPath = topologyPath;
}

::Hns::mapLongString EntityI::getCaptureJobIdPath(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QReadLocker locker(&_captureLock);
    ::Hns::mapLongString mapToReturn;
    foreach (int i,_jobIdPathHash.keys()){
        mapToReturn[i] = _jobIdPathHash.value(i);
    }
    return mapToReturn;
}

::Hns::PcapInfo EntityI::getPcapInfoByJobId(::Ice::Int jobId, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QReadLocker locker(&_captureLock);
    ::Hns::PcapInfo dummyEmptyPcap;

    dummyEmptyPcap.endDateTime = -1;
    dummyEmptyPcap.filters = "";
    dummyEmptyPcap.interfaces = -1;
    dummyEmptyPcap.packetsLimit = -1;
    dummyEmptyPcap.packetsNumber = -1;
    dummyEmptyPcap.path = "";
    dummyEmptyPcap.size = -1;
    dummyEmptyPcap.startDateTime = -1;
    dummyEmptyPcap.timeLimit = -1;

    if ( jobId == -1 || !_pcapInfosHash.contains(jobId))
        return dummyEmptyPcap;
    return _pcapInfosHash.value(jobId);
}

void EntityI::setIdentity(Ice::Identity identity)
{
    _identity = identity;
}

Ice::Identity EntityI::identity()
{
    return _identity;
}

void EntityI::_captureProgressChangedSlot(Ice::Int jobId, Ice::Double progress)
{
    if (progress == 1) {
        QWriteLocker locker(&_captureLock);
        JobManagerI *jobManager = JobManagerI::instance();
        Hns::CaptureJobI *job = qobject_cast<Hns::CaptureJobI *>(jobManager->job(jobId));
        if (job == 0) {
            qCritical() << tr("No capture job with id %1 found !").arg(jobId);
            return;
        }
        PcapInfo info = job->getPcapInfo();
        _pcapInfosHash.insert(jobId, info);
        int interface = _captureJobHash.key(jobId);
        _captureJobHash.remove(interface);
        _jobIdPathHash.remove(jobId);
        // Notice the gui
        emit interfaceReleased(interface, jobId);
    }
}

void EntityI::_updateCapturableInterfacesList(::Diamesh::IntList interfaceList)
{
    _availableInterfacesForCapture = QList<Ice::Int>::fromStdList(interfaceList);
    emit capturableInterfacesListChanged(interfaceList);
}

void EntityI::_statusChangedSlot(Ice::Int status, Ice::Long idx)
{
    if(idx > _idxCptNodeStatus){
        _idxCptNodeStatus = idx;
        _propagateStatus(Hns::Status(status));
    }
}

void EntityI::_nodeRemovedSlot()
{
    QWriteLocker captureLocker(&_captureLock);
    foreach (Ice::Int captureJobId, _captureJobHash) {
        _captureJobTerminated(captureJobId);
    }
    _nodeInfo.clear();
    captureLocker.unlock();
    undefine();
}

void EntityI::_propagateStatus(Hns::Status status)
{
    if (status != _status) {
        _status = status;
        emit statusChanged(_status, ++_idxCptStatus);
    }
}

bool EntityI::_createSnapshot(const Snapshot &snapshot)
{
    Q_UNUSED(snapshot)
    return true;
}

bool EntityI::_revertSnapshot(const std::string &snapshotName)
{
    Q_UNUSED(snapshotName)
    return true;
}

bool EntityI::_deleteSnapshot(const std::string &snapshotName)
{
    Q_UNUSED(snapshotName)
    return true;
}

Snapshot EntityI::_getSnapshot(const std::string &snapshotName)
{
    Q_UNUSED(snapshotName)
    return Hns::Snapshot();
}

std::string EntityI::_getCurrentSnapshotName()
{
    return "";
}

void EntityI::_initSnapshotFlags()
{
    //Used only by domainI for the moment
}

void EntityI::_initStatusesList()
{
    QWriteLocker locker(&_statusesLock);
    _statusesList.clear();
    if (_entityEntry->isModel) {
        _statusesList.append(Hns::UNIMPLEMENTED);
    } else {
        _statusesList.append(Hns::STARTED);
        _statusesList.append(Hns::STOPPED);
        _statusesList.append(Hns::UNDEFINED);
        _statusesList.append(Hns::BUSY);
        if (_entityEntryModel != 0)
            _statusesList.append(Hns::UNIMPLEMENTED);
    }
    emit allowedStatusListChanged();
}

void EntityI::_captureJobTerminated(Ice::Long id)
{
    JobManagerI *jobManager = JobManagerI::instance();
    Hns::CaptureJobI *job = qobject_cast<Hns::CaptureJobI *>(jobManager->job(id));
    if (job == 0) {
        qCritical() << tr("No capture job with id %1 found !").arg(id);
        return;
    }
    QMutableHashIterator<Ice::Int, PcapInfo> it(_pcapInfosHash);
    while (it.hasNext()) {
        it.next();
        if (it.key() == id)
            it.remove();
    }
    int interface = _captureJobHash.key(id);
    _captureJobHash.remove(interface);
    _jobIdPathHash.remove(id);
    // Notice the gui
    emit interfaceReleased(interface, id);
}

QString EntityI::defineDoneMessage()
{
    return tr("Define done for entity %1").arg(_entityEntry->uuid.c_str());
}

RemoteDisplayInfo EntityI::getRemoteDisplayInfo(const ::Ice::Current &current)
{
    if (!isDefined(current)) {
        qWarning() << tr("The domain isn't defined, cannot get RemoteDisplayInfo");
        return RemoteDisplayInfo();
    }

    if (!_isExecutable(current)) {
        qWarning() << tr("Insufficient permissions, cannot get RemoteDisplayInfo");
        return RemoteDisplayInfo();
    }

    return EntityPrx::uncheckedCast(_entityProxy)->getRemoteDisplayInfo();
}

bool EntityI::isImplementFromModel(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return (_entityEntryModel != 0);
}

void EntityI::checkCaptureFile(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QWriteLocker locker(&_captureLock);
    QMutableHashIterator<Ice::Int, PcapInfo> iterator(_pcapInfosHash);
    while (iterator.hasNext()) {
        iterator.next();
        QString pathFileToCheck = QString::fromStdString(MasterI::instance()->topologyCatalog()->getFullEntryPathFromMixed(iterator.value().path));
        if(!QFile::exists(pathFileToCheck)) {
            iterator.remove();
            emit captureJobFileDelete(pathFileToCheck.split("/").last()) ;
        }
    }
}

IntIntMap EntityI::getRunningCaptureJobs(const Ice::Current &current)
{
    Q_UNUSED(current)
    QReadLocker locker(&_captureLock);
    IntIntMap ret;
    QHashIterator<Ice::Int, Ice::Int> it(_captureJobHash);
    while (it.hasNext()) {
        it.next();
        ret.insert(std::pair<Ice::Int, Ice::Int>(it.key(), it.value()));
    }
    return ret;
}

QString EntityI::statusToString(Status status)
{
    switch(status) {
    case UNDEFINED:
        return tr("Undefined");
    case STOPPED:
        return tr("Stopped");
    case STARTED:
        return tr("Started");
    case BUSY:
        return tr("Busy");
    case PAUSED:
        return tr("Paused");
    case SAVED:
        return tr("Saved");
    case UNIMPLEMENTED:
        return tr("Unimplemented");
    default:
        return tr("%1").arg(status);
    }
}

EntityUndefineCallback::EntityUndefineCallback(QObject *receiver, const char *slot, QSharedPointer<NodeInfoI> nodeInfo, HnsNode::EntityPrx prx, QObject *parent)
    : QObject(parent)
    , _nodeInfo(nodeInfo)
    , _prx(prx)
{
    _qprx = HnsNode::EntityQPrx(prx, MasterI::instance()->adapter());
    connect(this, SIGNAL(completed()), receiver, slot);
}

EntityUndefineCallback::~EntityUndefineCallback()
{
    _qprx = 0;
}

void EntityUndefineCallback::start()
{
    _qprx->begin_undefine(this, SLOT(_entityUndefineSlot(bool)), this, SLOT(_entityUndefinedFailedSlot(QSharedPointer<Ice::Exception>)));
}

void EntityUndefineCallback::_entityUndefineSlot(bool result)
{
    if (result) {
        NodePrx nodePrx = NodePrx::uncheckedCast(_nodeInfo->node());
        nodePrx->getEntityFactory()->destroyEntity(_prx);
        _nodeInfo.data()->QObject::disconnect(this);
        _nodeInfo.clear();
        emit completed();
    }
    else { //Should never append
        qCritical() << QObject::tr("Unable to undefine entity %1").arg(_qprx->getId());
    }
    deleteLater();
}

void EntityUndefineCallback::_entityUndefinedFailedSlot(QSharedPointer<Ice::Exception> exception)
{
    qDebug() << tr("Caught Ice exception: %1").arg(QString::fromStdString(exception->ice_name()));
    deleteLater();
}

} // namespace Hns
