/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "entityfactoryI.h"
#include "masterI.h"
#include "catalog/entitycatalogI.h"
#include "domainI.h"
#include "switchI.h"
#include "wireI.h"
#include "hybridnetcardI.h"
#include "wifiapI.h"
#include "dynamipsrouterI.h"
#include "topologygateI.h"
#include "entryutils.h"

// Generated hynesim headers
#include "common/domainentry.h"
#include "common/switchentry.h"
#include "common/wireentry.h"
#include "common/hybridnetcardentry.h"
#include "common/wifiapentry.h"
#include "common/dynamipsrouterentry.h"
#include "common/topologygateentry.h"

// ZeroC Ice headers
#include <Ice/Current.h>

// diamesh headers
#include <diamesh/sessionManagerI.h>

// Qt headers
#include <QDebug>

namespace Hns {

EntityFactoryI::EntityFactoryI()
{
    registerMetaObject<DomainEntry, DomainI>();
    registerMetaObject<SwitchEntry, SwitchI>();
    registerMetaObject<WireEntry, WireI>();
    registerMetaObject<HybridNetcardEntry, HybridNetcardI>();
    registerMetaObject<WifiAPEntry,WifiAPI>();
    registerMetaObject<DynamipsRouterEntry,DynamipsRouterI>();
    registerMetaObject<TopologyGateEntry, TopologyGateI>();
}

EntityFactoryI::~EntityFactoryI()
{
}

EntityPrx EntityFactoryI::createEntity(const::Hns::EntityEntryPtr &entry, const ::Ice::Current &current)
{
    DomainEntryPtr domainEntry = DomainEntryPtr::dynamicCast(entry);
    if (domainEntry && domainEntry->diffUuids.size()) {
        qCritical() << tr("Cannot create domain which has linked clones");
        LinkedDomainError exception;
        throw exception;
        return NULL;
    }
    EntityEntryPtr entityEntry = entry;
    if ((entry->isTemplate && !entry->hasResources) || entry->isModel) {
        entityEntry = Hns::EntityEntryPtr::dynamicCast(entityEntry->ice_clone());
        entityEntry->id = 0;
        entityEntry->path = "";
        entityEntry->uuid = MasterI::instance()->entityCatalog()->generateUnusedUUID().toStdString();
        entityEntry->isTemplate = false;
        entityEntry->hasResources = false;
    }
    QString className = QString::fromStdString(entityEntry->ice_id());

    if (!_factoryHash.contains(className)) {
        qCritical() << tr("The class name %1 is unknown in the factory. Unable to create the corresponding entity (id: %2).").arg(className).arg(entityEntry->id);
        return NULL;
    }

    int status = entry->status;
    if (status == ::Hns::ENTITY_STATUS_UUID_CONFLICT) {
        qCritical() << tr("There is a conflict with this UUID : %1").arg(entry->uuid.c_str());
        return NULL;
    } else if (status == Hns::ENTITY_STATUS_PROVISIONING) {
        qCritical() << tr("Errors occured during provisioning");
        return NULL;
    }

    EntryPropertiesManagerPrx propertiesManager = MasterI::instance()->getEntryPropertiesManager();
    if (!propertiesManager) {
        qCritical() << tr("Could not get entry properties manager");
        return NULL;
    }

    if (propertiesManager->canValidateEntry(entityEntry)) {
        EntityEntryPtr newEntityEntry = Hns::EntityEntryPtr::dynamicCast(propertiesManager->validateEntry(entityEntry));
        if (!newEntityEntry) {
            qCritical() << tr("Entry %1 is not valid. Unable to create entity").arg(entityEntry->uuid.c_str());
            return NULL;
        } else {
            entityEntry = newEntityEntry;
        }
    }

    EntityCatalogI *catalog = MasterI::instance()->entityCatalog();
    int entryId = entityEntry->id;

    if (!entryId)
        initEntry(entityEntry, MasterI::instance()->sessionManager()->currentUser(current));

    if (entryId && !catalog->lockEntry(entryId)) {
        qCritical() << tr("The entry %1 could not be locked. Unable to create entity").arg(entryId);
        EntryLockedError exception;
        throw exception;
        return NULL;
    }

    QMetaObject metaObject = _factoryHash.value(className);
    EntityI *entity(qobject_cast<EntityI *>(metaObject.newInstance()));
    if (!entryId && entityEntry->shortName.empty())
        entityEntry->shortName = entity->objectName().toStdString();
    entity->setEntry(entityEntry);
    entity->initPlugs();

    Ice::ObjectAdapterPtr adapter = Diamesh::MasterI::instance()->adapter();
    Ice::ObjectPrx proxy = adapter->addWithUUID(entity);
    entity->setIdentity(proxy->ice_getIdentity());

    EntityPrx entityPrx = EntityPrx::uncheckedCast(proxy);
    if (!entityPrx) {
        if (entryId && !catalog->unlockEntry(entryId))
            qCritical() << tr("The entry %1 could not be unlocked.").arg(entryId);
        return NULL;
    }

    ::Ice::Int entityId = entityPrx->getId();
    _uuidHash[entityEntry->uuid.c_str()] = entityId;

    return entityPrx;
}

EntityPrx EntityFactoryI::createEntityFromId(::Ice::Long entryId, const::Ice::Current &current)
{
    EntityEntryPtr entityEntry = MasterI::instance()->entityCatalog()->getEntityEntry(entryId, current);
    if (!entityEntry) {
        qCritical() << tr("Unable to get the catalog entry %1").arg(entryId);
        return NULL;
    }
    EntityPrx entity = createEntity(entityEntry, current);
    if (!entity) {
        qCritical() << tr("The entry %1 could not be created.").arg(entryId);
        return NULL;
    }

    return entity;
}

bool EntityFactoryI::matchClass(const QString &classId , const QString &classMeta)
{
    if(_factoryHash.find(classId) == _factoryHash.end() ) return false;
    if(_factoryHash[classId].className() == classMeta) return true;
    return false;
}

} // namespace Hns
