/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "entrypropertiessetter.h"

// Generated hynesim headers
#include "common/entityentryproperties.h"
#include "common/domainentry.h"
#include "common/domainentryproperties.h"
#include "common/wireentry.h"
#include "common/wireentryproperties.h"
#include "common/switchentry.h"
#include "common/switchentryproperties.h"
#include "common/wifiapentry.h"
#include "common/topologygateentry.h"
#include "common/hybridnetcardentry.h"

// Qt headers
#include <QDebug>
#include <QList>

namespace Hns {

///////////////////////////////////////////////////////////////////////////////
//////                   Class EntryPropertiesSetter                     //////
///////////////////////////////////////////////////////////////////////////////

EntryPropertiesSetter::EntryPropertiesSetter(QObject *parent)
    : QObject(parent)
    , _properties(NULL)
{
}

EntryPropertiesSetter::~EntryPropertiesSetter()
{
}

EntryPropertiesPtr EntryPropertiesSetter::getProperties(const EntryPtr &entry)
{
    _properties = new EntryProperties();
    _fillProperties(entry);
    return _properties;
}

EntryPtr EntryPropertiesSetter::validateEntry(const EntryPtr &entry)
{
    return Hns::EntryPtr::dynamicCast(entry->ice_clone());
}

bool EntryPropertiesSetter::_fillProperties(const EntryPtr &entry)
{
    Q_UNUSED(entry)
    _properties->isExecutable = true;
    return true;
}

///////////////////////////////////////////////////////////////////////////////
//////                Class EntityEntryPropertiesSetter                  //////
///////////////////////////////////////////////////////////////////////////////

EntityEntryPropertiesSetter::EntityEntryPropertiesSetter(QObject *parent)
    : EntryPropertiesSetter(parent)
{
}

EntryPropertiesPtr EntityEntryPropertiesSetter::getProperties(const EntryPtr &entry)
{
    _properties = new EntityEntryProperties();
    _fillProperties(entry);
    return _properties;
}

EntryPtr EntityEntryPropertiesSetter::validateEntry(const EntryPtr &entry)
{
    EntryPtr newEntry = EntryPropertiesSetter::validateEntry(entry);
    if (!newEntry)
        return NULL;

    return newEntry;
}

bool EntityEntryPropertiesSetter::validateEntryWhenDefined(const EntityEntryPtr &oldEntry, const EntityEntryPtr &newEntry)
{
    Q_UNUSED(oldEntry)
    Q_UNUSED(newEntry)
    return true;
}

bool EntityEntryPropertiesSetter::_fillProperties(const EntryPtr &entry)
{
    EntryPropertiesSetter::_fillProperties(entry);
    return true;
}

///////////////////////////////////////////////////////////////////////////////
//////                Class DomainEntryPropertiesSetter                  //////
///////////////////////////////////////////////////////////////////////////////

DomainEntryPropertiesSetter::DomainEntryPropertiesSetter(QObject *parent)
    : EntityEntryPropertiesSetter(parent)
{
}

EntryPropertiesPtr DomainEntryPropertiesSetter::getProperties(const EntryPtr &entry)
{
    _properties = new DomainEntryProperties();
    _fillProperties(entry);
    return _properties;
}

EntryPtr DomainEntryPropertiesSetter::validateEntry(const EntryPtr &entry)
{    
    EntryPtr newEntry = EntityEntryPropertiesSetter::validateEntry(entry);
    if (!newEntry)
        return NULL;

    DomainEntryPtr domainEntry = DomainEntryPtr::dynamicCast(newEntry);
    if (!domainEntry) {
        qCritical() << tr("Entry is not a domain entry, unable to check if it is valid");
        return NULL;
    }

    if (domainEntry->type == "vbox") {
        uint nbNetworkCards = 8;
        if (domainEntry->netcards.size() > nbNetworkCards) {
            qCritical() << tr("Entry %1 is not valid : it should not have more than %2 network cards").arg(newEntry->uuid.c_str()).arg(nbNetworkCards);
            return NULL;
        }
    }
    else if (domainEntry->type == "openvz") {
        int nbCpuMax = 1;
        if (domainEntry->vcpu > nbCpuMax) {
            qCritical() << tr("Entry %1 is not valid : it should not have more than %2 cpus").arg(newEntry->uuid.c_str()).arg(nbCpuMax);
            return NULL;
        }
    }

    QMap<Ice::Long, Hns::Netcard> netcards(domainEntry->netcards);
    foreach (Hns::Netcard netcard, netcards) {
        if (netcard.id < 0) { // new netcard
            netcards.remove(netcard.id);
            domainEntry->netcardAutoId++;
            netcard.id = domainEntry->netcardAutoId;
            netcards.insert(netcard.id, netcard);
        }
    }

    domainEntry->netcards = netcards.toStdMap();

    return newEntry;
}

bool DomainEntryPropertiesSetter::validateEntryWhenDefined(const EntityEntryPtr &oldEntry, const EntityEntryPtr &newEntry)
{
    if (!EntityEntryPropertiesSetter::validateEntryWhenDefined(oldEntry, newEntry))
        return false;

    DomainEntryPtr oldDomainEntry = DomainEntryPtr::dynamicCast(oldEntry);
    if (!oldDomainEntry) {
        qCritical() << tr("Entry is not a domain entry, unable to check if it is valid");
        return NULL;
    }

    DomainEntryPtr newDomainEntry = DomainEntryPtr::dynamicCast(newEntry);
    if (!newDomainEntry) {
        qCritical() << tr("Entry is not a domain entry, unable to check if it is valid");
        return NULL;
    }

    return ((oldDomainEntry->type == newDomainEntry->type) && (oldDomainEntry->os == newDomainEntry->os) &&
        (oldDomainEntry->memory == newDomainEntry->memory) && (oldDomainEntry->vcpu == newDomainEntry->vcpu) &&
        (oldDomainEntry->pae == newDomainEntry->pae) && (oldDomainEntry->acpi == newDomainEntry->acpi) &&
        (oldDomainEntry->apic == newDomainEntry->apic) && (oldDomainEntry->netcards == newDomainEntry->netcards) &&
        (oldDomainEntry->disks == newDomainEntry->disks) && (oldDomainEntry->usbControllerType == newDomainEntry->usbControllerType) &&
        (oldDomainEntry->vid.videoModelType == newDomainEntry->vid.videoModelType));
}

bool DomainEntryPropertiesSetter::_fillProperties(const EntryPtr &entry)
{
    EntityEntryPropertiesSetter::_fillProperties(entry);

    DomainEntryPropertiesPtr properties = DomainEntryPropertiesPtr::dynamicCast(_properties);

    DomainEntryPtr domainEntry = DomainEntryPtr::dynamicCast(entry);
    if (!domainEntry) {
        qCritical() << tr("Entry is not a domain entry, unable to fill its properties");
        return false;
    }

    properties->nbCpuMax=-1;
    properties->nbNetworkCards=-1;
    properties->hasKeymap = false;
    QList<std::string> networkCardsTypesList;
    QList<std::string> networkCardsModelsList;
    QList<std::string> archList = QList<std::string>() << "x86_64";
    QList<std::string> keymapList;
    QList<std::string> usbControllerTypesList;
    QList<std::string> videoModelTypesList;

    properties->hasStorageSettings = false;
    QList<std::string> storageBusTypeList;
    QMap<std::string, std::string> storageTargetMap;


    if (domainEntry->type == "vbox") {
        properties->nbNetworkCards=8;
        properties->nbCpuMax = 32;
        networkCardsTypesList << "bridge";
        networkCardsModelsList << "am79c970a" << "am79c973" << "82540em" << "82545em" << "virtio";
    } else if (domainEntry->type == "openvz") {
        networkCardsTypesList << "ethernet";
        properties->nbNetworkCards=1000;
        properties->nbCpuMax = 8;
    } else if (domainEntry->type == "kvm") {
        properties->nbNetworkCards=28;
        networkCardsTypesList << "ethernet";
        networkCardsModelsList << "ne2k_pci" << "i82551" << "i82557b" << "i82559er" << "rtl8139" << "e1000" << "pcnet" << "virtio";
        properties->nbCpuMax = 255;
        properties->hasKeymap = true;
        keymapList << "ar" << "da" << "de" << "de-ch" << "en-gb" << "en-us"
                   << "es" << "et" << "fi" << "fo" << "fr" << "fr-be" << "fr-ca"
                   << "fr-ch" << "hr" << "hu" << "is" << "it" << "ja" << "lt"
                   << "lv" << "mk" << "nl" << "nl-be" << "no" << "pl" << "pt"
                   << "pt-br" << "ru" << "sl" << "sv" << "th" << "tr";
         usbControllerTypesList = QList<std::string>() << "" << USB_CONTROLLER_USB1 << USB_CONTROLLER_USB2 << USB_CONTROLLER_USB3;
         videoModelTypesList << "" << "vga" << "cirrus" << "vmvga" << "qxl";
    } else if (domainEntry->type == "qemu") {
        archList << "i686" << "armv7l" << "microblaze" << "mips" << "mipsel" << "sparc" << "ppc" << "ppc64";
        networkCardsTypesList << "ethernet";
        properties->nbCpuMax = 1;
        properties->hasKeymap = true;
        keymapList << "ar" << "da" << "de" << "de-ch" << "en-gb" << "en-us"
                   << "es" << "et" << "fi" << "fo" << "fr" << "fr-be" << "fr-ca"
                   << "fr-ch" << "hr" << "hu" << "is" << "it" << "ja" << "lt"
                   << "lv" << "mk" << "nl" << "nl-be" << "no" << "pl" << "pt"
                   << "pt-br" << "ru" << "sl" << "sv" << "th" << "tr";
        usbControllerTypesList = QList<std::string>() << "" << USB_CONTROLLER_USB1 << USB_CONTROLLER_USB2 << USB_CONTROLLER_USB3;
        videoModelTypesList << "" << "vga" << "cirrus" << "vmvga" << "qxl";
    } else if (domainEntry->type == "vmware") {
        archList << "i686";
        networkCardsTypesList << "bridge";
        networkCardsModelsList << "e1000" << "vlance" << "vmxnet" << "vmxnet3";
        properties->nbCpuMax = 8;
        properties->nbNetworkCards = 10;
        properties->hasKeymap = true;
        keymapList << "de" << "de-ch" << "es" << "fi" << "fr" << "fr-be"
                   << "fr-ch" << "is" << "it" << "jp" << "nl-be" << "no"
                   << "pt" << "uk" << "us";
    } else if (domainEntry->type == "lxc") {
        networkCardsTypesList << "bridge";
        properties->nbNetworkCards = 1000;
        properties->nbCpuMax = 255;
        properties->hasKeymap = true;
        keymapList << "ar" << "da" << "de" << "de-ch" << "en-gb" << "en-us"
                   << "es" << "et" << "fi" << "fo" << "fr" << "fr-be" << "fr-ca"
                   << "fr-ch" << "hr" << "hu" << "is" << "it" << "ja" << "lt"
                   << "lv" << "mk" << "nl" << "nl-be" << "no" << "pl" << "pt"
                   << "pt-br" << "ru" << "sl" << "sv" << "th" << "tr";
    }

    if (domainEntry->type != "openvz") {
        properties->nbDisksMax = 4;
        properties->hasStorageSettings = true;
        storageBusTypeList << "ide" << "scsi";
        storageTargetMap.insert("ide", "hd");
        storageTargetMap.insert("scsi", "sd");
        if (domainEntry->type != "vmware") {
            storageBusTypeList << "sata";
            storageTargetMap.insert("sata", "sd");
            if (domainEntry->type == "qemu" || domainEntry->type == "kvm") {
                storageBusTypeList << "virtio" << "sd";
                storageTargetMap.insert("virtio", "vd");
                storageTargetMap.insert("sd", "sd");
            }
        }
    }

    properties->networkCardsModelsList = networkCardsModelsList.toStdList();
    properties->networkCardsTypesList = networkCardsTypesList.toStdList();
    properties->archList = archList.toStdList();
    properties->keymapList = keymapList.toStdList();
    properties->storageBusTypesList = storageBusTypeList.toStdList();
    properties->storageTargetMap = storageTargetMap.toStdMap();
    properties->usbControllerTypesList = usbControllerTypesList.toStdList();
    properties->videoModelTypesList = videoModelTypesList.toStdList();

    return true;
}

///////////////////////////////////////////////////////////////////////////////
//////               Class TopologyEntryPropertiesSetter                 //////
///////////////////////////////////////////////////////////////////////////////

TopologyEntryPropertiesSetter::TopologyEntryPropertiesSetter(QObject *parent)
    : EntryPropertiesSetter(parent)
{
}

EntryPropertiesPtr TopologyEntryPropertiesSetter::getProperties(const EntryPtr &entry)
{
    _properties = new EntityEntryProperties();
    _fillProperties(entry);
    return _properties;
}

EntryPtr TopologyEntryPropertiesSetter::validateEntry(const EntryPtr &entry)
{
    EntryPtr newEntry = EntryPropertiesSetter::validateEntry(entry);
    if (!newEntry)
        return NULL;

    return newEntry;
}

bool TopologyEntryPropertiesSetter::_fillProperties(const EntryPtr &entry)
{
    EntryPropertiesSetter::_fillProperties(entry);
    return true;
}

///////////////////////////////////////////////////////////////////////////////
//////               Class WireEntryPropertiesSetter                 //////
///////////////////////////////////////////////////////////////////////////////

WireEntryPropertiesSetter::WireEntryPropertiesSetter(QObject *parent)
    : EntityEntryPropertiesSetter(parent)
{
}

EntryPropertiesPtr WireEntryPropertiesSetter::getProperties(const EntryPtr &entry)
{
    _properties = new WireEntryProperties();
    _fillProperties(entry);
    return _properties;
}

EntryPtr WireEntryPropertiesSetter::validateEntry(const EntryPtr &entry)
{
    EntryPtr newEntry = EntityEntryPropertiesSetter::validateEntry(entry);
    if (!newEntry)
        return NULL;

    return newEntry;
}

bool WireEntryPropertiesSetter::_fillProperties(const EntryPtr &entry)
{
    EntityEntryPropertiesSetter::_fillProperties(entry);
    WireEntryPtr wireEntry = WireEntryPtr::dynamicCast(entry);
    if (!wireEntry) {
        qCritical() << tr("Entry is not a wire entry, unable to fill its properties");
        return false;
    }
    WireEntryPropertiesPtr properties = WireEntryPropertiesPtr::dynamicCast(_properties);
    properties->canModified = true;
    properties->implementationsSupported.insert(std::make_pair("nativeWire",tr("Native Wire").toStdString()));
    properties->networkPropertiesSupported = true;
    return true;
}

///////////////////////////////////////////////////////////////////////////////
//////               Class SwitchEntryPropertiesSetter                   //////
///////////////////////////////////////////////////////////////////////////////

SwitchEntryPropertiesSetter::SwitchEntryPropertiesSetter(QObject *parent)
    : EntityEntryPropertiesSetter(parent)
{
}

EntryPtr SwitchEntryPropertiesSetter::validateEntry(const EntryPtr &entry)
{
    EntryPtr newEntry = EntityEntryPropertiesSetter::validateEntry(entry);
    if (!newEntry)
        return NULL;

    SwitchEntryPtr switchEntry = SwitchEntryPtr::dynamicCast(newEntry);
    if (!switchEntry) {
        qCritical() << tr("Entry is not a switch entry, unable to check if it is valid");
        return NULL;
    }

    QList<int> unassignedVlans;
    for (int i = 0 ; i < switchEntry->ports ; i++) {
        unassignedVlans.append(i);
    }
    Hns::VlanMap vlans = switchEntry->vlans;
    QMap<Ice::Int, Hns::Vlan>map = QMap<Ice::Int, Hns::Vlan>(vlans);
    QMapIterator<Ice::Int, Hns::Vlan> iterator(map);
    while (iterator.hasNext()) {
        iterator.next();
        QMap<Ice::Int, Hns::PortTag> portsMap(iterator.value().portsMap);
        QMapIterator<Ice::Int, Hns::PortTag> it(portsMap);
        while (it.hasNext()) {
            it.next();
            if (it.key() >= switchEntry->ports)
                portsMap.remove(it.key());
            if (it.value() == Untagged && it.key() < switchEntry->ports)
                unassignedVlans.removeOne(it.key());
        }
        map[iterator.key()].portsMap = portsMap.toStdMap();
    }
    if (!unassignedVlans.isEmpty()) {
        for (int i = 0 ; i <unassignedVlans.size() ; i++)
            map[0].portsMap[unassignedVlans.at(i)] = Hns::Untagged;
    }
    vlans = map.toStdMap();
    switchEntry->vlans = vlans;
    return newEntry;
}

EntryPropertiesPtr SwitchEntryPropertiesSetter::getProperties(const EntryPtr &entry)
{
    _properties = new SwitchEntryProperties();
    _fillProperties(entry);
    return _properties;
}

bool SwitchEntryPropertiesSetter::validateEntryWhenDefined(const EntityEntryPtr &oldEntry, const EntityEntryPtr &newEntry)
{
    if (!EntityEntryPropertiesSetter::validateEntryWhenDefined(oldEntry, newEntry))
        return false;

    SwitchEntryPtr oldSwitchEntry = SwitchEntryPtr::dynamicCast(oldEntry);
    if (!oldSwitchEntry) {
        qCritical() << tr("Entry is not a switch entry, unable to check if it is valid");
        return NULL;
    }

    SwitchEntryPtr newSwitchEntry = SwitchEntryPtr::dynamicCast(newEntry);
    if (!newSwitchEntry) {
        qCritical() << tr("Entry is not a switch entry, unable to check if it is valid");
        return NULL;
    }
    bool validate = (newSwitchEntry->ports == oldSwitchEntry->ports);
    validate &= (newSwitchEntry->isHub == oldSwitchEntry->isHub);
    validate &= (newSwitchEntry->stpEnabled == oldSwitchEntry->stpEnabled);
    validate &= (newSwitchEntry->type == oldSwitchEntry->type);
    validate &= (newSwitchEntry->portMirroring == oldSwitchEntry->portMirroring);
    return validate;
}

bool SwitchEntryPropertiesSetter::_fillProperties(const EntryPtr &entry)
{
    EntityEntryPropertiesSetter::_fillProperties(entry);
    
    SwitchEntryPropertiesPtr properties = SwitchEntryPropertiesPtr::dynamicCast(_properties);
    
    SwitchEntryPtr switchEntry = SwitchEntryPtr::dynamicCast(entry);
    if (!switchEntry) {
        qCritical() << tr("Entry is not a switch entry, unable to fill its properties");
        return false;
    }
    
    properties->implementationsSupported.insert(std::make_pair("kernelBridge",tr("Kernel Bridge Switch").toStdString()));
    properties->implementationsSupported.insert(std::make_pair("openVSwitch", tr("OpenVSwitch").toStdString()));

    if (switchEntry->type == "kernelBridge") {
        properties->vlanSupported = false;
        properties->stpSupported = false;
        properties->portMirroringSupported = false;
    }
    else if (switchEntry->type == "openVSwitch") {
        properties->vlanSupported = true;
        properties->stpSupported = true;
        properties->portMirroringSupported = true;
    }
    return true;
}

///////////////////////////////////////////////////////////////////////////////
//////               Class WifiApPropertiesSetter                        //////
///////////////////////////////////////////////////////////////////////////////

WifiApPropertiesSetter::WifiApPropertiesSetter(QObject *parent)
    : EntityEntryPropertiesSetter(parent)
{
}

EntryPtr WifiApPropertiesSetter::validateEntry(const EntryPtr &entry)
{
    EntryPtr newEntry = EntityEntryPropertiesSetter::validateEntry(entry);
    if (!newEntry)
        return NULL;

    return newEntry;
}

EntryPropertiesPtr WifiApPropertiesSetter::getProperties(const EntryPtr &entry)
{
    _properties = new EntityEntryProperties();
    _fillProperties(entry);
    return _properties;
}

bool WifiApPropertiesSetter::validateEntryWhenDefined(const EntityEntryPtr &oldEntry, const EntityEntryPtr &newEntry)
{
    if (!EntityEntryPropertiesSetter::validateEntryWhenDefined(oldEntry, newEntry))
        return false;

    WifiAPEntryPtr oldWifiApEntry = WifiAPEntryPtr::dynamicCast(oldEntry);
    if (!oldWifiApEntry) {
        qCritical() << tr("Entry is not a wifi entry, unable to check if it is valid");
        return NULL;
    }

    WifiAPEntryPtr newWifiApEntry = WifiAPEntryPtr::dynamicCast(newEntry);
    if (!newWifiApEntry) {
        qCritical() << tr("Entry is not a wifi entry, unable to check if it is valid");
        return NULL;
    }
    bool validate = (newWifiApEntry->macAdress == oldWifiApEntry->macAdress);
    validate &= (newWifiApEntry->ssid == oldWifiApEntry->ssid);
    validate &= (newWifiApEntry->authType == oldWifiApEntry->authType);
    validate &= (newWifiApEntry->key == oldWifiApEntry->key);
    validate &= (newWifiApEntry->channel == oldWifiApEntry->channel);
    validate &= (newWifiApEntry->hidden == oldWifiApEntry->hidden);
    return validate;
}

bool WifiApPropertiesSetter::_fillProperties(const EntryPtr &entry)
{
    EntryPropertiesSetter::_fillProperties(entry);
    return true;
}

///////////////////////////////////////////////////////////////////////////////
//////               Class TopologyGatePropertiesSetter                  //////
///////////////////////////////////////////////////////////////////////////////

TopologyGatePropertiesSetter::TopologyGatePropertiesSetter(QObject *parent)
    : EntityEntryPropertiesSetter(parent)
{
}

EntryPtr TopologyGatePropertiesSetter::validateEntry(const EntryPtr &entry)
{
    EntryPtr newEntry = EntityEntryPropertiesSetter::validateEntry(entry);
    if (!newEntry)
        return NULL;

    return newEntry;
}

EntryPropertiesPtr TopologyGatePropertiesSetter::getProperties(const EntryPtr &entry)
{
    _properties = new EntityEntryProperties();
    _fillProperties(entry);
    return _properties;
}

bool TopologyGatePropertiesSetter::validateEntryWhenDefined(const EntityEntryPtr &oldEntry, const EntityEntryPtr &newEntry)
{
    if (!EntityEntryPropertiesSetter::validateEntryWhenDefined(oldEntry, newEntry))
        return false;

    TopologyGateEntryPtr oldTopologyGateEntry = TopologyGateEntryPtr::dynamicCast(oldEntry);
    if (!oldTopologyGateEntry) {
        qCritical() << tr("Entry is not a topology gate entry, unable to check if it is valid");
        return NULL;
    }

    TopologyGateEntryPtr newTopologyGateEntry = TopologyGateEntryPtr::dynamicCast(newEntry);
    if (!newTopologyGateEntry) {
        qCritical() << tr("Entry is not a topology gate entry, unable to check if it is valid");
        return NULL;
    }
    bool validate = (newTopologyGateEntry->networkName == oldTopologyGateEntry->networkName);
    return validate;
}

bool TopologyGatePropertiesSetter::_fillProperties(const EntryPtr &entry)
{
    EntryPropertiesSetter::_fillProperties(entry);
    return true;
}

///////////////////////////////////////////////////////////////////////////////
//////               Class HybridNetcardPropertiesSetter                 //////
///////////////////////////////////////////////////////////////////////////////

HybridNetcardPropertiesSetter::HybridNetcardPropertiesSetter(QObject *parent)
    : EntityEntryPropertiesSetter(parent)
{
}

EntryPtr HybridNetcardPropertiesSetter::validateEntry(const EntryPtr &entry)
{
    EntryPtr newEntry = EntityEntryPropertiesSetter::validateEntry(entry);
    if (!newEntry)
        return NULL;

    return newEntry;
}

EntryPropertiesPtr HybridNetcardPropertiesSetter::getProperties(const EntryPtr &entry)
{
    _properties = new EntityEntryProperties();
    _fillProperties(entry);
    return _properties;
}

bool HybridNetcardPropertiesSetter::validateEntryWhenDefined(const EntityEntryPtr &oldEntry, const EntityEntryPtr &newEntry)
{
    if (!EntityEntryPropertiesSetter::validateEntryWhenDefined(oldEntry, newEntry))
        return false;

    HybridNetcardEntryPtr oldHybridNetcardEntry = HybridNetcardEntryPtr::dynamicCast(oldEntry);
    if (!oldHybridNetcardEntry) {
        qCritical() << tr("Entry is not an hybrid entry, unable to check if it is valid");
        return NULL;
    }

    HybridNetcardEntryPtr newHybridNetcardEntry = HybridNetcardEntryPtr::dynamicCast(newEntry);
    if (!newHybridNetcardEntry) {
        qCritical() << tr("Entry is not an hybrid entry, unable to check if it is valid");
        return NULL;
    }
    bool validate = (newHybridNetcardEntry->nodeName == oldHybridNetcardEntry->nodeName);
    validate &= (newHybridNetcardEntry->plugIndex == oldHybridNetcardEntry->plugIndex);
    return validate;
}

bool HybridNetcardPropertiesSetter::_fillProperties(const EntryPtr &entry)
{
    EntryPropertiesSetter::_fillProperties(entry);
    return true;
}

} // namespace Hns
