/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "entryserializer.h"
#include "catalog/entitycatalogI.h"
#include "dynamipsrouterdbI.h"
#include "masterI.h"

// hynesim common headers
#include "common/initializer.h"
#include "common/constants.h"

// Generated hynesim headers
#include "common/domainentry.h"
#include "common/switchentry.h"
#include "common/hybridnetcardentry.h"
#include "common/topologyentry.h"
#include "common/wifiapentry.h"
#include "common/dynamipsrouterentry.h"
#include "common/wireentry.h"
#include "common/topologygateentry.h"
#include "common/topologylink.h"

// diamesh headers
#include <diamesh/types.h>

// Qt headers
#include <QDir>
#include <QUuid>
#include <QDebug>

namespace Hns {

HNS_STATIC_HASH_INITIALIZER(_readSerializerTypes, ReadSerializerTypes, QString, QMetaObject)
ReadSerializerTypes::ReadSerializerTypes()
{
    insert("Entity/domain", DomainEntrySerializer::staticMetaObject);
    insert("Entity/switch", SwitchEntrySerializer::staticMetaObject);
    insert("Entity/hybridnetcard", HybridNetcardEntrySerializer::staticMetaObject);
    insert("Topology", TopologyEntrySerializer::staticMetaObject);
    insert("Entity/wifiap",WifiAPEntrySerializer::staticMetaObject);
    insert("Entity/dynamipsrouter",DynamipsRouterEntrySerializer::staticMetaObject);
    insert("Entity/wire",WireEntrySerializer::staticMetaObject);
    insert("Entity/topologygate",TopologyGateEntrySerializer::staticMetaObject);
}

HNS_STATIC_HASH_INITIALIZER(_writeSerializerTypes, WriteSerializerTypes, QString, QMetaObject)
WriteSerializerTypes::WriteSerializerTypes()
{
    insert(QString::fromStdString(DomainEntry::ice_staticId()), DomainEntrySerializer::staticMetaObject);
    insert(QString::fromStdString(SwitchEntry::ice_staticId()), SwitchEntrySerializer::staticMetaObject);
    insert(QString::fromStdString(HybridNetcardEntry::ice_staticId()), HybridNetcardEntrySerializer::staticMetaObject);
    insert(QString::fromStdString(TopologyEntry::ice_staticId()), TopologyEntrySerializer::staticMetaObject);
    insert(QString::fromStdString(WifiAPEntry::ice_staticId()),WifiAPEntrySerializer::staticMetaObject);
    insert(QString::fromStdString(DynamipsRouterEntry::ice_staticId()),DynamipsRouterEntrySerializer::staticMetaObject);
    insert(QString::fromStdString(WireEntry::ice_staticId()),WireEntrySerializer::staticMetaObject);
    insert(QString::fromStdString(TopologyGateEntry::ice_staticId()),TopologyGateEntrySerializer::staticMetaObject);
}

HNS_STATIC_HASH_INITIALIZER(_hashAuthTypeString, HashAuthTypeString, ::Hns::AuthTypeEnum, QString)
HashAuthTypeString::HashAuthTypeString()
{
    insert(::Hns::NONE,"NONE");
    insert(::Hns::WEP,"WEP");
    insert(::Hns::WPA1,"WPA1");
    insert(::Hns::WPA2,"WPA2");
    insert(::Hns::WPA12,"WPA12");
}

/*****************************************************************************/
/* EntrySerializer class                                                     */
/*****************************************************************************/
EntryPtr EntrySerializer::readEntry(const QString &path, const QString &docType)
{
    QString dirName = QDir(path).dirName();
    if (QUuid(dirName).isNull()) {
        qCritical() << tr("The entry directory name is not a valid UUID: %1").arg(dirName);
        return NULL;
    }

    QDir entryDir(path);
    if (!entryDir.exists()) {
        qCritical() << tr("The entry directory does not exist: %1").arg(path);
        return NULL;
    }

    entryDir.setNameFilters(QStringList("HnsEntryConfig.xml"));
    entryDir.setFilter(QDir::Files | QDir::NoDotAndDotDot | QDir::Readable);
    entryDir.setSorting(QDir::Type | QDir::Name | QDir::Time | QDir::Reversed);
    QFileInfoList list = entryDir.entryInfoList();
    if (list.size() == 0) {
        qWarning() << tr("No hynesim XML configuration file (HnsEntryConfig.xml) was found in the directory %1").
                      arg(QDir::cleanPath(path));
        return NULL;
    } else if (list.size() > 1) {
        qCritical() << tr("There should be only one HnsEntryConfig.xml file. %1 found").arg(list.size());
        return NULL;
    }

    QString configFilePath = list.at(0).absoluteFilePath();

    QFile file(configFilePath);
    if(!file.open(QIODevice::ReadOnly | QFile::Text)) {
        qCritical() << tr("Unable to read file %1").arg(configFilePath);
        return NULL;
    }

    QString errorStr;
    int errorLine;
    int errorColumn;
    QDomDocument domDocument;

    if (!domDocument.setContent(&file, true, &errorStr, &errorLine, &errorColumn)) {
        qCritical() << tr("Error at line %1, column %2: %3").arg(errorLine).arg(errorColumn).arg(errorStr);
        qCritical() << tr("Unable to deserialize %1").arg(path);
        file.close();
        return NULL;
    }
    file.close();

    if (domDocument.doctype().name() != docType) {
        qCritical() << tr("Doctype '%1' is not matching expected doctype '%2'").arg(domDocument.doctype().name()).arg(docType);
        qCritical() << tr("Unable to deserialize %1").arg(path);
        return NULL;
    }

    EntryPtr entry = readEntry(domDocument, dirName);
    if (!entry) {
        qCritical() << tr("Unable to deserialize %1").arg(path);
        return NULL;
    }

    entry->uuid = dirName.toStdString();
    entry->path = QDir::cleanPath(QString("%1/..").arg(path)).toStdString();

    return entry;
}

// static
EntryPtr EntrySerializer::readEntry(const QDomDocument &doc, const QString &uuid)
{
    QDomElement root = doc.documentElement();
    if (root.tagName() != doc.doctype().name()) {
        qCritical() << tr("The file is not a valid %1 file").arg(doc.doctype().name());
        return NULL;
    } else if (root.hasAttribute("version") && root.attribute("version").toDouble() > Constants::CURRENT_VERSION_XML) {
        qCritical() << tr("The file version %1 is not supported (current version is %2)").arg(root.attribute("version")).arg(QString::number(Hns::Constants::CURRENT_VERSION_XML, 'f', 1));
        return NULL;
    }

    QDomNodeList headerNodeList = root.elementsByTagName("Header");
    if (headerNodeList.size() != 1) {
        qCritical() << tr("There should be exactly one <Header> tag in the configuration file");
        return NULL;
    }

    QDomElement header = headerNodeList.item(0).toElement();
    QDomElement currentNode = header.nextSiblingElement();

    QStringList serializerTypes = _readSerializerTypes()->keys();
    QString serializerType;
    while (!currentNode.isNull()) {
        serializerType += currentNode.nodeName();
        QRegExp regExp(QString("^%1").arg(serializerType));
        serializerTypes = serializerTypes.filter(regExp);
        if (serializerTypes.count() <= 1)
            break;
        currentNode = currentNode.firstChildElement();
        serializerType += "/";
    }

    if (serializerTypes.count() != 1) {
        qCritical() << tr("There is no serializer available for the type %1").arg(serializerType);
        return NULL;
    }

    QMetaObject metaObject = _readSerializerTypes()->value(serializerType);
    QScopedPointer<EntrySerializer> serializer(qobject_cast<EntrySerializer *>(metaObject.newInstance(Q_ARG(QDomDocument, doc))));

    serializer->init();
    serializer->_entryUUID = uuid;
    if (!serializer->read()) {
        return NULL;
    }
    return serializer->getEntry();
}

// static
bool EntrySerializer::writeEntry(const EntryPtr &entry, const QString &docType, const QString& fullPath)
{
    QString className = QString::fromStdString(entry->ice_id());
    QMetaObject metaObject = _writeSerializerTypes()->value(className);
    QScopedPointer<EntrySerializer>  serializer(qobject_cast<EntrySerializer *>(metaObject.newInstance()));
    if (!serializer) {
        qCritical() << tr("There is no serializer available for the type %1")
                       .arg(className);
        return NULL;
    }

    return serializer->write(entry, docType, fullPath);
}

// static
QString EntrySerializer::writeEntryToString(const EntryPtr &entry, const QString &docType)
{
    QString className = QString::fromStdString(entry->ice_id());
    QMetaObject metaObject = _writeSerializerTypes()->value(className);
    QScopedPointer<EntrySerializer>  serializer(qobject_cast<EntrySerializer *>(metaObject.newInstance()));
    if (!serializer) {
        qCritical() << tr("There is no serializer available for the type %1")
                       .arg(className);
        return QString();
    }

    return serializer->writeToString(entry, docType);
}

EntrySerializer::EntrySerializer(const QDomDocument &doc)
    : _entry(NULL)
    , _docType("")
    , _version(0.0)
{
    _doc = doc;
}

EntrySerializer::EntrySerializer()
{
}

void EntrySerializer::init()
{
    _entry = new Entry();
}

EntryPtr EntrySerializer::getEntry()
{
    return _entry;
}

QString EntrySerializer::getEntryUUID()
{
    if (!_entryUUID.isEmpty()) {
        return _entryUUID;
    }

    if (_entry && _entry->uuid.length() > 0) {
        return QString::fromStdString(_entry->uuid);
    }

    return QString("[UNKNOWN]");
}

bool EntrySerializer::read()
{
    QDomElement root = _doc.documentElement();

    if (root.hasAttribute("version"))
        _version = root.attribute("version").toDouble();

    QDomNodeList headerNodeList = root.elementsByTagName("Header");
    if(headerNodeList.size() != 1) {
        qCritical() << tr("Entry %1: There should be exactly one <Header> tag in the configuration file")
                       .arg(getEntryUUID());
        return false;
    }

    QDomElement header = headerNodeList.item(0).toElement();

    QString shortName = header.attribute("shortName");
    if(shortName.isEmpty()) {
        qCritical() << tr("Entry %1: The \"shortName\" attribute is empty or missing from the <Header> tag")
                       .arg(getEntryUUID());
        return false;
    }

    QString longName = header.attribute("longName");
    QString description = header.attribute("description");
    QString tags = header.attribute("tags");

    ::Diamesh::StringList tagList;
    if (!tags.isEmpty()){
        foreach (QString tag, tags.split(", ")){
            tagList.push_back(tag.toStdString());
        }
    }
    QString creationDateTime = header.attribute("creationDateTime");
    if(creationDateTime.isEmpty()) {
        qCritical() << tr("Entry %1: The \"creationDateTime\" attribute is empty or missing from the <Header> tag")
                       .arg(getEntryUUID());
        return false;
    }

    QString lastModificationDateTime = header.attribute("lastModificationDateTime");
    if(lastModificationDateTime.isEmpty()) {
        qDebug() << tr("Entry %1: The \"lastModificationDateTime\" attribute is empty or missing from the <Header> tag")
                    .arg(getEntryUUID());
    }

    _entry->shortName = shortName.toStdString();
    _entry->longName = longName.toStdString();
    _entry->description = description.toStdString();
    _entry->creationDateTime = creationDateTime.toLongLong();
    _entry->lastModificationDateTime = lastModificationDateTime.toLongLong();
    _entry->hSetId = -1;
    if (tagList.size() !=0 ){
        _entry->tagList = tagList;
    }

    return true;
}

bool EntrySerializer::write(const EntryPtr &entry, const QString &docType, const QString &fullPath)
{
    _entry = entry;
    _docType = docType;

    if (!_fillDoc())
        return false;

    QDir dir(fullPath);
    if (!QFile::exists(fullPath) && !dir.mkpath(".")) {
        qCritical() << tr("Unable to create entry directory %1").arg(fullPath);
        return false;
    }

    QString fileName = QString("%1/hnsEntryConfig.xml").arg(fullPath);

    QFile file(fileName);
    if (!file.open(QIODevice::WriteOnly | QFile::Text)) {
        qCritical() << tr("Unable to open file %1").arg(fileName);
        return false;
    }
    QTextStream out(&file);
    const int xmlIndentSize = 2;
    _doc.save(out, xmlIndentSize);
    file.close();

    return true;
}

QString EntrySerializer::writeToString(const EntryPtr &entry, const QString &docType)
{
    _entry = entry;
    _docType = docType;

    if (!_fillDoc())
        return QString();

    return _doc.toString(-1);
}

bool EntrySerializer::_fillDoc()
{
    if (!_entry) {
        qCritical() << tr("Entry %1: The entry is not defined. Unable to fill the DOM Document")
                       .arg(getEntryUUID());
        return false;
    }

    _doc = QDomDocument(_docType);

    QDomElement configElement = _doc.createElement(_docType);
    configElement.setAttribute("version", QString::number(Hns::Constants::CURRENT_VERSION_XML, 'f', 1));

    QDomElement headerElement = _doc.createElement("Header");
    headerElement.setAttribute("shortName", _entry->shortName.c_str());
    headerElement.setAttribute("longName", _entry->longName.c_str());
    headerElement.setAttribute("description", _entry->description.c_str());
    headerElement.setAttribute("creationDateTime", QString("%1").arg(_entry->creationDateTime));
    headerElement.setAttribute("lastModificationDateTime", QString("%1").arg(_entry->lastModificationDateTime));

    QList<std::string> tagList = QList<std::string>::fromStdList(_entry->tagList);
    if (!tagList.isEmpty()){
        QString tags;
        foreach (std::string tag, tagList) {
            if (!tags.isEmpty())
                tags.append(", ");
            tags.append(tag.c_str());
        }
        headerElement.setAttribute("tags", tags);
    }

    configElement.appendChild(headerElement);
    _doc.appendChild(configElement);
    return true;
}

QDomElement EntrySerializer::_addNode(QDomElement &parent, const QString &nodeName, const QString &nodeValue)
{
    QDomElement element = _doc.createElement(nodeName);
    if (!nodeValue.isEmpty()) {
        QDomText nodeText = _doc.createTextNode(nodeValue);
        element.appendChild(nodeText);
    }
    parent.appendChild(element);

    return element;
}

/*****************************************************************************/
/* EntityEntrySerializer class                                               */
/*****************************************************************************/
EntityEntrySerializer::EntityEntrySerializer(const QDomDocument &doc)
    : EntrySerializer(doc)
{
}

EntityEntrySerializer::EntityEntrySerializer()
{
}

bool EntityEntrySerializer::read()
{
    if (!EntrySerializer::read())
        return false;

    QDomElement root = _doc.documentElement();
    QDomNodeList entityNodeList = root.elementsByTagName("Entity");
    if(entityNodeList.size() != 1) {
        qCritical() << tr("Entry %1: There should be exactly one <Entity> tag in the configuration file")
                       .arg(getEntryUUID());
        return false;
    }

    QDomElement entity = entityNodeList.item(0).toElement();
    EntityEntryPtr entityEntry = EntityEntryPtr::dynamicCast(_entry);

    QString restrictions = entity.attribute("restrictions");    
    if (!restrictions.isEmpty()){
        foreach (QString restriction, restrictions.split(", ")){
            entityEntry->restrictionList.push_back(restriction.toStdString());
        }
    }
    entityEntry->saved = entity.attribute("saved", "0").toInt();
    entityEntry->isTemplate = entity.attribute("isTemplate", "0").toInt();
    entityEntry->isModel = entity.attribute("isModel", "0").toInt();
    entityEntry->customGraphic = entity.attribute("customGraphic", "").toStdString();
    entityEntry->hasResources = false;

    return true;
}

bool EntityEntrySerializer::_fillDoc()
{
    if (!EntrySerializer::_fillDoc())
        return false;

    QDomElement entityElement = _doc.createElement("Entity");

    EntityEntryPtr entityEntry = EntityEntryPtr::dynamicCast(_entry);

    if (!entityEntry) {
        qCritical() << tr("Entry %1: The entry is not an entity entry. Unable to fill the DOM Document")
                       .arg(getEntryUUID());
        return false;
    }

    // generic attributes
    entityElement.setAttribute("saved", entityEntry->saved);
    entityElement.setAttribute("isTemplate", entityEntry->isTemplate);
    entityElement.setAttribute("isModel", entityEntry->isModel);
    entityElement.setAttribute("customGraphic", QString::fromStdString(entityEntry->customGraphic));
    QList<std::string> restrictionList = QList<std::string>::fromStdList(entityEntry->restrictionList);
    if (!restrictionList.isEmpty()) {
        QString restrictions;
        foreach (std::string restriction, restrictionList) {
            if (!restrictions.isEmpty())
                restrictions.append(", ");
            restrictions.append(restriction.c_str());
        }
        entityElement.setAttribute("restrictions", restrictions);
    }

    QDomElement configElement = _doc.firstChildElement(_docType);
    configElement.appendChild(entityElement);


    return true;
}

/*****************************************************************************/
/* DomainEntrySerializer class                                               */
/*****************************************************************************/
DomainEntrySerializer::DomainEntrySerializer(const QDomDocument &doc)
    : EntityEntrySerializer(doc)
{
}

DomainEntrySerializer::DomainEntrySerializer()
{
}

void DomainEntrySerializer::init()
{
    _entry = new DomainEntry();
}

bool DomainEntrySerializer::read()
{
    if (!EntityEntrySerializer::read())
        return false;

    QDomElement root = _doc.documentElement();
    QDomNodeList domainNodeList = root.elementsByTagName("domain");
    if(domainNodeList.size() != 1) {
        qCritical() << tr("Entry %1: There should be exactly one <domain> tag in the configuration file")
                       .arg(getEntryUUID());
        return false;
    }

    QDomElement domain = domainNodeList.item(0).toElement();
    DomainEntryPtr domainEntry = DomainEntryPtr::dynamicCast(_entry);

    // generic domain attributes
    domainEntry->hasResources = !domainEntry->isModel;

    QString type = domain.attribute("type");
    if(type.isEmpty()) {
        qCritical() << tr("Entry %1: The \"type\" attribute is empty or missing from the <domain> tag")
                       .arg(getEntryUUID());
        return false;
    }

    QString nodeKey = domain.attribute("nodeKey");

    qlonglong aminterface;
    QString aminterfacestring = domain.attribute("actionmanagerinterface");
    if(aminterfacestring.isEmpty()) {
        aminterface = -1;
    }else{
        aminterface = aminterfacestring.toLongLong();
    }

    QString memory = domain.firstChildElement("memory").text();
    if(memory.isEmpty()) {
        qCritical() << tr("Entry %1: The \"memory\" tag is empty or missing from the <domain> tag")
                       .arg(getEntryUUID());
        return false;
    }
    QString vcpu = domain.firstChildElement("vcpu").text();
    if(vcpu.isEmpty()) {
        qCritical() << tr("Entry %1: The \"vcpu\" tag is empty or missing from the <domain> tag")
                       .arg(getEntryUUID());
        return false;
    }

    QString usbRedirectionCount = domain.firstChildElement("usbRedirectionCount").text();
    if(!usbRedirectionCount.isEmpty())
        domainEntry->usbRedirectionCount = usbRedirectionCount.toInt();

    domainEntry->usbControllerType = domain.firstChildElement("usbControllerType").text().toStdString();

    QDomElement osElement = domain.firstChildElement("os");
    QDomElement osTypeElement = osElement.firstChildElement("type");
    QString osType = osTypeElement.text();
    if(osType.isEmpty()) {
        qCritical() << tr("Entry %1: The \"os/type\" tag is empty or missing from the <domain> tag")
                       .arg(getEntryUUID());
        return false;
    }

    QString osArch = osTypeElement.attribute("arch");
    QString osMachine = osTypeElement.attribute("machine");

    QString osKernel = osElement.firstChildElement("kernel").text();
    QString osInitrd = osElement.firstChildElement("initrd").text();
    QString osCmdline = osElement.firstChildElement("cmdline").text();

    bool pae = !domain.firstChildElement("features").firstChildElement("pae").isNull();
    bool acpi = !domain.firstChildElement("features").firstChildElement("acpi").isNull();
    bool apic = !domain.firstChildElement("features").firstChildElement("apic").isNull();
    bool immutable = !domain.firstChildElement("features").firstChildElement("immutable").isNull();
    bool sharedFolder = !domain.firstChildElement("features").firstChildElement("sharedFolder").isNull();

    domainEntry->type = type.toStdString();
    domainEntry->nodeKey = nodeKey.toStdString();
    domainEntry->aminterface = aminterface;
    if (aminterface != -1 && !QList<std::string>::fromStdList(domainEntry->restrictionList).contains("actionManager"))
        domainEntry->restrictionList.push_back("actionManager");
    domainEntry->memory = memory.toInt();
    domainEntry->vcpu = vcpu.toInt();
    domainEntry->os.type = osType.toStdString();
    domainEntry->os.arch = osArch.toStdString();
    domainEntry->os.machine = osMachine.toStdString();
    domainEntry->pae = pae;
    domainEntry->acpi = acpi;
    domainEntry->apic = apic;
    domainEntry->sharedFolder = sharedFolder;
    domainEntry->immutable = immutable;
    if (!osKernel.isEmpty())
        domainEntry->os.kernel = osKernel.toStdString();
    if (!osInitrd.isEmpty())
        domainEntry->os.initrd = osInitrd.toStdString();
    if(!osCmdline.isEmpty())
        domainEntry->os.cmdline= osCmdline.toStdString();
    // disks
    QList<Hns::Disk> diskList;
    QDomNodeList diskNodeList = domain.elementsByTagName("disk");
    for (uint i = 0; i < diskNodeList.length(); i++) {
        QDomElement diskElement = diskNodeList.item(i).toElement();
        Hns::Disk disk;

        QString diskDevice = diskElement.attribute("device");
        if(diskDevice.isEmpty()) {
            qCritical() << tr("Entry %1: The \"device\" attribute is empty or missing from the <disk> tag")
                           .arg(getEntryUUID());
            return false;
        }
        QString diskType = diskElement.attribute("type");
        if(diskType.isEmpty()) {
            qCritical() << tr("Entry %1: The \"type\" attribute is empty or missing from the <disk> tag")
                           .arg(getEntryUUID());
            return false;
        }
        QString diskSource = diskElement.firstChildElement("source").attribute("file");
        if(diskSource.isEmpty()) {
            qCritical() << tr("Entry %1: The \"file\" attribute is empty or missing from the <disk/source> tag")
                           .arg(getEntryUUID());
            return false;
        }
        QString diskTarget = diskElement.firstChildElement("target").attribute("dev");
        if(diskSource.isEmpty()) {
            qCritical() << tr("Entry %1: The \"dev\" attribute is empty or missing from the <disk/target> tag")
                           .arg(getEntryUUID());
            return false;
        }
        QString diskTargetBus = diskElement.firstChildElement("target").attribute("bus");
        if(diskTargetBus.isEmpty()) {
            qCritical() << tr("Entry %1: The \"bus\" attribute is empty or missing from the <disk/target> tag")
                           .arg(getEntryUUID());
            return false;
        }
        QString diskDriverName = diskElement.firstChildElement("driver").attribute("name");
        if(diskTargetBus.isEmpty()) {
            qCritical() << tr("Entry %1: The \"name\" attribute is empty or missing from the <disk/driver> tag")
                           .arg(getEntryUUID());
            return false;
        }
        QString diskDriverType = diskElement.firstChildElement("driver").attribute("type");
        if(diskTargetBus.isEmpty()) {
            qCritical() << tr("Entry %1: The \"type\" attribute is empty or missing from the <disk/driver> tag")
                           .arg(getEntryUUID());
            return false;
        }

        disk.device = diskDevice.toStdString();
        disk.type = diskType.toStdString();
        disk.source = diskSource.toStdString();
        disk.target = diskTarget.toStdString();
        disk.targetBus = diskTargetBus.toStdString();
        disk.driverName = diskDriverName.toStdString();
        disk.driverType = diskDriverType.toStdString();
        diskList.append(disk);
    }

    QDomNodeList filesystemNodeList = domain.elementsByTagName("filesystem");
    for (uint i = 0; i < filesystemNodeList.length(); i++) {
        QDomElement filesystemElement = filesystemNodeList.item(i).toElement();
        Hns::Disk disk;

        QString filesystemType = filesystemElement.attribute("type");
        if(filesystemType.isEmpty()) {
            qCritical() << tr("Entry %1: The \"type\" attribute is empty or missing from the <filesystem> tag")
                           .arg(getEntryUUID());
            return false;
        }
        QString filesystemSourceDir = filesystemElement.firstChildElement("source").attribute("dir");
        if(filesystemSourceDir.isEmpty()) {
            qCritical() << tr("Entry %1: The \"dir\" attribute is empty or missing from the <filesystem/source> tag")
                           .arg(getEntryUUID());
            return false;
        }
        QString filesystemTargetDir = filesystemElement.firstChildElement("target").attribute("dir");
        if(filesystemTargetDir.isEmpty()) {
            qCritical() << tr("Entry %1: The \"dir\" attribute is empty or missing from the <filesystem/target> tag")
                           .arg(getEntryUUID());
            return false;
        }

        disk.type = filesystemType.toStdString();
        disk.source = filesystemSourceDir.toStdString();
        disk.target = filesystemTargetDir.toStdString();
        diskList.append(disk);
    }

    domainEntry->disks = diskList.toStdList();

    // netcards
    QDomElement devicesNode = domain.elementsByTagName("devices").item(0).toElement();
    Ice::Long netcardAutoId = devicesNode.attribute("netcardAutoId", 0).toLongLong();
    domainEntry->netcardAutoId = netcardAutoId;

    QMap<Ice::Long, Hns::Netcard> netcardMap;
    QDomNodeList netcardNodeList = domain.elementsByTagName("interface");

    for (uint i = 0; i < netcardNodeList.length(); i++) {
        QDomElement netcardElement = netcardNodeList.item(i).toElement();
        Hns::Netcard netcard;
        QString netcardType = netcardElement.attribute("type");
        if(netcardType.isEmpty()) {
            qCritical() << tr("Entry %1: The \"type\" attribute is empty or missing from the <interface> tag")
                           .arg(getEntryUUID());
            return false;
        }
        qulonglong netcardId = netcardElement.attribute("id", QString::number(i)).toULongLong();
        QString netcardMacAddress = netcardElement.firstChildElement("mac").attribute("address");
        if(netcardMacAddress.isEmpty()) {
            qCritical() << tr("Entry %1: The \"address\" attribute is empty or missing from the <interface/mac> tag")
                           .arg(getEntryUUID());
            return false;
        }
        QString sourceAttribute;
        if (netcardType == "ethernet") {
            sourceAttribute = "dev";
        } else if (netcardType == "bridge") {
            sourceAttribute = "bridge";
        } else {
            qCritical() << tr("Entry %1: The \"type\" attribute from the <interface> tag should either be \"ethernet\" or \"bridge\".")
                           .arg(getEntryUUID());
            return false;
        }

        QString netcardSource = netcardElement.firstChildElement("source").attribute(sourceAttribute);
        if (netcardSource.isEmpty())
            netcardSource = QString("eth%1").arg(netcardId);
        QString netcardTarget = netcardElement.firstChildElement("target").attribute("dev");
        QString netcardModel = netcardElement.firstChildElement("model").attribute("type");
        QString netcardDescription = netcardElement.attribute("description");

        netcard.id = netcardId;
        netcard.type = netcardType.toStdString();
        netcard.macAddress = netcardMacAddress.toStdString();
        netcard.source = netcardSource.toStdString();
        netcard.target = netcardTarget.toStdString();
        netcard.model = netcardModel.toStdString();
        netcard.description = netcardDescription.toStdString();

        netcardMap.insert(netcardId, netcard);
    }

    domainEntry->netcards = netcardMap.toStdMap();

    QDomElement graphics = domain.firstChildElement("devices").firstChildElement("graphics");
    domainEntry->graphic.keymap = graphics.attribute("keymap").toStdString();

    QDomNodeList inputNodeList = domain.elementsByTagName("input");
    for (uint i = 0; i < inputNodeList.length(); i++) {
        QDomElement inputElement = inputNodeList.item(i).toElement();
        Hns::Input input;

        QString inputType = inputElement.attribute("type");
        if(inputType.isEmpty()) {
            qCritical() << tr("Entry %1: The \"type\" attribute is empty or missing from the <input> tag")
                           .arg(getEntryUUID());
            return false;
        }

        QString inputBus = inputElement.attribute("bus");
        if(inputBus.isEmpty()) {
            qCritical() << tr("Entry %1: The \"bus\" attribute is empty or missing from the <input> tag")
                           .arg(getEntryUUID());
            return false;
        }

        input.type = inputType.toStdString();
        input.bus = inputBus.toStdString();
        domainEntry->inputs.push_back(input);
    }

    QDomElement video = domain.firstChildElement("devices").firstChildElement("video");
    QDomElement videoModel = video.firstChildElement("model");
    QString videoModelType = videoModel.attribute("type");
    domainEntry->vid.videoModelType = videoModelType.toStdString();

    QDomNodeList domainSnapshotsNodeList = root.elementsByTagName("domainsnapshots");
    if(domainSnapshotsNodeList.size() > 1) {
        qCritical() << tr("Entry %1: There should be no more than one <domainsnapshots> tag in the configuration file")
                       .arg(getEntryUUID());
        return false;
    }

    QDomElement domainSnapshots = domainSnapshotsNodeList.item(0).toElement();
    QString currentSnapshot = domainSnapshots.attribute("current");
    domainEntry->currentSnapshotName = currentSnapshot.toStdString();

    QDomNodeList domainSnapshotNodeList = domainSnapshots.elementsByTagName("domainsnapshot");
    for (uint i = 0; i < domainSnapshotNodeList.length(); i++) {
        QDomElement domainSnapshotElement = domainSnapshotNodeList.item(i).toElement();
        Hns::Snapshot snapshot;
        QString name = domainSnapshotElement.firstChildElement("name").text();
        QString description = domainSnapshotElement.firstChildElement("description").text();
        QString creationDateTime = domainSnapshotElement.firstChildElement("creationDateTime").text();
        QString parent = domainSnapshotElement.firstChildElement("parent").text();

        QDomNodeList disksElement = domainSnapshotElement.firstChildElement("disks").childNodes();
        QMap<std::string, std::string> disksMap; /*<name, file>*/
        for (int i = 0; i < disksElement.count(); ++i) {
            QString name = disksElement.item(i).toElement().attribute("name");
            QString file = disksElement.item(i).toElement().firstChildElement().attribute("file");
            disksMap.insert(name.toStdString(), file.toStdString());
        }
        QDomNodeList parentDisksElement = domainSnapshotElement.firstChildElement("parentDisks").childNodes();
        QMap<std::string, std::string> parentDisksMap; /*<name, file>*/
        for (int i = 0; i < parentDisksElement.count(); ++i) {
            QString name = parentDisksElement.item(i).toElement().attribute("name");
            QString file = parentDisksElement.item(i).toElement().firstChildElement().attribute("file");
            parentDisksMap.insert(name.toStdString(), file.toStdString());
        }

        snapshot.name = name.toStdString();
        snapshot.description = description.toStdString();
        snapshot.creationDateTime = creationDateTime.toStdString();
        snapshot.parent = parent.toStdString();
        snapshot.childDisks = disksMap.toStdMap();
        snapshot.parentDisks = parentDisksMap.toStdMap();

        domainEntry->snapshots.push_back(snapshot);
    }

    return true;
}

bool DomainEntrySerializer::_fillDoc()
{
    if (!EntityEntrySerializer::_fillDoc())
        return false;

    QDomElement domainElement = _doc.createElement("domain");

    DomainEntryPtr domainEntry = DomainEntryPtr::dynamicCast(_entry);

    if (!domainEntry) {
        qCritical() << tr("Entry %1: The entry is not a domain entry. Unable to fill the DOM Document")
                       .arg(getEntryUUID());
        return false;
    }

    // generic attributes
    domainElement.setAttribute("type", domainEntry->type.c_str());
    domainElement.setAttribute("actionmanagerinterface",  QString::number(domainEntry->aminterface));
    domainElement.setAttribute("nodeKey", QString::fromStdString(domainEntry->nodeKey));

    _addNode(domainElement, "vcpu", QString::number(domainEntry->vcpu));
    _addNode(domainElement, "memory", QString::number(domainEntry->memory));

    _addNode(domainElement,"usbRedirectionCount", QString::number(domainEntry->usbRedirectionCount));

    if (!domainEntry->usbControllerType.empty())
        _addNode(domainElement, "usbControllerType", QString::fromStdString(domainEntry->usbControllerType));

    QDomElement osElement = _addNode(domainElement, "os");
    QDomElement osTypeElement =_addNode(osElement, "type", QString::fromStdString(domainEntry->os.type));
    if (domainEntry->os.arch != "")
        osTypeElement.setAttribute("arch", QString::fromStdString(domainEntry->os.arch));
    if (!domainEntry->os.machine.empty())
        osTypeElement.setAttribute("machine", QString::fromStdString(domainEntry->os.machine));
    QDomElement featuresElement = _addNode(domainElement, "features");
    /*ARM Elements*/
    if (domainEntry->os.kernel !="")
        _addNode(osElement,"kernel",QString::fromStdString(domainEntry->os.kernel));
    if (domainEntry->os.initrd !="")
        _addNode(osElement,"initrd",QString::fromStdString(domainEntry->os.initrd));
    if (domainEntry->os.cmdline !="")
        _addNode(osElement,"cmdline",QString::fromStdString(domainEntry->os.cmdline));
    /*End of ARM*/
    if (domainEntry->pae)
        _addNode(featuresElement, "pae");
    if (domainEntry->apic)
        _addNode(featuresElement, "apic");
    if (domainEntry->acpi)
        _addNode(featuresElement, "acpi");
    if (domainEntry->sharedFolder)
        _addNode(featuresElement, "sharedFolder");
    if (domainEntry->immutable)
        _addNode(featuresElement, "immutable");

    QDomElement devicesElement = _addNode(domainElement, "devices");

    // disks
    QList<Hns::Disk> diskList = QList<Hns::Disk>::fromStdList(domainEntry->disks);
    foreach(Hns::Disk disk, diskList) {
        QDomElement diskElement = _addNode(
                    devicesElement,
                    (disk.type == "file" ? "disk" : "filesystem"));
        diskElement.setAttribute("type", QString::fromStdString(disk.type));

        if (disk.type == "file") {
            diskElement.setAttribute("device", QString::fromStdString(disk.device));
            QDomElement sourceElement = _addNode(diskElement, "source");
            sourceElement.setAttribute("file", QString::fromStdString(disk.source));
            QDomElement targetElement = _addNode(diskElement, "target");
            targetElement.setAttribute("bus", QString::fromStdString(disk.targetBus));
            targetElement.setAttribute("dev", QString::fromStdString(disk.target));
            QDomElement driverElement = _addNode(diskElement, "driver");
            driverElement.setAttribute("name", QString::fromStdString(disk.driverName));
            driverElement.setAttribute("type", QString::fromStdString(disk.driverType));
        } else {
            QDomElement sourceElement = _addNode(diskElement, "source");
            sourceElement.setAttribute("dir", QString::fromStdString(disk.source));
            QDomElement targetElement = _addNode(diskElement, "target");
            targetElement.setAttribute("dir", QString::fromStdString(disk.target));
        }
    }

    // netcards
    devicesElement.setAttribute("netcardAutoId", QString::number(domainEntry->netcardAutoId));

    QMap<Ice::Long, Hns::Netcard> netcardMap = QMap<Ice::Long, Hns::Netcard>(domainEntry->netcards);
    for (QMap<Ice::Long, Hns::Netcard>::const_iterator it = netcardMap.constBegin(); it != netcardMap.constEnd(); ++it) {
        Ice::Long id = it.key();
        Hns::Netcard netcard = it.value();
        QDomElement netcardElement = _addNode(devicesElement, "interface");
        netcardElement.setAttribute("type", QString::fromStdString(netcard.type));
        netcardElement.setAttribute("id", QString::number(id));
        QDomElement macElement = _addNode(netcardElement, "mac");
        macElement.setAttribute("address", QString::fromStdString(netcard.macAddress));
        if (netcard.source != "") {
            QDomElement sourceElement = _addNode(netcardElement, "source");
            QString sourceAttribute;
            if (netcard.type == "ethernet")
                sourceAttribute = "dev";
            else if (netcard.type == "bridge")
                sourceAttribute = "bridge";
            sourceElement.setAttribute(sourceAttribute, QString::fromStdString(netcard.source));
        }
        if (netcard.target != "") {
            QDomElement targetElement = _addNode(netcardElement, "target");
            targetElement.setAttribute("dev", QString::fromStdString(netcard.target));
        }
        if (netcard.model != "") {
            QDomElement modelElement = _addNode(netcardElement, "model");
            modelElement.setAttribute("type", QString::fromStdString(netcard.model));
        }
        netcardElement.setAttribute("description", QString::fromStdString(netcard.description));
    }

    if (domainEntry->graphic.keymap !="") {
        QDomElement graphics = _addNode(devicesElement, "graphics");
        graphics.setAttribute("keymap", QString::fromStdString(domainEntry->graphic.keymap));
    }

    QList<Hns::Input> inputList = QList<Hns::Input>::fromStdList(domainEntry->inputs);
    foreach (const Hns::Input &input, inputList) {
        QDomElement inputElement = _addNode(devicesElement, "input");
        inputElement.setAttribute("type", QString::fromStdString(input.type));
        inputElement.setAttribute("bus", QString::fromStdString(input.bus));
    }

    if (!QString::fromStdString(domainEntry->vid.videoModelType).isEmpty()) {
        QDomElement videoElement = _addNode(devicesElement, "video");
        QDomElement videoModelElement = _addNode(videoElement, "model");
        videoModelElement.setAttribute("type", QString::fromStdString(domainEntry->vid.videoModelType));
    }

    QDomElement domainsnapshotsElement = _doc.createElement("domainsnapshots");
    domainsnapshotsElement.setAttribute("current",  QString::fromStdString(domainEntry->currentSnapshotName));

    foreach (const Hns::Snapshot &snapshot, QList<Hns::Snapshot>::fromStdList(domainEntry->snapshots)) {
        QDomElement domainSnapshotElement = _addNode(domainsnapshotsElement, "domainsnapshot");
        _addNode(domainSnapshotElement, "name", QString::fromStdString(snapshot.name));
        _addNode(domainSnapshotElement, "description", QString::fromStdString(snapshot.description));
        _addNode(domainSnapshotElement, "creationDateTime", QString::fromStdString(snapshot.creationDateTime));
        _addNode(domainSnapshotElement, "parent", QString::fromStdString(snapshot.parent));
        QDomElement disksElement = _addNode(domainSnapshotElement, "disks");
        QMap<std::string, std::string>childDisksMap(snapshot.childDisks);
        foreach (std::string name, childDisksMap.keys()) {
            QDomElement diskElement = _addNode(disksElement, "disk");
            diskElement.setAttribute("name", QString::fromStdString(name));
            QDomElement sourceElement = _addNode(diskElement, "source");
            sourceElement.setAttribute("file", QString::fromStdString(childDisksMap.value(name)));
        }
        QDomElement parentDisksElement = _addNode(domainSnapshotElement, "parentDisks");
        QMap<std::string, std::string>parentDisksMap(snapshot.parentDisks);
        foreach (std::string name, parentDisksMap.keys()) {
            QDomElement diskElement = _addNode(parentDisksElement, "disk");
            diskElement.setAttribute("name", QString::fromStdString(name));
            QDomElement sourceElement = _addNode(diskElement, "source");
            sourceElement.setAttribute("file", QString::fromStdString(parentDisksMap.value(name)));
        }
    }

    QDomElement configElement = _doc.firstChildElement(_docType);
    QDomElement entityElement = configElement.firstChildElement("Entity");
    entityElement.appendChild(domainElement);
    entityElement.appendChild(domainsnapshotsElement);

    return true;
}

/*****************************************************************************/
/* SwitchEntrySerializer class                                               */
/*****************************************************************************/
SwitchEntrySerializer::SwitchEntrySerializer(const QDomDocument &doc)
    : EntityEntrySerializer(doc)
{
}

SwitchEntrySerializer::SwitchEntrySerializer()
{
}

void SwitchEntrySerializer::init()
{
    _entry = new SwitchEntry();
}

bool SwitchEntrySerializer::read()
{
    if (!EntityEntrySerializer::read())
        return false;
    QDomElement root = _doc.documentElement();
    QDomNodeList switchNodeList = root.elementsByTagName("switch");

    if(switchNodeList.size() != 1) {
        qCritical() << tr("Entry %1: There should be exactly one <switch> tag in the configuration file")
                       .arg(getEntryUUID());
        return false;
    }

    QDomElement switchElement = switchNodeList.item(0).toElement();
    SwitchEntryPtr switchEntry = SwitchEntryPtr::dynamicCast(_entry);

    // generic switch attributes
    QString type = switchElement.attribute("type", QString());
    if (type.isEmpty()) {
        qCritical() << tr("Entry %1: The \"type\" attribute value is not defined")
                       .arg(getEntryUUID());
        return false;
    } else if (type == "vde") {
        qWarning() << tr("Entry %1: Vde switches are not supported anymore, turning this switch into an OpenVSwitch switch")
                      .arg(getEntryUUID());
        type = "openVSwitch";
    }
    QString ports = switchElement.attribute("ports");
    if(ports.isEmpty()) {
        qCritical() << tr("Entry %1: The \"ports\" attribute value is not defined")
                       .arg(getEntryUUID());
        return false;
    }
    QString isHub = switchElement.attribute("isHub");
    if(isHub.isEmpty()) {
        qCritical() << tr("Entry %1: The \"isHub\" attribute value is not defined")
                       .arg(getEntryUUID());
        return false;
    }
    QString stpEnabled = switchElement.attribute("stpEnabled");
    if(stpEnabled.isEmpty()) {
        qCritical() << tr("Entry %1: The \"stpEnabled\" attribute value is not defined")
                       .arg(getEntryUUID());
        return false;
    }
    QString portMirroring = switchElement.attribute("portMirroring");
    if(portMirroring.isEmpty())
        switchEntry->portMirroring = -1;
    else
        switchEntry->portMirroring = portMirroring.toInt();

    switchEntry->type = type.toStdString();
    switchEntry->ports = ports.toInt();
    switchEntry->isHub = isHub.toInt();
    switchEntry->stpEnabled = stpEnabled.toInt();

    // vlans
    QMap<Ice::Int, Hns::Vlan> vlanList;
    QDomNodeList vlanNodeList = switchElement.elementsByTagName("vlan");
    for (uint i = 0; i < vlanNodeList.length(); i++) {
        QDomElement vlanElement = vlanNodeList.item(i).toElement();
        Hns::Vlan vlan;
        QString vlanTag = vlanElement.attribute("tag");
        if(vlanTag.isEmpty()) {
            qCritical() << tr("Entry %1: The \"tag\" attribute is empty or missing from the <vlan> tag")
                           .arg(getEntryUUID());
            return false;
        }

        vlan.tag = vlanTag.toInt();

        QString vlanName = vlanElement.attribute("name");
        if (vlanName.isEmpty())
            vlanName = "";

        vlan.name = vlanName.toStdString();

        QString vlanDescription = vlanElement.attribute("description");
        if (vlanDescription.isEmpty())
            vlanDescription = "";

        QDomNodeList portNodeList = vlanElement.elementsByTagName("port");
        for (uint j = 0; j < portNodeList.length(); j++) {
            QDomElement portElement = portNodeList.item(j).toElement();

            QString portId = portElement.attribute("id");
            if(portId.isEmpty()) {
                qCritical() << tr("Entry %1: The \"id\" attribute is empty or missing from the <port> tag")
                               .arg(getEntryUUID());
                return false;
            }
            QString portTagged = portElement.attribute("tagged");
            if(portTagged.isEmpty()) {
                qCritical() << tr("Entry %1: The \"tagged\" attribute is empty or missing from the <port> tag")
                               .arg(getEntryUUID());
                return false;
            }

            vlan.portsMap[portId.toInt() - 1] = static_cast<Hns::PortTag>(portTagged.toInt());
        }
        vlanList[vlan.tag] = vlan;
    }
    if (vlanList.isEmpty()) {
        Hns::Vlan vlan;
        vlan.tag = 0;
        for (int i = 0 ; i < ports.toInt() ; i++) {
            vlan.portsMap[i] = Untagged;
        }
        vlan.name = "default";
        vlan.description = "default";
        vlanList[vlan.tag] = vlan;
    }
    switchEntry->vlans = vlanList.toStdMap();

    return true;
}

bool SwitchEntrySerializer::_fillDoc()
{
    if (!EntityEntrySerializer::_fillDoc())
        return false;

    QDomElement switchElement = _doc.createElement("switch");

    SwitchEntryPtr switchEntry = SwitchEntryPtr::dynamicCast(_entry);

    if (!switchEntry) {
        qCritical() << tr("Entry %1: The entry is not a switch entry. Unable to fill the DOM Document")
                       .arg(getEntryUUID());
        return false;
    }

    // generic attributes
    switchElement.setAttribute("type", QString::fromStdString(switchEntry->type));
    switchElement.setAttribute("ports", switchEntry->ports);
    switchElement.setAttribute("isHub", switchEntry->isHub);
    switchElement.setAttribute("stpEnabled", switchEntry->stpEnabled);
    switchElement.setAttribute("portMirroring", switchEntry->portMirroring);

    // vlans
    QDomElement vlansElement = _addNode(switchElement, "vlans");
    QMap<Ice::Int, Hns::Vlan> vlanList = QMap<Ice::Int, Hns::Vlan>(switchEntry->vlans);
    QMapIterator<Ice::Int, Hns::Vlan> iterator(vlanList);
    while(iterator.hasNext()) {
        iterator.next();
        QDomElement vlanElement = _addNode(vlansElement, "vlan");
        vlanElement.setAttribute("tag", iterator.value().tag);
        vlanElement.setAttribute("name", QString::fromStdString(iterator.value().name));
        vlanElement.setAttribute("description", QString::fromStdString(iterator.value().description));

        QMap<Ice::Int, Hns::PortTag> portsMap(iterator.value().portsMap);
        QMapIterator<Ice::Int, Hns::PortTag> portsIt(portsMap);

        while (portsIt.hasNext()) {
            portsIt.next();
            QDomElement portElement = _addNode(vlanElement, "port");
            portElement.setAttribute("id", portsIt.key() + 1);
            portElement.setAttribute("tagged", portsIt.value());
        }
    }

    QDomElement configElement = _doc.firstChildElement(_docType);
    QDomElement entityElement = configElement.firstChildElement("Entity");
    entityElement.appendChild(switchElement);

    return true;
}

/*****************************************************************************/
/* HybridNetcardEntrySerializer class                                        */
/*****************************************************************************/

HybridNetcardEntrySerializer::HybridNetcardEntrySerializer(const QDomDocument &doc)
    : EntityEntrySerializer(doc)
{
}

HybridNetcardEntrySerializer::HybridNetcardEntrySerializer()
{
}

void HybridNetcardEntrySerializer::init()
{
    _entry = new HybridNetcardEntry();
}

bool HybridNetcardEntrySerializer::read()
{
    if (!EntityEntrySerializer::read())
        return false;
    QDomElement root = _doc.documentElement();
    QDomNodeList hybridNetcardNodeList = root.elementsByTagName("hybridnetcard");
    if(hybridNetcardNodeList.size() != 1) {
        qCritical() << tr("Entry %1: There should be exactly one <hybridnetcard> tag in the configuration file")
                       .arg(getEntryUUID());
        return false;
    }

    QDomElement hybridNetcardElement = hybridNetcardNodeList.item(0).toElement();
    HybridNetcardEntryPtr hybridNetcardEntry = HybridNetcardEntryPtr::dynamicCast(_entry);

    QString nodeName = hybridNetcardElement.attribute("nodeName");
    if(nodeName.isEmpty()) {
        qCritical() << tr("Entry %1: The \"nodeName\" attribute value is not defined")
                       .arg(getEntryUUID());
        return false;
    }

    QString plugIndex = hybridNetcardElement.attribute("plugIndex");
    if(plugIndex.isEmpty()) {
        qCritical() << tr("Entry %1: The \"plugIndex\" attribute value is not defined")
                       .arg(getEntryUUID());
        return false;
    }

    hybridNetcardEntry->nodeName = nodeName.toStdString();
    hybridNetcardEntry->plugIndex = plugIndex.toStdString();

    return true;
}

bool HybridNetcardEntrySerializer::_fillDoc()
{
    if (!EntityEntrySerializer::_fillDoc())
        return false;

    QDomElement hybridNetcardElement = _doc.createElement("hybridnetcard");
    HybridNetcardEntryPtr hybridNetcardEntry = HybridNetcardEntryPtr::dynamicCast(_entry);
    if (!hybridNetcardEntry) {
        qCritical() << tr("Entry %1: The entry is not an hybrid netcard entry. Unable to fill the DOM Document")
                       .arg(getEntryUUID());
        return false;
    }

    hybridNetcardElement.setAttribute("nodeName", QString::fromStdString(hybridNetcardEntry->nodeName));
    hybridNetcardElement.setAttribute("plugIndex", QString::fromStdString(hybridNetcardEntry->plugIndex));
    QDomElement configElement = _doc.firstChildElement(_docType);
    QDomElement entityElement = configElement.firstChildElement("Entity");
    entityElement.appendChild(hybridNetcardElement);
    return true;
}

/*****************************************************************************/
/* TopologyEntrySerializer class                                             */
/*****************************************************************************/
TopologyEntrySerializer::TopologyEntrySerializer(const QDomDocument &doc)
    : EntrySerializer(doc)
{
}

TopologyEntrySerializer::TopologyEntrySerializer()
{
}

void TopologyEntrySerializer::init()
{
    _entry = new TopologyEntry();
}

bool TopologyEntrySerializer::read()
{
    if (!EntrySerializer::read())
        return false;

    TopologyEntryPtr topologyEntry = TopologyEntryPtr::dynamicCast(_entry);
    QList<int> entityIds;
    QList<int> wireIds;

    QDomElement root = _doc.documentElement();

    QDomNodeList topologyNodeList = root.elementsByTagName("Topology");
    if(topologyNodeList.size() != 1) {
        qCritical() << tr("Entry %1: There should be exactly one <Topology> tag in the configuration file")
                       .arg(getEntryUUID());
        return false;
    }

    QDomElement topologyElement = topologyNodeList.item(0).toElement();
    topologyEntry->isModel = topologyElement.attribute("isModel", "0").toInt();
    topologyEntry->isPersistent = topologyElement.attribute("isPersistent", "0").toInt();

    EntityCatalogI* entityCatalog = MasterI::instance()->entityCatalog();

    QDomNodeList entitiesNodeList = topologyElement.elementsByTagName("Entities");
    if(entitiesNodeList.size() != 1) {
        qCritical() << tr("Entry %1: There should be exactly one <Entities> tag in the configuration file")
                       .arg(getEntryUUID());
        return false;
    }
    QDomNodeList entityNodeList = topologyElement.elementsByTagName("Entity");

    for (int e = 0 ; e < entityNodeList.size() ; e++) {

        QDomElement entityElement = entityNodeList.item(e).toElement();
        QString entityUuid = entityElement.attribute("UUID");

        int entityId = entityElement.attribute("id").toInt();
        entityIds.append(entityId);
        if (!entityId) {
            qCritical() << tr("Entry %1: Entity \"id\" attribute is missing")
                           .arg(getEntryUUID());
            return false;
        }

        EntityEntryPtr entityEntry;

        if (entityUuid.isEmpty()) { // entity is not in the catalog and is defined in a CDATA section
            QString entityXml = entityElement.text();
            if (entityXml.isEmpty() || !entityElement.firstChild().isCDATASection()) {
                qCritical() << tr("Unable to read CDATA section for the unidentified entry");
                return false;
            }
            QDomDocument entityDoc;
            entityDoc.setContent(entityXml);
            entityEntry = EntityEntryPtr::dynamicCast(EntrySerializer::readEntry(entityDoc, _entryUUID));
            if (!entityEntry) {
                qCritical() << tr("Entry %1: Unable to deserialize embedded entry in topology")
                               .arg(getEntryUUID());
                return false;
            }
            entityEntry->uuid = entityCatalog->generateUnusedUUID().toStdString();
            entityEntry->id = 0; //Workaround for rights to work
        } else {
            entityEntry = EntityEntryPtr::dynamicCast(entityCatalog->getEntryByUuid(entityUuid));
            if (!entityEntry) {
                qCritical() << tr("Entry %1: Unable to get entry from entity catalog").arg(entityUuid);
                topologyEntry->status = ::Hns::TOPOLOGY_LOADING_ERROR;
                continue;
            }
        }
        if (entityEntry->status == ::Hns::ENTITY_STATUS_UUID_CONFLICT) {
            topologyEntry->status = ::Hns::TOPOLOGY_CONTAINS_ENTITY_WITH_UUID_CONFLICT;
        }
        if (topologyEntry->entitiesMap.find(entityId) != topologyEntry->entitiesMap.end()) {
            qCritical() << tr("Entry %1: The topology already contains entity %2").arg(entityUuid, entityId);
            return false;
        }
        topologyEntry->entitiesMap[entityId]=entityEntry;
    }

    QDomNodeList wiresNodeList = topologyElement.elementsByTagName("Wires");
    if(wiresNodeList.size() != 1) {
        qCritical() << tr("Entry %1: There should be exactly one <Wires> tag in the configuration file")
                       .arg(getEntryUUID());
        return false;
    }
    QDomNodeList wireNodeList = topologyElement.elementsByTagName("Wire");
    for (int w = 0 ; w < wireNodeList.size() ; w++) {
        QDomElement wireElement = wireNodeList.item(w).toElement();

        int wireId = wireElement.attribute("id").toInt();
        if (!wireId) {
            qCritical() << tr("Entry %1: Wire \"id\" attribute is missing")
                           .arg(getEntryUUID());
            return false;
        }
        wireIds.append(wireId);

        if (_version == 1.0) {
            QString from = wireElement.attribute("from");
            if (from.isEmpty()) {
                qCritical() << tr("Entry %1: Wire \"from\" attribute is missing")
                               .arg(getEntryUUID());
                return false;
            }
            TopologyLink link;
            link.wireSide = QString("%1:0").arg(wireId).toStdString(); //from is 0
            link.entitySide = from.toStdString();
            topologyEntry->topologyLinks.push_back(link);

            QString to = wireElement.attribute("to");
            if (to.isEmpty()) {
                qCritical() << tr("Entry %1: Wire \"to\" attribute is missing")
                               .arg(getEntryUUID());
                return false;
            }
            link.wireSide = QString("%1:1").arg(wireId).toStdString(); //to is 1
            link.entitySide = to.toStdString();
            topologyEntry->topologyLinks.push_back(link);
        }

        QString wireUuid = wireElement.attribute("UUID");

        WireEntryPtr wireEntry = NULL;
        if (wireUuid.isEmpty()) {
            QString entityXml = wireElement.text();
            if (entityXml.isEmpty() || !wireElement.firstChild().isCDATASection()) {
                wireEntry = new WireEntry();
            } else {
                QDomDocument entityDoc;
                entityDoc.setContent(entityXml);
                wireEntry = WireEntryPtr::dynamicCast(WireEntrySerializer::readEntry(entityDoc, _entryUUID));
                if (!wireEntry) {
                    qCritical() << tr("Entry %1: Unable to deserialize embedded entry in topology")
                                   .arg(getEntryUUID());
                    wireEntry = new WireEntry();
                }
            }
        } else {
            wireEntry = WireEntryPtr::dynamicCast(entityCatalog->getEntryByUuid(wireUuid));
            if (!wireEntry) {
                qCritical() << tr("Entry %1: Unable to retrieve wire by UUID %2").arg(getEntryUUID(), wireUuid);
                return false;
            }
        }

        if (topologyEntry->wiresMap.find(wireId) != topologyEntry->wiresMap.end()) {
            qCritical() << tr("Entry %1: The topology already contains wire %2").arg(getEntryUUID(), wireId);
            return false;
        }
        topologyEntry->wiresMap[wireId]=wireEntry;
    }
    //TopologyLink v1.1
    if (_version >= 1.1) {
        QDomNodeList topologyLinksNodeList = topologyElement.elementsByTagName("TopologyLinks");
        if(topologyLinksNodeList.size() != 1) {
            qCritical() << tr("Entry %1: There should be exactly one <TopologyLinks> tag in the configuration file")
                           .arg(getEntryUUID());
            return false;
        }
        QDomNodeList topologyLinkList = topologyElement.elementsByTagName("TopologyLink");
        for (int t = 0, topologyLinkListSize = topologyLinkList.size() ; t < topologyLinkListSize ; t++) {
            QDomElement topologyLinkElement = topologyLinkList.item(t).toElement();
            QString entitySide = topologyLinkElement.attribute("entity");
            QString wireSide = topologyLinkElement.attribute("wire");
            if (wireSide.isEmpty()) {
                qCritical() << tr("Entry %1: TopologyLink \"wire\" attribute is missing")
                               .arg(getEntryUUID());
                return false;
            }

            int wireSideId = wireSide.split(":").first().toInt();
            if(!wireIds.contains(wireSideId)) {
                qCritical() << tr("Entry %1: Topology link \"wire\" isn't associated to an existing wire")
                             .arg(getEntryUUID());
                return false;
            }

            if(entitySide != "") {
                int entitySideId = entitySide.split(":").first().toInt();
                if(!entityIds.contains(entitySideId)) {
                    qCritical() << tr("Entry %1: Topology link \"entity\" isn't associated to an existing entity")
                                 .arg(getEntryUUID());
                    return false;
                }
            }

            TopologyLink link;
            link.entitySide = entitySide.toStdString();
            link.wireSide = wireSide.toStdString();

            topologyEntry->topologyLinks.push_back(link);
        }
    }

    QDomNodeList pcapInfosNodeList = topologyElement.elementsByTagName("PcapInfos");
    if (pcapInfosNodeList.size() > 1){
        qCritical() << tr("Entry %1: There should maximum one <PcapInfos> tag in the configuration file")
                       .arg(getEntryUUID());
        return false;
    }
    QMap<int,PcapInfoList> mapPcapInfos(topologyEntry->pcapInfoMap);
    QDomNodeList pcapInfoNodeList = topologyElement.elementsByTagName("PcapInfo");
    for (int p = 0; p < pcapInfoNodeList.size() ; p++){
        QDomElement pcapInfoElement = pcapInfoNodeList.item(p).toElement();
        QString entityIdString = pcapInfoElement.attribute("entityId");
        if (entityIdString.isEmpty()){
            qCritical()<<tr("Entry %1: PcapInfo \"entityId\" attribute is missing")
                         .arg(getEntryUUID());
            return false;
        }
        bool ok;
        int entityId = entityIdString.toInt(&ok);
        if (ok == false) {
            qCritical()<<tr("Entry %1: PcapInfo \"entityId\" should be an integer")
                         .arg(getEntryUUID());
            return false;
        }
        QString path = pcapInfoElement.attribute("path");
        if (path.isEmpty()){
            qCritical()<<tr("Entry %1: PcapInfo \"path\" attribute is missing")
                         .arg(getEntryUUID());
            continue;
        }
        int interfaces = pcapInfoElement.attribute("interfaces", "-1").toInt();
        if (interfaces == -1)
            qDebug() << tr("Entry %1: PcapInfo \"interfaces\" attribute is missing, setting to default value")
                        .arg(getEntryUUID());

        int timeLimit = pcapInfoElement.attribute("timeLimit", "-1").toInt();
        if (timeLimit == -1)
            qDebug() << tr("Entry %1: PcapInfo \"timeLimit\" attribute is missing, setting to default value")
                        .arg(getEntryUUID());

        int packetsLimit = pcapInfoElement.attribute("packetsLimit", "-1").toInt();
        if (packetsLimit == -1)
            qDebug() << tr("Entry %1: PcapInfo \"packetsLimit\" attribute is missing, setting to default value")
                        .arg(getEntryUUID());

        int packetsNumber = pcapInfoElement.attribute("packetsNumber", "-1").toInt();
        if (packetsNumber == -1)
            qDebug() << tr("Entry %1: PcapInfo \"packetsNumber\" attribute is missing, setting to default value")
                        .arg(getEntryUUID());

        long size = pcapInfoElement.attribute("size", "-1").toLong();
        if (size == -1)
            qDebug() << tr("Entry %1: PcapInfo \"size\" attribute is missing, setting to default value")
                        .arg(getEntryUUID());

        long startDateTime = pcapInfoElement.attribute("startDateTime", "-1").toLong();
        if (startDateTime == -1)
            qDebug() << tr("Entry %1: PcapInfo \"startDateTime\" attribute is missing, setting to default value")
                        .arg(getEntryUUID());

        long endDateTime = pcapInfoElement.attribute("endDateTime", "-1").toLong();
        if (endDateTime == -1)
            qDebug() << tr("Entry %1: PcapInfo \"endDateTime\" attribute is missing, setting to default value")
                        .arg(getEntryUUID());

        QString filters = pcapInfoElement.attribute("filters");

        PcapInfo pcapInfo;
        pcapInfo.path = path.toStdString();
        pcapInfo.filters = filters.toStdString();
        pcapInfo.interfaces = interfaces;
        pcapInfo.packetsNumber = packetsNumber;
        pcapInfo.packetsLimit = packetsLimit;
        pcapInfo.size = size;
        pcapInfo.timeLimit = timeLimit;
        pcapInfo.startDateTime = startDateTime;
        pcapInfo.endDateTime = endDateTime;

        if (mapPcapInfos.contains(entityId)){
            PcapInfoList pcapInfoList = mapPcapInfos.value(entityId);
            mapPcapInfos.remove(entityId);
            pcapInfoList.push_back(pcapInfo);
            mapPcapInfos.insert(entityId,pcapInfoList);
        } else {
            PcapInfoList pcapInfoList;
            pcapInfoList.push_back(pcapInfo);
            mapPcapInfos.insert(entityId,pcapInfoList);
        }

        topologyEntry->pcapInfoMap = mapPcapInfos.toStdMap();

    }

    QDomNodeList memoryDumpInfosNodeList = topologyElement.elementsByTagName("MemoryDumpInfos");
    if (memoryDumpInfosNodeList.size() > 1) {
        qCritical() << tr("Entry %1: There should maximum one <MemoryDumpInfos> tag in the configuration file")
                       .arg(getEntryUUID());
        return false;
    }
    QMap<int,MemoryDumpInfoList> memoryDumpInfosMap(topologyEntry->memoryDumpInfoMap);
    QDomNodeList memoryDumpInfoNodeList = topologyElement.elementsByTagName("MemoryDumpInfo");
    for (int i = 0 ; i < memoryDumpInfoNodeList.size() ; ++i) {
        QDomElement memoryDumpInfoElement = memoryDumpInfoNodeList.item(i).toElement();
        QString entityIdString = memoryDumpInfoElement.attribute("entityId");
        if (entityIdString.isEmpty()) {
            qCritical() << tr("Entry %1: MemoryDumpInfo \"entityId\" attribute is missing")
                           .arg(getEntryUUID());
            return false;
        }
        bool ok;
        int entityId = entityIdString.toInt(&ok);
        if (ok == false) {
            qCritical() << tr("Entry %1: MemoryDumpInfo \"entityId\" should be an integer")
                           .arg(getEntryUUID());
            continue;
        }
        QString fileName = memoryDumpInfoElement.attribute("fileName");
        if (fileName.isEmpty()) {
            qCritical() << tr("Entry %1: MemoryDumpInfo \"fileName\" attribute is missing")
                           .arg(getEntryUUID());
            continue;
        }
        QString name = memoryDumpInfoElement.attribute("name");
        if (name.isEmpty()) {
            qCritical() << tr("Entry %1: MemoryDumpInfo \"name\" attribute is missing")
                           .arg(getEntryUUID());
            continue;
        }
        long dateTime = memoryDumpInfoElement.attribute("dateTime", "-1").toLong();
        if (dateTime == -1)
            qDebug() << tr("Entry %1: MemoryDumpInfo \"dateTime\" attribute is missing, setting to default value")
                        .arg(getEntryUUID());
        long size= memoryDumpInfoElement.attribute("size", "-1").toLong();
        if (size == -1)
            qDebug() << tr("Entry %1: MemoryDumpInfo \"size\" attribute is missing, setting to default value")
                        .arg(getEntryUUID());


        MemoryDumpInfo memoryDumpInfo;
        memoryDumpInfo.fileName = fileName.toStdString();
        memoryDumpInfo.name = name.toStdString();
        memoryDumpInfo.dateTime = dateTime;
        memoryDumpInfo.size = size;

        if (memoryDumpInfosMap.contains(entityId)) {
            MemoryDumpInfoList memoryDumpInfoList = memoryDumpInfosMap.take(entityId);
            memoryDumpInfoList.push_back(memoryDumpInfo);
            memoryDumpInfosMap.insert(entityId, memoryDumpInfoList);
        } else {
            MemoryDumpInfoList memoryDumpInfoList;
            memoryDumpInfoList.push_back(memoryDumpInfo);
            memoryDumpInfosMap.insert(entityId, memoryDumpInfoList);
        }
        topologyEntry->memoryDumpInfoMap = memoryDumpInfosMap.toStdMap();
    }

    return true;
}

bool TopologyEntrySerializer::_fillDoc()
{
    if (!EntrySerializer::_fillDoc())
        return false;

    TopologyEntryPtr topologyEntry = TopologyEntryPtr::dynamicCast(_entry);

    if (!topologyEntry) {
        qCritical() << tr("Entry %1: The entry is not a topology entry. Unable to fill the DOM Document")
                       .arg(getEntryUUID());
        return false;
    }

    QDomElement topologyElement = _doc.createElement("Topology");
    topologyElement.setAttribute("isModel", topologyEntry->isModel);
    topologyElement.setAttribute("isPersistent", topologyEntry->isPersistent);

    // entities
    QDomElement entitiesElement = _addNode(topologyElement, "Entities");

    QMap<Ice::Int, EntityEntryPtr> entitiesMap(topologyEntry->entitiesMap);
    QMapIterator<Ice::Int, EntityEntryPtr> entitiesIt(entitiesMap);
    while (entitiesIt.hasNext()) {
        entitiesIt.next();
        Ice::Int entityId = entitiesIt.key();
        EntityEntryPtr entry = entitiesIt.value();
        QDomElement entityElement = _addNode(entitiesElement, "Entity");
        if (entry->id != 0) { // entry->id is 0 if the entry is not in the catalog
            entityElement.setAttribute("UUID", QString::fromStdString(entry->uuid));
        } else {
            QString entityXml = EntrySerializer::writeEntryToString(entry, "HnsEntityConfig");
            QDomCDATASection cDataSection = _doc.createCDATASection(entityXml);
            entityElement.appendChild(cDataSection);
        }
        entityElement.setAttribute("id", entityId);
    }

    // wires
    QDomElement wiresElement = _addNode(topologyElement, "Wires");

    QMap<Ice::Int, WireEntryPtr> wiresMap(topologyEntry->wiresMap);
    QMapIterator<Ice::Int, WireEntryPtr> wiresIt(wiresMap);
    while (wiresIt.hasNext()) {
        wiresIt.next();
        Ice::Int wireId = wiresIt.key();
        EntityEntryPtr entry = wiresIt.value();
        QDomElement wireElement = _addNode(wiresElement, "Wire");
        wireElement.setAttribute("id", wireId);

        if (entry->id != 0) { // entry->id is 0 if the entry is not in the catalog
            wireElement.setAttribute("UUID", QString::fromStdString(entry->uuid));
        }
        else {
            QString entityXml = EntrySerializer::writeEntryToString(entry, "HnsEntityConfig");
            QDomCDATASection cDataSection = _doc.createCDATASection(entityXml);
            wireElement.appendChild(cDataSection);
        }
    }

    // links
    QDomElement linksElement = _addNode(topologyElement, "TopologyLinks");
    TopologyLinkList topologyLinkList(topologyEntry->topologyLinks);
    foreach (TopologyLink topologyLink, topologyLinkList) {
        QDomElement linkElement = _addNode(linksElement, "TopologyLink");
        linkElement.setAttribute("entity", QString::fromStdString(topologyLink.entitySide));
        linkElement.setAttribute("wire", QString::fromStdString(topologyLink.wireSide));
    }

    //pcapinfos
    QDomElement pcapInfosElement = _addNode(topologyElement, "PcapInfos");

    QMap<int, PcapInfoList> pcapInfosMap(topologyEntry->pcapInfoMap);
    QMapIterator<int,PcapInfoList> pcapInfosIt(pcapInfosMap);
    while(pcapInfosIt.hasNext()){
        pcapInfosIt.next();
        int entityId = pcapInfosIt.key();
        QList<PcapInfo> listPcapInfos = QList<PcapInfo>::fromStdList(pcapInfosIt.value());
        foreach (PcapInfo pcapInfo,listPcapInfos){
            QDomElement pcapInfoElement = _addNode(pcapInfosElement, "PcapInfo");
            pcapInfoElement.setAttribute("entityId",entityId);
            pcapInfoElement.setAttribute("path" , QString::fromStdString(pcapInfo.path));
            pcapInfoElement.setAttribute("interfaces", QString::number(pcapInfo.interfaces));
            pcapInfoElement.setAttribute("timeLimit", QString::number(pcapInfo.timeLimit));
            pcapInfoElement.setAttribute("packetsLimit", QString::number(pcapInfo.packetsLimit));
            pcapInfoElement.setAttribute("packetsNumber", QString::number(pcapInfo.packetsNumber));
            pcapInfoElement.setAttribute("size", QString::number(pcapInfo.size));
            pcapInfoElement.setAttribute("startDateTime", QString::number(pcapInfo.startDateTime));
            pcapInfoElement.setAttribute("endDateTime", QString::number(pcapInfo.endDateTime));
            pcapInfoElement.setAttribute("filters", QString::fromStdString(pcapInfo.filters));
        }

    }

    //memorydumpinfos
    QDomElement memoryDumpInfosElements = _addNode(topologyElement, "MemoryDumpInfos");
    QMap<int, MemoryDumpInfoList> memoryDumpInfosMap(topologyEntry->memoryDumpInfoMap);
    QMapIterator<int,MemoryDumpInfoList> memoryDumpInfosIt(memoryDumpInfosMap);
    while (memoryDumpInfosIt.hasNext()) {
        memoryDumpInfosIt.next();
        int entityId = memoryDumpInfosIt.key();
        QList<MemoryDumpInfo> memoryDumpInfosList = QList<MemoryDumpInfo>::fromStdList(memoryDumpInfosIt.value());
        foreach (MemoryDumpInfo memoryDumpInfo, memoryDumpInfosList) {
            QDomElement memoryDumpInfoElement = _addNode(memoryDumpInfosElements, "MemoryDumpInfo");
            memoryDumpInfoElement.setAttribute("entityId", entityId);
            memoryDumpInfoElement.setAttribute("fileName", QString::fromStdString(memoryDumpInfo.fileName));
            memoryDumpInfoElement.setAttribute("name", QString::fromStdString(memoryDumpInfo.name));
            memoryDumpInfoElement.setAttribute("dateTime", QString::number(memoryDumpInfo.dateTime));
            memoryDumpInfoElement.setAttribute("size", QString::number(memoryDumpInfo.size));
        }
    }

    QDomElement configElement = _doc.firstChildElement(_docType);
    configElement.appendChild(topologyElement);

    return true;
}

/*****************************************************************************/
/* WifiEntryAPSerializer class                                               */
/*****************************************************************************/

WifiAPEntrySerializer::WifiAPEntrySerializer(const QDomDocument &doc)
    : EntityEntrySerializer(doc)
{
}

WifiAPEntrySerializer::WifiAPEntrySerializer()
{
}

void WifiAPEntrySerializer::init()
{
    _entry = new WifiAPEntry();
}

bool WifiAPEntrySerializer::read()
{
    if (!EntityEntrySerializer::read())
        return false;
    QDomElement root = _doc.documentElement();
    QDomNodeList wifiAPNodeList = root.elementsByTagName("wifiap");
    if(wifiAPNodeList.size() != 1) {
        qCritical() << tr("Entry %1: There should be exactly one <wifiap> tag in the configuration file")
                       .arg(getEntryUUID());
        return false;
    }

    QDomElement wifiAPElement = wifiAPNodeList.item(0).toElement();
    WifiAPEntryPtr wifiAPEntry = WifiAPEntryPtr::dynamicCast(_entry);

    QString ssid = wifiAPElement.attribute("ssid");
    if(ssid.isEmpty()) {
        qCritical() << tr("Entry %1: The \"ssid\" attribute value is not defined")
                       .arg(getEntryUUID());
        return false;
    }

    QString macAdress = wifiAPElement.attribute("macAdress");
    if(macAdress.isEmpty()) {
        qCritical() << tr("Entry %1: The \"macAdress\" attribute value is not defined")
                       .arg(getEntryUUID());
        return false;
    }

    QString channel = wifiAPElement.attribute("channel");
    if(channel.isEmpty()) {
        qDebug() << tr("Entry %1: The \"channel\" attribute value is not defined, default : 6")
                    .arg(getEntryUUID());
        channel = QString::number(6);
    }

    ::Hns::AuthTypeEnum authType;
    QString authTypeString = wifiAPElement.attribute("authType");
    if (authTypeString.isEmpty()){
        qCritical() << tr("Entry %1: The \"authType\" attribute value is not defined")
                       .arg(getEntryUUID());
        return false;
    }
    authType = _hashAuthTypeString()->key(authTypeString);

    QString key = wifiAPElement.attribute("key");
    if(key.isEmpty() && authType != ::Hns::NONE){
        qCritical() << tr("Entry %1: The \"key\" attribute value should be defined if there is a \"authType\" different of NONE")
                       .arg(getEntryUUID());
        return false;
    }

    QString hiddenString = wifiAPElement.attribute("hidden");
    if (hiddenString.isEmpty()){
        qCritical() << tr("Entry %1: The \"hidden\" attribute value is not defined")
                       .arg(getEntryUUID());
        return false;
    }
    bool hidden = QVariant(hiddenString).toBool();
    wifiAPEntry->macAdress = macAdress.toStdString();
    wifiAPEntry->ssid = ssid.toStdString();
    wifiAPEntry->authType = authType;
    wifiAPEntry->key = key.toStdString();
    wifiAPEntry->hidden = hidden;
    wifiAPEntry->channel = channel.toInt();
    return true;
}

bool WifiAPEntrySerializer::_fillDoc()
{
    if (!EntityEntrySerializer::_fillDoc())
        return false;

    QDomElement wifiAPElement = _doc.createElement("wifiap");
    WifiAPEntryPtr wifiAPEntry = WifiAPEntryPtr::dynamicCast(_entry);
    if (!wifiAPEntry) {
        qCritical() << tr("Entry %1: The entry is not a wifiap entry. Unable to fill the DOM Document")
                       .arg(getEntryUUID());
        return false;
    }
    wifiAPElement.setAttribute("macAdress",QString::fromStdString(wifiAPEntry->macAdress));
    wifiAPElement.setAttribute("ssid",QString::fromStdString(wifiAPEntry->ssid));
    wifiAPElement.setAttribute("channel", QString::number(wifiAPEntry->channel));
    wifiAPElement.setAttribute("authType",_hashAuthTypeString()->value(wifiAPEntry->authType));
    wifiAPElement.setAttribute("key",QString::fromStdString(wifiAPEntry->key));
    wifiAPElement.setAttribute("hidden",wifiAPEntry->hidden);
    QDomElement configElement = _doc.firstChildElement(_docType);
    QDomElement entityElement = configElement.firstChildElement("Entity");
    entityElement.appendChild(wifiAPElement);
    return true;
}

DynamipsRouterEntrySerializer::DynamipsRouterEntrySerializer(const QDomDocument &doc)
    : EntityEntrySerializer(doc)
{
}

DynamipsRouterEntrySerializer::DynamipsRouterEntrySerializer()
{
}

void DynamipsRouterEntrySerializer::init()
{
    _entry = new DynamipsRouterEntry();
}

bool DynamipsRouterEntrySerializer::read()
{
    if (!EntityEntrySerializer::read())
        return false;

    QDomElement root = _doc.documentElement();
    QDomNodeList dynamipsrouterNodeList = root.elementsByTagName("dynamipsrouter");
    if(dynamipsrouterNodeList.size() != 1) {
        qCritical() << tr("Entry %1: There should be exactly one <dynamipsrouter> tag in the configuration file")
                       .arg(getEntryUUID());
        return false;
    }

    QDomElement dynamipsRouterElement = dynamipsrouterNodeList.item(0).toElement();
    DynamipsRouterEntryPtr dynamipsRouterEntry = DynamipsRouterEntryPtr::dynamicCast(_entry);
    dynamipsRouterEntry->hasResources = true;
    dynamipsRouterEntry->router = dynamipsRouterElement.attribute("router").toStdString();
    dynamipsRouterEntry->chassis = dynamipsRouterElement.attribute("chassis").toStdString();
    dynamipsRouterEntry->macAddress = dynamipsRouterElement.attribute("macAddress").toStdString();
    dynamipsRouterEntry->idlePc = dynamipsRouterElement.attribute("idlepc").toStdString();
    QDomNodeList imageDiskList = dynamipsRouterElement.elementsByTagName("imageDisk");
    if(imageDiskList.size() !=1){
        qCritical() << tr("Entry %1: There should be exactly one <imageDisk> tag in the configuration file")
                       .arg(getEntryUUID());
        return false;
    }

    QDomElement imageDiskElement = imageDiskList.item(0).toElement();
    dynamipsRouterEntry->imageDisk = imageDiskElement.attribute("file").toStdString();

    bool ramOk;
    dynamipsRouterEntry->ram = dynamipsRouterElement.attribute("ram").toInt(&ramOk);
    if (!ramOk) {
        qCritical() << tr("Entry %1: There is no value for ram in the configuration file")
                       .arg(getEntryUUID());
        return false;
    }

    QDomNodeList moduleRoot = dynamipsRouterElement.elementsByTagName("modules");
    if (!moduleRoot.isEmpty()) {
        QDomNodeList  listModules = moduleRoot.item(0).toElement().elementsByTagName("module");
        int idxModule = 0;
        int sizeModules = listModules.size();
        while (sizeModules != idxModule) {
            QDomElement moduleElement  = listModules.item(idxModule).toElement();
            DynamipsRouterModule moduleDynamips;
            bool ok;
            moduleDynamips.name = moduleElement.attribute("name").toStdString();
            int position = moduleElement.attribute("position").toInt(&ok);
            if (!ok) {
                qCritical() << QString(tr("Entry %1: Position of slot %2 must be specified (in %3)")
                                       .arg(getEntryUUID(), moduleElement.attribute("name")).arg(_entry->shortName.c_str()));
                return false;
            }

            dynamipsRouterEntry->modulesMap[position] = moduleDynamips;
            idxModule++;
        }
    }
    DynamipsRouterDBI::instance()->populate(_entry);
    QDomNodeList optionRoot = dynamipsRouterElement.elementsByTagName("options");
    if (!optionRoot.isEmpty()) {
        QDomNodeList  listOptions = optionRoot.item(0).toElement().elementsByTagName("option");
        int idxOption = 0;
        int sizeOptions = listOptions.size();
        while (sizeOptions != idxOption) {
            QDomElement optionElement  = listOptions.item(idxOption).toElement();
            QString key = optionElement.attribute("key");
            QString value = optionElement.attribute("value");
            if (!key.isEmpty() && !value.isEmpty()) {
                dynamipsRouterEntry->mapOptions[key.toStdString()]  = value.toStdString();
            }
            idxOption++;
        }
    }
    QDomNodeList nodeBaseConfigList = dynamipsRouterElement.elementsByTagName("configuration");
    if (nodeBaseConfigList.size() == 1) {
        QDomElement configurationElement = nodeBaseConfigList.item(0).toElement();
        dynamipsRouterEntry->applyOnBoot = (configurationElement.attribute("applyOnBoot", 0) == "1");
        dynamipsRouterEntry->autoExtract = (configurationElement.attribute("autoExtract", 0) == "1");
        dynamipsRouterEntry->configuration = configurationElement.attribute("config64", "").toStdString();
    }
    QString error;
    if (!DynamipsRouterDBI::instance()->isValid(_entry, error)) {
        error += tr(" (for entry \"%1\")").arg(_entry->shortName.c_str());
        qCritical() << error;
        return false;
    }

    return true;
}


bool DynamipsRouterEntrySerializer::_fillDoc()
{
    if (!EntityEntrySerializer::_fillDoc())
        return false;

    QDomElement dynamipsRouterElement = _doc.createElement("dynamipsrouter");
    DynamipsRouterEntryPtr dynamipsRouterEntry = DynamipsRouterEntryPtr::dynamicCast(_entry);
    if(!dynamipsRouterEntry){
        qCritical() << tr("Entry %1: The entry is not a dynamipsrouter entry. Unable to fill the DOM Document")
                       .arg(getEntryUUID());
        return false;
    }

    dynamipsRouterElement.setAttribute("macAddress",QString::fromStdString(dynamipsRouterEntry->macAddress));
    dynamipsRouterElement.setAttribute("chassis",QString::fromStdString(dynamipsRouterEntry->chassis));
    //save idle but can be change each time...
    dynamipsRouterElement.setAttribute("idlepc",QString::fromStdString(dynamipsRouterEntry->idlePc));
    dynamipsRouterElement.setAttribute("router",QString::fromStdString(dynamipsRouterEntry->router));
    dynamipsRouterElement.setAttribute("ram", QString::number(dynamipsRouterEntry->ram));
    QDomElement imageDiskElement = _addNode(dynamipsRouterElement, "imageDisk");
    imageDiskElement.setAttribute("file",QString::fromStdString(dynamipsRouterEntry->imageDisk));
    //add slots
    QDomElement modulesElement = _addNode(dynamipsRouterElement, "modules");
    QMap< ::Ice::Int, ::Hns::DynamipsRouterModule> modulesMap(dynamipsRouterEntry->modulesMap);
    foreach ( ::Ice::Int position, modulesMap.keys()) {
        QDomElement moduleElement = _addNode(modulesElement, "module");
        moduleElement.setAttribute("name",QString::fromStdString(modulesMap[position].name) );
        moduleElement.setAttribute("position", position);
    }
    //add options
    QDomElement optionsElement = _addNode(dynamipsRouterElement, "options");
    QMap<std::string, std::string> mapOption(dynamipsRouterEntry->mapOptions);
    foreach (std::string key , mapOption.keys()) {
        QDomElement optionElement = _addNode(optionsElement, "option");
        optionElement.setAttribute("key", QString::fromStdString(key));
        optionElement.setAttribute("value", QString::fromStdString(mapOption[key]));
    }
    QDomElement configurationElement = _addNode(dynamipsRouterElement, "configuration");
    //can not use QDomText if the first char is digit and QDomCDATASection (problem with the CDATA of model topology)
    configurationElement.setAttribute("config64",QString::fromStdString(dynamipsRouterEntry->configuration));
    QString applyOnBoot = "0";
    if (dynamipsRouterEntry->applyOnBoot)
        applyOnBoot = "1";
    configurationElement.setAttribute("applyOnBoot",applyOnBoot);
    QString autoExtract = "0";
    if (dynamipsRouterEntry->autoExtract)
        autoExtract = "1";
    configurationElement.setAttribute("autoExtract",autoExtract);
    QDomElement configElement = _doc.firstChildElement(_docType);
    QDomElement entityElement = configElement.firstChildElement("Entity");
    entityElement.appendChild(dynamipsRouterElement);
    return true;
}

WireEntrySerializer::WireEntrySerializer(const QDomDocument &doc)
    : EntityEntrySerializer(doc)
{
}

WireEntrySerializer::WireEntrySerializer()
{
}

void WireEntrySerializer::init()
{
    _entry = new WireEntry();
}

bool WireEntrySerializer::read()
{
    if (!EntityEntrySerializer::read())
        return false;

    QDomElement root = _doc.documentElement();
    QDomNodeList wireNodeList = root.elementsByTagName("wire");
    if(wireNodeList.size() != 1) {
        qCritical() << tr("Entry %1: There should be exactly one <wire> tag in the configuration file")
                       .arg(getEntryUUID());
        return false;
    }

    QDomElement wireElement = wireNodeList.item(0).toElement();
    WireEntryPtr wireEntry = WireEntryPtr::dynamicCast(_entry);
    QString type = wireElement.attribute("type", QString());
    if (type.isEmpty()) {
        qCritical() << tr("Entry %1: The \"type\" attribute value is not defined")
                       .arg(getEntryUUID());
        return false;
    } else if (type == "wireFilter") {
        qCritical() << tr("Entry %1: Wire WireFilter are not supported anymore")
                       .arg(getEntryUUID());
        return false;
    }

    wireEntry->type = type.toStdString();
    QDomNodeList rightToLeft = wireElement.elementsByTagName("RtoL");
    if(rightToLeft.size() !=1){
        qCritical() << tr("Entry %1: There should be exactly one <RtoL> tag in the configuration file")
                       .arg(getEntryUUID());
        return false;
    }

    QDomElement rightToLeftElement = rightToLeft.item(0).toElement();
    wireEntry->propertyRL.loss = rightToLeftElement.attribute("loss","0").toInt();
    wireEntry->propertyRL.delay = rightToLeftElement.attribute("delay","0").toInt();
    wireEntry->propertyRL.dup = rightToLeftElement.attribute("dup","0").toInt();
    wireEntry->propertyRL.bandwidth = rightToLeftElement.attribute("bandwith","0").toInt();
    wireEntry->propertyRL.mtu = rightToLeftElement.attribute("mtu","0").toInt();
    QDomNodeList leftToRight = wireElement.elementsByTagName("LtoR");
    if(leftToRight.size() !=1){
        qCritical() << tr("Entry %1: There should be exactly one <LtoR> tag in the configuration file")
                       .arg(getEntryUUID());
        return false;
    }

    QDomElement leftToRightElement = leftToRight.item(0).toElement();
    wireEntry->propertyLR.loss = leftToRightElement.attribute("loss","0").toInt();
    wireEntry->propertyLR.delay = leftToRightElement.attribute("delay","0").toInt();
    wireEntry->propertyLR.dup = leftToRightElement.attribute("dup","0").toInt();
    wireEntry->propertyLR.bandwidth = leftToRightElement.attribute("bandwith","0").toInt();
    wireEntry->propertyLR.mtu = leftToRightElement.attribute("mtu","0").toInt();
    return true;
}

bool WireEntrySerializer::_fillDoc()
{
    if (!EntityEntrySerializer::_fillDoc())
        return false;

    QDomElement wireElement = _doc.createElement("wire");
    WireEntryPtr wireEntry = WireEntryPtr::dynamicCast(_entry);
    if(!wireEntry){
        qCritical() << tr("Entry %1: The entry is not a wire entry. Unable to fill the DOM Document")
                       .arg(getEntryUUID());
        return false;
    }

    wireElement.setAttribute("type", QString::fromStdString(wireEntry->type));
    QDomElement leftToRightElement = _addNode(wireElement, "LtoR");
    leftToRightElement.setAttribute("delay", wireEntry->propertyLR.delay);
    leftToRightElement.setAttribute("loss", wireEntry->propertyLR.loss);
    leftToRightElement.setAttribute("dup", wireEntry->propertyLR.dup);
    leftToRightElement.setAttribute("bandwith", wireEntry->propertyLR.bandwidth);
    leftToRightElement.setAttribute("mtu", wireEntry->propertyLR.mtu);
    QDomElement rightToLeftElement = _addNode(wireElement, "RtoL");
    rightToLeftElement.setAttribute("delay", wireEntry->propertyRL.delay);
    rightToLeftElement.setAttribute("loss", wireEntry->propertyRL.loss);
    rightToLeftElement.setAttribute("dup", wireEntry->propertyRL.dup);
    rightToLeftElement.setAttribute("bandwith", wireEntry->propertyRL.bandwidth);
    rightToLeftElement.setAttribute("mtu", wireEntry->propertyRL.mtu);
    QDomElement configElement = _doc.firstChildElement(_docType);
    QDomElement entityElement = configElement.firstChildElement("Entity");
    entityElement.appendChild(wireElement);
    return true;
}

/*****************************************************************************/
/* TopologyGateEntrySerializer class                                        */
/*****************************************************************************/

TopologyGateEntrySerializer::TopologyGateEntrySerializer(const QDomDocument &doc)
    : EntityEntrySerializer(doc)
{
}

TopologyGateEntrySerializer::TopologyGateEntrySerializer()
{
}

void TopologyGateEntrySerializer::init()
{
    _entry = new TopologyGateEntry();
}

bool TopologyGateEntrySerializer::read()
{
    if (!EntityEntrySerializer::read())
        return false;
    QDomElement root = _doc.documentElement();
    QDomNodeList topologyGateNodeList = root.elementsByTagName("topologygate");
    if(topologyGateNodeList.size() != 1) {
        qCritical() << tr("Entry %1: There should be exactly one <topologygate> tag in the configuration file")
                       .arg(getEntryUUID());
        return false;
    }

    QDomElement topologyGateElement = topologyGateNodeList.item(0).toElement();
    TopologyGateEntryPtr topologyGateEntry = TopologyGateEntryPtr::dynamicCast(_entry);

    QString networkName = topologyGateElement.attribute("networkName");
    if (networkName.isEmpty()) {
        qCritical() << tr("Entry %1: The \"networkName\" attribute value is not defined")
                       .arg(getEntryUUID());
        return false;
    }

    topologyGateEntry->networkName = networkName.toStdString();

    return true;
}

bool TopologyGateEntrySerializer::_fillDoc()
{
    if (!EntityEntrySerializer::_fillDoc())
        return false;

    QDomElement topologyGateElement = _doc.createElement("topologygate");
    TopologyGateEntryPtr topologyGateEntry = TopologyGateEntryPtr::dynamicCast(_entry);

    if (!topologyGateEntry) {
        qCritical() << tr("Entry %1: The entry is not an topology bridge entry. Unable to fill the DOM Document")
                       .arg(getEntryUUID());
        return false;
    }

    topologyGateElement.setAttribute("networkName", QString::fromStdString(topologyGateEntry->networkName));

    QDomElement configElement = _doc.firstChildElement(_docType);
    QDomElement entityElement = configElement.firstChildElement("Entity");
    entityElement.appendChild(topologyGateElement);

    return true;
}

} // namespace Hns
