/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of HYNESIM.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "archiveimporter.h"
#include "catalog/entitycatalogI.h"
#include "entryserializer.h"
#include "catalog/topologycatalogI.h"
#include "masterI.h"
#include "libarchive/libarchivehandler.h"
#include "macaddressmanagerI.h"
#include "customresourcemanagerI.h"

// hynesim libhnsserver headers
#include "libhnsserver/processrunner.h"
#include "libhnsserver/storagemanager.h"

// hynesim common headers
#include "common/constants.h"
#include "common/initializer.h"
#include "common/rightslist.h"
#include "common/domainentry.h"

// Generated hynesim headers
#include "common/topologyentry.h"
#include "common/dynamipsrouterentry.h"

// diamesh headers
#include <diamesh/utils.h>
#include <diamesh/accessrightshelper.h>
#include <diamesh/aclmanagerI.h>
#include <diamesh/diamesh.h>

// Qt headers
#include <QDir>
#include <QDirIterator>
#include <QFile>
#include <QProcess>
#include <QDebug>
#include <QDateTime>
#include <QSettings>

namespace Hns {

void _finishImportModelDynamipsRouter(const::Hns::EntityEntryPtr &,const QString &, const ImportOptions &);
void _finishImportModelDomain(const ::Hns::EntityEntryPtr &, const QString &, const ImportOptions &);

typedef void (*FinishImportModel)(const::Hns::EntityEntryPtr &,const QString &, const ImportOptions &);

HNS_STATIC_HASH_INITIALIZER(_finishImportModelHash, FinishImportModelHash, QString, FinishImportModel)
FinishImportModelHash::FinishImportModelHash()
{
    insert(QString::fromStdString(DynamipsRouterEntry::ice_staticId()), &_finishImportModelDynamipsRouter);
    insert(QString::fromStdString(DomainEntry::ice_staticId()), &_finishImportModelDomain);
}

void _finishImportModelDynamipsRouter(const::Hns::EntityEntryPtr &entityEntry,const QString & folder, const ImportOptions &options)
{
    Q_UNUSED(options)
    Hns::DynamipsRouterEntryPtr dynamipsEntry =Hns::DynamipsRouterEntryPtr::dynamicCast(entityEntry);
    QStringList list;
    list << QString("*");
    QDirIterator dirIterator(folder, list ,QDir::Files | QDir::NoSymLinks);

    QStringList fileList;
    while(dirIterator.hasNext()){
        dirIterator.next();
        fileList << dirIterator.fileName();
    }

    QRegExp regExp("[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}");
    foreach(const QString &value, fileList){
        QString copy = value;
        copy.replace(regExp ,QString::fromStdString(dynamipsEntry->uuid)) ;
        if(copy != value)
            QFile().rename(QString("%1/%2").arg(folder).arg(value),QString("%1/%2").arg(folder).arg(copy) );
    }
}

void _finishImportModelDomain(const EntityEntryPtr &entityEntry, const QString &folder, const ImportOptions & options)
{
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(entityEntry);

    if (options.flags & ImportOptionsFlags::GENERATE_MAC_ADDRESSES) {
        QMap<Ice::Long, Hns::Netcard> netcards(domainEntry->netcards);
        QMapIterator<Ice::Long,Hns::Netcard> iterator(netcards);
        QMap<Ice::Long, Hns::Netcard> newNetcards;
        while (iterator.hasNext()) {
            iterator.next();
            Ice::Long id = iterator.key();
            Hns::Netcard netcard = iterator.value();
            netcard.macAddress = MacAddressManagerI::instance()->generateRandomAddress("");
            newNetcards.insert(id, netcard);
        }
        domainEntry->netcards = newNetcards.toStdMap();
    }

    if (domainEntry->type == "vbox") {
        QSettings settings(Diamesh::instance()->configFileName(), QSettings::IniFormat);
        settings.beginGroup(Constants::SETTINGS_HYNESIM_GROUP);
        QString changeDisksUUIDsScript = QString("%1/%2");
        changeDisksUUIDsScript = changeDisksUUIDsScript.arg(settings.value(Constants::SETTINGS_SCRIPTS_DIRECTORY).toString());
        changeDisksUUIDsScript = changeDisksUUIDsScript.arg(Constants::SETTINGS_VBOX_UUIDS_SCRIPT);

        QProcess process;
        QStringList args = QStringList() << folder;
        process.start(changeDisksUUIDsScript, args);
        if (!process.waitForStarted()) {
            qWarning() << QObject::tr("Failed to start %1 script").arg(changeDisksUUIDsScript);
            return;
        }
        if (!process.waitForFinished(-1)) {
            qWarning() << QObject::tr("%1 script did not finish correctly").arg(changeDisksUUIDsScript);
            return;
        }
    }
}

ArchiveImporter::ArchiveImporter(QString tStorage, QString eStorage, QString exportFolder, QObject *parent)
    : QObject(parent)
    , _topologyStorage(tStorage)
    , _entityStorage(eStorage)
    , _exportFolder(exportFolder)
{
    _compressedSuffixes.append("tar");
    _compressedSuffixes.append("tar.gz");
    _compressedSuffixes.append("tar.xz");
    _copyFlag = COPY;
    _isArchive = false;
}

ArchiveImporter::~ArchiveImporter()
{

}

bool ArchiveImporter::importArchive(ImportableArchivePtr archive, const Hns::ImportOptions &options, const Ice::Current &current)
{
    QList<FolderDescriptor> fdList = QList<FolderDescriptor>::fromStdList(archive->archive->folders);
    QList<EntityDescriptor> edList = QList<EntityDescriptor>::fromStdList(archive->archive->entities);
    QList<TopologyDescriptor> tdList = QList<TopologyDescriptor>::fromStdList(archive->archive->topologies);

    //Creation of a category "import - name - date"
    EntityCatalogI *catalog = MasterI::instance()->entityCatalog();
    FolderPtr folderImport = new Folder();
    QDateTime date;
    folderImport->shortName = QString("import - %1 - %2").arg(QString::fromStdString(options.newName)).arg(date.currentDateTime().toString("yyyyMMdd-hh:mm:ss")).toStdString();
    folderImport->longName = QObject::tr("Import category").toStdString();
    folderImport->description = QObject::tr("Category in which Imported entries are").toStdString();
    _folderImportId = catalog->addFolder(folderImport, current);

    QFileInfo fileInfo(QString::fromStdString(archive->path)); //Archive FileInfo
    //End of Creation of a category "import"

    //importFolderEntry();
    QList<qlonglong> foldersId = _importFolderEntryInArchive(fdList, current);
    foldersId.append(_folderImportId);

    //Processing Entries then topologies
    QList<qlonglong> topologiesId;
    QList<qlonglong> entriesId;
    if ((!fileInfo.isDir()) && (_compressedSuffixes.contains(fileInfo.completeSuffix()))) {
        _processCustomResourcesInCompressedArchive(QString::fromStdString(archive->path), archive->archive->customResources);
        entriesId = _processEntriesInCompressedArchive(QString::fromStdString(archive->path), edList, options, current);
        topologiesId = _processTopologiesInCompressedArchive(QString::fromStdString(archive->path), tdList, options, current);
    } else if (fileInfo.isDir()) {
        _processCustomResourcesInFolderArchive(QString::fromStdString(archive->path), archive->archive->customResources);
        entriesId = _processEntriesInFolderArchive(QString::fromStdString(archive->path), edList, options, current);
        topologiesId = _processTopologiesInFolderArchive(QString::fromStdString(archive->path), tdList, options, current);
    }
    _oldNewResourcesHash.clear();
    _applyACL(entriesId, topologiesId, foldersId, options);
    _rebaseEntities(archive->archive->baseEntities);
    return true;
}

qlonglong ArchiveImporter::importEntityEntry(const std::string &folder, const std::string &storage, const Ice::Current &iceCurrent)
{
    ImportOptions options;
    options.flags = 0;
    return _importEntityEntry(folder, storage, MOVE, options, iceCurrent);
}

bool ArchiveImporter::exportArchive(const ArchiveDescriptorPtr &archiveDescriptor, bool isArchive)
{
    _isArchive = isArchive;
    EntityDescriptorList entities = archiveDescriptor->entities;

    foreach (const EntityDescriptor &entityDescriptor, entities) {
        if (!_exportEntityEntry(entityDescriptor, _exportFolder, isArchive))
            return false;
        _exportCustomResources(QString::fromStdString(entityDescriptor.customGraphic), isArchive);
    }
    //before export topologies we have to check if all entities are exported
    //for now no need to implement
    TopologyDescriptorList topologies = archiveDescriptor->topologies;
    foreach (const TopologyDescriptor &topologyDescriptor, topologies) {
        if (!_exportTopology(topologyDescriptor))
            return false;
    }

    return true;
}

bool ArchiveImporter::exportEntityEntry(EntityDescriptor &descriptor)
{
    QSettings settings(MasterI::instance()->entityCatalog()->getSettingName(),QSettings::IniFormat);
    settings.beginGroup(Constants::SETTINGS_HYNESIM_GROUP);
    QString path = settings.value("exportCatalog/path").toString();
    settings.endGroup();
    return _exportEntityEntry(descriptor, path, false, true);
}

void ArchiveImporter::setCopyFlag(ArchiveImporter::CopyPolicy copyFlag)
{
    _copyFlag = copyFlag;
}

bool ArchiveImporter::_exportTopology(const TopologyDescriptor &topologyDescriptor)
{
    QString topologyDescriptorPath = QString::fromStdString(topologyDescriptor.path);
    QStringList stringList = topologyDescriptorPath.split("/");
    topologyDescriptorPath.replace(stringList[0], "");
    /**/
    QString srcPath = QString::fromStdString(MasterI::instance()->topologyCatalog()->getFullEntryPathFromMixed(topologyDescriptor.path, topologyDescriptor.uuid));

    QString dstPath = _exportFolder;
    QDir dir(dstPath);
    if (!dir.exists()) {
        qCritical() << QObject::tr("unable to find export folder ! operation aborted");
        return false;
    }
    if (_isArchive) {
        dir.cd(".tmp/");
        dstPath += "/.tmp";
    }
    dir.mkdir(QString::fromStdString(topologyDescriptor.uuid));
    _copyDirectory(srcPath, dstPath);

    QString topoPath = QString("%1/%2/hnsEntryConfig.xml").arg(dstPath).arg(topologyDescriptor.uuid.c_str());
    QFile file(topoPath);
    if (!file.open(QIODevice::ReadOnly)) {
        qCritical() << QObject::tr("Unable to open file %1 for reading").arg(file.fileName());
        return false;
    }

    QString content(file.readAll());
    QDomDocument doc;
    doc.setContent(content);
    QDomElement elem = doc.elementsByTagName("Header").item(0).toElement();
    file.close();

    if (!(elem.attribute("shortName") == QString::fromStdString(topologyDescriptor.shortName))) {
        if (!file.open(QIODevice::WriteOnly | QFile::Text)) {
            qCritical() << QObject::tr("Unable to open file %1 for writing").arg(file.fileName());
            return false;
        }
        elem.setAttribute("shortName", QString::fromStdString(topologyDescriptor.shortName));
        QTextStream out(&file);
        const int xmlIndentSize = 2;
        doc.save(out, xmlIndentSize);
        file.close();
    }

    return true;
}

QList<qlonglong> ArchiveImporter::_importFolderEntryInArchive(QList<FolderDescriptor> fdList, const ::Ice::Current &current)
{
    QList<qlonglong> ret;
    EntityCatalogI *catalog = MasterI::instance()->entityCatalog();

    QMutableListIterator<FolderDescriptor> iterator(fdList);
    while (iterator.hasNext()) {
        FolderDescriptor& fd =  iterator.next();
        FolderPtr folder = new Folder();
        folder->shortName = fd.shortName;
        folder->longName = fd.longName;
        folder->description = fd.description;
        folder->tagList = fd.tags;
        if (fd.parentId == 0) {
            folder->parentId = _folderImportId;
            qlonglong newId = catalog->addFolder(folder, current);
            if (newId != 0)
                ret.append(newId);
            _folderIDHash.insert(fd.id, newId);
            iterator.remove();
        } else if (_folderIDHash.contains(fd.parentId)) {
            folder->parentId = _folderIDHash.value(fd.parentId);
            qlonglong newId = catalog->addFolder(folder, current);
            if (newId != 0)
                ret.append(newId);
            _folderIDHash.insert(fd.id, newId);
            iterator.remove();
        }
        if (!iterator.hasNext() && !fdList.isEmpty()) {
            iterator.toFront();
            continue;
        }
    }
    return ret;
}

qlonglong ArchiveImporter::_importEntityEntryInArchive(EntityDescriptor entityDescriptor, const Hns::ImportOptions &options, const Ice::Current &current)
{
    qlonglong id = _importEntityEntry(entityDescriptor.path, _entityStorage.toStdString(), _copyFlag, options, current);
    EntityCatalogI *catalog = MasterI::instance()->entityCatalog();
    _entityUUIDHash.insert(QString::fromStdString(entityDescriptor.uuid),QString::fromStdString(catalog->getEntityEntry(id)->uuid));
    EntityEntryPtr entityEntryPtr = catalog->getEntityEntry(id);
    if (entityDescriptor.parentId != 0)
        entityEntryPtr->parentId = _folderIDHash.value(entityDescriptor.parentId, 0);
    else
        entityEntryPtr->parentId = _folderImportId;
    if (!entityEntryPtr->customGraphic.empty()) {
        entityEntryPtr->customGraphic = _oldNewResourcesHash.value(QString::fromStdString(entityEntryPtr->customGraphic)).toStdString();
    }
    catalog->updateEntry(entityEntryPtr,Ice::Current());
    return id;
}

qlonglong ArchiveImporter::_importTopologyInArchive(TopologyDescriptor topologyDescriptor, QString newName, const Ice::Current &current)
{
    QString path  = _topologyStorage+QString("/import");
    QDir dir(path);
    if (!dir.mkpath(dir.absolutePath())) {
        qCritical() << QObject::tr("Unable to create directory %1").arg(path);
        return 0;
    }

    QSettings settings(MasterI::instance()->entityCatalog()->getSettingName(),QSettings::IniFormat);
    settings.beginGroup(Constants::SETTINGS_HYNESIM_GROUP);
    QString pathTopology = settings.value((QString("%1Import/path").arg(MasterI::instance()->entityCatalog()->idName().c_str()))).toString();
    settings.endGroup();
    QString uuid = MasterI::instance()->entityCatalog()->generateUnusedUUID();
    QString fromDirectory = (QString("%1/%2").arg(pathTopology,QString::fromStdString(topologyDescriptor.path)));
    QString toDirectory = dir.absolutePath().append("/").append(uuid);

    if (!Diamesh::copyDirectory(fromDirectory,toDirectory)) {
        qCritical() << QObject::tr("Unable to copy directory %1").arg(QString::fromStdString(topologyDescriptor.path));
        return 0;
    }
    if (_copyFlag == MOVE) {
        if (!Diamesh::removeDirectory(fromDirectory)) {
            qDebug() << QObject::tr("Unable to delete %1").arg(fromDirectory);
        }
    }
    QDir topologyDir(QString("%1/%2").arg(dir.absolutePath(),uuid));
    QFile file(topologyDir.absolutePath().append("/hnsEntryConfig.xml"));
    if (!file.open(QIODevice::ReadOnly)) {
        qCritical() << QObject::tr("Unable to open file %1 for reading").arg(file.fileName());
        return 0;
    }
    QString content(file.readAll());
    file.close();

    if (content.contains(QString::fromStdString(topologyDescriptor.uuid)))
        content.replace(QString::fromStdString(topologyDescriptor.uuid), uuid);

    foreach (QString oldUuid, _entityUUIDHash.keys()) {
        if (content.contains(oldUuid)) {
            content.replace(oldUuid,_entityUUIDHash.value(oldUuid));
        }
    }
    if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) {
        qCritical() << QObject::tr("Unable to open file %1 for writing").arg(file.fileName());
        return 0;
    }
    QTextStream out(&file);
    QDomDocument doc;
    doc.setContent(content);
    if (!(newName == QString::fromStdString(topologyDescriptor.shortName)))
        doc.elementsByTagName("Header").item(0).toElement().setAttribute("shortName", newName);

    // rename PCAP files
    QDomNodeList pcapInfoNodeList = doc.elementsByTagName("PcapInfo");
    for (int p = 0; p < pcapInfoNodeList.size() ; p++) {
        QDomElement pcapInfoElement = pcapInfoNodeList.item(p).toElement();
        QString path = pcapInfoElement.attribute("path");
        QStringList list = path.split("/");
        QString name = list.last();
        QString first = list.takeFirst();
        QString newPath = first + "/import/" + list.join("/");
        QString oldName = name;
        foreach (QString oldUuid, _entityUUIDHash.keys()) {
            QString newUuid = _entityUUIDHash.value(oldUuid);
            if (name.contains(newUuid)) {
                oldName.replace(newUuid, oldUuid);
                QString fileName = toDirectory + "/pcap/" + oldName;
                QFile file(fileName);
                if (!file.exists()) {
                    qCritical() << QObject::tr("Pcap file %1 not found").arg(fileName);
                    continue;
                }
                if (!file.rename(fileName, toDirectory + "/pcap/" + name)) {
                    qCritical() << QObject::tr("Unable to rename pcap file %1").arg(fileName);
                    continue;
                }
                pcapInfoElement.setAttribute("path", newPath);
            }
        }
    }

    QDomNodeList memoryDumpInfoNodeList = doc.elementsByTagName("MemoryDumpInfo");
    int listSize = memoryDumpInfoNodeList.size();
    for (int i = 0 ; i < listSize ; ++i) {
        QDomElement memoryDumpInfoElement = memoryDumpInfoNodeList.item(i).toElement();
        QString path = memoryDumpInfoElement.attribute("path");
        QStringList list = path.split("/");
        QString topologyCatalogName = list.takeFirst();
        QString newPath = topologyCatalogName + "/import/" + list.join("/");
        memoryDumpInfoElement.setAttribute("path", newPath);
    }


    const int xmlIndentSize = 2;
    doc.save(out, xmlIndentSize);
    file.close();

    EntrySerializer serializer;
    TopologyEntryPtr topologyPtr = TopologyEntryPtr::dynamicCast(serializer.readEntry(doc, QString::fromStdString(topologyDescriptor.uuid)));
    if (!topologyPtr)
        return 0;
    topologyPtr->uuid = uuid.toStdString();
    TopologyCatalogI *catalog = MasterI::instance()->topologyCatalog();
    topologyPtr->path = QString("%1:/%2").arg(QString::fromStdString(catalog->getFileStorages()[_topologyStorage.toStdString()]),QString("import")).toStdString();

    return catalog->addEntry(topologyPtr, current);
}

QList<qlonglong> ArchiveImporter::_processEntriesInCompressedArchive(QString archivePath, QList<EntityDescriptor> edList, const Hns::ImportOptions &options, const Ice::Current &current)
{
    QList<qlonglong> ret;
    QSettings settings(MasterI::instance()->entityCatalog()->getSettingName(),QSettings::IniFormat);
    settings.beginGroup(Constants::SETTINGS_HYNESIM_GROUP);
    QString importDirectory = settings.value((QString("%1Import/path").arg(MasterI::instance()->entityCatalog()->idName().c_str()))).toString();
    settings.endGroup();
    QDir extractionDirectory;
    extractionDirectory.setPath(QString("%1/%2").arg(importDirectory,QString(".tmp")));
    if (!extractionDirectory.mkpath(extractionDirectory.absolutePath())) {
        qCritical() << QObject::tr("Unable to create directory %1").arg(extractionDirectory.absolutePath());
        return ret;
    }
    LibArchiveHandler archiveHandler(archivePath);
    archiveHandler.list(); //This is needed for constructing a list of the directories in the archive
    QMutableListIterator<EntityDescriptor> edIt(edList);
    while (edIt.hasNext()) {
        EntityDescriptor& ed =  edIt.next();
        if (ed.fileType == FileType::COMPRESSED_FILE) {
            archiveHandler.extractOnDisk(QStringList(QString::fromStdString(ed.fileLocation)),extractionDirectory.absolutePath()); //We extract only the necessary file for importing the entry
            QString tempEntryFile = QString("%1/%2").arg(extractionDirectory.absolutePath(),QString::fromStdString(ed.fileLocation));
            LibArchiveHandler entryArchiveHandler(tempEntryFile);
            entryArchiveHandler.extractOnDisk(QStringList(),extractionDirectory.absolutePath(), true); //We extract all the files in the entry archive and so we don't need to call list() because we don't need to preconstruct a list of directories...
        } else {
            archiveHandler.extractOnDisk(QStringList(QString("%1/").arg(QString::fromStdString(ed.path))),extractionDirectory.absolutePath()); //We extract only the necessary file for importing the entry

        }
        QString edPath = QString::fromStdString(ed.path);
        edPath.prepend(".tmp/");
        ed.path = edPath.toStdString();
        qlonglong id = _importEntityEntryInArchive(ed, options, current);
        if (id != 0)
            ret.append(id);
    }
    return ret;
}

QList<qlonglong> ArchiveImporter::_processEntriesInFolderArchive(QString archivePath, QList<EntityDescriptor> edList, const Hns::ImportOptions &options, const Ice::Current &current)
{
    QList<qlonglong> ret;
    QSettings settings(MasterI::instance()->entityCatalog()->getSettingName(),QSettings::IniFormat);
    settings.beginGroup(Constants::SETTINGS_HYNESIM_GROUP);
    QString importDirectory = settings.value((QString("%1Import/path").arg(MasterI::instance()->entityCatalog()->idName().c_str()))).toString();
    settings.endGroup();
    QDir extractionDirectory;
    extractionDirectory.setPath(QString("%1/%2").arg(archivePath,QString(".tmp")));
    if (!extractionDirectory.mkpath(extractionDirectory.absolutePath())) {
        qCritical() << QObject::tr("Unable to create directory %1").arg(extractionDirectory.absolutePath());
        return ret;
    }
    QMutableListIterator<EntityDescriptor> edIt(edList);
    QString relativeArchivePath = archivePath;
    relativeArchivePath.remove(QDir(importDirectory).absolutePath());
    if (relativeArchivePath.startsWith(QString("/"))) {
        relativeArchivePath.remove(0,1);
    }
    while (edIt.hasNext()) {
        EntityDescriptor& ed =  edIt.next();
        QString edPath = QString::fromStdString(ed.path);
        if (ed.fileType == FileType::COMPRESSED_FILE) {
            QString tempEntryFile = QString("%1/%2").arg(archivePath,QString::fromStdString(ed.fileLocation));
            LibArchiveHandler entryArchiveHandler(tempEntryFile);
            entryArchiveHandler.extractOnDisk(QStringList(),extractionDirectory.absolutePath()); //We extract all the files in the entry archive and so we don't need to call list() because we don't need to preconstruct a list of directories...
            edPath.prepend(QString("%1/%2").arg(relativeArchivePath,QString(".tmp/")));
        } else {
            edPath.prepend(QString("%1/").arg(relativeArchivePath));
        }
        ed.path = edPath.toStdString();
        qlonglong id = _importEntityEntryInArchive(ed, options, current);
        if (id != 0)
            ret.append(id);
    }
    return ret;
}

QList<qlonglong> ArchiveImporter::_processTopologiesInCompressedArchive(QString archivePath, QList<TopologyDescriptor> tdList, const Hns::ImportOptions &options, const Ice::Current &current)
{
    QList<qlonglong> ret;
    QSettings settings(MasterI::instance()->entityCatalog()->getSettingName(),QSettings::IniFormat);
    settings.beginGroup(Constants::SETTINGS_HYNESIM_GROUP);
    QString importDirectory = settings.value((QString("%1Import/path").arg(MasterI::instance()->entityCatalog()->idName().c_str()))).toString();
    settings.endGroup();
    QDir extractionDirectory;
    extractionDirectory.setPath(QString("%1/%2").arg(importDirectory,QString(".tmp")));
    LibArchiveHandler archiveHandler(archivePath);
    archiveHandler.list();
    QStringList topologyList;
    foreach (const TopologyDescriptor& td, tdList) {
        topologyList << QString("%1/").arg(QString::fromStdString(td.path));
    }
    if (!topologyList.isEmpty()) {
        archiveHandler.extractOnDisk(topologyList,extractionDirectory.absolutePath());
        QMutableListIterator<TopologyDescriptor> tdIt(tdList);
        while (tdIt.hasNext()) {
            TopologyDescriptor& td =  tdIt.next();
            QString tdPath = QString::fromStdString(td.path);
            tdPath.prepend(".tmp/");
            td.path = tdPath.toStdString();
        }
    }
    QListIterator<TopologyDescriptor> tdIt(tdList);
    while (tdIt.hasNext()) {
        TopologyDescriptor td =  tdIt.next();
        qlonglong id = _importTopologyInArchive(td, QString::fromStdString(options.newName), current);
        if (id != 0)
            ret.append(id);
    }
    return ret;
}

QList<qlonglong> ArchiveImporter::_processTopologiesInFolderArchive(QString archivePath, QList<TopologyDescriptor> tdList, const Hns::ImportOptions &options, const Ice::Current &current)
{
    QList<qlonglong> ret;
    QSettings settings(MasterI::instance()->entityCatalog()->getSettingName(),QSettings::IniFormat);
    settings.beginGroup(Constants::SETTINGS_HYNESIM_GROUP);
    QString importDirectory = settings.value((QString("%1Import/path").arg(MasterI::instance()->entityCatalog()->idName().c_str()))).toString();
    settings.endGroup();
    QStringList topologyList;
    foreach (const TopologyDescriptor& td, tdList) {
        topologyList << QString::fromStdString(td.path);
    }
    if (!topologyList.isEmpty()) {
        QMutableListIterator<TopologyDescriptor> tdIt(tdList);
        while (tdIt.hasNext()) {
            TopologyDescriptor& td =  tdIt.next();
            QString tdPath = QString::fromStdString(td.path);
            QString relativeArchivePath = archivePath;
            relativeArchivePath.remove(QDir(importDirectory).absolutePath());
            tdPath.prepend(QString("%1/").arg(relativeArchivePath));
            td.path = tdPath.toStdString();
        }
    }
    QListIterator<TopologyDescriptor> tdIt(tdList);
    while (tdIt.hasNext()) {
        TopologyDescriptor td =  tdIt.next();
        qlonglong id = _importTopologyInArchive(td, QString::fromStdString(options.newName), current);
        if (id != 0)
            ret.append(id);
    }
    if (_copyFlag == ArchiveImporter::MOVE)
        Diamesh::removeDirectory(archivePath);
    return ret;
}

void ArchiveImporter::_processCustomResourcesInCompressedArchive(const QString &archivePath, Diamesh::StringList customResources)
{
    QSettings settings(MasterI::instance()->entityCatalog()->getSettingName(),QSettings::IniFormat);
    settings.beginGroup(Constants::SETTINGS_HYNESIM_GROUP);
    QString importDirectory = settings.value((QString("%1Import/path").arg(MasterI::instance()->entityCatalog()->idName().c_str()))).toString();
    settings.endGroup();

    QDir extractionDirectory;
    extractionDirectory.setPath(QString("%1/%2").arg(importDirectory,QString(".tmp")));
    LibArchiveHandler archiveHandler(archivePath);
    archiveHandler.list();

    QStringList customGraphics;
    foreach (const std::string &customGraphic, customResources) {
        customGraphics << QString("customResources/%1").arg(customGraphic.c_str());
    }
    archiveHandler.extractOnDisk(customGraphics, extractionDirectory.absolutePath());

    foreach (const std::string &customGraphic, customResources) {
        QString oldKey = QString::fromStdString(customGraphic);
        QString newKey = MasterI::instance()->customResourceManager()->generateNewKey(oldKey);
        _oldNewResourcesHash.insert(oldKey, newKey);
    }

    QHashIterator<QString, QString> it(_oldNewResourcesHash);
    while (it.hasNext()) {
        it.next();
        QDir resourceDirectory(MasterI::instance()->customResourceManager()->getResourceDirectory());
        QString dstPath = resourceDirectory.absolutePath() + "/" + it.value();
        QString srcPath = extractionDirectory.absolutePath() + "/customResources/" + it.key();
        if (!QFile::copy(srcPath, dstPath)) {
            qCritical() << tr("Failed to copy file %1 to %2").arg(srcPath).arg(dstPath);
            continue;
        }
        MasterI::instance()->customResourceManager()->addResource(it.value());
    }

    Diamesh::removeDirectory(extractionDirectory.absolutePath() + "/customResources");
}

void ArchiveImporter::_processCustomResourcesInFolderArchive(const QString &archivePath, Diamesh::StringList customResources)
{
    foreach (const std::string &customGraphic, customResources) {
        QString oldKey = QString::fromStdString(customGraphic);
        QString newKey = MasterI::instance()->customResourceManager()->generateNewKey(oldKey);
        _oldNewResourcesHash.insert(oldKey, newKey);
    }
    QHashIterator<QString, QString> it(_oldNewResourcesHash);
    while (it.hasNext()) {
        it.next();
        QDir resourceDirectory(MasterI::instance()->customResourceManager()->getResourceDirectory());
        QString dstPath = resourceDirectory.absolutePath() + "/" + it.value();
        QString srcPath = archivePath + "/customResources/" + it.key();
        if (!QFile::copy(srcPath, dstPath)) {
            qCritical() << tr("Failed to copy file %1 to %2").arg(srcPath).arg(dstPath);
            continue;
        }
        MasterI::instance()->customResourceManager()->addResource(it.value());
    }
}

bool ArchiveImporter::_exportEntityEntry(const EntityDescriptor &entityDescriptor, QString dstPath, bool insideArchive, bool appendDataToName)
{
    QString entityDescriptorPath = QString::fromStdString(entityDescriptor.path);
    QStringList stringList = entityDescriptorPath.split("/");
    entityDescriptorPath.replace(stringList[0], "");
    QString srcPath = QString::fromStdString(MasterI::instance()->entityCatalog()->getFullEntryPathFromMixed(entityDescriptor.path,entityDescriptor.uuid).c_str());
    QDateTime time;
    QString appendTimeStamp = appendDataToName ? QString("-%1").arg(time.currentDateTime().toString("yyyyMMdd-hhmmss")) : QString();
    QString appendNameWithoutSpace = appendDataToName ? "-" + QString::fromStdString(entityDescriptor.shortName).replace(" ", "_") : QString();
    if (entityDescriptor.fileType == Hns::FileType::COMPRESSED_FILE) {
        LibArchiveHandler libArchiveHandler("");
        //compress folder in export/.tmp/uuid
        //uuid is get by parsing srcPath
        //export in dstPath
        QDir dir(dstPath);
        if (!dir.cd(".tmp")) {
            dir.mkdir(".tmp");
            dir.cd(".tmp/");
        }
        dstPath += "/.tmp/" + srcPath.split("/").last();
        if (!libArchiveHandler.compress(srcPath, dstPath, COMPRESS_TAR_GZ))
            return false;
        if (!insideArchive) {
            //mv archive
            QString archivePath = dstPath + ".tar.gz";
            QString archiveDstPath = dstPath + appendTimeStamp + ".tar.gz";
            archiveDstPath.replace("/.tmp", "");
            QFile::rename(archivePath, archiveDstPath);
        }
    }
    else {
        QDir dir(dstPath);
        if (!dir.exists()) {
            qCritical() << QObject::tr("unable to find export folder ! operation aborted");
            return false;
        }
        if (insideArchive) {
            if (!dir.cd(".tmp")) {
                dir.mkdir(".tmp");
                dir.cd(".tmp/");
            }
            dstPath += "/.tmp";
        }
        QString newFolder = dstPath + QDir::separator() + srcPath.split(QDir::separator()).last() + appendNameWithoutSpace + appendTimeStamp;
        _copyDirectory(srcPath, newFolder, true);
        //todo : patch diamesh
        //        Diamesh::copyDirectory(srcPath, dstPath);
    }
    return true;
}

void ArchiveImporter::_exportCustomResources(const QString &customGraphic, bool insideArchive)
{
    if (customGraphic.isEmpty())
        return;

    QString customResourcesPath = _exportFolder + "/";
    if (insideArchive)
        customResourcesPath += ".tmp/customResources";
    else
        customResourcesPath += "customResources";

    QDir dir(customResourcesPath);
    if (!dir.exists()) {
        qDebug() << tr("Directory %1 doesn't exist, creating it.").arg(dir.absolutePath());
        dir.mkpath(".");
    }
    QString dstPath = customResourcesPath + "/" + customGraphic;

    if (dir.exists(dstPath)) //file already exists
        return;

    QString srcPath = MasterI::instance()->customResourceManager()->getResourcePath(customGraphic);
    if (!QFile::copy(srcPath, dstPath)) {
        qCritical() << tr("Unable to copy file %1 to file %2").arg(srcPath).arg(dstPath);
        return;
    }
    qDebug() << tr("File %1 copied").arg(dstPath);
}

void ArchiveImporter::_copyDirectory(const QString &srcPath, const QString &dstPath, bool letCpCreateDir)
{
    QDir dstDir(dstPath);
    QDir srcDir(srcPath);

    if (!srcDir.exists()) {
        qCritical() << QObject::tr("unable to find source folder ") << srcPath;
    }
    if (!letCpCreateDir && !dstDir.mkpath(dstDir.absolutePath())) {
        qCritical() << QObject::tr("unable to find destination folder ") << dstPath;
    }
    qDebug() << tr("Copying %1 to %2").arg(srcPath, dstPath);
    QProcess process;
    QString name = "cp";
    QStringList args;
    args << "-a" << srcPath << dstPath;
    process.start(name, args);
    if (!process.waitForStarted()){
        qWarning() << tr("%1 can not start").arg(name);
        return;
    }
    if (!process.waitForFinished(-1)){
        qWarning() << tr("%1 can not end").arg(name);
        return;
    }
}

qlonglong ArchiveImporter::_importEntityEntry(const std::string &folderName, const std::string &storage, ArchiveImporter::CopyPolicy policy,const Hns::ImportOptions &options, const Ice::Current &iceCurrent)
{
    ::std::string catalogName= MasterI::instance()->entityCatalog()->getFileStorages()[storage];
    QString uuid = MasterI::instance()->entityCatalog()->generateUnusedUUID();
    //it is assumed that the file is correct but we make a verification
    QSettings settings(MasterI::instance()->entityCatalog()->getSettingName(),QSettings::IniFormat);
    settings.beginGroup(Constants::SETTINGS_HYNESIM_GROUP);
    QString path = settings.value((QString("%1Import/path").arg(MasterI::instance()->entityCatalog()->idName().c_str()))).toString();
    settings.endGroup();
    //verification import folder in storage
    QString strPathToImport = QString("%1/import").arg(storage.c_str());

    QDir catalogImportDir(strPathToImport);
    if (!catalogImportDir.mkpath(catalogImportDir.absolutePath())) {
        qCritical() << QString(QObject::tr("Can't create import folder in %1")).arg(catalogImportDir.absolutePath());
        return 0;
    }
    // create a lock file
    QString filePath = QString("%1/%2").arg(path).arg(folderName.c_str());
    QFile file(QString("%1/.lock.xml").arg(filePath));

    QDir dirToCopy(filePath);
    if(!dirToCopy.exists()) {
        qCritical() << QObject::tr("Folder %1 does not exist").arg(dirToCopy.absolutePath());
        return 0;
    }
    if (policy == MOVE) {
        if(file.exists()){
            qCritical() << QObject::tr("Entity is lock");
            return 0;
        }
        //before we lock the file
        file.open(QIODevice::WriteOnly);
        file.write("lockFile");
        file.close();

        //mv folder
        if(!QDir().rename(filePath , QString("%1/%2").arg(strPathToImport).arg(uuid))){
            qCritical() << QObject::tr("Can't move the file with rename");
            if(!Diamesh::copyDirectory(filePath,QString("%1/%2").arg(strPathToImport).arg(uuid))){
                qCritical() << QObject::tr("Unable to copy file \"%1\" to \"%2\"").arg(filePath).arg(QString("%1/%2").arg(strPathToImport).arg(uuid));
                file.setFileName(QString("%1/.lock.xml").arg(filePath));
                file.remove();
                return 0;
            }
            if(!Diamesh::removeDirectory(filePath)){
                //if lock is not here, make it
                file.setFileName(QString("%1/.lock.xml").arg(filePath));
                if(!file.exists()){
                    file.open(QIODevice::WriteOnly);
                    file.write("lockFile");
                    //return 0; not 0 because entity is imported
                }
            }
        }

        //suppress lock
        file.setFileName(QString("%1/%2/.lock.xml").arg(strPathToImport).arg(uuid));
        file.remove();
    } else if (policy == COPY){
        HnsServer::ProcessRunner process;
        QString program = "rsync";
        QStringList args;
        args << QString("%1/").arg(filePath) << QString("%1/%2").arg(strPathToImport).arg(uuid) << "-aH" << "--numeric-ids";
        process.runSynchronous(program, args, -1);
    }

    QString folder = QString("%1/%2").arg(strPathToImport).arg(uuid);

    QFile fileConfig(QString("%1/hnsEntryConfig.xml").arg(folder));
    if (fileConfig.exists() && fileConfig.open(QFile::ReadOnly | QFile::Text)) {
        QDomDocument doc;
        doc.setContent(&fileConfig, false);
        ::Hns::EntityEntryPtr newEntry = EntityEntryPtr::dynamicCast(EntityEntrySerializer::readEntry(doc));
        if (newEntry) {
            newEntry->uuid = uuid.toStdString(); //EntityCatalogI::instance()->getUnusedUUID().toStdString();
            newEntry->path = QString("%1:/import").arg(catalogName.c_str()).toStdString();
            QString className = QString::fromStdString(newEntry->ice_id());
            if (_finishImportModelHash()->contains(className)) {
                FinishImportModel finishImportModel = _finishImportModelHash()->value(className);
                finishImportModel(newEntry, folder, options);
            }
            fileConfig.close();
            return MasterI::instance()->entityCatalog()->addEntry(newEntry, iceCurrent);
        }
    }
    return 0;
}

void ArchiveImporter::_applyACL(QList<qlonglong> entries, QList<qlonglong> topologies, QList<qlonglong> folders, const Hns::ImportOptions &options)
{
    foreach (qlonglong id, entries) {
        EntityCatalogI *catalog = MasterI::instance()->entityCatalog();
        QList<Diamesh::AccessControlList> oldACLS = QList<Diamesh::AccessControlList>::fromStdList(options.acls);
        QList<Diamesh::AccessControlList> newACLs;
        EntryPtr entry = catalog->getEntry(id);
        if (!entry)
            continue;
        foreach (Diamesh::AccessControlList oldAcl, oldACLS)
            newACLs.append(_generateACL(entry, oldAcl, Hns::Rights::ENTITY_ENTRY_PREFIX));
        newACLs.append(QList<Diamesh::AccessControlList>::fromStdList(MasterI::instance()->aclManager()->getACLsForTarget(entry)));
        if (!catalog->setACLsForCatalogItem(id, newACLs.toStdList()))
            qCritical() << tr("Unable to set ACL when importing an entity entry");
    }
    foreach (qlonglong id, topologies) {
        TopologyCatalogI *catalog = MasterI::instance()->topologyCatalog();
        QList<Diamesh::AccessControlList> oldACLS = QList<Diamesh::AccessControlList>::fromStdList(options.acls);
        QList<Diamesh::AccessControlList> newACLs;
        EntryPtr entry = catalog->getEntry(id);
        if (!entry)
            continue;
        foreach (Diamesh::AccessControlList oldAcl, oldACLS)
                newACLs.append(_generateACL(entry, oldAcl, Hns::Rights::TOPOLOGY_ENTRY_PREFIX));
        newACLs.append(QList<Diamesh::AccessControlList>::fromStdList(MasterI::instance()->aclManager()->getACLsForTarget(entry)));
        if (!catalog->setACLsForCatalogItem(id, newACLs.toStdList()))
            qCritical() << tr("Unable to set ACL when importing a topology");
    }
    foreach (qlonglong id, folders) {
        EntityCatalogI *catalog = MasterI::instance()->entityCatalog();
        QList<Diamesh::AccessControlList> oldACLS = QList<Diamesh::AccessControlList>::fromStdList(options.acls);
        QList<Diamesh::AccessControlList> newACLs;
        FolderPtr folder = catalog->getFolder(id);
        if (!folder)
            continue;
        foreach (Diamesh::AccessControlList oldAcl, oldACLS)
                newACLs.append(_generateFolderACL(folder, oldAcl));
        newACLs.append(QList<Diamesh::AccessControlList>::fromStdList(MasterI::instance()->aclManager()->getACLsForTarget(folder)));
        if (!catalog->setACLsForCatalogItem(id, newACLs.toStdList()))
            qCritical() << tr("Unable to set ACL when importing a folder");
    }
}

Diamesh::AccessControlList ArchiveImporter::_generateACL(const Diamesh::ACLTargetPtr &target, const Diamesh::AccessControlList &oldAcl, const QString &rightPrefix)
{
    QStringList rightsNames;
    rightsNames.append(rightPrefix + QString(Hns::Rights::ENTRY_SUFFIX_READ));
    rightsNames.append(rightPrefix + QString(Hns::Rights::ENTRY_SUFFIX_UPDATE_ENTRY));
    rightsNames.append(rightPrefix + QString(Hns::Rights::ENTRY_SUFFIX_USE));

    Diamesh::AccessControlList newACL;
    newACL.hSetId = oldAcl.hSetId;
    newACL.subject = oldAcl.subject;
    newACL.target = MasterI::instance()->aclManager()->helper()->targetFromObject(target).toStdString();
    newACL.rights = MasterI::instance()->aclManager()->helper()->rightsFromNames(rightsNames);

    return newACL;
}

Diamesh::AccessControlList ArchiveImporter::_generateFolderACL(const Diamesh::ACLTargetPtr &target, const Diamesh::AccessControlList &oldAcl)
{
    QString rightsPrefix(Hns::Rights::FOLDER_PREFIX);
    QStringList rightsNames;
    rightsNames.append(rightsPrefix + QString(Hns::Rights::FOLDER_SUFFIX_DELETE));
    rightsNames.append(rightsPrefix + QString(Hns::Rights::FOLDER_SUFFIX_READ));
    rightsNames.append(rightsPrefix + QString(Hns::Rights::FOLDER_SUFFIX_UPDATE_FOLDER));

    Diamesh::AccessControlList newACL;
    newACL.hSetId = oldAcl.hSetId;
    newACL.subject = oldAcl.subject;
    newACL.target = MasterI::instance()->aclManager()->helper()->targetFromObject(target).toStdString();
    newACL.rights = MasterI::instance()->aclManager()->helper()->rightsFromNames(rightsNames);

    return newACL;
}

void ArchiveImporter::_rebaseEntities(BaseEntityMap relationships) {
    EntityCatalogI *catalog = MasterI::instance()->entityCatalog();

    for (auto it = relationships.begin() ; it != relationships.end() ; ++it) {
        DomainEntryPtr child = DomainEntryPtr::dynamicCast(catalog->getEntryByUuid(_entityUUIDHash.value(QString::fromStdString(it->first))));
        DomainEntryPtr parent = DomainEntryPtr::dynamicCast(catalog->getEntryByUuid(_entityUUIDHash.value(QString::fromStdString(it->second))));
        qDebug() << tr("Import: rebasing %1 domain to %2").arg(QString::fromStdString(child->uuid)).arg(QString::fromStdString(parent->uuid));
        parent->diffUuids.push_back(_entityUUIDHash.value(QString::fromStdString(it->first)).toStdString());
        catalog->updateEntry(parent);
        child->baseUuid = _entityUUIDHash.value(QString::fromStdString(it->second)).toStdString();
        catalog->updateEntry(child);

        for (const auto &disk : child->disks) {
            qDebug() << tr("Import: rebasing disk %1 ...").arg(QString::fromStdString(disk.source));
            HnsServer::StorageManager *storageManager = MasterI::instance()->storageManager();
            QString clonedDiskPath = storageManager->getFullPathFromMixed("entity", QString::fromStdString(child->path), QString::fromStdString(child->uuid))
                    + "/" + QString::fromStdString(disk.source);
            QString originalDiskPath = storageManager->getFullPathFromMixed("entity", QString::fromStdString(parent->path), QString::fromStdString(parent->uuid))
                    + "/" + QString::fromStdString(disk.source);

            QStringList args = QStringList() << "rebase" << "-u" << "-b" << originalDiskPath << clonedDiskPath;
            HnsServer::ProcessRunner process;
            if (!process.runSynchronous("qemu-img", args, 3000000))
                qCritical() << tr("An error occured when rebasing the domain %1: %2")
                               .arg(child->uuid.c_str(), process.getError());
            else
                qDebug() << tr("Import: disk %1 rebased").arg(QString::fromStdString(disk.source));
        }
    }
}

} // namespace Hns
