/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of HYNESIM.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "catalogimporterI.h"
#include "handler.h"
#include "archiveimporter.h"
#include "catalog/entitycatalogI.h"
#include "masterI.h"
#include "libarchive/libarchivehandler.h"
#include "catalog/topologycatalogI.h"
#include "entryserializer.h"
#include "catalog/catalogfilestorage.h"
#include "importableserializer.h"

// hynesim common headers
#include "common/constants.h"

// diamesh headers
#include <diamesh/utils.h>

// Qt headers
#include <QSettings>
#include <QDirIterator>
#include <QDebug>
#include <QDateTime>

namespace Hns {

CatalogImporterI *CatalogImporterI::_instance = NULL;

CatalogImporterI::CatalogImporterI() :
    CatalogImporter()
{
    _id.name = "catalogImporterI";
    MasterI::instance()->adapter()->add(this, _id);
    FolderHandler *fHandler = new FolderHandler();
    CompressedHandler *cHandler = new CompressedHandler();
    _handlerList.append(fHandler);
    _handlerList.append(cHandler);
}

CatalogImporterI *CatalogImporterI::create()
{
    if (!_instance) {
        _instance = new CatalogImporterI();
    }
    return _instance;
}

CatalogImporterI *CatalogImporterI::instance()
{
    return _instance;
}

CatalogImporterPrx CatalogImporterI::proxy()
{
    return CatalogImporterPrx::uncheckedCast(MasterI::instance()->adapter()->createProxy(_id));
}


CatalogImporterI::~CatalogImporterI()
{

}

bool CatalogImporterI::importData(const ImportablePtr &importable, const std::string &entityStorage, const std::string &topologyStorage, const ImportOptionsList &list, const Ice::Current &current)
{
    ImportableArchivePtr archive = ImportableArchivePtr::dynamicCast(importable);
    if (archive) {
        QList<ImportOptions> optionsQList = QList<ImportOptions>::fromStdList(list);
        ArchiveImporter::CopyPolicy copyFlag = ArchiveImporter::COPY;
        int counter = 0;
        for (QList<ImportOptions>::const_iterator i = optionsQList.constBegin() ; i != optionsQList.constEnd() ; ++i) {
            ArchiveImporter importer(QString::fromStdString(topologyStorage), QString::fromStdString(entityStorage));
            if ((i + 1) == optionsQList.constEnd())
                copyFlag = ArchiveImporter::MOVE;
            importer.setCopyFlag(copyFlag);
            if (!importer.importArchive(archive, *i, current))
                return false;
            emit progressUpdated(++counter);
        }
        return true;
    }
    ImportableEntryPtr entry = ImportableEntryPtr::dynamicCast(importable);
    if (entry) {
        ArchiveImporter importer(QString(),QString::fromStdString(entityStorage));
        if (importer.importEntityEntry(entry->path, entityStorage, current))
            return true;
    }
    return false;
}

bool CatalogImporterI::exportData(const DescriptorPtr &desc, bool isArchive, const Ice::Current &current)
{
    Q_UNUSED(current)
    ArchiveDescriptorPtr archive = ArchiveDescriptorPtr::dynamicCast(desc);
    if (archive)
        return exportTopology(archive, isArchive);

    EntityEntryDescriptorPtr entityEntry = EntityEntryDescriptorPtr::dynamicCast(desc);
    if (entityEntry) {
        ArchiveImporter importer("");
        MasterI::instance()->entityCatalog()->lockEntry(MasterI::instance()->entityCatalog()->getEntryByUuid(entityEntry->entity.uuid.c_str())->id, LOCK_TYPE_EXPORT);
        bool ret = importer.exportEntityEntry(entityEntry->entity);
        MasterI::instance()->entityCatalog()->unlockEntry(MasterI::instance()->entityCatalog()->getEntryByUuid(entityEntry->entity.uuid.c_str())->id);
        return ret;
    }
    return false;
}

bool CatalogImporterI::exportTopology(const ArchiveDescriptorPtr &archiveDesc, bool isArchive, const Ice::Current &current)
{
    TopologyCatalogI *catalog = MasterI::instance()->topologyCatalog();
    if (!catalog) {
        qCritical() << QObject::tr("Could not access Topology Catalog. Export operation aborted.");
        return false;
    }
    QList<TopologyEntryPtr> topologies;
    QList<TopologyDescriptor> tdList = QList<TopologyDescriptor>::fromStdList(archiveDesc->topologies);
    bool topologiesAvailable = true;
    foreach (const TopologyDescriptor &tDescriptor, tdList) {
        QString topologyUUID = QString::fromStdString(tDescriptor.uuid);
        Hns::TopologyEntryPtr topology = Hns::TopologyEntryPtr::dynamicCast(catalog->getEntryByUuid(topologyUUID));
        if (topology) {
            switch (topology->status) {
            case Hns::TOPOLOGY_BUSY_EXPORTING: {
                qCritical() << QObject::tr("Topology %1 with UUID %2 is already being exported. Operation aborted.")
                               .arg(QString::fromStdString(topology->shortName))
                               .arg(topologyUUID);
                topologiesAvailable = false;
                break;
            }
            case Hns::TOPOLOGY_STATUS_OK: {
                topology->status = Hns::TOPOLOGY_BUSY_EXPORTING;
                if (!catalog->updateEntry(topology, current)) {
                    qCritical() << QObject::tr("Failed to mark topology %1 with UUID %2 as busy.")
                                   .arg(QString::fromStdString(topology->shortName))
                                   .arg(QString::fromStdString(topology->uuid));
                    topologiesAvailable = false;
                } else
                    topologies.append(topology);
                break;
            }
            default: {
                    qCritical() << QObject::tr("Topology %1 with UUID %2 is not available. Operation aborted.")
                                   .arg(QString::fromStdString(topology->shortName))
                                   .arg(topologyUUID);
                    topologiesAvailable = false;
                break;
            }
            }
        }
        if (!topologiesAvailable) {
            _releaseTopologies(topologies, current);
            return false;
        }
    }

    //check if entities are locked
    QList<EntityDescriptor> entities = QList<EntityDescriptor>::fromStdList(archiveDesc->entities);
    EntityCatalogI *entityCatalog = MasterI::instance()->entityCatalog();
    foreach (EntityDescriptor entityDescriptor, entities) {
        QString uuid = QString::fromStdString(entityDescriptor.uuid);
        EntityEntryPtr entityEntry = EntityEntryPtr::dynamicCast(entityCatalog->getEntryByUuid(uuid));
        if (!entityEntry) {
            continue;
        }
        if (entityCatalog->entryIsLocked(entityEntry->id)) {
            qCritical() << QObject::tr("Entity %1 with id %2 is already used. Operation aborted").arg(QString::fromStdString(entityEntry->shortName)).arg(entityEntry->id);
            return false;
        }
    }

    QList<qlonglong> entityIdLockedList;
    bool lockOk = true;
    foreach (EntityDescriptor entityDescriptor, entities) {
        QString uuid = QString::fromStdString(entityDescriptor.uuid);
        EntityEntryPtr entityEntry = EntityEntryPtr::dynamicCast(entityCatalog->getEntryByUuid(uuid));
        if (!entityEntry) {
            continue;
        }
        if (entityCatalog->lockEntry(entityEntry->id)) {
            entityIdLockedList.append(entityEntry->id);
        }
        else {
            lockOk = false;
            break;
        }
    }
    if (!lockOk) { //an entity is already lock, need to abort operation
        _unlockEntities(entityIdLockedList);
        qCritical() << QObject::tr("Unable to lock all entities. Operation aborted");
        return false;
    }

//    QString eStorage = QString::fromStdString(entityStorage);
    ArchiveDescriptorPtr archiveDescNoConst = archiveDesc;
    QSettings settings(MasterI::instance()->entityCatalog()->getSettingName(),QSettings::IniFormat);
    settings.beginGroup(Constants::SETTINGS_HYNESIM_GROUP);
    QString exportFolder = settings.value("exportCatalog/path").toString();
    settings.endGroup();
    if (exportFolder.isEmpty()) {
        qCritical() << tr("export folder cannot be found, please add exportCatalog/path in master.ini. Operation aborted");
        _unlockEntities(entityIdLockedList);
        return false;
    }
    if (!exportFolder.endsWith("/"))
        exportFolder += "/";

    QDir dir(exportFolder);
    if (!dir.exists()) {
        qCritical() << QObject::tr("unable to find export folder aborting");
        _unlockEntities(entityIdLockedList);
        return false;
    }

    //Manage fake switches
//    QString pathToLab = QString::fromStdString(entityStorage + "/.tmp/");
    QString pathToLab = MasterI::instance()->getFirstEntityFileStorage()->storagePath()+"/.tmp/";
    QDir dirLab(pathToLab);
    dirLab.mkdir(pathToLab);

    QList<std::string> customResources = QList<std::string>::fromStdList(archiveDesc->customResources);

    foreach (TopologyEntryPtr tp, topologies) {
        QMap<int, Hns::EntityEntryPtr> entityEntryMap(tp->entitiesMap);
        foreach (const Hns::EntityEntryPtr &entryPtr, entityEntryMap.values()) {
            if (!entryPtr->hasResources && (entryPtr->path == "")) {
                QString entryPath = pathToLab + QString::fromStdString(entryPtr->uuid) + "/";
                QDir dirEntryPath(entryPath);
                dirEntryPath.mkdir(entryPath);
                EntrySerializer serializer;
                entryPtr->isTemplate = true;
                if (serializer.writeEntry(entryPtr, "HnsEntityConfig", entryPath)) {
                    Hns::EntityDescriptor entityDescriptor;
                    entityDescriptor.type = entryPtr->ice_id();
                    entityDescriptor.shortName = entryPtr->shortName;
                    entityDescriptor.longName = entryPtr->longName;
                    entityDescriptor.description = entryPtr->description;
                    entityDescriptor.path = MasterI::instance()->getFirstEntityFileStorage()->storageName().toStdString()+":/.tmp/";
                    entityDescriptor.uuid = entryPtr->uuid;
                    entityDescriptor.fileType = Hns::FileType::PLAIN_FOLDER;
                    entityDescriptor.tags = entryPtr->tagList;
                    entityDescriptor.parentId = 0;
                    entityDescriptor.customGraphic = entryPtr->customGraphic;
                    archiveDescNoConst->entities.push_back(entityDescriptor);
                    if (!entryPtr->customGraphic.empty() && !customResources.contains(entryPtr->customGraphic))
                        customResources << entryPtr->customGraphic;
                }
            }
        }
    }
    archiveDescNoConst->customResources = customResources.toStdList();

    //get back archive descriptor
    //create xml
    ImportableArchiveSerializer serializer;
    QString pathToXml = serializer.serialize(archiveDescNoConst);


    QString folderToCreate;
    QDateTime date;
    folderToCreate += QString::fromStdString(archiveDescNoConst->name);
    QString currentDateTime = date.currentDateTime().toString("yyyyMMdd-hhmmss");
    folderToCreate += QString("-%1").arg(currentDateTime);

    if (!dir.mkdir(QString::fromStdString(archiveDescNoConst->name))) {
        qCritical() << tr("Unable to create folder. Operation aborted") << QString::fromStdString(archiveDescNoConst->name);
        Diamesh::removeDirectory(pathToLab);
        _unlockEntities(entityIdLockedList);
        _releaseTopologies(topologies, current);
        return false;
    }
    exportFolder += QString::fromStdString(archiveDescNoConst->name);
    if (isArchive) {
        QFile file(exportFolder + ".tar");
        if (file.exists()) {
            qDebug() << QString(exportFolder + ".tar") << tr("already exists, operation aborted");
            Diamesh::removeDirectory(pathToLab);
            _unlockEntities(entityIdLockedList);
            _releaseTopologies(topologies, current);
            return false;
        }
    }

    //compress/cp in the folder
    ArchiveImporter archiveImporter(QString(), "", exportFolder);
    if (!archiveImporter.exportArchive(archiveDescNoConst, isArchive)) {
        Diamesh::removeDirectory(pathToLab);
        _unlockEntities(entityIdLockedList);
        _releaseTopologies(topologies, current);
        return false;
    }

    if (isArchive) {
        //mv xml
        QFile::rename(pathToXml, exportFolder + "/.tmp/hnsArchive.xml");
        LibArchiveHandler handler("");
        handler.compress(exportFolder + "/.tmp", exportFolder, COMPRESS_TAR);
        Diamesh::removeDirectory(exportFolder);
    }
    else {
        QFile::rename(pathToXml, exportFolder + "/hnsArchive.xml");
        Diamesh::removeDirectory(exportFolder + "/.tmp");
    }
    Diamesh::removeDirectory(pathToLab);
    _unlockEntities(entityIdLockedList);
    _releaseTopologies(topologies, current);
    return true;
}

ImportableArchiveList CatalogImporterI::getArchiveList(const Ice::Current &)
{
    QList<ImportablePtr> list = _getImportableList(CatalogImporterFlags::FLAG_IMPORT_ARCHIVES);
    ImportableArchiveList returnList;
    foreach (ImportablePtr ptr, list) {
        ImportableArchivePtr archivePtr = ImportableArchivePtr::dynamicCast(ptr);
        if (archivePtr != 0) {
            returnList.push_back(archivePtr);
        }
    }
    return returnList;
}

::Hns::EntityEntryToImportMap CatalogImporterI::showEntityEntry(const ::Ice::Current&)
{
    ::Hns::EntityEntryToImportMap importMap;
    QSettings settings(MasterI::instance()->entityCatalog()->getSettingName(),QSettings::IniFormat);

    settings.beginGroup(Constants::SETTINGS_HYNESIM_GROUP);
    QString path = settings.value((QString("%1Import/path").arg(MasterI::instance()->entityCatalog()->idName().c_str()))).toString();
    settings.endGroup();
    if (path.isEmpty()) {
        qCritical() << QObject::tr("The import path is empty.");
        return importMap;
    }

    //test if folder exists, creation otherwise
    QFile file(path);
    if(!file.exists()){
        QDir().mkpath(path);
        if(!file.exists()){
            qCritical() << QString(QObject::tr("The import path: %1 do not exist and can't be create.")).arg(path);
            return importMap;
        }
    }
    _addCatalogImporter(path, importMap);

    return importMap;
}

QList<ImportablePtr> CatalogImporterI::_getImportableList(int flags)
{
    QList<ImportablePtr>list;
    _parseImportFolder(flags,&list);
    return list;
}

void CatalogImporterI::_parseImportFolder(int flags, QList<ImportablePtr> *list, QString path)
{
    bool flagEntry = (flags && CatalogImporterFlags::FLAG_IMPORT_ENTITIES);
    bool flagArchives = (flags && CatalogImporterFlags::FLAG_IMPORT_ARCHIVES);
    QString pathToExplore;
    if (path == "") {
        QSettings settings(MasterI::instance()->entityCatalog()->getSettingName(),QSettings::IniFormat);
        settings.beginGroup(Constants::SETTINGS_HYNESIM_GROUP);
        pathToExplore = settings.value("entityCatalogImport/path").toString();
        settings.endGroup();
    } else {
        pathToExplore = path;
    }
    QDir dir(pathToExplore);
    dir.setFilter(QDir::NoDotAndDotDot | QDir::AllDirs | QDir::Files | QDir::NoSymLinks);
    QDirIterator iterator(dir);
    while(iterator.hasNext()) {
        iterator.next();
        foreach (Handler *handler,_handlerList) {
            bool handlerResult = handler->canHandle(iterator.fileInfo().absoluteFilePath());
            if (handlerResult) {
                ImportablePtr importablePtr = handler->handle(iterator.fileInfo().absoluteFilePath());
                if (importablePtr != 0) {
                    list->append(importablePtr);
                } else if (iterator.fileInfo().isDir() && importablePtr == 0) {
                    _parseImportFolder(flags,list,iterator.fileInfo().absoluteFilePath());
                }
            }
        }
    }
    //We delete items from the list if they don't match the flags
    QListIterator<ImportablePtr> importableIterator(*list);
    while (importableIterator.hasNext()) {
        ImportablePtr importablePtr = importableIterator.next();
        ImportableArchivePtr archivePtr = ImportableArchivePtr::dynamicCast(importablePtr);
        ImportableEntryPtr entryPtr = ImportableEntryPtr::dynamicCast(importablePtr);
        if (archivePtr != 0 && !flagArchives) {
            list->removeOne(importablePtr);
        }
        if (entryPtr != 0 && !flagEntry) {
            list->removeOne(importablePtr);
        }
    }
}

Ice::Long CatalogImporterI::_importEntityEntry(const std::string &folder, const std::string &storage, const Ice::Current &iceCurrent)
{
    ArchiveImporter importer(QString(),QString::fromStdString(storage));
    return importer.importEntityEntry(folder, storage, iceCurrent);
}

void CatalogImporterI::_unlockEntities(QList<qlonglong> idList)
{
    EntityCatalogI *entityCatalog = MasterI::instance()->entityCatalog();
    foreach (qlonglong id, idList) {
        entityCatalog->unlockEntry(id);
    }
}

void CatalogImporterI::_addCatalogImporter(const QString &storagePath, ::Hns::EntityEntryToImportMap &entryMap)
{
    //change if exit subdirectory (with QDirIterator... or other)
    QDir dirImport(storagePath);
    QDirIterator dirIterator(dirImport);
    QDomDocument doc;
    while (dirIterator.hasNext()) {
        dirIterator.next();
        if (dirIterator.fileName()[0]!= '.') {
            QFile file(QString("%1/hnsEntryConfig.xml").arg( dirIterator.fileInfo().absoluteFilePath()));
            if (file.exists() && file.open(QFile::ReadOnly | QFile::Text)) {
                doc.setContent(&file, false);
                ::Hns::EntityEntryPtr newEntry = EntityEntryPtr::dynamicCast(EntityEntrySerializer::readEntry(doc, dirImport.dirName()));
                if (newEntry) {
                    entryMap[dirIterator.fileName().toStdString()] = newEntry;
                }
                else {
                    qDebug() << QObject::tr("entity not valid");
                }
            }
        }
    }
}

void CatalogImporterI::_releaseTopologies(QList<TopologyEntryPtr> topologies, const Ice::Current &current)
{
    TopologyCatalogI *catalog = MasterI::instance()->topologyCatalog();
    if (!catalog) {
        qCritical() << QObject::tr("Could not access Topology Catalog. Export operation aborted.");
        return;
    }
    foreach(const TopologyEntryPtr &topology, topologies) {
        topology->status = Hns::TOPOLOGY_STATUS_OK;
        if (!catalog->updateEntry(topology, current)) {
            qCritical() << QObject::tr("Failed to release topology %1 with UUID %2.")
                           .arg(QString::fromStdString(topology->shortName))
                           .arg(QString::fromStdString(topology->uuid));
        }
    }
}

} // namespace Hns
