/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of HYNESIM.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "importableserializer.h"
#include "entryserializer.h"

// hynesim common headers
#include "common/constants.h"

// Qt headers
#include <QDebug>
#include <QStringList>
#include <QFile>
#include <QDomDocument>

namespace Hns {
/************************************************************
 **********************IMPORTABLESERIALIZER******************
 ***********************************************************/
ImportableSerializer::ImportableSerializer(QObject *parent)
    : QObject(parent)
{
}

ImportableSerializer::~ImportableSerializer()
{
}

/************************************************************
 **********************IMPORTABLEENTRYSERIALIZER*************
 ***********************************************************/
ImportableEntrySerializer::ImportableEntrySerializer(QObject *parent)
    : ImportableSerializer(parent)
{
}

ImportableEntrySerializer::~ImportableEntrySerializer()
{
}

ImportablePtr ImportableEntrySerializer::deserialize(const QString &fileContent)
{
    QDomDocument doc;
    if (!doc.setContent(fileContent)) {
        qCritical()<<QObject::tr("Can not read the entry data");
        return NULL;
    }
    ImportableEntryPtr importableEntry = new ImportableEntry();
    ::Hns::EntityEntryPtr entityEntry =  EntityEntryPtr::dynamicCast(EntityEntrySerializer::readEntry(doc));
    if (!entityEntry)
        qCritical() << QObject::tr("ImportableEntrySerializer : Unable to create entityEntry !");
    importableEntry->entry = entityEntry;
    return importableEntry;

}

/************************************************************
 **********************IMPORTABLEARCHIVESERIALIZER***********
 ***********************************************************/
ImportableArchiveSerializer::ImportableArchiveSerializer(QObject *parent)
    : ImportableSerializer(parent)
{
}
ImportableArchiveSerializer::~ImportableArchiveSerializer()
{
}

ImportablePtr ImportableArchiveSerializer::deserialize(const QString &fileContent)
{
    QDomDocument doc;
    QString errorMsg;
    if (!doc.setContent(fileContent)) {
        qCritical()<<QObject::tr("Can not read the archive data: \n") << fileContent;
        return NULL;
    }
    ImportableArchivePtr importableArchive = new ImportableArchive();
    ArchiveDescriptorPtr archiveDescriptor = new ArchiveDescriptor();

    QDomElement root = doc.documentElement();
    QDomNodeList hnsArchiveList = root.elementsByTagName("HnsArchive");
    if(hnsArchiveList.size() != 1) {
        qCritical() << QObject::tr("There should be exactly one <HnsArchive> tag in the archive descriptor file");
        return NULL;
    }
    QDomElement hnsArchive = hnsArchiveList.item(0).toElement();
    QString name = hnsArchive.attribute("name");
    long creationDateTime = hnsArchive.attribute("creationDateTime").toLong();
    //The version allows us to depreciate XML when we need to bring changes
    double version = hnsArchive.attribute("version").toDouble();
    if (!version) {
        version = Constants::DEFAULT_VERSION_ARCHIVE_XML;
    }
    if (name.isEmpty() || creationDateTime == 0) {
        qCritical() << QObject::tr("You need to put a name and a creationDateTime");
        return NULL;
    }

    archiveDescriptor->name = name.toStdString();
    archiveDescriptor->creationDateTime = creationDateTime;


    //FOLDER
    QDomNodeList folderEntriesList = hnsArchive.elementsByTagName("FolderEntries");
    if (folderEntriesList.size() != 1) {
        qCritical() << QObject::tr("There should be exactly one <FolderEntries> tag in the archive descriptor file");
        return NULL;
    }

    QDomElement folderEntries = folderEntriesList.item(0).toElement();
    QDomNodeList folderEntryList = folderEntries.childNodes();

    QList<FolderDescriptor> folderDescriptorList;
    for (int i = 0 ; i < folderEntryList.size() ; ++i) {
        QDomNode node = folderEntryList.item(i);
        FolderDescriptor folder;
        QDomElement element = node.toElement();
        QString shortName = element.attribute("shortName");
        QString longName = element.attribute("longName");
        QString description = element.attribute("description");
        QString tags = element.attribute("tags");
        int id = element.attribute("id").toInt();
        int parentId = element.attribute("parentId").toInt();
        if (shortName.isEmpty()) {
            qCritical() << QString(QObject::tr("The folder %1 must have a short and a long name")).arg(i);
            return NULL;
        }

        if (id == 0) {
            qCritical() << QObject::tr("The id of a folder must not be 0");
            return NULL;
        }
        if (longName.isEmpty())
            longName = "";

        folder.shortName = shortName.toStdString();
        folder.longName = longName.toStdString();
        folder.description = description.toStdString();
        QStringList tagsList = tags.split(",");
        foreach (QString tag, tagsList) {
            folder.tags.push_back(tag.toStdString());
        }

        folder.id = id;
        folder.parentId = parentId;

        folderDescriptorList.append(folder);
    }

    archiveDescriptor->folders = folderDescriptorList.toStdList();


    //ENTITY
    QDomNodeList entityEntriesList = hnsArchive.elementsByTagName("EntityEntries");
    if (entityEntriesList.size() != 1) {
        qCritical() << QObject::tr("There should be exactly one <EntityEntries> tag in the archive descriptor file");
        return NULL;
    }

    QDomElement entityEntries = entityEntriesList.item(0).toElement();
    QDomNodeList entityEntryList = entityEntries.childNodes();
    QList<EntityDescriptor> entityDescriptorList;

    for (int i = 0 ; i < entityEntryList.size() ; i++) {
        QDomNode node = entityEntryList.item(i);
        EntityDescriptor entityDescriptor;
        QDomElement element = node.toElement();
        QString type = element.attribute("type");
        QString shortName = element.attribute("shortName");
        QString longName = element.attribute("longName");
        QString description = element.attribute("description");
        QString path = element.attribute("path");
        QString uuid = element.attribute("uuid");
        QString parentId = element.attribute("parentId", "0");
        int fileType = element.attribute("fileType").toInt();
        QString fileLocation = element.attribute("fileLocation");

        //We need to depreciate version 1.0 of the XML
        if (version >= Constants::CURRENT_COMPATIBILITY_VERSION_ARCHIVE_XML) {
            if (!fileType) {
                qCritical() << QObject::tr("File type must be precised");
                return NULL;
            }
            if (fileType != FileType::PLAIN_FOLDER && fileLocation.isEmpty()) {
                qCritical() << QObject::tr("File location is necessary in case file type is not a plain folder");
                return NULL;
            }
        } else {
            fileType = FileType::PLAIN_FOLDER;
            fileLocation = path;
        }
        if (type.isEmpty() || shortName.isEmpty() || path.isEmpty() || uuid.isEmpty()) {
            qCritical() << QString(QObject::tr("The entity %1 is bad declared")).arg(i);
            return NULL;
        }

        entityDescriptor.type = type.toStdString();
        entityDescriptor.shortName = shortName.toStdString();
        entityDescriptor.longName = longName.toStdString();
        entityDescriptor.description = description.toStdString();
        entityDescriptor.path = path.toStdString();
        entityDescriptor.uuid = uuid.toStdString();
        entityDescriptor.fileType = fileType;
        entityDescriptor.fileLocation = fileLocation.toStdString();
        entityDescriptor.parentId = parentId.toLongLong();

        entityDescriptorList.append(entityDescriptor);
    }

    archiveDescriptor->entities = entityDescriptorList.toStdList();


    //TOPOLOGY
    QDomNodeList topologyEntriesList = hnsArchive.elementsByTagName("TopologyEntries");
    if (topologyEntriesList.size() != 1) {
        qCritical() << QObject::tr("There should be exactly one <TopologyEntries> tag in the archive descriptor file");
        return NULL;
    }

    QDomElement topologyEntries = topologyEntriesList.item(0).toElement();
    QDomNodeList topologyEntryList = topologyEntries.childNodes();
    QList<TopologyDescriptor> topologyDescriptorList;

    for (int i = 0; i < topologyEntryList.size(); i++) {
        QDomNode node = topologyEntryList.item(i);
        TopologyDescriptor topologyDescriptor;
        QDomElement element = node.toElement();
        QString shortName = element.attribute("shortName");
        QString longName = element.attribute("longName");
        QString description = element.attribute("description");
        QString path = element.attribute("path");
        QString uuid = element.attribute("uuid");

        if (shortName.isEmpty() || uuid.isEmpty()) {
            qCritical() << QObject::tr("The topology is bad declared");
            return NULL;
        }
        if (path.isEmpty())
            path = "/";

        QDomNodeList dependenciesNodeList = element.elementsByTagName("Dependencies");
        if (dependenciesNodeList.size() != 1) {
            qCritical() << QString(QObject::tr("There should be exactly one <Dependencies> tag in the archive descriptor file at the topology %1")).arg(i);
            return NULL;
        }

        QDomElement dependencies = dependenciesNodeList.item(0).toElement();
        QDomNodeList dependencyList = dependencies.childNodes();
        QList<std::string> uuidList;
        for (int j = 0; j< dependencyList.size(); j++){
            QDomNode uuidNode = dependencyList.item(j);
            QDomElement uuidElement = uuidNode.toElement();
            QString uuid = uuidElement.attribute("uuid");
            if (uuid.isEmpty()) {
                qCritical() << QString(QObject::tr("The UUID must not be empty at topology %1 in dependency %2").arg(i).arg(j));
                return NULL;
            }
            uuidList.append(uuid.toStdString());
        }

        topologyDescriptor.dependencies = uuidList.toStdList();
        topologyDescriptor.shortName = shortName.toStdString();
        topologyDescriptor.longName = longName.toStdString();
        topologyDescriptor.description = description.toStdString();
        topologyDescriptor.path = path.toStdString();
        topologyDescriptor.uuid = uuid.toStdString();

        topologyDescriptorList.append(topologyDescriptor);
    }
    archiveDescriptor->topologies = topologyDescriptorList.toStdList();

    QDomElement customResourcesElement = hnsArchive.elementsByTagName("CustomResources").item(0).toElement();
    QDomNodeList customGraphicList = customResourcesElement.childNodes();
    for (int i = 0, size = customGraphicList.size() ; i < size ; ++i) {
        QDomElement customGraphicNode = customGraphicList.at(i).toElement();
        QString customGraphic = customGraphicNode.text();
        archiveDescriptor->customResources.push_back(customGraphic.toStdString());
    }

    QDomNodeList baseEntitiesElements = hnsArchive.elementsByTagName("BaseEntities");
    if (baseEntitiesElements.length() == 1) {
        QDomElement baseEntitiesElement = baseEntitiesElements.item(0).toElement();
        QDomNodeList baseEntityList = baseEntitiesElement.childNodes();
        for (int i = 0, size = baseEntityList.size() ; i < size ; ++i) {
            QString childUUID = baseEntityList.at(i).toElement().attribute("childUUID");
            QString parentUUID = baseEntityList.at(i).toElement().attribute("parentUUID");
            archiveDescriptor->baseEntities[childUUID.toStdString()] = parentUUID.toStdString();
        }
    }


    importableArchive->archive = archiveDescriptor;
    return importableArchive;
}

QString ImportableArchiveSerializer::serialize(const ArchiveDescriptorPtr &archiveDescriptor)
{
    QString path = QString("/tmp/hnsArchive-") + QString::number(archiveDescriptor->creationDateTime) + QString(".xml");

    QDomDocument doc("HnsArchiveConfig");
    QDomElement root = doc.createElement("HnsArchiveConfig");
    doc.appendChild(root);

    QDomElement hnsArchiveElement = doc.createElement("HnsArchive");
    //add info about hnsArchive
    hnsArchiveElement.setAttribute("version", QString("%1").arg(Constants::CURRENT_VERSION_ARCHIVE_XML));
    hnsArchiveElement.setAttribute("name", QString::fromStdString(archiveDescriptor->name));
    hnsArchiveElement.setAttribute("creationDateTime", QString("%1").arg(archiveDescriptor->creationDateTime));

    QDomElement folderEntriesElements = doc.createElement("FolderEntries");
    //add informations about folderentries
    foreach (const FolderDescriptor &fd, archiveDescriptor->folders) {
        QDomElement folderEntryElement = doc.createElement("FolderEntry");
        folderEntryElement.setAttribute("shortName", QString::fromStdString(fd.shortName));
        folderEntryElement.setAttribute("longName", QString::fromStdString(fd.longName));
        folderEntryElement.setAttribute("description", QString::fromStdString(fd.description));
        QString list = "";
        foreach (const std::string &str, fd.tags) {
            list += QString::fromStdString(str) + ",";
        }
        list.remove(list.size() - 1 , 1);

        folderEntryElement.setAttribute("tags", list);
        folderEntryElement.setAttribute("id", QString("%1").arg(fd.id));
        folderEntryElement.setAttribute("parentId", QString::number(fd.parentId));
        folderEntriesElements.appendChild(folderEntryElement);
    }
    hnsArchiveElement.appendChild(folderEntriesElements);

    QDomElement entityEntriesElements = doc.createElement("EntityEntries");
    //add informations about entitiesEntries
    foreach(const EntityDescriptor &entityDescriptor, archiveDescriptor->entities) {
        QDomElement entityEntryElement = doc.createElement("EntityEntry");
        entityEntryElement.setAttribute("type", QString::fromStdString(entityDescriptor.type));
        entityEntryElement.setAttribute("shortName", QString::fromStdString(entityDescriptor.shortName));
        entityEntryElement.setAttribute("longName", QString::fromStdString(entityDescriptor.longName));
        entityEntryElement.setAttribute("path", QString::fromStdString(entityDescriptor.uuid));
        entityEntryElement.setAttribute("uuid", QString::fromStdString(entityDescriptor.uuid));
        entityEntryElement.setAttribute("fileType", entityDescriptor.fileType);
        entityEntryElement.setAttribute("fileLocation", QString::fromStdString(entityDescriptor.fileLocation));
        entityEntryElement.setAttribute("parentId", QString::number(entityDescriptor.parentId));
        entityEntriesElements.appendChild(entityEntryElement);
    }

    hnsArchiveElement.appendChild(entityEntriesElements);

    QDomElement topologiesEntriesElements = doc.createElement("TopologyEntries");
    //add informations about Topologies
    foreach (const TopologyDescriptor &topologyDescriptor, archiveDescriptor->topologies) {
        QDomElement topologyEntryElement = doc.createElement("TopologyEntry");
        topologyEntryElement.setAttribute("shortName", QString::fromStdString(topologyDescriptor.shortName));
        topologyEntryElement.setAttribute("longName", QString::fromStdString(topologyDescriptor.longName));
        topologyEntryElement.setAttribute("description", QString::fromStdString(topologyDescriptor.description));

        topologyEntryElement.setAttribute("path", QString::fromStdString(topologyDescriptor.uuid));
        topologyEntryElement.setAttribute("uuid", QString::fromStdString(topologyDescriptor.uuid));
        //add dependencies
        QDomElement dependenciesElements = doc.createElement("Dependencies");
        foreach (const std::string &str, topologyDescriptor.dependencies) {
            QDomElement dependencyElement = doc.createElement("Dependency");
            dependencyElement.setAttribute("uuid", QString::fromStdString(str));
            dependenciesElements.appendChild(dependencyElement);
        }
        topologyEntryElement.appendChild(dependenciesElements);
        topologiesEntriesElements.appendChild(topologyEntryElement);
    }

    hnsArchiveElement.appendChild(topologiesEntriesElements);

    QDomElement customResourcesElement = doc.createElement("CustomResources");
    foreach (const std::string &customResource,archiveDescriptor->customResources) {
        QDomElement customResourceElement = doc.createElement("CustomGraphic");
        QDomText domText = doc.createTextNode(QString::fromStdString(customResource));
        customResourceElement.appendChild(domText);
        customResourcesElement.appendChild(customResourceElement);
    }
    hnsArchiveElement.appendChild(customResourcesElement);

    QDomElement baseEntitiesElement = doc.createElement("BaseEntities");
    for (auto it = archiveDescriptor->baseEntities.begin() ; it != archiveDescriptor->baseEntities.end() ; ++it) {
        QDomElement baseEntity = doc.createElement("BaseEntity");
        baseEntity.setAttribute("childUUID", QString::fromStdString(it->first));
        baseEntity.setAttribute("parentUUID", QString::fromStdString(it->second));
        baseEntitiesElement.appendChild(baseEntity);
    }
    hnsArchiveElement.appendChild(baseEntitiesElement);

    root.appendChild(hnsArchiveElement);

    QFile file(path);
    if (!file.open(QIODevice::WriteOnly | QFile::Text)) {
        qCritical() << tr("Unable to open file %1").arg(path);
        return 0;
    }
    QTextStream out(&file);
    const int xmlIndentSize = 2;
    doc.save(out, xmlIndentSize);
    file.close();

    return path;
}

} // namespace Hns
