// Project headers
#include "libarchivehandler.h"

// Qt headers
#include <QFile>
#include <QFileInfo>
#include <QDebug>
#include <QDir>

struct LibArchiveHandler::ArchiveReadCustomDeleter
{
    static inline void cleanup(struct archive *a)
    {
        if (a) {
            archive_read_free(a);
        }
    }
};

struct LibArchiveHandler::ArchiveWriteCustomDeleter
{
    static inline void cleanup(struct archive *a)
    {
        if (a) {
            archive_write_free(a);
        }
    }
};

LibArchiveHandler::LibArchiveHandler(QString fileName, QObject *parent)
    : QObject(parent)
    , _fileName(fileName)
    , _cachedArchiveEntryCount(0)
    , _extractedFilesSize(0)
    , _archiveReadDisk(archive_read_disk_new())
{
    QFileInfo fileInfo(fileName);
    _archiveSize = fileInfo.size();
}

LibArchiveHandler::~LibArchiveHandler()
{
}

QString LibArchiveHandler::getFilename()
{
    return _fileName;
}

bool LibArchiveHandler::list()
{
    qDebug() << tr("Listing %1").arg(_fileName);
    ArchiveRead archReader(archive_read_new());
    if (!(archReader.data())) {
        return false;
    }
    if (archive_read_support_filter_all(archReader.data()) != ARCHIVE_OK) {
        return false;
    }

    if (archive_read_support_format_all(archReader.data()) != ARCHIVE_OK) {
        return false;
    }
    if (archive_read_open_filename(archReader.data(), QFile::encodeName(_fileName), 10240) != ARCHIVE_OK) {
        qWarning() << QObject::tr("Could not open the archive %1, libarchive cannot handle it").arg(_fileName);
        return false;
    }

    _cachedArchiveEntryCount = 0;
    _extractedFilesSize = 0;
    struct archive_entry *archiveEntry;
    int result;
    while ((result = archive_read_next_header(archReader.data(),&archiveEntry)) == ARCHIVE_OK) {
        const bool entryIsDir = S_ISDIR(archive_entry_mode(archiveEntry));
        if (entryIsDir) {
            QString entryRoot = QDir::fromNativeSeparators(QFile::decodeName(archive_entry_pathname(archiveEntry))).split("/").first()+"/";
            _directories.append(QDir::fromNativeSeparators(QFile::decodeName(archive_entry_pathname(archiveEntry))));
            if (!_rootDirectories.contains(entryRoot)) {
                _rootDirectories.append(entryRoot);
            }
        }
        _extractedFilesSize += (qlonglong)archive_entry_size(archiveEntry);
        _cachedArchiveEntryCount++;
        archive_read_data_skip(archReader.data());
    }
    qDebug() << tr("The uncompressed size is %1 bytes").arg(_extractedFilesSize);
    qDebug() << tr("It contains %1 entries").arg(_cachedArchiveEntryCount);
    qDebug() << tr("First level directories are :") << _rootDirectories;
    if (result != ARCHIVE_EOF) {
        qWarning() << QObject::tr("The archive reading failed with the following error:%1").arg(QString::fromLatin1(archive_error_string(archReader.data())));
        return false;
    }
    return archive_read_close(archReader.data()) == ARCHIVE_OK;
}

bool LibArchiveHandler::extractOnDisk(QStringList files, const QString &destinationDirectory/*, ExtractionOptions options*/, bool removeArchive)
{
    if (_rootDirectories.isEmpty()) {
        _constructRootDirectories();
    }
    qDebug() << tr("Extracting %1").arg(_fileName);
    if (!files.isEmpty()) {
        qDebug() << tr("Extracting ") << files << tr("in %1").arg(_fileName);
    }
    QString directoryBeforeExtract = QDir::currentPath();
    QDir::setCurrent(destinationDirectory);

    const bool extractAll = files.isEmpty();

    ArchiveRead arch(archive_read_new());
    if (!_checkArchiveRead(arch)) {
        return false;
    }
    ArchiveWrite writer(archive_write_disk_new());
    if (!_checkArchiveWrite(writer)) {
        return false;
    }
    struct archive_entry *entry;
    while (archive_read_next_header(arch.data(), &entry) == ARCHIVE_OK) {
        const bool entryIsDir = S_ISDIR(archive_entry_mode(entry));
        QString entryName = QDir::fromNativeSeparators(QFile::decodeName(archive_entry_pathname(entry)));
        if (entryName.startsWith(QLatin1Char( '/' ))) {
            //for now we just can't handle absolute filenames in a tar archive.
            //TODO: find out what to do here!!
            qWarning() << QObject::tr("This archive contains archive entries with absolute paths");
            QDir::setCurrent(directoryBeforeExtract);
            return false;
        }
        bool extractThisEntry = false;
        if (!extractAll) {
            foreach (const QString &root, _rootDirectories) {
                if (files.contains(root) && !files.contains(entryName) && entryName.startsWith(root)) {
                    extractThisEntry = true;
                }
            }
        }
        if (!extractAll && (files.contains(entryName) || (extractThisEntry))) {

            // entryFI is the fileinfo pointing to where the file will be
            // written from the archive
            QFileInfo entryFI(entryName);
            const QString fileWithoutPath(entryFI.fileName());
            //if there is an already existing directory:
            if (entryIsDir && entryFI.exists()) {
                if (entryFI.isWritable()) {
                    qDebug() << tr("Warning, existing, but writable dir");
                } else {
                    qDebug() << tr("Warning, existing, but non-writable dir. skipping");
                    archive_entry_clear(entry);
                    archive_read_data_skip(arch.data());
                    continue;
                }
            }
            int header_response;
            qDebug() << tr("Writing %1 to %2").arg(fileWithoutPath,entryName);

            if ((header_response = archive_write_header(writer.data(), entry)) == ARCHIVE_OK) {
                _copyDataOnDisk(arch.data(), writer.data());
            } else if (header_response == ARCHIVE_WARN) {
                qDebug() << tr("Warning while writing %1").arg(entryName);
            } else {
                qDebug() << tr("Writing header failed with error code %1 while attempting to write %2").arg(QString::number(header_response), entryName);
                qDebug() << archive_error_string(writer.data());
            }
            archive_entry_clear(entry);
        } else if (extractAll){
            int flags = ARCHIVE_EXTRACT_TIME;
            flags |= ARCHIVE_EXTRACT_SECURE_NODOTDOT;
            flags |= ARCHIVE_EXTRACT_SECURE_SYMLINKS;
            archive_read_extract(arch.data(), entry, flags);
        } else {
            archive_read_data_skip(arch.data());
        }
    }
    QDir::setCurrent(directoryBeforeExtract);
    if (removeArchive) {
        qDebug() << tr("I remove %1").arg(_fileName);
        QFile::remove(_fileName);
    }
    return archive_read_close(arch.data()) == ARCHIVE_OK;
}

QList<QByteArray> LibArchiveHandler::extractOnMemory(QStringList files, bool removeArchive)
{
    qDebug() << tr("Extracting %1").arg(_fileName);
    if (!files.isEmpty()) {
        qDebug() << tr("Extracting ") << files << tr("in %1").arg(_fileName);
    }
    QList<QByteArray> returnList;
    const bool extractAll = files.isEmpty();

    ArchiveRead arch(archive_read_new());
    if (!_checkArchiveRead(arch)) {
        return QList<QByteArray>();
    }

    struct archive_entry *entry;

    while (archive_read_next_header(arch.data(), &entry) == ARCHIVE_OK) {

        QString entryName = QDir::fromNativeSeparators(QFile::decodeName(archive_entry_pathname(entry)));
        if (entryName.startsWith(QLatin1Char( '/' ))) {
            //for now we just can't handle absolute filenames in a tar archive.
            //TODO: find out what to do here!!
            qWarning() << QObject::tr("This archive contains archive entries with absolute paths");
            return QList<QByteArray>();
        }
        //In case files contains a dir name
        foreach (const QString& dir, _directories) {
            if (files.contains(dir) && entryName.startsWith(dir) && (entryName != dir) ) {
                files << entryName;
            }
        }
        if (files.contains(entryName) || extractAll) {
            returnList.append(_copyDataOnMemory(arch.data()));
            archive_entry_clear(entry);
        } else {
            archive_read_data_skip(arch.data());
        }
    }
    if (!archive_read_close(arch.data()) == ARCHIVE_OK) {
        return QList<QByteArray>();
    }
    if (removeArchive) {
        qDebug() << tr("I remove %1").arg(_fileName);
        QFile::remove(_fileName);
    }
    return returnList;
}

qint64 LibArchiveHandler::getSize()
{
    return _archiveSize;
}

void LibArchiveHandler::_copyDataOnDisk(archive *source, archive *dest)
{
    char buff[10240];
    ssize_t readBytes;

    readBytes = archive_read_data(source, buff, sizeof(buff));
    while (readBytes > 0) {
        archive_write_data(dest, buff, readBytes);
        if (archive_errno(dest) != ARCHIVE_OK) {
            qWarning() << tr("Error while extracting %1 (error nb = %2)").arg(QString::fromLatin1(archive_error_string(dest)), QString::number(archive_errno(dest)));
            return;
        }
        readBytes = archive_read_data(source, buff, sizeof(buff));
    }
}

QByteArray LibArchiveHandler::_copyDataOnMemory(archive *source)
{
    QByteArray ba;
    char *buff = new char[10240];
    ssize_t readBytes;
    readBytes = archive_read_data(source, buff, 10240);
    ba.clear();
    ba.resize(0);
    while (readBytes > 0) {
        ba.append(buff,readBytes);
        readBytes = archive_read_data(source, buff, 10240);
    }
    delete[] buff;
    return ba;
}

bool LibArchiveHandler::_checkArchiveRead(LibArchiveHandler::ArchiveRead &archive)
{
    if (!(archive.data())) {
        return false;
    }
    if (archive_read_support_filter_all(archive.data()) != ARCHIVE_OK) {
        return false;
    }

    if (archive_read_support_format_all(archive.data()) != ARCHIVE_OK) {
        return false;
    }
    if (archive_read_open_filename(archive.data(), QFile::encodeName(_fileName), 10240) != ARCHIVE_OK) {
        qWarning() << QObject::tr("Could not open the archive %1. libarchive cannot handle it").arg(_fileName);
        return false;
    }
    return true;
}

bool LibArchiveHandler::_checkArchiveWrite(LibArchiveHandler::ArchiveWrite &archive)
{
    if (!(archive.data())) {
        return false;
    }
    // Workaround because ARCHIVE_EXTRACT_TIME and ARCHIVE_EXTRACT_SECURE_NODOTDOT are not int
    int flags = ARCHIVE_EXTRACT_TIME;
    flags |= ARCHIVE_EXTRACT_SECURE_NODOTDOT;
    flags |= ARCHIVE_EXTRACT_SECURE_SYMLINKS;
    if (!(archive_write_disk_set_options(archive.data(), flags)) == ARCHIVE_OK) {
        return false;
    }
    return true;
}
bool LibArchiveHandler::compress(QString srcPath, QString dstPath, int flag)
{
    struct archive *arch;
    int r;
    arch = archive_write_new();
    r = archive_write_set_format_gnutar(arch);
    if (r != ARCHIVE_OK) {
        qDebug() << QObject::tr("Error format") << archive_error_string(arch);
    }
    if (flag == COMPRESS_TAR_GZ) {
        r = archive_write_add_filter_gzip(arch);
        dstPath += ".tar.gz";
    }
    else if (flag == COMPRESS_TAR) {
        r = archive_write_add_filter_none(arch);
        dstPath += ".tar";
    }
    if (r != ARCHIVE_OK) {
        qDebug() << QObject::tr("Setting the compression method failed with the following error : %1").arg(archive_error_string(arch));
        return false;
    }

    r = archive_write_open_filename(arch, QFile::encodeName(dstPath));
    if (r != ARCHIVE_OK) {
        qDebug() << QObject::tr("opening the archive for writing failed with the following error : %1").arg(archive_error_string(arch));
        return false;
    }
    struct archive_entry *entry=NULL;
    struct archive *archReader;
    struct archive_entry *entry2;
    Q_UNUSED (entry2)
    if (r != ARCHIVE_OK) {
        qDebug() << QObject::tr("Error while readink disk");
        archive_entry_free(entry);
        return false;
    }
    if ((archReader = archive_read_disk_new()) == NULL) {
        qDebug() << QObject::tr("Can not create archive reader");
        return false;
    }
    r = archive_read_disk_set_symlink_physical(archReader);
    if (r != ARCHIVE_OK) {
        qDebug() << QObject::tr("Error while setting symlink mode");
        return false;
    }

    r = archive_read_disk_open(archReader, QFile::encodeName(srcPath));
    if (r != ARCHIVE_OK) {
        qDebug() << QObject::tr("Error while setting path");
        return false;
    }
    struct archive *disk = archReader;
    archive_entry_free(entry);
    entry = archive_entry_new();
    while(((r = archive_read_next_header2(disk,entry)) != ARCHIVE_EOF)) {
        if (r != ARCHIVE_OK) {
            qWarning() << QObject::tr("Error while reading header") <<archive_error_string(arch) <<archive_entry_pathname(entry);
        }
        QString fileName(archive_entry_pathname(entry));
        const bool trailingSlash = fileName.endsWith("/");
        QString relativeName = fileName;
        //todo : find a better way because it's dirty
        if (srcPath.contains("/.tmp")) {
            relativeName.replace(srcPath + "/", "");
        }
        else {
            relativeName.replace(srcPath, srcPath.split("/").last()) + (trailingSlash ? "/" : "" );
        }
        if (relativeName.endsWith(".tmp")){
            if (archive_read_disk_can_descend(disk)){
                archive_read_disk_descend(disk);
                archive_entry_free(entry);
                entry = archive_entry_new();
                continue;
            }
        }
        archive_entry_set_pathname(entry, QFile::encodeName(relativeName).constData());
        archive_entry_copy_sourcepath(entry, QFile::encodeName(fileName).constData());
        qDebug() << QObject::tr("Adding %1 to archive").arg(QString(archive_entry_pathname(entry)));
        if (archive_entry_filetype(entry) != AE_IFREG) {
            archive_entry_set_size(entry, 0);
        }
        if (archive_entry_filetype(entry) == AE_IFDIR) {
            int a = archive_write_header(arch,entry);
            if (a != ARCHIVE_OK) {
                qWarning() << QObject::tr("Error while writing header: ") << archive_entry_pathname(entry) << archive_error_string(arch);
                continue;
            }
            if (archive_read_disk_can_descend(disk)){
                archive_read_disk_descend(disk);
                archive_entry_free(entry);
                entry = archive_entry_new();
                continue;
            }
        }
        _writeEntry(arch, entry,archReader);
        archive_entry_free(entry);
        entry = archive_entry_new();
    }
    entry = NULL;
    archive_write_close(arch);
    archive_entry_free(entry);
    archive_read_close(disk);
    return true;
}

void LibArchiveHandler::_writeEntry(archive *arch, archive_entry *entry, archive *archReader)
{
    int e;
    int r;
    e = archive_write_header(arch, entry);
    if (e != ARCHIVE_OK) {
        qWarning() << QObject::tr("Error while writing header:") << archive_entry_pathname(entry) << archive_error_string(arch);
        return;
    }
    if (e >= ARCHIVE_WARN && archive_entry_size(entry) > 0) {
        size_t	bytes_read;
        ssize_t	bytes_written;
        int64_t	offset, progress = 0;
        char *null_buff = NULL;
        const void *buff;
        int buffsize = 64 * 1024;
        while (buffsize < 10240) {
            buffsize *=2;
        }
        buffsize += 16*1024;


        while ((r = archive_read_data_block(archReader, &buff, &bytes_read, &offset)) == ARCHIVE_OK) {
            if (offset > progress) {
                int64_t sparse = offset - progress;
                size_t ns;

                if (null_buff == NULL) {
                    null_buff = (char*)malloc(buffsize);;
                    memset(null_buff, 0, buffsize);
                }
                while (sparse > 0) {
                    if (sparse > (int64_t)buffsize) {
                        ns = buffsize;
                    } else {
                        ns = (size_t)sparse;
                    }
                    bytes_written = archive_write_data(arch, null_buff, ns);
                    if (bytes_written < 0) {
                        /* Write failed; this is bad */
                        qCritical() << QObject::tr("Write failed");
                        return;
                    }
                    if ((size_t)bytes_written < bytes_read) {
                        /* Write was truncated; warn but continue. */
                        qCritical() << QObject::tr("%1: truncated write, file may have grown").arg(QString(archive_entry_pathname((entry))));
                    }
                    progress += bytes_written;
                    sparse -= bytes_written;
                }
            }
            bytes_written = archive_write_data(arch, buff, bytes_read);
            if (bytes_written < 0) {
                /* Write failed; this is bad */
                qCritical() << QObject::tr("Write failed");
                return;
            }
            if ((size_t)bytes_written < bytes_read) {
                /* Write was truncated; warn but continue. */
                qCritical() << QObject::tr("%1: truncated write, file may have grown").arg(QString(archive_entry_pathname((entry))));
                return;
            }
            progress += bytes_written;
        }
    }
}

void LibArchiveHandler::_constructRootDirectories()
{
    foreach (const QString &dir, _directories) {
        QString root = dir.split("/").first()+"/";
        if (!_rootDirectories.contains(root)) {
            _rootDirectories.append(root);
        }
    }
}
