/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "macaddressmanagerI.h"
#include "masterI.h"

// hynesim common headers
#include "common/constants.h"

// diamesh headers
#include <diamesh/diamesh.h>

// ZeroC Ice headers
#include <Ice/ObjectAdapter.h>
#include <Ice/Identity.h>

// Qt headers
#include <QSettings>
#include <QDir>
#include <QFile>
#include <QDateTime>
#include <QDebug>

namespace Hns {

MacAddressManagerI *MacAddressManagerI::_instance = NULL;

MacAddressManagerI::MacAddressManagerI()
{
    _fillVendorPrefixData();
    _id.name = "macAddressManagerI";
    MasterI::instance()->adapter()->add(this, _id);
}

MacAddressManagerI::~MacAddressManagerI()
{
}

MacAddressManagerI *MacAddressManagerI::create()
{
    if (!_instance)
        _instance = new MacAddressManagerI();
    return instance();
}

MacAddressManagerI *MacAddressManagerI::instance()
{
    return _instance;
}

bool MacAddressManagerI::isValid(QString macAddress)
{
    QRegExp validPattern("00(?::[0-9a-fA-F]{2}){5}");
    return (validPattern.exactMatch(macAddress));
}

MacAddressManagerPrx MacAddressManagerI::proxy()
{
    return MacAddressManagerPrx::uncheckedCast(MasterI::instance()->adapter()->createProxy(_id));
}

::Diamesh::StringList MacAddressManagerI::getVendorList(const::Ice::Current &current)
{
    Q_UNUSED(current)
    QList<std::string> vendorList;
    foreach (QString s,_vendorPrefixMap.keys())
        vendorList.append(s.toStdString());
    return vendorList.toStdList();
}

::std::string MacAddressManagerI::getVendorFromPrefix(const ::std::string &macPrefix, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QMapIterator<QString, QStringList> i(_vendorPrefixMap);
    while (i.hasNext()) {
        i.next();
        if (i.value().contains(QString::fromStdString(macPrefix)))
            return i.key().toStdString();
    }
    return QString("").toStdString();
}

QSet<QString> MacAddressManagerI::_getUsedMacAddresses()
{
    QSet<QString> usedMacAddresses;
    for (QMap<Ice::Long, QSet<QString> >::const_iterator it = _usedMacAddresses.begin(); it != _usedMacAddresses.end(); ++it) {
        usedMacAddresses.unite(it.value());
    }
    return usedMacAddresses;
}

QSet<QString> MacAddressManagerI::_getReservedMacAddresses()
{
    QSet<QString> reservedMacAddresses;
    for (QMap<Ice::Long, QSet<QString> >::const_iterator it = _reservedMacAddresses.begin(); it != _reservedMacAddresses.end(); ++it) {
        reservedMacAddresses.unite(it.value());
    }
    return reservedMacAddresses;
}

::std::string MacAddressManagerI::generateRandomAddress(const ::std::string &vendorStr, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    static uint seed = QDateTime::currentDateTime().toTime_t();
    qsrand(seed++);

    QString vendor = QString::fromStdString(vendorStr);
    bool vendorRandom = vendor.isEmpty() || !_vendorPrefixMap.contains(vendor);
    bool allRandom = (_vendorPrefixMap.size() == 0);
    QString addr;
    do {
        if (allRandom) {
            addr = _getRandomMacSuffix().mid(1);
            addr.replace(0, 2, "00");
        } else if(vendorRandom) {
            addr = _getRandomVendorPrefix(_getRandomVendor());
        } else {
            addr = _getRandomVendorPrefix(vendor);
        }

        addr += _getRandomMacSuffix();
    } while (isReserved(addr));
    reserveMacAddress(-1, addr);
    return addr.toStdString();
}

bool MacAddressManagerI::reserveMacAddress(Ice::Long entryId, const QString &macAddress)
{
    QWriteLocker locker(&_macAddressesLock);
    QSet<QString> usedMacAddresses = _usedMacAddresses.value(entryId);
    if( ! MacAddressManagerI::isValid(macAddress) ||
            (_getUsedMacAddresses().contains(macAddress) && !usedMacAddresses.contains(macAddress)))
        return false;

    for (QMap<Ice::Long, QSet<QString> >::const_iterator it = _reservedMacAddresses.begin(); it != _reservedMacAddresses.end(); ++it) {
        if (it.key() != entryId) {
            if (it.value().contains(macAddress))
                return false;
        }
    }

    qDebug() << tr("Reserve mac address") << macAddress;
    QSet<QString> &reservedMacAddresses = _reservedMacAddresses[entryId];
    if (!usedMacAddresses.contains(macAddress) && !reservedMacAddresses.contains(macAddress)) {
        reservedMacAddresses.insert(macAddress);
    }
    return true;
}

bool MacAddressManagerI::registerMacAddress(Ice::Long entryId, const QString &macAddress)
{
    QWriteLocker locker(&_macAddressesLock);
    QSet<QString> &usedMacAddresses = _usedMacAddresses[entryId];
    QSet<QString> &reservedMacAddresses = _reservedMacAddresses[entryId];

    if( ! MacAddressManagerI::isValid(macAddress) ||
            _getUsedMacAddresses().contains(macAddress) ||
            !reservedMacAddresses.contains(macAddress))
        return false;

    qDebug() << tr("Register mac address") << macAddress;
    reservedMacAddresses.remove(macAddress);
    usedMacAddresses.insert(macAddress);

    return true;
}

bool MacAddressManagerI::unregisterMacAddress(Ice::Long entryId, const QString &macAddress)
{
    QWriteLocker locker(&_macAddressesLock);
    QSet<QString> usedMacAddresses = _usedMacAddresses.value(entryId);

    if( ! usedMacAddresses.contains(macAddress))
        return false;

    qDebug() << tr("Unregister mac address") << macAddress;
    usedMacAddresses.remove(macAddress);

    return true;
}

bool MacAddressManagerI::isUsed(const ::std::string &macAddress, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QReadLocker locker(&_macAddressesLock);
    return _getUsedMacAddresses().contains(QString::fromStdString(macAddress));
}

bool MacAddressManagerI::isReserved(const QString &macAddress)
{
    QReadLocker locker(&_macAddressesLock);
    return _getReservedMacAddresses().contains(macAddress) || _getUsedMacAddresses().contains(macAddress);
}

QString MacAddressManagerI::_getRandomVendor()
{
    QMap<QString,QStringList>::const_iterator it = _vendorPrefixMap.constBegin();
    int index = (int)(qrand() / (((double)RAND_MAX + 1)/ _vendorPrefixMap.size()));
    it += index;
    return it.key();
}

QString MacAddressManagerI::_getRandomVendorPrefix(const QString &vendor)
{
    QStringList prefixes = _vendorPrefixMap.value(vendor);
    int size = prefixes.size();
    int index = (int)(qrand() / (((double)RAND_MAX + 1)/ size));
    return prefixes.at(index);
}

QString MacAddressManagerI::_getRandomMacSuffix()
{
    union { int randInt; unsigned char randChar[4];} r;
    r.randInt = qrand();
    QString suffix = QString(":%1:%2:%3").arg(r.randChar[0], 2, 16, QChar('0'))
            .arg(r.randChar[1], 2, 16, QChar('0'))
            .arg(r.randChar[2], 2, 16, QChar('0'));
    return suffix;
}

void MacAddressManagerI::_fillVendorPrefixData()
{
    QSettings settings(Diamesh::instance()->configFileName(), QSettings::IniFormat, this);
    settings.beginGroup(Constants::SETTINGS_HYNESIM_GROUP);
    QString dataPath = settings.value("dataDirectory", ".").toString();
    settings.endGroup();
    if (!dataPath.endsWith("/"))
        dataPath.append("/");

    QDir dataDir(dataPath);
    if (!dataDir.exists()) {
        qWarning() << tr("The directory \"%1\" does not exist.").arg(dataDir.path());
        return;
    }

    QFile dataFile(dataDir.absoluteFilePath("macAddrPrefixes.txt"));
    if (!dataFile.exists()) {
        qWarning() << tr("The file \"%1\" does not exist.").arg(dataFile.fileName());
        return;
    }

    if (!dataFile.open(QIODevice::ReadOnly)) {
        qWarning() << tr("Unable to open the file \"%1\" in readonly mode").arg(dataFile.fileName());
        return;
    }

    QRegExp pattern("\\s*([a-zA-Z0-9]{2}(?:\\-[a-zA-Z0-9]{2}){2})\\s+\\(hex\\)\\s+([^\\s].+)");
    while (!dataFile.atEnd()) {
        QString line = QString::fromLatin1(dataFile.readLine().constData());
        line.chop(1);
        if ( pattern.exactMatch(line) ) {
            QString vendor = pattern.cap(2);
            QString prefix = pattern.cap(1).replace('-',':').toLower();
            _vendorPrefixMap[vendor] += prefix;
        }
    }
}

} // namespace Hns
