/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "masterI.h"
#include "nodeinfoI.h"
#include "entityfactoryI.h"
#include "topologymanagerI.h"
#include "entrypropertiesmanagerI.h"
#include "customresourcemanagerI.h"
#include "catalog/catalogfilestorage.h"
#include "catalog/entitycatalogI.h"
#include "catalog/topologycatalogI.h"
#include "tagmanagerI.h"
#include "jobmanagerI.h"
#include "tracerI.h"
#include "macaddressmanagerI.h"
#include "dynamipsrouterdbI.h"
#include "importer/catalogimporterI.h"

// hynesim common headers
#include "common/constants.h"
#include "common/hns.h"
#include "common/version.h"
#include "common/rightslist.h"

// hynesim server headers
#include "libhnsserver/storagemanager.h"

// diamesh headers
#include <diamesh/diamesh.h>
#include <diamesh/aclmanagerI.h>

// Qt headers
#include <QDebug>
#include <QDateTime>
#include <QSettings>

namespace Hns {

MasterI::MasterI()
    : _Hns_Master()
{
}

MasterI::~MasterI()
{
}

MasterI *MasterI::instance()
{
    return dynamic_cast<Hns::MasterI*>(_instance);
}

EntityCatalogPrx MasterI::getEntityCatalog(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return _entityCatalog->proxy();
}

EntityCatalogI *MasterI::entityCatalog()
{
    return _entityCatalog;
}

TopologyCatalogPrx MasterI::getTopologyCatalog(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return _topologyCatalog->proxy();
}

TopologyCatalogI *MasterI::topologyCatalog()
{
    return _topologyCatalog;
}

TagManagerPrx MasterI::getTagManager(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return _tagManager->proxy();
}

TagManagerI *MasterI::tagManager()
{
    return _tagManager;
}

EntityFactoryPrx MasterI::getEntityFactory(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    Ice::Identity id;
    id.name = "entityFactory";
    return EntityFactoryPrx::uncheckedCast(_adapter->createProxy(id));
}

TopologyManagerPrx MasterI::getTopologyManager(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    Ice::Identity id;
    id.name = "topologyManager";
    return TopologyManagerPrx::uncheckedCast(_adapter->createProxy(id));
}

EntryPropertiesManagerPrx MasterI::getEntryPropertiesManager(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    Ice::Identity id;
    id.name = "entryPropertiesManager";
    return EntryPropertiesManagerPrx::uncheckedCast(_adapter->createProxy(id));
}

EntryPropertiesManagerI *MasterI::entryPropertiesManager()
{
    return _entryPropertiesManager;
}

JobManagerPrx MasterI::getJobManager(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return JobManagerI::instance()->proxy();
}

CatalogImporterPrx MasterI::getCatalogImporter(const Ice::Current &current)
{
    Q_UNUSED(current)
    return CatalogImporterI::instance()->proxy();
}

MacAddressManagerPrx MasterI::getMacAddressManager(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return MacAddressManagerI::instance()->proxy();
}

DynamipsRouterDBPrx  MasterI::getDynamipsRouterDB(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return DynamipsRouterDBI::instance()->proxy();
}

TracerPrx MasterI::getTracer(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return TracerI::get()->proxy();
}

NodeInfoMap MasterI::getEntryNodeInfoMap(const EntityEntryPtr &entry, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    NodeInfoMap map;
    QList<QSharedPointer<NodeInfoI> > infoList = nodeInfoList(entry);
    foreach (const QSharedPointer<NodeInfoI> &info, infoList)
        map[info->getName()] = NodeInfoPrx::uncheckedCast(_adapter->createProxy(info->identity()));
    return map;
}

bool MasterI::getAdvancedMode(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return _advancedMode;
}

Diamesh::VersionInfo MasterI::getVersionInfo(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return Hns::versionInfo();
}

bool MasterI::_init()
{
    if (!Diamesh::MasterI::_init())
        return false;
    QString config = Diamesh::instance()->configFileName();
    _tagManager = new TagManagerI(_adapter, config);
    _entityCatalog = new EntityCatalogI(communicator(), _adapter, config);
    _topologyCatalog = new TopologyCatalogI(communicator(), _adapter, config);
    JobManagerI::create();
    MacAddressManagerI::create();
    CatalogImporterI::create();
    _customResourceManager = new CustomResourceManagerI(config);
    _customResourceManager->_init();

    _entityFactory = new EntityFactoryI();
    Ice::Identity id;
    id.name = "entityFactory";
    _adapter->add(_entityFactory, id);

    _topologyManager = new TopologyManagerI();
    id.name = "topologyManager";
    _adapter->add(_topologyManager, id);

    _entryPropertiesManager = new EntryPropertiesManagerI();
    id.name = "entryPropertiesManager";
    _adapter->add(_entryPropertiesManager, id);

    QSettings settings(config, QSettings::IniFormat);
    settings.beginGroup(Constants::SETTINGS_HYNESIM_GROUP);
    _advancedMode = settings.value("advancedMode", false).toBool();
    bool monitoringEnabled = settings.value("monitoringEnabled", false).toBool();
    if (monitoringEnabled)
        _flags |= MonitoringEnabled;

    _storageManager = new HnsServer::StorageManager(Diamesh::instance()->configFileName());

    return true;
}

bool MasterI::_addNode(const Diamesh::NodePrx &node, const QString &key)
{
    NodePrx hnsNode = NodePrx::checkedCast(node);
    if (!hnsNode) {
        qWarning() << tr("The node isn't a hynesim node");
        return false;
    }

    NodeInfoI *nodeInfo = new NodeInfoI(key, hnsNode);
    return _addNodeInfo(nodeInfo);
}

void MasterI::_registerACLTargetTypes()
{
    Diamesh::MasterI::_registerACLTargetTypes();
    _aclManager->registerACLTargetType(EntityEntry::ice_staticId().c_str(), Rights::ACL_TARGET_ENTITYENTRY);
    _aclManager->registerACLTargetType(TopologyEntry::ice_staticId().c_str(), Rights::ACL_TARGET_TOPOLOGYENTRY);
    _aclManager->registerACLTargetType(Folder::ice_staticId().c_str(), Rights::ACL_TARGET_FOLDER);
}

void MasterI::_registerRights()
{
    Diamesh::MasterI::_registerRights();
    QStringList prefixes;
    prefixes << Rights::ENTITY_ENTRY_PREFIX << Rights::TOPOLOGY_ENTRY_PREFIX;

    foreach (const QString& prefix, prefixes) {
        _aclManager->registerAccessRight(prefix + Rights::ENTRY_SUFFIX_READ);
        _aclManager->registerAccessRight(prefix + Rights::ENTRY_SUFFIX_DELETE);
        _aclManager->registerAccessRight(prefix + Rights::ENTRY_SUFFIX_UPDATE_ENTRY);
        _aclManager->registerAccessRight(prefix + Rights::ENTRY_SUFFIX_USE);
    }
    QString entityEntryPrefix(Rights::ENTITY_ENTRY_PREFIX);
    _aclManager->registerAccessRight(entityEntryPrefix + Rights::ENTRY_SUFFIX_USE_CLONE);
    _aclManager->registerAccessRight(entityEntryPrefix + Rights::ENTRY_SUFFIX_USE_INSTANTIATE);
    _aclManager->registerAccessRight(entityEntryPrefix + Rights::ENTRY_SUFFIX_USE_DISCARD_SAVE_STATE);
    _aclManager->registerAccessRight(entityEntryPrefix + Rights::ENTRY_SUFFIX_USE_BUILD_MODEL_FROM);

    QString folderPrefix(Rights::FOLDER_PREFIX);
    _aclManager->registerAccessRight(folderPrefix + Rights::FOLDER_SUFFIX_READ);
    _aclManager->registerAccessRight(folderPrefix + Rights::FOLDER_SUFFIX_DELETE);
    _aclManager->registerAccessRight(folderPrefix + Rights::FOLDER_SUFFIX_UPDATE_FOLDER);
}

QList<QSharedPointer<NodeInfoI> > MasterI::nodeInfoList(const::Hns::EntityEntryPtr &entry, QHash<QString, QList<Hns::EntityEntryPtr> > alreadyComputedEntries)
{
    QList<QSharedPointer<Diamesh::NodeInfoI> > list = Diamesh::MasterI::nodeInfoList();
    QList<QSharedPointer<NodeInfoI> > hnsList;
    foreach(const QSharedPointer<Diamesh::NodeInfoI> &diameshNodeInfo, list) {
        QSharedPointer<NodeInfoI> hnsNodeInfo = qSharedPointerDynamicCast<NodeInfoI>(diameshNodeInfo);
        if (!hnsNodeInfo)
            continue;
        // alreadyComputedEntries contains all the entities that have been assigned to a node
        // that node is the key of the QHash
        if (!alreadyComputedEntries.keys().contains(QString::fromStdString(hnsNodeInfo->getName()))) {
            if (hnsNodeInfo->canBuild(entry, ::Ice::Current())) {
                hnsList.append(hnsNodeInfo);
            }
        } else {
            if (hnsNodeInfo->canBuildAll(entry, alreadyComputedEntries.value(QString::fromStdString(hnsNodeInfo->getName())).toStdList(), ::Ice::Current())) {
                hnsList.append(hnsNodeInfo);
            }
        }
    }
    return hnsList;
}

QSharedPointer<NodeInfoI> MasterI::getSuitableNode(const ::Hns::EntityEntryPtr &entry, QHash<QString, QList<EntityEntryPtr> > alreadyComputedEntries)
{
    QList<QSharedPointer<NodeInfoI> > list = nodeInfoList(entry, alreadyComputedEntries);
    if (list.size() == 0) {
        qCritical() << tr("No node is able to build the entity : %1.").arg(QString::fromStdString(entry->shortName));
        return QSharedPointer<NodeInfoI>(NULL);
    }

    static uint seed = QDateTime::currentDateTime().toTime_t();
    qsrand(seed++);

    int index = (int)(qrand() / (((double)RAND_MAX + 1)/ list.size()));
    return list.at(index);
}

void MasterI::refreshEntityClassSlot(QString className)
{
    emit refreshClassName(className);
}

CustomResourceManagerPrx MasterI::getCustomResourceManager(const Ice::Current &current)
{
    Q_UNUSED(current)
    return _customResourceManager->proxy();
}

CustomResourceManagerI *MasterI::customResourceManager()
{
    return _customResourceManager;
}

CatalogFileStorage *MasterI::getFirstEntityFileStorage()
{
    return _entityCatalog->getDefaultFileStorage();
}

HnsServer::StorageManager *MasterI::storageManager()
{
    return _storageManager;
}

} // namespace Hns
