/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "sqluserstorage.h"

// Generated hynesim headers
#include "common/usermanager.h"

// diamesh headers
#include <diamesh/sqldatabase.h>

// Qt headers
#include <QList>
#include <QDebug>
#include <QSqlRecord>

// C++ std headers
#include <stdexcept>

namespace Hns {

SqlUserStorage::SqlUserStorage(Diamesh::SqlDatabase *db)
    : Diamesh::SqlUserStorage(db)
{
}

// static
SqlUserStorage* SqlUserStorage::create(Diamesh::SqlDatabase *db)
{
    SqlUserStorage * storage = new SqlUserStorage(db);
    storage->init();
    return storage;
}

Diamesh::UserPtr SqlUserStorage::getUser(::Ice::Long userId)
{
    QReadLocker locker(&_usersLock);
    if (!_usersHash.contains(userId)) {
        qCritical() << QObject::tr("No user with id %1 exists").arg(userId);
        return 0;
    }

    UserPtr hashUser = UserPtr::dynamicCast(_usersHash.value(userId));
    return new User(*hashUser);
}

Ice::Long SqlUserStorage::addUser(const ::Diamesh::UserPtr &user, const QHash<QString, QVariant> &additionalFields)
{
    UserPtr hnsUser = UserPtr::dynamicCast(user);
    if (!hnsUser) {
        qCritical() << QObject::tr("User is not a Hns user, unable to add it to sql user storage");
        return 0;
    }

    QHash<QString, QVariant> fields(additionalFields);
    fields["firstname"] = hnsUser->firstname.c_str();
    fields["lastname"] = hnsUser->lastname.c_str();
    fields["isAdvanced"] = hnsUser->isAdvanced;
    return Diamesh::SqlUserStorage::addUser(user, fields);
}

bool SqlUserStorage::updateUser(const ::Diamesh::UserPtr &user, const QHash<QString, QVariant> &additionalContent)
{
    UserPtr hnsUser = UserPtr::dynamicCast(user);
    if (!hnsUser) {
        qCritical() << QObject::tr("User is not a Hns user, unable to update it in sql user storage");
        return 0;
    }

    QHash<QString, QVariant> content = additionalContent;
    content["firstname"] = hnsUser->firstname.c_str();
    content["lastname"] = hnsUser->lastname.c_str();
    content["isAdvanced"] = hnsUser->isAdvanced;
    if (!Diamesh::SqlUserStorage::updateUser(user, content))
        return false;

    UserPtr hnsLocalUser = UserPtr::dynamicCast(_usersHash.value(user->id));

    if (!hnsLocalUser) {
        qCritical() << QObject::tr("Local user is not a Hns user, unable to update it in sql user storage");
        return 0;
    }
    hnsLocalUser->firstname = hnsUser->firstname;
    hnsLocalUser->lastname = hnsUser->lastname;
    hnsLocalUser->isAdvanced = hnsUser->isAdvanced;
    return true;
}

void SqlUserStorage::_loadFromDb()
{
    QWriteLocker usersLocker(&_usersLock);
    QWriteLocker groupsLocker(&_groupsLock);

    QList<QSqlRecord> results;
    if (_db->executeQuery("select * from users", QList<QVariant>(), &results) == -1)
        throw std::runtime_error("SqlUserStorage: Unable to get users from DB");

    foreach(const QSqlRecord &r, results) {
        UserPtr user = new User();
        user->id = r.value("id").toLongLong();
        user->name = r.value("name").toString().toStdString();
        user->isEnabled = r.value("isEnabled").toBool();
        user->createdOn = r.value("createdOn").toLongLong();
        user->modifiedOn = r.value("modifiedOn").toLongLong();
        user->firstname = r.value("firstname").toString().toStdString();
        user->hSetId = r.value("hsetid").toLongLong();
        user->lastname = r.value("lastname").toString().toStdString();
        user->isAdvanced = r.value("isAdvanced").toBool();
        _usersHash.insert(user->id, user);
        _usersNamesIdsHash.insert(user->name.c_str(), user->id);
    }

    results.clear();
    if (_db->executeQuery("select * from groups", QList<QVariant>(), &results) == -1)
        throw std::runtime_error("SqlUserStorage: Unable to get groups from DB");

    foreach(const QSqlRecord &r, results) {
        Diamesh::GroupPtr group = new Diamesh::Group();
        group->id = r.value("id").toLongLong();
        group->name = r.value("name").toString().toStdString();
        group->createdOn = r.value("createdOn").toLongLong();
        group->modifiedOn = r.value("modifiedOn").toLongLong();
        group->hSetId = r.value("hsetid").toLongLong();
        _groupsHash.insert(group->id, group);
    }

    results.clear();
    if (!_db->executeQuery("select * from join_groups_users", QList<QVariant>(), &results) == -1)
        throw std::runtime_error("SqlUserStorage: Unable to get join table from DB");

    foreach(const QSqlRecord &r, results) {
        qulonglong groupId = r.value("groupId").toLongLong();
        qulonglong userId = r.value("userId").toLongLong();

        Diamesh::GroupPtr group = _groupsHash.value(groupId);
        UserPtr user = UserPtr::dynamicCast(_usersHash.value(userId));

        // ignore unknown groups and users
        if (!group || !user)
            continue;

        group->userList.push_back(userId);
        user->groupList.push_back(groupId);
    }
}

} // namespace Hns
