/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "tagmanagerI.h"

// hynesim common headers
#include "common/constants.h"

// diamesh headers
#include <diamesh/sqldatabase.h>

// ZeroC Ice headers
#include <Ice/ObjectAdapter.h>
#include <Ice/Identity.h>
#include <Ice/Current.h>

// Qt headers
#include <QSettings>
#include <QDebug>
#include <QSqlField>
#include <QSqlRecord>

namespace Hns {

TagManagerI::TagManagerI(Ice::ObjectAdapterPtr adapter, const QString &settingsFileName)
    : _adapter(adapter)
{
    _id.name = "tagManagerI";
    _adapter->add(this, _id);

    QSettings settings(settingsFileName, QSettings::IniFormat, this);
    settings.beginGroup(Constants::SETTINGS_HYNESIM_GROUP);
    QString dbName(settings.value("catalog/dbName", "/tmp/hnsCatalogI.db").toString());
    _database = new Diamesh::SqlDatabase("QSQLITE", dbName);
    settings.endGroup();
    _init();
}

TagManagerI::~TagManagerI()
{
}

TagManagerPrx TagManagerI::proxy()
{
    return TagManagerPrx::uncheckedCast(_adapter->createProxy(_id));
}

bool TagManagerI::addTag(Ice::Long elementId, const std::string &tagName, const Ice::Current &current)
{
    Q_UNUSED(current)
    QWriteLocker locker(&_tagsLock);
    QString qTagName = QString::fromStdString(tagName);
    qlonglong tagId = _tags.key(qTagName, -1);
    if (tagId == -1) {
        tagId = _database->executeQuery("INSERT INTO tags (name) VALUES(?)", qTagName);
        if (!tagId) {
            qCritical() << tr("Unable to add tag %1 to database").arg(qTagName);
            return false;
        }
        _tags.insert(tagId, qTagName);
    }
    if (!_tagsItems.contains(elementId))
        _tagsItems.insert(elementId, QList<qlonglong>());

    if (_tagsItems[elementId].contains(tagId)) {
        qCritical() << tr("The catalog item already has this tag, unable to add it twice");
        return false;
    }
    QString query("INSERT INTO join_elements_tags (elementId, tagId) VALUES(?, ?)");
    QVariantList values;
    values.append(qlonglong(elementId));
    values.append(qlonglong(tagId));
    if (_database->executeQuery(query, values) < 1) {
        qCritical() << tr("Unable to add tag %1 to catalog item %2").arg(qTagName).arg(elementId);
        return false;
    }
    _tagsItems[elementId].append(tagId);
    return true;
}

bool TagManagerI::addTags(qlonglong elementId, Diamesh::StringList tags)
{
    foreach (const std::string &tag, tags) {
        if (!addTag(elementId, tag))
            return false;
    }
    return true;
}

bool TagManagerI::removeTag(Ice::Long elementId, const std::string &tagName, const Ice::Current &current)
{
    Q_UNUSED(current)
    QWriteLocker locker(&_tagsLock);
    QString qTagName = QString::fromStdString(tagName);
    if (!_tagsItems.contains(elementId)) {
        qCritical() << tr("Catalog item %1 has no tag").arg(elementId);
        return false;
    }
    qlonglong tagId = _tags.key(qTagName, -1);
    if (tagId == -1) {
        qCritical() << tr("No tag with name %1 exists").arg(qTagName);
        return false;
    }
    if (!_tagsItems[elementId].contains(tagId)) {
        qCritical() << tr("Catalog Item %1 does not have tag %2").arg(elementId).arg(qTagName);
        return false;
    }

    QString query("DELETE FROM join_elements_tags WHERE elementId=? AND tagId=?");
    QVariantList values;
    values.append(qlonglong(elementId));
    values.append(qlonglong(tagId));
    if (_database->executeQuery(query, values) < 0) {
        qCritical() << tr("Unable to remove tag %1 link to catalog item %2 in database").arg(qTagName).arg(elementId);
        return false;
    }
    _tagsItems[elementId].removeAll(tagId);
    return true;
}

bool TagManagerI::removeTags(Ice::Long elementId, const Ice::Current &current)
{
    Q_UNUSED(current)
    QWriteLocker locker(&_tagsLock);
    if (!_tagsItems.contains(elementId)) {
        qCritical() << tr("Catalog item %1 has no tag").arg(elementId);
        return false;
    }
    QString query("DELETE FROM join_elements_tags WHERE elementId=?");
    QVariantList values = QVariantList() << qlonglong(elementId);
    if (_database->executeQuery(query, values) < 0) {
        qCritical() << tr("Unable to remove tags for catalog item %1 in database").arg(elementId);
        return false;
    }
    _tagsItems.remove(elementId);
    return true;
}

Diamesh::StringList TagManagerI::tags(qlonglong elementId)
{
    QReadLocker locker(&_tagsLock);
    Diamesh::StringList ret;
    if (!_tagsItems.contains(elementId))
        return ret;
    QList<qlonglong> tags = _tagsItems.value(elementId);
    foreach (qlonglong tagId, tags) {
        ret.push_back(_tags.value(tagId).toStdString());
    }
    return ret;
}

bool TagManagerI::updateTags(qlonglong elementId, Diamesh::StringList newTags)
{
    QWriteLocker locker(&_tagsLock);

    foreach (const std::string &newTag, newTags) {
        QString qNewtag = QString::fromStdString(newTag);
        qlonglong tagId = _tags.key(qNewtag, -1);
        if (tagId < 0) {
            tagId = _database->executeQuery("INSERT INTO tags (name) VALUES(?)", qNewtag);
            if (tagId < 0) {
                qCritical() << tr("Unable to add tag %1 to database").arg(qNewtag);
                return false;
            }
            _tags.insert(tagId, qNewtag);
        }
    }

    if (!_tagsItems.contains(elementId))
        _tagsItems.insert(elementId, QList<qlonglong>());

    QSet<qlonglong> newTagsSet;
    foreach (const std::string &newTag, newTags) {
        newTagsSet << _tags.key(QString::fromStdString(newTag));
    }

    QSet<qlonglong> tagsToRemove = _tagsItems.value(elementId).toSet().subtract(newTagsSet);
    QSet<qlonglong> tagsToAdd = newTagsSet.subtract(_tagsItems.value(elementId).toSet());

    foreach (qlonglong tagToRemove, tagsToRemove) {
        QString query("DELETE FROM join_elements_tags WHERE elementId=? AND tagId=?");
        QVariantList values;
        values.append(qlonglong(elementId));
        values.append(qlonglong(tagToRemove));
        if (_database->executeQuery(query, values) < 0) {
            qCritical() << tr("Unable to remove tag %1 link to catalog item %2 in database").arg(_tags.value(tagToRemove)).arg(elementId);
            return false;
        }
        _tagsItems[elementId].removeAll(tagToRemove);
    }

    foreach (qlonglong tagToAdd, tagsToAdd) {
        QString query("INSERT INTO join_elements_tags (elementId, tagId) VALUES(?, ?)");
        QVariantList values;
        values.append(qlonglong(elementId));
        values.append(qlonglong(tagToAdd));
        if (_database->executeQuery(query, values) < 1) {
            qCritical() << tr("Unable to add tag %1 to catalog item %2").arg(_tags.value(tagToAdd)).arg(elementId);
            return false;
        }
        _tagsItems[elementId].append(tagToAdd);
    }

    return true;
}

void TagManagerI::_init()
{
    QList<QSqlRecord> tagSqlResult;
    QString tagQuery = "SELECT * FROM tags";
    if (_database->executeQuery(tagQuery, QVariantList(), &tagSqlResult) < 0) {
        qCritical() << tr("Error while getting tags");
        return;
    }
    foreach (const QSqlRecord &rec, tagSqlResult) {
        qlonglong tagId = rec.value("id").toLongLong();
        QString name = rec.value("name").toString();
        _tags.insert(tagId, name);
    }

    QList<QSqlRecord> joinSqlResult;
    QString joinQuery = "SELECT * FROM join_elements_tags";
    if (_database->executeQuery(joinQuery, QVariantList(), &joinSqlResult) < 0) {
        qCritical() << tr("Error while getting link between tags and catalog items");
        return;
    }
    foreach (const QSqlRecord &rec, joinSqlResult) {
        qlonglong tagId = rec.value("tagId").toLongLong();
        qlonglong elementId = rec.value("elementId").toLongLong();

        if (!_tagsItems.contains(elementId))
            _tagsItems.insert(elementId, QList<qlonglong>());
        if (_tags.contains(tagId))
            _tagsItems[elementId].append(tagId);
    }
}

} // namespace Hns
