/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "topologyI.h"
#include "entityI.h"
#include "tracerI.h"
#include "masterI.h"
#include "catalog/topologycatalogI.h"
#include "catalog/entitycatalogI.h"
#include "wireI.h"
#include "nodeinfoI.h"
#include "jobmanagerI.h"
#include "catalog/catalogfilestorage.h"
#include "clonejobs/entityclonejob.h"
#include "domainI.h"

// hynesim common headers
#include "common/canimplementmodel.h"
#include "common/resourceutils.h"
#include "common/clonejobflags.h"

// Generated hynesim headers
#include "common/wireentry.h"
#include "common/wire.h"

// diamesh headers
#include <diamesh/sessionManagerI.h>

// Qt headers
#include <QCoreApplication>
#include <QDebug>
#include <QtConcurrentRun>
#include <QDir>
#include <QRect>
#include <QTime>
#include <QSharedPointer>

namespace Hns {

QHash<Hns::BalancingBehavior, TopologyI::ComputeBalancing> *TopologyI::_computationalBehaviors = NULL;

Q_GLOBAL_STATIC(QReadWriteLock, _behaviorsLock)

Q_GLOBAL_STATIC_WITH_ARGS(QAtomicInt, _idCounter,(1))

TopologyI::TopologyI(const::Hns::TopologyEntryPtr &topologyEntry)
    : _entitiesLock(QReadWriteLock::Recursive)
    , _wiresLock(QReadWriteLock::Recursive)
    ,_modified(false)

{
    _topologyEntry = topologyEntry;
    _id = _idCounter()->fetchAndAddOrdered(1);

    _statusSignalMapper = new QSignalMapper();
    _statusSignalMapper->moveToThread(qApp->thread());
    QObject::connect(_statusSignalMapper, SIGNAL(mapped(QObject *)),
                     this, SLOT(entityChanged(QObject *)));

    _plugCountSignalMapper = new QSignalMapper();
    _plugCountSignalMapper->moveToThread(qApp->thread());
    QObject::connect(_plugCountSignalMapper, SIGNAL(mapped(QObject *)),
                     this, SLOT(checkWires(QObject *)));

    _checkcpapFileSignalMapper = new QSignalMapper();
    _checkcpapFileSignalMapper->moveToThread(qApp->thread());
    QObject::connect(_checkcpapFileSignalMapper, SIGNAL(mapped(QObject *)),
                     this, SLOT(checkCpapFiles(QObject *)));

    QObject::connect(MasterI::instance()->topologyCatalog(), SIGNAL(entryACLsChanged(Ice::Long,Diamesh::ACLList)), this, SLOT(_entryACLsChangedSlot(Ice::Long,Diamesh::ACLList)));

    QString path = _topologyEntry->path.c_str();

    if(!path.isEmpty())
        path = QString::fromStdString(MasterI::instance()->topologyCatalog()->getFullEntryPathFromMixed(_topologyEntry->path, _topologyEntry->uuid));
    if (!path.isEmpty())
        _tracer = TracerI::get(path);
    else
        _tracer = new TracerI();

    Ice::ObjectAdapterPtr adapter = Diamesh::MasterI::instance()->adapter();
    Ice::ObjectPrx proxy = adapter->addWithUUID(this);
    _proxyIdentity = proxy->ice_getIdentity();

    QMetaObject::invokeMethod(this, "_init", Qt::QueuedConnection);
}

void TopologyI::_init()
{
    qsrand(QTime::currentTime().msec());

    _statusSignalMapper->setParent(this);
    _plugCountSignalMapper->setParent(this);
}

void TopologyI::_entityEntryChanged()
{
    setModified(true);
}

void TopologyI::_wireEntryChanged()
{
    setModified(true);
}

void TopologyI::_captureJobStartedSlot()
{
    setModified(true);
}

void TopologyI::_memoryDumpJobStartedSlot()
{
    setModified(true);
}

TopologyI::~TopologyI()
{
    _tracer->destroy();
}

EntityTopologyLinkStruct TopologyI::createEntityFromEntry(const ::Hns::EntityEntryPtr &entityEntry, const Hns::LayoutPtr &layout, const::Ice::Current &current, bool initialLoading, const TopologyLinkList &links)
{
    bool isWire = (WireEntryPtr::dynamicCast(entityEntry)) ? true:false;
    if (isWire)
        entityEntry->isModel = false;
    /*Creation of the entity*/
    QWriteLocker locker(&_entitiesLock);
    QWriteLocker wiresLocker(&_wiresLock);
    EntityFactoryPrx entityFactory = MasterI::instance()->getEntityFactory(::Ice::Current());
    EntityPrx entity = NULL;
    // Topology does not propagate context in order to allow to create Entities based on entity entries invisible to the user
    if (entityEntry->id)
        entity = entityFactory->createEntityFromId(entityEntry->id);
    else
        entity = entityFactory->createEntity(entityEntry);
    if (!entity) {
        qCritical() << tr("Unable to create a topology entity for entity %1 -- id: %2").arg(QString::fromStdString(entityEntry->shortName)).arg(QString::number(entityEntry->id));
        return EntityTopologyLinkStruct();
    }
    EntityI *entityI = dynamic_cast<EntityI*>(MasterI::instance()->adapter()->findByProxy(entity).get());
    LayoutPtr entityLayout(layout);
    entityLayout->id = entity->getId();
    entityLayout->labelPolicyState = Hns::LabelPolicyDefault;
    if (!isWire)
        _layout.entityLayouts[entityLayout->id] = EntityLayoutPtr::dynamicCast(entityLayout);
    if (isWire) {
        WirePrx wire = WirePrx::uncheckedCast(entity);
        WireI *wirePointer = dynamic_cast<WireI*>(MasterI::instance()->adapter()->findByProxy(wire).get());
        QObject::connect(wirePointer, SIGNAL(entryChanged()), this, SLOT(_wireEntryChanged()));
        updateWireLayout(WireLayoutPtr::dynamicCast(entityLayout), initialLoading, current);
        emit wireAdded(_id, entity->getId());
    }

    QList<TopologyLink> linksList = QList<TopologyLink>::fromStdList(links);
    QList<TopologyLink> returnList;
    foreach (const TopologyLink &l, linksList) {
        QPair<Ice::Long, int> entitySide;
        QPair<Ice::Long, int> wireSide;
        entitySide.first = QString::fromStdString(l.entitySide).split(":").first().toLong();
        entitySide.second = QString::fromStdString(l.entitySide).split(":").last().toInt();
        wireSide.first = QString::fromStdString(l.wireSide).split(":").first().toLong();
        wireSide.second = QString::fromStdString(l.wireSide).split(":").last().toInt();
        TopologyLink returnLink;
        if (isWire) {
            WirePrx wire = WirePrx::uncheckedCast(entity);

            returnLink.wireProxy = wire;
            returnLink.entitySide = l.entitySide;
            returnLink.wireSide = QString("%1:%2").arg(entity->getId()).arg(wireSide.second).toStdString();

            bool plugSucceeded = false;

            EntityI *entityToPlug = _getLinkedEntity(l);
            if (entityToPlug) {
                plugSucceeded = entityToPlug->plug(entitySide.second);
                if (!plugSucceeded) {
                    qCritical() << tr("Can not plug entity %1 with wire %2").arg(QString::number(entitySide.first), QString::number(entity->getId()));
                    returnLink.entitySide.clear();
                }
            } else {
              qCritical() << tr("Can not plug non-existent entity %1 with wire %2").arg(QString::number(entitySide.first), QString::number(entity->getId()));
              returnLink.entitySide.clear();
            }
            _wiresLink.insertMulti(entity->getId(), returnLink);
            if (entityToPlug && plugSucceeded)
                _defineWireIfNeeded(entityToPlug);
        } else {
            WirePrx wire = _getWirePrx(wireSide.first);
            if (!entityI->plug(entitySide.second)) {
                qCritical() << tr("Can not plug entity %1 with wire %2").arg(QString::number(entity->getId()), QString::number(wireSide.first));
                returnLink.wireProxy = 0;
                returnLink.wireSide.clear();
            } else {
                returnLink.wireProxy = wire;
                returnLink.wireSide = l.wireSide;
            }
            returnLink.entitySide = QString("%1:%2").arg(entity->getId(),entitySide.second).toStdString();
        }

        returnList.append(returnLink);
    }

    /*Entity created*/

    if (!isWire)
        _addEntity(entity, current, initialLoading);


    EntityTopologyLinkStruct returnStruct;
    returnStruct.e= entity;
    returnStruct.tl = returnList.toStdList();
    return returnStruct;
}

EntityTopologyLinkStruct TopologyI::createEntityFromEntry(const ::Hns::EntityEntryPtr &entityEntry, const Hns::LayoutPtr &layout, const TopologyLinkList &links, const ::Ice::Current &current)
{
    return createEntityFromEntry(entityEntry, layout, current, false, links);
}

EntityTopologyLinkStruct TopologyI::createEntityFromEntryId(::Ice::Long entityId, const Hns::LayoutPtr &layout, const TopologyLinkList &links, const ::Ice::Current &current)
{
    QWriteLocker locker(&_entitiesLock);

    EntityEntryPtr entityEntry = MasterI::instance()->entityCatalog()->getEntityEntry(entityId, current);
    if (!entityEntry) {
        qCritical() << tr("Unable to get the catalog entry %1").arg(entityId);
        return EntityTopologyLinkStruct();
    }

    if (_topologyEntry->isModel && !entityEntry->isModel) {
        entityEntry = Hns::EntityEntryPtr::dynamicCast(entityEntry->ice_clone());
        entityEntry->isModel = true;
        entityEntry->id = 0;
    }
    locker.unlock();
    return createEntityFromEntry(entityEntry, layout, links, current);
}

EntityPrx TopologyI::implementModelEntity(Ice::Long entityId, Ice::Long entryId, const Ice::Current &current)
{
    EntityPrx entityPrx = getEntity(entityId);
    if (!entityPrx)
        return NULL;

    EntityEntryPtr entityEntry = MasterI::instance()->entityCatalog()->getEntityEntry(entryId, current);

    if (!entityEntry) {
        qCritical() << tr("Unable to get the catalog entry %1").arg(entryId);
        return NULL;
    }

    EntityI *entity = dynamic_cast<EntityI *>(MasterI::instance()->adapter()->findByProxy(entityPrx).get());

    QList<QPair<ImplementModelResult, QString> > errorList;
    if(canImplementModel(entity->getEntry(), entityEntry, errorList, false) == ERROR) {
        QStringList errors;
        QListIterator<QPair<Hns::ImplementModelResult, QString> > errorIterator(errorList);
        while(errorIterator.hasNext()) {
            QPair<Hns::ImplementModelResult, QString> error = errorIterator.next();
            errors.append(error.second);
        }
        qWarning() << errors;
        return NULL;
    }

    if (entityEntry->isTemplate && !entityEntry->hasResources) {
        entityEntry = Hns::EntityEntryPtr::dynamicCast(entityEntry->ice_clone());
        entityEntry->id = 0;
        entityEntry->path = "";
        entityEntry->uuid = MasterI::instance()->entityCatalog()->generateUnusedUUID().toStdString();
        entityEntry->isTemplate = false;
        entityEntry->hasResources = false;
    }

    entity->setEntry(entityEntry);
    QHash<Ice::Int, Ice::Int> listOld2NewPlugs;
    listOld2NewPlugs = entity->changePlugsForImplementation();
    _plugChanged(entityId , listOld2NewPlugs);

    if(entityEntry->id != 0)
        MasterI::instance()->entityCatalog()->lockEntry(entityEntry->id);

    emit entityImplemented(entity->getId() ,entityEntry->id );
    setModified(true);
    return entityPrx;
}

bool TopologyI::unImplementModelEntity(Ice::Long entityId, const Ice::Current &current)
{
    EntityPrx entityPrx = getEntity(entityId);
    if (!entityPrx)
        return false;

    EntityI *entity = dynamic_cast<EntityI *>(MasterI::instance()->adapter()->findByProxy(entityPrx).get());


    if(entity->getStatus() != Hns::UNDEFINED)
        return false;
    ::Hns::EntityEntryPtr entityEntry = entity->getEntry(current);

    if (!entityEntry) {
        qCritical() << tr("Unable to get the catalog entry");
        return false;
    }

    if(entity->unImplement()) {
        QHash<Ice::Int, Ice::Int> listOld2NewPlugs;
        listOld2NewPlugs = entity->changePlugsForImplementation();
        if (!listOld2NewPlugs.isEmpty())
            _plugChanged(entityId , listOld2NewPlugs);
        if(entityEntry->id != 0) {
            emit entityUnImplemented(entityEntry->id);
            MasterI::instance()->entityCatalog()->unlockEntry(entityEntry->id);
        }
        setModified(true);
        return true;
    }
    return false;
}

Hns::IdImportedList TopologyI::importTopology(Ice::Long topologyEntryId, const EntityLayoutPtr &positionLayout, const Ice::Current &current)
{
    Hns::IdImportedList importedList;
    // get the topology to import
    TopologyCatalogI *catalog = MasterI::instance()->topologyCatalog();
    TopologyEntryPtr importedEntry = catalog->getTopologyEntry(topologyEntryId, current);
    if (!importedEntry) {
        qCritical() << tr("Unable to get the catalog entry %1").arg(topologyEntryId);
        return importedList;
    }
    // check whether the topology to import is a model
    if (!importedEntry->isModel) {
        qCritical() << tr("Topology entry '%1' is not a model").arg(QString::fromStdString(importedEntry->shortName));
        return importedList;
    }
    // compute the imported bounding box
    QRect boundingBox;
    int maxSize = 128;
    TopologyLayout importedLayout = catalog->getTopologyLayout(importedEntry);
    QMap<Ice::Long, EntityLayoutPtr> entityLayoutMap(importedLayout.entityLayouts);
    QMap<Ice::Long, WireLayoutPtr> wireLayoutMap(importedLayout.wireLayouts);
    foreach (EntityLayoutPtr importedEntityLayout, entityLayoutMap) {
        QRect entityBounds(importedEntityLayout->x, importedEntityLayout->y, maxSize, maxSize);
        if (boundingBox.isNull()) {
            boundingBox = entityBounds;
        } else {
            boundingBox = boundingBox.united(entityBounds);
        }
    }
    QPoint boundingBoxCenter = boundingBox.center();

    // load entities
    QHash<int, int> oldNewEntityIdHash;
    QMap<int, EntityEntryPtr> entitiesHash(importedEntry->entitiesMap);
    QMapIterator<int, EntityEntryPtr> entityIt(entitiesHash);
    while (entityIt.hasNext()) {
        entityIt.next();
        int oldEntityId = entityIt.key();
        EntityEntryPtr entityEntry = entityIt.value();

        EntityLayoutPtr entityLayout = entityLayoutMap.value(oldEntityId);
        entityLayout->x += (positionLayout->x - boundingBoxCenter.x());
        entityLayout->y += (positionLayout->y - boundingBoxCenter.y());
        EntityPrx entity = NULL;
        try {
            entity = createEntityFromEntry(entityEntry, entityLayout, current, true).e;
        } catch (EntryLockedError &) {
            qCritical() << tr("Caught an 'entry locked' exception");
        } catch (LinkedDomainError &) {
            qCritical() << tr("Caught a 'linked domain' exception");
        }

        if (!entity) {
            qCritical() << tr("Unable to create entity %1").arg(QString::fromStdString(entityEntry->shortName));
            return importedList;
        }
        if (oldNewEntityIdHash.contains(oldEntityId)) {
            qCritical() << tr("Old entity %1 has already been loaded").arg(oldEntityId);
            return importedList;
        }
        oldNewEntityIdHash[oldEntityId] = entity->getId();

        QMap<int, PcapInfoList> qmapPcapInfoList(_topologyEntry->pcapInfoMap);
        entity->setPcapInfoList(qmapPcapInfoList.value(oldNewEntityIdHash.key(entity->getId())));
        importedList.push_back(entity->getId());
    }

    QMultiHash<int, TopologyLink> topologyLinkMultiHash; //id: wireId
    QList<TopologyLink> topologyLinkList = QList<TopologyLink>::fromStdList(importedEntry->topologyLinks);
    foreach (TopologyLink topologyLink, topologyLinkList) {
        int wireId = QString::fromStdString(topologyLink.wireSide).split(":").first().toInt(); //TODO: refactor, use a method
        /*need to get the new entityId to modify topologyLinkMultiHash*/
        if (!QString::fromStdString(topologyLink.entitySide).isEmpty()) {
            int entityPortId = QString::fromStdString(topologyLink.entitySide).split(':').last().toInt();
            qlonglong entityId = QString::fromStdString(topologyLink.entitySide).split(':').first().toLongLong();
            topologyLink.entitySide = QString("%1:%2").arg(oldNewEntityIdHash[entityId]).arg(entityPortId).toStdString();
        }
        topologyLinkMultiHash.insertMulti(wireId, topologyLink);
    }

    QHash<int, int> oldNewWireIdHash;
    QMap<int, WireEntryPtr> wiresMap(importedEntry->wiresMap);
    QMapIterator<int, WireEntryPtr> wiresMapIterator(wiresMap);
    while (wiresMapIterator.hasNext()) {
        wiresMapIterator.next();
        int oldWireId = wiresMapIterator.key();
        WireLayoutPtr wireLayout = wireLayoutMap.value(oldWireId);
        wireLayout->x1 += (positionLayout->x - boundingBoxCenter.x());
        wireLayout->y1 += (positionLayout->y - boundingBoxCenter.y());
        wireLayout->x2 += (positionLayout->x - boundingBoxCenter.x());
        wireLayout->y2 += (positionLayout->y - boundingBoxCenter.y());

        QRectF sceneRect = QRectF(0, 0, _layout.layoutOptions.width, _layout.layoutOptions.height);
        QRectF itemRect = QRectF(-10, -10, 20, 20);
        _validateWireLayout(wireLayout, itemRect, sceneRect);

        WireEntryPtr wireEntry = wiresMapIterator.value();
        EntityTopologyLinkStruct etls;
        try {
            etls = createEntityFromEntry(wireEntry, wireLayout, current, true, topologyLinkMultiHash.values(oldWireId).toStdList());
        } catch (EntryLockedError &) {
            qCritical() << tr("Caught an 'entry locked' exception");
        }
        if (!etls.e) {
            qCritical() << tr("Unable to create wire %1").arg(QString::number(etls.e->getId()));
            return importedList;
        }
        if (oldNewWireIdHash.contains(oldWireId)) {
            qCritical() << tr("Old wire %1 has already been loaded").arg(oldWireId);
            return importedList;
        }
        oldNewEntityIdHash[oldWireId] = etls.e->getId();
        importedList.push_back(etls.e->getId());
    }

    if(!importedList.empty())
        setModified(true);
    return importedList;
}

void TopologyI::_validateWireLayout(WireLayoutPtr wireLayout, QRectF itemRect, QRectF sceneRect)
{
    qreal minX = sceneRect.x() - itemRect.x();
    qreal minY = sceneRect.y() - itemRect.y();
    qreal maxX = sceneRect.x() + sceneRect.width() - itemRect.width();
    qreal maxY = sceneRect.y() + sceneRect.height() - itemRect.height();

    if (wireLayout->x1 > maxX)
        wireLayout->x1 = maxX;
    if (wireLayout->y1 > maxY)
        wireLayout->y1 = maxY;
    if (wireLayout->x1 < minX)
        wireLayout->x1 = minX;
    if (wireLayout->y1 < minY)
        wireLayout->y1 = minY;

    if (wireLayout->x2 > maxX)
        wireLayout->x2 = maxX;
    if (wireLayout->y2 > maxY)
        wireLayout->y2 = maxY;
    if (wireLayout->x2 < minX)
        wireLayout->x2 = minX;
    if (wireLayout->y2 < minY)
        wireLayout->y2 = minY;
}

Ice::Int TopologyI::createEntityFromClonedEntryId(::Ice::Long entryId, const Hns::EntityLayoutPtr &layout, ::Ice::Long entryModelId , const ::Ice::Current &current)
{
    int jobId = 0;
    try {
        jobId = MasterI::instance()->entityCatalog()->cloneEntityEntry(entryId, Hns::REMOVE_TEMPLATE, current);
    } catch (HnsError) {
        return jobId;
    }
    JobManagerI *jobManager = JobManagerI::instance();
    EntityCloneJob *cloneJob = dynamic_cast<EntityCloneJob *>(jobManager->job(jobId));
    JobInfo jobInfo;
    jobInfo.cloneJob = cloneJob;
    jobInfo.entryModelId = entryModelId;
    jobInfo.layout = layout;
    jobInfo.current = current;
    _jobInfoMutex.lock();
    _jobInfoMap[cloneJob->id()] = jobInfo;
    _jobInfoMutex.unlock();
    QObject::connect(cloneJob, SIGNAL(finished(Ice::Int)), this , SLOT(_createOrImplementFromCloneJob(Ice::Int)) );
    return cloneJob->id();
}



void TopologyI::_createOrImplementFromCloneJob(Ice::Int jobId)
{
    _jobInfoMutex.lock();
    if (_jobInfoMap.contains(jobId)) {
        JobInfo jobInfo = _jobInfoMap.take(jobId);
        _jobInfoMutex.unlock();
        Ice::Long clonedEntryId =jobInfo.cloneJob->clonedEntry()->id;
        if(jobInfo.entryModelId == 0) {
            QtConcurrent::run(this, &TopologyI::createEntityFromEntryId, clonedEntryId, jobInfo.layout, TopologyLinkList(), jobInfo.current);
        } else
            QtConcurrent::run(this, &TopologyI::implementModelEntity, jobInfo.entryModelId,clonedEntryId, jobInfo.current);
    } else {
        _jobInfoMutex.unlock();
    }

}

void TopologyI::_entryACLsChangedSlot(Ice::Long entryId, Diamesh::ACLList aclList)
{
    if (entryId == _topologyEntry->id)
        emit aclsChanged(aclList);
}

bool TopologyI::removeEntity(::Ice::Long entityId, const ::Ice::Current &current)
{

    QWriteLocker _lockerBusy(&_entitiesSupLock);
    if(_setEntityBusy.insert(entityId) == _setEntityBusy.end()) return false;
    _lockerBusy.unlock();
    QWriteLocker entityLocker(&_entitiesLock);
    QWriteLocker wireLocker(&_wiresLock);
    if (!_entities.contains(entityId) && !_wiresLink.contains(entityId)) {
        qCritical() << tr("Entity %1 does not exist in topology. Unable to remove it.").arg(entityId);
        _lockerBusy.relock();
        _setEntityBusy.erase(_setEntityBusy.find(entityId));
        return false;
    }

    EntityI *entity = NULL;
    EntityPrx entityProxy = NULL;
    if (_entities.contains(entityId)) {
        entityProxy = _entities.value(entityId);
        entity = dynamic_cast<EntityI*>(MasterI::instance()->adapter()->findByProxy(entityProxy).get());
    }
    else { //it means that we have a wireEntity
        entityProxy = _wiresLink.value(entityId).wireProxy;
        entity = dynamic_cast<EntityI*>(MasterI::instance()->adapter()->findByProxy(entityProxy).get());
    }


    Diamesh::SessionManagerI * sessionManager = Diamesh::MasterI::instance()->sessionManager();
    Diamesh::UserPtr user = sessionManager->currentUser(current);

    // TODO:TestRight
    //    if (sessionManager->isClientCall(current) && !user->isAdmin) {
    //            qCritical() << tr("The current user is not allowed to remove entity %1").arg(entityId);
    //            _lockerBusy.relock();
    //            _setEntityBusy.erase(_setEntityBusy.find(entityId));
    //            return false;
    //    };
    disconnectSignalsFromEntity(entityProxy);
    entity->destroy(current);
    bool isWire = false;
    if (qobject_cast<WireI *>(entity))
        isWire = true;

    if (isWire) {
        foreach (const TopologyLink &l, _wiresLink.values(entityId)) {
            if (!l.entitySide.empty()) {
                EntityI * entity(_entity(_wireEntityId(l.entitySide)));
                if (entity)
                    entity->unplug(_wireEntityPort(l.entitySide));
                else
                    qDebug() << tr("Unable to unplug wire %1, entity does not exist").arg(entityId);
            }
        }
        _wiresLink.remove(entityId);
        _layout.wireLayouts.erase(entityId);
    }
    else {
        foreach (const TopologyLink &link, _wiresLink) {
            if(_wireEntityId(link.entitySide) == entityId) {
                TopologyLink newLink = link;
                newLink.entitySide = "";
                qlonglong wireId = _wireEntityId(link.wireSide);
                WireI *wirePointer = dynamic_cast<WireI*>(MasterI::instance()->adapter()->findByProxy(link.wireProxy).get());
                if (wirePointer) {
                    wirePointer->undefine(current);
                }
                _wiresLink.remove(wireId, link);
                _wiresLink.insertMulti(wireId, newLink);
                QString entitySide = QString::fromStdString(link.entitySide);
                QString wireSide = QString::fromStdString(link.wireSide);
                emit entityUnplugged(wireSide, entitySide);
            }
        }
        _entities.remove(entityId);
        _layout.entityLayouts.erase(entityId);
    }

    setModified(true);
    if (isWire) {
        emit wireRemoved(entityId);
    }
    else {
        emit entityRemoved(entityId);
    }
    _lockerBusy.relock();
    _setEntityBusy.erase(_setEntityBusy.find(entityId));
    return true;
}

void TopologyI::disconnectSignalsFromEntity(EntityPrx entity)
{
    EntityI *entityPointer = dynamic_cast<EntityI*>(MasterI::instance()->adapter()->findByProxy(entity).get());
    QObject::disconnect(entityPointer, SIGNAL(statusChanged(Ice::Int, Ice::Long)),
                        _statusSignalMapper, SLOT(map()));
    QObject::disconnect(entityPointer, SIGNAL(plugsRemoved()),
                        _plugCountSignalMapper, SLOT(map()));

}

QString TopologyI::_getPath()
{
    QString topologyPath;

    if (_topologyEntry->path == "") {
        TopologyCatalogI *catalog = MasterI::instance()->topologyCatalog();
        CatalogFileStorage* fs = catalog->getDefaultFileStorage();
        if (!fs) {
            qCritical() << tr("No file storage was found, unable to delete the topology entry");
            return QString();
        }

        topologyPath = QString("%1:%2%3").arg(fs->storageName()).arg("/.tmp/").arg(QString::fromStdString(_topologyEntry->uuid));
    } else if (QString::fromStdString(_topologyEntry->path).endsWith(":")) {
        topologyPath = QString::fromStdString(_topologyEntry->path)+QString::fromStdString(_topologyEntry->uuid);
    } else {
        topologyPath = QString::fromStdString(_topologyEntry->path) + "/" + QString::fromStdString(_topologyEntry->uuid);
    }

    return QDir::cleanPath(topologyPath);
}

bool TopologyI::_addEntity(EntityPrx entity, const::Ice::Current &current, bool initialLoading)
{
    Q_UNUSED(current)
    _entities.insert(entity->getId(), entity);

    EntityI *entityPointer = dynamic_cast<EntityI*>(MasterI::instance()->adapter()->findByProxy(entity).get());
    QObject::connect(entityPointer, SIGNAL(statusChanged(Ice::Int , Ice::Long)),
                     _statusSignalMapper, SLOT(map()));
    _statusSignalMapper->setMapping(entityPointer, entityPointer);
    QObject::connect(entityPointer, SIGNAL(plugsRemoved()),
                     _plugCountSignalMapper, SLOT(map()));
    _plugCountSignalMapper->setMapping(entityPointer, entityPointer);

    QObject::connect(entityPointer, SIGNAL(checkPcapFiles()),
                     _checkcpapFileSignalMapper, SLOT(map()));
    _checkcpapFileSignalMapper->setMapping(entityPointer, entityPointer);

    Hns::EntityEntryPtr entityEntry = entityPointer->getEntry();
    if (entityEntry->id == 0 || _topologyEntry->isModel) {
        QObject::connect(entityPointer, SIGNAL(entryChanged()),
                         this, SLOT(_entityEntryChanged()));
    }

    entityPointer->setTopologyPath(_getPath());

    QObject::connect(entityPointer, SIGNAL(captureJobStarted()), this, SLOT(_captureJobStartedSlot()));
    DomainI *domainPointer = qobject_cast<DomainI*>(entityPointer);
    if (domainPointer)
        QObject::connect(domainPointer, SIGNAL(memoryDumpJobStarted()), this, SLOT(_memoryDumpJobStartedSlot()));

    if (!initialLoading)
        setModified(true);
    emit entityAdded(_id ,entity->getId());
    if(entity->getEntry())
        emit entryUsed(entity->getEntry()->id);
    return entity;
}

EntityPrx TopologyI::_getEntity(::Ice::Long entityId)
{
    if (!_entities.contains(entityId)) {
        qCritical() << tr("Entity %1 was not found in topology").arg(entityId);
        return NULL;
    }

    return _entities.value(entityId);
}

bool TopologyI::plugEntity(Ice::Long entityId, Ice::Int entityPortId, Ice::Long wireId, Ice::Int wirePortId, const Ice::Current &current)
{
    Q_UNUSED(current)
    QReadLocker entitiesLocker(&_entitiesLock);
    QWriteLocker wiresLocker(&_wiresLock);
    EntityPrx entity = _entities.value(entityId);
    if (!entity) {
        qCritical() << tr("Entity %1 does not exist").arg(entityId);
        return false;
    }
    QList<TopologyLink> links = _wiresLink.values(wireId);
    if (links.isEmpty()) {
        qCritical() << tr("Wire %1 does not exist").arg(wireId);
        return false;
    }
    QString wireSide = QString("%1:%2").arg(QString::number(wireId)).arg(QString::number(wirePortId));
    QString entitySide = QString("%1:%2").arg(QString::number(entityId)).arg(QString::number(entityPortId));
    TopologyLink newLink;
    foreach (TopologyLink link, links) {
        if (link.wireSide == wireSide.toStdString()) {
            if (QString::fromStdString(link.entitySide).isEmpty()) {
                if (_entity(entityId)->plug(entityPortId)) {
                    //plugging
                    newLink = link;
                    newLink.entitySide = entitySide.toStdString();
                    _wiresLink.remove(wireId, link);
                    _wiresLink.insertMulti(wireId, newLink);
                }
                else {
                    return false;
                }
            }
            else { //entity already plugged, something is wrong
                qCritical() << tr("Entity %1 port %2 is already plugged").arg(entityId).arg(entityPortId);
                return false;
            }
        }
    }

    emit entityPlugged(QString::fromStdString(newLink.wireSide), QString::fromStdString(newLink.entitySide));
    _defineWireIfNeeded(_entity(entityId));
    return true;
}

bool TopologyI::unplugEntity(Ice::Long entityId, Ice::Int entityPortId, Ice::Long wireId, Ice::Int wirePortId, const Ice::Current &current)
{
    Q_UNUSED(current)
    QReadLocker entitiesLocker(&_entitiesLock);
    QWriteLocker wiresLocker(&_wiresLock);
    EntityPrx entity = _entities.value(entityId);
    if (!entity) {
        qCritical() << tr("Entity %1 does not exist").arg(entityId);
        return false;
    }
    QList<TopologyLink> links = _wiresLink.values(wireId);
    if (links.isEmpty()) {
        qCritical() << tr("Wire %1 does not exist").arg(wireId);
        return false;
    }
    //undefine wire first
    WirePrx wirePrx = links.first().wireProxy;
    if (!wirePrx) {
        qCritical() << tr("Wire %1 does not exist").arg(wireId);
        return false;
    }
    if (wirePrx->isDefined()) {
        bool undefinedResult = wirePrx->undefine();
        if (!undefinedResult) {
            qCritical() << tr("Unable to undefine Wire %1").arg(wireId);
            return false;
        }
    }
    //unplug entity
    QString wireSide = QString("%1:%2").arg(QString::number(wireId)).arg(QString::number(wirePortId));
    QString entitySide = QString("%1:%2").arg(QString::number(entityId)).arg(QString::number(entityPortId));
    TopologyLink newLink;
    foreach (TopologyLink link, links) {
        if (link.wireSide == wireSide.toStdString()) {
            if (!QString::fromStdString(link.entitySide).isEmpty()) {
                //unplugging
                newLink = link;
                newLink.entitySide = "";
                _wiresLink.remove(wireId,link);
                _wiresLink.insertMulti(wireId, newLink);
                _entity(entityId)->unplug(entityPortId);
            }
            else { //entity already unplugged, something is wrong
                qCritical() << tr("Entity %1 port %2 is already unplugged").arg(entityId).arg(entityPortId);
                return false;
            }
        }
    }
    emit entityUnplugged(wireSide, entitySide);
    return true;
}

LongEntityMap TopologyI::getEntities(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QReadLocker locker(&_entitiesLock);
    LongEntityMap map;
    foreach (EntityPrx entity, _entities)
        map[entity->getId()] = entity;

    return map;
}

EntityPrx TopologyI::getEntity(::Ice::Long entityId, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QReadLocker locker(&_entitiesLock);
    return _getEntity(entityId);
}

LongLinkListMap TopologyI::getTopologyLinks(const Ice::Current &current)
{
    Q_UNUSED(current)
    QReadLocker locker(&_wiresLock);
    QMap<Ice::Long, TopologyLinkList> mapToReturn;
    QList<Ice::Long> keys = _wiresLink.uniqueKeys();
    foreach (qlonglong wireId, keys) {
        QList<TopologyLink> links = _wiresLink.values(wireId);
        mapToReturn.insert(wireId, links.toStdList());
    }
    return mapToReturn.toStdMap();
}

TopologyLinkList TopologyI::getTopologyLink(Ice::Long wireId, const Ice::Current &current)
{
    Q_UNUSED(current)
    QReadLocker locker(&_wiresLock);
    QList<TopologyLink> links = _wiresLink.values(wireId);
    return links.toStdList();
}

Ice::Long TopologyI::getId(const::Ice::Current &)
{
    return _id;
}

TopologyEntryPtr TopologyI::getEntry(const::Ice::Current &current)
{
    Q_UNUSED(current)
    return _topologyEntry;
}

bool TopologyI::updateEntry(const TopologyEntryPtr &entry, const ::Ice::Current &current)
{
    if (!entry) {
        qCritical() << tr("No entry to update");
        return false;
    }
    if (!entry->id) {
        qCritical() << tr("The topology entry has no id. Unable to update it.");
        return false;
    }

    if (_topologyEntry->id == entry->id
            && _topologyEntry->hSetId == entry->hSetId
            && _topologyEntry->uuid == entry->uuid
            && _topologyEntry->shortName == entry->shortName
            && _topologyEntry->longName == entry->longName
            && _topologyEntry->description == entry->description
            && _topologyEntry->creationDateTime == entry->creationDateTime
            && _topologyEntry->lastModificationDateTime == entry->lastModificationDateTime
            && _topologyEntry->path == entry->path
            && _topologyEntry->tagList == entry->tagList
            && _topologyEntry->status == entry->status
            && _topologyEntry->entitiesMap == entry->entitiesMap
            && _topologyEntry->wiresMap == entry->wiresMap
            && _topologyEntry->pcapInfoMap == entry->pcapInfoMap
            && _topologyEntry->memoryDumpInfoMap == entry->memoryDumpInfoMap
            && _topologyEntry->isModel == entry->isModel
            && _topologyEntry->isPersistent == entry->isPersistent) {
        // Update is not required
        return true;
    }

    TopologyCatalogI *catalog = MasterI::instance()->topologyCatalog();
    if (!catalog) {
        qCritical() << tr("Unable to get the topology catalog");
        return false;
    }

    Diamesh::SessionManagerI *sessionManager = Diamesh::MasterI::instance()->sessionManager();
    if(!sessionManager) {
        qCritical() << tr("Unable to get the session manager");
        return false;
    }

    TopologyEntryPtr newEntry = 0;

    if (sessionManager->isClientCall(current)) {
        newEntry = TopologyEntryPtr::dynamicCast(_topologyEntry->ice_clone());
        if (newEntry == 0) {
            qCritical() << tr("Unable to clone topology entry");
            return false;
        }
        // Update only descriptive fields
        newEntry->shortName = entry->shortName;
        newEntry->shortName = entry->shortName;
        newEntry->longName = entry->longName;
        newEntry->description = entry->description;
        newEntry->tagList = entry->tagList;
        newEntry->isPersistent = entry->isPersistent;
        if (newEntry->path.empty())
            newEntry->path = entry->path;
    } else {
        bool updateNeeded = (_topologyEntry->path != entry->path || _topologyEntry->uuid != entry->uuid);
        QString exceptionMessage;
        if (updateNeeded && !canBeSaved(exceptionMessage)) {
            qCritical() << tr("Unable to update the topology entry. %1").arg(exceptionMessage);
            return false;
        }
        newEntry = entry;
    }

    if (!catalog->updateEntry(newEntry)) {
        qCritical() << tr("Unable to update the topology entry");
        return false;
    }
    if (!sessionManager->isClientCall(current)) {
        if (!catalog->updateTopologyLayout(newEntry, _layout)) {
            qCritical() << tr("Unable to update layout topology");
            return false;
        }
    }

    setEntry(newEntry);
    return true;
}

void TopologyI::setEntry(const TopologyEntryPtr &entry)
{
    // if the topology was never saved
    bool updateNeeded = (_topologyEntry->path != entry->path || _topologyEntry->uuid != entry->uuid);
    if (updateNeeded) {
        TopologyCatalogI *catalog = MasterI::instance()->topologyCatalog();
        if (!catalog) {
            qCritical() << tr("Unable to get the topology catalog");
            return;
        }
        catalog->copyEntryResources(_topologyEntry, entry);
    }
    _topologyEntry = entry;

    if (updateNeeded) {
        _entitiesLock.lockForRead();
        foreach (EntityPrx entityPrx, _entities) {
            EntityI *entityI = _entity(entityPrx->getId());
            entityI->setTopologyPath(_getPath());
        }
        _entitiesLock.unlock();
    }
    emit entryModified(_topologyEntry->uuid.c_str());
}

bool TopologyI::canBeSaved(QString &exceptionMessage)
{
    QReadLocker locker(&_entitiesLock);
    foreach (EntityPrx entityPrx, _entities) {
        EntityI *entityI = _entity(entityPrx->getId());
        if (entityI->isCapturing()) {
            exceptionMessage = "The topology cannot be saved now because there still are capture jobs running";
            return false;
        }

    }

    return true;
}

bool TopologyI::isModified(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return _modified;
}

TracerPrx TopologyI::getTracer(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return _tracer->proxy();
}

TopologyLayout TopologyI::getLayout(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return _layout;
}

void TopologyI::setLayout(const ::Hns::TopologyLayout &layout, bool initialLoading, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    if (_layout != layout) {
        if (!initialLoading)
            setModified(true);
        _layout = layout;
        emit layoutChanged();
    }
}

void TopologyI::setLayout(const ::Hns::TopologyLayout &layout, const::Ice::Current &current)
{
    setLayout(layout, false, current);
}

TopologyLayoutOptions TopologyI::getLayoutOptions(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return _layout.layoutOptions;
}

void TopologyI::setLayoutOptions(const ::Hns::TopologyLayoutOptions &options, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    if (_layout.layoutOptions != options) {
        _layout.layoutOptions = options;
        setModified(true);
        emit layoutOptionsChanged();
    }
}

EntityLayoutPtr TopologyI::getEntityLayout(::Ice::Long entityId, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return _layout.entityLayouts[entityId];
}

WireLayoutPtr TopologyI::getWireLayout(::Ice::Long wireId, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return _layout.wireLayouts[wireId];
}

void TopologyI::updateEntityLayout(const Hns::EntityLayoutPtr &layout, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    _layout.entityLayouts[layout->id] = layout;
    emit entityLayoutChanged(layout);
}

void TopologyI::updateWireLayout(const WireLayoutPtr &layout, const ::Ice::Current &current)
{
    updateWireLayout(layout, false, current);
}

void TopologyI::updateWireLayout(const WireLayoutPtr &layout, bool initialLoading, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    if(_layout.wireLayouts[layout->id] != layout){
        _layout.wireLayouts[layout->id] = layout;
        if (!initialLoading)
            setModified(true);
        emit wireLayoutChanged(layout);
    }
}

::Diamesh::LongList TopologyI::canBuildWithNode(const ::std::string &nodeName, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QReadLocker locker(&_entitiesLock);

    QString nodeNameStr = QString::fromStdString(nodeName);
    QSharedPointer<NodeInfoI> nodeInfo = qSharedPointerDynamicCast<NodeInfoI>(MasterI::instance()->nodeInfo(nodeNameStr));

    QMap<Ice::Long, EntityEntryPtr> entryMap;
    if (nodeInfo == NULL) {
        qCritical() << tr("There is no node named %1").arg(nodeNameStr);
    } else {
        foreach (EntityPrx entity, _entities) {
            Ice::Long entityId = entity->getId();
            EntityEntryPtr entry = entity->getEntry();
            entryMap.insert(entityId, entry);
        }
    }
    return nodeInfo->canBuildFromEntryMap(entryMap);
}

::Hns::LongDefineInfoMap TopologyI::computeDefineInfos(const ::Hns::LongDefineInfoMap &defineInfoMap, ::Hns::BalancingBehavior behavior, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QMap<Ice::Long, Hns::DefineInfo> defineInfos(defineInfoMap);
    ComputeBalancing computeBalancing = TopologyI::getComputeBalancing(behavior);
    if (!computeBalancing)
        qWarning() << tr("There is no balancing method implemented yet for this behavior");
    else
        (this->*computeBalancing)(defineInfos);
    return defineInfos.toStdMap();
}

::Hns::BalancingBehaviorMap TopologyI::getBehaviorMap(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    // Order is important: The first element is used as the default behavior by hyneview.
    ::Hns::BalancingBehaviorMap map;
    ::Hns::BalancingBehaviorInfo balancingInfo;
    balancingInfo.name = tr("Smart").toStdString();
    balancingInfo.behavior= Hns::BEHAVIOR_AUTO;
    map[0] = balancingInfo;
    balancingInfo.name = tr("First").toStdString();
    balancingInfo.behavior= Hns::BEHAVIOR_FIRST;
    map[1] = balancingInfo;
    balancingInfo.name = tr("Random").toStdString();
    balancingInfo.behavior= Hns::BEHAVIOR_RANDOM;
    map[2] = balancingInfo;
    return map;
}

TopologyI::ComputeBalancing TopologyI::getComputeBalancing(::Hns::BalancingBehavior behavior)
{
    QWriteLocker locker(_behaviorsLock());
    if (!_computationalBehaviors) {
        _computationalBehaviors = new QHash<Hns::BalancingBehavior, TopologyI::ComputeBalancing>();
        _computationalBehaviors->insert(Hns::BEHAVIOR_RANDOM, &TopologyI::_computeRandomDefineInfos);
        _computationalBehaviors->insert(Hns::BEHAVIOR_FIRST, &TopologyI::_computeFirstDefineInfos);
        _computationalBehaviors->insert(Hns::BEHAVIOR_AUTO, &TopologyI::_computeAutoDefineInfos);
    }
    locker.unlock();
    return _computationalBehaviors->value(behavior, NULL);
}

void TopologyI::_computeRandomDefineInfos(QMap<Ice::Long, Hns::DefineInfo> &defineInfos)
{
    QHash<QString, QList<Hns::EntityEntryPtr> > alreadyComputedEntries = QHash<QString, QList<Hns::EntityEntryPtr> >(); /*<nodeName, computedEntriesOnThisNode>*/
    for (QMap<Ice::Long, Hns::DefineInfo>::iterator it = defineInfos.begin(); it != defineInfos.end(); ++it) {
        Hns::DefineInfo &defineInfo = it.value();
        if (defineInfo.isDefined && defineInfo.nodeId == "") {
            Ice::Long entityId = defineInfo.entityId;
            EntityPrx entity = _getEntity(entityId);
            QSharedPointer<NodeInfoI> nodeInfo = MasterI::instance()->getSuitableNode(entity->getEntry(), alreadyComputedEntries);
            if (nodeInfo) {
                defineInfo.nodeId = nodeInfo->getName();
                QList<Hns::EntityEntryPtr> computedEntriesForThisNode = alreadyComputedEntries.value(QString::fromStdString(nodeInfo->getName()));
                computedEntriesForThisNode.append(entity->getEntry());
                alreadyComputedEntries.insert(QString::fromStdString(nodeInfo->getName()), computedEntriesForThisNode);
            }
        }
    }
}

void TopologyI::_computeFirstDefineInfos(QMap<Ice::Long, Hns::DefineInfo> &defineInfos)
{
    MasterI *master = MasterI::instance();
    QHash<QString, QList<Hns::EntityEntryPtr> > alreadyComputedEntries = QHash<QString, QList<Hns::EntityEntryPtr> >(); /*<nodeName, computedEntriesOnThisNode>*/
    for (QMap<Ice::Long, Hns::DefineInfo>::iterator it = defineInfos.begin(); it != defineInfos.end(); ++it) {
        Hns::DefineInfo &defineInfo = it.value();
        if (defineInfo.isDefined && defineInfo.nodeId == "") {
            Ice::Long entityId = defineInfo.entityId;
            EntityPrx entity = _getEntity(entityId);

            QList<QSharedPointer<NodeInfoI> > suitableNodes = master->nodeInfoList(entity->getEntry(), alreadyComputedEntries);
            if (suitableNodes.size() > 0) {
                defineInfo.nodeId = suitableNodes.first()->getName();
                QList<Hns::EntityEntryPtr> computedEntriesForThisNode = alreadyComputedEntries.value(QString::fromStdString(suitableNodes.first()->getName()));
                computedEntriesForThisNode.append(entity->getEntry());
                alreadyComputedEntries.insert(QString::fromStdString(suitableNodes.first()->getName()), computedEntriesForThisNode);
            }
        }
    }
}

void TopologyI::_computeAutoDefineInfos(QMap<Ice::Long, Hns::DefineInfo> &defineInfos)
{
    MasterI *master = MasterI::instance();
    QHash<QString, QList<Hns::EntityEntryPtr> > alreadyComputedEntries = QHash<QString, QList<Hns::EntityEntryPtr> >(); /*<nodeName, computedEntriesOnThisNode>*/
    // compute how much resources are left for each node
    QHash<NodeInfoI*, Resource> resourcesLeft;
    QList<QSharedPointer<Diamesh::NodeInfoI> > diameshNodeInfos = master->nodeInfoList();
    foreach (const QSharedPointer<Diamesh::NodeInfoI> &diameshNodeInfo, diameshNodeInfos) {
        QSharedPointer<NodeInfoI> hnsNodeInfo = qSharedPointerDynamicCast<NodeInfoI>(diameshNodeInfo);
        HnsNode::ResourceInfo nodeResourceInfo = hnsNodeInfo->getResourceInfo();
        Resource resourceLeft;
        resourceLeft.cpu = nodeResourceInfo.maxResources.cpu - nodeResourceInfo.currentResources.cpu;
        resourceLeft.memory = nodeResourceInfo.maxResources.memory - nodeResourceInfo.currentResources.memory;
        resourcesLeft[hnsNodeInfo.data()] = resourceLeft;
    }

    // sort entities by resources or take the resources from the node
    // defined by the client
    QMultiMap<Resource, Ice::Long> entitiesSortedByResource;
    for (QMap<Ice::Long, Hns::DefineInfo>::iterator it = defineInfos.begin(); it != defineInfos.end(); ++it) {
        Hns::DefineInfo &defineInfo = it.value();
        if (defineInfo.isDefined) {
            Ice::Long entityId = defineInfo.entityId;
            EntityPrx entity = _getEntity(entityId);
            EntityEntryPtr entityEntry = entity->getEntry();
            Resource resource = Hns::getResourceFromEntry(entityEntry);
            if (defineInfo.nodeId != "") {
                // update the resources left
                QList<NodeInfoI *> nodeInfos = resourcesLeft.keys();
                NodeInfoI *nodeInfo = NULL;
                foreach (nodeInfo, nodeInfos) {
                    if (nodeInfo->getName() == defineInfo.nodeId)
                        break;
                }

                Resource &resourceLeft = resourcesLeft[nodeInfo];
                resourceLeft.cpu -= resource.cpu;
                resourceLeft.memory -= resource.memory;
                QString nodeName = QString::fromStdString(defineInfo.nodeId);
                if (!alreadyComputedEntries.contains(nodeName)) {
                    QList<Hns::EntityEntryPtr> list;
                    list.append(entityEntry);
                    alreadyComputedEntries.insert(nodeName, list);
                }
                else {
                    QList<Hns::EntityEntryPtr> list = alreadyComputedEntries.value(nodeName);
                    list.append(entityEntry);
                    alreadyComputedEntries.insert(nodeName, list);
                }
            } else {
                // add the entity to the map sorted by resources
                entitiesSortedByResource.insert(resource, entityId);
            }
        }
    }

    // load balancing
    // the map is iterated in reverse order (the most expensive resources first)
    QMutableMapIterator<Resource, Ice::Long> it(entitiesSortedByResource);
    it.toBack();
    while (it.hasPrevious()) {
        it.previous();

        Ice::Long entityId = it.value();
        Hns::DefineInfo &defineInfo = defineInfos[entityId];

        EntityPrx entity = _getEntity(entityId);
        EntityEntryPtr entityEntry = entity->getEntry();
        QList<QSharedPointer<NodeInfoI> > suitableNodes = master->nodeInfoList(entityEntry, alreadyComputedEntries);

        // get the node with the most available cpus or the most available
        // memory if all the nodes have the same cpu usage
        QMultiMap<Resource, NodeInfoI *> nodesSortedByResource;
        foreach (const QSharedPointer<NodeInfoI> &nodeInfo, suitableNodes) {
            Resource resourceLeft = resourcesLeft.value(nodeInfo.data());
            nodesSortedByResource.insert(resourceLeft, nodeInfo.data());
        }
        if (nodesSortedByResource.isEmpty())
            continue;
        NodeInfoI *mostAvailableNode = (nodesSortedByResource.end() - 1).value();

        // update the resources left
        Resource resource = it.key();
        Resource &resourceLeft = resourcesLeft[mostAvailableNode];
        resourceLeft.cpu -= resource.cpu;
        resourceLeft.memory -= resource.memory;
        defineInfo.nodeId = mostAvailableNode->getName();
        QList<Hns::EntityEntryPtr> computedEntriesForThisNode = alreadyComputedEntries.value(QString::fromStdString(mostAvailableNode->getName()));
        computedEntriesForThisNode.append(entity->getEntry());
        alreadyComputedEntries.insert(QString::fromStdString(mostAvailableNode->getName()), computedEntriesForThisNode);

    }
}

QList<Diamesh::SessionI*> TopologyI::getSessions()
{
    QList<Diamesh::SessionI*> sessions;

    Diamesh::SessionManagerI *sessionManager = MasterI::instance()->sessionManager();
    if (!sessionManager) {
        qCritical() << tr("Unable to get session manager");
        return sessions;
    }

    foreach (QString sessionId, _sessions) {
        Diamesh::SessionI *session = sessionManager->session(sessionId);
        if (session)
            sessions.append(session);
        else
            removeSession(sessionId);
    }
    return sessions;
}

QHash< ::Ice::Long, EntityPrx> TopologyI::entities()
{
    QReadLocker locker(&_entitiesLock);
    return _entities;
}

TopologyPrx TopologyI::proxy()
{
    Ice::ObjectAdapterPtr adapter = Diamesh::MasterI::instance()->adapter();
    return TopologyPrx::uncheckedCast(adapter->createProxy(_proxyIdentity));
}

void TopologyI::setModified(bool modified)
{
    _modified = modified;
    emit this->modified(_topologyEntry->uuid.c_str(), modified);
}

QMultiHash<Ice::Long, TopologyLink> TopologyI::wires()
{
    QReadLocker locker(&_wiresLock);
    return _wiresLink;
}

TopologyEntryPtr TopologyI::topologyEntry()
{
    return _topologyEntry;
}

TracerI *TopologyI::tracer()
{
    return _tracer;
}

void TopologyI::addSession(const QString &sessionId)
{
    _sessions.push_back(sessionId);
}

void TopologyI::removeSession(const QString &sessionId)
{
    _sessions.removeAll(sessionId);
}

bool TopologyI::hasSession(const QString &sessionId)
{
    foreach(QString session, _sessions) {
        if (sessionId == session) {
            return true;
        }
    }
    return false;
}

void TopologyI::checkWires(QObject *object)
{
    EntityI* entity = qobject_cast<EntityI *>(object);
    QMap<Ice::Long, Hns::Plug> plugMap = QMap<Ice::Long, Hns::Plug>(entity->getPlugs());
    QList<Ice::Long> plugList = plugMap.keys();
    ::Ice::Long entityId = entity->getId();

    QList< ::Ice::Long > wiresToDestroy;
    QWriteLocker writeLockWire(&_wiresLock);
    QHashIterator< ::Ice::Long, TopologyLink> it(_wiresLink);

    while (it.hasNext()) {
        it.next();
        const TopologyLink &link = it.value();
        Ice::Long linkedEntityId = _wireEntityId(link.entitySide);
        int linkedEntityPort = _wireEntityPort(link.entitySide);

        if (linkedEntityId == entityId && !plugList.contains(linkedEntityPort))
            wiresToDestroy.append(it.key());
    }


    foreach (::Ice::Long id, wiresToDestroy) {
        QList<TopologyLink> links = _wiresLink.values(id);
        foreach (TopologyLink link, links) {
            Ice::Long linkEntityId = _wireEntityId(link.entitySide);
            if (linkEntityId == entityId) {
                TopologyLink newLink = link;
                newLink.entitySide = "";
                _wiresLink.remove(id, link);
                _wiresLink.insertMulti(id, newLink);
                emit entityUnplugged(QString::fromStdString(link.wireSide), QString::fromStdString(link.entitySide));
            }
        }
    }
}

void TopologyI::checkCpapFiles(QObject *object)
{
    Q_UNUSED(object)
    QReadLocker locker(&_entitiesLock);
    foreach (EntityPrx entity, _entities)
        entity->checkCaptureFile();
}

void TopologyI::_plugChanged(::Ice::Long entityId, const QHash<Ice::Int, Ice::Int> &hashOld2New)
{
    QWriteLocker writeLockWire(&_wiresLock);
    QMutableHashIterator<Ice::Long, TopologyLink> it(_wiresLink);
    while (it.hasNext()) {
        it.next();
        Ice::Long linkedEntityId = _wireEntityId(it.value().entitySide);
        int linkedEntityPort = _wireEntityPort(it.value().entitySide);
        if (entityId == linkedEntityId && hashOld2New.keys().contains(linkedEntityPort)) {
            it.value().entitySide = _wireNewPort(it.value().entitySide, hashOld2New.value(linkedEntityPort)).toStdString();
            WireI *wirePointer = dynamic_cast<WireI*>(MasterI::instance()->adapter()->findByProxy(it.value().wireProxy).get());
            wirePointer->changePortFrom(hashOld2New.value(linkedEntityPort));
        }
    }
}

TopologyLink TopologyI::_getOtherSideLink(const TopologyLink &link)
{
    foreach (const TopologyLink &l, _wiresLink.values(_wireEntityId(link.wireSide))) {
        if ((_wireEntityId(l.entitySide) != _wireEntityId(link.entitySide)))
            return l;
    }

    return TopologyLink();
}

std::string TopologyI::_getRandomNodeNameForWire(const std::string &node1, const std::string &node2)
{
    double randNumber = (double)qrand();
    return (randNumber <= ((double)RAND_MAX * 0.5)) ? node1: node2;
}

WirePrx TopologyI::_getWirePrx(Ice::Long wireId)
{
    foreach (const TopologyLink &link, _wiresLink) {
        if (QString::fromStdString(link.wireSide).split(":").first().toLong() == wireId) {
            return link.wireProxy;
        }
    }
    return 0;
}

void TopologyI::entityChanged(QObject *object)
{
    EntityI* entity = qobject_cast<EntityI *>(object);
    Hns::Status status = entity->getStatus();
    qDebug() << tr("Entity %1 status has changed to %2").arg(entity->getEntry()->uuid.c_str()).arg(EntityI::statusToString(status));
    if (status == Hns::STOPPED || status == Hns::SAVED || status == Hns::STARTED)
        defineWireIfNeeded(entity);
    else if (status == Hns::UNDEFINED)
        entityUndefined(entity);
}

void TopologyI::defineWireIfNeeded(EntityI *entity)
{
    QReadLocker readLockerEntity(&_entitiesLock);
    QReadLocker readLockWire(&_wiresLock);
    _defineWireIfNeeded(entity);
}

void TopologyI::entityUndefined(EntityI *entity)
{
    /*
     *We need to undefine all the wires related to an entity as soon as the entity is
     *undefined
     */
    TopologyLinkList linksRelatedToUndefinableWires;
    foreach (const TopologyLink &link, _wiresLink) {
        if (_wireEntityId(link.entitySide) == entity->getId())
            linksRelatedToUndefinableWires.push_back(link);
    }
    foreach (const TopologyLink &link, linksRelatedToUndefinableWires) {
        WireI *wirePointer = dynamic_cast<WireI*>(MasterI::instance()->adapter()->findByProxy(link.wireProxy).get());
        if (wirePointer->getStatus() != Hns::UNDEFINED)
            wirePointer->undefine();
    }
}

void TopologyI::removeFromAdapter()
{
    emit requestDeletion(Diamesh::MasterI::instance()->adapter(), _proxyIdentity);
}

Diamesh::ACLList TopologyI::getACLs(const Ice::Current &current)
{
    return MasterI::instance()->topologyCatalog()->getACLsForCatalogItem(_topologyEntry->id, current);
}

bool TopologyI::setACLs(const Diamesh::ACLList &aclList, const Ice::Current &current)
{
    return MasterI::instance()->topologyCatalog()->setACLsForCatalogItem(_topologyEntry->id, aclList, current);
}

void TopologyI::_defineWireIfNeeded(EntityI *entity)
{
    TopologyLinkList linksRelatedToDefinableWires;
    foreach (const TopologyLink &link, _wiresLink) {
        if (_wireEntityId(link.entitySide) == entity->getId())
            linksRelatedToDefinableWires.push_back(link);
    }
    foreach (const TopologyLink &link, linksRelatedToDefinableWires) {
        TopologyLink otherSideLink = _getOtherSideLink(link);
        if (QString::fromStdString(link.entitySide).isEmpty() || QString::fromStdString(otherSideLink.entitySide).isEmpty()) {
            return;
        }
        Ice::Long entityId = _wireEntityId(link.entitySide);
        Ice::Long otherEntityId = _wireEntityId(otherSideLink.entitySide);
        int entityPort = _wireEntityPort(link.entitySide);
        int otherEntityPort = _wireEntityPort(otherSideLink.entitySide);

        if (!_getEntity(otherEntityId) || ! entity)
            return;

        Hns::Status entityStatus = _getEntity(otherEntityId)->getStatus();
        Hns::Status otherEntityStatus = _getEntity(entityId)->getStatus();
        if ((entityStatus == Hns::STOPPED || entityStatus == Hns::STARTED) && (otherEntityStatus == Hns::STOPPED || otherEntityStatus == Hns::STARTED)) {
            WireI *wirePointer = dynamic_cast<WireI*>(MasterI::instance()->adapter()->findByProxy(link.wireProxy).get());
            wirePointer->setLinkedEntities(_entities.value(entityId), entityPort, _entities.value(otherEntityId), otherEntityPort);
            wirePointer->define(_getRandomNodeNameForWire(entity->getNodeName(), _getEntity(otherEntityId)->getNodeName()));
        }
    }
}

bool TopologyI::_removeWire(Ice::Long wireId, const ::Ice::Current &current) //Should be deleted
{
    TopologyLink topologyLink = _wiresLink.value(wireId);

    Diamesh::UserPtr user = Diamesh::MasterI::instance()->sessionManager()->currentUser(current);

    // TODO:TestRight
    //    if (user && !user->isAdmin) {
    //        qCritical() << tr("The current user is not allowed to remove wire %1").arg(wireId);
    //        return false;
    //    };

    WirePrx proxy = topologyLink.wireProxy;
    EntityI *wireEntity = dynamic_cast<EntityI*>(MasterI::instance()->adapter()->findByProxy(proxy).get());
    wireEntity->destroy(current);
    foreach (const TopologyLink &l, _wiresLink.values(wireId)) {
        if (!l.entitySide.empty()) {
            EntityI * entity(_entity(_wireEntityId(l.entitySide)));
            if (entity)
                entity->unplug(_wireEntityPort(l.entitySide));
            else
                qDebug() << tr("Unable to unplug wire %1, entity does not exist").arg(wireId);
        }
    }
    _wiresLink.remove(wireId);
    _layout.wireLayouts.erase(wireId);
    setModified(true);
    emit wireRemoved(wireId);

    return true;
}

EntityI *TopologyI::_entity(::Ice::Long entityID)
{
    EntityI * entity = 0;
    if (_entities.contains(entityID)) {
        Ice::ObjectAdapterPtr adapter = MasterI::instance()->adapter();
        EntityPrx prx = _getEntity(entityID);
        if (prx)
            entity = dynamic_cast<EntityI *>(adapter->findByProxy(prx).get());
    }
    return entity;
}

EntityI *TopologyI::getEntityI(::Ice::Long entityID)
{
    QReadLocker locker(&_entitiesLock);
    return _entity(entityID);
}

EntityI *TopologyI::wireEntity(::Ice::Long wireID)
{
    QReadLocker locker(&_wiresLock);
    if (_wiresLink.contains(wireID))
        return dynamic_cast<EntityI *>(MasterI::instance()->adapter()->findByProxy(_wiresLink.value(wireID).wireProxy).get());
    return 0;
}

void TopologyI::clear(QList< Ice::Long > &entityRemoved)
{
    _entitiesLock.lockForRead();
    entityRemoved.clear();
    entityRemoved = _entities.keys();
    _wiresLock.lockForRead();
    QList<Ice::Long> wiresIds = _wiresLink.uniqueKeys();

    foreach (Ice::Long wireId, wiresIds)
        _removeWire(wireId);

    _entitiesLock.unlock();
    _wiresLock.unlock();
    QList< Ice::Long >::iterator itorList = entityRemoved.begin();
    while(itorList != entityRemoved.end()){
        if(!removeEntity((*itorList))){
            itorList = entityRemoved.erase(itorList);
        }
        else itorList++;
    }
}

EntityI *TopologyI::_getLinkedEntity(const TopologyLink &link)
{
    return _entity(_wireEntityId(link.entitySide));
}

int TopologyI::_wireEntityId(const ::std::string &wireEntity)
{
    return QString::fromStdString(wireEntity).split(":").first().toInt();
}

int TopologyI::_wireEntityPort(const ::std::string &wireEntity)
{
    return QString::fromStdString(wireEntity).split(":").last().toInt();
}

QString TopologyI::_wireNewPort(const ::std::string &wireEntity, int newPort)
{
    return QString::fromStdString(wireEntity).split(":").first()+":"+ QString::number(newPort);
}

} // namespace Hns
