/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "topologymanagerI.h"
#include "topologyI.h"
#include "entityinfoI.h"
#include "catalog/entitycatalogI.h"
#include "catalog/topologycatalogI.h"
#include "masterI.h"
#include "entityI.h"
#include "tracerI.h"
#include "domainI.h"

// hynesim common headers
#include "common/rightslist.h"

// Generated hynesim headers
#include "common/wire.h"
#include "common/topologylink.h"

// diamesh headers
#include <diamesh/sessionManagerI.h>
#include <diamesh/usermanager.h>
#include <diamesh/hsetmanagerI.h>
#include <diamesh/aclmanagerI.h>

// Qt headers
#include <QDebug>
#include <QDir>
#include <QFile>
#include <QFileInfo>
#include <QDateTime>

namespace Hns {

TopologyManagerI::TopologyManagerI()
    : _topologyCounter(0)
{
}

TopologyManagerI::~TopologyManagerI()
{
}

TopologyPrx TopologyManagerI::createTopology(bool isModel, const TopologyLayoutOptions &layoutOptions, const ::Ice::Current &current)
{
    QWriteLocker locker(&_topologiesLock);

    TopologyEntryPtr topologyEntry = new TopologyEntry();
    topologyEntry->shortName = QString("Session#%1").arg(++_topologyCounter).toStdString();
    topologyEntry->longName = "";
    topologyEntry->description = "";
    topologyEntry->creationDateTime = QDateTime::currentDateTime().toMSecsSinceEpoch();
    topologyEntry->isModel = isModel;
    Diamesh::UserPtr currentUser = MasterI::instance()->sessionManager()->currentUser(current);
    if (currentUser)
        topologyEntry->hSetId = currentUser->hSetId;
    else
        topologyEntry->hSetId = MasterI::instance()->hSetManager()->highestHSetId();

    TopologyCatalogI* catalog = MasterI::instance()->topologyCatalog();
    topologyEntry->uuid = catalog->generateUnusedUUID().toStdString();

    if (!catalog->addEntry(topologyEntry, current)) {
        qCritical() << tr("Unable to add topology entry to catalog");
        return NULL;
    }

    TopologyI *topology = new TopologyI(topologyEntry);
    _addTopologyToHashes(topology, topologyEntry->uuid.c_str());

    long topologyId = topology->getId();
    _setTopologyUsed(topologyId, current);


    // reset the topology entry path from /.tmp/ to an empty string
    // so that clients see the entry as a new one
    topologyEntry->path = "";

    catalog->lockEntry(topologyEntry->id);
    QObject::connect(topology, SIGNAL(entryUsed(Ice::Long)), this, SLOT(_entryUsedSlot(Ice::Long)));
    QObject::connect(topology, SIGNAL(entityAdded(Ice::Long, Ice::Long)), this, SLOT(_entityAddSlot(Ice::Long, Ice::Long)));
    QObject::connect(topology, SIGNAL(wireAdded(Ice::Long, Ice::Long)), this, SLOT(_wireAddSlot(Ice::Long, Ice::Long)));
    _connectToTopology(topology);

    topology->setLayoutOptions(layoutOptions);

    emit topologyLoaded(topologyEntry->uuid.c_str(), topologyId);
    return topology->proxy();
}

TopologyPrx TopologyManagerI::loadTopology(Ice::Long topologyEntryId, const ::Ice::Current &current)
{
    QReadLocker lockerBusy(&_topologiesBusyLock);
    if(_setBusyTopology.find(topologyEntryId) != _setBusyTopology.end()){
        HnsError exception;
        exception.message = tr("Please wait while closing topology.").toStdString();
        throw exception;
        return NULL;
    }
    lockerBusy.unlock();
    QWriteLocker locker(&_topologiesLock);

    TopologyCatalogI *catalog = MasterI::instance()->topologyCatalog();
    TopologyEntryPtr entry = TopologyEntryPtr::dynamicCast(catalog->getEntry(topologyEntryId));

    if (!entry) {
        qCritical() << tr("Unable to get topology entry %1").arg(topologyEntryId);
        return NULL;
    }
    Diamesh::UserPtr currentUser = MasterI::instance()->sessionManager()->currentUser(current);
    if (currentUser && !MasterI::instance()->aclManager()->testRightForObject(entry,currentUser,QString("%1%2").arg(Rights::TOPOLOGY_ENTRY_PREFIX).arg(Rights::ENTRY_SUFFIX_USE))) {
        qCritical() << tr("You (id %3) don't have the permission %1 on entry id:%2").arg(QString(Rights::TOPOLOGY_ENTRY_PREFIX) + Rights::ENTRY_SUFFIX_USE).arg(entry->id).arg(currentUser->id);
        return NULL;
    }
    int status = entry->status;
    if (status == ::Hns::TOPOLOGY_CONTAINS_ENTITY_WITH_UUID_CONFLICT){
        qCritical() << tr("There is some UUID conflicts with entities in this topology : %1").arg(entry->uuid.c_str());
        return NULL;
    } else if (status == ::Hns::TOPOLOGY_UUID_CONFLICT){
        qCritical() << tr("This topology has an UUID conflict :%1").arg(entry->uuid.c_str());
        return NULL;
    } else if (status == ::Hns::TOPOLOGY_LOADING_ERROR) {
        qCritical() << tr("This topology cannot be loaded, see logs for more details");
        return NULL;
    } else if (status == ::Hns::TOPOLOGY_BUSY_EXPORTING) {
        qCritical() << tr("Topology %1 with UUID %2 is currently busy (export) and can not be loaded.")
                       .arg(QString::fromStdString(entry->shortName))
                       .arg(QString::fromStdString(entry->uuid));
        return NULL;
    }

    QString uuid = entry->uuid.c_str();
    if (_uuidHash.contains(uuid)) {
        Ice::Long topologyId = _uuidHash[uuid];
        if (!_setTopologyUsed(topologyId, current)) {
            HnsError exception;
            exception.message = tr("Topology is already loaded").toStdString();
            throw exception;
            return NULL;
        }
        return _getTopologyById(topologyId, current);
    }

    if (!catalog->lockEntry(entry->id)) {
        qCritical() << tr("Topology entry %1 is locked, unable to load it").arg(entry->id);
        return NULL;
    }

    TopologyI* topology = new TopologyI(entry);
    _addTopologyToHashes(topology, uuid);
    QObject::connect(topology, SIGNAL(entryUsed(Ice::Long)), this, SLOT(_entryUsedSlot(Ice::Long)));
    QObject::connect(topology, SIGNAL(wireAdded(Ice::Long, Ice::Long)), this, SLOT(_wireAddSlot(Ice::Long, Ice::Long)));
    QObject::connect(topology, SIGNAL(entityAdded(Ice::Long, Ice::Long)), this, SLOT(_entityAddSlot(Ice::Long, Ice::Long)));

    QHash<int, int> oldNewEntityIdHash;
    QMap<int, EntityEntryPtr> entitiesHash(entry->entitiesMap);
    QMapIterator<int, EntityEntryPtr> entityIt(entitiesHash);

    //verification for pacp info files
    ::Hns::PcapMap::iterator itorPcapMap(entry->pcapInfoMap.begin());
    while(itorPcapMap != entry->pcapInfoMap.end()) {
        ::Hns::PcapInfoList::iterator itorList(itorPcapMap->second.begin());
        while(itorList != itorPcapMap->second.end()) {
            QString pathFileToCheck = QString::fromStdString(MasterI::instance()->topologyCatalog()->getFullEntryPathFromMixed((*itorList).path));
            if (!QFile::exists(pathFileToCheck)) {
                itorList =  itorPcapMap->second.erase(itorList);
            } else {
                itorList++;
            }
        }
        itorPcapMap++;
    }

    //verification for memory dump info file
    ::Hns::MemoryDumpMap::iterator itorMemoryDumpMap(entry->memoryDumpInfoMap.begin());
    while (itorMemoryDumpMap != entry->memoryDumpInfoMap.end()) {
        ::Hns::MemoryDumpInfoList::iterator itorList(itorMemoryDumpMap->second.begin());
        while (itorList != itorMemoryDumpMap->second.end()) {
            QString path = QString("%1/%2/memorydumps/%3")
                    .arg(QString::fromStdString(entry->path))
                    .arg(QString::fromStdString(entry->uuid))
                    .arg(QString::fromStdString((*itorList).fileName));
            QString filePathToCheck = QString::fromStdString(MasterI::instance()->topologyCatalog()->getFullEntryPathFromMixed(path.toStdString()));
            if (!QFile::exists(filePathToCheck))
                itorList = itorMemoryDumpMap->second.erase(itorList);
            else
                ++itorList;
        }
        ++itorMemoryDumpMap;
    }

    QMap<Ice::Int, Hns::PcapInfoList > qmapPcapInfoList(entry->pcapInfoMap);
    QMap<Ice::Int, Hns::MemoryDumpInfoList> memoryDumpInfoMap(entry->memoryDumpInfoMap);

    while (entityIt.hasNext()) {
        entityIt.next();
        int oldEntityId = entityIt.key();
        EntityEntryPtr entityEntry = entityIt.value();

        EntityPrx entity = NULL;
        bool entityEntryIsLocked = false;
        EntityLayoutPtr layout = new EntityLayout();
        try {
            entity = topology->createEntityFromEntry(entityEntry, layout, current, true).e;
        } catch (EntryLockedError &) {
            qCritical() << tr("Caught an 'entry locked' exception");
            entityEntryIsLocked = true;
        } catch (LinkedDomainError &) {
            qCritical() << tr("Caught a 'linked domain' exception");
        }

        if (!entity) {
            qCritical() << tr("Unable to recreate entity %1").arg(entityEntry->shortName.c_str());
            if (!entityEntryIsLocked) {
                entry->status = ::Hns::TOPOLOGY_LOADING_ERROR;
                catalog->updateEntry(entry);
            }
            _clearTopology(topology);
            _uuidHash.erase(_uuidHash.find(uuid));
            _topologyHash.erase(_topologyHash.find(topology->getId()));
            catalog->unlockEntry(entry->id);
            return NULL;
        }
        if (oldNewEntityIdHash.contains(oldEntityId)) {
            qCritical() << tr("Old entity %1 has already been loaded").arg(oldEntityId);
            entry->status = ::Hns::TOPOLOGY_LOADING_ERROR;
            _clearTopology(topology);
            _uuidHash.erase(_uuidHash.find(uuid));
            _topologyHash.erase(_topologyHash.find(topology->getId()));
            catalog->updateEntry(entry);
            catalog->unlockEntry(entry->id);
            return NULL;
        }
        oldNewEntityIdHash[oldEntityId] = entity->getId();

        entity->setPcapInfoList(qmapPcapInfoList.value(oldNewEntityIdHash.key(entity->getId())));
        DomainPrx domain = DomainPrx::checkedCast(entity);
        if (domain)
            domain->setMemoryDumpInfoList(memoryDumpInfoMap.value(oldNewEntityIdHash.key(entity->getId())));
    }

    QMultiHash<int, TopologyLink> topologyLinkMultiHash; //id: wireId
    QList<TopologyLink> topologyLinkList = QList<TopologyLink>::fromStdList(entry->topologyLinks);
    foreach (TopologyLink topologyLink, topologyLinkList) {
        int wireId = QString::fromStdString(topologyLink.wireSide).split(":").first().toInt(); //TODO: refactor, use a method
        /*need to get the new entityId to modify topologyLinkMultiHash*/
        if (!QString::fromStdString(topologyLink.entitySide).isEmpty()) {
            int entityPortId = QString::fromStdString(topologyLink.entitySide).split(':').last().toInt();
            qlonglong entityId = QString::fromStdString(topologyLink.entitySide).split(':').first().toLongLong();
            topologyLink.entitySide = QString("%1:%2").arg(oldNewEntityIdHash[entityId]).arg(entityPortId).toStdString();
        }
        topologyLinkMultiHash.insertMulti(wireId, topologyLink);
    }

    QHash<int, int> oldNewWireIdHash;
    QMap<int, WireEntryPtr> wiresMap(entry->wiresMap);
    QMapIterator<int, WireEntryPtr> wiresMapIterator(wiresMap);
    while (wiresMapIterator.hasNext()) {
        wiresMapIterator.next();
        int oldWireId = wiresMapIterator.key();
        WireEntryPtr wireEntry = wiresMapIterator.value();
        WireLayoutPtr layout = new WireLayout();
        EntityTopologyLinkStruct etls;
        bool entityEntryIsLocked = false;
        try {
            etls = topology->createEntityFromEntry(wireEntry, layout, current, true, topologyLinkMultiHash.values(oldWireId).toStdList());
        } catch (EntryLockedError &) {
            qCritical() << tr("Caught and 'entry locked' exception");
            entityEntryIsLocked = true;
        }
        if (!etls.e) {
            qCritical() << tr("Unable to create wire %1").arg(QString::fromStdString(wireEntry->shortName));
            if (!entityEntryIsLocked) { /*should never append*/
                entry->status = ::Hns::TOPOLOGY_LOADING_ERROR;
                catalog->updateEntry(entry, current);
            }
            _clearTopology(topology);
            _uuidHash.erase(_uuidHash.find(uuid));
            _topologyHash.erase(_topologyHash.find(topology->getId()));
            catalog->unlockEntry(entry->id);
            return NULL;
        }
        if (oldNewWireIdHash.contains(oldWireId)) {
            qCritical() << tr("Old wire %1 has already been loaded").arg(oldWireId);
            entry->status = ::Hns::TOPOLOGY_LOADING_ERROR;
            catalog->updateEntry(entry, current);
            _clearTopology(topology);
            _uuidHash.erase(_uuidHash.find(uuid));
            _topologyHash.erase(_topologyHash.find(topology->getId()));
            catalog->unlockEntry(entry->id);
            return NULL;
        }
        oldNewWireIdHash[oldWireId] = etls.e->getId();
    }

    TopologyLayout layout = catalog->getTopologyLayout(entry);
    QMap<Ice::Long, EntityLayoutPtr> entityLayoutMap(layout.entityLayouts);
    layout.entityLayouts.clear();
    foreach (EntityLayoutPtr entityLayout, entityLayoutMap) {
        int id = entityLayout->id;
        if (oldNewEntityIdHash.contains(id))
            id = oldNewEntityIdHash[id];
        entityLayout->id = id;
        layout.entityLayouts[id] = entityLayout;
    }
    QMap<Ice::Long, WireLayoutPtr> wireLayoutMap(layout.wireLayouts);
    layout.wireLayouts.clear();
    foreach (WireLayoutPtr wireLayout, wireLayoutMap) {
        int id = wireLayout->id;
        if (oldNewWireIdHash.contains(id))
            id = oldNewWireIdHash[id];
        wireLayout->id = id;
        layout.wireLayouts[id] = wireLayout;
        topology->updateWireLayout(wireLayout, true);
    }

    QString fullPath = QString::fromStdString(entry->path);
    QStringList entryDirList = fullPath.split(":", QString::SkipEmptyParts);
    QString entryDir("");
    if (!entryDirList.isEmpty())
        entryDir = entryDirList.last();
    // check whether the topology entry has already been saved once
    bool isTemporary = (entryDir == "/.tmp/");
    if (isTemporary)
        entry->path = "";
    topology->setLayout(layout, !isTemporary);
    long topologyId = topology->getId();

    _setTopologyUsed(topologyId, current);

    emit topologyLoaded(uuid, topologyId);
    _connectToTopology(topology);
    return topology->proxy();
}

bool TopologyManagerI::saveTopology(::Ice::Long topologyId, const ::Ice::Current &current)
{
    QWriteLocker locker(&_topologiesLock);
    if (!_topologyHash.contains(topologyId)) {
        qCritical() << "saveTopology:" << tr("Topology %1 was not found").arg(topologyId);
        return false;
    }

    TopologyI* topology = _topologyHash[topologyId];
    QString exceptionMessage;
    if (!topology->canBeSaved(exceptionMessage)) {
        SaveError exception;
        exception.message = exceptionMessage.toStdString();
        throw exception;
        return false;
    }

    return _saveTopology(topology, current);
}

bool TopologyManagerI::saveNewTopology(Ice::Long topologyId, const TopologyEntryPtr &entry, const Ice::Current &current)
{
    QWriteLocker locker(&_topologiesLock);
    if (!_topologyHash.contains(topologyId)) {
        qCritical() << "saveTopology:" << tr("Topology %1 was not found").arg(topologyId);
        return false;
    }

    TopologyI* topology = _topologyHash[topologyId];
    QString exceptionMessage;
    if (!topology->canBeSaved(exceptionMessage)) {
        SaveError exception;
        exception.message = exceptionMessage.toStdString();
        throw exception;
        return false;
    }
    topology->tracer()->close();
    if (topology->updateEntry(entry, current))
        return _saveTopology(topology, current);
    return false;
}

bool TopologyManagerI::saveTopologyAs(::Ice::Long topologyId, const TopologyEntryPtr &entry, const ::Ice::Current &current)
{
    QWriteLocker locker(&_topologiesLock);

    if (!_topologyHash.contains(topologyId)) {
        qCritical() << tr("Topology %1 was not found. Unable to clone it.").arg(topologyId);
        return false;
    }
    TopologyI* topology = _topologyHash[topologyId];
    QString exceptionMessage;
    if (!topology->canBeSaved(exceptionMessage)) {
        SaveError exception;
        exception.message = exceptionMessage.toStdString();
        throw exception;
        return false;
    }

    TopologyEntryPtr oldEntry = topology->getEntry(current);
    QString oldUuid = QString::fromStdString(oldEntry->uuid);
    Ice::Long oldEntryId = oldEntry->id;
    TopologyEntryPtr newEntry = TopologyEntryPtr::dynamicCast(oldEntry->ice_clone());

    if (newEntry == 0) {
        qCritical() << tr("Unable to clone topology entry");
        return false;
    }

    // Reset the entry ID and copy descriptive fields
    newEntry->id = 0;
    newEntry->shortName = entry->shortName;
    newEntry->longName = entry->longName;
    newEntry->description = entry->description;
    newEntry->tagList = entry->tagList;
    newEntry->hSetId = entry->hSetId;

    TopologyCatalogI* catalog = MasterI::instance()->topologyCatalog();
    QString newUuid = catalog->generateUnusedUUID();
    newEntry->uuid = newUuid.toStdString();

    Ice::Long newEntryId = catalog->addEntry(newEntry, current);
    if(!newEntryId) {
        qCritical() << tr("Unable to add topology entry");
        return false;
    }

    // unlock the original entry
    catalog->unlockEntry(oldEntryId);
    _uuidHash.remove(oldUuid);

    // lock the new entry
    _uuidHash[newUuid] = topologyId;
    newEntry->id = newEntryId;
    topology->updateEntry(newEntry);
    catalog->lockEntry(newEntryId);

    emit topologySavedAs(oldUuid, newUuid);

    return _saveTopology(topology, current);
}

Ice::Long TopologyManagerI::switchTopologyType(::Ice::Long topologyId, const ::std::string &newBackgroundColor, const ::Ice::Current &current)
{
    QWriteLocker locker(&_topologiesLock);

    TopologyI *topology = _topologyHash[topologyId];
    QString exceptionMessage;
    if (!topology->canBeSaved(exceptionMessage)) {
        SaveError exception;
        exception.message = exceptionMessage.toStdString();
        throw exception;
        return false;
    }

    TopologyEntryPtr entry = topology->getEntry(current);
    TopologyCatalogI* catalog = MasterI::instance()->topologyCatalog();
    TopologyEntryPtr newTopologyEntry = TopologyEntryPtr::dynamicCast(entry->ice_clone());
    newTopologyEntry->uuid = catalog->generateUnusedUUID().toStdString();
    // switch to or from model
    newTopologyEntry->isModel = !entry->isModel;
    newTopologyEntry->shortName = QString("Session#%1").arg(++_topologyCounter).toStdString();
    newTopologyEntry->path = ""; // so it is considered as a new topology (which has never been saved)
    if (!catalog->addEntry(newTopologyEntry, current)) {
        qCritical() << tr("Unable to add topology entry to catalog");
        return -1;
    }

    TopologyEntryPtr clonedEntry = TopologyEntryPtr::dynamicCast(newTopologyEntry->ice_clone());
    _exportTopology(topology, clonedEntry, current, newBackgroundColor);

    return newTopologyEntry->id;
}

bool TopologyManagerI::closeTopology(::Ice::Long topologyId, const ::Ice::Current &current)
{
    QWriteLocker topologiesLocker(&_topologiesLock);
    QWriteLocker busyLocker(&_topologiesBusyLock);

    if(!_topologyHash.contains(topologyId)) {
        qWarning() << tr("Topology %1 does not seem to be open. Not removing current user from counter").arg(topologyId);
        return true;
    }
    TopologyI* topology = _topologyHash[topologyId];

    Diamesh::UserPtr user = 0;

    if (!topology->getSessions().size()) {
        qWarning() << tr("Topology %1 does not seem to be open. Not removing current user from counter").arg(topologyId);
    } else {
        user = Diamesh::MasterI::instance()->sessionManager()->currentUser(current);
    }

    ::Ice::Long myId = -1;
    if(topology->getEntry()){
        myId = topology->getEntry()->id;
        _setBusyTopology.insert(myId);
    }
    busyLocker.unlock();
    if (user)
        emit userClosedTopology(user->id, topologyId);
    topology->removeSession(current.ctx.at("auth").c_str());

    bool returnValue = true;
    if (!topology->getSessions().size() && !topology->getEntry()->isPersistent)
        returnValue = _closeTopology(topologyId, current);
    if(myId != -1)
    {
        busyLocker.relock();
        _setBusyTopology.erase(_setBusyTopology.find(myId));
        busyLocker.unlock();
    }
    return returnValue;
}

bool TopologyManagerI::unloadTopology(::Ice::Long topologyId, const ::Ice::Current &current)
{
    QWriteLocker topologiesLocker(&_topologiesLock);

    if(!_topologyHash.contains(topologyId)) {
        qWarning() << tr("Topology %1 does not seem to be open. Not removing current user from counter").arg(topologyId);
        return true;
    }

    EntryPtr entry = _topologyHash.value(topologyId)->getEntry();
    if (!entry) {
        qCritical() << tr("Unable to get entry of topology %1. Unable to unload topology").arg(topologyId);
        return false;
    }

    Diamesh::UserPtr currentUser = MasterI::instance()->sessionManager()->currentUser(current);
    if (currentUser && !MasterI::instance()->aclManager()->testRightForObject(entry, currentUser,QString("%1%2").arg(Rights::TOPOLOGY_ENTRY_PREFIX).arg(Rights::ENTRY_SUFFIX_DELETE))) {
        qCritical() << tr("You (id %3) don't have the permission %1 on entry id:%2").arg(QString(Rights::TOPOLOGY_ENTRY_PREFIX) + Rights::ENTRY_SUFFIX_DELETE).arg(entry->id).arg(currentUser->id);
        return false;
    }

    return _closeTopology(topologyId, current);
}

bool TopologyManagerI::deleteTopology(::Ice::Long entryId, const ::Ice::Current &current)
{
    QWriteLocker topologyLocker(&_topologiesLock);

    TopologyCatalogI* catalog = MasterI::instance()->topologyCatalog();

    EntryPtr entry = catalog->getEntry(entryId, current);
    if (!entry) {
        qCritical() << tr("Unable to get entry %1. Unable to delete topology").arg(entryId);
        return false;
    }

    Diamesh::UserPtr currentUser = MasterI::instance()->sessionManager()->currentUser(current);
    if (currentUser && !MasterI::instance()->aclManager()->testRightForObject(entry, currentUser,QString("%1%2").arg(Rights::TOPOLOGY_ENTRY_PREFIX).arg(Rights::ENTRY_SUFFIX_DELETE))) {
        qCritical() << tr("You (id %3) don't have the permission %1 on entry id:%2").arg(QString(Rights::TOPOLOGY_ENTRY_PREFIX) + Rights::ENTRY_SUFFIX_DELETE).arg(entry->id).arg(currentUser->id);
        return false;
    }

    QString path = QString::fromStdString(entry->path);

    bool isLocked = catalog->entryIsLocked(entryId, current);
    if (isLocked) {
        QString topologyUuid = entry->uuid.c_str();

        if (!_uuidHash.contains(topologyUuid)) {
            qCritical() << tr("Topology with uuid %1 is locked but was not found in topology manager. Unable to delete it.").arg(topologyUuid);
            return false;
        }

        Ice::Long topologyId = _uuidHash[topologyUuid];

        if (!_topologyHash.contains(topologyId)) {
            qCritical() << tr("Topology with id %1 is locked but was not found in topology manager. Unable to delete it.").arg(topologyId);
            return false;
        }

        if (!_closeTopology(topologyId, current, true)) {
            qCritical() << tr("Unable to delete topology %1").arg(topologyId);
            return false;
        }

    }

    if (!path.isEmpty() || (path.isEmpty() && !isLocked)) { // if path is empty and entry locked, entry has been removed in _closeTopology
        if (!catalog->deleteEntry(entry, current)) {
            qCritical() << tr("Unable to delete topology %1").arg(entry->shortName.c_str());
            return false;
        }
    }
    return true;
}

Diamesh::LongList TopologyManagerI::getTopologies(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QReadLocker locker(&_topologiesLock);
    return _topologyHash.keys().toStdList();
}

TopologyPrx TopologyManagerI::getTopologyById(::Ice::Long id, const ::Ice::Current &current)
{
    QReadLocker locker(&_topologiesLock);
    return _getTopologyById(id, current);
}

TopologyPrx TopologyManagerI::getTopologyByUuid(const std::string &topologyUuid, const ::Ice::Current &current)
{
    QReadLocker locker(&_topologiesLock);
    return _getTopologyByUuid(topologyUuid, current);
}

::Diamesh::LongList TopologyManagerI::getTopologyUsers(::Ice::Long topologyId, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QReadLocker locker(&_topologiesLock);
    if (!_topologyHash.contains(topologyId)) {
        qCritical() << tr("Topology %1 was not found. Unable to get users").arg(topologyId);
        return Diamesh::LongList();
    }

    TopologyI *topology = _topologyHash[topologyId];

    ::Diamesh::LongList users;
    foreach(Diamesh::SessionI *session, topology->getSessions()) {
        Ice::Long userId = session->userId();
        users.push_back(userId);
    }
    return users;
}

EntityInfoPrx TopologyManagerI::getEntityInfo(::Ice::Long entryId, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QReadLocker locker(&_entityInfoLock);
    if(_entryEntityHash.contains(entryId) && _entityInfoHash.contains(_entryEntityHash[entryId]))
        return _entityInfoHash[_entryEntityHash[entryId]]->proxy();
    qDebug() << tr("No entity was found for entry %1").arg(entryId);
    return 0;
}

void TopologyManagerI::_addTopologyToHashes(TopologyI *topology, const QString &uuid)
{
    ::Ice::Long topologyId = topology->getId();
    _uuidHash[uuid] = topologyId;
    _topologyHash[topologyId] = topology;
}

void TopologyManagerI::_connectToTopology(TopologyI *topology)
{
    QObject::connect(topology, SIGNAL(modified(QString, bool)), this, SLOT(_topologyModifiedSlot(QString, bool)));
    QObject::connect(topology, SIGNAL(entityRemoved(Ice::Long)), this, SLOT(_entityRemovedSlot(Ice::Long)));
    QObject::connect(topology, SIGNAL(wireRemoved(Ice::Long)), this, SLOT(_entityRemovedSlot(Ice::Long)));
    QObject::connect(topology, SIGNAL(entityUnImplemented(Ice::Long)), this, SLOT(_entityUnImplemented(Ice::Long)));
    QObject::connect(topology, SIGNAL(entityImplemented(Ice::Long, Ice::Long)), this, SLOT(_entityImplemented(Ice::Long, Ice::Long)));
}

bool TopologyManagerI::_clearTopology(TopologyI *topology)
{
    QList< Ice::Long > entityRemoved;
    topology->clear(entityRemoved);

    foreach(Ice::Long entityID , entityRemoved){
        _entityRemovedSlot(entityID);
    }

    return true;
}

bool TopologyManagerI::_closeTopology(::Ice::Long topologyId, const ::Ice::Current &current, bool isDelete)
{
    Q_UNUSED(current)
    TopologyI* topology = _topologyHash[topologyId];

    TopologyEntryPtr entry = topology->topologyEntry();
    QString uuid = entry->uuid.c_str();
    QString path = entry->path.c_str();
    bool notSave = path.isEmpty() || path.split(":", QString::SkipEmptyParts).last() == "/.tmp/";
    emit topologyUnloaded(uuid, topology->getId(), notSave || isDelete);

    // Disconnect from the signals sent by the topology
    topology->QObject::disconnect(this);

    if (!_clearTopology(topology))
        return false;

    TopologyCatalogI* catalog = MasterI::instance()->topologyCatalog();
    catalog->unlockEntry(entry->id);

    _uuidHash.remove(uuid);
    if (notSave) {
        if (!catalog->deleteEntry(entry))
            qCritical() << tr("Unable to remove topology entry %1").arg(topologyId);
    } else {
        _deleteUnknownPcapFiles(topology);
        _deleteUnknownMemoryDumpFiles(topology);
    }

    topology->removeFromAdapter();
    _topologyHash.remove(topologyId);

    return true;
}

bool TopologyManagerI::_saveTopology(TopologyI *topology, const ::Ice::Current &current)
{
    TopologyEntryPtr entry = topology->topologyEntry();

    if (!_exportTopology(topology, entry, current))
        return false;

    topology->setModified(false);
    return true;
}

bool TopologyManagerI::_exportTopology(TopologyI *topology, TopologyEntryPtr &entry, const ::Ice::Current &current, const ::std::string &newBackgroundColor)
{
    if (entry->path == "") {
        qCritical() << tr("Topology path is not defined");
        return false;
    }

    bool isModel = entry->isModel;
    entry->entitiesMap.clear();
    entry->pcapInfoMap.clear();
    entry->memoryDumpInfoMap.clear();
    QMap<int,PcapInfoList> mapPcap(entry->pcapInfoMap);
    QMap<int,MemoryDumpInfoList> memoryDumpInfoMap;
    QHash< ::Ice::Long, EntityPrx> entities = topology->entities();
    QHashIterator< ::Ice::Long, EntityPrx> entityIt(entities);
    while (entityIt.hasNext()) {
        entityIt.next();
        EntityEntryPtr entityEntry = entityIt.value()->getEntry();
        if (isModel && !entityEntry->isModel) {
            entityEntry = Hns::EntityEntryPtr::dynamicCast(entityEntry->ice_clone());
            entityEntry->id = 0;
            entityEntry->path = "";
            entityEntry->uuid = MasterI::instance()->entityCatalog()->generateUnusedUUID().toStdString();
            entityEntry->isModel = true;
            entityEntry->isTemplate = false;
            entityEntry->hasResources = false;
        }
        entry->entitiesMap[entityIt.key()] = entityEntry;
        mapPcap.insert(entityIt.key(),entityIt.value()->getPcapInfoList());
        DomainPrx domain = DomainPrx::checkedCast(entityIt.value());
        if (domain)
            memoryDumpInfoMap.insert(entityIt.key(), domain->getMemoryDumpInfoList());
    }
    entry->pcapInfoMap = mapPcap.toStdMap();
    entry->memoryDumpInfoMap = memoryDumpInfoMap.toStdMap();

    entry->wiresMap.clear();
    QMultiHash<Ice::Long, TopologyLink> links = topology->wires();
    foreach (const Ice::Long &wireId, links.uniqueKeys()) {
        WireEntryPtr wireEntry = WireEntryPtr::dynamicCast(links.value(wireId).wireProxy->getEntry());
        entry->wiresMap.insert(std::make_pair(wireId,wireEntry));
    }

    entry->topologyLinks.clear();
    QMultiHash< ::Ice::Long, TopologyLink> wires = topology->wires();
    foreach (const TopologyLink &link, wires)
        entry->topologyLinks.push_back(link);

    TopologyLayout layout = topology->getLayout(current);
    if (newBackgroundColor != "")
        layout.layoutOptions.colorCode = newBackgroundColor;

    if (!MasterI::instance()->topologyCatalog()->updateEntry(entry)) {
        qCritical() << tr("Unable to update topology entry");
        return false;
    }
    if (!MasterI::instance()->topologyCatalog()->updateTopologyLayout(entry, layout)) {
        qCritical() << tr("Unable to update topology layout");
        return false;
    }

    QString path = QString::fromStdString(MasterI::instance()->topologyCatalog()->getFullEntryPathFromMixed(entry->path, entry->uuid));
    topology->tracer()->setDirectory(path);

    return true;
}

bool TopologyManagerI::_deleteUnknownPcapFiles(TopologyI *topology)
{
    TopologyEntryPtr entry = topology->topologyEntry();
    QString path = QString::fromStdString(MasterI::instance()->topologyCatalog()->getFullEntryPathFromMixed(entry->path,entry->uuid));

    QMap<int,PcapInfoList> qmapPcapInfoList(entry->pcapInfoMap);
    QStringList knownPcap;
    foreach (int i,qmapPcapInfoList.keys()){
        foreach(PcapInfo pcapInfo,qmapPcapInfoList.value(i)){
            knownPcap.append(QString::fromStdString(pcapInfo.path).split("/").last());
        }
    }

    QFileInfoList allPcap;
    QStringList capFilter;
    capFilter<<"*.cap";
    QDir dirPath;
    dirPath.cd(path+"/pcap/");
    if (!dirPath.exists()){
        qDebug()<< tr("There is no pcap files to remove");
        return false;
    } else {
        allPcap = dirPath.entryInfoList(capFilter,QDir::Files);
    }
    int removeCounter = 0;

    foreach (QFileInfo f,allPcap){
        if(!knownPcap.contains(f.fileName())){
            QFile fileToRemove(f.absoluteFilePath());
            if (fileToRemove.remove()){
                removeCounter++;
            }
        }
    }
    qDebug()<<tr("%1 pcap files have been deleted").arg(removeCounter);


    return true;
}

bool TopologyManagerI::_deleteUnknownMemoryDumpFiles(TopologyI *topology)
{
    TopologyEntryPtr entry = topology->topologyEntry();
    QString path = QString::fromStdString(MasterI::instance()->topologyCatalog()->getFullEntryPathFromMixed(entry->path, entry->uuid));

    QMap<int, MemoryDumpInfoList> memoryDumpInfoMap(entry->memoryDumpInfoMap);
    QStringList knownMemoryDumps;
    foreach(int i, memoryDumpInfoMap.keys()) {
        foreach (MemoryDumpInfo memoryDumpInfo, memoryDumpInfoMap.value(i)) {
            knownMemoryDumps.append(QString::fromStdString(memoryDumpInfo.fileName));
        }
    }

    QFileInfoList allMemoryDumps;
    QDir dumpsDir(path + "/memorydumps");
    if (!dumpsDir.exists()) {
        qDebug() << tr("There are no memory dump files to remove");
        return true;
    } else {
        QStringList dumpFilter = QStringList() << "*.tar.gz";
        allMemoryDumps = dumpsDir.entryInfoList(dumpFilter, QDir::Files);
    }

    int removeCounter = 0;
    foreach (QFileInfo f, allMemoryDumps) {
        if (!knownMemoryDumps.contains(f.fileName())) {
            QFile fileToRemove(f.absoluteFilePath());
            if (fileToRemove.remove())
                removeCounter++;
        }
    }
    qDebug() << tr("%1 memory dump file(s) have been deleted").arg(removeCounter);

    return true;
}

TopologyPrx TopologyManagerI::_getTopologyById(::Ice::Long id, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    if(!_topologyHash.contains(id)) {
        qCritical() << "getTopologyById:" << tr("Topology %1 was not found").arg(id);
        return NULL;
    }

    return _topologyHash[id]->proxy();
}

TopologyPrx TopologyManagerI::_getTopologyByUuid(const std::string &topologyUuid, const ::Ice::Current &current)
{
    if (!_uuidHash.contains(topologyUuid.c_str())) {
        qCritical() << tr("The topology %1 was not found.").arg(topologyUuid.c_str());
        return NULL;
    }
    return _getTopologyById(_uuidHash[topologyUuid.c_str()], current);
}

bool TopologyManagerI::_setTopologyUsed(long topologyId, const ::Ice::Current &current)
{
    Diamesh::UserPtr user = Diamesh::MasterI::instance()->sessionManager()->currentUser(current);
    if (!_topologyHash.contains(topologyId)) {
        qCritical() << tr("The topology %1 was not found.").arg(topologyId);
        return false;
    }

    if (_topologyHash[topologyId]->hasSession(current.ctx.at("auth").c_str())) {
        qCritical() << tr("The topology %1 have already session %2.").arg(topologyId).arg(current.ctx.at("auth").c_str());
        return false;
    }

    _topologyHash[topologyId]->addSession(current.ctx.at("auth").c_str());
    emit userOpenedTopology(user->id, topologyId);
    return true;
}

void TopologyManagerI::_topologyModifiedSlot(const QString &uuid, bool modified)
{
    QReadLocker locker(&_topologiesLock);
    if (_uuidHash.contains(uuid)) {
        ::Ice::Long idTopology = _uuidHash[uuid];
        QReadLocker lockerBusy(&_topologiesBusyLock);
        if(_setBusyTopology.find(idTopology) != _setBusyTopology.end())
            return;
        lockerBusy.unlock();

    }
    locker.unlock();

    emit topologyModified(uuid, modified);
}

void TopologyManagerI::_entityRemovedSlot(Ice::Long entityId)
{
    QWriteLocker locker(&_entityInfoLock);
    if (_entityInfoHash.contains(entityId)) {
        ::Ice::Long entryId = _entityInfoHash[entityId]->entryId();
        _entityInfoHash[entityId]->removeFromAdapter();
        _entityInfoHash.remove(entityId);
        _entryEntityHash.remove(entryId);
        locker.unlock();
        emit removeEntityInfo(entityId);
    }
}

void TopologyManagerI::_entityAddSlot(Ice::Long topologyId, Ice::Long entityId)
{
    QReadLocker readlocker(&_topologiesLock);
    QWriteLocker locker(&_entityInfoLock);
    if (_topologyHash.contains(topologyId)) {
        TopologyI* topology = _topologyHash[topologyId];
        EntityI * entity = topology->getEntityI(entityId);
        if (entity) {
            EntityInfoI *entityInfo = new EntityInfoI(entity);
            _entityInfoHash[entityId] = entityInfo;
            _entryEntityHash[entity->getEntry()->id] = entityId;
            readlocker.unlock();
            emit addEntityInfo(entityId,QString::fromStdString(_entityInfoHash[entityId]->proxy()->ice_toString()));
        }
    }
}

void TopologyManagerI::_wireAddSlot(Ice::Long topologyId, Ice::Long wireId)
{
    QReadLocker readlocker(&_topologiesLock);
    QWriteLocker locker(&_entityInfoLock);
    if (_topologyHash.contains(topologyId)) {
        TopologyI* topology = _topologyHash[topologyId];
        EntityI * entity = topology->wireEntity(wireId);
        if (entity) {
            EntityInfoI *entityInfo = new EntityInfoI(entity);
            _entityInfoHash[wireId] = entityInfo;
            _entryEntityHash[entity->getEntry()->id] = wireId;
            emit addEntityInfo(wireId,QString::fromStdString(_entityInfoHash[wireId]->proxy()->ice_toString()));
        }
    }
}

void TopologyManagerI::_entityUnImplemented(Ice::Long entityEntryID)
{
    QWriteLocker locker(&_entityInfoLock);
    if (_entryEntityHash.contains(entityEntryID)) {
        _entryEntityHash.remove(entityEntryID);
    }
}

void TopologyManagerI::_entityImplemented(Ice::Long entityId, Ice::Long entryId)
{
    QWriteLocker locker(&_entityInfoLock);
    if (_entityInfoHash.contains(entityId)) {
        _entryEntityHash[entryId] = entityId;
    }
}

void TopologyManagerI::_entryUsedSlot(Ice::Long entryId)
{
    QWriteLocker locker(&_entityWaitInfoLock);
    if(_setEntityWait.find(entryId) != _setEntityWait.end()){
        _setEntityWait.erase(_setEntityWait.find(entryId));
        emit requestEntityInfo(entryId);
    }
}

void TopologyManagerI::requestGetEntityInfo(Ice::Long entryId, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QWriteLocker locker(&_entityWaitInfoLock);
    if(getEntityInfo(entryId) !=  0)
        emit requestEntityInfo(entryId);
    else
        _setEntityWait.insert(entryId);
}

::Hns::EntityInfoMap TopologyManagerI::getListEntityInfo(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    ::Hns::EntityInfoMap hashEntityInfo;
    QHash< ::Ice::Long, EntityInfoI*>::const_iterator i = _entityInfoHash.begin();
    while (i != _entityInfoHash.end()) {
        hashEntityInfo[i.key()] = i.value()->proxy();
        ++i;
    }
    return hashEntityInfo;
}

} // namespace Hns
