/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "tracerI.h"

// hynesim common headers
#include "common/constants.h"

// Generated hynesim headers
#include "common/usermanager.h"

// diamesh headers
#include <diamesh/diamesh.h>
#include <diamesh/masterI.h>
#include <diamesh/sessionManagerI.h>
#include <diamesh/sqldatabase.h>

// ZeroC Ice headers
#include <Ice/Identity.h>

// Qt headers
#include <QDateTime>
#include <QFile>
#include <QHash>
#include <QScopedPointer>
#include <QSettings>
#include <QStringList>
#include <QDebug>
#include <QSqlRecord>

namespace Hns {

typedef QHash<QString, TracerI *> TracerHash;
typedef QHash<QString, int> TracerCountHash;
Q_GLOBAL_STATIC(TracerHash, tracerHash)
Q_GLOBAL_STATIC(QMutex, tracerMutex)
Q_GLOBAL_STATIC(TracerCountHash, tracerCountHash)

const char *const dbFilename = "hnsTrace.db";
const char *const createQuery = "create table trace(id integer primary key autoincrement, category text, content text, "
                                "userName text, userLogin text, userId integer, date integer)";
const char *const insertQuery = "insert into trace (category, content, userName, userLogin, userId, date) values(?, ?, ?, ?, ?, ?)";

TracerI::TracerI()
    : _Hns_Tracer()
    , _db(NULL)
{
    _prx = TracerPrx::uncheckedCast(Diamesh::MasterI::instance()->adapter()->addWithUUID(this));
}

TracerI::~TracerI()
{
    delete _db;
}

TracerI *TracerI::get(const QString &directory)
{
    QMutexLocker locker(tracerMutex());
    if (tracerHash()->contains(directory)) {
        tracerCountHash()->insert(directory, tracerCountHash()->value(directory) + 1);
        return tracerHash()->value(directory);
    }
    locker.unlock();

    TracerI *tracer = new TracerI();
    tracer->setDirectory(directory);
    return tracer;
}

Diamesh::SqlDatabase *TracerI::getTracerDb(const QString &directory)
{
    QString path(directory);
    if (path.isEmpty()) {
        QSettings settings(Diamesh::instance()->configFileName(), QSettings::IniFormat);
        settings.beginGroup(Constants::SETTINGS_HYNESIM_GROUP);
        path = settings.value("defaultTracePath", "/var/log/").toString();
    }

    QString dbName = path + "/" + dbFilename;
    bool createTable = false;
    if (!QFile::exists(dbName)) {
        createTable = true;
        QFile file(dbName);
        if (!file.open(QIODevice::WriteOnly)) {
            qWarning() << tr("Unable to create trace database (%1)").arg(dbName);
            return NULL;
        }
    }

    QScopedPointer<Diamesh::SqlDatabase> db(new Diamesh::SqlDatabase("QSQLITE", dbName));
    if (createTable && (db->executeQuery(createQuery, QVariantList()) != 0)) {
        qWarning() << tr("Unable to create trace table");
        return NULL;
    }

    return db.take();
}

void TracerI::setDirectory(const QString &directory)
{
    if (_db)
        return;

    _db = getTracerDb(directory);
    QMutexLocker locker(tracerMutex());
    if (_db && !tracerHash()->contains(directory)) {
        tracerHash()->insert(directory, this);
        tracerCountHash()->insert(directory, 1);
        _directory = directory;
    }

    QReadLocker tracesLocker(&_traceListLock);
    foreach (const Trace &t, _traceList)
        _sqlTrace(t);
}

const QString &TracerI::directory()
{
    return _directory;
}

void TracerI::close()
{
    if (_db) {
        delete _db;
        _db = NULL;
    }
}

void TracerI::trace(const QString &content, const Ice::Current &current, const QString &category)
{
    Trace t;
    t.category = category.toStdString();
    t.content = content.toStdString();
    t.date = QDateTime::currentMSecsSinceEpoch();
    UserPtr user = UserPtr::dynamicCast(Diamesh::MasterI::instance()->sessionManager()->currentUser(current));
    if (user) {
        t.userName = user->firstname + " " + user->lastname;
        t.userLogin = user->name;
        t.userId = user->id;
    }

    _traceListLock.lockForWrite();
    _traceList.append(t);
    _traceListLock.unlock();

    _sqlTrace(t);

    emit newTrace(t);
}

TracerPrx TracerI::proxy()
{
    return _prx;
}

void TracerI::destroy()
{
    QMutexLocker locker(tracerMutex());
    if (!_directory.isEmpty()) {
        tracerCountHash()->insert(directory(), tracerCountHash()->value(directory()) - 1);
        if (tracerCountHash()->value(directory()) == 0) {
            tracerHash()->remove(_directory);
            emit requestDeletion(Diamesh::MasterI::instance()->adapter(), _prx->ice_getIdentity());
        }
    } else {
        emit requestDeletion(Diamesh::MasterI::instance()->adapter(), _prx->ice_getIdentity());
    }
}

TraceList TracerI::getLatestTraces(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QReadLocker locker(&_traceListLock);
    return _traceList.toStdList();
}

TraceList TracerI::getHistory(::Ice::Int pageNumber, ::Ice::Int count, const::std::string &match, ::Ice::Long beginDate, ::Ice::Long endDate, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    if (!_db)
        return TraceList();

    QString query("select * from trace");
    QString conditions;
    QVariantList values;

    if (beginDate != 0) {
        conditions.append(QString("(date > %1").arg(beginDate));
        if (endDate != 0)
            conditions.append(QString(" and date < %1").arg(endDate));
        conditions.append(")");
    }

    QString search = QString::fromStdString(match);
    if (!search.isEmpty()) {
        conditions.append(conditions.isEmpty() ? "(" : " and (");
        QStringList columnList = QStringList() << "content" << "userName" << "userLogin";
        QStringList searchConditions;
        foreach (const QString &column, columnList) {
            searchConditions << QString("%1 like ?").arg(column);
            values.append(QString("%%1%").arg(search));
        }
        conditions.append(searchConditions.join(" or "));
        conditions.append(")");
    }

    if (!conditions.isEmpty())
        query.append(QString(" where %1").arg(conditions));

    query.append(" order by id desc");

    int offset = (pageNumber - 1) * count;
    query.append(QString(" limit %1 offset %2").arg(count).arg(offset));

    QList<QSqlRecord> results;
    if (_db->executeQuery(query, values, &results) == -1) {
        qWarning() << tr("Error getting traces from database");
        return TraceList();
    }

    TraceList traceList;
    foreach (const QSqlRecord & sqlRecord, results) {
        Trace t;
        t.category = sqlRecord.value("category").toString().toStdString();
        t.content = sqlRecord.value("content").toString().toStdString();
        t.userName = sqlRecord.value("userName").toString().toStdString();
        t.userLogin = sqlRecord.value("userLogin").toString().toStdString();
        t.userId = sqlRecord.value("userId").toULongLong();
        t.date = sqlRecord.value("date").toULongLong();
        traceList.push_back(t);
    }

    return traceList;
}

void TracerI::_sqlTrace(const Trace &t)
{
    if (!_db)
        return;

    QVariantList args;
    args << QString::fromStdString(t.category);
    args << QString::fromStdString(t.content);
    args << QString::fromStdString(t.userName);
    args << QString::fromStdString(t.userLogin);
    args << qulonglong(t.userId);
    args << qulonglong(t.date);

    _db->executeQuery(insertQuery, args);
}

} // namespace Hns
