/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "bridgemonitor.h"
#include "nodescriptrunner.h"

// Qt headers
#include <QDebug>
#include <QTimerEvent>
#include <QDateTime>
#include <QReadLocker>
#include <QSocketNotifier>

// C++ std headers
#include <net/if.h>
#include <arpa/inet.h>

#include <linux/netfilter.h>
#include <linux/netfilter/nfnetlink.h>
#include <linux/netfilter/nfnetlink_log.h>

namespace HnsNode {

/*******************************************************************************
* BridgeMonitor
*******************************************************************************/
BridgeMonitor *BridgeMonitor::_instance = NULL;

BridgeMonitor::BridgeMonitor(QObject *parent)
    : QObject(parent)
{
    _start();
}

BridgeMonitor::~BridgeMonitor()
{
    _stop();
    qDeleteAll(_nflogHash);
}

BridgeMonitor *BridgeMonitor::instance()
{
    return _instance;
}

void BridgeMonitor::create(QObject *parent)
{
    if (_instance == NULL)
        _instance = new BridgeMonitor(parent);
}

QHash<QString, bool> BridgeMonitor::getMacAddresses(QString bridgeName, int groupId)
{
    QReadLocker locker(&_lock);
    QPair<QString, int> key;
    key.first = bridgeName;
    key.second = groupId;
    if (_data.contains(key)) {
        QList<MacAddressInformation> value = _data.value(key);
        QHash<QString, bool> hashToReturn;
        foreach (MacAddressInformation info, value) {
            hashToReturn.insert(info.macAddress, info.isActive);
        }
        return hashToReturn;
    }
    return QHash<QString, bool>();
}

void BridgeMonitor::registerEntity(QString bridgeName, int groupId, QObject *entity)
{
    QPair<QString, int> info;
    info.first = bridgeName;
    info.second = groupId;
    ScriptRunner runner;
    QStringList args = QStringList() << "--start"<< bridgeName << QString::number(groupId);
    runner.runSynchronous("hnsNflogGroup.sh", args);

    QWriteLocker locker(&_lock);
    _instance->_objectHash.insert(info, entity);
    QNFlog *nFlog;
    if (_nflogHash.contains(groupId)) {
        nFlog = _nflogHash.value(groupId);
        nFlog->addBridge(bridgeName);
    }
    else {
        nFlog = new QNFlog(groupId);
        if (nFlog->start()) {
            _nflogHash.insert(groupId, nFlog);
            nFlog->addBridge(bridgeName);
        }
        else {
            nFlog->deleteLater();
        }
    }
}

void BridgeMonitor::unregisterEntity(QObject *entity)
{
    QWriteLocker locker(&_lock);
    QPair<QString, int> key = _objectHash.key(entity);
    _objectHash.remove(key);
    _nflogHash.value(key.second)->removeBridge(key.first);
    locker.unlock();

    ScriptRunner runner;
    QStringList args = QStringList() << "--stop"<< key.first << QString::number(key.second);
    runner.runSynchronous("hnsNflogGroup.sh", args);
}

void BridgeMonitor::timerEvent(QTimerEvent *event)
{
    Q_UNUSED(event);
    QWriteLocker locker(&_lock);
    QHashIterator<QPair<QString, int>, QObject*> iterator(_objectHash);
    while (iterator.hasNext()) {
        iterator.next();
        QPair<QString, int> key = iterator.key();
        QObject *entity = iterator.value();
        if (_nflogHash.contains(key.second) && _nflogHash.value(key.second)->canHandleBridge(key.first)) {
            QList<MacAddressInformation> newInfos = _nflogHash.value(key.second)->getMacAddressInformation(key.first);
            QList<MacAddressInformation> oldInfos = _data.value(key);
            foreach (MacAddressInformation info, newInfos) {
                QGenericArgument arg = Q_ARG(QString, info.macAddress);
                if (info.timestamp < QDateTime::currentMSecsSinceEpoch() - 64000) { //1min04sec
                    if (oldInfos.contains(info)) {
                        QMetaObject::invokeMethod(entity, "macAddrGone", arg);
                        oldInfos.removeOne(info);
                    }
                    continue;
                }
                if (info.timestamp < QDateTime::currentMSecsSinceEpoch() - 32000) { //32sec
                    info.isActive = false;
                }
                else {
                    info.isActive = true;
                }
                if (oldInfos.contains(info)) {
                    int index = oldInfos.indexOf(info);
                    if (oldInfos.at(index).isActive != info.isActive) {
                        if (info.isActive) {
                            QMetaObject::invokeMethod(entity, "macBecameActive", arg);
                        }
                        else {
                            QMetaObject::invokeMethod(entity, "macBecameInactive", arg);
                        }
                    }
                    oldInfos.replace(index, info);
                }
                else {
                    QMetaObject::invokeMethod(entity, "newMacAddr", arg);
                    oldInfos.append(info);
                }
            }
            _data.insert(key, oldInfos);
        }
    }
}

void BridgeMonitor::_start(int interval)
{
    _timerId = startTimer(interval);
}

void BridgeMonitor::_stop()
{
    killTimer(_timerId);
    foreach (QNFlog *nFlog, _nflogHash)
        nFlog->stop();
}

/*******************************************************************************
* QNFlog
*******************************************************************************/
QNFlog::QNFlog(int groupId, QObject *parent)
    : QObject(parent)
    , _groupId(groupId)
{
}

QNFlog::~QNFlog()
{
}

bool QNFlog::start()
{
    char buf[MNL_SOCKET_BUFFER_SIZE];
    struct nlmsghdr *nlMessageHandler = 0;
    int fd = -1;

    _mnlSocket = mnl_socket_open(NETLINK_NETFILTER);
    if (!_mnlSocket) {
        qCritical() << QObject::tr("Unable to open mnl socket");
        return false;
    }
    if (mnl_socket_bind(_mnlSocket, 0, MNL_SOCKET_AUTOPID) < 0) {
        qCritical() << QObject::tr("Unable to bind mnl socket");
        return false;
    }
    _portId = mnl_socket_get_portid(_mnlSocket);

    nlMessageHandler = _nflogBuildConfigRequest(buf, NFULNL_CFG_CMD_PF_UNBIND);

    if (mnl_socket_sendto(_mnlSocket, nlMessageHandler, nlMessageHandler->nlmsg_len) < 0) {
        qCritical() << QObject::tr("Unable to send CMD_PF_UNBIND to socket");
        return false;
    }

    nlMessageHandler = _nflogBuildConfigRequest(buf, NFULNL_CFG_CMD_PF_BIND);

    if (mnl_socket_sendto(_mnlSocket, nlMessageHandler, nlMessageHandler->nlmsg_len) < 0) {
        qCritical() << QObject::tr("Unable to send CMD_PF_BIND to socket");
        return false;
    }

    nlMessageHandler = _nflogBuildConfigRequest(buf, NFULNL_CFG_CMD_BIND, true);

    if (mnl_socket_sendto(_mnlSocket, nlMessageHandler, nlMessageHandler->nlmsg_len) < 0) {
        qCritical() << QObject::tr("Unable to send CMD_BIND to socket");
        return false;
    }

    nlMessageHandler = _nflogBuildConfigParams(buf, NFULNL_COPY_PACKET, 0xFFFF);

    if (mnl_socket_sendto(_mnlSocket, nlMessageHandler, nlMessageHandler->nlmsg_len) < 0) {
        qCritical() << QObject::tr("Unable to send COPY_PACKET to socket");
        return false;
    }

    fd = mnl_socket_get_fd(_mnlSocket);
    _socketNotifier = new QSocketNotifier(fd, QSocketNotifier::Read, this);
    connect(_socketNotifier, SIGNAL(activated(int)), this, SLOT(_notifierSlot(int)));

    return true;
}

void QNFlog::stop()
{
    _socketNotifier->disconnect();
    mnl_socket_close(_mnlSocket);
}

QList<MacAddressInformation> QNFlog::getMacAddressInformation(QString bridgeName)
{
    if (_data.contains(bridgeName))
        return _data.value(bridgeName);
    return QList<MacAddressInformation>();
}

void QNFlog::addBridge(QString bridgeName)
{
    QList<MacAddressInformation> list;
    _data.insert(bridgeName, list);
}

void QNFlog::removeBridge(QString bridgeName)
{
    if (_data.contains(bridgeName))
        _data.remove(bridgeName);
}

bool QNFlog::canHandleBridge(QString bridgeName)
{
    if (_data.contains(bridgeName))
        return true;
    return false;
}

void QNFlog::_notifierSlot(int fd)
{
    Q_UNUSED(fd)
    char buf[MNL_SOCKET_BUFFER_SIZE];

    int ret = mnl_socket_recvfrom(_mnlSocket, buf, sizeof(buf));
    if (ret < 0) {
        qCritical() << QObject::tr("Unable to receive data from the socket");
        return;
    }

    if (mnl_cb_run(buf, ret, 0, _portId, QNFlog::_QNFlogCallback, this) < 0) {
        qCritical() << QObject::tr("Unable to get ARP information");
        return;
    }
}

static int _parseAttributeCallback(const struct nlattr *attr, void *data)
{
    const struct nlattr **tb = (const struct nlattr **)data;
    int type = mnl_attr_get_type(attr);

    /* skip unsupported attribute in user-space */
    if (mnl_attr_type_valid(attr, NFULA_MAX) < 0)
        return MNL_CB_OK;

    switch(type) {
    case NFULA_IFINDEX_OUTDEV:
        if (mnl_attr_validate(attr, MNL_TYPE_U32) < 0) {
            qCritical() << QObject::tr("Unable to validate type of IFINDEX_OUTDEV"); //TODO
            return MNL_CB_ERROR;
        }
        break;
    case NFULA_HWADDR:
        if (mnl_attr_validate2(attr, MNL_TYPE_UNSPEC, sizeof(struct nfulnl_msg_packet_hw)) < 0) {
            qCritical() << QObject::tr("Unable to valide type of HWADDR");
            return MNL_CB_ERROR;
        }
        break;
    }
    tb[type] = attr;
    return MNL_CB_OK;
}

int QNFlog::_QNFlogCallback(const nlmsghdr *nlh, void *data)
{
    struct nlattr *tb[NFULA_MAX+1] = {};
    QString bridgeName;
    QString macAddress;

    QNFlog *me = static_cast<QNFlog *>(data);
    if (!me)
        return MNL_CB_ERROR;

    mnl_attr_parse(nlh, sizeof(struct nfgenmsg), _parseAttributeCallback, tb);

    if (tb[NFULA_IFINDEX_OUTDEV]) {
        int outdev = ntohl(mnl_attr_get_u32(tb[NFULA_IFINDEX_OUTDEV]));
        if (outdev > 0) {
            char ifName[IF_NAMESIZE];
            bridgeName = QString(if_indextoname(outdev, ifName));
        }
    }
    if (bridgeName.isEmpty() || !me->_data.contains(bridgeName))
        return MNL_CB_OK;

    if (tb[NFULA_HWADDR]) {
        struct nfulnl_msg_packet_hw *msg_packet_hw = (struct nfulnl_msg_packet_hw *)mnl_attr_get_payload(tb[NFULA_HWADDR]);
        if (msg_packet_hw) {
            int hlen = ntohs(msg_packet_hw->hw_addrlen);
            for (int i = 0 ; i < hlen ; ++i) {
                QString str;
                str = str.sprintf("%02x", msg_packet_hw->hw_addr[i]);
                macAddress.append(str);
                if (i == hlen -1)
                    break;
                else
                    macAddress.append(":");
            }
        } else {
            return MNL_CB_ERROR;
        }
    }

    qint64 timestamp = QDateTime::currentMSecsSinceEpoch();
    MacAddressInformation macAddressInformation;
    macAddressInformation.macAddress = macAddress;
    macAddressInformation.timestamp = timestamp;
    QList<MacAddressInformation> list;
    if (me->_data.value(bridgeName).contains(macAddressInformation)) {
        list = me->_data.value(bridgeName);
        int idx = me->_data.value(bridgeName).indexOf(macAddressInformation);
        list.replace(idx, macAddressInformation); //just change the timestamp
        me->_data.insert(bridgeName, list);
    }
    else {
        list = me->_data.value(bridgeName);
        list.append(macAddressInformation);
        me->_data.insert(bridgeName, list);
    }
    return MNL_CB_OK;

}

nlmsghdr *QNFlog::_nflogBuildConfigRequest(char *buf, uint8_t command, bool useGroupId)
{
    struct nlmsghdr *nlh = mnl_nlmsg_put_header(buf);
    nlh->nlmsg_type = (NFNL_SUBSYS_ULOG << 8) | NFULNL_MSG_CONFIG;
    nlh->nlmsg_flags = NLM_F_REQUEST;

    struct nfgenmsg *nfg = (struct nfgenmsg *)mnl_nlmsg_put_extra_header(nlh, sizeof(*nfg));
    nfg->nfgen_family = AF_BRIDGE;
    nfg->version = NFNETLINK_V0;
    if (useGroupId)
        nfg->res_id = htons(_groupId);

    struct nfulnl_msg_config_cmd cmd;
    cmd.command = command;
    mnl_attr_put(nlh, NFULA_CFG_CMD, sizeof(cmd), &cmd);

    return nlh;
}

nlmsghdr *QNFlog::_nflogBuildConfigParams(char *buf, uint8_t mode, int range)
{
    struct nlmsghdr *nlh = mnl_nlmsg_put_header(buf);
    nlh->nlmsg_type = (NFNL_SUBSYS_ULOG << 8) | NFULNL_MSG_CONFIG;
    nlh->nlmsg_flags = NLM_F_REQUEST;

    struct nfgenmsg *nfg = (struct nfgenmsg *) mnl_nlmsg_put_extra_header(nlh, sizeof(*nfg));
    nfg->nfgen_family = AF_UNSPEC;
    nfg->version = NFNETLINK_V0;
    nfg->res_id = htons(_groupId);

    struct nfulnl_msg_config_mode params;
    params.copy_range = htonl(range);
    params.copy_mode = mode;
    mnl_attr_put(nlh, NFULA_CFG_MODE, sizeof(params), &params);

    return nlh;
}

} // namespace HnsNode
