/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "capturehandler.h"
#include "nodeentityI.h"
#include "nodeI.h"

// hynesim server headers
#include "libhnsserver/storagemanager.h"

// Qt headers
#include <QDateTime>
#include <QFile>
#include <QTimer>
#include <QDebug>

namespace HnsNode {

CaptureHandler::CaptureHandler()
    : _entity(NULL),
      _started(false)
{
}

CaptureHandler::~CaptureHandler()
{
    delete(_captureTimer);
    delete(_progressTimer);
}

void CaptureHandler::init(::HnsNode::EntityI *entity, QString captureJobPath)
{
    // init timers
    /* Timer for the elapsed capture time */
    _captureTimer = new QTimer();
    _captureTimer->moveToThread(thread());
    QTimer::connect(_captureTimer, SIGNAL(timeout()), this, SLOT(endCapture()));

    /* Timer for the capture progress */
    _progressTimer = new QTimer();
    _progressTimer->moveToThread(thread());
    QTimer::connect(_progressTimer, SIGNAL(timeout()), this, SLOT(_updateCaptureProgress()));
    _progressTimer->setInterval(500);

    // init entity
    _entity = entity;
    _availableInterfaces = QList<Ice::Int>::fromStdList(_entity->getAvailablePlugsForCapture());
    QObject::connect(
                _entity, SIGNAL(capturableInterfacesListChanged(::Diamesh::IntList)),
                this, SLOT(_updateAvailableInterfacesList(::Diamesh::IntList)));

    // init capture path
    _captureMixedPath = captureJobPath + "/pcap/";
    HnsServer::StorageManager *storageManager = Hns::NodeI::instance()->storageManager();
    _captureFullPath = storageManager->getFullPathFromMixed("topology", captureJobPath) + "/pcap/";
    _startDateTime = QDateTime::currentMSecsSinceEpoch();
    _endDateTime = -1;

    QObject::connect(this, SIGNAL(progressChanged(double)), SLOT(_progressChangedSlot(double)));
}

bool CaptureHandler::setOptions(const Hns::CaptureJobOptions &jobOptions)
{
    _jobOptions = jobOptions;
    _generateFileName();
    return true;
}

void CaptureHandler::_generateFileName()
{
    QDateTime date;
    date.setMSecsSinceEpoch(_startDateTime);
    _captureFileName = QString("%1_%2_%3.cap")
            .arg(_getFileNamePrefix())
            .arg(date.toString("dd-MM-yyyy-HH-mm-ss"))
            .arg(QString::fromStdString(_entity->getEntry()->uuid));
}

bool CaptureHandler::started()
{
    QReadLocker locker(&_startedLock);
    return _started;
}

void CaptureHandler::setStarted(bool started)
{
    QWriteLocker locker(&_startedLock);
    _started = started;
}

QString CaptureHandler::getCaptureFileMixedPath()
{
    return _captureMixedPath + _captureFileName;
}

void CaptureHandler::_updateAvailableInterfacesList(::Diamesh::IntList listInterfaces)
{
    _availableInterfaces = QList<Ice::Int>::fromStdList(listInterfaces);
}

void CaptureHandler::_updateCaptureProgress()
{
    double progress = 0.0;
    if (_captureTimer->isActive()) {
        progress = _elapsedTime.elapsed() / 1000.0 / _jobOptions.timer;
    }
    if (_jobOptions.numberpackets != 0) {
        double packetProgress = _getPacketsCount() * 1.0 / _jobOptions.numberpackets;
        if (packetProgress > progress)
            progress = packetProgress;
    } else {
        _getPacketsCount(); //Needed for updating packet count in all cases
    }

    if (progress != 1.0)
        emit progressChanged(progress);
}

Hns::PcapInfo CaptureHandler::getPcapInfo()
{
    Hns::PcapInfo info;
    info.path = getCaptureFileMixedPath().toStdString();
    info.filters = _jobOptions.filteringterms;
    info.packetsLimit = _jobOptions.numberpackets;
    info.interfaces = _jobOptions.interfaceList.front();
    info.timeLimit = _jobOptions.timer;
    info.startDateTime = _startDateTime;
    if (!started()){
        info.packetsNumber = 0;
    } else {
        info.packetsNumber = _getPacketsCount();
    }
    info.endDateTime = _getEndDateTime();

    long size;
    QFile file(QString(_captureFullPath + _captureFileName));
    if (file.open(QIODevice::ReadOnly)) {
        size = file.size();
        file.close();
    }
    else {
        if (file.exists())
            qCritical() << tr("Cannot open ") << _captureFullPath << _captureFileName;
        size = -1;
    }
    info.size = size;

    return info;
}

void CaptureHandler::_setEndDateTime()
{
    QWriteLocker locker(&_finishTimeLock);
    _endDateTime = QDateTime::currentMSecsSinceEpoch();
}

long CaptureHandler::_getEndDateTime()
{
    QReadLocker locker(&_finishTimeLock);
    return _endDateTime;
}

void CaptureHandler::_progressChangedSlot(double progress)
{
    if (progress == 1.0)
        _setEndDateTime();
}

} // namespace HnsNode
