#include "entityplugI.h"
#include "nodeentityI.h"
#include "netutils.h"
#include "nodeswitchI.h"
#include "switchimplementation.h"
#include "nodeI.h"
#include "wireinterfacemanager.h"

#include <libhnsserver/processrunner.h>

#include <QDebug>
#include <QGenericArgument>
#include <QMetaObject>

namespace HnsNode {

/*******************************************************************************
 * EntityPlugI
 ******************************************************************************/
EntityPlugI::EntityPlugI(const QString &bridgeName)
    : _entity(0)
    , _bridgeName(bridgeName)
    , _vxlan(false)
{
}

EntityPlugI::~EntityPlugI()
{
}

std::string EntityPlugI::getHost(const Ice::Current &current)
{
    Q_UNUSED(current)
    std::string vxlanHost = Hns::NodeI::instance()->vxlanHost().toStdString();
    if (vxlanHost.empty())
        vxlanHost = Hns::NodeI::instance()->getHost();
    return vxlanHost;
}

std::string EntityPlugI::getBridgeName(const Ice::Current &current)
{
    Q_UNUSED(current)
    return _bridgeName.toStdString();
}

void EntityPlugI::plug(const Ice::Current &current)
{
    Q_UNUSED(current)
    addToBridge(_bridgeName, _interfaceName);
    emit interfaceStateChanged(_bridgeName, true);
}

void EntityPlugI::unplug(const Ice::Current &current)
{
    Q_UNUSED(current)
    removeFromBridge(_bridgeName, _interfaceName);
    removeVxlan();
    emit interfaceStateChanged(_bridgeName, false);
}

void EntityPlugI::setInterface(const std::string &interfaceName, const Ice::Current &current)
{
    Q_UNUSED(current)
    _interfaceName = QString::fromStdString(interfaceName);
}

std::string EntityPlugI::getInterfaceName(const Ice::Current &current)
{
    Q_UNUSED(current)
    return interfaceName().toStdString();
}

void EntityPlugI::createVxlan(Ice::Long id, const std::string &dstIp, const Ice::Current &current)
{
    _vxlan = true;
    QString interface = QString("hnsvx%1").arg(id, 6, 10, QChar('0'));
    Q_UNUSED(current)
    Hns::NodeI::instance()->wireInterfaceManager()->createVxlan(interface, id, Hns::NodeI::instance()->vxlanInterface(), QString::fromStdString(dstIp));
    setInterface(interface.toStdString());
}

void EntityPlugI::setEntity(EntityI *entity)
{
    _entity = entity;
    QObject::connect(this, SIGNAL(interfaceStateChanged(QString,bool)), _entity, SLOT(interfaceStateChangedSlot(QString,bool)));
}

QString EntityPlugI::interfaceName()
{
    return _interfaceName;
}

void EntityPlugI::applyProperties(const Hns::WireProperty &property, const Ice::Current &current)
{
    Q_UNUSED(current)
    HnsServer::ProcessRunner processRunner;

    //Delete the previous qdisc
    QString program  = "tc";
    QString qdiscDel = QString("qdisc del dev %1 root").arg(_interfaceName);
    QStringList delArgs = qdiscDel.split(" ");
    processRunner.runSynchronous(program, delArgs);

    //Reset the previous MTU to 1500

    QString resetMTU = QString("link set dev %1 mtu 1500").arg(_interfaceName);
    program = "ip";
    QStringList resetMTUArgs = resetMTU.split(" ");
    processRunner.runSynchronous(program, resetMTUArgs);


    //Add netem properties
    QString netemArgs = QString("netem delay %1ms loss %2\% duplicate %3\%").arg(property.delay).arg(property.loss).arg(property.dup);
    if (property.bandwidth) {
        QString htbQdisc = QString("qdisc add dev %1 root handle 1: htb default 1").arg(_interfaceName);
        QString htbRateQdisc = QString("class add dev %1 parent 1: classid 1:1 htb rate %2kbps").arg(_interfaceName).arg(property.bandwidth);
        QString netemQdisc = QString("qdisc add dev %1 parent 1:1 handle 10: %2").arg(_interfaceName).arg(netemArgs);

        program = "tc";
        QStringList args1 = htbQdisc.split(" ");
        QStringList args2 = htbRateQdisc.split(" ");
        QStringList args3 = netemQdisc.split(" ");

        processRunner.runSynchronous(program, args1);
        processRunner.runSynchronous(program, args2);
        processRunner.runSynchronous(program, args3);
    } else {
        QString netemQDisc = QString("qdisc add dev %1 root handle 1: %2").arg(_interfaceName).arg(netemArgs);

        program = "tc";
        QStringList args = netemQDisc.split(" ");

        processRunner.runSynchronous(program, args);
    }

    //Add MTU property
    if (property.mtu) {
        QString mtuArg = QString("link set dev %1 mtu %2").arg(_interfaceName).arg(property.mtu);

        program = "ip";
        QStringList args = mtuArg.split(" ");

        processRunner.runSynchronous(program, args);
    }
}

void EntityPlugI::removeVxlan(const Ice::Current &current)
{
    Q_UNUSED(current)
    if (_vxlan) {
        Hns::NodeI::instance()->wireInterfaceManager()->deleteInterface(_interfaceName);
    }
}

/*******************************************************************************
 * SwitchPlug
 ******************************************************************************/
SwitchPlug::SwitchPlug(const QString &bridgeName)
    : EntityPlugI(bridgeName)
    , _port(0)
{
}

SwitchPlug::~SwitchPlug()
{
}

void SwitchPlug::plug(const Ice::Current &current)
{
    Q_UNUSED(current)
    SwitchI *switchI = qobject_cast<SwitchI *>(_entity);
    if (switchI)
        switchI->implementation()->plug(_port, _interfaceName);
}

void SwitchPlug::unplug(const Ice::Current &current)
{
    Q_UNUSED(current)
    SwitchI *switchI = qobject_cast<SwitchI *>(_entity);
    if (switchI) {
        QGenericArgument arg = Q_ARG(int, _port);
        QMetaObject::invokeMethod(switchI->implementation(), "unplug", arg);
    }
    removeVxlan();
}

void SwitchPlug::setPort(int port)
{
    _port = port;
}

} // namespace HnsNode
