/******************************************************************************
**
** Copyright (C) 2013 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "libvirtnetworkhelper.h"
#include "nodescriptrunner.h"

// hynesim common headers
#include "common/initializer.h"

// Qt headers
#include <QDebug>
#include <QFile>
#include <QString>
#include <QStringList>
#include <QRegExp>

namespace HnsNode {

HNS_STATIC_SET_INITIALIZER(_xmlTagWhitelist, XmlTagWhitelist, QString)
XmlTagWhitelist::XmlTagWhitelist()
{
    insert("dns");
    insert("forward");
}

LibvirtNetworkHelper::LibvirtNetworkHelper()
    : _ipCount(0)
    , _customXmlPath(QString())
    , _connection(0)
    , _network(0)
{
}

LibvirtNetworkHelper::~LibvirtNetworkHelper()
{
}

bool LibvirtNetworkHelper::init(const QString &address, const QString &bridgeName, const QString &networkName)
{
    QRegExp regex("([0-9]{1,3}.){3}[0-9]{1,3}/[0-9]{1,2}");
    if (!regex.exactMatch(address)) {
        qCritical() << QObject::tr("The provided network %1 is not a valid network adress").arg(address);
        return false;
    }
    if (bridgeName.isEmpty()) {
        qCritical() << QObject::tr("Bridge name is empty");
        return false;
    }
    if (networkName.isEmpty()) {
        qCritical() << QObject::tr("Network name is empty");
        return false;
    }

    QStringList list = address.split('/');
    _address.first.setAddress(list.at(0));
    _address.second = list.at(1).toInt();

    _bridgeName = bridgeName;
    _networkName = networkName;

    return true;
}

bool LibvirtNetworkHelper::defineDHCP()
{
    _connection = virConnectOpen("qemu:///system");
    if (!_connection) {
        qCritical() << QObject::tr("connection to qemu:///system failed");
        return false;
    }
    _initStaticHosts();
    _createXML();
    _network = virNetworkDefineXML(_connection, _xml.toByteArray(-1).constData());
    if (!_network) {
        qCritical() << QObject::tr("unable to create %1").arg(_networkName);
        return false;
    }
    return true;
}

bool LibvirtNetworkHelper::startDHCP()
{
    if (!_network) {
        qCritical() << QObject::tr("%1 is not defined").arg(_networkName);
        return false;
    }
    if (!virNetworkCreate(_network)) {
        return true;
    }
    qCritical() << QObject::tr("unable to start %1").arg(_networkName);
    return false;
}

bool LibvirtNetworkHelper::destroyDHCP()
{
    if (!_network)
        return true;
    if (!virNetworkDestroy(_network))
        return true;
    qCritical() << QObject::tr("unable to stop %1").arg(_networkName);
    return false;
}

bool LibvirtNetworkHelper::undefineDHCP()
{
    if (!_network)
        return true; //already undefine
    if (!virNetworkUndefine(_network))
        return true;
    qCritical() << QObject::tr("unable to undefine %1").arg(_networkName);
    return false;
}

QString LibvirtNetworkHelper::addMacAddress(const QString &macAddress)
{
    if (_staticHosts.contains(macAddress))
        return _staticHosts.value(macAddress);

    QString ip;
    do {
        ip = _generateIP();
    } while (_isInStaticHosts(ip));
    if (ip.isEmpty())
        return QString();
    if (_macAddressHash.contains(macAddress)) {
        qCritical() << QObject::tr("The mac Address %1 is already register in the network").arg(macAddress);
        return QString();
    }
    QString xmlPart = QString("<host mac='%1' ip='%2'/>").arg(macAddress).arg(ip);
    int result = virNetworkUpdate(_network, VIR_NETWORK_UPDATE_COMMAND_ADD_FIRST,
                                  VIR_NETWORK_SECTION_IP_DHCP_HOST, -1, xmlPart.toLatin1().data(), VIR_NETWORK_UPDATE_AFFECT_CURRENT);
    if (!result) {
        qDebug() << QObject::tr("Adding %1 to %2 successfull").arg(macAddress).arg(ip);
        _macAddressHash.insert(macAddress, ip);
        return ip;
    }
    qCritical() << QObject::tr("Unable to add Mac Address %1 to the network %2").arg(macAddress).arg(_networkName);
    return QString();
}

void LibvirtNetworkHelper::removeMacAddress(const QString &macAddress)
{
    if (!_macAddressHash.contains(macAddress)) {
        if (_staticHosts.contains(macAddress)) {
            qDebug() << QObject::tr("Not removing mac address %1 from network %2 because it is a static IP").arg(macAddress).arg(_networkName);
            return;
        } else {
            qDebug() << QObject::tr("The network %1 does not contains the mac Address %2").arg(_networkName).arg(macAddress);
            return;
        }
    }

    QString xmlPart = QString("<host mac='%1' ip='%2'/>").arg(macAddress).arg(_macAddressHash.value(macAddress));
    int result = virNetworkUpdate(_network, VIR_NETWORK_UPDATE_COMMAND_DELETE,
                                  VIR_NETWORK_SECTION_IP_DHCP_HOST, -1, xmlPart.toLatin1().data(), VIR_NETWORK_UPDATE_AFFECT_CURRENT);
    if (!result) {
        qDebug() << QObject::tr("Removing %1 successfull").arg(macAddress);
        _macAddressHash.remove(macAddress);
        return;
    }
    qCritical() << QObject::tr("Unable to remove Mac Address %1 to the network %2").arg(macAddress).arg(_networkName);
}

bool LibvirtNetworkHelper::applyRules(const QString &fileName, bool adding)
{
    qDebug() << QObject::tr("Applying %1").arg(fileName);
    ScriptRunner runner;
    QStringList args;
    if (adding)
        args << "--start";
    else
        args << "--stop";
    args << _bridgeName;
    bool res = runner.runSynchronous(fileName, args);
    if(!res)
        qCritical() << QObject::tr("Applying %1 failed").arg(fileName);
    return res;
}

QString LibvirtNetworkHelper::networkIp() const
{
    return _address.first.toString();
}

void LibvirtNetworkHelper::setCustomXmlPath(const QString &path)
{
    _customXmlPath = path;
}

void LibvirtNetworkHelper::setStaticHostXmlPath(const QString &path)
{
    _staticHostsXmlPath = path;
}

QString LibvirtNetworkHelper::_generateIP()
{
    QHostAddress address(_address.first);
    ++_ipCount;
    address.setAddress(address.toIPv4Address() + _ipCount);
    if (!address.isInSubnet(_address)) {
        qCritical() << QObject::tr("The network %1 is full, unable to add another IP").arg(_networkName);
        return QString();
    }
    return address.toString();
}

void LibvirtNetworkHelper::_generatePlage()
{
}

void LibvirtNetworkHelper::_createXML()
{
    QDomElement root = _xml.createElement("network");

    QDomElement nameElement = _xml.createElement("name");
    QDomText textNameElement = _xml.createTextNode(_networkName);
    nameElement.appendChild(textNameElement);
    root.appendChild(nameElement);

    QDomElement bridgeElement = _xml.createElement("bridge");
    bridgeElement.setAttribute("name", _bridgeName);
    root.appendChild(bridgeElement);

    QDomElement ipElement = _xml.createElement("ip");
    QString ipAddress(_address.first.toString());
    ipElement.setAttribute("address", ipAddress);
    ipElement.setAttribute("prefix", QString::number(_address.second));

    QDomElement dhcpElement = _xml.createElement("dhcp");
    QDomElement rangeElement = _xml.createElement("range");
    dhcpElement.appendChild(rangeElement);
    ipElement.appendChild(dhcpElement);

    quint32 mask = (~quint32(0)) << (32 - _address.second);
    quint32 lastIpv4Address = (_address.first.toIPv4Address() |~ mask) - 1;
    rangeElement.setAttribute("start", ipAddress);
    rangeElement.setAttribute("end", QHostAddress(lastIpv4Address).toString());

    for (QHash<QString, QString>::const_iterator it = _staticHosts.begin() ; it != _staticHosts.end() ; ++it) {
        const QString &macAddr = it.key();
        const QString &ip = it.value();
        QDomElement hostElement = _xml.createElement("host");
        hostElement.setAttribute("mac", macAddr);
        hostElement.setAttribute("ip", ip);
        dhcpElement.appendChild(hostElement);
    }

    root.appendChild(ipElement);

    _xml.appendChild(root);

    if (!_customXmlPath.isEmpty())
        _importCustomXml();

    if (getenv("HNS_DEBUG"))
        qDebug() << QObject::tr("Going to define a network with the following XML:\n%1").arg(_xml.toString(2));
}

void LibvirtNetworkHelper::_importCustomXml()
{
    QFile file(_customXmlPath);
    if (!file.open(QIODevice::ReadOnly)) {
        qWarning() << QObject::tr("Couldn't open custom network XML file %1").arg(_customXmlPath);
        return;
    }

    QDomDocument doc;
    QString errorMsg;
    int errorLine;
    int errorColumn;

    if (!doc.setContent(&file, &errorMsg, &errorLine, &errorColumn)) {
        qWarning() << QObject::tr("Failed to parse %1, error at line %2, column %3:\n%4")
                      .arg(_customXmlPath)
                      .arg(errorLine)
                      .arg(errorColumn)
                      .arg(errorMsg);
        file.close();
        return;
    }
    file.close();

    QDomElement docRoot = _xml.documentElement();
    QString docRootName = docRoot.tagName();
    QDomElement customDocRoot = doc.documentElement();
    QString customDocRootName = customDocRoot.tagName();

    if (docRootName != customDocRootName) {
        qWarning() << QObject::tr("Expected custom network XML root element to be '%1', got '%2'")
                      .arg(docRootName)
                      .arg(customDocRootName);
        return;
    }

    QDomNodeList nodeList = customDocRoot.childNodes();
    int nodeListSize = nodeList.size();
    for (int i = 0 ; i < nodeListSize ; ++i) {
        QDomElement element = nodeList.at(i).toElement();
        QString elementName = element.tagName();
        if (!_xmlTagWhitelist()->contains(elementName)) {
            qWarning() << QObject::tr("Ignored unsupported element '%1' in custom network XML file")
                          .arg(elementName);
            continue;
        }
        QDomNode newNode = _xml.importNode(element, true);
        docRoot.appendChild(newNode);
    }
}

void LibvirtNetworkHelper::_initStaticHosts()
{
    if (_staticHostsXmlPath.isEmpty())
        return;

    QFile file(_staticHostsXmlPath);
    if (!file.open(QIODevice::ReadOnly)) {
        qWarning() << QObject::tr("Couldn't open static hosts XML file %1").arg(_staticHostsXmlPath);
        return;
    }

    QDomDocument doc;
    QString errorMsg;
    int errorLine;
    int errorColumn;

    if (!doc.setContent(&file, &errorMsg, &errorLine, &errorColumn)) {
        qWarning() << QObject::tr("Failed to parse %1, error at line 2, column %3\n%4")
                      .arg(_staticHostsXmlPath)
                      .arg(errorLine)
                      .arg(errorColumn)
                      .arg(errorMsg);
        file.close();
        return;
    }
    file.close();

    QDomElement docRoot = doc.documentElement();
    if (docRoot.tagName() != "hosts") {
        qWarning() << QObject::tr("Expected custom static hosts XML root element to be 'hosts', got '%2'").arg(docRoot.tagName());
        return;
    }
    QDomNodeList hostsNode = docRoot.elementsByTagName("host");
    for (int i = 0, hostsNodeSize = hostsNode.length() ; i < hostsNodeSize ; ++i) {
        QDomElement host = hostsNode.at(i).toElement();
        QString macAddr = host.attribute("mac");
        if (macAddr.isEmpty())
            continue;
        QString ip = host.attribute("ip");
        if (ip.isEmpty())
            continue;
        QHostAddress ipv4(ip);
        if (!ipv4.isInSubnet(_address)) {
            qWarning() << QObject::tr("Static IP %1 for MAC %2 doesn't belong to the network %3/%4").arg(ip).arg(macAddr).arg(_address.first.toString()).arg(_address.second);
            continue;
        }
        _staticHosts.insert(macAddr, ip);
    }
}

bool LibvirtNetworkHelper::_isInStaticHosts(const QString &ip)
{
    foreach (const QString &value, _staticHosts) {
        if (value == ip)
            return true;
    }
    return false;
}

} // namespace HnsNode
