/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "netutils.h"
#include "nodescriptrunner.h"

// hynesim common headers
#include "common/initializer.h"

// Qt headers
#include <QDebug>

Q_GLOBAL_STATIC(QAtomicInt, _nextDeviceId)

namespace HnsNode {

HNS_STATIC_HASH_INITIALIZER(_stateStringHash, StateStringHash, int, QString)
StateStringHash::StateStringHash()
{
    insert(Down, "down");
    insert(Up, "up");
}

bool createAMTap(const QString &tapName, const QString &actionManagerBridge, InterfaceState state)
{
    qDebug() << QObject::tr("Creating tap %1").arg(tapName);
    ScriptRunner runner;
    QStringList args = QStringList() << tapName << _stateStringHash()->value(state) << actionManagerBridge;
    return runner.runSynchronous("hnsCreateAMTap.sh", args);
}

bool createTap(const QString &tapName, InterfaceState state)
{
    qDebug() << QObject::tr("Creating tap %1").arg(tapName);
    ScriptRunner runner;
    QStringList args = QStringList() << tapName << _stateStringHash()->value(state);
    return runner.runSynchronous("hnsCreateTap.sh", args);
}

bool removeTap(const QString &tapName)
{
    qDebug() << QObject::tr("Remove tap %1").arg(tapName);
    ScriptRunner runner;
    QStringList args = QStringList() << tapName;
    return runner.runSynchronous("hnsRemoveTap.sh", args);
}

bool unplugTap(const QString &tapName)
{
    qDebug() << QObject::tr("Unplug tap %1").arg(tapName);
    ScriptRunner runner;
    QStringList args = QStringList() << tapName;
    return runner.runSynchronous("hnsUnplugTap.sh", args);
}

bool setupTap(const QString &tapName)
{
    qDebug() << QObject::tr("Setup tap %1").arg(tapName);
    ScriptRunner runner;
    QStringList args = QStringList() << tapName;
    return runner.runSynchronous("hnsSetupTap.sh", args);
}

bool unsetupTap(const QString &tapName)
{
    qDebug() << QObject::tr("Unsetup tap %1").arg(tapName);
    ScriptRunner runner;
    QStringList args = QStringList() << tapName;
    return runner.runSynchronous("hnsUnsetupTap.sh", args);
}

bool createBridge(const QString &devName, InterfaceState state, bool isHub)
{
    qDebug() << QObject::tr("Create bridge %1").arg(devName);
    ScriptRunner runner;
    QStringList args = QStringList() << devName << _stateStringHash()->value(state) << QString::number(isHub);
    return runner.runSynchronous("hnsCreateBridge.sh", args);
}

bool removeBridge(const QString &devName)
{
    qDebug() << QObject::tr("Remove bridge %1").arg(devName);
    ScriptRunner runner;
    QStringList args = QStringList() << devName;
    return runner.runSynchronous("hnsRemoveBridge.sh", args);
}

bool addToBridge(const QString &bridgeName, const QString &devName)
{
    qDebug() << QObject::tr("Add device %1 to bridge %2").arg(devName).arg(bridgeName);
    ScriptRunner runner;
    QStringList args = QStringList() << bridgeName << devName;
    return runner.runSynchronous("hnsAddToBridge.sh", args);
}

bool setInterfaceState(const QString &ifName, InterfaceState state)
{
    ScriptRunner runner;
    QStringList args = QStringList() << ifName << _stateStringHash()->value(state);
    return runner.runSynchronous("hnsSetIfState.sh", args);
}

void startSpiceTerm(QString vmName, int port, QString keymap)
{
    ScriptRunner runner;
    QStringList args = QStringList() << vmName << QString::number(port) << keymap;
    runner.runDetached("hnsSpiceTerm.sh", args);
}

void startVncTerm(QString veId, int port, QString type)
{
    ScriptRunner runner;
    QStringList args = QStringList() << "start" << veId << QString::number(port) << type;
    runner.runDetached("hnsVncTerm.sh", args);
}

void stopVncTerm(QString veId, int port, QString type)
{
    ScriptRunner runner;
    QStringList args = QStringList() << "stop" << veId << QString::number(port) << type;
    runner.runDetached("hnsVncTerm.sh", args);
}

QString generateHnsDevice(void)
{
    return QString("hns%1").arg(_nextDeviceId()->fetchAndAddOrdered(1), 7, 10, QChar('0'));
}

QString _vmnetId(const QString &vmnet)
{
    QRegExp rx(QString("%1(\\d+)$").arg(VMNET_NAME));
    rx.indexIn(vmnet);
    return rx.cap(1);
}

bool createVmnet(const QString &vmnet, bool setupIf)
{
    ScriptRunner runner;
    QStringList args = QStringList() << _vmnetId(vmnet) << QString::number(setupIf);
    return runner.runSynchronous("hnsCreateVmnet.sh", args);
}

bool removeVmnet(const QString &vmnet)
{
    ScriptRunner runner;
    QStringList args = QStringList() << _vmnetId(vmnet);
    return runner.runSynchronous("hnsRemoveVmnet.sh", args);
}

bool createVeth(const QString &firstInterface, const QString &secondInterface)
{
    qDebug() << QObject::tr("Create Veth interfaces %1 and %2").arg(firstInterface).arg(secondInterface);
    ScriptRunner runner;
    QStringList args = QStringList() << firstInterface << secondInterface;
    return runner.runSynchronous("hnsCreateVeth.sh", args);
}

bool createVxlan(const QString &interfaceName, qlonglong id, const QString &sourceInterface, const QString &dstIp)
{
    qDebug() << QObject::tr("Create VXLAN interface %1 with id %2, source interface %3 and destination IP %4").arg(interfaceName).arg(id).arg(sourceInterface).arg(dstIp);
    ScriptRunner runner;
    QStringList args = QStringList() << sourceInterface << interfaceName << QString::number(id) << dstIp;
    return runner.runSynchronous("hnsCreateVxlan.sh", args);
}

bool deleteInterface(const QString &interfaceName)
{
    if (interfaceName.isEmpty())
        return false;
    qDebug() << QObject::tr("Delete interface %1").arg(interfaceName);
    ScriptRunner runner;
    QStringList args = QStringList() << interfaceName;
    return runner.runSynchronous("hnsDeleteInterface.sh", args);
}

bool removeFromBridge(const QString &bridgeName, const QString &interfaceName)
{
    if (interfaceName.isEmpty())
        return false;
    qDebug() << QObject::tr("Remove interface %1 from bridge %2").arg(interfaceName).arg(bridgeName);
    ScriptRunner runner;
    QStringList args = QStringList() << bridgeName << interfaceName;
    return runner.runSynchronous("hnsRemoveFromBridge.sh", args);
}

} // namespace HnsNode
