/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "nodeI.h"
#include "bridgemonitor.h"
#include "nodeconstants.h"
#include "platform.h"
#include "nodeentityfactoryI.h"
#include "nodelocalmonitorI.h"
#include "wireinterfacemanager.h"
#include "nodescriptrunner.h"

// hynesim common headers
#include "common/pluginmanager.h"

// hynesim server headers
#include "libhnsserver/storagemanager.h"

// Generated hynesim headers
#include "common/platforminfo.h"
#include "common/config-prefix.h"
#include "common/version.h"

// diamesh headers
#include <diamesh/diamesh.h>
#include <diamesh/coreapplication.h>

// ZeroC Ice headers
#include <Ice/ObjectAdapter.h>

// Qt headers
#include <QSettings>

namespace Hns {

NodeI::NodeI()
    : _Hns_Node()
    , _actionManagerBridgeAvailable(false)
{
    _entityFactory = new HnsNode::EntityFactoryI();
    QObject::connect(_entityFactory,SIGNAL(addEntity(long)), this, SIGNAL(addEntity(long)));
    QObject::connect(_entityFactory,SIGNAL(removeEntity(long)), this, SIGNAL(removeEntity(long)));
    Ice::Identity id;
    id.name = "entityFactory";
    adapter()->add(_entityFactory, id);
    _pluginManager = new Hns::PluginManager(NODE_PLUGINS_DIR, this);
    _pluginManager->buildPluginList();

    _wireInterfaceDeleter = new HnsNode::WireInterfaceManager();

    HnsNode::Platform::createPlatforms();

    _storageManager = new HnsServer::StorageManager(Diamesh::instance()->configFileName());
    _resourceManager = new HnsNode::ResourceManager();
    QObject::connect(_resourceManager, SIGNAL(resourcesChanged(HnsNode::ResourceInfo)), this, SIGNAL(resourcesChanged(HnsNode::ResourceInfo)));

    QSettings settings(Diamesh::instance()->configFileName(), QSettings::IniFormat, this);
    settings.beginGroup(HnsNode::Constants::SETTINGS_NODE_GROUP);
    _sharedFolder = settings.value("sharedFolder", "").toString();
    _vxlanInterface = settings.value("VxlanInterface", "eth0").toString();
    _vxlanHost = settings.value(HnsNode::Constants::SETTINGS_VXLAN_HOST, "").toString();
    _actionManagerBridge = settings.value(HnsNode::Constants::SETTINGS_ACTIONMANAGER_BRIDGE_NAME, "ambr0").toString();
    if (!_actionManagerBridge.isEmpty()) {
        HnsNode::ScriptRunner runner;
        runner.runSynchronous("actionManagerBridgeAvailable.sh", QStringList() << _actionManagerBridge);
        _actionManagerBridgeAvailable = (runner.exitCode() == 0);
    }

    HnsNode::BridgeMonitor::create(this);
    _nodeMonitor = new HnsNode::NodeMonitorI(_entityFactory);
    Ice::Identity monitorId;
    monitorId.name = "nodeMonitor";
    adapter()->add(_nodeMonitor, monitorId);

    QObject::connect(Diamesh::CoreApplication::instance(), SIGNAL(shutdown()), this, SLOT(_shutdownSlot()));
}

NodeI::~NodeI()
{
}

NodeI *NodeI::instance()
{
    return dynamic_cast<Hns::NodeI*>(Diamesh::NodeI::instance());
}

HnsNode::EntityFactoryPrx NodeI::getEntityFactory(const ::Ice::Current &current)
{    
    Q_UNUSED(current)
    Ice::Identity id;
    id.name = "entityFactory";
    return HnsNode::EntityFactoryPrx::uncheckedCast(adapter()->createProxy(id));
}

Diamesh::VersionInfo NodeI::getVersionInfo(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return Hns::versionInfo();
}

HnsNode::ResourceInfo NodeI::getResourceInfo(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return _resourceManager->getResourceInfo();
}

HnsNode::EntityFactoryI *NodeI::entityFactory()
{
    return _entityFactory;
}

HnsServer::StorageManager *NodeI::storageManager()
{
    return _storageManager;
}

HnsNode::ResourceManager *NodeI::resourceManager()
{
    return _resourceManager;
}

QString NodeI::sharedFolder()
{
    return _sharedFolder;
}

MasterPrx NodeI::hnsMaster()
{
    return MasterPrx::uncheckedCast(master());
}

void NodeI::emitSignalRefresh(const QString &className)
{
    emit refreshClassName(_entityFactory->getNameKeyChecker(className));
}

Hns::PlatformInfoMap NodeI::getPlatforms(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QMap<std::string, Hns::PlatformInfo> platformsMap;
    foreach (Hns::PlatformInfo plat, HnsNode::Platform::getPlatforms())
        platformsMap.insert(plat.name, plat);
    return platformsMap.toStdMap();
}

Hns::PluginManager * NodeI::pluginManager()
{
    return _pluginManager;
}

HnsNode::NodeMonitorI *NodeI::nodeMonitor()
{
    return _nodeMonitor;
}

HnsNode::NodeMonitorPrx NodeI::getNodeMonitor(const Ice::Current &current)
{
    Q_UNUSED(current)
    Ice::Identity id;
    id.name = "nodeMonitor";
    return HnsNode::NodeMonitorPrx::uncheckedCast(adapter()->createProxy(id));
}

QString NodeI::vxlanInterface() const
{
    return _vxlanInterface;
}

QString NodeI::vxlanHost() const
{
    return _vxlanHost;
}

QString NodeI::actionManagerBridge() const
{
    return _actionManagerBridge;
}

HnsNode::WireInterfaceManager *NodeI::wireInterfaceManager()
{
    return _wireInterfaceDeleter;
}

bool NodeI::actionManagerBridgeAvailable() const
{
    return _actionManagerBridgeAvailable;
}

void NodeI::_shutdownSlot()
{
    delete _wireInterfaceDeleter;
}

} // namespace Hns
