/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "nodedomainI.h"
#include "platformdomain.h"
#include "netcard.h"
#include "entityplugI.h"
#include "nodeI.h"

// hynesim server headers
#include "libhnsserver/storagemanager.h"
#include "libhnsserver/processrunner.h"

// Generated hynesim headers
#include "common/domainentry.h"

// Qt headers
#include <QCoreApplication>
#include <QDebug>
#include <QFile>
#include <QFileInfo>

namespace HnsNode {

DomainI::DomainI()
    : _virDomain(0)
{
    _remoteDisplayInfo.host = getHost();
    _aliveTimer.setInterval(1000);
    QObject::connect(&_aliveTimer, SIGNAL(timeout()), this, SLOT(_checkAliveSlot()));
    _aliveTimer.moveToThread(qApp->thread());
    _monitoringData = new Hns::DomainMonitoringData();
}

DomainI::~DomainI()
{
    _monitoringData = 0;
    qDeleteAll(_netcardMap);
}

void DomainI::init()
{
    EntityI::init();
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(_entityEntry);
    PlatformDomain *platform = PlatformDomain::get(domainEntry->type);
    _flags |= platform->getEntityFlags();
}

bool DomainI::_define(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(_entityEntry);
    PlatformDomain *platform = PlatformDomain::get(domainEntry->type);
    platform->createNetcards(this);
    return platform->defineEntity(this);
}

bool DomainI::_destroy(const::Ice::Current &current)
{
    Q_UNUSED(current)
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(_entityEntry);
    PlatformDomain *platform = PlatformDomain::get(domainEntry->type);
    return platform->undefineEntity(this);
}

bool DomainI::_start(const::Ice::Current &current)
{
    Q_UNUSED(current)
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(_entityEntry);
    PlatformDomain *platform = PlatformDomain::get(domainEntry->type);
    if(platform->createEntity(this)) {
        QMetaObject::invokeMethod(&_aliveTimer, "start");
        return true;
    }
    return false;
}

bool DomainI::_stop(const::Ice::Current &current)
{
    Q_UNUSED(current)
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(_entityEntry);
    PlatformDomain *platform = PlatformDomain::get(domainEntry->type);
    if(platform->destroyEntity(this)) {
        QMetaObject::invokeMethod(&_aliveTimer, "stop");
        _updateMonitoringData();
        return true;
    }
    return false;
}

bool DomainI::_save(const::Ice::Current &current)
{
    Q_UNUSED(current)
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(_entityEntry);
    PlatformDomain *platform = PlatformDomain::get(domainEntry->type);
    if (platform->saveEntity(this)) {
         QMetaObject::invokeMethod(&_aliveTimer, "stop");
         return true;
    }
    return false;
}

bool DomainI::_restore(const::Ice::Current &current)
{
    Q_UNUSED(current)
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(_entityEntry);
    PlatformDomain *platform = PlatformDomain::get(domainEntry->type);
    if(platform->restoreEntity(this)) {
        QMetaObject::invokeMethod(&_aliveTimer, "start");
        return true;
    }
    return false;
}

bool DomainI::_suspend(const::Ice::Current &current)
{
    Q_UNUSED(current)
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(_entityEntry);
    PlatformDomain *platform = PlatformDomain::get(domainEntry->type);
    if(platform->suspendEntity(this)) {
        QMetaObject::invokeMethod(&_aliveTimer, "stop");
        return true;
    }
    return false;
}

bool DomainI::_resume(const::Ice::Current &current)
{
    Q_UNUSED(current)
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(_entityEntry);
    PlatformDomain *platform = PlatformDomain::get(domainEntry->type);
    if(platform->resumeEntity(this)) {
        QMetaObject::invokeMethod(&_aliveTimer, "start");
        return true;
    }
    return false;
}

void DomainI::createMemoryDump_effective(const::HnsNode::AMD_Domain_createMemoryDumpPtr &amdPtr, const std::string &path, const Ice::Current &current)
{
    Q_UNUSED(current)
    QWriteLocker locker(&_entityLocker);
    HnsServer::StorageManager *storageManager = Hns::NodeI::instance()->storageManager();
    QString fullPath = storageManager->getFullPathFromMixed("topology", QString::fromStdString(path));
    QString archiveFullPath = fullPath + ".tar.gz";
    qDebug() << tr("Dumping memory of domain %1 to %2").arg(_entityEntry->shortName.c_str()).arg(fullPath);
    int ret = virDomainCoreDump(_virDomain, fullPath.toStdString().c_str(), VIR_DUMP_LIVE|VIR_DUMP_MEMORY_ONLY);
    if (ret < 0) {
        qCritical() << tr("Memory dump of domain %1 failed").arg(_entityEntry->shortName.c_str());
        amdPtr->ice_response(ret);
        return;
    }
    qDebug() << tr("Memory dump %1 created").arg(fullPath);
    qDebug() << tr("Archiving %1 to %2").arg(fullPath).arg(archiveFullPath);
    //compress the file and delete it

    QFileInfo fileInfo(fullPath);

    HnsServer::ProcessRunner runner;
    QStringList args = QStringList() << "-czvf" << archiveFullPath << "-C" << fileInfo.absolutePath() << fileInfo.fileName();
    if (!runner.runSynchronous("tar", args, 300000))
        qCritical() << tr("An error occured when archiving the memory dump: %1").arg(runner.getError());

    qDebug() << tr("Archive %1 successfully created").arg(archiveFullPath);
    qDebug() << tr("Deleting %1").arg(fullPath);
    QFile dumpFile(fullPath);
    if (!dumpFile.remove())
        qCritical() << tr("Unable to delete file %1: %2").arg(dumpFile.fileName()).arg(dumpFile.errorString());

    amdPtr->ice_response(ret);
}

bool DomainI::_createSnapshot(QString xmlDesc)
{
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(_entityEntry);
    PlatformDomain *platform = PlatformDomain::get(domainEntry->type);
    return platform->createSnapshot(this, xmlDesc);
}

bool DomainI::_revertSnapshot(QString snapshotName)
{
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(_entityEntry);
    PlatformDomain *platform = PlatformDomain::get(domainEntry->type);
    return platform->revertSnapshot(this, snapshotName);
}

bool DomainI::_deleteSnapshot(QString snapshotName)
{
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(_entityEntry);
    PlatformDomain *platform = PlatformDomain::get(domainEntry->type);
    return platform->deleteSnapshot(this, snapshotName);
}

Hns::Snapshot DomainI::_getSnapshot(QString snapshotName)
{
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(_entityEntry);
    PlatformDomain *platform = PlatformDomain::get(domainEntry->type);
    return platform->getSnapshot(this, snapshotName);
}

QString DomainI::_getCurrentSnapshotName()
{
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(_entityEntry);
    PlatformDomain *platform = PlatformDomain::get(domainEntry->type);
    return platform->getCurrentSnapshotName(this);
}

virDomainPtr DomainI::virDomain()
{
    return _virDomain;
}

void DomainI::setVirDomainPtr(virDomainPtr newVirDomainPtr)
{
    _virDomain = newVirDomainPtr;
}

QList<Netcard *> DomainI::netcards()
{
    return _netcardMap.values();
}

EntityPlugI *DomainI::_createPlug(::Ice::Int port)
{
    if (!_netcardMap.contains(port)) {
        qWarning() << tr("No netcard on port %1").arg(port);
        return NULL;
    }
    Netcard *netcard = _netcardMap.value(port);
    EntityPlugI *entityPlug = new EntityPlugI(netcard->devName());
    return entityPlug;
}

void DomainI::_updateMonitoringData()
{
    if (!hasFlag(Monitoring))
        return;
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(_entityEntry);
    PlatformDomain *platform = PlatformDomain::get(domainEntry->type);
    bool updated = platform->updateMonitoringData(this);
    if (updated)
        emit monitoringDataUpdated(_monitoringData);
}

void DomainI::_interfaceStateChangedSlot(QString interfaceName, bool stateUp)
{
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(getEntry());
    PlatformDomain::get(domainEntry->type)->setInterfaceUp(this, interfaceName, stateUp);
}

::Diamesh::IntList DomainI::getPlugs(const::Ice::Current &current)
{
    Q_UNUSED(current)
    return _netcardMap.keys().toStdList();
}

void DomainI::_checkAliveSlot()
{
    QWriteLocker locker(&_entityLocker);
    if (_status != Hns::STARTED)
        return;
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(_entityEntry);
    PlatformDomain *platform = PlatformDomain::get(domainEntry->type);
    int status = platform->domainStatus(this);
    if (status == Hns::STOPPED && _status == Hns::STARTED) {
        _aliveTimer.stop();
        setStatus(Hns::STOPPED);
    }
    _updateMonitoringData();
}

} // namespace HnsNode
