/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "nodeentityI.h"
#include "nodeI.h"
#include "capturejobI.h"
#include "entityplugI.h"

// hynesim common headers
#include "common/resourceutils.h"

// hynesim server headers
#include "libhnsserver/storagemanager.h"

// ZeroC Ice headers
#include <Ice/ObjectAdapter.h>

#include <QDebug>
#include <QEventLoop>
#include <QTimer>

namespace HnsNode {

EntityI::EntityI()
    : _entityEntry(NULL)
    , _status(Hns::UNDEFINED)
    , _cptStatus(0)
    , _flags(0)
    , _monitoringData(NULL)
{
}

EntityI::~EntityI()
{    
    Hns::NodeI *node = Hns::NodeI::instance();
    ResourceManager *resourceManager = node->resourceManager();
    Hns::Resource resource = Hns::getResourceFromEntry(_entityEntry);
    resourceManager->removeResource(resource);
}

void EntityI::init()
{
}

Hns::EntityEntryPtr EntityI::getEntry(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return _entityEntry;
}

void EntityI::setEntry(const ::Hns::EntityEntryPtr &entityEntry)
{
    Hns::NodeI *node = Hns::NodeI::instance();
    ResourceManager *resourceManager = node->resourceManager();
    if (_entityEntry) {
        Hns::Resource oldResource = Hns::getResourceFromEntry(_entityEntry);
        resourceManager->removeResource(oldResource);
    }
    _entityEntry = entityEntry;
    Hns::Resource newResource = Hns::getResourceFromEntry(_entityEntry);
    resourceManager->addResource(newResource);
}

bool EntityI::updateEntry(const ::Hns::EntityEntryPtr &entityEntry, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    if (_checkEntry(entityEntry) && _updateEntry(entityEntry)) {
        setEntry(entityEntry);
        return true;
    }
    return false;
}

bool EntityI::define(const ::Ice::Current &current)
{    
    Q_UNUSED(current)
    QWriteLocker locker(&_entityLocker);
    if (_status != Hns::UNDEFINED) {
        qDebug() << tr("Entity should be undefined before it is defined.");
        return false;
    }
    Hns::Status previousStatus = _status;
    setStatus(Hns::BUSY);
    bool result = _define(current);
    if (result) {
        if (_entityEntry->saved == false) {
            setStatus(Hns::STOPPED);
        } else {
            setStatus(Hns::SAVED);
        }
    } else {
        setStatus(previousStatus);
    }
    _initAvailablePlugsForCapture(current);
    return result;
}

bool EntityI::start(const ::Ice::Current &current)
{        
    QWriteLocker locker(&_entityLocker);
    if (_status != Hns::STOPPED && _status != Hns::SAVED && _status != Hns::PAUSED) {
        qDebug() << tr("Entity should be stopped, saved or paused before it is started.");
        return false;
    }
    Hns::Status previousStatus = _status;
    setStatus(Hns::BUSY);
    bool result = _start(current);
    if (result)
        setStatus(Hns::STARTED);
    else
        setStatus(previousStatus);
    return result;
}

bool EntityI::stop(const ::Ice::Current &current)
{
    QWriteLocker locker(&_entityLocker);
    if (_status != Hns::STARTED) {
        qDebug() << tr("Entity should be started before it is stopped.");
        return false;
    }
    Hns::Status previousStatus = _status;
    setStatus(Hns::BUSY);
    bool result = _stop(current);
    if (result)
        setStatus(Hns::STOPPED);
    else
        setStatus(previousStatus);
    return result;
}

bool EntityI::save(const ::Ice::Current &current)
{
    QWriteLocker locker(&_entityLocker);
    if (_status != Hns::STARTED) {
        qDebug() << tr("Entity should be started before it is saved.");
        return false;
    }
    Hns::Status previousStatus = _status;
    setStatus(Hns::BUSY);
    bool result = _save(current);
    if (result)
        setStatus(Hns::SAVED);
    else
        setStatus(previousStatus);
    return result;
}

bool EntityI::restore(const ::Ice::Current &current)
{
    QWriteLocker locker(&_entityLocker);
    if (_status != Hns::SAVED) {
        qDebug() << tr("Entity should be saved before it is restored.");
        return false;
    }
    Hns::Status previousStatus = _status;
    setStatus(Hns::BUSY);
    bool result = _restore(current);
    if (result)
        setStatus(Hns::STARTED);
    else
        setStatus(previousStatus);
    return result;
}

bool EntityI::suspend(const ::Ice::Current &current)
{
    QWriteLocker locker(&_entityLocker);
    if (_status != Hns::STARTED) {
        qDebug() << tr("Entity should be started before it is suspended.");
        return false;
    }
    Hns::Status previousStatus = _status;
    setStatus(Hns::BUSY);
    bool result = _suspend(current);
    if (result)
        setStatus(Hns::PAUSED);
    else
        setStatus(previousStatus);
    return result;
}

bool EntityI::resume(const ::Ice::Current &current)
{
    QWriteLocker locker(&_entityLocker);
    if (_status != Hns::PAUSED) {
        qDebug() << tr("Entity should be paused before it is resumed.");
        return false;
    }
    Hns::Status previousStatus = _status;
    setStatus(Hns::BUSY);
    bool result = _resume(current);
    if (result)
        setStatus(Hns::STARTED);
    else
        setStatus(previousStatus);
    return result;
}

bool EntityI::undefine(const ::Ice::Current &current)
{    
    QWriteLocker locker(&_entityLocker);
    if (_status == Hns::UNDEFINED)
        return true;
    Hns::Status previousStatus = _status;
    setStatus(Hns::BUSY);
    if (previousStatus == Hns::STARTED || previousStatus == Hns::PAUSED) {
        if (!_stop(current))
            return false;
        // TODO: fixme
        // Workaround for libvirt or VBox bug when calling undefine immediately after destroy
        QEventLoop l;
        QTimer::singleShot(1000, &l, SLOT(quit()));
        l.exec();
    }
    _removePlugs();
    bool result = _destroy(current);
    if (result) {
        setStatus(Hns::UNDEFINED);
    } else {
        setStatus(previousStatus);
    }
    return result;
}

bool EntityI::createSnapshot(const std::string &xmlDesc, const Ice::Current &current)
{
    Q_UNUSED(current)
    QWriteLocker locker(&_entityLocker);
    if (_status != Hns::STOPPED) {
        qDebug() << tr("Entity should be stopped before it is snapshoted.");
        return false;
    }
    return _createSnapshot(QString::fromStdString(xmlDesc));
}

bool EntityI::revertSnapshot(const std::string &snapshotName, const Ice::Current &current)
{
    Q_UNUSED(current)
    QWriteLocker locker(&_entityLocker);
    if (_status != Hns::STOPPED) {
        qDebug() << tr("Entity should be stopped before it is snapshoted.");
        return false;
    }
    return _revertSnapshot(QString::fromStdString(snapshotName));
}

bool EntityI::deleteSnapshot(const std::string &snapshotName, const Ice::Current &current)
{
    Q_UNUSED(current)
    QWriteLocker locker(&_entityLocker);
    if (_status != Hns::STOPPED) {
        qDebug() << tr("Entity should be stopped before it is snapshoted.");
        return false;
    }
    return _deleteSnapshot(QString::fromStdString(snapshotName));
}

Hns::Snapshot EntityI::getSnapshot(const std::string &snapshotName, const Ice::Current &current)
{
    Q_UNUSED(current)
    QWriteLocker locker(&_entityLocker);
    return _getSnapshot(QString::fromStdString(snapshotName));
}

std::string EntityI::getCurrentSnapshotName(const Ice::Current &current)
{
    Q_UNUSED(current)
    QWriteLocker locker(&_entityLocker);
    return _getCurrentSnapshotName().toStdString();
}

bool EntityI::destroy()
{
    emit requestDeletion(Hns::NodeI::instance()->adapter(), _identity);
    return true;
}

bool EntityI::_define(const::Ice::Current &current)
{
    Q_UNUSED(current)
    return true;
}

bool EntityI::_start(const::Ice::Current &current)
{
    Q_UNUSED(current)
    return true;
}

bool EntityI::_stop(const::Ice::Current &current)
{
    Q_UNUSED(current)
    return true;
}

bool EntityI::_save(const::Ice::Current &current)
{
    Q_UNUSED(current)
    return true;
}

bool EntityI::_restore(const::Ice::Current &current)
{
    Q_UNUSED(current)
    return true;
}

bool EntityI::_suspend(const::Ice::Current &current)
{
    Q_UNUSED(current)
    return true;
}

bool EntityI::_resume(const::Ice::Current &current)
{
    Q_UNUSED(current)
    return true;
}

bool EntityI::_destroy(const::Ice::Current &current)
{
    Q_UNUSED(current)
    return true;
}

bool EntityI::_createSnapshot(QString xmlDesc)
{
    Q_UNUSED(xmlDesc)
    return true;
}

bool EntityI::_revertSnapshot(QString snapshotName)
{
    Q_UNUSED(snapshotName)
    return true;
}

bool EntityI::_deleteSnapshot(QString snaptshotName)
{
    Q_UNUSED(snaptshotName)
    return true;
}

Hns::Snapshot EntityI::_getSnapshot(QString snapshotName)
{
    Q_UNUSED(snapshotName)
    return Hns::Snapshot();
}

QString EntityI::_getCurrentSnapshotName()
{
    return QString();
}

QString EntityI::path()
{
    HnsServer::StorageManager *storageManager = Hns::NodeI::instance()->storageManager();
    return storageManager->getFullPathFromMixed("entity",QString::fromStdString(_entityEntry->path), QString::fromStdString(_entityEntry->uuid));
}

std::string EntityI::getHost(const ::Ice::Current &current)
{    
    Q_UNUSED(current)
    return Hns::NodeI::instance()->getHost();
}

EntityPlugI *EntityI::_createPlug(::Ice::Int port)
{
    Q_UNUSED(port);
    return NULL;
}

bool EntityI::_removePlugs()
{
    QMapIterator< ::Ice::Int, EntityPlugPrx> i(_plugs);
    while (i.hasNext()) {
        i.next();
        EntityPlugPrx plug = i.value();
        plug->removeVxlan();
        //qDebug() << "remove plug:" << QString::fromStdString(plug->ice_getIdentity().name);
        Hns::NodeI::instance()->adapter()->remove(plug->ice_getIdentity());
    }
    return true;
}

bool EntityI::_checkEntry(const ::Hns::EntityEntryPtr &current)
{
    Q_UNUSED(current)
    return true;
}

bool EntityI::_updateEntry(const ::Hns::EntityEntryPtr &current)
{
    Q_UNUSED(current)
    return true;
}

EntityPlugPrx EntityI::getPlug(::Ice::Int port, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    if (!_plugs.contains(port)) {
        EntityPlugI *plug = _createPlug(port);
        if (!plug) {
            qCritical() << tr("Unable to create plug for %1").arg(QString::fromStdString(_entityEntry->shortName));
            return NULL;
        }
        plug->setEntity(this);

        Ice::ObjectAdapterPtr adapter = Hns::NodeI::instance()->adapter();
        Ice::ObjectPrx proxy = adapter->addWithUUID(plug);
        EntityPlugPrx plugProxy =  EntityPlugPrx::uncheckedCast(proxy);
        _plugs.insert(port, plugProxy);
        return plugProxy;
    } else {
        EntityPlugPrx plugProxy = _plugs.value(port);
        return plugProxy;
    }
}

HnsNode::CaptureJobPrx EntityI::createCaptureJob(const std::string& captureJobPath, const Ice::Current &current)
{
    Q_UNUSED(current)
    HnsNode::CaptureJobI *captureJob = new CaptureJobI(this, QString::fromStdString(captureJobPath));
    Ice::ObjectAdapterPtr adapter = Hns::NodeI::instance()->adapter();
    Ice::ObjectPrx proxy = adapter->addWithUUID(captureJob);
    HnsNode::CaptureJobPrx captureJobPrx = HnsNode::CaptureJobPrx::uncheckedCast(proxy);
    captureJob->setIdentity(proxy->ice_getIdentity());
    return captureJobPrx;
}

::Diamesh::IntList EntityI::getPlugs(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return ::Diamesh::IntList();
}

::Diamesh::IntList EntityI::getAvailablePlugsForCapture(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return _availablePlugsForCapture.toStdList();
}

void EntityI::setAvailablePlugsForCapture(const ::Diamesh::IntList &interfacesList, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    _availablePlugsForCapture = QList<Ice::Int>::fromStdList(interfacesList);
    emit capturableInterfacesListChanged(_availablePlugsForCapture.toStdList());
}

void EntityI::removeInterfaceForCapture(Ice::Int interface, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    _availablePlugsForCapture.removeAll(interface);
    emit capturableInterfacesListChanged(_availablePlugsForCapture.toStdList());
}

void EntityI::addInterfaceForCapture(Ice::Int interface, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    _availablePlugsForCapture.append(interface);
    qSort(_availablePlugsForCapture.begin(),_availablePlugsForCapture.end());
    emit capturableInterfacesListChanged(_availablePlugsForCapture.toStdList());
}

Hns::Status EntityI::getStatus(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return _status;
}

void EntityI::setIdentity(Ice::Identity identity)
{
    _identity = identity;
}

void EntityI::setId(::Ice::Long id, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    _id = id;
}

::Ice::Long EntityI::getId(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return _id;
}

bool EntityI::hasFlag(EntityI::EntityFlags flag)
{
    return _flags & flag;
}

Hns::EntityMonitoringDataPtr EntityI::getMonitoringData()
{
    return _monitoringData;
}

void EntityI::interfaceStateChangedSlot(QString interfaceName, bool stateUp)
{
    _interfaceStateChangedSlot(interfaceName, stateUp);
}

void EntityI::setStatus(Hns::Status status)
{
    if (status != _status) {
        _status = status;
        emit statusChanged(_status, ++_cptStatus);
    }
}

QString EntityI::_getStatusName()
{
    QString statusName;
    switch(_status) {
    case Hns::BUSY:
        statusName = "BUSY";
        break;
    case Hns::UNDEFINED:
        statusName = "UNDEFINED";
        break;
    case Hns::STARTED:
        statusName = "STARTED";
        break;
    case Hns::STOPPED:
        statusName = "STOPPED";
        break;
    case Hns::PAUSED:
        statusName = "PAUSED";
        break;
    case Hns::SAVED:
        statusName = "SAVED";
        break;
    default:
        break;
    }
    return statusName;
}

Hns::RemoteDisplayInfo EntityI::getRemoteDisplayInfo(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return _remoteDisplayInfo;
}

void EntityI::setRemoteDisplayPort(int remoteDisplayPort)
{
    _remoteDisplayInfo.port = remoteDisplayPort;
}

int EntityI::getRemoteDisplayPort()
{
    return _remoteDisplayInfo.port;
}

void EntityI::setRemoteDisplayType(int remoteDisplayType)
{
    _remoteDisplayInfo.type = Hns::RemoteDisplayType(remoteDisplayType);
}

int EntityI::getRemoteDisplayType()
{
    return _remoteDisplayInfo.type;
}

void EntityI::setRemoteDisplayPassword(QString password)
{
    _remoteDisplayInfo.password = password.toStdString();
}

void EntityI::setRemoteDisplayTls(bool tls)
{
    _remoteDisplayInfo.tls = tls;
}

void EntityI::setRemoteDisplayCa(QString ca)
{
    _remoteDisplayInfo.ca = ca.toStdString();
}

void EntityI::setRemoteDisplayCertSubject(QString certSubject)
{
    _remoteDisplayInfo.certSubject = certSubject.toStdString();
}

QString EntityI::getRemoteDisplayPassword()
{
    return QString::fromStdString(_remoteDisplayInfo.password);
}

void EntityI::_initAvailablePlugsForCapture(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    _availablePlugsForCapture = QList<Ice::Int>::fromStdList(getPlugs());
}

void EntityI::_interfaceStateChangedSlot(QString interfaceName, bool stateUp)
{
    Q_UNUSED(interfaceName)
    Q_UNUSED(stateUp)
}

} // namespace HnsNode
