/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "nodeentityfactoryI.h"
#include "nodeentityI.h"
#include "nodeI.h"

#include "nodewireI.h"

// hynesim server headers
#include "libhnsserver/storagemanager.h"

// Generated hynesim headers
#include "common/wireentry.h"

// ZeroC Ice headers
#include <Ice/ObjectAdapter.h>

// Qt headers
#include <QDebug>
#include <QScopedPointer>
#include <QMetaObject>
#include <QMutex>
#include <QFile>

namespace HnsNode {

EntityFactoryI::EntityFactoryI()
    : _HnsNode_EntityFactory()
{
    registerDefaultChecker<Hns::WireEntry, WireI>();
}

EntityFactoryI::~EntityFactoryI()
{
    QMultiHash<QString, FactoryHelper*>::iterator i = _helpers.begin();
    while (i != _helpers.end()) {
        FactoryHelper *factoryHelper = i.value();
        delete factoryHelper;
        ++i;
    }
}

Q_GLOBAL_STATIC(QMutex, entitiesMutex)

EntityPrx EntityFactoryI::createEntity(const::Hns::EntityEntryPtr &entityEntry, ::Ice::Long id, const::Ice::Current &current)
{
    Q_UNUSED(current)
    QMutexLocker locker(entitiesMutex());
    QString className = QString::fromStdString(entityEntry->ice_id());

    QMetaObject metaObject;
    FactoryHelper *factoryHelper = _getFactoryHelper(entityEntry);
    if (factoryHelper != NULL) {
        metaObject =  factoryHelper->metaObject;
    } else {
        qCritical() << tr("Unable to create the entity for the entry (id : %1).").arg(entityEntry->id);
        return NULL;
    }

    QScopedPointer<EntityI> entity(qobject_cast<EntityI *>(metaObject.newInstance()));
    entity->setEntry(entityEntry);
    entity->setId(id);
    Ice::ObjectAdapterPtr adapter = Hns::NodeI::instance()->adapter();
    Ice::ObjectPrx proxy = adapter->addWithUUID(entity.data());
    EntityI *validEntity = entity.take();
    validEntity->setIdentity(proxy->ice_getIdentity());
    validEntity->init();
    EntityPrx entityPrx = EntityPrx::uncheckedCast(proxy);
    emit entityAdded(validEntity);
    emit addEntity(id);
    return entityPrx;
}

EntityPrx EntityFactoryI::createEntityFromId(::Ice::Long entryId, ::Ice::Long id , const ::Ice::Current &current)
{
    const Hns::EntityCatalogPrx &catalog = Hns::NodeI::instance()->hnsMaster()->getEntityCatalog();
    ::Hns::EntityEntryPtr entityEntry = catalog->getEntityEntry(entryId, current.ctx);
    if (!entityEntry) {
        qCritical() << tr("Unable to get the catalog entry %1").arg(entryId);
        return NULL;
    }
    return createEntity(entityEntry, id, current);
}

bool EntityFactoryI::destroyEntity(const EntityPrx &entityPrx, const Ice::Current &current)
{
    Q_UNUSED(current)
    EntityI *entity = dynamic_cast<EntityI*>(Hns::NodeI::instance()->adapter()->findByProxy(entityPrx).get());
    if (entity) {
        emit removeEntity(entity->getId());
        return entity->destroy();
    }
    return false;
}

bool EntityFactoryI::canBuild(const::Hns::EntityEntryPtr &entityEntry, const::Ice::Current &current)
{
    Q_UNUSED(current)
    if (!entityEntry->path.empty()) {
        HnsServer::StorageManager *storageManager = Hns::NodeI::instance()->storageManager();
        QString path = storageManager->getFullPathFromMixed("entity", QString::fromStdString(entityEntry->path), QString::fromStdString(entityEntry->uuid));
        if (path.isEmpty() || !QFile::exists(path))
            return false;
    }
    return ((entityEntry->isModel || _getFactoryHelper(entityEntry) != NULL));
}

bool EntityFactoryI::canBuildAll(const Hns::EntityEntryPtr &entityEntry, const Hns::EntityEntryList &alreadyDefinedEntities, const Ice::Current &current)
{
    Q_UNUSED(current)
    if (!entityEntry->path.empty()) {
        HnsServer::StorageManager *storageManager = Hns::NodeI::instance()->storageManager();
        QString path = storageManager->getFullPathFromMixed("entity", QString::fromStdString(entityEntry->path), QString::fromStdString(entityEntry->uuid));
        if (path.isEmpty() || !QFile::exists(path))
            return false;
    }
    return ((entityEntry->isModel || _getFactoryHelper(entityEntry, QList<Hns::EntityEntryPtr>::fromStdList(alreadyDefinedEntities)) != NULL));
}

FactoryHelper *EntityFactoryI::_getFactoryHelper(const::Hns::EntityEntryPtr &entityEntry, const QList<Hns::EntityEntryPtr> &alreadyDefinedEntities)
{
    QString className = QString::fromStdString(entityEntry->ice_id());
    QList<FactoryHelper*> factoryHelpers = _helpers.values(className);
    foreach (FactoryHelper *factoryHelper, factoryHelpers) {
        BuildChecker check = factoryHelper->buildChecker;
        if (check(entityEntry, alreadyDefinedEntities))
            return factoryHelper;
    }
    return NULL;
}

const QString  &EntityFactoryI::getNameKeyChecker(const QString & metaClass) const
{
    QHash<QString, QString>::const_iterator itor = _refresh.constFind(metaClass);
    if(itor == _refresh.constEnd()){
        static QString strVoid("");
        return strVoid;
    }
    return itor.value();
}

} // namespace HnsNode
