/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of Hynesim.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "nodelocalmonitorI.h"
#include "nodeentityI.h"
#include "nodeentityfactoryI.h"

// Qt headers
#include <QTimerEvent>
#include <QSignalMapper>

namespace HnsNode {

NodeMonitorI::NodeMonitorI(EntityFactoryI *factory)
    : _HnsNode_NodeMonitor()
    , _updateTimerId(0)
{
    _entityMapper = new QSignalMapper(this);
    QObject::connect(_entityMapper, SIGNAL(mapped(QObject*)), this, SLOT(_entityStatusChanged(QObject*)));
    _updateTimerId = startTimer(1000);
    if (factory) {
        QObject::connect(factory, SIGNAL(entityAdded(EntityI*)), this, SLOT(_entityAdded(EntityI*)));
        QObject::connect(factory, SIGNAL(removeEntity(long)), this, SLOT(_removeEntity(long)));
    }
}

NodeMonitorI::~NodeMonitorI()
{
    killTimer(_updateTimerId);
    _updateTimerId = 0;
    _entitiesDataMap.clear();
}

Hns::EntityMonitorDataMap NodeMonitorI::getEntityMonitoringDataMap(const Ice::Current &current)
{
    Q_UNUSED(current)
    QReadLocker locker(&_dataLocker);
    return _dataAsStdMap();
}

void NodeMonitorI::timerEvent(QTimerEvent *event)
{
    if (event->timerId() == _updateTimerId) {
        QReadLocker locker(&_dataLocker);
        _dataChanged();
    }
}

Hns::EntityMonitorDataMap NodeMonitorI::_dataAsStdMap()
{
    return _entitiesDataMap.toStdMap();
}

void NodeMonitorI::_entityAdded(EntityI *entity)
{
    if (entity->hasFlag(EntityI::Monitoring)) {
        Hns::Status status = entity->getStatus();
        if (status == Hns::UNDEFINED || status == Hns::BUSY) {
            _entityMapper->setMapping(entity, entity);
            QObject::connect(entity, SIGNAL(statusChanged(Ice::Int,Ice::Long)), _entityMapper, SLOT(map()));
        } else
            _monitorEntity(entity);
    }
}

void NodeMonitorI::_removeEntity(long entityId)
{
    QWriteLocker locker(&_dataLocker);
    if (_entitiesDataMap.contains(entityId)) {
        _entitiesDataMap.remove(entityId);
        _dataChanged();
    }
}

void NodeMonitorI::_monitorEntity(EntityI *entity)
{
    QWriteLocker locker(&_dataLocker);
    _entitiesDataMap.insert(entity->getId(), entity->getMonitoringData());
    QObject::connect(entity, SIGNAL(monitoringDataUpdated(::Hns::EntityMonitoringDataPtr)), this, SLOT(_monitoringDataUpdated(::Hns::EntityMonitoringDataPtr)));
    _dataChanged();
}

void NodeMonitorI::_monitoringDataUpdated(Hns::EntityMonitoringDataPtr data)
{
    QWriteLocker locker(&_dataLocker);
    if (_entitiesDataMap.contains(data->entityId)) {
        _entitiesDataMap[data->entityId] = data;
    }
}

void NodeMonitorI::_entityStatusChanged(QObject *object)
{
    EntityI *entity = qobject_cast<HnsNode::EntityI*>(object);
    if (entity) {
        Hns::Status status = entity->getStatus();
        if (status != Hns::UNDEFINED && status != Hns::BUSY) {
            QObject::disconnect(entity, 0, _entityMapper, 0);
            _monitorEntity(entity);
        }
    }
}

void NodeMonitorI::_dataChanged()
{
    emit entityDataChanged(_dataAsStdMap());
}

} // namespace HnsNode
