// hynesim node headers
#include "openvswitchimplementation.h"
#include "nodeentityI.h"
#include "entityplugI.h"
#include "netutils.h"

// hynesim server headers
#include "libhnsserver/processrunner.h"

// Generated hynesim headers
#include "common/switchentry.h"

// Qt headers
#include <QDebug>
#include <QSemaphore>
#include <QStringList>

namespace HnsNode {

Q_GLOBAL_STATIC_WITH_ARGS(QAtomicInt, _idCounter, (1))
Q_GLOBAL_STATIC_WITH_ARGS(QSemaphore, _semaphore, (1))

OpenVSwitchImplementation::OpenVSwitchImplementation(EntityI *entity)
    : SwitchImplementation(entity)
    , _isStarted(false)
{
    setBridgeName(QString("hnsOVS%1").arg(_idCounter()->fetchAndAddOrdered(1)));
}

bool OpenVSwitchImplementation::define()
{
    _semaphore()->acquire();
    bool result = _define();
    _semaphore()->release();
    return result;
}

bool OpenVSwitchImplementation::destroy()
{
    _semaphore()->acquire();
    bool result = _destroy();
    _semaphore()->release();
    return result;
}

bool OpenVSwitchImplementation::start()
{
    _semaphore()->acquire();
    bool result = _start();
    _semaphore()->release();
    return result;
}

bool OpenVSwitchImplementation::stop()
{
    _semaphore()->acquire();
    bool result = _stop();
    _semaphore()->release();
    return result;
}

bool OpenVSwitchImplementation::updateEntry(const Hns::EntityEntryPtr &entityEntry)
{
    _semaphore()->acquire();
    bool result = _updateEntry(entityEntry);
    _semaphore()->release();
    return result;
}

EntityPlugI *OpenVSwitchImplementation::createPlug(Ice::Int port)
{
    _semaphore()->acquire();
    EntityPlugI *result = _createPlug(port);
    _semaphore()->release();
    return result;
}

bool OpenVSwitchImplementation::_define()
{
    Hns::SwitchEntryPtr switchEntry = Hns::SwitchEntryPtr::dynamicCast(entityEntry());
    QString program = "ovs-vsctl";
    QStringList args;
    args << "add-br" << getBridgeName();

    HnsServer::ProcessRunner process;
    if (!process.runSynchronous(program, args)) {
        qCritical() << tr("unable to define OpenVSwitch %1").arg(getBridgeName());
        return false;
    }

    args.clear();
    if (switchEntry->isHub) {
        /* Set hub mode */
        args << "add-flow" << getBridgeName() << "actions=all";
        program = "ovs-ofctl";

        if (!process.runSynchronous(program, args))
            qCritical() << tr("unable to set HUB mode");
    }
    else {
        /* Set STP */
        QString enableStp = switchEntry->stpEnabled ? "true" : "false";
        args << "set" << "BRIDGE" << getBridgeName() << QString("stp_enable=%1").arg(enableStp);

        if (!process.runSynchronous(program, args))
            qCritical() << tr("unable to set STP, set to false by default");
    }

    _vlanHash = _generateVlanHash(entityEntry());
    _portMirroring = switchEntry->portMirroring;
    return true;
}

bool OpenVSwitchImplementation::_destroy()
{
    if (getBridgeName().isEmpty())
        return false;

    QString program = "ovs-vsctl";
    QStringList args;
    args << "del-br" << getBridgeName();

    HnsServer::ProcessRunner process;
    if(process.runSynchronous(program, args))
        return true;

    return false;
}

bool OpenVSwitchImplementation::_start()
{
    _isStarted = true;
    foreach (const QString &portName, getTapHash()) {
        if (!_setPortEnabled(portName, true)) {
            qCritical() << tr("unable to enable port %1 on switch %2").arg(portName).arg(getBridgeName());
            return false;
        }
    }
    return true;
}

bool OpenVSwitchImplementation::_stop()
{
    _isStarted = false;
    foreach (const QString &portName, getTapHash()) {
        if (!_setPortEnabled(portName, false)) {
            // Ignore error because disabling a port can fail if it has already been destroyed
            qCritical() << tr("unable to disable port %1 on switch %2").arg(portName).arg(getBridgeName());
        }
    }
    return true;
}

bool OpenVSwitchImplementation::_updateEntry(const Hns::EntityEntryPtr &entityEntry)
{
    QHash<int, QPair<int, QList<int> > > newVlanHash = _generateVlanHash(entityEntry);
    QList<int> keyHash = _vlanHash.keys();

    for (int i = 0 ; i < keyHash.size() ; ++i) {
        if (getTapHash().contains(i) && newVlanHash.value(i) != _vlanHash.value(i))
            _applyVlan(i, newVlanHash.value(i));
    }

    _vlanHash = newVlanHash;
    return true;
}

EntityPlugI *OpenVSwitchImplementation::_createPlug(Ice::Int port)
{
    if (port > Hns::SwitchEntryPtr::dynamicCast(entityEntry())->ports)
        return NULL;

    SwitchPlug *plug = new SwitchPlug(getBridgeName());
    plug->setPort(port);

    return plug;
}

void OpenVSwitchImplementation::plug(int port, const QString &interfaceName)
{
    _semaphore()->acquire();
    addPort(port, interfaceName);
    QString program = "ovs-vsctl";
    QStringList args;
    args << "add-port" << getBridgeName() << interfaceName;
    HnsServer::ProcessRunner process;
    process.runSynchronous(program, args);

    if (!_isStarted)
        _setPortEnabled(interfaceName, false);
    _applyVlan(port, _vlanHash.value(port));
    if(port == _portMirroring)
        _applyPortMirroring();
    entity()->addInterfaceForCapture(port);
    _semaphore()->release();
}

void OpenVSwitchImplementation::unplug(int port)
{
    _semaphore()->acquire();
    QString program = "ovs-vsctl";
    QStringList args;
    args << "--retry" << "del-port" << getBridgeName() << getTapHash().value(port);

    removePort(port);
    entity()->removeInterfaceForCapture(port);
    _semaphore()->release();

    if(port == _portMirroring)
        _disapplyPortMirroring();

    HnsServer::ProcessRunner process;
    process.runSynchronous(program, args);
}

QList<Ice::Int> OpenVSwitchImplementation::initAvailablePlugsForCapture()
{
    QList<Ice::Int> availablePlugs = getTapHash().keys();
    qSort(availablePlugs.begin(), availablePlugs.end());
    return availablePlugs;
}

bool OpenVSwitchImplementation::_setPortEnabled(QString portName, bool enabled)
{
    QStringList args;
    args << "mod-port" << getBridgeName() << portName;

    QStringList commands;
    if (enabled)
        commands << "receive" << "receive-stp";
    else
        commands << "no-receive" << "no-receive-stp";

    foreach (const QString &command, commands) {
        HnsServer::ProcessRunner process;
        bool ret = process.runSynchronous("ovs-ofctl", args + QStringList(command));
        if (!ret) {
            qCritical() << tr("ovs-ofctl failed");
            return ret;
        }
    }
    return true;
}

QHash<int, QPair<int, QList<int> > > OpenVSwitchImplementation::_generateVlanHash(const::Hns::EntityEntryPtr &entityEntry)
{
    QHash<int, QPair<int, QList<int> > > vlanHash;
    Hns::SwitchEntryPtr switchEntry = Hns::SwitchEntryPtr::dynamicCast(entityEntry);
    QMap<Ice::Int, Hns::Vlan> map = QMap<Ice::Int, Hns::Vlan>(switchEntry->vlans);
    QMapIterator<Ice::Int, Hns::Vlan> iterator(map);

    while (iterator.hasNext()) {
        iterator.next();

        Hns::Vlan vlan = iterator.value();
        int vlanId = vlan.tag;
        QMap<Ice::Int, Hns::PortTag> portTagMap = QMap<Ice::Int, Hns::PortTag>(vlan.portsMap);
        QMapIterator<Ice::Int, Hns::PortTag> portTagMapIterator(portTagMap);

        while (portTagMapIterator.hasNext()) {
            portTagMapIterator.next();

            int port = portTagMapIterator.key();
            Hns::PortTag tag = portTagMapIterator.value();
            QPair<int, QList<int> > data;

            if (vlanHash.contains(port))
                data = vlanHash.value(port);

            if (tag == Hns::Tagged)
                data.second.append(vlanId);
            else
                data.first = vlanId;

            vlanHash.insert(port, data);
        }
    }
    return vlanHash;
}

void OpenVSwitchImplementation::_applyVlan(int port, QPair<int, QList<int> > pair)
{
    QString program = "ovs-vsctl";
    QStringList args;
    QString portName = getTapHash().value(port);
    QString vlanMode;

    if (pair.second.isEmpty()) {
        vlanMode = QString("vlan_mode=access tag=%1").arg(pair.first);
    } else {
        QString trunks;
        int listSize = pair.second.size();

        for (int i = 0 ; i < listSize ; ++i) {
            trunks += QString::number(pair.second.at(i));
            if (i < listSize)
                trunks += ",";
        }
        vlanMode = QString("vlan_mode=native-untagged tag=%1 trunks=[%2]").arg(pair.first).arg(trunks);
    }

    args << "set" << "PORT" << portName << vlanMode.split(" ");
    HnsServer::ProcessRunner process;
    process.runSynchronous(program, args);
}

void OpenVSwitchImplementation::_applyPortMirroring()
{
    QString program = "ovs-vsctl";
    QString mirror_name = getBridgeName() + "_" + getTapHash().value(_portMirroring) + "_mirror";
    QStringList args;
    args << "set" << "Bridge" << getBridgeName() << "mirrors=@m" << "--" << "--id=@outputInterface" << "get" << "Port" << getTapHash().value(_portMirroring);
    args << "--" << "--id=@m" << "create" << "Mirror" << QString("name=%1").arg(mirror_name) << "select-all=true" << "output-port=@outputInterface";

    HnsServer::ProcessRunner process;
    process.runSynchronous(program, args);
}

void OpenVSwitchImplementation::_disapplyPortMirroring()
{
    QString program = "ovs-vsctl";
    QStringList args;
    args << "--retry" << "clear" << "bridge" << getBridgeName() << "mirrors";

    HnsServer::ProcessRunner process;
    process.runSynchronous(program, args);
}

} // namespace HnsNode
