/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "pcapcapturehandler.h"
#include "nodeentityI.h"

// Qt headers
#include <QDebug>
#include <QDir>
#include <QTimer>

namespace HnsNode {

PcapCaptureHandler::PcapCaptureHandler()
    : CaptureHandler()
    , _packetsCount(0)
    , _dumper(NULL)
    , _qpcap(NULL)
    , _creating(false)
{
}

PcapCaptureHandler::~PcapCaptureHandler()
{
    delete(_qpcap);
}

bool PcapCaptureHandler::initCapture()
{
    if (_jobOptions.interfaceList.empty() || _jobOptions.interfaceList.size() > 1) {
        qCritical() << tr("There is no interface filtered in the options or more than one interface is filtered");
        return false;
    }

    QString interfaceToCapture = _getInterfaceToCapture();

    if (interfaceToCapture.isEmpty()) {
        qCritical() << tr("No interface found with id %1").arg(_jobOptions.interfaceList.front());
        return false;
    } else {
        _interfaceToCapture = interfaceToCapture;
    }

    return true;
}

void PcapCaptureHandler::startCapture()
{
    qDebug() << "PcapCaptureHandler::startCapture";
    /* Boolean to avoid segfault with pcap_dump_close in _endCapture */
    _creating = true;

    _qpcap = new QPcap();
    if (!_qpcap->open(_interfaceToCapture)) {
        qCritical() << tr("Can not open interface %1 for capture. Please check if it is UP").arg(_interfaceToCapture);
        return;
    }

    if (!_jobOptions.filteringterms.empty())
        _qpcap->setFilter(QString::fromStdString(_jobOptions.filteringterms));

    if (!QPcap::connect(_qpcap, SIGNAL(packetReady(QPcapHeader, const uchar *)),
                        this, SLOT(_packetReceived(QPcapHeader,const uchar *))))
        return;

    if (!QPcap::connect(_qpcap, SIGNAL(pcapClosed()),
                        this, SLOT(_afterPcapClosed())))
        return;

    /* Open file dumper */
    QDir directory;
    if (!directory.exists(_captureFullPath) && !directory.mkdir(_captureFullPath)) {
        qCritical() << tr("Can not build capture directory");
        return;
    }
    directory.setPath(_captureFullPath);

    _dumper = pcap_dump_open(_qpcap->handle(), directory.absoluteFilePath(_captureFileName).toAscii());

    if (!_dumper) {
        qCritical() << tr("Unable to open file for dumping, filename is : %1\nError is : %2")
                       .arg(_captureFileName).arg(pcap_geterr(_qpcap->handle()));
        return;
    }

    if (_jobOptions.timer != 0) {
        _captureTimer->start(_jobOptions.timer * 1000);
        _elapsedTime.start();
    }

    /* Progress not updated if capture is infinite in time and packets */
    if (_jobOptions.numberpackets != 0 || _jobOptions.timer != 0)
        _progressTimer->start();

    _qpcap->start();
    setStarted(true);
    _entity->removeInterfaceForCapture(_jobOptions.interfaceList.front());
    _creating=false;
}

bool PcapCaptureHandler::setOptions(const Hns::CaptureJobOptions &jobOptions)
{
    if (!CaptureHandler::setOptions(jobOptions)) {
        return false;
    }
    if (_jobOptions.interfaceList.empty() || _jobOptions.interfaceList.size() > 1) {
        qCritical() << tr("There is no interface filtered in the options or more than one interface is filtered");
        return false;
    }

    QString interfaceToCapture = _getInterfaceToCapture();

    if (interfaceToCapture.isEmpty()) {
        qCritical() << tr("No interface found with id %1").arg(_jobOptions.interfaceList.front());
        return false;
    }
    return true;
}

void PcapCaptureHandler::_packetReceived(QPcapHeader packetHeader, const uchar *packet)
{
    _packetsCount++;
    const struct pcap_pkthdr *pcapHeader = packetHeader.pcap_header();
    pcap_dump((uchar *) _dumper, pcapHeader, packet);
    pcap_dump_flush(_dumper);

    if (_jobOptions.numberpackets != 0 && _packetsCount == _jobOptions.numberpackets)
        endCapture();
}

void PcapCaptureHandler::endCapture()
{
    if (_creating){
        qDebug()<<tr("Error, you cannot end a capture so soon after launching it.");
        return;
    }
    if (_qpcap == 0) {
        return;
    }

    if(_progressTimer->isActive())
        _progressTimer->stop();
    if (_captureTimer->isActive())
        _captureTimer->stop();

    QPcap::disconnect(_qpcap, SIGNAL(packetReady(QPcapHeader, const uchar *)),
                      this, SLOT(_packetReceived(QPcapHeader,const uchar*)));
    _qpcap->stop();
    _entity->addInterfaceForCapture(_jobOptions.interfaceList.front());
    /* Close file and qpcap */
    /* pcap_dispatch throws a segfault on QPcap::close if called by the slot _pktRecieve,
            hence the call on the main event loop */
    QMetaObject::invokeMethod(_qpcap, "close", Qt::QueuedConnection);
}

void PcapCaptureHandler::_afterPcapClosed()
{
    pcap_dump_close(_dumper);
    emit progressChanged(1.0);
}

int PcapCaptureHandler::_getPacketsCount()
{
    return _packetsCount;
}

} // namespace HnsNode
