/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "platformdomain.h"
#include "netcard.h"
#include "nodeI.h"
#include "nodedomainI.h"

// hynesim server headers
#include "libhnsserver/storagemanager.h"

// hynesim common headers
#include "common/initializer.h"

// diamesh headers
#include <diamesh/utils.h>

// Qt headers
#include <QAtomicInt>
#include <QDebug>
#include <QHash>
#include <QList>
#include <QDomDocument>
#include <QDir>
#include <QDateTime>
#include <QCryptographicHash>

// C++ std headers
#include <stdexcept>

namespace HnsNode {

const char * PlatformDomain::saveFileName = "hnsGuestSave";

typedef QHash<QString, PlatformDomain*> PlatformDomainHash;
Q_GLOBAL_STATIC(PlatformDomainHash, platformDomainHash)

Q_GLOBAL_STATIC(QMutex, remoteDisplayPortPoolMutex)
const int MIN_REMOTE_DISPLAY_PORT = 5900;
const int MAX_REMOTE_DISPLAY_PORT = 6900;

HNS_STATIC_HASH_INITIALIZER(_remoteDisplayPortPool, RemoteDisplayPortPool, int, bool)
RemoteDisplayPortPool::RemoteDisplayPortPool()
{
    for (int i = MIN_REMOTE_DISPLAY_PORT ; i <= MAX_REMOTE_DISPLAY_PORT ; ++i)
        insert(i, false);
}

PlatformDomain::PlatformDomain(const QStringList &domainType, const QString &uri, bool needPooledConnection)
    : Platform()
    , _needPooledConnection(needPooledConnection)
    , _closeTemporaryConnections(true)
    , _uri(uri)
    , _hostNbCpu(0)
{
    _connection = virConnectOpen(qPrintable(_uri));

    if (!_connection) {
        qCritical() << tr("PlatformDomain: connection failed to %1").arg(_uri);
        throw LibvirtConnectException();
    }
    foreach (QString _domainType,domainType){
        _registerPlatformDomain(_domainType);
    }
    _extendedCapabilities.append("actionManager");
    _hostNbCpu = Hns::NodeI::instance()->getSystemInfo().processors.size();
}

PlatformDomain::~PlatformDomain()
{
}

virConnectPtr PlatformDomain::_openNewConnectionIfNeeded()
{
    if (!_needPooledConnection)
        return _connection;
    return _getConnection();
}

void PlatformDomain::_cleanupIfNeeded(virConnectPtr connection, DomainI *hnsDomain, virDomainPtr domain)
{
    _closeConnection(connection);

    if (domain && (domain != hnsDomain->_virDomain))
        virDomainFree(domain);

    if (connection && _closeTemporaryConnections && !_needPooledConnection && (connection != _connection)) {
        QMutexLocker locker(&_connectionMutex);
        virConnectClose(connection);
    }
}

virConnectPtr HnsNode::PlatformDomain::_getConnection()
{
    return _connection;
}

void HnsNode::PlatformDomain::_closeConnection(virConnectPtr connection)
{
    Q_UNUSED(connection)
}

virDomainPtr PlatformDomain::_getVirDomainForConnection(virDomainPtr domain, virConnectPtr connection)
{
    virConnectPtr domainConnection = virDomainGetConnect(domain);
    virDomainPtr dom;
    // This is already the domain connection, no need to get a new virDomainPtr
    if (connection == domainConnection)
        return domain;
    char uuid[VIR_UUID_STRING_BUFLEN];
    if (virDomainGetUUIDString(domain, &uuid[0]))
        throw std::runtime_error(tr("PlatformDomain: virDomainGetUUIDString() failed").toStdString());
    if (connection) {
        dom = virDomainLookupByUUIDString(connection, &uuid[0]);
    }
    else {
        dom = virDomainLookupByUUIDString(_getConnection(), &uuid[0]);
    }
    if (!dom)
        throw std::runtime_error(tr("PlatformDomain: virDomainLookupByUUIDString() failed").toStdString());

    return dom;
}

PlatformDomain *PlatformDomain::get(const std::string &domainType)
{
    return platformDomainHash()->value(QString::fromStdString(domainType));
}

bool PlatformDomain::defineEntity(EntityI *entity)
{
    DomainI *domain = qobject_cast<DomainI *>(entity);
    if (!domain) {
        qCritical() << "PlatformDomain::defineEntity: NULL DomainI";
        return false;
    }

    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(domain->getEntry());
    if (!domainEntry) {
        qCritical() << "PlatformDomain::defineEntity: NULL DomainEntry";
        return false;
    }

    QString domainPath = domain->path();
    if (domainPath.isEmpty()) {
        qCritical() << "PlatformDomain::defineEntity: NULL Domain FullPath";
        return false;
    }

    QDomDocument xml;
    QDomElement xmlDomain = xml.createElement("domain");
    xml.appendChild(xmlDomain);
    xmlDomain.setAttribute("type", QString::fromStdString(domainEntry->type));

    _addTextNode(xmlDomain, "name", QString::fromStdString(domainEntry->uuid));
    _addTextNode(xmlDomain, "uuid", QString::fromStdString(domainEntry->uuid));
    _addTextNode(xmlDomain, "vcpu", QString::number(domainEntry->vcpu));
    _addTextNode(xmlDomain, "memory", QString::number(domainEntry->memory));

    QDomElement xmlDomOs = xml.createElement("os");
    xmlDomain.appendChild(xmlDomOs);

    QDomElement xmlDomFeatures = xml.createElement("features");
    xmlDomain.appendChild(xmlDomFeatures);

    if(domainEntry->pae)
      _addTextNode(xmlDomFeatures, "pae", "");
    if(domainEntry->acpi)
      _addTextNode(xmlDomFeatures, "acpi", "");
    if(domainEntry->apic)
      _addTextNode(xmlDomFeatures, "apic", "");

    QDomElement xmlDomDevices = xml.createElement("devices");
    xmlDomain.appendChild(xmlDomDevices);  

    if (!_beforeLibvirtDefine(xml, domain))
        return false;

    bool ret = true;
    virConnectPtr connection = NULL;
    virDomainPtr dom = NULL;
    try {
        connection = _openNewConnectionIfNeeded();

        if (getenv("HNS_DEBUG"))
            qDebug() << tr("Going to define a domain with the following XML:\n%1").arg(xml.toString(2));

        dom = virDomainDefineXML(connection, xml.toByteArray(-1).constData());
        if (!dom)
            throw std::runtime_error(tr("PlatformDomain::defineEntity: libvirt define failed").toStdString());

        // get VirDomainPtr for the main connection
        domain->_virDomain = _getVirDomainForConnection(dom, _connection);

        if (!_afterLibvirtDefine(domain))
            ret = false;

    } catch (std::runtime_error &e) {
        ret = false;
        qCritical() << e.what();
        // Cancel what was done during _beforeLibvirtDefine()
        _afterLibvirtUndefine(domain);
    }
    _cleanupIfNeeded(connection, domain, dom);

    return ret;
}

bool PlatformDomain::undefineEntity(EntityI *entity)
{
    DomainI *domain = qobject_cast<DomainI *>(entity);
    if (!domain) {
        qCritical() << "PlatformDomain::undefineEntity: NULL DomainI";
        return false;
    }

    if (!_beforeLibvirtUndefine(domain))
        return false;

    bool ret = true;
    virConnectPtr connection = NULL;
    virDomainPtr dom = NULL;
    try {
        connection = _openNewConnectionIfNeeded();
        dom = _getVirDomainForConnection(domain->_virDomain, connection);

        if (virDomainUndefine(dom) != 0)
            throw std::runtime_error(tr("PlatformDomain::undefineEntity: libvirt undefine failed").toStdString());

    } catch (std::runtime_error &e) {
        ret = false;
        qCritical() << e.what();
    }

    _cleanupIfNeeded(connection, domain, dom);

    if (!_afterLibvirtUndefine(domain))
        ret = false;

    return ret;
}

bool PlatformDomain::createEntity(EntityI *entity)
{
    DomainI *domain = qobject_cast<DomainI *>(entity);
    if (!domain) {
        qCritical() << "PlatformDomain::createEntity: NULL DomainI";
        return false;
    }
    if (!_beforeLibvirtCreate(domain)) {
        qCritical() << "PlatformDomain::createEntity: error while doing _beforeLibvirtCreate()";
        return false;
    }
    bool ret = true;
    virConnectPtr connection = NULL;
    virDomainPtr dom = NULL;
    try {
        connection = _openNewConnectionIfNeeded();
        dom = _getVirDomainForConnection(domain->_virDomain, connection);

        if (virDomainCreate(dom) != 0)
            throw std::runtime_error(tr("PlatformDomain::createEntity: libvirt create failed").toStdString());

        if (!_afterLibvirtCreate(domain))
            throw std::runtime_error(tr("PlatformDomain::createEntity: libvirt after create failed").toStdString());

    } catch (std::runtime_error &e) {
        _createEntityCleanup();
        ret = false;
        qCritical() << e.what();
    }

    _cleanupIfNeeded(connection, domain, dom);

    return ret;
}

bool PlatformDomain::destroyEntity(EntityI *entity)
{
    DomainI *domain = qobject_cast<DomainI *>(entity);
    if (!domain) {
        qCritical() << "PlatformDomain::destroyEntity: NULL DomainI";
        return false;
    }

    bool ret = true;
    virConnectPtr connection = NULL;
    virDomainPtr dom = NULL;
    try {
        connection = _openNewConnectionIfNeeded();
        dom = _getVirDomainForConnection(domain->_virDomain, connection);

        if (!_beforeLibvirtDestroy(domain))
            throw std::runtime_error(tr("PlatformDomain::destroyEntity: libvirt before destroy failed").toStdString());

        if (virDomainDestroy(dom) != 0)
            throw std::runtime_error(tr("PlatformDomain::destroyEntity: libvirt destroy failed").toStdString());

        if (!_afterLibvirtDestroy(domain))
            throw std::runtime_error(tr("PlatformDomain::destroyEntity: libvirt after destroy failed").toStdString());

    } catch (std::runtime_error &e) {
        ret = false;
        qCritical() << e.what();
    }

    _cleanupIfNeeded(connection, domain, dom);

    return ret;
}

bool PlatformDomain::suspendEntity(EntityI *entity)
{
    DomainI *domain = qobject_cast<DomainI *>(entity);
    if (!domain) {
        qCritical() << "PlatformDomain::suspendEntity: NULL DomainI";
        return false;
    }

    bool ret = true;
    virConnectPtr connection = NULL;
    virDomainPtr dom = NULL;
    try {
        connection = _openNewConnectionIfNeeded();
        dom = _getVirDomainForConnection(domain->_virDomain, connection);

        if (!_beforeLibvirtDestroy(domain))
            throw std::runtime_error(tr("PlatformDomain::suspendEntity: libvirt before destroy failed").toStdString());

        if (virDomainSuspend(dom) != 0)
            throw std::runtime_error(tr("PlatformDomain::suspendEntity: libvirt suspend failed").toStdString());

    } catch (std::runtime_error &e) {
        ret = false;
        qCritical() << e.what();
    }

    _cleanupIfNeeded(connection, domain, dom);

    return ret;
}

bool PlatformDomain::resumeEntity(EntityI *entity)
{
    DomainI *domain = qobject_cast<DomainI *>(entity);
    if (!domain) {
        qCritical() << "PlatformDomain::resumeEntity: NULL DomainI";
        return false;
    }

    bool ret = true;
    virConnectPtr connection = NULL;
    virDomainPtr dom = NULL;
    try {
        connection = _openNewConnectionIfNeeded();
        dom = _getVirDomainForConnection(domain->_virDomain, connection);

        if (virDomainResume(dom) != 0)
            throw std::runtime_error(tr("PlatformDomain::resumeEntity: libvirt resume failed").toStdString());

        if (!_afterLibvirtResume(domain))
            throw std::runtime_error(tr("PlatformDomain::resumeEntity: libvirt after resume failed").toStdString());

    } catch (std::runtime_error &e) {
        ret = false;
        qCritical() << e.what();
    }

    _cleanupIfNeeded(connection, domain, dom);

    return ret;
}

bool PlatformDomain::createSnapshot(EntityI *entity, QString xmlDesc)
{
    if (!getFeatures(HnsNode::Snapshot))
        return false;

    DomainI *domain = qobject_cast<DomainI *>(entity);
    if (!domain) {
        qCritical() << "PlatformDomain::createSnapshot NULL DomainI";
        return false;
    }

    bool ret = true;
    virConnectPtr connection = NULL;
    virDomainPtr dom = NULL;
    virDomainSnapshotPtr snapshot = NULL;
    QString snapshotDescription;
    try {
        connection = _openNewConnectionIfNeeded();
        dom = _getVirDomainForConnection(domain->_virDomain, connection);
        snapshot = virDomainSnapshotCreateXML(dom, xmlDesc.toLatin1(), 0);
        if (snapshot == NULL)
            throw std::runtime_error(tr("PlatformDomain::createSnapshot: libvirt create snapshot failed").toStdString());

        snapshotDescription = QString::fromUtf8(virDomainSnapshotGetXMLDesc(snapshot, 0));

    } catch (std::runtime_error &e) {
        ret = false;
        qCritical() << e.what();
    }

    _cleanupIfNeeded(connection, domain, dom);
    return ret;
}

bool PlatformDomain::revertSnapshot(EntityI *entity, QString snapshotName)
{
    if (!getFeatures(HnsNode::Snapshot))
        return false;

    DomainI *domain = qobject_cast<DomainI *>(entity);
    if (!domain) {
        qCritical() << "PlatformDomain::revertSnapshot NULL DomainI";
        return false;
    }

    bool ret = true;
    virConnectPtr connection = NULL;
    virDomainPtr dom = NULL;
    try {
        connection = _openNewConnectionIfNeeded();
        dom = _getVirDomainForConnection(domain->_virDomain, connection);

        virDomainSnapshotPtr snapToRevert = virDomainSnapshotLookupByName(dom, snapshotName.toLatin1(), 0);
        if (snapToRevert == NULL)
            throw std::runtime_error(tr("PlatformDomain::revertSnapshot libvirt virDomainSnapshotLookupByName failed").toStdString());

        if (virDomainRevertToSnapshot(snapToRevert, 0) != 0)
            throw std::runtime_error(tr("PlatformDomain::revertSnapshot libvirt virDomainRevertToSnapshot failed").toStdString());

    } catch (std::runtime_error &e) {
        ret = false;
        qCritical() << e.what();
    }
    _refreshSnapshotList(domain);

    _cleanupIfNeeded(connection, domain, dom);

    return ret;
}

bool PlatformDomain::deleteSnapshot(EntityI *entity, QString snapshotName)
{
    if (!getFeatures(HnsNode::Snapshot))
        return false;

    DomainI *domain = qobject_cast<DomainI *>(entity);
    if (!domain) {
        qCritical() << "PlatformDomain::deleteSnapshot NULL DomainI";
        return false;
    }
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(domain->getEntry());
    bool ret = true;
    virConnectPtr connection = NULL;
    virDomainPtr dom = NULL;
    try {
        connection = _openNewConnectionIfNeeded();
        dom = _getVirDomainForConnection(domain->_virDomain, connection);

        virDomainSnapshotPtr snapToDelete = virDomainSnapshotLookupByName(dom, snapshotName.toLatin1(), 0);
        if (snapToDelete == NULL)
            throw std::runtime_error(tr("PlatformDomain::deleteSnapshot libvirt virDomainSnapshotLookupByName failed").toStdString());
        foreach (Hns::Snapshot snap, domainEntry->snapshots) {
            /*Workaround because vbox driver in libvirt does not handle virDomainSnapshotNumChildren*/
            if (QString::fromStdString(snap.parent) == snapshotName) {
                qCritical() << tr("You want to delete snapshots that have many children, this is not possible for the moment");
                throw std::runtime_error(tr("PlatformDomain::deleteSnapshot: (too many children)").toStdString());
            }
        }

        if (virDomainSnapshotDelete(snapToDelete, 0) != 0)
            throw std::runtime_error(tr("PlatformDomain::deleteSnapshot libvirt virDomainSnapshotDelete failed").toStdString());

    } catch (std::runtime_error &e) {
        ret = false;
        qCritical() << e.what();
    }
    _cleanupIfNeeded(connection, domain, dom);
    return ret;
}

Hns::Snapshot PlatformDomain::getSnapshot(EntityI *entity, QString snapshotName)
{
    if (!getFeatures(HnsNode::Snapshot))
        return Hns::Snapshot();

    DomainI *domain = qobject_cast<DomainI *>(entity);
    if (!domain) {
        qCritical() << "PlatformDomain::getSnapshot NULL DomainI";
        return Hns::Snapshot();
    }

    virConnectPtr connection = NULL;
    virDomainPtr dom = NULL;
    QString xml;
    try {
        connection = _openNewConnectionIfNeeded();
        dom = _getVirDomainForConnection(domain->_virDomain, connection);

        virDomainSnapshotPtr snapToDescribe = virDomainSnapshotLookupByName(dom, snapshotName.toLatin1(), 0);
        if (snapToDescribe == NULL)
            throw std::runtime_error(tr("PlatformDomain::getSnapshot libvirt virDomainSnapshotLookupByName failed").toStdString());

        xml = virDomainSnapshotGetXMLDesc(snapToDescribe, 0);
        if (xml.isEmpty())
            throw std::runtime_error(tr("PlatformDomain::getSnapshot libvirt virDomainSnapshotGetXMLDesc failed").toStdString());
    } catch (std::runtime_error &e) {
        qCritical() << e.what();
    }

    if (xml.isEmpty())
        return Hns::Snapshot();

    QDomDocument doc;
    doc.setContent(xml);
    QDomElement e = doc.firstChildElement("domainsnapshot");

    QString name = e.firstChildElement("name").text();
    QString parent = e.firstChildElement("parent").text();
    QString creationDateTime = e.firstChildElement("creationTime").text();
    QString description = e.firstChildElement("description").text();

    Hns::Snapshot snapshot;
    snapshot.name = name.toStdString();
    snapshot.description = description.toStdString();
    snapshot.creationDateTime = creationDateTime.toStdString();
    snapshot.parent = parent.toStdString();
    _cleanupIfNeeded(connection, domain, dom);
    return snapshot;
}

QString PlatformDomain::getCurrentSnapshotName(EntityI *entity)
{
    if (!getFeatures(HnsNode::Snapshot))
        return QString();

    DomainI *domain = qobject_cast<DomainI *>(entity);
    if (!domain) {
        qCritical() << "PlatformDomain::getCurrentSnapshotName NULL DomainI";
        return QString();
    }

    virConnectPtr connection = NULL;
    virDomainPtr dom = NULL;
    QString name;
    try {
        connection = _openNewConnectionIfNeeded();
        dom = _getVirDomainForConnection(domain->_virDomain, connection);

        if (virDomainHasCurrentSnapshot(dom, 0) == 1) {
                virDomainSnapshotPtr snapCurrent = virDomainSnapshotCurrent(dom, 0);
                if (snapCurrent == NULL)
                    throw std::runtime_error(tr("PlatformDomain::getCurrentSnapshotName libvirt virDomainSnapshotCurrent failed").toStdString());
                name = virDomainSnapshotGetName(snapCurrent);
        }

    } catch (std::runtime_error &e) {
        qCritical() << e.what();
    }
    _cleanupIfNeeded(connection, domain, dom);
    return name;
}

int PlatformDomain::domainStatus(DomainI *domain)
{
    // using domain->_virDomain directly here, because this is always called from the main thread
    int state;
    if (virDomainGetState(domain->_virDomain, &state, NULL, 0) == 0) {
        switch (state) {
        case VIR_DOMAIN_RUNNING:
            return Hns::STARTED;
        case VIR_DOMAIN_PAUSED:
            return Hns::PAUSED;
        case VIR_DOMAIN_SHUTOFF:
            return Hns::STOPPED;
        default:
            return -1;
        }
    }
    qCritical() << tr("PlatformDomain::domainStatus: unknown error");
    return -1;
}

void PlatformDomain::createNetcards(DomainI *domain)
{
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(domain->_entityEntry);
    Hns::NetcardMap netcards = domainEntry->netcards;
    for (Hns::NetcardMap::const_iterator it = netcards.begin(); it != netcards.end(); ++it)
        domain->_netcardMap.insert((*it).first, _createNetcard((*it).second));
}

bool PlatformDomain::_saveDomain(DomainI *domain)
{
    bool ret = true;
    virConnectPtr connection = NULL;
    virDomainPtr dom = NULL;
    try {
        connection = _openNewConnectionIfNeeded();
        dom = _getVirDomainForConnection(domain->_virDomain, connection);

        if (virDomainSave(dom, qPrintable(suspendPath(domain))) != 0)
            throw std::runtime_error(tr("PlatformDomain::saveEntity: libvirt save failed").toStdString());

    } catch (std::runtime_error &e) {
        ret = false;
        qCritical() << e.what();
    }
    _cleanupIfNeeded(connection, domain, dom);

    return ret;
}

bool PlatformDomain::saveEntity(EntityI *entity)
{
    DomainI *domain = qobject_cast<DomainI *>(entity);
    if (!domain) {
        qCritical() << "PlatformDomain::saveEntity: NULL DomainI";
        return false;
    }


    if (!_beforeLibvirtDestroy(domain)) {
        qCritical() << tr("PlatformDomain::saveEntity: libvirt before destroy failed");
        return false;
    }

    if (!_saveDomain(domain))
        return false;

    qDebug() << tr("PlatformDomain::saveEntity: saved on disk -%1").arg(suspendPath(domain));
    return true;
}

bool PlatformDomain::_restoreDomain(DomainI *domain)
{
    bool ret = true;
    virConnectPtr connection = NULL;
    try {
        connection = _openNewConnectionIfNeeded();

        if (virDomainRestore(connection, qPrintable(suspendPath(domain))) != 0)
            throw std::runtime_error(tr("PlatformDomain::restoreEntity: libvirt restore failed").toStdString());

    } catch (std::runtime_error &e) {
        ret = false;
        qCritical() << e.what();
    }
    _cleanupIfNeeded(connection, domain, NULL);

    return ret;
}

void PlatformDomain::_createEntityCleanup()
{
}

Netcard *PlatformDomain::_createNetcard(const Hns::Netcard &netcardEntry)
{
    return new Netcard(netcardEntry);
}

bool PlatformDomain::restoreEntity(EntityI *entity)
{
    DomainI *domain = qobject_cast<DomainI *>(entity);
    if (!domain) {
        qCritical() << "PlatformDomain::restoreEntity: NULL DomainI";
        return false;
    }

    if (!_beforeLibvirtRestore(domain)) {
        qCritical() << tr("PlatformDomain::restoreEntity: libvirt before restore failed");
        return false;
    }

    if (!_restoreDomain(domain))
        return false;

    qDebug() << tr("PlatformDomain::restoreEntity: restored from disk -%1").arg(suspendPath(domain));
    return true;
}

bool PlatformDomain::_beforeLibvirtDefine(QDomDocument &/*xml*/, DomainI *domain)
{
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(domain->getEntry());
    int actionManagerInterface = domainEntry->aminterface;

    foreach (Netcard *netcard, domain->netcards()) {
        Hns::Netcard netcardEntry = netcard->netcardEntry();

        if(actionManagerInterface != netcardEntry.id){
            if (!createBridge(netcard->devName(), HnsNode::Up, true)) {
                qCritical() << tr("PlatformDomain::_beforeLibvirtDefine: can't create bridge %1").arg(netcard->devName());
                return false;
            }
        }
    }
    return true;
}

bool PlatformDomain::_afterLibvirtDefine(DomainI *domain)
{
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(domain->getEntry());
    QString entryAbsolutePath = Hns::NodeI::instance()->storageManager()->getFullPathFromMixed("entity",QString::fromStdString(domainEntry->path), QString::fromStdString(domainEntry->uuid));
    QString xmlDomainLibvirt;
    if (getFeatures(HnsNode::Snapshot) && domainEntry->snapshots.size() > 0) {
        virConnectPtr connection = NULL;
        virDomainPtr dom = NULL;
        try {
            connection = _openNewConnectionIfNeeded();
            foreach (Hns::Snapshot snapshot, QList<Hns::Snapshot>::fromStdList(_sortSnapshotListForParentFirst(domainEntry->snapshots))) {
                dom = virDomainLookupByName(connection, domainEntry->uuid.c_str());
                domain->setVirDomainPtr(dom);
                xmlDomainLibvirt = QString::fromUtf8(virDomainGetXMLDesc(dom, 0));
                if (xmlDomainLibvirt.isEmpty())
                    throw std::runtime_error(tr("PlatformVBox::_afterLibvirtDefine libvirt virDomainGetXMLDesc failed").toStdString());
                QDomDocument snapXml;
                QDomElement root = snapXml.createElement("domainsnapshot");
                snapXml.appendChild(root);
                QDomElement name = snapXml.createElement("name");
                name.appendChild(snapXml.createTextNode(QString::fromStdString(snapshot.name)));
                root.appendChild(name);
                QDomElement description = snapXml.createElement("description");
                description.appendChild(snapXml.createTextNode(QString::fromStdString(snapshot.description)));
                root.appendChild(description);
                QDomElement parent = snapXml.createElement("parent");
                QDomElement parentName = snapXml.createElement("name");
                parentName.appendChild(snapXml.createTextNode(QString::fromStdString(snapshot.parent)));
                parent.appendChild(parentName);
                root.appendChild(parent);
                QDomElement state = snapXml.createElement("state");
                state.appendChild(snapXml.createTextNode("shutoff"));
                root.appendChild(state);
                QDomElement creationTime = snapXml.createElement("creationTime");
                creationTime.appendChild(snapXml.createTextNode(QString::fromStdString(snapshot.creationDateTime)));
                root.appendChild(creationTime);
                QDomElement disks = snapXml.createElement("disks");
                QMap<std::string, std::string>childDisks(snapshot.childDisks);
                foreach (std::string name, childDisks.keys()) {
                    if (!childDisks.value(name).empty()) {
                        QDomElement disk = snapXml.createElement("disk");
                        disk.setAttribute("name", QString::fromStdString(name));
                        QDomElement source = snapXml.createElement("source");
                        QString absolutePath;
                        if (!QDir::isAbsolutePath(QString::fromStdString(childDisks.value(name))))
                            absolutePath = QDir::cleanPath(entryAbsolutePath + QString("/") + QString::fromStdString(childDisks.value(name)));
                        else
                            absolutePath = QString::fromStdString(childDisks.value(name));
                        source.setAttribute("file", absolutePath);
                        disk.appendChild(source);
                        disks.appendChild(disk);
                    }
                }
                root.appendChild(disks);

                QDomDocument domXml;
                domXml.setContent(xmlDomainLibvirt);
                if (domainEntry->type == "vbox") {
                    QMap<std::string, std::string> parentDisks(snapshot.parentDisks);
                    QDomElement domainElement = domXml.firstChildElement("domain");
                    QDomElement devicesElement = domainElement.firstChildElement("devices");
                    QDomNodeList diskNodeList = devicesElement.elementsByTagName("disk");
                    for (int j = 0, total = diskNodeList.length() ; j < total ; ++j) {
                        QDomElement diskElement = diskNodeList.at(j).toElement();
                        QDomElement sourceElement = diskElement.firstChildElement("source");
                        QDomElement targetElement = diskElement.firstChildElement("target");
                        std::string keyMap = targetElement.attribute("dev").toStdString();
                        QString absolutePath;
                        if (!QDir::isAbsolutePath(QString::fromStdString(parentDisks.value(keyMap))))
                            absolutePath = QDir::cleanPath(entryAbsolutePath + QString("/") + QString::fromStdString(parentDisks.value(keyMap)));
                        else
                            absolutePath = QString::fromStdString(parentDisks.value(keyMap));
                        sourceElement.setAttribute("file", absolutePath);
                    }
                }
                root.appendChild(domXml);
                if (snapshot.name == domainEntry->currentSnapshotName) {
                    if (virDomainSnapshotCreateXML(dom, snapXml.toString().toUtf8(), VIR_DOMAIN_SNAPSHOT_CREATE_REDEFINE | VIR_DOMAIN_SNAPSHOT_CREATE_CURRENT) == NULL)
                        throw std::runtime_error(tr("PlatformDomain::_afterLibvirtDefine libvirt virDomainSnapshotCreateXML failed CURRENT").toStdString());
                } else {
                    if (virDomainSnapshotCreateXML(dom, snapXml.toString().toUtf8(), VIR_DOMAIN_SNAPSHOT_CREATE_REDEFINE) == NULL)
                        throw std::runtime_error(tr("PlatformDomain::_afterLibvirtDefine libvirt virDomainSnapshotCreateXML failed").toStdString());
                }
            }
            _cleanupIfNeeded(connection, domain, dom);
        } catch (std::runtime_error &e) {
            qCritical() << e.what();
            return false;
        }
    }
    if (domain->hasFlag(EntityI::Monitoring))
        initMonitoringInformation(domain);
    return true;
}

bool HnsNode::PlatformDomain::_beforeLibvirtCreate(HnsNode::DomainI *domain)
{
    Q_UNUSED(domain)
    return true;
}

bool PlatformDomain::_afterLibvirtCreate(DomainI *domain)
{
    Q_UNUSED(domain)
    return true;
}

bool PlatformDomain::_beforeLibvirtDestroy(DomainI *domain)
{
    Q_UNUSED(domain)
    return true;
}

bool PlatformDomain::_afterLibvirtDestroy(DomainI *domain)
{
    Q_UNUSED(domain)
    return true;
}

bool PlatformDomain::_beforeLibvirtUndefine(DomainI *domain)
{
    bool ret = true;
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(domain->getEntry());
    if (getFeatures(HnsNode::Snapshot) && domainEntry->snapshots.size() > 0) {

        virConnectPtr connection = NULL;
        virDomainPtr dom = NULL;
        try {
            connection = _openNewConnectionIfNeeded();
            QList<Hns::Snapshot> snapshotsList = QList<Hns::Snapshot>::fromStdList(_sortSnapshotListForChildrenFirst(domainEntry->snapshots));
            foreach (Hns::Snapshot snapshot, snapshotsList) {
                dom = virDomainLookupByUUIDString(connection, domainEntry->uuid.c_str());
                if (!dom) {
                    throw std::runtime_error(tr("PlatformDomain::_beforeLibvirtUndefine libvirt virDomainLookupByUUIDString(%1) failed").arg(domainEntry->uuid.c_str()).toStdString());
                }
                domain->setVirDomainPtr(dom);
                virDomainSnapshotPtr libvirtSnap = virDomainSnapshotLookupByName(dom, snapshot.name.c_str(), 0);
                if (libvirtSnap == NULL)
                    throw std::runtime_error(tr("PlatformDomain::_beforeLibvirtUndefine libvirt virDomainSnapshotLookupByName failed").toStdString());

                if ((virDomainSnapshotDelete(libvirtSnap, VIR_DOMAIN_SNAPSHOT_DELETE_METADATA_ONLY)) != 0)
                    throw std::runtime_error(tr("PlatformDomain::_beforeLibvirtUndefine libvirt virDomainSnapshotDelete failed").toStdString());
            }
            _cleanupIfNeeded(connection, domain, dom);
        } catch (std::runtime_error &e) {
            ret = false;
            qCritical() << e.what();
        }
    }
    return ret;
}

bool PlatformDomain::_afterLibvirtUndefine(DomainI *domain)
{    
    int actionManagerInterface = Hns::DomainEntryPtr::dynamicCast(domain->getEntry())->aminterface;
    foreach (Netcard *netcard, domain->netcards()) {
        if (actionManagerInterface != netcard->netcardEntry().id)
            removeBridge(netcard->devName());
    }
    _releaseRemoteDisplayPort(domain->getRemoteDisplayPort());

    return true;
}

bool PlatformDomain::_beforeLibvirtRestore(DomainI *domain)
{
    Q_UNUSED(domain)
    return true;
}

bool PlatformDomain::_afterLibvirtResume(DomainI *domain)
{
    Q_UNUSED(domain)
    return true;
}

bool PlatformDomain::_addDisks(DomainI *domain, QDomElement &xmlDevices)
{
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(domain->getEntry());
    if (domainEntry->disks.empty()) {
        qCritical() << tr("Can't define a domain with no disk");
        return false;
    }

    QString domainPath = domain->path();
    if (domainPath.isEmpty()) {
        qCritical() << tr("Could not get domain full path");
        return false;
    }

    QDomDocument xml = xmlDevices.ownerDocument();

    QList<Hns::Disk> diskList = QList<Hns::Disk>::fromStdList(domainEntry->disks);
    foreach (const Hns::Disk &disk, diskList) {
        QDomElement xmlDisk = xml.createElement("disk");
        xmlDisk.setAttribute("type", QString::fromStdString(disk.type));
        xmlDisk.setAttribute("device", QString::fromStdString(disk.device));
        xmlDevices.appendChild(xmlDisk);

        QDomElement diskSource = xml.createElement("source");
        QString filePath = Diamesh::absolutePath(QString("%1/%2").arg(domainPath).arg(QString::fromStdString(disk.source)));
        diskSource.setAttribute("file", filePath);
        xmlDisk.appendChild(diskSource);
        QDomElement diskTarget = xml.createElement("target");
        diskTarget.setAttribute("bus", QString::fromStdString(disk.targetBus));
        diskTarget.setAttribute("dev", QString::fromStdString(disk.target));
        xmlDisk.appendChild(diskTarget);

        if (disk.driverName != "") {
            QDomElement diskDriver = xml.createElement("driver");
            diskDriver.setAttribute("name", QString::fromStdString(disk.driverName));

            if (disk.driverType != "")
                diskDriver.setAttribute("type", QString::fromStdString(disk.driverType));

            xmlDisk.appendChild(diskDriver);
        }
    }

    return true;
}

QDomElement PlatformDomain::_addTextNode(QDomElement &elem, const QString &name, const QString &value)
{
    QDomDocument doc = elem.ownerDocument();
    QDomElement domElem = doc.createElement(name);
    QDomText domText = doc.createTextNode(value);
    domElem.appendChild(domText);
    elem.appendChild(domElem);
    return domElem;
}

int PlatformDomain::_nextRemoteDisplayPort()
{
    QMutexLocker locker(remoteDisplayPortPoolMutex());
    int nextPort = 0;
    QHashIterator<int, bool> it(*_remoteDisplayPortPool());
    while (it.hasNext()) {
        it.next();
        if (it.value() == false) {
            nextPort = it.key();
            break;
        }
    }
    if (nextPort != 0)
        _remoteDisplayPortPool()->insert(nextPort, true);
    return nextPort;
}

void PlatformDomain::_releaseRemoteDisplayPort(int port)
{
    QMutexLocker locker(remoteDisplayPortPoolMutex());
    _remoteDisplayPortPool()->insert(port, false);
}

QString PlatformDomain::_nextRemoteDisplayPassword()
{
    qsrand(QDateTime::currentDateTime().toMSecsSinceEpoch());
    QByteArray passByteArray;
    // Generate a 20 hexadecimal characters pass
    for (int i = 0 ; i < 10 ; ++i)
        passByteArray.append((char)qrand());

    QString pass(passByteArray.toHex());
    return QCryptographicHash::hash(pass.toUtf8(), QCryptographicHash::Sha1).toHex();
}

void PlatformDomain::_registerPlatformDomain(const QString &domainType)
{
    platformDomainHash()->insert(domainType, this);
}

Hns::SnapshotList PlatformDomain::_sortSnapshotListForChildrenFirst(Hns::SnapshotList snapList)
{
    Hns::SnapshotList retList;
    QMap<QString, QString>parentMap; /*<name, parent> ("" if no parent)*/
    QMap<QString, Hns::Snapshot>snapshotMap;/*<name, snapshot>*/
    foreach (Hns::Snapshot snapshot, snapList) {
        parentMap.insert(QString::fromStdString(snapshot.name), QString::fromStdString(snapshot.parent));
        snapshotMap.insert(QString::fromStdString(snapshot.name), snapshot);
    }
    while (!parentMap.keys().isEmpty()) {
        QMutableMapIterator<QString, QString>it(parentMap);
        while (it.hasNext()) {
            it.next();
            QString key = it.key();
            QString value = parentMap.value(key);
            if (!parentMap.values().contains(key) && !value.isEmpty()) { //you can add the element if it is not in the parent list
                retList.push_back(snapshotMap.value(key));
                it.remove();
            }
            if (value.isEmpty() && parentMap.keys().size() == 1) {
                retList.push_back(snapshotMap.value(key));
                it.remove();
            }
        }
    }

    return retList;
}

Hns::SnapshotList PlatformDomain::_sortSnapshotListForParentFirst(Hns::SnapshotList snapList)
{
    Hns::SnapshotList retList;
    QMap<QString, QString>parentMap; /*<name, parent> ("" if no parent)*/
    QMap<QString, Hns::Snapshot>snapshotMap;/*<name, snapshot>*/
    foreach (Hns::Snapshot snapshot, snapList) {
        parentMap.insert(QString::fromStdString(snapshot.name), QString::fromStdString(snapshot.parent));
        snapshotMap.insert(QString::fromStdString(snapshot.name), snapshot);
    }
    while (!parentMap.keys().isEmpty()) {
        QMutableMapIterator<QString, QString>it(parentMap);
        while (it.hasNext()) {
            it.next();
            QString key = it.key();
            QString value = parentMap.value(key);
            if (value.isEmpty()) { //if there is no parent
                retList.push_back(snapshotMap.value(key));
                it.remove();
            } else {
                if (_contains(retList, value)) { //if the parent has already been added
                    retList.push_back(snapshotMap.value(key));
                    it.remove();
                }
            }
        }
    }
    return retList;
}

bool HnsNode::PlatformDomain::_contains(Hns::SnapshotList snapList, QString name)
{
    /*Return true if there is a snapshot in @snapList with the name @name*/
    bool ret = false;
    foreach (Hns::Snapshot snapshot, snapList) {
        ret = ret | (QString::fromStdString(snapshot.name) == name);
    }
    return ret;
}

void PlatformDomain::_refreshSnapshotList(DomainI *domain)
{
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(domain->getEntry());
    QString entryAbsolutePath = Hns::NodeI::instance()->storageManager()->getFullPathFromMixed("entity",QString::fromStdString(domain->getEntry()->path), QString::fromStdString(domain->getEntry()->uuid));
    virConnectPtr connection = NULL;
    virDomainPtr dom = NULL;
    Hns::SnapshotList updatedList;
    try {
        connection = _openNewConnectionIfNeeded();
        dom = _getVirDomainForConnection(domain->_virDomain, connection);
        unsigned int numberSnapshots = virDomainSnapshotNum(dom, 0);
        char* snapNames[numberSnapshots];
        if (virDomainSnapshotListNames(dom, snapNames, numberSnapshots, 0) < 0)
            throw std::runtime_error(tr("PlatformDomain::_refreshSnapshotList libvirt virDomainSnapshotListName failed").toStdString());

        for (unsigned int i = 0; i < numberSnapshots; ++i){
            QString snapName(snapNames[i]);
            virDomainSnapshotPtr libvirtSnap = virDomainSnapshotLookupByName(dom, snapName.toUtf8(), 0);
            if (libvirtSnap == NULL)
                throw std::runtime_error(tr("PlatformDomain::_refreshSnapshotList libvirt virDomainSnapshotLookupByName failed").toStdString());

            QString snapshotXmlDesc = QString::fromUtf8(virDomainSnapshotGetXMLDesc(libvirtSnap, 0));
            if (snapshotXmlDesc.isEmpty())
                throw std::runtime_error(tr("PlatformDomain::_refreshSnapshotList libvirt virDomainSnapshotGetXMLDesc failed").toStdString());
            QDomDocument doc;
            doc.setContent(snapshotXmlDesc);
            QDomElement e = doc.firstChildElement("domainsnapshot");
            QString name = e.firstChildElement("name").text();
            QString parent = e.firstChildElement("parent").text();
            QString creationDateTime = e.firstChildElement("creationTime").text();
            QString description = e.firstChildElement("description").text();
            QDomNodeList disksElement = e.firstChildElement("disks").childNodes();
            QMap<std::string, std::string> disksMap; /*<name, file>*/
            for (int i = 0; i < disksElement.count(); ++i) {
                QString name = disksElement.item(i).toElement().attribute("name");
                QString file = disksElement.item(i).toElement().firstChildElement().attribute("file");
                if (file.contains(entryAbsolutePath)) {
                    file.remove(entryAbsolutePath); //Saving only relative path
                    if (file.startsWith(QString("/")))
                        file.replace(0, 1, QString());
                }
                if (!file.isEmpty())
                    disksMap.insert(name.toStdString(), file.toStdString());
            }

            QMap<std::string, std::string> parentDisksMap;
            QDomElement domainElement = e.firstChildElement("domain");
            QDomElement devicesElement = domainElement.firstChildElement("devices");
            QDomNodeList diskNodeList = devicesElement.elementsByTagName("disk");
            for (int j = 0, total = diskNodeList.length(); j < total; ++j) {
                QDomElement diskElement = diskNodeList.at(j).toElement();
                QDomElement sourceElement = diskElement.firstChildElement("source");
                QString file = sourceElement.attribute("file");
                if (file.contains(entryAbsolutePath)) {
                    file.remove(entryAbsolutePath); //Saving only relative path
                    if (file.startsWith(QString("/")))
                        file.replace(0, 1, QString());
                }
                QDomElement targetElement = diskElement.firstChildElement("target");
                QString name = targetElement.attribute("dev");
                parentDisksMap.insert(name.toStdString(), file.toStdString());
            }
            Hns::Snapshot snapshot;
            snapshot.name = name.toStdString();
            snapshot.description = description.toStdString();
            snapshot.creationDateTime = creationDateTime.toStdString();
            snapshot.parent = parent.toStdString();
            snapshot.childDisks = disksMap.toStdMap();
            snapshot.parentDisks = parentDisksMap.toStdMap();
            updatedList.push_back(snapshot);
        }

        _cleanupIfNeeded(connection, domain, dom);
    } catch (std::runtime_error &e) {
        qCritical() << e.what();
    }
    domainEntry->snapshots = updatedList;
    _cleanupIfNeeded(connection, domain, dom);

}

void PlatformDomain::_buildFactoryHelper(BuildChecker buildChecker)
{
    Hns::NodeI::instance()->entityFactory()->registerChecker<Hns::DomainEntry, DomainI>(buildChecker);
}

void PlatformDomain::_monitorCpu(DomainI *domain, Hns::DomainMonitoringDataPtr data)
{
    qint64 actualTimeStamp = QDateTime::currentMSecsSinceEpoch();
    virDomainInfo domainInfo;
    virDomainGetInfo(domain->virDomain(), &domainInfo);
    qulonglong actualCPUTime = domainInfo.cpuTime;
    qulonglong cpuTime = actualCPUTime - domain->_monitoringInfo.previousCPUTime;
    qreal perCentBase = (cpuTime * 100.0) / ((actualTimeStamp - domain->_monitoringInfo.monitoringTimeStamp) * 1000.0 * 1000.0);
    qreal pcentHostCpu = perCentBase / _hostNbCpu;
    qreal pcentGuestCpu = perCentBase / domainInfo.nrVirtCpu;

    data->cpuData.hostCPU = qMax(0.0, qMin(100.0, pcentHostCpu));
    data->cpuData.guestCPU = qMax(0.0, qMin(100.0, pcentGuestCpu));

    domain->_monitoringInfo.monitoringTimeStamp = actualTimeStamp;
    domain->_monitoringInfo.previousCPUTime = actualCPUTime;
}

void PlatformDomain::_monitorMemory(DomainI *domain, Hns::DomainMonitoringDataPtr data)
{
    virDomainInfo domainInfo;
    virDomainGetInfo(domain->virDomain(), &domainInfo);
    data->memoryData.totalSize = domainInfo.maxMem;
    data->memoryData.used = domainInfo.memory;
    data->memoryData.available = data->memoryData.totalSize - data->memoryData.used;
}

void PlatformDomain::_monitorNetwork(DomainI *domain, Hns::DomainMonitoringDataPtr data)
{
    foreach(HnsNode::Netcard *netcard, domain->netcards()) {
        virDomainInterfaceStatsStruct stats;
        virDomainInterfaceStats(domain->virDomain(), netcard->vethName().toLatin1().constData(), &stats, sizeof(stats));
        qlonglong rx = stats.rx_bytes;
        qlonglong tx = stats.tx_bytes;
        Hns::NetworkMonitoringData netData = data->networkDataMap[netcard->netcardEntry().source];
        netData.rRate = rx - netData.received;
        netData.tRate = tx - netData.transmitted;
        netData.received = rx;
        netData.transmitted = tx;
        data->networkDataMap[netcard->netcardEntry().source] = netData;
    }
}

void PlatformDomain::_monitorIO(DomainI *domain, Hns::DomainMonitoringDataPtr data, Hns::DomainEntryPtr domainEntry)
{
    QList<Hns::Disk> diskList = QList<Hns::Disk>::fromStdList(domainEntry->disks);
    foreach(const Hns::Disk &disk, diskList) {
        virDomainBlockStatsStruct stats;
        virDomainBlockStats(domain->virDomain(), disk.target.c_str(), &stats, sizeof(stats));
        Hns::IOMonitoringData diskData = data->ioDataMap[disk.target];
        qlonglong byteRead = stats.rd_bytes;
        qlonglong byteWrite = stats.wr_bytes;
        diskData.rRate = byteRead - diskData.rBytes;
        diskData.wRate = byteWrite - diskData.wBytes;
        diskData.rBytes = byteRead;
        diskData.wBytes = byteWrite;
        data->ioDataMap[disk.target] = diskData;
    }
}

bool PlatformDomain::canBuild(const Hns::EntityEntryPtr &entityEntry, const QList<Hns::EntityEntryPtr> &alreadyDefinedEntities)
{
    Q_UNUSED(alreadyDefinedEntities)
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(entityEntry);
    if (!domainEntry)
        return false;
    if (domainEntry->nodeKey != "" && domainEntry->nodeKey != Hns::NodeI::instance()->key().toStdString())
        return false;
    if (domainEntry->aminterface != -1 && !Hns::NodeI::instance()->actionManagerBridgeAvailable())
        return false;
    return true;
}

QString PlatformDomain::suspendPath(DomainI *domain)
{    
    QString path = QString("%1/%2").arg(domain->path()).arg(saveFileName);
    return Diamesh::absolutePath(path);
}

void PlatformDomain::initMonitoringInformation(DomainI *domain)
{
    Hns::DomainMonitoringDataPtr data = Hns::DomainMonitoringDataPtr::dynamicCast(domain->getMonitoringData());
    if (data == 0)
        return;

    if (domain->hasFlag(EntityI::CPUMonitoring)) {
        virDomainInfo domainInfo;
        virDomainGetInfo(domain->virDomain(), &domainInfo);

        domain->_monitoringInfo.monitoringTimeStamp = QDateTime::currentMSecsSinceEpoch();
        domain->_monitoringInfo.previousCPUTime = domainInfo.cpuTime;
        data->cpuData.hostCPU = 0;
        data->cpuData.guestCPU = 0;
    }
    if (domain->hasFlag(EntityI::MemoryMonitoring)) {
        data->memoryData.totalSize = 0;
        data->memoryData.used = 0;
        data->memoryData.available = 0;
    }
    if (domain->hasFlag(EntityI::NetworkMonitoring)) {
        foreach(HnsNode::Netcard *netcard, domain->netcards())
            data->networkDataMap[netcard->netcardEntry().source] = Hns::NetworkMonitoringData();
    }

    if (domain->hasFlag(EntityI::IOMonitoring)) {
        Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(domain->getEntry());
        QList<Hns::Disk> diskList = QList<Hns::Disk>::fromStdList(domainEntry->disks);
        foreach(const Hns::Disk &disk, diskList)
            data->ioDataMap[disk.target] = Hns::IOMonitoringData();
    }
}

bool PlatformDomain::updateMonitoringData(DomainI *domain)
{
    Hns::DomainMonitoringDataPtr data = Hns::DomainMonitoringDataPtr::dynamicCast(domain->getMonitoringData());
    if (data == 0)
        return false;

    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(domain->getEntry());

    int status = domainStatus(domain);
    if (status == Hns::STARTED) {
        if (domain->hasFlag(EntityI::CPUMonitoring))
            _monitorCpu(domain, data);
        if (domain->hasFlag(EntityI::MemoryMonitoring)) {
            _monitorMemory(domain, data);
        }
        if (domain->hasFlag(EntityI::NetworkMonitoring))
            _monitorNetwork(domain, data);
        if (domain->hasFlag(EntityI::IOMonitoring))
            _monitorIO(domain, data, domainEntry);
    } else {
        if (domain->hasFlag(EntityI::CPUMonitoring)) {
            data->cpuData.hostCPU = 0;
            data->cpuData.guestCPU = 0;
        }
        else {
            data->cpuData = Hns::CPUMonitoringData();
        }
        if (domain->hasFlag(EntityI::MemoryMonitoring)) {
            data->memoryData.totalSize = 0;
            data->memoryData.used = 0;
            data->memoryData.available = 0;
        }
        else {
            data->memoryData = Hns::MemoryMonitoringData();
        }
        if (domain->hasFlag(EntityI::NetworkMonitoring)) {
            foreach(HnsNode::Netcard *netcard, domain->netcards())
                data->networkDataMap[netcard->netcardEntry().source] = Hns::NetworkMonitoringData();
        }
        else {
            data->networkDataMap.clear();
        }
        if (domain->hasFlag(EntityI::IOMonitoring)) {
            QList<Hns::Disk> diskList = QList<Hns::Disk>::fromStdList(domainEntry->disks);
            foreach(const Hns::Disk &disk, diskList)
                data->ioDataMap[disk.target] = Hns::IOMonitoringData();
        }
        else {
            data->ioDataMap.clear();
        }
    }
    return true;
}

void PlatformDomain::setInterfaceUp(DomainI *domain, const QString &devName, bool up)
{
    if (!getFeatures(HnsNode::StateLink) || !domain) {
        return;
    }

    virDomainPtr virDom = NULL;

    try {
        virConnectPtr connection = _openNewConnectionIfNeeded();
        virDom = _getVirDomainForConnection(domain->_virDomain, connection);
    } catch (std::runtime_error &e) {
        qCritical() << e.what();
        return;
    }

    // VIR_DOMAIN_AFFECT_CONFIG always needs to be used, the changes need to be persistant
    int flags = VIR_DOMAIN_AFFECT_CONFIG;

    if (virDomainIsActive(virDom)) {
        // VIR_DOMAIN_AFFECT_LIVE can only be used if a domain is active
        flags |= VIR_DOMAIN_AFFECT_LIVE;
    }

    QString desc = virDomainGetXMLDesc(virDom, flags);
    QDomDocument document;
    document.setContent(desc);

    QDomNodeList interfaces = document.elementsByTagName("interface");
    QDomNode interface;
    QDomElement link;
    bool interfaceFound = false;
    bool linkFound = false;

    QString vethName;

    foreach (Netcard* netcard, domain->netcards()) {
        if (netcard->devName() == devName) {
            vethName = netcard->vethName();
            break;
        }
    }

    for (int i = 0, interfacesLength = interfaces.length() ; i < interfacesLength ; ++i) {
        QDomNodeList interfaceChildren = interfaces.at(i).childNodes();

        for (int j = 0, interfaceChildrenLength = interfaceChildren.length() ; j < interfaceChildrenLength ; ++j) {
            QDomNode child = interfaceChildren.at(j);

            if (child.nodeName() == "target"
                    && !child.attributes().isEmpty()
                    && child.attributes().item(0).toAttr().value() == vethName) {
                interface = interfaces.at(i);
                interfaceFound = true;
            }

            if (child.nodeName() == "link") {
                link = child.toElement();
                linkFound = true;
            }
        }

        if (interfaceFound && linkFound) {
            QString state(up ? "up" : "down");
            QString interfaceString;
            QTextStream stream(&interfaceString);

            link.setAttribute("state", state);
            interface.appendChild(link);
            interface.save(stream, 0);

            if (virDomainUpdateDeviceFlags(virDom, interfaceString.toStdString().c_str(), flags) < 0) {
                qWarning() << tr("Unable to set device %1 to state %2").arg(devName).arg(state);
            }

            break;
        }
    }
}

} // namespace HnsNode
