/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "platformdynamipsrouter.h"
#include "nodeI.h"
#include "nodeentityI.h"
#include "nodedynamipsrouterI.h"
#include "telnetwrapper.h"
#include "nodeconstants.h"
#include "netutils.h"

// hynesim common headers
#include "common/pluginmanager.h"

// hynesim server headers
#include "libhnsserver/storagemanager.h"
#include "libhnsserver/processrunner.h"

// Generated hynesim headers
#include "common/dynamipsrouterentry.h"

// diamesh headers
#include <diamesh/diamesh.h>
#include <diamesh/utils.h>

// Qt headers
#include <QSettings>
#include <QtDebug>
#include <QScopedPointer>
#include <QDirIterator>

// C++ std headers
#include <stdexcept>

namespace HnsNode {

/*******************************************************************************
* DynamipsPlugin
*/
DynamipsPlugin::DynamipsPlugin(QObject *parent)
    : Plugin(parent)
{
}

bool DynamipsPlugin::define(DynamipsRouterI *dynamipsRouterI)
{
    if (_preDefine(dynamipsRouterI)) {
        if (_define(dynamipsRouterI)) {
            return _postDefine(dynamipsRouterI);
        }
    }
    return false;
}

bool DynamipsPlugin::_preDefine(DynamipsRouterI *dynamipsRouter)
{
    Q_UNUSED(dynamipsRouter)
    return true;
}

bool DynamipsPlugin::_postDefine(DynamipsRouterI *dynamipsRouter)
{
    Q_UNUSED(dynamipsRouter)
    return true;
}

bool DynamipsPlugin::_define(DynamipsRouterI *dynamipsRouter)
{
    Hns::DynamipsRouterEntryPtr dynamipsRouterEntry = Hns::DynamipsRouterEntryPtr::dynamicCast(dynamipsRouter->getEntry());

    Hns::ModulePositionMap::iterator itorPositionSlot = dynamipsRouterEntry->modulesMap.begin();
    while (itorPositionSlot != dynamipsRouterEntry->modulesMap.end()) {
        for (int i = 0; i < itorPositionSlot->second.interfaces; i++ ) {
            _addVdeTapPlug(dynamipsRouter);
        }
        itorPositionSlot++;
    }

    QScopedPointer<TelnetWrapper> telnet(PlatformDynamipsRouter::instance()->getTelnet(dynamipsRouter));
    if (!telnet)
        return false;

    try {
        QString uuid = QString::fromStdString(dynamipsRouterEntry->uuid);
        QString routerName = QString::fromStdString(dynamipsRouterEntry->router);

        //before all, if there is a _rom file, and we want to use configuration, we must destroy this file
        if (!dynamipsRouterEntry->configuration.empty() && dynamipsRouterEntry->applyOnBoot) {
            _deleteRomFile(dynamipsRouter);
        }

        telnet->send(QString("vm create %1 %2 %3").arg(uuid).arg(dynamipsRouter->routerId()).arg(routerName));
        telnet->send(QString("vm set_ios %1 %2").arg(uuid).arg(dynamipsRouterEntry->imageDisk.c_str()));
        if (dynamipsRouterEntry->needSetChassis)
            telnet->send(QString("%1 set_chassis %2 %3").arg(routerName).arg(uuid).arg(dynamipsRouterEntry->chassis.c_str()));
        telnet->send(QString("vm set_con_tcp_port %1 %2").arg(uuid).arg(dynamipsRouter->_telnetPort));
        telnet->send(QString("vm set_ram %1 %2").arg(uuid).arg(dynamipsRouterEntry->ram));
        telnet->send(QString("vm set_nvram %1 %2").arg(uuid).arg(dynamipsRouterEntry->nvram));
        if (!dynamipsRouterEntry->macAddress.empty())
            telnet->send(QString("%1 set_mac_addr %2 %3").arg(routerName).arg(uuid).arg(dynamipsRouterEntry->macAddress.c_str()));

        QString tapName, nioName;
        int vdeTapPlugId = 0;
        itorPositionSlot = dynamipsRouterEntry->modulesMap.begin();

        while (itorPositionSlot != dynamipsRouterEntry->modulesMap.end()) {
            if (!itorPositionSlot->second.isMB)
                telnet->send(QString("vm slot_add_binding %1 %2 0 %3").arg(uuid).arg(itorPositionSlot->first).arg(itorPositionSlot->second.name.c_str()));

            for (int i = 0; i < itorPositionSlot->second.interfaces; i++ ) {
                QString strtapplug  = _getTapPlugName(dynamipsRouter, vdeTapPlugId);
                if (strtapplug  != NULL) {
                    tapName = QString("%1v").arg(strtapplug);
                    nioName = QString("nio_%1").arg(tapName);
                    telnet->send(QString("nio create_tap %1 %2").arg(nioName).arg(tapName));
                    telnet->send(QString("vm slot_add_nio_binding %1 %2 %3 %4").arg(uuid).arg(itorPositionSlot->first).arg(i).arg(nioName));
                    createBridge(strtapplug, HnsNode::Up, true);
                    addToBridge(strtapplug, tapName);
                    setInterfaceState(tapName, HnsNode::Up);
                }
                vdeTapPlugId++;
            }
            itorPositionSlot++;
        }

        if (!dynamipsRouterEntry->configuration.empty() && dynamipsRouterEntry->applyOnBoot) {
            //add a space before push (the first character is eaten)
            QString configuration = QString(" ")+QByteArray::fromBase64(dynamipsRouterEntry->configuration.c_str()).data();
            QByteArray data = QByteArray(configuration.toStdString().c_str());
            QByteArray data64 = data.toBase64();
            telnet->send(QString("vm push_config %1 %2").arg(uuid).arg(QString(data64.data())));
        }
    } catch(const DynamipsError & error) {
        qCritical("%s", error.what());
        return false;
    }
    return true;
}

bool DynamipsPlugin::undefine(DynamipsRouterI *dynamipsRouter)
{
    if (_preUndefine(dynamipsRouter)) {
        if (_undefine(dynamipsRouter)) {
            return _postUndefine(dynamipsRouter);
        }
    }
    return false;
}

bool DynamipsPlugin::_preUndefine(DynamipsRouterI *dynamipsRouter)
{
    Q_UNUSED(dynamipsRouter)
    return true;
}

bool DynamipsPlugin::_postUndefine(DynamipsRouterI *dynamipsRouter)
{
    Q_UNUSED(dynamipsRouter)
    return true;
}

bool DynamipsPlugin::_undefine(DynamipsRouterI *dynamipsRouter)
{
    Hns::DynamipsRouterEntryPtr dynamipsRouterEntry = Hns::DynamipsRouterEntryPtr::dynamicCast(dynamipsRouter->getEntry());
    QScopedPointer<TelnetWrapper> telnet(PlatformDynamipsRouter::instance()->getTelnet(dynamipsRouter));
    if(!telnet) return false;
    try{
        Hns::ModulePositionMap::iterator itorPositionSlot = dynamipsRouterEntry->modulesMap.begin();
        int vdeTapPlugId = 0;
        QString tapName, nioName;
        QString uuid = QString::fromStdString(dynamipsRouterEntry->uuid);
        while (itorPositionSlot != dynamipsRouterEntry->modulesMap.end()) {
            for (int i = 0; i < itorPositionSlot->second.interfaces; i++ ) {
                QString strtapplug  = dynamipsRouter->_vectVdeTapPlug.value(vdeTapPlugId);
                if (strtapplug  != NULL) {
                    tapName = QString("%1v").arg(strtapplug);
                    nioName = QString("nio_%1").arg(tapName);
                    setInterfaceState(tapName, HnsNode::Down);
                    removeBridge(strtapplug);
                    telnet->send(QString("vm slot_remove_nio_binding %1 %2 %3").arg(uuid).arg(itorPositionSlot->first).arg(i));
                    telnet->send(QString("nio delete %1").arg(nioName));
                }
                vdeTapPlugId++;
            }
            if (!itorPositionSlot->second.isMB)
                telnet->send(QString("vm slot_remove_binding %1 %2 0").arg(uuid).arg(itorPositionSlot->first));
            itorPositionSlot++;
        }
        telnet->send(QString("vm delete %1").arg(uuid));
        dynamipsRouter->_vectVdeTapPlug.clear();
    } catch(const DynamipsError & error){
        qCritical("%s", error.what());
        return false;
    }
    return true;
}

bool DynamipsPlugin::create(DynamipsRouterI *dynamipsRouter)
{
    if (_preCreate(dynamipsRouter)) {
        if (_create(dynamipsRouter)) {
            return _postCreate(dynamipsRouter);
        }
    }
    return false;
}

bool DynamipsPlugin::_preCreate(DynamipsRouterI *dynamipsRouter)
{
    Q_UNUSED(dynamipsRouter)
    return true;
}

bool DynamipsPlugin::_postCreate(DynamipsRouterI *dynamipsRouter)
{
    Q_UNUSED(dynamipsRouter)
    return true;
}

bool DynamipsPlugin::_create(DynamipsRouterI *dynamipsRouter)
{
    Hns::DynamipsRouterEntryPtr dynamipsRouterEntry = Hns::DynamipsRouterEntryPtr::dynamicCast(dynamipsRouter->getEntry());

    QScopedPointer<TelnetWrapper> telnet(PlatformDynamipsRouter::instance()->getTelnet(dynamipsRouter));
    if(!telnet) return false;
    try{
        QString uuid = QString::fromStdString(dynamipsRouterEntry->uuid);
        telnet->send(QString("vm start %1").arg(uuid));
        if(!dynamipsRouterEntry->idlePc.empty()){
            telnet->send(QString("vm set_idle_pc_online %1 0 %2").arg(uuid).arg(dynamipsRouterEntry->idlePc.c_str()));
        }
    } catch(const DynamipsError & error){
        qCritical("%s", error.what());
        return false;
    }
    return true;
}

bool DynamipsPlugin::destroy(DynamipsRouterI *dynamipsRouter)
{
    if (_preDestroy(dynamipsRouter)) {
        if (_destroy(dynamipsRouter)) {
            return _postDestroy(dynamipsRouter);
        }
    }
    return false;
}

bool DynamipsPlugin::_preDestroy(DynamipsRouterI *dynamipsRouter)
{
    Q_UNUSED(dynamipsRouter)
    return true;
}

bool DynamipsPlugin::_postDestroy(DynamipsRouterI *dynamipsRouter)
{
    Q_UNUSED(dynamipsRouter)
    return true;
}

bool DynamipsPlugin::_destroy(DynamipsRouterI *dynamipsRouter)
{
    Hns::DynamipsRouterEntryPtr dynamipsRouterEntry = Hns::DynamipsRouterEntryPtr::dynamicCast(dynamipsRouter->getEntry());
    QScopedPointer<TelnetWrapper> telnet(PlatformDynamipsRouter::instance()->getTelnet(dynamipsRouter));
    if(!telnet) return false;
    try{
        telnet->send(QString("vm stop %1").arg(dynamipsRouterEntry->uuid.c_str()));
    } catch(const DynamipsError & error){
        qCritical("%s", error.what());
        return false;
    }
    return true;
}

bool DynamipsPlugin::suspend(DynamipsRouterI *dynamipsRouter)
{
    if (_preSuspend(dynamipsRouter)) {
        if (_suspend(dynamipsRouter)) {
            return _postSuspend(dynamipsRouter);
        }
    }
    return false;
}

bool DynamipsPlugin::_preSuspend(DynamipsRouterI *dynamipsRouter)
{
    Q_UNUSED(dynamipsRouter)
    return true;
}

bool DynamipsPlugin::_postSuspend(DynamipsRouterI *dynamipsRouter)
{
    Q_UNUSED(dynamipsRouter)
    return true;
}

bool DynamipsPlugin::_suspend(DynamipsRouterI *dynamipsRouter)
{
    Hns::DynamipsRouterEntryPtr dynamipsRouterEntry = Hns::DynamipsRouterEntryPtr::dynamicCast(dynamipsRouter->getEntry());
    QScopedPointer<TelnetWrapper> telnet(PlatformDynamipsRouter::instance()->getTelnet(dynamipsRouter));
    if(!telnet) return false;
    try{
        telnet->send(QString("vm suspend %1").arg(dynamipsRouterEntry->uuid.c_str()));
    } catch(const DynamipsError & error){
        qCritical("%s", error.what());
        return false;
    }
    return true;
}

bool DynamipsPlugin::resume(DynamipsRouterI *dynamipsRouter)
{
    if (_preResume(dynamipsRouter)) {
        if (_resume(dynamipsRouter)) {
            return _postResume(dynamipsRouter);
        }
    }
    return false;
}

bool DynamipsPlugin::_preResume(DynamipsRouterI *dynamipsRouter)
{
    Q_UNUSED(dynamipsRouter)
    return true;
}

bool DynamipsPlugin::_postResume(DynamipsRouterI *dynamipsRouter)
{
    Q_UNUSED(dynamipsRouter)
    return true;
}

bool DynamipsPlugin::_resume(DynamipsRouterI *dynamipsRouter)
{
    Hns::DynamipsRouterEntryPtr dynamipsRouterEntry = Hns::DynamipsRouterEntryPtr::dynamicCast(dynamipsRouter->getEntry());
    QScopedPointer<TelnetWrapper> telnet(PlatformDynamipsRouter::instance()->getTelnet(dynamipsRouter));
    if(!telnet) return false;
    try{
        telnet->send(QString("vm resume %1").arg(dynamipsRouterEntry->uuid.c_str()));
    } catch(const DynamipsError & error){
        qCritical("%s", error.what());
        return false;
    }
    return true;
}

void DynamipsPlugin::_addVdeTapPlug(DynamipsRouterI *dynamipsRouter)
{
    dynamipsRouter->_vectVdeTapPlug.push_back(generateHnsDevice());
}

QString DynamipsPlugin::_getTapPlugName(DynamipsRouterI *dynamipsRouter, int plugId)
{
    return dynamipsRouter->_vectVdeTapPlug.value(plugId);
}

void DynamipsPlugin::_clearVdeTapPlug(DynamipsRouterI *dynamipsRouter)
{
    dynamipsRouter->_vectVdeTapPlug.clear();
}

void DynamipsPlugin::_deleteRomFile(DynamipsRouterI *dynamipsRouter)
{
    Hns::DynamipsRouterEntryPtr dynamipsRouterEntry = Hns::DynamipsRouterEntryPtr::dynamicCast(dynamipsRouter->getEntry());
    HnsServer::StorageManager *storageManager = Hns::NodeI::instance()->storageManager();
    QString fullpath = storageManager->getFullPathFromMixed("entity",QString::fromStdString(dynamipsRouterEntry->path), QString::fromStdString(dynamipsRouterEntry->uuid));
    QStringList  listFilter;
    listFilter << QString("*%1*_rom").arg(QString::fromStdString(dynamipsRouterEntry->uuid));
    QDirIterator dirIterator(fullpath, listFilter ,QDir::Files | QDir::NoSymLinks);
    QString program = "rm";
    QStringList arguments;

    while (dirIterator.hasNext()) {
        dirIterator.next();
        arguments.clear();
        arguments << dirIterator.fileInfo().absoluteFilePath();
        HnsServer::ProcessRunner process;
        process.runSynchronous(program, arguments, -1);
    }
}

/*******************************************************************************
* PlatformDynamipsRouter
*/
static RegisterPlatform<PlatformDynamipsRouter> registerMe(Constants::SETTINGS_DYNAMIPS_PLATFORM_GROUP);

PlatformDynamipsRouter *PlatformDynamipsRouter::_instance = 0;

PlatformDynamipsRouter::PlatformDynamipsRouter()
    : Platform()
    , _nextRouterId(0)
{
    QSettings settings(Diamesh::instance()->configFileName(), QSettings::IniFormat);
    settings.beginGroup(Constants::SETTINGS_PLATFORMS_GROUP);
    settings.beginGroup(Constants::SETTINGS_DYNAMIPS_PLATFORM_GROUP);

    if (!settings.contains("port")) {
        throw std::runtime_error("No port found for dynamips");
    }
    _portDynamips = settings.value("port").toInt();
    settings.endGroup();

    settings.beginGroup("Ice");
    QString nameHost = "localhost";
    if (settings.contains("Ice.Default.Host")) {
        nameHost = settings.value("Ice.Default.Host").toString();
    }

    _dynamipsPluginList = Hns::NodeI::instance()->pluginManager()->pluginListByType<DynamipsPlugin>();

    TelnetWrapper * telnet = new TelnetWrapper(this);

    if (!telnet->connectToHost(nameHost, _portDynamips)) {
        delete telnet;
        throw std::runtime_error("Failed to load dynamips");
    }
    delete telnet;
    Hns::NodeI::instance()->entityFactory()->registerChecker<Hns::DynamipsRouterEntry, DynamipsRouterI>(&PlatformDynamipsRouter::canBuild);
    _buildFactoryHelper(&PlatformDynamipsRouter::canBuild);
}

PlatformDynamipsRouter::~PlatformDynamipsRouter()
{
}

Platform *PlatformDynamipsRouter::create()
{
    if (!_instance) {
        try {
            _instance = new PlatformDynamipsRouter();
            return _instance;
        } catch (const std::runtime_error & error) {
            qCritical() << error.what();
            qCritical() << tr("Failed to create DynamipsRouter platform");
        }
    }
    return NULL;
}

PlatformDynamipsRouter *PlatformDynamipsRouter::instance()
{
    return _instance;
}

bool PlatformDynamipsRouter::canBuild(const::Hns::EntityEntryPtr &entityEntry, const QList<Hns::EntityEntryPtr> &alreadyDefinedEntities)
{
    Q_UNUSED(alreadyDefinedEntities)
    Hns::DynamipsRouterEntryPtr dynamipsRouterEntry = Hns::DynamipsRouterEntryPtr::dynamicCast(entityEntry);
    if (dynamipsRouterEntry) {
        foreach(DynamipsPlugin * plugin, PlatformDynamipsRouter::instance()->_dynamipsPluginList) {
            if (plugin->canBuild(entityEntry))
                return true;
        }
    }
    return false;
}

bool PlatformDynamipsRouter::defineEntity(EntityI *entity)
{
    QWriteLocker locker(&_lock);

    DynamipsRouterI *dynamipsRouterI = dynamic_cast<DynamipsRouterI *>(entity);
    Hns::DynamipsRouterEntryPtr dynamipsrouter = Hns::DynamipsRouterEntryPtr::dynamicCast(dynamipsRouterI->getEntry());

    if (!dynamipsRouterI->plugin()) {
        foreach(DynamipsPlugin * plugin, _dynamipsPluginList) {
            if (plugin->canBuild(dynamipsrouter))
                dynamipsRouterI->setPlugin(plugin);
        }
        if (!dynamipsRouterI->plugin()) {
            qCritical() << QObject::tr("Can't find a plugin for router %1").arg(dynamipsrouter->shortName.c_str());
            return false;
        }
    }

    int id = ++_nextRouterId;
    dynamipsRouterI->_routerId = id;
    dynamipsRouterI->_telnetPort = _portDynamips + id;
    QMutexLocker telnetLocker(&_telnetLocker);
    return dynamipsRouterI->plugin()->define(dynamipsRouterI);
}

bool PlatformDynamipsRouter::undefineEntity(EntityI *entity)
{
    QWriteLocker locker(&_lock);
    DynamipsRouterI *dynamipsRouterI = dynamic_cast<DynamipsRouterI *>(entity);
    if (!dynamipsRouterI->plugin())
        return false;
    QMutexLocker telnetLocker(&_telnetLocker);
    return dynamipsRouterI->plugin()->undefine(dynamipsRouterI);
}

bool PlatformDynamipsRouter::createEntity(EntityI *entity)
{
    DynamipsRouterI *dynamipsRouterI= dynamic_cast<DynamipsRouterI *>(entity);
    if (!dynamipsRouterI->plugin())
        return false;
    QMutexLocker telnetLocker(&_telnetLocker);
    return dynamipsRouterI->plugin()->create(dynamipsRouterI);
}

bool PlatformDynamipsRouter::destroyEntity(EntityI *entity)
{
    DynamipsRouterI *dynamipsRouterI= dynamic_cast<DynamipsRouterI *>(entity);
    if (!dynamipsRouterI->plugin())
        return false;
    QMutexLocker telnetLocker(&_telnetLocker);
    return dynamipsRouterI->plugin()->destroy(dynamipsRouterI);
}

bool PlatformDynamipsRouter::suspendEntity(EntityI *entity)
{
    DynamipsRouterI *dynamipsRouterI= dynamic_cast<DynamipsRouterI *>(entity);
    if (!dynamipsRouterI->plugin())
        return false;
    QMutexLocker telnetLocker(&_telnetLocker);
    return dynamipsRouterI->plugin()->suspend(dynamipsRouterI);
}

bool PlatformDynamipsRouter::resumeEntity(EntityI *entity)
{
    DynamipsRouterI *dynamipsRouterI= dynamic_cast<DynamipsRouterI *>(entity);
    if (!dynamipsRouterI->plugin())
        return false;
    QMutexLocker telnetLocker(&_telnetLocker);
    return dynamipsRouterI->plugin()->resume(dynamipsRouterI);
}

std::string PlatformDynamipsRouter::generateIdleEntity(EntityI *entity)
{
    Hns::DynamipsRouterEntryPtr dynamipsRouterEntry = Hns::DynamipsRouterEntryPtr::dynamicCast(entity->getEntry());
    DynamipsRouterI *dynamipsRouterI= dynamic_cast<DynamipsRouterI *>(entity);
    QMutexLocker telnetLocker(&_telnetLocker);
    QScopedPointer<TelnetWrapper> telnet(getTelnet(dynamipsRouterI));
    if(!telnet) return "";

    QString bestValue ="";
    try{
        telnet->send(QString("vm set_idle_pc_online %1 0 0").arg(dynamipsRouterEntry->uuid.c_str())); //RAZ
        QByteArray qByteArray = telnet->send(QString("vm get_idle_pc_prop %1 0").arg(dynamipsRouterEntry->uuid.c_str()), true);
        QString strData = QString(qByteArray.data());
        QStringList stringList = strData.split("\n");
        QRegExp regExpHexa("(0x[0-9a-f]{8})");
        QMap<int , QString> mapValuePossible;
        foreach(const QString &value, stringList ){
            if(value.contains(regExpHexa)){
                QString strPriority = value;
                strPriority =  strPriority.mid(strPriority.indexOf("[")+1,2);
                mapValuePossible[qAbs(strPriority.toInt()-55)] = regExpHexa.cap(); //best value between 50-60 -> near 55
            }
        }
        if(! mapValuePossible.empty()){
            bestValue = mapValuePossible.begin().value();
            telnet->send(QString("vm set_idle_pc_online %1 0 %2").arg(dynamipsRouterEntry->uuid.c_str()).arg(bestValue));
        }
    } catch(const DynamipsError & error){
        qCritical("%s", error.what());
        return "";
    }
    return bestValue.toStdString();
}

std::string PlatformDynamipsRouter::extractConfigurationEntity(EntityI *entity)
{
    Hns::DynamipsRouterEntryPtr dynamipsRouterEntry = Hns::DynamipsRouterEntryPtr::dynamicCast(entity->getEntry());
    DynamipsRouterI *dynamipsRouterI= dynamic_cast<DynamipsRouterI *>(entity);
    QMutexLocker telnetLocker(&_telnetLocker);
    QScopedPointer<TelnetWrapper> telnet(getTelnet(dynamipsRouterI));
    if(!telnet) return "";

    QString configuration ="";
    try{
        QByteArray qByteArray = telnet->send(QString("vm extract_config %1").arg(dynamipsRouterEntry->uuid.c_str()));
        QString strData = QString(qByteArray.data()).split(' ').at(2).trimmed();
        configuration = strData.toAscii().data();
    } catch(const DynamipsError & error){
        qCritical("%s", error.what());
        return "";
    }
    return configuration.toStdString();
}

bool PlatformDynamipsRouter::pushConfigurationEntity(EntityI *entity, const QString &qConfiguration)
{
    Hns::DynamipsRouterEntryPtr dynamipsRouterEntry = Hns::DynamipsRouterEntryPtr::dynamicCast(entity->getEntry());
    DynamipsRouterI *dynamipsRouterI= dynamic_cast<DynamipsRouterI *>(entity);
    QMutexLocker telnetLocker(&_telnetLocker);
    QScopedPointer<TelnetWrapper> telnet(getTelnet(dynamipsRouterI));
    if(!telnet) return false;

    try{
        //add a space before push (the first character is eaten)
        QString configuration =QString(" ")+QByteArray::fromBase64(qConfiguration.toAscii()).data();
        QByteArray data = QByteArray(configuration.toStdString().c_str());
        QByteArray data64 = data.toBase64();
        telnet->send(QString("vm push_config %1 %2").arg(dynamipsRouterEntry->uuid.c_str()).arg(QString(data64.data())));
    } catch(const DynamipsError &error){
        qCritical("%s", error.what());
        return false;
    }
    return true;
}

TelnetWrapper *PlatformDynamipsRouter::getTelnet(DynamipsRouterI *dynamipsRouterI)
{
    TelnetWrapper *telnet = new TelnetWrapper();
    try {
        QString host = QString::fromStdString(Hns::NodeI::instance()->getHost());
        telnet->connectToHost(host, _portDynamips);
        telnet->send(QString("hypervisor working_dir %1").arg(Diamesh::absolutePath(dynamipsRouterI->path())));
    } catch(const DynamipsError &error) {
        qCritical("%s", error.what());
        delete telnet;
        return NULL;
    }
    return telnet;
}

void PlatformDynamipsRouter::_buildFactoryHelper(BuildChecker buildChecker)
{
    Hns::NodeI::instance()->entityFactory()->registerChecker<Hns::DynamipsRouterEntry, DynamipsRouterI>(buildChecker);
}

} // namespace HnsNode
