/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "platformhybridnetcard.h"
#include "nodeentityfactoryI.h"
#include "nodeconstants.h"
#include "nodeI.h"
#include "nodehybridnetcardI.h"
#include "netutils.h"

// Generated hynesim headers
#include "common/entityentry.h"
#include "common/hybridnetcardentry.h"

// diamesh headers
#include <diamesh/diamesh.h>

// Qt headers
#include <QSettings>
#include <QDebug>

// C++ std headers
#include <stdexcept>

namespace HnsNode {

static RegisterPlatform<PlatformHybridNetcard> registerMe(Constants::SETTINGS_HYBRID_NETCARD_PLATFORM_GROUP);

PlatformHybridNetcard *PlatformHybridNetcard::_instance = 0;

PlatformHybridNetcard::PlatformHybridNetcard()
    : Platform()
{
    if (!_loadHybridNetcards())
        throw std::runtime_error("Failed to load hybrid netcards"); // TODO: Translation
    Hns::NodeI::instance()->entityFactory()->registerChecker<Hns::HybridNetcardEntry, HybridNetcardI>(&PlatformHybridNetcard::canBuild);
}

PlatformHybridNetcard::~PlatformHybridNetcard()
{
}

Platform *PlatformHybridNetcard::create()
{
    if (!_instance) {
        try {
            _instance = new PlatformHybridNetcard();
            return _instance;
        } catch (const std::runtime_error &ex) {
            Q_UNUSED(ex)
            qCritical() << tr("Failed to create HybridNetcard platform");
        }
    }
    return NULL;
}

PlatformHybridNetcard *PlatformHybridNetcard::instance()
{
    return _instance;
}

bool PlatformHybridNetcard::canBuild(const::Hns::EntityEntryPtr &entityEntry, const QList<Hns::EntityEntryPtr> &alreadyDefinedEntities)
{
    Hns::HybridNetcardEntryPtr hybridNetcardEntry = Hns::HybridNetcardEntryPtr::dynamicCast(entityEntry);
    if (hybridNetcardEntry) {
        if (Hns::NodeI::instance()->key() != QString::fromStdString(hybridNetcardEntry->nodeName))
            return false;
    } else {
        return false;
    }

    if (alreadyDefinedEntities.isEmpty()) {
        PlatformHybridNetcard *platform = PlatformHybridNetcard::instance();
        QString plugIndex(QString::fromStdString(hybridNetcardEntry->plugIndex));
        QString devName(platform->_hybridNetcards.value(plugIndex));

        if (devName.isEmpty())
            return false;

        QReadLocker locker(&platform->_lock);
        if (platform->_hybridNetcardsInUse.contains(plugIndex))
            return false;

        return true;
    } else {
        /*If the list is not empty, we are in the case of a call by the load balancer*/
        foreach (Hns::EntityEntryPtr definedEntry, alreadyDefinedEntities) {
            Hns::HybridNetcardEntryPtr hybridNetcardDefinedEntry = Hns::HybridNetcardEntryPtr::dynamicCast(definedEntry);
            if(!hybridNetcardEntry || !hybridNetcardDefinedEntry)
                continue;
            if (!instance()->getHybridNetCards().keys().contains(QString::fromStdString(hybridNetcardEntry->plugIndex)))/*If the node doesn't contain this plug index*/
                return false;
            if (instance()->getHybridNetCards().value(QString::fromStdString(hybridNetcardEntry->plugIndex)) == instance()->getHybridNetCards().value(QString::fromStdString(hybridNetcardDefinedEntry->plugIndex)))
                /*If there is already an hybrid netcard with the same device*/
                return false;
        }
        return true;
    }
    return false;
}

bool PlatformHybridNetcard::defineEntity(EntityI *entity)
{
    QWriteLocker locker(&_lock);

    Hns::HybridNetcardEntryPtr hybridNetcardEntry = Hns::HybridNetcardEntryPtr::dynamicCast(entity->getEntry());
    QString plugIndex = QString::fromStdString(hybridNetcardEntry->plugIndex);

    QString devName = _hybridNetcards.value(plugIndex);
    if (devName.isEmpty()) {
        qCritical() << tr("There is no hybrid netcard with this plug index: %1").arg(plugIndex);
        return false;
    }

    if (_hybridNetcardsInUse.contains(plugIndex)) {
        qCritical() << tr("There is already a hybrid netcard with this plug index: %1").arg(plugIndex);
        return false;
    }

    QString bridgeName = devName + "b";

    if (!createBridge(bridgeName, HnsNode::Down, true)) {
        qCritical() << tr("Can't create bridge %1").arg(bridgeName);
        return false;
    }

    if (!addToBridge(bridgeName, devName)) {
        qCritical() << tr("Can't add device %1 to bridge %2").arg(devName).arg(bridgeName);
        return false;
    }

    HybridNetcardI *hybridNetcard = dynamic_cast<HybridNetcardI *>(entity);
    hybridNetcard->_bridgeName = bridgeName;
    hybridNetcard->_interfaceName = devName;

    _hybridNetcardsInUse.append(plugIndex);

    if (!setInterfaceState(devName, HnsNode::Up))
        return false;

    Hns::NodeI::instance()->emitSignalRefresh(hybridNetcard->metaObject()->className());
    return true;
}

bool PlatformHybridNetcard::undefineEntity(EntityI *entity)
{
    QWriteLocker locker(&_lock);

    Hns::HybridNetcardEntryPtr hybridNetcardEntry = Hns::HybridNetcardEntryPtr::dynamicCast(entity->getEntry());
    QString plugIndex = QString::fromStdString(hybridNetcardEntry->plugIndex);
    HybridNetcardI *hybridNetcard = dynamic_cast<HybridNetcardI *>(entity);
    QString bridgeName(hybridNetcard->_bridgeName);

    _hybridNetcardsInUse.removeAll(plugIndex);

    if (!bridgeName.isEmpty())
        removeBridge(bridgeName);

    Hns::NodeI::instance()->emitSignalRefresh(hybridNetcard->metaObject()->className());
    return true;
}

bool PlatformHybridNetcard::createEntity(EntityI *entity)
{
    Hns::HybridNetcardEntryPtr hybridNetcardEntry = Hns::HybridNetcardEntryPtr::dynamicCast(entity->getEntry());
    QString plugIndex = QString::fromStdString(hybridNetcardEntry->plugIndex);
    QString devName = _hybridNetcards.value(plugIndex);
    HybridNetcardI *hybridNetcard = dynamic_cast<HybridNetcardI *>(entity);
    QString bridgeName(hybridNetcard->_bridgeName);

    if (!setInterfaceState(bridgeName, HnsNode::Up))
        return false;

    if (!setInterfaceState(devName, HnsNode::Up))
        return false;

    return true;
}

bool PlatformHybridNetcard::destroyEntity(EntityI *entity)
{
    Hns::HybridNetcardEntryPtr hybridNetcardEntry = Hns::HybridNetcardEntryPtr::dynamicCast(entity->getEntry());
    QString plugIndex = QString::fromStdString(hybridNetcardEntry->plugIndex);
    QString devName = _hybridNetcards.value(plugIndex);
    HybridNetcardI *hybridNetcard = dynamic_cast<HybridNetcardI *>(entity);
    QString bridgeName(hybridNetcard->_bridgeName);

    if (!setInterfaceState(bridgeName, HnsNode::Down))
        return false;

    if (!setInterfaceState(devName, HnsNode::Down))
        return false;

    return true;
}

QHash<QString, QString> PlatformHybridNetcard::getHybridNetCards()
{
    return _hybridNetcards;
}

bool PlatformHybridNetcard::_loadHybridNetcards()
{
    QSettings settings(Diamesh::instance()->configFileName(), QSettings::IniFormat, this);
    settings.beginGroup(Constants::SETTINGS_PLATFORMS_GROUP);
    settings.beginGroup(Constants::SETTINGS_HYBRID_NETCARD_PLATFORM_GROUP);

    int nbNodes = settings.beginReadArray("cards");
    for (int i = 0; i < nbNodes; ++i) {
        settings.setArrayIndex(i);

        QString hybridNetcardDevice = settings.value("device").toString();
        if (hybridNetcardDevice.isEmpty()) {
            qCritical() << tr("The hybrid netcard device path is empty.");
            return false;
        }
        QString hybridNetcardPlug = settings.value("plug").toString();
        if (hybridNetcardPlug.isEmpty()) {
            qCritical() << tr("The hybrid netcard plug is empty.");
            return false;
        }

        if (!setInterfaceState(hybridNetcardDevice, HnsNode::Down)) {
            qCritical() << tr("Hybrid interface %1 doesn't seem to exist").arg(hybridNetcardDevice);
            return false;
        }

        _hybridNetcards.insert(hybridNetcardPlug, hybridNetcardDevice);
    }
    settings.endArray();
    settings.endGroup();
    return true;
}

} // namespace HnsNode
