/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "platformkvmqemu.h"
#include "nodeI.h"
#include "nodedomainI.h"
#include "netcard.h"
#include "netutils.h"
#include "nodeconstants.h"

// Generated hynesim headers
#include "common/domainentry.h"
#include "common/domainentryproperties.h"

// hynesim common headers
#include "common/hns.h"

// diamesh headers
#include <diamesh/diamesh.h>
#include <diamesh/utils.h>

// Qt headers
#include <QDebug>
#include <QDomDocument>
#include <QRegExp>
#include <QFile>
#include <QStringList>
#include <QTextCodec>
#include <QSettings>
#include <QSslCertificate>

namespace HnsNode {

static RegisterPlatform<PlatformKvmQemu> registerMeKvmQemu("KvmQemu");

PlatformKvmQemu *PlatformKvmQemu::_instance = 0;

PlatformKvmQemu::PlatformKvmQemu()
    : PlatformDomain(QStringList("kvm") << ("qemu"), "qemu:///system")
{
    _buildFactoryHelper(&PlatformKvmQemu::canBuild);
    _features |= HnsNode::Snapshot;

    QSettings settings(Diamesh::instance()->configFileName(), QSettings::IniFormat);
    settings.beginGroup(Constants::SETTINGS_PLATFORMS_GROUP);
    bool handleStateLink = settings.value(HnsNode::Constants::SETTINGS_HANDLE_STATE_LINK, false).toBool();
    settings.endGroup();
    if (handleStateLink) {
        qDebug() << tr("Enable State Link feature for Platform KVM/QEMU");
        _features |= HnsNode::StateLink;
    }
}

PlatformKvmQemu::~PlatformKvmQemu()
{
}

Platform *PlatformKvmQemu::create()
{
    if (!_instance) {
        try {
            _instance = new PlatformKvmQemu();
            return _instance;
        } catch (const LibvirtConnectException &ex) {
            qCritical() << tr("Unable to create KVM/QEMU platform");
        }
    }
    return NULL;
}

PlatformKvmQemu *PlatformKvmQemu::instance()
{
    return _instance;
}

bool PlatformKvmQemu::_beforeLibvirtDefine(QDomDocument &xml, DomainI *domain)
{
    if (!PlatformDomain::_beforeLibvirtDefine(xml, domain))
        return false;
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(domain->getEntry());
    if (!domainEntry){
        qCritical()<<("PlatformKvmQemu::_beforeLibvirtDefine: NULL domainEntry");
        return false;
    }
    QDomElement xmlDomain = xml.firstChildElement("domain");

    QDomElement xmlOs = xmlDomain.firstChildElement("os");
    QDomElement xmlOsType = _addTextNode(xmlOs, "type", "hvm");
    const Hns::OperatingSystem &os = domainEntry->os;

    if (!os.arch.empty())
        xmlOsType.setAttribute("arch", QString::fromStdString(os.arch));
    if (!os.machine.empty())
        xmlOsType.setAttribute("machine", QString::fromStdString(os.machine));

    QString absDomainPath = Diamesh::absolutePath(domain->path());
    if (!os.kernel.empty())
        _addTextNode(xmlOs, "kernel", QString("%1/%2").arg(absDomainPath).arg(QString::fromStdString(os.kernel)));
    if (!os.initrd.empty())
        _addTextNode(xmlOs, "initrd", QString("%1/%2").arg(absDomainPath).arg(QString::fromStdString(os.initrd)));
    if (!os.cmdline.empty())
        _addTextNode(xmlOs, "cmdline", QString::fromStdString(domainEntry->os.cmdline));

    QDomElement xmlDevices = xmlDomain.firstChildElement("devices");

    QList<Hns::Input> inputs = QList<Hns::Input>::fromStdList(domainEntry->inputs);
    // Add a USB tablet by default
    if (inputs.isEmpty()) {
        Hns::Input tabletInput;
        tabletInput.type = "tablet";
        tabletInput.bus = "usb";
        inputs.append(tabletInput);
    }

    foreach (const Hns::Input &input, inputs) {
        QDomElement xmlInput = xml.createElement("input");
        xmlInput.setAttribute("type", QString::fromStdString(input.type));
        xmlInput.setAttribute("bus", QString::fromStdString(input.bus));
        xmlDevices.appendChild(xmlInput);
    }

    // hard disks
    if (!_addDisks(domain, xmlDevices))
        return false;

    // shared folder
    if (domainEntry->sharedFolder) {
        QString sharedFolder = Hns::NodeI::instance()->sharedFolder();
        if (sharedFolder.isEmpty()) {
            qCritical() << tr("No shared folder has been defined in the node configuration");
            return false;
        }

        QDomElement xmlFilesystem = xml.createElement("filesystem");
        xmlFilesystem.setAttribute("type", "mount");
        xmlFilesystem.setAttribute("access", "passthrough");
        xmlDevices.appendChild(xmlFilesystem);

        QDomElement filesystemSource = xml.createElement("source");
        filesystemSource.setAttribute("dir", sharedFolder);
        xmlFilesystem.appendChild(filesystemSource);
        QDomElement filesystemTarget = xml.createElement("target");
        filesystemTarget.setAttribute("dir", "hynesim-shared");
        xmlFilesystem.appendChild(filesystemTarget);
    }

    // Network configuration
    int actionManagerInterface = domainEntry->aminterface;
    bool networkInterfacesCreated = true;
    QStringList networkInterfaces;
    foreach (Netcard *netcard, domain->netcards()) {
        Hns::Netcard netcardEntry = netcard->netcardEntry();
        bool isActionManagerInterface = (actionManagerInterface == netcardEntry.id);

        QDomElement devIf = xml.createElement("interface");
        devIf.setAttribute("type", !isActionManagerInterface ? "ethernet" : "bridge");

        QDomElement ifMac = xml.createElement("mac");
        ifMac.setAttribute("address", netcard->macAddress());
        devIf.appendChild(ifMac);

        if (!isActionManagerInterface) {
            if (getFeatures(HnsNode::StateLink)) {
                QDomElement ifState = xml.createElement("link");
                ifState.setAttribute("state", "down");
                devIf.appendChild(ifState);
            }

            QDomElement ifScript = xml.createElement("script");
            ifScript.setAttribute("path", "no");
            devIf.appendChild(ifScript);
        } else {
            QDomElement ifSource = xml.createElement("source");
            ifSource.setAttribute("bridge", Hns::NodeI::instance()->actionManagerBridge());
            devIf.appendChild(ifSource);

            QDomElement filterRef = xml.createElement("filterref");
            filterRef.setAttribute("filter", Constants::ACTION_MANAGER_NWFILTER_NAME);
            devIf.appendChild(filterRef);
        }

        QDomElement ifTarget = xml.createElement("target");
        ifTarget.setAttribute("dev", netcard->vethName());
        devIf.appendChild(ifTarget);

        QString modelType = QString::fromStdString(netcard->netcardEntry().model);
        if (!modelType.isEmpty()) {
            QDomElement ifModel = xml.createElement("model");
            ifModel.setAttribute("type", modelType);
            devIf.appendChild(ifModel);
        }

        xmlDevices.appendChild(devIf);

        // libvirt creates the AM TAP interface and adds it to the AM bridge
        if (!isActionManagerInterface) {
            networkInterfacesCreated &= createTap(netcard->vethName());
            if (networkInterfacesCreated) {
                if (addToBridge(netcard->devName(), netcard->vethName()))
                    networkInterfaces << netcard->vethName();
                else
                    networkInterfacesCreated = false;
            }
        }
        if (!networkInterfacesCreated)
            break;
    }
    if (!networkInterfacesCreated) {
        _cleanNetworkInterfaces(networkInterfaces);
        return false;
    }

    // Video configuration
    if (!QString::fromStdString(domainEntry->vid.videoModelType).isEmpty()) {
        QDomElement xmlVideo = xml.createElement("video");
        QDomElement xmlModel = xml.createElement("model");
        xmlModel.setAttribute("type", QString::fromStdString(domainEntry->vid.videoModelType));
        xmlVideo.appendChild(xmlModel);
        xmlDevices.appendChild(xmlVideo);
    }

    // Usb redirection configuration
    for (int i=0 ; i < domainEntry->usbRedirectionCount ; i++) {
        QDomElement xmlUsb = xml.createElement("redirdev");
        xmlUsb.setAttribute("bus", "usb");
        xmlUsb.setAttribute("type", "spicevmc");
        xmlDevices.appendChild(xmlUsb);
    }

    // USB controller configuration. Inspired by https://www.spice-space.org/usbredir.html
    if (domainEntry->usbControllerType == ::Hns::USB_CONTROLLER_USB1) {
        QDomElement usbController = xml.createElement("controller");
        usbController.setAttribute("type", "usb");
        xmlDevices.appendChild(usbController);
    } else if (domainEntry->usbControllerType == ::Hns::USB_CONTROLLER_USB2) {
        QDomElement ehciUsbController = xml.createElement("controller");
        ehciUsbController.setAttribute("type", "usb");

        QDomElement uhci1UsbController = ehciUsbController.cloneNode().toElement();
        QDomElement uhci2UsbController = ehciUsbController.cloneNode().toElement();
        QDomElement uhci3UsbController = ehciUsbController.cloneNode().toElement();

        QDomElement uhci1UsbMaster = xml.createElement("master");
        QDomElement uhci2UsbMaster = uhci1UsbMaster.cloneNode().toElement();
        QDomElement uhci3UsbMaster = uhci1UsbMaster.cloneNode().toElement();

        ehciUsbController.setAttribute("model", "ich9-ehci1");

        uhci1UsbController.setAttribute("model", "ich9-uhci1");
        uhci1UsbMaster.setAttribute("startport", 0);
        uhci1UsbController.appendChild(uhci1UsbMaster);

        uhci2UsbController.setAttribute("model", "ich9-uhci2");
        uhci2UsbMaster.setAttribute("startport", 2);
        uhci2UsbController.appendChild(uhci2UsbMaster);

        uhci3UsbController.setAttribute("model", "ich9-uhci3");
        uhci3UsbMaster.setAttribute("startport", 4);
        uhci3UsbController.appendChild(uhci3UsbMaster);

        xmlDevices.appendChild(ehciUsbController);
        xmlDevices.appendChild(uhci1UsbController);
        xmlDevices.appendChild(uhci2UsbController);
        xmlDevices.appendChild(uhci3UsbController);
    } else if (domainEntry->usbControllerType == ::Hns::USB_CONTROLLER_USB3) {
        QDomElement usbController = xml.createElement("controller");
        usbController.setAttribute("type", "usb");
        usbController.setAttribute("model", "nec-xhci");
        xmlDevices.appendChild(usbController);
    }

    // Spice configuration
    int spicePort = _nextRemoteDisplayPort();
    QString spicePassword = _nextRemoteDisplayPassword();
    domain->setRemoteDisplayPort(spicePort);
    domain->setRemoteDisplayPassword(spicePassword);
    domain->setRemoteDisplayType(Hns::SpiceViewer);
    QDomElement xmlGraphics = xml.createElement("graphics");
    xmlGraphics.setAttribute("type", "spice");
    xmlGraphics.setAttribute("autoport", "no");
    xmlGraphics.setAttribute("listen", "0.0.0.0");
    xmlGraphics.setAttribute("passwd", spicePassword);

    QSettings settings(Diamesh::instance()->configFileName(), QSettings::IniFormat);
    settings.beginGroup(Constants::SETTINGS_NODE_GROUP);
    bool enableSpiceCompression = settings.value(Constants::SETTINGS_ENABLE_SPICE_COMPRESSION, false).toBool();
    bool enableSpiceTLS = settings.value(Constants::SETTINGS_ENABLE_SPICE_TLS, false).toBool();

    QString ca = "";
    QString certSubject = "";

    if (enableSpiceTLS) {
        QString caPath = settings.value(Constants::SETTINGS_SPICE_TLS_CA_CERT_PATH, "").toString();
        QString serverPath = settings.value(Constants::SETTINGS_SPICE_TLS_SERVER_CERT_PATH, "").toString();

        if (caPath.isEmpty()) {
            qCritical() << tr("There is no CA certificate path in node configuration file");
            return false;
        }
        QFile caFile(caPath);
        if (!caFile.exists()) {
            qCritical() << tr("The CA certificate path defined in node configuration file doesn't exist");
        }
        if (!caFile.open(QIODevice::ReadOnly)) {
            qCritical() << tr("The CA certificate path defined in node configuration cannot be read");
        }

        if (serverPath.isEmpty()) {
            qCritical() << tr("There is no server certificate path in node configuration file");
            return false;
        }
        QFile serverFile(serverPath);
        if (!serverFile.exists()) {
            qCritical() << tr("The server certificate path defined in node configuration file doesn't exist");
        }
        if (!serverFile.open(QIODevice::ReadOnly)) {
            qCritical() << tr("The server certificate path defined in node configuration cannot be read");
        }

        QByteArray caArray = caFile.readAll();
        ca = QString(caArray);

        QSslCertificate serverSslCertificate(&serverFile, QSsl::Pem);
        certSubject.append("C=");
        certSubject.append(serverSslCertificate.subjectInfo("C"));
        certSubject.append(", L=");
        certSubject.append(serverSslCertificate.subjectInfo("L"));
        certSubject.append(", O=");
        certSubject.append(serverSslCertificate.subjectInfo("O"));
        certSubject.append(", CN=");
        certSubject.append(serverSslCertificate.subjectInfo("CN"));
    }
    settings.endGroup();

    if (enableSpiceCompression) {
        QDomElement image = xml.createElement("image");
        image.setAttribute("compression", "glz");
        xmlGraphics.appendChild(image);

        QDomElement streaming = xml.createElement("streaming");
        streaming.setAttribute("mode", "filter");
        xmlGraphics.appendChild(streaming);

        QDomElement jpeg = xml.createElement("jpeg");
        jpeg.setAttribute("compression", "always");
        xmlGraphics.appendChild(jpeg);

        QDomElement zlib = xml.createElement("zlib");
        zlib.setAttribute("compression", "always");
        xmlGraphics.appendChild(zlib);
    }
    if (enableSpiceTLS) {
        domain->setRemoteDisplayTls(true);
        xmlGraphics.setAttribute("tlsPort", QString::number(spicePort));
        domain->setRemoteDisplayCa(ca);
        domain->setRemoteDisplayCertSubject(certSubject);
    } else {
        xmlGraphics.setAttribute("port", QString::number(spicePort));
    }

    QString keymap = QString::fromStdString(domainEntry->graphic.keymap);
    if (!keymap.isEmpty())
        xmlGraphics.setAttribute("keymap", keymap);
    xmlDevices.appendChild(xmlGraphics);
    /*adding spice channel*/
    QDomElement xmlSpiceChannel = xml.createElement("channel");
    xmlSpiceChannel.setAttribute("type", "spicevmc");
    QDomElement xmlSpiceChannelType = xml.createElement("target");
    xmlSpiceChannelType.setAttribute("type", "virtio");
    xmlSpiceChannelType.setAttribute("name", "com.redhat.spice.0");
    xmlSpiceChannel.appendChild(xmlSpiceChannelType);
    QDomElement xmlSpiceChannelAlias = xml.createElement("alias");
    xmlSpiceChannelAlias.setAttribute("name", "channel0");
    xmlSpiceChannel.appendChild(xmlSpiceChannelAlias);
    QDomElement xmlSpiceChannelAddress = xml.createElement("address");
    xmlSpiceChannelAddress.setAttribute("type", "virtio-serial");
    xmlSpiceChannelAddress.setAttribute("controller", 0);
    xmlSpiceChannelAddress.setAttribute("bus", 0);
    xmlSpiceChannelAddress.setAttribute("port", 1);
    xmlSpiceChannel.appendChild(xmlSpiceChannelAddress);
    xmlDevices.appendChild(xmlSpiceChannel);

    // QEMU commandline
    xmlDomain.setAttribute("xmlns:qemu", "http://libvirt.org/schemas/domain/qemu/1.0");
    QDomElement qemuCommandLine = xml.createElement("qemu:commandline");
    xmlDomain.appendChild(qemuCommandLine);

    // QEMU commandline environment
    settings.beginGroup(Constants::SETTINGS_PLATFORMS_GROUP);
    settings.beginGroup(Constants::SETTINGS_KVMQEMU_PLATFORM_GROUP);
    QString qemuTmpDir = settings.value(Constants::SETTINGS_KVMQEMU_TMPDIR, Constants::SETTINGS_KVMQEMU_DEFAULT_TMPDIR).toString();
    settings.endGroup();
    settings.endGroup();
    QDomElement qemuEnvTmpDir = xml.createElement("qemu:env");
    qemuEnvTmpDir.setAttribute("name", "TMPDIR");
    qemuEnvTmpDir.setAttribute("value", qemuTmpDir);
    qemuCommandLine.appendChild(qemuEnvTmpDir);

    // QEMU command line arguments
    QStringList qemuCommandLineArgs;

    if (os.arch == "armv7l" && os.machine == "vexpress-a9") {
        /* The vexpress-a9 doesn't support vga cards so we disable it explicitely.
         * This is a workaround until something is done in libvirt about this.
         * See https://www.redhat.com/archives/libvir-list/2014-July/msg01483.html
         */
        qemuCommandLineArgs << "-vga" << "none";
    }

    if (domainEntry->immutable)
        qemuCommandLineArgs << "-snapshot";

    if (!qemuCommandLineArgs.isEmpty()) {
        foreach (const QString &arg, qemuCommandLineArgs) {
            QDomElement qemuArg = xml.createElement("qemu:arg");
            qemuArg.setAttribute("value", arg);
            qemuCommandLine.appendChild(qemuArg);
        }
    }

    return true;
}

bool PlatformKvmQemu::_afterLibvirtUndefine(DomainI *domain)
{
    int actionManagerInterface = Hns::DomainEntryPtr::dynamicCast(domain->getEntry())->aminterface;
    foreach (Netcard *netcard, domain->netcards()) {
        if (actionManagerInterface != netcard->netcardEntry().id)
            removeTap(netcard->vethName());
    }

    return PlatformDomain::_afterLibvirtUndefine(domain);
}

bool PlatformKvmQemu::_beforeLibvirtRestore(DomainI *domain)
{
    QStringList tapNames;
    foreach (Netcard *netcard, domain->netcards())
        tapNames.append(netcard->vethName());

    QString port = QString::number(domain->getRemoteDisplayPort());
    QString password = domain->getRemoteDisplayPassword();
    QRegExp tapRx("^\\s+<target dev='hns\\d+v'/>$");
    QRegExp endRx("^\\s+</devices>$");
    QRegExp portRx("\\s+<graphics type='spice' port='");
    QRegExp passwordRx(".*\\s+passwd='");
    passwordRx.setMinimal(true);

    QFile file(suspendPath(domain));
    if(!file.open(QIODevice::ReadWrite | QIODevice::Unbuffered)) {
        qDebug() << tr("Unable to open the domain's saved file").arg(file.error());
        return false;
    }

    QTextStream stream(&file);
    stream.setCodec(QTextCodec::codecForName("ISO-8859-1"));
    QString line;
    int pos;

    do {
        pos = stream.pos();
        line = stream.readLine();
        if (tapRx.exactMatch(line)) {
            if(tapNames.isEmpty()) {
                qCritical() << tr("The interfaces count in the domain is different from the saved interfaces count");
                return false;
            }
            QString interfaceName = tapNames.takeFirst();
            stream.seek(pos + line.size() - interfaceName.size() - 3);
            stream << interfaceName;
            stream.flush();
        } else if (line.contains(portRx)) {
            stream.seek(pos + portRx.matchedLength());
            stream << port;
            stream.flush();
        } else if (line.contains(passwordRx)) {
            stream.seek(pos + passwordRx.matchedLength());
            stream << password;
            stream.flush();
        }
    } while (!endRx.exactMatch(line));

    if(!tapNames.isEmpty()) {
        qCritical() << tr("The interfaces count in the domain is different from the saved interfaces count");
        return false;
    }
    file.close();
    return true;
}

int PlatformKvmQemu::getEntityFlags()
{
    qlonglong masterFlags = Hns::NodeI::instance()->hnsMaster()->getCapabilitiesFlags();
    int flags = 0;
    if (masterFlags & Hns::MonitoringEnabled) {
        flags |= (EntityI::Monitoring
                  | EntityI::CPUMonitoring
                  | EntityI::MemoryMonitoring
                  | EntityI::NetworkMonitoring
                  | EntityI::IOMonitoring);
    }
    return flags;
}

bool PlatformKvmQemu::canBuild(const::Hns::EntityEntryPtr &entityEntry, const QList<Hns::EntityEntryPtr> &alreadyDefinedEntities)
{
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(entityEntry);
    if (!domainEntry)
        return false;
    QString domainType = QString::fromStdString(domainEntry->type);
    if (domainType != "kvm" && domainType != "qemu")
        return false;
    return PlatformDomain::canBuild(entityEntry, alreadyDefinedEntities);
}

void PlatformKvmQemu::_cleanNetworkInterfaces(const QStringList &interfaces)
{
    foreach (const QString &interface, interfaces)
        removeTap(interface);
}

} // namespace HnsNode
