// hynesim node headers
#include "platformlxc.h"
#include "netcard.h"
#include "nodedomainI.h"
#include "nodeI.h"
#include "nodeconstants.h"

// hynesim common headers
#include "common/hns.h"

// Generated hynesim headers
#include "common/domainentry.h"

// diamesh headers
#include <diamesh/diamesh.h>

// Qt headers
#include <QDebug>
#include <QDomDocument>
#include <QSettings>

namespace HnsNode {

static RegisterPlatform<PlatformLxc> registerMe("LXC");

PlatformLxc *PlatformLxc::_instance = 0;

PlatformLxc::PlatformLxc()
    : PlatformDomain(QStringList("lxc"), "lxc:///")
    , _enableUserNS(false)
    , _uidMappingTarget(0)
    , _uidMappingCount(0)
    , _gidMappingTarget(0)
    , _gidMappingCount(0)
{
    _buildFactoryHelper(&PlatformLxc::canBuild);
    QSettings settings(Diamesh::instance()->configFileName(), QSettings::IniFormat);
    settings.beginGroup(Constants::SETTINGS_PLATFORMS_GROUP);
    settings.beginGroup(Constants::SETTINGS_LXC_PLATFORM_GROUP);
    if (settings.childGroups().contains(Constants::SETTINGS_LXC_IDMAP_GROUP)) {
        settings.beginGroup(Constants::SETTINGS_LXC_IDMAP_GROUP);
        _enableUserNS = true;
        _uidMappingTarget = settings.value(Constants::SETTINGS_LXC_IDMAP_UID_TARGET, 0).toInt();
        _uidMappingCount = settings.value(Constants::SETTINGS_LXC_IDMAP_UID_COUNT, 65536).toInt();
        _gidMappingTarget = settings.value(Constants::SETTINGS_LXC_IDMAP_GID_TARGET, 0).toInt();
        _gidMappingCount = settings.value(Constants::SETTINGS_LXC_IDMAP_GID_COUNT, 65536).toInt();
    }
}

PlatformLxc::~PlatformLxc()
{
}

Platform *PlatformLxc::create()
{
    if (!_instance) {
        try {
            _instance = new PlatformLxc();
            return _instance;
        } catch (const LibvirtConnectException &ex) {
            qCritical() << tr("Unable to create LXC platform");
        }
    }
    return NULL;
}

PlatformLxc *PlatformLxc::instance()
{
    return _instance;
}

bool PlatformLxc::_beforeLibvirtDefine(QDomDocument &xml, DomainI *domain)
{
    if (!PlatformDomain::_beforeLibvirtDefine(xml, domain))
        return false;

    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(domain->getEntry());

    QDomElement xmlDomain = xml.firstChildElement("domain");
    QDomText xmlName = xmlDomain.firstChildElement("name").firstChild().toText();
    xmlName.setNodeValue(QString::fromStdString(domainEntry->uuid));

    QDomElement xmlOs = xmlDomain.firstChildElement("os");
    _addTextNode(xmlOs, "type", "exe");
    _addTextNode(xmlOs, "init", "/sbin/init");

    if (_enableUserNS) {
        QDomElement xmlIdMap = xml.createElement("idmap");
        QDomElement uidMapping = xml.createElement("uid");
        uidMapping.setAttribute("start", 0);
        uidMapping.setAttribute("target", _uidMappingTarget);
        uidMapping.setAttribute("count", _uidMappingCount);
        xmlIdMap.appendChild(uidMapping);
        QDomElement gidMapping = xml.createElement("gid");
        gidMapping.setAttribute("start", 0);
        gidMapping.setAttribute("target", _gidMappingTarget);
        gidMapping.setAttribute("count", _gidMappingCount);
        xmlIdMap.appendChild(gidMapping);
        xmlDomain.appendChild(xmlIdMap);
    }

    QDomElement xmlDevices = xmlDomain.firstChildElement("devices");
    _addTextNode(xmlDevices, "emulator", "/usr/lib/libvirt/libvirt_lxc");
    // filesystem
    if (domainEntry->disks.empty()) {
        qCritical() << tr("PlatformLxc::_beforeLibvirtDefine: no disk in DomainEntry");
        return false;
    }

    QString domainPath = domain->path();
    if (domainPath.isEmpty()) {
        qCritical() << tr("Could not get domain full path");
        return false;
    }

    const Hns::Disk &disk = domainEntry->disks.front();
    QDomElement xmlFs = xml.createElement("filesystem");
    xmlFs.setAttribute("type", QString::fromStdString(disk.type));
    xmlDevices.appendChild(xmlFs);

    QDomElement fsSource = xml.createElement("source");
    QString sourceDir = QString("%1/%2").arg(domainPath).arg(QString::fromStdString(disk.source));
    fsSource.setAttribute("dir", sourceDir);
    xmlFs.appendChild(fsSource);

    if (domainEntry->sharedFolder) {
        QString sharedFolder = Hns::NodeI::instance()->sharedFolder();
        if (sharedFolder.isEmpty()) {
            qCritical() << tr("No shared folder has been defined in the node configuration");
            return false;
        }

        QDomElement xmlFilesystem = xml.createElement("filesystem");
        xmlFilesystem.setAttribute("type", "mount");
        xmlFilesystem.setAttribute("access", "passthrough");
        xmlDevices.appendChild(xmlFilesystem);

        QDomElement filesystemSource = xml.createElement("source");
        filesystemSource.setAttribute("dir", sharedFolder);
        xmlFilesystem.appendChild(filesystemSource);
        QDomElement filesystemTarget = xml.createElement("target");
        filesystemTarget.setAttribute("dir", "/mnt/hynesim-shared");
        xmlFilesystem.appendChild(filesystemTarget);
    }

    QDomElement fsTarget = xml.createElement("target");
    fsTarget.setAttribute("dir", QString::fromStdString(disk.target));
    xmlFs.appendChild(fsTarget);

    QDomElement consoleElement = xml.createElement("console");
    consoleElement.setAttribute("type", "pty");
    xmlDevices.appendChild(consoleElement);

    // This node is already created in PlatformDomain::defineEntity() before the current method call
    QDomElement xmlDomFeatures = xml.elementsByTagName("features").at(0).toElement();
    _addTextNode(xmlDomFeatures, "privnet", "");

    // Network configuration
    int actionManagerInterface = domainEntry->aminterface;
    foreach (Netcard *netcard, domain->netcards()) {
        Hns::Netcard netcardEntry = netcard->netcardEntry();

        QDomElement devIf = xml.createElement("interface");
        devIf.setAttribute("type", "bridge");

        QDomElement ifMac = xml.createElement("mac");
        ifMac.setAttribute("address", netcard->macAddress());
        devIf.appendChild(ifMac);

        if (actionManagerInterface == netcardEntry.id) {
            QDomElement filterRef = xml.createElement("filterref");
            filterRef.setAttribute("filter", Constants::ACTION_MANAGER_NWFILTER_NAME);
            devIf.appendChild(filterRef);
        } else if (getFeatures(HnsNode::StateLink)) {
            QDomElement ifState = xml.createElement("link");
            ifState.setAttribute("state", "down");
            devIf.appendChild(ifState);
        }

        QDomElement ifSource = xml.createElement("source");
        QString sourceBridge = (actionManagerInterface == netcardEntry.id) ? Hns::NodeI::instance()->actionManagerBridge() : netcard->devName();
        ifSource.setAttribute("bridge", sourceBridge);
        devIf.appendChild(ifSource);

        QDomElement ifTarget = xml.createElement("target");
        ifTarget.setAttribute("dev", netcard->vethName());
        devIf.appendChild(ifTarget);

        xmlDevices.appendChild(devIf);

    }

    // Spice configuration
    int spicePort = _nextRemoteDisplayPort();
    domain->setRemoteDisplayPort(spicePort);
    domain->setRemoteDisplayType(Hns::SpiceViewer);

    return true;
}

bool PlatformLxc::_beforeLibvirtCreate(DomainI *domain)
{
    Q_UNUSED(domain)
    /*Workaround for a race condition while starting multiple LXC containers with libvirt*/
    _oneDomainIsStarting.lockForWrite();
    return true;
}

bool PlatformLxc::_afterLibvirtCreate(DomainI *domain)
{
    _oneDomainIsStarting.unlock();
    QString vmName = virDomainGetName(domain->virDomain());
    int spicePort = domain->getRemoteDisplayPort();

    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(domain->getEntry());
    if (!domainEntry) {
        qCritical() << QObject::tr("PlatformLXC::_afterLibvirtCreate: NULL domainEntry");
        return false;
    }

    QString keymap = QString::fromStdString(domainEntry->graphic.keymap);
    startSpiceTerm(vmName, spicePort, keymap);
    return true;
}

void PlatformLxc::_createEntityCleanup()
{
    _oneDomainIsStarting.unlock();
}

int PlatformLxc::getEntityFlags()
{
    qlonglong masterFlags = Hns::NodeI::instance()->hnsMaster()->getCapabilitiesFlags();
    int flags = 0;
    if (masterFlags & Hns::MonitoringEnabled) {
        flags |= (EntityI::Monitoring
                  | EntityI::CPUMonitoring
                  | EntityI::MemoryMonitoring
                  | EntityI::NetworkMonitoring);
    }
    return flags;
}

bool PlatformLxc::canBuild(const Hns::EntityEntryPtr &entityEntry, const QList<Hns::EntityEntryPtr> &alreadyDefinedEntities)
{
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(entityEntry);
    if (!domainEntry)
        return false;
    if (QString::fromStdString(domainEntry->type) != "lxc")
        return false;
    return PlatformDomain::canBuild(entityEntry, alreadyDefinedEntities);
}

} // namespace HnsNode
