/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "platformopenvz.h"
#include "netcard.h"
#include "netutils.h"
#include "nodeI.h"
#include "nodedomainI.h"
#include "nodeconstants.h"

// hynesim server headers
#include "libhnsserver/processrunner.h"

// Generated hynesim headers
#include "common/domainentry.h"

// diamesh headers
#include <diamesh/diamesh.h>

// Libvirt headers
#include <libvirt/libvirt.h>

// Qt headers
#include <QAtomicInt>
#include <QDebug>
#include <QDir>
#include <QFile>
#include <QTextStream>
#include <QDomDocument>
#include <QSettings>

namespace HnsNode {

Q_GLOBAL_STATIC_WITH_ARGS(QAtomicInt, _nextVeId,(100))

static const char * const CT_CONF_DIR = "/etc/vz/conf";

static RegisterPlatform<PlatformOpenVz> registerMe("OpenVZ");

PlatformOpenVz *PlatformOpenVz::_instance = 0;

PlatformOpenVz::PlatformOpenVz()
    : PlatformDomain(QStringList("openvz"), "openvz:///system")
{
    _buildFactoryHelper(&PlatformOpenVz::canBuild);
}

PlatformOpenVz::~PlatformOpenVz()
{
}

Platform *PlatformOpenVz::create()
{
    if (!_instance) {
        try {
            _instance = new PlatformOpenVz();
            return _instance;
        } catch (const LibvirtConnectException &ex) {
            qCritical() << tr("Unable to create OpenVZ platform");
        }
    }
    return NULL;
}

PlatformOpenVz *PlatformOpenVz::instance()
{
    return _instance;
}

bool PlatformOpenVz::_beforeLibvirtDefine(QDomDocument &xml, DomainI *domain)
{
    if (!PlatformDomain::_beforeLibvirtDefine(xml, domain))
        return false;

    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(domain->getEntry());

    QDomElement xmlDomain = xml.firstChildElement("domain");
    QDomText xmlName = xmlDomain.firstChildElement("name").firstChild().toText();
    xmlName.setNodeValue(QString::number(_nextVeId()->fetchAndAddOrdered(1)));

    QDomElement xmlOs = xmlDomain.firstChildElement("os");
    _addTextNode(xmlOs, "type", "exe");

    QDomElement xmlDevices = xmlDomain.firstChildElement("devices");

    // filesystem
    if (domainEntry->disks.empty()) {
        qCritical() << tr("PlatformOpenVz::_beforeLibvirtDefine: no disk in DomainEntry");
        return false;
    }

    QString domainPath = domain->path();
    if (domainPath.isEmpty()) {
        qCritical() << tr("Could not get domain full path");
        return false;
    }

    const Hns::Disk &disk = domainEntry->disks.front();
    QDomElement xmlFs = xml.createElement("filesystem");
    xmlFs.setAttribute("type", QString::fromStdString(disk.type));
    xmlDevices.appendChild(xmlFs);

    QDomElement fsSource = xml.createElement("source");
    QString sourceDir = QString("%1/%2").arg(domainPath).arg(QString::fromStdString(disk.source));
    fsSource.setAttribute("dir", sourceDir);
    xmlFs.appendChild(fsSource);
    QDomElement fsTarget = xml.createElement("target");
    fsTarget.setAttribute("dir", QString::fromStdString(disk.target));
    xmlFs.appendChild(fsTarget);

    // Network configuration
    foreach (Netcard *netcard, domain->netcards()) {
        QDomElement devIf = xml.createElement("interface");
        devIf.setAttribute("type", "ethernet");

        QDomElement ifMac = xml.createElement("mac");
        ifMac.setAttribute("address", netcard->macAddress());
        devIf.appendChild(ifMac);
        QDomElement ifScript = xml.createElement("source");
        ifScript.setAttribute("dev", QString::fromStdString(netcard->netcardEntry().source));
        devIf.appendChild(ifScript);
        QDomElement ifTarget = xml.createElement("target");
        ifTarget.setAttribute("dev", netcard->vethName());
        devIf.appendChild(ifTarget);

        xmlDevices.appendChild(devIf);
    }

    // VNC configuration
    int vncPort = _nextRemoteDisplayPort();
    domain->setRemoteDisplayPort(vncPort);
    domain->setRemoteDisplayType(Hns::VncViewer);

    return true;
}

bool PlatformOpenVz::_afterLibvirtDefine(DomainI *domain)
{
    QString veId = virDomainGetName(domain->virDomain());

    // disable mac_filter on all veth devices
    foreach (Netcard *netcard, domain->netcards()) {
        HnsServer::ProcessRunner process;
        QString program = "vzctl";
        QStringList arguments;
        QString ifName = QString::fromStdString(netcard->netcardEntry().source);
        arguments << "set" << veId << "--save" << "--ifname" << ifName << "--mac_filter" << "off";
        process.runSynchronous(program, arguments, -1);
    }

    // shared folder
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(domain->getEntry());
    if (domainEntry->sharedFolder) {
        QString sharedFolder = Hns::NodeI::instance()->sharedFolder();
        if (sharedFolder.isEmpty()) {
            qCritical() << tr("No shared folder has been defined in the node configuration");
            return false;
        }
        QSettings settings(Diamesh::instance()->configFileName(), QSettings::IniFormat);
        settings.beginGroup(Constants::SETTINGS_NODE_GROUP);
        QString scriptsDirectory = settings.value("scriptsDirectory", ".").toString();

        QFile inputFile(QString("%1/ctid.mount").arg(scriptsDirectory));
        QFile outputFile(QString("%1/%2.mount").arg(CT_CONF_DIR).arg(veId));
        if (!inputFile.open(QIODevice::ReadOnly | QIODevice::Text) || !outputFile.open(QIODevice::WriteOnly | QIODevice::Text)) {
            qWarning() << tr("PlatformOpenVz::_afterLibvirtDefine: error copying ctid.mount file");
            return false;
        }
        QTextStream in(&inputFile);
        QTextStream out(&outputFile);

        int lineIndex = 0;
        while (!in.atEnd()) {
            if (lineIndex == 1)
                out << QString("HYNESIM_SHARED_FOLDER=%1\n").arg(sharedFolder);
            out << in.readLine() << "\n";
            lineIndex++;
        }

        inputFile.close();
        outputFile.close();
        outputFile.setPermissions(QFile::ReadUser | QFile::WriteUser | QFile::ExeUser);
    }

    return PlatformDomain::_afterLibvirtDefine(domain);
}

bool PlatformOpenVz::_afterLibvirtCreate(DomainI *domain)
{
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(domain->getEntry());
    int actionManagerInterface = domainEntry->aminterface;

    foreach (Netcard *netcard, domain->netcards()) {

        Hns::Netcard netcardEntry = netcard->netcardEntry();
        if(actionManagerInterface != netcardEntry.id)
            addToBridge(netcard->devName(), netcard->vethName());
        else
            setupTap(netcard->vethName());
    }

    QString veId = virDomainGetName(domain->virDomain());
    int vncPort = domain->getRemoteDisplayPort();
    startVncTerm(veId, vncPort, "ovz");

    return true;
}

bool PlatformOpenVz::_beforeLibvirtDestroy(DomainI *domain)
{
    QString veId = virDomainGetName(domain->virDomain());
    int vncPort = domain->getRemoteDisplayPort();
    stopVncTerm(veId, vncPort, "ovz");
    return true;
}

bool PlatformOpenVz::_afterLibvirtDestroy(DomainI *domain)
{
    foreach (Netcard *netcard, domain->netcards())
        unsetupTap(netcard->vethName());

    return true;
}

bool PlatformOpenVz::_beforeLibvirtUndefine(DomainI *domain)
{
    if (!PlatformDomain::_beforeLibvirtUndefine(domain))
        return false;

    QString veId = virDomainGetName(domain->virDomain());

    // Change VE_PRIVATE in ctid.conf because we don't want the real private area to be destroyed
    QString fileName = QString("%1/%2.conf").arg(CT_CONF_DIR).arg(veId);
    QFile file(fileName);
    if (!file.open(QFile::ReadOnly | QFile::Text)) {
        qCritical() << tr("PlatformOpenVz::_afterLibvirtUndefine: unable to open %1 for reading").arg(fileName);
        return false;
    }
    QString contents = file.readAll();
    file.close();

    QString tmpName = QString("%1/veprivate_%2").arg(QDir::tempPath()).arg(veId);
    QDir dir;
    if (!dir.mkpath(tmpName)) {
        qCritical() << tr("PlatformOpenVz::_afterLibvirtUndefine: unable to create temporary private area %1").arg(tmpName);
        return false;
    }

    QRegExp regexp("VE_PRIVATE=\"\[^\\0]+\"\\n");
    regexp.setMinimal(true);
    contents.replace(regexp, QString("VE_PRIVATE=\"%1\"\n").arg(tmpName));

    if (!file.open(QFile::WriteOnly | QFile::Truncate | QFile::Text)) {
        qCritical() << tr("PlatformOpenVz::_afterLibvirtUndefine: unable to open %1 for writing").arg(fileName);
        return false;
    }
    QTextStream stream(&file);
    stream << contents;

    // shared folder
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(domain->getEntry());
    if (domainEntry->sharedFolder) {
        if (!QFile::remove(QString("%1/%2.mount").arg(CT_CONF_DIR).arg(veId)))
            qWarning() << tr("PlatformOpenVz::_afterLibvirtUndefine: error removing mount file");
    }

    return true;
}

bool PlatformOpenVz::_afterLibvirtResume(DomainI *domain)
{
    QString veId = virDomainGetName(domain->virDomain());
    int vncPort = domain->getRemoteDisplayPort();
    startVncTerm(veId, vncPort, "ovz");
    return true;
}

bool PlatformOpenVz::_saveDomain(DomainI *domain)
{
    QString veId = virDomainGetName(domain->virDomain());

    _beforeLibvirtDestroy(domain);

    HnsServer::ProcessRunner process;
    QString program = "vzctl";
    QStringList arguments;
    arguments << "chkpnt" << veId << "--dumpfile" << suspendPath(domain);
    process.runSynchronous(program, arguments);
    if (process.exitCode() != 0) {
        qCritical() << tr("Unable to save the state of OpenVZ machine %1").arg(veId);
        return false;
    }

    return true;
}

bool PlatformOpenVz::_restoreDomain(DomainI *domain)
{
    QString veId = virDomainGetName(domain->virDomain());

    HnsServer::ProcessRunner process;
    QString program = "vzctl";
    QStringList arguments;
    arguments << "restore" << veId << "--dumpfile" << suspendPath(domain);
    process.runSynchronous(program, arguments);
    if (process.exitCode() != 0) {
        qCritical() << tr("Unable to restore the state of OpenVZ machine %1").arg(veId);
        return false;
    }

    _afterLibvirtCreate(domain);

    return true;
}

bool PlatformOpenVz::canBuild(const::Hns::EntityEntryPtr &entityEntry, const QList<Hns::EntityEntryPtr> &alreadyDefinedEntities)
{
    Q_UNUSED(alreadyDefinedEntities)
    if (Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(entityEntry)) {
        return (QString::fromStdString(domainEntry->type) == "openvz");
    }
    return false;
}

} // namespace HnsNode
