/******************************************************************************
**
** Copyright (C) 2003-2013 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "platformtopologygate.h"
#include "nodeentityfactoryI.h"
#include "netutils.h"
#include "nodeconstants.h"
#include "nodeI.h"
#include "nodetopologygateI.h"
#include "nodeentityI.h"

// Generated hynesim headers
#include "common/topologygateentry.h"

// diamesh headers
#include <diamesh/diamesh.h>

// Qt headers
#include <QAtomicInt>
#include <QSettings>
#include <QDebug>

// C++ std headers
#include <stdexcept>

namespace HnsNode {

Q_GLOBAL_STATIC_WITH_ARGS(QAtomicInt, _tgId, (1))

PlatformTopologyGate *PlatformTopologyGate::_instance = 0;
static RegisterPlatform<PlatformTopologyGate> registerMe(Constants::SETTINGS_TOPOLOGY_GATE_PLATFORM_GROUP);

PlatformTopologyGate::PlatformTopologyGate()
    : Platform()
{
    if (!_loadPlatformTopologyGate())
        throw std::runtime_error("Failed to load topology bridges");
    Hns::NodeI::instance()->entityFactory()->registerChecker<Hns::TopologyGateEntry, TopologyGateI>(&PlatformTopologyGate::canBuild);
}

PlatformTopologyGate::~PlatformTopologyGate()
{
}

Platform *PlatformTopologyGate::create()
{
    if (!_instance) {
        try {
            _instance = new PlatformTopologyGate();
            return _instance;
        } catch (const std::runtime_error &) {
            qCritical() << tr("Failed to create TopologyGate platform");
        }
    }
    return NULL;
}

PlatformTopologyGate *PlatformTopologyGate::instance()
{
    return _instance;
}

bool PlatformTopologyGate::canBuild(const::Hns::EntityEntryPtr &entityEntry, const QList<Hns::EntityEntryPtr> &alreadyDefinedEntities)
{
    Q_UNUSED(alreadyDefinedEntities)
    Hns::TopologyGateEntryPtr topologyGate = Hns::TopologyGateEntryPtr::dynamicCast(entityEntry);
    if (topologyGate) {
        PlatformTopologyGate *platform = PlatformTopologyGate::instance();
        if (platform && platform->_authorizedGates.keys().contains(QString::fromStdString(topologyGate->networkName)))
            return true;
    }
    return false;
}

bool PlatformTopologyGate::defineEntity(EntityI *entity)
{
    QWriteLocker locker(&_lock);
    TopologyGateI *topologyGate = dynamic_cast<TopologyGateI *>(entity);
    QString networkName = topologyGate->getNetworkName();
    int tgId = _tgId()->fetchAndAddOrdered(1);

    if (!_counterHash.value(networkName)) {
        QString bridgeName = QString("hnstgb%1").arg(tgId);
        if (!createBridge(bridgeName, InterfaceState::Up, _authorizedGates.value(networkName))) {
            qCritical() << tr("Unable to create bridge %1").arg(bridgeName);
            return false;
        }
        _bridgeHash.insert(networkName, bridgeName);
    }

    QString bridgeName = QString("hnstg%1").arg(tgId);
    topologyGate->setBridgeName(bridgeName);
    if (!createBridge(bridgeName, InterfaceState::Down, true)) {
        qCritical() << tr("Unable to create bridge %1").arg(bridgeName);
        return false;
    }
    if (!createVeth(QString("%1a").arg(bridgeName), QString("%1b").arg(bridgeName))) {
        qCritical() << tr("Unable to create veth interfaces %1").arg(bridgeName);
        return false;
    }
    if (!addToBridge(_bridgeHash.value(networkName), QString("%1a").arg(bridgeName))) {
        qCritical() << tr("Unable to add veth interface to bridge %1").arg(_bridgeHash.value(networkName));
        return false;
    }
    if (!addToBridge(bridgeName, QString("%1b").arg(bridgeName))) {
        qCritical() << tr("Unable to add veth interface to bridge %1").arg(bridgeName);
        return false;
    }

    ++_counterHash[networkName];

    Hns::NodeI::instance()->emitSignalRefresh(topologyGate->metaObject()->className());
    return true;
}

bool PlatformTopologyGate::undefineEntity(EntityI *entity)
{
    QWriteLocker locker(&_lock);

    TopologyGateI *topologyGate = dynamic_cast<TopologyGateI *>(entity);
    QString networkName = topologyGate->getNetworkName();

    QString bridgeName = topologyGate->getBridgeName();
    if (!removeBridge(bridgeName)) {
        qCritical() << tr("Unable to remove bridge %1").arg(bridgeName);
    }
    if (!deleteInterface(QString("%1a").arg(bridgeName))) { //removing only one veth pair is enough
        qCritical() << tr("Unable to remove veth interface %1a").arg(bridgeName);
    }
    if (!--_counterHash[networkName]) {
        QString brName = _bridgeHash.value(networkName);
        if (!removeBridge(brName)) {
            qCritical() << tr("Unable to remove bridge %1").arg(brName);
        }
        _bridgeHash.remove(networkName);
    }

    Hns::NodeI::instance()->emitSignalRefresh(topologyGate->metaObject()->className());
    return true;
}

bool PlatformTopologyGate::createEntity(EntityI *entity)
{
    TopologyGateI *topologyGate = dynamic_cast<TopologyGateI *>(entity);
    setInterfaceState(topologyGate->getBridgeName(), HnsNode::Up);
    return true;
}

bool PlatformTopologyGate::destroyEntity(EntityI *entity)
{
    TopologyGateI *topologyGate = dynamic_cast<TopologyGateI *>(entity);
    setInterfaceState(topologyGate->getBridgeName(), HnsNode::Down);
    return true;
}

bool PlatformTopologyGate::_loadPlatformTopologyGate()
{
    QSettings settings(Diamesh::instance()->configFileName(), QSettings::IniFormat, this);
    settings.beginGroup(Constants::SETTINGS_PLATFORMS_GROUP);
    settings.beginGroup(Constants::SETTINGS_TOPOLOGY_GATE_PLATFORM_GROUP);

    int nbNodes = settings.beginReadArray("gates");
    for (int i = 0; i < nbNodes; ++i) {
        settings.setArrayIndex(i);

        QString networkName = settings.value("name").toString();
        if (networkName.isEmpty()) {
            qCritical() << tr("The topology bridge name is empty in the configuration file.");
            return false;
        }
        bool isHub = settings.value("isHub", false).toBool();

        _authorizedGates.insert(networkName, isHub);
    }

    settings.endArray();
    settings.endGroup();
    return true;
}

} // namespace HnsNode
