/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "platformvbox.h"
#include "netcard.h"
#include "netutils.h"
#include "nodeI.h"
#include "nodeentityI.h"
#include "nodedomainI.h"
#include "nodeconstants.h"

// hynesim common headers
#include "common/hns.h"

// hynesim server headers
#include "libhnsserver/processrunner.h"

// Generated hynesim headers
#include "common/domainentry.h"

// diamesh headers
#include <diamesh/coreapplication.h>
#include <diamesh/diamesh.h>

// Qt headers
#include <QDebug>
#include <QDir>
#include <QSettings>
#include <QDomDocument>

// C++ std headers
#include <stdexcept>

namespace HnsNode {

template <class T>
LibvirtVboxConnectionPoolHandler<T>::LibvirtVboxConnectionPoolHandler(double nmax)
    : Hns::PoolHandler<T>(nmax)
{
}

template <class T>
LibvirtVboxConnectionPoolHandler<T>::~LibvirtVboxConnectionPoolHandler()
{
}

template <class T>
T LibvirtVboxConnectionPoolHandler<T>::create()
{
    virConnectPtr connection = virConnectOpen("vbox:///session");
    if (!connection) {
        qCritical() << QObject::tr("LibvirtVboxConnectionPoolHandler: failed to create new connection");
        throw LibvirtConnectException();
    }
    return connection;
}

template <class T>
void LibvirtVboxConnectionPoolHandler<T>::closeMethod(T item)
{
    if (item) {
        virConnectClose(item);
    }
}

static RegisterPlatform<PlatformVBox> registerMe(Constants::SETTINGS_VIRTUALBOX_PLATFORM_GROUP);

PlatformVBox *PlatformVBox::_instance = 0;

PlatformVBox::PlatformVBox()
    : PlatformDomain(QStringList("vbox"), "vbox:///session", true)
{
    _buildFactoryHelper(&PlatformVBox::canBuild);
    _features += HnsNode::Snapshot;
    Diamesh::CoreApplication::instance()->restoreSigChldHandler();
    _poolHandler = new LibvirtVboxConnectionPoolHandler<virConnectPtr>(10);
}

PlatformVBox::~PlatformVBox()
{
    _poolHandler->preDestroy();
    delete _poolHandler;
}

Platform *PlatformVBox::create()
{
    if (!_instance) {
        try {
            _instance = new PlatformVBox();
            // Workaround vbox driver thread-safety issues
            _instance->_closeTemporaryConnections = false;
            return _instance;
        } catch (const LibvirtConnectException &ex) {
            qCritical() << tr("Unable to create VBox platform");
        }
    }
    return NULL;
}

PlatformVBox *PlatformVBox::instance()
{
    return _instance;
}

Hns::Snapshot PlatformVBox::getSnapshot(EntityI *entity, QString snapshotName)
{
    if (!getFeatures(HnsNode::Snapshot))
        return Hns::Snapshot();

    DomainI *domain = qobject_cast<DomainI *>(entity);
    if (!domain) {
        qCritical() << "PlatformVbox::getSnapshot NULL DomainI";
        return Hns::Snapshot();
    }
    QString entryAbsolutePath = entity->path();
    virConnectPtr connection = NULL;
    virDomainPtr dom = NULL;
    QString xml;
    try {
        connection = _getConnection();
        dom = _getVirDomainForConnection(domain->virDomain(), connection);

        virDomainSnapshotPtr snapToDescribe = virDomainSnapshotLookupByName(dom, snapshotName.toLatin1(), 0);
        if (snapToDescribe == NULL)
            throw std::runtime_error(tr("PlatformVbox::getSnapshot libvirt virDomainSnapshotLookupByName failed, name: '%1'").arg(snapshotName).toStdString());

        xml = virDomainSnapshotGetXMLDesc(snapToDescribe, 0);
        if (xml.isEmpty())
            throw std::runtime_error(tr("PlatformVbox::getSnapshot libvirt virDomainSnapshotGetXMLDesc failed").toStdString());
    } catch (std::runtime_error &e) {
        qCritical() << e.what();
    }
    if (xml.isEmpty()) {
        _cleanupIfNeeded(connection, domain, dom);
        return Hns::Snapshot();
    }

    QDomDocument doc;
    doc.setContent(xml);
    QDomElement e = doc.firstChildElement("domainsnapshot");

    QString name = e.firstChildElement("name").text();
    QString parent = e.firstChildElement("parent").text();
    QString creationDateTime = e.firstChildElement("creationTime").text();
    QString description = e.firstChildElement("description").text();

    QDomNodeList disksElement = e.firstChildElement("disks").childNodes();
    QMap<std::string, std::string> disksMap; /*<name, file>*/
    for (int i = 0; i < disksElement.count(); ++i) {
        QString name = disksElement.item(i).toElement().attribute("name");
        QString file = disksElement.item(i).toElement().firstChildElement().attribute("file");
        if (file.contains(entryAbsolutePath)) {
            file.remove(entryAbsolutePath); //Saving only relative path
            if (file.startsWith(QString("/")))
                file.replace(0, 1, QString());
        }
        disksMap.insert(name.toStdString(), file.toStdString());
    }
    QMap<std::string, std::string> parentDisksMap;
    QDomElement domainElement = e.firstChildElement("domain");
    QDomElement devicesElement = domainElement.firstChildElement("devices");
    QDomNodeList diskNodeList = devicesElement.elementsByTagName("disk");
    for (int j = 0, total = diskNodeList.length(); j < total; ++j) {
        QDomElement diskElement = diskNodeList.at(j).toElement();
        QDomElement sourceElement = diskElement.firstChildElement("source");
        QString file = sourceElement.attribute("file");
        if (file.contains(entryAbsolutePath)) {
            file.remove(entryAbsolutePath); //Saving only relative path
            if (file.startsWith(QString("/")))
                file.replace(0, 1, QString());
        }
        QDomElement targetElement = diskElement.firstChildElement("target");
        QString name = targetElement.attribute("dev");
        parentDisksMap.insert(name.toStdString(), file.toStdString());
    }

    Hns::Snapshot snapshot;
    snapshot.name = name.toStdString();
    snapshot.description = description.toStdString();
    snapshot.creationDateTime = creationDateTime.toStdString();
    snapshot.parent = parent.toStdString();
    snapshot.childDisks = disksMap.toStdMap();
    snapshot.parentDisks = parentDisksMap.toStdMap();
    QMutexLocker locker(&_connectionMutex);

    _cleanupIfNeeded(connection, domain, dom);
    return snapshot;
}

int PlatformVBox::getEntityFlags()
{
    qlonglong masterFlags = Hns::NodeI::instance()->hnsMaster()->getCapabilitiesFlags();
    int flags = 0;
    if (masterFlags & Hns::MonitoringEnabled) {
        flags |= (EntityI::Monitoring
                  | EntityI::CPUMonitoring
                  | EntityI::MemoryMonitoring
                  | EntityI::NetworkMonitoring
                  | EntityI::IOMonitoring);
    }
    return flags;
}

bool PlatformVBox::_beforeLibvirtDefine(QDomDocument &xml, DomainI *domain)
{
    if (!PlatformDomain::_beforeLibvirtDefine(xml, domain))
        return false;

    QDomElement xmlDomain = xml.firstChildElement("domain");

    QDomElement xmlOs = xmlDomain.firstChildElement("os");
    _addTextNode(xmlOs, "type", "hvm");

    QDomElement xmlDevices = xmlDomain.firstChildElement("devices");

    // hard disks
    if (!_addDisks(domain, xmlDevices))
        return false;

    // Network configuration
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(domain->getEntry());
    int actionManagerInterface = domainEntry->aminterface;
    bool networkInterfacesCreated = true;
    QStringList networkInterfaces;
    foreach (Netcard *netcard, domain->netcards()) {
        QDomElement devIf = xml.createElement("interface");
        devIf.setAttribute("type", "bridge");

        QDomElement ifSource = xml.createElement("source");
        ifSource.setAttribute("bridge", netcard->vethName());
        devIf.appendChild(ifSource);

        QDomElement ifMac = xml.createElement("mac");
        ifMac.setAttribute("address", netcard->macAddress());
        devIf.appendChild(ifMac);

        QDomElement ifModel = xml.createElement("model");
        ifModel.setAttribute("type", QString::fromStdString(netcard->netcardEntry().model));
        devIf.appendChild(ifModel);

        Hns::Netcard netcardEntry = netcard->netcardEntry();
        if(actionManagerInterface != netcardEntry.id)
        {
            if (getFeatures(HnsNode::StateLink)) {
                QDomElement ifState = xml.createElement("link");
                ifState.setAttribute("state", "down");
                devIf.appendChild(ifState);
            }

            networkInterfacesCreated &= createTap(netcard->vethName());
            if (networkInterfacesCreated) {
                if (addToBridge(netcard->devName(), netcard->vethName()))
                    networkInterfaces << netcard->vethName();
                else
                    networkInterfacesCreated = false;
            }
        } else {
            networkInterfacesCreated &= createAMTap(netcard->vethName(), Hns::NodeI::instance()->actionManagerBridge());
            if (networkInterfacesCreated)
                networkInterfaces << netcard->vethName();
        }

        xmlDevices.appendChild(devIf);

        if (!networkInterfacesCreated)
            break;
    }
    if (!networkInterfacesCreated) {
        _cleanNetworkInterfaces(networkInterfaces);
        return false;
    }

    // RDP configuration
    int rdpPort = _nextRemoteDisplayPort();
    domain->setRemoteDisplayPort(rdpPort);
    domain->setRemoteDisplayType(Hns::RdpViewer);
    QDomElement xmlGraphics = xml.createElement("graphics");
    xmlGraphics.setAttribute("type", "rdp");
    xmlGraphics.setAttribute("port", QString::number(rdpPort));
    xmlGraphics.setAttribute("multiUser", "yes");
    xmlDevices.appendChild(xmlGraphics);



    if (domainEntry->sharedFolder) {
        QString sharedFolder = Hns::NodeI::instance()->sharedFolder();
        if (sharedFolder.isEmpty()) {
            qCritical() << tr("No shared folder has been defined in the node configuration");
            return false;
        }

        QDomElement shareF = xml.createElement("filesystem");
        shareF.setAttribute("type", "mount");
        QDomElement shareSource = xml.createElement("source");
        shareSource.setAttribute("dir", sharedFolder);
        shareF.appendChild(shareSource);
        QDomElement shareTarget = xml.createElement("target");
        shareTarget.setAttribute("dir", "hynesim-shared");
        shareF.appendChild(shareTarget);
        xmlDevices.appendChild(shareF);
    }

    return true;
}

bool PlatformVBox::_afterLibvirtDefine(DomainI *domain)
{
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(domain->getEntry());
    QString vboxUuid = domainEntry->uuid.c_str();

    /*We move the default snapshot directory with VBoxManage*/
    QDir dir;
    QString entryAbsolutePath = domain->path();
    QString entryAbsoluteSnapshotPath = entryAbsolutePath.append("/Snapshots");
    if (!dir.exists(entryAbsoluteSnapshotPath))
        dir.mkpath(entryAbsoluteSnapshotPath);

    HnsServer::ProcessRunner process;
    QString program = "VBoxManage";
    QStringList arguments;
    arguments << "modifyvm" << vboxUuid;
    arguments << "--snapshotfolder" << entryAbsoluteSnapshotPath;
    process.runSynchronous(program, arguments);
    if (process.exitCode() != 0) {
        qCritical() << tr("Unable to change the snapshot folder for machine %1").arg(vboxUuid);
        return false;
    }

    if (!PlatformDomain::_afterLibvirtDefine(domain))
        return false;

    for (int netcardSlot = 0 ; netcardSlot < domain->netcards().size() ; ++netcardSlot) {
        HnsServer::ProcessRunner process;
        QString program = "VBoxManage";
        QStringList arguments;
        QString configString = QString("--nicpromisc%1").arg(netcardSlot + 1);
        arguments << "modifyvm" << vboxUuid << configString << "allow-all";
        process.runSynchronous(program, arguments);
        if (process.exitCode() != 0) {
            qCritical() << tr("Failed to set nicpromisc for interface #%1 of VirtualBox machine %2").arg(netcardSlot).arg(vboxUuid);
            return false;
        }
    }

    if (QList<std::string>::fromStdList(domainEntry->restrictionList).contains("mac")) {
        HnsServer::ProcessRunner process;
        QString program = "VBoxManage";
        QStringList arguments;
        arguments << "modifyvm" << vboxUuid;
        arguments << "--firmware" << "efi";
        arguments << "--mouse" << "usbtablet" << "--keyboard" << "usb";
        arguments << "--ostype" << "MacOS_64";
        process.runSynchronous(program, arguments);
        if (process.exitCode() != 0) {
            qCritical() << tr("Unable to update the VirtualBox machine %1 with Mac OSX restrictions").arg(vboxUuid);
            return false;
        }

        arguments.clear();
        arguments << "setextradata" << vboxUuid;
        arguments << "VBoxInternal2/SmcDeviceKey" << "ourhardworkbythesewordsguardedpleasedontsteal(c)AppleComputerInc";
        process.runSynchronous(program, arguments);
        if (process.exitCode() != 0) {
            qCritical() << tr("Unable to update the VirtualBox machine %1 with Mac OSX restrictions").arg(vboxUuid);
            return false;
        }
    } else {
        HnsServer::ProcessRunner process;
        QString program = "VBoxManage";
        QStringList arguments;
        arguments << "modifyvm" << vboxUuid;
        arguments << "--mouse" << "usbtablet";
        process.runSynchronous(program, arguments);
        if (process.exitCode() != 0) {
            qCritical() << tr("Unable to update the VirtualBox machine %1 with absolute pointing device").arg(vboxUuid);
            return false;
        }
    }

    return true;
}

bool PlatformVBox::_afterLibvirtUndefine(DomainI *domain)
{

    foreach (Netcard *netcard, domain->netcards())
        removeTap(netcard->vethName());

    return PlatformDomain::_afterLibvirtUndefine(domain);
}

bool PlatformVBox::_saveDomain(DomainI *domain)
{
    QString vboxUuid = domain->getEntry()->uuid.c_str();

    HnsServer::ProcessRunner process;
    QString program = "VBoxManage";
    QStringList arguments;
    arguments << "controlvm" << vboxUuid << "savestate";
    if (!process.runSynchronous(program, arguments, -1)) {
        qCritical() << tr("Unable to save the state of VirtualBox machine %1").arg(vboxUuid);
        return false;
    }

    QSettings settings(Diamesh::instance()->configFileName(), QSettings::IniFormat);
    settings.beginGroup(Constants::SETTINGS_PLATFORMS_GROUP);
    settings.beginGroup(Constants::SETTINGS_VIRTUALBOX_PLATFORM_GROUP);
    QString vmSavePath = domain->path().append("/Snapshots");
    QDir vmSaveDir(vmSavePath);
    (vmSaveDir.exists(vmSavePath)) ? : vmSaveDir.mkpath(vmSavePath);
    program = "cp";
    /*We need to use a QDir, because we can't know the exact filename
     *which is a timestamp and the joker "*" doesn't work with QProcess
    */
    QFileInfoList listFiles = vmSaveDir.entryInfoList(QStringList()<<"*.sav",QDir::Files | QDir::NoDotAndDotDot);
    arguments.clear();
    foreach (QFileInfo fileInfo, listFiles) {
        arguments << fileInfo.absoluteFilePath();
    }
    arguments << suspendPath(domain);
    if (!process.runSynchronous(program, arguments, -1)) {
        qCritical() << tr("Unable to copy the saved state of VirtualBox machine %1").arg(vboxUuid);
        return false;
    }

    program = "VBoxManage";
    arguments.clear();
    arguments << "discardstate" << vboxUuid;
    if (!process.runSynchronous(program, arguments, -1)) {
        qCritical() << tr("Unable to discard the state of VirtualBox machine %1").arg(vboxUuid);
        return false;
    }

    return true;
}

bool PlatformVBox::_restoreDomain(DomainI *domain)
{
    QString vboxUuid = domain->getEntry()->uuid.c_str();

    HnsServer::ProcessRunner process;
    QString program = "VBoxManage";
    QStringList arguments;
    arguments << "adoptstate" << vboxUuid << suspendPath(domain);
    process.runSynchronous(program, arguments);
    if (process.exitCode() != 0) {
        qCritical() << tr("Unable to restore the state of VirtualBox machine %1").arg(vboxUuid);
        return false;
    }
    if (!createEntity(domain)) {
        qCritical() << tr("Unable to restart the VirtualBox machine %1").arg(vboxUuid);
        return false;
    }
    return true;
}

virConnectPtr PlatformVBox::_getConnection()
{
    return _poolHandler->getOrCreate();
}

void PlatformVBox::_closeConnection(virConnectPtr connection)
{
    if (connection)
        _poolHandler->release(connection);
}

void PlatformVBox::_monitorMemory(DomainI *domain, Hns::DomainMonitoringDataPtr data)
{
    data->memoryData.available = -1;
    data->memoryData.used = -1;
    data->memoryData.totalSize = -1;
    virDomainMemoryStatStruct stats[VIR_DOMAIN_MEMORY_STAT_NR];
    int nr_stats = virDomainMemoryStats(domain->virDomain(), stats, VIR_DOMAIN_MEMORY_STAT_NR, 0);
    for (int i = 0 ; i < nr_stats ; i++) {
        switch (stats[i].tag) {
        case VIR_DOMAIN_MEMORY_STAT_UNUSED:
            data->memoryData.available = stats[i].val;
            break;
        case VIR_DOMAIN_MEMORY_STAT_AVAILABLE:
            data->memoryData.totalSize = stats[i].val;
            break;
        default:
            break;
        }
    }
    if (data->memoryData.available != -1 && data->memoryData.totalSize != -1)
        data->memoryData.used = data->memoryData.totalSize - data->memoryData.available;
}

bool PlatformVBox::canBuild(const ::Hns::EntityEntryPtr &entityEntry, const QList<Hns::EntityEntryPtr> &alreadyDefinedEntities)
{
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(entityEntry);
    if (!domainEntry)
        return false;
    if (QString::fromStdString(domainEntry->type) != "vbox")
        return false;
    // TODO: move the restriction check to a generic canBuild method
    foreach (const std::string &restriction, QList<std::string>::fromStdList(entityEntry->restrictionList)) {
        if (!_instance->_extendedCapabilities.contains(QString::fromStdString(restriction)))
            return false;
    }
    return PlatformDomain::canBuild(entityEntry, alreadyDefinedEntities);
}

void PlatformVBox::_cleanNetworkInterfaces(const QStringList &interfaces)
{
    foreach (const QString &interface, interfaces)
        removeTap(interface);
}

} // namespace HnsNode
