/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "platformvmware.h"
#include "netcard.h"
#include "netutils.h"
#include "nodeI.h"
#include "nodedomainI.h"

// hynesim server headers
#include "libhnsserver/processrunner.h"

// diamesh headers
#include <diamesh/utils.h>

// Qt headers
#include <QDir>
#include <QDomDocument>
#include <QFile>
#include <QFileInfo>
#include <QStringList>
#include <QTextStream>
#include <QDebug>

namespace HnsNode {

Q_GLOBAL_STATIC(QAtomicInt, _nextVmnetId)

static RegisterPlatform<PlatformVMware> registerMe("VMware");

PlatformVMware *PlatformVMware::_instance = 0;

PlatformVMware::PlatformVMware()
    : PlatformDomain(QStringList("vmware"), "vmwarews:///session")
{
    _buildFactoryHelper(&PlatformVMware::canBuild);
}

PlatformVMware::~PlatformVMware()
{
}

Platform *PlatformVMware::create()
{
    if (!_instance) {
        try {
            _instance = new PlatformVMware();
            return _instance;
        } catch (const LibvirtConnectException &ex) {
            qCritical() << tr("Unable to create VMware platform");
        }
    }
    return NULL;
}

PlatformVMware *PlatformVMware::instance()
{
    return _instance;
}

bool PlatformVMware::_beforeLibvirtDefine(QDomDocument &xml, DomainI *domain)
{
    if (!PlatformDomain::_beforeLibvirtDefine(xml, domain))
        return false;

    QDomElement xmlDomain = xml.firstChildElement("domain");

    QDomElement xmlOs = xmlDomain.firstChildElement("os");
    QDomElement xmlOsType = _addTextNode(xmlOs, "type", "hvm");

    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(domain->getEntry());
    const Hns::OperatingSystem &os = domainEntry->os;
    if (!os.arch.empty())
        xmlOsType.setAttribute("arch", QString::fromStdString(os.arch));

    QDomElement xmlDevices = xmlDomain.firstChildElement("devices");

    // hard disks
    if (!_addDisks(domain, xmlDevices))
        return false;

    // Network configuration
    int actionManagerInterface = domainEntry->aminterface;
    bool networkInterfacesCreated = true;
    QStringList networkInterfaces;
    foreach (Netcard *netcard, domain->netcards()) {
        QDomElement devIf = xml.createElement("interface");
        devIf.setAttribute("type", "bridge");

        QDomElement ifSource = xml.createElement("source");
        ifSource.setAttribute("bridge", QString());
        devIf.appendChild(ifSource);

        QDomElement ifTarget = xml.createElement("target");
        ifTarget.setAttribute("dev", QString("/dev/%1").arg(netcard->vethName()));
        devIf.appendChild(ifTarget);

        QDomElement ifMac = xml.createElement("mac");
        ifMac.setAttribute("address", netcard->macAddress());
        devIf.appendChild(ifMac);

        QDomElement ifModel = xml.createElement("model");
        ifModel.setAttribute("type", QString::fromStdString(netcard->netcardEntry().model));
        devIf.appendChild(ifModel);

        Hns::Netcard netcardEntry = netcard->netcardEntry();
        bool isActionManagerInterface = (actionManagerInterface == netcardEntry.id);
        networkInterfacesCreated &= createVmnet(netcard->vethName(), !isActionManagerInterface);
        if (networkInterfacesCreated) {
            if (!isActionManagerInterface) {
                if (getFeatures(HnsNode::StateLink)) {
                    QDomElement ifState = xml.createElement("link");
                    ifState.setAttribute("state", "down");
                    devIf.appendChild(ifState);
                }

                if(addToBridge(netcard->devName(), netcard->vethName()))
                    networkInterfaces << netcard->vethName();
                else
                    networkInterfacesCreated = false;
            }
        }

        xmlDevices.appendChild(devIf);

        if (!networkInterfacesCreated)
            break;
    }
    if (!networkInterfacesCreated) {
        _cleanNetworkIntefaces(networkInterfaces);
        return false;
    }

    // VNC configuration
    int vncPort = _nextRemoteDisplayPort();
    QString vncPassword = _nextRemoteDisplayPassword();
    domain->setRemoteDisplayPort(vncPort);
    domain->setRemoteDisplayPassword(vncPassword);
    domain->setRemoteDisplayType(Hns::VncViewer);
    QDomElement xmlGraphics = xml.createElement("graphics");
    xmlGraphics.setAttribute("type", "vnc");
    xmlGraphics.setAttribute("port", QString::number(vncPort));
    xmlGraphics.setAttribute("autoport", "no");
    xmlGraphics.setAttribute("listen", "0.0.0.0");
    xmlGraphics.setAttribute("passwd", vncPassword);
    QString keymap = QString::fromStdString(domainEntry->graphic.keymap);
    if (!keymap.isEmpty())
        xmlGraphics.setAttribute("keymap", keymap);
    xmlDevices.appendChild(xmlGraphics);

    return true;
}

bool PlatformVMware::_afterLibvirtDefine(DomainI *domain)
{
    if (!PlatformDomain::_afterLibvirtDefine(domain))
        return false;

    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(domain->getEntry());

    QString vmxPath = _vmxPath(domain);
    QFile vmxFile(vmxPath);
    if (!vmxFile.open(QIODevice::ReadWrite | QIODevice::Text | QIODevice::Append)) {
        qCritical() << tr("Unable to open %1: %2").arg(vmxPath).arg(vmxFile.errorString());
        return false;
    }
    QTextStream stream(&vmxFile);

    if (domainEntry->sharedFolder) {
        QString sharedFolder = Hns::NodeI::instance()->sharedFolder();
        if (sharedFolder.isEmpty()) {
            qCritical() << tr("No shared folder has been defined in the node configuration");
            return false;
        }

        stream << "isolation.tools.hgfs.disable = \"FALSE\"" << endl
               << "sharedFolder.maxNum = \"1\"" << endl
               << "sharedFolder0.present = \"TRUE\"" << endl
               << "sharedFolder0.enabled = \"TRUE\"" << endl
               << "sharedFolder0.readAccess = \"TRUE\"" << endl
               << "sharedFolder0.writeAccess = \"TRUE\"" << endl
               << QString("sharedFolder0.hostPath = \"%1\"").arg(sharedFolder) << endl
               << "sharedFolder0.guestName = \"hynesim-shared\"" << endl
               << "sharedFolder0.expiration = \"never\"" << endl;
    }

    stream << "pciBridge0.present = \"TRUE\"" << endl
           << "pciBridge4.present = \"TRUE\"" << endl
           << "pciBridge4.virtualDev = \"pcieRootPort\"" << endl
           << "pciBridge4.functions = \"8\"" << endl
           << "pciBridge5.present = \"TRUE\"" << endl
           << "pciBridge5.virtualDev = \"pcieRootPort\"" << endl
           << "pciBridge5.functions = \"8\"" << endl
           << "pciBridge6.present = \"TRUE\"" << endl
           << "pciBridge6.virtualDev = \"pcieRootPort\"" << endl
           << "pciBridge6.functions = \"8\"" << endl
           << "pciBridge7.present = \"TRUE\"" << endl
           << "pciBridge7.virtualDev = \"pcieRootPort\"" << endl
           << "pciBridge7.functions = \"8\"" << endl;

    QString uuid = QString::fromStdString(domain->getEntry()->uuid);
    QString lockDir = QString("%1/%2.vmx.lck").arg(domain->path()).arg(uuid);
    Diamesh::removeDirectory(lockDir);

    return true;
}

bool PlatformVMware::_afterLibvirtUndefine(DomainI *domain)
{
    foreach (Netcard *netcard, domain->netcards())
        removeVmnet(netcard->vethName());

    return PlatformDomain::_afterLibvirtUndefine(domain);
}

bool PlatformVMware::_saveDomain(DomainI *domain)
{
    HnsServer::ProcessRunner process;
    QString program = "vmrun";
    QStringList arguments = QStringList() << "-T" << "ws" << "suspend" << _vmxPath(domain);
    process.runSynchronous(program, arguments, -1);
    if (process.exitCode() != 0) {
        qCritical() << tr("Unable to save the state of VMware domain");
        return false;
    }

    return true;
}

bool PlatformVMware::_restoreDomain(DomainI *domain)
{
    QString vmxPath = _vmxPath(domain);
    QFile vmxFile(vmxPath);
    QFileInfo vmxFileInfo(vmxFile);

    QDir vmxDir = vmxFileInfo.absoluteDir();
    QStringList vmssList = vmxDir.entryList(QStringList("*.vmss"), QDir::Files);
    if (vmssList.isEmpty()) {
        qCritical() << tr("Unable to find VMware state file in %1").arg(vmxDir.absolutePath());
        return false;
    } else if (vmssList.size() > 1) {
        qCritical() << tr("There should be only one VMware state file in %1").arg(vmxDir.absolutePath());
        return false;
    }

    if (!vmxFile.open(QIODevice::ReadWrite | QIODevice::Text)) {
        qCritical() << tr("Unable to open %1: %2").arg(vmxPath).arg(vmxFile.errorString());
        return false;
    }
    QTextStream stream(&vmxFile);

    QString vmssFileName = vmssList.first();
    QString vmStateLine = QString("checkpoint.vmState = \"%1\"").arg(vmssFileName);
    QString data = stream.readAll();
    if (!data.contains(vmStateLine))
        stream << vmStateLine << endl;
    return createEntity(domain);
}

Netcard *PlatformVMware::_createNetcard(const Hns::Netcard &netcardEntry)
{
    QString vmnet = QString("%1%2").arg(VMNET_NAME).arg(_nextVmnetId()->fetchAndAddOrdered(1));
    return new Netcard(netcardEntry, vmnet + "b", vmnet);
}

bool PlatformVMware::canBuild(const ::Hns::EntityEntryPtr &entityEntry, const QList<Hns::EntityEntryPtr> &alreadyDefinedEntities)
{
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(entityEntry);
    if (!domainEntry)
        return false;
    if (QString::fromStdString(domainEntry->type) != "vmware")
        return false;
    return PlatformDomain::canBuild(entityEntry, alreadyDefinedEntities);
}

void PlatformVMware::_cleanNetworkIntefaces(const QStringList &interfaces)
{
    foreach (const QString &interface, interfaces)
        removeVmnet(interface);
}

QString PlatformVMware::_vmxPath(DomainI *domain)
{
    QString uuid = QString::fromStdString(domain->getEntry()->uuid);
    return Diamesh::absolutePath(QString("%1/%2.vmx").arg(domain->path()).arg(uuid));
}

} // namespace HnsNode
