/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "platformwifiap.h"
#include "nodeentityfactoryI.h"
#include "nodescriptrunner.h"
#include "nodeentityI.h"
#include "nodewifiapI.h"
#include "nodeI.h"
#include "nodeconstants.h"
#include "netutils.h"
#include "entityplugI.h"

// hynesim common headers
#include "common/initializer.h"

// Generated hynesim headers
#include "common/wifiapentry.h"

// diamesh headers
#include <diamesh/diamesh.h>

// Qt headers
#include <QSettings>
#include <QDebug>

// C++ std headers
#include <stdexcept>

namespace HnsNode {

HNS_STATIC_HASH_INITIALIZER(_hashAuthTypeString, HashAuthTypeString, ::Hns::AuthTypeEnum, QString)
HashAuthTypeString::HashAuthTypeString()
{
    insert(::Hns::NONE,"NONE");
    insert(::Hns::WEP,"WEP");
    insert(::Hns::WPA1,"WPA1");
    insert(::Hns::WPA2,"WPA2");
    insert(::Hns::WPA12,"WPA12");
}

static RegisterPlatform<PlatformWifiAp> registerMe(Constants::SETTINGS_WIFIAP_PLATFORM_GROUP);
PlatformWifiAp *PlatformWifiAp::_instance = 0;

PlatformWifiAp::PlatformWifiAp()
    : Platform()
    , _maxSsidPerNode(0)
    , _singleCounter(0)
{
    if (!_loadWifiDevices())
        throw std::runtime_error("Failed to load wifiap");
    Hns::NodeI::instance()->entityFactory()->registerChecker<Hns::WifiAPEntry, WifiAPI>(&PlatformWifiAp::canBuild);
}

PlatformWifiAp::~PlatformWifiAp(){
}

PlatformWifiAp *PlatformWifiAp::instance()
{
    return _instance;
}

Platform *PlatformWifiAp::create()
{
    if (!_instance) {
        try {
        _instance = new PlatformWifiAp();
        return _instance;
        } catch (const std::runtime_error &) {
            qCritical() << tr("Failed to create WifiAP platform");
        }
    }
    return NULL;
}

bool PlatformWifiAp::defineEntity(HnsNode::EntityI *entity)
{
    WifiAPI *wifiAp = qobject_cast<WifiAPI *>(entity);
    if (!wifiAp) {
        qCritical() << Q_FUNC_INFO << tr("Entity is not a WifiAP");
        return false;
    }
    Hns::WifiAPEntryPtr wifiEntry = Hns::WifiAPEntryPtr::dynamicCast(wifiAp->getEntry());
    if (!wifiEntry) {
        qCritical() << Q_FUNC_INFO << tr("This entry is not a WifiAPEntry");
        return false;
    }

    if (_ssidList.contains(QString::fromStdString(wifiEntry->ssid)))
    {
        qCritical() << Q_FUNC_INFO << tr("There is already an AP with this SSID");
        return false;
    }
    if (_ssidList.size() >= _maxSsidPerNode)
    {
        qCritical() << Q_FUNC_INFO << tr("There is too much AP on this node");
        return false;
    }

    if (_macAdressList.contains(QString::fromStdString(wifiEntry->macAdress)))
    {
        qCritical() << Q_FUNC_INFO << tr("There is already an AP with this MAC Adress");
        return false;
    }

    QString interface = "hnsAp" + QString::number(_singleCounter.fetchAndAddOrdered(1));
    QWriteLocker lock(&_locker);
    _ssidInterface.insert(QString::fromStdString(wifiEntry->ssid),interface);

    QString wifiDevice;
    foreach (QString device, _wifiDevices) {
        if (_busyDevicesHash.value(device) < _maxSsidPerDevicesHash.value(device))
            wifiDevice = device;
    }

    if (wifiDevice.isEmpty()) {
        qCritical() << Q_FUNC_INFO << tr("No wifi devices available");
        return false;
    }
    ScriptRunner addSsid;
    QStringList args;

    args.append(wifiDevice);
    args.append(QString::fromStdString(wifiEntry->macAdress));
    args.append(interface);
    args.append(QString::fromStdString(wifiEntry->ssid));
    args.append(QString::number(int(wifiEntry->hidden)));
    args.append(_hashAuthTypeString()->value(wifiEntry->authType));
    QString passWord = QString::fromStdString(wifiEntry->key);
    //Adding quotes if WEP and non hexadecimal key
    if ((passWord.size() == 5 || passWord.size() == 13 || passWord.size() == 16) && _hashAuthTypeString()->value(wifiEntry->authType) == "WEP"){ //TODO : Creating Constants with this values
        QString key = QString("\"%1\"").arg(passWord);
        args.append(key);
    } else {
        args.append(passWord);
    }
    //Adding Channel
    args.append(QString::number(wifiEntry->channel));

    QString script = "hnsWifiApAddSsid.sh";
    if (!addSsid.runSynchronous(script, args)) {
        qCritical() << Q_FUNC_INFO << tr("Error while running %1").arg(script);
        return false;
    }
    QString bridgeName = interface + "b";

    if (!createBridge(bridgeName, HnsNode::Up, true)) {
        qCritical() << tr("Can't create bridge %1").arg(bridgeName);
        return false;
    }

    WifiAPI *wifiApi = dynamic_cast<WifiAPI *>(entity);
    wifiApi->setSsid(QString::fromStdString(wifiEntry->ssid));
    wifiApi->setBridgeName(bridgeName);
    wifiApi->setInterfaceName(interface);

    _ssidList.append(QString::fromStdString(wifiEntry->ssid));
    _macAdressList.append(QString::fromStdString(wifiEntry->macAdress));
    _busyDevicesHash.insert(wifiDevice, _busyDevicesHash.value(wifiDevice) + 1);
    _ssidDevicesHash.insert(QString::fromStdString(wifiEntry->ssid), wifiDevice);

    if (_ssidList.size() >= _maxSsidPerNode)
        Hns::NodeI::instance()->emitSignalRefresh(wifiApi->metaObject()->className());

    return true;
}

bool PlatformWifiAp::undefineEntity(EntityI *entity)
{
    WifiAPI *wifiAp = qobject_cast<WifiAPI *>(entity);
    if (!wifiAp) {
        qCritical() << Q_FUNC_INFO << tr("Entity is not a WifiAP");
        return false;
    }

    Hns::WifiAPEntryPtr wifiEntry = Hns::WifiAPEntryPtr::dynamicCast(wifiAp->getEntry());
    if (!wifiEntry) {
        qCritical() << Q_FUNC_INFO << tr("This entry is not a WifiAPEntry");
        return false;
    }

    WifiAPI *wifiApi = dynamic_cast<WifiAPI *>(entity);
    QString bridgeName(wifiApi->getBridgeName());
    if (!bridgeName.isEmpty())
        removeBridge(bridgeName);

    ScriptRunner removeIface;
    QStringList args(_ssidInterface.value(QString::fromStdString(wifiEntry->ssid)));
    QString script = "hnsWifiApRemoveIface.sh";
    if (!removeIface.runSynchronous(script, args)) {
        qCritical() << Q_FUNC_INFO << tr("Error while running %1").arg(script);
        return false;
    }
    QWriteLocker lock(&_locker);
    _ssidInterface.remove(QString::fromStdString(wifiEntry->ssid));
    _ssidList.removeAll(QString::fromStdString(wifiEntry->ssid));
    _macAdressList.removeAll(QString::fromStdString(wifiEntry->macAdress));
    QString wifiDevice = _ssidDevicesHash.take(QString::fromStdString(wifiEntry->ssid));
    _busyDevicesHash.insert(wifiDevice, _busyDevicesHash.value(wifiDevice) - 1);

    if(_ssidList.size() == (_maxSsidPerNode- 1) )
        Hns::NodeI::instance()->emitSignalRefresh(wifiApi->metaObject()->className());

    return true;
}

bool PlatformWifiAp::createEntity(EntityI *entity)
{
    WifiAPI *wifiAp = qobject_cast<WifiAPI *>(entity);
    if (!wifiAp) {
        qCritical() << Q_FUNC_INFO << tr("Entity is not a WifiAP");
        return false;
    }

    Hns::WifiAPEntryPtr wifiEntry = Hns::WifiAPEntryPtr::dynamicCast(wifiAp->getEntry());
    if (!wifiEntry) {
        qCritical() << Q_FUNC_INFO << tr("This entry is not a WifiAPEntry");
        return false;
    }
    ScriptRunner startIface;
    QStringList args(_ssidInterface.value(QString::fromStdString(wifiEntry->ssid)));
    QString script = "hnsWifiApStartIface.sh";
    if (!startIface.runSynchronous(script, args, 10000)) {
        qCritical() << Q_FUNC_INFO << tr("Error while running %1").arg(script) << tr("See /var/run/hostapdOutput.%1").arg(args.at(0));
        return false;
    }
    return true;
}

bool PlatformWifiAp::destroyEntity(EntityI *entity)
{
    WifiAPI *wifiAp = qobject_cast<WifiAPI *>(entity);
    if (!wifiAp) {
        qCritical() << Q_FUNC_INFO << tr("Entity is not a WifiAP");
        return false;
    }

    Hns::WifiAPEntryPtr wifiEntry = Hns::WifiAPEntryPtr::dynamicCast(wifiAp->getEntry());
    if (!wifiEntry) {
        qCritical() << Q_FUNC_INFO << tr("This entry is not a WifiAPEntry");
        return false;
    }

    ScriptRunner stopIface;
    QStringList args(_ssidInterface.value(QString::fromStdString(wifiEntry->ssid)));
    QString script = "hnsWifiApStopIface.sh";
    if (!stopIface.runSynchronous(script, args)) {
        qCritical() << Q_FUNC_INFO << tr("Error while running %1").arg(script);
        return false;
    }
    return true;
}

QStringList PlatformWifiAp::checkPeers(QString ssid)
{
    ScriptRunner checkPeers;
    QStringList args;
    args.append(_ssidInterface.value(ssid));
    QString script = "hnsWifiApListConnectedPeers.sh";
    if (!checkPeers.runSynchronous(script, args)) {
        qCritical() << Q_FUNC_INFO << tr("Error while running %1").arg(script);
        return QStringList();
    }
    QString output = checkPeers.getOutput();
    QStringList listPeers = output.split("\n",QString::SkipEmptyParts);
    return (listPeers);
}

bool PlatformWifiAp::canBuild(const::Hns::EntityEntryPtr &entityEntry, const QList<Hns::EntityEntryPtr> &alreadyComputedEntities)
{
    QList<Hns::WifiAPEntryPtr> wifiComputedEntities;
    foreach (Hns::EntityEntryPtr entry, alreadyComputedEntities) {
        if (Hns::WifiAPEntryPtr wifiApEntryPtr = Hns::WifiAPEntryPtr::dynamicCast(entry))
            wifiComputedEntities.append(wifiApEntryPtr);
    }

    if (alreadyComputedEntities.isEmpty()) {
        if (Hns::WifiAPEntryPtr wifiApEntryPtr = Hns::WifiAPEntryPtr::dynamicCast(entityEntry)) {
            QReadLocker lock(&instance()->_locker);
            if (instance()->_ssidList.contains(QString::fromStdString(wifiApEntryPtr->ssid))) {
                // SSID is already used
                return false;
            }
            if (instance()->_macAdressList.contains(QString::fromStdString(wifiApEntryPtr->macAdress))){
                // MAC Address is already used
                return false;
            }
            if (instance()->_ssidList.size() >= instance()->_maxSsidPerNode){
                // Maximum number of SSID reached for the node
                return false;
            }
        }
    } else {
        /*If the list is not empty, we are in the case it is called by the loadbalancer*/
        if (wifiComputedEntities.size() >= ((instance()->_maxSsidPerNode) - instance()->_ssidDevicesHash.size())) /*<-- Case we have asked already for an amount of WifiAp bigger than what the node can handle*/
            return false;
        foreach (Hns::WifiAPEntryPtr wifiApComputedEntry, wifiComputedEntities) {
            Hns::WifiAPEntryPtr wifiApEntryPtr = Hns::WifiAPEntryPtr::dynamicCast(entityEntry);
            if(!wifiApEntryPtr)
                continue;
            if (wifiApEntryPtr->ssid == wifiApComputedEntry->ssid || wifiApEntryPtr->macAdress == wifiApComputedEntry->macAdress)
                return false;

        }
    }
    return true;
}

bool PlatformWifiAp::_loadWifiDevices()
{
    QSettings settings(Diamesh::instance()->configFileName(), QSettings::IniFormat, this);
    settings.beginGroup(Constants::SETTINGS_PLATFORMS_GROUP);
    settings.beginGroup(Constants::SETTINGS_WIFIAP_PLATFORM_GROUP);
    int nbNodes = settings.beginReadArray("wifiDevice");

    for (int i = 0; i < nbNodes; ++i) {
        settings.setArrayIndex(i);

        QString wifiDevice = settings.value("device").toString();
        if (wifiDevice.isEmpty()) {
            qCritical() << tr("Wifi device not provided");
            return false;
        }

        bool ok = false;
        int maxSsidPerDevice = settings.value("maxSsid",4).toInt(&ok);
        _maxSsidPerNode += maxSsidPerDevice;
        if (!ok){
            qCritical() << tr("Maximum number of SSID on this device should be provided");
            return false;
        }
        _wifiDevices.append(wifiDevice);
        _maxSsidPerDevicesHash.insert(wifiDevice, maxSsidPerDevice);
        _busyDevicesHash.insert(wifiDevice, 0);
    }
    settings.endArray();
    settings.endGroup();
    if (_wifiDevices.isEmpty()){
        return false;
    }
    QStringList args(_wifiDevices);
    ScriptRunner checkWirelessCards;
    QString script = "hnsWifiApCheckWirelessCards.sh";
    if (!checkWirelessCards.runSynchronous(script, args)) {
        qCritical() << Q_FUNC_INFO << tr("Error while running %1").arg(script);
        return false;
    }
    _availableWifiDevices = _wifiDevices;
    return true;
}

} // namespace HnsNode
