/******************************************************************************
**
** Copyright (C) 2003-2017 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "wireinterfacemanager.h"
#include "netutils.h"
#include "nodeI.h"

// diamesh headers
#include <diamesh/coreapplication.h>

// Qt headers
#include <QTimer>
#include <QDateTime>
#include <QDebug>

const int timerInterval = 15000;

namespace HnsNode {

WireInterfaceManager::WireInterfaceManager(QObject *parent)
    : QObject(parent)
    , _timer(new QTimer(this))
{
    connect(_timer, SIGNAL(timeout()), this, SLOT(_timeoutSlot()));
    _timer->start(timerInterval);
}

WireInterfaceManager::~WireInterfaceManager()
{
    _timeoutSlot(false);
}

void WireInterfaceManager::deleteInterface(const QString &interfaceName)
{
    QMutexLocker locker(&_mutex);
    qDebug() << tr("Deletion of interface %1 requested").arg(interfaceName);
    _interfaces.insert(interfaceName, QDateTime::currentMSecsSinceEpoch());
}

void WireInterfaceManager::createVxlan(const QString &interfaceName, qlonglong id, const QString &sourceInterface, const QString &dstIp)
{
    QMutexLocker locker(&_mutex);
    if (_interfaces.contains(interfaceName))
        _interfaces.remove(interfaceName);
    ::HnsNode::createVxlan(interfaceName, id, sourceInterface, dstIp);
}

void WireInterfaceManager::createVeth(const QString &fromInterface, const QString &toInterface)
{
    QMutexLocker locker(&_mutex);
    if (_interfaces.contains(fromInterface))
        _interfaces.remove(fromInterface);
    if (_interfaces.contains(toInterface))
        _interfaces.remove(toInterface);
    ::HnsNode::createVeth(fromInterface, toInterface);
}

void WireInterfaceManager::_timeoutSlot(bool wait)
{
    QMutexLocker locker(&_mutex);
    qint64 currentDateTime = QDateTime::currentMSecsSinceEpoch();
    QMutableHashIterator<QString, qint64> it(_interfaces);
    while (it.hasNext()) {
        it.next();
        QString interface = it.key();
        qint64 interfaceDate = it.value();
        if (wait && (interfaceDate + timerInterval > currentDateTime))
            continue;
        if (!HnsNode::deleteInterface(interface))
            qDebug() << tr("Unable to delete interface %1, it may already have been destroyed").arg(interface);
        it.remove();
    }
}

} // namespace HnsNode
