/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include <diamesh/coreapplication.h>
#include <diamesh/sessionhelper.h>
#include <diamesh/diamesh.h>
#include <diamesh/sessionManager.h>
#include <diamesh/utils.h>
#include <QtCore/QDebug>
#include <QtCore/QUuid>
#include <QtCore/QFile>
#include <QtCore/QDir>
#include <QtXml/QDomDocument>
#include <QtCore/QDateTime>

#include "client.h"
#include "common/domainentry.h"

bool setShortName(const QString &entryPath, const QString& shortName)
{
    QFile file(entryPath);
    if(!file.open(QIODevice::ReadOnly | QFile::Text))
    {
        qCritical() << QObject::tr("Test: Unable to read file %1").arg(entryPath);
        return false;
    }

    QString errorStr;
    int errorLine;
    int errorColumn;
    QDomDocument domDocument;

    if (!domDocument.setContent(&file, true, &errorStr, &errorLine, &errorColumn))
    {
        qCritical() << QObject::tr("Error at line %1, column %2: %3").arg(errorLine).arg(errorColumn).arg(errorStr);
        file.close();
        return false;
    }
    file.close();

    QDomElement root = domDocument.documentElement();
    QDomNodeList headerNodeList = root.elementsByTagName("Header");
    QDomElement header = headerNodeList.item(0).toElement();

    header.setAttribute("shortName", shortName);
    header.setAttribute("lastModificationDateTime", QDateTime::currentDateTime().toMSecsSinceEpoch());

    if (!file.open(QIODevice::WriteOnly | QFile::Text)) {
        qCritical() << QObject::tr("Test: Unable to open file").arg(entryPath);
        return false;
    }
    QTextStream out(&file);
    const int xmlIndentSize = 2;
    domDocument.save(out, xmlIndentSize);
    file.close();

    return true;
}

bool testAddEntryFiles(const Hns::EntityCatalogPrx &catalog)
{
    Hns::DomainEntryPtr entry = new Hns::DomainEntry();
    entry->type = "kvm";

    entry->uuid = QUuid::createUuid().toString().mid(1, 36).toLower().toStdString();
    entry->path="/bad/path";
    if (catalog->addEntry(entry)) {
        qCritical() << QObject::tr("The entry should not have been added, as its path is not in any file storage path");
        return false;
    }

    entry->path="Test Catalog:/";
    //entry->path.append(entry->uuid);
    if (!catalog->addEntry(entry)) {
        qCritical() << QObject::tr("The entry should have been added, as its path is in the file storage path");
        return false;
    }

    return true;
}

bool testUuidCollision(const Hns::EntryPtr &baseEntry, const Hns::EntityCatalogPrx &catalog)
{

    QDir dir(baseEntry->path.c_str());
    dir.mkdir("../collision");
    dir.cd("../collision");
    dir.mkdir(baseEntry->uuid.c_str());
    QString XmlPath = QString("%1/hnsEntryConfig.xml").arg(catalog->getFullEntryPathFromMixed(baseEntry->path, baseEntry->uuid).c_str());
    QFile baseXml(XmlPath);
    if(!baseXml.exists()) {
        qDebug() << QObject::tr("The XML file does not exist: %1").arg(XmlPath);
        return false;
    }
    baseXml.copy(XmlPath, QString("%1/%2/hnsEntryConfig.xml").arg(dir.absolutePath()).arg(baseEntry->uuid.c_str()));

    return true;
}

int main(int argc, char **argv)
{
    Diamesh::CoreApplication * app = Diamesh::CoreApplication::create(argc, argv, NULL);
    Q_UNUSED(app);

    Diamesh::SessionHelperFactory factory;
    QSharedPointer<Diamesh::SessionHelper> sessionHelper = factory.connect();
    if(!sessionHelper)
    {
        qCritical() << QObject::tr("The session should have been created, as the password is right.");
        return 1;
    }

    Hns::Client client(sessionHelper.data());
    Hns::EntityCatalogPrx catalog = client.entityCatalog();
    if (!catalog) {
        qCritical() << QObject::tr("The catalog has not been created");
        return 1;
    }

    QMap<std::string, std::list<std::string> > fileStoragesDirs(catalog->getFileStoragesDirs());
    QMap<std::string, std::list<std::string> >::iterator it = fileStoragesDirs.begin();
    while (it != fileStoragesDirs.end()) {
        qDebug() << QObject::tr("File storage : %1").arg(it.key().c_str());

        QList<std::string> dirList = QList<std::string>::fromStdList(it.value());
        for (int i = 0 ; i < dirList.size() ; i++)
            qDebug() << "              " << dirList.at(i).c_str();
        it++;
    }
    // Entries

    // Create an entry with an uuid already used in the catalog

    /*Hns::EntryPtr entry1 = new Hns::Entry(); // now several entries can have the same uuid, this test is obsolete
    entry1->uuid = "011b3585-661c-4a77-a389-714c11eba314";

    QString entry1id = entry1->uuid.c_str();

    if (catalog->addEntry(entry1)) {
        qCritical() << QObject::tr("The entry %1 already exists, it should not have been added to the catalog").arg(entry1->uuid.c_str());
        return 1;
    }*/

    qulonglong entry1id = 3;
    Hns::EntryPtr entry1 = catalog->getEntry(entry1id);

    if (entry1->path == "") {
        qCritical() << QObject::tr("The entry %1 path should not be empty").arg(entry1id);
        return 1;
    }

    entry1->shortName="mysql";

    if (!catalog->updateEntry(entry1)) {
        qCritical() << QObject::tr("Unable to update entry %1").arg(entry1id);
        return 1;
    }

    entry1 = catalog->getEntry(entry1->id);

    if (entry1->shortName != "mysql") {
        qCritical() << QObject::tr("The entry %1 shortName should be \"mysql\"").arg(entry1id);
        return 1;
    }

    QString entryPath = QString("%1/hnsEntryConfig.xml").arg(catalog->getFullEntryPathFromMixed(entry1->path, entry1->uuid).c_str());

    if (!setShortName(entryPath, "server"))
        return 1;

    qulonglong entry2id = 4;
    Hns::EntryPtr entry2 = catalog->getEntry(entry2id);

    if (!entry2) {
        qCritical() << QObject::tr("The entry %1 has not been found").arg(entry2id);
        return 1;
    }

    if (entry2->path == "") {
        qCritical() << QObject::tr("The entry %1 should have a path defined").arg(entry2id);
        return 1;
    }

    QString entry2RealPath = catalog->getFullEntryPathFromMixed(entry2->path, entry2->uuid).c_str();
    QDir entry2Dir(entry2RealPath);
    if (!entry2Dir.exists()) {
        qCritical() << QObject::tr("The entry directory %1 does not exist %1").arg(entry2RealPath);
        return 1;
    }

    if (!Diamesh::removeDirectory(entry2Dir.absolutePath())) {
        qCritical() << QObject::tr("Unable to remove the entry directory %1").arg(entry2Dir.absolutePath());
        return 1;
    }    

    if (!catalog->refresh()) {
        qCritical() << QObject::tr("The catalog could not be refreshed");
        return 1;
    }

    entry1 = catalog->getEntry(entry1->id);

    if (entry1->shortName == "mysql") {
        qCritical() << QObject::tr("The entry %1 shortName should not be \"mysql\" any more").arg(entry1id);
        return 1;
    }

    entry2 = catalog->getEntry(entry2->id);

    if (entry2->path != "") {
        qCritical() << QObject::tr("The entry %1 should not have a path defined any more").arg(entry2id);
        return 1;
    }

    if (!testUuidCollision(entry1, catalog)) return 1;

    if (!testAddEntryFiles(catalog)) return 1;

    return 0;
}
