/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include <diamesh/coreapplication.h>
#include <diamesh/sessionhelper.h>
#include <diamesh/diamesh.h>
#include <diamesh/sessionManager.h>
#include <QtCore/QDebug>
#include <QDir>
#include <QUuid>
#include <QTimer>

#include "client.h"
#include "common/entity.h"
#include "common/wire.h"
#include "common/entry.h"
#include "common/domainentry.h"
#include "common/wireentry.h"
#include "common/hybridnetcardentry.h"
#include "common/switchentry.h"
#include "common/topology.h"
#include "entryserializer.h"
#include "common/proxies_entity.h"
#include "common/nodeinfo.h"
#include "common/proxies_nodeinfo.h"
#include "qiceobj_nodeinfo.h"


void printNodeInfo(Hns::Client &client)
{
    Diamesh::NodeInfoMap nodeInfoMap = client.master()->getNodeInfoMap();
    QMap<QString, Hns::NodeInfoPrx> nodeInfos;
    for (Diamesh::NodeInfoMap::iterator it = nodeInfoMap.begin(); it != nodeInfoMap.end(); it++) {
        QString nodeKey = QString::fromStdString(it->first);
        Diamesh::NodeInfoPrx diameshNodeInfo = it->second;
        Hns::NodeInfoPrx hnsNodeInfo = Hns::NodeInfoPrx::checkedCast(diameshNodeInfo);

        nodeInfos.insert(nodeKey, hnsNodeInfo);
    }
    qDebug() << "%$ ====================================";
    qDebug() << "%$ nbNodes" << nodeInfos.size();
    foreach (Hns::NodeInfoPrx nodeInfo, nodeInfos) {
        qDebug() << "%$ ------------------------------------";
        HnsNode::ResourceInfo resourceInfo = nodeInfo->getResourceInfo();
        qDebug() << "%$ nodeId:" << nodeInfo->getName().c_str();
        qDebug() << "%$ definedEntities:" << resourceInfo.definedEntities;
        qDebug() << "%$ maxCpu:" << resourceInfo.maxResources.cpu;
        qDebug() << "%$ currentCpu:" << resourceInfo.currentResources.cpu;
        qDebug() << "%$ maxMemory:" << resourceInfo.maxResources.memory;
        qDebug() << "%$ currentMemory:" << resourceInfo.currentResources.memory;
    }
    qDebug() << "%$ ====================================";
}

int main(int argc, char **argv)
{
    Diamesh::CoreApplication * app = Diamesh::CoreApplication::create(argc, argv, NULL);
    Q_UNUSED(app);

    Diamesh::SessionHelperFactory factory;
    QSharedPointer<Diamesh::SessionHelper> sessionHelper = factory.connect();
    if(!sessionHelper)
    {
        qCritical() << QObject::tr("The session should have been created, as the password is right.");
        return 1;
    }

    Hns::Client client(sessionHelper.data());

    Hns::TopologyManagerPrx topologyManager = client.master()->getTopologyManager();
    Hns::TopologyPrx topology = topologyManager->createTopology(false);

    Hns::EntityLayoutPtr layout = new Hns::EntityLayout();

    Hns::DomainEntryPtr domainEntry = NULL;
    QList<Hns::EntityEntryPtr> domainEntries;

    domainEntry = new Hns::DomainEntry();
    domainEntry->uuid = QUuid::createUuid().toString().mid(1, 36).toStdString();
    domainEntry->path = "Test Catalog:/";
    domainEntry->shortName = "short name test 1";
    domainEntry->longName = "long name test 1";
    domainEntry->type = "vbox";
    domainEntry->vcpu = 1;
    domainEntry->memory = 262144;
    Hns::Disk disk;
    disk.type = "file";
    disk.device = "disk";
    disk.source = "dummy";
    disk.target = "sda";
    disk.targetBus = "sata";
    domainEntry->disks.push_back(disk);
    domainEntries << domainEntry;

    domainEntry = Hns::DomainEntryPtr::dynamicCast(domainEntry->ice_clone());
    domainEntry->uuid = QUuid::createUuid().toString().mid(1, 36).toStdString();
    domainEntry->shortName = "short name test 2";
    domainEntry->longName = "long name test 2";
    domainEntry->vcpu = 2;
    domainEntry->memory = 524288;
    domainEntries << domainEntry;

    domainEntry = Hns::DomainEntryPtr::dynamicCast(domainEntry->ice_clone());
    domainEntry->uuid = QUuid::createUuid().toString().mid(1, 36).toStdString();
    domainEntry->shortName = "short name test 3";
    domainEntry->longName = "long name test 3";
    domainEntry->vcpu = 1;
    domainEntry->memory = 262144;
    domainEntries << domainEntry;


    QMap<int, Hns::EntityPrx> domainMap;
    foreach (Hns::EntityEntryPtr entry, domainEntries) {
        Hns::EntityPrx domainProxy = topology->createEntityFromEntry(entry, layout);
        if (!domainProxy) {
            qCritical() << QObject::tr("The domain should have been created.");
            return 1;
        }
        domainMap.insert(domainProxy->getId(), domainProxy);
    }

    printNodeInfo(client);

    QMap<Ice::Long, Hns::DefineInfo> defineInfos;
    foreach (Hns::EntityPrx domainProxy, domainMap) {
        Hns::DefineInfo defineInfo;
        Ice::Long entityId = domainProxy->getId();
        defineInfo.entityId = entityId;
        defineInfo.isDefined = true;
        //defineInfo.nodeId = "node1";
        defineInfos.insert(entityId, defineInfo);
    }
    Hns::DefineInfo &defineInfo = defineInfos.begin().value();
    defineInfo.nodeId = "node2";

    defineInfos = QMap<Ice::Long, Hns::DefineInfo>(topology->computeDefineInfos(defineInfos.toStdMap(), Hns::BEHAVIOR_AUTO));

    foreach (Hns::DefineInfo defineInfo, defineInfos) {
        qDebug() << "%$ defineInfo" << defineInfo.nodeId.c_str();
    }

    foreach (Hns::EntityPrx domainProxy, domainMap) {
        Ice::Long entityId = domainProxy->getId();
        Hns::DefineInfo defineInfo = defineInfos.value(entityId);
        if (defineInfo.isDefined)
            domainProxy->define(defineInfo.nodeId);
    }

    printNodeInfo(client);
/*
    QEventLoop l;
    QTimer::singleShot(5000, &l, SLOT(quit()));
    l.exec();
*/

    foreach (Hns::EntityPrx domainProxy, domainMap) {
        topology->removeEntity(domainProxy->getId());
    }

    printNodeInfo(client);

    return 0;
}
