/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of QtIce.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include "qiceobject.h"

#include <Ice/Connection.h>
#include <QtCore/QCoreApplication>
#include <Ice/ObjectAdapter.h>
#include <Ice/Initialize.h>
#include <QtIce/utils.h>

using namespace std;
using namespace QtIce;

QIceObject::QIceObject(QObject *parent) :
    QObject(parent)
{
  moveToThread(qApp->thread());
  QObject::connect(this, SIGNAL(requestDeletion(Ice::ObjectAdapterPtr,Ice::Identity)),
                   this, SLOT(_removeFromAdapter(Ice::ObjectAdapterPtr,Ice::Identity)));
  QObject::connect(this, SIGNAL(removedFromAdapter()), this, SLOT(deleteLater()));
}

QIceObject::~QIceObject()
{
}

void QIceObject::connect(const SignalForwarderPrx & client, const string & signal, const Ice::Current & current)
{
  QWriteLocker locker(&_signalsLock);
  if (qgetenv("QTICE_DEBUG").toInt())
    cout << "QIceObject::connect " << signal << endl;
  QString sigName = QString::fromStdString(signal);

  Ice::ObjectPrx oneWayPrx = client;
  if(client->ice_getEndpoints().empty())
    oneWayPrx = current.con->createProxy(client->ice_getIdentity());

  oneWayPrx = oneWayPrx->ice_oneway();
  SignalForwarderPrx forwarder = SignalForwarderPrx::uncheckedCast(oneWayPrx);

  // Check for duplicate connection
  if (!_signalsHash.contains(sigName, forwarder))
      _signalsHash.insert(sigName, forwarder);
}

void QIceObject::disconnect(const SignalForwarderPrx & client, const string & signal, const Ice::Current &)
{
  QWriteLocker locker(&_signalsLock);
  if (qgetenv("QTICE_DEBUG").toInt())
    cout << "QIceObject::disconnect " << signal << endl;
  QString key = QString::fromStdString(signal);
  QMultiHash<QString, SignalForwarderPrx>::iterator i;
  if(!key.isEmpty())
  {
    i = _signalsHash.find(key);
    while(i != _signalsHash.end() && i.key() == key)
    {
      if(i.value()->ice_getIdentity() == client->ice_getIdentity())
      {
        i = _signalsHash.erase(i);
      }
      else
      {
        ++i;
      }
    }
  }
  else
  {
    // Signal name is empty, disconnect all signals for this client
    i = _signalsHash.begin();
    while(i != _signalsHash.end())
    {
      if(i.value()->ice_getIdentity() == client->ice_getIdentity())
      {
        i = _signalsHash.erase(i);
      }
      else
      {
        ++i;
      }
    }
  }
}

void QIceObject::emitSignal(const char * signal, const char *internalSignal, const QVariantList & args)
{
  QList<SignalForwarderPrx> proxies;
  {
    QReadLocker locker(&_signalsLock);
    proxies = _signalsHash.values(signal);
    if(proxies.isEmpty())
      return;
  }

  QByteArray block;
  QDataStream stream(&block, QIODevice::WriteOnly);
  stream.setVersion(QDataStream::Qt_4_6);
  stream << args;
  if (stream.status() != QDataStream::Ok) {
      cerr << "error serializing signal \"" << signal << "\" arguments" << endl;
      return;
  }

  const char * data = block.constData();
  SignalArgs sigArgs(data, data+block.size());

  foreach(const SignalForwarderPrx & proxy, proxies)
  {
//    cout << "emitSignal " << signal << " to " << proxy->ice_getIdentity().name << endl;
    proxy->begin_forward(signal, internalSignal, sigArgs);
  }
}

void QIceObject::timerEvent(QTimerEvent */*event*/)
{
  if (!__getRef())
      emit removedFromAdapter();
}

Ice::CommunicatorPtr QIceObject::_getIceCommunicator()
{
  return QtIce::iceCommunicator();
}

Ice::OutputStreamPtr QIceObject::_createOutputStream(const Ice::CommunicatorPtr &communicator)
{
    return Ice::createOutputStream(communicator);
}

void QIceObject::_displayError(const QString &message)
{
  cerr << message.toStdString() << endl;
}

void QIceObject::_removeFromAdapter(Ice::ObjectAdapterPtr adapter, Ice::Identity identity)
{
  __setNoDelete(true);
  adapter->remove(identity);
  startTimer(1000);
}

