/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of QtIce.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include <QtCore/QCoreApplication>
#include <QtCore/QStringList>
#include <QtCore/QDebug>
#include <QtCore/QTextCodec>
#include <stdlib.h>
#include <cstdio>
#include <getopt.h>

#include "generatorset.h"
#include "proxygenerator.h"
#include "qiceobjectgenerator.h"

void colorMsgHandler(QtMsgType type, const char * msg)
{
  static const char * colors[] = {
    "\e[36mDebug:", // debug
    "\e[33mWarning:", // warning
    "\e[35mCritical:", // critical
    "\e[31mFatal:" // fatal
  };
  fprintf(stderr, "!!! slice2qt %s %s\e[0m\n", colors[(int)type], msg);
}

void displayHelp();

struct option long_options[] = {
  { "help", no_argument, NULL, 'h'},
  { "output-dir", required_argument, NULL, 'o' },
  { "generator", required_argument, NULL, 'g' },
  { "prefix", required_argument, NULL, 'p'},
  { NULL, 0, NULL, 0 }
};

int main(int argc, char** argv)
{
  QCoreApplication myApp(argc, argv);
  QTextCodec::setCodecForCStrings(QTextCodec::codecForName("UTF-8"));

  qInstallMsgHandler(&colorMsgHandler);

  QHash<QString, GeneratorFactory*> generators;
  generators.insert("proxies", new ProxyGeneratorFactory);
  generators.insert("qiceobj", new QIceObjectGeneratorFactory);

  int c = 0;

  QString generator;
  QString outputDir;
  QStringList includeDirs;
  QString prefix;

  while((c = getopt_long(argc, argv, "I:o:g:p:h", long_options, 0)) != -1)
  {
    switch(c)
    {
      case 'I':
      {
        includeDirs << optarg;
        break;
      }
      case 'o':
      {
        outputDir = optarg;
        break;
      }
      case 'g':
      {
        generator = optarg;
        break;
      }
      case 'p':
      {
        prefix = optarg;
        prefix = prefix.trimmed();
        break;
      }
      case 'h':
      default:
      {
        displayHelp();
      }
    }
  }

  if(argc < 2 || optind > argc)
  {
    qCritical() << "No input file given";
    displayHelp();
  }

  QString inputFile = argv[optind];

  if(!generators.contains(generator))
  {
    qCritical() << "No generator named" << generator << "exists";
    exit(-1);
  }

  GeneratorSet gs;
  gs.addGenerator(generators.value(generator));
  gs.generate(inputFile, outputDir, includeDirs, prefix);

  return 0;
}

void displayHelp()
{
  printf("Usage:\n  %s [options] input file\n\n", qPrintable(qApp->applicationFilePath()));
  printf("Available options:\n");
  printf("-h, --help               Show this message\n");
  printf("-o DIR, --output-dir DIR Create files in the directory DIR\n");
  printf("-I DIR                   Put DIR in the include file search path\n");
  printf("-g GEN, --generator GEN  Use generator GEN to generate code\n");
  printf("-p PREFIX, --prefix PREFIC  Use prefix for include files\n");

  exit(0);
}
