/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of QtIce.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include "proxygenerator.h"

#include <Slice/Parser.h>
#include <Slice/CPlusPlusUtil.h>
#include <Slice/Util.h>
#include <QtCore/QDebug>
#include <QtCore/QFileInfo>
#include <QtCore/QStringList>

using namespace Slice;
using namespace std;

ProxyGenerator::ProxyGenerator(Slice::UnitPtr u, const QString & outputDirPath, const QStringList &includeDirs, const QString &prefix) :
    Generator(u, outputDirPath, includeDirs, "proxies", prefix)
{
}

bool ProxyGenerator::generate()
{
  QString basePath = QString("%1/%2").arg(_outputDirPath).arg(_baseName());

  QFile proxyHeaderFile(QString("%1.h").arg(basePath));
  if(!proxyHeaderFile.open(QIODevice::WriteOnly | QIODevice::Truncate))
  {
    qWarning("Could not open %s for writing!", qPrintable(proxyHeaderFile.fileName()));
    return false;
  }

  QFile proxySourceFile(QString("%1.cpp").arg(basePath));
  if(!proxySourceFile.open(QIODevice::WriteOnly | QIODevice::Truncate))
  {
    qWarning("Could not open %s for writing!", qPrintable(proxySourceFile.fileName()));
    return false;
  }

  QTextStream hStream(&proxyHeaderFile);
  QTextStream sStream(&proxySourceFile);

  ClassList classes = _classList(_unit->modules());

  // use first class namespace as module prefix
  QString modulePrefix;
  if(classes.size()) {
    modulePrefix = QString::fromStdString(classes.front()->scoped().c_str());
    modulePrefix = modulePrefix.left(modulePrefix.lastIndexOf("::"));
    modulePrefix = modulePrefix.replace("::", "_").toUpper() + "_";
  }

  _writeProxyHeaderHeader(hStream, modulePrefix);
  _writeProxySourceHeader(sStream, QFileInfo(proxyHeaderFile).fileName());

  for(ClassList::const_iterator ci = classes.begin() ; ci != classes.end() ; ++ci) {
      if(!_writeProxyFromClass(hStream, sStream, *ci))
          return false;
  }

  _writeProxyHeaderFooter(hStream);

  proxyHeaderFile.close();
  proxySourceFile.close();

  return true;
}

void ProxyGenerator::_writeProxyHeaderHeader(QTextStream & stream, QString modulePrefix)
{
  QFileInfo inputFileInfo(QString::fromStdString(_unit->topLevelFile()));

  QString headerGuard = modulePrefix + _headerGuardString(inputFileInfo.completeBaseName());
  stream << "// This file was generated with slice2qt." << endl
         << "// All modifications will be overwritten." << endl
         << endl;

  stream << endl
         << "#ifndef " << headerGuard << endl
         << "#define " << headerGuard << endl
         << endl;

  QString inputFileHeader = QString("%1.h").arg(inputFileInfo.completeBaseName());

  stream << "#include <QtIce/qiceobjectproxy.h>" << endl
         << "#include \"" << inputFileHeader << "\"" << endl
         << "#include <QtCore/QSharedPointer>" << endl << endl;
}

void ProxyGenerator::_writeProxyHeaderFooter(QTextStream & stream)
{
  QFileInfo inputFileInfo(QString::fromStdString(_unit->topLevelFile()));

  stream << endl << endl
         << "#endif // " << _headerGuardString(inputFileInfo.completeBaseName()) << endl;
}

void ProxyGenerator::_writeProxySourceHeader(QTextStream & stream, const QString & header)
{
  stream << "#include \"" << header << "\"" << endl << endl
         << "#include <QtCore/QMetaType>" << endl
         << "#include <QtCore/QDebug>" << endl
         << endl;
}

Q_GLOBAL_STATIC_WITH_INITIALIZER(QStringList, iceModules, {
  x->append("Freeze");
  x->append("Glacier2");
  x->append("Ice");
  x->append("IceBox");
  x->append("IceGrid");
  x->append("IcePatch2");
  x->append("IceSSL");
  x->append("IceStorm");
});

bool ProxyGenerator::_writeProxyFromClass(QTextStream & hStream, QTextStream & sStream, Slice::ClassDefPtr c)
{
  if(c->bases().empty())
    return true;

  bool inheritsSignalEmitter = false;
  ClassList allBases = c->allBases();
  for(ClassList::const_iterator i = allBases.begin() ; i != allBases.end() ; ++i)
  {
    ClassDefPtr b = *i;
    if(b->name() == "SignalEmitter")
    {
      inheritsSignalEmitter = true;
      break;
    }
  }

  if(!inheritsSignalEmitter)
    return true;

  QString iName = QString::fromStdString(c->scoped().c_str());
  qDebug("Generating proxy for %s", qPrintable(iName));
  QStringList namespaces = iName.split("::", QString::SkipEmptyParts);
  namespaces.removeLast();

  QString baseProxyName = "QIceObjectProxy";
  ClassDefPtr base = c->bases().front();
  QString baseIName = QString::fromStdString(base->name());

  if(baseIName != "SignalEmitter")
  {
    QString scope(QString::fromStdString(base->scope()).split("::", QString::SkipEmptyParts).first());
    // include nothing if the base interface is from an Ice builtin module
    if(!iceModules()->contains(scope))
    {
      if(base->file() != c->file())
      {

        std::vector<std::string> paths;
        foreach(const QString &str, _includeDirs)
          paths.push_back(QFileInfo(str).canonicalFilePath().toStdString());
        QFileInfo baseFileInfo(QString::fromStdString(base->file()));
        QString fullPathFile = QString("%1/%2_%3.h").arg(baseFileInfo.absolutePath()).arg(_baseNamePrefix).arg(baseFileInfo.completeBaseName());

        QString baseHeader = QString("\"%1.h\"").arg(QString::fromStdString(changeInclude(fullPathFile.toStdString(), paths)));
        hStream << "#include " << baseHeader << endl << endl;
      }
      baseProxyName = "Proxy" + baseIName;
      if(base->scope() != "")
        baseProxyName = QString::fromStdString(base->scope()) + baseProxyName;
    }
  }

  foreach (const QString &ns, namespaces) {
      QString beginNamespace = QString("namespace %1 {").arg(ns);
      hStream << beginNamespace << endl << endl;
      sStream << beginNamespace << endl << endl;
  }

  QString proxyName = "Proxy" + QString::fromStdString(c->name());
  hStream << "class " << proxyName << ";" << endl
          << "typedef QIceObjectProxyPointer< " << proxyName << "> " << QString::fromStdString(c->name()) << "QPrx;" << endl << endl
          << "class " << proxyName << " : public " << baseProxyName << endl
          << "{" << endl
          << "  friend class QIceObjectProxyPointer< " << proxyName << ">;" << endl
          << "  Q_OBJECT" << endl << endl
          << "protected:" << endl;

  QString ctorDecl = proxyName + "(" + iName + "Prx prx, Ice::ObjectAdapterPtr adapter = 0)";
  QString dtorDecl = "~" + proxyName + "()";
  hStream << "  " << ctorDecl << ";" << endl
          << endl << _metaDataToSignalsForwarderDeclaration(QList<string>::fromStdList(c->getMetaData()), c)
          << "public:" << endl
          << "  typedef " << iName << "Prx proxy_type;" << endl << endl
          << "  virtual " << dtorDecl << ";" << endl << endl;

  QString ctorDef = proxyName + "(" + iName + "Prx prx, Ice::ObjectAdapterPtr adapter)";
  sStream << proxyName << "::" << ctorDef << " :" << endl
          << "  " << baseProxyName << "(prx, adapter), _prx(prx)" << endl
          << "{" << endl
          << "}" << endl << endl
          << proxyName << "::" << dtorDecl << endl
          << "{" << endl
          << "}" << endl << endl;

  sStream << _metaDataToSignalsForwarderDefinition(QList<string>::fromStdList(c->getMetaData()), proxyName + "::", c);


  ContainedList contents = c->contents();
  QList<OperationPtr> opList;
  for(ContainedList::const_iterator contentsIt = contents.begin() ; contentsIt != contents.end() ; ++contentsIt)
  {
    OperationPtr op = OperationPtr::dynamicCast(*contentsIt);
    if(op)
    {
      opList << op;
      hStream << _operationToDeclarations(op, c) << endl;
      sStream << _operationToDefinition(op, c, proxyName + "::") << endl << endl;
    }
  }

  hStream << endl << "signals:" << endl
          << _metaDataToSignals(QList<string>::fromStdList(c->getMetaData())) << endl
          << endl
          << "private:" << endl
          << iName << "Prx _prx;" << endl
          << "};" << endl << endl;

  foreach(const OperationPtr & op, opList)
  {
    hStream << _operationToCbDeclaration(op, c);
    sStream << _operationToCbDefinition(op, c);
  }

  foreach (const QString &ns, namespaces) {
      QString endNamespace = QString("} // %1").arg(ns);
      hStream << endNamespace << endl << endl;
      sStream << endNamespace << endl << endl;
  }

  return true;
}

QString ProxyGenerator::_operationToDeclarations(Slice::OperationPtr op, Slice::ClassDefPtr c)
{
  Q_UNUSED(c)
  QString decl;
  QTextStream stream(&decl);

  QString opName = QString::fromStdString(op->name());

  stream << "  " << QString::fromStdString(returnTypeToString(op->returnType(), false)) << " ";
  stream << opName << "(";
  ParamDeclList params = op->parameters();
  QStringList paramsList;
  QStringList asyncParams;
  for(ParamDeclList::const_iterator pi = params.begin() ; pi != params.end() ; ++pi)
  {
    ParamDeclPtr p = *pi;
    std::string type = p->isOutParam() ? outputTypeToString(p->type(), false) : inputTypeToString(p->type(), false);
    paramsList << QString::fromStdString(type) + " " + QString::fromStdString(p->name());
    if(!p->isOutParam())
      asyncParams << QString::fromStdString(type) + " " + QString::fromStdString(p->name());
  }
  stream << paramsList.join(", ") << ");" << endl;

  // async
  asyncParams << "QObject * __receiver = 0" << "const char * __successSlot = 0" << "QObject * __fReceiver = 0" << "const char * __failureSlot = 0";
  asyncParams<<QString("QObject *cookie = 0");
  stream << "  void begin_" << opName << "(" << asyncParams.join(", ") << ");" << endl;

  return decl;
}

QString ProxyGenerator::_operationToDefinition(Slice::OperationPtr op, Slice::ClassDefPtr c, const QString & prefix)
{
  QString def;
  QTextStream stream(&def);

  ParamDeclList params = op->parameters();
  QString name = QString::fromStdString(op->name());
  QString cName = QString::fromStdString(c->name());
  QString scope = QString::fromStdString(c->scope());
  QStringList paramsList;
  QStringList asyncParams;
  QStringList argsList;
  QStringList asyncArgs;
  QStringList sigArgs;

  if(op->returnsData() && op->returnType())
    sigArgs << QMetaObject::normalizedType(returnTypeToString(op->returnType(), false).c_str());

  for(ParamDeclList::const_iterator pi = params.begin() ; pi != params.end() ; ++pi)
  {
    ParamDeclPtr p = *pi;
    std::string type = p->isOutParam() ? outputTypeToString(p->type(), false) : inputTypeToString(p->type(), false);
    QString arg = QString::fromStdString(p->name());
    QString param = QString::fromStdString(type) + " " + arg;
    paramsList << param;
    argsList << arg;
    if(p->isOutParam())
    {
      sigArgs << QMetaObject::normalizedType(typeToString(p->type()).c_str());
    }
    else
    {
      asyncParams << param;
      asyncArgs << arg;
    }
  }

  sigArgs<<QString("QObject*");

  stream << QString::fromStdString(returnTypeToString(op->returnType(), false)) << " " << prefix << name
         << "(" << paramsList.join(", ") << ")" << endl
         << "{" << endl
         << "  " << (op->returnsData() ? "return " : "") << "_prx->" << name << "(" << argsList.join(", ") << ");" << endl
         << "}" << endl << endl;

  asyncParams << "QObject * __receiver" << "const char * __successSlot" << "QObject * __fReceiver" << "const char * __failureSlot";
  paramsList<<QString("QObject *cookie");
  QString resultsCbType = "Callback_Proxy" + cName + "_" + name;
  asyncParams<<QString("QObject *cookie");
  stream << "void " << prefix << "begin_" << name
         << "(" << asyncParams.join(", ") << ")" << endl
         << "{" << endl
         << "  " << resultsCbType << "Ptr resultsCb = new " << resultsCbType << ";" << endl;

  QString successSignal = "succeeded(" + sigArgs.join(",") + ")";
  QString failureSignal = "failed(QSharedPointer<Ice::Exception>, QObject*)";
  QString cbType = "Callback_" + cName + "_" + name;
  asyncArgs << "__cb";
  stream << "  resultsCb->moveToThread(this->thread());" << endl;
  stream << "  resultsCb->_cookie = cookie;"<<endl;

  foreach(const QString & sig, sigArgs)
    stream << "  qRegisterMetaType< " << sig << " >(\"" << sig << "\");" << endl;

  stream << "  if (__receiver && __successSlot)" << endl
         << "      QObject::connect(resultsCb.get(), SIGNAL(" << successSignal << "), __receiver, __successSlot);" << endl
         << "  if (__fReceiver && __failureSlot)" << endl
         << "      QObject::connect(resultsCb.get(), SIGNAL(" << failureSignal << "), __fReceiver, __failureSlot);" << endl
         << "  " << scope << cbType << "Ptr __cb = " << scope << "new" << cbType << "(resultsCb, &" << resultsCbType << "::_success, &" << resultsCbType << "::_failure);" << endl
         << "  _prx->begin_" << name << "(" << asyncArgs.join(", ") << ");" << endl
         << "}" << endl << endl;

  return def;
}

QString ProxyGenerator::_operationToCbDeclaration(Slice::OperationPtr op, Slice::ClassDefPtr c)
{
  QString out;
  QTextStream stream(&out);

  QString cName = QString::fromStdString(c->name());
  QString opName = QString::fromStdString(op->name());
  QString className = "Callback_Proxy" + cName + "_" + opName;
  stream << "class " << className << " : public QObject, public IceUtil::Shared" << endl
         << "{" << endl
         << "  Q_OBJECT" << endl << endl
         << "  public:" << endl;

  QStringList args;
  QStringList sigArgs;
  if(op->returnsData() && op->returnType())
  {
    args << QString::fromStdString(inputTypeToString(op->returnType(), false));
    sigArgs << QMetaObject::normalizedType(typeToString(op->returnType()).c_str());
  }

  ParamDeclList params = op->parameters();
  for(ParamDeclList::const_iterator pi = params.begin() ; pi != params.end() ; ++pi)
  {
    ParamDeclPtr p = *pi;
    if(p->isOutParam())
    {
      args << QString::fromStdString(inputTypeToString(p->type(), false));
      sigArgs << QMetaObject::normalizedType(typeToString(p->type()).c_str());
    }
  }
  sigArgs<<QString("QObject*");

  stream << "    void _success(" << args.join(", ") << ");" << endl
         << "    void _failure(const Ice::Exception&);" << endl << endl
         << endl<<endl
         << "    QObject *_cookie;"
         << "  signals:" << endl
         << "    void succeeded(" << sigArgs.join(", ") << ");" << endl
         << "    void failed(QSharedPointer<Ice::Exception>, QObject*);" << endl
         << "};" << endl
         << "typedef IceUtil::Handle<" << className << "> " << className << "Ptr;" << endl << endl;

  return out;
}

QString ProxyGenerator::_operationToCbDefinition(Slice::OperationPtr op, Slice::ClassDefPtr c)
{
  QString out;
  QTextStream stream(&out);

  QString className = "Callback_Proxy" + QString::fromStdString(c->name()) + "_" + QString::fromStdString(op->name());

  QStringList args;
  QStringList sigArgs;
  if(op->returnsData() && op->returnType())
  {
    QString argName = "returnArg";
    args << QString::fromStdString(inputTypeToString(op->returnType(), false)) + " " + argName;
    sigArgs << argName;
  }

  ParamDeclList params = op->parameters();
  for(ParamDeclList::const_iterator pi = params.begin() ; pi != params.end() ; ++pi)
  {
    ParamDeclPtr p = *pi;
    if(p->isOutParam())
    {
      QString argName = QString::fromStdString(p->name());
      args << QString::fromStdString(inputTypeToString(p->type(), false)) + " " + argName;
      sigArgs << argName;
    }
  }
  sigArgs<<QString("_cookie");

  stream << "void " << className << "::" << "_success(" << args.join(", ") << ")" << endl
         << "{" << endl
         << "  emit succeeded(" << sigArgs.join(", ") << ");" << endl
         << "}" << endl << endl;

  stream << "void " << className << "::" << "_failure(const Ice::Exception& ex)" << endl
         << "{" << endl
         << "  emit failed(QSharedPointer<Ice::Exception>(ex.ice_clone()), _cookie);" << endl
         << "}" << endl << endl;

  return out;
}

QString ProxyGenerator::_metaDataToSignals(const QList<std::string> & metaData)
{
  QStringList signalList;
  foreach(const string & str, metaData)
    signalList << QString::fromStdString(str);
  QRegExp rx("^signal:");
  signalList = signalList.filter(rx).replaceInStrings(rx, "");
  signalList.append("");

  QString decl = "  ";
  decl.append(signalList.join(";\n  "));
  return decl;
}

QString ProxyGenerator::_metaDataToSignalsForwarderDeclaration(const QList<std::string> &metaData, ClassDefPtr c)
{
  QStringList signalList;
  foreach(const std::string &str, metaData)
    signalList << QString::fromStdString(str);

  QStringList internalSignalList;
  QRegExp rx("^signal:\\s*void\\s+(.*)\\((.*)\\)");
  foreach(const QString &signal, signalList) {
    if (rx.exactMatch(signal)) {
      QStringList argumentsTypes = rx.cap(2).split(",");
      QStringList internalTypes;
      foreach(QString type, argumentsTypes) {
        if (type.endsWith("Ptr"))
          type.truncate(type.length() - 3);

        bool serialized = false;
        QList<TypePtr> tList = QList<TypePtr>::fromStdList(c->unit()->lookupType(type.toStdString(), false));
        foreach(const TypePtr &tPtr, tList) {
          if (QString::fromStdString(tPtr->typeId()).contains(type) && tPtr->usesClasses()) {
            serialized = true;
            break;
          }
        }
        if (serialized)
          internalTypes.append("Ice::ByteSeq");
        else
          internalTypes.append(type);
      }

      QString internalSignal = "Q_INVOKABLE void _forward_" + rx.cap(1) + "(" + internalTypes.join(",") + ")";
      internalSignalList.append(internalSignal);
    }
  }
  QString decl = "  ";
  internalSignalList.append("");
  decl.append(internalSignalList.join(";\n  "));
  return decl;
}

QString ProxyGenerator::_metaDataToSignalsForwarderDefinition(const QList<string> &metaData, const QString &prefix, ClassDefPtr c)
{
  QString def;
  QTextStream stream(&def);
  QStringList signalList;
  foreach(const string &str, metaData)
    signalList << QString::fromStdString(str);

  QRegExp rx("^signal:\\s*void\\s+(.*)\\((.*)\\)");

  foreach(const QString &signal, signalList) {
    if (rx.exactMatch(signal)) {
      QString signalName = rx.cap(1);
      QString arguments = rx.cap(2);
      QStringList argumentsTypes;
      if (!arguments.isEmpty())
        argumentsTypes = arguments.split(",");
      QStringList methodArguments;
      QStringList emittedArguments;
      QString preEmit;
      for(int i = 0; i < argumentsTypes.count(); ++i) {
        QString argName = "arg" + QString::number(i);
        QString methodArgument;
        QString emittedArgument = argName;
        QString type = argumentsTypes.at(i);

        if (type.endsWith("Ptr"))
          type.truncate(type.length() - 3);

        bool serialized = false;
        QList<TypePtr> tList = QList<TypePtr>::fromStdList(c->unit()->lookupType(type.toStdString(), false));
        foreach(const TypePtr &tPtr, tList) {
          if (QString::fromStdString(tPtr->typeId()).contains(type) && tPtr->usesClasses()) {
            serialized = true;
            break;
          }
        }

        if (serialized) {
          type = argumentsTypes.at(i);
          methodArgument.append("Ice::ByteSeq");
          emittedArgument.append("Final");
          preEmit += "  " + type + " " + emittedArgument + ";\n";
          preEmit += "  if (_deserializeData(" + argName + ", " + emittedArgument + ") == false) {\n";
          preEmit += "    qCritical() << QObject::tr(\"Unable to deserialize signal argument of type '%1'. Could not emit signal '%2'\").arg(\"" + type + "\").arg(\"" + signalName + "\");\n";
          preEmit += "    return;\n";
          preEmit += "  }\n";
        } else {
          methodArgument.append(type);
        }
        methodArgument += " " + argName;
        methodArguments.append(methodArgument);
        emittedArguments.append(emittedArgument);
      }
      stream << "void " << prefix << "_forward_" << signalName << "(" << methodArguments.join(",") << ")" << endl
             << "{" << endl
             << preEmit << endl
             << "  emit " << signalName << "(" << emittedArguments.join(",") << ");" << endl
             << "}" << endl << endl;
    }
  }
  return def;
}

Generator * ProxyGeneratorFactory::create(Slice::UnitPtr u, const QString & outputDirPath, const QStringList &includeDirs, const QString &prefix)
{
  return new ProxyGenerator(u, outputDirPath, includeDirs, prefix);
}
