/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of QtIce.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include "qiceobjectgenerator.h"

#include <Slice/CPlusPlusUtil.h>
#include <Slice/Util.h>
#include <QtCore/QDebug>
#include <QtCore/QFileInfo>
#include <QtCore/QStringList>

using namespace Slice;
using namespace std;

Q_GLOBAL_STATIC(QStringList, signalsTypes)

QIceObjectGenerator::QIceObjectGenerator(Slice::UnitPtr u, const QString & outputDirPath, const QStringList &includeDirs, const QString &prefix) :
    Generator(u, outputDirPath, includeDirs, "qiceobj", prefix)
{
}

bool QIceObjectGenerator::generate()
{
  QString basePath = QString("%1/%2").arg(_outputDirPath).arg(_baseName());

  QFile headerFile(QString("%1.h").arg(basePath));
  if(!headerFile.open(QIODevice::WriteOnly | QIODevice::Truncate))
  {
    qWarning("Could not open %s for writing!", qPrintable(headerFile.fileName()));
    return false;
  }

  QFile sourceFile(QString("%1.cpp").arg(basePath));
  if(!sourceFile.open(QIODevice::WriteOnly | QIODevice::Truncate))
  {
    qWarning("Could not open %s for writing!", qPrintable(sourceFile.fileName()));
    return false;
  }

  QTextStream hStream(&headerFile);
  QTextStream sStream(&sourceFile);

  ClassList classes = _classList(_unit->modules());

  for(ClassList::const_iterator ci = classes.begin() ; ci != classes.end() ; ++ci) {
      ClassDefPtr c = ClassDefPtr::dynamicCast(*ci);
      ContainedList contents = c->contents();
      for(ContainedList::const_iterator contentsIt = contents.begin() ; contentsIt != contents.end() ; ++contentsIt)
        {
            OperationPtr op = OperationPtr::dynamicCast(*contentsIt);
            if(_extractMetaData(op->getMetaData()).contains("amd"))
                _amdOpList.append(op);
        }
  }

  // use first class namespace as module prefix
  QString modulePrefix;
  if(classes.size()) {
    modulePrefix = QString::fromStdString(classes.front()->scoped().c_str());
    modulePrefix = modulePrefix.left(modulePrefix.lastIndexOf("::"));
    modulePrefix = modulePrefix.replace("::", "_").toUpper() + "_";
  }

  _writeProxyHeaderHeader(hStream, modulePrefix);
  _writeProxySourceHeader(sStream, QFileInfo(headerFile).fileName());

  for(ClassList::const_iterator ci = classes.begin() ; ci != classes.end() ; ++ci) {
      if(!_writeProxyFromClass(hStream, sStream, *ci))
          return false;
  }

  _writeProxyHeaderFooter(hStream);

  headerFile.close();
  sourceFile.close();

  return true;
}

void QIceObjectGenerator::_writeProxyHeaderHeader(QTextStream & stream, QString modulePrefix)
{
  QFileInfo inputFileInfo(QString::fromStdString(_unit->topLevelFile()));

  QString headerGuard = modulePrefix + _headerGuardString(inputFileInfo.completeBaseName());
  stream << "// This file was generated with slice2qt." << endl
         << "// All modifications will be overwritten." << endl
         << endl;

  stream << endl
         << "#ifndef " << headerGuard << endl
         << "#define " << headerGuard << endl
         << endl;

  QString inputFileHeader = QString("%1.h").arg(inputFileInfo.completeBaseName());

  stream << "#include <QtIce/qiceobject.h>" << endl
         << "#include \"" << _prefix <<inputFileHeader << "\"" << endl
         << endl;
}

void QIceObjectGenerator::_writeProxyHeaderFooter(QTextStream & stream)
{
  QFileInfo inputFileInfo(QString::fromStdString(_unit->topLevelFile()));

  stream << endl << endl
         << "#endif // " << _headerGuardString(inputFileInfo.completeBaseName()) << endl;
}

void QIceObjectGenerator::_writeProxySourceHeader(QTextStream & stream, const QString & header)
{
  stream << "#include \"" << header << "\"" << endl
         << "#include <QtIce/utils.h>" << endl
         << "#include <QtCore/QDebug>" << endl
         << endl;
}

bool QIceObjectGenerator::_writeProxyFromClass(QTextStream & hStream, QTextStream & sStream, Slice::ClassDefPtr c)
{
  if(c->bases().empty())
    return true;

  bool inheritsSignalEmitter = false;
  ClassList allBases = c->allBases();
  for(ClassList::const_iterator i = allBases.begin() ; i != allBases.end() ; ++i)
  {
    ClassDefPtr b = *i;
    if(b->name() == "SignalEmitter")
    {
      inheritsSignalEmitter = true;
      break;
    }
  }

  if(!inheritsSignalEmitter)
    return true;

  QString iName = QString::fromStdString(c->scoped().c_str());
  qDebug("Generating QIceObject for %s", qPrintable(iName));

  QString baseName = "QIceObject";
  ClassDefPtr base = c->bases().front();
  QString baseIName = QString::fromStdString(base->name());

  QMultiHash<QString, QString> qiceobjMetaData = _extractMetaData(c->getMetaData(), "qtice:qiceobj");
  if(baseIName != "SignalEmitter")
  {
    QString baseHeader = qiceobjMetaData.value("include");

    if(baseHeader.isEmpty() && (c->file() != base->file()))
    {
      QFileInfo baseHeaderFileInfo = QFileInfo(QString::fromStdString(base->file()));
      //baseHeader = QString("%1_%2.h").arg(_baseNamePrefix).arg(baseHeaderFileInfo.completeBaseName());

      std::vector<std::string> paths;
      foreach(const QString &str, _includeDirs)
      paths.push_back(str.toStdString());
      QFileInfo baseFileInfo(QString::fromStdString(base->file()));
      QString fullPathFile = QString("%1/%2_%3.h").arg(baseHeaderFileInfo.absolutePath()).arg(_baseNamePrefix).arg(baseHeaderFileInfo.completeBaseName());
      baseHeader = QString::fromStdString(changeInclude(fullPathFile.toStdString(), paths));
    }
    if(!baseHeader.endsWith(".h"))
      baseHeader.append(".h");
    if(!baseHeader.isEmpty())
      hStream << "#include " << QString("\"%1\"").arg(baseHeader) << endl << endl;

    if(!qiceobjMetaData.value("baseClass").isEmpty())
      baseName = qiceobjMetaData.value("baseClass");
    else
      baseName = QString::fromStdString(base->scoped()).replace("::", "_");
  }

  QString scopedIName = QString::fromStdString(c->scoped());
  QString className = QString(scopedIName).replace("::", "_");
  hStream << "class " << className << " : public " << baseName << ", virtual public " << scopedIName << endl
          << "{" << endl
          << "  Q_OBJECT" << endl << endl
          << "public:" << endl;

  QString ctorParams = qiceobjMetaData.value("baseCtor", "()");
  QString ctorDecl = className + ctorParams;
  QString dtorDecl = "~" + className + "()";
  hStream << "  " << ctorDecl << ";" << endl
          << "  virtual " << dtorDecl << ";" << endl << endl;

  QStringList ctorArgs;
  if (ctorParams != "()") {
      ctorDecl = ctorParams.replace("(", "").replace(")", "");
      int ctorArgsCount = ctorParams.count(',')+1;
      QStringList ctorDeclList = ctorDecl.split(",");
      QStringList ctorDeclArgs;
      ctorDecl.clear();
      for(int i = 0 ; i < ctorArgsCount ; ++i) {
          QString argName = QString("arg%1").arg(i);
          ctorArgs << argName;
          ctorDeclArgs << QString("%1 %2").arg(ctorDeclList.at(i)).arg(argName);
      }
      ctorDecl = QString("%1(%2)").arg(className).arg(ctorDeclArgs.join(", "));
  }
  if (!_amdOpList.isEmpty()){
      sStream<<"#include<QtCore/QFuture>"<<endl;
      sStream<<"#include <QtCore/QtConcurrentRun>"<<endl<<endl<<endl;
  }

  sStream << className << "::" << ctorDecl << " :" << endl
          << "  " << baseName << "(" << ctorArgs.join(", ") << ")" << endl
          << "{" << endl;


  foreach(OperationPtr op,_amdOpList){
      _writeAmdDeclarations(hStream,op,c);
  }

  QString ctorDef;
  QString helpersDefs;

  QStringList sigList = _extractMetaData(c->getMetaData(), "signal").values();
  QStringList sigTypesList;
  if(!sigList.isEmpty())
  {
    sigList.replaceInStrings(QRegExp("^\\w+ "), "");
    QString sigDefs = "signals:\n";

    hStream << "private slots:" << endl;
    foreach(const QString & s, sigList)
    {
      QString signal = QMetaObject::normalizedSignature(qPrintable(s));
      QString helperName = _signalHelperName(signal);

      sigDefs.append("  void " + signal + ";\n");
      ctorDef.append("  QObject::connect(this, SIGNAL(" + signal + "), this, SLOT(" + helperName + "));\n");

      hStream << "  void " << helperName << ";" << endl;
      helpersDefs.append(_defineSignalHelper(className, signal, sigTypesList, c));
    }
    hStream << endl << sigDefs << endl;
  }

  sigTypesList.removeDuplicates();
  foreach(const QString & type, sigTypesList)
    sStream << "  qRegisterMetaType< " << type << " >(\"" << type << "\");" << endl;

  sStream << ctorDef
          << "}" << endl << endl
          << className << "::" << dtorDecl << endl
          << "{" << endl
          << "}" << endl << endl;

  foreach(OperationPtr op,_amdOpList){
      _writeAmdDefinitions(sStream,op,c);
  }

  foreach(const QString & type, sigTypesList)
  {
    if(!signalsTypes()->contains(type))
    {
      sStream << "Q_DECLARE_METATYPE(" << type << ")" << endl;
      signalsTypes()->append(type);
    }
  }

  sStream << endl << helpersDefs << endl;

  hStream << "};" << endl << endl;

  return true;
}

QString QIceObjectGenerator::_signalHelperName(const QString & signal)
{
  return "_emit_" + signal;
}

QString QIceObjectGenerator::_defineSignalHelper(const QString & className, const QString & signal, QStringList & sigTypesList, ClassDefPtr c)
{
  QString helperName = _signalHelperName(signal);
  QString helper;

  QRegExp rx("^(.*)\\((.*)\\)");
  if (rx.exactMatch(signal)) {
    QString signalName = rx.cap(1);
    QString arguments = rx.cap(2);
    QStringList argumentsTypes;
    if (!arguments.isEmpty())
      argumentsTypes = arguments.split(",");

    QStringList methodArguments;
    QStringList emittedTypes;
    QString preEmit;

    for(int i = 0; i < argumentsTypes.count(); ++i) {
      QString type = argumentsTypes.at(i);
      QString emittedType = type;
      QString argName = "arg" + QString::number(i);
      QString methodArgument = type + " " + argName;
      sigTypesList.append(type);

      if (type.endsWith("Ptr"))
        type.truncate(type.length() - 3);

      bool preSerialize = false;
      QList<TypePtr> tList = QList<TypePtr>::fromStdList(c->unit()->lookupType(type.toStdString(), false));
      foreach(const TypePtr &tPtr, tList) {
        if (QString::fromStdString(tPtr->typeId()).contains(type) && tPtr->usesClasses()) {
          preSerialize = true;
          break;
        }
      }

      if (preSerialize) {
        type = argumentsTypes.at(i);
        QString argAsBytes = argName + "Bytes";
        preEmit += "  Ice::ByteSeq " + argAsBytes + ";\n";
        preEmit += "  if (_serializeData(" + argName + ", " + argAsBytes + "))\n";
        preEmit += "    args << QVariant::fromValue(" + argAsBytes + ");\n";
        preEmit += "  else {\n";
        preEmit += "    qCritical() << QObject::tr(\"Unable to serialize signal argument of type '%1'. Could not emit signal '%2'.\").arg(\"" + type + "\").arg(\"" + signal + "\");\n";
        preEmit += "    return;\n";
        preEmit += "  }\n";
        emittedType = "Ice::ByteSeq";
      }
      else
      {
        preEmit += "  args << QVariant::fromValue(" + argName + ");\n";
      }
      methodArguments.append(methodArgument);
      emittedTypes.append(emittedType);
    }
    QString internalSignal = signalName + "(" + emittedTypes.join(",") + ")";

    helper += "void " + className + "::_emit_" + signalName + "(" + methodArguments.join(", ") +  ")\n";
    helper += "{\n";
    helper += "  QVariantList args;\n";
    helper += preEmit;
    helper += "  emitSignal(\"" + signal + "\", \"" + internalSignal + "\", args);\n";
    helper += "}\n\n";
  }
  return helper;
}

Generator * QIceObjectGeneratorFactory::create(Slice::UnitPtr u, const QString & outputDirPath, const QStringList &includeDirs, const QString &prefix)
{
  return new QIceObjectGenerator(u, outputDirPath, includeDirs, prefix);
}

void QIceObjectGenerator::_writeAmdDeclarations(QTextStream &hStream, Slice::OperationPtr op, Slice::ClassDefPtr c)
{
  QString opName = QString::fromStdString(op->name());
  QStringList paramsList;
  QStringList paramNameList;
  //Create foo_async function declaration
  _createPrototypeAsyncFunction(opName,hStream,op,c,false,&paramsList,&paramNameList);
  //Create virtual foo_effective function declaration
  QString effectiveType = QString("virtual void");
  QString fullOpEffectiveName = opName + QString("_effective");
  hStream << "  " << effectiveType << " ";
  hStream << fullOpEffectiveName << "(";
  hStream << paramsList.join(", ") << " = 0;"<< endl;
}

void QIceObjectGenerator::_writeAmdDefinitions(QTextStream &sStream, OperationPtr op, ClassDefPtr c)
{
    QString opName = QString::fromStdString(op->name());
    //Create foo_async function definition
    QStringList paramsList;
    QStringList paramNameList;
    _createPrototypeAsyncFunction(opName,sStream,op,c,true,&paramsList,&paramNameList);
    sStream<<"  QFuture<void> result = QtConcurrent::run(this, ";
    sStream<<"&";
    sStream<<QString::fromStdString(c->scoped()).replace("::","_")<<"::";
    sStream<<opName + QString("_effective");
    sStream<<", ";
    sStream<<paramNameList.join(", ");
    sStream<<", current"<<");";
    sStream<<endl<<"  Q_UNUSED(result)"<<endl<<"  return;"<<endl<<"}";

}

void QIceObjectGenerator::_createPrototypeAsyncFunction(QString name, QTextStream &stream, OperationPtr op, ClassDefPtr c, bool isDef, QStringList *paramsList, QStringList *paramsNameList)
{
    QString fullOpAsyncName = name + QString("_async");
    if (!isDef){
        stream<<QString("  ");
    }
    stream<<"void ";
    if (isDef){
        stream<<QString::fromStdString(c->scoped()).replace("::","_")<<"::";
    }
    stream<<fullOpAsyncName<<"(";

    ParamDeclList params = op->parameters();

    QStringList amdParams;
    QString amdPtr = QString::fromStdString(c->scoped()).split("::").value(1);
    amdPtr.append(QString("::"));
    amdPtr.append(QString("AMD_"));
    amdPtr.append(QString::fromStdString(c->scoped()).split("::").value(2));
    amdPtr.append(QString("_"));
    amdPtr.append(name);
    amdPtr.append(QString("Ptr"));
    amdPtr.append(QString(" &"));
    amdPtr.append(QString("ptr"));
    paramsNameList->append(QString("ptr"));
    amdParams<<QString("const ") + amdPtr;
    *paramsList<<amdParams;

    for(ParamDeclList::const_iterator pi = params.begin() ; pi != params.end() ; ++pi)
    {
      ParamDeclPtr p = *pi;
      std::string type = p->isOutParam() ? outputTypeToString(p->type(), false) : inputTypeToString(p->type(), false);
      *paramsList << QString::fromStdString(type) + " " + QString::fromStdString(p->name());
      paramsNameList->append(QString::fromStdString(p->name()));
    }
    if (isDef){
        stream << paramsList->join(", ") <<QString(", const Ice::Current &current")<< ")";
    } else {
        *paramsList<<QString("const::Ice::Current & = ::Ice::Current())");
        stream<< paramsList->join(", ");
    }


    if (isDef){
        stream<<"{"<<endl;
    } else {
        stream<<";"<<endl;
    }
}
